Package: TemporalHazard
Title: Temporal Parametric Hazard Modeling
Version: 1.1.0
Authors@R: person("John", "Ehrlinger", email = "john.ehrlinger@gmail.com", role = c("aut", "cre", "cph"))
URL: https://ehrlinger.github.io/temporal_hazard/,
        https://github.com/ehrlinger/temporal_hazard
BugReports: https://github.com/ehrlinger/temporal_hazard/issues
Description: Provides native R implementations of the multiphase parametric
    hazard model of Blackstone, Naftel, and Turner (1986)
    <doi:10.1080/01621459.1986.10478314> with a focus on behavioral parity,
    transparent numerics, and reproducible validation against reference
    outputs from the original 'C'/'SAS' HAZARD program, originally developed at
    the University of Alabama at Birmingham (UAB). The 'SAS'/'C' code and this R
    package are currently developed and maintained at The Cleveland Clinic
    Foundation, and the R code was wholly developed at The Cleveland Clinic
    Foundation. The generalized temporal
    decomposition family extends to longitudinal mixed-effects settings
    (Rajeswaran et al. 2018 <doi:10.1177/0962280215623583>). The package is
    intentionally implemented in pure R first; performance-critical paths may
    later be accelerated with 'Rcpp' without changing the public interface.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
VignetteBuilder: quarto
Depends: R (>= 4.1.0)
Imports: survival
Suggests: covr, ggplot2, knitr, lintr, numDeriv, pkgdown, quarto,
        roxygen2, rmarkdown, scales, testthat (>= 3.0.0)
LazyData: true
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
RoxygenNote: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-12 16:43:32 UTC; ehrlinj
Author: John Ehrlinger [aut, cre, cph]
Maintainer: John Ehrlinger <john.ehrlinger@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 17:30:02 UTC
Built: R 4.5.2; ; 2026-06-12 20:12:04 UTC; unix

Package: TemporalHazard
Title: Temporal Parametric Hazard Modeling
Version: 1.1.0
Authors@R: person("John", "Ehrlinger", email = "john.ehrlinger@gmail.com", role = c("aut", "cre", "cph"))
URL: https://ehrlinger.github.io/temporal_hazard/,
        https://github.com/ehrlinger/temporal_hazard
BugReports: https://github.com/ehrlinger/temporal_hazard/issues
Description: Provides native R implementations of the multiphase parametric
    hazard model of Blackstone, Naftel, and Turner (1986)
    <doi:10.1080/01621459.1986.10478314> with a focus on behavioral parity,
    transparent numerics, and reproducible validation against reference
    outputs from the original 'C'/'SAS' HAZARD program, originally developed at
    the University of Alabama at Birmingham (UAB). The 'SAS'/'C' code and this R
    package are currently developed and maintained at The Cleveland Clinic
    Foundation, and the R code was wholly developed at The Cleveland Clinic
    Foundation. The generalized temporal
    decomposition family extends to longitudinal mixed-effects settings
    (Rajeswaran et al. 2018 <doi:10.1177/0962280215623583>). The package is
    intentionally implemented in pure R first; performance-critical paths may
    later be accelerated with 'Rcpp' without changing the public interface.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
VignetteBuilder: quarto
Depends: R (>= 4.1.0)
Imports: survival
Suggests: covr, ggplot2, knitr, lintr, numDeriv, pkgdown, quarto,
        roxygen2, rmarkdown, scales, testthat (>= 3.0.0)
LazyData: true
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
RoxygenNote: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-12 16:43:32 UTC; ehrlinj
Author: John Ehrlinger [aut, cre, cph]
Maintainer: John Ehrlinger <john.ehrlinger@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 17:30:02 UTC
Built: R 4.5.2; ; 2026-06-12 20:31:54 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,hzr_stepwise)
S3method(coef,hazard)
S3method(predict,hazard)
S3method(print,hazard)
S3method(print,hzr_bootstrap)
S3method(print,hzr_calibrate)
S3method(print,hzr_competing_risks)
S3method(print,hzr_deciles)
S3method(print,hzr_gof)
S3method(print,hzr_kaplan)
S3method(print,hzr_nelson)
S3method(print,hzr_phase)
S3method(print,hzr_stepwise)
S3method(print,summary.hazard)
S3method(print,summary.hzr_stepwise)
S3method(summary,hazard)
S3method(summary,hzr_stepwise)
S3method(vcov,hazard)
export(hazard)
export(hzr_argument_mapping)
export(hzr_bootstrap)
export(hzr_calibrate)
export(hzr_clamp_prob)
export(hzr_competing_risks)
export(hzr_deciles)
export(hzr_decompos)
export(hzr_decompos_g3)
export(hzr_gof)
export(hzr_kaplan)
export(hzr_log1mexp)
export(hzr_log1pexp)
export(hzr_nelson)
export(hzr_phase)
export(hzr_phase_cumhaz)
export(hzr_phase_hazard)
export(hzr_stepwise)
export(is_hzr_phase)
export(stepwise_trace)
importFrom(stats,dnorm)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,setNames)
importFrom(survival,Surv)
importFrom(utils,read.table)
importFrom(utils,write.csv)

# TemporalHazard

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/TemporalHazard)](https://CRAN.R-project.org/package=TemporalHazard)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/TemporalHazard)](https://CRAN.R-project.org/package=TemporalHazard)
[![CRAN downloads total](https://cranlogs.r-pkg.org/badges/grand-total/TemporalHazard)](https://CRAN.R-project.org/package=TemporalHazard)
[![R-CMD-check](https://github.com/ehrlinger/temporal_hazard/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ehrlinger/temporal_hazard/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/ehrlinger/temporal_hazard/branch/main/graph/badge.svg)](https://app.codecov.io/gh/ehrlinger/temporal_hazard?branch=main)
[![lint](https://github.com/ehrlinger/temporal_hazard/actions/workflows/lint.yaml/badge.svg)](https://github.com/ehrlinger/temporal_hazard/actions/workflows/lint.yaml)
[![pkgdown site](https://img.shields.io/badge/docs-pkgdown-blue)](https://ehrlinger.github.io/temporal_hazard/)
![active](https://www.repostatus.org/badges/latest/active.svg)
<!-- badges: end -->

**TemporalHazard** is a pure-R implementation of the multiphase parametric
hazard model of Blackstone, Naftel, and Turner (1986). It decomposes the
overall hazard of an event into additive temporal phases --- early, constant,
and late --- each governed by the generalized temporal decomposition family.
This structure captures real clinical risk patterns that standard single-distribution
models (Weibull, log-normal) cannot represent.

## Provenance and maintenance

The original SAS/C HAZARD code was developed at the University of Alabama at
Birmingham (UAB). The SAS/C code and this R package are currently developed
and maintained at The Cleveland Clinic Foundation. The R code in
TemporalHazard was wholly developed at The Cleveland Clinic Foundation.

## Why multiphase?

After cardiac surgery, the risk of death is not constant. It starts high in
the immediate post-operative period (early phase), settles to a low background
rate (constant phase), and eventually rises again as patients age (late phase).
A single Weibull curve forces a monotone shape; a multiphase model captures all
three regimes simultaneously.

<img src="man/figures/readme-hazard-phases.png" width="100%" alt="Additive phase decomposition showing early, constant, and late hazard components summing to the total hazard curve (CABGKUL, n=5,880)" />

The resulting survival curve closely tracks the nonparametric Kaplan-Meier estimate
while providing a smooth, parametric representation that supports covariate
adjustment, prediction, and extrapolation.

<img src="man/figures/readme-survival.png" width="100%" alt="Multiphase parametric survival curve overlaid on the Kaplan-Meier estimate from the CABGKUL dataset (n=5,880)" />

## Key capabilities

| Feature | Status |
|:---|:---:|
| Multi-phase hazard modeling (early, constant, late phases) | :white_check_mark: |
| Five parametric distributions (Weibull, exponential, log-logistic, log-normal, multiphase) | :white_check_mark: |
| Right, left, interval, and counting-process censoring | :white_check_mark: |
| Repeating events (epoch decomposition via `Surv(start, stop, event)`) | :white_check_mark: |
| Time-varying covariates (piecewise windows) | :white_check_mark: |
| Weighted events across all distributions | :white_check_mark: |
| Automatic stepwise covariate selection (forward, backward, stepwise; Wald or AIC) | :white_check_mark: |
| Conservation of Events theorem for numerically stable parameter estimation | :white_check_mark: |
| Covariance and correlation matrix estimation | :white_check_mark: |
| Delta-method confidence limits on `predict()` (`se.fit = TRUE`) | :white_check_mark: |
| Seven `hzr_*` utility functions (Kaplan-Meier, Nelson, GOF, deciles, calibration, bootstrap, competing risks) | :white_check_mark: |
| Per-phase confidence limits (`decompose = TRUE, se.fit = TRUE`) | :construction: |
| 4-phase example dataset and vignette | :construction: |

:white_check_mark: = implemented &nbsp;&nbsp; :construction: = planned

## Installation

```r
# Install from CRAN
install.packages("TemporalHazard")
```

```r
# Or install the development version from GitHub
remotes::install_github("ehrlinger/temporal_hazard")
```

TemporalHazard requires R >= 4.1.0 and depends on the
[survival](https://CRAN.R-project.org/package=survival) package.
Optional packages for visualization and vignettes include ggplot2, numDeriv,
and quarto.

## Quick start

### Single-phase model

```r
library(TemporalHazard)
data(cabgkul)

# Intercept-only Weibull on 5,880 CABG patients
fit <- hazard(
  survival::Surv(int_dead, dead) ~ 1,
  data  = cabgkul,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0),
  fit   = TRUE
)
summary(fit)
```

### Multiphase model

```r
# Three-phase additive hazard decomposition
fit_mp <- hazard(
  survival::Surv(int_dead, dead) ~ 1,
  data   = cabgkul,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",  tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  ),
  fit = TRUE
)
summary(fit_mp)

# Per-phase decomposition of cumulative hazard
t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.9, length.out = 200)
predict(fit_mp, newdata = data.frame(time = t_grid),
        type = "cumulative_hazard", decompose = TRUE)
```

Each phase is specified with `hzr_phase()`, which sets the temporal shape
type and starting values. The optimizer estimates both the phase-specific
scale parameters and shape parameters jointly. Covariates are supported via
the formula interface (see `vignette("fitting-hazard-models")`).

## Documentation

- **[Clinical Analysis Walkthrough](https://ehrlinger.github.io/temporal_hazard/articles/clinical-analysis-walkthrough.html)** --- complete end-to-end workflow from Kaplan-Meier baseline through validated multivariable model.
- **[Getting Started](https://ehrlinger.github.io/temporal_hazard/articles/getting-started.html)** --- first fit-predict workflow with visualizations.
- **[Fitting Hazard Models](https://ehrlinger.github.io/temporal_hazard/articles/fitting-hazard-models.html)** --- intercept-only through multiphase and multi-endpoint models.
- **[Prediction & Visualization](https://ehrlinger.github.io/temporal_hazard/articles/prediction-visualization.html)** --- survival curves, decomposed hazard, patient-specific risk profiles.
- **[Inference & Diagnostics](https://ehrlinger.github.io/temporal_hazard/articles/inference-diagnostics.html)** --- bootstrap CIs, decile-of-risk validation, sensitivity analysis.
- **[Mathematical Foundations](https://ehrlinger.github.io/temporal_hazard/articles/mf-mathematical-foundations.html)** --- the generalized decomposition, additive hazard model, censoring likelihood, and time-varying covariates.
- **[Package Architecture](https://ehrlinger.github.io/temporal_hazard/articles/ar-architecture.html)** --- internal design, golden fixtures, and dataset catalog.
- **[SAS-to-R Migration](https://ehrlinger.github.io/temporal_hazard/articles/sas-to-r-migration.html)** --- statement-by-statement mapping from SAS HAZARD syntax.

## Development

```r
install.packages(c("devtools", "roxygen2", "pkgdown", "testthat"))
devtools::install_deps(dependencies = TRUE)
devtools::test()
devtools::check()
```

GitHub Actions runs multi-platform `R CMD check` on every push and pull request. Coverage is published to Codecov and the pkgdown site deploys automatically from `main`.

See the development plan in `inst/dev/DEVELOPMENT-PLAN.md` for the full roadmap covering the C/SAS migration, multiphase implementation, CRAN release, and planned feature parity work.
See `.github/BRANCH_PROTECTION.md` for recommended required-check settings that block merges when CI fails.

# SAS Stepwise Parity Fixtures

This directory holds capture instructions and templates for
SAS-reference stepwise-selection runs.  The captured output lives as
`.rds` files under `inst/fixtures/` and is consumed by
`tests/testthat/test-stepwise-parity.R`.

Parity tests skip gracefully when a fixture is missing, so the package
still installs and passes CI without these files — they are only
required for claiming SAS-exact behaviour.

## What we capture

For each scenario (e.g. CABGKUL forward-Wald, AVC multiphase backward),
we want:

1. **The step-by-step trace** — which variable entered / dropped at
   each step, its Wald χ² statistic, degrees of freedom, p-value, and
   (for multiphase) which phase the move applied to.
2. **The final coefficient table** — estimates, standard errors,
   z-values, p-values at termination.
3. **Goodness-of-fit totals** — log-likelihood, AIC, iteration count.
4. **Meta** — SAS version, threshold settings, date captured, dataset,
   criterion, the exact `PROC HAZARD` invocation used.

Tolerances in the R-side comparison:

| Quantity                 | Tolerance           |
|--------------------------|---------------------|
| Sequence of (var, phase) | Exact               |
| Each step action         | Exact               |
| Wald χ² statistic        | 1e-3 (relative)     |
| p-value                  | 1e-3 (absolute)     |
| Final log-likelihood     | 1e-3 (absolute)     |
| Final coefficient        | 1e-2 (relative)     |

## Workflow

1. Run the SAS template in `cabgkul-forward-wald.sas` (adjust the
   `%LET` macros for other scenarios).
2. The template writes three text files into the SAS work library:
   - `stepwise_trace.csv` — one row per step
   - `stepwise_final.csv` — final coefficient table
   - `stepwise_meta.txt`  — key=value settings + GOF totals
3. In R, convert those three files into an `.rds` fixture with
   `TemporalHazard:::.hzr_build_stepwise_fixture()`, which
   validates the schema and writes to
   `inst/fixtures/stepwise-<scenario>.rds`.
4. Re-run `devtools::test()`; the parity test will pick up the
   fixture and assert each expectation.

See `schema.md` for the exact shape of the fixture list.

# SAS Fractional-Weight Parity Fixture

This directory holds the capture payload for a SAS-reference run that
confirms `PROC HAZARD`'s `WEIGHT` statement matches R's
`hazard(weights = ...)` for **non-integer** (inverse-probability-style)
weights. It closes roadmap item **7a** / FIXTURE-GAP-LIST **B5**:

> Uniform non-integer weights (fractional sampling weights) — verify the
> SAS `WEIGHT` statement matches R behaviour for non-integer weights.

R-side correctness is already proven *without* SAS by the additive-split
and linear-scaling invariants in `tests/testthat/test-weights.R`. This
fixture is the external SAS confirmation. The parity test
(`tests/testthat/test-weights-sas-parity.R`) **skips gracefully when the
fixture is absent**, so the package installs and CI passes without any SAS
output — the fixture is only required to claim SAS-exact weighted behaviour.

## The model

A fixed (non-stepwise) two-phase fit on the AVC data:

- **Early** phase: Weibull CDF shape, `THALF`/`NU`/`M` fixed at the AVC
  unconditional values, covariate `AGE`.
- **Constant** phase: flat exponential hazard, covariate `AGE`.
- **Weight**: the fractional column `IPW` (`WEIGHT IPW;` in SAS,
  `weights = ipw` in R).
- Conservation of events on (`CONDITION=14`).

## The weight column

`payload/avc-weighted.csv` is the AVC analysis data with a deterministic,
non-integer `ipw` column appended. `ipw` is a closed-form function of
`age` only (logistic, centred at 60, scale 10 → range `(0.5, 1.5)`), with
**no RNG and no cohort-derived statistics**, so SAS and R read identical
values. Regenerate it with `make-avc-weighted-csv.R`. The exact formula is
illustrative; only the *shared, fractional* values matter for parity.

## Tolerances (R-side comparison)

| Quantity                       | Tolerance        |
|--------------------------------|------------------|
| Covariate coefficient (per phase) | 0.02 (relative) |
| Final log-likelihood           | 0.5 (absolute)   |

## Workflow

1. On a SAS box with the CCF HAZARD install, from `payload/`:
   ```sh
   ./run.sh           # auto-discovers HAZAPPS / HZ_MACROS; or set them
   ```
   This writes `payload/capture/avc-weighted.lst` and
   `payload/capture/weighted_meta.txt`.
2. Back in R, from the project root:
   ```r
   source("inst/extdata/weights-fixtures/parse-weighted-lst.R")
   ```
   This parses the listing's coefficient table + log-likelihood and calls
   `TemporalHazard:::.hzr_build_weighted_fixture()`, writing
   `inst/fixtures/weighted-avc-fractional.rds`.
3. Re-run `devtools::test(filter = "weights-sas-parity")`; the parity test
   picks up the fixture and asserts the covariate estimates and
   log-likelihood agree with SAS.

## Files

| File | Purpose |
|------|---------|
| `make-avc-weighted-csv.R` | Regenerates `payload/avc-weighted.csv` deterministically |
| `payload/avc-weighted.csv` | AVC analysis data + fractional `ipw` column |
| `payload/avc-weighted.sas` | `PROC HAZARD` weighted two-phase fit template |
| `payload/run.sh` | Discovers the HAZARD install and runs SAS |
| `payload/capture/` | Where the `.lst` + parsed outputs land |
| `parse-weighted-lst.R` | Parses the `.lst` and builds the `.rds` fixture |

The fixture list shape is defined by `.hzr_weighted_fixture_schema()` in
`R/weights-fixture.R`.

## -----------------------------------------------------------------------------
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)


## -----------------------------------------------------------------------------
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)


## -----------------------------------------------------------------------------
#| label: km-baseline
data(avc)
avc <- na.omit(avc)
km <- survfit(Surv(int_dead, dead) ~ 1, data = avc)


## -----------------------------------------------------------------------------
#| label: fig-km
#| fig-cap: "Kaplan-Meier survival estimate: death after AVC repair"
#| fig-width: 7
#| fig-height: 4
km_df <- data.frame(time = km$time, survival = km$surv * 100)

ggplot(km_df, aes(time, survival)) +
  geom_step(linewidth = 0.6) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after repair", y = "Freedom from death (%)") +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: fit-mv
fit <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 4)),
  fit   = TRUE,
  control = list(maxit = 500)
)


## -----------------------------------------------------------------------------
#| label: pred-types
t_grid <- seq(0.01, max(avc$int_dead) * 0.95, length.out = 200)
profile <- data.frame(
  time   = t_grid,
  age    = median(avc$age),
  status = 2,
  mal    = 0,
  com_iv = 0
)

surv   <- predict(fit, newdata = profile, type = "survival")
cumhaz <- predict(fit, newdata = profile, type = "cumulative_hazard")

profile$survival          <- surv
profile$cumulative_hazard <- cumhaz

head(profile[, c("time", "survival", "cumulative_hazard")])


## -----------------------------------------------------------------------------
#| label: fig-surv-overlay
#| fig-cap: "Weibull parametric survival vs. Kaplan-Meier (AVC death)"
#| fig-width: 7
#| fig-height: 4.5
ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_line(data = profile,
            aes(time, survival * 100, colour = "Parametric (Weibull)"),
            linewidth = 1) +
  scale_colour_manual(
    values = c("Parametric (Weibull)" = "#0072B2",
               "Kaplan-Meier"         = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: pred-with-se
# Build a clean newdata frame (the earlier chunk appended result
# columns to `profile`, which would confuse predict()'s column count).
profile_ci <- data.frame(
  time   = t_grid,
  age    = median(avc$age),
  status = 2,
  mal    = 0,
  com_iv = 0
)
surv_ci <- predict(fit, newdata = profile_ci,
                   type = "survival", se.fit = TRUE, level = 0.95)
head(surv_ci)


## -----------------------------------------------------------------------------
#| label: fig-surv-ci
#| fig-cap: "Parametric survival with 95% delta-method confidence band"
#| fig-width: 7
#| fig-height: 4.5
ci_df <- data.frame(
  time     = profile_ci$time,
  survival = surv_ci$fit * 100,
  lower    = surv_ci$lower * 100,
  upper    = surv_ci$upper * 100
)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_ribbon(data = ci_df,
              aes(time, ymin = lower, ymax = upper),
              fill = "#0072B2", alpha = 0.15) +
  geom_line(data = ci_df,
            aes(time, survival, colour = "Parametric + 95% CI"),
            linewidth = 1) +
  scale_colour_manual(
    values = c("Parametric + 95% CI" = "#0072B2",
               "Kaplan-Meier"        = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: fit-mp
data(cabgkul)

fit_mp <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = cabgkul,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",  tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)


## -----------------------------------------------------------------------------
#| label: fig-decomposed-hazard
#| fig-cap: "Additive phase decomposition: total hazard rate (solid) = early + constant + late (dashed)"
#| fig-width: 7
#| fig-height: 4.5
t_mp <- seq(0.01, max(cabgkul$int_dead) * 0.95, length.out = 200)
nd   <- data.frame(time = t_mp)

decomp <- predict(fit_mp, newdata = nd, type = "cumulative_hazard",
                  decompose = TRUE)

# Numerical differentiation: h(t) ≈ ΔH(t) / Δt
num_hazard <- function(cumhaz, time) {
  dt <- diff(time)
  dH <- diff(cumhaz)
  c(dH[1] / dt[1], dH / dt)
}

h_long <- rbind(
  data.frame(time = t_mp, hazard = num_hazard(decomp$early, t_mp),
             Phase = "Early"),
  data.frame(time = t_mp, hazard = num_hazard(decomp$constant, t_mp),
             Phase = "Constant"),
  data.frame(time = t_mp, hazard = num_hazard(decomp$late, t_mp),
             Phase = "Late"),
  data.frame(time = t_mp, hazard = num_hazard(decomp$total, t_mp),
             Phase = "Total")
)
h_long$Phase <- factor(h_long$Phase,
                       levels = c("Total", "Early", "Constant", "Late"))

ggplot(h_long, aes(time, hazard, colour = Phase, linetype = Phase)) +
  geom_line(aes(linewidth = Phase)) +
  scale_colour_manual(values = c(Total = "#222222", Early = "#E69F00",
                                 Constant = "#56B4E9", Late = "#CC79A7")) +
  scale_linetype_manual(values = c(Total = "solid", Early = "dashed",
                                   Constant = "dashed", Late = "dashed")) +
  scale_linewidth_manual(values = c(Total = 1.3, Early = 0.7,
                                    Constant = 0.7, Late = 0.7)) +
  labs(x = "Months after CABG", y = "Hazard rate",
       colour = "Phase", linetype = "Phase", linewidth = "Phase") +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: fig-mp-surv
#| fig-cap: "Multiphase parametric survival vs. Kaplan-Meier"
#| fig-width: 7
#| fig-height: 4.5
surv_mp <- predict(fit_mp, newdata = nd, type = "survival") * 100

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_line(data = data.frame(time = t_grid, survival = surv_mp),
            aes(time, survival, colour = "Multiphase (3-phase)"),
            linewidth = 1) +
  scale_colour_manual(
    values = c("Multiphase (3-phase)" = "#0072B2",
               "Kaplan-Meier"         = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: fig-risk-profiles
#| fig-cap: "Predicted survival by risk profile"
#| fig-width: 7
#| fig-height: 4.5
profiles <- list(
  "Low risk"  = data.frame(age = quantile(avc$age, 0.25),
                            status = 1, mal = 0, com_iv = 0),
  "Median"    = data.frame(age = median(avc$age),
                            status = 2, mal = 0, com_iv = 0),
  "High risk" = data.frame(age = quantile(avc$age, 0.90),
                            status = 4, mal = 1, com_iv = 1)
)

curves <- do.call(rbind, lapply(names(profiles), function(nm) {
  nd <- profiles[[nm]][rep(1, length(t_grid)), ]
  nd$time <- t_grid
  data.frame(time = t_grid,
             survival = predict(fit, newdata = nd, type = "survival") * 100,
             Profile = nm)
}))
curves$Profile <- factor(curves$Profile,
                         levels = c("Low risk", "Median", "High risk"))

ggplot(curves, aes(time, survival, colour = Profile)) +
  geom_line(linewidth = 0.9) +
  scale_colour_manual(values = c("Low risk" = "#009E73",
                                 "Median"   = "#0072B2",
                                 "High risk" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: fig-valves-endpoints
#| fig-cap: "Freedom from death and PVE after valve replacement"
#| fig-width: 7
#| fig-height: 4.5
data(valves)
valves <- na.omit(valves)

km_death <- survfit(Surv(int_dead, dead) ~ 1, data = valves)
km_pve   <- survfit(Surv(int_pve, pve) ~ 1, data = valves)

ep_df <- rbind(
  data.frame(time = km_death$time, survival = km_death$surv * 100,
             Endpoint = "Death"),
  data.frame(time = km_pve$time, survival = km_pve$surv * 100,
             Endpoint = "PVE")
)

ggplot(ep_df, aes(time, survival, colour = Endpoint)) +
  geom_step(linewidth = 0.7) +
  scale_y_continuous(limits = c(0, 100)) +
  scale_colour_manual(values = c("Death" = "#D55E00", "PVE" = "#0072B2")) +
  labs(x = "Months after valve replacement",
       y = "Freedom from event (%)", colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")

## -----------------------------------------------------------------------------
#| include: false
has_pkg    <- requireNamespace("TemporalHazard", quietly = TRUE)
has_ggplot2 <- requireNamespace("ggplot2",       quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = has_pkg
)


## -----------------------------------------------------------------------------
library(TemporalHazard)
library(survival)

set.seed(1001)
n <- 180
dat <- data.frame(
  time = rexp(n, rate = 0.35) + 0.05,
  status = rbinom(n, size = 1, prob = 0.6),
  age = rnorm(n, mean = 62, sd = 11),
  nyha = sample(1:4, n, replace = TRUE),
  shock = rbinom(n, size = 1, prob = 0.18)
)

fit <- TemporalHazard::hazard(
  Surv(time, status) ~ age + nyha + shock,
  data = dat,
  theta = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
  dist = "weibull",
  fit = TRUE,
  control = list(maxit = 300)
)

summary(fit)


## -----------------------------------------------------------------------------
new_patients <- data.frame(
  time = c(0.5, 1.5, 3.0),
  age = c(50, 65, 75),
  nyha = c(1, 3, 4),
  shock = c(0, 0, 1)
)

pred_input <- new_patients

new_patients$linear_predictor <- predict(fit, newdata = pred_input, type = "linear_predictor")
new_patients$hazard_multiplier <- predict(fit, newdata = pred_input, type = "hazard")
new_patients$survival <- predict(fit, newdata = pred_input, type = "survival")
new_patients$cumulative_hazard <- predict(fit, newdata = pred_input, type = "cumulative_hazard")

new_patients


## -----------------------------------------------------------------------------
#| label: fig-survival
#| fig-cap: "Parametric Weibull survival curve with Kaplan-Meier overlay"
#| fig-width: 7
#| fig-height: 5
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
library(ggplot2)

# Parametric curve on a fine grid
t_grid <- seq(0.05, max(dat$time), length.out = 80)
curve_df <- data.frame(
  time  = t_grid,
  age   = median(dat$age),
  nyha  = 2,
  shock = 0
)
curve_df$survival <- predict(fit, newdata = curve_df, type = "survival") * 100

# Kaplan-Meier empirical overlay
km <- survival::survfit(survival::Surv(time, status) ~ 1, data = dat)
km_df <- data.frame(time = km$time, survival = km$surv * 100)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier")) +
  geom_line(data = curve_df, aes(time, survival,
                                  colour = "Parametric (Weibull)")) +
  scale_colour_manual(
    values = c("Parametric (Weibull)" = "#0072B2",
               "Kaplan-Meier"         = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after surgery", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: multiphase-fit
# CABGKUL is the benchmark dataset for 3-phase decomposition (n = 5,880)
data(cabgkul)

fit_mp <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = cabgkul,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",  tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)

summary(fit_mp)


## -----------------------------------------------------------------------------
#| label: fig-hazard-phases
#| fig-cap: "Additive phase decomposition: total hazard (solid) = early + constant + late (dashed)"
#| fig-width: 7
#| fig-height: 4.5
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.95, length.out = 200)
nd     <- data.frame(time = t_grid)

# decompose = TRUE returns per-phase cumulative hazard columns
decomp <- predict(fit_mp, newdata = nd, type = "cumulative_hazard",
                  decompose = TRUE)

# Numerical differentiation: h(t) ≈ ΔH(t) / Δt
num_hazard <- function(cumhaz, time) {
  dt <- diff(time)
  dH <- diff(cumhaz)
  c(dH[1] / dt[1], dH / dt)
}

h_long <- rbind(
  data.frame(time = t_grid, hazard = num_hazard(decomp$early, t_grid),
             Phase = "Early"),
  data.frame(time = t_grid, hazard = num_hazard(decomp$constant, t_grid),
             Phase = "Constant"),
  data.frame(time = t_grid, hazard = num_hazard(decomp$late, t_grid),
             Phase = "Late"),
  data.frame(time = t_grid, hazard = num_hazard(decomp$total, t_grid),
             Phase = "Total")
)
h_long$Phase <- factor(h_long$Phase,
                       levels = c("Total", "Early", "Constant", "Late"))

ggplot(h_long, aes(time, hazard, colour = Phase, linetype = Phase)) +
  geom_line(aes(linewidth = Phase)) +
  scale_colour_manual(values = c(Total = "#222222", Early = "#E69F00",
                                 Constant = "#56B4E9", Late = "#CC79A7")) +
  scale_linetype_manual(values = c(Total = "solid", Early = "dashed",
                                   Constant = "dashed", Late = "dashed")) +
  scale_linewidth_manual(values = c(Total = 1.3, Early = 0.7,
                                    Constant = 0.7, Late = 0.7)) +
  labs(x = "Months after CABG", y = "Hazard rate",
       colour = "Phase", linetype = "Phase", linewidth = "Phase") +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: fig-mp-survival
#| fig-cap: "Multiphase parametric survival vs. Kaplan-Meier"
#| fig-width: 7
#| fig-height: 4.5
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
surv_df <- data.frame(
  time     = t_grid,
  survival = predict(fit_mp, newdata = nd, type = "survival") * 100
)

km    <- survfit(Surv(int_dead, dead) ~ 1, data = cabgkul)
km_df <- data.frame(time = km$time, survival = km$surv * 100)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.6) +
  geom_line(data = surv_df, aes(time, survival, colour = "Multiphase model"),
            linewidth = 1.1) +
  scale_colour_manual(
    values = c("Multiphase model" = "#0072B2", "Kaplan-Meier" = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after CABG", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| eval: false
# hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 1)   # Early risk (bounded)
# hzr_phase("constant")                                   # Flat background rate
# hzr_phase("cdf",      t_half = 10,  nu = 1, m = 0)    # Late risk (CDF-based)
# hzr_phase("g3",       tau = 1, gamma = 3, alpha = 1,   # Late risk (G3 power law)
#                        eta = 1)


## -----------------------------------------------------------------------------
TemporalHazard::hzr_log1pexp(c(-2, 0, 2))

## -----------------------------------------------------------------------------
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)


## -----------------------------------------------------------------------------
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)


## -----------------------------------------------------------------------------
#| label: explore-data
data(avc)
avc <- na.omit(avc)

# Candidate covariates
candidates <- c("age", "status", "mal", "com_iv", "inc_surg", "orifice")


## -----------------------------------------------------------------------------
#| label: explore-logit
logit_fits <- lapply(candidates, function(var) {
  fmla <- as.formula(paste("dead ~", var))
  glm(fmla, data = avc, family = binomial())
})
names(logit_fits) <- candidates

# Which covariates are significant univariably?
p_values <- vapply(logit_fits, function(f) {
  coef(summary(f))[2, "Pr(>|z|)"]
}, numeric(1))

data.frame(covariate = names(p_values),
           p_value   = round(p_values, 4),
           row.names = NULL)


## -----------------------------------------------------------------------------
#| label: fig-explore-age
#| fig-cap: "Exploratory: age vs. mortality with LOESS smooth"
#| fig-width: 7
#| fig-height: 4
ggplot(avc, aes(age, dead)) +
  geom_jitter(height = 0.05, width = 0, alpha = 0.3, size = 1) +
  geom_smooth(method = "loess", se = TRUE, colour = "#0072B2",
              linewidth = 0.8) +
  labs(x = "Age at operation (months)", y = "Death (0/1)") +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: calibrate-age
age_cal <- hzr_calibrate(
  x      = avc$age,
  event  = avc$dead,
  groups = 10,
  link   = "logit"
)
print(age_cal)


## -----------------------------------------------------------------------------
#| label: km-baseline
km <- hzr_kaplan(time = avc$int_dead, status = avc$dead, conf_level = 0.95)
print(km)


## -----------------------------------------------------------------------------
#| label: nelson-baseline
nel <- hzr_nelson(time = avc$int_dead, event = avc$dead, conf_level = 0.95)
print(nel)


## -----------------------------------------------------------------------------
#| label: fit-base
# Fit the base model that bootstrapping will use
fit <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 4)),
  fit   = TRUE,
  control = list(maxit = 500)
)

# Prediction grid at a median-risk profile
t_grid <- seq(0.01, max(avc$int_dead) * 0.9, length.out = 100)
base_nd <- data.frame(time = t_grid, age = median(avc$age),
                      status = 2, mal = 0, com_iv = 0)

surv_point <- predict(fit, newdata = base_nd, type = "survival")


## -----------------------------------------------------------------------------
#| label: bootstrap
set.seed(42)
boot <- hzr_bootstrap(fit, n_boot = 30)  # kept small for vignette build time
print(boot)


## -----------------------------------------------------------------------------
#| label: gof-summary
gof <- hzr_gof(fit)
print(gof)


## -----------------------------------------------------------------------------
#| label: deciles
deciles <- hzr_deciles(fit, time = 120, groups = 10)
print(deciles)


## -----------------------------------------------------------------------------
#| label: fig-deciles
#| fig-cap: "Observed vs. expected mortality by decile of risk"
#| fig-width: 7
#| fig-height: 4.5
decile_df <- as.data.frame(deciles)
cal_long <- rbind(
  data.frame(group = decile_df$group,
             rate  = decile_df$observed_rate,
             Series = "Observed"),
  data.frame(group = decile_df$group,
             rate  = decile_df$expected_rate,
             Series = "Expected")
)
ggplot(cal_long, aes(group, rate, fill = Series)) +
  geom_col(position = position_dodge(width = 0.7), alpha = 0.8) +
  scale_fill_manual(values = c("Observed" = "#56B4E9",
                                "Expected" = "#E69F00")) +
  scale_x_continuous(breaks = seq_len(nrow(decile_df))) +
  scale_y_continuous(labels = scales::percent_format(accuracy = 1)) +
  labs(x = "Risk decile (1 = lowest)", y = "Mortality rate", fill = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: sensitivity
# Define reference and high-risk profiles
ref_profile <- data.frame(
  age    = median(avc$age),
  status = 2,
  mal    = 0,
  com_iv = 0
)

high_risk <- data.frame(
  age    = quantile(avc$age, 0.90),
  status = 4,
  mal    = 1,
  com_iv = 1
)


## -----------------------------------------------------------------------------
#| label: fig-sensitivity
#| fig-cap: "Sensitivity analysis: reference vs. high-risk profile"
#| fig-width: 7
#| fig-height: 4.5
sens_curves <- do.call(rbind, lapply(
  list("Reference" = ref_profile, "High risk" = high_risk),
  function(p) {
    nd <- p[rep(1, length(t_grid)), ]
    nd$time <- t_grid
    data.frame(time = t_grid,
               survival = predict(fit, newdata = nd, type = "survival") * 100,
               Profile = deparse(substitute(p)))
  }
))
# Fix profile labels
sens_curves$Profile <- rep(c("Reference", "High risk"),
                           each = length(t_grid))
sens_curves$Profile <- factor(sens_curves$Profile,
                              levels = c("Reference", "High risk"))

ggplot(sens_curves, aes(time, survival, colour = Profile)) +
  geom_line(linewidth = 0.9) +
  scale_colour_manual(values = c("Reference" = "#009E73",
                                 "High risk" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")

# parse-weighted-lst.R -- Parse a HAZARD .lst listing from the weighted fit
# into the files consumed by .hzr_build_weighted_fixture().
#
# Usage (from the project root, after the SAS run via payload/run.sh):
#
#   source("inst/extdata/weights-fixtures/parse-weighted-lst.R")
#
# Inputs:
#   inst/extdata/weights-fixtures/payload/capture/avc-weighted.lst
#   inst/extdata/weights-fixtures/payload/capture/weighted_meta.txt
#
# Outputs (written in-place to payload/capture/):
#   weighted_coef.csv  -- final coefficient table
#   weighted_meta.txt  -- updated with logLik= entry
#
# Then builds:
#   inst/fixtures/weighted-avc-fractional.rds
#
# The coefficient-table and log-likelihood parsing mirror
# stepwise-fixtures/parse-hazard-lst.R (same PROC HAZARD listing format).

lst_file    <- "inst/extdata/weights-fixtures/payload/capture/avc-weighted.lst"
meta_file   <- "inst/extdata/weights-fixtures/payload/capture/weighted_meta.txt"
out_coef    <- "inst/extdata/weights-fixtures/payload/capture/weighted_coef.csv"
fixture_out <- "inst/fixtures/weighted-avc-fractional.rds"

stopifnot(file.exists(lst_file), file.exists(meta_file))

lines <- readLines(lst_file, warn = FALSE)

# ---- Final coefficient table ------------------------------------------------
# Lives in the "Parameter Estimate Summary" block.  Fixed-width columns:
#
#   Phase      Parameter    Estimate       Std error             Z    Prob>|Z|
#   ----------------------...
#   Early:     E0             -2.20788      0.4338312         -5.089    <.0001
#              AGE          -0.00544498    0.002572203         -2.117    0.0343
#              ------...
#   Constant:  C0             -10.6589       1.726197         -6.175    <.0001
#              AGE           ...
#   ------...

param_summary_idx <- grep("Parameter Estimate Summary", lines, fixed = TRUE)[1]
stopifnot(!is.na(param_summary_idx))

col_header_idx <- grep(
  "Phase\\s+Parameter\\s+Estimate\\s+Std error",
  lines[param_summary_idx:length(lines)]
)[1] + param_summary_idx - 1L

data_start <- col_header_idx + 2L   # +1 dashes, +1 first data line

stop_idx <- grep("Estimates for Model Parameters", lines, fixed = TRUE)
stop_idx <- stop_idx[stop_idx > data_start][1L]

final_rows    <- list()
current_phase <- NA_character_

for (i in seq(data_start, stop_idx - 1L)) {
  line    <- lines[i]
  trimmed <- trimws(line)

  if (nchar(trimmed) == 0L) next
  if (grepl("^-{5,}", trimmed))  next

  phase_lbl <- regmatches(
    line, regexpr("(Early|Constant):", line, perl = TRUE, ignore.case = TRUE)
  )
  if (length(phase_lbl) > 0L && nchar(phase_lbl) > 0L) {
    current_phase <- tolower(sub(":", "", phase_lbl))
    line <- sub(".*(?:Early|Constant):\\s+", "", line, ignore.case = TRUE,
                perl = TRUE)
  }

  m <- regexpr(
    paste0("^\\s*(\\S+)\\s+([+-]?[0-9.Ee+-]+)\\s+([0-9.Ee+-]+)\\s+",
           "([+-]?[0-9.Ee+-]+)\\s+([<]?[.0-9]+)"),
    line, perl = TRUE
  )
  if (m < 0L) next

  tok <- strsplit(trimws(regmatches(line, m)), "\\s+")[[1]]
  if (length(tok) < 5L) next

  p_raw <- tok[5L]
  p_val <- if (startsWith(p_raw, "<")) {
    as.numeric(sub("^<\\.?", "0.", p_raw))
  } else {
    as.numeric(p_raw)
  }

  final_rows[[length(final_rows) + 1L]] <- data.frame(
    variable  = tok[1L],
    phase     = current_phase,
    estimate  = as.numeric(tok[2L]),
    std_error = as.numeric(tok[3L]),
    z_stat    = as.numeric(tok[4L]),
    p_value   = p_val,
    stringsAsFactors = FALSE
  )
}

final_df <- do.call(rbind, final_rows)
final_df$variable <- tolower(final_df$variable)
utils::write.csv(final_df, out_coef, row.names = FALSE)
message("Wrote ", out_coef, " (", nrow(final_df), " parameters)")

# ---- Augment meta.txt with logLik -------------------------------------------
ll_val <- as.numeric(trimws(sub(
  ".*Log likelihood\\s*=\\s*", "",
  tail(grep("Log likelihood\\s*=", lines, value = TRUE), 1L)
)))
message("Final log-likelihood: ", ll_val)

meta_lines <- readLines(meta_file)
meta_lines <- meta_lines[!grepl("^logLik\\s*=", meta_lines)]
meta_lines <- c(meta_lines, paste0("logLik=", ll_val))
writeLines(meta_lines, meta_file)
message("Updated ", meta_file)

# ---- Build the .rds fixture -------------------------------------------------
devtools::load_all(quiet = TRUE)

fix <- TemporalHazard:::.hzr_build_weighted_fixture(
  coef_csv = out_coef,
  meta_txt = meta_file,
  out_path = fixture_out
)
message("Built fixture: ", fixture_out)
message("\nFinal coefficients:")
print(fix$final$coef)
message("\nlogLik: ", fix$final$logLik)

## ----setup, include=FALSE-----------------------------------------------------
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE)
if (has_pkg) library(TemporalHazard)
knitr::opts_chunk$set(echo = TRUE, collapse = TRUE, comment = "#>",
                      eval = has_pkg)


## ----file-layout, echo=FALSE--------------------------------------------------
layout <- data.frame(
  Layer = c(
    rep("User API", 2),
    rep("Multiphase engine", 3),
    rep("Single-phase likelihoods", 4),
    rep("Shared infrastructure", 5)
  ),
  File = c(
    "hazard_api.R", "argument_mapping.R",
    "likelihood-multiphase.R", "phase-spec.R", "decomposition.R",
    "likelihood-weibull.R", "likelihood-exponential.R",
    "likelihood-loglogistic.R", "likelihood-lognormal.R",
    "optimizer.R", "math_primitives.R", "formula-helpers.R",
    "golden_fixtures.R", "parity-helpers.R"
  ),
  Purpose = c(
    "hazard(), predict.hazard(), print/summary/coef/vcov S3 methods",
    "SAS HAZARD -> R parameter translation table",
    "Additive N-phase likelihood, cumhaz/hazard evaluators, multi-start optimizer",
    "hzr_phase() constructor, validators, starting-value assembly",
    "Unified decompos(t; t_half, nu, m) parametric family",
    "Weibull PH likelihood, gradient, optimizer",
    "Exponential (constant hazard) likelihood",
    "Log-logistic (proportional odds) likelihood",
    "Log-normal (AFT) likelihood",
    "Generic L-BFGS-B/BFGS optimizer with Hessian-based vcov",
    "Numerically stable log1pexp, log1mexp, clamp_prob",
    "Surv() formula parsing for right/left/interval censoring",
    "Synthetic fixture generators (.rds reference outputs)",
    "Stubs for cross-validating against C HAZARD binary"
  )
)
knitr::kable(layout, caption = "R source files by architectural layer")


## ----theta-layout, echo=FALSE-------------------------------------------------
theta_layout <- data.frame(
  `Phase type` = c("cdf / hazard", "g3", "constant"),
  `Sub-vector` = c(
    "[log_mu, log_t_half, nu, m, beta_1, ..., beta_p]",
    "[log_mu, log_tau, gamma, alpha, eta, beta_1, ..., beta_p]",
    "[log_mu, beta_1, ..., beta_p]"
  ),
  `Count` = c("4 + p", "5 + p", "1 + p")
)
knitr::kable(theta_layout, caption = "Internal theta layout per phase")


## ----fixtures, echo=FALSE-----------------------------------------------------
fixtures <- data.frame(
  File = c(
    "hz_univariate.rds",
    "hm_multivariate.rds",
    "hm_edge_case.rds",
    "hz_loglogistic.rds",
    "hz_lognormal.rds",
    "mp_synthetic_3phase.rds",
    "mp_c_reference_kul.rds"
  ),
  Distribution = c(
    "Weibull", "Weibull", "Weibull",
    "Log-logistic", "Log-normal",
    "Multiphase (3)", "Multiphase (3)"
  ),
  Description = c(
    "Univariable shape estimation (n=100)",
    "2 covariates (n=100)",
    "Edge case: n=20, 3 covariates",
    "Univariable (n=80)",
    "Univariable (n=80)",
    "Synthetic early CDF + constant + late hazard (n=500)",
    "KUL CABG dataset + C HAZARD binary reference output (n=5880)"
  ),
  Source = c(
    rep("Synthetic (seed=42)", 5),
    "Synthetic (seed=42)",
    "Clinical + C binary"
  )
)
knitr::kable(fixtures, caption = "Golden fixture inventory")


## ----test-tiers, echo=FALSE---------------------------------------------------
tiers <- data.frame(
  Tier = c(
    "Unit tests",
    "Distribution tests",
    "Integration tests",
    "Parity tests"
  ),
  Files = c(
    "test-math-primitives, test-decomposition, test-phase-spec, test-argument-mapping",
    paste("test-gradient-weibull, test-exponential-dist,",
          "test-loglogistic-dist, test-lognormal-dist, test-multiphase-gradient"),
    paste("test-hazard-api, test-predict-types, test-interval-censoring-*,",
          "test-time-varying-*, test-multiphase-likelihood, test-multiphase-api"),
    "test-parity-core, test-parity-edge-cases, test-parity-c-binary, test-multiphase-parity"
  ),
  Purpose = c(
    "Verify individual functions in isolation",
    "Likelihood, gradient, and optimizer for each distribution",
    "End-to-end: hazard() -> predict() -> summary() pipeline, censoring types, multiphase wiring",
    "Golden fixture round-trip, C binary cross-validation"
  )
)
knitr::kable(tiers, caption = "Test suite tiers")


## ----datasets, echo=FALSE-----------------------------------------------------
datasets <- data.frame(
  File = c("avc.csv", "cabgkul.csv", "omc.csv", "tga.csv", "valves.csv"),
  Study = c(
    "Atrioventricular canal repair",
    "Coronary artery bypass grafting (KU Leuven)",
    "Open mitral commissurotomy",
    "Transposition of great arteries (arterial switch)",
    "Primary valve replacement"
  ),
  n = c(310, 5880, 339, 470, 1533),
  Events = c("70 deaths", "545 deaths", "thromboembolic events (repeated)", "deaths", "deaths, PVE, reoperation"),
  Covariates = c(
    "NYHA, age, anatomy, era",
    "None (intercept only)",
    "TE events (repeated measures)",
    "anatomy, coronary pattern, era",
    "age, NYHA, valve position, pathology, race"
  ),
  `SAS origin` = c(
    "hz.death.AVC, hm.death.AVC",
    "hz.deadp.KUL",
    "hz.te123.OMC",
    "hs.dthar.TGA",
    "hm.deadp.VALVES"
  )
)
knitr::kable(datasets, caption = "Reference datasets (lazy-loaded via `data()`)")


## ----load-demo----------------------------------------------------------------
# Datasets are lazy-loaded with the package — just reference them directly.
# Raw CSVs are also available in inst/extdata/ for advanced use:
#   read.csv(system.file("extdata", "cabgkul.csv", package = "TemporalHazard"))

data(cabgkul)
str(cabgkul)

data(avc)
str(avc)


## ----avc-detail, echo=FALSE---------------------------------------------------
avc_vars <- data.frame(
  Variable = c("study", "status", "inc_surg", "opmos", "age", "mal",
               "com_iv", "orifice", "dead", "int_dead", "op_age"),
  Label = c("Study number", "NYHA functional class (I-V)", "Surgical grade of AV valve incompetence",
            "Date of operation (months since Jan 1967)", "Age (months) at repair",
            "Important associated cardiac anomaly", "Interventricular communication",
            "Accessory left AV valve orifice", "Death (event indicator)",
            "Follow-up interval (months)", "Interaction: opmos x age"),
  Type = c("character", rep("numeric", 10))
)
knitr::kable(avc_vars, caption = "AVC dataset variables")


## ----mapping------------------------------------------------------------------
mapping <- hzr_argument_mapping()
knitr::kable(
  mapping[, c("legacy_input", "r_parameter", "implementation_status", "notes")],
  caption = "SAS HAZARD to R parameter mapping (excerpt)"
)

/* =======================================================================
 * cabgkul-forward-wald.sas — Reference capture for R-side stepwise parity
 *
 * Runs forward Wald stepwise selection on CABGKUL using SAS PROC HAZARD
 * with SLENTRY = 0.30 / SLSTAY = 0.20 (SAS defaults) and emits three
 * text files consumed by .hzr_build_stepwise_fixture() in R.
 *
 * Usage:
 *   1. Point LIBNAME to the directory containing cabgkul.sas7bdat.
 *   2. Adjust %LET out_dir if you want the capture written elsewhere.
 *   3. Run in SAS 9.4+.  The three output files can then be copied
 *      into the R session for fixture construction.
 * ===================================================================== */

%LET in_dir  = /path/to/cabgkul;
%LET out_dir = /path/to/capture;

LIBNAME hvti "&in_dir";

/* -- PROC HAZARD invocation --------------------------------------------- */
ODS LISTING CLOSE;
ODS OUTPUT
  SelectionSummary = _sel_trace     /* step-by-step history         */
  ParameterEstimates = _sel_final   /* final coefficient table      */
  ConvergenceStatus = _sel_conv     /* iterations, convergence code */
  FitStatistics = _sel_fit;         /* logLik, AIC, BIC             */

PROC HAZARD DATA = hvti.cabgkul
            METHOD = STEPWISE
            SLENTRY = 0.30
            SLSTAY  = 0.20;
  TIME INT_DEAD * DEAD(0);
  MODEL INT_DEAD = AGE MALE INC_SURG ORIFICE COM_IV
        / DIST = WEIBULL;
RUN;

ODS LISTING;

/* -- Write the three capture files -------------------------------------- */

/* 1. Step trace: one row per step. */
DATA _step_trace;
  SET _sel_trace;
  LENGTH action $ 8 variable $ 32 phase $ 8;
  step_num = _N_;
  /* Map SAS "Effect Entered" / "Effect Removed" columns → action */
  IF MISSING(EffectRemoved) THEN DO;
    action   = "enter";
    variable = EffectEntered;
  END;
  ELSE DO;
    action   = "drop";
    variable = EffectRemoved;
  END;
  phase    = ".";                      /* placeholder for single-dist */
  stat     = ChiSquare;
  df       = DF;
  p_value  = ProbChiSq;
  KEEP step_num action variable phase stat df p_value;
RUN;

PROC EXPORT DATA = _step_trace
            OUTFILE = "&out_dir/stepwise_trace.csv"
            DBMS = CSV REPLACE;
RUN;

/* 2. Final coefficient table. */
DATA _final_coef;
  SET _sel_final;
  LENGTH variable $ 32 phase $ 8;
  variable  = Parameter;
  phase     = ".";
  estimate  = Estimate;
  std_error = StdErr;
  z_stat    = Estimate / StdErr;
  p_value   = ProbChiSq;
  KEEP variable phase estimate std_error z_stat p_value;
RUN;

PROC EXPORT DATA = _final_coef
            OUTFILE = "&out_dir/stepwise_final.csv"
            DBMS = CSV REPLACE;
RUN;

/* 3. Meta / GOF totals — a tiny key=value text file. */
DATA _NULL_;
  FILE "&out_dir/stepwise_meta.txt";
  SET _sel_fit(WHERE = (UPCASE(Criterion) = "-2 LOG L" OR
                         UPCASE(Criterion) = "AIC"));
  IF UPCASE(Criterion) = "-2 LOG L" THEN DO;
    logLik_twice = Value;
    PUT "logLik=" (logLik_twice / -2) 12.6;
  END;
  ELSE IF UPCASE(Criterion) = "AIC" THEN DO;
    PUT "aic=" Value 12.6;
  END;
RUN;

DATA _NULL_;
  FILE "&out_dir/stepwise_meta.txt" MOD;
  SET _sel_conv;
  PUT "iterations=" _N_;
  PUT "converged=" Status;
  PUT "sas_version=9.4";
  PUT "dataset=cabgkul";
  PUT "dist=weibull";
  PUT "criterion=wald";
  PUT "direction=forward";
  PUT "slentry=0.30";
  PUT "slstay=0.20";
  PUT "captured_on=2026-04-17";
RUN;

/* =======================================================================
 * avc-forward-wald.sas  --  TemporalHazard stepwise parity capture
 *
 * Run via run.sh (which sets HAZAPPS + HZ_MACROS before calling SAS):
 *
 *   cd /path/to/this/folder
 *   ./run.sh
 *
 * Produces ./capture/avc-forward-wald.lst, which the R fixture builder
 * parses to create inst/fixtures/stepwise-avc-forward-wald.rds.
 *
 * Syntax based on hm.death.AVC.sas from the HAZARD example corpus.
 * Starting parameters from the unconditional hz.death.AVC fit.
 * Shaping parameters (THALF, NU, M) are fixed during stepwise for speed
 * and numerical stability (standard practice).
 * ===================================================================== */

/* -- Load HAZARD macros ------------------------------------------------- */
/* HZ_MACROS env var is set by run.sh to the dir containing hazard.sas     */
%let _macros = %sysget(HZ_MACROS);
%if %length(&_macros) = 0 %then %do;
  %put ERROR: HZ_MACROS env var not set -- run this script via run.sh.;
  %abort;
%end;
OPTIONS SASAUTOS=("&_macros" SASAUTOS);

/* Override TMPDIR so %HAZARD macro and C binary agree on the XPORT path.
   SAS Foundation overwrites the shell's $TMPDIR; OPTIONS SET= patches it
   back before %HAZARD writes the handoff file.                            */
%let _hz_tmp = %sysget(HZ_TMPDIR);
%if %length(&_hz_tmp) = 0 %then %do;
  %let _hz_tmp = /tmp;
%end;
OPTIONS SET=TMPDIR "&_hz_tmp";

/* SAS site config overrides HAZAPPS; override it back to the local build.  */
%let _hz_apps = %sysget(HAZAPPS);
%if %length(&_hz_apps) > 0 %then %do;
  OPTIONS SET=HAZAPPS "&_hz_apps";
%end;

/* -- Paths ---------------------------------------------------------------- */
%LET csv_path = ./avc.csv;
%LET out_dir  = ./capture;
%LET run_date = 2026-06-10;

/* -- Import avc.csv ------------------------------------------------------- */
PROC IMPORT DATAFILE = "&csv_path"
            OUT      = _avc
            DBMS     = CSV
            REPLACE;
  GETNAMES = YES;
RUN;

/* Exclude rows with any missing analysis variable (matches R na.omit) */
DATA _avc_clean;
  SET _avc;
  IF NMISS(INT_DEAD, DEAD, AGE, MAL, INC_SURG, ORIFICE, COM_IV) = 0;
RUN;

/* -- Forward Wald stepwise via PROC HAZARD -------------------------------- */
/*                                                                           */
/* Model: two-phase (early + constant) hazard with Weibull shaping.         */
/* Candidates: AGE, MAL, INC_SURG, ORIFICE, COM_IV in both phases.         */
/* SLE=0.30, SLS=0.20 match the R hzr_stepwise() call in the parity test.  */
/*                                                                           */
/* Output goes to the SAS listing (avc-forward-wald.lst); run.sh copies it  */
/* to ./capture/ for the R fixture builder.                                 */

%let hzrdelds = 1;
%HAZARD(
PROC HAZARD DATA = _avc_clean NOCOV NOCOR CONDITION=14;
     TIME INT_DEAD;
     EVENT DEAD;
     PARMS MUE=0.2361727 THALF=0.1512095 NU=1.438652 M=1 FIXM
           FIXTHALF FIXNU
           MUC=0.0005436977;
     SELECTION SLE=0.30 SLS=0.20 MAXSTEPS=50;
     EARLY
       AGE, MAL, INC_SURG, ORIFICE, COM_IV;
     CONSTANT
       AGE, MAL, INC_SURG, ORIFICE, COM_IV;
);

/* -- Metadata file -------------------------------------------------------- */
OPTIONS DLCREATEDIR;
LIBNAME _cap "&out_dir";
LIBNAME _cap CLEAR;

DATA _NULL_;
  FILE "&out_dir/stepwise_meta.txt";
  PUT "sas_version=9.4";
  PUT "dataset=avc";
  PUT "dist=weibull";
  PUT "criterion=wald";
  /* SELECTION with both SLE and SLS is forward-backward; the listing includes
     drop steps.  Record direction=both so a rebuild matches the committed
     fixture (the R parity runner maps this to direction = "both"). */
  PUT "direction=both";
  PUT "slentry=0.30";
  PUT "slstay=0.20";
  PUT "proc_hazard=SAS HAZARD (PROC HAZARD via %HAZARD macro)";
  PUT "captured_on=&run_date";
RUN;

/* =======================================================================
 * avc-forward-wald.sas — Reference capture for R-side stepwise parity
 *
 * Runs forward Wald stepwise selection on the AVC dataset using SAS
 * PROC HAZARD with SLENTRY = 0.30 / SLSTAY = 0.20 (SAS defaults) and
 * emits three text files consumed by .hzr_build_stepwise_fixture() in R.
 *
 * Dataset: hz.death.AVC  (n = 310, 68 deaths after na.omit)
 * Covariates: AGE MAL INC_SURG ORIFICE COM_IV
 * Distribution: Weibull
 * Direction: Forward
 * Criterion: Wald chi-square
 *
 * Usage:
 *   1. Upload inst/extdata/avc.csv to the server (or point to an
 *      existing SAS dataset).
 *   2. Set the three macro variables below.
 *   3. Run in SAS 9.4+ with PROC HAZARD available.
 *   4. Copy the three output files back to your R session and run:
 *
 *        TemporalHazard:::.hzr_build_stepwise_fixture(
 *          trace_csv = "path/to/stepwise_trace.csv",
 *          final_csv = "path/to/stepwise_final.csv",
 *          meta_txt  = "path/to/stepwise_meta.txt",
 *          out_path  = "inst/fixtures/stepwise-avc-forward-wald.rds"
 *        )
 *
 * ===================================================================== */

%LET csv_path = /path/to/avc.csv;       /* path to uploaded avc.csv  */
%LET out_dir  = /path/to/capture;       /* directory for output files */
%LET run_date = 2026-06-10;             /* update to actual run date  */

/* -- Import the CSV ------------------------------------------------------ */
PROC IMPORT DATAFILE = "&csv_path"
            OUT      = _avc
            DBMS     = CSV
            REPLACE;
  GETNAMES = YES;
RUN;

/* Remove incomplete cases (matches R's na.omit(avc)) */
DATA _avc_clean;
  SET _avc;
  IF NMISS(INT_DEAD, DEAD, AGE, MAL, INC_SURG, ORIFICE, COM_IV) = 0;
RUN;

/* -- PROC HAZARD invocation --------------------------------------------- */
ODS LISTING CLOSE;
ODS OUTPUT
  SelectionSummary  = _sel_trace    /* step-by-step history         */
  ParameterEstimates = _sel_final   /* final coefficient table      */
  ConvergenceStatus  = _sel_conv    /* iterations, convergence code */
  FitStatistics      = _sel_fit;    /* logLik, AIC, BIC             */

PROC HAZARD DATA = _avc_clean
            METHOD = STEPWISE
            SLENTRY = 0.30
            SLSTAY  = 0.20;
  TIME INT_DEAD * DEAD(0);
  MODEL INT_DEAD = AGE MAL INC_SURG ORIFICE COM_IV
        / DIST = WEIBULL;
RUN;

ODS LISTING;

/* -- Write the three capture files -------------------------------------- */

/* 1. Step trace: one row per step. */
DATA _step_trace;
  SET _sel_trace;
  LENGTH action $ 8 variable $ 32 phase $ 8;
  step_num = _N_;
  IF MISSING(EffectRemoved) THEN DO;
    action   = "enter";
    variable = LOWCASE(STRIP(EffectEntered));
  END;
  ELSE DO;
    action   = "drop";
    variable = LOWCASE(STRIP(EffectRemoved));
  END;
  phase   = ".";
  stat    = ChiSquare;
  df      = DF;
  p_value = ProbChiSq;
  KEEP step_num action variable phase stat df p_value;
RUN;

PROC EXPORT DATA = _step_trace
            OUTFILE = "&out_dir/stepwise_trace.csv"
            DBMS = CSV REPLACE;
RUN;

/* 2. Final coefficient table. */
DATA _final_coef;
  SET _sel_final;
  LENGTH variable $ 32 phase $ 8;
  variable  = LOWCASE(STRIP(Parameter));
  phase     = ".";
  estimate  = Estimate;
  std_error = StdErr;
  z_stat    = Estimate / StdErr;
  p_value   = ProbChiSq;
  KEEP variable phase estimate std_error z_stat p_value;
RUN;

PROC EXPORT DATA = _final_coef
            OUTFILE = "&out_dir/stepwise_final.csv"
            DBMS = CSV REPLACE;
RUN;

/* 3. Meta / GOF totals. */
DATA _NULL_;
  FILE "&out_dir/stepwise_meta.txt";
  SET _sel_fit(WHERE = (UPCASE(STRIP(Criterion)) = "-2 LOG L" OR
                        UPCASE(STRIP(Criterion)) = "AIC"));
  IF UPCASE(STRIP(Criterion)) = "-2 LOG L" THEN
    PUT "logLik=" (Value / -2) 12.6;
  ELSE IF UPCASE(STRIP(Criterion)) = "AIC" THEN
    PUT "aic=" Value 12.6;
RUN;

DATA _NULL_;
  FILE "&out_dir/stepwise_meta.txt" MOD;
  SET _sel_conv(OBS = 1);
  PUT "iterations=" _N_;
  PUT "converged=" Status;
RUN;

DATA _NULL_;
  FILE "&out_dir/stepwise_meta.txt" MOD;
  PUT "sas_version=9.4";
  PUT "dataset=avc";
  PUT "dist=weibull";
  PUT "criterion=wald";
  PUT "direction=forward";
  PUT "slentry=0.30";
  PUT "slstay=0.20";
  PUT "proc_hazard=SAS HAZARD";
  PUT "captured_on=&run_date";
RUN;

/* =======================================================================
 * avc-weighted.sas  --  TemporalHazard fractional-weight parity capture
 *
 * Run via run.sh (which sets HAZAPPS + HZ_MACROS before calling SAS):
 *
 *   cd /path/to/this/folder
 *   ./run.sh
 *
 * Produces ./capture/avc-weighted.lst, which parse-weighted-lst.R parses
 * into inst/fixtures/weighted-avc-fractional.rds.
 *
 * Purpose: confirm SAS PROC HAZARD's WEIGHT statement matches R's
 * hazard(weights = ...) for NON-INTEGER (inverse-probability-style)
 * weights -- roadmap 7a / FIXTURE-GAP-LIST B5.  R-side correctness is
 * already proven by the additive-split + linear-scaling invariants in
 * test-weights.R; this fixture is the external SAS confirmation.
 *
 * Model: two-phase (early CDF + constant) hazard with Weibull shaping,
 * AGE as a covariate in both phases, weighted by the fractional column
 * IPW.  Shape parameters (THALF, NU, M) are fixed at the AVC
 * unconditional fit values (same as the stepwise capture) for numerical
 * stability.  Starting MU values likewise.
 * ===================================================================== */

/* -- Load HAZARD macros ------------------------------------------------- */
%let _macros = %sysget(HZ_MACROS);
%if %length(&_macros) = 0 %then %do;
  %put ERROR: HZ_MACROS env var not set -- run this script via run.sh.;
  %abort;
%end;
OPTIONS SASAUTOS=("&_macros" SASAUTOS);

/* Override TMPDIR so %HAZARD macro and C binary agree on the XPORT path. */
%let _hz_tmp = %sysget(HZ_TMPDIR);
%if %length(&_hz_tmp) = 0 %then %do;
  %let _hz_tmp = /tmp;
%end;
OPTIONS SET=TMPDIR "&_hz_tmp";

/* SAS site config overrides HAZAPPS; override it back to the local build. */
%let _hz_apps = %sysget(HAZAPPS);
%if %length(&_hz_apps) > 0 %then %do;
  OPTIONS SET=HAZAPPS "&_hz_apps";
%end;

/* -- Paths ---------------------------------------------------------------- */
%LET csv_path = ./avc-weighted.csv;
%LET out_dir  = ./capture;
/* Stamp the capture date from the run so the metadata is correct on whatever
   day the fixture is captured (yields YYYY-MM-DD). */
%LET run_date = %sysfunc(today(), yymmdd10.);

/* -- Import avc-weighted.csv ---------------------------------------------- */
PROC IMPORT DATAFILE = "&csv_path"
            OUT      = _avc
            DBMS     = CSV
            REPLACE;
  GETNAMES = YES;
RUN;

/* Exclude rows with any missing analysis variable (matches R na.omit).
   IPW is a function of AGE, so it is non-missing wherever AGE is.          */
DATA _avc_clean;
  SET _avc;
  IF NMISS(INT_DEAD, DEAD, AGE, IPW) = 0;
RUN;

/* -- Weighted two-phase fit via PROC HAZARD ------------------------------- */
/*                                                                           */
/* WEIGHT IPW applies the fractional column as Fisher observation weights.   */
/* No SELECTION -- AGE is forced into both phases so the model is fixed and  */
/* the R parity refit uses the identical specification.                      */

%let hzrdelds = 1;
%HAZARD(
PROC HAZARD DATA = _avc_clean NOCOV NOCOR CONDITION=14;
     TIME INT_DEAD;
     EVENT DEAD;
     WEIGHT IPW;
     PARMS MUE=0.2361727 THALF=0.1512095 NU=1.438652 M=1 FIXM
           FIXTHALF FIXNU
           MUC=0.0005436977;
     EARLY
       AGE;
     CONSTANT
       AGE;
);

/* -- Metadata file -------------------------------------------------------- */
OPTIONS DLCREATEDIR;
LIBNAME _cap "&out_dir";
LIBNAME _cap CLEAR;

DATA _NULL_;
  FILE "&out_dir/weighted_meta.txt";
  PUT "sas_version=9.4";
  PUT "dataset=avc";
  PUT "dist=multiphase";
  PUT "model=early_cdf+constant";
  PUT "covariates=age";
  PUT "weight_var=ipw";
  PUT "weight_kind=fractional";
  PUT "conserve=1";
  PUT "proc_hazard=SAS HAZARD (PROC HAZARD via %HAZARD macro), WEIGHT IPW";
  PUT "captured_on=&run_date";
RUN;

## -----------------------------------------------------------------------------
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE)
has_ggplot <- requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = has_pkg,
  fig.width = 7,
  fig.height = 4.5
)


## -----------------------------------------------------------------------------
#| label: setup
library(TemporalHazard)


## -----------------------------------------------------------------------------
#| label: basic-decompos
t_grid <- seq(0.01, 10, length.out = 200)

# Standard sigmoidal (Case 1: m > 0, nu > 0)
d <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 1)

# Verify the half-life property: G(t_half) = 0.5
d_half <- hzr_decompos(3, t_half = 3, nu = 2, m = 1)
cat("G(t_half) =", round(d_half$G, 6), "\n")

# Verify the identity: h(t) = g(t) / (1 - G(t))
h_check <- d$g / (1 - d$G)
cat("Max |h - g/(1-G)| =", max(abs(d$h - h_check)), "\n")


## -----------------------------------------------------------------------------
#| label: six-cases-table
#| echo: false
cases <- data.frame(
  Case = c("1", "1L", "2", "2L", "3", "3L"),
  m     = c("> 0", "= 0", "< 0", "< 0", "> 0", "= 0"),
  nu    = c("> 0", "> 0", "> 0", "= 0", "< 0", "< 0"),
  Behavior = c(
    "Standard sigmoidal",
    "Weibull-like (exponential limit as m -> 0)",
    "Heavy-tailed",
    "Exponential decay",
    "Bounded cumulative",
    "Bounded exponential"
  )
)
knitr::kable(cases, caption = "Six valid parameter sign combinations")


## -----------------------------------------------------------------------------
#| label: fig-six-cases
#| fig-cap: "G(t), g(t), and h(t) for all six valid decomposition cases"
#| fig-height: 8
if (has_ggplot) {
  library(ggplot2)

  params <- list(
    "Case 1: m=1, nu=2"     = list(nu = 2,   m = 1),
    "Case 1L: m=0, nu=2"    = list(nu = 2,   m = 0),
    "Case 2: m=-0.5, nu=2"  = list(nu = 2,   m = -0.5),
    "Case 2L: m=-0.5, nu=0" = list(nu = 0,   m = -0.5),
    "Case 3: m=1, nu=-0.5"  = list(nu = -0.5, m = 1),
    "Case 3L: m=0, nu=-0.5" = list(nu = -0.5, m = 0)
  )

  t_grid <- seq(0.01, 10, length.out = 200)
  rows <- list()

  for (nm in names(params)) {
    p <- params[[nm]]
    d <- hzr_decompos(t_grid, t_half = 3, nu = p$nu, m = p$m)
    rows <- c(rows, list(data.frame(
      time = rep(t_grid, 3),
      value = c(d$G, d$g, d$h),
      quantity = rep(c("G(t) — CDF", "g(t) — density", "h(t) — hazard"),
                     each = length(t_grid)),
      case = nm
    )))
  }

  df <- do.call(rbind, rows)

  # Cap hazard display at reasonable values for readability
  df$value[df$quantity == "h(t) — hazard" & df$value > 5] <- NA

  ggplot(df, aes(x = time, y = value)) +
    geom_line(linewidth = 0.6) +
    facet_grid(quantity ~ case, scales = "free_y") +
    labs(x = "Time", y = NULL) +
    theme_minimal(base_size = 10) +
    theme(strip.text = element_text(size = 7))
}


## -----------------------------------------------------------------------------
#| label: phase-construction
# Classic three-phase pattern
early <- hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0)
const <- hzr_phase("constant")
late  <- hzr_phase("hazard",   t_half = 5,   nu = 1, m = 0)

print(early)
print(const)
print(late)


## -----------------------------------------------------------------------------
#| label: fig-phase-shapes
#| fig-cap: "Phi(t) and phi(t) for each phase type"
#| fig-height: 5
if (has_ggplot) {
  t_grid <- seq(0.01, 12, length.out = 200)

  phi_early <- hzr_phase_cumhaz(t_grid, t_half = 0.5, nu = 2, m = 0,
                                 type = "cdf")
  phi_late  <- hzr_phase_cumhaz(t_grid, t_half = 5, nu = 1, m = 0,
                                 type = "hazard")
  phi_const <- hzr_phase_cumhaz(t_grid, type = "constant")

  dphi_early <- hzr_phase_hazard(t_grid, t_half = 0.5, nu = 2, m = 0,
                                  type = "cdf")
  dphi_late  <- hzr_phase_hazard(t_grid, t_half = 5, nu = 1, m = 0,
                                  type = "hazard")
  dphi_const <- hzr_phase_hazard(t_grid, type = "constant")

  df <- data.frame(
    time = rep(t_grid, 6),
    value = c(phi_early, phi_late, phi_const,
              dphi_early, dphi_late, dphi_const),
    phase = rep(rep(c("cdf (early)", "hazard (late)", "constant"),
                    each = length(t_grid)), 2),
    quantity = rep(c("Phi(t) — cumulative", "phi(t) — instantaneous"),
                   each = 3 * length(t_grid))
  )

  ggplot(df, aes(x = time, y = value, colour = phase)) +
    geom_line(linewidth = 0.8) +
    facet_wrap(~ quantity, scales = "free_y", ncol = 1) +
    scale_colour_manual(values = c(
      "cdf (early)" = "#0072B2",
      "hazard (late)" = "#D55E00",
      "constant" = "#009E73"
    )) +
    labs(x = "Time", y = NULL, colour = "Phase type") +
    theme_minimal(base_size = 11) +
    theme(legend.position = "bottom")
}


## -----------------------------------------------------------------------------
#| label: fig-additive-demo
#| fig-cap: "Three-phase additive cumulative hazard: total = early + constant + late"
#| fig-height: 4
if (has_ggplot) {
  t_grid <- seq(0.01, 15, length.out = 300)

  mu_early <- 0.3
  mu_const <- 0.05
  mu_late  <- 0.2

  H_early <- mu_early * hzr_phase_cumhaz(t_grid, t_half = 0.5, nu = 2,
                                           m = 0, type = "cdf")
  H_const <- mu_const * hzr_phase_cumhaz(t_grid, type = "constant")
  H_late  <- mu_late  * hzr_phase_cumhaz(t_grid, t_half = 5, nu = 1,
                                           m = 0, type = "hazard")
  H_total <- H_early + H_const + H_late

  df <- data.frame(
    time = rep(t_grid, 4),
    cumhaz = c(H_early, H_const, H_late, H_total),
    component = rep(c("Early (cdf)", "Constant", "Late (hazard)", "Total"),
                    each = length(t_grid))
  )
  df$component <- factor(df$component,
    levels = c("Total", "Early (cdf)", "Constant", "Late (hazard)"))

  ggplot(df, aes(x = time, y = cumhaz, colour = component,
                 linewidth = component)) +
    geom_line() +
    scale_colour_manual(values = c(
      "Total" = "black", "Early (cdf)" = "#0072B2",
      "Constant" = "#009E73", "Late (hazard)" = "#D55E00"
    )) +
    scale_linewidth_manual(values = c(
      "Total" = 1.2, "Early (cdf)" = 0.6,
      "Constant" = 0.6, "Late (hazard)" = 0.6
    )) +
    labs(x = "Time", y = "Cumulative hazard H(t)",
         colour = NULL, linewidth = NULL) +
    theme_minimal(base_size = 11) +
    theme(legend.position = "bottom")
}


## -----------------------------------------------------------------------------
#| label: log1mexp-demo
# Naive log(1 - exp(-x)) fails near zero
x_small <- 1e-15
cat("Naive:  ", log(1 - exp(-x_small)), "\n")
cat("Stable: ", hzr_log1mexp(x_small), "\n")


## -----------------------------------------------------------------------------
#| label: global-covariates
#| eval: false
# # Both phases use age and nyha from the global formula
# hazard(
#   survival::Surv(time, status) ~ age + nyha,
#   data   = dat,
#   dist   = "multiphase",
#   phases = list(
#     early = hzr_phase("cdf",    t_half = 0.5, nu = 2, m = 0),
#     late  = hzr_phase("hazard", t_half = 5,   nu = 1, m = 0)
#   ),
#   fit = TRUE
# )


## -----------------------------------------------------------------------------
#| label: phase-specific-covariates
#| eval: false
# # Early risk depends on surgical factors; late risk on chronic conditions
# hazard(
#   survival::Surv(time, status) ~ age + nyha + shock,
#   data   = dat,
#   dist   = "multiphase",
#   phases = list(
#     early = hzr_phase("cdf",    t_half = 0.5, nu = 2, m = 0,
#                       formula = ~ age + shock),
#     late  = hzr_phase("hazard", t_half = 5,   nu = 1, m = 0,
#                       formula = ~ age + nyha)
#   ),
#   fit = TRUE
# )


## -----------------------------------------------------------------------------
#| label: time-varying-demo
#| eval: false
# # Two windows: [0, 2] and (2, infinity)
# hazard(
#   survival::Surv(time, status) ~ age + nyha,
#   data         = dat,
#   theta        = c(mu = 0.25, nu = 1.1,
#                    age_w1 = 0, nyha_w1 = 0,
#                    age_w2 = 0, nyha_w2 = 0),
#   dist         = "weibull",
#   time_windows = 2,
#   fit          = TRUE
# )

# make-avc-weighted-csv.R -- regenerate payload/avc-weighted.csv.
#
# Writes the AVC analysis data with a deterministic, NON-INTEGER weight
# column `ipw` appended.  The weight is a closed-form function of `age`
# only -- no RNG, no cohort-derived statistics -- so the column is exactly
# reproducible and SAS (PROC HAZARD `WEIGHT ipw;`) and R (`hazard(weights =
# ipw)`) read identical values.  The precise formula is illustrative (an
# inverse-probability-style weight in (0.5, 1.5)); what matters for the
# parity test is only that both engines weight the same fractional column.
#
# Run from the project root after devtools::load_all():
#   source("inst/extdata/weights-fixtures/make-avc-weighted-csv.R")

suppressMessages(devtools::load_all(quiet = TRUE))
data(avc, package = "TemporalHazard")

# Deterministic fractional weight: logistic in age, centred at 60, scale 10.
# Range (0.5, 1.5); rounded to 6 dp so the CSV is byte-stable.
avc$ipw <- round(0.5 + 1 / (1 + exp(-(avc$age - 60) / 10)), 6)

out <- "inst/extdata/weights-fixtures/payload/avc-weighted.csv"
utils::write.csv(avc, out, row.names = FALSE, na = ".")
message("Wrote ", out, " (", nrow(avc), " rows; ipw range ",
        paste(round(range(avc$ipw, na.rm = TRUE), 4), collapse = "-"), ")")

# parse-hazard-lst.R -- Parse a HAZARD C-binary .lst listing into the three
# files consumed by .hzr_build_stepwise_fixture().
#
# Usage (from the project root, after devtools::load_all()):
#
#   source("inst/extdata/stepwise-fixtures/parse-hazard-lst.R")
#
# Inputs:
#   inst/extdata/stepwise-fixtures/payload/capture/avc-forward-wald.lst
#   inst/extdata/stepwise-fixtures/payload/capture/stepwise_meta.txt
#
# Outputs (written in-place to payload/capture/):
#   stepwise_trace.csv   -- one row per accepted step
#   stepwise_final.csv   -- final coefficient table
#   stepwise_meta.txt    -- updated with logLik= entry
#
# Then builds:
#   inst/fixtures/stepwise-avc-forward-wald.rds

lst_file    <- "inst/extdata/stepwise-fixtures/payload/capture/avc-forward-wald.lst"
meta_file   <- "inst/extdata/stepwise-fixtures/payload/capture/stepwise_meta.txt"
out_trace   <- "inst/extdata/stepwise-fixtures/payload/capture/stepwise_trace.csv"
out_final   <- "inst/extdata/stepwise-fixtures/payload/capture/stepwise_final.csv"
fixture_out <- "inst/fixtures/stepwise-avc-forward-wald.rds"

stopifnot(file.exists(lst_file), file.exists(meta_file))

lines <- readLines(lst_file, warn = FALSE)

# ---- 1. Step trace ----------------------------------------------------------
# Banner format (lines after the *---* separator):
#   |Step   N. VAR   added to PHASE PHASE                  |
#   |                   with Z =   Z_VAL and P = P_VAL     |
# or
#   |Step   N. VAR   deleted from PHASE PHASE              |
#   |                   with Z =   Z_VAL and P = P_VAL     |
# Step 0 is the intercept-only baseline; skip it.

step_idx <- grep("^\\|Step\\s+\\d+\\.", lines)
step_rows <- vector("list", length(step_idx))
n_steps <- 0L

for (i in step_idx) {
  line <- lines[i]
  step_num <- as.integer(sub("^\\|Step\\s+(\\d+)\\..*", "\\1", line))
  if (step_num == 0L) next

  if (grepl("added to", line, ignore.case = TRUE)) {
    action <- "enter"
    rest   <- sub("^\\|Step\\s+\\d+\\.\\s+", "", line)
    variable <- trimws(sub("\\s+added to.*", "", rest))
    phase_str <- trimws(sub(".*added to\\s+(\\S+)\\s+PHASE.*", "\\1", line,
                             ignore.case = TRUE))
  } else if (grepl("deleted from", line, ignore.case = TRUE)) {
    action <- "drop"
    rest   <- sub("^\\|Step\\s+\\d+\\.\\s+", "", line)
    variable <- trimws(sub("\\s+deleted from.*", "", rest))
    phase_str <- trimws(sub(".*deleted from\\s+(\\S+)\\s+PHASE.*", "\\1", line,
                             ignore.case = TRUE))
  } else {
    next
  }

  phase <- tolower(phase_str)

  # The next line carries "with Z = ... and P = ..."
  z_line <- if (i + 1L <= length(lines)) lines[i + 1L] else ""
  z_val  <- as.numeric(sub(".*with Z\\s*=\\s*([+-]?[0-9.]+).*", "\\1", z_line))

  p_raw  <- trimws(sub(".*and P\\s*=\\s*(.+?)\\s*\\|.*", "\\1", z_line))
  p_val  <- if (startsWith(p_raw, "<")) {
    as.numeric(trimws(sub("^<\\s*\\.?", "0.", p_raw)))
  } else {
    as.numeric(p_raw)
  }

  n_steps <- n_steps + 1L
  step_rows[[n_steps]] <- data.frame(
    step_num = step_num,
    action   = action,
    variable = variable,
    phase    = phase,
    stat     = z_val^2,   # Wald chi-square = Z^2 (recorded for reference; the
                          # current parity test compares final-model agreement,
                          # not per-step statistics)
    df       = 1L,
    p_value  = p_val,
    stringsAsFactors = FALSE
  )
}

trace_df <- do.call(rbind, step_rows[seq_len(n_steps)])
# SAS uppercases all variable names; R's avc uses lowercase.
trace_df$variable <- tolower(trace_df$variable)
utils::write.csv(trace_df, out_trace, row.names = FALSE)
message("Wrote ", out_trace, " (", nrow(trace_df), " steps)")

# ---- 2. Final coefficient table ---------------------------------------------
# Lives in the "Parameter Estimate Summary" block under "Final Results:".
# Format (fixed-width columns):
#
#   Phase      Parameter    Estimate       Std error             Z    Prob>|Z|
#   ----------------------...
#   Early:     E0             -2.20788      0.4338312         -5.089    <.0001
#              AGE          -0.00544498    0.002572203         -2.117    0.0343
#              ...
#              ------...
#   Constant:  C0             -10.6589       1.726197         -6.175    <.0001
#              ...
#   ------...
#
# The inner --------... separates phases.
# The outer (longer) --------... ends the table.

param_summary_idx <- grep("Parameter Estimate Summary", lines, fixed = TRUE)[1]
stopifnot(!is.na(param_summary_idx))

# Header line: "Phase  Parameter  Estimate  Std error  Z  Prob>|Z|"
col_header_idx <- grep(
  "Phase\\s+Parameter\\s+Estimate\\s+Std error",
  lines[param_summary_idx:length(lines)]
)[1] + param_summary_idx - 1L

# Data starts after the dashes line immediately below the header
data_start <- col_header_idx + 2L   # +1 dashes, +1 first data line

# Stop when we hit "Estimates for Model Parameters" (next section)
stop_idx <- grep("Estimates for Model Parameters", lines, fixed = TRUE)
stop_idx <- stop_idx[stop_idx > data_start][1L]

final_rows    <- list()
current_phase <- NA_character_

for (i in seq(data_start, stop_idx - 1L)) {
  line    <- lines[i]
  trimmed <- trimws(line)

  if (nchar(trimmed) == 0L) next
  if (grepl("^-{5,}", trimmed))  next   # separator line

  # Phase label on this line?
  phase_lbl <- regmatches(
    line, regexpr("(Early|Constant):", line, perl = TRUE, ignore.case = TRUE)
  )
  if (length(phase_lbl) > 0L && nchar(phase_lbl) > 0L) {
    current_phase <- tolower(sub(":", "", phase_lbl))
    # Strip the "Early:  " or "Constant:  " prefix before parsing numeric fields
    line <- sub(".*(?:Early|Constant):\\s+", "", line, ignore.case = TRUE,
                perl = TRUE)
  }

  # Expect: PARAM  estimate  std_error  z_stat  prob
  # Prob may be "<.0001"
  m <- regexpr(
    paste0("^\\s*(\\S+)\\s+([+-]?[0-9.Ee+-]+)\\s+([0-9.Ee+-]+)\\s+",
           "([+-]?[0-9.Ee+-]+)\\s+([<]?[.0-9]+)"),
    line, perl = TRUE
  )
  if (m < 0L) next

  tok <- strsplit(trimws(regmatches(line, m)), "\\s+")[[1]]
  if (length(tok) < 5L) next

  p_raw <- tok[5L]
  p_val <- if (startsWith(p_raw, "<")) {
    as.numeric(sub("^<\\.?", "0.", p_raw))
  } else {
    as.numeric(p_raw)
  }

  final_rows[[length(final_rows) + 1L]] <- data.frame(
    variable  = tok[1L],
    phase     = current_phase,
    estimate  = as.numeric(tok[2L]),
    std_error = as.numeric(tok[3L]),
    z_stat    = as.numeric(tok[4L]),
    p_value   = p_val,
    stringsAsFactors = FALSE
  )
}

final_df <- do.call(rbind, final_rows)
final_df$variable <- tolower(final_df$variable)
utils::write.csv(final_df, out_final, row.names = FALSE)
message("Wrote ", out_final, " (", nrow(final_df), " parameters)")

# ---- 3. Augment meta.txt with logLik ----------------------------------------
# Grab the LAST "Log likelihood = ..." line in the file (= final model's).
ll_val <- as.numeric(trimws(sub(
  ".*Log likelihood\\s*=\\s*", "",
  tail(grep("Log likelihood\\s*=", lines, value = TRUE), 1L)
)))
message("Final log-likelihood: ", ll_val)

meta_lines <- readLines(meta_file)
# Remove any existing logLik entry then append the fresh one.
meta_lines <- meta_lines[!grepl("^logLik\\s*=", meta_lines)]
meta_lines <- c(meta_lines, paste0("logLik=", ll_val))
writeLines(meta_lines, meta_file)
message("Updated ", meta_file)

# ---- 4. Build the .rds fixture ----------------------------------------------
devtools::load_all(quiet = TRUE)

fix <- TemporalHazard:::.hzr_build_stepwise_fixture(
  trace_csv = out_trace,
  final_csv = out_final,
  meta_txt  = meta_file,
  out_path  = fixture_out
)
message("Built fixture: ", fixture_out)
message("\nStep trace:")
print(fix$steps)
message("\nFinal coefficients:")
print(fix$final$coef)
message("\nlogLik: ", fix$final$logLik)

## -----------------------------------------------------------------------------
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE)
has_ggplot2 <- requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5,
  eval = has_pkg
)


## -----------------------------------------------------------------------------
#| label: setup
library(TemporalHazard)
if (has_ggplot2) library(ggplot2)

data(avc)
avc <- na.omit(avc)
str(avc)


## -----------------------------------------------------------------------------
#| label: km-life-table
km <- hzr_kaplan(time = avc$int_dead, status = avc$dead)
head(km)


## -----------------------------------------------------------------------------
#| label: fig-km-baseline
#| fig-cap: "Kaplan-Meier survival estimate for AVC patients (n = 310)"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
km_df <- data.frame(
  time     = km$time,
  survival = km$survival * 100,
  lower    = km$cl_lower * 100,
  upper    = km$cl_upper * 100
)

ggplot(km_df, aes(time, survival)) +
  geom_step(colour = "#D55E00", linewidth = 0.8) +
  geom_ribbon(aes(ymin = lower, ymax = upper),
              stat = "identity", alpha = 0.15, fill = "#D55E00") +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: weibull-fit
fit_weib <- hazard(
  survival::Surv(int_dead, dead) ~ 1,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.05, nu = 0.5),
  fit   = TRUE
)
summary(fit_weib)


## -----------------------------------------------------------------------------
#| label: fig-weibull-overlay
#| fig-cap: "Single Weibull vs. Kaplan-Meier"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
t_grid <- seq(0.01, max(avc$int_dead) * 0.9, length.out = 200)
surv_weib <- predict(fit_weib,
                     newdata = data.frame(time = t_grid),
                     type = "survival") * 100

ggplot() +
  geom_step(data = km_df, aes(time, survival),
            colour = "#D55E00", linewidth = 0.6) +
  geom_line(data = data.frame(time = t_grid, survival = surv_weib),
            aes(time, survival), colour = "#0072B2", linewidth = 0.8) +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: multiphase-fit
fit_mp <- hazard(
  survival::Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)
summary(fit_mp)


## -----------------------------------------------------------------------------
#| label: fig-multiphase-overlay
#| fig-cap: "Two-phase parametric model vs. Kaplan-Meier"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
surv_mp <- predict(fit_mp,
                   newdata = data.frame(time = t_grid),
                   type = "survival") * 100

ggplot() +
  geom_step(data = km_df, aes(time, survival),
            colour = "#D55E00", linewidth = 0.6) +
  geom_line(data = data.frame(time = t_grid, survival = surv_mp),
            aes(time, survival), colour = "#0072B2", linewidth = 0.8) +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: fig-decomposed
#| fig-cap: "Phase decomposition of cumulative hazard"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
decomp <- predict(fit_mp,
                  newdata = data.frame(time = t_grid),
                  type = "cumulative_hazard",
                  decompose = TRUE)

decomp_df <- data.frame(
  time  = t_grid,
  total = decomp[, "total"],
  early = decomp[, "early"],
  const = decomp[, "constant"]
)

ggplot(decomp_df, aes(x = time)) +
  geom_line(aes(y = total), linewidth = 0.9, colour = "black") +
  geom_line(aes(y = early), linewidth = 0.7, colour = "#E69F00",
            linetype = "dashed") +
  geom_line(aes(y = const), linewidth = 0.7, colour = "#56B4E9",
            linetype = "dashed") +
  labs(x = "Months after AVC repair", y = "Cumulative hazard") +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: screening
covariates <- c("age", "status", "mal", "com_iv", "orifice",
                "inc_surg", "opmos")

screen_results <- data.frame(
  variable = covariates,
  coef     = numeric(length(covariates)),
  p_value  = numeric(length(covariates))
)

for (i in seq_along(covariates)) {
  v <- covariates[i]
  fml <- as.formula(paste("dead ~", v))
  lg <- glm(fml, data = avc, family = binomial)
  s <- summary(lg)$coefficients
  if (nrow(s) >= 2) {
    screen_results$coef[i] <- s[2, 1]
    screen_results$p_value[i] <- s[2, 4]
  }
}

screen_results <- screen_results[order(screen_results$p_value), ]
screen_results$significant <- ifelse(screen_results$p_value < 0.05,
                                      "*", "")
screen_results


## -----------------------------------------------------------------------------
#| label: age-calibrate
cal_age <- hzr_calibrate(x = avc$age, event = avc$dead,
                          groups = 10, link = "logit")
cal_age


## -----------------------------------------------------------------------------
#| label: fig-calibrate-age
#| fig-cap: "Decile calibration: age vs. logit(P(death))"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(cal_age, aes(mean, link_value)) +
  geom_point(size = 3, colour = "#0072B2") +
  geom_line(colour = "#0072B2", alpha = 0.4) +
  geom_smooth(method = "lm", formula = y ~ x, se = FALSE,
              colour = "#D55E00", linetype = "dashed") +
  labs(x = "Age at repair (months, decile midpoint)",
       y = "logit(P(death))") +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: multivariable-fit
fit_mv <- hazard(
  survival::Surv(int_dead, dead) ~ age + status + mal + com_iv,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)
summary(fit_mv)


## -----------------------------------------------------------------------------
#| label: stepwise-forward
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE)
base_mp <- hazard(
  survival::Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 3, maxit = 500)
)

fit_step <- hzr_stepwise(
  base_mp,
  scope     = list(
    early    = ~ age + status + mal + com_iv,
    constant = ~ age + status + mal + com_iv
  ),
  data      = avc,
  direction = "both",
  criterion = "wald",
  trace     = FALSE,
  control   = list(n_starts = 2, maxit = 500)
)
fit_step


## -----------------------------------------------------------------------------
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE)
fit_step$steps[, c("step_num", "action", "variable", "phase",
                   "p_value", "aic")]


## -----------------------------------------------------------------------------
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE)
logLik_manual <- fit_mv$fit$objective
logLik_step   <- fit_step$fit$objective
c(manual = logLik_manual, stepwise = logLik_step,
  aic_manual = 2 * length(fit_mv$fit$theta) - 2 * logLik_manual,
  aic_step   = 2 * length(fit_step$fit$theta) - 2 * logLik_step)


## -----------------------------------------------------------------------------
#| label: fig-prediction
#| fig-cap: "Reference patient survival with 95% delta-method CI and KM overlay"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ref_patient <- data.frame(
  time   = t_grid,
  age    = median(avc$age),
  status = 1,
  mal    = 0,
  com_iv = 0
)

surv_ref <- predict(fit_mv, newdata = ref_patient,
                    type = "survival", se.fit = TRUE)

ref_df <- data.frame(
  time     = t_grid,
  survival = surv_ref$fit * 100,
  lower    = surv_ref$lower * 100,
  upper    = surv_ref$upper * 100
)

ggplot() +
  geom_step(data = km_df, aes(time, survival),
            colour = "#D55E00", linewidth = 0.6, alpha = 0.7) +
  geom_ribbon(data = ref_df, aes(time, ymin = lower, ymax = upper),
              fill = "#0072B2", alpha = 0.15) +
  geom_line(data = ref_df, aes(time, survival),
            colour = "#0072B2", linewidth = 0.8) +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: fig-sensitivity
#| fig-cap: "Survival by risk profile: low-risk (blue) vs. high-risk (red)"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
low_risk <- data.frame(
  time   = t_grid,
  age    = quantile(avc$age, 0.25),
  status = 1,
  mal    = 0,
  com_iv = 0
)

high_risk <- data.frame(
  time   = t_grid,
  age    = quantile(avc$age, 0.90),
  status = 4,
  mal    = 1,
  com_iv = 1
)

surv_lo <- predict(fit_mv, newdata = low_risk,
                   type = "survival", se.fit = TRUE)
surv_hi <- predict(fit_mv, newdata = high_risk,
                   type = "survival", se.fit = TRUE)

sens_df <- data.frame(
  time     = rep(t_grid, 2),
  survival = c(surv_lo$fit, surv_hi$fit) * 100,
  lower    = c(surv_lo$lower, surv_hi$lower) * 100,
  upper    = c(surv_lo$upper, surv_hi$upper) * 100,
  profile  = rep(c("Low risk", "High risk"), each = length(t_grid))
)

ggplot(sens_df, aes(time, survival, colour = profile, fill = profile)) +
  geom_ribbon(aes(ymin = lower, ymax = upper),
              alpha = 0.15, colour = NA) +
  geom_line(linewidth = 0.8) +
  scale_colour_manual(values = c("Low risk" = "#0072B2",
                                  "High risk" = "#D55E00")) +
  scale_fill_manual(values = c("Low risk" = "#0072B2",
                                "High risk" = "#D55E00")) +
  labs(x = "Months after AVC repair", y = "Survival (%)",
       colour = NULL, fill = NULL) +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: fig-deciles
#| fig-cap: "Observed vs. expected event rates by risk decile"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
cal <- hzr_deciles(fit_mv, time = max(avc$int_dead))
print(cal)


## -----------------------------------------------------------------------------
#| label: fig-decile-plot
#| fig-cap: "Decile calibration: observed (bars) vs. expected (points) event rates"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(cal, aes(x = group)) +
  geom_col(aes(y = observed_rate), fill = "#56B4E9", alpha = 0.7) +
  geom_point(aes(y = expected_rate), colour = "#D55E00", size = 3) +
  geom_line(aes(y = expected_rate), colour = "#D55E00", linewidth = 0.5) +
  scale_x_continuous(breaks = seq_len(nrow(cal))) +
  labs(x = "Risk decile (1 = lowest)", y = "Event rate",
       caption = paste("Overall chi-sq =",
                        round(attr(cal, "overall")$chi_sq, 2),
                        ", p =",
                        format.pval(attr(cal, "overall")$p_value,
                                     digits = 3))) +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: gof-summary
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE)
gof <- hzr_gof(fit_mv)
print(gof)


## -----------------------------------------------------------------------------
#| label: fig-gof-survival
#| fig-cap: "Parametric (blue) vs. Kaplan-Meier (orange) survival"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(gof, aes(x = time)) +
  geom_step(aes(y = km_surv * 100), colour = "#D55E00", linewidth = 0.6) +
  geom_line(aes(y = par_surv * 100), colour = "#0072B2", linewidth = 0.8) +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: fig-gof-events
#| fig-cap: "Cumulative observed (orange) vs. expected (blue) events"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(gof, aes(x = time)) +
  geom_line(aes(y = cum_observed), colour = "#D55E00", linewidth = 0.8) +
  geom_line(aes(y = cum_expected), colour = "#0072B2",
            linewidth = 0.8, linetype = "dashed") +
  labs(x = "Months after AVC repair", y = "Cumulative events") +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: fig-gof-residual
#| fig-cap: "Conservation-of-events residual (expected minus observed) over time"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(gof, aes(x = time, y = residual)) +
  geom_line(linewidth = 0.7, colour = "grey30") +
  geom_hline(yintercept = 0, linetype = "dashed", colour = "grey60") +
  labs(x = "Months after AVC repair",
       y = "Residual (expected - observed)") +
  theme_minimal()

## -----------------------------------------------------------------------------
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)


## -----------------------------------------------------------------------------
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)


## -----------------------------------------------------------------------------
#| label: kul-data
data(cabgkul)
str(cabgkul)


## -----------------------------------------------------------------------------
#| label: kul-weibull
fit_kul <- hazard(
  Surv(int_dead, dead) ~ 1,
  data  = cabgkul,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0),
  fit   = TRUE
)

fit_kul


## -----------------------------------------------------------------------------
#| label: fig-kul-km
#| fig-cap: "Weibull parametric survival vs. Kaplan-Meier (CABG, KU Leuven)"
#| fig-width: 7
#| fig-height: 4.5
t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.9, length.out = 200)
nd     <- data.frame(time = t_grid)
surv   <- predict(fit_kul, newdata = nd, type = "survival") * 100

km     <- survfit(Surv(int_dead, dead) ~ 1, data = cabgkul)
km_df  <- data.frame(time = km$time, survival = km$surv * 100)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_line(data = data.frame(time = t_grid, survival = surv),
            aes(time, survival, colour = "Weibull"), linewidth = 1) +
  scale_colour_manual(values = c("Weibull" = "#0072B2",
                                 "Kaplan-Meier" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after CABG", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: avc-data
data(avc)
avc <- na.omit(avc)
str(avc)


## -----------------------------------------------------------------------------
#| label: avc-mv
fit_avc <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv + inc_surg + orifice,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 6)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_avc


## -----------------------------------------------------------------------------
#| label: avc-mp
fit_mp <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)

summary(fit_mp)


## -----------------------------------------------------------------------------
#| label: fig-avc-compare
#| fig-cap: "Single-phase Weibull vs. multiphase model against Kaplan-Meier (AVC)"
#| fig-width: 7
#| fig-height: 4.5
t_grid <- seq(0.01, max(avc$int_dead) * 0.95, length.out = 200)
nd     <- data.frame(time = t_grid)

km_avc <- survfit(Surv(int_dead, dead) ~ 1, data = avc)
km_df  <- data.frame(time = km_avc$time, survival = km_avc$surv * 100)

fit_wb <- hazard(
  Surv(int_dead, dead) ~ 1, data = avc, dist = "weibull",
  theta = c(mu = 0.20, nu = 1.0), fit = TRUE
)

surv_wb <- predict(fit_wb, newdata = nd, type = "survival") * 100
surv_mp <- predict(fit_mp, newdata = nd, type = "survival") * 100

plot_df <- rbind(
  data.frame(time = t_grid, survival = surv_wb, Model = "Single Weibull"),
  data.frame(time = t_grid, survival = surv_mp, Model = "Multiphase (2-phase)")
)

ggplot() +
  geom_step(data = km_df, aes(time, survival), colour = "grey50",
            linewidth = 0.5) +
  geom_line(data = plot_df, aes(time, survival, colour = Model),
            linewidth = 1) +
  scale_colour_manual(values = c("Single Weibull" = "#E69F00",
                                 "Multiphase (2-phase)" = "#0072B2")) +
  scale_y_continuous(limits = c(0, 100)) +
  annotate("text", x = max(t_grid) * 0.6, y = 95, label = "KM (grey)",
           size = 3, colour = "grey50") +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: valves-death
data(valves)
valves <- na.omit(valves)

fit_death <- hazard(
  Surv(int_dead, dead) ~ age_cop + nyha + mechvalv,
  data  = valves,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0, rep(0, 3)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_death


## -----------------------------------------------------------------------------
#| label: valves-pve
fit_pve <- hazard(
  Surv(int_pve, pve) ~ age_cop + nve + mechvalv,
  data  = valves,
  dist  = "weibull",
  theta = c(mu = 0.02, nu = 1.0, rep(0, 3)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_pve


## -----------------------------------------------------------------------------
#| label: interval-simulate
set.seed(101)
n         <- 400
visit_gap <- 6   # months between visits

# True event times: Weibull mu = 0.025, nu = 1.0 (median ~28 months)
true_time <- rexp(n, rate = 0.025)  # nu=1 → exponential

n_visits  <- sample(4:8, n, replace = TRUE)

status     <- integer(n)
time       <- numeric(n)
time_lower <- numeric(n)
time_upper <- numeric(n)

for (i in seq_len(n)) {
  visits <- seq(visit_gap, n_visits[i] * visit_gap, by = visit_gap)
  t_ev   <- true_time[i]
  brk    <- findInterval(t_ev, c(0, visits))  # which interval contains t_ev

  if (brk > length(visits)) {
    # Event after last visit: right-censored.
    # time_lower = 0: with H(time_lower) = H(0) = 0, the right-censored
    # contribution reduces to -(H(time) - H(0)) = -H(time), which is correct.
    status[i]     <- 0L
    time[i]       <- max(visits)
    time_lower[i] <- 0
    time_upper[i] <- max(visits)
  } else {
    # Event within the visit window: interval-censored in (lower, upper)
    lower_bound   <- if (brk == 1L) 0 else visits[brk - 1L]
    upper_bound   <- visits[brk]
    status[i]     <- 2L
    time[i]       <- lower_bound
    time_lower[i] <- lower_bound
    time_upper[i] <- upper_bound
  }
}

table(status)


## -----------------------------------------------------------------------------
#| label: interval-fit
fit_ic <- hazard(
  time       = time,
  status     = status,
  time_lower = time_lower,
  time_upper = time_upper,
  dist  = "weibull",
  theta = c(mu = 0.025, nu = 1.0),
  fit   = TRUE
)

fit_ic


## -----------------------------------------------------------------------------
#| label: interval-vs-naive
# Naive: treat each interval-censored row as an exact event at time_upper
# (the visit when the death was first recorded)
fit_naive <- hazard(
  time   = ifelse(status == 2L, time_upper, time),
  status = ifelse(status == 2L, 1L,         status),
  dist  = "weibull",
  theta = c(mu = 0.025, nu = 1.0),
  fit   = TRUE
)

# Compare MLEs; true parameters are mu = 0.025, nu = 1.0
rbind(
  interval_censored = round(coef(fit_ic)[1:2],   4),
  naive_exact_upper = round(coef(fit_naive)[1:2], 4),
  truth             = c(mu = 0.025, nu = 1.0)
)


## -----------------------------------------------------------------------------
#| label: starting-values
nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)

# Drop the zero-time boundary point before log-transforming
nel_clean <- nel[nel$time > 0 & nel$cumhaz > 0, ]
log_t <- log(nel_clean$time)
log_H <- log(nel_clean$cumhaz)

lm_fit  <- lm(log_H ~ log_t)
nu_hat  <- unname(coef(lm_fit)[2])
mu_hat  <- exp(unname(coef(lm_fit)[1]) / nu_hat)
c(mu = round(mu_hat, 4), nu = round(nu_hat, 4))


## -----------------------------------------------------------------------------
#| label: fig-cumhaz-shape
#| fig-cap: "Nelson-Aalen cumulative hazard for CABGKUL — three-phase shape visible as two kinks"
#| fig-width: 7
#| fig-height: 4
ggplot(data.frame(time = nel$time, cumhaz = nel$cumhaz),
       aes(time, cumhaz)) +
  geom_step(colour = "#D55E00", linewidth = 0.7) +
  labs(x = "Months after CABG", y = "Cumulative hazard") +
  theme_minimal()


## -----------------------------------------------------------------------------
#| label: overparameterized
set.seed(42)
fit_3ph <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 5, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  ),
  fit = TRUE, control = list(n_starts = 5, maxit = 1000)
)

# Scale magnitudes: exp(log_mu) ≈ 0 for any phase the data doesn't support
log_mu_idx <- grep("log_mu", names(coef(fit_3ph)))
round(exp(coef(fit_3ph)[log_mu_idx]), 6)


## -----------------------------------------------------------------------------
#| label: nstarts
set.seed(42)
fit_robust <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 10, maxit = 1000)
)
fit_robust$fit$converged

## -----------------------------------------------------------------------------
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)


## -----------------------------------------------------------------------------
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)


## -----------------------------------------------------------------------------
#| label: kul-data
data(cabgkul)
str(cabgkul)


## -----------------------------------------------------------------------------
#| label: kul-weibull
fit_kul <- hazard(
  Surv(int_dead, dead) ~ 1,
  data  = cabgkul,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0),
  fit   = TRUE
)

fit_kul


## -----------------------------------------------------------------------------
#| label: fig-kul-km
#| fig-cap: "Weibull parametric survival vs. Kaplan-Meier (CABG, KU Leuven)"
#| fig-width: 7
#| fig-height: 4.5
t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.9, length.out = 200)
nd     <- data.frame(time = t_grid)
surv   <- predict(fit_kul, newdata = nd, type = "survival") * 100

km     <- survfit(Surv(int_dead, dead) ~ 1, data = cabgkul)
km_df  <- data.frame(time = km$time, survival = km$surv * 100)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_line(data = data.frame(time = t_grid, survival = surv),
            aes(time, survival, colour = "Weibull"), linewidth = 1) +
  scale_colour_manual(values = c("Weibull" = "#0072B2",
                                 "Kaplan-Meier" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after CABG", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: avc-data
data(avc)
avc <- na.omit(avc)
str(avc)


## -----------------------------------------------------------------------------
#| label: avc-mv
fit_avc <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv + inc_surg + orifice,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 6)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_avc


## -----------------------------------------------------------------------------
#| label: avc-mp
fit_mp <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)

summary(fit_mp)


## -----------------------------------------------------------------------------
#| label: fig-avc-compare
#| fig-cap: "Single-phase Weibull vs. multiphase model against Kaplan-Meier (AVC)"
#| fig-width: 7
#| fig-height: 4.5
t_grid <- seq(0.01, max(avc$int_dead) * 0.95, length.out = 200)
nd     <- data.frame(time = t_grid)

km_avc <- survfit(Surv(int_dead, dead) ~ 1, data = avc)
km_df  <- data.frame(time = km_avc$time, survival = km_avc$surv * 100)

fit_wb <- hazard(
  Surv(int_dead, dead) ~ 1, data = avc, dist = "weibull",
  theta = c(mu = 0.20, nu = 1.0), fit = TRUE
)

surv_wb <- predict(fit_wb, newdata = nd, type = "survival") * 100
surv_mp <- predict(fit_mp, newdata = nd, type = "survival") * 100

plot_df <- rbind(
  data.frame(time = t_grid, survival = surv_wb, Model = "Single Weibull"),
  data.frame(time = t_grid, survival = surv_mp, Model = "Multiphase (2-phase)")
)

ggplot() +
  geom_step(data = km_df, aes(time, survival), colour = "grey50",
            linewidth = 0.5) +
  geom_line(data = plot_df, aes(time, survival, colour = Model),
            linewidth = 1) +
  scale_colour_manual(values = c("Single Weibull" = "#E69F00",
                                 "Multiphase (2-phase)" = "#0072B2")) +
  scale_y_continuous(limits = c(0, 100)) +
  annotate("text", x = max(t_grid) * 0.6, y = 95, label = "KM (grey)",
           size = 3, colour = "grey50") +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")


## -----------------------------------------------------------------------------
#| label: valves-death
data(valves)
valves <- na.omit(valves)

fit_death <- hazard(
  Surv(int_dead, dead) ~ age_cop + nyha + mechvalv,
  data  = valves,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0, rep(0, 3)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_death


## -----------------------------------------------------------------------------
#| label: valves-pve
fit_pve <- hazard(
  Surv(int_pve, pve) ~ age_cop + nve + mechvalv,
  data  = valves,
  dist  = "weibull",
  theta = c(mu = 0.02, nu = 1.0, rep(0, 3)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_pve

#' @keywords internal
NULL

# predict-cl.R -- Delta-method confidence limits for predict.hazard()
#
# PROBLEM
# -------
# predict.hazard() returns a point estimate eta(theta) for each row.  We want
# a standard error and a (1 - alpha) confidence interval around each one,
# matching SAS HAZARD's `hzp_calc_haz_CL.c` / `hzp_calc_srv_CL.c`.
#
# METHOD
# ------
# Delta method: if theta_hat ~ N(theta, V) then any smooth function eta(theta)
# is approximately Gaussian with
#
#   var(eta_hat)  ~  (d eta/d theta)^T  V  (d eta/d theta).
#
# For a batch of n prediction points, we assemble J (n x p) where
# J[i, ] = d eta_i / d theta, then
#
#   se_i  =  sqrt( (J V J^T)_{ii} )  =  sqrt( rowSums( (J V) * J ) ).
#
# SCALE CHOICE
# ------------
# Symmetric CLs on the natural scale can leave a [0, 1] survival or a
# [0, Inf) hazard, so we build them on a transformed scale and back-transform:
#
#   hazard / cumhaz   -> log scale:
#     se(log eta)  =  se(eta) / eta,
#     [eta * exp(-z se_log),  eta * exp(+z se_log)]
#
#   survival          -> log-log scale (log(-log S) = log H):
#     se(log(-log S))  =  se(H) / H,
#     S_lwr  =  exp(-H * exp(+z se_logH)),
#     S_upr  =  exp(-H * exp(-z se_logH))
#
#   linear_predictor  -> natural scale (symmetric):
#     [eta - z se, eta + z se]
#
# ENTRY POINT
# -----------
# `.hzr_predict_with_se()` takes a fitted `hazard` object, a type, and the
# materialised `x` / `time` for prediction.  It dispatches to the appropriate
# distribution-specific jacobian builder, runs the delta-method sandwich, and
# returns a list(fit, se.fit, lower, upper).

# ---------------------------------------------------------------------------
# Jacobian: Weibull
# ---------------------------------------------------------------------------
#
# Parameter layout (natural scale, matching .hzr_logl_weibull):
#   theta = [mu, nu, beta_1, ..., beta_p]
#
# H(t|x) = (mu t)^nu exp(eta),   h_rel = exp(eta),   eta = x beta.
#
# Analytical derivatives used below:
#   dH/dmu       = (nu / mu) H
#   dH/dnu       = log(mu t) H
#   dH/dbeta_j   = x_ij H
#
#   d exp(eta)/dbeta_j = x_ij exp(eta)    (other entries are 0)
#
#   eta itself is linear: d eta / d beta_j = x_ij, rest 0.

#' Jacobian of Weibull predictions with respect to theta
#'
#' @param type Prediction type (see `.hzr_predict_with_se`).
#' @param theta MLE parameter vector `c(mu, nu, beta_1, ...)`.
#' @param time Prediction times (may be NULL for `hazard` / `linear_predictor`).
#' @param x Design matrix (n x p_cov) or NULL.
#' @param p Length of theta.
#' @return Numeric n x p Jacobian.
#' @keywords internal
.hzr_predict_jacobian_weibull <- function(type, theta, time, x, p) {
  n_shape <- 2L
  mu <- theta[1]
  nu <- theta[2]
  beta <- if (p > n_shape) theta[(n_shape + 1L):p] else numeric(0)

  if (!is.null(x) && length(beta) > 0L) {
    eta <- as.numeric(x %*% beta)
  } else if (!is.null(x)) {
    eta <- rep(0, nrow(x))
  } else {
    # No covariates: treat all rows as a single reference
    eta <- if (!is.null(time)) rep(0, length(time)) else 0
  }
  n <- length(eta)
  J <- matrix(0, nrow = n, ncol = p)

  if (type %in% c("cumulative_hazard", "survival")) {
    H <- (mu * time) ^ nu * exp(eta)
    J[, 1L] <- (nu / mu) * H
    J[, 2L] <- log(mu * time) * H
    if (length(beta) > 0L && !is.null(x)) {
      J[, (n_shape + 1L):p] <- x * H
    }
  } else if (type == "hazard") {
    # Current API returns relative hazard = exp(eta); shape params drop out.
    h_rel <- exp(eta)
    if (length(beta) > 0L && !is.null(x)) {
      J[, (n_shape + 1L):p] <- x * h_rel
    }
  } else if (type == "linear_predictor") {
    if (length(beta) > 0L && !is.null(x)) {
      J[, (n_shape + 1L):p] <- x
    }
  }

  J
}

# ---------------------------------------------------------------------------
# Jacobian: multiphase
# ---------------------------------------------------------------------------
#
# H(t|x) = sum_j mu_j(x) Phi_j(t),   mu_j(x) = exp(log_mu_j + x_j beta_j).
#
# For each phase j, its sub-vector in theta contains (in order):
#   log_mu_j, (log_t_half_j, nu_j, m_j) or (log_tau_j, gamma_j, alpha_j, eta_j)
#   or () for a "constant" phase, then beta_j covariates.
#
# Derivatives:
#   dH/d log_mu_j    = mu_j Phi_j
#   dH/d shape_s_j   = mu_j dPhi_j/d shape_s     (from .hzr_phase_derivatives
#                                                 / .hzr_g3_phase_derivatives)
#   dH/d beta_jk     = x_ik mu_j Phi_j

#' Jacobian of multiphase cumulative-hazard predictions
#'
#' Only `cumulative_hazard` and `survival` are delegated here -- `hazard`
#' and `linear_predictor` are rejected upstream for multiphase models.
#'
#' @param theta MLE parameter vector.
#' @param time Prediction times (length n).
#' @param phases Named list of `hzr_phase` objects.
#' @param covariate_counts Named integer vector.
#' @param x_list Named list of per-phase design matrices.
#' @param p Length of theta.
#' @return Numeric n x p Jacobian of H(t|x).
#' @keywords internal
.hzr_predict_jacobian_multiphase <- function(theta, time, phases,
                                              covariate_counts, x_list, p) {
  n <- length(time)
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  J <- matrix(0, nrow = n, ncol = p)

  pos <- 1L
  for (nm in names(phases)) {
    ph <- phases[[nm]]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], ph)

    # mu_j(x_i)
    if (length(pars$beta) > 0L && !is.null(x_list[[nm]])) {
      eta_j <- pars$log_mu + as.numeric(x_list[[nm]] %*% pars$beta)
    } else {
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)

    # Phi_j and shape derivatives
    if (ph$type == "constant") {
      Phi_j <- time
      # log_mu only
      J[, pos] <- mu_j * Phi_j
      pos <- pos + 1L
    } else if (ph$type == "g3") {
      tau_j <- exp(pars$log_tau)
      pd <- .hzr_g3_phase_derivatives(time, tau = tau_j,
                                        gamma = pars$gamma,
                                        alpha = pars$alpha,
                                        eta = pars$eta)
      Phi_j <- pd$Phi
      J[, pos] <- mu_j * Phi_j  # log_mu
      J[, pos + 1L] <- mu_j * pd$dPhi_dlog_tau
      J[, pos + 2L] <- mu_j * pd$dPhi_dgamma
      J[, pos + 3L] <- mu_j * pd$dPhi_dalpha
      J[, pos + 4L] <- mu_j * pd$dPhi_deta
      pos <- pos + 5L
    } else {
      t_half_j <- exp(pars$log_t_half)
      pd <- .hzr_phase_derivatives(time, t_half = t_half_j,
                                    nu = pars$nu, m = pars$m,
                                    type = ph$type)
      Phi_j <- pd$Phi
      J[, pos] <- mu_j * Phi_j  # log_mu
      J[, pos + 1L] <- mu_j * pd$dPhi_dlog_thalf
      J[, pos + 2L] <- mu_j * pd$dPhi_dnu
      J[, pos + 3L] <- mu_j * pd$dPhi_dm
      pos <- pos + 4L
    }

    # Covariate betas
    n_beta <- covariate_counts[[nm]]
    if (n_beta > 0L && !is.null(x_list[[nm]])) {
      x_phase <- x_list[[nm]]
      for (k in seq_len(n_beta)) {
        J[, pos] <- x_phase[, k] * mu_j * Phi_j
        pos <- pos + 1L
      }
    } else if (n_beta > 0L) {
      pos <- pos + n_beta
    }
  }

  J
}

# ---------------------------------------------------------------------------
# Numeric jacobian fallback for exp / loglogistic / lognormal
# ---------------------------------------------------------------------------
#
# Rather than hand-code each distribution's closed form, we differentiate the
# prediction function numerically through the existing single-distribution
# cumhaz/survival formulas in predict.hazard().  The prediction function is
# built once per call and returns a length-n vector for any candidate theta.

#' Numeric Jacobian of a single-distribution prediction w.r.t. theta
#'
#' @param predict_fn Function(theta) -> numeric vector of length n.
#' @param theta MLE parameter vector.
#' @return Numeric n x p Jacobian.
#' @keywords internal
.hzr_predict_jacobian_numeric <- function(predict_fn, theta) {
  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(numDeriv::jacobian(predict_fn, theta))
  }
  eps <- (.Machine$double.eps) ^ (1 / 3)
  p <- length(theta)
  fit <- predict_fn(theta)
  n <- length(fit)
  J <- matrix(0, nrow = n, ncol = p)
  for (j in seq_len(p)) {
    h <- eps * max(abs(theta[j]), 1)
    tp <- theta
    tm <- theta
    tp[j] <- tp[j] + h
    tm[j] <- tm[j] - h
    J[, j] <- (predict_fn(tp) - predict_fn(tm)) / (2 * h)
  }
  J
}

# ---------------------------------------------------------------------------
# Delta-method sandwich + scale transform
# ---------------------------------------------------------------------------

#' Per-row standard errors via the delta method
#'
#' @param J Jacobian (n x p).
#' @param vcov p x p variance-covariance matrix.
#' @return Numeric vector of length n.
#' @keywords internal
.hzr_predict_se_from_jacobian <- function(J, vcov) {
  JV <- J %*% vcov
  se <- sqrt(pmax(rowSums(JV * J), 0))
  se
}

#' Apply a scale transform + back-transform for CLs
#'
#' @param fit Point estimate (numeric vector length n).
#' @param se_nat SE on the natural scale (numeric vector length n).
#' @param level Confidence level.
#' @param scale One of "natural", "log", "loglog_survival".
#' @return data.frame with columns fit, se.fit, lower, upper.
#' @keywords internal
.hzr_predict_cl_from_se <- function(fit, se_nat, level, scale) {
  alpha <- 1 - level
  z <- stats::qnorm(1 - alpha / 2)

  lower <- rep(NA_real_, length(fit))
  upper <- rep(NA_real_, length(fit))

  if (scale == "natural") {
    lower <- fit - z * se_nat
    upper <- fit + z * se_nat
  } else if (scale == "log") {
    # eta > 0 required; any non-positive fit gets NA CLs.
    pos <- is.finite(fit) & fit > 0
    se_log <- ifelse(pos, se_nat / abs(fit), NA_real_)
    lower[pos] <- fit[pos] * exp(-z * se_log[pos])
    upper[pos] <- fit[pos] * exp(z * se_log[pos])
  } else if (scale == "loglog_survival") {
    # fit is S = exp(-H); se_nat is the SE of H (not S).  Build CLs on
    # log(-log S) = log H.
    #   S_lwr = exp(-H * exp(+z * se(log H)))
    #   S_upr = exp(-H * exp(-z * se(log H)))
    # The caller passes `fit = S` and `se_nat = se(H)`; we reconstruct
    # H = -log(S) to compute se(log H) = se(H) / H.
    H <- -log(pmin(pmax(fit, .Machine$double.xmin), 1))
    pos <- is.finite(H) & H > 0
    se_logH <- ifelse(pos, se_nat / H, NA_real_)
    lower[pos] <- exp(-H[pos] * exp(z * se_logH[pos]))
    upper[pos] <- exp(-H[pos] * exp(-z * se_logH[pos]))
  } else {
    stop("Unknown CL scale: '", scale, "'.", call. = FALSE)
  }

  data.frame(fit = fit, se.fit = se_nat, lower = lower, upper = upper)
}

# ---------------------------------------------------------------------------
# Public entry point
# ---------------------------------------------------------------------------

#' Compute point predictions + delta-method CLs
#'
#' Dispatched from predict.hazard() when `se.fit = TRUE`.
#'
#' DELTA-METHOD TARGET
#' -------------------
#' The quantity we actually differentiate depends on the prediction type and
#' the CL scale the type uses:
#'
#'   type = "cumulative_hazard" -> target = H, log-scale CL
#'   type = "survival"          -> target = H, log-log-survival CL
#'                                 (final `fit` is exp(-H))
#'   type = "hazard"            -> target = exp(eta) (single-dist only),
#'                                 log-scale CL
#'   type = "linear_predictor"  -> target = eta, natural-scale CL
#'
#' The caller supplies `diff_fn(theta)` that returns the target vector of
#' length n.  For Weibull and multiphase we build J analytically; for
#' exp / loglogistic / lognormal we fall through to a numeric jacobian of
#' `diff_fn`.
#'
#' @param object Fitted `hazard` object.
#' @param type One of "hazard", "linear_predictor", "survival",
#'   "cumulative_hazard".
#' @param time Numeric prediction times or NULL (for type = "hazard" /
#'   "linear_predictor").
#' @param x Design matrix (single-distribution) or NULL.  Ignored for
#'   multiphase; use `x_list` instead.
#' @param x_list Named list of per-phase design matrices (multiphase only).
#' @param cov_counts Named integer covariate counts (multiphase only).
#' @param phases Named list of `hzr_phase` objects (multiphase only).
#' @param level Confidence level (default 0.95).
#' @param diff_fn Required function(theta) -> numeric vector of length n
#'   returning the delta-method target (H, exp(eta), or eta depending on
#'   `type`).  Used for the point estimate AND for the numeric jacobian
#'   fallback in exp / loglogistic / lognormal.
#' @return data.frame with columns `fit`, `se.fit`, `lower`, `upper`.
#' @keywords internal
.hzr_predict_with_se <- function(object, type, time = NULL,
                                   x = NULL, x_list = NULL,
                                   cov_counts = NULL, phases = NULL,
                                   level = 0.95, diff_fn) {

  theta <- object$fit$theta
  p <- length(theta)
  dist <- object$spec$dist
  target <- diff_fn(theta)

  vcov_mat <- object$fit$vcov
  vcov_ok <- !is.null(vcov_mat) && is.matrix(vcov_mat) &&
               nrow(vcov_mat) == p && ncol(vcov_mat) == p
  if (!vcov_ok) {
    warning("Variance-covariance matrix is unavailable; ",
            "standard errors and CLs will be NA.", call. = FALSE)
    n <- length(target)
    fit <- if (type == "survival") exp(-target) else target
    na_vec <- rep(NA_real_, n)
    return(data.frame(fit = fit, se.fit = na_vec,
                      lower = na_vec, upper = na_vec))
  }

  # Fixed parameters (from `fixed = "shapes"` or CoE) leave NA rows/cols in
  # the expanded vcov.  Treat them as known-with-zero-variance: restrict the
  # delta-method sandwich to the free submatrix of vcov and the
  # corresponding columns of J.
  diag_vcov <- diag(vcov_mat)
  free_idx <- which(is.finite(diag_vcov))
  if (length(free_idx) < p) {
    # At least one parameter is fixed -- check that the off-diagonals among
    # free params are finite; if they aren't, something is wrong and we
    # fall back to NA CLs.
    free_submat <- vcov_mat[free_idx, free_idx, drop = FALSE]
    if (anyNA(free_submat)) {
      warning("Variance-covariance matrix has NA entries outside the ",
              "fixed-parameter rows/cols; standard errors and CLs will be NA.",
              call. = FALSE)
      n <- length(target)
      fit <- if (type == "survival") exp(-target) else target
      na_vec <- rep(NA_real_, n)
      return(data.frame(fit = fit, se.fit = na_vec,
                        lower = na_vec, upper = na_vec))
    }
    vcov_use <- free_submat
  } else if (anyNA(vcov_mat)) {
    warning("Variance-covariance matrix has NA entries; ",
            "standard errors and CLs will be NA.", call. = FALSE)
    n <- length(target)
    fit <- if (type == "survival") exp(-target) else target
    na_vec <- rep(NA_real_, n)
    return(data.frame(fit = fit, se.fit = na_vec,
                      lower = na_vec, upper = na_vec))
  } else {
    vcov_use <- vcov_mat
  }

  # --- Build Jacobian of the delta-method target ---------------------------
  if (dist == "weibull") {
    J <- .hzr_predict_jacobian_weibull(type, theta, time, x, p)
  } else if (dist == "multiphase") {
    # hazard / linear_predictor are rejected upstream for multiphase.
    J <- .hzr_predict_jacobian_multiphase(theta, time, phases,
                                            cov_counts, x_list, p)
  } else {
    J <- .hzr_predict_jacobian_numeric(diff_fn, theta)
  }

  # Restrict J to the free columns so the sandwich dimensions match.
  J <- J[, free_idx, drop = FALSE]

  # --- Sandwich -------------------------------------------------------------
  se_target <- .hzr_predict_se_from_jacobian(J, vcov_use)

  # --- Scale transform -----------------------------------------------------
  if (type == "survival") {
    fit <- exp(-target)
    .hzr_predict_cl_from_se(fit, se_target, level, "loglog_survival")
  } else if (type == "cumulative_hazard" || type == "hazard") {
    .hzr_predict_cl_from_se(target, se_target, level, "log")
  } else if (type == "linear_predictor") {
    .hzr_predict_cl_from_se(target, se_target, level, "natural")
  } else {
    stop("Unknown prediction type '", type, "' for se.fit.", call. = FALSE)
  }
}

# tests/testthat/test-predict-cl.R
# Phase 4g -- predict.hazard(..., se.fit = TRUE, level = 0.95)
#
# Coverage:
# (1) API contract: output shape, level validation, decompose incompat
# (2) Jacobian parity: analytic vs numeric for Weibull + multiphase
# (3) Delta-method scale: log-CL for hazard/cumhaz, log-log for survival
# (4) Monotonicity: lower <= fit <= upper
# (5) Survival CLs stay in [0, 1]; cumhaz CLs stay in [0, Inf)
# (6) `vcov` NA (non-converged fit): returns NA CLs with a warning
# (7) Fixed shapes / CoE: free-parameter submatrix recovers meaningful CLs
# (8) Default of `se.fit = FALSE` is byte-identical to pre-0.9.8 predict()

make_toy <- function(seed = 17, n = 60) {
  set.seed(seed)
  data.frame(
    time = rexp(n, 0.3),
    status = rbinom(n, 1, 0.6),
    x = rnorm(n)
  )
}

# ---------------------------------------------------------------------------
# (1) API contract
# ---------------------------------------------------------------------------

test_that("se.fit returns a data frame with fit/se.fit/lower/upper", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2), x = 0),
                 type = "survival", se.fit = TRUE)
  expect_s3_class(res, "data.frame")
  expect_named(res, c("fit", "se.fit", "lower", "upper"))
  expect_equal(nrow(res), 3L)
  expect_true(all(is.finite(res$fit)))
  expect_true(all(is.finite(res$se.fit)))
  expect_true(all(res$se.fit >= 0))
})

test_that("invalid level triggers a clean error", {
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  expect_error(
    predict(fit, newdata = data.frame(time = 1, x = 0),
            type = "survival", se.fit = TRUE, level = 1.1),
    "level"
  )
  expect_error(
    predict(fit, newdata = data.frame(time = 1, x = 0),
            type = "survival", se.fit = TRUE, level = 0),
    "level"
  )
})

test_that("se.fit = TRUE with decompose = TRUE errors clearly", {
  df <- make_toy()
  phases <- list(early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                                     fixed = "shapes"),
                 constant = hzr_phase("constant"))
  fit <- hazard(survival::Surv(time, status) ~ 1, data = df,
                dist = "multiphase", phases = phases, fit = TRUE,
                control = list(n_starts = 2))
  expect_error(
    predict(fit, newdata = data.frame(time = c(0.5, 1)),
            type = "cumulative_hazard", se.fit = TRUE, decompose = TRUE),
    "decompose"
  )
})

# ---------------------------------------------------------------------------
# (2) Jacobian parity: analytic vs numeric
# ---------------------------------------------------------------------------

test_that("Weibull analytic jacobian matches numDeriv (cumhaz)", {
  skip_if_not_installed("numDeriv")
  set.seed(3)
  t_new <- c(0.3, 0.8, 1.5, 2.2)
  x_new <- matrix(c(-0.5, 0, 0.4, 1.2), ncol = 1)
  theta <- c(mu = 0.6, nu = 1.2, beta = 0.25)

  cumhaz_fn <- function(th) {
    (th[1] * t_new) ^ th[2] * exp(as.numeric(x_new %*% th[3:length(th)]))
  }
  J_ana <- TemporalHazard:::.hzr_predict_jacobian_weibull(
    "cumulative_hazard", theta, t_new, x_new, length(theta)
  )
  J_num <- numDeriv::jacobian(cumhaz_fn, theta)
  expect_equal(J_ana, J_num, tolerance = 1e-6)
})

test_that("Weibull analytic jacobian matches numDeriv (hazard / relative)", {
  skip_if_not_installed("numDeriv")
  x_new <- matrix(c(-0.5, 0, 0.4), ncol = 1)
  theta <- c(mu = 0.6, nu = 1.2, beta = 0.25)
  haz_fn <- function(th) exp(as.numeric(x_new %*% th[3:length(th)]))
  J_ana <- TemporalHazard:::.hzr_predict_jacobian_weibull(
    "hazard", theta, time = NULL, x_new, length(theta)
  )
  J_num <- numDeriv::jacobian(haz_fn, theta)
  expect_equal(J_ana, J_num, tolerance = 1e-6)
})

test_that("Multiphase analytic jacobian matches numDeriv (cumhaz)", {
  skip_if_not_installed("numDeriv")
  skip_on_cran()
  t_new <- c(0.3, 0.8, 1.5, 2.2)
  phases <- list(
    early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  cov_counts <- c(early = 0L, constant = 0L)
  x_list <- list(early = NULL, constant = NULL)
  theta <- c(-3, log(0.3), 1, 1, -2)

  J_ana <- TemporalHazard:::.hzr_predict_jacobian_multiphase(
    theta, t_new, phases, cov_counts, x_list, length(theta)
  )
  J_num <- numDeriv::jacobian(
    function(th) {
      TemporalHazard:::.hzr_multiphase_cumhaz(
        t_new, th, phases, cov_counts, x_list
      )
    },
    theta
  )
  expect_equal(J_ana, J_num, tolerance = 1e-6)
})

# ---------------------------------------------------------------------------
# (3, 4, 5) Scale, monotonicity, and range
# ---------------------------------------------------------------------------

test_that("survival CLs stay in [0, 1] and straddle the point estimate", {
  skip_on_cran()
  df <- make_toy()
  for (dist in c("weibull", "exponential", "loglogistic", "lognormal")) {
    theta_init <- switch(
      dist,
      weibull = c(0.5, 1, 0),
      exponential = c(log(0.3), 0),
      loglogistic = c(0, 0, 0),
      lognormal = c(0, 0, 0)
    )
    fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = dist,
                  theta = theta_init, fit = TRUE)
    res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2), x = 0),
                   type = "survival", se.fit = TRUE)
    expect_true(all(res$lower >= 0), info = dist)
    expect_true(all(res$upper <= 1), info = dist)
    expect_true(all(res$lower <= res$fit + 1e-8), info = dist)
    expect_true(all(res$fit <= res$upper + 1e-8), info = dist)
  }
})

test_that("cumulative_hazard CLs are strictly positive and log-symmetric", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2), x = 0),
                 type = "cumulative_hazard", se.fit = TRUE)
  expect_true(all(res$lower > 0))
  expect_true(all(res$upper > res$lower))
  # log-symmetric: log(upper) - log(fit) ~= log(fit) - log(lower)
  log_halfwidth_upper <- log(res$upper) - log(res$fit)
  log_halfwidth_lower <- log(res$fit) - log(res$lower)
  expect_equal(log_halfwidth_upper, log_halfwidth_lower, tolerance = 1e-6)
})

test_that("linear_predictor CLs are symmetric on the natural scale", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  res <- predict(fit, newdata = data.frame(x = c(-1, 0, 1)),
                 type = "linear_predictor", se.fit = TRUE)
  halfwidth_upper <- res$upper - res$fit
  halfwidth_lower <- res$fit - res$lower
  expect_equal(halfwidth_upper, halfwidth_lower, tolerance = 1e-10)
})

# ---------------------------------------------------------------------------
# (6) Missing vcov -> NA CLs with a warning
# ---------------------------------------------------------------------------

test_that("missing vcov produces NA CLs with a warning", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  # Knock out vcov to simulate a non-converged / ill-conditioned fit
  fit$fit$vcov <- NULL
  expect_warning(
    res <- predict(fit, newdata = data.frame(time = 1, x = 0),
                   type = "survival", se.fit = TRUE),
    "Variance-covariance"
  )
  expect_true(all(is.na(res$se.fit)))
  expect_true(all(is.na(res$lower)))
  expect_true(all(is.na(res$upper)))
  expect_false(anyNA(res$fit))
})

# ---------------------------------------------------------------------------
# (7) Fixed-shapes multiphase recovers CLs from the free-parameter submatrix
# ---------------------------------------------------------------------------

test_that("multiphase CLs work with fixed shapes + CoE (free submatrix)", {
  skip_on_cran()
  df <- make_toy()
  phases <- list(
    early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  fit <- hazard(survival::Surv(time, status) ~ 1, data = df,
                dist = "multiphase", phases = phases, fit = TRUE,
                control = list(n_starts = 2))
  res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2)),
                 type = "survival", se.fit = TRUE)
  expect_true(all(is.finite(res$se.fit)))
  expect_true(all(res$se.fit > 0))
  expect_true(all(res$lower >= 0))
  expect_true(all(res$upper <= 1))
})

# ---------------------------------------------------------------------------
# (8) Backward compat: se.fit = FALSE reproduces the old scalar-vector return
# ---------------------------------------------------------------------------

test_that("se.fit = FALSE preserves pre-0.9.8 return shape", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)

  # Each type should return a bare numeric vector with se.fit = FALSE.
  for (type in c("survival", "cumulative_hazard", "hazard",
                 "linear_predictor")) {
    nd <- if (type %in% c("survival", "cumulative_hazard")) {
      data.frame(time = c(0.5, 1, 2), x = 0)
    } else {
      data.frame(x = c(-1, 0, 1))
    }
    out <- predict(fit, newdata = nd, type = type, se.fit = FALSE)
    expect_type(out, "double")
    expect_length(out, 3L)
    expect_false(is.data.frame(out))
  }
})

#' @keywords internal
NULL

# hessian-multiphase.R -- Analytic NLL Hessian for multiphase hazard models.
#
# Three helpers then the main function:
#   .hzr_phase_second_derivatives()     -- d2Phi/dss' and d2phi/dss' for CDF/hazard
#   .hzr_g3_phase_second_derivatives()  -- same for G3 (late) phases
#   .hzr_hessian_multiphase()           -- full p x p NLL Hessian

# Step size for 4-point second-difference formulas: h ~ eps^(1/4).
# Optimal for central second differences.
.hzr_h2 <- .Machine$double.eps^(1 / 4)

# ---------------------------------------------------------------------------
# CDF / hazard phase second derivatives
# ---------------------------------------------------------------------------

#' Second derivatives of CDF/hazard phase shape functions
#'
#' Computes the six unique mixed second partial derivatives of
#' \eqn{\Phi_j(t)} (cumulative shape) and \eqn{\phi_j(t)} (instantaneous
#' shape) with respect to the internal shape parameters
#' \eqn{\{log\_t\_half, \nu, m\}} via second-order central differences.
#'
#' This is the second-derivative analogue of \code{.hzr_phase_derivatives()}.
#'
#' @param time  Numeric vector of observation times.
#' @param t_half,nu,m  Phase shape parameters (natural scale).
#' @param type  Phase type: \code{"cdf"} or \code{"hazard"}.
#' @return A list with elements \code{d2Phi_dlog_thalf2},
#'   \code{d2Phi_dnu2}, \code{d2Phi_dm2},
#'   \code{d2Phi_dlog_thalf_dnu}, \code{d2Phi_dlog_thalf_dm},
#'   \code{d2Phi_dnu_dm}, and the matching \code{d2phi_*} entries.
#' @noRd
.hzr_phase_second_derivatives <- function(time, t_half, nu, m, type) {
  h <- .hzr_h2

  # Evaluate Phi and phi at given (t_half_p, nu_p, m_p)
  eval_at <- function(th, nup, mp) {
    d <- .hzr_phase_derivatives(time, t_half = th, nu = nup, m = mp, type = type)
    list(Phi = d$Phi, phi = d$phi)
  }

  d00 <- eval_at(t_half, nu, m)

  # Perturbed values on each internal scale:
  #   log_t_half: steps are multiplicative -> t_half * exp(+/-h)
  #   nu, m:      additive steps
  t_p  <- t_half * exp(h)
  t_m  <- t_half * exp(-h)
  nu_p <- nu + h
  nu_m <- nu - h
  m_p  <- m  + h
  m_m  <- m  - h

  # Six single-parameter perturbations for diagonal second differences
  d_tp  <- eval_at(t_p,    nu,   m)
  d_tm  <- eval_at(t_m,    nu,   m)
  d_np  <- eval_at(t_half, nu_p, m)
  d_nm  <- eval_at(t_half, nu_m, m)
  d_mp  <- eval_at(t_half, nu,   m_p)
  d_mm  <- eval_at(t_half, nu,   m_m)

  h2 <- h * h

  # Diagonal second differences
  d2Phi_tt  <- (d_tp$Phi - 2 * d00$Phi + d_tm$Phi) / h2
  d2Phi_nn  <- (d_np$Phi - 2 * d00$Phi + d_nm$Phi) / h2
  d2Phi_mm_ <- (d_mp$Phi - 2 * d00$Phi + d_mm$Phi) / h2
  d2phi_tt  <- (d_tp$phi - 2 * d00$phi + d_tm$phi) / h2
  d2phi_nn  <- (d_np$phi - 2 * d00$phi + d_nm$phi) / h2
  d2phi_mm_ <- (d_mp$phi - 2 * d00$phi + d_mm$phi) / h2

  # t_half / nu cross: corners (t_p, nu_p), (t_p, nu_m), (t_m, nu_p), (t_m, nu_m)
  e_tn_pp <- eval_at(t_p, nu_p, m)
  e_tn_pm <- eval_at(t_p, nu_m, m)
  e_tn_mp <- eval_at(t_m, nu_p, m)
  e_tn_mm <- eval_at(t_m, nu_m, m)
  d2_tn_Phi <- (e_tn_pp$Phi - e_tn_pm$Phi - e_tn_mp$Phi + e_tn_mm$Phi) / (4 * h2)
  d2_tn_phi <- (e_tn_pp$phi - e_tn_pm$phi - e_tn_mp$phi + e_tn_mm$phi) / (4 * h2)

  # t_half / m cross
  e_tm_pp <- eval_at(t_p, nu, m_p)
  e_tm_pm <- eval_at(t_p, nu, m_m)
  e_tm_mp <- eval_at(t_m, nu, m_p)
  e_tm_mm <- eval_at(t_m, nu, m_m)
  d2_tm_Phi <- (e_tm_pp$Phi - e_tm_pm$Phi - e_tm_mp$Phi + e_tm_mm$Phi) / (4 * h2)
  d2_tm_phi <- (e_tm_pp$phi - e_tm_pm$phi - e_tm_mp$phi + e_tm_mm$phi) / (4 * h2)

  # nu / m cross
  e_nm_pp <- eval_at(t_half, nu_p, m_p)
  e_nm_pm <- eval_at(t_half, nu_p, m_m)
  e_nm_mp <- eval_at(t_half, nu_m, m_p)
  e_nm_mm <- eval_at(t_half, nu_m, m_m)
  d2_nm_Phi <- (e_nm_pp$Phi - e_nm_pm$Phi - e_nm_mp$Phi + e_nm_mm$Phi) / (4 * h2)
  d2_nm_phi <- (e_nm_pp$phi - e_nm_pm$phi - e_nm_mp$phi + e_nm_mm$phi) / (4 * h2)

  list(
    d2Phi_dlog_thalf2    = d2Phi_tt,
    d2Phi_dnu2           = d2Phi_nn,
    d2Phi_dm2            = d2Phi_mm_,
    d2Phi_dlog_thalf_dnu = d2_tn_Phi,
    d2Phi_dlog_thalf_dm  = d2_tm_Phi,
    d2Phi_dnu_dm         = d2_nm_Phi,

    d2phi_dlog_thalf2    = d2phi_tt,
    d2phi_dnu2           = d2phi_nn,
    d2phi_dm2            = d2phi_mm_,
    d2phi_dlog_thalf_dnu = d2_tn_phi,
    d2phi_dlog_thalf_dm  = d2_tm_phi,
    d2phi_dnu_dm         = d2_nm_phi
  )
}

# ---------------------------------------------------------------------------
# G3 (late) phase second derivatives
# ---------------------------------------------------------------------------

#' Second derivatives of G3 phase shape functions
#'
#' Analogue of \code{.hzr_g3_phase_derivatives()} for second-order
#' shape derivatives over \code{\{log\_tau, gamma, alpha, eta\}}.
#'
#' @param time,tau,gamma,alpha,eta  G3 phase parameters.
#' @return List with \code{d2Phi_*} and \code{d2phi_*} entries for the ten
#'   unique pairs from \code{\{log\_tau, gamma, alpha, eta\}}.
#' @noRd
.hzr_g3_phase_second_derivatives <- function(time, tau, gamma, alpha, eta) {
  h <- .hzr_h2

  eval_at <- function(tau_p, gam_p, alp_p, eta_p) {
    # Guard against non-positive G3 parameters
    if (tau_p <= 0 || gam_p <= 0 || eta_p <= 0) return(NULL)
    tryCatch(
      {
        d <- hzr_decompos_g3(time, tau = tau_p, gamma = gam_p,
                              alpha = alp_p, eta = eta_p)
        list(Phi = d$G3, phi = d$g3)
      },
      error = function(e) NULL
    )
  }

  d00 <- eval_at(tau, gamma, alpha, eta)
  if (is.null(d00)) {
    n <- length(time)
    zeros <- rep(0, n)
    nms <- c("d2Phi_dlog_tau2", "d2Phi_dgamma2", "d2Phi_dalpha2",
             "d2Phi_deta2",
             "d2Phi_dlog_tau_dgamma", "d2Phi_dlog_tau_dalpha",
             "d2Phi_dlog_tau_deta",   "d2Phi_dgamma_dalpha",
             "d2Phi_dgamma_deta",     "d2Phi_dalpha_deta",
             "d2phi_dlog_tau2", "d2phi_dgamma2", "d2phi_dalpha2",
             "d2phi_deta2",
             "d2phi_dlog_tau_dgamma", "d2phi_dlog_tau_dalpha",
             "d2phi_dlog_tau_deta",   "d2phi_dgamma_dalpha",
             "d2phi_dgamma_deta",     "d2phi_dalpha_deta")
    return(setNames(replicate(length(nms), zeros, simplify = FALSE), nms))
  }

  # Perturbed values: log_tau, gamma, alpha, eta steps
  tau_p <- tau * exp(h)
  tau_m <- tau * exp(-h)
  gam_p <- gamma + h
  gam_m <- gamma - h
  alp_p <- alpha + h
  alp_m <- alpha - h
  eta_p <- eta   + h
  eta_m <- eta   - h

  # Fallback to d00 if perturbed eval returns NULL
  safe_eval <- function(tau_v, gam_v, alp_v, eta_v) {
    r <- eval_at(tau_v, gam_v, alp_v, eta_v)
    if (is.null(r)) d00 else r
  }

  d_Tp <- safe_eval(tau_p, gamma, alpha, eta)
  d_Tm <- safe_eval(tau_m, gamma, alpha, eta)
  d_Gp <- safe_eval(tau, gam_p, alpha, eta)
  d_Gm <- safe_eval(tau, gam_m, alpha, eta)
  d_Ap <- safe_eval(tau, gamma, alp_p, eta)
  d_Am <- safe_eval(tau, gamma, alp_m, eta)
  d_Ep <- safe_eval(tau, gamma, alpha, eta_p)
  d_Em <- safe_eval(tau, gamma, alpha, eta_m)

  h2 <- h * h

  # Diagonal second differences
  diag2 <- function(dp, dm, d0, which) {
    (dp[[which]] - 2 * d0[[which]] + dm[[which]]) / h2
  }

  # tau / gamma
  d_TpGp <- safe_eval(tau_p, gam_p, alpha, eta)
  d_TpGm <- safe_eval(tau_p, gam_m, alpha, eta)
  d_TmGp <- safe_eval(tau_m, gam_p, alpha, eta)
  d_TmGm <- safe_eval(tau_m, gam_m, alpha, eta)
  TG_Phi <- (d_TpGp$Phi - d_TpGm$Phi - d_TmGp$Phi + d_TmGm$Phi) / (4 * h2)
  TG_phi <- (d_TpGp$phi - d_TpGm$phi - d_TmGp$phi + d_TmGm$phi) / (4 * h2)

  # tau / alpha
  d_TpAp <- safe_eval(tau_p, gamma, alp_p, eta)
  d_TpAm <- safe_eval(tau_p, gamma, alp_m, eta)
  d_TmAp <- safe_eval(tau_m, gamma, alp_p, eta)
  d_TmAm <- safe_eval(tau_m, gamma, alp_m, eta)
  TA_Phi <- (d_TpAp$Phi - d_TpAm$Phi - d_TmAp$Phi + d_TmAm$Phi) / (4 * h2)
  TA_phi <- (d_TpAp$phi - d_TpAm$phi - d_TmAp$phi + d_TmAm$phi) / (4 * h2)

  # tau / eta
  d_TpEp <- safe_eval(tau_p, gamma, alpha, eta_p)
  d_TpEm <- safe_eval(tau_p, gamma, alpha, eta_m)
  d_TmEp <- safe_eval(tau_m, gamma, alpha, eta_p)
  d_TmEm <- safe_eval(tau_m, gamma, alpha, eta_m)
  TE_Phi <- (d_TpEp$Phi - d_TpEm$Phi - d_TmEp$Phi + d_TmEm$Phi) / (4 * h2)
  TE_phi <- (d_TpEp$phi - d_TpEm$phi - d_TmEp$phi + d_TmEm$phi) / (4 * h2)

  # gamma / alpha
  d_GpAp <- safe_eval(tau, gam_p, alp_p, eta)
  d_GpAm <- safe_eval(tau, gam_p, alp_m, eta)
  d_GmAp <- safe_eval(tau, gam_m, alp_p, eta)
  d_GmAm <- safe_eval(tau, gam_m, alp_m, eta)
  GA_Phi <- (d_GpAp$Phi - d_GpAm$Phi - d_GmAp$Phi + d_GmAm$Phi) / (4 * h2)
  GA_phi <- (d_GpAp$phi - d_GpAm$phi - d_GmAp$phi + d_GmAm$phi) / (4 * h2)

  # gamma / eta
  d_GpEp <- safe_eval(tau, gam_p, alpha, eta_p)
  d_GpEm <- safe_eval(tau, gam_p, alpha, eta_m)
  d_GmEp <- safe_eval(tau, gam_m, alpha, eta_p)
  d_GmEm <- safe_eval(tau, gam_m, alpha, eta_m)
  GE_Phi <- (d_GpEp$Phi - d_GpEm$Phi - d_GmEp$Phi + d_GmEm$Phi) / (4 * h2)
  GE_phi <- (d_GpEp$phi - d_GpEm$phi - d_GmEp$phi + d_GmEm$phi) / (4 * h2)

  # alpha / eta
  d_ApEp <- safe_eval(tau, gamma, alp_p, eta_p)
  d_ApEm <- safe_eval(tau, gamma, alp_p, eta_m)
  d_AmEp <- safe_eval(tau, gamma, alp_m, eta_p)
  d_AmEm <- safe_eval(tau, gamma, alp_m, eta_m)
  AE_Phi <- (d_ApEp$Phi - d_ApEm$Phi - d_AmEp$Phi + d_AmEm$Phi) / (4 * h2)
  AE_phi <- (d_ApEp$phi - d_ApEm$phi - d_AmEp$phi + d_AmEm$phi) / (4 * h2)

  list(
    d2Phi_dlog_tau2        = diag2(d_Tp, d_Tm, d00, "Phi"),
    d2Phi_dgamma2          = diag2(d_Gp, d_Gm, d00, "Phi"),
    d2Phi_dalpha2          = diag2(d_Ap, d_Am, d00, "Phi"),
    d2Phi_deta2            = diag2(d_Ep, d_Em, d00, "Phi"),
    d2Phi_dlog_tau_dgamma  = TG_Phi,
    d2Phi_dlog_tau_dalpha  = TA_Phi,
    d2Phi_dlog_tau_deta    = TE_Phi,
    d2Phi_dgamma_dalpha    = GA_Phi,
    d2Phi_dgamma_deta      = GE_Phi,
    d2Phi_dalpha_deta      = AE_Phi,

    d2phi_dlog_tau2        = diag2(d_Tp, d_Tm, d00, "phi"),
    d2phi_dgamma2          = diag2(d_Gp, d_Gm, d00, "phi"),
    d2phi_dalpha2          = diag2(d_Ap, d_Am, d00, "phi"),
    d2phi_deta2            = diag2(d_Ep, d_Em, d00, "phi"),
    d2phi_dlog_tau_dgamma  = TG_phi,
    d2phi_dlog_tau_dalpha  = TA_phi,
    d2phi_dlog_tau_deta    = TE_phi,
    d2phi_dgamma_dalpha    = GA_phi,
    d2phi_dgamma_deta      = GE_phi,
    d2phi_dalpha_deta      = AE_phi
  )
}

# ---------------------------------------------------------------------------
# Main analytic Hessian function
# ---------------------------------------------------------------------------
#' Analytic Hessian of the multiphase NLL
#'
#' Returns the \eqn{p \times p} Hessian of the **objective** (negative
#' log-likelihood) on the internal parameter scale, matching
#' \code{numDeriv::hessian(objective)}.  Coverage: event + right-censored
#' rows, with counting-process start-time corrections when
#' \code{time_lower > 0}. Returns \code{NULL} for data containing
#' left-censored (\code{status == -1}) or interval-censored
#' (\code{status == 2}) rows; the caller falls back to numDeriv in that case.
#'
#' **Structure** (see derivation in PLAN-hessian-analytic-multiphase.md):
#' \deqn{H = (\text{block-diagonal } A) + (\text{dense outer-product } B) + (\text{block-diagonal } C)}
#'
#' Term A: curvature of \eqn{\sum_i w_i H_i}; block-diagonal in phases.
#' Term B: \eqn{\sum_{i\in E} (w_i/h_i^2) \nabla h_i \nabla h_i^T}; dense.
#' Term C: curvature of \eqn{-\sum_{i\in E} w_i \log h_i}; block-diagonal.
#'
#' @noRd
.hzr_hessian_multiphase <- function(
    theta, time, status,
    time_lower = NULL, time_upper = NULL,
    x = NULL, weights = NULL,
    phases, covariate_counts, x_list) {

  n <- length(time)
  p <- length(theta)
  if (is.null(weights)) weights <- rep(1, n)

  # Coverage contract: decline for left/interval-censored rows
  if (any(status %in% c(-1L, 2L))) return(NULL)

  idx_event <- which(status == 1L)
  n_e       <- length(idx_event)

  # Counting-process start-time handling (mirrors gradient)
  need_start <- !is.null(time_lower) &&
                any(time_lower > 0 & time_lower < time & status %in% c(0L, 1L))
  start_vec <- if (need_start) {
    sv <- rep(0, n)
    epoch_idx <- status %in% c(0L, 1L) & time_lower < time
    sv[epoch_idx] <- time_lower[epoch_idx]
    sv
  } else {
    NULL
  }

  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)

  # --- Per-phase quantities --------------------------------------------------
  n_phases        <- length(phases)
  phase_mu        <- vector("list", n_phases)
  phase_Phi       <- vector("list", n_phases)
  phase_phi       <- vector("list", n_phases)
  phase_d1        <- vector("list", n_phases)   # .hzr_phase_derivatives output
  phase_d2        <- vector("list", n_phases)   # second-derivative output
  phase_Phi_start <- vector("list", n_phases)
  phase_d1_start  <- vector("list", n_phases)
  phase_d2_start  <- vector("list", n_phases)
  phase_x_tilde   <- vector("list", n_phases)   # [1 | x_j], n x (1+p_j)

  H_t <- rep(0, n)
  h_t <- rep(0, n)

  for (j in seq_along(phases)) {
    nm   <- names(phases)[j]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    # mu_j(i) and augmented covariate matrix
    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      x_j  <- x_list[[nm]]
      eta_j <- pars$log_mu + as.numeric(x_j %*% pars$beta)
    } else {
      x_j   <- NULL
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)
    phase_mu[[j]] <- mu_j

    x_tilde_j <- if (is.null(x_j)) matrix(1, n, 1L) else cbind(1, x_j)
    phase_x_tilde[[j]] <- x_tilde_j

    # Shape quantities
    if (phases[[nm]]$type == "constant") {
      phase_Phi[[j]] <- time
      phase_phi[[j]] <- rep(1, n)
      phase_d1[j]  <- list(NULL)
      phase_d2[j]  <- list(NULL)
      if (need_start) {
        phase_Phi_start[[j]] <- start_vec
        phase_d1_start[j]  <- list(NULL)
        phase_d2_start[j]  <- list(NULL)
      }
    } else if (phases[[nm]]$type == "g3") {
      tau_j <- exp(pars$log_tau)
      d3    <- hzr_decompos_g3(time, tau = tau_j, gamma = pars$gamma,
                                alpha = pars$alpha, eta = pars$eta)
      phase_Phi[[j]] <- d3$G3
      phase_phi[[j]] <- d3$g3
      phase_d1[[j]]  <- .hzr_g3_phase_derivatives(
        time, tau = tau_j, gamma = pars$gamma,
        alpha = pars$alpha, eta = pars$eta)
      phase_d2[[j]]  <- .hzr_g3_phase_second_derivatives(
        time, tau = tau_j, gamma = pars$gamma,
        alpha = pars$alpha, eta = pars$eta)
      if (need_start) {
        d3s <- hzr_decompos_g3(start_vec, tau = tau_j, gamma = pars$gamma,
                                alpha = pars$alpha, eta = pars$eta)
        phase_Phi_start[[j]] <- d3s$G3
        phase_d1_start[[j]]  <- .hzr_g3_phase_derivatives(
          start_vec, tau = tau_j, gamma = pars$gamma,
          alpha = pars$alpha, eta = pars$eta)
        phase_d2_start[[j]]  <- .hzr_g3_phase_second_derivatives(
          start_vec, tau = tau_j, gamma = pars$gamma,
          alpha = pars$alpha, eta = pars$eta)
      }
    } else {
      # CDF / hazard
      t_half_j <- exp(pars$log_t_half)
      d1 <- .hzr_phase_derivatives(time, t_half = t_half_j, nu = pars$nu,
                                    m = pars$m, type = phases[[nm]]$type)
      phase_Phi[[j]] <- d1$Phi
      phase_phi[[j]] <- d1$phi
      phase_d1[[j]]  <- d1
      phase_d2[[j]]  <- .hzr_phase_second_derivatives(
        time, t_half = t_half_j, nu = pars$nu, m = pars$m,
        type = phases[[nm]]$type)
      if (need_start) {
        d1s <- .hzr_phase_derivatives(start_vec, t_half = t_half_j,
                                       nu = pars$nu, m = pars$m,
                                       type = phases[[nm]]$type)
        phase_Phi_start[[j]] <- d1s$Phi
        phase_d1_start[[j]]  <- d1s
        phase_d2_start[[j]]  <- .hzr_phase_second_derivatives(
          start_vec, t_half = t_half_j, nu = pars$nu, m = pars$m,
          type = phases[[nm]]$type)
      }
    }

    H_t <- H_t + mu_j * phase_Phi[[j]]
    h_t <- h_t + mu_j * phase_phi[[j]]
    if (need_start) {
      H_t <- H_t - mu_j * phase_Phi_start[[j]]  # H(t) - H(start) net
    }
  }

  if (any(!is.finite(H_t)) || any(!is.finite(h_t))) {
    return(NULL)
  }

  # h_e = h at event times (guard against 0)
  h_e <- pmax(h_t[idx_event], .Machine$double.xmin)

  # w/h_e^2 and w/h_e vectors (length n_e)
  w_e     <- weights[idx_event]
  wh2_e   <- w_e / h_e^2     # for Term B
  wh1_e   <- w_e / h_e       # for Term C

  # --- Build the full p x p Hessian matrix ----------------------------------
  H_mat <- matrix(0, p, p)
  dimnames(H_mat) <- list(names(theta), names(theta))

  # We need a per-phase parameter offset so we know where in H_mat to write.
  # Build an offset table.
  phase_param_counts <- vapply(seq_along(phases), function(j) {
    nm <- names(phases)[j]
    if (phases[[nm]]$type == "constant") {
      1L + covariate_counts[[nm]]
    } else if (phases[[nm]]$type == "g3") {
      5L + covariate_counts[[nm]]  # log_mu, log_tau, gamma, alpha, eta
    } else {
      4L + covariate_counts[[nm]]  # log_mu, log_t_half, nu, m
    }
  }, integer(1))

  phase_offsets <- c(0L, cumsum(phase_param_counts))

  # --- Term B: dense outer product of h-gradient ----------------------------
  # Assemble X_h (n_e x p): row i is the gradient of h(t_i) w.r.t. theta.
  X_h <- matrix(0, n_e, p)

  for (j in seq_along(phases)) {
    nm   <- names(phases)[j]
    pos  <- phase_offsets[j]
    mu_j <- phase_mu[[j]]
    phi_j <- phase_phi[[j]]
    d1    <- phase_d1[[j]]

    mu_phi_e <- mu_j[idx_event] * phi_j[idx_event]  # length n_e

    # log_mu column: dh/d(log_mu_j) = mu_j phi_j
    X_h[, pos + 1L] <- mu_phi_e

    # Shape columns
    if (phases[[nm]]$type %in% c("cdf", "hazard")) {
      shape_dphi <- list(d1$dphi_dlog_thalf, d1$dphi_dnu, d1$dphi_dm)
      for (s in seq_along(shape_dphi)) {
        X_h[, pos + 1L + s] <- mu_j[idx_event] * shape_dphi[[s]][idx_event]
      }
      beta_start <- pos + 5L
    } else if (phases[[nm]]$type == "g3") {
      shape_dphi <- list(d1$dphi_dlog_tau, d1$dphi_dgamma,
                         d1$dphi_dalpha,   d1$dphi_deta)
      for (s in seq_along(shape_dphi)) {
        X_h[, pos + 1L + s] <- mu_j[idx_event] * shape_dphi[[s]][idx_event]
      }
      beta_start <- pos + 6L
    } else {
      # constant phase: no shape cols
      beta_start <- pos + 2L
    }

    # Beta columns: dh/d(beta_jk) = mu_j phi_j x_jk
    if (covariate_counts[[nm]] > 0 && !is.null(x_list[[nm]])) {
      x_j_e <- x_list[[nm]][idx_event, , drop = FALSE]
      for (k in seq_len(covariate_counts[[nm]])) {
        X_h[, beta_start + k - 1L] <- mu_phi_e * x_j_e[, k]
      }
    }
  }

  # Term B = X_h' diag(wh2_e) X_h (each row of X_h weighted by wh2_e)
  B_mat <- crossprod(X_h, wh2_e * X_h)
  H_mat <- H_mat + B_mat

  # --- Terms A and C: per-phase block contributions -------------------------
  for (j in seq_along(phases)) {
    nm   <- names(phases)[j]
    pos  <- phase_offsets[j]
    mu_j <- phase_mu[[j]]
    Phi_j <- phase_Phi[[j]]
    phi_j <- phase_phi[[j]]
    d1    <- phase_d1[[j]]
    d2    <- phase_d2[[j]]
    x_t   <- phase_x_tilde[[j]]

    wmu_Phi   <- weights * mu_j * Phi_j               # Term A weight (all rows)
    wmu_phi_e <- wh1_e * mu_j[idx_event] * phi_j[idx_event]  # Term C weight (events)

    # Counting-process start correction for Term A:
    if (need_start) {
      wmu_Phi_start <- weights * mu_j * phase_Phi_start[[j]]
      wmu_Phi_net <- wmu_Phi - wmu_Phi_start   # net for H(t) - H(start)
    } else {
      wmu_Phi_net <- wmu_Phi
    }

    # --- mu/beta sub-block (crossprod style) ---------------------------------
    # Term A mu/beta: X_t' diag(wmu_Phi_net) X_t
    # Term C mu/beta: - X_t_E' diag(wmu_phi_e) X_t_E
    x_t_e <- x_t[idx_event, , drop = FALSE]

    block_A_mub <- crossprod(x_t, wmu_Phi_net * x_t)
    block_C_mub <- crossprod(x_t_e, wmu_phi_e * x_t_e)

    # Number of mu/beta parameters: 1 + p_j
    n_mub <- 1L + covariate_counts[[nm]]

    if (phases[[nm]]$type == "constant") {
      # Only mu/beta block; no shape params
      idx_block <- pos + seq_len(n_mub)
      H_mat[idx_block, idx_block] <- H_mat[idx_block, idx_block] +
        block_A_mub - block_C_mub

    } else {
      # Shape params come AFTER log_mu, BEFORE beta (log_mu | shapes | beta)
      n_shape <- if (phases[[nm]]$type == "g3") 4L else 3L

      idx_mub   <- pos + 1L                             # log_mu index
      idx_shape <- pos + 1L + seq_len(n_shape)          # shape indices
      idx_beta  <- if (covariate_counts[[nm]] > 0)
                     pos + 1L + n_shape + seq_len(covariate_counts[[nm]])
                   else integer(0)

      # log_mu / log_mu (scalar)
      H_mat[idx_mub, idx_mub] <- H_mat[idx_mub, idx_mub] +
        sum(wmu_Phi_net) - sum(wmu_phi_e)

      # log_mu / beta cross terms (only if covariates present)
      if (length(idx_beta) > 0) {
        x_j <- x_list[[nm]]
        x_j_e <- x_j[idx_event, , drop = FALSE]
        cross_A <- colSums(wmu_Phi_net * x_j)
        cross_C <- colSums(wmu_phi_e * x_j_e)
        H_mat[idx_mub, idx_beta] <- H_mat[idx_mub, idx_beta] + cross_A - cross_C
        H_mat[idx_beta, idx_mub] <- H_mat[idx_beta, idx_mub] + cross_A - cross_C
      }

      # beta / beta (only if covariates)
      if (length(idx_beta) > 0) {
        x_j   <- x_list[[nm]]
        x_j_e <- x_j[idx_event, , drop = FALSE]
        bb_A  <- crossprod(x_j,   wmu_Phi_net * x_j)
        bb_C  <- crossprod(x_j_e, wmu_phi_e   * x_j_e)
        H_mat[idx_beta, idx_beta] <- H_mat[idx_beta, idx_beta] + bb_A - bb_C
      }

      # Shape parameters: extract first-derivative vectors
      if (phases[[nm]]$type %in% c("cdf", "hazard")) {
        shape_keys_d1_Phi <- c("dPhi_dlog_thalf", "dPhi_dnu", "dPhi_dm")
        shape_keys_d1_phi <- c("dphi_dlog_thalf", "dphi_dnu", "dphi_dm")
        shape_keys_d2_Phi <- list(
          c("d2Phi_dlog_thalf2",    "d2Phi_dlog_thalf2"),
          c("d2Phi_dnu2",           "d2Phi_dnu2"),
          c("d2Phi_dm2",            "d2Phi_dm2"),
          c("d2Phi_dlog_thalf_dnu", "d2Phi_dlog_thalf_dnu"),
          c("d2Phi_dlog_thalf_dm",  "d2Phi_dlog_thalf_dm"),
          c("d2Phi_dnu_dm",         "d2Phi_dnu_dm")
        )
        shape_keys_d2_phi <- list(
          c("d2phi_dlog_thalf2",    "d2phi_dlog_thalf2"),
          c("d2phi_dnu2",           "d2phi_dnu2"),
          c("d2phi_dm2",            "d2phi_dm2"),
          c("d2phi_dlog_thalf_dnu", "d2phi_dlog_thalf_dnu"),
          c("d2phi_dlog_thalf_dm",  "d2phi_dlog_thalf_dm"),
          c("d2phi_dnu_dm",         "d2phi_dnu_dm")
        )
        # Which pairs of shape indices correspond to the 6 unique pairs?
        shape_pairs <- list(c(1, 1), c(2, 2), c(3, 3), c(1, 2), c(1, 3), c(2, 3))
      } else {
        # G3: 4 shape params -> 10 unique pairs
        shape_keys_d1_Phi <- c("dPhi_dlog_tau", "dPhi_dgamma",
                                "dPhi_dalpha",   "dPhi_deta")
        shape_keys_d1_phi <- c("dphi_dlog_tau", "dphi_dgamma",
                                "dphi_dalpha",   "dphi_deta")
        shape_keys_d2_Phi <- list(
          c("d2Phi_dlog_tau2",       "d2Phi_dlog_tau2"),
          c("d2Phi_dgamma2",         "d2Phi_dgamma2"),
          c("d2Phi_dalpha2",         "d2Phi_dalpha2"),
          c("d2Phi_deta2",           "d2Phi_deta2"),
          c("d2Phi_dlog_tau_dgamma", "d2Phi_dlog_tau_dgamma"),
          c("d2Phi_dlog_tau_dalpha", "d2Phi_dlog_tau_dalpha"),
          c("d2Phi_dlog_tau_deta",   "d2Phi_dlog_tau_deta"),
          c("d2Phi_dgamma_dalpha",   "d2Phi_dgamma_dalpha"),
          c("d2Phi_dgamma_deta",     "d2Phi_dgamma_deta"),
          c("d2Phi_dalpha_deta",     "d2Phi_dalpha_deta")
        )
        shape_keys_d2_phi <- lapply(shape_keys_d2_Phi, function(k) {
          sub("Phi", "phi", k)
        })
        shape_pairs <- c(
          list(c(1, 1), c(2, 2), c(3, 3), c(4, 4)),
          list(c(1, 2), c(1, 3), c(1, 4), c(2, 3), c(2, 4), c(3, 4))
        )
      }

      n_shape <- length(shape_keys_d1_Phi)

      # log_mu / shape cross terms (Term A and C)
      for (s in seq_len(n_shape)) {
        dPhi_ds <- d1[[shape_keys_d1_Phi[s]]]
        dphi_ds <- d1[[shape_keys_d1_phi[s]]]
        if (need_start && !is.null(phase_d1_start[[j]])) {
          dPhi_ds_start <- phase_d1_start[[j]][[shape_keys_d1_Phi[s]]]
          a_val <- sum(weights * mu_j * dPhi_ds) -
                   sum(weights * mu_j * dPhi_ds_start)
        } else {
          a_val <- sum(weights * mu_j * dPhi_ds)
        }
        c_val <- sum(wh1_e * mu_j[idx_event] * dphi_ds[idx_event])
        col_s <- idx_shape[s]
        H_mat[idx_mub, col_s] <- H_mat[idx_mub, col_s] + a_val - c_val
        H_mat[col_s, idx_mub] <- H_mat[col_s, idx_mub] + a_val - c_val
      }

      # shape / shape block
      for (pair_idx in seq_along(shape_pairs)) {
        s_a <- shape_pairs[[pair_idx]][1]
        s_b <- shape_pairs[[pair_idx]][2]
        key_d2_Phi <- shape_keys_d2_Phi[[pair_idx]][1]
        key_d2_phi <- shape_keys_d2_phi[[pair_idx]][1]

        d2Phi_sasb <- d2[[key_d2_Phi]]
        d2phi_sasb <- d2[[key_d2_phi]]

        a_val <- sum(weights * mu_j * d2Phi_sasb)
        if (need_start && !is.null(phase_d2_start[[j]])) {
          a_val <- a_val - sum(weights * mu_j * phase_d2_start[[j]][[key_d2_Phi]])
        }
        c_val <- sum(wh1_e * mu_j[idx_event] * d2phi_sasb[idx_event])

        i_idx <- idx_shape[s_a]
        j_idx <- idx_shape[s_b]
        H_mat[i_idx, j_idx] <- H_mat[i_idx, j_idx] + a_val - c_val
        if (s_a != s_b) {
          H_mat[j_idx, i_idx] <- H_mat[j_idx, i_idx] + a_val - c_val
        }
      }

      # beta / shape cross terms
      if (length(idx_beta) > 0) {
        x_j   <- x_list[[nm]]
        x_j_e <- x_j[idx_event, , drop = FALSE]
        for (s in seq_len(n_shape)) {
          dPhi_ds <- d1[[shape_keys_d1_Phi[s]]]
          dphi_ds <- d1[[shape_keys_d1_phi[s]]]
          if (need_start && !is.null(phase_d1_start[[j]])) {
            dPhi_ds_start <- phase_d1_start[[j]][[shape_keys_d1_Phi[s]]]
            a_vec <- colSums((weights * mu_j * dPhi_ds -
                              weights * mu_j * dPhi_ds_start) * x_j)
          } else {
            a_vec <- colSums((weights * mu_j * dPhi_ds) * x_j)
          }
          c_vec <- colSums((wh1_e * mu_j[idx_event] *
                              dphi_ds[idx_event]) * x_j_e)
          col_s <- idx_shape[s]
          H_mat[idx_beta, col_s] <- H_mat[idx_beta, col_s] + a_vec - c_vec
          H_mat[col_s, idx_beta] <- H_mat[col_s, idx_beta] + a_vec - c_vec
        }
      }
    }
  }

  # Guard: any non-finite entry means the Hessian is unreliable; return NULL so
  # the caller falls back to numDeriv rather than silently passing a corrupted matrix.
  if (any(!is.finite(H_mat))) return(NULL)

  H_mat
}

# test-multiphase-hessian.R
# Analytic Hessian tests for multiphase models.
# Validates shape second-derivative helpers and .hzr_hessian_multiphase()
# against numDeriv references.

# ---------------------------------------------------------------------------
# Task 1: Shape second-derivative helpers
# ---------------------------------------------------------------------------

test_that(".hzr_phase_second_derivatives diagonals match numDeriv", {
  skip_if_not_installed("numDeriv")
  t <- c(0.2, 0.5, 1.0, 2.0, 4.0)
  t_half <- 1.0
  nu <- 1.5
  m <- 1.0

  d2 <- .hzr_phase_second_derivatives(t, t_half = t_half, nu = nu, m = m,
                                       type = "cdf")

  # Reference: numDeriv Hessian of Phi_i w.r.t. (log_t_half, nu, m)
  par0 <- c(log(t_half), nu, m)
  for (i in seq_along(t)) {
    phi_fn_i <- function(par) {
      hzr_phase_cumhaz(t[i], t_half = exp(par[1]), nu = par[2], m = par[3],
                       type = "cdf")
    }
    H_nd <- numDeriv::hessian(phi_fn_i, par0)
    expect_equal(d2$d2Phi_dlog_thalf2[i],       H_nd[1, 1], tolerance = 1e-3,
                 label = paste0("d2Phi/d(log_thalf)^2 at t=", t[i]))
    expect_equal(d2$d2Phi_dnu2[i],              H_nd[2, 2], tolerance = 1e-3,
                 label = paste0("d2Phi/dnu^2 at t=", t[i]))
    expect_equal(d2$d2Phi_dm2[i],               H_nd[3, 3], tolerance = 1e-3,
                 label = paste0("d2Phi/dm^2 at t=", t[i]))
    expect_equal(d2$d2Phi_dlog_thalf_dnu[i],    H_nd[1, 2], tolerance = 1e-3,
                 label = paste0("d2Phi/d(log_thalf)dnu at t=", t[i]))
    expect_equal(d2$d2Phi_dlog_thalf_dm[i],     H_nd[1, 3], tolerance = 1e-3,
                 label = paste0("d2Phi/d(log_thalf)dm at t=", t[i]))
    expect_equal(d2$d2Phi_dnu_dm[i],            H_nd[2, 3], tolerance = 1e-3,
                 label = paste0("d2Phi/dnudm at t=", t[i]))
  }
})

test_that(".hzr_phase_second_derivatives phi second derivs match numDeriv", {
  skip_if_not_installed("numDeriv")
  t <- c(0.3, 0.8, 1.5, 3.0)
  t_half <- 0.5
  nu <- 2.0
  m <- 1.0
  d2 <- .hzr_phase_second_derivatives(t, t_half = t_half, nu = nu, m = m,
                                       type = "cdf")
  par0 <- c(log(t_half), nu, m)
  for (i in seq_along(t)) {
    haz_fn_i <- function(par) {
      hzr_phase_hazard(t[i], t_half = exp(par[1]), nu = par[2], m = par[3],
                       type = "cdf")
    }
    H_nd <- numDeriv::hessian(haz_fn_i, par0)
    expect_equal(d2$d2phi_dlog_thalf2[i], H_nd[1, 1], tolerance = 1e-3,
                 label = paste0("d2phi/d(log_thalf)^2 at t=", t[i]))
    expect_equal(d2$d2phi_dnu2[i],        H_nd[2, 2], tolerance = 1e-3,
                 label = paste0("d2phi/dnu^2 at t=", t[i]))
  }
})

test_that(".hzr_g3_phase_second_derivatives diagonals match numDeriv", {
  skip_if_not_installed("numDeriv")
  t <- c(0.5, 1.0, 3.0, 8.0)
  tau <- 2.0
  gamma <- 1.5
  alpha <- 0.8
  eta <- 0.6

  d2 <- .hzr_g3_phase_second_derivatives(t, tau = tau, gamma = gamma,
                                          alpha = alpha, eta = eta)
  par0 <- c(log(tau), gamma, alpha, eta)
  for (i in seq_along(t)) {
    g3_fn_i <- function(par) {
      hzr_decompos_g3(t[i], tau = exp(par[1]), gamma = par[2],
                       alpha = par[3], eta = par[4])$G3
    }
    H_nd <- numDeriv::hessian(g3_fn_i, par0)
    expect_equal(d2$d2Phi_dlog_tau2[i], H_nd[1, 1], tolerance = 1e-2,
                 label = paste0("d2G3/d(log_tau)^2 at t=", t[i]))
    expect_equal(d2$d2Phi_dgamma2[i],   H_nd[2, 2], tolerance = 1e-2,
                 label = paste0("d2G3/dgamma^2 at t=", t[i]))
    expect_equal(d2$d2Phi_dalpha2[i],   H_nd[3, 3], tolerance = 1e-2,
                 label = paste0("d2G3/dalpha^2 at t=", t[i]))
    expect_equal(d2$d2Phi_deta2[i],     H_nd[4, 4], tolerance = 1e-2,
                 label = paste0("d2G3/deta^2 at t=", t[i]))
  }
})

# ---------------------------------------------------------------------------
# Task 2: .hzr_hessian_multiphase() cross-check tests
# ---------------------------------------------------------------------------

test_that(".hzr_hessian_multiphase matches numDeriv (2-phase no covariates)", {
  skip_on_cran()
  skip_if_not_installed("numDeriv")
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
    constant = hzr_phase("constant")
  )
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data    = avc,
    dist    = "multiphase",
    phases  = phases,
    fit     = TRUE,
    control = list(n_starts = 1, conserve = FALSE)
  )
  theta      <- fit$fit$par
  cov_counts <- fit$fit$covariate_counts
  x_list     <- fit$fit$x_list

  H_an <- .hzr_hessian_multiphase(
    theta, time = avc$int_dead, status = avc$dead,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )
  expect_true(is.matrix(H_an))

  obj <- function(par) {
    -.hzr_logl_multiphase(par, avc$int_dead, avc$dead,
                           phases = phases,
                           covariate_counts = cov_counts,
                           x_list = x_list)
  }
  H_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(H_an), unname(H_nd), tolerance = 1e-3,
               label = "2-phase no-cov NLL Hessian vs numDeriv")
})

test_that(".hzr_hessian_multiphase matches numDeriv (2-phase with covariates)", {
  skip_on_cran()
  skip_if_not_installed("numDeriv")
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)

  phases <- list(
    early    = hzr_phase("cdf",
                          formula = ~ age + com_iv,
                          t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
    constant = hzr_phase("constant")
  )
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ early(age + com_iv),
    data    = avc,
    dist    = "multiphase",
    phases  = phases,
    fit     = TRUE,
    control = list(n_starts = 1, conserve = FALSE)
  )
  theta      <- fit$fit$par
  cov_counts <- fit$fit$covariate_counts
  x_list     <- fit$fit$x_list

  H_an <- .hzr_hessian_multiphase(
    theta, time = avc$int_dead, status = avc$dead,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )
  obj <- function(par) {
    -.hzr_logl_multiphase(par, avc$int_dead, avc$dead,
                           phases = phases,
                           covariate_counts = cov_counts,
                           x_list = x_list)
  }
  H_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(H_an), unname(H_nd), tolerance = 1e-3,
               label = "2-phase + covariates Hessian vs numDeriv")
})

test_that(".hzr_hessian_multiphase returns NULL for interval-censored rows", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  status_mixed <- avc$dead
  status_mixed[1:3] <- 2L

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
    constant = hzr_phase("constant")
  )
  theta <- c(early.log_mu = 0, early.log_t_half = log(0.15), early.nu = 1.4,
             early.m = 1.0, constant.log_mu = 0)
  cov_counts <- c(early = 0L, constant = 0L)
  expect_null(.hzr_hessian_multiphase(
    theta, time = avc$int_dead, status = status_mixed,
    phases = phases, covariate_counts = cov_counts,
    x_list = list(early = NULL, constant = NULL)
  ))
})

test_that(".hzr_hessian_multiphase returns NULL for left-censored rows", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  status_left <- avc$dead
  status_left[1:3] <- -1L

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
    constant = hzr_phase("constant")
  )
  theta <- c(early.log_mu = 0, early.log_t_half = log(0.15), early.nu = 1.4,
             early.m = 1.0, constant.log_mu = 0)
  cov_counts <- c(early = 0L, constant = 0L)
  expect_null(.hzr_hessian_multiphase(
    theta, time = avc$int_dead, status = status_left,
    phases = phases, covariate_counts = cov_counts,
    x_list = list(early = NULL, constant = NULL)
  ))
})

# ---------------------------------------------------------------------------
# Task 3: End-to-end wiring tests
# ---------------------------------------------------------------------------

test_that("multiphase fit vcov uses analytic Hessian (matches numDeriv to 1e-3)", {
  skip_on_cran()
  skip_if_not_installed("numDeriv")
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
    constant = hzr_phase("constant")
  )
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1, data = avc, dist = "multiphase",
    phases = phases, fit = TRUE,
    control = list(n_starts = 1, conserve = FALSE)
  )
  theta      <- fit$fit$par
  cov_counts <- fit$fit$covariate_counts
  x_list     <- fit$fit$x_list
  free_idx   <- which(!fit$fit$fixed_mask)

  # numDeriv reference on the FREE-parameter objective (matches optimizer)
  obj_free <- function(par_free) {
    th <- theta
    th[free_idx] <- par_free
    -.hzr_logl_multiphase(th, avc$int_dead, avc$dead,
                           phases = phases,
                           covariate_counts = cov_counts,
                           x_list = x_list)
  }
  vcov_nd <- solve(numDeriv::hessian(obj_free, theta[free_idx]))

  # fit$fit$vcov rows/cols for free params only
  vcov_fit_free <- fit$fit$vcov[free_idx, free_idx]
  expect_equal(unname(vcov_fit_free), unname(vcov_nd),
               tolerance = 1e-3,
               label = "analytic vcov (free params) matches numDeriv reference")
})

test_that("CoE fit vcov (full-info) uses analytic Hessian", {
  skip_on_cran()
  skip_if_not_installed("numDeriv")
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
    constant = hzr_phase("constant")
  )
  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1, data = avc, dist = "multiphase",
    phases = phases, fit = TRUE,
    control = list(n_starts = 1, conserve = TRUE)
  )
  # The conserved log_mu must now carry a finite SE (CoE full-info path)
  se <- sqrt(diag(fit_coe$fit$vcov))
  expect_true(all(is.finite(se[!is.na(se)])))
  expect_true(is.finite(fit_coe$fit$rcond))
})

# ---------------------------------------------------------------------------
# Task 4: Scale-invariance and 13-parameter anchor
# ---------------------------------------------------------------------------

test_that("multiphase SEs are invariant to covariate rescaling (log-mu z-stat)", {
  skip_on_cran()
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  phases_base <- list(
    early    = hzr_phase("cdf", formula = ~ age,
                          t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
    constant = hzr_phase("constant")
  )
  avc2 <- avc
  avc2$age <- avc$age / 10  # rescaled covariate

  f1 <- hazard(survival::Surv(int_dead, dead) ~ early(age),
               data = avc,  dist = "multiphase", phases = phases_base, fit = TRUE,
               control = list(n_starts = 1, conserve = FALSE))
  f2 <- hazard(survival::Surv(int_dead, dead) ~ early(age),
               data = avc2, dist = "multiphase", phases = phases_base, fit = TRUE,
               control = list(n_starts = 1, conserve = FALSE))

  # The beta z-statistic (estimate / SE) must be invariant under x -> x/10.
  se1 <- sqrt(diag(f1$fit$vcov))
  se2 <- sqrt(diag(f2$fit$vcov))
  beta_name <- grep("age", names(f1$fit$par), value = TRUE)[1]
  z1 <- f1$fit$par[beta_name] / se1[beta_name]
  # f2 beta for age is 10x larger (rescaled covariate), SE is 10x larger too
  beta_name2 <- grep("age", names(f2$fit$par), value = TRUE)[1]
  z2 <- f2$fit$par[beta_name2] / se2[beta_name2]
  expect_equal(unname(z1), unname(z2), tolerance = 1e-2,
               label = "beta z-stat invariant under covariate rescaling")
})

test_that("13-parameter multiphase anchor: stable SEs and rcond (supersedes placeholder)", {
  skip_on_cran()
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)

  # 6 early-phase covariates + constant intercept = 10 free params
  # (log_mu_early + log_t_half + nu + 6 betas + log_mu_constant; m is fixed)
  # NOTE: AVC column names: age, com_iv, mal, opmos (NOT op_mos), op_age, status
  phases_hd <- list(
    early    = hzr_phase("cdf",
                          formula = ~ age + com_iv + mal + opmos + op_age + status,
                          t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
    constant = hzr_phase("constant")
  )
  fit <- hazard(
    survival::Surv(int_dead, dead) ~
      early(age + com_iv + mal + opmos + op_age + status),
    data = avc, dist = "multiphase", phases = phases_hd, fit = TRUE,
    control = list(n_starts = 3, maxit = 800, conserve = TRUE)
  )

  expect_true(fit$fit$converged)
  free <- which(is.finite(diag(fit$fit$vcov)))
  expect_gte(length(free), 10L)
  se <- sqrt(diag(fit$fit$vcov)[free])
  expect_true(all(is.finite(se)))
  expect_true(all(se > 0))
  expect_true(is.finite(fit$fit$rcond))
  expect_true(isTRUE(fit$fit$pd),
              label = "10-free-param fit is positive-definite at optimum")
})

#' @importFrom stats optim
#' @keywords internal
NULL

# likelihood-exponential.R -- Exponential parametric hazard likelihood, gradient, and optimizer
#
# MODEL
# -----
# Constant-hazard (memoryless) proportional-hazards model:
#
#   h(t | x)  = lambda exp(eta)                    hazard (constant in t)
#   H(t | x)  = lambda t exp(eta)                  cumulative hazard
#   S(t | x)  = exp(-lambda t exp(eta))            survival
#
# where lambda > 0 (baseline rate), eta = x beta (linear predictor).
#
# THETA LAYOUT
# ------------
#   theta[1]   = log(lambda)   (unconstrained; lambda recovered via exp())
#   theta[2:p] = beta        (covariate coefficients; unrestricted)
#
# The exponential is a special case of Weibull with nu = 1.  It has no shape
# parameter so theta is one element shorter than Weibull for the same data.
# Unconstrained BFGS is used (no box bounds needed).
#
# FUNCTIONS
# ---------
#   .hzr_logl_exponential()     -- log-likelihood (optionally returning gradient)
#   .hzr_gradient_exponential() -- analytical score vector
#   .hzr_optim_exponential()    -- unconstrained BFGS wrapper

# Exponential Parametric Hazard Likelihood
#
# Evaluate the log-likelihood and its derivatives for exponential hazard models.
# The exponential is the simplest ALT model: constant hazard lambda, no shape parameter.

#' Log-likelihood for exponential hazard with covariates
#'
#' Computes the negative log-likelihood for right-censored data under the exponential
#' parametric hazard model with optional linear-predictor covariates.
#'
#' @param theta Vector of parameters:
#'   theta\[1\] = log(lambda) where lambda > 0 is the baseline hazard rate
#'   theta\[2:length\]: Covariate coefficients (linear on log-hazard scale)
#'
#' @param time Numeric vector of follow-up times (n)
#' @param status Numeric vector of event indicators: 1 = event, 0 = censored (n)
#' @param time_lower Optional numeric lower bound vector for interval-censored rows.
#'   Defaults to time if NULL.
#' @param time_upper Optional numeric upper bound vector for left/interval-censored rows.
#'   Defaults to time if NULL.
#' @param x Design matrix of covariates (n x p_coef); NULL for no covariates
#' @param return_gradient Logical; if TRUE, attach gradient vector as attribute
#'
#' @return Scalar log-likelihood value. If return_gradient = TRUE, gradient vector
#' is attached as attribute \code{"gradient"}.
#'
#' @details
#' The exponential hazard model is parameterized as:
#'
#' \deqn{h(t | x) = \lambda \exp(\eta)}
#'
#' where:
#' - \eqn{\lambda > 0} is baseline hazard rate
#' - \eqn{\eta = x \beta} is log-relative-hazard (covariates)
#'
#' The survival function is:
#'
#' \deqn{S(t | x) = \exp(-\lambda t \exp(\eta))}
#'
#' The log-likelihood for right-censored data is:
#'
#' \deqn{\ell(\theta) = \sum_{i: \delta_i = 1} [\log\lambda + \eta_i]
#'   - \lambda \sum_i t_i \exp(\eta_i)}
#'
#' Reparameterization: \u03b8\[1\] = log(\u03bb) avoids constrained optimization.
#'
#' Mixed censoring status coding:
#' - 1: exact event at time
#' - 0: right-censored at time
#' - -1: left-censored with upper bound time_upper \(or time\)
#' - 2: interval-censored in the interval \(time_lower, time_upper\)
#'
#' @noRd
.hzr_logl_exponential <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    return_gradient = FALSE) {

  n <- length(time)

  # Default unit weights
  if (is.null(weights)) weights <- rep(1, n)

  # Extract parameters
  log_lambda <- theta[1]
  lambda <- exp(log_lambda)  # Always positive

  # Covariate coefficients (if any)
  if (!is.null(x)) {
    if (is.null(attr(x, "dimnames")[[2]])) {
      colnames(x) <- paste0("beta_", seq_len(ncol(x)))
    }
    beta <- theta[2:length(theta)]
    eta <- as.numeric(x %*% beta)  # n-vector
  } else {
    eta <- rep(0, n)
  }

  # Resolve censoring bounds once so each likelihood term can reference
  # a consistent lower/upper representation regardless of user input style.
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  if (any(status %in% c(-1, 2)) && any(upper <= 0)) {
    return(Inf)
  }
  if (any(status == 2) && any(lower[status == 2] >= upper[status == 2])) {
    return(Inf)
  }

  # Cumulative hazards at event/lower/upper times; all terms can be
  # expressed using H(t) for the exponential model.
  cumhaz_event <- lambda * time * exp(eta)
  cumhaz_lower <- lambda * lower * exp(eta)
  cumhaz_upper <- lambda * upper * exp(eta)

  idx_event <- status == 1
  idx_right <- status == 0
  idx_left <- status == -1
  idx_interval <- status == 2

  # Event: w * [log h(t) + log S(t)] = w * [log(lambda)+eta - H(t)]
  ll_event <- if (any(idx_event)) {
    sum(weights[idx_event] *
          ((log_lambda + eta[idx_event]) - cumhaz_event[idx_event]))
  } else {
    0
  }

  # Right-censored: w * log S(t) = -w * H(t)
  ll_right <- if (any(idx_right)) {
    -sum(weights[idx_right] * cumhaz_event[idx_right])
  } else {
    0
  }

  # Left-censored: w * log(1 - exp(-H(u)))
  ll_left <- if (any(idx_left)) {
    sum(weights[idx_left] * hzr_log1mexp(cumhaz_upper[idx_left]))
  } else {
    0
  }

  # Interval-censored: w * [log(S(l)-S(u))]
  ll_interval <- if (any(idx_interval)) {
    delta_h <- cumhaz_upper[idx_interval] - cumhaz_lower[idx_interval]
    sum(weights[idx_interval] *
          (-cumhaz_lower[idx_interval] + hzr_log1mexp(delta_h)))
  } else {
    0
  }

  logl <- ll_event + ll_right + ll_left + ll_interval

  if (!is.finite(logl)) {
    return(Inf)
  }

  # If gradient requested, compute score vector
  if (return_gradient) {
    grad <- .hzr_gradient_exponential(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      eta = eta, cumhaz = cumhaz_event,
      lambda = lambda, log_lambda = log_lambda
    )
    attr(logl, "gradient") <- grad
  }

  logl
}

#' Score vector (gradient of log-likelihood) for exponential
#'
#' Computes the score vector of the exponential log-likelihood w.r.t. all parameters.
#'
#' The log-likelihood is:
#'   \eqn{L = \sum(\delta_i * [\log(\lambda) + \eta_i]) - \sum(\lambda * t_i * \exp(\eta_i))}
#'
#' Derivatives:
#' dL/d(log lambda) = sum(delta_i) - sum(lambda * t_i * exp(eta_i)) = sum(delta_i) - sum(H_i)
#' dL/dbeta_j    = sum(delta_i * x_ij) - sum(lambda * t_i * exp(eta_i) * x_ij) = t(X) %*% (delta - H)
#'
#' @noRd
.hzr_gradient_exponential <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    eta = NULL,
    cumhaz = NULL,
    lambda = NULL,
    log_lambda = NULL) {

  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)

  # Default unit weights -- keep gradient consistent with the LL when weights
  # are omitted.
  if (is.null(weights)) weights <- rep(1, n)

  # Numerical fallback for left/interval censoring contributions.
  # Right-censored/event-only rows still use the closed-form score below.
  if (any(status %in% c(-1, 2))) {
    return(.hzr_numeric_grad_exponential(theta, time, status, time_lower,
                                          time_upper, x, weights = weights))
  }

  # Recompute if not provided
  if (is.null(eta) || is.null(cumhaz) || is.null(lambda)) {
    log_lambda <- theta[1]
    lambda <- exp(log_lambda)

    if (!is.null(x)) {
      beta <- theta[2:length(theta)]
      eta <- as.numeric(x %*% beta)
    } else {
      eta <- rep(0, n)
    }

    cumhaz <- lambda * time * exp(eta)
  }

  # Weighted building blocks: every term below is the unweighted form with
  # `status` -> `w * status` and `cumhaz` -> `w * cumhaz`.
  w_status <- weights * status
  w_cumhaz <- weights * cumhaz

  # ===== Gradient w.r.t. log(lambda) =====
  # dL/d(log lambda) = sum(w * delta) - sum(w * H)
  grad[1] <- sum(w_status) - sum(w_cumhaz)

  # ===== Gradient w.r.t. beta (covariate coefficients) =====
  if (p > 1 && !is.null(x)) {
    # dL/dbeta = t(X) %*% (w * delta - w * H)
    residual <- w_status - w_cumhaz
    grad[2:p] <- as.numeric(crossprod(x, residual))
  }

  grad
}

#' @noRd
.hzr_optim_exponential <- function(
    time, status, time_lower = NULL, time_upper = NULL,
    x = NULL, theta_start, weights = NULL, control = list()) {
  .hzr_optim_generic(
    logl_fn = .hzr_logl_exponential,
    gradient_fn = .hzr_gradient_exponential,
    time = time, status = status,
    time_lower = time_lower, time_upper = time_upper,
    x = x, theta_start = theta_start, weights = weights,
    control = control, use_bounds = FALSE
  )
}

.hzr_numeric_grad_exponential <- function(theta, time, status,
                                            time_lower = NULL, time_upper = NULL,
                                            x = NULL, weights = NULL) {
  # Reuse log-likelihood as scalar objective and differentiate numerically.
  objective <- function(par) {
    .hzr_logl_exponential(
      theta = par,
      time = time,
      status = status,
      time_lower = time_lower,
      time_upper = time_upper,
      x = x,
      weights = weights,
      return_gradient = FALSE
    )
  }

  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(as.numeric(numDeriv::grad(objective, theta)))
  }

  eps <- 1e-6
  grad <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tm <- theta
    tp[i] <- tp[i] + eps
    tm[i] <- tm[i] - eps
    grad[i] <- (objective(tp) - objective(tm)) / (2 * eps)
  }
  grad
}

test_that("exponential gradient computes correct shape for univariate case", {
  # Simple univariate exponential (no covariates)
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  # log(lambda) is the parameter
  theta <- c(log_lambda = log(0.5))

  # Compute gradient analytically
  grad <- .hzr_gradient_exponential(
    theta = theta,
    time = time,
    status = status,
    x = NULL
  )

  expect_length(grad, 1)
  expect_true(all(is.finite(grad)))
})

test_that("exponential gradient computes correct shape with covariates", {
  # Exponential with one covariate
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)
  x <- matrix(c(0.5, 1.2, -0.3, 0.8, 1.5), ncol = 1)

  theta <- c(log_lambda = log(0.5), beta = 0.3)

  # Compute gradient analytically
  grad <- .hzr_gradient_exponential(
    theta = theta,
    time = time,
    status = status,
    x = x
  )

  expect_length(grad, 2)
  expect_true(all(is.finite(grad)))
})

test_that("exponential gradient matches numerical gradient", {
  skip_if_not_installed("numDeriv")

  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)
  x <- matrix(c(0.5, 1.2, -0.3, 0.8, 1.5), ncol = 1)

  theta <- c(log_lambda = log(0.5), beta = 0.3)

  # Analytical gradient
  grad_analytical <- .hzr_gradient_exponential(
    theta = theta,
    time = time,
    status = status,
    x = x
  )

  # Numerical gradient via numDeriv
  objective <- function(th) {
    .hzr_logl_exponential(
      theta = th,
      time = time,
      status = status,
      x = x,
      return_gradient = FALSE
    )
  }

  grad_numerical <- numDeriv::grad(objective, theta)

  # Should match to machine precision
  expect_equal(grad_analytical, grad_numerical, tolerance = 1e-4)
})

test_that("exponential optimizer converges", {
  time <- c(1, 2, 3, 4, 5, 6)
  status <- c(1, 0, 1, 1, 0, 1)

  theta_start <- c(log_lambda = log(0.5))

  result <- .hzr_optim_exponential(
    time = time,
    status = status,
    x = NULL,
    theta_start = theta_start,
    control = list(maxit = 100)
  )

  # Check that optimization ran
  expect_type(result, "list")
  expect_true("par" %in% names(result))
  expect_true("value" %in% names(result))

  # Parameters should be finite
  expect_true(all(is.finite(result$par)))
  expect_true(is.finite(result$value))
})

test_that("hazard() accepts exponential distribution", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(log_lambda = log(0.5)),
    dist = "exponential",
    fit = FALSE
  )

  expect_s3_class(fit, "hazard")
  expect_equal(fit$spec$dist, "exponential")
  expect_equal(fit$fit$theta, c(log_lambda = log(0.5)))
})

test_that("hazard() with fit=TRUE estimates exponential parameters", {
  time <- c(1, 2, 3, 4, 5, 6)
  status <- c(1, 0, 1, 1, 0, 1)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(log_lambda = log(0.3)),
    dist = "exponential",
    fit = TRUE,
    control = list(maxit = 200)
  )

  # Should produce fitted model
  expect_true(!is.null(fit$fit$theta))
  expect_true(is.finite(fit$fit$objective))
})

test_that("predict() survival works for exponential", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(log_lambda = log(0.5)),
    dist = "exponential",
    fit = FALSE
  )

  # Predict survival
  surv <- predict(fit, type = "survival")

  expect_true(is.numeric(surv))
  expect_equal(length(surv), length(time))
  expect_true(all(surv > 0 & surv <= 1))
})

test_that("predict() cumulative_hazard works for exponential", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(log_lambda = log(0.5)),
    dist = "exponential",
    fit = FALSE
  )

  # Predict cumulative hazard
  cumhaz <- predict(fit, type = "cumulative_hazard")

  expect_true(is.numeric(cumhaz))
  expect_equal(length(cumhaz), length(time))
  expect_true(all(cumhaz > 0))
})

test_that("exponential: S(t) = exp(-H(t)) relationship holds", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(log_lambda = log(0.5)),
    dist = "exponential",
    fit = FALSE
  )

  surv <- predict(fit, type = "survival")
  cumhaz <- predict(fit, type = "cumulative_hazard")

  # S(t) = exp(-H(t))
  expected_cumhaz <- -log(surv)

  expect_equal(cumhaz, expected_cumhaz, tolerance = 1e-12)
})

test_that("predict() works with covariates for exponential", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)
  x <- matrix(c(0.5, 1.2, -0.3, 0.8, 1.5), ncol = 1)

  fit <- hazard(
    time = time,
    status = status,
    x = x,
    theta = c(log_lambda = log(0.5), beta = 0.3),
    dist = "exponential",
    fit = FALSE
  )

  # Predict survival at original times and data
  newdata <- data.frame(time = time, X1 = x[, 1])
  surv <- predict(fit, newdata = newdata, type = "survival")

  expect_true(is.numeric(surv))
  expect_equal(length(surv), length(time))
  expect_true(all(surv > 0 & surv <= 1))
})

test_that("exponential: monotonicity of survival", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = NULL,
    theta = c(log_lambda = log(0.5)),
    dist = "exponential",
    fit = FALSE
  )

  # Predict across increasing times
  new_times <- seq(0.1, 5, by = 0.5)
  newdata <- data.frame(time = new_times)
  surv <- predict(fit, newdata = newdata, type = "survival")

  # Survival should be monotonically decreasing
  expect_true(all(diff(surv) < 0))
})

test_that("exponential: monotonicity of cumulative hazard", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = NULL,
    theta = c(log_lambda = log(0.5)),
    dist = "exponential",
    fit = FALSE
  )

  # Predict across increasing times
  new_times <- seq(0.1, 5, by = 0.5)
  newdata <- data.frame(time = new_times)
  cumhaz <- predict(fit, newdata = newdata, type = "cumulative_hazard")

  # Cumulative hazard should be monotonically increasing
  expect_true(all(diff(cumhaz) > 0))
})

test_that("exponential baseline hazard is constant", {
  # For exponential with no covariates, hazard should be constant
  # h(t) = lambda (does not depend on t)

  log_lambda <- log(0.5)
  lambda <- exp(log_lambda)

  time <- c(1, 2, 5, 10)
  fit <- hazard(
    time = time,
    status = c(1, 0, 1, 0),
    x = NULL,
    theta = c(log_lambda = log_lambda),
    dist = "exponential",
    fit = FALSE
  )

  # For exponential, cumulative hazard = lambda * t
  # So H(t) should be proportional to t
  cumhaz <- predict(fit, type = "cumulative_hazard")

  # Check proportionality: H(t) / t should be constant
  ratio <- cumhaz / time
  expect_true(all(abs(ratio - lambda) < 1e-12))
})

library(testthat)

test_that("log-logistic fixture refits reproducibly", {
  fixture_file <- system.file("fixtures", "hz_loglogistic.rds", package = "TemporalHazard")
  skip_if_not(file.exists(fixture_file))

  fixture <- readRDS(fixture_file)

  refit <- hazard(
    time = fixture$data$time,
    status = fixture$data$status,
    theta = c(log_alpha = 0.0, log_beta = 0.2),
    dist = "loglogistic",
    fit = TRUE,
    control = list(maxit = 500, reltol = 1e-6)
  )

  expect_true(is.finite(refit$fit$objective))
  expect_true(all(is.finite(refit$fit$theta)))
  expect_equal(refit$fit$objective, fixture$fit$logl, tolerance = 5e-2)
})

test_that("log-normal fixture refits reproducibly", {
  fixture_file <- system.file("fixtures", "hz_lognormal.rds", package = "TemporalHazard")
  skip_if_not(file.exists(fixture_file))

  fixture <- readRDS(fixture_file)

  refit <- hazard(
    time = fixture$data$time,
    status = fixture$data$status,
    theta = c(mu = 1.0, log_sigma = 0.0),
    dist = "lognormal",
    fit = TRUE,
    control = list(maxit = 500, reltol = 1e-6)
  )

  expect_true(is.finite(refit$fit$objective))
  expect_true(all(is.finite(refit$fit$theta)))
  expect_equal(refit$fit$objective, fixture$fit$logl, tolerance = 1e-6)
})

test_that("near-zero positive times remain numerically stable across distributions", {
  time <- c(1e-6, 2e-6, 5e-6, 1e-5, 2e-5)
  status <- c(1, 0, 1, 0, 1)

  expect_true(is.finite(.hzr_logl_weibull(c(0.7, 1.2), time, status)))
  expect_true(is.finite(.hzr_logl_exponential(c(log(0.3)), time, status)))
  expect_true(is.finite(.hzr_logl_loglogistic(c(log(0.7), log(1.2)), time, status)))
  expect_true(is.finite(.hzr_logl_lognormal(c(0.8, log(0.6)), time, status)))
})

test_that("high-dimensional (p > n) fit returns finite objective", {
  set.seed(2026)
  n <- 25
  p <- 40

  time <- rexp(n, rate = 0.5) + 0.05
  status <- rbinom(n, 1, 0.6)
  x <- matrix(rnorm(n * p), nrow = n, ncol = p)

  theta <- c(log(0.4), rep(0, p))

  fit <- hazard(
    time = time,
    status = status,
    x = x,
    theta = theta,
    dist = "exponential",
    fit = TRUE,
    control = list(maxit = 120)
  )

  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$fit$theta), p + 1)
})

# tests/testthat/test-weights.R
# Tests for the `weights` argument added in 0.9.4.
#
# The key invariant we check is that Fisher weighting is equivalent to
# row duplication for integer weights:
#
#   logL(theta; weights = w,  data = D)
#     ==  logL(theta; weights = 1,  data = expand(D, w))
#
# where expand(D, w) duplicates row i exactly w_i times.  This verifies
# that weights propagate through every likelihood term (event,
# right-censored, interval, left).  Running the same invariant through
# `hazard(fit = TRUE)` additionally exercises the optimizer, gradient,
# and Hessian paths because any mis-weighting there would show up as a
# different MLE or vcov between the two fits.

make_toy <- function(seed = 17) {
  set.seed(seed)
  n <- 30
  x <- rnorm(n)
  # Weibull-ish times
  u <- runif(n)
  t <- (-log(u) / exp(0.3 * x))^(1 / 1.2)
  # Mix of events, right-censored, and interval-censored
  status <- ifelse(t < 0.6, 1, 0)
  data.frame(time = t, status = status, x = x)
}

expand_rows <- function(df, weights) {
  df[rep(seq_len(nrow(df)), times = weights), , drop = FALSE]
}

# ---------------------------------------------------------------------------
# Argument validation
# ---------------------------------------------------------------------------

test_that("weights must be numeric and the right length", {
  df <- make_toy()
  expect_error(
    hazard(survival::Surv(time, status) ~ x, data = df,
           dist = "weibull", weights = "a"),
    "numeric vector"
  )
  expect_error(
    hazard(survival::Surv(time, status) ~ x, data = df,
           dist = "weibull", weights = rep(1, nrow(df) - 1)),
    "numeric vector"
  )
})

test_that("weights must be non-negative and finite", {
  df <- make_toy()
  expect_error(
    hazard(survival::Surv(time, status) ~ x, data = df,
           dist = "weibull", weights = c(-1, rep(1, nrow(df) - 1))),
    "non-negative"
  )
  expect_error(
    hazard(survival::Surv(time, status) ~ x, data = df,
           dist = "weibull", weights = c(Inf, rep(1, nrow(df) - 1))),
    "non-negative"
  )
})

# ---------------------------------------------------------------------------
# Weighted log-likelihood equals duplicated-row log-likelihood (Weibull)
# ---------------------------------------------------------------------------

test_that("integer weights match row duplication at an arbitrary theta (Weibull)", {
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)
  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)

  ll_weighted <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta,
    time = df$time, status = df$status,
    x = matrix(df$x, ncol = 1),
    weights = w
  )
  ll_expanded <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta,
    time = df_exp$time, status = df_exp$status,
    x = matrix(df_exp$x, ncol = 1),
    weights = rep(1, nrow(df_exp))
  )

  expect_equal(ll_weighted, ll_expanded, tolerance = 1e-10)
})

test_that("integer weights match row duplication for all censoring flavours", {
  set.seed(3)
  n <- 20
  x <- rnorm(n)
  t <- runif(n, 0.1, 2)
  # Mix: 10 events, 5 right-censored, 5 left-censored
  status <- c(rep(1, 10), rep(0, 5), rep(-1, 5))
  # The parser's convention for left-censored data is time_upper = time for
  # every row (non-left rows ignore it). Mirror that here so the LL's
  # `any(upper <= 0)` guard doesn't trip on NAs.
  time_upper <- t
  w <- sample(1:3, n, replace = TRUE)

  theta <- c(mu = 0.7, nu = 1.3, beta = 0.1)
  ll_w <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = t, status = status,
    time_upper = time_upper, x = matrix(x, ncol = 1), weights = w
  )

  idx <- rep(seq_len(n), times = w)
  ll_dup <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = t[idx], status = status[idx],
    time_upper = time_upper[idx], x = matrix(x[idx], ncol = 1),
    weights = rep(1, sum(w))
  )

  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

# ---------------------------------------------------------------------------
# Weighted fit == duplicated-row fit (MLE + vcov)
# ---------------------------------------------------------------------------

test_that("fit with integer weights matches the duplicated-row fit (Weibull)", {
  skip_on_cran()
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)

  fit_w <- hazard(
    survival::Surv(time, status) ~ x,
    data = df, dist = "weibull",
    weights = w, fit = TRUE
  )
  fit_dup <- hazard(
    survival::Surv(time, status) ~ x,
    data = df_exp, dist = "weibull",
    fit = TRUE
  )

  # Coefficients should match up to optimizer tolerance.
  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  # Log-likelihood should match tightly.
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

# ---------------------------------------------------------------------------
# Unit weights == unweighted (sanity)
# ---------------------------------------------------------------------------

test_that("unit weights are equivalent to no weights (Weibull)", {
  df <- make_toy()
  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)

  ll_noweight <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status,
    x = matrix(df$x, ncol = 1)
  )
  ll_unit <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status,
    x = matrix(df$x, ncol = 1), weights = rep(1, nrow(df))
  )
  expect_equal(ll_noweight, ll_unit, tolerance = 1e-12)
})

# ---------------------------------------------------------------------------
# Multiphase weighted likelihood equals duplicated-row likelihood
# ---------------------------------------------------------------------------

test_that("integer weights match duplication in multiphase likelihood", {
  skip_on_cran()
  set.seed(11)
  n <- 40
  t <- runif(n, 0.05, 3)
  status <- rbinom(n, 1, 0.4)
  w <- sample(1:2, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  # Start values: [log_mu_early, t_half, nu, m, log_mu_constant]
  theta <- c(-4, log(0.3), 1, 1, -3)

  ll_w <- TemporalHazard:::.hzr_logl_multiphase(
    theta = theta, time = t, status = status,
    phases = phases,
    covariate_counts = c(early = 0L, constant = 0L),
    x_list = list(early = NULL, constant = NULL),
    weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_multiphase(
    theta = theta, time = t[idx], status = status[idx],
    phases = phases,
    covariate_counts = c(early = 0L, constant = 0L),
    x_list = list(early = NULL, constant = NULL),
    weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

# ---------------------------------------------------------------------------
# Gradient is weighted (numerical check on Weibull)
# ---------------------------------------------------------------------------

test_that("weighted log-likelihood gradient equals numerical gradient", {
  skip_if_not_installed("numDeriv")
  df <- make_toy()
  w <- sample(1:3, nrow(df), replace = TRUE)
  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  # Numerical gradient of the weighted LL
  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_weibull(
        theta = th, time = df$time, status = df$status,
        x = xm, weights = w
      )
    },
    theta
  )

  ana_g <- TemporalHazard:::.hzr_gradient_weibull(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )

  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-5)
})

# ---------------------------------------------------------------------------
# Zero weight excludes the row
# ---------------------------------------------------------------------------

test_that("a zero weight drops the row's contribution to the likelihood", {
  df <- make_toy()
  w <- rep(1, nrow(df))
  w[1] <- 0

  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  ll_drop <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time[-1], status = df$status[-1],
    x = xm[-1, , drop = FALSE],
    weights = rep(1, nrow(df) - 1)
  )
  expect_equal(ll_w, ll_drop, tolerance = 1e-12)
})

# ---------------------------------------------------------------------------
# Per-distribution duplication-parity for exp / log-logistic / log-normal
# (wired up in v0.9.6 as part of Phase 4e).
# ---------------------------------------------------------------------------

test_that("integer weights match row duplication (exponential LL)", {
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(log_lambda = log(0.6), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta, time = df$time[idx], status = df$status[idx],
    x = matrix(df$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("integer weights match row duplication (log-logistic LL)", {
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(log_alpha = log(0.6), log_beta = log(1.2), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_loglogistic(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_loglogistic(
    theta = theta, time = df$time[idx], status = df$status[idx],
    x = matrix(df$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("integer weights match row duplication (log-normal LL)", {
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(mu = 0, log_sigma = log(1), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta, time = df$time[idx], status = df$status[idx],
    x = matrix(df$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

# Analytic gradients for exp / log-logistic / log-normal match numerical
# gradients of the weighted LL.  Catches the "accepts formal but never
# applies it" pattern in the analytic score path.

test_that("weighted exponential gradient equals numerical gradient", {
  skip_if_not_installed("numDeriv")
  df <- make_toy()
  w <- sample(1:3, nrow(df), replace = TRUE)
  theta <- c(log_lambda = log(0.6), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_exponential(
        theta = th, time = df$time, status = df$status,
        x = xm, weights = w
      )
    },
    theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_exponential(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-5)
})

test_that("weighted log-logistic gradient equals numerical gradient", {
  skip_if_not_installed("numDeriv")
  df <- make_toy()
  w <- sample(1:3, nrow(df), replace = TRUE)
  theta <- c(log_alpha = log(0.6), log_beta = log(1.2), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_loglogistic(
        theta = th, time = df$time, status = df$status,
        x = xm, weights = w
      )
    },
    theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_loglogistic(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-5)
})

test_that("weighted log-normal gradient equals numerical gradient", {
  skip_if_not_installed("numDeriv")
  df <- make_toy()
  w <- sample(1:3, nrow(df), replace = TRUE)
  theta <- c(mu = 0, log_sigma = log(1), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_lognormal(
        theta = th, time = df$time, status = df$status,
        x = xm, weights = w
      )
    },
    theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_lognormal(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-5)
})

# End-to-end: weighted fit == duplicated-row fit for each distribution.

test_that("fit with integer weights matches the duplicated-row fit (exponential)", {
  skip_on_cran()
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)

  fit_w <- hazard(
    survival::Surv(time, status) ~ x, data = df,
    dist = "exponential", weights = w, fit = TRUE
  )
  fit_dup <- hazard(
    survival::Surv(time, status) ~ x, data = df_exp,
    dist = "exponential", fit = TRUE
  )

  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

test_that("fit with integer weights matches the duplicated-row fit (log-logistic)", {
  skip_on_cran()
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)

  fit_w <- hazard(
    survival::Surv(time, status) ~ x, data = df,
    dist = "loglogistic", weights = w, fit = TRUE
  )
  fit_dup <- hazard(
    survival::Surv(time, status) ~ x, data = df_exp,
    dist = "loglogistic", fit = TRUE
  )

  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

test_that("fit with integer weights matches the duplicated-row fit (log-normal)", {
  skip_on_cran()
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)

  fit_w <- hazard(
    survival::Surv(time, status) ~ x, data = df,
    dist = "lognormal", weights = w, fit = TRUE
  )
  fit_dup <- hazard(
    survival::Surv(time, status) ~ x, data = df_exp,
    dist = "lognormal", fit = TRUE
  )

  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

# ---------------------------------------------------------------------------
# Mixed-censoring (status == -1 and status == 2) duplication-parity for
# exp / log-logistic / log-normal.  The weighted LL now applies row
# weights to the left-censored term `log(1 - exp(-H(u)))` and the
# interval-censored term `log(F(u) - F(l))` as well; these tests
# exercise those newly weighted branches.
# ---------------------------------------------------------------------------

make_mixed <- function(seed = 23) {
  set.seed(seed)
  n <- 24
  x <- rnorm(n)
  t <- runif(n, 0.1, 2.0)
  # 10 events, 4 right-censored, 5 left-censored, 5 interval-censored
  status <- c(rep(1L, 10), rep(0L, 4), rep(-1L, 5), rep(2L, 5))
  # Interval bounds must be strictly positive with lower < upper.
  # Left-censored rows set time_upper = time; non-interval rows set
  # time_lower = time (ignored by the LL except on interval rows).
  time_lower <- t
  time_upper <- t
  idx_iv <- which(status == 2)
  time_lower[idx_iv] <- t[idx_iv]
  time_upper[idx_iv] <- t[idx_iv] + runif(length(idx_iv), 0.05, 0.3)
  list(time = t, status = status, x = x,
       time_lower = time_lower, time_upper = time_upper)
}

test_that("weighted LL matches duplicated-row LL under mixed censoring (exponential)", {
  m <- make_mixed()
  n <- length(m$time)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(log_lambda = log(0.5), beta = 0.2)
  xm <- matrix(m$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta, time = m$time, status = m$status,
    time_lower = m$time_lower, time_upper = m$time_upper,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta,
    time = m$time[idx], status = m$status[idx],
    time_lower = m$time_lower[idx], time_upper = m$time_upper[idx],
    x = matrix(m$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("weighted LL matches duplicated-row LL under mixed censoring (log-logistic)", {
  m <- make_mixed()
  n <- length(m$time)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(log_alpha = log(0.5), log_beta = log(1.1), beta = 0.2)
  xm <- matrix(m$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_loglogistic(
    theta = theta, time = m$time, status = m$status,
    time_lower = m$time_lower, time_upper = m$time_upper,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_loglogistic(
    theta = theta,
    time = m$time[idx], status = m$status[idx],
    time_lower = m$time_lower[idx], time_upper = m$time_upper[idx],
    x = matrix(m$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("weighted LL matches duplicated-row LL under mixed censoring (log-normal)", {
  m <- make_mixed()
  n <- length(m$time)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(mu = 0, log_sigma = log(1), beta = 0.2)
  xm <- matrix(m$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta, time = m$time, status = m$status,
    time_lower = m$time_lower, time_upper = m$time_upper,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta,
    time = m$time[idx], status = m$status[idx],
    time_lower = m$time_lower[idx], time_upper = m$time_upper[idx],
    x = matrix(m$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("weighted multiphase analytic gradient matches numerical gradient", {
  # `.hzr_gradient_multiphase()` must apply `weights` to the per-row score
  # (w_H, inv_h, and the interval-censored finite-difference correction).
  # Prior to 0.9.6 it ignored the argument, so weighted multiphase fits
  # optimized a weighted objective with an unweighted gradient -- the fit
  # still converged near the right MLE via line search but the final
  # gradient norm did not go to zero.  This test would have caught it.
  skip_if_not_installed("numDeriv")

  set.seed(31)
  n <- 40
  t <- runif(n, 0.1, 3)
  status <- rbinom(n, 1, 0.45)
  w <- sample(1:3, n, replace = TRUE)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  cov_counts <- c(early = 0L, constant = 0L)
  x_list <- list(early = NULL, constant = NULL)
  # [log_mu_e, log_thalf, nu, m, log_mu_c]
  theta <- c(-4, log(0.3), 1, 1, -3)

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_multiphase(
        theta = th, time = t, status = status,
        phases = phases, covariate_counts = cov_counts,
        x_list = x_list, weights = w
      )
    },
    theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_multiphase(
    theta = theta, time = t, status = status,
    weights = w,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-4)
})

test_that("zero weight drops the row's contribution across mixed censoring (log-normal)", {
  # Targeted sanity check of the per-term weighting: zeroing one row of each
  # censoring flavour must equal dropping those rows entirely.
  m <- make_mixed()
  n <- length(m$time)
  w <- rep(1, n)
  # Zero out one event, one RC, one left, one interval row.
  w[c(1, 11, 15, 20)] <- 0
  keep <- w > 0
  theta <- c(mu = 0, log_sigma = log(1), beta = 0.2)
  xm <- matrix(m$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta, time = m$time, status = m$status,
    time_lower = m$time_lower, time_upper = m$time_upper,
    x = xm, weights = w
  )
  ll_drop <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta,
    time = m$time[keep], status = m$status[keep],
    time_lower = m$time_lower[keep], time_upper = m$time_upper[keep],
    x = xm[keep, , drop = FALSE], weights = rep(1, sum(keep))
  )
  expect_equal(ll_w, ll_drop, tolerance = 1e-12)
})

library(testthat)

.make_mixed_censor_data <- function(n = 70, seed = 321) {
  set.seed(seed)
  base_time <- rexp(n, rate = 0.4) + 0.05
  status <- sample(c(1, 0, -1, 2), n, replace = TRUE, prob = c(0.5, 0.3, 0.1, 0.1))

  time <- base_time
  time_lower <- base_time
  time_upper <- base_time

  idx_left <- which(status == -1)
  if (length(idx_left) > 0) {
    time_upper[idx_left] <- base_time[idx_left] + 0.2
    time_lower[idx_left] <- pmax(base_time[idx_left] - 0.2, 0.01)
    time[idx_left] <- time_upper[idx_left]
  }

  idx_interval <- which(status == 2)
  if (length(idx_interval) > 0) {
    time_lower[idx_interval] <- pmax(base_time[idx_interval] * 0.8, 0.01)
    time_upper[idx_interval] <- base_time[idx_interval] * 1.3 + 0.05
    time[idx_interval] <- (time_lower[idx_interval] + time_upper[idx_interval]) / 2
  }

  list(time = time, status = status, time_lower = time_lower, time_upper = time_upper)
}

test_that("Exponential log-likelihood supports mixed censoring", {
  d <- .make_mixed_censor_data(50, 11)
  ll <- .hzr_logl_exponential(
    theta = c(log_lambda = log(0.4)),
    time = d$time,
    status = d$status,
    time_lower = d$time_lower,
    time_upper = d$time_upper
  )
  expect_true(is.finite(ll))
})

test_that("Log-logistic log-likelihood supports mixed censoring", {
  d <- .make_mixed_censor_data(50, 12)
  ll <- .hzr_logl_loglogistic(
    theta = c(log_alpha = log(0.6), log_beta = log(1.3)),
    time = d$time,
    status = d$status,
    time_lower = d$time_lower,
    time_upper = d$time_upper
  )
  expect_true(is.finite(ll))
})

test_that("Log-normal log-likelihood supports mixed censoring", {
  d <- .make_mixed_censor_data(50, 13)
  ll <- .hzr_logl_lognormal(
    theta = c(mu = 0.8, log_sigma = log(0.7)),
    time = d$time,
    status = d$status,
    time_lower = d$time_lower,
    time_upper = d$time_upper
  )
  expect_true(is.finite(ll))
})

test_that("hazard() fits exponential with mixed censoring", {
  d <- .make_mixed_censor_data(70, 21)
  fit <- hazard(
    time = d$time,
    status = d$status,
    time_lower = d$time_lower,
    time_upper = d$time_upper,
    theta = c(log_lambda = log(0.5)),
    dist = "exponential",
    fit = TRUE,
    control = list(maxit = 250)
  )

  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$fit$theta), 1)
})

test_that("hazard() fits log-logistic with mixed censoring", {
  d <- .make_mixed_censor_data(70, 22)
  fit <- hazard(
    time = d$time,
    status = d$status,
    time_lower = d$time_lower,
    time_upper = d$time_upper,
    theta = c(log_alpha = log(0.6), log_beta = log(1.2)),
    dist = "loglogistic",
    fit = TRUE,
    control = list(maxit = 300)
  )

  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$fit$theta), 2)
})

test_that("hazard() fits log-normal with mixed censoring", {
  d <- .make_mixed_censor_data(70, 23)
  fit <- hazard(
    time = d$time,
    status = d$status,
    time_lower = d$time_lower,
    time_upper = d$time_upper,
    theta = c(mu = 0.8, log_sigma = log(0.9)),
    dist = "lognormal",
    fit = TRUE,
    control = list(maxit = 300)
  )

  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$fit$theta), 2)
})

test_that("Interval bounds validation is enforced for all distributions", {
  bad_lower <- c(1.5, 2.0)
  bad_upper <- c(1.0, 2.5)
  time <- c(1.2, 2.2)
  status <- c(2, 1)

  expect_true(is.infinite(.hzr_logl_exponential(
    theta = c(log(0.3)),
    time = time,
    status = status,
    time_lower = bad_lower,
    time_upper = bad_upper
  )))

  expect_true(is.infinite(.hzr_logl_loglogistic(
    theta = c(log(0.6), log(1.2)),
    time = time,
    status = status,
    time_lower = bad_lower,
    time_upper = bad_upper
  )))

  expect_true(is.infinite(.hzr_logl_lognormal(
    theta = c(0.8, log(0.7)),
    time = time,
    status = status,
    time_lower = bad_lower,
    time_upper = bad_upper
  )))
})

# tests/testthat/test-weights.R
# Tests for the `weights` argument added in 0.9.4.
#
# The key invariant we check is that Fisher weighting is equivalent to
# row duplication for integer weights:
#
#   logL(theta; weights = w,  data = D)
#     ==  logL(theta; weights = 1,  data = expand(D, w))
#
# where expand(D, w) duplicates row i exactly w_i times.  This verifies
# that weights propagate through every likelihood term (event,
# right-censored, interval, left).  Running the same invariant through
# `hazard(fit = TRUE)` additionally exercises the optimizer, gradient,
# and Hessian paths because any mis-weighting there would show up as a
# different MLE or vcov between the two fits.

make_toy <- function(seed = 17) {
  set.seed(seed)
  n <- 30
  x <- rnorm(n)
  # Weibull-ish times
  u <- runif(n)
  t <- (-log(u) / exp(0.3 * x))^(1 / 1.2)
  # Mix of events, right-censored, and interval-censored
  status <- ifelse(t < 0.6, 1, 0)
  data.frame(time = t, status = status, x = x)
}

expand_rows <- function(df, weights) {
  df[rep(seq_len(nrow(df)), times = weights), , drop = FALSE]
}

# ---------------------------------------------------------------------------
# Argument validation
# ---------------------------------------------------------------------------

test_that("weights must be numeric and the right length", {
  df <- make_toy()
  expect_error(
    hazard(survival::Surv(time, status) ~ x, data = df,
           dist = "weibull", weights = "a"),
    "numeric vector"
  )
  expect_error(
    hazard(survival::Surv(time, status) ~ x, data = df,
           dist = "weibull", weights = rep(1, nrow(df) - 1)),
    "numeric vector"
  )
})

test_that("weights must be non-negative and finite", {
  df <- make_toy()
  expect_error(
    hazard(survival::Surv(time, status) ~ x, data = df,
           dist = "weibull", weights = c(-1, rep(1, nrow(df) - 1))),
    "non-negative"
  )
  expect_error(
    hazard(survival::Surv(time, status) ~ x, data = df,
           dist = "weibull", weights = c(Inf, rep(1, nrow(df) - 1))),
    "non-negative"
  )
})

# ---------------------------------------------------------------------------
# Weighted log-likelihood equals duplicated-row log-likelihood (Weibull)
# ---------------------------------------------------------------------------

test_that("integer weights match row duplication at an arbitrary theta (Weibull)", {
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)
  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)

  ll_weighted <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta,
    time = df$time, status = df$status,
    x = matrix(df$x, ncol = 1),
    weights = w
  )
  ll_expanded <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta,
    time = df_exp$time, status = df_exp$status,
    x = matrix(df_exp$x, ncol = 1),
    weights = rep(1, nrow(df_exp))
  )

  expect_equal(ll_weighted, ll_expanded, tolerance = 1e-10)
})

test_that("integer weights match row duplication for all censoring flavours", {
  set.seed(3)
  n <- 20
  x <- rnorm(n)
  t <- runif(n, 0.1, 2)
  # Mix: 10 events, 5 right-censored, 5 left-censored
  status <- c(rep(1, 10), rep(0, 5), rep(-1, 5))
  # The parser's convention for left-censored data is time_upper = time for
  # every row (non-left rows ignore it). Mirror that here so the LL's
  # `any(upper <= 0)` guard doesn't trip on NAs.
  time_upper <- t
  w <- sample(1:3, n, replace = TRUE)

  theta <- c(mu = 0.7, nu = 1.3, beta = 0.1)
  ll_w <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = t, status = status,
    time_upper = time_upper, x = matrix(x, ncol = 1), weights = w
  )

  idx <- rep(seq_len(n), times = w)
  ll_dup <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = t[idx], status = status[idx],
    time_upper = time_upper[idx], x = matrix(x[idx], ncol = 1),
    weights = rep(1, sum(w))
  )

  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

# ---------------------------------------------------------------------------
# Weighted fit == duplicated-row fit (MLE + vcov)
# ---------------------------------------------------------------------------

test_that("fit with integer weights matches the duplicated-row fit (Weibull)", {
  skip_on_cran()
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)

  # Single-distribution fits only run the optimizer when start values are
  # supplied (hazard()'s `fit && !is.null(theta)` branch), so pass `theta`.
  fit_w <- hazard(
    survival::Surv(time, status) ~ x,
    data = df, dist = "weibull", theta = c(0.3, 1, 0),
    weights = w, fit = TRUE
  )
  fit_dup <- hazard(
    survival::Surv(time, status) ~ x,
    data = df_exp, dist = "weibull", theta = c(0.3, 1, 0),
    fit = TRUE
  )

  # Coefficients should match up to optimizer tolerance.
  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  # Log-likelihood should match tightly.
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

# ---------------------------------------------------------------------------
# Unit weights == unweighted (sanity)
# ---------------------------------------------------------------------------

test_that("unit weights are equivalent to no weights (Weibull)", {
  df <- make_toy()
  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)

  ll_noweight <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status,
    x = matrix(df$x, ncol = 1)
  )
  ll_unit <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status,
    x = matrix(df$x, ncol = 1), weights = rep(1, nrow(df))
  )
  expect_equal(ll_noweight, ll_unit, tolerance = 1e-12)
})

# ---------------------------------------------------------------------------
# Multiphase weighted likelihood equals duplicated-row likelihood
# ---------------------------------------------------------------------------

test_that("integer weights match duplication in multiphase likelihood", {
  skip_on_cran()
  set.seed(11)
  n <- 40
  t <- runif(n, 0.05, 3)
  status <- rbinom(n, 1, 0.4)
  w <- sample(1:2, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  # Start values: [log_mu_early, t_half, nu, m, log_mu_constant]
  theta <- c(-4, log(0.3), 1, 1, -3)

  ll_w <- TemporalHazard:::.hzr_logl_multiphase(
    theta = theta, time = t, status = status,
    phases = phases,
    covariate_counts = c(early = 0L, constant = 0L),
    x_list = list(early = NULL, constant = NULL),
    weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_multiphase(
    theta = theta, time = t[idx], status = status[idx],
    phases = phases,
    covariate_counts = c(early = 0L, constant = 0L),
    x_list = list(early = NULL, constant = NULL),
    weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

# ---------------------------------------------------------------------------
# Gradient is weighted (numerical check on Weibull)
# ---------------------------------------------------------------------------

test_that("weighted log-likelihood gradient equals numerical gradient", {
  skip_if_not_installed("numDeriv")
  df <- make_toy()
  w <- sample(1:3, nrow(df), replace = TRUE)
  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  # Numerical gradient of the weighted LL
  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_weibull(
        theta = th, time = df$time, status = df$status,
        x = xm, weights = w
      )
    },
    theta
  )

  ana_g <- TemporalHazard:::.hzr_gradient_weibull(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )

  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-5)
})

# ---------------------------------------------------------------------------
# Zero weight excludes the row
# ---------------------------------------------------------------------------

test_that("a zero weight drops the row's contribution to the likelihood", {
  df <- make_toy()
  w <- rep(1, nrow(df))
  w[1] <- 0

  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  ll_drop <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time[-1], status = df$status[-1],
    x = xm[-1, , drop = FALSE],
    weights = rep(1, nrow(df) - 1)
  )
  expect_equal(ll_w, ll_drop, tolerance = 1e-12)
})

# ---------------------------------------------------------------------------
# Per-distribution duplication-parity for exp / log-logistic / log-normal
# (wired up in v0.9.6 as part of Phase 4e).
# ---------------------------------------------------------------------------

test_that("integer weights match row duplication (exponential LL)", {
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(log_lambda = log(0.6), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta, time = df$time[idx], status = df$status[idx],
    x = matrix(df$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("integer weights match row duplication (log-logistic LL)", {
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(log_alpha = log(0.6), log_beta = log(1.2), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_loglogistic(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_loglogistic(
    theta = theta, time = df$time[idx], status = df$status[idx],
    x = matrix(df$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("integer weights match row duplication (log-normal LL)", {
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(mu = 0, log_sigma = log(1), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta, time = df$time[idx], status = df$status[idx],
    x = matrix(df$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

# Analytic gradients for exp / log-logistic / log-normal match numerical
# gradients of the weighted LL.  Catches the "accepts formal but never
# applies it" pattern in the analytic score path.

test_that("weighted exponential gradient equals numerical gradient", {
  skip_if_not_installed("numDeriv")
  df <- make_toy()
  w <- sample(1:3, nrow(df), replace = TRUE)
  theta <- c(log_lambda = log(0.6), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_exponential(
        theta = th, time = df$time, status = df$status,
        x = xm, weights = w
      )
    },
    theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_exponential(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-5)
})

test_that("weighted log-logistic gradient equals numerical gradient", {
  skip_if_not_installed("numDeriv")
  df <- make_toy()
  w <- sample(1:3, nrow(df), replace = TRUE)
  theta <- c(log_alpha = log(0.6), log_beta = log(1.2), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_loglogistic(
        theta = th, time = df$time, status = df$status,
        x = xm, weights = w
      )
    },
    theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_loglogistic(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-5)
})

test_that("weighted log-normal gradient equals numerical gradient", {
  skip_if_not_installed("numDeriv")
  df <- make_toy()
  w <- sample(1:3, nrow(df), replace = TRUE)
  theta <- c(mu = 0, log_sigma = log(1), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_lognormal(
        theta = th, time = df$time, status = df$status,
        x = xm, weights = w
      )
    },
    theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_lognormal(
    theta = theta, time = df$time, status = df$status,
    x = xm, weights = w
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-5)
})

# End-to-end: weighted fit == duplicated-row fit for each distribution.

test_that("fit with integer weights matches the duplicated-row fit (exponential)", {
  skip_on_cran()
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)

  fit_w <- hazard(
    survival::Surv(time, status) ~ x, data = df,
    dist = "exponential", theta = c(log(0.6), 0), weights = w, fit = TRUE
  )
  fit_dup <- hazard(
    survival::Surv(time, status) ~ x, data = df_exp,
    dist = "exponential", theta = c(log(0.6), 0), fit = TRUE
  )

  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

test_that("fit with integer weights matches the duplicated-row fit (log-logistic)", {
  skip_on_cran()
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)

  fit_w <- hazard(
    survival::Surv(time, status) ~ x, data = df,
    dist = "loglogistic", theta = c(0, 0.2, 0), weights = w, fit = TRUE
  )
  fit_dup <- hazard(
    survival::Surv(time, status) ~ x, data = df_exp,
    dist = "loglogistic", theta = c(0, 0.2, 0), fit = TRUE
  )

  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

test_that("fit with integer weights matches the duplicated-row fit (log-normal)", {
  skip_on_cran()
  df <- make_toy()
  n <- nrow(df)
  w <- sample(1:3, n, replace = TRUE)
  df_exp <- expand_rows(df, w)

  fit_w <- hazard(
    survival::Surv(time, status) ~ x, data = df,
    dist = "lognormal", theta = c(1, 0, 0), weights = w, fit = TRUE
  )
  fit_dup <- hazard(
    survival::Surv(time, status) ~ x, data = df_exp,
    dist = "lognormal", theta = c(1, 0, 0), fit = TRUE
  )

  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

# ---------------------------------------------------------------------------
# Mixed-censoring (status == -1 and status == 2) duplication-parity for
# exp / log-logistic / log-normal.  The weighted LL now applies row
# weights to the left-censored term `log(1 - exp(-H(u)))` and the
# interval-censored term `log(F(u) - F(l))` as well; these tests
# exercise those newly weighted branches.
# ---------------------------------------------------------------------------

make_mixed <- function(seed = 23) {
  set.seed(seed)
  n <- 24
  x <- rnorm(n)
  t <- runif(n, 0.1, 2.0)
  # 10 events, 4 right-censored, 5 left-censored, 5 interval-censored
  status <- c(rep(1L, 10), rep(0L, 4), rep(-1L, 5), rep(2L, 5))
  # Interval bounds must be strictly positive with lower < upper.
  # Left-censored rows set time_upper = time; non-interval rows set
  # time_lower = time (ignored by the LL except on interval rows).
  time_lower <- t
  time_upper <- t
  idx_iv <- which(status == 2)
  time_lower[idx_iv] <- t[idx_iv]
  time_upper[idx_iv] <- t[idx_iv] + runif(length(idx_iv), 0.05, 0.3)
  list(time = t, status = status, x = x,
       time_lower = time_lower, time_upper = time_upper)
}

test_that("weighted LL matches duplicated-row LL under mixed censoring (exponential)", {
  m <- make_mixed()
  n <- length(m$time)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(log_lambda = log(0.5), beta = 0.2)
  xm <- matrix(m$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta, time = m$time, status = m$status,
    time_lower = m$time_lower, time_upper = m$time_upper,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta,
    time = m$time[idx], status = m$status[idx],
    time_lower = m$time_lower[idx], time_upper = m$time_upper[idx],
    x = matrix(m$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("weighted LL matches duplicated-row LL under mixed censoring (log-logistic)", {
  m <- make_mixed()
  n <- length(m$time)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(log_alpha = log(0.5), log_beta = log(1.1), beta = 0.2)
  xm <- matrix(m$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_loglogistic(
    theta = theta, time = m$time, status = m$status,
    time_lower = m$time_lower, time_upper = m$time_upper,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_loglogistic(
    theta = theta,
    time = m$time[idx], status = m$status[idx],
    time_lower = m$time_lower[idx], time_upper = m$time_upper[idx],
    x = matrix(m$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("weighted LL matches duplicated-row LL under mixed censoring (log-normal)", {
  m <- make_mixed()
  n <- length(m$time)
  w <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)
  theta <- c(mu = 0, log_sigma = log(1), beta = 0.2)
  xm <- matrix(m$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta, time = m$time, status = m$status,
    time_lower = m$time_lower, time_upper = m$time_upper,
    x = xm, weights = w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta,
    time = m$time[idx], status = m$status[idx],
    time_lower = m$time_lower[idx], time_upper = m$time_upper[idx],
    x = matrix(m$x[idx], ncol = 1), weights = rep(1, sum(w))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("weighted multiphase analytic gradient matches numerical gradient", {
  # `.hzr_gradient_multiphase()` must apply `weights` to the per-row score
  # (w_H, inv_h, and the interval-censored finite-difference correction).
  # Prior to 0.9.6 it ignored the argument, so weighted multiphase fits
  # optimized a weighted objective with an unweighted gradient -- the fit
  # still converged near the right MLE via line search but the final
  # gradient norm did not go to zero.  This test would have caught it.
  skip_if_not_installed("numDeriv")

  set.seed(31)
  n <- 40
  t <- runif(n, 0.1, 3)
  status <- rbinom(n, 1, 0.45)
  w <- sample(1:3, n, replace = TRUE)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  cov_counts <- c(early = 0L, constant = 0L)
  x_list <- list(early = NULL, constant = NULL)
  # [log_mu_e, log_thalf, nu, m, log_mu_c]
  theta <- c(-4, log(0.3), 1, 1, -3)

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_multiphase(
        theta = th, time = t, status = status,
        phases = phases, covariate_counts = cov_counts,
        x_list = x_list, weights = w
      )
    },
    theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_multiphase(
    theta = theta, time = t, status = status,
    weights = w,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-4)
})

test_that("zero weight drops the row's contribution across mixed censoring (log-normal)", {
  # Targeted sanity check of the per-term weighting: zeroing one row of each
  # censoring flavour must equal dropping those rows entirely.
  m <- make_mixed()
  n <- length(m$time)
  w <- rep(1, n)
  # Zero out one event, one RC, one left, one interval row.
  w[c(1, 11, 15, 20)] <- 0
  keep <- w > 0
  theta <- c(mu = 0, log_sigma = log(1), beta = 0.2)
  xm <- matrix(m$x, ncol = 1)

  ll_w <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta, time = m$time, status = m$status,
    time_lower = m$time_lower, time_upper = m$time_upper,
    x = xm, weights = w
  )
  ll_drop <- TemporalHazard:::.hzr_logl_lognormal(
    theta = theta,
    time = m$time[keep], status = m$status[keep],
    time_lower = m$time_lower[keep], time_upper = m$time_upper[keep],
    x = xm[keep, , drop = FALSE], weights = rep(1, sum(keep))
  )
  expect_equal(ll_w, ll_drop, tolerance = 1e-12)
})

# ---------------------------------------------------------------------------
# Multiphase WITH covariates: weighted == duplicated (roadmap 7c)
# ---------------------------------------------------------------------------

# Shared fixture: multiphase data with a covariate in BOTH phases, integer
# weights, and the row-duplication index. covariate enters early + constant.
make_mp_cov <- function(seed = 41, n = 40) {
  set.seed(seed)
  t      <- runif(n, 0.05, 3)
  status <- rbinom(n, 1, 0.4)
  x      <- rnorm(n)
  w      <- sample(1:3, n, replace = TRUE)
  idx    <- rep(seq_len(n), times = w)
  list(t = t, status = status, x = x, w = w, idx = idx, n = n)
}

# Phases + theta shared by the direct-LL and gradient tests.
# Layout: [log_mu_e, log_thalf, nu, m, beta_e, log_mu_c, beta_c]
mp_cov_phases <- function() {
  list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
}
mp_cov_theta   <- c(-4, log(0.3), 1, 1, 0.5, -3, -0.3)
mp_cov_counts  <- c(early = 1L, constant = 1L)

test_that("integer weights match duplication in multiphase LL WITH covariates", {
  skip_on_cran()
  d <- make_mp_cov()
  x_list   <- list(early    = matrix(d$x, ncol = 1),
                   constant = matrix(d$x, ncol = 1))
  x_list_d <- list(early    = matrix(d$x[d$idx], ncol = 1),
                   constant = matrix(d$x[d$idx], ncol = 1))

  ll_w <- TemporalHazard:::.hzr_logl_multiphase(
    theta = mp_cov_theta, time = d$t, status = d$status,
    phases = mp_cov_phases(), covariate_counts = mp_cov_counts,
    x_list = x_list, weights = d$w
  )
  ll_dup <- TemporalHazard:::.hzr_logl_multiphase(
    theta = mp_cov_theta, time = d$t[d$idx], status = d$status[d$idx],
    phases = mp_cov_phases(), covariate_counts = mp_cov_counts,
    x_list = x_list_d, weights = rep(1, length(d$idx))
  )
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("weighted multiphase analytic gradient matches numerical WITH covariates", {
  skip_if_not_installed("numDeriv")
  d <- make_mp_cov(seed = 42)
  x_list <- list(early    = matrix(d$x, ncol = 1),
                 constant = matrix(d$x, ncol = 1))

  num_g <- numDeriv::grad(
    function(th) {
      TemporalHazard:::.hzr_logl_multiphase(
        theta = th, time = d$t, status = d$status,
        phases = mp_cov_phases(), covariate_counts = mp_cov_counts,
        x_list = x_list, weights = d$w
      )
    },
    mp_cov_theta
  )
  ana_g <- TemporalHazard:::.hzr_gradient_multiphase(
    theta = mp_cov_theta, time = d$t, status = d$status, weights = d$w,
    phases = mp_cov_phases(), covariate_counts = mp_cov_counts, x_list = x_list
  )
  expect_equal(as.numeric(ana_g), num_g, tolerance = 1e-4)
})

test_that("weighted multiphase fit matches duplicated-row fit, covariates, CoE off", {
  skip_on_cran()
  d  <- make_mp_cov(seed = 43)
  df <- data.frame(time = d$t, status = d$status, x = d$x)
  df_exp <- expand_rows(df, d$w)

  mk_fit <- function(data, weights) {
    hazard(
      survival::Surv(time, status) ~ 1,
      data = data, dist = "multiphase",
      phases = list(
        early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                             fixed = "shapes", formula = ~ x),
        constant = hzr_phase("constant", formula = ~ x)
      ),
      weights = weights, fit = TRUE,
      control = list(conserve = FALSE, n_starts = 1L, maxit = 500L)
    )
  }

  fit_w   <- mk_fit(df,     d$w)
  fit_dup <- mk_fit(df_exp, NULL)

  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

test_that(".hzr_logl_weibull gradient attribute respects weights", {
  # Regression: return_gradient = TRUE previously called .hzr_gradient_weibull
  # without forwarding `weights`, so the attached gradient was unit-weighted
  # even for weighted data.
  skip_if_not_installed("numDeriv")
  set.seed(314)
  n <- 120
  x <- cbind(z = rnorm(n))
  time <- rexp(n, 0.5) + 0.01
  status <- rbinom(n, 1, 0.75)
  w <- runif(n, 0.5, 2.5)
  theta <- c(mu = 0.8, nu = 1.4, z = 0.3)

  ll <- .hzr_logl_weibull(theta, time, status, x = x, weights = w,
                          return_gradient = TRUE)
  g_attr <- attr(ll, "gradient")

  obj <- function(th) {
    .hzr_logl_weibull(th, time, status, x = x, weights = w)
  }
  g_num <- numDeriv::grad(obj, theta)

  expect_equal(g_attr, g_num, tolerance = 1e-5)
})

# ---------------------------------------------------------------------------
# FRACTIONAL (non-integer) weights -- roadmap 7a.
#
# Every test above proves weighting via *integer* row duplication.  Fractional
# weights (e.g. inverse-probability or aggregated-cell weights) cannot be
# checked that way -- you cannot duplicate a row 2.5 times.  Instead we verify
# the two algebraic properties that DEFINE a correctly weighted, per-row
# additive log-likelihood  L(theta; w) = sum_i w_i * l_i(theta):
#
#   (A) ADDITIVE SPLIT.  Splitting one row of weight w_i into two identical
#       copies whose weights sum to w_i leaves L unchanged.  This is the
#       continuous generalisation of duplication -- it reduces to the integer
#       duplication tests when the split weights are 1, and exercises the
#       fractional arithmetic when they are not.  If any term failed to scale
#       linearly in w_i (e.g. a w_i applied inside a log, or a stray w_i^2),
#       the split would not balance.
#   (B) LINEAR SCALING.  L(theta; c*w) = c * L(theta; w) and the gradient
#       scales likewise; the MLE (argmax) is therefore invariant to a positive
#       scalar reweight while the objective scales by c.
#
# Together (A) and (B) pin the fractional-weight path without a SAS fixture --
# a SAS `WEIGHT`-statement parity run (roadmap 7a) remains the external check,
# but these establish internal correctness now.
# ---------------------------------------------------------------------------

# Replace each row i (weight w_i) with two identical copies carrying weights
# w_i * f_i and w_i * (1 - f_i).  Returns the doubled vectors + split weights.
.frac_split <- function(time, status, x, w, f) {
  list(
    time   = c(time, time),
    status = c(status, status),
    x      = rbind(x, x),
    w      = c(w * f, w * (1 - f))
  )
}

test_that("fractional weights: additive split leaves the LL unchanged (Weibull)", {
  df <- make_toy()
  n  <- nrow(df)
  set.seed(71)                      # seed the weight/split draws (make_toy() seeds its own data)
  w  <- runif(n, 0.3, 2.7)          # fractional
  f  <- runif(n, 0.1, 0.9)          # fractional split point per row
  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  s <- .frac_split(df$time, df$status, xm, w, f)

  ll_w <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status, x = xm, weights = w)
  ll_split <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = s$time, status = s$status, x = s$x, weights = s$w)

  expect_equal(ll_w, ll_split, tolerance = 1e-10)
})

test_that("fractional weights: additive split leaves the LL unchanged (exponential)", {
  df <- make_toy()
  n  <- nrow(df)
  set.seed(72)                      # seed the weight/split draws (make_toy() seeds its own data)
  w  <- runif(n, 0.3, 2.7)
  f  <- runif(n, 0.1, 0.9)
  theta <- c(log_lambda = log(0.6), beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  s <- .frac_split(df$time, df$status, xm, w, f)

  ll_w <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta, time = df$time, status = df$status, x = xm, weights = w)
  ll_split <- TemporalHazard:::.hzr_logl_exponential(
    theta = theta, time = s$time, status = s$status, x = s$x, weights = s$w)

  expect_equal(ll_w, ll_split, tolerance = 1e-10)
})

test_that("fractional weights: additive split leaves the LL unchanged (multiphase + covariates)", {
  d <- make_mp_cov(seed = 73)
  set.seed(73)                      # seed the weight/split draws (make_mp_cov() seeds its own data)
  w <- runif(d$n, 0.3, 2.7)
  f <- runif(d$n, 0.1, 0.9)
  xm <- matrix(d$x, ncol = 1)

  x_list   <- list(early = xm, constant = xm)
  x_list_s <- list(early = rbind(xm, xm), constant = rbind(xm, xm))

  ll_w <- TemporalHazard:::.hzr_logl_multiphase(
    theta = mp_cov_theta, time = d$t, status = d$status,
    phases = mp_cov_phases(), covariate_counts = mp_cov_counts,
    x_list = x_list, weights = w)
  ll_split <- TemporalHazard:::.hzr_logl_multiphase(
    theta = mp_cov_theta, time = c(d$t, d$t), status = c(d$status, d$status),
    phases = mp_cov_phases(), covariate_counts = mp_cov_counts,
    x_list = x_list_s, weights = c(w * f, w * (1 - f)))

  expect_equal(ll_w, ll_split, tolerance = 1e-10)
})

test_that("fractional weights: LL and gradient scale linearly in the weights (Weibull)", {
  # No numDeriv needed: this checks the analytic LL/gradient scale exactly in w.
  df <- make_toy()
  set.seed(74)                      # seed the weight draws (make_toy() seeds its own data)
  w  <- runif(nrow(df), 0.3, 2.7)
  cc <- 2.5                         # fractional scale factor
  theta <- c(mu = 0.8, nu = 1.2, beta = 0.25)
  xm <- matrix(df$x, ncol = 1)

  ll_1 <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status, x = xm, weights = w)
  ll_c <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status, x = xm, weights = cc * w)
  expect_equal(ll_c, cc * ll_1, tolerance = 1e-10)

  g_1 <- TemporalHazard:::.hzr_gradient_weibull(
    theta = theta, time = df$time, status = df$status, x = xm, weights = w)
  g_c <- TemporalHazard:::.hzr_gradient_weibull(
    theta = theta, time = df$time, status = df$status, x = xm, weights = cc * w)
  expect_equal(as.numeric(g_c), cc * as.numeric(g_1), tolerance = 1e-10)
})

test_that("fractional weights: the MLE is invariant to a positive scalar reweight (Weibull)", {
  skip_on_cran()
  df <- make_toy()
  set.seed(75)                      # seed the weight draws (make_toy() seeds its own data)
  w  <- runif(nrow(df), 0.3, 2.7)
  cc <- 2.0
  # Single-distribution fits run the optimizer only when start values are
  # supplied (the `fit && !is.null(theta)` branch in hazard()); pass `theta`
  # so this exercises a real fit rather than an unfitted object.
  theta0 <- c(0.3, 1, 0)

  fit_w  <- hazard(survival::Surv(time, status) ~ x, data = df,
                   dist = "weibull", theta = theta0, weights = w,      fit = TRUE)
  fit_cw <- hazard(survival::Surv(time, status) ~ x, data = df,
                   dist = "weibull", theta = theta0, weights = cc * w, fit = TRUE)

  # argmax unchanged; the maximised objective scales by the constant.
  expect_equal(coef(fit_cw), coef(fit_w), tolerance = 5e-3)
  expect_equal(fit_cw$fit$objective / fit_w$fit$objective, cc, tolerance = 1e-4)
})

test_that("fractional weights: additive-split fit matches the original fit (multiphase + covariates)", {
  skip_on_cran()
  d  <- make_mp_cov(seed = 76)
  set.seed(76)                      # seed the weight/split draws (make_mp_cov() seeds its own data)
  w  <- runif(d$n, 0.3, 2.7)
  f  <- runif(d$n, 0.1, 0.9)
  df <- data.frame(time = d$t, status = d$status, x = d$x)
  df_split <- rbind(df, df)
  w_split  <- c(w * f, w * (1 - f))

  mk_fit <- function(data, weights) {
    hazard(
      survival::Surv(time, status) ~ 1,
      data = data, dist = "multiphase",
      phases = list(
        early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                             fixed = "shapes", formula = ~ x),
        constant = hzr_phase("constant", formula = ~ x)
      ),
      weights = weights, fit = TRUE,
      control = list(conserve = FALSE, n_starts = 1L, maxit = 500L)
    )
  }

  fit_w     <- suppressWarnings(mk_fit(df,       w))
  fit_split <- suppressWarnings(mk_fit(df_split, w_split))

  expect_equal(coef(fit_w), coef(fit_split), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_split$fit$objective, tolerance = 1e-4)
})

# ---------------------------------------------------------------------------
# Conservation of Events under left truncation (counting-process entry times).
#
# The multiphase log-likelihood scores mu via Sum E = Sum [H(stop) - H(start)]
# for left-truncated rows (Surv(start, stop, event)). The Conservation-of-Events
# reparameterization must solve the SAME constraint, i.e. it must subtract the
# entry-time cumulative hazard H(start). If CoE instead conserves Sum H(stop)
# (ignoring the truncation term), it constrains the fit to the wrong manifold
# and the attained maximum log-likelihood drops below the unconstrained fit.
#
# Invariant exploited here: CoE only removes a redundant parameter direction, so
# objective(conserve = TRUE) must equal objective(conserve = FALSE). This holds
# trivially for untruncated data (start = 0) and is the discriminating check for
# truncated data. This is the synthetic analogue of the hz.te123.OMC fit-1 gap
# (gap-list P1 #6).
# ---------------------------------------------------------------------------

# Simulate a fittable two-phase data set with optional left truncation. The
# exact data-generating process is unimportant; what matters is that (a) both
# the CoE and free fits converge and (b) entry times are a non-negligible
# fraction of follow-up so the omitted H(start) term is material.
.sim_coe_lt <- function(n = 600, seed = 101, truncate = TRUE) {
  set.seed(seed)
  u <- runif(n)
  # Early burst of events plus a long constant-ish tail -> two phases.
  t_event <- ifelse(u < 0.5, rexp(n, rate = 3), rexp(n, rate = 0.4))
  cens    <- runif(n, 0.2, 6)
  stop    <- pmin(t_event, cens)
  event   <- as.integer(t_event <= cens)
  start   <- if (truncate) pmin(runif(n, 0, 1.0), 0.8 * stop) else rep(0, n)
  data.frame(start = start, stop = stop, event = event)
}

.coe_phases <- function() {
  list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
}

.coe_fit <- function(d, conserve, seed = 202) {
  # Reset the RNG before fitting so the multi-start perturbations are
  # deterministic: the CoE and free fits must not drift into different local
  # optima from back-to-back RNG draws (which would make the objective
  # comparison flaky across platforms/R versions).
  set.seed(seed)
  hazard(
    survival::Surv(start, stop, event) ~ 1,
    data    = d,
    dist    = "multiphase",
    phases  = .coe_phases(),
    fit     = TRUE,
    control = list(n_starts = 5, maxit = 1500, reltol = 1e-9, conserve = conserve)
  )
}

# Recompute the multiphase log-likelihood at a fitted object's returned
# coefficients (no further CoE applied). A self-consistent fit must reproduce
# its own reported objective here.
.coe_ll_at_par <- function(fit, d) {
  TemporalHazard:::.hzr_logl_multiphase(
    theta = fit$fit$theta, time = d$stop, status = d$event,
    time_lower = d$start, phases = fit$fit$phases,
    covariate_counts = fit$fit$covariate_counts, x_list = fit$fit$x_list
  )
}

test_that("CoE conserves events on the entry-time scale (left-truncated)", {
  skip_on_cran()
  skip_if_not_installed("survival")

  d <- .sim_coe_lt(truncate = TRUE)
  fit_coe  <- .coe_fit(d, conserve = TRUE)
  fit_free <- .coe_fit(d, conserve = FALSE)

  expect_true(isTRUE(fit_coe$fit$converged))
  expect_true(isTRUE(fit_free$fit$converged))

  # A correct CoE cannot lower the attainable maximum log-likelihood.
  expect_equal(fit_coe$fit$objective, fit_free$fit$objective,
               tolerance = 1e-3,
               label = "objective(conserve=TRUE) vs conserve=FALSE, left-truncated")

  # The returned coefficients must be self-consistent with the objective: the
  # final post-optimization CoE adjustment has to solve the conserved log_mu on
  # the same entry-time scale, else the returned mu drifts off the optimum.
  expect_equal(.coe_ll_at_par(fit_coe, d), fit_coe$fit$objective,
               tolerance = 1e-6,
               label = "LL at returned coefficients vs reported objective")
})

test_that("CoE matches the free fit for untruncated data (control)", {
  skip_on_cran()
  skip_if_not_installed("survival")

  d <- .sim_coe_lt(truncate = FALSE)
  fit_coe  <- .coe_fit(d, conserve = TRUE)
  fit_free <- .coe_fit(d, conserve = FALSE)

  expect_equal(fit_coe$fit$objective, fit_free$fit$objective,
               tolerance = 1e-3,
               label = "objective(conserve=TRUE) vs conserve=FALSE, untruncated")
})

#' @keywords internal
NULL

# predict-cl.R -- Delta-method confidence limits for predict.hazard()
#
# PROBLEM
# -------
# predict.hazard() returns a point estimate eta(theta) for each row.  We want
# a standard error and a (1 - alpha) confidence interval around each one,
# matching SAS HAZARD's `hzp_calc_haz_CL.c` / `hzp_calc_srv_CL.c`.
#
# METHOD
# ------
# Delta method: if theta_hat ~ N(theta, V) then any smooth function eta(theta)
# is approximately Gaussian with
#
#   var(eta_hat)  ~  (d eta/d theta)^T  V  (d eta/d theta).
#
# For a batch of n prediction points, we assemble J (n x p) where
# J[i, ] = d eta_i / d theta, then
#
#   se_i  =  sqrt( (J V J^T)_{ii} )  =  sqrt( rowSums( (J V) * J ) ).
#
# SCALE CHOICE
# ------------
# Symmetric CLs on the natural scale can leave a [0, 1] survival or a
# [0, Inf) hazard, so we build them on a transformed scale and back-transform:
#
#   hazard / cumhaz   -> log scale:
#     se(log eta)  =  se(eta) / eta,
#     [eta * exp(-z se_log),  eta * exp(+z se_log)]
#
#   survival          -> log-log scale (log(-log S) = log H):
#     se(log(-log S))  =  se(H) / H,
#     S_lwr  =  exp(-H * exp(+z se_logH)),
#     S_upr  =  exp(-H * exp(-z se_logH))
#
#   linear_predictor  -> natural scale (symmetric):
#     [eta - z se, eta + z se]
#
# ENTRY POINT
# -----------
# `.hzr_predict_with_se()` takes a fitted `hazard` object, a type, and the
# materialised `x` / `time` for prediction.  It dispatches to the appropriate
# distribution-specific jacobian builder, runs the delta-method sandwich, and
# returns a list(fit, se.fit, lower, upper).

# ---------------------------------------------------------------------------
# Jacobian: Weibull
# ---------------------------------------------------------------------------
#
# Parameter layout (natural scale, matching .hzr_logl_weibull):
#   theta = [mu, nu, beta_1, ..., beta_p]
#
# H(t|x) = (mu t)^nu exp(eta),   h_rel = exp(eta),   eta = x beta.
#
# Analytical derivatives used below:
#   dH/dmu       = (nu / mu) H
#   dH/dnu       = log(mu t) H
#   dH/dbeta_j   = x_ij H
#
#   d exp(eta)/dbeta_j = x_ij exp(eta)    (other entries are 0)
#
#   eta itself is linear: d eta / d beta_j = x_ij, rest 0.

#' Jacobian of Weibull predictions with respect to theta
#'
#' @param type Prediction type (see `.hzr_predict_with_se`).
#' @param theta MLE parameter vector `c(mu, nu, beta_1, ...)`.
#' @param time Prediction times (may be NULL for `hazard` / `linear_predictor`).
#' @param x Design matrix (n x p_cov) or NULL.
#' @param p Length of theta.
#' @return Numeric n x p Jacobian.
#' @keywords internal
.hzr_predict_jacobian_weibull <- function(type, theta, time, x, p) {
  n_shape <- 2L
  mu <- theta[1]
  nu <- theta[2]
  beta <- if (p > n_shape) theta[(n_shape + 1L):p] else numeric(0)

  if (!is.null(x) && length(beta) > 0L) {
    eta <- as.numeric(x %*% beta)
  } else if (!is.null(x)) {
    eta <- rep(0, nrow(x))
  } else {
    # No covariates: treat all rows as a single reference
    eta <- if (!is.null(time)) rep(0, length(time)) else 0
  }
  n <- length(eta)
  J <- matrix(0, nrow = n, ncol = p)

  if (type %in% c("cumulative_hazard", "survival")) {
    H <- (mu * time) ^ nu * exp(eta)
    J[, 1L] <- (nu / mu) * H
    J[, 2L] <- log(mu * time) * H
    if (length(beta) > 0L && !is.null(x)) {
      J[, (n_shape + 1L):p] <- x * H
    }
  } else if (type == "hazard") {
    # Current API returns relative hazard = exp(eta); shape params drop out.
    h_rel <- exp(eta)
    if (length(beta) > 0L && !is.null(x)) {
      J[, (n_shape + 1L):p] <- x * h_rel
    }
  } else if (type == "linear_predictor") {
    if (length(beta) > 0L && !is.null(x)) {
      J[, (n_shape + 1L):p] <- x
    }
  }

  J
}

# ---------------------------------------------------------------------------
# Jacobian: multiphase
# ---------------------------------------------------------------------------
#
# H(t|x) = sum_j mu_j(x) Phi_j(t),   mu_j(x) = exp(log_mu_j + x_j beta_j).
#
# For each phase j, its sub-vector in theta contains (in order):
#   log_mu_j, (log_t_half_j, nu_j, m_j) or (log_tau_j, gamma_j, alpha_j, eta_j)
#   or () for a "constant" phase, then beta_j covariates.
#
# Derivatives:
#   dH/d log_mu_j    = mu_j Phi_j
#   dH/d shape_s_j   = mu_j dPhi_j/d shape_s     (from .hzr_phase_derivatives
#                                                 / .hzr_g3_phase_derivatives)
#   dH/d beta_jk     = x_ik mu_j Phi_j

#' Jacobian of multiphase cumulative-hazard predictions
#'
#' Only `cumulative_hazard` and `survival` are delegated here -- `hazard`
#' and `linear_predictor` are rejected upstream for multiphase models.
#'
#' @param theta MLE parameter vector.
#' @param time Prediction times (length n).
#' @param phases Named list of `hzr_phase` objects.
#' @param covariate_counts Named integer vector.
#' @param x_list Named list of per-phase design matrices.
#' @param p Length of theta.
#' @param per_phase Logical; if `TRUE`, return a named list of per-phase
#'   `n x p` Jacobians (each with only that phase's columns nonzero) instead
#'   of their sum.
#' @return Numeric `n x p` Jacobian of `H(t|x)` (default), or a named list of
#'   per-phase `n x p` Jacobians when `per_phase = TRUE`.
#' @keywords internal
.hzr_predict_jacobian_multiphase <- function(theta, time, phases,
                                              covariate_counts, x_list, p,
                                              per_phase = FALSE) {
  n <- length(time)
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  # One n x p matrix per phase; each phase fills only its own columns.
  J_list <- stats::setNames(
    lapply(seq_along(phases), function(i) matrix(0, nrow = n, ncol = p)),
    names(phases)
  )

  pos <- 1L
  for (nm in names(phases)) {
    ph <- phases[[nm]]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], ph)
    Jp <- J_list[[nm]]

    # mu_j(x_i)
    if (length(pars$beta) > 0L && !is.null(x_list[[nm]])) {
      eta_j <- pars$log_mu + as.numeric(x_list[[nm]] %*% pars$beta)
    } else {
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)

    # Phi_j and shape derivatives
    if (ph$type == "constant") {
      Phi_j <- time
      Jp[, pos] <- mu_j * Phi_j
      pos <- pos + 1L
    } else if (ph$type == "g3") {
      tau_j <- exp(pars$log_tau)
      pd <- .hzr_g3_phase_derivatives(time, tau = tau_j,
                                        gamma = pars$gamma,
                                        alpha = pars$alpha,
                                        eta = pars$eta)
      Phi_j <- pd$Phi
      Jp[, pos] <- mu_j * Phi_j
      Jp[, pos + 1L] <- mu_j * pd$dPhi_dlog_tau
      Jp[, pos + 2L] <- mu_j * pd$dPhi_dgamma
      Jp[, pos + 3L] <- mu_j * pd$dPhi_dalpha
      Jp[, pos + 4L] <- mu_j * pd$dPhi_deta
      pos <- pos + 5L
    } else {
      t_half_j <- exp(pars$log_t_half)
      pd <- .hzr_phase_derivatives(time, t_half = t_half_j,
                                    nu = pars$nu, m = pars$m,
                                    type = ph$type)
      Phi_j <- pd$Phi
      Jp[, pos] <- mu_j * Phi_j
      Jp[, pos + 1L] <- mu_j * pd$dPhi_dlog_thalf
      Jp[, pos + 2L] <- mu_j * pd$dPhi_dnu
      Jp[, pos + 3L] <- mu_j * pd$dPhi_dm
      pos <- pos + 4L
    }

    # Covariate betas
    n_beta <- covariate_counts[[nm]]
    if (n_beta > 0L && !is.null(x_list[[nm]])) {
      x_phase <- x_list[[nm]]
      for (k in seq_len(n_beta)) {
        Jp[, pos] <- x_phase[, k] * mu_j * Phi_j
        pos <- pos + 1L
      }
    } else if (n_beta > 0L) {
      pos <- pos + n_beta
    }

    J_list[[nm]] <- Jp
  }

  if (per_phase) {
    return(J_list)
  }
  Reduce(`+`, J_list)
}

# ---------------------------------------------------------------------------
# Numeric jacobian fallback for exp / loglogistic / lognormal
# ---------------------------------------------------------------------------
#
# Rather than hand-code each distribution's closed form, we differentiate the
# prediction function numerically through the existing single-distribution
# cumhaz/survival formulas in predict.hazard().  The prediction function is
# built once per call and returns a length-n vector for any candidate theta.

#' Numeric Jacobian of a single-distribution prediction w.r.t. theta
#'
#' @param predict_fn Function(theta) -> numeric vector of length n.
#' @param theta MLE parameter vector.
#' @return Numeric n x p Jacobian.
#' @keywords internal
.hzr_predict_jacobian_numeric <- function(predict_fn, theta) {
  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(numDeriv::jacobian(predict_fn, theta))
  }
  eps <- (.Machine$double.eps) ^ (1 / 3)
  p <- length(theta)
  fit <- predict_fn(theta)
  n <- length(fit)
  J <- matrix(0, nrow = n, ncol = p)
  for (j in seq_len(p)) {
    h <- eps * max(abs(theta[j]), 1)
    tp <- theta
    tm <- theta
    tp[j] <- tp[j] + h
    tm[j] <- tm[j] - h
    J[, j] <- (predict_fn(tp) - predict_fn(tm)) / (2 * h)
  }
  J
}

# ---------------------------------------------------------------------------
# Delta-method sandwich + scale transform
# ---------------------------------------------------------------------------

#' Per-row standard errors via the delta method
#'
#' @param J Jacobian (n x p).
#' @param vcov p x p variance-covariance matrix.
#' @return Numeric vector of length n.
#' @keywords internal
.hzr_predict_se_from_jacobian <- function(J, vcov) {
  JV <- J %*% vcov
  se <- sqrt(pmax(rowSums(JV * J), 0))
  se
}

#' Apply a scale transform + back-transform for CLs
#'
#' @param fit Point estimate (numeric vector length n).
#' @param se_nat SE on the natural scale (numeric vector length n).
#' @param level Confidence level.
#' @param scale One of "natural", "log", "loglog_survival".
#' @return data.frame with columns fit, se.fit, lower, upper.
#' @keywords internal
.hzr_predict_cl_from_se <- function(fit, se_nat, level, scale) {
  alpha <- 1 - level
  z <- stats::qnorm(1 - alpha / 2)

  lower <- rep(NA_real_, length(fit))
  upper <- rep(NA_real_, length(fit))

  if (scale == "natural") {
    lower <- fit - z * se_nat
    upper <- fit + z * se_nat
  } else if (scale == "log") {
    # eta > 0 required; any non-positive fit gets NA CLs.
    pos <- is.finite(fit) & fit > 0
    se_log <- ifelse(pos, se_nat / abs(fit), NA_real_)
    lower[pos] <- fit[pos] * exp(-z * se_log[pos])
    upper[pos] <- fit[pos] * exp(z * se_log[pos])
  } else if (scale == "loglog_survival") {
    # fit is S = exp(-H); se_nat is the SE of H (not S).  Build CLs on
    # log(-log S) = log H.
    #   S_lwr = exp(-H * exp(+z * se(log H)))
    #   S_upr = exp(-H * exp(-z * se(log H)))
    # The caller passes `fit = S` and `se_nat = se(H)`; we reconstruct
    # H = -log(S) to compute se(log H) = se(H) / H.
    H <- -log(pmin(pmax(fit, .Machine$double.xmin), 1))
    pos <- is.finite(H) & H > 0
    se_logH <- ifelse(pos, se_nat / H, NA_real_)
    lower[pos] <- exp(-H[pos] * exp(z * se_logH[pos]))
    upper[pos] <- exp(-H[pos] * exp(-z * se_logH[pos]))
  } else if (scale == "logit_survival") {
    # SAS HAZPRED's survival CL: build on the logit of cumulative incidence,
    # Z = logit(1 - S) = log(e^H - 1), with se(Z) = se(H) / (1 - S), then
    # back-transform S = 1 / (e^Z + 1).  `fit` is S = exp(-H); `se_nat` is
    # se(H).  (Reproduces hzp_calc_srv_CL.c.)
    H <- -log(pmin(pmax(fit, .Machine$double.xmin), 1))
    Fc <- 1 - fit
    pos <- is.finite(H) & H > 0 & Fc > 0
    Z <- ifelse(pos, log(expm1(H)), NA_real_)
    seZ <- ifelse(pos, se_nat / Fc, NA_real_)
    # S = 1 / (e^Z + 1) = plogis(-Z); use plogis for a numerically stable
    # logistic back-transform (avoids exp() overflow at large |Z +/- z*seZ|).
    lower[pos] <- stats::plogis(-(Z[pos] + z * seZ[pos]))
    upper[pos] <- stats::plogis(-(Z[pos] - z * seZ[pos]))
  } else {
    stop("Unknown CL scale: '", scale, "'.", call. = FALSE)
  }

  data.frame(fit = fit, se.fit = se_nat, lower = lower, upper = upper)
}

# ---------------------------------------------------------------------------
# Free-parameter vcov helper (shared by aggregate and decomposed se.fit paths)
# ---------------------------------------------------------------------------

#' Free-parameter vcov submatrix for the delta-method sandwich
#'
#' Fixed parameters (e.g. `fixed = "shapes"`) leave NA rows/cols in the expanded
#' vcov. Treat them as known-with-zero-variance: restrict the sandwich to the
#' free submatrix. (The CoE-conserved `log_mu` normally participates -- CoE fits
#' use the full-information vcov -- but it leaves an NA row, like a fixed
#' parameter, when that recomputation was unavailable.) Returns `NULL` (with a
#' warning) when CLs cannot be computed. Shared by the aggregate and decomposed
#' se.fit paths.
#'
#' @param vcov_mat The fitted vcov (or NULL / wrong shape).
#' @param p Length of the parameter vector.
#' @return `list(vcov_use, free_idx)`, or `NULL` if unusable.
#' @keywords internal
.hzr_free_vcov <- function(vcov_mat, p) {
  vcov_ok <- !is.null(vcov_mat) && is.matrix(vcov_mat) &&
               nrow(vcov_mat) == p && ncol(vcov_mat) == p
  if (!vcov_ok) {
    warning("Variance-covariance matrix is unavailable; ",
            "standard errors and CLs will be NA.", call. = FALSE)
    return(NULL)
  }
  free_idx <- which(is.finite(diag(vcov_mat)))
  if (length(free_idx) < p) {
    free_submat <- vcov_mat[free_idx, free_idx, drop = FALSE]
    if (anyNA(free_submat)) {
      warning("Variance-covariance matrix has NA entries outside the ",
              "fixed-parameter rows/cols; standard errors and CLs will be NA.",
              call. = FALSE)
      return(NULL)
    }
    return(list(vcov_use = free_submat, free_idx = free_idx))
  }
  if (anyNA(vcov_mat)) {
    warning("Variance-covariance matrix has NA entries; ",
            "standard errors and CLs will be NA.", call. = FALSE)
    return(NULL)
  }
  list(vcov_use = vcov_mat, free_idx = free_idx)
}

# ---------------------------------------------------------------------------
# Public entry point
# ---------------------------------------------------------------------------

#' Compute point predictions + delta-method CLs
#'
#' Dispatched from predict.hazard() when `se.fit = TRUE`.
#'
#' DELTA-METHOD TARGET
#' -------------------
#' The quantity we actually differentiate depends on the prediction type and
#' the CL scale the type uses:
#'
#'   type = "cumulative_hazard" -> target = H, log-scale CL
#'   type = "survival"          -> target = H, log-log-survival CL
#'                                 (final `fit` is exp(-H))
#'   type = "hazard"            -> target = exp(eta) (single-dist only),
#'                                 log-scale CL
#'   type = "linear_predictor"  -> target = eta, natural-scale CL
#'
#' The caller supplies `diff_fn(theta)` that returns the target vector of
#' length n.  For Weibull and multiphase we build J analytically; for
#' exp / loglogistic / lognormal we fall through to a numeric jacobian of
#' `diff_fn`.
#'
#' @param object Fitted `hazard` object.
#' @param type One of "hazard", "linear_predictor", "survival",
#'   "cumulative_hazard".
#' @param time Numeric prediction times or NULL (for type = "hazard" /
#'   "linear_predictor").
#' @param x Design matrix (single-distribution) or NULL.  Ignored for
#'   multiphase; use `x_list` instead.
#' @param x_list Named list of per-phase design matrices (multiphase only).
#' @param cov_counts Named integer covariate counts (multiphase only).
#' @param phases Named list of `hzr_phase` objects (multiphase only).
#' @param level Confidence level (default 0.95).
#' @param diff_fn Required function(theta) -> numeric vector of length n
#'   returning the delta-method target (H, exp(eta), or eta depending on
#'   `type`).  Used for the point estimate AND for the numeric jacobian
#'   fallback in exp / loglogistic / lognormal.
#' @param conf_type Survival CL transform: `"log-log"` (default, on
#'   `log(-log S)`) or `"logit"` (on `logit(1 - S)`, HAZPRED-compatible).
#'   Only consulted when `type == "survival"`.
#' @return data.frame with columns `fit`, `se.fit`, `lower`, `upper`.
#' @keywords internal
.hzr_predict_with_se <- function(object, type, time = NULL,
                                   x = NULL, x_list = NULL,
                                   cov_counts = NULL, phases = NULL,
                                   level = 0.95, diff_fn,
                                   conf_type = c("log-log", "logit")) {
  # Only survival CLs use conf_type; validate only then so an ignored value
  # (hazard / cumulative_hazard / linear_predictor) is not an error.
  if (type == "survival") conf_type <- match.arg(conf_type)

  theta <- object$fit$theta
  p <- length(theta)
  dist <- object$spec$dist
  target <- diff_fn(theta)

  fv <- .hzr_free_vcov(object$fit$vcov, p)
  if (is.null(fv)) {
    n <- length(target)
    fit <- if (type == "survival") exp(-target) else target
    na_vec <- rep(NA_real_, n)
    return(data.frame(fit = fit, se.fit = na_vec,
                      lower = na_vec, upper = na_vec))
  }
  vcov_use <- fv$vcov_use
  free_idx <- fv$free_idx

  # --- Build Jacobian of the delta-method target ---------------------------
  if (dist == "weibull") {
    J <- .hzr_predict_jacobian_weibull(type, theta, time, x, p)
  } else if (dist == "multiphase") {
    # The analytic multiphase Jacobian is for the cumulative hazard; the
    # instantaneous hazard has no analytic Jacobian here, so fall back to a
    # numeric Jacobian of its evaluator. (linear_predictor is rejected
    # upstream for multiphase.)
    if (type == "hazard") {
      J <- .hzr_predict_jacobian_numeric(diff_fn, theta)
    } else {
      J <- .hzr_predict_jacobian_multiphase(theta, time, phases,
                                              cov_counts, x_list, p)
    }
  } else {
    J <- .hzr_predict_jacobian_numeric(diff_fn, theta)
  }

  # Restrict J to the free columns so the sandwich dimensions match.
  J <- J[, free_idx, drop = FALSE]

  # --- Sandwich -------------------------------------------------------------
  se_target <- .hzr_predict_se_from_jacobian(J, vcov_use)

  # --- Scale transform -----------------------------------------------------
  if (type == "survival") {
    fit <- exp(-target)
    surv_scale <- if (conf_type == "logit") "logit_survival" else "loglog_survival"
    .hzr_predict_cl_from_se(fit, se_target, level, surv_scale)
  } else if (type == "cumulative_hazard" || type == "hazard") {
    .hzr_predict_cl_from_se(target, se_target, level, "log")
  } else if (type == "linear_predictor") {
    .hzr_predict_cl_from_se(target, se_target, level, "natural")
  } else {
    stop("Unknown prediction type '", type, "' for se.fit.", call. = FALSE)
  }
}

#' Per-phase + total cumulative-hazard predictions with delta-method CLs
#'
#' Long-format companion to [.hzr_predict_with_se()] for multiphase models under
#' `decompose = TRUE`. Computes log-scale CLs for the total cumulative hazard
#' and for each phase's additive contribution `H_j(t) = mu_j(x) Phi_j(t)`.
#' Per-phase CLs use only that phase's parameter block, so they do NOT sum to
#' the total CL (cross-phase covariance contributes only to the total).
#'
#' @param object Fitted multiphase `hazard` object.
#' @param time Numeric prediction times (length n).
#' @param x_list Named list of per-phase design matrices.
#' @param cov_counts Named integer covariate counts.
#' @param phases Named list of `hzr_phase` objects.
#' @param level Confidence level.
#' @return Long `data.frame(time, component, fit, se.fit, lower, upper)`;
#'   `component` is an ordered factor with levels `c("total", names(phases))`.
#' @keywords internal
.hzr_predict_with_se_decomposed <- function(object, time, x_list,
                                             cov_counts, phases,
                                             level = 0.95) {
  theta <- object$fit$theta
  p <- length(theta)
  components <- c("total", names(phases))

  # Point estimates: total + per-phase cumulative hazard.
  ph <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list,
                               per_phase = TRUE)
  fit_by_comp <- c(list(total = ph$total),
                   stats::setNames(lapply(names(phases), function(nm) ph[[nm]]),
                                   names(phases)))

  make_long <- function(cl_list) {
    rows <- lapply(components, function(cmp) {
      data.frame(time = time, component = cmp,
                 fit = cl_list[[cmp]]$fit, se.fit = cl_list[[cmp]]$se.fit,
                 lower = cl_list[[cmp]]$lower, upper = cl_list[[cmp]]$upper,
                 stringsAsFactors = FALSE)
    })
    res <- do.call(rbind, rows)
    res$component <- factor(res$component, levels = components, ordered = TRUE)
    rownames(res) <- NULL
    res
  }

  fv <- .hzr_free_vcov(object$fit$vcov, p)
  if (is.null(fv)) {
    na_cl <- lapply(components, function(cmp) {
      n <- length(time)
      data.frame(fit = fit_by_comp[[cmp]], se.fit = rep(NA_real_, n),
                 lower = rep(NA_real_, n), upper = rep(NA_real_, n))
    })
    names(na_cl) <- components
    return(make_long(na_cl))
  }

  # Per-phase Jacobians; the total is their sum.
  J_list <- .hzr_predict_jacobian_multiphase(theta, time, phases, cov_counts,
                                             x_list, p, per_phase = TRUE)
  J_by_comp <- c(list(total = Reduce(`+`, J_list)), J_list)

  cl_list <- lapply(components, function(cmp) {
    J <- J_by_comp[[cmp]][, fv$free_idx, drop = FALSE]
    se <- .hzr_predict_se_from_jacobian(J, fv$vcov_use)
    .hzr_predict_cl_from_se(fit_by_comp[[cmp]], se, level, "log")
  })
  names(cl_list) <- components
  make_long(cl_list)
}

# tests/testthat/test-predict-cl.R
# Phase 4g -- predict.hazard(..., se.fit = TRUE, level = 0.95)
#
# Coverage:
# (1) API contract: output shape, level validation, decompose incompat
# (2) Jacobian parity: analytic vs numeric for Weibull + multiphase
# (3) Delta-method scale: log-CL for hazard/cumhaz, log-log for survival
# (4) Monotonicity: lower <= fit <= upper
# (5) Survival CLs stay in [0, 1]; cumhaz CLs stay in [0, Inf)
# (6) `vcov` NA (non-converged fit): returns NA CLs with a warning
# (7) Fixed shapes / CoE: free-parameter submatrix recovers meaningful CLs
# (8) Default of `se.fit = FALSE` is byte-identical to pre-0.9.8 predict()

make_toy <- function(seed = 17, n = 60) {
  set.seed(seed)
  data.frame(
    time = rexp(n, 0.3),
    status = rbinom(n, 1, 0.6),
    x = rnorm(n)
  )
}

# ---------------------------------------------------------------------------
# (1) API contract
# ---------------------------------------------------------------------------

test_that("se.fit returns a data frame with fit/se.fit/lower/upper", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2), x = 0),
                 type = "survival", se.fit = TRUE)
  expect_s3_class(res, "data.frame")
  expect_named(res, c("fit", "se.fit", "lower", "upper"))
  expect_equal(nrow(res), 3L)
  expect_true(all(is.finite(res$fit)))
  expect_true(all(is.finite(res$se.fit)))
  expect_true(all(res$se.fit >= 0))
})

test_that("invalid level triggers a clean error", {
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  expect_error(
    predict(fit, newdata = data.frame(time = 1, x = 0),
            type = "survival", se.fit = TRUE, level = 1.1),
    "level"
  )
  expect_error(
    predict(fit, newdata = data.frame(time = 1, x = 0),
            type = "survival", se.fit = TRUE, level = 0),
    "level"
  )
})

test_that("decompose + se.fit: cumulative_hazard works, survival errors", {
  skip_on_cran()
  df <- make_toy()
  phases <- list(early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                                     fixed = "shapes"),
                 constant = hzr_phase("constant"))
  fit <- hazard(survival::Surv(time, status) ~ 1, data = df,
                dist = "multiphase", phases = phases, fit = TRUE,
                control = list(n_starts = 2))
  t_new <- c(0.5, 1, 2)

  # cumulative_hazard: long per-phase + total frame.
  res <- predict(fit, newdata = data.frame(time = t_new),
                 type = "cumulative_hazard", se.fit = TRUE, decompose = TRUE)
  expect_named(res, c("time", "component", "fit", "se.fit", "lower", "upper"))
  expect_equal(levels(res$component), c("total", "early", "constant"))
  expect_equal(nrow(res), length(t_new) * 3L)

  # Total rows reproduce the aggregate se.fit output exactly.
  agg <- predict(fit, newdata = data.frame(time = t_new),
                 type = "cumulative_hazard", se.fit = TRUE, decompose = FALSE)
  tot <- res[res$component == "total", ]
  expect_equal(tot$fit, agg$fit, tolerance = 1e-10)
  expect_equal(tot$se.fit, agg$se.fit, tolerance = 1e-10)
  expect_equal(tot$lower, agg$lower, tolerance = 1e-10)
  expect_equal(tot$upper, agg$upper, tolerance = 1e-10)

  # Per-phase point estimates reproduce the decompose=TRUE, se.fit=FALSE wide
  # cumhaz columns.
  wide <- predict(fit, newdata = data.frame(time = t_new),
                  type = "cumulative_hazard", se.fit = FALSE, decompose = TRUE)
  expect_equal(res[res$component == "early", "fit"], wide$early, tolerance = 1e-10)
  expect_equal(res[res$component == "constant", "fit"], wide$constant,
               tolerance = 1e-10)

  # component is an ordered factor (documented semantics).
  expect_true(is.ordered(res$component))

  # survival still rejected for multiphase.
  expect_error(
    predict(fit, newdata = data.frame(time = t_new),
            type = "survival", se.fit = TRUE, decompose = TRUE),
    "cumulative_hazard"
  )
})

test_that("decompose is ignored (not errored) for single-distribution se.fit", {
  # `decompose` applies only to multiphase models. A single-distribution fit
  # must not error on decompose = TRUE -- it returns the standard aggregate
  # se.fit frame regardless of type (decompose silently ignored).
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2), x = 0),
                 type = "survival", se.fit = TRUE, decompose = TRUE)
  expect_named(res, c("fit", "se.fit", "lower", "upper"))
  expect_equal(nrow(res), 3L)
})

test_that("decompose + se.fit per-phase SE matches a numeric jacobian", {
  skip_if_not_installed("numDeriv")
  skip_on_cran()
  df <- make_toy()
  phases <- list(early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                                     fixed = "shapes"),
                 constant = hzr_phase("constant"))
  fit <- hazard(survival::Surv(time, status) ~ 1, data = df,
                dist = "multiphase", phases = phases, fit = TRUE,
                control = list(n_starts = 2))
  t_new <- c(0.5, 1, 2)
  cov_counts <- c(early = 0L, constant = 0L)
  x_list <- list(early = NULL, constant = NULL)
  theta <- fit$fit$theta
  V <- fit$fit$vcov
  free_idx <- which(is.finite(diag(V)))

  res <- predict(fit, newdata = data.frame(time = t_new),
                 type = "cumulative_hazard", se.fit = TRUE, decompose = TRUE)

  # Numeric per-phase SE for the "early" component via numDeriv.
  H_early <- function(th) {
    TemporalHazard:::.hzr_multiphase_cumhaz(
      t_new, th, phases, cov_counts, x_list, per_phase = TRUE
    )$early
  }
  Jn <- numDeriv::jacobian(H_early, theta)[, free_idx, drop = FALSE]
  se_num <- sqrt(diag(Jn %*% V[free_idx, free_idx, drop = FALSE] %*% t(Jn)))
  se_ana <- res[res$component == "early", "se.fit"]
  expect_equal(se_ana, se_num, tolerance = 1e-5)
})

# ---------------------------------------------------------------------------
# (2) Jacobian parity: analytic vs numeric
# ---------------------------------------------------------------------------

test_that("Weibull analytic jacobian matches numDeriv (cumhaz)", {
  skip_if_not_installed("numDeriv")
  set.seed(3)
  t_new <- c(0.3, 0.8, 1.5, 2.2)
  x_new <- matrix(c(-0.5, 0, 0.4, 1.2), ncol = 1)
  theta <- c(mu = 0.6, nu = 1.2, beta = 0.25)

  cumhaz_fn <- function(th) {
    (th[1] * t_new) ^ th[2] * exp(as.numeric(x_new %*% th[3:length(th)]))
  }
  J_ana <- TemporalHazard:::.hzr_predict_jacobian_weibull(
    "cumulative_hazard", theta, t_new, x_new, length(theta)
  )
  J_num <- numDeriv::jacobian(cumhaz_fn, theta)
  expect_equal(J_ana, J_num, tolerance = 1e-6)
})

test_that("Weibull analytic jacobian matches numDeriv (hazard / relative)", {
  skip_if_not_installed("numDeriv")
  x_new <- matrix(c(-0.5, 0, 0.4), ncol = 1)
  theta <- c(mu = 0.6, nu = 1.2, beta = 0.25)
  haz_fn <- function(th) exp(as.numeric(x_new %*% th[3:length(th)]))
  J_ana <- TemporalHazard:::.hzr_predict_jacobian_weibull(
    "hazard", theta, time = NULL, x_new, length(theta)
  )
  J_num <- numDeriv::jacobian(haz_fn, theta)
  expect_equal(J_ana, J_num, tolerance = 1e-6)
})

test_that("Multiphase analytic jacobian matches numDeriv (cumhaz)", {
  skip_if_not_installed("numDeriv")
  skip_on_cran()
  t_new <- c(0.3, 0.8, 1.5, 2.2)
  phases <- list(
    early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  cov_counts <- c(early = 0L, constant = 0L)
  x_list <- list(early = NULL, constant = NULL)
  theta <- c(-3, log(0.3), 1, 1, -2)

  J_ana <- TemporalHazard:::.hzr_predict_jacobian_multiphase(
    theta, t_new, phases, cov_counts, x_list, length(theta)
  )
  J_num <- numDeriv::jacobian(
    function(th) {
      TemporalHazard:::.hzr_multiphase_cumhaz(
        t_new, th, phases, cov_counts, x_list
      )
    },
    theta
  )
  expect_equal(J_ana, J_num, tolerance = 1e-6)
})

# ---------------------------------------------------------------------------
# (3, 4, 5) Scale, monotonicity, and range
# ---------------------------------------------------------------------------

test_that("survival CLs stay in [0, 1] and straddle the point estimate", {
  skip_on_cran()
  df <- make_toy()
  for (dist in c("weibull", "exponential", "loglogistic", "lognormal")) {
    theta_init <- switch(
      dist,
      weibull = c(0.5, 1, 0),
      exponential = c(log(0.3), 0),
      loglogistic = c(0, 0, 0),
      lognormal = c(0, 0, 0)
    )
    fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = dist,
                  theta = theta_init, fit = TRUE)
    res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2), x = 0),
                   type = "survival", se.fit = TRUE)
    expect_true(all(res$lower >= 0), info = dist)
    expect_true(all(res$upper <= 1), info = dist)
    expect_true(all(res$lower <= res$fit + 1e-8), info = dist)
    expect_true(all(res$fit <= res$upper + 1e-8), info = dist)
  }
})

test_that("cumulative_hazard CLs are strictly positive and log-symmetric", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2), x = 0),
                 type = "cumulative_hazard", se.fit = TRUE)
  expect_true(all(res$lower > 0))
  expect_true(all(res$upper > res$lower))
  # log-symmetric: log(upper) - log(fit) ~= log(fit) - log(lower)
  log_halfwidth_upper <- log(res$upper) - log(res$fit)
  log_halfwidth_lower <- log(res$fit) - log(res$lower)
  expect_equal(log_halfwidth_upper, log_halfwidth_lower, tolerance = 1e-6)
})

test_that("linear_predictor CLs are symmetric on the natural scale", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  res <- predict(fit, newdata = data.frame(x = c(-1, 0, 1)),
                 type = "linear_predictor", se.fit = TRUE)
  halfwidth_upper <- res$upper - res$fit
  halfwidth_lower <- res$fit - res$lower
  expect_equal(halfwidth_upper, halfwidth_lower, tolerance = 1e-10)
})

# ---------------------------------------------------------------------------
# (6) Missing vcov -> NA CLs with a warning
# ---------------------------------------------------------------------------

test_that("missing vcov produces NA CLs with a warning", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)
  # Knock out vcov to simulate a non-converged / ill-conditioned fit
  fit$fit$vcov <- NULL
  expect_warning(
    res <- predict(fit, newdata = data.frame(time = 1, x = 0),
                   type = "survival", se.fit = TRUE),
    "Variance-covariance"
  )
  expect_true(all(is.na(res$se.fit)))
  expect_true(all(is.na(res$lower)))
  expect_true(all(is.na(res$upper)))
  expect_false(anyNA(res$fit))
})

# ---------------------------------------------------------------------------
# (7) Fixed-shapes multiphase recovers CLs from the free-parameter submatrix
# ---------------------------------------------------------------------------

test_that("multiphase CLs work with fixed shapes + CoE (free submatrix)", {
  skip_on_cran()
  df <- make_toy()
  phases <- list(
    early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  fit <- hazard(survival::Surv(time, status) ~ 1, data = df,
                dist = "multiphase", phases = phases, fit = TRUE,
                control = list(n_starts = 2))
  res <- predict(fit, newdata = data.frame(time = c(0.5, 1, 2)),
                 type = "survival", se.fit = TRUE)
  expect_true(all(is.finite(res$se.fit)))
  expect_true(all(res$se.fit > 0))
  expect_true(all(res$lower >= 0))
  expect_true(all(res$upper <= 1))
})

# ---------------------------------------------------------------------------
# per_phase Jacobian mode (decompose + se.fit support)
# ---------------------------------------------------------------------------

test_that("multiphase jacobian per_phase=TRUE returns blocks that sum to the total", {
  skip_on_cran()
  t_new <- c(0.3, 0.8, 1.5, 2.2)
  phases <- list(
    early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  cov_counts <- c(early = 0L, constant = 0L)
  x_list <- list(early = NULL, constant = NULL)
  theta <- c(-3, log(0.3), 1, 1, -2)
  p <- length(theta)

  J_total <- TemporalHazard:::.hzr_predict_jacobian_multiphase(
    theta, t_new, phases, cov_counts, x_list, p
  )
  J_list <- TemporalHazard:::.hzr_predict_jacobian_multiphase(
    theta, t_new, phases, cov_counts, x_list, p, per_phase = TRUE
  )

  expect_type(J_list, "list")
  expect_named(J_list, c("early", "constant"))
  expect_equal(dim(J_list$early), c(length(t_new), p))
  # Blocks sum to the aggregate Jacobian.
  expect_equal(Reduce(`+`, J_list), J_total, tolerance = 1e-12)
  # Each block touches only its own phase's columns:
  # early occupies cols 1:4 (log_mu, log_thalf, nu, m), constant col 5 (log_mu).
  expect_true(all(J_list$early[, 5] == 0))
  expect_true(all(J_list$constant[, 1:4] == 0))
})

test_that(".hzr_free_vcov restricts to finite-diagonal free parameters", {
  # All-free 2x2.
  V <- matrix(c(0.04, 0.01, 0.01, 0.09), 2, 2)
  res <- TemporalHazard:::.hzr_free_vcov(V, p = 2L)
  expect_equal(res$free_idx, 1:2)
  expect_equal(res$vcov_use, V)

  # One fixed parameter (NA row/col) -> restricted to the free submatrix.
  Vf <- matrix(NA_real_, 3, 3)
  Vf[c(1, 3), c(1, 3)] <- c(0.04, 0.00, 0.00, 0.09)
  res2 <- TemporalHazard:::.hzr_free_vcov(Vf, p = 3L)
  expect_equal(res2$free_idx, c(1L, 3L))
  expect_equal(dim(res2$vcov_use), c(2L, 2L))

  # Unusable vcov -> NULL with a warning.
  expect_warning(
    bad <- TemporalHazard:::.hzr_free_vcov(NULL, p = 3L),
    "unavailable"
  )
  expect_null(bad)
})

test_that(".hzr_predict_with_se_decomposed returns a tidy long frame", {
  skip_on_cran()
  df <- make_toy()
  phases <- list(
    early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  fit <- hazard(survival::Surv(time, status) ~ 1, data = df,
                dist = "multiphase", phases = phases, fit = TRUE,
                control = list(n_starts = 2))
  t_new <- c(0.5, 1, 2)
  cov_counts <- c(early = 0L, constant = 0L)
  x_list <- list(early = NULL, constant = NULL)

  res <- TemporalHazard:::.hzr_predict_with_se_decomposed(
    object = fit, time = t_new, x_list = x_list,
    cov_counts = cov_counts, phases = phases, level = 0.95
  )

  expect_s3_class(res, "data.frame")
  expect_named(res, c("time", "component", "fit", "se.fit", "lower", "upper"))
  expect_equal(levels(res$component), c("total", "early", "constant"))
  expect_equal(nrow(res), length(t_new) * 3L)  # total + 2 phases
  # Monotone, non-negative CLs everywhere SEs are finite.
  ok <- is.finite(res$se.fit)
  expect_true(all(res$lower[ok] <= res$fit[ok] + 1e-9))
  expect_true(all(res$fit[ok] <= res$upper[ok] + 1e-9))
  expect_true(all(res$lower[ok] >= -1e-12))
})

# ---------------------------------------------------------------------------
# (8) Backward compat: se.fit = FALSE reproduces the old scalar-vector return
# ---------------------------------------------------------------------------

test_that("se.fit = FALSE preserves pre-0.9.8 return shape", {
  skip_on_cran()
  df <- make_toy()
  fit <- hazard(survival::Surv(time, status) ~ x, data = df, dist = "weibull",
                theta = c(0.5, 1, 0), fit = TRUE)

  # Each type should return a bare numeric vector with se.fit = FALSE.
  for (type in c("survival", "cumulative_hazard", "hazard",
                 "linear_predictor")) {
    nd <- if (type %in% c("survival", "cumulative_hazard")) {
      data.frame(time = c(0.5, 1, 2), x = 0)
    } else {
      data.frame(x = c(-1, 0, 1))
    }
    out <- predict(fit, newdata = nd, type = type, se.fit = FALSE)
    expect_type(out, "double")
    expect_length(out, 3L)
    expect_false(is.data.frame(out))
  }
})

# ---------------------------------------------------------------------------
# conf.type for survival confidence limits (log-log default vs logit/HAZPRED)
# ---------------------------------------------------------------------------
test_that("predict(type='survival') conf.type selects the CL transform", {
  skip_if_not_installed("numDeriv")
  set.seed(7)
  n <- 400
  tev <- rexp(n, 0.3) + 0.01
  cens <- runif(n, 0.2, 6)
  d <- data.frame(time = pmin(tev, cens), status = as.integer(tev <= cens))
  set.seed(1)
  fit <- hazard(survival::Surv(time, status) ~ 1, data = d, dist = "multiphase",
    phases = list(early = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
                  constant = hzr_phase("constant")),
    fit = TRUE, control = list(n_starts = 1, maxit = 500, conserve = TRUE))
  grid <- data.frame(time = c(0.5, 1, 3))

  default <- predict(fit, newdata = grid, type = "survival", se.fit = TRUE)
  loglog  <- predict(fit, newdata = grid, type = "survival", se.fit = TRUE,
                     conf.type = "log-log")
  logit   <- predict(fit, newdata = grid, type = "survival", se.fit = TRUE,
                     conf.type = "logit")

  # Default is log-log.
  expect_equal(default$lower, loglog$lower)
  expect_equal(default$upper, loglog$upper)
  # logit differs from log-log but shares the point estimate and stays in (0,1).
  expect_equal(logit$fit, loglog$fit, tolerance = 1e-10)
  expect_false(isTRUE(all.equal(logit$lower, loglog$lower)))
  expect_true(all(logit$lower > 0 & logit$upper < 1))
  expect_true(all(logit$lower <= logit$fit & logit$fit <= logit$upper))

  # Invalid conf.type is rejected.
  expect_error(predict(fit, newdata = grid, type = "survival", se.fit = TRUE,
                       conf.type = "bogus"))
})

# stepwise-fixture.R -- Load and validate SAS stepwise parity fixtures.
#
# Fixtures live under `inst/fixtures/stepwise-<scenario>.rds` (and so
# reside under `<pkg root>/fixtures/` at install time).  They are
# deliberately optional -- tests that consume them skip when the file
# is absent, so the package installs and tests pass without any SAS
# output.
#
# The workflow for adding a new fixture is documented in
# `inst/extdata/stepwise-fixtures/README.md`.  .hzr_build_stepwise_fixture
# packages three SAS capture files into the expected list shape;
# .hzr_load_stepwise_fixture is the lookup called from the parity test.

#' Required top-level fields of a stepwise parity fixture
#'
#' Used by both the loader and the builder to validate the list shape.
#' @keywords internal
#' @noRd
.hzr_stepwise_fixture_schema <- function() {
  list(
    meta = c(
      "dataset", "dist", "criterion", "direction",
      "slentry", "slstay", "sas_version", "captured_on",
      "proc_hazard"
    ),
    scope = c("candidates", "force_in", "force_out"),
    steps = c("step_num", "action", "variable", "phase",
              "stat", "df", "p_value"),
    final = c("coef", "logLik", "iterations")
  )
}


#' Validate a stepwise parity fixture against the schema
#'
#' @param fix A list as produced by `.hzr_build_stepwise_fixture()` or
#'   deserialised from `inst/fixtures/stepwise-*.rds`.
#' @return The fixture, unchanged, if valid.  Throws with a list of
#'   missing fields otherwise.
#'
#' @keywords internal
#' @noRd
.hzr_validate_stepwise_fixture <- function(fix) {
  schema <- .hzr_stepwise_fixture_schema()
  problems <- character()

  for (group in names(schema)) {
    if (!group %in% names(fix)) {
      problems <- c(problems, paste0("missing top-level: ", group))
      next
    }
    missing_fields <- setdiff(schema[[group]], names(fix[[group]]))
    if (length(missing_fields) > 0L) {
      problems <- c(problems,
                    sprintf("missing %s: %s", group,
                            paste(missing_fields, collapse = ", ")))
    }
  }

  if (is.data.frame(fix$steps) && nrow(fix$steps) > 0L) {
    bad_actions <- setdiff(fix$steps$action, c("enter", "drop"))
    if (length(bad_actions) > 0L) {
      problems <- c(problems,
                    sprintf("unknown steps$action: %s",
                            paste(bad_actions, collapse = ", ")))
    }
  } else if (!is.null(fix$steps)) {
    problems <- c(problems, "steps must be a data.frame")
  }

  if (length(problems) > 0L) {
    stop("Invalid stepwise parity fixture:\n  - ",
         paste(problems, collapse = "\n  - "),
         call. = FALSE)
  }
  invisible(fix)
}


#' Load a stepwise parity fixture by name
#'
#' Returns NULL (not an error) when the fixture file is absent, so
#' callers can `skip_if_null()`.
#'
#' @param name Short name like `"cabgkul-forward-wald"`.
#' @return The fixture list or NULL.
#'
#' @keywords internal
#' @noRd
.hzr_load_stepwise_fixture <- function(name) {
  file_name <- paste0("stepwise-", name, ".rds")
  path <- system.file("fixtures", file_name, package = "TemporalHazard")
  if (!nzchar(path) || !file.exists(path)) {
    return(NULL)
  }
  fix <- readRDS(path)
  .hzr_validate_stepwise_fixture(fix)
}


#' Build a stepwise parity fixture from SAS capture files
#'
#' Reads the three files emitted by
#' `inst/extdata/stepwise-fixtures/cabgkul-forward-wald.sas` and
#' assembles them into the fixture list (see `schema.md`).  Writes the
#' result to `inst/fixtures/` if `out_path` is supplied.
#'
#' @param trace_csv,final_csv,meta_txt Paths to the SAS outputs.
#' @param out_path Optional output `.rds` path.  When set, the
#'   validated fixture is written there before being returned.
#'
#' @return The fixture list.
#'
#' @keywords internal
#' @noRd
.hzr_build_stepwise_fixture <- function(trace_csv, final_csv, meta_txt,
                                         out_path = NULL) {
  if (!file.exists(trace_csv)) {
    stop("trace_csv not found: ", trace_csv, call. = FALSE)
  }
  if (!file.exists(final_csv)) {
    stop("final_csv not found: ", final_csv, call. = FALSE)
  }
  if (!file.exists(meta_txt)) {
    stop("meta_txt not found: ", meta_txt, call. = FALSE)
  }

  trace <- utils::read.csv(trace_csv, stringsAsFactors = FALSE)
  # SAS may emit "." for missing phase -- translate to NA for the R side.
  if ("phase" %in% names(trace)) {
    trace$phase[trace$phase == "."] <- NA_character_
  }

  final_tbl <- utils::read.csv(final_csv, stringsAsFactors = FALSE)
  if ("phase" %in% names(final_tbl)) {
    final_tbl$phase[final_tbl$phase == "."] <- NA_character_
  }

  meta_lines <- readLines(meta_txt)
  meta_kv    <- .hzr_parse_keyvals(meta_lines)

  loglik <- as.numeric(meta_kv[["logLik"]] %||% NA_real_)
  iters  <- as.integer(meta_kv[["iterations"]] %||% NA_integer_)

  fix <- list(
    meta = list(
      dataset     = meta_kv[["dataset"]]     %||% NA_character_,
      dist        = meta_kv[["dist"]]        %||% NA_character_,
      criterion   = meta_kv[["criterion"]]   %||% "wald",
      direction   = meta_kv[["direction"]]   %||% NA_character_,
      slentry     = as.numeric(meta_kv[["slentry"]] %||% NA_real_),
      slstay      = as.numeric(meta_kv[["slstay"]]  %||% NA_real_),
      sas_version = meta_kv[["sas_version"]] %||% NA_character_,
      captured_on = meta_kv[["captured_on"]] %||% NA_character_,
      proc_hazard = meta_kv[["proc_hazard"]] %||% NA_character_,
      notes       = meta_kv[["notes"]]
    ),
    scope = list(
      candidates = unique(trace$variable),
      force_in   = character(),
      force_out  = character()
    ),
    steps = trace,
    final = list(
      coef       = final_tbl,
      logLik     = loglik,
      iterations = iters
    )
  )

  .hzr_validate_stepwise_fixture(fix)

  if (!is.null(out_path)) {
    dir.create(dirname(out_path), showWarnings = FALSE, recursive = TRUE)
    saveRDS(fix, out_path)
  }
  fix
}


#' Parse a `key=value` text file into a named character list
#'
#' @keywords internal
#' @noRd
.hzr_parse_keyvals <- function(lines) {
  nonblank <- lines[nzchar(lines) & !startsWith(trimws(lines), "#")]
  kv <- strsplit(nonblank, "=", fixed = TRUE)
  out <- vapply(kv, function(p) trimws(paste(p[-1], collapse = "=")),
                character(1L))
  names(out) <- vapply(kv, function(p) trimws(p[[1L]]), character(1L))
  as.list(out)
}

# SAS parity tests for hzr_stepwise().  Skip gracefully when the
# fixture file under inst/fixtures/ is absent, so the package installs
# and passes CI without any SAS capture.
#
# To enable: run the SAS template in
# `inst/extdata/stepwise-fixtures/cabgkul-forward-wald.sas`, then
# convert its three output files via:
#
#   TemporalHazard:::.hzr_build_stepwise_fixture(
#     trace_csv = "path/to/stepwise_trace.csv",
#     final_csv = "path/to/stepwise_final.csv",
#     meta_txt  = "path/to/stepwise_meta.txt",
#     out_path  = "inst/fixtures/stepwise-cabgkul-forward-wald.rds"
#   )

.stepwise_parity_tolerance <- list(
  stat        = 1e-3,   # Wald chi-square, relative
  p_value     = 1e-3,   # absolute
  logLik      = 1e-3,   # absolute
  coefficient = 1e-2    # relative
)


.stepwise_parity_run <- function(fix) {
  if (fix$meta$dataset != "cabgkul") {
    testthat::skip(paste0("Parity runner only wired up for CABGKUL so far; saw ",
                           fix$meta$dataset))
  }
  data(cabgkul, envir = environment())

  # Base model: intercept-only, same distribution as SAS, fit = TRUE
  base <- hazard(
    Surv(int_dead, dead) ~ 1,
    data  = cabgkul,
    theta = c(0.5, 1.0),
    dist  = fix$meta$dist,
    fit   = TRUE
  )

  hzr_stepwise(
    base,
    scope     = fix$scope$candidates,
    data      = cabgkul,
    direction = fix$meta$direction,
    criterion = fix$meta$criterion,
    slentry   = fix$meta$slentry,
    slstay    = fix$meta$slstay,
    force_in  = fix$scope$force_in,
    force_out = fix$scope$force_out,
    trace     = FALSE
  )
}


test_that("CABGKUL forward-Wald stepwise matches SAS output", {
  fix <- .hzr_load_stepwise_fixture("cabgkul-forward-wald")
  if (is.null(fix)) {
    skip("Stepwise parity fixture cabgkul-forward-wald.rds not found")
  }

  r_fit <- .stepwise_parity_run(fix)
  sas   <- fix$steps
  r_steps <- r_fit$steps[r_fit$steps$action %in% c("enter", "drop"), ]

  # 1. Same number of accepted steps
  expect_equal(nrow(r_steps), nrow(sas))

  # 2. Same (action, variable, phase) sequence, step by step
  expect_identical(r_steps$action,   sas$action)
  expect_identical(r_steps$variable, sas$variable)
  if ("phase" %in% names(sas)) {
    expect_identical(r_steps$phase, sas$phase)
  }

  # 3. Wald chi-square agreement.  The R driver stores z or chi^2 in
  # $stat depending on df; for df = 1 we square to compare.
  r_chi <- ifelse(r_steps$df == 1L, r_steps$stat ^ 2, r_steps$stat)
  expect_equal(r_chi, sas$stat,
               tolerance = .stepwise_parity_tolerance$stat)

  # 4. p-value agreement
  expect_equal(r_steps$p_value, sas$p_value,
               tolerance = .stepwise_parity_tolerance$p_value,
               scale = 1)

  # 5. Final log-likelihood agreement
  expect_equal(r_fit$fit$objective, fix$final$logLik,
               tolerance = .stepwise_parity_tolerance$logLik,
               scale = 1)

  # 6. Final coefficient table agreement (joined on variable name)
  r_summary <- summary(r_fit)$coefficients
  r_coef <- data.frame(
    variable  = rownames(r_summary),
    estimate  = r_summary$estimate,
    std_error = r_summary$std_error,
    stringsAsFactors = FALSE
  )
  sas_coef <- fix$final$coef[, c("variable", "estimate", "std_error")]

  common <- intersect(r_coef$variable, sas_coef$variable)
  expect_true(length(common) >= 1L,
              info = "no shared variables between R and SAS coef tables")

  for (v in common) {
    r_row   <- r_coef[r_coef$variable  == v, ]
    sas_row <- sas_coef[sas_coef$variable == v, ]
    expect_equal(r_row$estimate, sas_row$estimate,
                 tolerance = .stepwise_parity_tolerance$coefficient,
                 info = paste("estimate for", v))
    expect_equal(r_row$std_error, sas_row$std_error,
                 tolerance = .stepwise_parity_tolerance$coefficient,
                 info = paste("std_error for", v))
  }
})


# Sanity: the fixture loader + validator at least work on a synthetic
# fixture (doesn't depend on SAS output being present).
test_that("fixture validator rejects malformed input", {
  bad <- list(
    meta = list(dataset = "x", dist = "weibull", criterion = "wald"),
    scope = list(candidates = "age", force_in = character(),
                 force_out = character()),
    steps = data.frame(step_num = 1L, action = "bogus",
                       variable = "x", phase = NA_character_,
                       stat = 1, df = 1L, p_value = 0.1,
                       stringsAsFactors = FALSE),
    final = list(coef = data.frame(), logLik = -100, iterations = 5L)
  )
  expect_error(
    TemporalHazard:::.hzr_validate_stepwise_fixture(bad),
    "Invalid stepwise parity fixture"
  )
})

test_that("fixture validator accepts a well-formed fixture skeleton", {
  ok <- list(
    meta = list(
      dataset     = "x", dist = "weibull",
      criterion   = "wald", direction = "forward",
      slentry     = 0.30, slstay = 0.20,
      sas_version = "9.4", captured_on = "2026-04-17",
      proc_hazard = "PROC HAZARD ..."
    ),
    scope = list(candidates = "age", force_in = character(),
                 force_out = character()),
    steps = data.frame(
      step_num = 1L, action = "enter", variable = "age",
      phase = NA_character_, stat = 5, df = 1L, p_value = 0.025,
      stringsAsFactors = FALSE
    ),
    final = list(coef = data.frame(), logLik = -100, iterations = 7L)
  )
  expect_silent(TemporalHazard:::.hzr_validate_stepwise_fixture(ok))
})

# SAS parity tests for hzr_stepwise().  Skip gracefully when the
# fixture file under inst/fixtures/ is absent, so the package installs
# and passes CI without any SAS capture.
#
# To enable: run the SAS template in
# `inst/extdata/stepwise-fixtures/avc-forward-wald.sas`, then
# convert its three output files via:
#
#   TemporalHazard:::.hzr_build_stepwise_fixture(
#     trace_csv = "path/to/stepwise_trace.csv",
#     final_csv = "path/to/stepwise_final.csv",
#     meta_txt  = "path/to/stepwise_meta.txt",
#     out_path  = "inst/fixtures/stepwise-avc-forward-wald.rds"
#   )
#
# ALGORITHMIC NOTE: SAS HAZARD uses Q-statistics (score test, evaluated at the
# current parameter estimates without refitting) for stepwise candidate
# selection.  R's hzr_stepwise(criterion = "wald") uses full model refits and
# Wald chi-square from the fitted model.  These produce different intermediate
# statistics and different step sequences, so row-by-row step comparison is not
# meaningful.  The test instead verifies that R selects largely the same final
# variable set and achieves a competitive log-likelihood.

.stepwise_parity_tolerance <- list(
  logLik      = 10,    # absolute; generous to account for Q-stat vs Wald path difference
  coefficient = 0.10   # relative; final estimates for shared covariates
)


.stepwise_parity_run <- function(fix) {
  if (fix$meta$dataset != "avc") {
    testthat::skip(paste0("Parity runner only wired up for AVC so far; saw ",
                           fix$meta$dataset))
  }
  data(avc, envir = environment())
  avc <- na.omit(avc)

  # Multiphase fits with n_starts > 1 perturb the initial values using the
  # ambient RNG.  Seed locally and restore the stream on exit so the test is
  # reproducible and does not leak RNG state into later tests.
  if (exists(".Random.seed", envir = globalenv(), inherits = FALSE)) {
    old_seed <- get(".Random.seed", envir = globalenv(), inherits = FALSE)
    on.exit(assign(".Random.seed", old_seed, envir = globalenv()), add = TRUE)
  } else {
    on.exit(
      suppressWarnings(rm(".Random.seed", envir = globalenv())),
      add = TRUE
    )
  }
  set.seed(20260610L)

  # Two-phase (early CDF + constant) model matching SAS CONDITION=14:
  #   Early phase    -- Weibull CDF, THALF/NU/M fixed at the unconditional fit values
  #   Constant phase -- flat exponential hazard rate
  base <- suppressWarnings(hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = avc,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.1512095, nu = 1.438652, m = 1,
                            fixed = c("t_half", "nu", "m")),
      constant = hzr_phase("constant")
    ),
    fit    = TRUE,
    control = list(n_starts = 3L, maxit = 500L, conserve = TRUE)
  ))

  # For multiphase stepwise, scope must be a named list of one-sided formulas.
  cands    <- fix$scope$candidates
  scope_mp <- list(early = reformulate(cands), constant = reformulate(cands))

  hzr_stepwise(
    base,
    scope     = scope_mp,
    data      = avc,
    direction = fix$meta$direction,
    criterion = fix$meta$criterion,
    slentry   = fix$meta$slentry,
    slstay    = fix$meta$slstay,
    force_in  = fix$scope$force_in,
    force_out = fix$scope$force_out,
    trace     = FALSE
  )
}


test_that("AVC forward-Wald stepwise: R selects similar variables to SAS", {
  skip_on_cran()
  fix <- .hzr_load_stepwise_fixture("avc-forward-wald")
  if (is.null(fix)) {
    skip("Stepwise parity fixture avc-forward-wald.rds not found")
  }

  r_fit   <- suppressWarnings(.stepwise_parity_run(fix))

  # Final fitted model's coefficient names (e.g. "early.com_iv").  Covariates
  # are the rows whose suffix (after "phase.") is one of the scope candidates;
  # intercepts (log_mu) and shape parameters (log_t_half, nu, m, ...) are not.
  r_summary    <- summary(r_fit)$coefficients
  r_coef_names <- rownames(r_summary)
  cands        <- fix$scope$candidates
  r_covariates <- r_coef_names[sub("^[^.]+\\.", "", r_coef_names) %in% cands]

  # SAS non-intercept final covariates.
  sas_coef       <- fix$final$coef
  sas_covariates <- sas_coef[!sas_coef$variable %in% c("e0", "c0"), ]

  # R's *final model* must retain at least as many covariates as SAS selected.
  # (Count coefficients in the fitted model, not accepted entry steps, so a
  # variable that entered and was later dropped does not inflate the count.)
  expect_gte(length(r_covariates),
             nrow(sas_covariates),
             label = "R final model should retain at least as many covariates as SAS selected")

  # Final log-likelihood: R may find a different (even better) optimum via a
  # different variable path; we only require it stays within the generous
  # tolerance that accounts for the Q-stat vs Wald path difference.
  expect_gte(r_fit$fit$objective,
             fix$final$logLik - .stepwise_parity_tolerance$logLik,
             label = paste0("R logLik (", round(r_fit$fit$objective, 3),
                            ") should be within ", .stepwise_parity_tolerance$logLik,
                            " of SAS logLik (", fix$final$logLik, ")"))

  # Final variable set: SAS non-intercept covariates should largely appear in R.
  # Map fixture coef rows (variable + phase columns) to R's "phase.variable" naming.
  sas_varphase <- paste0(sas_covariates$phase, ".", sas_covariates$variable)
  n_shared <- sum(sas_varphase %in% r_coef_names)

  expect_gte(n_shared / length(sas_varphase), 0.5,
             label = paste0("At least half of SAS final covariates should appear in R model.",
                            " SAS: ", paste(sas_varphase, collapse = ", "),
                            " | R: ", paste(r_coef_names, collapse = ", ")))

  # Coefficient comparison is intentionally omitted: because R and SAS take
  # different selection paths (Wald refit vs Q-statistic), their final models
  # include different covariates.  Estimates for nominally "shared" variables
  # differ because the surrounding model composition differs, making direct
  # comparison invalid.
})


# Sanity: the fixture loader + validator at least work on a synthetic
# fixture (doesn't depend on SAS output being present).
test_that("fixture validator rejects malformed input", {
  bad <- list(
    meta = list(dataset = "x", dist = "weibull", criterion = "wald"),
    scope = list(candidates = "age", force_in = character(),
                 force_out = character()),
    steps = data.frame(step_num = 1L, action = "bogus",
                       variable = "x", phase = NA_character_,
                       stat = 1, df = 1L, p_value = 0.1,
                       stringsAsFactors = FALSE),
    final = list(coef = data.frame(), logLik = -100, iterations = 5L)
  )
  expect_error(
    TemporalHazard:::.hzr_validate_stepwise_fixture(bad),
    "Invalid stepwise parity fixture"
  )
})

test_that("fixture validator accepts a well-formed fixture skeleton", {
  ok <- list(
    meta = list(
      dataset     = "x", dist = "weibull",
      criterion   = "wald", direction = "forward",
      slentry     = 0.30, slstay = 0.20,
      sas_version = "9.4", captured_on = "2026-04-17",
      proc_hazard = "PROC HAZARD ..."
    ),
    scope = list(candidates = "age", force_in = character(),
                 force_out = character()),
    steps = data.frame(
      step_num = 1L, action = "enter", variable = "age",
      phase = NA_character_, stat = 5, df = 1L, p_value = 0.025,
      stringsAsFactors = FALSE
    ),
    final = list(coef = data.frame(), logLik = -100, iterations = 7L)
  )
  expect_silent(TemporalHazard:::.hzr_validate_stepwise_fixture(ok))
})

# math_primitives.R -- Numerically stable elementary functions
#
# PURPOSE
# -------
# Floating-point naive implementations of log(1+exp(x)), log(1-exp(-x)), and
# probability clamping produce incorrect results near the boundaries of
# double precision.  These primitives use piece-wise stable formulas that are
# safe across the full range of inputs encountered in survival analysis
# (very small or very large hazard rates, near-zero probabilities).
#
# All three functions are exported so users can call them directly when
# constructing custom likelihoods or diagnostics.

#' Numerically stable log(1 + exp(x))
#'
#' @param x Numeric vector.
#' @return Numeric vector with element-wise `log(1 + exp(x))`.
#' @examples
#' hzr_log1pexp(c(-50, 0, 50))
#' @export
hzr_log1pexp <- function(x) {
  ifelse(x > 0, x + log1p(exp(-x)), log1p(exp(x)))
}

#' Numerically stable log(1 - exp(-x)) for x > 0
#'
#' @param x Numeric vector with positive values.
#' @return Numeric vector with element-wise `log(1 - exp(-x))`.
#' @examples
#' hzr_log1mexp(c(0.01, 0.5, 5))
#' @export
hzr_log1mexp <- function(x) {
  out <- rep(NA_real_, length(x))
  cutoff <- log(2)

  valid <- is.finite(x) & (x > 0)
  small <- valid & (x <= cutoff)
  large <- valid & (x > cutoff)

  out[small] <- log(-expm1(-x[small]))
  out[large] <- log1p(-exp(-x[large]))
  out
}

#' Clamp probabilities away from 0 and 1
#'
#' @param p Numeric vector of probabilities.
#' @param eps Small positive tolerance.
#' @return Numeric vector bounded to `[eps, 1 - eps]`.
#' @examples
#' hzr_clamp_prob(c(0, 0.5, 1))
#' @export
hzr_clamp_prob <- function(p, eps = 1e-12) {
  if (!is.numeric(eps) || length(eps) != 1 || !is.finite(eps) || eps <= 0 || eps >= 0.5) {
    stop("'eps' must be a finite scalar in (0, 0.5).", call. = FALSE)
  }

  pmin(1 - eps, pmax(eps, p))
}

library(testthat)

test_that("Weibull log-likelihood handles mixed censoring statuses", {
  time <- c(1.0, 2.0, 1.5, 0.8)
  status <- c(1, 0, -1, 2)
  time_lower <- c(1.0, 2.0, 0.0, 1.2)
  time_upper <- c(1.0, 2.0, 1.5, 2.5)

  theta <- c(mu = 0.7, nu = 1.2)

  ll <- .hzr_logl_weibull(
    theta = theta,
    time = time,
    status = status,
    time_lower = time_lower,
    time_upper = time_upper
  )

  expect_true(is.finite(ll))
})

test_that("Weibull interval contribution matches manual formula", {
  theta <- c(mu = 0.8, nu = 1.4)
  time <- 1.0
  status <- 2
  time_lower <- 0.9
  time_upper <- 1.7

  ll <- .hzr_logl_weibull(
    theta = theta,
    time = time,
    status = status,
    time_lower = time_lower,
    time_upper = time_upper
  )

  H_l <- (theta[1] * time_lower) ^ theta[2]
  H_u <- (theta[1] * time_upper) ^ theta[2]
  manual <- -H_l + hzr_log1mexp(H_u - H_l)

  expect_equal(unname(ll), unname(manual), tolerance = 1e-10)
})

test_that("Weibull left-censored contribution matches manual formula", {
  theta <- c(mu = 0.9, nu = 1.1)
  time <- 1.3
  status <- -1

  ll <- .hzr_logl_weibull(
    theta = theta,
    time = time,
    status = status
  )

  H_u <- (theta[1] * time) ^ theta[2]
  manual <- hzr_log1mexp(H_u)

  expect_equal(ll, manual, tolerance = 1e-10)
})

test_that("Weibull right-censored contribution remains -H(t)", {
  theta <- c(mu = 0.6, nu = 1.5)
  time <- c(0.7, 1.2, 2.0)
  status <- c(0, 0, 0)

  ll <- .hzr_logl_weibull(theta, time, status)
  manual <- -sum((theta[1] * time) ^ theta[2])

  expect_equal(ll, manual, tolerance = 1e-10)
})

test_that("Weibull fit converges for mixed censoring sample", {
  set.seed(123)
  n <- 60
  time <- rexp(n, rate = 0.5)

  status <- sample(c(1, 0, -1, 2), size = n, replace = TRUE, prob = c(0.45, 0.35, 0.1, 0.1))

  time_lower <- time
  time_upper <- time

  idx_left <- which(status == -1)
  if (length(idx_left) > 0) {
    time_upper[idx_left] <- pmax(time[idx_left], 0.2)
    time_lower[idx_left] <- 0
  }

  idx_interval <- which(status == 2)
  if (length(idx_interval) > 0) {
    time_lower[idx_interval] <- pmax(time[idx_interval] * 0.8, 0.05)
    time_upper[idx_interval] <- time[idx_interval] * 1.2 + 0.1
  }

  fit <- hazard(
    time = time,
    status = status,
    time_lower = time_lower,
    time_upper = time_upper,
    theta = c(mu = 0.5, nu = 1.0),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 300)
  )

  expect_true(isTRUE(fit$fit$converged) || identical(fit$fit$converged, FALSE))
  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$fit$theta), 2)
})

test_that("Interval bounds are validated (lower must be < upper)", {
  expect_true(is.infinite(.hzr_logl_weibull(
    theta = c(0.7, 1.2),
    time = 1.0,
    status = 2,
    time_lower = 2.0,
    time_upper = 1.0
  )))
})

test_that("Non-Weibull distributions accept interval/left censoring inputs", {
  fit <- hazard(
    time = c(1, 2, 3),
    status = c(1, 2, 0),
    time_lower = c(1, 1.5, 3),
    time_upper = c(1, 2.5, 3),
    theta = c(0.2, 0.1),
    dist = "lognormal",
    fit = FALSE
  )

  expect_s3_class(fit, "hazard")
})

test_that("Right-censored legacy path unchanged when bounds are absent", {
  time <- c(0.5, 1.0, 2.0)
  status <- c(1, 0, 1)
  theta <- c(0.8, 1.3)

  ll_old <- .hzr_logl_weibull(theta, time, status)
  ll_new <- .hzr_logl_weibull(theta, time, status, time_lower = NULL, time_upper = NULL)

  expect_equal(ll_old, ll_new, tolerance = 1e-12)
})

library(testthat)

test_that("Weibull log-likelihood handles mixed censoring statuses", {
  time <- c(1.0, 2.0, 1.5, 0.8)
  status <- c(1, 0, -1, 2)
  time_lower <- c(1.0, 2.0, 0.0, 1.2)
  time_upper <- c(1.0, 2.0, 1.5, 2.5)

  theta <- c(mu = 0.7, nu = 1.2)

  ll <- .hzr_logl_weibull(
    theta = theta,
    time = time,
    status = status,
    time_lower = time_lower,
    time_upper = time_upper
  )

  expect_true(is.finite(ll))
})

test_that("Weibull interval contribution matches manual formula", {
  theta <- c(mu = 0.8, nu = 1.4)
  time <- 1.0
  status <- 2
  time_lower <- 0.9
  time_upper <- 1.7

  ll <- .hzr_logl_weibull(
    theta = theta,
    time = time,
    status = status,
    time_lower = time_lower,
    time_upper = time_upper
  )

  H_l <- (theta[1] * time_lower) ^ theta[2]
  H_u <- (theta[1] * time_upper) ^ theta[2]
  manual <- -H_l + hzr_log1mexp(H_u - H_l)

  expect_equal(unname(ll), unname(manual), tolerance = 1e-10)
})

test_that("Weibull left-censored contribution matches manual formula", {
  theta <- c(mu = 0.9, nu = 1.1)
  time <- 1.3
  status <- -1

  ll <- .hzr_logl_weibull(
    theta = theta,
    time = time,
    status = status
  )

  H_u <- (theta[1] * time) ^ theta[2]
  manual <- hzr_log1mexp(H_u)

  expect_equal(ll, manual, tolerance = 1e-10)
})

test_that("Weibull right-censored contribution remains -H(t)", {
  theta <- c(mu = 0.6, nu = 1.5)
  time <- c(0.7, 1.2, 2.0)
  status <- c(0, 0, 0)

  ll <- .hzr_logl_weibull(theta, time, status)
  manual <- -sum((theta[1] * time) ^ theta[2])

  expect_equal(ll, manual, tolerance = 1e-10)
})

test_that("Weibull fit converges for mixed censoring sample", {
  set.seed(123)
  n <- 60
  time <- rexp(n, rate = 0.5)

  status <- sample(c(1, 0, -1, 2), size = n, replace = TRUE, prob = c(0.45, 0.35, 0.1, 0.1))

  time_lower <- time
  time_upper <- time

  idx_left <- which(status == -1)
  if (length(idx_left) > 0) {
    time_upper[idx_left] <- pmax(time[idx_left], 0.2)
    time_lower[idx_left] <- 0
  }

  idx_interval <- which(status == 2)
  if (length(idx_interval) > 0) {
    time_lower[idx_interval] <- pmax(time[idx_interval] * 0.8, 0.05)
    time_upper[idx_interval] <- time[idx_interval] * 1.2 + 0.1
  }

  fit <- hazard(
    time = time,
    status = status,
    time_lower = time_lower,
    time_upper = time_upper,
    theta = c(mu = 0.5, nu = 1.0),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 300)
  )

  expect_true(isTRUE(fit$fit$converged) || identical(fit$fit$converged, FALSE))
  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$fit$theta), 2)
})

test_that("Interval bounds are validated (lower must be < upper)", {
  expect_true(is.infinite(.hzr_logl_weibull(
    theta = c(0.7, 1.2),
    time = 1.0,
    status = 2,
    time_lower = 2.0,
    time_upper = 1.0
  )))
})

test_that("Non-Weibull distributions accept interval/left censoring inputs", {
  fit <- hazard(
    time = c(1, 2, 3),
    status = c(1, 2, 0),
    time_lower = c(1, 1.5, 3),
    time_upper = c(1, 2.5, 3),
    theta = c(0.2, 0.1),
    dist = "lognormal",
    fit = FALSE
  )

  expect_s3_class(fit, "hazard")
})

test_that("Right-censored legacy path unchanged when bounds are absent", {
  time <- c(0.5, 1.0, 2.0)
  status <- c(1, 0, 1)
  theta <- c(0.8, 1.3)

  ll_old <- .hzr_logl_weibull(theta, time, status)
  ll_new <- .hzr_logl_weibull(theta, time, status, time_lower = NULL, time_upper = NULL)

  expect_equal(ll_old, ll_new, tolerance = 1e-12)
})

test_that("Mixed IC+RC: right-censored contribution not silently zeroed when time_lower supplied", {
  # Regression test for the time_lower dual-use bug:
  # When time_lower is provided for a mix of interval-censored (status=2)
  # and right-censored (status=0) rows, right-censored rows must contribute
  # -H(stop) to the log-likelihood, NOT H(stop)-H(stop)=0.
  theta      <- c(mu = 0.4, nu = 1.2)
  mu <- theta[1]
  nu <- theta[2]

  # 3 interval-censored observations in (1, 2)
  # 2 right-censored observations at t = 5
  time       <- c(1, 1, 1, 5, 5)
  status     <- c(2L, 2L, 2L, 0L, 0L)
  time_lower <- c(1, 1, 1, 0, 0)   # correct: 0 for RC rows
  time_upper <- c(2, 2, 2, 5, 5)

  # Manual LL
  H_lo <- (mu * 1)^nu
  H_hi <- (mu * 2)^nu
  H_rc <- (mu * 5)^nu
  ll_ic_manual <- 3 * (-H_lo + hzr_log1mexp(H_hi - H_lo))
  ll_rc_manual <- 2 * (-H_rc)
  ll_manual <- unname(ll_ic_manual + ll_rc_manual)

  ll_pkg <- .hzr_logl_weibull(theta, time, status,
                               time_lower = time_lower,
                               time_upper = time_upper)

  expect_equal(ll_pkg, ll_manual, tolerance = 1e-10)

  # Also verify the *wrong* setup (time_lower = time for RC) now gives same
  # result after the fix — previously it zeroed the RC contribution
  time_lower_wrong <- c(1, 1, 1, 5, 5)  # time_lower == time for RC rows
  ll_wrong_setup <- .hzr_logl_weibull(theta, time, status,
                                       time_lower = time_lower_wrong,
                                       time_upper = time_upper)
  # With the fix, time_lower == time means NOT a genuine epoch, so RC
  # contribution is still -H(5), same result as the correct setup
  expect_equal(ll_wrong_setup, ll_manual, tolerance = 1e-10)
})

test_that("Mixed IC+RC: fitting recovers correct parameters with time_lower=0 for RC", {
  set.seed(42)
  n <- 600
  true_time <- rexp(n, rate = 0.05)
  # Interval censoring in 6-month windows; right-censor at 48 months
  status     <- integer(n)
  time       <- numeric(n)
  time_lower <- numeric(n)
  time_upper <- numeric(n)
  for (i in seq_len(n)) {
    brk <- ceiling(true_time[i] / 6)
    if (brk > 8L) {
      status[i]     <- 0L
      time[i]       <- 48
      time_lower[i] <- 0   # correct: 0 for RC rows
      time_upper[i] <- 48
    } else {
      status[i]     <- 2L
      time[i]       <- (brk - 1L) * 6
      time_lower[i] <- (brk - 1L) * 6
      time_upper[i] <- brk * 6
    }
  }
  fit <- hazard(time = time, status = status,
                time_lower = time_lower, time_upper = time_upper,
                dist = "weibull", theta = c(mu = 0.05, nu = 1.0), fit = TRUE)
  expect_true(fit$fit$converged)
  # MLE should be within 20% of truth (nu=1 for exponential, mu=0.05)
  coefs <- coef(fit)[1:2]
  expect_lt(abs(coefs["nu"] - 1.0), 0.2)
  expect_lt(abs(coefs["mu"] - 0.05) / 0.05, 0.2)
})

test_that("hzr_log1pexp is numerically stable", {
  x <- c(-1000, -10, 0, 10, 1000)
  y <- hzr_log1pexp(x)

  expect_true(all(is.finite(y)))
  expect_lt(abs(y[3] - log(2)), 1e-14)
  expect_lt(abs(y[4] - log1p(exp(10))), 1e-12)
  expect_lt(abs(y[5] - 1000), 1e-10)
})

test_that("hzr_log1mexp behaves for positive inputs", {
  x <- c(1e-8, 0.1, 1, 10)
  y <- hzr_log1mexp(x)

  expect_true(all(is.finite(y)))
  expect_equal(y[3], log(1 - exp(-1)), tolerance = 1e-12)
  expect_equal(y[4], log(1 - exp(-10)), tolerance = 1e-12)
})

test_that("hzr_log1mexp returns NA for non-positive inputs", {
  y <- hzr_log1mexp(c(-1, 0, 1))
  expect_true(is.na(y[1]))
  expect_true(is.na(y[2]))
  expect_false(is.na(y[3]))
})

test_that("hzr_clamp_prob bounds values", {
  p <- c(-1, 0, 0.25, 1, 2)
  y <- hzr_clamp_prob(p, eps = 1e-6)

  expect_equal(y[1], 1e-6)
  expect_equal(y[2], 1e-6)
  expect_equal(y[3], 0.25)
  expect_equal(y[4], 1 - 1e-6)
  expect_equal(y[5], 1 - 1e-6)
})

test_that("hzr_clamp_prob validates eps", {
  expect_error(hzr_clamp_prob(0.1, eps = 0))
  expect_error(hzr_clamp_prob(0.1, eps = 0.5))
  expect_error(hzr_clamp_prob(0.1, eps = NA_real_))
})

#' @importFrom stats optim
#' @keywords internal
NULL

# likelihood-exponential.R -- Exponential parametric hazard likelihood, gradient, and optimizer
#
# MODEL
# -----
# Constant-hazard (memoryless) proportional-hazards model:
#
#   h(t | x)  = lambda exp(eta)                    hazard (constant in t)
#   H(t | x)  = lambda t exp(eta)                  cumulative hazard
#   S(t | x)  = exp(-lambda t exp(eta))            survival
#
# where lambda > 0 (baseline rate), eta = x beta (linear predictor).
#
# THETA LAYOUT
# ------------
#   theta[1]   = log(lambda)   (unconstrained; lambda recovered via exp())
#   theta[2:p] = beta        (covariate coefficients; unrestricted)
#
# The exponential is a special case of Weibull with nu = 1.  It has no shape
# parameter so theta is one element shorter than Weibull for the same data.
# Unconstrained BFGS is used (no box bounds needed).
#
# FUNCTIONS
# ---------
#   .hzr_logl_exponential()     -- log-likelihood (optionally returning gradient)
#   .hzr_gradient_exponential() -- analytical score vector
#   .hzr_optim_exponential()    -- unconstrained BFGS wrapper

# Exponential Parametric Hazard Likelihood
#
# Evaluate the log-likelihood and its derivatives for exponential hazard models.
# The exponential is the simplest ALT model: constant hazard lambda, no shape parameter.

#' Log-likelihood for exponential hazard with covariates
#'
#' Computes the negative log-likelihood for right-censored data under the exponential
#' parametric hazard model with optional linear-predictor covariates.
#'
#' @param theta Vector of parameters:
#'   theta\[1\] = log(lambda) where lambda > 0 is the baseline hazard rate
#'   theta\[2:length\]: Covariate coefficients (linear on log-hazard scale)
#'
#' @param time Numeric vector of follow-up times (n)
#' @param status Numeric vector of event indicators: 1 = event, 0 = censored (n)
#' @param time_lower Optional numeric lower bound vector for interval-censored rows.
#'   Defaults to time if NULL.
#' @param time_upper Optional numeric upper bound vector for left/interval-censored rows.
#'   Defaults to time if NULL.
#' @param x Design matrix of covariates (n x p_coef); NULL for no covariates
#' @param return_gradient Logical; if TRUE, attach gradient vector as attribute
#'
#' @return Scalar log-likelihood value. If return_gradient = TRUE, gradient vector
#' is attached as attribute \code{"gradient"}.
#'
#' @details
#' The exponential hazard model is parameterized as:
#'
#' \deqn{h(t | x) = \lambda \exp(\eta)}
#'
#' where:
#' - \eqn{\lambda > 0} is baseline hazard rate
#' - \eqn{\eta = x \beta} is log-relative-hazard (covariates)
#'
#' The survival function is:
#'
#' \deqn{S(t | x) = \exp(-\lambda t \exp(\eta))}
#'
#' The log-likelihood for right-censored data is:
#'
#' \deqn{\ell(\theta) = \sum_{i: \delta_i = 1} [\log\lambda + \eta_i]
#'   - \lambda \sum_i t_i \exp(\eta_i)}
#'
#' Reparameterization: \u03b8\[1\] = log(\u03bb) avoids constrained optimization.
#'
#' Mixed censoring status coding:
#' - 1: exact event at time
#' - 0: right-censored at time
#' - -1: left-censored with upper bound time_upper \(or time\)
#' - 2: interval-censored in the interval \(time_lower, time_upper\)
#'
#' @noRd
.hzr_logl_exponential <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    return_gradient = FALSE) {

  n <- length(time)

  # Default unit weights
  if (is.null(weights)) weights <- rep(1, n)

  # Extract parameters
  log_lambda <- theta[1]
  lambda <- exp(log_lambda)  # Always positive

  # Covariate coefficients (if any)
  if (!is.null(x)) {
    if (is.null(attr(x, "dimnames")[[2]])) {
      colnames(x) <- paste0("beta_", seq_len(ncol(x)))
    }
    beta <- theta[2:length(theta)]
    eta <- as.numeric(x %*% beta)  # n-vector
  } else {
    eta <- rep(0, n)
  }

  # Resolve censoring bounds once so each likelihood term can reference
  # a consistent lower/upper representation regardless of user input style.
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  if (any(status %in% c(-1, 2)) && any(upper <= 0)) {
    return(Inf)
  }
  if (any(status == 2) && any(lower[status == 2] >= upper[status == 2])) {
    return(Inf)
  }

  # Cumulative hazards at event/lower/upper times; all terms can be
  # expressed using H(t) for the exponential model.
  cumhaz_event <- lambda * time * exp(eta)
  cumhaz_lower <- lambda * lower * exp(eta)
  cumhaz_upper <- lambda * upper * exp(eta)

  idx_event <- status == 1
  idx_right <- status == 0
  idx_left <- status == -1
  idx_interval <- status == 2

  # Event: w * [log h(t) + log S(t)] = w * [log(lambda)+eta - H(t)]
  ll_event <- if (any(idx_event)) {
    sum(weights[idx_event] *
          ((log_lambda + eta[idx_event]) - cumhaz_event[idx_event]))
  } else {
    0
  }

  # Right-censored: w * log S(t) = -w * H(t)
  ll_right <- if (any(idx_right)) {
    -sum(weights[idx_right] * cumhaz_event[idx_right])
  } else {
    0
  }

  # Left-censored: w * log(1 - exp(-H(u)))
  ll_left <- if (any(idx_left)) {
    sum(weights[idx_left] * hzr_log1mexp(cumhaz_upper[idx_left]))
  } else {
    0
  }

  # Interval-censored: w * [log(S(l)-S(u))]
  ll_interval <- if (any(idx_interval)) {
    delta_h <- cumhaz_upper[idx_interval] - cumhaz_lower[idx_interval]
    sum(weights[idx_interval] *
          (-cumhaz_lower[idx_interval] + hzr_log1mexp(delta_h)))
  } else {
    0
  }

  logl <- ll_event + ll_right + ll_left + ll_interval

  if (!is.finite(logl)) {
    return(Inf)
  }

  # If gradient requested, compute score vector
  if (return_gradient) {
    grad <- .hzr_gradient_exponential(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      eta = eta, cumhaz = cumhaz_event,
      lambda = lambda, log_lambda = log_lambda
    )
    attr(logl, "gradient") <- grad
  }

  logl
}

#' Score vector (gradient of log-likelihood) for exponential
#'
#' Computes the score vector of the exponential log-likelihood w.r.t. all parameters.
#'
#' The log-likelihood is:
#'   \eqn{L = \sum(\delta_i * [\log(\lambda) + \eta_i]) - \sum(\lambda * t_i * \exp(\eta_i))}
#'
#' Derivatives:
#' dL/d(log lambda) = sum(delta_i) - sum(lambda * t_i * exp(eta_i)) = sum(delta_i) - sum(H_i)
#' dL/dbeta_j    = sum(delta_i * x_ij) - sum(lambda * t_i * exp(eta_i) * x_ij) = t(X) %*% (delta - H)
#'
#' @noRd
.hzr_gradient_exponential <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    eta = NULL,
    cumhaz = NULL,
    lambda = NULL,
    log_lambda = NULL) {

  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)

  # Default unit weights -- keep gradient consistent with the LL when weights
  # are omitted.
  if (is.null(weights)) weights <- rep(1, n)

  # Numerical fallback for left/interval censoring contributions.
  # Right-censored/event-only rows still use the closed-form score below.
  if (any(status %in% c(-1, 2))) {
    return(.hzr_numeric_grad_exponential(theta, time, status, time_lower,
                                          time_upper, x, weights = weights))
  }

  # Recompute if not provided
  if (is.null(eta) || is.null(cumhaz) || is.null(lambda)) {
    log_lambda <- theta[1]
    lambda <- exp(log_lambda)

    if (!is.null(x)) {
      beta <- theta[2:length(theta)]
      eta <- as.numeric(x %*% beta)
    } else {
      eta <- rep(0, n)
    }

    cumhaz <- lambda * time * exp(eta)
  }

  # Weighted building blocks: every term below is the unweighted form with
  # `status` -> `w * status` and `cumhaz` -> `w * cumhaz`.
  w_status <- weights * status
  w_cumhaz <- weights * cumhaz

  # ===== Gradient w.r.t. log(lambda) =====
  # dL/d(log lambda) = sum(w * delta) - sum(w * H)
  grad[1] <- sum(w_status) - sum(w_cumhaz)

  # ===== Gradient w.r.t. beta (covariate coefficients) =====
  if (p > 1 && !is.null(x)) {
    # dL/dbeta = t(X) %*% (w * delta - w * H)
    residual <- w_status - w_cumhaz
    grad[2:p] <- as.numeric(crossprod(x, residual))
  }

  grad
}

#' Analytic Hessian of the exponential negative log-likelihood
#'
#' Returns the Hessian of the **objective** (negative log-likelihood) on the
#' internal parameter scale, matching \code{numDeriv::hessian(objective)} so it
#' can be used directly by \code{.hzr_optim_generic()}.  Coverage mirrors the
#' analytic gradient: closed form for event + right-censored rows only.  When any
#' row is left- or interval-censored (\code{status \%in\% c(-1, 2)}) it returns
#' \code{NULL} to request the numerical-Hessian fallback.
#'
#' Derivation: with \eqn{H_i = \lambda t_i e^{\eta_i}} and design matrix
#' \eqn{\tilde X = [1 | X]}, the log-likelihood Hessian is
#' \eqn{-\tilde X^\top \mathrm{diag}(w H) \tilde X}; the objective Hessian is its
#' negation, \eqn{+\tilde X^\top \mathrm{diag}(w H) \tilde X}.
#'
#' @noRd
.hzr_hessian_exponential <- function(
    theta, time, status,
    time_lower = NULL, time_upper = NULL, x = NULL, weights = NULL) {

  n <- length(time)
  if (is.null(weights)) weights <- rep(1, n)

  # Mirror the analytic gradient's coverage: closed form is event + right
  # censored only.  Decline (NULL -> numDeriv fallback) on left/interval rows.
  if (any(status %in% c(-1, 2))) {
    return(NULL)
  }

  log_lambda <- theta[1]
  lambda <- exp(log_lambda)
  # Fail loud on inconsistent theta/x: a mismatch would otherwise yield a wrong
  # or non-conformant Hessian used for vcov without a clear error.
  p_cov <- length(theta) - 1L
  if (!is.null(x)) x <- as.matrix(x)
  n_x <- if (is.null(x)) 0L else NCOL(x)
  if (n_x != p_cov) {
    stop(".hzr_hessian_exponential(): ncol(x) (", n_x, ") must equal the number ",
         "of covariate parameters in theta (", p_cov, ").", call. = FALSE)
  }
  if (p_cov > 0L) {
    beta <- theta[2:length(theta)]
    eta <- as.numeric(x %*% beta)
  } else {
    eta <- rep(0, n)
  }

  # Weighted cumulative hazard per row.
  wH <- weights * lambda * time * exp(eta)

  # Design matrix augmented with the log(lambda) "intercept" column.
  x_tilde <- if (is.null(x)) matrix(1, nrow = n, ncol = 1L) else cbind(1, x)

  # Objective (negative log-likelihood) Hessian: + X~' diag(wH) X~.
  hess <- crossprod(x_tilde, wH * x_tilde)
  dimnames(hess) <- list(names(theta), names(theta))
  hess
}

#' @noRd
.hzr_optim_exponential <- function(
    time, status, time_lower = NULL, time_upper = NULL,
    x = NULL, theta_start, weights = NULL, control = list()) {
  hessian_fn <- function(par) {
    .hzr_hessian_exponential(
      theta = par, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights
    )
  }
  .hzr_optim_generic(
    logl_fn = .hzr_logl_exponential,
    gradient_fn = .hzr_gradient_exponential,
    time = time, status = status,
    time_lower = time_lower, time_upper = time_upper,
    x = x, theta_start = theta_start, weights = weights,
    control = control, use_bounds = FALSE,
    hessian_fn = hessian_fn
  )
}

.hzr_numeric_grad_exponential <- function(theta, time, status,
                                            time_lower = NULL, time_upper = NULL,
                                            x = NULL, weights = NULL) {
  # Reuse log-likelihood as scalar objective and differentiate numerically.
  objective <- function(par) {
    .hzr_logl_exponential(
      theta = par,
      time = time,
      status = status,
      time_lower = time_lower,
      time_upper = time_upper,
      x = x,
      weights = weights,
      return_gradient = FALSE
    )
  }

  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(as.numeric(numDeriv::grad(objective, theta)))
  }

  eps <- 1e-6
  grad <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tm <- theta
    tp[i] <- tp[i] + eps
    tm[i] <- tm[i] - eps
    grad[i] <- (objective(tp) - objective(tm)) / (2 * eps)
  }
  grad
}

test_that(".hzr_hessian_exponential matches numDeriv (no covariates)", {
  skip_if_not_installed("numDeriv")
  set.seed(2)
  n <- 300
  time <- rexp(n, rate = 0.7)
  status <- rbinom(n, 1, 0.7)
  theta <- c(log_rate = log(0.7))
  obj <- function(par) -.hzr_logl_exponential(par, time, status, x = NULL)
  H_an <- .hzr_hessian_exponential(theta, time, status, x = NULL)
  H_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(H_an), unname(H_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_exponential matches numDeriv (covariates + weights)", {
  skip_if_not_installed("numDeriv")
  set.seed(3)
  n <- 400
  x <- cbind(a = rnorm(n), b = rbinom(n, 1, 0.4))
  eta <- as.numeric(x %*% c(0.3, -0.5))
  time <- rexp(n, rate = exp(log(0.5) + eta))
  status <- rbinom(n, 1, 0.8)
  w <- runif(n, 0.5, 2)
  theta <- c(log_rate = log(0.5), 0.3, -0.5)
  obj <- function(par) -.hzr_logl_exponential(par, time, status, x = x, weights = w)
  H_an <- .hzr_hessian_exponential(theta, time, status, x = x, weights = w)
  H_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(H_an), unname(H_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_exponential returns NULL for left/interval censoring", {
  set.seed(4)
  n <- 50
  time <- rexp(n, 0.5)
  status <- rep(1L, n)
  status[1:5] <- 2L
  tl <- time
  tu <- time + 0.5
  expect_null(.hzr_hessian_exponential(c(log_rate = 0), time, status,
                                       time_lower = tl, time_upper = tu, x = NULL))
  status2 <- rep(1L, n)
  status2[1:5] <- -1L
  expect_null(.hzr_hessian_exponential(c(log_rate = 0), time, status2,
                                       time_upper = tu, x = NULL))
})

test_that("exponential fit vcov uses the analytic Hessian (matches numDeriv)", {
  skip_if_not_installed("numDeriv")
  set.seed(5)
  n <- 500
  z <- rnorm(n)
  time <- rexp(n, rate = exp(log(0.4) + 0.6 * z))
  status <- rbinom(n, 1, 0.85)
  fit <- hazard(
    survival::Surv(time, status) ~ z,
    data = data.frame(time, status, z),
    dist = "exponential", theta = c(log_rate = 0, z = 0), fit = TRUE
  )
  obj <- function(par) -.hzr_logl_exponential(par, time, status, x = cbind(z))
  v_nd <- solve(numDeriv::hessian(obj, fit$fit$theta))
  expect_equal(unname(fit$fit$vcov), unname(v_nd), tolerance = 1e-4)
  expect_true(isTRUE(fit$fit$pd))
})

test_that("exponential SEs are invariant to covariate rescaling", {
  set.seed(6)
  n <- 600
  z <- rnorm(n, mean = 0, sd = 1)   # centered -> well-conditioned design
  time <- rexp(n, rate = exp(log(0.3) + 0.5 * z))
  status <- rbinom(n, 1, 0.9)
  f1 <- hazard(survival::Surv(time, status) ~ z,
               data = data.frame(time, status, z = z),
               dist = "exponential", theta = c(log_rate = 0, z = 0), fit = TRUE)
  f2 <- hazard(survival::Surv(time, status) ~ z,
               data = data.frame(time, status, z = z / 100),
               dist = "exponential", theta = c(log_rate = 0, z = 0), fit = TRUE)
  se1 <- sqrt(diag(f1$fit$vcov))
  se2 <- sqrt(diag(f2$fit$vcov))
  # log_rate SE is invariant to covariate rescaling.
  expect_equal(unname(se1[1]), unname(se2[1]), tolerance = 1e-3)
  # The beta z-statistic is invariant under x -> x / 100.
  z1 <- unname(f1$fit$theta[2] / se1[2])
  z2 <- unname(f2$fit$theta[2] / se2[2])
  expect_equal(z1, z2, tolerance = 1e-2)
})

test_that(".hzr_hessian_exponential errors on inconsistent theta/x", {
  set.seed(61)
  n <- 30
  time <- rexp(n, 0.5) + 0.01
  status <- rbinom(n, 1, 0.7)
  expect_error(
    .hzr_hessian_exponential(c(log_rate = 0, z = 0.3), time, status, x = NULL),
    "ncol")
  x2 <- cbind(a = rnorm(n), b = rnorm(n))
  expect_error(
    .hzr_hessian_exponential(c(log_rate = 0, z = 0.3), time, status, x = x2),
    "ncol")
})

test_that(".hzr_hessian_exponential guard handles vector x (NCOL robustness)", {
  set.seed(64)
  n <- 30
  time <- rexp(n, 0.5) + 0.01
  status <- rbinom(n, 1, 0.7)
  # x as a bare vector (NCOL = 1) with a 2-covariate theta -> clean "ncol" error,
  # not a cryptic "argument is of length zero" from ncol(vector) == NULL.
  expect_error(
    .hzr_hessian_exponential(c(log_rate = 0, a = 0.1, b = 0.2), time, status,
                             x = rnorm(n)),
    "ncol")
})

test_that(".hzr_hessian_exponential coerces a vector x to a 1-column design", {
  skip_if_not_installed("numDeriv")
  set.seed(65)
  n <- 200
  z <- rnorm(n)
  time <- rexp(n, rate = exp(-0.4 * z) * 0.5) + 0.01
  status <- rbinom(n, 1, 0.8)
  theta <- c(log_rate = log(0.5), z = -0.4)
  h_vec <- .hzr_hessian_exponential(theta, time, status, x = z)            # bare vector
  h_mat <- .hzr_hessian_exponential(theta, time, status, x = matrix(z, n, 1))
  expect_equal(unname(h_vec), unname(h_mat), tolerance = 1e-10)
  expect_equal(dim(h_vec), c(2L, 2L))
})

# diagnostics.R -- Model validation and goodness-of-fit utilities
#
# Implements SAS HAZARD macro equivalents for calibration and validation.

#' @importFrom stats predict
NULL

#' Decile-of-risk calibration
#'
#' Partition observations into groups (default 10) by predicted risk and
#' compare observed vs. expected event counts in each group.  Good
#' calibration means the two track each other across the risk spectrum.
#'
#' This implements the workflow of the SAS `deciles.hazard.sas` macro.
#' Patients are ranked by predicted cumulative hazard at a specified time
#' point, grouped into quantile bins, and each bin is tested with a
#' chi-square goodness-of-fit statistic. Subjects censored before the
#' requested horizon are excluded from the observed-vs-expected comparison.
#'
#' @param object A fitted `hazard` object (with `fit = TRUE`).
#' @param time Numeric scalar: the time point at which to evaluate
#'   predicted survival / cumulative hazard.  For example, `time = 12`
#'   evaluates 12-month predictions.
#' @param groups Integer: number of risk groups (default 10 for deciles).
#' @param status Optional numeric vector of event indicators (1 = event,
#'   0 = censored).
#'   If `NULL` (default), extracted from the fitted object's stored data.
#' @param event_time Optional numeric vector of observed event/censoring
#'   times.  If `NULL` (default), extracted from the fitted object.
#'
#' @return A data frame with one row per risk group and columns:
#' \describe{
#'   \item{group}{Integer group label (1 = lowest risk).}
#'   \item{n}{Number of observations in the group.}
#'   \item{events}{Observed event count by the requested horizon
#'     (event indicator = 1 and event time <= \code{time}).}
#'   \item{expected}{Expected event count by the requested horizon,
#'     computed as the sum of individual event probabilities
#'     (\eqn{1 - S(time)}).}
#'   \item{observed_rate}{Observed event rate (events / n).}
#'   \item{expected_rate}{Expected event rate (expected / n).}
#'   \item{chi_sq}{Chi-square contribution: (events - expected)^2 /
#'     expected.}
#'   \item{p_value}{Upper-tail p-value from the chi-square test for
#'     this group (1 df).}
#'   \item{mean_survival}{Mean predicted survival probability in the
#'     group.}
#'   \item{mean_cumhaz}{Mean predicted cumulative hazard in the group.}
#' }
#'
#' An attribute `"overall"` is attached with the overall chi-square
#' statistic, degrees of freedom, and p-value.
#'
#' @examples
#' \donttest{
#' data(avc)
#' avc <- na.omit(avc)
#' fit <- hazard(
#'   survival::Surv(int_dead, dead) ~ age + mal,
#'   data  = avc,
#'   dist  = "weibull",
#'   theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
#'   fit   = TRUE
#' )
#' cal <- hzr_deciles(fit, time = 120)
#' print(cal)
#' }
#'
#' @seealso [predict.hazard()] for the prediction types used internally.
#' @export
hzr_deciles <- function(object, time, groups = 10L,
                        status = NULL, event_time = NULL) {
  if (!inherits(object, "hazard")) {
    stop("'object' must be a fitted hazard object.", call. = FALSE)
  }

  if (is.null(object$fit$theta) ||
      (is.logical(object$fit$converged) && is.na(object$fit$converged))) {
    stop("'object' has no fitted parameters. Refit with fit = TRUE.",
         call. = FALSE)
  }
  if (!is.numeric(time) || length(time) != 1L || time <= 0) {
    stop("'time' must be a single positive number.", call. = FALSE)
  }
  groups <- as.integer(groups)
  if (groups < 2L) {
    stop("'groups' must be at least 2.", call. = FALSE)
  }


  # --- Extract observed data ------------------------------------------------
  if (is.null(status)) {
    status <- object$data$status
  }
  if (is.null(event_time)) {
    event_time <- object$data$time
  }
  n_obs <- length(status)
  if (length(event_time) != n_obs) {
    stop("'status' and 'event_time' must have the same length.",
         call. = FALSE)
  }
  if (any(!is.finite(event_time)) || any(event_time < 0)) {
    stop("'event_time' must be finite and non-negative.", call. = FALSE)
  }
  if (any(!is.finite(status)) || any(!status %in% c(0, 1))) {
    stop("'status' must be coded as 0/1 with finite values.", call. = FALSE)
  }

  n_model <- length(object$data$time)
  if (n_obs != n_model) {
    stop("'status'/'event_time' lengths must match the fitted data length (",
         n_model, ").", call. = FALSE)
  }

  # --- Predicted cumulative hazard and survival at the target time ----------
  # Compute per-observation predictions at the calibration horizon.
  if (identical(object$spec$dist, "multiphase")) {
    phases <- object$fit$phases
    if (is.null(phases)) phases <- object$spec$phases
    cov_counts <- object$fit$covariate_counts
    x_list <- object$fit$x_list
    cumhaz <- .hzr_multiphase_cumhaz(
      rep(time, n_model), object$fit$theta, phases, cov_counts, x_list,
      per_phase = FALSE
    )
  } else if (!is.null(object$data$x) && ncol(object$data$x) > 0) {
    nd <- as.data.frame(object$data$x)
    nd$time <- time
    cumhaz <- predict(object, newdata = nd, type = "cumulative_hazard")
  } else {
    nd <- data.frame(time = rep(time, n_model))
    cumhaz <- predict(object, newdata = nd, type = "cumulative_hazard")
  }
  survival <- exp(-cumhaz)

  # Exclude subjects censored before the calibration horizon.
  include <- (status == 1) | (event_time >= time)
  n_excluded <- sum(!include)
  if (!any(include)) {
    stop("No observations are available at the requested horizon after ",
         "excluding subjects censored before 'time'.", call. = FALSE)
  }

  status <- status[include]
  event_time <- event_time[include]
  cumhaz <- cumhaz[include]
  survival <- survival[include]
  observed <- as.integer(status == 1 & event_time <= time)
  n_included <- length(observed)

  if (groups > n_included) {
    stop("'groups' must be <= number of included observations at horizon (",
         n_included, ").", call. = FALSE)
  }

  # --- Rank into groups by predicted risk (cumulative hazard) ---------------
  # Higher cumhaz = higher risk; group 1 = lowest risk.
  # Use ntile-style assignment that handles ties gracefully (no duplicate

  # breaks).  When all predictions are identical (intercept-only model),
  # observations are distributed as evenly as possible across groups.
  ranks <- rank(cumhaz, ties.method = "first")
  group <- as.integer(cut(ranks,
                          breaks = seq(0, n_included,
                                       length.out = groups + 1L),
                          include.lowest = TRUE,
                          labels = seq_len(groups)))

  # --- Aggregate by group ---------------------------------------------------
  result <- data.frame(
    group = seq_len(groups),
    n = integer(groups),
    events = numeric(groups),
    expected = numeric(groups),
    observed_rate = numeric(groups),
    expected_rate = numeric(groups),
    chi_sq = numeric(groups),
    p_value = numeric(groups),
    mean_survival = numeric(groups),
    mean_cumhaz = numeric(groups)
  )

  for (g in seq_len(groups)) {
    idx <- which(group == g)
    ng <- length(idx)
    obs_events <- sum(observed[idx] == 1)
    exp_events <- sum(1 - survival[idx])

    result$n[g] <- ng
    result$events[g] <- obs_events
    result$expected[g] <- exp_events
    result$observed_rate[g] <- if (ng > 0) obs_events / ng else NA_real_
    result$expected_rate[g] <- if (ng > 0) exp_events / ng else NA_real_
    result$mean_survival[g] <- if (ng > 0) mean(survival[idx]) else NA_real_
    result$mean_cumhaz[g] <- if (ng > 0) mean(cumhaz[idx]) else NA_real_

    # Per-group chi-square: (O - E)^2 / E
    if (exp_events > 0) {
      result$chi_sq[g] <- (obs_events - exp_events)^2 / exp_events
      # Upper-tail p-value from chi-square with 1 df
      result$p_value[g] <- stats::pchisq(result$chi_sq[g], df = 1,
                                          lower.tail = FALSE)
    } else {
      result$chi_sq[g] <- NA_real_
      result$p_value[g] <- NA_real_
    }
  }

  # --- Overall chi-square ---------------------------------------------------
  valid <- !is.na(result$chi_sq)
  overall_chi_sq <- sum(result$chi_sq[valid])
  overall_df <- sum(valid) - 1L
  overall_p <- if (overall_df > 0) {
    stats::pchisq(overall_chi_sq, df = overall_df, lower.tail = FALSE)
  } else {
    NA_real_
  }

  attr(result, "overall") <- list(
    chi_sq = overall_chi_sq,
    df = overall_df,
    p_value = overall_p,
    time = time,
    groups = groups,
    total_events = sum(observed == 1),
    total_expected = sum(1 - survival),
    n_included = n_included,
    n_excluded = n_excluded
  )

  class(result) <- c("hzr_deciles", "data.frame")
  result
}

#' Print method for hzr_deciles
#'
#' @param x An `hzr_deciles` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @return The object \code{x} of class \code{c("hzr_deciles", "data.frame")},
#'   invisibly. The data frame has one row per risk group and columns:
#'   \code{group} (integer group index, 1 = lowest risk),
#'   \code{n} (group size),
#'   \code{events} (observed event count),
#'   \code{expected} (expected event count from model predictions),
#'   \code{observed_rate}, \code{expected_rate} (events / n),
#'   \code{chi_sq} (per-group (O-E)^2/E contribution),
#'   \code{p_value} (1-df chi-square upper-tail p),
#'   \code{mean_survival}, \code{mean_cumhaz} (mean predicted values in group).
#'   An \code{"overall"} attribute contains the omnibus chi-square test
#'   (fields: \code{chi_sq}, \code{df}, \code{p_value}, \code{time},
#'   \code{groups}, \code{total_events}, \code{total_expected},
#'   \code{n_included}, \code{n_excluded}).
#' @export
print.hzr_deciles <- function(x, digits = 3, ...) {
  ov <- attr(x, "overall")
  cat("Decile-of-risk calibration at time =", ov$time, "\n")
  if (!is.null(ov$n_included) && !is.null(ov$n_excluded)) {
    cat("Included", ov$n_included, "observations (excluded", ov$n_excluded,
        "censored before horizon).\n")
  }
  cat(ov$groups, "groups,", ov$total_events, "observed events,",
      signif(ov$total_expected, digits), "expected\n\n")

  # Format for display
  display <- x
  display$expected <- signif(display$expected, digits)
  display$observed_rate <- signif(display$observed_rate, digits)
  display$expected_rate <- signif(display$expected_rate, digits)
  display$chi_sq <- signif(display$chi_sq, digits)
  display$p_value <- signif(display$p_value, digits)
  display$mean_survival <- signif(display$mean_survival, digits)
  display$mean_cumhaz <- signif(display$mean_cumhaz, digits)
  class(display) <- "data.frame"
  print(display, row.names = FALSE)

  cat("\nOverall: chi-sq =", signif(ov$chi_sq, digits),
      "on", ov$df, "df, p =",
      format.pval(ov$p_value, digits = digits), "\n")

  invisible(x)
}


# =========================================================================
# hzr_gof -- Observed vs. expected goodness-of-fit
# =========================================================================

#' Goodness-of-fit: observed vs. predicted events
#'
#' Compare a fitted hazard model against the nonparametric Kaplan-Meier
#' estimate by computing observed and expected (parametric) event counts
#' at each distinct event time.  This is the R equivalent of the SAS
#' `hazplot.sas` macro and implements the conservation-of-events
#' diagnostic.
#'
#' At each observed event time the function computes:
#' \itemize{
#'   \item The Kaplan-Meier survival and cumulative hazard.
#'   \item The parametric survival and cumulative hazard from the fitted
#'     model (and per-phase components for multiphase models).
#'   \item Cumulative observed events vs. cumulative expected events
#'     (sum of individual cumulative hazards for those exiting the risk
#'     set at each time).
#'   \item The running residual (expected minus observed).
#' }
#'
#' Perfect model fit implies the expected and observed event counts track
#' each other (residual near zero).  This is the conservation-of-events
#' principle.
#'
#' @param object A fitted `hazard` object (with `fit = TRUE`).
#' @param time_grid Optional numeric vector of time points at which to
#'   evaluate the parametric model.
#'   If `NULL` (default), uses the sorted unique event times from the
#'   fitted data.
#'
#' @return A data frame with one row per time point and columns:
#' \describe{
#'   \item{time}{Evaluation time.}
#'   \item{n_risk}{Number at risk (Kaplan-Meier).}
#'   \item{n_event}{Number of events at this time.}
#'   \item{n_censor}{Number censored at this time.}
#'   \item{km_surv}{Kaplan-Meier survival estimate.}
#'   \item{km_cumhaz}{Kaplan-Meier cumulative hazard
#'     (\eqn{-\log(\text{km\_surv})}).}
#'   \item{par_surv}{Parametric survival from the fitted model.}
#'   \item{par_cumhaz}{Parametric cumulative hazard.}
#'   \item{cum_observed}{Cumulative observed events to this time.}
#'   \item{cum_expected}{Cumulative expected events (sum of individual
#'     cumulative hazards for observations exiting the risk set).}
#'   \item{residual}{Expected minus observed
#'     (\code{cum_expected - cum_observed}).}
#' }
#'
#' For multiphase models, additional columns are appended for each
#' phase: \code{par_cumhaz_<phase>}.
#'
#' An attribute `"summary"` is attached with scalar diagnostics:
#' total observed events, total expected events, and the final residual.
#'
#' @examples
#' \donttest{
#' data(avc)
#' avc <- na.omit(avc)
#' fit <- hazard(
#'   survival::Surv(int_dead, dead) ~ age + mal,
#'   data  = avc,
#'   dist  = "weibull",
#'   theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
#'   fit   = TRUE
#' )
#' gof <- hzr_gof(fit)
#' print(gof)
#'
#' # Plot observed vs expected events
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(gof, aes(x = time)) +
#'     geom_line(aes(y = cum_observed), colour = "#D55E00") +
#'     geom_line(aes(y = cum_expected), colour = "#0072B2") +
#'     labs(x = "Time", y = "Cumulative events") +
#'     theme_minimal()
#' }
#' }
#'
#' @seealso [hzr_deciles()] for decile-of-risk calibration,
#'   [predict.hazard()] for prediction types.
#' @export
hzr_gof <- function(object, time_grid = NULL) {
  if (!inherits(object, "hazard")) {
    stop("'object' must be a fitted hazard object.", call. = FALSE)
  }
  if (is.null(object$fit$theta) ||
      (is.logical(object$fit$converged) && is.na(object$fit$converged))) {
    stop("'object' has no fitted parameters. Refit with fit = TRUE.",
         call. = FALSE)
  }

  # --- Extract observed data ------------------------------------------------
  obs_time   <- object$data$time
  obs_status <- object$data$status
  n_total    <- length(obs_time)

  if (length(obs_status) != n_total) {
    stop("Stored 'time' and 'status' vectors must have the same length.",
         call. = FALSE)
  }
  if (any(!is.finite(obs_time)) || any(obs_time < 0)) {
    stop("Stored event/censoring times must be finite and non-negative.",
         call. = FALSE)
  }
  if (any(!is.finite(obs_status)) || any(!obs_status %in% c(0, 1))) {
    stop("Stored status must be coded as 0/1 with finite values.",
         call. = FALSE)
  }

  # --- Kaplan-Meier via survival::survfit -----------------------------------
  km_fit <- survival::survfit(survival::Surv(obs_time, obs_status) ~ 1)

  # survfit output: time, n.risk, n.event, n.censor, surv
  km_times   <- km_fit$time
  km_n_risk  <- km_fit$n.risk
  km_n_event <- km_fit$n.event
  km_n_censor <- km_fit$n.censor
  km_surv    <- km_fit$surv

  # --- Decide time grid -----------------------------------------------------
  if (is.null(time_grid)) {
    time_grid <- km_times
  }

  # --- Parametric predictions at each time point ----------------------------
  is_multiphase <- (object$spec$dist == "multiphase")

  if (!is.null(object$data$x) && ncol(object$data$x) > 0) {
    # For covariate models, evaluate at covariate means (baseline patient)
    x_means <- colMeans(object$data$x)
    nd <- as.data.frame(t(x_means))
    nd <- nd[rep(1, length(time_grid)), , drop = FALSE]
    nd$time <- time_grid
  } else {
    nd <- data.frame(time = time_grid)
  }

  par_cumhaz <- predict(object, newdata = nd, type = "cumulative_hazard")

  # Phase decomposition for multiphase models
  phase_cumhaz <- NULL
  if (is_multiphase) {
    decomp <- predict(object, newdata = nd, type = "cumulative_hazard",
                      decompose = TRUE)
    # decomp is a matrix; first column is "total", rest are phase names
    phase_cols <- colnames(decomp)[colnames(decomp) != "total"]
    phase_cumhaz <- as.data.frame(decomp[, phase_cols, drop = FALSE])
  }

  par_surv <- exp(-par_cumhaz)

  # --- Interpolate KM at the time grid --------------------------------------
  # Use stepfun-style interpolation for KM (right-continuous)
  km_surv_at_grid <- stats::approx(
    x = c(0, km_times), y = c(1, km_surv),
    xout = time_grid, method = "constant", f = 0, rule = 2
  )$y
  km_cumhaz_at_grid <- -log(pmax(km_surv_at_grid, .Machine$double.xmin))

  # Interpolate n.risk, n.event, n.censor at grid times
  # For event counts, sum events at matching times; 0 otherwise
  km_n_risk_grid <- stats::approx(
    x = c(0, km_times), y = c(n_total, km_n_risk),
    xout = time_grid, method = "constant", f = 0, rule = 2
  )$y
  km_n_event_grid <- rep(0, length(time_grid))
  km_n_censor_grid <- rep(0, length(time_grid))
  for (i in seq_along(km_times)) {
    match_idx <- which(abs(time_grid - km_times[i]) < .Machine$double.eps * 100)
    if (length(match_idx) > 0) {
      km_n_event_grid[match_idx[1]] <- km_n_event[i]
      km_n_censor_grid[match_idx[1]] <- km_n_censor[i]
    }
  }

  # --- Conservation of Events accounting ------------------------------------
  # At each event time, accumulate:
  #   cum_observed: running sum of observed events
  #   cum_expected: running sum of individual cumulative hazards for
  #                 observations exiting the risk set (events + censored)
  #
  # The expected events for observations leaving at time t is:
  #   (n_event + n_censor) * parametric_cumhaz(t)
  # This is the SAS hazplot approach: total * _CUMHAZ at that interval.

  cum_observed <- cumsum(km_n_event_grid)
  interval_expected <- (km_n_event_grid + km_n_censor_grid) * par_cumhaz
  cum_expected <- cumsum(interval_expected)
  residual <- cum_expected - cum_observed

  # --- Assemble result ------------------------------------------------------
  result <- data.frame(
    time         = time_grid,
    n_risk       = km_n_risk_grid,
    n_event      = km_n_event_grid,
    n_censor     = km_n_censor_grid,
    km_surv      = km_surv_at_grid,
    km_cumhaz    = km_cumhaz_at_grid,
    par_surv     = par_surv,
    par_cumhaz   = par_cumhaz,
    cum_observed = cum_observed,
    cum_expected = cum_expected,
    residual     = residual
  )

  # Add phase columns for multiphase
  if (!is.null(phase_cumhaz)) {
    for (ph in names(phase_cumhaz)) {
      result[[paste0("par_cumhaz_", ph)]] <- phase_cumhaz[[ph]]
    }
  }

  # Summary diagnostics
  attr(result, "summary") <- list(
    total_observed = cum_observed[length(cum_observed)],
    total_expected = cum_expected[length(cum_expected)],
    final_residual = residual[length(residual)],
    dist = object$spec$dist,
    n = n_total
  )

  class(result) <- c("hzr_gof", "data.frame")
  result
}

#' Print method for hzr_gof
#'
#' @param x An `hzr_gof` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @return The object \code{x} of class \code{c("hzr_gof", "data.frame")},
#'   invisibly. The data frame has one row per time point and columns:
#'   \code{time}, \code{n_risk}, \code{n_event}, \code{n_censor},
#'   \code{km_surv} (Kaplan-Meier survival), \code{km_cumhaz},
#'   \code{par_surv} (parametric survival), \code{par_cumhaz},
#'   \code{cum_observed} (cumulative observed events),
#'   \code{cum_expected} (cumulative expected events from model),
#'   \code{residual} (cum_expected - cum_observed).
#'   Multiphase models additionally include \code{par_cumhaz_<phase>} columns
#'   for per-phase cumulative hazard contributions.
#'   A \code{"summary"} attribute contains scalar diagnostics:
#'   \code{total_observed}, \code{total_expected}, \code{final_residual},
#'   \code{dist}, \code{n}.
#' @export
print.hzr_gof <- function(x, digits = 3, ...) {
  s <- attr(x, "summary")
  cat("Goodness-of-fit: observed vs. expected events\n")
  cat("Distribution:", s$dist, " | n =", s$n, "\n\n")
  cat("Total observed events:", s$total_observed, "\n")
  cat("Total expected events:", round(s$total_expected, digits), "\n")
  cat("Final residual (E - O):", round(s$final_residual, digits), "\n")
  cat("Conservation ratio (E/O):",
      round(s$total_expected / max(s$total_observed, 1), digits), "\n")
  cat("\nUse plot columns: time, km_surv, par_surv, cum_observed,",
      "cum_expected, residual\n")
  invisible(x)
}


# =========================================================================
# hzr_kaplan -- Kaplan-Meier with exact logit confidence limits
# =========================================================================

#' Kaplan-Meier survival with exact logit confidence limits
#'
#' Compute the product-limit (Kaplan-Meier) survival estimate with
#' logit-transformed confidence limits that respect the \eqn{[0, 1]}
#' boundary.
#' This is the R equivalent of the SAS `kaplan.sas` macro.
#'
#' The standard Greenwood confidence interval can exceed \eqn{[0, 1]} in the
#' tails. The logit-transformed interval avoids this by working on the
#' log-odds scale:
#'
#' \deqn{
#'   \text{CL}_{\text{lower}} = S / \bigl(S + (1-S)\,
#'   \exp(z_\alpha\,\text{SI})\bigr)
#' }
#' \deqn{
#'   \text{CL}_{\text{upper}} = S / \bigl(S + (1-S)\,
#'   \exp(-z_\alpha\,\text{SI})\bigr)
#' }
#'
#' where \eqn{\text{SI} = \sqrt{V_P - 1} / (1 - S)}, \eqn{V_P} is the
#' cumulative Greenwood variance product, and \eqn{z_\alpha} is the
#' normal quantile for the requested confidence level.
#'
#' @param time Numeric vector of follow-up times.
#' @param status Numeric event indicator (1 = event, 0 = censored).
#' @param conf_level Confidence level for the interval (default 0.95).
#'   The SAS default of 0.68268948 corresponds to a 1-SD interval.
#' @param event_only Logical; if `TRUE` (default), only return rows at
#'   event times (where `n_event > 0`).
#'   If `FALSE`, return rows at all times reported by
#'   `survival::survfit()` (events and censoring times).
#'
#' @return A data frame with one row per time point and columns:
#' \describe{
#'   \item{time}{Event/censoring time.}
#'   \item{n_risk}{Number at risk at start of interval.}
#'   \item{n_event}{Number of events at this time.}
#'   \item{n_censor}{Number censored at this time.}
#'   \item{survival}{Kaplan-Meier survival estimate.}
#'   \item{std_err}{Standard error of survival (Greenwood).}
#'   \item{cl_lower}{Lower confidence limit (logit-transformed).}
#'   \item{cl_upper}{Upper confidence limit (logit-transformed).}
#'   \item{cumhaz}{Cumulative hazard \eqn{= -\log(S)}.}
#'   \item{hazard}{Interval hazard rate
#'     \eqn{= \log(S_{t-1} / S_t) / \Delta t}.}
#'   \item{density}{Probability density estimate
#'     \eqn{= (S_{t-1} - S_t) / \Delta t}.}
#'   \item{life}{Restricted mean survival time (area under curve to
#'     this time).}
#' }
#'
#' @examples
#' data(cabgkul)
#' km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)
#' head(km)
#'
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(km, aes(time)) +
#'     geom_step(aes(y = survival * 100)) +
#'     geom_ribbon(aes(ymin = cl_lower * 100, ymax = cl_upper * 100),
#'                 stat = "identity", alpha = 0.2) +
#'     labs(x = "Months", y = "Survival (%)") +
#'     theme_minimal()
#' }
#' }
#'
#' @seealso [hzr_gof()] for parametric vs. nonparametric comparison.
#' @export
hzr_kaplan <- function(time, status, conf_level = 0.95,
                       event_only = TRUE) {
  if (!is.numeric(time) || !is.numeric(status)) {
    stop("'time' and 'status' must be numeric vectors.", call. = FALSE)
  }
  if (length(time) != length(status)) {
    stop("'time' and 'status' must have the same length.", call. = FALSE)
  }
  if (conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1.", call. = FALSE)
  }

  z_alpha <- stats::qnorm(0.5 + 0.5 * conf_level)

  # Use survival::survfit for the core KM computation
  km_fit <- survival::survfit(survival::Surv(time, status) ~ 1)

  n_times <- length(km_fit$time)

  # --- Build output vectors at each event/censoring time --------------------
  km_time    <- km_fit$time
  km_n_risk  <- km_fit$n.risk
  km_n_event <- km_fit$n.event
  km_n_censor <- km_fit$n.censor
  km_surv    <- km_fit$surv

  # --- Greenwood variance product and exact logit CL -----------------------
  # VAR_PROD = prod_i [1/(n_i - d_i) - 1/n_i + 1] for all event times
  # The product is cumulative: each time contributes a multiplicative factor.
  var_prod <- rep(1.0, n_times)
  for (i in seq_len(n_times)) {
    ni <- km_n_risk[i]
    di <- km_n_event[i]
    if (ni > 0 && (ni - di) > 0 && di > 0) {
      factor_i <- 1.0 / (ni - di) - 1.0 / ni + 1.0
      if (i == 1L) {
        var_prod[i] <- factor_i
      } else {
        var_prod[i] <- var_prod[i - 1L] * factor_i
      }
    } else if (i > 1L) {
      var_prod[i] <- var_prod[i - 1L]
    }
  }

  # Standard error (Greenwood)
  var_exact <- km_surv^2 * pmax(var_prod - 1, 0)
  std_err <- sqrt(var_exact)

  # Logit-transformed confidence limits (SAS SI_EXACT formula)
  si_exact <- rep(0.0, n_times)
  idx_valid <- km_surv < 1 & km_surv > 0
  si_exact[idx_valid] <- sqrt(pmax(var_prod[idx_valid] - 1, 0)) /
    (1 - km_surv[idx_valid])

  cl_lower <- km_surv / (km_surv + (1 - km_surv) *
                            exp(z_alpha * si_exact))
  cl_upper <- km_surv / (km_surv + (1 - km_surv) *
                            exp(-z_alpha * si_exact))

  # Edge cases: if S = 1 or S = 0, CL = S

  cl_lower[km_surv >= 1] <- 1
  cl_upper[km_surv >= 1] <- 1
  cl_lower[km_surv <= 0] <- 0
  cl_upper[km_surv <= 0] <- 0

  # --- Cumulative hazard -----------------------------------------------------
  cumhaz <- -log(pmax(km_surv, .Machine$double.xmin))

  # --- Interval hazard and density ------------------------------------------
  lag_surv <- c(1, km_surv[-n_times])
  lag_time <- c(0, km_time[-n_times])
  delta_t <- km_time - lag_time

  hazard <- rep(NA_real_, n_times)
  density <- rep(NA_real_, n_times)
  idx_haz <- km_n_event > 0 & delta_t > 0 & km_surv > 0
  hazard[idx_haz] <- log(lag_surv[idx_haz] / km_surv[idx_haz]) /
    delta_t[idx_haz]
  density[idx_haz] <- (lag_surv[idx_haz] - km_surv[idx_haz]) /
    delta_t[idx_haz]

  # --- Restricted mean survival (life integral) -----------------------------
  # KM survival is a right-continuous step function.  RMST is accumulated
  # as the sum of rectangle areas: each interval contributes dt * S(t-1).
  life <- rep(0, n_times)
  lag_life <- 0
  for (i in seq_len(n_times)) {
    life[i] <- lag_life + delta_t[i] * lag_surv[i]
    lag_life <- life[i]
  }

  # --- Assemble result ------------------------------------------------------
  result <- data.frame(
    time      = km_time,
    n_risk    = km_n_risk,
    n_event   = km_n_event,
    n_censor  = km_n_censor,
    survival  = km_surv,
    std_err   = std_err,
    cl_lower  = cl_lower,
    cl_upper  = cl_upper,
    cumhaz    = cumhaz,
    hazard    = hazard,
    density   = density,
    life      = life
  )

  if (event_only) {
    result <- result[result$n_event > 0, , drop = FALSE]
    rownames(result) <- NULL
  }

  class(result) <- c("hzr_kaplan", "data.frame")
  result
}

#' Print method for hzr_kaplan
#'
#' @param x An `hzr_kaplan` object.
#' @param digits Number of decimal places for formatting.
#' @param n Maximum rows to print (default 20).
#' @param ... Additional arguments (ignored).
#' @return The object \code{x} of class \code{c("hzr_kaplan", "data.frame")},
#'   invisibly. The data frame has one row per event time (or all times when
#'   \code{event_only = FALSE}) and columns:
#'   \code{time} (follow-up time),
#'   \code{n_risk} (number at risk),
#'   \code{n_event} (events in interval),
#'   \code{n_censor} (censored observations in interval),
#'   \code{survival} (Kaplan-Meier survival estimate),
#'   \code{std_err} (Greenwood standard error on log-hazard scale),
#'   \code{cl_lower}, \code{cl_upper} (logit-transformed confidence limits
#'   on the survival scale),
#'   \code{cumhaz} (Nelson-Aalen cumulative hazard),
#'   \code{hazard} (interval hazard estimate),
#'   \code{density} (estimated event density),
#'   \code{life} (life-table life expectancy contribution).
#' @export
print.hzr_kaplan <- function(x, digits = 4, n = 20, ...) {
  cat("Kaplan-Meier estimate with logit confidence limits\n")
  total_events <- sum(x$n_event)
  cat("Events:", total_events, " | Time points:", nrow(x), "\n")
  if (nrow(x) > 0) {
    cat("Survival range:",
        round(min(x$survival), digits), "to",
        round(max(x$survival), digits), "\n")
    cat("RMST at last event:", round(x$life[nrow(x)], digits), "\n\n")
  }
  display <- x
  display$survival <- round(display$survival, digits)
  display$std_err <- round(display$std_err, digits)
  display$cl_lower <- round(display$cl_lower, digits)
  display$cl_upper <- round(display$cl_upper, digits)
  display$cumhaz <- round(display$cumhaz, digits)
  display$hazard <- round(display$hazard, digits)
  display$density <- round(display$density, digits)
  display$life <- round(display$life, digits)
  class(display) <- "data.frame"
  if (nrow(display) > n) {
    print(utils::head(display, n), row.names = FALSE)
    cat("... (", nrow(display) - n, " more rows)\n")
  } else {
    print(display, row.names = FALSE)
  }
  invisible(x)
}


# =========================================================================
# hzr_calibrate -- Variable calibration (logit / Gompertz / Cox transform)
# =========================================================================

#' Calibrate a continuous variable against an outcome
#'
#' Group a continuous covariate into quantile bins, compute the event
#' probability (or hazard rate) per bin, and apply a link transform
#' (logit, Gompertz, or Cox).
#' This is the R equivalent of the SAS `logit.sas` and `logitgr.sas`
#' macros.
#'
#' Use this function before model entry to assess whether the covariate
#' relationship with the outcome is approximately linear on the link
#' scale. If the transformed probabilities are roughly linear against
#' the group means, the covariate can enter the model untransformed.
#' Curvature suggests a transformation (log, quadratic) may improve fit.
#'
#' @param x Numeric vector: the continuous covariate to calibrate.
#' @param event Numeric vector: event indicator (1 = event, 0 = no event).
#' @param groups Integer: number of quantile bins (default 10).
#' @param by Optional factor or character vector for stratified calibration
#'   (SAS `logitgr.sas` functionality). If provided, calibration is
#'   computed within each stratum. Default `NULL` (no stratification).
#' @param link Character: transform to apply to event probabilities.
#'   One of `"logit"` (default), `"gompertz"` (complementary log-log),
#'   or `"cox"`.
#' @param time Optional numeric vector: follow-up time, required when
#'   `link = "cox"`. The Cox link computes
#'   \eqn{\log(\text{events} / \sum \text{time})} (constant hazard rate).
#'
#' @return A data frame with one row per group (or per group-by-stratum
#'   combination) and columns:
#' \describe{
#'   \item{group}{Integer group label.}
#'   \item{by}{Stratum level (only present when `by` is provided).}
#'   \item{n}{Number of observations in the group.}
#'   \item{events}{Number of events.}
#'   \item{mean}{Mean of `x` within the group.}
#'   \item{min}{Minimum of `x` within the group.}
#'   \item{max}{Maximum of `x` within the group.}
#'   \item{prob}{Event probability (events / n), or for Cox link:
#'     events / sum(time).}
#'   \item{link_value}{Transformed probability on the chosen link scale.}
#' }
#'
#' @examples
#' data(avc)
#' avc <- na.omit(avc)
#'
#' # Logit calibration of age
#' cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)
#' print(cal)
#'
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(cal, aes(mean, link_value)) +
#'     geom_point(size = 3) +
#'     geom_smooth(method = "lm", formula = y ~ x, se = FALSE,
#'                 linetype = "dashed") +
#'     labs(x = "Age at repair (months)", y = "Logit(P(death))") +
#'     theme_minimal()
#' }
#' }
#'
#' @seealso [hzr_deciles()] for model-based calibration after fitting.
#' @export
hzr_calibrate <- function(x, event, groups = 10L, by = NULL,
                          link = c("logit", "gompertz", "cox"),
                          time = NULL) {
  link <- match.arg(link)

  if (!is.numeric(x) || !is.numeric(event)) {
    stop("'x' and 'event' must be numeric vectors.", call. = FALSE)
  }
  if (length(x) != length(event)) {
    stop("'x' and 'event' must have the same length.", call. = FALSE)
  }
  groups <- as.integer(groups)
  if (groups < 2L) {
    stop("'groups' must be at least 2.", call. = FALSE)
  }
  if (link == "cox" && is.null(time)) {
    stop("'time' is required when link = 'cox'.", call. = FALSE)
  }
  if (!is.null(time) && length(time) != length(x)) {
    stop("'time' must have the same length as 'x'.", call. = FALSE)
  }

  # --- Assign quantile groups -----------------------------------------------
  ranks <- rank(x, ties.method = "first")
  n <- length(x)
  group <- as.integer(cut(ranks,
                          breaks = seq(0, n, length.out = groups + 1L),
                          include.lowest = TRUE,
                          labels = seq_len(groups)))

  # --- Build data frame for aggregation ------------------------------------
  df <- data.frame(x = x, event = event, group = group)
  if (!is.null(time)) df$time <- time
  if (!is.null(by)) df$by <- by

  # --- Aggregation function -------------------------------------------------
  .calibrate_group <- function(d) {
    ng <- nrow(d)
    ev <- sum(d$event)
    if (link == "cox") {
      prob <- if (sum(d$time) > 0) ev / sum(d$time) else NA_real_
    } else {
      prob <- if (ng > 0) ev / ng else NA_real_
    }

    # Apply link transform
    link_val <- if (is.na(prob) || prob <= 0 || prob >= 1) {
      NA_real_
    } else if (link == "logit") {
      log(prob / (1 - prob))
    } else if (link == "gompertz") {
      log(-log(1 - prob))
    } else {
      # Cox: prob is already events/time (hazard rate)
      if (prob > 0) log(prob) else NA_real_
    }

    data.frame(
      n      = ng,
      events = ev,
      mean   = mean(d$x),
      min    = min(d$x),
      max    = max(d$x),
      prob   = prob,
      link_value = link_val
    )
  }

  # --- Aggregate by group (and optionally by stratum) -----------------------
  if (is.null(by)) {
    result_list <- lapply(split(df, df$group), .calibrate_group)
    result <- do.call(rbind, result_list)
    result$group <- as.integer(rownames(result))
    rownames(result) <- NULL
    result <- result[order(result$group),
                     c("group", "n", "events", "mean", "min", "max",
                       "prob", "link_value")]
  } else {
    splits <- split(df, list(df$by, df$group))
    result_list <- lapply(splits, function(d) {
      if (nrow(d) == 0) return(NULL)
      r <- .calibrate_group(d)
      r$by <- d$by[1]
      r$group <- d$group[1]
      r
    })
    result <- do.call(rbind, Filter(Negate(is.null), result_list))
    rownames(result) <- NULL
    result <- result[order(result$by, result$group),
                     c("group", "by", "n", "events", "mean", "min", "max",
                       "prob", "link_value")]
  }

  attr(result, "link") <- link
  attr(result, "groups") <- groups
  class(result) <- c("hzr_calibrate", "data.frame")
  result
}

#' Print method for hzr_calibrate
#'
#' @param x An `hzr_calibrate` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @return The object \code{x} of class \code{c("hzr_calibrate", "data.frame")},
#'   invisibly. The data frame has one row per quantile group and columns:
#'   \code{group} (group index),
#'   \code{n} (group size),
#'   \code{events} (event count),
#'   \code{mean}, \code{min}, \code{max} (covariate summary within group),
#'   \code{prob} (observed event probability),
#'   \code{link_value} (transformed probability on the link scale).
#'   When stratified via the \code{by} argument, a \code{by} column is also
#'   present. Attributes: \code{"link"} (the transform applied,
#'   e.g. \code{"logit"}) and \code{"groups"} (number of quantile bins).
#' @export
print.hzr_calibrate <- function(x, digits = 3, ...) {
  lnk <- attr(x, "link")
  grp <- attr(x, "groups")
  cat("Variable calibration (", lnk, " link, ", grp, " groups)\n",
      sep = "")
  if ("by" %in% names(x)) {
    cat("Stratified by:", paste(unique(x$by), collapse = ", "), "\n")
  }
  cat("\n")
  display <- x
  display$mean <- round(display$mean, digits)
  display$min <- round(display$min, digits)
  display$max <- round(display$max, digits)
  display$prob <- round(display$prob, digits)
  display$link_value <- round(display$link_value, digits)
  class(display) <- "data.frame"
  print(display, row.names = FALSE)
  invisible(x)
}


# =========================================================================
# hzr_nelson -- Wayne Nelson cumulative hazard estimator
# =========================================================================

#' Wayne Nelson cumulative hazard estimator with lognormal confidence limits
#'
#' Compute the Nelson-Aalen cumulative hazard estimate with lognormal
#' confidence limits. Supports weighted events for severity-adjusted
#' analyses of repeated/recurrent events.
#' This is the R equivalent of the SAS `nelsonl.sas` macro.
#'
#' Unlike `survival::survfit()` which uses the Breslow estimator with
#' Greenwood variance, this function uses the Wayne Nelson estimator
#' with lognormal confidence limits that are always non-negative.
#'
#' @param time Numeric vector of follow-up times.
#' @param event Numeric event indicator (1 = event, 0 = censored).
#' @param weight Optional numeric vector of event weights (default 1).
#'   Weights are applied only to events (censored observations contribute
#'   zero weight). Use for severity-weighted repeated events.
#' @param conf_level Confidence level for the interval (default 0.95).
#'
#' @return A data frame with one row per unique event time and columns:
#' \describe{
#'   \item{time}{Event time.}
#'   \item{n_risk}{Number at risk.}
#'   \item{n_event}{Number of events at this time.}
#'   \item{weight_sum}{Sum of event weights at this time.}
#'   \item{cumhaz}{Nelson cumulative hazard estimate.}
#'   \item{std_err}{Standard error.}
#'   \item{cl_lower}{Lower lognormal confidence limit.}
#'   \item{cl_upper}{Upper lognormal confidence limit.}
#'   \item{hazard}{Interval hazard rate.}
#'   \item{cum_events}{Cumulative (weighted) event count.}
#' }
#'
#' @examples
#' data(cabgkul)
#' nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)
#' head(nel)
#'
#' @seealso [hzr_kaplan()] for survival estimation.
#' @export
hzr_nelson <- function(time, event, weight = NULL, conf_level = 0.95) {
  if (!is.numeric(time) || !is.numeric(event)) {
    stop("'time' and 'event' must be numeric vectors.", call. = FALSE)
  }
  n <- length(time)
  if (length(event) != n) {
    stop("'time' and 'event' must have the same length.", call. = FALSE)
  }
  if (is.null(weight)) weight <- rep(1, n)
  if (length(weight) != n) {
    stop("'weight' must have the same length as 'time'.", call. = FALSE)
  }
  if (conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1.", call. = FALSE)
  }

  z_alpha <- stats::qnorm(0.5 + 0.5 * conf_level)

  # Effective weight: weight * event (censored get 0)
  e_wght <- weight * event

  # Sort by time

  ord <- order(time)
  time_s <- time[ord]
  event_s <- event[ord]
  e_wght_s <- e_wght[ord]

  # Collapse to unique times
  u_times <- sort(unique(time_s[event_s == 1]))
  if (length(u_times) == 0) {
    out <- data.frame(time = numeric(0), n_risk = numeric(0),
                      n_event = numeric(0), weight_sum = numeric(0),
                      cumhaz = numeric(0), std_err = numeric(0),
                      cl_lower = numeric(0), cl_upper = numeric(0),
                      hazard = numeric(0), cum_events = numeric(0))
    class(out) <- c("hzr_nelson", "data.frame")
    return(out)
  }

  out_n <- length(u_times)
  n_risk <- numeric(out_n)
  n_event_out <- numeric(out_n)
  weight_sum <- numeric(out_n)
  cumhaz <- numeric(out_n)

  # Single pass: compute cumhaz, variance, and lognormal CL together
  std_err <- numeric(out_n)
  cl_lower <- numeric(out_n)
  cl_upper <- numeric(out_n)

  run_i_nrisk <- 0
  run_it <- 0
  cum_dist <- 0

  for (k in seq_along(u_times)) {
    t_k <- u_times[k]
    n_risk[k] <- sum(time_s >= t_k)
    at_time <- time_s == t_k & event_s == 1
    n_event_out[k] <- sum(at_time)
    weight_sum[k] <- sum(e_wght_s[at_time])

    # Nelson estimator: dist = sum(weight) / n_risk
    dist_k <- if (n_risk[k] > 0) weight_sum[k] / n_risk[k] else 0
    cum_dist <- cum_dist + dist_k
    cumhaz[k] <- cum_dist

    # Variance and lognormal CL (running accumulators)
    if (n_risk[k] > 0) run_i_nrisk <- run_i_nrisk + 1 / n_risk[k]
    run_it <- run_it + n_event_out[k]

    if (run_it > 0 && cum_dist > 0) {
      var_cef <- run_i_nrisk * cum_dist / run_it
      std_err[k] <- sqrt(var_cef)

      sigma2 <- log(run_i_nrisk / (run_it * cum_dist) + 1)
      sigma <- sqrt(sigma2)
      mu_ln <- log(cum_dist) - sigma2 / 2

      cl_upper[k] <- exp(mu_ln + z_alpha * sigma)
      cl_lower[k] <- exp(mu_ln - z_alpha * sigma)
    }
  }

  # Interval hazard
  lag_cumhaz <- c(0, cumhaz[-out_n])
  lag_time <- c(0, u_times[-out_n])
  delta_t <- u_times - lag_time
  hazard <- rep(NA_real_, out_n)
  idx <- delta_t > 0
  hazard[idx] <- (cumhaz[idx] - lag_cumhaz[idx]) / delta_t[idx]

  result <- data.frame(
    time       = u_times,
    n_risk     = n_risk,
    n_event    = n_event_out,
    weight_sum = weight_sum,
    cumhaz     = cumhaz,
    std_err    = std_err,
    cl_lower   = cl_lower,
    cl_upper   = cl_upper,
    hazard     = hazard,
    cum_events = cumsum(n_event_out)
  )

  class(result) <- c("hzr_nelson", "data.frame")
  result
}

#' @rdname hzr_nelson
#' @param x An `hzr_nelson` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @export
print.hzr_nelson <- function(x, digits = 4, ...) {
  cat("Nelson cumulative hazard estimate with lognormal CL\n")
  cat("Events:", sum(x$n_event), " | Time points:", nrow(x), "\n\n")
  display <- x
  for (col in c("cumhaz", "std_err", "cl_lower", "cl_upper", "hazard")) {
    display[[col]] <- round(display[[col]], digits)
  }
  class(display) <- "data.frame"
  print(utils::head(display, 20), row.names = FALSE)
  if (nrow(display) > 20) cat("... (", nrow(display) - 20, " more rows)\n")
  invisible(x)
}


# =========================================================================
# hzr_bootstrap -- Bootstrap inference for hazard models
# =========================================================================

#' Bootstrap resampling for hazard model coefficients
#'
#' Resample data with replacement, refit the hazard model on each
#' replicate, and accumulate coefficient distributions. Returns a tidy
#' data frame of per-replicate estimates with summary statistics.
#' This is the R equivalent of the SAS `bootstrap.hazard.sas` macro.
#'
#' @param object A fitted `hazard` object (with `fit = TRUE`).
#' @param n_boot Integer: number of bootstrap replicates (default 200).
#' @param fraction Numeric in (0, 1]: fraction of data to sample per
#'   replicate (default 1.0 for full bootstrap; < 1 for bagging).
#' @param seed Optional integer random seed for reproducibility. When
#'   supplied, `set.seed(seed)` is called at function entry, jumping the
#'   global RNG to the seeded state; it is not restored on exit. Pass
#'   `NULL` (the default) to skip the `set.seed()` call and start from
#'   the caller's current RNG state. Note that the bootstrap consumes
#'   random numbers either way, so the global RNG state will advance
#'   during the call -- `seed = NULL` avoids the *reset* at entry, not
#'   the advance during resampling.
#' @param verbose Logical; if `TRUE`, print progress every 50 replicates.
#'
#' @return A list with class `"hzr_bootstrap"` containing:
#' \describe{
#'   \item{replicates}{Data frame with columns `replicate`, `parameter`,
#'     and `estimate` -- one row per parameter per successful replicate.}
#'   \item{summary}{Data frame with columns `parameter`, `n`, `pct`,
#'     `mean`, `sd`, `min`, `max`, `ci_lower`, `ci_upper` -- one row per
#'     parameter.}
#'   \item{n_success}{Number of successfully converged replicates.}
#'   \item{n_failed}{Number of replicates that failed to converge.}
#' }
#'
#' @examples
#' \donttest{
#' data(avc)
#' avc <- na.omit(avc)
#' fit <- hazard(
#'   survival::Surv(int_dead, dead) ~ age + mal,
#'   data  = avc,
#'   dist  = "weibull",
#'   theta = c(mu = 0.01, nu = 0.5, 0, 0),
#'   fit   = TRUE
#' )
#' bs <- hzr_bootstrap(fit, n_boot = 50, seed = 123)
#' print(bs)
#' }
#'
#' @seealso [hazard()] for model fitting, [vcov.hazard()] for
#'   Hessian-based standard errors.
#' @export
hzr_bootstrap <- function(object, n_boot = 200L, fraction = 1.0,
                           seed = NULL, verbose = FALSE) {
  if (!inherits(object, "hazard")) {
    stop("'object' must be a fitted hazard object.", call. = FALSE)
  }
  if (is.null(object$fit$theta) ||
      (is.logical(object$fit$converged) && is.na(object$fit$converged))) {
    stop("'object' has no fitted parameters. Refit with fit = TRUE.",
         call. = FALSE)
  }

  n_boot <- as.integer(n_boot)
  if (n_boot < 1L) stop("'n_boot' must be at least 1.", call. = FALSE)
  if (fraction <= 0 || fraction > 1) {
    stop("'fraction' must be in (0, 1].", call. = FALSE)
  }

  if (!is.null(seed)) set.seed(seed)

  # Reconstruct the call components
  cl <- object$call
  orig_data <- eval(cl$data, envir = parent.frame())
  n_obs <- nrow(orig_data)
  sample_size <- max(1L, as.integer(n_obs * fraction))

  # Parameter names from the fitted model. Shape parameters (e.g. mu, nu) are
  # named in theta, but covariate betas often come through with empty names.
  # Covariate coefficients occupy the last ncol(x) positions of theta; fill
  # any blanks within that block from the design matrix column names by
  # relative index, so downstream pivots (e.g. reshape(wide)) get a distinct
  # column per covariate -- even when some betas are already named and
  # others are not.
  param_names <- names(object$fit$theta)
  if (is.null(param_names)) {
    param_names <- character(length(object$fit$theta))
  }
  if (!is.null(object$data$x)) {
    x_names <- colnames(object$data$x)
    p <- ncol(object$data$x)
    n_theta <- length(param_names)
    if (!is.null(x_names) && p > 0L && n_theta >= p) {
      cov_idx <- seq.int(n_theta - p + 1L, n_theta)
      blank_in_block <- !nzchar(param_names[cov_idx])
      param_names[cov_idx[blank_in_block]] <- x_names[blank_in_block]
    }
  }
  still_blank <- !nzchar(param_names)
  if (any(still_blank)) {
    param_names[still_blank] <- paste0("param_", which(still_blank))
  }

  # Accumulate results
  rep_list <- vector("list", n_boot)
  n_success <- 0L
  n_failed <- 0L

  for (b in seq_len(n_boot)) {
    if (verbose && b %% 50 == 0) {
      cat("Bootstrap replicate", b, "/", n_boot, "\n")
    }

    # Resample with replacement
    idx <- sample.int(n_obs, size = sample_size, replace = TRUE)
    boot_data <- orig_data[idx, , drop = FALSE] # nolint: object_usage_linter.

    # Refit using the same call but with resampled data
    # (boot_data is referenced via quote() inside eval -- lintr cannot trace this)
    boot_fit <- tryCatch({
      cl_boot <- cl
      cl_boot$data <- quote(boot_data)
      cl_boot$fit <- TRUE
      eval(cl_boot)
    }, error = function(e) NULL)

    if (!is.null(boot_fit) && is.finite(boot_fit$fit$objective)) {
      n_success <- n_success + 1L
      theta_b <- boot_fit$fit$theta
      rep_list[[b]] <- data.frame(
        replicate = b,
        parameter = param_names[seq_along(theta_b)],
        estimate  = as.numeric(theta_b),
        stringsAsFactors = FALSE
      )
    } else {
      n_failed <- n_failed + 1L
    }
  }

  # Combine replicates
  replicates <- do.call(rbind, Filter(Negate(is.null), rep_list))
  if (is.null(replicates)) {
    replicates <- data.frame(replicate = integer(0),
                              parameter = character(0),
                              estimate = numeric(0),
                              stringsAsFactors = FALSE)
  }
  rownames(replicates) <- NULL

  # Summary statistics per parameter
  if (nrow(replicates) > 0) {
    summary_list <- lapply(split(replicates, replicates$parameter), function(d) {
      data.frame(
        parameter = d$parameter[1],
        n         = nrow(d),
        pct       = 100 * nrow(d) / n_boot,
        mean      = mean(d$estimate),
        sd        = stats::sd(d$estimate),
        min       = min(d$estimate),
        max       = max(d$estimate),
        ci_lower  = stats::quantile(d$estimate, 0.025),
        ci_upper  = stats::quantile(d$estimate, 0.975),
        stringsAsFactors = FALSE
      )
    })
    summary_df <- do.call(rbind, summary_list)
    rownames(summary_df) <- NULL
    # Sort by parameter order in the original model
    idx_order <- match(summary_df$parameter, param_names)
    summary_df <- summary_df[order(idx_order), ]
  } else {
    summary_df <- data.frame(parameter = character(0), n = integer(0),
                              pct = numeric(0), mean = numeric(0),
                              sd = numeric(0), min = numeric(0),
                              max = numeric(0), ci_lower = numeric(0),
                              ci_upper = numeric(0),
                              stringsAsFactors = FALSE)
  }

  result <- list(
    replicates = replicates,
    summary    = summary_df,
    n_success  = n_success,
    n_failed   = n_failed
  )
  class(result) <- "hzr_bootstrap"
  result
}

#' @rdname hzr_bootstrap
#' @param x An `hzr_bootstrap` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @export
print.hzr_bootstrap <- function(x, digits = 4, ...) {
  cat("Bootstrap inference for hazard model\n")
  cat("Replicates:", x$n_success, "successful,", x$n_failed, "failed\n\n")
  if (nrow(x$summary) > 0) {
    display <- x$summary
    for (col in c("pct", "mean", "sd", "min", "max",
                   "ci_lower", "ci_upper")) {
      display[[col]] <- round(display[[col]], digits)
    }
    print(display, row.names = FALSE)
  }
  invisible(x)
}


# =========================================================================
# hzr_competing_risks -- Cumulative incidence with Greenwood variance
# =========================================================================

#' Competing risks cumulative incidence
#'
#' Compute cumulative incidence functions for multiple competing event
#' types using the Aalen-Johansen estimator with Greenwood variance.
#' This is the R equivalent of the SAS `markov.sas` macro.
#'
#' Unlike the naive 1 - KM estimator (which overestimates incidence when
#' competing risks exist), this provides the correct marginal cumulative
#' incidence for each event type.
#'
#' @param time Numeric vector of follow-up times.
#' @param event Integer vector of event type indicators:
#'   0 = censored, 1 = event type 1, 2 = event type 2, etc.
#'
#' @return A data frame with one row per unique event time and columns:
#' \describe{
#'   \item{time}{Event time.}
#'   \item{n_risk}{Number at risk.}
#'   \item{n_event_1, n_event_2, ...}{Events of each type at this time.}
#'   \item{n_censor}{Number censored at this time.}
#'   \item{surv}{Overall event-free survival (freedom from all events).}
#'   \item{incid_1, incid_2, ...}{Cumulative incidence for each event type.}
#'   \item{se_surv}{Standard error of overall survival.}
#'   \item{se_1, se_2, ...}{Standard error of each cumulative incidence.}
#' }
#'
#' @examples
#' data(valves)
#' valves_cc <- na.omit(valves)
#' # Combine death and PVE into a competing risks event variable
#' # 0 = censored, 1 = death, 2 = PVE
#' event_cr <- ifelse(valves_cc$dead == 1, 1L,
#'                    ifelse(valves_cc$pve == 1, 2L, 0L))
#' time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
#' cr <- hzr_competing_risks(time_cr, event_cr)
#' head(cr)
#'
#' @seealso [hzr_kaplan()] for single-event survival estimation.
#' @export
hzr_competing_risks <- function(time, event) {
  if (!is.numeric(time) || !is.numeric(event)) {
    stop("'time' and 'event' must be numeric vectors.", call. = FALSE)
  }
  if (length(time) != length(event)) {
    stop("'time' and 'event' must have the same length.", call. = FALSE)
  }

  event_types <- sort(setdiff(unique(event), 0))
  n_types <- length(event_types)
  if (n_types == 0) {
    stop("No events found (all observations are censored).", call. = FALSE)
  }

  # Sort by time
  ord <- order(time)
  time_s <- time[ord]
  event_s <- event[ord]
  n_total <- length(time_s)

  # Unique event times (where at least one event occurred)
  u_times <- sort(unique(time_s[event_s > 0]))
  out_n <- length(u_times)

  # Initialize output
  n_risk <- numeric(out_n)
  n_censor_out <- numeric(out_n)
  n_event_mat <- matrix(0, nrow = out_n, ncol = n_types)
  colnames(n_event_mat) <- paste0("n_event_", event_types)

  # Cumulative incidence vectors
  surv <- numeric(out_n)
  incid <- matrix(0, nrow = out_n, ncol = n_types)
  colnames(incid) <- paste0("incid_", event_types)

  # Variance (diagonal of Greenwood matrix -- simplified)
  var_surv <- numeric(out_n)
  var_incid <- matrix(0, nrow = out_n, ncol = n_types)

  prev_surv <- 1.0
  prev_incid <- rep(0, n_types)
  prev_var_surv <- 0
  prev_var_incid <- rep(0, n_types)
  at_risk <- n_total

  for (k in seq_along(u_times)) {
    t_k <- u_times[k]

    # Censored before this time (between previous event time and t_k)
    prev_t <- if (k == 1) 0 else u_times[k - 1]
    censored_between <- sum(event_s == 0 & time_s > prev_t & time_s < t_k)
    at_risk <- at_risk - censored_between

    n_risk[k] <- at_risk

    # Events and censored AT this time
    at_time <- time_s == t_k
    for (j in seq_along(event_types)) {
      n_event_mat[k, j] <- sum(event_s == event_types[j] & at_time)
    }
    n_censor_out[k] <- sum(event_s == 0 & at_time)
    total_events_k <- sum(n_event_mat[k, ])

    # Aalen-Johansen update
    if (at_risk > 0) {
      # Transition probabilities
      d_total <- total_events_k / at_risk
      surv[k] <- prev_surv * (1 - d_total)

      for (j in seq_along(event_types)) {
        d_j <- n_event_mat[k, j] / at_risk
        incid[k, j] <- prev_incid[j] + prev_surv * d_j
      }

      # Greenwood variance update (simplified diagonal)
      if (at_risk > total_events_k && at_risk > 0) {
        greenwood_term <- total_events_k / (at_risk * (at_risk - total_events_k))
        var_surv[k] <- surv[k]^2 *
          (prev_var_surv / max(prev_surv^2, .Machine$double.xmin) +
             greenwood_term)

        for (j in seq_along(event_types)) {
          d_j <- n_event_mat[k, j] / at_risk
          var_incid[k, j] <- prev_var_incid[j] +
            prev_surv^2 * d_j * (1 - d_j) / at_risk
        }
      } else {
        var_surv[k] <- prev_var_surv
        var_incid[k, ] <- prev_var_incid
      }
    } else {
      surv[k] <- prev_surv
      incid[k, ] <- prev_incid
      var_surv[k] <- prev_var_surv
      var_incid[k, ] <- prev_var_incid
    }

    # Decrease at-risk by events + censored AT this time
    at_risk <- at_risk - total_events_k - n_censor_out[k]

    prev_surv <- surv[k]
    prev_incid <- incid[k, ]
    prev_var_surv <- var_surv[k]
    prev_var_incid <- var_incid[k, ]
  }

  # Assemble result
  result <- data.frame(
    time     = u_times,
    n_risk   = n_risk,
    n_event_mat,
    n_censor = n_censor_out,
    surv     = surv,
    incid,
    se_surv  = sqrt(pmax(var_surv, 0)),
    stringsAsFactors = FALSE
  )

  # Add SE columns for each event type
  for (j in seq_along(event_types)) {
    result[[paste0("se_", event_types[j])]] <-
      sqrt(pmax(var_incid[, j], 0))
  }

  class(result) <- c("hzr_competing_risks", "data.frame")
  result
}

#' @rdname hzr_competing_risks
#' @param x An `hzr_competing_risks` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @export
print.hzr_competing_risks <- function(x, digits = 4, ...) {
  incid_cols <- grep("^incid_", names(x), value = TRUE)
  cat("Competing risks cumulative incidence\n")
  cat("Event types:", length(incid_cols), " | Time points:", nrow(x), "\n")
  if (nrow(x) > 0) {
    cat("Final survival:", round(x$surv[nrow(x)], digits), "\n")
    for (col in incid_cols) {
      cat("Final", col, ":", round(x[[col]][nrow(x)], digits), "\n")
    }
  }
  cat("\n")
  display <- x
  for (col in c("surv", incid_cols, "se_surv",
                 grep("^se_\\d", names(x), value = TRUE))) {
    if (col %in% names(display)) {
      display[[col]] <- round(display[[col]], digits)
    }
  }
  class(display) <- "data.frame"
  print(utils::head(display, 15), row.names = FALSE)
  if (nrow(display) > 15) cat("... (", nrow(display) - 15, " more rows)\n")
  invisible(x)
}

# tests/testthat/test-diagnostics.R
# Tests for hzr_deciles() and related diagnostic utilities

# Helper cache: fit once to reduce repeated optimizer runs across tests.
.avc_fixture <- local({
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ age + mal,
    data  = avc,
    dist  = "weibull",
    theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
    fit   = TRUE
  )
  stopifnot(isTRUE(fit$fit$converged))
  list(avc = avc, fit = fit)
})

.fit_avc_weibull <- function() .avc_fixture$fit
.avc_data <- function() .avc_fixture$avc

test_that("hzr_deciles returns correct structure", {
  fit <- .fit_avc_weibull()
  cal <- hzr_deciles(fit, time = 120)

  expect_s3_class(cal, "hzr_deciles")
  expect_s3_class(cal, "data.frame")
  expect_equal(nrow(cal), 10)
  expect_named(cal, c("group", "n", "events", "expected", "observed_rate",
                       "expected_rate", "chi_sq", "p_value",
                       "mean_survival", "mean_cumhaz"))

  # Overall attribute
  ov <- attr(cal, "overall")
  expect_true(is.list(ov))
  expect_named(ov, c("chi_sq", "df", "p_value", "time", "groups",
                      "total_events", "total_expected",
                      "n_included", "n_excluded"))
  expect_equal(ov$time, 120)
  expect_equal(ov$groups, 10)
})

test_that("hzr_deciles group counts sum to evaluable n", {
  fit <- .fit_avc_weibull()
  avc <- .avc_data()
  cal <- hzr_deciles(fit, time = 120)
  included <- with(avc, dead == 1 | int_dead >= 120)
  events_120 <- with(avc, dead == 1 & int_dead <= 120 & included)

  expect_equal(sum(cal$n), sum(included))
  expect_equal(sum(cal$events), sum(events_120))
  expect_equal(attr(cal, "overall")$n_excluded, sum(!included))
})

test_that("hzr_deciles works with non-default groups", {
  fit <- .fit_avc_weibull()
  avc <- .avc_data()
  cal5 <- hzr_deciles(fit, time = 60, groups = 5)
  expect_equal(nrow(cal5), 5)
  expect_equal(sum(cal5$n), sum(avc$dead == 1 | avc$int_dead >= 60))
})

test_that("hzr_deciles events and expected totals increase with horizon", {
  fit <- .fit_avc_weibull()
  avc <- .avc_data()
  status_all <- avc$dead
  time_all <- ifelse(avc$dead == 1, avc$int_dead, 1e9)

  cal60 <- hzr_deciles(fit, time = 60, status = status_all,
                       event_time = time_all)
  cal120 <- hzr_deciles(fit, time = 120, status = status_all,
                        event_time = time_all)

  events_60 <- with(avc, sum(dead == 1 & int_dead <= 60))
  events_120 <- with(avc, sum(dead == 1 & int_dead <= 120))

  expect_equal(sum(cal60$events), events_60)
  expect_equal(sum(cal120$events), events_120)
  expect_equal(sum(cal60$n), nrow(avc))
  expect_equal(sum(cal120$n), nrow(avc))
  expect_true(sum(cal120$events) >= sum(cal60$events))
  expect_true(sum(cal120$expected) >= sum(cal60$expected))
})

test_that("hzr_deciles risk ordering is monotone", {
  fit <- .fit_avc_weibull()
  cal <- hzr_deciles(fit, time = 120)

  # Mean cumhaz should increase across groups (group 1 = lowest risk)
  expect_true(all(diff(cal$mean_cumhaz) >= -1e-10),
              label = "mean cumulative hazard should increase across groups")
  # Mean survival should decrease across groups
  expect_true(all(diff(cal$mean_survival) <= 1e-10),
              label = "mean survival should decrease across groups")
})

test_that("hzr_deciles chi-square values are non-negative", {
  fit <- .fit_avc_weibull()
  cal <- hzr_deciles(fit, time = 120)
  valid <- !is.na(cal$chi_sq)
  expect_true(all(cal$chi_sq[valid] >= 0))
  expect_true(all(cal$p_value[valid] >= 0 & cal$p_value[valid] <= 1))

  ov <- attr(cal, "overall")
  expect_true(ov$chi_sq >= 0)
  expect_true(ov$df > 0)
})

test_that("hzr_deciles works with intercept-only model", {
  # Intercept-only: all predictions identical, but grouping should still work
  data(cabgkul, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data  = cabgkul,
    dist  = "weibull",
    theta = c(mu = 0.10, nu = 1.0),
    fit   = TRUE
  )
  cal <- hzr_deciles(fit, time = 12)
  included <- with(cabgkul, dead == 1 | int_dead >= 12)

  expect_s3_class(cal, "hzr_deciles")
  expect_equal(nrow(cal), 10)
  expect_equal(sum(cal$n), sum(included))
  # All expected rates should be equal (same prediction for everyone)
  expect_true(max(cal$expected_rate) - min(cal$expected_rate) < 1e-10)
})

test_that("hzr_deciles works with multiphase model", {
  data(cabgkul, package = "TemporalHazard")
  fit_mp <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  cal <- hzr_deciles(fit_mp, time = 60)
  included <- with(cabgkul, dead == 1 | int_dead >= 60)

  expect_s3_class(cal, "hzr_deciles")
  expect_equal(nrow(cal), 10)
  expect_equal(sum(cal$n), sum(included))
})

test_that("hzr_deciles rejects invalid inputs", {
  fit <- .fit_avc_weibull()
  avc <- .avc_data()

  expect_error(hzr_deciles(fit, time = -1), "positive")
  expect_error(hzr_deciles(fit, time = 120, groups = 1), "at least 2")
  expect_error(hzr_deciles(fit, time = 120, groups = nrow(avc) + 1),
               "included observations")
  expect_error(hzr_deciles("not_a_model", time = 12), "hazard object")
})

test_that("hzr_deciles unfitted model is rejected", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit_unfitted <- hazard(
    survival::Surv(int_dead, dead) ~ age + status,
    data  = avc,
    dist  = "weibull",
    theta = c(mu = 0.05, nu = 0.5, 0, 0),
    fit   = FALSE
  )

  expect_error(hzr_deciles(fit_unfitted, time = 120), "fit = TRUE")
})

test_that("print.hzr_deciles runs without error", {
  fit <- .fit_avc_weibull()
  cal <- hzr_deciles(fit, time = 120)
  expect_output(print(cal), "Decile-of-risk calibration")
  expect_output(print(cal), "Overall")
})


# =========================================================================
# hzr_gof tests
# =========================================================================

test_that("hzr_gof returns correct structure", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)

  expect_s3_class(gof, "hzr_gof")
  expect_s3_class(gof, "data.frame")
  expect_true(nrow(gof) > 0)
  expect_true(all(c("time", "n_risk", "n_event", "n_censor",
                     "km_surv", "km_cumhaz", "par_surv", "par_cumhaz",
                     "cum_observed", "cum_expected", "residual") %in%
                    names(gof)))

  # Summary attribute
  s <- attr(gof, "summary")
  expect_true(is.list(s))
  expect_named(s, c("total_observed", "total_expected", "final_residual",
                     "dist", "n"))
})

test_that("hzr_gof cumulative observed equals total events", {
  fit <- .fit_avc_weibull()
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  gof <- hzr_gof(fit)

  s <- attr(gof, "summary")
  expect_equal(s$total_observed, sum(avc$dead))
  expect_equal(s$n, nrow(avc))
})

test_that("hzr_gof survival values are in [0, 1]", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)

  expect_true(all(gof$km_surv >= 0 & gof$km_surv <= 1))
  expect_true(all(gof$par_surv >= 0 & gof$par_surv <= 1))
})

test_that("hzr_gof cumulative hazard is non-negative and increasing", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)

  expect_true(all(gof$km_cumhaz >= 0))
  expect_true(all(gof$par_cumhaz >= 0))
  # Parametric cumhaz should be non-decreasing
  expect_true(all(diff(gof$par_cumhaz) >= -1e-10))
})

test_that("hzr_gof conservation ratio is reasonable", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)

  s <- attr(gof, "summary")
  ratio <- s$total_expected / s$total_observed
  # For a reasonable fit, E/O should be in the ballpark (0.5 to 2.0)
  expect_true(ratio > 0.2 && ratio < 5.0,
              label = paste("conservation ratio", round(ratio, 3),
                            "should be between 0.2 and 5.0"))
})

test_that("hzr_gof works with custom time grid", {
  fit <- .fit_avc_weibull()
  t_grid <- seq(1, 200, by = 10)
  gof <- hzr_gof(fit, time_grid = t_grid)

  expect_equal(nrow(gof), length(t_grid))
  expect_equal(gof$time, t_grid)
})

test_that("hzr_gof works with intercept-only model", {
  data(cabgkul, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data  = cabgkul,
    dist  = "weibull",
    theta = c(mu = 0.10, nu = 1.0),
    fit   = TRUE
  )
  gof <- hzr_gof(fit)

  expect_s3_class(gof, "hzr_gof")
  expect_true(nrow(gof) > 0)
  s <- attr(gof, "summary")
  expect_equal(s$total_observed, sum(cabgkul$dead))
})

test_that("hzr_gof works with multiphase model", {
  data(cabgkul, package = "TemporalHazard")
  fit_mp <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  gof <- hzr_gof(fit_mp)

  expect_s3_class(gof, "hzr_gof")
  # Should have phase-specific columns
  expect_true(any(grepl("^par_cumhaz_", names(gof))))
})

test_that("hzr_gof rejects invalid inputs", {
  expect_error(hzr_gof("not_a_model"), "hazard object")

  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit_unfitted <- hazard(
    survival::Surv(int_dead, dead) ~ age + mal,
    data  = avc,
    dist  = "weibull",
    theta = c(mu = 0.01, nu = 0.5, 0, 0),
    fit   = FALSE
  )
  expect_error(hzr_gof(fit_unfitted), "fit = TRUE")
})

test_that("print.hzr_gof runs without error", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)
  expect_output(print(gof), "Goodness-of-fit")
  expect_output(print(gof), "Conservation ratio")
})


# =========================================================================
# hzr_kaplan tests
# =========================================================================

test_that("hzr_kaplan returns correct structure", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  expect_s3_class(km, "hzr_kaplan")
  expect_s3_class(km, "data.frame")
  expect_true(nrow(km) > 0)
  expect_named(km, c("time", "n_risk", "n_event", "n_censor",
                      "survival", "std_err", "cl_lower", "cl_upper",
                      "cumhaz", "hazard", "density", "life"))
})

test_that("hzr_kaplan survival is monotone non-increasing", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(diff(km$survival) <= 0),
              label = "survival should be non-increasing")
  expect_true(all(km$survival >= 0 & km$survival <= 1))
})

test_that("hzr_kaplan confidence limits respect [0, 1]", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(km$cl_lower >= 0 & km$cl_lower <= 1))
  expect_true(all(km$cl_upper >= 0 & km$cl_upper <= 1))
  # Lower <= survival <= upper
  expect_true(all(km$cl_lower <= km$survival + 1e-10))
  expect_true(all(km$cl_upper >= km$survival - 1e-10))
})

test_that("hzr_kaplan matches survfit survival values", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead, event_only = FALSE)
  sf <- survival::survfit(survival::Surv(int_dead, dead) ~ 1,
                           data = cabgkul)

  # Survival values at survfit times should match exactly
  expect_equal(km$survival, sf$surv, tolerance = 1e-12)
  expect_equal(km$time, sf$time)
})

test_that("hzr_kaplan cumhaz is consistent with survival", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  # cumhaz = -log(S)
  expected_cumhaz <- -log(pmax(km$survival, .Machine$double.xmin))
  expect_equal(km$cumhaz, expected_cumhaz, tolerance = 1e-10)
})

test_that("hzr_kaplan life integral is non-negative and non-decreasing", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(km$life >= 0))
  expect_true(all(diff(km$life) >= -1e-10),
              label = "life integral should be non-decreasing")
})

test_that("hzr_kaplan event_only filters correctly", {
  data(cabgkul, package = "TemporalHazard")
  km_events <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead,
                            event_only = TRUE)
  km_all <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead,
                         event_only = FALSE)

  expect_true(nrow(km_all) >= nrow(km_events))
  expect_true(all(km_events$n_event > 0))
})

test_that("hzr_kaplan conf_level parameter works", {
  data(cabgkul, package = "TemporalHazard")
  km_95 <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead,
                        conf_level = 0.95)
  km_68 <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead,
                        conf_level = 0.68268948)

  # 95% intervals should be wider than 68% (1-SD) intervals
  width_95 <- km_95$cl_upper - km_95$cl_lower
  width_68 <- km_68$cl_upper - km_68$cl_lower
  # Compare at matching times where both have events
  expect_true(all(width_95 >= width_68 - 1e-10))
})

test_that("hzr_kaplan rejects invalid inputs", {
  expect_error(hzr_kaplan("a", 1), "numeric")
  expect_error(hzr_kaplan(1:5, 1:3), "same length")
  expect_error(hzr_kaplan(1:5, rep(1, 5), conf_level = 1.5),
               "between 0 and 1")
})

test_that("print.hzr_kaplan runs without error", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)
  expect_output(print(km), "Kaplan-Meier")
  expect_output(print(km), "RMST")
})


# =========================================================================
# hzr_calibrate tests
# =========================================================================

test_that("hzr_calibrate returns correct structure", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)

  expect_s3_class(cal, "hzr_calibrate")
  expect_s3_class(cal, "data.frame")
  expect_equal(nrow(cal), 10)
  expect_named(cal, c("group", "n", "events", "mean", "min", "max",
                       "prob", "link_value"))
  expect_equal(attr(cal, "link"), "logit")
  expect_equal(attr(cal, "groups"), 10L)
})

test_that("hzr_calibrate group counts sum to n", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 5)

  expect_equal(sum(cal$n), nrow(avc))
  expect_equal(sum(cal$events), sum(avc$dead))
})

test_that("hzr_calibrate group means are monotone", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)

  # Group means should increase (group 1 = lowest x)
  expect_true(all(diff(cal$mean) > 0),
              label = "group means should be strictly increasing")
})

test_that("hzr_calibrate logit values are finite where prob in (0,1)", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)

  valid <- cal$prob > 0 & cal$prob < 1
  expect_true(all(is.finite(cal$link_value[valid])))
})

test_that("hzr_calibrate gompertz link works", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 5, link = "gompertz")

  expect_equal(attr(cal, "link"), "gompertz")
  # Gompertz = log(-log(1-p)), should be finite for 0 < p < 1
  valid <- cal$prob > 0 & cal$prob < 1
  expect_true(all(is.finite(cal$link_value[valid])))
})

test_that("hzr_calibrate cox link requires time", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)

  expect_error(
    hzr_calibrate(avc$age, avc$dead, link = "cox"),
    "time.*required"
  )

  cal <- hzr_calibrate(avc$age, avc$dead, link = "cox",
                        time = avc$int_dead, groups = 5)
  expect_equal(attr(cal, "link"), "cox")
  expect_equal(nrow(cal), 5)
})

test_that("hzr_calibrate stratified by grouping variable", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 5,
                        by = ifelse(avc$mal == 1, "mal", "no_mal"))

  expect_true("by" %in% names(cal))
  # Should have rows for both strata
  expect_true(all(c("mal", "no_mal") %in% cal$by))
  # Total n across all rows should equal nrow(avc)
  expect_equal(sum(cal$n), nrow(avc))
})

test_that("hzr_calibrate rejects invalid inputs", {
  expect_error(hzr_calibrate("a", 1), "numeric")
  expect_error(hzr_calibrate(1:5, 1:3), "same length")
  expect_error(hzr_calibrate(1:5, rep(1, 5), groups = 1), "at least 2")
})

test_that("print.hzr_calibrate runs without error", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 5)
  expect_output(print(cal), "Variable calibration")
  expect_output(print(cal), "logit")
})


# =========================================================================
# hzr_nelson tests
# =========================================================================

test_that("hzr_nelson returns correct structure", {
  data(cabgkul, package = "TemporalHazard")
  nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)

  expect_s3_class(nel, "hzr_nelson")
  expect_true(nrow(nel) > 0)
  expect_named(nel, c("time", "n_risk", "n_event", "weight_sum",
                       "cumhaz", "std_err", "cl_lower", "cl_upper",
                       "hazard", "cum_events"))
})

test_that("hzr_nelson cumhaz is non-decreasing", {
  data(cabgkul, package = "TemporalHazard")
  nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(diff(nel$cumhaz) >= -1e-10))
  expect_true(all(nel$cumhaz >= 0))
})

test_that("hzr_nelson CL are non-negative and ordered", {
  data(cabgkul, package = "TemporalHazard")
  nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(nel$cl_lower >= 0))
  expect_true(all(nel$cl_upper >= 0))
  expect_true(all(nel$cl_lower <= nel$cumhaz + 1e-10))
  expect_true(all(nel$cl_upper >= nel$cumhaz - 1e-10))
})

test_that("hzr_nelson weighted events change cumhaz", {
  data(cabgkul, package = "TemporalHazard")
  nel_unw <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)
  # Double weights should roughly double cumhaz
  nel_w2 <- hzr_nelson(cabgkul$int_dead, cabgkul$dead,
                         weight = rep(2, nrow(cabgkul)))
  expect_true(
    nel_w2$cumhaz[nrow(nel_w2)] > nel_unw$cumhaz[nrow(nel_unw)] * 1.5
  )
})

test_that("hzr_nelson rejects invalid inputs", {
  expect_error(hzr_nelson("a", 1), "numeric")
  expect_error(hzr_nelson(1:5, 1:3), "same length")
})

test_that("print.hzr_nelson runs without error", {
  data(cabgkul, package = "TemporalHazard")
  nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)
  expect_output(print(nel), "Nelson")
})


# =========================================================================
# hzr_bootstrap tests
# =========================================================================

test_that("hzr_bootstrap returns correct structure", {
  set.seed(42)
  fit <- .fit_avc_weibull()
  bs <- hzr_bootstrap(fit, n_boot = 10, seed = 123)

  expect_s3_class(bs, "hzr_bootstrap")
  expect_true(is.data.frame(bs$replicates))
  expect_true(is.data.frame(bs$summary))
  expect_true(bs$n_success + bs$n_failed == 10)
  expect_named(bs$summary, c("parameter", "n", "pct", "mean", "sd",
                              "min", "max", "ci_lower", "ci_upper"))
})

test_that("hzr_bootstrap seed gives reproducible results", {
  fit <- .fit_avc_weibull()
  bs1 <- hzr_bootstrap(fit, n_boot = 5, seed = 999)
  bs2 <- hzr_bootstrap(fit, n_boot = 5, seed = 999)

  expect_equal(bs1$replicates$estimate, bs2$replicates$estimate)
})

test_that("hzr_bootstrap fraction parameter works", {
  fit <- .fit_avc_weibull()
  bs <- hzr_bootstrap(fit, n_boot = 5, fraction = 0.5, seed = 42)

  expect_s3_class(bs, "hzr_bootstrap")
  expect_true(bs$n_success > 0)
})

test_that("hzr_bootstrap rejects invalid inputs", {
  expect_error(hzr_bootstrap("not_a_model"), "hazard object")
})

test_that("print.hzr_bootstrap runs without error", {
  fit <- .fit_avc_weibull()
  bs <- hzr_bootstrap(fit, n_boot = 5, seed = 42)
  expect_output(print(bs), "Bootstrap")
})

test_that("hzr_bootstrap labels covariate rows with design-matrix names", {
  # Regression test: covariate parameter rows used to be emitted with empty
  # strings, which broke wide pivots of $replicates. They should now carry
  # the design-matrix column names (e.g. "x1", "x2").
  set.seed(7)
  n <- 80
  df <- data.frame(
    t = stats::rexp(n, 0.1),
    d = stats::rbinom(n, 1, 0.6),
    x1 = stats::rnorm(n),
    x2 = stats::rnorm(n)
  )
  fit <- hazard(
    survival::Surv(t, d) ~ x1 + x2,
    data  = df,
    dist  = "weibull",
    theta = c(mu = 0.05, nu = 0.8, 0, 0),
    fit   = TRUE
  )

  bs <- hzr_bootstrap(fit, n_boot = 5, seed = 123)
  params <- bs$replicates$parameter

  expect_false(any(!nzchar(params)))
  expect_true(all(c("x1", "x2") %in% params))
  # Each successful replicate should contribute one row per covariate.
  expect_equal(sum(params == "x1"), bs$n_success)
  expect_equal(sum(params == "x2"), bs$n_success)
  # Summary table should also carry the covariate labels.
  expect_true(all(c("x1", "x2") %in% bs$summary$parameter))
})

test_that("hzr_bootstrap preserves named betas while filling blanks", {
  # Mixed-naming case: one beta is explicitly named in the starting theta
  # and the other is left blank. The named slot must be preserved; the
  # blank slot should pick up its design-matrix column name.
  set.seed(11)
  n <- 80
  df <- data.frame(
    t = stats::rexp(n, 0.1),
    d = stats::rbinom(n, 1, 0.6),
    x1 = stats::rnorm(n),
    x2 = stats::rnorm(n)
  )
  fit <- hazard(
    survival::Surv(t, d) ~ x1 + x2,
    data  = df,
    dist  = "weibull",
    theta = c(mu = 0.05, nu = 0.8, b_custom = 0, 0),
    fit   = TRUE
  )

  bs <- hzr_bootstrap(fit, n_boot = 5, seed = 321)
  params <- bs$replicates$parameter

  expect_false(any(!nzchar(params)))
  # Named beta retained; blank beta filled from colnames(x) -> "x2".
  expect_true("b_custom" %in% params)
  expect_true("x2" %in% params)
  expect_false("x1" %in% params)
})


# =========================================================================
# hzr_competing_risks tests
# =========================================================================

test_that("hzr_competing_risks returns correct structure", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)

  expect_s3_class(cr, "hzr_competing_risks")
  expect_true(nrow(cr) > 0)
  expect_true("surv" %in% names(cr))
  expect_true("incid_1" %in% names(cr))
  expect_true("incid_2" %in% names(cr))
  expect_true("se_surv" %in% names(cr))
})

test_that("hzr_competing_risks survival + incidences sum to ~1", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)

  # At each time: surv + sum(incid) should be approximately 1
  total <- cr$surv + cr$incid_1 + cr$incid_2
  expect_true(all(abs(total - 1) < 0.01),
              label = "surv + incidences should sum to ~1")
})

test_that("hzr_competing_risks survival is non-increasing", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)

  expect_true(all(diff(cr$surv) <= 1e-10))
  expect_true(all(cr$surv >= 0 & cr$surv <= 1))
})

test_that("hzr_competing_risks incidences are non-decreasing", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)

  expect_true(all(diff(cr$incid_1) >= -1e-10))
  expect_true(all(diff(cr$incid_2) >= -1e-10))
})

test_that("hzr_competing_risks rejects invalid inputs", {
  expect_error(hzr_competing_risks("a", 1), "numeric")
  expect_error(hzr_competing_risks(1:5, rep(0, 5)), "No events")
})

test_that("print.hzr_competing_risks runs without error", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)
  expect_output(print(cr), "Competing risks")
})

# tests/testthat/test-diagnostics.R
# Tests for hzr_deciles() and related diagnostic utilities

# Helper cache: fit once to reduce repeated optimizer runs across tests.
.avc_fixture <- local({
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ age + mal,
    data  = avc,
    dist  = "weibull",
    theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
    fit   = TRUE
  )
  stopifnot(isTRUE(fit$fit$converged))
  list(avc = avc, fit = fit)
})

.fit_avc_weibull <- function() .avc_fixture$fit
.avc_data <- function() .avc_fixture$avc

test_that("hzr_deciles returns correct structure", {
  fit <- .fit_avc_weibull()
  cal <- hzr_deciles(fit, time = 120)

  expect_s3_class(cal, "hzr_deciles")
  expect_s3_class(cal, "data.frame")
  expect_equal(nrow(cal), 10)
  expect_named(cal, c("group", "n", "events", "expected", "observed_rate",
                       "expected_rate", "chi_sq", "p_value",
                       "mean_survival", "mean_cumhaz"))

  # Overall attribute
  ov <- attr(cal, "overall")
  expect_true(is.list(ov))
  expect_named(ov, c("chi_sq", "df", "p_value", "time", "groups",
                      "total_events", "total_expected",
                      "n_included", "n_excluded"))
  expect_equal(ov$time, 120)
  expect_equal(ov$groups, 10)
})

test_that("hzr_deciles includes all subjects and counts all events (SAS method)", {
  fit <- .fit_avc_weibull()
  avc <- .avc_data()
  cal <- hzr_deciles(fit, time = 120)

  # SAS %DECILES uses every subject; the horizon only stratifies risk.
  expect_equal(sum(cal$n), nrow(avc))
  expect_equal(sum(cal$events), sum(avc$dead == 1))
  expect_equal(attr(cal, "overall")$n_excluded, 0L)
})

test_that("hzr_deciles works with non-default groups", {
  fit <- .fit_avc_weibull()
  avc <- .avc_data()
  cal5 <- hzr_deciles(fit, time = 60, groups = 5)
  expect_equal(nrow(cal5), 5)
  expect_equal(sum(cal5$n), nrow(avc))
})

test_that("hzr_deciles totals are horizon-invariant; only the grouping changes", {
  fit <- .fit_avc_weibull()
  avc <- .avc_data()

  cal60  <- hzr_deciles(fit, time = 60)
  cal120 <- hzr_deciles(fit, time = 120)

  # Expected = sum of cumulative hazard at each subject's own follow-up time,
  # and events = all observed deaths: both totals are independent of the
  # grouping horizon. Only the assignment of subjects to risk groups changes.
  expect_equal(sum(cal60$n), nrow(avc))
  expect_equal(sum(cal120$n), nrow(avc))
  expect_equal(sum(cal60$events), sum(avc$dead == 1))
  expect_equal(sum(cal120$events), sum(avc$dead == 1))
  expect_equal(sum(cal60$expected), sum(cal120$expected), tolerance = 1e-8)
})

test_that("hzr_deciles risk ordering is monotone in horizon survival", {
  fit <- .fit_avc_weibull()
  cal <- hzr_deciles(fit, time = 120)

  # Groups are ranked by predicted survival at the horizon, group 1 = lowest
  # risk (highest survival), so mean horizon survival decreases across groups.
  expect_true(all(diff(cal$mean_survival) <= 1e-10),
              label = "mean horizon survival should decrease across groups")
})

test_that("hzr_deciles chi-square values are non-negative", {
  fit <- .fit_avc_weibull()
  cal <- hzr_deciles(fit, time = 120)
  valid <- !is.na(cal$chi_sq)
  expect_true(all(cal$chi_sq[valid] >= 0))
  expect_true(all(cal$p_value[valid] >= 0 & cal$p_value[valid] <= 1))

  ov <- attr(cal, "overall")
  expect_true(ov$chi_sq >= 0)
  expect_true(ov$df > 0)
})

test_that("hzr_deciles works with intercept-only model", {
  # Intercept-only: all predictions identical, but grouping should still work
  data(cabgkul, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data  = cabgkul,
    dist  = "weibull",
    theta = c(mu = 0.10, nu = 1.0),
    fit   = TRUE
  )
  cal <- hzr_deciles(fit, time = 12)

  expect_s3_class(cal, "hzr_deciles")
  expect_equal(nrow(cal), 10)
  expect_equal(sum(cal$n), nrow(cabgkul))
  # Everyone shares the same horizon survival, so risk grouping is arbitrary
  # but mean horizon survival is identical across groups.
  expect_true(max(cal$mean_survival) - min(cal$mean_survival) < 1e-8)
})

test_that("hzr_deciles works with multiphase model", {
  data(cabgkul, package = "TemporalHazard")
  fit_mp <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  cal <- hzr_deciles(fit_mp, time = 60)

  expect_s3_class(cal, "hzr_deciles")
  expect_equal(nrow(cal), 10)
  expect_equal(sum(cal$n), nrow(cabgkul))
})

test_that("hzr_deciles rejects invalid inputs", {
  fit <- .fit_avc_weibull()
  avc <- .avc_data()

  expect_error(hzr_deciles(fit, time = -1), "positive")
  expect_error(hzr_deciles(fit, time = 120, groups = 1), "at least 2")
  expect_error(hzr_deciles(fit, time = 120, groups = nrow(avc) + 1),
               "number of observations")
  expect_error(hzr_deciles("not_a_model", time = 12), "hazard object")
})

test_that("hzr_deciles unfitted model is rejected", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit_unfitted <- hazard(
    survival::Surv(int_dead, dead) ~ age + status,
    data  = avc,
    dist  = "weibull",
    theta = c(mu = 0.05, nu = 0.5, 0, 0),
    fit   = FALSE
  )

  expect_error(hzr_deciles(fit_unfitted, time = 120), "fit = TRUE")
})

test_that("print.hzr_deciles runs without error", {
  fit <- .fit_avc_weibull()
  cal <- hzr_deciles(fit, time = 120)
  expect_output(print(cal), "Decile-of-risk calibration")
  expect_output(print(cal), "Overall")
})


# =========================================================================
# hzr_gof tests
# =========================================================================

test_that("hzr_gof returns correct structure", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)

  expect_s3_class(gof, "hzr_gof")
  expect_s3_class(gof, "data.frame")
  expect_true(nrow(gof) > 0)
  expect_true(all(c("time", "n_risk", "n_event", "n_censor",
                     "km_surv", "km_cumhaz", "par_surv", "par_cumhaz",
                     "cum_observed", "cum_expected", "residual") %in%
                    names(gof)))

  # Summary attribute
  s <- attr(gof, "summary")
  expect_true(is.list(s))
  expect_named(s, c("total_observed", "total_expected", "final_residual",
                     "dist", "n"))
})

test_that("hzr_gof cumulative observed equals total events", {
  fit <- .fit_avc_weibull()
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  gof <- hzr_gof(fit)

  s <- attr(gof, "summary")
  expect_equal(s$total_observed, sum(avc$dead))
  expect_equal(s$n, nrow(avc))
})

test_that("hzr_gof survival values are in [0, 1]", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)

  expect_true(all(gof$km_surv >= 0 & gof$km_surv <= 1))
  expect_true(all(gof$par_surv >= 0 & gof$par_surv <= 1))
})

test_that("hzr_gof cumulative hazard is non-negative and increasing", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)

  expect_true(all(gof$km_cumhaz >= 0))
  expect_true(all(gof$par_cumhaz >= 0))
  # Parametric cumhaz should be non-decreasing
  expect_true(all(diff(gof$par_cumhaz) >= -1e-10))
})

test_that("hzr_gof conservation ratio is reasonable", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)

  s <- attr(gof, "summary")
  ratio <- s$total_expected / s$total_observed
  # For a reasonable fit, E/O should be in the ballpark (0.5 to 2.0)
  expect_true(ratio > 0.2 && ratio < 5.0,
              label = paste("conservation ratio", round(ratio, 3),
                            "should be between 0.2 and 5.0"))
})

test_that("hzr_gof works with custom time grid", {
  fit <- .fit_avc_weibull()
  t_grid <- seq(1, 200, by = 10)
  gof <- hzr_gof(fit, time_grid = t_grid)

  expect_equal(nrow(gof), length(t_grid))
  expect_equal(gof$time, t_grid)
})

test_that("hzr_gof works with intercept-only model", {
  data(cabgkul, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data  = cabgkul,
    dist  = "weibull",
    theta = c(mu = 0.10, nu = 1.0),
    fit   = TRUE
  )
  gof <- hzr_gof(fit)

  expect_s3_class(gof, "hzr_gof")
  expect_true(nrow(gof) > 0)
  s <- attr(gof, "summary")
  expect_equal(s$total_observed, sum(cabgkul$dead))
})

test_that("hzr_gof works with multiphase model", {
  data(cabgkul, package = "TemporalHazard")
  fit_mp <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  gof <- hzr_gof(fit_mp)

  expect_s3_class(gof, "hzr_gof")
  # Should have phase-specific columns
  expect_true(any(grepl("^par_cumhaz_", names(gof))))
})

test_that("hzr_gof rejects invalid inputs", {
  expect_error(hzr_gof("not_a_model"), "hazard object")

  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit_unfitted <- hazard(
    survival::Surv(int_dead, dead) ~ age + mal,
    data  = avc,
    dist  = "weibull",
    theta = c(mu = 0.01, nu = 0.5, 0, 0),
    fit   = FALSE
  )
  expect_error(hzr_gof(fit_unfitted), "fit = TRUE")
})

test_that("print.hzr_gof runs without error", {
  fit <- .fit_avc_weibull()
  gof <- hzr_gof(fit)
  expect_output(print(gof), "Goodness-of-fit")
  expect_output(print(gof), "Conservation ratio")
})


# =========================================================================
# hzr_kaplan tests
# =========================================================================

test_that("hzr_kaplan returns correct structure", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  expect_s3_class(km, "hzr_kaplan")
  expect_s3_class(km, "data.frame")
  expect_true(nrow(km) > 0)
  expect_named(km, c("time", "n_risk", "n_event", "n_censor",
                      "survival", "std_err", "cl_lower", "cl_upper",
                      "cumhaz", "hazard", "density", "life"))
})

test_that("hzr_kaplan survival is monotone non-increasing", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(diff(km$survival) <= 0),
              label = "survival should be non-increasing")
  expect_true(all(km$survival >= 0 & km$survival <= 1))
})

test_that("hzr_kaplan confidence limits respect [0, 1]", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(km$cl_lower >= 0 & km$cl_lower <= 1))
  expect_true(all(km$cl_upper >= 0 & km$cl_upper <= 1))
  # Lower <= survival <= upper
  expect_true(all(km$cl_lower <= km$survival + 1e-10))
  expect_true(all(km$cl_upper >= km$survival - 1e-10))
})

test_that("hzr_kaplan matches survfit survival values", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead, event_only = FALSE)
  sf <- survival::survfit(survival::Surv(int_dead, dead) ~ 1,
                           data = cabgkul)

  # Survival values at survfit times should match exactly
  expect_equal(km$survival, sf$surv, tolerance = 1e-12)
  expect_equal(km$time, sf$time)
})

test_that("hzr_kaplan cumhaz is consistent with survival", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  # cumhaz = -log(S)
  expected_cumhaz <- -log(pmax(km$survival, .Machine$double.xmin))
  expect_equal(km$cumhaz, expected_cumhaz, tolerance = 1e-10)
})

test_that("hzr_kaplan life integral is non-negative and non-decreasing", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(km$life >= 0))
  expect_true(all(diff(km$life) >= -1e-10),
              label = "life integral should be non-decreasing")
})

test_that("hzr_kaplan event_only filters correctly", {
  data(cabgkul, package = "TemporalHazard")
  km_events <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead,
                            event_only = TRUE)
  km_all <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead,
                         event_only = FALSE)

  expect_true(nrow(km_all) >= nrow(km_events))
  expect_true(all(km_events$n_event > 0))
})

test_that("hzr_kaplan conf_level parameter works", {
  data(cabgkul, package = "TemporalHazard")
  km_95 <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead,
                        conf_level = 0.95)
  km_68 <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead,
                        conf_level = 0.68268948)

  # 95% intervals should be wider than 68% (1-SD) intervals
  width_95 <- km_95$cl_upper - km_95$cl_lower
  width_68 <- km_68$cl_upper - km_68$cl_lower
  # Compare at matching times where both have events
  expect_true(all(width_95 >= width_68 - 1e-10))
})

test_that("hzr_kaplan rejects invalid inputs", {
  expect_error(hzr_kaplan("a", 1), "numeric")
  expect_error(hzr_kaplan(1:5, 1:3), "same length")
  expect_error(hzr_kaplan(1:5, rep(1, 5), conf_level = 1.5),
               "between 0 and 1")
})

test_that("print.hzr_kaplan runs without error", {
  data(cabgkul, package = "TemporalHazard")
  km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)
  expect_output(print(km), "Kaplan-Meier")
  expect_output(print(km), "RMST")
})


# =========================================================================
# hzr_calibrate tests
# =========================================================================

test_that("hzr_calibrate returns correct structure", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)

  expect_s3_class(cal, "hzr_calibrate")
  expect_s3_class(cal, "data.frame")
  expect_equal(nrow(cal), 10)
  expect_named(cal, c("group", "n", "events", "mean", "min", "max",
                       "prob", "link_value"))
  expect_equal(attr(cal, "link"), "logit")
  expect_equal(attr(cal, "groups"), 10L)
})

test_that("hzr_calibrate group counts sum to n", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 5)

  expect_equal(sum(cal$n), nrow(avc))
  expect_equal(sum(cal$events), sum(avc$dead))
})

test_that("hzr_calibrate group means are monotone", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)

  # Group means should increase (group 1 = lowest x)
  expect_true(all(diff(cal$mean) > 0),
              label = "group means should be strictly increasing")
})

test_that("hzr_calibrate logit values are finite where prob in (0,1)", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)

  valid <- cal$prob > 0 & cal$prob < 1
  expect_true(all(is.finite(cal$link_value[valid])))
})

test_that("hzr_calibrate gompertz link works", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 5, link = "gompertz")

  expect_equal(attr(cal, "link"), "gompertz")
  # Gompertz = log(-log(1-p)), should be finite for 0 < p < 1
  valid <- cal$prob > 0 & cal$prob < 1
  expect_true(all(is.finite(cal$link_value[valid])))
})

test_that("hzr_calibrate cox link requires time", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)

  expect_error(
    hzr_calibrate(avc$age, avc$dead, link = "cox"),
    "time.*required"
  )

  cal <- hzr_calibrate(avc$age, avc$dead, link = "cox",
                        time = avc$int_dead, groups = 5)
  expect_equal(attr(cal, "link"), "cox")
  expect_equal(nrow(cal), 5)
})

test_that("hzr_calibrate stratified by grouping variable", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 5,
                        by = ifelse(avc$mal == 1, "mal", "no_mal"))

  expect_true("by" %in% names(cal))
  # Should have rows for both strata
  expect_true(all(c("mal", "no_mal") %in% cal$by))
  # Total n across all rows should equal nrow(avc)
  expect_equal(sum(cal$n), nrow(avc))
})

test_that("hzr_calibrate rejects invalid inputs", {
  expect_error(hzr_calibrate("a", 1), "numeric")
  expect_error(hzr_calibrate(1:5, 1:3), "same length")
  expect_error(hzr_calibrate(1:5, rep(1, 5), groups = 1), "at least 2")
})

test_that("print.hzr_calibrate runs without error", {
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  cal <- hzr_calibrate(avc$age, avc$dead, groups = 5)
  expect_output(print(cal), "Variable calibration")
  expect_output(print(cal), "logit")
})


# =========================================================================
# hzr_nelson tests
# =========================================================================

test_that("hzr_nelson returns correct structure", {
  data(cabgkul, package = "TemporalHazard")
  nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)

  expect_s3_class(nel, "hzr_nelson")
  expect_true(nrow(nel) > 0)
  expect_named(nel, c("time", "n_risk", "n_event", "weight_sum",
                       "cumhaz", "std_err", "cl_lower", "cl_upper",
                       "hazard", "cum_events"))
})

test_that("hzr_nelson cumhaz is non-decreasing", {
  data(cabgkul, package = "TemporalHazard")
  nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(diff(nel$cumhaz) >= -1e-10))
  expect_true(all(nel$cumhaz >= 0))
})

test_that("hzr_nelson CL are non-negative and ordered", {
  data(cabgkul, package = "TemporalHazard")
  nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)

  expect_true(all(nel$cl_lower >= 0))
  expect_true(all(nel$cl_upper >= 0))
  expect_true(all(nel$cl_lower <= nel$cumhaz + 1e-10))
  expect_true(all(nel$cl_upper >= nel$cumhaz - 1e-10))
})

test_that("hzr_nelson weighted events change cumhaz", {
  data(cabgkul, package = "TemporalHazard")
  nel_unw <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)
  # Double weights should roughly double cumhaz
  nel_w2 <- hzr_nelson(cabgkul$int_dead, cabgkul$dead,
                         weight = rep(2, nrow(cabgkul)))
  expect_true(
    nel_w2$cumhaz[nrow(nel_w2)] > nel_unw$cumhaz[nrow(nel_unw)] * 1.5
  )
})

test_that("hzr_nelson rejects invalid inputs", {
  expect_error(hzr_nelson("a", 1), "numeric")
  expect_error(hzr_nelson(1:5, 1:3), "same length")
})

test_that("print.hzr_nelson runs without error", {
  data(cabgkul, package = "TemporalHazard")
  nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)
  expect_output(print(nel), "Nelson")
})


# =========================================================================
# hzr_bootstrap tests
# =========================================================================

test_that("hzr_bootstrap returns correct structure", {
  set.seed(42)
  fit <- .fit_avc_weibull()
  bs <- hzr_bootstrap(fit, n_boot = 10, seed = 123)

  expect_s3_class(bs, "hzr_bootstrap")
  expect_true(is.data.frame(bs$replicates))
  expect_true(is.data.frame(bs$summary))
  expect_true(bs$n_success + bs$n_failed == 10)
  expect_named(bs$summary, c("parameter", "n", "pct", "mean", "sd",
                              "min", "max", "ci_lower", "ci_upper"))
})

test_that("hzr_bootstrap seed gives reproducible results", {
  fit <- .fit_avc_weibull()
  bs1 <- hzr_bootstrap(fit, n_boot = 5, seed = 999)
  bs2 <- hzr_bootstrap(fit, n_boot = 5, seed = 999)

  expect_equal(bs1$replicates$estimate, bs2$replicates$estimate)
})

test_that("hzr_bootstrap fraction parameter works", {
  fit <- .fit_avc_weibull()
  bs <- hzr_bootstrap(fit, n_boot = 5, fraction = 0.5, seed = 42)

  expect_s3_class(bs, "hzr_bootstrap")
  expect_true(bs$n_success > 0)
})

test_that("hzr_bootstrap rejects invalid inputs", {
  expect_error(hzr_bootstrap("not_a_model"), "hazard object")
})

test_that("print.hzr_bootstrap runs without error", {
  fit <- .fit_avc_weibull()
  bs <- hzr_bootstrap(fit, n_boot = 5, seed = 42)
  expect_output(print(bs), "Bootstrap")
})

test_that("hzr_bootstrap labels covariate rows with design-matrix names", {
  # Regression test: covariate parameter rows used to be emitted with empty
  # strings, which broke wide pivots of $replicates. They should now carry
  # the design-matrix column names (e.g. "x1", "x2").
  set.seed(7)
  n <- 80
  df <- data.frame(
    t = stats::rexp(n, 0.1),
    d = stats::rbinom(n, 1, 0.6),
    x1 = stats::rnorm(n),
    x2 = stats::rnorm(n)
  )
  fit <- hazard(
    survival::Surv(t, d) ~ x1 + x2,
    data  = df,
    dist  = "weibull",
    theta = c(mu = 0.05, nu = 0.8, 0, 0),
    fit   = TRUE
  )

  bs <- hzr_bootstrap(fit, n_boot = 5, seed = 123)
  params <- bs$replicates$parameter

  expect_false(any(!nzchar(params)))
  expect_true(all(c("x1", "x2") %in% params))
  # Each successful replicate should contribute one row per covariate.
  expect_equal(sum(params == "x1"), bs$n_success)
  expect_equal(sum(params == "x2"), bs$n_success)
  # Summary table should also carry the covariate labels.
  expect_true(all(c("x1", "x2") %in% bs$summary$parameter))
})

test_that("hzr_bootstrap preserves named betas while filling blanks", {
  # Mixed-naming case: one beta is explicitly named in the starting theta
  # and the other is left blank. The named slot must be preserved; the
  # blank slot should pick up its design-matrix column name.
  set.seed(11)
  n <- 80
  df <- data.frame(
    t = stats::rexp(n, 0.1),
    d = stats::rbinom(n, 1, 0.6),
    x1 = stats::rnorm(n),
    x2 = stats::rnorm(n)
  )
  fit <- hazard(
    survival::Surv(t, d) ~ x1 + x2,
    data  = df,
    dist  = "weibull",
    theta = c(mu = 0.05, nu = 0.8, b_custom = 0, 0),
    fit   = TRUE
  )

  bs <- hzr_bootstrap(fit, n_boot = 5, seed = 321)
  params <- bs$replicates$parameter

  expect_false(any(!nzchar(params)))
  # Named beta retained; blank beta filled from colnames(x) -> "x2".
  expect_true("b_custom" %in% params)
  expect_true("x2" %in% params)
  expect_false("x1" %in% params)
})

test_that("hzr_bootstrap resamples weights alongside the data", {
  # Regression test for the weighted-bootstrap bug: the resample loop used to
  # rewire only `data`, leaving the original full-length `weights` vector bound
  # to the refit call. Two observable consequences this guards against:
  #   1. fraction < 1 -> weights length (n_obs) no longer matches the smaller
  #      resampled data, so every replicate errored out: n_success == 0.
  #   2. fraction == 1 -> length matched, so no error, but weights were applied
  #      to the wrong (resampled) rows: silently incorrect estimates.
  set.seed(202)
  n <- 100
  df <- data.frame(
    t = stats::rexp(n, 0.1),
    d = stats::rbinom(n, 1, 0.6),
    x1 = stats::rnorm(n)
  )
  w <- stats::runif(n, 0.5, 2.0)

  fit <- hazard(
    survival::Surv(t, d) ~ x1,
    data    = df,
    weights = w,
    dist    = "weibull",
    theta   = c(mu = 0.05, nu = 0.8, 0),
    fit     = TRUE
  )

  # (1) Subsampled weighted bootstrap must produce successful replicates.
  bs_frac <- hzr_bootstrap(fit, n_boot = 10, fraction = 0.5, seed = 123)
  expect_gt(bs_frac$n_success, 0)
  expect_equal(bs_frac$n_failed, 0)

  # (2) Full-size weighted bootstrap must also succeed and be well-formed.
  bs_full <- hzr_bootstrap(fit, n_boot = 10, seed = 123)
  expect_gt(bs_full$n_success, 0)
  expect_true(all(is.finite(bs_full$summary$mean)))
})

test_that("hzr_bootstrap uses stored weights when the original symbol is out of scope", {
  # Robustness regression: the resample setup previously re-evaluated the call's
  # `weights` expression in parent.frame(). If the original weights symbol is no
  # longer in scope at bootstrap time -- e.g. the fit was built inside a helper
  # that has since returned -- that eval() raised "object not found". The fit now
  # carries its evaluated weights on object$data$weights, so bootstrap no longer
  # depends on a caller-frame lookup.
  set.seed(77)
  n <- 80
  # `df` stays in the test frame so the call's `data` symbol still resolves;
  # only the `weights` symbol is confined to the local() scope and gone by the
  # time we bootstrap, isolating the weights lookup.
  df <- data.frame(
    t  = stats::rexp(n, 0.1),
    d  = stats::rbinom(n, 1, 0.6),
    x1 = stats::rnorm(n)
  )
  fit <- local({
    local_w <- stats::runif(n, 0.5, 2.0)
    hazard(
      survival::Surv(t, d) ~ x1,
      data = df, weights = local_w, dist = "weibull",
      theta = c(mu = 0.05, nu = 0.8, 0), fit = TRUE
    )
  })
  # `local_w` is now out of scope; bootstrap must still resample weights via the
  # stored copy and produce successful replicates.
  bs <- hzr_bootstrap(fit, n_boot = 10, fraction = 0.5, seed = 5)
  expect_gt(bs$n_success, 0)
  expect_equal(bs$n_failed, 0)
})

test_that("hzr_bootstrap uses stored data frame when the original symbol is out of scope", {
  # Robustness regression (sibling of the weights case): the resample setup
  # re-evaluated the call's `data` expression via eval(cl$data, parent.frame()).
  # If the original data symbol is no longer in scope at bootstrap time -- e.g.
  # the entire fit was built inside a helper that has since returned -- that
  # eval() failed and n_obs was malformed, so bootstrap errored before the loop.
  # The fit now carries its evaluated model frame on object$data$frame, so the
  # resample no longer depends on a caller-frame lookup of the data symbol.
  set.seed(91)
  n <- 80
  # Build EVERYTHING inside local(): both the `data` (`bdf`) and `weights`
  # (`bw`) symbols are gone by the time we bootstrap.
  fit <- local({
    bdf <- data.frame(
      t  = stats::rexp(n, 0.1),
      d  = stats::rbinom(n, 1, 0.6),
      x1 = stats::rnorm(n)
    )
    bw <- stats::runif(n, 0.5, 2.0)
    hazard(
      survival::Surv(t, d) ~ x1,
      data = bdf, weights = bw, dist = "weibull",
      theta = c(mu = 0.05, nu = 0.8, 0), fit = TRUE
    )
  })
  bs <- hzr_bootstrap(fit, n_boot = 10, fraction = 0.5, seed = 7)
  expect_gt(bs$n_success, 0)
  expect_equal(bs$n_failed, 0)
})


# =========================================================================
# hzr_competing_risks tests
# =========================================================================

test_that("hzr_competing_risks returns correct structure", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)

  expect_s3_class(cr, "hzr_competing_risks")
  expect_true(nrow(cr) > 0)
  expect_true("surv" %in% names(cr))
  expect_true("incid_1" %in% names(cr))
  expect_true("incid_2" %in% names(cr))
  expect_true("se_surv" %in% names(cr))
})

test_that("hzr_competing_risks survival + incidences sum to ~1", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)

  # At each time: surv + sum(incid) should be approximately 1
  total <- cr$surv + cr$incid_1 + cr$incid_2
  expect_true(all(abs(total - 1) < 0.01),
              label = "surv + incidences should sum to ~1")
})

test_that("hzr_competing_risks survival is non-increasing", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)

  expect_true(all(diff(cr$surv) <= 1e-10))
  expect_true(all(cr$surv >= 0 & cr$surv <= 1))
})

test_that("hzr_competing_risks incidences are non-decreasing", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)

  expect_true(all(diff(cr$incid_1) >= -1e-10))
  expect_true(all(diff(cr$incid_2) >= -1e-10))
})

test_that("hzr_competing_risks rejects invalid inputs", {
  expect_error(hzr_competing_risks("a", 1), "numeric")
  expect_error(hzr_competing_risks(1:5, rep(0, 5)), "No events")
})

test_that("print.hzr_competing_risks runs without error", {
  data(valves, package = "TemporalHazard")
  valves_cc <- na.omit(valves)
  event_cr <- ifelse(valves_cc$dead == 1, 1L,
                     ifelse(valves_cc$pve == 1, 2L, 0L))
  time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
  cr <- hzr_competing_risks(time_cr, event_cr)
  expect_output(print(cr), "Competing risks")
})

# math_primitives.R -- Numerically stable elementary functions
#
# PURPOSE
# -------
# Floating-point naive implementations of log(1+exp(x)), log(1-exp(-x)), and
# probability clamping produce incorrect results near the boundaries of
# double precision.  These primitives use piece-wise stable formulas that are
# safe across the full range of inputs encountered in survival analysis
# (very small or very large hazard rates, near-zero probabilities).
#
# All three functions are exported so users can call them directly when
# constructing custom likelihoods or diagnostics.

#' Numerically stable log(1 + exp(x))
#'
#' Computes the "softplus" function
#' \deqn{\mathrm{log1pexp}(x) = \log\!\bigl(1 + e^{x}\bigr)}
#' without overflow.  A naive `log(1 + exp(x))` overflows to `Inf` for
#' \eqn{x \gtrsim 710} even though the true value is \eqn{\approx x}.  The
#' identity \eqn{\log(1 + e^{x}) = x + \log(1 + e^{-x})} for \eqn{x > 0} keeps
#' every intermediate finite.
#'
#' @param x Numeric vector.
#' @return Numeric vector with element-wise \eqn{\log(1 + e^{x})}.
#' @references
#' \enc{Mächler}{Machler} M (2012). *Accurately Computing \eqn{\log(1 - e^{-|a|})}.* R package
#' Rmpfr vignette.
#' \url{https://CRAN.R-project.org/package=Rmpfr/vignettes/log1mexp-note.pdf}
#' @seealso [hzr_log1mexp()] for the complementary \eqn{\log(1 - e^{-x})},
#'   [hzr_clamp_prob()] for boundary-safe probabilities.
#' @examples
#' hzr_log1pexp(c(-50, 0, 50))
#' @export
hzr_log1pexp <- function(x) {
  ifelse(x > 0, x + log1p(exp(-x)), log1p(exp(x)))
}

#' Numerically stable log(1 - exp(-x)) for x > 0
#'
#' Computes
#' \deqn{\mathrm{log1mexp}(x) = \log\!\bigl(1 - e^{-x}\bigr), \qquad x > 0}
#' accurately across the full range.  Following \enc{Mächler}{Machler} (2012), the evaluation
#' switches at \eqn{x = \log 2}: for small \eqn{x} it uses
#' \eqn{\log(-\mathrm{expm1}(-x))} (avoiding cancellation as \eqn{x \to 0}), and
#' for larger \eqn{x} it uses \eqn{\mathrm{log1p}(-e^{-x})} (avoiding loss of
#' precision as \eqn{x \to \infty}).  Non-positive or non-finite inputs return
#' `NA`.
#'
#' @param x Numeric vector with positive values.
#' @return Numeric vector with element-wise \eqn{\log(1 - e^{-x})}.
#' @references
#' \enc{Mächler}{Machler} M (2012). *Accurately Computing \eqn{\log(1 - e^{-|a|})}.* R package
#' Rmpfr vignette.
#' \url{https://CRAN.R-project.org/package=Rmpfr/vignettes/log1mexp-note.pdf}
#' @seealso [hzr_log1pexp()] for the softplus \eqn{\log(1 + e^{x})},
#'   [hzr_clamp_prob()] for boundary-safe probabilities.
#' @examples
#' hzr_log1mexp(c(0.01, 0.5, 5))
#' @export
hzr_log1mexp <- function(x) {
  out <- rep(NA_real_, length(x))
  cutoff <- log(2)

  valid <- is.finite(x) & (x > 0)
  small <- valid & (x <= cutoff)
  large <- valid & (x > cutoff)

  out[small] <- log(-expm1(-x[small]))
  out[large] <- log1p(-exp(-x[large]))
  out
}

#' Clamp probabilities away from 0 and 1
#'
#' Bounds a probability vector to \eqn{[\epsilon,\, 1 - \epsilon]} so that
#' downstream \eqn{\log p} or \eqn{\log(1 - p)} evaluations stay finite.  Used
#' throughout the likelihood and prediction code to guard survival and CDF
#' values against exact 0 or 1.
#'
#' @param p Numeric vector of probabilities.
#' @param eps Small positive tolerance in \eqn{(0, 0.5)}; default `1e-12`.
#' @return Numeric vector bounded to `[eps, 1 - eps]`.
#' @seealso [hzr_log1pexp()] and [hzr_log1mexp()] for the companion
#'   stable-logarithm primitives.
#' @examples
#' hzr_clamp_prob(c(0, 0.5, 1))
#' @export
hzr_clamp_prob <- function(p, eps = 1e-12) {
  if (!is.numeric(eps) || length(eps) != 1 || !is.finite(eps) || eps <= 0 || eps >= 0.5) {
    stop("'eps' must be a finite scalar in (0, 0.5).", call. = FALSE)
  }

  pmin(1 - eps, pmax(eps, p))
}

# weights-fixture.R -- Load and validate SAS fractional-weight parity fixtures.
#
# Fixtures live under `inst/fixtures/weighted-<scenario>.rds`.  Like the
# stepwise fixtures they are optional: the parity test skips when the file
# is absent, so the package installs and CI passes without any SAS output.
#
# The workflow for capturing one is documented in
# `inst/extdata/weights-fixtures/README.md`.  .hzr_build_weighted_fixture()
# packages the SAS coefficient table + meta into the expected list shape;
# .hzr_load_weighted_fixture() is the lookup called from the parity test.
#
# `.hzr_parse_keyvals()` is shared with stepwise-fixture.R.

#' Required top-level fields of a weighted-fit parity fixture
#'
#' @keywords internal
#' @noRd
.hzr_weighted_fixture_schema <- function() {
  list(
    meta = c(
      "dataset", "dist", "weight_var", "weight_kind",
      "sas_version", "captured_on", "proc_hazard"
    ),
    final = c("coef", "logLik")
  )
}


#' Validate a weighted-fit parity fixture against the schema
#'
#' @param fix A list as produced by `.hzr_build_weighted_fixture()` or
#'   deserialised from `inst/fixtures/weighted-*.rds`.
#' @return The fixture, unchanged, if valid.  Throws otherwise.
#'
#' @keywords internal
#' @noRd
.hzr_validate_weighted_fixture <- function(fix) {
  schema <- .hzr_weighted_fixture_schema()
  problems <- character()

  for (group in names(schema)) {
    if (!group %in% names(fix)) {
      problems <- c(problems, paste0("missing top-level: ", group))
      next
    }
    missing_fields <- setdiff(schema[[group]], names(fix[[group]]))
    if (length(missing_fields) > 0L) {
      problems <- c(problems,
                    sprintf("missing %s: %s", group,
                            paste(missing_fields, collapse = ", ")))
    }
  }

  if (!is.null(fix$final) && !is.null(fix$final$coef)) {
    needed <- c("variable", "phase", "estimate")
    miss   <- setdiff(needed, names(fix$final$coef))
    if (length(miss) > 0L) {
      problems <- c(problems,
                    sprintf("final$coef missing columns: %s",
                            paste(miss, collapse = ", ")))
    }
  }

  if (length(problems) > 0L) {
    stop("Invalid weighted parity fixture:\n  - ",
         paste(problems, collapse = "\n  - "),
         call. = FALSE)
  }
  invisible(fix)
}


#' Load a weighted-fit parity fixture by name
#'
#' Returns NULL (not an error) when the fixture file is absent.
#'
#' @param name Short name like `"avc-fractional"`.
#' @return The fixture list or NULL.
#'
#' @keywords internal
#' @noRd
.hzr_load_weighted_fixture <- function(name) {
  file_name <- paste0("weighted-", name, ".rds")
  path <- system.file("fixtures", file_name, package = "TemporalHazard")
  if (!nzchar(path) || !file.exists(path)) {
    return(NULL)
  }
  fix <- readRDS(path)
  .hzr_validate_weighted_fixture(fix)
}


#' Build a weighted-fit parity fixture from SAS capture files
#'
#' Reads the coefficient CSV and `key=value` meta file emitted by
#' `inst/extdata/weights-fixtures/parse-weighted-lst.R` and assembles the
#' fixture list.  Writes the result to `out_path` if supplied.
#'
#' @param coef_csv Path to the parsed coefficient table CSV (columns
#'   `variable`, `phase`, `estimate`, and optionally `std_error`,
#'   `z_stat`, `p_value`).
#' @param meta_txt Path to the `key=value` meta file (must include a
#'   `logLik=` line, added by the parser).
#' @param out_path Optional output `.rds` path.
#'
#' @return The fixture list.
#'
#' @keywords internal
#' @noRd
.hzr_build_weighted_fixture <- function(coef_csv, meta_txt, out_path = NULL) {
  if (!file.exists(coef_csv)) {
    stop("coef_csv not found: ", coef_csv, call. = FALSE)
  }
  if (!file.exists(meta_txt)) {
    stop("meta_txt not found: ", meta_txt, call. = FALSE)
  }

  coef_tbl <- utils::read.csv(coef_csv, stringsAsFactors = FALSE)
  if ("phase" %in% names(coef_tbl)) {
    coef_tbl$phase[coef_tbl$phase == "."] <- NA_character_
  }

  meta_kv <- .hzr_parse_keyvals(readLines(meta_txt))

  fix <- list(
    meta = list(
      dataset     = meta_kv[["dataset"]]     %||% NA_character_,
      dist        = meta_kv[["dist"]]        %||% "multiphase",
      model       = meta_kv[["model"]]       %||% NA_character_,
      covariates  = meta_kv[["covariates"]]  %||% NA_character_,
      weight_var  = meta_kv[["weight_var"]]  %||% NA_character_,
      weight_kind = meta_kv[["weight_kind"]] %||% NA_character_,
      conserve    = as.integer(meta_kv[["conserve"]] %||% NA_integer_),
      sas_version = meta_kv[["sas_version"]] %||% NA_character_,
      captured_on = meta_kv[["captured_on"]] %||% NA_character_,
      proc_hazard = meta_kv[["proc_hazard"]] %||% NA_character_,
      notes       = meta_kv[["notes"]]
    ),
    final = list(
      coef   = coef_tbl,
      logLik = as.numeric(meta_kv[["logLik"]] %||% NA_real_)
    )
  )

  .hzr_validate_weighted_fixture(fix)

  if (!is.null(out_path)) {
    dir.create(dirname(out_path), showWarnings = FALSE, recursive = TRUE)
    saveRDS(fix, out_path)
  }
  fix
}

# SAS parity test for FRACTIONAL (non-integer) observation weights.
# Roadmap 7a / FIXTURE-GAP-LIST B5: confirm SAS PROC HAZARD's WEIGHT
# statement matches hazard(weights = ...) for non-integer weights.
#
# R-side correctness is already proven without SAS by the additive-split
# and linear-scaling invariants in test-weights.R.  This test adds the
# external SAS confirmation, and like the other parity tests it skips
# gracefully when the capture fixture is absent, so the package installs
# and passes CI with no SAS output.
#
# To enable: run the SAS template in
#   inst/extdata/weights-fixtures/payload/avc-weighted.sas  (via run.sh)
# then convert the listing with
#   source("inst/extdata/weights-fixtures/parse-weighted-lst.R")
# which writes inst/fixtures/weighted-avc-fractional.rds.

.weighted_parity_tolerance <- list(
  logLik      = 0.5,    # absolute
  coefficient = 0.02    # relative; covariate betas for shared phases
)

# Re-fit the exact SAS specification in R: two-phase early-CDF + constant,
# AGE in both phases, fractional IPW weights, conservation of events on.
# Uses the identical committed avc-weighted.csv that the SAS run imports.
.weighted_parity_run <- function() {
  csv <- system.file("extdata", "weights-fixtures", "payload",
                     "avc-weighted.csv", package = "TemporalHazard")
  if (!nzchar(csv) || !file.exists(csv)) {
    testthat::skip("avc-weighted.csv payload not found")
  }
  d <- utils::read.csv(csv, stringsAsFactors = FALSE)
  d <- d[stats::complete.cases(d[, c("int_dead", "dead", "age", "ipw")]), ]

  # Multiphase fits with n_starts > 1 perturb the initial values from the
  # ambient RNG.  Seed locally and restore the stream on exit so the test is
  # reproducible and does not leak RNG state into later tests (mirrors
  # test-stepwise-parity.R).
  if (exists(".Random.seed", envir = globalenv(), inherits = FALSE)) {
    old_seed <- get(".Random.seed", envir = globalenv(), inherits = FALSE)
    on.exit(assign(".Random.seed", old_seed, envir = globalenv()), add = TRUE)
  } else {
    on.exit(suppressWarnings(rm(".Random.seed", envir = globalenv())),
            add = TRUE)
  }
  set.seed(20260611L)
  suppressWarnings(hazard(
    survival::Surv(int_dead, dead) ~ 1, data = d, dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.1512095, nu = 1.438652, m = 1,
                           fixed = "shapes", formula = ~ age),
      constant = hzr_phase("constant", formula = ~ age)),
    weights = d$ipw, fit = TRUE,
    control = list(n_starts = 3L, maxit = 500L, conserve = TRUE)))
}


test_that("AVC fractional-weight fit: R covariate estimates match SAS", {
  skip_on_cran()
  fix <- .hzr_load_weighted_fixture("avc-fractional")
  if (is.null(fix)) {
    skip("Weighted parity fixture weighted-avc-fractional.rds not found")
  }

  r_fit  <- .weighted_parity_run()
  r_coef <- coef(r_fit)

  # SAS covariate rows (exclude the intercepts E0 / C0); map each to R's
  # "phase.variable" coefficient name (e.g. "early.age").
  sas <- fix$final$coef
  sas_cov <- sas[!sas$variable %in% c("e0", "c0"), ]

  for (i in seq_len(nrow(sas_cov))) {
    nm <- paste0(sas_cov$phase[i], ".", sas_cov$variable[i])
    expect_true(nm %in% names(r_coef),
                label = paste0("R model has coefficient ", nm))
    expect_equal(unname(r_coef[[nm]]), sas_cov$estimate[i],
                 tolerance = .weighted_parity_tolerance$coefficient,
                 label = paste0(nm, " (R ", round(r_coef[[nm]], 5),
                                " vs SAS ", sas_cov$estimate[i], ")"))
  }
})

test_that("AVC fractional-weight fit: R log-likelihood matches SAS", {
  skip_on_cran()
  fix <- .hzr_load_weighted_fixture("avc-fractional")
  if (is.null(fix)) {
    skip("Weighted parity fixture weighted-avc-fractional.rds not found")
  }

  r_fit <- .weighted_parity_run()
  expect_equal(r_fit$fit$objective, fix$final$logLik,
               tolerance = .weighted_parity_tolerance$logLik,
               label = paste0("R logLik (", round(r_fit$fit$objective, 3),
                              ") vs SAS logLik (", fix$final$logLik, ")"))
})


# Sanity: the loader/validator work on a synthetic fixture (no SAS output
# required).  Mirrors the stepwise fixture-validator tests.
test_that("weighted fixture validator rejects malformed input", {
  bad <- list(meta = list(dataset = "avc"), final = list(logLik = -100))
  expect_error(
    TemporalHazard:::.hzr_validate_weighted_fixture(bad),
    "Invalid weighted parity fixture"
  )
})

test_that("weighted fixture validator accepts a well-formed skeleton", {
  ok <- list(
    meta = list(
      dataset = "avc", dist = "multiphase", weight_var = "ipw",
      weight_kind = "fractional", sas_version = "9.4",
      captured_on = "2026-06-11", proc_hazard = "PROC HAZARD ... WEIGHT IPW"
    ),
    final = list(
      coef = data.frame(
        variable = c("e0", "age", "c0", "age"),
        phase    = c("early", "early", "constant", "constant"),
        estimate = c(-0.97, -0.008, -7.88, 0.0018),
        stringsAsFactors = FALSE
      ),
      logLik = -175.5
    )
  )
  expect_silent(TemporalHazard:::.hzr_validate_weighted_fixture(ok))
})

# argument_mapping.R -- SAS HAZARD -> R argument translation table
#
# PURPOSE
# -------
# During the SAS-to-R migration, analysts may need to know how legacy HAZARD
# procedure statements map to hazard() arguments.  hzr_argument_mapping()
# exposes the mapping as a data frame so it can be consulted programmatically
# or rendered in vignettes.
#
# The internal table .hzr_argument_mapping_table drives the exported function.
# Add rows as new arguments are implemented; update implementation_status from
# "planned" to "implemented" when the feature lands.

#' Legacy HAZARD to TemporalHazard argument mapping
#'
#' Returns a formal mapping table that defines how legacy SAS HAZARD/C-style
#' inputs map to `hazard(...)` arguments in this package.
#'
#' @param include_planned Logical; if `FALSE`, only rows marked as implemented
#'   are returned.
#'
#' @return A data frame with one row per mapping rule.
#' @examples
#' hzr_argument_mapping()
#' hzr_argument_mapping(include_planned = FALSE)
#' @export
hzr_argument_mapping <- function(include_planned = TRUE) {
  map <- .hzr_argument_mapping_table
  if (!isTRUE(include_planned)) {
    map <- map[map$implementation_status == "implemented", , drop = FALSE]
  }
  rownames(map) <- NULL
  map
}

.hzr_argument_mapping_table <- data.frame(
  sas_statement = c(
    # --- Core hazard() arguments -----------------------------------------------
    "HAZARD", "HAZARD", "HAZARD", "HAZARD", "HAZARD", "HAZARD", "HAZARD",
    "TIME", "EVENT", "PARMS", "DIST",
    # --- Multiphase (phases / hzr_phase) arguments -----------------------------
    "HAZARD", "HAZARD",
    "G1", "G1", "G1", "G1",
    "G2",
    "G3", "G3", "G3", "G3"
  ),
  legacy_input = c(
    "TIME variable", "EVENT/censor variable", "X covariate block",
    "initial parameters", "baseline distribution", "control options",
    "additional legacy options", "t", "status", "theta0", "dist",
    # Multiphase
    "phases (3-phase structure)", "MU_1, MU_2, MU_3",
    "THALF / RHO (early)", "NU (early)", "M (early)", "DELTA (early)",
    "G2 constant phase",
    "TAU (late)", "GAMMA (late)", "ALPHA (late)", "ETA (late)"
  ),
  c_concept = c(
    "obs time array", "event indicator array", "design matrix",
    "parameter vector", "phase distribution selector", "optimizer/control struct",
    "misc legacy switches", "time vector", "event vector", "starting coef", "dist selector",
    # Multiphase
    "3-phase Early/Const/Late", "per-phase scale factors",
    "early half-life", "early time exponent", "early shape", "early time transform",
    "constant hazard rate phase",
    "late G3 scale", "late G3 time exponent", "late G3 shape", "late G3 outer exponent"
  ),
  r_parameter = c(
    "time", "status", "x", "theta", "dist", "control", "...",
    "time", "status", "theta", "dist",
    # Multiphase
    "phases (list of hzr_phase())", "mu (via exp(log_mu) in theta)",
    "hzr_phase(t_half=)", "hzr_phase(nu=)", "hzr_phase(m=)", "(absorbed by decompos)",
    "hzr_phase('constant')",
    "hzr_phase('g3', tau=)", "hzr_phase('g3', gamma=)", "hzr_phase('g3', alpha=)", "hzr_phase('g3', eta=)"
  ),
  required = c(
    TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE, FALSE, FALSE,
    # Multiphase
    FALSE, FALSE,
    FALSE, FALSE, FALSE, FALSE,
    FALSE,
    FALSE, FALSE, FALSE, FALSE
  ),
  expected_type = c(
    "numeric vector", "numeric/logical vector", "numeric matrix or data.frame",
    "numeric vector", "character scalar", "named list", "named arguments",
    "numeric vector", "numeric/logical vector", "numeric vector", "character scalar",
    # Multiphase
    "list of hzr_phase objects", "numeric (per-phase)",
    "positive scalar", "numeric scalar", "numeric scalar", "numeric scalar",
    "hzr_phase('constant')",
    "positive scalar", "numeric scalar", "numeric scalar", "numeric scalar"
  ),
  transform_rule = c(
    "pass through as numeric", "coerce to numeric 0/1", "data.frame -> data.matrix",
    "length must equal ncol(x) when x is present", "normalized lower-case label",
    "stored in spec$control", "stored in legacy_args for parity",
    "pass through", "coerce to numeric", "map PARMS/INITIAL to theta", "map DIST= to dist",
    # Multiphase
    "list(early=hzr_phase('cdf',...), constant=hzr_phase('constant'), late=hzr_phase('g3',...))",
    "exp(alpha_j) in internal parameterization; estimated on log scale",
    "maps directly to hzr_phase(t_half=) starting value",
    "maps directly to hzr_phase(nu=) starting value",
    "maps directly to hzr_phase(m=) starting value",
    "time transform B(t) = (exp(delta*t)-1)/delta absorbed into decompos shape",
    "hzr_phase('constant') with no shape parameters",
    "maps directly to hzr_phase('g3', tau=) for late phase",
    "maps directly to hzr_phase('g3', gamma=) for late phase",
    "maps directly to hzr_phase('g3', alpha=) for late phase",
    "maps directly to hzr_phase('g3', eta=) for late phase"
  ),
  implementation_status = c(
    "implemented", "implemented", "implemented", "implemented", "implemented",
    "implemented", "implemented", "implemented", "implemented", "planned", "implemented",
    # Multiphase
    "implemented", "implemented",
    "implemented", "implemented", "implemented", "implemented",
    "implemented",
    "implemented", "implemented", "implemented", "implemented"
  ),
  notes = c(
    "Core observation time input.",
    "Event indicator currently retained as numeric in object$data$status.",
    "Future versions will support richer design encoding helpers.",
    "Used by predict.hazard as coefficient vector.",
    "Current default is 'weibull'; more options planned.",
    "Control list is stored and reserved for optimizer parity.",
    "Supports legacy-style pass-through options during migration.",
    "Canonical SAS migration uses TIME= mapping.",
    "Canonical SAS migration uses EVENT= mapping.",
    "SAS PARMS syntax parser not yet implemented.",
    "SAS DIST keyword maps directly to dist.",
    # Multiphase
    "Use dist='multiphase' with phases argument. N-phase generalization of legacy 3-phase model.",
    "Each phase has its own scale mu_j(x) = exp(alpha_j + x*beta_j). Starting value via hzr_phase().",
    "Half-life: time at which G(t_half) = 0.5. Same concept as SAS RHO/THALF.",
    "Time exponent controlling rate dynamics. Same parameter name as SAS early NU.",
    "Shape exponent controlling distributional form. Same parameter name as SAS early M.",
    "The C DELTA controlled B(t) = (exp(delta*t)-1)/delta. This transform is absorbed by decompos().",
    "Flat background rate. No shape parameters estimated. SAS G2 equivalent.",
    "Late-phase G3 scale parameter. Maps directly to hzr_phase('g3', tau=).",
    "Late-phase G3 time exponent. Maps directly to hzr_phase('g3', gamma=).",
    "Late-phase G3 shape parameter. alpha=0 gives exponential case. Maps directly to hzr_phase('g3', alpha=).",
    "Late-phase G3 outer exponent. Maps directly to hzr_phase('g3', eta=).")
  ,
  stringsAsFactors = FALSE
)

test_that("hzr_argument_mapping returns required schema", {
  map <- hzr_argument_mapping()

  expect_true(is.data.frame(map))
  expect_true(nrow(map) >= 8)

  required_cols <- c(
    "sas_statement", "legacy_input", "c_concept", "r_parameter",
    "required", "expected_type", "transform_rule",
    "implementation_status", "notes"
  )
  expect_true(all(required_cols %in% names(map)))
})

test_that("mapping includes core hazard() arguments", {
  map <- hzr_argument_mapping(include_planned = FALSE)
  params <- unique(map$r_parameter)

  expect_true(all(c("time", "status", "x", "theta", "dist", "control", "...") %in% params))
})

test_that("mapping table has no duplicate legacy-to-parameter rows", {
  map <- hzr_argument_mapping()
  key <- paste(map$legacy_input, map$r_parameter, sep = "::")
  expect_equal(anyDuplicated(key), 0)
})

# wald.R -- Wald inference for stepwise covariate selection.
#
# The stepwise engine (hzr_stepwise, Phase 4b of the development plan)
# needs a lightweight Wald-test helper that works against any fitted
# hazard model:
#
#   - Scalar test for a single named coefficient  -> two-sided z-test
#   - Joint test for a set of named coefficients  -> chi-square Wald
#
# Both cases produce a p-value on the same scale so the stepwise loop
# can compare candidates uniformly.
#
# This helper is intentionally forgiving about boundary / fixed
# parameters: if the vcov matrix is absent or any requested coefficient
# has a non-finite variance, it returns p = NA rather than erroring,
# matching the summary.hazard() convention.

#' Wald test for one or more coefficients in a fitted hazard model
#'
#' Computes the Wald statistic and p-value for a subset of coefficients
#' from a fitted `hazard` object.  A single name yields a two-sided
#' z-test on the standardised coefficient; multiple names yield a joint
#' chi-square Wald test on the corresponding submatrix of the vcov.
#'
#' @param fit A fitted `hazard` object produced by [hazard()].
#' @param names Character vector of coefficient names to test.  Must be
#'   a subset of `names(coef(fit))`.
#'
#' @return A list with elements:
#' \describe{
#'   \item{stat}{Test statistic -- `z` if `length(names) == 1L`,
#'     chi-square otherwise.}
#'   \item{df}{Degrees of freedom (1 for scalar, `length(names)` joint).}
#'   \item{p_value}{Two-sided p-value, or `NA_real_` if the test could
#'     not be computed (no vcov or non-finite SE for any coefficient in
#'     `names`).}
#'   \item{names}{Character vector of the coefficients tested.}
#'   \item{estimate}{Numeric vector of coefficient estimates for the
#'     requested `names`.}
#' }
#'
#' @keywords internal
#' @noRd
.hzr_wald_p <- function(fit, names) {
  if (!inherits(fit, "hazard")) {
    stop("`fit` must be a `hazard` object.", call. = FALSE)
  }
  if (!is.character(names) || length(names) == 0L || any(!nzchar(names))) {
    stop("`names` must be a non-empty character vector of coefficient names.",
         call. = FALSE)
  }

  theta <- fit$fit$theta
  if (is.null(theta)) {
    stop("`fit` has no fitted coefficients (did you call hazard(..., fit = TRUE)?).",
         call. = FALSE)
  }

  # Canonical coefficient names: multiphase stores them directly on
  # theta; single-distribution fits are unnamed, so regenerate via the
  # same helper summary.hazard() uses.
  coef_names <- base::names(theta)
  if (is.null(coef_names) || !all(nzchar(coef_names))) {
    p <- if (is.null(fit$data$x)) 0L else ncol(fit$data$x)
    coef_names <- .hzr_parameter_names(
      theta = theta,
      dist  = fit$spec$dist,
      p     = p
    )
  }

  missing_names <- setdiff(names, coef_names)
  if (length(missing_names) > 0L) {
    stop(
      "Unknown coefficient name(s): ",
      paste(sQuote(missing_names), collapse = ", "),
      ".  Available: ",
      paste(sQuote(coef_names), collapse = ", "),
      call. = FALSE
    )
  }

  idx <- match(names, coef_names)
  estimate <- setNames(theta[idx], names)

  # Access vcov directly on the fit rather than via vcov.hazard(), which
  # collapses to NA if any diagonal entry is NA (common for fixed shape
  # parameters). summary.hazard() handles partial NAs by masking, so
  # match that behaviour.
  vcov_mat <- fit$fit$vcov

  na_result <- function(stat = NA_real_, df = length(names)) {
    list(
      stat     = stat,
      df       = df,
      p_value  = NA_real_,
      names    = names,
      estimate = unname(estimate)
    )
  }

  if (is.null(vcov_mat) || !is.matrix(vcov_mat)) {
    return(na_result())
  }

  v_sub <- vcov_mat[idx, idx, drop = FALSE]
  diag_v <- diag(v_sub)
  if (any(!is.finite(diag_v)) || any(diag_v <= 0)) {
    return(na_result())
  }

  if (length(names) == 1L) {
    # Scalar z-test
    se <- sqrt(diag_v)
    z <- estimate / se
    p <- 2 * stats::pnorm(-abs(z))
    return(list(
      stat     = unname(z),
      df       = 1L,
      p_value  = unname(p),
      names    = names,
      estimate = unname(estimate)
    ))
  }

  # Joint chi-square Wald: W = beta' V^{-1} beta  ~  chi^2_{df}
  v_inv <- tryCatch(solve(v_sub), error = function(e) NULL)
  if (is.null(v_inv)) {
    return(na_result())
  }
  stat <- as.numeric(crossprod(estimate, v_inv %*% estimate))
  if (!is.finite(stat) || stat < 0) {
    return(na_result())
  }
  p <- stats::pchisq(stat, df = length(names), lower.tail = FALSE)

  list(
    stat     = stat,
    df       = length(names),
    p_value  = p,
    names    = names,
    estimate = unname(estimate)
  )
}

# `.fit_weibull()` is defined in `helper-stepwise.R` and auto-sourced
# by testthat before any test-*.R files run.

test_that("scalar Wald p-value matches summary.hazard", {
  fit <- .fit_weibull()

  s <- summary(fit)
  # Find the single covariate row in the summary table
  covariate_row <- setdiff(rownames(s$coefficients), c("mu", "nu"))
  expect_length(covariate_row, 1L)

  wald <- .hzr_wald_p(fit, covariate_row)

  expect_equal(wald$df, 1L)
  expect_equal(wald$stat,     s$coefficients[covariate_row, "z_stat"],
               tolerance = 1e-10)
  expect_equal(wald$p_value,  s$coefficients[covariate_row, "p_value"],
               tolerance = 1e-10)
  expect_equal(wald$estimate, unname(s$coefficients[covariate_row, "estimate"]),
               tolerance = 1e-10)
})

test_that("joint Wald chi-square matches hand calculation on 2 coefficients", {
  fit <- .fit_weibull(n = 300L, betas = c(0.4, -0.3), seed = 7L)

  s <- summary(fit)
  covariate_rows <- setdiff(rownames(s$coefficients), c("mu", "nu"))
  expect_length(covariate_rows, 2L)

  w <- .hzr_wald_p(fit, covariate_rows)

  # Hand calc: W = b' V^{-1} b.  fit$fit$theta / vcov are unnamed for
  # single-distribution fits, so index by position via match() against
  # the canonical names summary() prints.
  all_names <- rownames(s$coefficients)
  idx <- match(covariate_rows, all_names)
  coef_vec <- fit$fit$theta[idx]
  V <- fit$fit$vcov[idx, idx]
  expected_stat <- as.numeric(crossprod(coef_vec, solve(V) %*% coef_vec))
  expected_p <- pchisq(expected_stat, df = 2, lower.tail = FALSE)

  expect_equal(w$df, 2L)
  expect_equal(w$stat, expected_stat, tolerance = 1e-10)
  expect_equal(w$p_value, expected_p, tolerance = 1e-10)
})

test_that("scalar z^2 equals the df=1 chi-square via pchisq", {
  fit <- .fit_weibull(n = 150L, betas = 0.5, seed = 99L)

  covariate_row <- setdiff(rownames(summary(fit)$coefficients),
                           c("mu", "nu"))

  w <- .hzr_wald_p(fit, covariate_row)
  expect_equal(
    pchisq(w$stat^2, df = 1L, lower.tail = FALSE),
    w$p_value,
    tolerance = 1e-10
  )
})

test_that("unknown coefficient name produces an informative error", {
  fit <- .fit_weibull(n = 100L, seed = 11L)
  expect_error(
    .hzr_wald_p(fit, "not_in_model"),
    "Unknown coefficient name"
  )
})

test_that("no vcov on the fit returns p_value = NA", {
  fit <- .fit_weibull(n = 80L, seed = 5L)
  covariate_row <- setdiff(rownames(summary(fit)$coefficients),
                           c("mu", "nu"))

  fit$fit$vcov <- NULL

  w <- .hzr_wald_p(fit, covariate_row)
  expect_true(is.na(w$p_value))
  expect_equal(w$df, 1L)
  expect_identical(w$names, covariate_row)
})

test_that("non-finite variance (boundary parameter) returns p_value = NA", {
  fit <- .fit_weibull(n = 80L, seed = 5L)
  s <- summary(fit)
  covariate_row <- setdiff(rownames(s$coefficients), c("mu", "nu"))

  # vcov for single-dist fits has no dimnames; poison by positional
  # index
  idx <- match(covariate_row, rownames(s$coefficients))
  V <- fit$fit$vcov
  V[idx, idx] <- NA_real_
  fit$fit$vcov <- V

  w <- .hzr_wald_p(fit, covariate_row)
  expect_true(is.na(w$p_value))
})

test_that("works on a multiphase fit with phase-prefixed coefficient names", {
  data(avc)
  set.seed(1)
  fit <- hazard(
    Surv(int_dead, dead) ~ age,
    data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 3L, maxit = 1000L)
  )

  coef_names <- names(coef(fit))
  expect_true(!is.null(coef_names))
  age_name <- grep("age", coef_names, value = TRUE)
  expect_true(length(age_name) >= 1L)

  w <- .hzr_wald_p(fit, age_name[1])
  expect_equal(w$df, 1L)
  # p_value is either a finite probability in [0, 1] or NA
  expect_true(is.na(w$p_value) ||
                (is.finite(w$p_value) && w$p_value >= 0 && w$p_value <= 1))
})

test_that("non-hazard object is rejected", {
  expect_error(.hzr_wald_p(list(), "x1"), "must be a `hazard` object")
})

test_that("empty or invalid names argument is rejected", {
  fit <- .fit_weibull(n = 50L, seed = 1L)
  expect_error(.hzr_wald_p(fit, character()), "non-empty character vector")
  expect_error(.hzr_wald_p(fit, c("mu", "")), "non-empty character vector")
  expect_error(.hzr_wald_p(fit, 1), "non-empty character vector")
})

#' @importFrom stats optim setNames
#' @keywords internal
NULL

# likelihood-weibull.R -- Weibull parametric hazard likelihood, gradient, and optimizer
#
# MODEL
# -----
# Proportional-hazards (PH) parameterization:
#
#   h(t | x)  = mu nu t^(nu-1) exp(eta)         hazard
#   H(t | x)  = (mu t)^nu exp(eta)              cumulative hazard
#   S(t | x)  = exp(-H(t | x))              survival
#
# where mu > 0 (scale), nu > 0 (shape), eta = x beta (linear predictor).
#
# THETA LAYOUT
# ------------
#   theta[1] = mu   (must be > 0; enforced by L-BFGS-B lower bound)
#   theta[2] = nu   (must be > 0; enforced by L-BFGS-B lower bound)
#   theta[3:p] = beta (covariate coefficients; unrestricted)
#
# Note: unlike the other distributions, Weibull optimises on the natural scale
# and uses L-BFGS-B with lower = 1e-6 instead of log-reparameterisation.
# This preserves direct interpretability of the mu/nu estimates.
#
# FUNCTIONS
# ---------
#   .hzr_logl_weibull()     -- log-likelihood (optionally returning gradient)
#   .hzr_gradient_weibull() -- analytical score vector
#   .hzr_optim_weibull()    -- L-BFGS-B wrapper with post-fit Hessian SE

# Weibull Parametric Hazard Likelihood
#
# Evaluate the log-likelihood and its derivatives for Weibull hazard models.
# This is the core mathematical engine for model fitting.

#' Log-likelihood for Weibull hazard with covariates
#'
#' Computes the negative log-likelihood for a sample under the Weibull
#' parametric hazard model with optional linear-predictor covariates.
#'
#' @param theta Vector of parameters:
#'   theta\[1:n_shape\]: Shape/scale parameters (distribution-specific)
#'   theta\[n_shape+1:length\]: Covariate coefficients (linear on log-hazard scale)
#'   where n_shape = number of shape parameters (e.g., 2 for Weibull scale/shape)
#'
#' @param time Numeric vector of follow-up times (n)
#' @param status Numeric vector of event indicators: 1 = event, 0 = censored (n)
#' @param time_lower Optional numeric lower bound vector for interval-censored rows.
#'   Defaults to time if NULL.
#' @param time_upper Optional numeric upper bound vector for left/interval-censored rows.
#'   Defaults to time if NULL.
#' @param x Design matrix of covariates (n x p_coef); NULL for no covariates
#' @param dist_name Character name of baseline distribution ("weibull", etc.)
#' @param return_gradient Logical; if TRUE, attach gradient vector as attribute
#' @param return_hessian Logical; if TRUE, attach Hessian matrix as attribute
#'
#' @return Scalar log-likelihood value. If return_gradient = TRUE, gradient vector
#' is attached as attribute \code{"gradient"}. If return_hessian = TRUE, Hessian
#' matrix is attached as \code{"hessian"}.
#'
#' @details
#' The Weibull hazard model is parameterized as:
#'
#' \deqn{h(t | x) = \mu \nu t^{\nu-1} \exp(\eta)}
#'
#' where:
#' - \eqn{\mu > 0} is scale (MU)
#' - \eqn{\nu > 0} is shape (NU)
#' - \eqn{\eta = x \beta} is log-relative-hazard (covariates)
#'
#' The log-likelihood for right-censored data is:
#'
#' \deqn{\ell(\theta) = \sum_{i: \delta_i = 1} \log h(t_i | x_i)
#'   - \sum_i S(t_i | x_i)}
#'
#' where S is the survival function (1 - CDF).
#'
#' Mixed censoring status coding:
#' - 1: exact event at time
#' - 0: right-censored at time
#' - -1: left-censored with upper bound time_upper \(or time\)
#' - 2: interval-censored in the interval \(time_lower, time_upper\)
#'
#' @noRd
.hzr_logl_weibull <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    dist_name = "weibull",
    return_gradient = FALSE,
    return_hessian = FALSE) {

  # Shape parameter count for Weibull
  n_shape <- 2  # mu, nu (scale, shape)
  n <- length(time)

  # Default unit weights
  if (is.null(weights)) weights <- rep(1, n)

  # Extract parameters
  mu <- theta[1]    # scale > 0
  nu <- theta[2]    # shape > 0

  # Covariate coefficients (if any)
  if (!is.null(x)) {
    if (is.null(attr(x, "dimnames")[[2]])) {
      colnames(x) <- paste0("beta_", seq_len(ncol(x)))
    }
    beta <- theta[3:length(theta)]
    eta <- as.numeric(x %*% beta)  # n-vector
  } else {
    eta <- rep(0, n)
  }

  # Stability checks
  if (mu <= 0 || nu <= 0) {
    return(Inf)  # Infeasible: return large penalty
  }

  # Normalize censoring bounds to lower/upper vectors for unified formulas.
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  # Exact events require strictly positive time because log(h(t)) is used.
  # Right-censored observations can be at time 0 (valid: log S(0) = 0).
  if (any(status == 1 & time <= 0)) {
    return(Inf)
  }
  if (any(status %in% c(-1, 2)) && any(upper <= 0)) {
    return(Inf)
  }
  if (any(status == 2) && any(lower[status == 2] >= upper[status == 2])) {
    return(Inf)
  }

  # Event-time hazard/cumulative-hazard (exact events only).
  haz_event <- mu * nu * (time ^ (nu - 1)) * exp(eta)
  cumhaz_event <- (mu * time) ^ nu * exp(eta)

  # Lower/upper cumulative hazards for censoring contributions.
  cumhaz_lower <- (mu * lower) ^ nu * exp(eta)
  cumhaz_upper <- (mu * upper) ^ nu * exp(eta)

  # Counting-process entry-time cumulative hazard (H(start)) for
  # event/right-censored rows.  When `time_lower` is NULL the entry time is
  # implicitly 0 and H(start) = 0, recovering the plain-Surv likelihood.
  start_vec <- if (is.null(time_lower)) rep(0, n) else time_lower
  cumhaz_start <- (mu * start_vec) ^ nu * exp(eta)

  idx_event <- status == 1
  idx_right <- status == 0
  idx_left <- status == -1
  idx_interval <- status == 2

  # Exact events: w * [log h(stop) - (H(stop) - H(start))]
  ll_event <- if (any(idx_event)) {
    sum(weights[idx_event] *
          (log(haz_event[idx_event]) -
             (cumhaz_event[idx_event] - cumhaz_start[idx_event])))
  } else {
    0
  }

  # Right-censored: w * [-(H(stop) - H(start))]
  ll_right <- if (any(idx_right)) {
    -sum(weights[idx_right] *
           (cumhaz_event[idx_right] - cumhaz_start[idx_right]))
  } else {
    0
  }

  # Left-censored: w * [log(1 - exp(-H(u)))]
  ll_left <- if (any(idx_left)) {
    sum(weights[idx_left] * hzr_log1mexp(cumhaz_upper[idx_left]))
  } else {
    0
  }

  # Interval-censored: w * [-H(l) + log(1 - exp(-(H(u)-H(l))))]
  ll_interval <- if (any(idx_interval)) {
    delta_h <- cumhaz_upper[idx_interval] - cumhaz_lower[idx_interval]
    sum(weights[idx_interval] *
          (-cumhaz_lower[idx_interval] + hzr_log1mexp(delta_h)))
  } else {
    0
  }

  logl <- ll_event + ll_right + ll_left + ll_interval

  if (!is.finite(logl)) {
    return(Inf)
  }

  # If gradient requested, compute score vector.
  if (return_gradient) {
    grad <- .hzr_gradient_weibull(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      eta = eta, cumhaz = cumhaz_event, haz = haz_event,
      mu = mu, nu = nu, n_shape = n_shape
    )
    attr(logl, "gradient") <- grad
  }

  # If Hessian requested (stub for M2)
  if (return_hessian) {
    attr(logl, "hessian") <- matrix(NA, length(theta), length(theta))
  }

  logl
}

#' Score vector (gradient of log-likelihood)
#'
#' Computes the score vector (gradient) of the Weibull log-likelihood w.r.t. all parameters.
#'
#' @param time_lower Optional lower bounds for interval-censored rows.
#' @param time_upper Optional upper bounds for left/interval-censored rows.
#'
#' The log-likelihood is:
#'   L = sum(delta_i * log h(t_i)) - sum(H(t_i))
#'
#' where h is hazard and H is cumulative hazard. Derivatives are:
#'
#' dL/dmu  = sum(delta_i / mu) - (nu / mu) * sum(H(t_i))
#' dL/dnu  = sum(delta_i / nu) + sum(delta_i * log(t_i)) - sum(log(mu * t_i) * H(t_i))
#' dL/dbeta_j = sum(delta_i * x_ij) - sum(H(t_i) * x_ij)  = t(X) %*% (delta - H)
#'
#' @noRd
.hzr_gradient_weibull <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    eta = NULL,
    cumhaz = NULL,
    haz = NULL,
    mu = NULL,
    nu = NULL,
    n_shape = 2) {

  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)

  # Default unit weights -- keep gradient consistent with the LL when weights
  # are omitted.
  if (is.null(weights)) weights <- rep(1, n)

  # Interval/left censoring currently uses a robust numerical gradient fallback.
  # The closed-form derivatives for these terms are deferred to a later pass.
  if (any(status %in% c(-1, 2))) {
    return(.hzr_numeric_grad_weibull(theta, time, status, time_lower,
                                       time_upper, x, weights = weights))
  }

  # Sanity check: if eta, cumhaz, haz not provided, compute them
  if (is.null(eta) || is.null(cumhaz) || is.null(haz)) {
    # Need to recompute from theta
    mu <- theta[1]
    nu <- theta[2]

    if (!is.null(x)) {
      beta <- theta[3:length(theta)]
      eta <- as.numeric(x %*% beta)
    } else {
      eta <- rep(0, n)
    }

    haz <- mu * nu * (time ^ (nu - 1)) * exp(eta)
    cumhaz <- (mu * time) ^ nu * exp(eta)
  }

  # Counting-process entry-time cumulative hazard; H(start) = 0 when no
  # `time_lower` supplied (plain right-censored data).
  start_vec <- if (is.null(time_lower)) rep(0, n) else time_lower
  cumhaz_start <- (mu * start_vec) ^ nu * exp(eta)

  # Weighted building blocks: each per-row contribution below is the
  # unweighted form scaled by `weights`, and the cumulative-hazard term is
  # the epoch difference `H(stop) - H(start)` (degenerates to `H(stop)` when
  # start = 0).
  w_status <- weights * status
  w_cumhaz_net <- weights * (cumhaz - cumhaz_start)

  # ===== Gradient w.r.t. mu (scale parameter) =====
  # dH(t)/dmu = (nu / mu) * H(t), so dL/dmu picks up a matching
  # contribution at start: -(nu/mu) * (H(stop) - H(start)).
  grad[1] <- sum(w_status) / mu - (nu / mu) * sum(w_cumhaz_net)

  # ===== Gradient w.r.t. nu (shape parameter) =====
  # dH(t)/dnu = log(mu*t) * H(t).  When start = 0, H(start) = 0 and the
  # log(mu*start) term is undefined; `ifelse` picks the well-defined 0
  # limit there.
  log_mu_start <- ifelse(start_vec > 0, log(mu * start_vec), 0)
  d_nu_start <- weights * log_mu_start * cumhaz_start
  grad[2] <- sum(w_status) / nu +
             sum(w_status * log(time)) -
             (sum(log(mu * time) * weights * cumhaz) - sum(d_nu_start))

  # ===== Gradient w.r.t. beta (covariate coefficients) =====
  if (n_shape < p && !is.null(x)) {
    # dL/dbeta = t(X) %*% (w * (delta - (H(stop) - H(start))))
    residual <- w_status - w_cumhaz_net
    grad[3:p] <- as.numeric(crossprod(x, residual))
  }

  grad
}

#' @noRd
.hzr_optim_weibull <- function(
    time, status, time_lower = NULL, time_upper = NULL,
    x = NULL, theta_start, weights = NULL, control = list()) {

  # -- Internal reparameterisation --------------------------------------------
  # The original hazard C code parameterises the cumulative hazard as
  #
  #     H(t | x) = exp(alpha + Xbeta) * t^gamma
  #
  # where alpha is a free intercept, gamma = exp(psi) is the shape, and beta enters
  # the same log-hazard-rate space as alpha.  This decouples scale from shape
  # and avoids the degenerate ridge that appears when covariates with large
  # means (e.g. age ~ 62) interact with the (log mu, log nu) parameterisation.
  #
  # We optimise over phi = (alpha, psi, beta_1, ...) -- all unconstrained -- then
  # back-transform to the user-facing (mu, nu, beta) scale:
  #
  #     nu = exp(psi)      mu = exp(alpha / nu)
  #
  # and apply the delta method for the variance-covariance matrix.
  # --------------------------------------------------------------------------

  n_shape <- 2L
  p <- length(theta_start)
  n <- length(time)

  # -- Convert user theta (mu, nu, beta) -> internal (alpha, psi, beta) ------
  mu_start    <- unname(theta_start[1])
  nu_start    <- unname(theta_start[2])
  alpha_start <- nu_start * log(mu_start)   # alpha = nu * log(mu)
  psi_start   <- log(nu_start)              # psi = log(nu)
  beta_start  <- if (p > n_shape) unname(theta_start[(n_shape + 1):p]) else numeric(0)
  phi_start   <- c(alpha_start, psi_start, beta_start)

  # -- Internal likelihood: H(t|x) = exp(alpha + Xbeta) * t^exp(psi) ----------------
  # `weights` is declared as a formal (with a default of NULL) so that when
  # .hzr_optim_generic forwards weights via ..., it matches this formal
  # rather than landing in ... -- which would collide with the explicit
  # `weights =` on the delegated call below.
  .outer_w <- if (is.null(weights)) rep(1, n) else weights

  logl_internal <- function(theta, time, status, time_lower, time_upper,
                            x, weights = NULL, ...) {
    w_use <- if (is.null(weights)) .outer_w else weights
    alpha <- theta[1]
    g     <- exp(theta[2])   # nu = shape
    beta  <- if (p > n_shape) theta[(n_shape + 1):p] else numeric(0)
    eta   <- if (!is.null(x) && length(beta) > 0) {
      as.numeric(x %*% beta)
    } else {
      rep(0, n)
    }

    # Left / interval censoring: delegate to full Weibull likelihood via
    # the natural-scale function, converting params on the fly.
    if (any(status %in% c(-1, 2))) {
      mu <- exp(alpha / g)
      return(.hzr_logl_weibull(c(mu, g, beta), time, status,
                               time_lower, time_upper, x,
                               weights = w_use, ...))
    }

    log_t <- log(pmax(time, .Machine$double.xmin))
    H     <- exp(alpha + eta) * (time ^ g)

    # Counting-process entry-time H(start); zero when no `time_lower` given.
    start_vec <- if (is.null(time_lower)) rep(0, n) else time_lower
    H_start   <- exp(alpha + eta) * (start_vec ^ g)

    idx_event <- status == 1
    idx_right <- status == 0

    ll_event <- if (any(idx_event)) {
      sum(w_use[idx_event] * (alpha + eta[idx_event] + theta[2] +
          (g - 1) * log_t[idx_event] -
          (H[idx_event] - H_start[idx_event])))
    } else {
      0
    }

    ll_right <- if (any(idx_right)) {
      -sum(w_use[idx_right] * (H[idx_right] - H_start[idx_right]))
    } else {
      0
    }

    logl <- ll_event + ll_right
    if (!is.finite(logl)) return(Inf)
    logl
  }

  # -- Internal gradient -----------------------------------------------------
  # `weights` is captured from the enclosing optim frame via `.outer_w`
  # (same trick used by logl_internal above) so the analytic gradient stays
  # consistent with the weighted LL whether or not .hzr_optim_generic
  # forwards the argument via ...
  grad_internal <- function(theta, time, status, time_lower, time_upper, x,
                            weights = NULL, ...) {
    w_use <- if (is.null(weights)) .outer_w else weights
    alpha <- theta[1]
    psi   <- theta[2]
    g     <- exp(psi)
    beta  <- if (p > n_shape) theta[(n_shape + 1):p] else numeric(0)
    eta   <- if (!is.null(x) && length(beta) > 0) as.numeric(x %*% beta) else rep(0, n)

    # Left / interval censoring -> numerical gradient fallback (already weighted
    # because logl_internal propagates weights via the closure).
    if (any(status %in% c(-1, 2))) {
      obj <- function(th) {
        logl_internal(th, time, status, time_lower, time_upper, x,
                      weights = w_use)
      }
      if (requireNamespace("numDeriv", quietly = TRUE))
        return(as.numeric(numDeriv::grad(obj, theta)))
      eps <- 1e-6
      grd <- numeric(p)
      for (i in seq_along(theta)) {
        tp <- tm <- theta
        tp[i] <- tp[i] + eps
        tm[i] <- tm[i] - eps
        grd[i] <- (obj(tp) - obj(tm)) / (2 * eps)
      }
      return(grd)
    }

    log_t <- log(pmax(time, .Machine$double.xmin))
    H     <- exp(alpha + eta) * (time ^ g)

    # H(start) for counting-process rows; zero when time_lower is NULL or
    # start == 0.  `log_t_start` is only used where start > 0 (guarded below)
    # since log(0) = -Inf would otherwise propagate NaNs through the score.
    start_vec <- if (is.null(time_lower)) rep(0, n) else time_lower
    H_start   <- exp(alpha + eta) * (start_vec ^ g)
    log_t_start <- ifelse(start_vec > 0, log(pmax(start_vec, .Machine$double.xmin)), 0)

    grad  <- numeric(p)

    # Weighted building blocks.  Every term below is the unweighted form with
    # `status` -> `w * status` and `H` -> `w * (H(stop) - H(start))`.
    w_status   <- w_use * status
    w_H_net    <- w_use * (H - H_start)
    w_H        <- w_use * H
    w_H_start  <- w_use * H_start

    # dL/dalpha = sum(w * delta) - sum(w * (H(stop) - H(start)))
    grad[1] <- sum(w_status) - sum(w_H_net)

    # dL/dpsi = sum(w * delta) + g * sum(w * delta * log(stop))
    #          - g * [sum(w * H(stop) * log(stop)) - sum(w * H(start) * log(start))]
    grad[2] <- sum(w_status) +
               g * sum(w_status * log_t) -
               g * (sum(w_H * log_t) - sum(w_H_start * log_t_start))

    # dL/dbeta_j = X_j' (w * (delta - (H(stop) - H(start))))
    if (p > n_shape && !is.null(x)) {
      grad[(n_shape + 1):p] <- as.numeric(crossprod(x, w_status - w_H_net))
    }

    grad
  }

  # -- Optimise (all unconstrained) ------------------------------------------
  result <- .hzr_optim_generic(
    logl_fn     = logl_internal,
    gradient_fn = grad_internal,
    time        = time, status = status,
    time_lower  = time_lower, time_upper = time_upper,
    x           = x,
    theta_start = phi_start,
    weights     = weights,
    control     = control,
    use_bounds  = FALSE
  )

  # -- Back-transform to natural scale (mu, nu, beta) ----------------------------
  alpha_hat <- result$par[1]
  psi_hat   <- result$par[2]
  nu_hat    <- exp(psi_hat)
  mu_hat    <- exp(alpha_hat / nu_hat)
  beta_hat  <- if (p > n_shape) result$par[(n_shape + 1):p] else numeric(0)

  result$par <- setNames(c(mu_hat, nu_hat, beta_hat), names(theta_start))

  # Delta method: Cov(mu, nu, beta) = J * Cov(alpha, psi, beta) * J^T
  #
  #   dmu/dalpha = mu / nu             dmu/dpsi = -mu alpha / nu
  #   dnu/dalpha = 0                 dnu/dpsi = nu
  #   dbeta/dalpha = 0                 dbeta/dpsi = 0         dbeta/dbeta = I
  if (is.matrix(result$vcov) && !anyNA(result$vcov)) {
    J <- diag(p)
    J[1, 1] <- mu_hat / nu_hat
    J[1, 2] <- -mu_hat * alpha_hat / nu_hat
    J[2, 1] <- 0
    J[2, 2] <- nu_hat
    result$vcov <- J %*% result$vcov %*% t(J)
  }

  result
}

.hzr_numeric_grad_weibull <- function(theta, time, status, time_lower = NULL,
                                       time_upper = NULL, x = NULL,
                                       weights = NULL) {
  objective <- function(par) {
    .hzr_logl_weibull(
      theta = par,
      time = time,
      status = status,
      time_lower = time_lower,
      time_upper = time_upper,
      x = x,
      weights = weights,
      return_gradient = FALSE
    )
  }

  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(as.numeric(numDeriv::grad(objective, theta)))
  }

  # Fallback central differences if numDeriv is unavailable.
  eps <- 1e-6
  grad <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tm <- theta
    tp[i] <- tp[i] + eps
    tm[i] <- tm[i] - eps
    grad[i] <- (objective(tp) - objective(tm)) / (2 * eps)
  }
  grad
}

# Regression tests for the Weibull event-hazard / cumulative-hazard consistency
# bug: haz_event must equal dH/dt for H = (mu*t)^nu*exp(eta), i.e.
# h = nu*mu^nu*t^(nu-1)*exp(eta) = (nu/t)*H  (Form A, matching C/SAS HAZARD).

test_that(".hzr_logl_weibull event hazard is consistent with its cumulative hazard", {
  set.seed(11)
  n <- 150
  time <- rexp(n, 0.6) + 0.01
  status <- rbinom(n, 1, 0.7)
  mu <- 0.7
  nu <- 1.6
  cumhaz <- (mu * time)^nu                  # Form A cumulative hazard
  haz <- nu * mu^nu * time^(nu - 1)         # correct hazard = dH/dt = (nu/t)*H
  ll_expected <- sum(status * (log(haz) - cumhaz)) - sum((1 - status) * cumhaz)
  ll_actual <- .hzr_logl_weibull(c(mu, nu), time, status)
  expect_equal(ll_actual, ll_expected, tolerance = 1e-8)
})

test_that(".hzr_gradient_weibull matches numDeriv of the (corrected) log-likelihood", {
  skip_if_not_installed("numDeriv")
  set.seed(12)
  n <- 120
  x <- cbind(z = rnorm(n))
  time <- rexp(n, 0.5) + 0.01
  status <- rbinom(n, 1, 0.75)
  theta <- c(mu = 0.8, nu = 1.4, z = 0.3)
  ll <- .hzr_logl_weibull(theta, time, status, x = x, return_gradient = TRUE)
  g_an <- attr(ll, "gradient")
  obj <- function(th) .hzr_logl_weibull(th, time, status, x = x)
  g_nd <- numDeriv::grad(obj, theta)
  expect_equal(g_an, g_nd, tolerance = 1e-5)
})

test_that(".hzr_gradient_weibull is finite for right-censored time = 0 rows", {
  skip_if_not_installed("numDeriv")
  set.seed(41)
  n <- 60
  time <- c(0, rexp(n - 1, 0.5) + 0.05)
  status <- c(0L, rbinom(n - 1, 1, 0.7))
  theta <- c(mu = 0.6, nu = 1.3)
  ll <- .hzr_logl_weibull(theta, time, status, return_gradient = TRUE)
  g_an <- attr(ll, "gradient")
  expect_true(all(is.finite(g_an)))
  obj <- function(th) .hzr_logl_weibull(th, time, status)
  g_nd <- numDeriv::grad(obj, theta)
  expect_equal(g_an, g_nd, tolerance = 1e-5)
})

test_that("gradient_weibull computes correct shape for univariate case", {
  # Simple univariate Weibull (no covariates)
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  theta <- c(mu = 0.5, nu = 1.2)

  # Compute gradient analytically
  grad <- .hzr_gradient_weibull(
    theta = theta,
    time = time,
    status = status,
    x = NULL
  )

  expect_length(grad, 2)
  expect_true(all(is.finite(grad)))
})

test_that("gradient_weibull computes correct shape with covariates", {
  # Weibull with one covariate
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)
  x <- matrix(c(0.5, 1.2, -0.3, 0.8, 1.5), ncol = 1)

  theta <- c(mu = 0.5, nu = 1.2, beta = 0.3)

  # Compute gradient analytically
  grad <- .hzr_gradient_weibull(
    theta = theta,
    time = time,
    status = status,
    x = x
  )

  expect_length(grad, 3)
  expect_true(all(is.finite(grad)))
})

test_that("gradient_weibull matches numerical gradient", {
  # Use numDeriv to verify analytical gradient
  skip_if_not_installed("numDeriv")

  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)
  x <- matrix(c(0.5, 1.2, -0.3, 0.8, 1.5), ncol = 1)

  theta <- c(mu = 0.5, nu = 1.2, beta = 0.3)

  # Analytical gradient
  grad_analytical <- .hzr_gradient_weibull(
    theta = theta,
    time = time,
    status = status,
    x = x
  )

  # Numerical gradient via numDeriv
  objective <- function(th) {
    .hzr_logl_weibull(
      theta = th,
      time = time,
      status = status,
      x = x,
      return_gradient = FALSE
    )
  }

  grad_numerical <- numDeriv::grad(objective, theta)

  # Should match to machine precision (allowing small numerical errors)
  expect_equal(grad_analytical, grad_numerical, tolerance = 1e-4)
})

test_that("optimizer converges using analytical gradient", {
  # Test that optimizer runs with our gradient enabled
  time <- c(1, 2, 3, 4, 5, 6)
  status <- c(1, 0, 1, 1, 0, 1)

  theta_start <- c(mu = 0.3, nu = 1.0)

  result <- .hzr_optim_weibull(
    time = time,
    status = status,
    x = NULL,
    theta_start = theta_start,
    control = list(maxit = 50)
  )

  # Check that optimization ran
  expect_type(result, "list")
  expect_true("par" %in% names(result))
  expect_true("value" %in% names(result))

  # Parameters should be positive and finite
  expect_true(all(result$par > 0))
  expect_true(all(is.finite(result$par)))
})

test_that("optimizer with gradient is faster than without (heuristic)", {
  skip_if_not_installed("numDeriv")

  # Simulate proper Weibull data with censoring
  set.seed(123)
  n <- 50
  event_time <- (-log(runif(n)) / 0.5) ^ (1 / 1.2)
  cens_time <- runif(n, 0, quantile(event_time, 0.8))
  status <- as.integer(event_time <= cens_time)
  time <- pmin(event_time, cens_time)

  theta_start <- c(mu = 0.5, nu = 1.0)

  # Run with gradient (should converge faster)
  result_with_grad <- .hzr_optim_weibull(
    time = time,
    status = status,
    x = NULL,
    theta_start = theta_start,
    control = list(maxit = 1000)
  )

  # Should converge
  expect_equal(result_with_grad$convergence, 0)

  # Final estimates should be reasonable
  expect_true(result_with_grad$par[1] > 0)
  expect_true(result_with_grad$par[2] > 0)
})

# tests/testthat/test-repeating-events.R
# Tests for Surv(start, stop, event) counting-process / epoch-decomposition
# support added in 0.9.4.
#
# The defining invariant of epoch decomposition is:
#
#   L([0, T], event=d)  ==  L([0, t1], 0) * L([t1, T], d)
#
# i.e. splitting a single observation into two contiguous epochs on a time
# s in (0, T) produces the same likelihood as the unsplit observation.
# This holds because each epoch contributes H(stop) - H(start), and the
# telescoping sum recovers H(T).
#
# These tests exercise that invariant on both Weibull and multiphase.
# They also verify the trivial case that Surv(0, t, event) matches
# Surv(t, event) exactly.

make_toy_rc <- function(seed = 91) {
  set.seed(seed)
  n <- 40
  x <- rnorm(n)
  u <- runif(n)
  t <- (-log(u) / exp(0.2 * x)) ^ (1 / 1.1)
  status <- as.integer(t < quantile(t, 0.6))
  data.frame(time = t, status = status, x = x)
}

# ---------------------------------------------------------------------------
# Trivial case: start = 0 matches right-censored Surv
# ---------------------------------------------------------------------------

test_that("Surv(0, t, d) fits identically to Surv(t, d) on Weibull", {
  skip_on_cran()
  df <- make_toy_rc()
  df$start <- 0

  fit_rc <- hazard(
    survival::Surv(time, status) ~ x,
    data = df, dist = "weibull", fit = TRUE
  )
  fit_cp <- hazard(
    survival::Surv(start, time, status) ~ x,
    data = df, dist = "weibull", fit = TRUE
  )

  expect_equal(coef(fit_rc), coef(fit_cp), tolerance = 1e-3)
  expect_equal(fit_rc$fit$objective, fit_cp$fit$objective, tolerance = 1e-4)
})

# ---------------------------------------------------------------------------
# Epoch split invariance: splitting at an interior time preserves LL
# ---------------------------------------------------------------------------

split_epochs <- function(df, split_frac = 0.5) {
  # Each row [time_i, status_i] becomes two rows:
  #   [0,  s_i, 0]            -- left epoch, never an event
  #   [s_i, time_i, status_i] -- right epoch, preserves original status
  # where s_i = split_frac * time_i.
  s <- split_frac * df$time
  pre <- data.frame(
    start = 0, stop = s, status = 0L,
    x = df$x
  )
  post <- data.frame(
    start = s, stop = df$time, status = df$status,
    x = df$x
  )
  rbind(pre, post)
}

# The two split-invariance tests below are the canonical check that
# `H(stop) - H(start)` is actually applied to counting-process rows.
# Wired up in v0.9.6 (Phase 4f).

test_that("splitting each row into two epochs preserves Weibull log-lik", {
  df <- make_toy_rc()
  df_split <- split_epochs(df, split_frac = 0.3)

  theta <- c(mu = 0.8, nu = 1.1, beta = 0.2)

  ll_orig <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta, time = df$time, status = df$status,
    x = matrix(df$x, ncol = 1)
  )

  ll_split <- TemporalHazard:::.hzr_logl_weibull(
    theta = theta,
    time = df_split$stop, status = df_split$status,
    time_lower = df_split$start,
    x = matrix(df_split$x, ncol = 1)
  )

  expect_equal(ll_orig, ll_split, tolerance = 1e-8)
})

test_that("splitting each row into two epochs preserves multiphase log-lik", {
  df <- make_toy_rc()
  df_split <- split_epochs(df, split_frac = 0.4)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
  # Theta layout: [log_mu_e, log_thalf, nu, m, beta_e, log_mu_c, beta_c].
  # Match the phase spec's nu = m = 1 so the decomposition case dispatch
  # hits Case 1 (m > 0, nu > 0); otherwise nu = m = 0 lands in a gap.
  theta <- c(-4, log(0.3), 1, 1, -3, 0.1, 0.05)

  ll_orig <- TemporalHazard:::.hzr_logl_multiphase(
    theta = theta, time = df$time, status = df$status,
    phases = phases,
    covariate_counts = c(early = 1L, constant = 1L),
    x_list = list(early = matrix(df$x, ncol = 1),
                  constant = matrix(df$x, ncol = 1))
  )

  ll_split <- TemporalHazard:::.hzr_logl_multiphase(
    theta = theta,
    time = df_split$stop, status = df_split$status,
    time_lower = df_split$start,
    phases = phases,
    covariate_counts = c(early = 1L, constant = 1L),
    x_list = list(early = matrix(df_split$x, ncol = 1),
                  constant = matrix(df_split$x, ncol = 1))
  )

  expect_equal(ll_orig, ll_split, tolerance = 1e-8)
})

# ---------------------------------------------------------------------------
# Parser plumbing: Surv(start, stop, event) routes via the formula interface
# ---------------------------------------------------------------------------

test_that("Surv(start, stop, event) routes start -> time_lower in formula parser", {
  df <- data.frame(
    start = c(0, 0.5, 1.0),
    stop  = c(1.0, 1.5, 2.0),
    event = c(1, 0, 1)
  )
  parsed <- TemporalHazard:::.hzr_parse_formula(
    survival::Surv(start, stop, event) ~ 1,
    data = df
  )
  expect_equal(parsed$time_lower, df$start)
  expect_equal(parsed$time, df$stop)
  expect_equal(parsed$status, df$event)
})

# ---------------------------------------------------------------------------
# End-to-end fit: counting-process Surv converges and the split dataset
# yields the same MLE as the unsplit.
# ---------------------------------------------------------------------------

test_that("Weibull fit on split-epoch data matches unsplit fit", {
  df <- make_toy_rc()
  df_split <- split_epochs(df, split_frac = 0.35)

  fit_orig <- hazard(
    survival::Surv(time, status) ~ x,
    data = df, dist = "weibull", fit = TRUE
  )
  fit_split <- hazard(
    survival::Surv(start, stop, status) ~ x,
    data = df_split, dist = "weibull", fit = TRUE
  )

  expect_equal(coef(fit_orig), coef(fit_split), tolerance = 1e-2)
  expect_equal(fit_orig$fit$objective, fit_split$fit$objective, tolerance = 1e-3)
})

# ---------------------------------------------------------------------------
# Counting-process data with nonzero starts is now accepted (v0.9.6)
# ---------------------------------------------------------------------------
# The v0.9.5 narrowing rejected `Surv(start, stop, event)` with any
# `start > 0`; v0.9.6 wires `H(stop) - H(start)` into the Weibull and
# multiphase likelihoods, so the guard is gone.

test_that("hazard() accepts counting-process Surv when every start is 0", {
  df <- data.frame(
    start = c(0, 0, 0, 0),
    stop  = c(1.5, 2.0, 3.0, 3.5),
    event = c(1, 0, 1, 0),
    x     = c(0.1, 0.2, 0.3, 0.4)
  )
  # Should not error -- start = 0 is equivalent to right-censored Surv.
  fit <- hazard(
    survival::Surv(start, stop, event) ~ x,
    data = df, dist = "weibull", fit = FALSE
  )
  expect_s3_class(fit, "hazard")
})

# diagnostics.R -- Model validation and goodness-of-fit utilities
#
# Implements SAS HAZARD macro equivalents for calibration and validation.

#' @importFrom stats predict
NULL

#' Decile-of-risk calibration
#'
#' Partition observations into groups (default 10) by predicted risk and
#' compare observed vs. expected event counts in each group.  Good
#' calibration means the two track each other across the risk spectrum.
#'
#' This reproduces the SAS `deciles.hazard.sas` macro. **All** subjects are
#' ranked by predicted survival at the horizon `time` and split into equal-sized
#' risk groups. Within each group the **expected** event count is the sum of
#' each subject's predicted cumulative hazard at its *own* follow-up time, and
#' the **observed** count is its number of events; under conservation of events
#' the group totals sum to the total observed events. The horizon therefore only
#' stratifies subjects into risk groups -- it does not restrict or exclude any
#' subject, and the expected/observed totals are independent of it.
#'
#' @param object A fitted `hazard` object (with `fit = TRUE`).
#' @param time Numeric scalar: the horizon at which predicted survival is used
#'   to **rank subjects into risk groups** (e.g. `time = 12` ranks by 12-month
#'   predicted survival). It does not restrict the event/expected counts, which
#'   are accumulated over each subject's full follow-up.
#' @param groups Integer: number of risk groups (default 10 for deciles).
#' @param status Optional numeric vector of event indicators (1 = event,
#'   0 = censored).
#'   If `NULL` (default), extracted from the fitted object's stored data.
#' @param event_time Optional numeric vector of observed event/censoring
#'   times.  If `NULL` (default), extracted from the fitted object.
#'
#' @return A data frame with one row per risk group and columns:
#' \describe{
#'   \item{group}{Integer group label (1 = lowest risk, ranked by predicted
#'     survival at \code{time}).}
#'   \item{n}{Number of observations in the group.}
#'   \item{events}{Observed event count in the group (all events over
#'     follow-up).}
#'   \item{expected}{Expected event count: the sum of each subject's predicted
#'     cumulative hazard at its own follow-up time.}
#'   \item{observed_rate}{Observed event rate (events / n).}
#'   \item{expected_rate}{Expected event rate (expected / n).}
#'   \item{chi_sq}{Chi-square contribution: (events - expected)^2 /
#'     expected.}
#'   \item{p_value}{Upper-tail p-value from the chi-square test for
#'     this group (1 df).}
#'   \item{mean_survival}{Mean predicted survival probability at the horizon
#'     in the group.}
#'   \item{mean_cumhaz}{Mean predicted cumulative hazard at follow-up in the
#'     group.}
#' }
#'
#' An attribute `"overall"` is attached with the overall chi-square
#' statistic, degrees of freedom, and p-value.
#'
#' @examples
#' \donttest{
#' data(avc)
#' avc <- na.omit(avc)
#' fit <- hazard(
#'   survival::Surv(int_dead, dead) ~ age + mal,
#'   data  = avc,
#'   dist  = "weibull",
#'   theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
#'   fit   = TRUE
#' )
#' cal <- hzr_deciles(fit, time = 120)
#' print(cal)
#' }
#'
#' @seealso [predict.hazard()] for the prediction types used internally.
#' @export
hzr_deciles <- function(object, time, groups = 10L,
                        status = NULL, event_time = NULL) {
  if (!inherits(object, "hazard")) {
    stop("'object' must be a fitted hazard object.", call. = FALSE)
  }

  if (is.null(object$fit$theta) ||
      (is.logical(object$fit$converged) && is.na(object$fit$converged))) {
    stop("'object' has no fitted parameters. Refit with fit = TRUE.",
         call. = FALSE)
  }
  if (!is.numeric(time) || length(time) != 1L || time <= 0) {
    stop("'time' must be a single positive number.", call. = FALSE)
  }
  groups <- as.integer(groups)
  if (groups < 2L) {
    stop("'groups' must be at least 2.", call. = FALSE)
  }


  # --- Extract observed data ------------------------------------------------
  if (is.null(status)) {
    status <- object$data$status
  }
  if (is.null(event_time)) {
    event_time <- object$data$time
  }
  n_obs <- length(status)
  if (length(event_time) != n_obs) {
    stop("'status' and 'event_time' must have the same length.",
         call. = FALSE)
  }
  if (any(!is.finite(event_time)) || any(event_time < 0)) {
    stop("'event_time' must be finite and non-negative.", call. = FALSE)
  }
  if (any(!is.finite(status)) || any(!status %in% c(0, 1))) {
    stop("'status' must be coded as 0/1 with finite values.", call. = FALSE)
  }

  n_model <- length(object$data$time)
  if (n_obs != n_model) {
    stop("'status'/'event_time' lengths must match the fitted data length (",
         n_model, ").", call. = FALSE)
  }

  # --- Per-observation predicted cumulative hazard --------------------------
  # SAS %DECILES method (Blackstone/Naftel HAZARD): the "expected events" in a
  # group are the sum of predicted cumulative hazard at each subject's OWN
  # follow-up time, so the total expected equals the observed event count under
  # conservation of events.  Risk grouping is by predicted survival at the
  # calibration horizon `time`; the horizon only stratifies subjects into risk
  # groups -- it does not restrict the event or expected counts, and all
  # subjects are included.
  cumhaz_at <- function(times) {
    if (identical(object$spec$dist, "multiphase")) {
      phases <- object$fit$phases
      if (is.null(phases)) phases <- object$spec$phases
      .hzr_multiphase_cumhaz(times, object$fit$theta, phases,
                             object$fit$covariate_counts, object$fit$x_list,
                             per_phase = FALSE)
    } else if (!is.null(object$data$x) && ncol(object$data$x) > 0) {
      nd <- as.data.frame(object$data$x)
      nd$time <- times
      predict(object, newdata = nd, type = "cumulative_hazard")
    } else {
      predict(object, newdata = data.frame(time = times),
              type = "cumulative_hazard")
    }
  }

  cumhaz_fu    <- cumhaz_at(event_time)        # expected-event contribution
  cumhaz_hor   <- cumhaz_at(rep(time, n_obs))  # risk grouping at the horizon
  survival_hor <- exp(-cumhaz_hor)
  observed     <- as.integer(status == 1)
  n_included   <- n_obs
  n_excluded   <- 0L

  if (groups > n_obs) {
    stop("'groups' must be <= number of observations (", n_obs, ").",
         call. = FALSE)
  }

  # --- Rank into equal-sized groups by predicted risk at the horizon --------
  # Rank by cumulative hazard at the horizon, ascending, so group 1 = lowest
  # risk (highest predicted survival).  This is the same partition SAS PROC
  # RANK GROUPS= produces on predicted survival, with the opposite label order
  # (SAS _DECILE_ 0 = highest risk).  Equal-count bins via PROC RANK's floor
  # rule.  Ties are broken by order of appearance ("first") so that an
  # intercept-only model (all predictions identical) still yields equal-sized
  # groups rather than collapsing into one; for models with distinct
  # predictions the tie rule is irrelevant.
  ranks <- rank(cumhaz_hor, ties.method = "first")
  group <- as.integer(floor(groups * (ranks - 1) / n_obs)) + 1L
  group[group > groups] <- groups

  # --- Aggregate by group ---------------------------------------------------
  result <- data.frame(
    group = seq_len(groups),
    n = integer(groups),
    events = numeric(groups),
    expected = numeric(groups),
    observed_rate = numeric(groups),
    expected_rate = numeric(groups),
    chi_sq = numeric(groups),
    p_value = numeric(groups),
    mean_survival = numeric(groups),
    mean_cumhaz = numeric(groups)
  )

  for (g in seq_len(groups)) {
    idx <- which(group == g)
    ng <- length(idx)
    obs_events <- sum(observed[idx] == 1)
    exp_events <- sum(cumhaz_fu[idx])

    result$n[g] <- ng
    result$events[g] <- obs_events
    result$expected[g] <- exp_events
    result$observed_rate[g] <- if (ng > 0) obs_events / ng else NA_real_
    result$expected_rate[g] <- if (ng > 0) exp_events / ng else NA_real_
    result$mean_survival[g] <- if (ng > 0) mean(survival_hor[idx]) else NA_real_
    result$mean_cumhaz[g] <- if (ng > 0) mean(cumhaz_fu[idx]) else NA_real_

    # Per-group chi-square: (O - E)^2 / E
    if (exp_events > 0) {
      result$chi_sq[g] <- (obs_events - exp_events)^2 / exp_events
      # Upper-tail p-value from chi-square with 1 df
      result$p_value[g] <- stats::pchisq(result$chi_sq[g], df = 1,
                                          lower.tail = FALSE)
    } else {
      result$chi_sq[g] <- NA_real_
      result$p_value[g] <- NA_real_
    }
  }

  # --- Overall chi-square ---------------------------------------------------
  # Only groups with a positive expected count contribute (a zero-expected
  # group has an undefined (O-E)^2/E term and NA chi_sq). Clamp df at 0 so
  # degenerate inputs (<=1 contributing group) never yield a negative df.
  valid <- result$expected > 0 & !is.na(result$chi_sq)
  overall_chi_sq <- sum(result$chi_sq[valid])
  overall_df <- max(0L, sum(valid) - 1L)
  overall_p <- if (overall_df > 0) {
    stats::pchisq(overall_chi_sq, df = overall_df, lower.tail = FALSE)
  } else {
    NA_real_
  }

  attr(result, "overall") <- list(
    chi_sq = overall_chi_sq,
    df = overall_df,
    p_value = overall_p,
    time = time,
    groups = groups,
    total_events = sum(observed == 1),
    total_expected = sum(cumhaz_fu),
    n_included = n_included,
    n_excluded = n_excluded
  )

  class(result) <- c("hzr_deciles", "data.frame")
  result
}

#' Print method for hzr_deciles
#'
#' @param x An `hzr_deciles` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @return The object \code{x} of class \code{c("hzr_deciles", "data.frame")},
#'   invisibly. The data frame has one row per risk group and columns:
#'   \code{group} (integer group index, 1 = lowest risk),
#'   \code{n} (group size),
#'   \code{events} (observed event count),
#'   \code{expected} (expected event count from model predictions),
#'   \code{observed_rate}, \code{expected_rate} (events / n),
#'   \code{chi_sq} (per-group (O-E)^2/E contribution),
#'   \code{p_value} (1-df chi-square upper-tail p),
#'   \code{mean_survival}, \code{mean_cumhaz} (mean predicted values in group).
#'   An \code{"overall"} attribute contains the omnibus chi-square test
#'   (fields: \code{chi_sq}, \code{df}, \code{p_value}, \code{time},
#'   \code{groups}, \code{total_events}, \code{total_expected},
#'   \code{n_included}, \code{n_excluded}).
#' @export
print.hzr_deciles <- function(x, digits = 3, ...) {
  ov <- attr(x, "overall")
  cat("Decile-of-risk calibration (risk grouped at time =", ov$time, ")\n")
  if (!is.null(ov$n_included)) {
    cat(ov$n_included, "subjects, all included.\n")
  }
  cat(ov$groups, "groups,", ov$total_events, "observed events,",
      signif(ov$total_expected, digits), "expected\n\n")

  # Format for display
  display <- x
  display$expected <- signif(display$expected, digits)
  display$observed_rate <- signif(display$observed_rate, digits)
  display$expected_rate <- signif(display$expected_rate, digits)
  display$chi_sq <- signif(display$chi_sq, digits)
  display$p_value <- signif(display$p_value, digits)
  display$mean_survival <- signif(display$mean_survival, digits)
  display$mean_cumhaz <- signif(display$mean_cumhaz, digits)
  class(display) <- "data.frame"
  print(display, row.names = FALSE)

  cat("\nOverall: chi-sq =", signif(ov$chi_sq, digits),
      "on", ov$df, "df, p =",
      format.pval(ov$p_value, digits = digits), "\n")

  invisible(x)
}


# =========================================================================
# hzr_gof -- Observed vs. expected goodness-of-fit
# =========================================================================

#' Goodness-of-fit: observed vs. predicted events
#'
#' Compare a fitted hazard model against the nonparametric Kaplan-Meier
#' estimate by computing observed and expected (parametric) event counts
#' at each distinct event time.  This is the R equivalent of the SAS
#' `hazplot.sas` macro and implements the conservation-of-events
#' diagnostic.
#'
#' At each observed event time the function computes:
#' \itemize{
#'   \item The Kaplan-Meier survival and cumulative hazard.
#'   \item The parametric survival and cumulative hazard from the fitted
#'     model (and per-phase components for multiphase models).
#'   \item Cumulative observed events vs. cumulative expected events
#'     (sum of individual cumulative hazards for those exiting the risk
#'     set at each time).
#'   \item The running residual (expected minus observed).
#' }
#'
#' Perfect model fit implies the expected and observed event counts track
#' each other (residual near zero).  This is the conservation-of-events
#' principle.
#'
#' @param object A fitted `hazard` object (with `fit = TRUE`).
#' @param time_grid Optional numeric vector of time points at which to
#'   evaluate the parametric model.
#'   If `NULL` (default), uses the sorted unique event times from the
#'   fitted data.
#'
#' @return A data frame with one row per time point and columns:
#' \describe{
#'   \item{time}{Evaluation time.}
#'   \item{n_risk}{Number at risk (Kaplan-Meier).}
#'   \item{n_event}{Number of events at this time.}
#'   \item{n_censor}{Number censored at this time.}
#'   \item{km_surv}{Kaplan-Meier survival estimate.}
#'   \item{km_cumhaz}{Kaplan-Meier cumulative hazard
#'     (\eqn{-\log(\text{km\_surv})}).}
#'   \item{par_surv}{Parametric survival from the fitted model.}
#'   \item{par_cumhaz}{Parametric cumulative hazard.}
#'   \item{cum_observed}{Cumulative observed events to this time.}
#'   \item{cum_expected}{Cumulative expected events (sum of individual
#'     cumulative hazards for observations exiting the risk set).}
#'   \item{residual}{Expected minus observed
#'     (\code{cum_expected - cum_observed}).}
#' }
#'
#' For multiphase models, additional columns are appended for each
#' phase: \code{par_cumhaz_<phase>}.
#'
#' An attribute `"summary"` is attached with scalar diagnostics:
#' total observed events, total expected events, and the final residual.
#'
#' @examples
#' \donttest{
#' data(avc)
#' avc <- na.omit(avc)
#' fit <- hazard(
#'   survival::Surv(int_dead, dead) ~ age + mal,
#'   data  = avc,
#'   dist  = "weibull",
#'   theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
#'   fit   = TRUE
#' )
#' gof <- hzr_gof(fit)
#' print(gof)
#'
#' # Plot observed vs expected events
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(gof, aes(x = time)) +
#'     geom_line(aes(y = cum_observed), colour = "#D55E00") +
#'     geom_line(aes(y = cum_expected), colour = "#0072B2") +
#'     labs(x = "Time", y = "Cumulative events") +
#'     theme_minimal()
#' }
#' }
#'
#' @seealso [hzr_deciles()] for decile-of-risk calibration,
#'   [predict.hazard()] for prediction types.
#' @export
hzr_gof <- function(object, time_grid = NULL) {
  if (!inherits(object, "hazard")) {
    stop("'object' must be a fitted hazard object.", call. = FALSE)
  }
  if (is.null(object$fit$theta) ||
      (is.logical(object$fit$converged) && is.na(object$fit$converged))) {
    stop("'object' has no fitted parameters. Refit with fit = TRUE.",
         call. = FALSE)
  }

  # --- Extract observed data ------------------------------------------------
  obs_time   <- object$data$time
  obs_status <- object$data$status
  n_total    <- length(obs_time)

  if (length(obs_status) != n_total) {
    stop("Stored 'time' and 'status' vectors must have the same length.",
         call. = FALSE)
  }
  if (any(!is.finite(obs_time)) || any(obs_time < 0)) {
    stop("Stored event/censoring times must be finite and non-negative.",
         call. = FALSE)
  }
  if (any(!is.finite(obs_status)) || any(!obs_status %in% c(0, 1))) {
    stop("Stored status must be coded as 0/1 with finite values.",
         call. = FALSE)
  }

  # --- Kaplan-Meier via survival::survfit -----------------------------------
  km_fit <- survival::survfit(survival::Surv(obs_time, obs_status) ~ 1)

  # survfit output: time, n.risk, n.event, n.censor, surv
  km_times   <- km_fit$time
  km_n_risk  <- km_fit$n.risk
  km_n_event <- km_fit$n.event
  km_n_censor <- km_fit$n.censor
  km_surv    <- km_fit$surv

  # --- Decide time grid -----------------------------------------------------
  if (is.null(time_grid)) {
    time_grid <- km_times
  }

  # --- Parametric predictions at each time point ----------------------------
  is_multiphase <- (object$spec$dist == "multiphase")

  if (!is.null(object$data$x) && ncol(object$data$x) > 0) {
    # For covariate models, evaluate at covariate means (baseline patient)
    x_means <- colMeans(object$data$x)
    nd <- as.data.frame(t(x_means))
    nd <- nd[rep(1, length(time_grid)), , drop = FALSE]
    nd$time <- time_grid
  } else {
    nd <- data.frame(time = time_grid)
  }

  par_cumhaz <- predict(object, newdata = nd, type = "cumulative_hazard")

  # Phase decomposition for multiphase models
  phase_cumhaz <- NULL
  if (is_multiphase) {
    decomp <- predict(object, newdata = nd, type = "cumulative_hazard",
                      decompose = TRUE)
    # decomp is a matrix; first column is "total", rest are phase names
    phase_cols <- colnames(decomp)[colnames(decomp) != "total"]
    phase_cumhaz <- as.data.frame(decomp[, phase_cols, drop = FALSE])
  }

  par_surv <- exp(-par_cumhaz)

  # --- Interpolate KM at the time grid --------------------------------------
  # Use stepfun-style interpolation for KM (right-continuous)
  km_surv_at_grid <- stats::approx(
    x = c(0, km_times), y = c(1, km_surv),
    xout = time_grid, method = "constant", f = 0, rule = 2
  )$y
  km_cumhaz_at_grid <- -log(pmax(km_surv_at_grid, .Machine$double.xmin))

  # Interpolate n.risk, n.event, n.censor at grid times
  # For event counts, sum events at matching times; 0 otherwise
  km_n_risk_grid <- stats::approx(
    x = c(0, km_times), y = c(n_total, km_n_risk),
    xout = time_grid, method = "constant", f = 0, rule = 2
  )$y
  km_n_event_grid <- rep(0, length(time_grid))
  km_n_censor_grid <- rep(0, length(time_grid))
  for (i in seq_along(km_times)) {
    match_idx <- which(abs(time_grid - km_times[i]) < .Machine$double.eps * 100)
    if (length(match_idx) > 0) {
      km_n_event_grid[match_idx[1]] <- km_n_event[i]
      km_n_censor_grid[match_idx[1]] <- km_n_censor[i]
    }
  }

  # --- Conservation of Events accounting ------------------------------------
  # At each event time, accumulate:
  #   cum_observed: running sum of observed events
  #   cum_expected: running sum of individual cumulative hazards for
  #                 observations exiting the risk set (events + censored)
  #
  # The expected events for observations leaving at time t is:
  #   (n_event + n_censor) * parametric_cumhaz(t)
  # This is the SAS hazplot approach: total * _CUMHAZ at that interval.

  cum_observed <- cumsum(km_n_event_grid)
  interval_expected <- (km_n_event_grid + km_n_censor_grid) * par_cumhaz
  cum_expected <- cumsum(interval_expected)
  residual <- cum_expected - cum_observed

  # --- Assemble result ------------------------------------------------------
  result <- data.frame(
    time         = time_grid,
    n_risk       = km_n_risk_grid,
    n_event      = km_n_event_grid,
    n_censor     = km_n_censor_grid,
    km_surv      = km_surv_at_grid,
    km_cumhaz    = km_cumhaz_at_grid,
    par_surv     = par_surv,
    par_cumhaz   = par_cumhaz,
    cum_observed = cum_observed,
    cum_expected = cum_expected,
    residual     = residual
  )

  # Add phase columns for multiphase
  if (!is.null(phase_cumhaz)) {
    for (ph in names(phase_cumhaz)) {
      result[[paste0("par_cumhaz_", ph)]] <- phase_cumhaz[[ph]]
    }
  }

  # Summary diagnostics
  attr(result, "summary") <- list(
    total_observed = cum_observed[length(cum_observed)],
    total_expected = cum_expected[length(cum_expected)],
    final_residual = residual[length(residual)],
    dist = object$spec$dist,
    n = n_total
  )

  class(result) <- c("hzr_gof", "data.frame")
  result
}

#' Print method for hzr_gof
#'
#' @param x An `hzr_gof` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @return The object \code{x} of class \code{c("hzr_gof", "data.frame")},
#'   invisibly. The data frame has one row per time point and columns:
#'   \code{time}, \code{n_risk}, \code{n_event}, \code{n_censor},
#'   \code{km_surv} (Kaplan-Meier survival), \code{km_cumhaz},
#'   \code{par_surv} (parametric survival), \code{par_cumhaz},
#'   \code{cum_observed} (cumulative observed events),
#'   \code{cum_expected} (cumulative expected events from model),
#'   \code{residual} (cum_expected - cum_observed).
#'   Multiphase models additionally include \code{par_cumhaz_<phase>} columns
#'   for per-phase cumulative hazard contributions.
#'   A \code{"summary"} attribute contains scalar diagnostics:
#'   \code{total_observed}, \code{total_expected}, \code{final_residual},
#'   \code{dist}, \code{n}.
#' @export
print.hzr_gof <- function(x, digits = 3, ...) {
  s <- attr(x, "summary")
  cat("Goodness-of-fit: observed vs. expected events\n")
  cat("Distribution:", s$dist, " | n =", s$n, "\n\n")
  cat("Total observed events:", s$total_observed, "\n")
  cat("Total expected events:", round(s$total_expected, digits), "\n")
  cat("Final residual (E - O):", round(s$final_residual, digits), "\n")
  cat("Conservation ratio (E/O):",
      round(s$total_expected / max(s$total_observed, 1), digits), "\n")
  cat("\nUse plot columns: time, km_surv, par_surv, cum_observed,",
      "cum_expected, residual\n")
  invisible(x)
}


# =========================================================================
# hzr_kaplan -- Kaplan-Meier with exact logit confidence limits
# =========================================================================

#' Kaplan-Meier survival with exact logit confidence limits
#'
#' Compute the product-limit (Kaplan-Meier) survival estimate with
#' logit-transformed confidence limits that respect the \eqn{[0, 1]}
#' boundary.
#' This is the R equivalent of the SAS `kaplan.sas` macro.
#'
#' The standard Greenwood confidence interval can exceed \eqn{[0, 1]} in the
#' tails. The logit-transformed interval avoids this by working on the
#' log-odds scale:
#'
#' \deqn{
#'   \text{CL}_{\text{lower}} = S / \bigl(S + (1-S)\,
#'   \exp(z_\alpha\,\text{SI})\bigr)
#' }
#' \deqn{
#'   \text{CL}_{\text{upper}} = S / \bigl(S + (1-S)\,
#'   \exp(-z_\alpha\,\text{SI})\bigr)
#' }
#'
#' where \eqn{\text{SI} = \sqrt{V_P - 1} / (1 - S)}, \eqn{V_P} is the
#' cumulative Greenwood variance product, and \eqn{z_\alpha} is the
#' normal quantile for the requested confidence level.
#'
#' @param time Numeric vector of follow-up times.
#' @param status Numeric event indicator (1 = event, 0 = censored).
#' @param conf_level Confidence level for the interval (default 0.95).
#'   The SAS default of 0.68268948 corresponds to a 1-SD interval.
#' @param event_only Logical; if `TRUE` (default), only return rows at
#'   event times (where `n_event > 0`).
#'   If `FALSE`, return rows at all times reported by
#'   `survival::survfit()` (events and censoring times).
#'
#' @return A data frame with one row per time point and columns:
#' \describe{
#'   \item{time}{Event/censoring time.}
#'   \item{n_risk}{Number at risk at start of interval.}
#'   \item{n_event}{Number of events at this time.}
#'   \item{n_censor}{Number censored at this time.}
#'   \item{survival}{Kaplan-Meier survival estimate.}
#'   \item{std_err}{Standard error of survival (Greenwood).}
#'   \item{cl_lower}{Lower confidence limit (logit-transformed).}
#'   \item{cl_upper}{Upper confidence limit (logit-transformed).}
#'   \item{cumhaz}{Cumulative hazard \eqn{= -\log(S)}.}
#'   \item{hazard}{Interval hazard rate
#'     \eqn{= \log(S_{t-1} / S_t) / \Delta t}.}
#'   \item{density}{Probability density estimate
#'     \eqn{= (S_{t-1} - S_t) / \Delta t}.}
#'   \item{life}{Restricted mean survival time (area under curve to
#'     this time).}
#' }
#'
#' @examples
#' data(cabgkul)
#' km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)
#' head(km)
#'
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(km, aes(time)) +
#'     geom_step(aes(y = survival * 100)) +
#'     geom_ribbon(aes(ymin = cl_lower * 100, ymax = cl_upper * 100),
#'                 stat = "identity", alpha = 0.2) +
#'     labs(x = "Months", y = "Survival (%)") +
#'     theme_minimal()
#' }
#' }
#'
#' @references
#' Kaplan EL, Meier P (1958). Nonparametric estimation from incomplete
#' observations. *J Am Stat Assoc* 53(282):457--481.
#' \doi{10.1080/01621459.1958.10501452}
#'
#' Greenwood M (1926). The natural duration of cancer. *Reports on Public
#' Health and Medical Subjects* 33:1--26.
#' @seealso [hzr_gof()] for parametric vs. nonparametric comparison.
#' @export
hzr_kaplan <- function(time, status, conf_level = 0.95,
                       event_only = TRUE) {
  if (!is.numeric(time) || !is.numeric(status)) {
    stop("'time' and 'status' must be numeric vectors.", call. = FALSE)
  }
  if (length(time) != length(status)) {
    stop("'time' and 'status' must have the same length.", call. = FALSE)
  }
  if (conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1.", call. = FALSE)
  }

  z_alpha <- stats::qnorm(0.5 + 0.5 * conf_level)

  # Use survival::survfit for the core KM computation
  km_fit <- survival::survfit(survival::Surv(time, status) ~ 1)

  n_times <- length(km_fit$time)

  # --- Build output vectors at each event/censoring time --------------------
  km_time    <- km_fit$time
  km_n_risk  <- km_fit$n.risk
  km_n_event <- km_fit$n.event
  km_n_censor <- km_fit$n.censor
  km_surv    <- km_fit$surv

  # --- Greenwood variance product and exact logit CL -----------------------
  # VAR_PROD = prod_i [1/(n_i - d_i) - 1/n_i + 1] for all event times
  # The product is cumulative: each time contributes a multiplicative factor.
  var_prod <- rep(1.0, n_times)
  for (i in seq_len(n_times)) {
    ni <- km_n_risk[i]
    di <- km_n_event[i]
    if (ni > 0 && (ni - di) > 0 && di > 0) {
      factor_i <- 1.0 / (ni - di) - 1.0 / ni + 1.0
      if (i == 1L) {
        var_prod[i] <- factor_i
      } else {
        var_prod[i] <- var_prod[i - 1L] * factor_i
      }
    } else if (i > 1L) {
      var_prod[i] <- var_prod[i - 1L]
    }
  }

  # Standard error (Greenwood)
  var_exact <- km_surv^2 * pmax(var_prod - 1, 0)
  std_err <- sqrt(var_exact)

  # Logit-transformed confidence limits (SAS SI_EXACT formula)
  si_exact <- rep(0.0, n_times)
  idx_valid <- km_surv < 1 & km_surv > 0
  si_exact[idx_valid] <- sqrt(pmax(var_prod[idx_valid] - 1, 0)) /
    (1 - km_surv[idx_valid])

  cl_lower <- km_surv / (km_surv + (1 - km_surv) *
                            exp(z_alpha * si_exact))
  cl_upper <- km_surv / (km_surv + (1 - km_surv) *
                            exp(-z_alpha * si_exact))

  # Edge cases: if S = 1 or S = 0, CL = S

  cl_lower[km_surv >= 1] <- 1
  cl_upper[km_surv >= 1] <- 1
  cl_lower[km_surv <= 0] <- 0
  cl_upper[km_surv <= 0] <- 0

  # --- Cumulative hazard -----------------------------------------------------
  cumhaz <- -log(pmax(km_surv, .Machine$double.xmin))

  # --- Interval hazard and density ------------------------------------------
  lag_surv <- c(1, km_surv[-n_times])
  lag_time <- c(0, km_time[-n_times])
  delta_t <- km_time - lag_time

  hazard <- rep(NA_real_, n_times)
  density <- rep(NA_real_, n_times)
  idx_haz <- km_n_event > 0 & delta_t > 0 & km_surv > 0
  hazard[idx_haz] <- log(lag_surv[idx_haz] / km_surv[idx_haz]) /
    delta_t[idx_haz]
  density[idx_haz] <- (lag_surv[idx_haz] - km_surv[idx_haz]) /
    delta_t[idx_haz]

  # --- Restricted mean survival (life integral) -----------------------------
  # KM survival is a right-continuous step function.  RMST is accumulated
  # as the sum of rectangle areas: each interval contributes dt * S(t-1).
  life <- rep(0, n_times)
  lag_life <- 0
  for (i in seq_len(n_times)) {
    life[i] <- lag_life + delta_t[i] * lag_surv[i]
    lag_life <- life[i]
  }

  # --- Assemble result ------------------------------------------------------
  result <- data.frame(
    time      = km_time,
    n_risk    = km_n_risk,
    n_event   = km_n_event,
    n_censor  = km_n_censor,
    survival  = km_surv,
    std_err   = std_err,
    cl_lower  = cl_lower,
    cl_upper  = cl_upper,
    cumhaz    = cumhaz,
    hazard    = hazard,
    density   = density,
    life      = life
  )

  if (event_only) {
    result <- result[result$n_event > 0, , drop = FALSE]
    rownames(result) <- NULL
  }

  class(result) <- c("hzr_kaplan", "data.frame")
  result
}

#' Print method for hzr_kaplan
#'
#' @param x An `hzr_kaplan` object.
#' @param digits Number of decimal places for formatting.
#' @param n Maximum rows to print (default 20).
#' @param ... Additional arguments (ignored).
#' @return The object \code{x} of class \code{c("hzr_kaplan", "data.frame")},
#'   invisibly. The data frame has one row per event time (or all times when
#'   \code{event_only = FALSE}) and columns:
#'   \code{time} (follow-up time),
#'   \code{n_risk} (number at risk),
#'   \code{n_event} (events in interval),
#'   \code{n_censor} (censored observations in interval),
#'   \code{survival} (Kaplan-Meier survival estimate),
#'   \code{std_err} (Greenwood standard error on log-hazard scale),
#'   \code{cl_lower}, \code{cl_upper} (logit-transformed confidence limits
#'   on the survival scale),
#'   \code{cumhaz} (Nelson-Aalen cumulative hazard),
#'   \code{hazard} (interval hazard estimate),
#'   \code{density} (estimated event density),
#'   \code{life} (life-table life expectancy contribution).
#' @export
print.hzr_kaplan <- function(x, digits = 4, n = 20, ...) {
  cat("Kaplan-Meier estimate with logit confidence limits\n")
  total_events <- sum(x$n_event)
  cat("Events:", total_events, " | Time points:", nrow(x), "\n")
  if (nrow(x) > 0) {
    cat("Survival range:",
        round(min(x$survival), digits), "to",
        round(max(x$survival), digits), "\n")
    cat("RMST at last event:", round(x$life[nrow(x)], digits), "\n\n")
  }
  display <- x
  display$survival <- round(display$survival, digits)
  display$std_err <- round(display$std_err, digits)
  display$cl_lower <- round(display$cl_lower, digits)
  display$cl_upper <- round(display$cl_upper, digits)
  display$cumhaz <- round(display$cumhaz, digits)
  display$hazard <- round(display$hazard, digits)
  display$density <- round(display$density, digits)
  display$life <- round(display$life, digits)
  class(display) <- "data.frame"
  if (nrow(display) > n) {
    print(utils::head(display, n), row.names = FALSE)
    cat("... (", nrow(display) - n, " more rows)\n")
  } else {
    print(display, row.names = FALSE)
  }
  invisible(x)
}


# =========================================================================
# hzr_calibrate -- Variable calibration (logit / Gompertz / Cox transform)
# =========================================================================

#' Calibrate a continuous variable against an outcome
#'
#' Group a continuous covariate into quantile bins, compute the event
#' probability (or hazard rate) per bin, and apply a link transform
#' (logit, Gompertz, or Cox).
#' This is the R equivalent of the SAS `logit.sas` and `logitgr.sas`
#' macros.
#'
#' Use this function before model entry to assess whether the covariate
#' relationship with the outcome is approximately linear on the link
#' scale. If the transformed probabilities are roughly linear against
#' the group means, the covariate can enter the model untransformed.
#' Curvature suggests a transformation (log, quadratic) may improve fit.
#'
#' @param x Numeric vector: the continuous covariate to calibrate.
#' @param event Numeric vector: event indicator (1 = event, 0 = no event).
#' @param groups Integer: number of quantile bins (default 10).
#' @param by Optional factor or character vector for stratified calibration
#'   (SAS `logitgr.sas` functionality). If provided, calibration is
#'   computed within each stratum. Default `NULL` (no stratification).
#' @param link Character: transform to apply to event probabilities.
#'   One of `"logit"` (default), `"gompertz"` (complementary log-log),
#'   or `"cox"`.
#' @param time Optional numeric vector: follow-up time, required when
#'   `link = "cox"`. The Cox link computes
#'   \eqn{\log(\text{events} / \sum \text{time})} (constant hazard rate).
#'
#' @return A data frame with one row per group (or per group-by-stratum
#'   combination) and columns:
#' \describe{
#'   \item{group}{Integer group label.}
#'   \item{by}{Stratum level (only present when `by` is provided).}
#'   \item{n}{Number of observations in the group.}
#'   \item{events}{Number of events.}
#'   \item{mean}{Mean of `x` within the group.}
#'   \item{min}{Minimum of `x` within the group.}
#'   \item{max}{Maximum of `x` within the group.}
#'   \item{prob}{Event probability (events / n), or for Cox link:
#'     events / sum(time).}
#'   \item{link_value}{Transformed probability on the chosen link scale.}
#' }
#'
#' @examples
#' data(avc)
#' avc <- na.omit(avc)
#'
#' # Logit calibration of age
#' cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)
#' print(cal)
#'
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(cal, aes(mean, link_value)) +
#'     geom_point(size = 3) +
#'     geom_smooth(method = "lm", formula = y ~ x, se = FALSE,
#'                 linetype = "dashed") +
#'     labs(x = "Age at repair (months)", y = "Logit(P(death))") +
#'     theme_minimal()
#' }
#' }
#'
#' @seealso [hzr_deciles()] for model-based calibration after fitting.
#' @export
hzr_calibrate <- function(x, event, groups = 10L, by = NULL,
                          link = c("logit", "gompertz", "cox"),
                          time = NULL) {
  link <- match.arg(link)

  if (!is.numeric(x) || !is.numeric(event)) {
    stop("'x' and 'event' must be numeric vectors.", call. = FALSE)
  }
  if (length(x) != length(event)) {
    stop("'x' and 'event' must have the same length.", call. = FALSE)
  }
  groups <- as.integer(groups)
  if (groups < 2L) {
    stop("'groups' must be at least 2.", call. = FALSE)
  }
  if (link == "cox" && is.null(time)) {
    stop("'time' is required when link = 'cox'.", call. = FALSE)
  }
  if (!is.null(time) && length(time) != length(x)) {
    stop("'time' must have the same length as 'x'.", call. = FALSE)
  }

  # --- Assign quantile groups -----------------------------------------------
  ranks <- rank(x, ties.method = "first")
  n <- length(x)
  group <- as.integer(cut(ranks,
                          breaks = seq(0, n, length.out = groups + 1L),
                          include.lowest = TRUE,
                          labels = seq_len(groups)))

  # --- Build data frame for aggregation ------------------------------------
  df <- data.frame(x = x, event = event, group = group)
  if (!is.null(time)) df$time <- time
  if (!is.null(by)) df$by <- by

  # --- Aggregation function -------------------------------------------------
  .calibrate_group <- function(d) {
    ng <- nrow(d)
    ev <- sum(d$event)
    if (link == "cox") {
      prob <- if (sum(d$time) > 0) ev / sum(d$time) else NA_real_
    } else {
      prob <- if (ng > 0) ev / ng else NA_real_
    }

    # Apply link transform
    link_val <- if (is.na(prob) || prob <= 0 || prob >= 1) {
      NA_real_
    } else if (link == "logit") {
      log(prob / (1 - prob))
    } else if (link == "gompertz") {
      log(-log(1 - prob))
    } else {
      # Cox: prob is already events/time (hazard rate)
      if (prob > 0) log(prob) else NA_real_
    }

    data.frame(
      n      = ng,
      events = ev,
      mean   = mean(d$x),
      min    = min(d$x),
      max    = max(d$x),
      prob   = prob,
      link_value = link_val
    )
  }

  # --- Aggregate by group (and optionally by stratum) -----------------------
  if (is.null(by)) {
    result_list <- lapply(split(df, df$group), .calibrate_group)
    result <- do.call(rbind, result_list)
    result$group <- as.integer(rownames(result))
    rownames(result) <- NULL
    result <- result[order(result$group),
                     c("group", "n", "events", "mean", "min", "max",
                       "prob", "link_value")]
  } else {
    splits <- split(df, list(df$by, df$group))
    result_list <- lapply(splits, function(d) {
      if (nrow(d) == 0) return(NULL)
      r <- .calibrate_group(d)
      r$by <- d$by[1]
      r$group <- d$group[1]
      r
    })
    result <- do.call(rbind, Filter(Negate(is.null), result_list))
    rownames(result) <- NULL
    result <- result[order(result$by, result$group),
                     c("group", "by", "n", "events", "mean", "min", "max",
                       "prob", "link_value")]
  }

  attr(result, "link") <- link
  attr(result, "groups") <- groups
  class(result) <- c("hzr_calibrate", "data.frame")
  result
}

#' Print method for hzr_calibrate
#'
#' @param x An `hzr_calibrate` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @return The object \code{x} of class \code{c("hzr_calibrate", "data.frame")},
#'   invisibly. The data frame has one row per quantile group and columns:
#'   \code{group} (group index),
#'   \code{n} (group size),
#'   \code{events} (event count),
#'   \code{mean}, \code{min}, \code{max} (covariate summary within group),
#'   \code{prob} (observed event probability),
#'   \code{link_value} (transformed probability on the link scale).
#'   When stratified via the \code{by} argument, a \code{by} column is also
#'   present. Attributes: \code{"link"} (the transform applied,
#'   e.g. \code{"logit"}) and \code{"groups"} (number of quantile bins).
#' @export
print.hzr_calibrate <- function(x, digits = 3, ...) {
  lnk <- attr(x, "link")
  grp <- attr(x, "groups")
  cat("Variable calibration (", lnk, " link, ", grp, " groups)\n",
      sep = "")
  if ("by" %in% names(x)) {
    cat("Stratified by:", paste(unique(x$by), collapse = ", "), "\n")
  }
  cat("\n")
  display <- x
  display$mean <- round(display$mean, digits)
  display$min <- round(display$min, digits)
  display$max <- round(display$max, digits)
  display$prob <- round(display$prob, digits)
  display$link_value <- round(display$link_value, digits)
  class(display) <- "data.frame"
  print(display, row.names = FALSE)
  invisible(x)
}


# =========================================================================
# hzr_nelson -- Wayne Nelson cumulative hazard estimator
# =========================================================================

#' Wayne Nelson cumulative hazard estimator with lognormal confidence limits
#'
#' Compute the Nelson-Aalen cumulative hazard estimate with lognormal
#' confidence limits. Supports weighted events for severity-adjusted
#' analyses of repeated/recurrent events.
#' This is the R equivalent of the SAS `nelsonl.sas` macro.
#'
#' Unlike `survival::survfit()` which uses the Breslow estimator with
#' Greenwood variance, this function uses the Wayne Nelson estimator
#' with lognormal confidence limits that are always non-negative.
#'
#' @param time Numeric vector of follow-up times.
#' @param event Numeric event indicator (1 = event, 0 = censored).
#' @param weight Optional numeric vector of event weights (default 1).
#'   Weights are applied only to events (censored observations contribute
#'   zero weight). Use for severity-weighted repeated events.
#' @param conf_level Confidence level for the interval (default 0.95).
#'
#' @return A data frame with one row per unique event time and columns:
#' \describe{
#'   \item{time}{Event time.}
#'   \item{n_risk}{Number at risk.}
#'   \item{n_event}{Number of events at this time.}
#'   \item{weight_sum}{Sum of event weights at this time.}
#'   \item{cumhaz}{Nelson cumulative hazard estimate.}
#'   \item{std_err}{Standard error.}
#'   \item{cl_lower}{Lower lognormal confidence limit.}
#'   \item{cl_upper}{Upper lognormal confidence limit.}
#'   \item{hazard}{Interval hazard rate.}
#'   \item{cum_events}{Cumulative (weighted) event count.}
#' }
#'
#' @examples
#' data(cabgkul)
#' nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)
#' head(nel)
#'
#' @references
#' Nelson W (1972). Theory and applications of hazard plotting for censored
#' failure data. *Technometrics* 14(4):945--966.
#' \doi{10.1080/00401706.1972.10488991}
#'
#' Aalen O (1978). Nonparametric inference for a family of counting processes.
#' *Ann Statist* 6(4):701--726. \doi{10.1214/aos/1176344247}
#' @seealso [hzr_kaplan()] for survival estimation.
#' @export
hzr_nelson <- function(time, event, weight = NULL, conf_level = 0.95) {
  if (!is.numeric(time) || !is.numeric(event)) {
    stop("'time' and 'event' must be numeric vectors.", call. = FALSE)
  }
  n <- length(time)
  if (length(event) != n) {
    stop("'time' and 'event' must have the same length.", call. = FALSE)
  }
  if (is.null(weight)) weight <- rep(1, n)
  if (length(weight) != n) {
    stop("'weight' must have the same length as 'time'.", call. = FALSE)
  }
  if (conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1.", call. = FALSE)
  }

  z_alpha <- stats::qnorm(0.5 + 0.5 * conf_level)

  # Effective weight: weight * event (censored get 0)
  e_wght <- weight * event

  # Sort by time

  ord <- order(time)
  time_s <- time[ord]
  event_s <- event[ord]
  e_wght_s <- e_wght[ord]

  # Collapse to unique times
  u_times <- sort(unique(time_s[event_s == 1]))
  if (length(u_times) == 0) {
    out <- data.frame(time = numeric(0), n_risk = numeric(0),
                      n_event = numeric(0), weight_sum = numeric(0),
                      cumhaz = numeric(0), std_err = numeric(0),
                      cl_lower = numeric(0), cl_upper = numeric(0),
                      hazard = numeric(0), cum_events = numeric(0))
    class(out) <- c("hzr_nelson", "data.frame")
    return(out)
  }

  out_n <- length(u_times)
  n_risk <- numeric(out_n)
  n_event_out <- numeric(out_n)
  weight_sum <- numeric(out_n)
  cumhaz <- numeric(out_n)

  # Single pass: compute cumhaz, variance, and lognormal CL together
  std_err <- numeric(out_n)
  cl_lower <- numeric(out_n)
  cl_upper <- numeric(out_n)

  run_i_nrisk <- 0
  run_it <- 0
  cum_dist <- 0

  for (k in seq_along(u_times)) {
    t_k <- u_times[k]
    n_risk[k] <- sum(time_s >= t_k)
    at_time <- time_s == t_k & event_s == 1
    n_event_out[k] <- sum(at_time)
    weight_sum[k] <- sum(e_wght_s[at_time])

    # Nelson estimator: dist = sum(weight) / n_risk
    dist_k <- if (n_risk[k] > 0) weight_sum[k] / n_risk[k] else 0
    cum_dist <- cum_dist + dist_k
    cumhaz[k] <- cum_dist

    # Variance and lognormal CL (running accumulators)
    if (n_risk[k] > 0) run_i_nrisk <- run_i_nrisk + 1 / n_risk[k]
    run_it <- run_it + n_event_out[k]

    if (run_it > 0 && cum_dist > 0) {
      var_cef <- run_i_nrisk * cum_dist / run_it
      std_err[k] <- sqrt(var_cef)

      sigma2 <- log(run_i_nrisk / (run_it * cum_dist) + 1)
      sigma <- sqrt(sigma2)
      mu_ln <- log(cum_dist) - sigma2 / 2

      cl_upper[k] <- exp(mu_ln + z_alpha * sigma)
      cl_lower[k] <- exp(mu_ln - z_alpha * sigma)
    }
  }

  # Interval hazard
  lag_cumhaz <- c(0, cumhaz[-out_n])
  lag_time <- c(0, u_times[-out_n])
  delta_t <- u_times - lag_time
  hazard <- rep(NA_real_, out_n)
  idx <- delta_t > 0
  hazard[idx] <- (cumhaz[idx] - lag_cumhaz[idx]) / delta_t[idx]

  result <- data.frame(
    time       = u_times,
    n_risk     = n_risk,
    n_event    = n_event_out,
    weight_sum = weight_sum,
    cumhaz     = cumhaz,
    std_err    = std_err,
    cl_lower   = cl_lower,
    cl_upper   = cl_upper,
    hazard     = hazard,
    cum_events = cumsum(n_event_out)
  )

  class(result) <- c("hzr_nelson", "data.frame")
  result
}

#' @rdname hzr_nelson
#' @param x An `hzr_nelson` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @export
print.hzr_nelson <- function(x, digits = 4, ...) {
  cat("Nelson cumulative hazard estimate with lognormal CL\n")
  cat("Events:", sum(x$n_event), " | Time points:", nrow(x), "\n\n")
  display <- x
  for (col in c("cumhaz", "std_err", "cl_lower", "cl_upper", "hazard")) {
    display[[col]] <- round(display[[col]], digits)
  }
  class(display) <- "data.frame"
  print(utils::head(display, 20), row.names = FALSE)
  if (nrow(display) > 20) cat("... (", nrow(display) - 20, " more rows)\n")
  invisible(x)
}


# =========================================================================
# hzr_bootstrap -- Bootstrap inference for hazard models
# =========================================================================

#' Bootstrap resampling for hazard model coefficients
#'
#' Resample data with replacement, refit the hazard model on each
#' replicate, and accumulate coefficient distributions. Returns a tidy
#' data frame of per-replicate estimates with summary statistics.
#' This is the R equivalent of the SAS `bootstrap.hazard.sas` macro.
#'
#' @param object A fitted `hazard` object (with `fit = TRUE`).
#' @param n_boot Integer: number of bootstrap replicates (default 200).
#' @param fraction Numeric in (0, 1]: fraction of data to sample per
#'   replicate (default 1.0 for full bootstrap; < 1 for bagging).
#' @param seed Optional integer random seed for reproducibility. When
#'   supplied, `set.seed(seed)` is called at function entry, jumping the
#'   global RNG to the seeded state; it is not restored on exit. Pass
#'   `NULL` (the default) to skip the `set.seed()` call and start from
#'   the caller's current RNG state. Note that the bootstrap consumes
#'   random numbers either way, so the global RNG state will advance
#'   during the call -- `seed = NULL` avoids the *reset* at entry, not
#'   the advance during resampling.
#' @param verbose Logical; if `TRUE`, print progress every 50 replicates.
#'
#' @return A list with class `"hzr_bootstrap"` containing:
#' \describe{
#'   \item{replicates}{Data frame with columns `replicate`, `parameter`,
#'     and `estimate` -- one row per parameter per successful replicate.}
#'   \item{summary}{Data frame with columns `parameter`, `n`, `pct`,
#'     `mean`, `sd`, `min`, `max`, `ci_lower`, `ci_upper` -- one row per
#'     parameter.}
#'   \item{n_success}{Number of successfully converged replicates.}
#'   \item{n_failed}{Number of replicates that failed to converge.}
#' }
#'
#' @examples
#' \donttest{
#' data(avc)
#' avc <- na.omit(avc)
#' fit <- hazard(
#'   survival::Surv(int_dead, dead) ~ age + mal,
#'   data  = avc,
#'   dist  = "weibull",
#'   theta = c(mu = 0.01, nu = 0.5, 0, 0),
#'   fit   = TRUE
#' )
#' bs <- hzr_bootstrap(fit, n_boot = 50, seed = 123)
#' print(bs)
#' }
#'
#' @seealso [hazard()] for model fitting, [vcov.hazard()] for
#'   Hessian-based standard errors.
#' @export
hzr_bootstrap <- function(object, n_boot = 200L, fraction = 1.0,
                           seed = NULL, verbose = FALSE) {
  if (!inherits(object, "hazard")) {
    stop("'object' must be a fitted hazard object.", call. = FALSE)
  }
  if (is.null(object$fit$theta) ||
      (is.logical(object$fit$converged) && is.na(object$fit$converged))) {
    stop("'object' has no fitted parameters. Refit with fit = TRUE.",
         call. = FALSE)
  }

  n_boot <- as.integer(n_boot)
  if (n_boot < 1L) stop("'n_boot' must be at least 1.", call. = FALSE)
  if (fraction <= 0 || fraction > 1) {
    stop("'fraction' must be in (0, 1].", call. = FALSE)
  }

  if (!is.null(seed)) set.seed(seed)

  # Reconstruct the call components
  cl <- object$call
  # Prefer the evaluated `data` argument stored on the fitted object
  # (`object$data$frame` -- the data frame passed to hazard(), not a
  # model.frame() result): it is guaranteed available and needs no caller-frame
  # lookup. Re-evaluating `cl$data` in `parent.frame()` is fragile -- the
  # original `data` symbol may no longer be in scope (e.g. the fit was built
  # inside a helper that has returned) -- so fall back to it only for objects
  # fitted before the frame was stored on `object$data`.
  orig_data <- if (!is.null(object$data$frame)) {
    object$data$frame
  } else {
    eval(cl$data, envir = parent.frame())
  }
  n_obs <- nrow(orig_data)
  sample_size <- max(1L, as.integer(n_obs * fraction))

  # Observation weights, if any, must be resampled in lockstep with the data.
  # Prefer the weights already evaluated and stored on the fitted object: they
  # are guaranteed aligned with the original rows and need no caller-frame
  # lookup. Re-evaluating `cl$weights` in `parent.frame()` is fragile -- the
  # original symbol/expression may no longer be in scope, or may now resolve to
  # a different value -- so fall back to it only for objects fitted before
  # weights were stored on `object$data`. Each replicate is then rewired to a
  # locally bound, resampled copy (mirroring `data`).
  orig_weights <- if (!is.null(object$data$weights)) {
    object$data$weights
  } else if (!is.null(cl$weights)) {
    eval(cl$weights, envir = parent.frame())
  } else {
    NULL
  }

  # Parameter names from the fitted model. Shape parameters (e.g. mu, nu) are
  # named in theta, but covariate betas often come through with empty names.
  # Covariate coefficients occupy the last ncol(x) positions of theta; fill
  # any blanks within that block from the design matrix column names by
  # relative index, so downstream pivots (e.g. reshape(wide)) get a distinct
  # column per covariate -- even when some betas are already named and
  # others are not.
  param_names <- names(object$fit$theta)
  if (is.null(param_names)) {
    param_names <- character(length(object$fit$theta))
  }
  if (!is.null(object$data$x)) {
    x_names <- colnames(object$data$x)
    p <- ncol(object$data$x)
    n_theta <- length(param_names)
    if (!is.null(x_names) && p > 0L && n_theta >= p) {
      cov_idx <- seq.int(n_theta - p + 1L, n_theta)
      blank_in_block <- !nzchar(param_names[cov_idx])
      param_names[cov_idx[blank_in_block]] <- x_names[blank_in_block]
    }
  }
  still_blank <- !nzchar(param_names)
  if (any(still_blank)) {
    param_names[still_blank] <- paste0("param_", which(still_blank))
  }

  # Accumulate results
  rep_list <- vector("list", n_boot)
  n_success <- 0L
  n_failed <- 0L

  for (b in seq_len(n_boot)) {
    if (verbose && b %% 50 == 0) {
      cat("Bootstrap replicate", b, "/", n_boot, "\n")
    }

    # Resample with replacement
    idx <- sample.int(n_obs, size = sample_size, replace = TRUE)
    boot_data <- orig_data[idx, , drop = FALSE] # nolint: object_usage_linter.
    # boot_weights is referenced via quote() inside eval -- lintr cannot trace it
    boot_weights <- if (is.null(orig_weights)) NULL else orig_weights[idx] # nolint: object_usage_linter.

    # Refit using the same call but with resampled data (and weights, if any)
    # (boot_data/boot_weights are referenced via quote() inside eval)
    boot_fit <- tryCatch({
      cl_boot <- cl
      cl_boot$data <- quote(boot_data)
      if (!is.null(orig_weights)) cl_boot$weights <- quote(boot_weights)
      cl_boot$fit <- TRUE
      eval(cl_boot)
    }, error = function(e) NULL)

    if (!is.null(boot_fit) && is.finite(boot_fit$fit$objective)) {
      n_success <- n_success + 1L
      theta_b <- boot_fit$fit$theta
      rep_list[[b]] <- data.frame(
        replicate = b,
        parameter = param_names[seq_along(theta_b)],
        estimate  = as.numeric(theta_b),
        stringsAsFactors = FALSE
      )
    } else {
      n_failed <- n_failed + 1L
    }
  }

  # Combine replicates
  replicates <- do.call(rbind, Filter(Negate(is.null), rep_list))
  if (is.null(replicates)) {
    replicates <- data.frame(replicate = integer(0),
                              parameter = character(0),
                              estimate = numeric(0),
                              stringsAsFactors = FALSE)
  }
  rownames(replicates) <- NULL

  # Summary statistics per parameter
  if (nrow(replicates) > 0) {
    summary_list <- lapply(split(replicates, replicates$parameter), function(d) {
      data.frame(
        parameter = d$parameter[1],
        n         = nrow(d),
        pct       = 100 * nrow(d) / n_boot,
        mean      = mean(d$estimate),
        sd        = stats::sd(d$estimate),
        min       = min(d$estimate),
        max       = max(d$estimate),
        ci_lower  = stats::quantile(d$estimate, 0.025),
        ci_upper  = stats::quantile(d$estimate, 0.975),
        stringsAsFactors = FALSE
      )
    })
    summary_df <- do.call(rbind, summary_list)
    rownames(summary_df) <- NULL
    # Sort by parameter order in the original model
    idx_order <- match(summary_df$parameter, param_names)
    summary_df <- summary_df[order(idx_order), ]
  } else {
    summary_df <- data.frame(parameter = character(0), n = integer(0),
                              pct = numeric(0), mean = numeric(0),
                              sd = numeric(0), min = numeric(0),
                              max = numeric(0), ci_lower = numeric(0),
                              ci_upper = numeric(0),
                              stringsAsFactors = FALSE)
  }

  result <- list(
    replicates = replicates,
    summary    = summary_df,
    n_success  = n_success,
    n_failed   = n_failed
  )
  class(result) <- "hzr_bootstrap"
  result
}

#' @rdname hzr_bootstrap
#' @param x An `hzr_bootstrap` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @export
print.hzr_bootstrap <- function(x, digits = 4, ...) {
  cat("Bootstrap inference for hazard model\n")
  cat("Replicates:", x$n_success, "successful,", x$n_failed, "failed\n\n")
  if (nrow(x$summary) > 0) {
    display <- x$summary
    for (col in c("pct", "mean", "sd", "min", "max",
                   "ci_lower", "ci_upper")) {
      display[[col]] <- round(display[[col]], digits)
    }
    print(display, row.names = FALSE)
  }
  invisible(x)
}


# =========================================================================
# hzr_competing_risks -- Cumulative incidence with Greenwood variance
# =========================================================================

#' Competing risks cumulative incidence
#'
#' Compute cumulative incidence functions for multiple competing event
#' types using the Aalen-Johansen estimator with Greenwood variance.
#' This is the R equivalent of the SAS `markov.sas` macro.
#'
#' Unlike the naive 1 - KM estimator (which overestimates incidence when
#' competing risks exist), this provides the correct marginal cumulative
#' incidence for each event type.
#'
#' @note This estimator is **unweighted**: every observation contributes a
#'   unit count to the at-risk and event tallies. There is no `weights`
#'   argument, so case or inverse-probability weights are not yet supported
#'   for competing-risks incidence (unlike [hazard()], which accepts
#'   `weights`). Pre-expand weighted rows to individual records if an
#'   approximate weighted estimate is needed.
#'
#' @param time Numeric vector of follow-up times.
#' @param event Integer vector of event type indicators:
#'   0 = censored, 1 = event type 1, 2 = event type 2, etc.
#'
#' @return A data frame with one row per unique event time and columns:
#' \describe{
#'   \item{time}{Event time.}
#'   \item{n_risk}{Number at risk.}
#'   \item{n_event_1, n_event_2, ...}{Events of each type at this time.}
#'   \item{n_censor}{Number censored at this time.}
#'   \item{surv}{Overall event-free survival (freedom from all events).}
#'   \item{incid_1, incid_2, ...}{Cumulative incidence for each event type.}
#'   \item{se_surv}{Standard error of overall survival.}
#'   \item{se_1, se_2, ...}{Standard error of each cumulative incidence.}
#' }
#'
#' @examples
#' data(valves)
#' valves_cc <- na.omit(valves)
#' # Combine death and PVE into a competing risks event variable
#' # 0 = censored, 1 = death, 2 = PVE
#' event_cr <- ifelse(valves_cc$dead == 1, 1L,
#'                    ifelse(valves_cc$pve == 1, 2L, 0L))
#' time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
#' cr <- hzr_competing_risks(time_cr, event_cr)
#' head(cr)
#'
#' @references
#' Aalen O, Johansen S (1978). An empirical transition matrix for
#' non-homogeneous Markov chains based on censored observations.
#' *Scand J Statist* 5(3):141--150.
#'
#' Kalbfleisch JD, Prentice RL (1980). *The Statistical Analysis of Failure
#' Time Data.* Wiley, New York.
#' @seealso [hzr_kaplan()] for single-event survival estimation.
#' @export
hzr_competing_risks <- function(time, event) {
  if (!is.numeric(time) || !is.numeric(event)) {
    stop("'time' and 'event' must be numeric vectors.", call. = FALSE)
  }
  if (length(time) != length(event)) {
    stop("'time' and 'event' must have the same length.", call. = FALSE)
  }

  event_types <- sort(setdiff(unique(event), 0))
  n_types <- length(event_types)
  if (n_types == 0) {
    stop("No events found (all observations are censored).", call. = FALSE)
  }

  # Sort by time
  ord <- order(time)
  time_s <- time[ord]
  event_s <- event[ord]
  n_total <- length(time_s)

  # Unique event times (where at least one event occurred)
  u_times <- sort(unique(time_s[event_s > 0]))
  out_n <- length(u_times)

  # Initialize output
  n_risk <- numeric(out_n)
  n_censor_out <- numeric(out_n)
  n_event_mat <- matrix(0, nrow = out_n, ncol = n_types)
  colnames(n_event_mat) <- paste0("n_event_", event_types)

  # Cumulative incidence vectors
  surv <- numeric(out_n)
  incid <- matrix(0, nrow = out_n, ncol = n_types)
  colnames(incid) <- paste0("incid_", event_types)

  # Variance (diagonal of Greenwood matrix -- simplified)
  var_surv <- numeric(out_n)
  var_incid <- matrix(0, nrow = out_n, ncol = n_types)

  prev_surv <- 1.0
  prev_incid <- rep(0, n_types)
  prev_var_surv <- 0
  prev_var_incid <- rep(0, n_types)
  at_risk <- n_total

  for (k in seq_along(u_times)) {
    t_k <- u_times[k]

    # Censored before this time (between previous event time and t_k)
    prev_t <- if (k == 1) 0 else u_times[k - 1]
    censored_between <- sum(event_s == 0 & time_s > prev_t & time_s < t_k)
    at_risk <- at_risk - censored_between

    n_risk[k] <- at_risk

    # Events and censored AT this time
    at_time <- time_s == t_k
    for (j in seq_along(event_types)) {
      n_event_mat[k, j] <- sum(event_s == event_types[j] & at_time)
    }
    n_censor_out[k] <- sum(event_s == 0 & at_time)
    total_events_k <- sum(n_event_mat[k, ])

    # Aalen-Johansen update
    if (at_risk > 0) {
      # Transition probabilities
      d_total <- total_events_k / at_risk
      surv[k] <- prev_surv * (1 - d_total)

      for (j in seq_along(event_types)) {
        d_j <- n_event_mat[k, j] / at_risk
        incid[k, j] <- prev_incid[j] + prev_surv * d_j
      }

      # Greenwood variance update (simplified diagonal)
      if (at_risk > total_events_k && at_risk > 0) {
        greenwood_term <- total_events_k / (at_risk * (at_risk - total_events_k))
        var_surv[k] <- surv[k]^2 *
          (prev_var_surv / max(prev_surv^2, .Machine$double.xmin) +
             greenwood_term)

        for (j in seq_along(event_types)) {
          d_j <- n_event_mat[k, j] / at_risk
          var_incid[k, j] <- prev_var_incid[j] +
            prev_surv^2 * d_j * (1 - d_j) / at_risk
        }
      } else {
        var_surv[k] <- prev_var_surv
        var_incid[k, ] <- prev_var_incid
      }
    } else {
      surv[k] <- prev_surv
      incid[k, ] <- prev_incid
      var_surv[k] <- prev_var_surv
      var_incid[k, ] <- prev_var_incid
    }

    # Decrease at-risk by events + censored AT this time
    at_risk <- at_risk - total_events_k - n_censor_out[k]

    prev_surv <- surv[k]
    prev_incid <- incid[k, ]
    prev_var_surv <- var_surv[k]
    prev_var_incid <- var_incid[k, ]
  }

  # Assemble result
  result <- data.frame(
    time     = u_times,
    n_risk   = n_risk,
    n_event_mat,
    n_censor = n_censor_out,
    surv     = surv,
    incid,
    se_surv  = sqrt(pmax(var_surv, 0)),
    stringsAsFactors = FALSE
  )

  # Add SE columns for each event type
  for (j in seq_along(event_types)) {
    result[[paste0("se_", event_types[j])]] <-
      sqrt(pmax(var_incid[, j], 0))
  }

  class(result) <- c("hzr_competing_risks", "data.frame")
  result
}

#' @rdname hzr_competing_risks
#' @param x An `hzr_competing_risks` object.
#' @param digits Number of decimal places for formatting.
#' @param ... Additional arguments (ignored).
#' @export
print.hzr_competing_risks <- function(x, digits = 4, ...) {
  incid_cols <- grep("^incid_", names(x), value = TRUE)
  cat("Competing risks cumulative incidence\n")
  cat("Event types:", length(incid_cols), " | Time points:", nrow(x), "\n")
  if (nrow(x) > 0) {
    cat("Final survival:", round(x$surv[nrow(x)], digits), "\n")
    for (col in incid_cols) {
      cat("Final", col, ":", round(x[[col]][nrow(x)], digits), "\n")
    }
  }
  cat("\n")
  display <- x
  for (col in c("surv", incid_cols, "se_surv",
                 grep("^se_\\d", names(x), value = TRUE))) {
    if (col %in% names(display)) {
      display[[col]] <- round(display[[col]], digits)
    }
  }
  class(display) <- "data.frame"
  print(utils::head(display, 15), row.names = FALSE)
  if (nrow(display) > 15) cat("... (", nrow(display) - 15, " more rows)\n")
  invisible(x)
}

# likelihood-multiphase.R -- Multiphase additive hazard likelihood and optimizer
#
# REFERENCES
# ----------
# Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
# hazard into phases, each incorporating a separate stream of concomitant
# information. J Am Stat Assoc. 1986;81(395):615-624.
#
# Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
# Probability of atrial fibrillation after ablation: Using a parametric
# nonlinear temporal decomposition mixed effects model. Stat Methods Med Res.
# 2018;27(1):126-141.
#
# MODEL
# -----
# Additive cumulative hazard across J phases:
#
#   H(t | x) = sum_{j=1}^{J}  mu_j(x) * Phi_j(t; t_half_j, nu_j, m_j)
#
# where:
#   mu_j(x) = exp(alpha_j + x_j * beta_j)  -- phase-specific log-linear scale
#   Phi_j(t) depends on phase type:
#     "cdf"      -> G(t)             bounded [0, 1]   (early risk)
#     "hazard"   -> -log(1 - G(t))   monotone incr.   (late risk)
#     "constant" -> t                flat rate         (background)
#
# Total hazard:
#   h(t | x) = sum_j  mu_j(x) * phi_j(t)
#
# where phi_j = dPhi_j/dt.
#
# Survival:
#   S(t | x) = exp(-H(t | x))
#
# THETA LAYOUT (internal / estimation scale)
# ------------------------------------------
# For each phase j, the sub-vector is:
#   [log_mu_j, log_t_half_j, nu_j, m_j, beta_j_1, ..., beta_j_pj]
#   (constant phases omit log_t_half, nu, m)
#
# The full theta is the concatenation of all phase sub-vectors.
#
# FUNCTIONS
# ---------
#   .hzr_logl_multiphase()     -- log-likelihood
#   .hzr_optim_multiphase()    -- optimizer using .hzr_optim_generic()
#   .hzr_multiphase_cumhaz()   -- compute total H(t|x) from theta + phases
#   .hzr_multiphase_hazard()   -- compute total h(t|x) from theta + phases
#   .hzr_split_theta()         -- split theta into per-phase sub-vectors


# ============================================================================
# Theta vector manipulation
# ============================================================================

#' Split the full theta vector into per-phase sub-vectors
#'
#' @param theta Numeric vector -- full parameter vector (internal scale).
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector -- number of covariates per phase.
#' @return Named list of numeric vectors, one per phase.
#' @keywords internal
.hzr_split_theta <- function(theta, phases, covariate_counts) {
  result <- vector("list", length(phases))
  names(result) <- names(phases)
  pos <- 1L

  for (nm in names(phases)) {
    np <- .hzr_phase_n_params(phases[[nm]], n_covariates = covariate_counts[[nm]])
    result[[nm]] <- theta[pos:(pos + np - 1L)]
    pos <- pos + np
  }

  result
}


#' Extract parameters from a phase sub-vector (internal scale)
#'
#' @param theta_j Numeric sub-vector for one phase.
#' @param phase An `hzr_phase` object.
#' @return Named list with `log_mu`, `log_t_half`, `nu`, `m`, `beta`.
#' @keywords internal
.hzr_unpack_phase_theta <- function(theta_j, phase) {
  pos <- 1L
  log_mu <- theta_j[pos]
  pos <- pos + 1L

  if (phase$type == "g3") {
    log_tau <- theta_j[pos]
    pos <- pos + 1L
    gamma_  <- theta_j[pos]
    pos <- pos + 1L
    alpha_  <- theta_j[pos]
    pos <- pos + 1L
    eta_    <- theta_j[pos]
    pos <- pos + 1L
    log_t_half <- NA_real_
    nu <- NA_real_
    m <- NA_real_
  } else if (phase$type != "constant") {
    log_t_half <- theta_j[pos]
    pos <- pos + 1L
    nu <- theta_j[pos]
    pos <- pos + 1L
    m <- theta_j[pos]
    pos <- pos + 1L
    log_tau <- NA_real_
    gamma_ <- NA_real_
    alpha_ <- NA_real_
    eta_ <- NA_real_
  } else {
    log_t_half <- NA_real_
    nu <- NA_real_
    m <- NA_real_
    log_tau <- NA_real_
    gamma_ <- NA_real_
    alpha_ <- NA_real_
    eta_ <- NA_real_
  }

  beta <- if (pos <= length(theta_j)) theta_j[pos:length(theta_j)] else numeric(0)

  list(log_mu = log_mu, log_t_half = log_t_half, nu = nu, m = m,
       log_tau = log_tau, gamma = gamma_, alpha = alpha_, eta = eta_,
       beta = beta)
}


# ============================================================================
# Cumulative hazard and hazard computations
# ============================================================================

#' Compute total cumulative hazard H(t|x) for multiphase model
#'
#' @param time Numeric vector of times (n).
#' @param theta Full parameter vector (internal scale).
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of covariate counts per phase.
#' @param x_list Named list of design matrices, one per phase. Each element is
#'   an n x p_j matrix or NULL.
#' @param per_phase Logical; if TRUE return a list with total and per-phase
#'   contributions.
#' @return If `per_phase = FALSE`: numeric vector of length n (total H(t|x)).
#'   If `per_phase = TRUE`: named list with `$total` and one element per phase.
#' @keywords internal
.hzr_multiphase_cumhaz <- function(time, theta, phases, covariate_counts,
                                    x_list, per_phase = FALSE) {
  n <- length(time)
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  total <- rep(0, n)
  phase_contributions <- if (per_phase) vector("list", length(phases)) else NULL

  for (i in seq_along(phases)) {
    nm <- names(phases)[i]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    # mu_j(x) = exp(log_mu + x_j * beta_j)
    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      eta_j <- pars$log_mu + as.numeric(x_list[[nm]] %*% pars$beta)
    } else {
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)

    # Phi_j(t)
    if (phases[[nm]]$type == "constant") {
      phi_j <- time
    } else if (phases[[nm]]$type == "g3") {
      tau_j <- exp(pars$log_tau)
      d3 <- hzr_decompos_g3(time, tau = tau_j, gamma = pars$gamma,
                              alpha = pars$alpha, eta = pars$eta)
      phi_j <- d3$G3
    } else {
      t_half_j <- exp(pars$log_t_half)
      # Guard against invalid decomposition parameters
      if (pars$m < 0 && pars$nu < 0) return(rep(Inf, n))
      phi_j <- hzr_phase_cumhaz(time, t_half = t_half_j, nu = pars$nu,
                                  m = pars$m, type = phases[[nm]]$type)
    }

    contrib <- mu_j * phi_j
    total <- total + contrib

    if (per_phase) phase_contributions[[i]] <- contrib
  }

  if (per_phase) {
    names(phase_contributions) <- names(phases)
    phase_contributions$total <- total
    return(phase_contributions)
  }

  total
}


#' Compute total instantaneous hazard h(t|x) for multiphase model
#'
#' @inheritParams .hzr_multiphase_cumhaz
#' @return Numeric vector of length n.
#' @keywords internal
.hzr_multiphase_hazard <- function(time, theta, phases, covariate_counts,
                                    x_list) {
  n <- length(time)
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  total <- rep(0, n)

  for (i in seq_along(phases)) {
    nm <- names(phases)[i]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      eta_j <- pars$log_mu + as.numeric(x_list[[nm]] %*% pars$beta)
    } else {
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)

    if (phases[[nm]]$type == "constant") {
      dphi_j <- rep(1, n)
    } else if (phases[[nm]]$type == "g3") {
      tau_j <- exp(pars$log_tau)
      d3 <- hzr_decompos_g3(time, tau = tau_j, gamma = pars$gamma,
                              alpha = pars$alpha, eta = pars$eta)
      dphi_j <- d3$g3
    } else {
      t_half_j <- exp(pars$log_t_half)
      if (pars$m < 0 && pars$nu < 0) return(rep(Inf, n))
      dphi_j <- hzr_phase_hazard(time, t_half = t_half_j, nu = pars$nu,
                                   m = pars$m, type = phases[[nm]]$type)
    }

    total <- total + mu_j * dphi_j
  }

  total
}


# ============================================================================
# Conservation of Events (CoE) -- Turner's theorem
# ============================================================================
#
# For the additive hazard model H(t|x) = Sigma_j mu_j(x) * Phi_j(t),
# the MLE constraint (score equation for mu) implies:
#
#   Sigma_i E(i) = Sigma_i H(t_i | x_i)
#
# i.e., total observed events equals total predicted cumulative hazard.
#
# This allows one phase's log_mu to be solved analytically at each
# optimizer iteration, reducing the optimization dimension by 1.
#
# Reference: Turner ME Jr., as implemented in C HAZARD setcoe.c / consrv.c.

#' Find the position of each phase's log_mu in the full theta vector
#'
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of per-phase covariate counts.
#' @return Named integer vector: position of log_mu for each phase.
#' @keywords internal
.hzr_log_mu_positions <- function(phases, covariate_counts) {
  positions <- setNames(integer(length(phases)), names(phases))
  pos <- 1L
  for (nm in names(phases)) {
    positions[[nm]] <- pos
    np <- .hzr_phase_n_params(phases[[nm]],
                               n_covariates = covariate_counts[[nm]])
    pos <- pos + np
  }
  positions
}


#' Select the phase whose log_mu will be solved by conservation
#'
#' Chooses the phase contributing the largest share of total cumulative
#' hazard, matching the C HAZARD SETCOE strategy.
#'
#' @param theta Full parameter vector (internal scale).
#' @param time Numeric vector of follow-up times.
#' @param status Numeric event indicator.
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector.
#' @param x_list Named list of per-phase design matrices.
#' @param weights Optional numeric vector of row weights (length n). Defaults to
#'   unit weights. Applied when summing per-phase cumhaz so that selection
#'   happens on the same scale as the (weighted) observed event count.
#' @return Character: name of the phase to fix.
#' @keywords internal
.hzr_select_fixmu_phase <- function(theta, time, status,
                                     phases, covariate_counts, x_list,
                                     weights = NULL) {
  decomp <- .hzr_multiphase_cumhaz(time, theta, phases,
                                     covariate_counts, x_list,
                                     per_phase = TRUE)
  if (is.null(weights)) weights <- rep(1, length(time))
  phase_sums <- vapply(names(phases), function(nm) {
    sum(weights * decomp[[nm]])
  }, numeric(1))

  names(which.max(phase_sums))
}


#' Apply the Conservation of Events adjustment to one phase's log_mu
#'
#' Given the current theta vector, analytically solve the fixmu phase's
#' log_mu so that total predicted events = total observed events.
#'
#' This is called BEFORE each likelihood evaluation inside the optimizer,
#' matching the C HAZARD `CONSRV` entry point.
#'
#' @param theta Full parameter vector (internal scale).
#' @param fixmu_phase Character: name of the phase whose log_mu is solved.
#' @param fixmu_pos Integer: position of that log_mu in theta.
#' @param time Numeric vector of follow-up times.
#' @param status Numeric event indicator.
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector.
#' @param x_list Named list of per-phase design matrices.
#' @param total_events Numeric: sum of observed events (precomputed, weighted
#'   when `weights` is supplied).
#' @param weights Optional numeric vector of row weights (length n). Defaults to
#'   unit weights. Applied when summing per-phase cumhaz so Turner's adjustment
#'   is computed on the same scale as `total_events`.
#' @return Updated theta vector with fixmu phase's log_mu adjusted.
#' @keywords internal
.hzr_conserve_events <- function(theta, fixmu_phase, fixmu_pos,
                                  time, status,
                                  phases, covariate_counts, x_list,
                                  total_events, weights = NULL) {
  # Compute per-phase cumulative hazard contributions
  decomp <- .hzr_multiphase_cumhaz(time, theta, phases,
                                     covariate_counts, x_list,
                                     per_phase = TRUE)

  if (is.null(weights)) weights <- rep(1, length(time))

  # Total predicted events (weighted sum of cumhaz across all observations)
  sumcz <- sum(weights * decomp$total)

  # Weighted contribution from the fixmu phase alone
  sumcj <- sum(weights * decomp[[fixmu_phase]])

  if (sumcj <= 0 || !is.finite(sumcj)) return(theta)

  # Discrepancy: how many events are unaccounted for
  devent <- total_events - sumcz

  # Events the fixmu phase should absorb
  jevent <- sumcj + devent

  if (jevent <= 0 || !is.finite(jevent)) return(theta)

  # Multiplicative adjustment in log scale
  lfactor <- log(jevent / sumcj)

  if (!is.finite(lfactor)) return(theta)

  theta[fixmu_pos] <- theta[fixmu_pos] + lfactor
  theta
}


# ============================================================================
# Log-likelihood
# ============================================================================

#' Log-likelihood for multiphase additive hazard model
#'
#' @param theta Full parameter vector (internal scale).
#' @param time Numeric vector of follow-up times (n).
#' @param status Numeric event indicator: 1 = event, 0 = right-censored,
#'   -1 = left-censored, 2 = interval-censored.
#' @param time_lower Optional lower bounds for interval censoring.
#' @param time_upper Optional upper bounds for left/interval censoring.
#' @param x Design matrix (unused directly; kept for interface compatibility).
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of per-phase covariate counts.
#' @param x_list Named list of per-phase design matrices.
#' @param return_gradient Logical (ignored; gradient via separate function).
#' @param return_hessian Logical (ignored).
#' @param ... Ignored.
#' @return Scalar log-likelihood. Returns -Inf for infeasible parameters.
#' @keywords internal
.hzr_logl_multiphase <- function(theta, time, status,
                                  time_lower = NULL, time_upper = NULL,
                                  x = NULL, weights = NULL,
                                  phases, covariate_counts, x_list,
                                  return_gradient = FALSE,
                                  return_hessian = FALSE, ...) {

  n <- length(time)
  if (is.null(weights)) weights <- rep(1, n)

  # Feasibility: check parameter constraints

  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  for (nm in names(phases)) {
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])
    if (phases[[nm]]$type %in% c("cdf", "hazard")) {
      if (pars$m < 0 && pars$nu < 0) return(-Inf)
    }
    if (phases[[nm]]$type == "g3") {
      # G3 requires gamma > 0, alpha >= 0, eta > 0
      if (pars$gamma <= 0 || pars$eta <= 0 || pars$alpha < 0) return(-Inf)
    }
  }

  # Validate event times
  if (any(status == 1 & time <= 0)) return(-Inf)

  # Normalize censoring bounds
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  # Cumulative hazard at event/censoring times
  cumhaz <- .hzr_multiphase_cumhaz(time, theta, phases, covariate_counts, x_list)
  if (any(!is.finite(cumhaz))) return(-Inf)

  # Counting-process entry-time cumulative hazard; H(start) = 0 when no
  # `time_lower` supplied (plain right-censored data).  For status in
  # {0, 1}, the contribution becomes H(stop) - H(start); for status == 2,
  # this value is unused (interval-censoring handled separately below).
  if (is.null(time_lower)) {
    cumhaz_start <- rep(0, n)
  } else {
    cumhaz_start <- .hzr_multiphase_cumhaz(time_lower, theta, phases,
                                             covariate_counts, x_list)
    if (any(!is.finite(cumhaz_start))) return(-Inf)
  }

  # Instantaneous hazard at event times (only needed for exact events)
  idx_event <- status == 1

  logl <- 0

  # Exact events: w * [log h(stop) - (H(stop) - H(start))]
  if (any(idx_event)) {
    haz <- .hzr_multiphase_hazard(time, theta, phases, covariate_counts, x_list)
    if (any(!is.finite(haz[idx_event])) || any(haz[idx_event] <= 0)) return(-Inf)
    logl <- logl + sum(weights[idx_event] *
                         (log(haz[idx_event]) -
                            (cumhaz[idx_event] - cumhaz_start[idx_event])))
  }

  # Right-censored: w * [-(H(stop) - H(start))]
  idx_right <- status == 0
  if (any(idx_right)) {
    logl <- logl - sum(weights[idx_right] *
                         (cumhaz[idx_right] - cumhaz_start[idx_right]))
  }

  # Left-censored: w * [log(1 - exp(-H(u)))]
  idx_left <- status == -1
  if (any(idx_left)) {
    cumhaz_upper <- .hzr_multiphase_cumhaz(upper, theta, phases,
                                             covariate_counts, x_list)
    logl <- logl + sum(weights[idx_left] *
                         hzr_log1mexp(cumhaz_upper[idx_left]))
  }

  # Interval-censored: w * [-H(l) + log(1 - exp(-(H(u) - H(l))))]
  idx_interval <- status == 2
  if (any(idx_interval)) {
    cumhaz_lower <- .hzr_multiphase_cumhaz(lower, theta, phases,
                                             covariate_counts, x_list)
    cumhaz_upper_iv <- .hzr_multiphase_cumhaz(upper, theta, phases,
                                                covariate_counts, x_list)
    delta_h <- cumhaz_upper_iv[idx_interval] - cumhaz_lower[idx_interval]
    logl <- logl + sum(weights[idx_interval] *
                         (-cumhaz_lower[idx_interval] +
                            hzr_log1mexp(delta_h)))
  }

  if (!is.finite(logl)) return(-Inf)

  logl
}


# ============================================================================
# Analytic gradient
# ============================================================================

#' Gradient of the multiphase log-likelihood
#'
#' Computes the score vector \eqn{d\ell / d\theta} using analytic chain-rule
#' formulas for `log_mu` and `beta` parameters, and central-difference
#' derivatives (via `.hzr_phase_derivatives()`) for shape parameters
#' (`log_t_half`, `nu`, `m`).
#'
#' @inheritParams .hzr_logl_multiphase
#' @return Numeric vector of length `length(theta)` -- the gradient.
#'   Returns a zero vector if any component is non-finite (guards optimizer).
#' @keywords internal
.hzr_gradient_multiphase <- function(theta, time, status,
                                      time_lower = NULL, time_upper = NULL,
                                      x = NULL, weights = NULL,
                                      phases, covariate_counts, x_list,
                                      ...) {
  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)
  if (is.null(weights)) weights <- rep(1, n)

  # Feasibility check
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  for (nm in names(phases)) {
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])
    if (phases[[nm]]$type %in% c("cdf", "hazard")) {
      if (pars$m < 0 && pars$nu < 0) return(grad)
    }
    if (phases[[nm]]$type == "g3") {
      if (pars$gamma <= 0 || pars$eta <= 0 || pars$alpha < 0) return(grad)
    }
  }

  # Normalize censoring bounds
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  # Observation type masks
  idx_event    <- which(status == 1)
  idx_right    <- which(status == 0)
  idx_left     <- which(status == -1)

  idx_interval <- which(status == 2)

  has_left     <- length(idx_left) > 0
  has_interval <- length(idx_interval) > 0

  # -- Per-phase quantities --------------------------------------------------
  # We need, for each phase j and each observation i:
  #   mu_j(x_i), Phi_j(t_i), phi_j(t_i), and shape derivatives of Phi and phi
  #
  # Then the per-observation gradient contributions are assembled from:
  #   H(t) = sum_j mu_j * Phi_j    (cumulative hazard)
  #   h(t) = sum_j mu_j * phi_j    (instantaneous hazard)

  # Pre-compute total H(t) and h(t) at observation times (and censoring bounds)
  H_t <- rep(0, n)      # total cumulative hazard at time[i]
  h_t <- rep(0, n)      # total instantaneous hazard at time[i]

  # Storage for per-phase intermediate quantities
  n_phases <- length(phases)
  phase_mu         <- vector("list", n_phases)  # mu_j(x_i) for each i
  phase_Phi        <- vector("list", n_phases)  # Phi_j(t_i) for each i
  phase_phi        <- vector("list", n_phases)  # phi_j(t_i) for each i
  phase_deriv      <- vector("list", n_phases)  # derivative list at time
  phase_Phi_start  <- vector("list", n_phases)  # Phi_j(start_i), if needed
  phase_deriv_start <- vector("list", n_phases) # derivative list at start
  # Whether to actually compute start-time quantities for the gradient.
  # Without any counting-process rows (time_lower = NULL or all zeros with
  # status in {0, 1}) these additions are identically zero and we can skip.
  need_start <- !is.null(time_lower) &&
                any(time_lower > 0 & status %in% c(0L, 1L))
  start_vec <- if (need_start) time_lower else NULL

  for (j in seq_along(phases)) {
    nm <- names(phases)[j]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    # mu_j(x_i) = exp(log_mu + x * beta)
    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      eta_j <- pars$log_mu + as.numeric(x_list[[nm]] %*% pars$beta)
    } else {
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)
    phase_mu[[j]] <- mu_j

    # Phi_j, phi_j, and shape derivatives at observation time
    if (phases[[nm]]$type == "constant") {
      phase_Phi[[j]] <- time
      phase_phi[[j]] <- rep(1, n)
      phase_deriv[[j]] <- NULL  # no shape params for constant
    } else if (phases[[nm]]$type == "g3") {
      tau_j <- exp(pars$log_tau)
      d3 <- hzr_decompos_g3(time, tau = tau_j, gamma = pars$gamma,
                              alpha = pars$alpha, eta = pars$eta)
      phase_Phi[[j]] <- d3$G3
      phase_phi[[j]] <- d3$g3
      # G3 shape derivatives via finite differences (for now)
      phase_deriv[[j]] <- .hzr_g3_phase_derivatives(
        time, tau = tau_j, gamma = pars$gamma,
        alpha = pars$alpha, eta = pars$eta)
    } else {
      t_half_j <- exp(pars$log_t_half)
      pd <- .hzr_phase_derivatives(time, t_half = t_half_j, nu = pars$nu,
                                    m = pars$m, type = phases[[nm]]$type)
      phase_Phi[[j]]   <- pd$Phi
      phase_phi[[j]]   <- pd$phi
      phase_deriv[[j]] <- pd
    }

    # Matching quantities at start_vec -- only Phi_j and its shape
    # derivatives matter for the H(start) gradient contribution.
    if (need_start) {
      if (phases[[nm]]$type == "constant") {
        phase_Phi_start[[j]] <- start_vec
        phase_deriv_start[[j]] <- NULL
      } else if (phases[[nm]]$type == "g3") {
        tau_j <- exp(pars$log_tau)
        d3s <- hzr_decompos_g3(start_vec, tau = tau_j, gamma = pars$gamma,
                                 alpha = pars$alpha, eta = pars$eta)
        phase_Phi_start[[j]] <- d3s$G3
        phase_deriv_start[[j]] <- .hzr_g3_phase_derivatives(
          start_vec, tau = tau_j, gamma = pars$gamma,
          alpha = pars$alpha, eta = pars$eta)
      } else {
        t_half_j <- exp(pars$log_t_half)
        pds <- .hzr_phase_derivatives(start_vec, t_half = t_half_j,
                                        nu = pars$nu, m = pars$m,
                                        type = phases[[nm]]$type)
        phase_Phi_start[[j]]   <- pds$Phi
        phase_deriv_start[[j]] <- pds
      }
    }

    H_t <- H_t + mu_j * phase_Phi[[j]]
    h_t <- h_t + mu_j * phase_phi[[j]]
  }

  # Guard: if total hazard or cumhaz is non-finite, return zero gradient
  if (any(!is.finite(H_t)) || any(!is.finite(h_t))) return(grad)

  # -- Per-observation weight vectors ----------------------------------------
  # dLogl/dH(t_i) depends on observation type:
  #   event:         d/dH [ log h(t) - H(t) ] = -1        (H part)
  #   right-cens:    d/dH [ -H(t) ]            = -1
  #   left-cens:     d/dH [ log(1-exp(-H(u))) ] = exp(-H) / (1 - exp(-H))
  #   interval-cens: more complex (handled separately)
  #
  # For events, there's also the d/d[theta] of log h(t):
  #   d(log h) / d(theta_j) = (1/h) * dh/d(theta_j)

  # Weight for the cumulative hazard part at `time[i]`: w_H_i = dLogl/dH(t_i),
  # scaled by Fisher weights so the score matches the weighted log-likelihood.
  w_H <- numeric(n)
  w_H[idx_event] <- -weights[idx_event]
  w_H[idx_right] <- -weights[idx_right]

  if (has_left) {
    H_upper <- .hzr_multiphase_cumhaz(upper, theta, phases, covariate_counts,
                                       x_list)
    # d/dH log(1 - exp(-H)) = exp(-H) / (1 - exp(-H))
    exp_neg_H_u <- exp(-H_upper[idx_left])
    one_minus   <- pmax(1 - exp_neg_H_u, .Machine$double.xmin)
    w_H[idx_left] <- weights[idx_left] * (exp_neg_H_u / one_minus)
    # Left-censored uses H(upper), not H(time), so we handle below
  }

  # H(start) contribution for counting-process rows (status in {0, 1}).
  # Per-row contribution is `+H(start_i)` inside the log-likelihood, so
  # the derivative w.r.t. theta picks up `+weights_i * dH(start)/dtheta`.
  has_start <- !is.null(time_lower) && any(time_lower > 0 & status %in% c(0L, 1L))
  w_H_start <- numeric(n)
  if (has_start) {
    w_H_start[idx_event] <- weights[idx_event]
    w_H_start[idx_right] <- weights[idx_right]
  }

  # Weight for the instantaneous hazard part (events only): w_i / h(t_i).
  # This multiplies dh/d(theta_j).
  inv_h <- numeric(n)
  if (length(idx_event) > 0) {
    h_event <- h_t[idx_event]
    h_event <- pmax(h_event, .Machine$double.xmin)
    inv_h[idx_event] <- weights[idx_event] / h_event
  }

  # -- Assemble gradient per phase -------------------------------------------
  pos <- 1L  # position in the full theta vector

  for (j in seq_along(phases)) {
    nm <- names(phases)[j]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])
    mu_j  <- phase_mu[[j]]
    Phi_j <- phase_Phi[[j]]
    phi_j <- phase_phi[[j]]
    Phi_j_start <- phase_Phi_start[[j]]  # NULL when need_start = FALSE

    # -- d(logl) / d(log_mu_j) ------------------------------------------
    # From H part: w_H_i * dH/d(log_mu) = w_H_i * mu_j * Phi_j
    # From h part (events): inv_h_i * dh/d(log_mu) = inv_h_i * mu_j * phi_j
    dlogl_dlog_mu <- sum(w_H * mu_j * Phi_j) +
                     sum(inv_h[idx_event] * mu_j[idx_event] * phi_j[idx_event])

    # Counting-process entry-time contribution: +H(start) inside the LL,
    # so its derivative picks up +w_H_start * dH(start)/d(log_mu).
    if (need_start) {
      dlogl_dlog_mu <- dlogl_dlog_mu +
        sum(w_H_start * mu_j * Phi_j_start)
    }

    # Left-censored: uses H(upper), need separate Phi_j evaluation there
    if (has_left) {
      if (phases[[nm]]$type == "constant") {
        Phi_j_upper <- upper[idx_left]
      } else if (phases[[nm]]$type == "g3") {
        tau_j <- exp(pars$log_tau)
        d3_upper <- hzr_decompos_g3(upper[idx_left], tau = tau_j,
                                      gamma = pars$gamma, alpha = pars$alpha,
                                      eta = pars$eta)
        Phi_j_upper <- d3_upper$G3
      } else {
        t_half_j <- exp(pars$log_t_half)
        Phi_j_upper <- hzr_phase_cumhaz(upper[idx_left], t_half = t_half_j,
                                          nu = pars$nu, m = pars$m,
                                          type = phases[[nm]]$type)
      }
      dlogl_dlog_mu <- dlogl_dlog_mu +
        sum(w_H[idx_left] * mu_j[idx_left] * Phi_j_upper)
      # Subtract the w_H * mu * Phi at time[i] that we already added above
      # (for left-censored, the H contribution is at upper, not time)
      dlogl_dlog_mu <- dlogl_dlog_mu -
        sum(w_H[idx_left] * mu_j[idx_left] * Phi_j[idx_left])
    }

    # Interval-censored: use numerical fallback for these observations
    # (handled at end of loop via finite-difference correction)

    grad[pos] <- dlogl_dlog_mu
    pos <- pos + 1L

    # -- Shape parameters (non-constant phases only) ---------------------
    if (phases[[nm]]$type %in% c("cdf", "hazard")) {
      pd <- phase_deriv[[j]]

      # Compute shape derivatives at upper[idx_left] for left-censoring
      pd_upper <- NULL
      if (has_left) {
        t_half_j <- exp(pars$log_t_half)
        pd_upper <- .hzr_phase_derivatives(upper[idx_left],
          t_half = t_half_j, nu = pars$nu, m = pars$m,
          type = phases[[nm]]$type)
      }

      pd_start <- if (need_start) phase_deriv_start[[j]] else NULL

      # For each shape param s in {log_t_half, nu, m}:
      shape_derivs <- list(
        list(dPhi = pd$dPhi_dlog_thalf, dphi = pd$dphi_dlog_thalf,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dlog_thalf,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dlog_thalf),
        list(dPhi = pd$dPhi_dnu,        dphi = pd$dphi_dnu,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dnu,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dnu),
        list(dPhi = pd$dPhi_dm,         dphi = pd$dphi_dm,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dm,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dm)
      )

      for (s in seq_along(shape_derivs)) {
        dPhi_ds <- shape_derivs[[s]]$dPhi
        dphi_ds <- shape_derivs[[s]]$dphi

        dlogl_ds <- sum(w_H * mu_j * dPhi_ds) +
                    sum(inv_h[idx_event] * mu_j[idx_event] * dphi_ds[idx_event])

        # Counting-process start contribution: +w_H_start * mu * dPhi(start)
        if (need_start && !is.null(shape_derivs[[s]]$dPhi_start)) {
          dlogl_ds <- dlogl_ds +
            sum(w_H_start * mu_j * shape_derivs[[s]]$dPhi_start)
        }

        # Left-censoring correction: swap dPhi at time for dPhi at upper
        if (has_left && !is.null(shape_derivs[[s]]$dPhi_upper)) {
          dlogl_ds <- dlogl_ds +
            sum(w_H[idx_left] * mu_j[idx_left] * shape_derivs[[s]]$dPhi_upper) -
            sum(w_H[idx_left] * mu_j[idx_left] * dPhi_ds[idx_left])
        }

        grad[pos] <- dlogl_ds
        pos <- pos + 1L
      }
    } else if (phases[[nm]]$type == "g3") {
      pd <- phase_deriv[[j]]

      # Compute shape derivatives at upper[idx_left] for left-censoring
      pd_upper <- NULL
      if (has_left) {
        tau_j <- exp(pars$log_tau)
        pd_upper <- .hzr_g3_phase_derivatives(upper[idx_left],
          tau = tau_j, gamma = pars$gamma,
          alpha = pars$alpha, eta = pars$eta)
      }

      pd_start <- if (need_start) phase_deriv_start[[j]] else NULL

      # For each shape param s in {log_tau, gamma, alpha, eta}:
      shape_derivs <- list(
        list(dPhi = pd$dPhi_dlog_tau, dphi = pd$dphi_dlog_tau,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dlog_tau,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dlog_tau),
        list(dPhi = pd$dPhi_dgamma,   dphi = pd$dphi_dgamma,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dgamma,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dgamma),
        list(dPhi = pd$dPhi_dalpha,   dphi = pd$dphi_dalpha,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dalpha,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dalpha),
        list(dPhi = pd$dPhi_deta,     dphi = pd$dphi_deta,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_deta,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_deta)
      )

      for (s in seq_along(shape_derivs)) {
        dPhi_ds <- shape_derivs[[s]]$dPhi
        dphi_ds <- shape_derivs[[s]]$dphi

        dlogl_ds <- sum(w_H * mu_j * dPhi_ds) +
                    sum(inv_h[idx_event] * mu_j[idx_event] * dphi_ds[idx_event])

        # Counting-process start contribution
        if (need_start && !is.null(shape_derivs[[s]]$dPhi_start)) {
          dlogl_ds <- dlogl_ds +
            sum(w_H_start * mu_j * shape_derivs[[s]]$dPhi_start)
        }

        # Left-censoring correction: swap dPhi at time for dPhi at upper
        if (has_left && !is.null(shape_derivs[[s]]$dPhi_upper)) {
          dlogl_ds <- dlogl_ds +
            sum(w_H[idx_left] * mu_j[idx_left] * shape_derivs[[s]]$dPhi_upper) -
            sum(w_H[idx_left] * mu_j[idx_left] * dPhi_ds[idx_left])
        }

        grad[pos] <- dlogl_ds
        pos <- pos + 1L
      }
    }

    # -- Covariate coefficients beta_jk ----------------------------------
    # dH/d(beta_jk) = x_ik * mu_j_i * Phi_j_i
    # dh/d(beta_jk) = x_ik * mu_j_i * phi_j_i
    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      x_j <- x_list[[nm]]
      for (k in seq_along(pars$beta)) {
        x_k <- x_j[, k]
        dlogl_dbeta <- sum(w_H * x_k * mu_j * Phi_j) +
                       sum(inv_h[idx_event] * x_k[idx_event] *
                           mu_j[idx_event] * phi_j[idx_event])

        # Counting-process start contribution: +w_H_start * x_k * mu_j * Phi_start
        if (need_start) {
          dlogl_dbeta <- dlogl_dbeta +
            sum(w_H_start * x_k * mu_j * Phi_j_start)
        }

        if (has_left) {
          dlogl_dbeta <- dlogl_dbeta +
            sum(w_H[idx_left] * x_k[idx_left] *
                mu_j[idx_left] * Phi_j_upper) -
            sum(w_H[idx_left] * x_k[idx_left] *
                mu_j[idx_left] * Phi_j[idx_left])
        }

        grad[pos] <- dlogl_dbeta
        pos <- pos + 1L
      }
    } else {
      pos <- pos + covariate_counts[[nm]]
    }
  }

  # -- Interval-censored fallback: add correction via finite difference ---
  # Interval-censored observations are uncommon; their gradient contribution
  # is added via one-sided numerical difference on the weighted
  # log-likelihood of just those observations.
  if (has_interval) {
    w_iv <- weights[idx_interval]
    logl_iv <- function(th) {
      cumhaz_l <- .hzr_multiphase_cumhaz(lower, th, phases, covariate_counts,
                                          x_list)
      cumhaz_u <- .hzr_multiphase_cumhaz(upper, th, phases, covariate_counts,
                                          x_list)
      delta <- cumhaz_u[idx_interval] - cumhaz_l[idx_interval]
      sum(w_iv *
            (-cumhaz_l[idx_interval] + hzr_log1mexp(delta)))
    }
    eps_rel <- sqrt(.Machine$double.eps)
    ll0_iv <- logl_iv(theta)
    for (i in seq_len(p)) {
      h_i <- eps_rel * max(abs(theta[i]), 1)
      theta_p <- theta
      theta_p[i] <- theta_p[i] + h_i
      grad[i] <- grad[i] + (logl_iv(theta_p) - ll0_iv) / h_i
    }
  }

  # Safety: zero out non-finite entries
  grad[!is.finite(grad)] <- 0

  grad
}


# ============================================================================
# Optimizer
# ============================================================================

#' Fit a multiphase additive hazard model via maximum likelihood
#'
#' Assembles starting values from phase specifications, resolves per-phase
#' design matrices, and delegates to `.hzr_optim_generic()`.
#'
#' @param time Numeric vector of follow-up times.
#' @param status Numeric event indicator vector.
#' @param time_lower Optional lower bounds for interval censoring.
#' @param time_upper Optional upper bounds for left/interval censoring.
#' @param x Global design matrix (n x p) or NULL.
#' @param theta_start Starting parameter vector (full internal scale).
#'   If NULL, assembled automatically from phase specs.
#' @param control Named list of control options.
#' @param phases Named list of validated `hzr_phase` objects.
#' @param formula_global The global formula (used when phases have no
#'   phase-specific formula).
#' @param data Data frame containing covariates (needed for phase-specific
#'   formula evaluation).
#' @return List with par (internal scale), value, convergence, vcov, etc.
#' @keywords internal
.hzr_optim_multiphase <- function(time, status,
                                   time_lower = NULL, time_upper = NULL,
                                   x = NULL,
                                   theta_start = NULL,
                                   weights = NULL,
                                   control = list(),
                                   phases,
                                   formula_global = NULL,
                                   data = NULL) {

  if (is.null(weights)) weights <- rep(1, length(time))

  phases <- .hzr_validate_phases(phases)

  # --- Resolve per-phase design matrices and covariate counts ----------------
  # Build phase-specific design matrices with na.action = na.pass so that
  # rows with NA covariates are preserved (not silently dropped).  We then
  # align all observation-aligned inputs (time, status, weights, ...) by
  # removing any row that has NA in any phase's design matrix.  Without this
  # alignment, model.matrix()'s default na.omit shortens x_j relative to
  # time, and mu_j * phi_j emits a vector-recycling warning on every
  # likelihood evaluation.
  x_list <- vector("list", length(phases))
  names(x_list) <- names(phases)
  covariate_counts <- setNames(integer(length(phases)), names(phases))

  for (nm in names(phases)) {
    ph <- phases[[nm]]
    if (!is.null(ph$formula) && !is.null(data)) {
      # Phase-specific formula: build design matrix from data.  We route
      # through model.frame(na.action = na.pass) explicitly because
      # model.matrix() consults getOption("na.action") when it constructs
      # the frame internally, which defaults to na.omit and silently drops
      # rows with NA covariates.
      mf_j <- stats::model.frame(ph$formula, data = data,
                                   na.action = stats::na.pass)
      x_j <- stats::model.matrix(ph$formula, data = mf_j)[, -1L,
                                                           drop = FALSE]
      x_list[[nm]] <- x_j
      covariate_counts[[nm]] <- ncol(x_j)
    } else if (!is.null(x)) {
      # Inherit global design matrix
      x_list[[nm]] <- x
      covariate_counts[[nm]] <- ncol(x)
    } else {
      x_list[[nm]] <- NULL
      covariate_counts[[nm]] <- 0L
    }
  }

  # Align rows: drop any observation with an NA in any phase-specific design
  # matrix (or a row-count mismatch relative to time).
  n_time <- length(time)
  drop_rows <- logical(n_time)
  for (nm in names(phases)) {
    xm <- x_list[[nm]]
    if (is.null(xm) || ncol(xm) == 0L) next
    if (nrow(xm) != n_time) {
      stop("Design matrix for phase '", nm, "' has ", nrow(xm),
           " rows but length(time) is ", n_time,
           ". Check that 'data' matches 'time'/'status'.", call. = FALSE)
    }
    drop_rows <- drop_rows |
      apply(xm, 1L, function(row) any(is.na(row)))
  }
  if (any(drop_rows)) {
    keep <- !drop_rows
    time   <- time[keep]
    status <- status[keep]
    if (!is.null(time_lower)) time_lower <- time_lower[keep]
    if (!is.null(time_upper)) time_upper <- time_upper[keep]
    if (!is.null(weights))    weights    <- weights[keep]
    if (!is.null(x))          x          <- x[keep, , drop = FALSE]
    for (nm in names(phases)) {
      if (!is.null(x_list[[nm]])) {
        x_list[[nm]] <- x_list[[nm]][keep, , drop = FALSE]
      }
    }
  }

  # --- Assemble starting values if not provided ------------------------------
  if (is.null(theta_start)) {
    theta_start <- unlist(lapply(names(phases), function(nm) {
      .hzr_phase_start(phases[[nm]], n_covariates = covariate_counts[[nm]])
    }))
  }

  # Build parameter names for the full theta vector
  theta_names <- unlist(lapply(names(phases), function(nm) {
    cov_names <- if (covariate_counts[[nm]] > 0 && !is.null(x_list[[nm]])) {
      colnames(x_list[[nm]])
    } else {
      character(0)
    }
    # Ensure we have names even if colnames are NULL
    if (covariate_counts[[nm]] > 0 && length(cov_names) == 0) {
      cov_names <- paste0("x", seq_len(covariate_counts[[nm]]))
    }
    .hzr_phase_theta_names(phases[[nm]], nm, cov_names)
  }))

  names(theta_start) <- theta_names

  # --- Likelihood wrapper matching .hzr_optim_generic() signature -----------
  # NOTE: `weights` is an explicit formal so that .hzr_optim_generic can pass
  # it through without colliding with a closure capture via `...`
  # (which would trigger "formal argument 'weights' matched by multiple
  # actual arguments").
  logl_fn <- function(theta, time, status, time_lower, time_upper, x,
                      weights = NULL, ...) {
    .hzr_logl_multiphase(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights,
      phases = phases, covariate_counts = covariate_counts,
      x_list = x_list, ...
    )
  }

  # --- Analytic gradient (semi-analytic: chain-rule for mu/beta, central ----
  # differences for shape parameters via .hzr_phase_derivatives()).
  # Falls back to numerical gradient if analytic returns all zeros.
  #
  # IMPORTANT: Save a reference to the ORIGINAL (unwrapped) logl_fn for use

  # inside the gradient function's numerical fallback.  After this point,
  # logl_fn will be re-bound by CoE and fixed-mask wrappers, but the gradient
  # function always receives a FULL theta vector from its own wrapper chain,
  # so it must call the unwrapped version that also expects full theta.
  logl_fn_unwrapped <- logl_fn

  gradient_fn <- function(theta, time, status, time_lower, time_upper, x,
                          weights = NULL, ...) {
    grad <- .hzr_gradient_multiphase(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      phases = phases, covariate_counts = covariate_counts, x_list = x_list
    )

    # Fallback: if gradient is all zero (e.g. at infeasible point), try
    # numerical gradient of the *weighted* LL to keep optimizer moving.
    if (all(grad == 0)) {
      eps_rel <- sqrt(.Machine$double.eps)
      p <- length(theta)
      ll0 <- logl_fn_unwrapped(theta, time, status, time_lower,
                                time_upper, x, weights = weights, ...)
      for (i in seq_len(p)) {
        h_i <- eps_rel * max(abs(theta[i]), 1)
        theta_plus <- theta
        theta_plus[i] <- theta_plus[i] + h_i
        ll_plus <- logl_fn_unwrapped(theta_plus, time, status,
                                      time_lower, time_upper, x,
                                      weights = weights, ...)
        grad[i] <- (ll_plus - ll0) / h_i
      }
    }

    grad
  }

  # --- Conservation of Events (CoE) setup -------------------------------------
  use_conserve <- if (!is.null(control$conserve)) control$conserve else TRUE
  control$conserve <- NULL  # remove before passing to optim

  fixmu_phase <- NULL
  fixmu_pos <- NULL
  total_events <- NULL
  log_mu_positions <- .hzr_log_mu_positions(phases, covariate_counts)

  # CoE is only valid for right-censored + exact-event data.  Interval and
  # left censoring require a different event-counting formulation.
  coe_supported_data <- all(status %in% c(0, 1))

  if (use_conserve && coe_supported_data && length(phases) >= 2L) {
    total_events <- sum(weights[status == 1])
    if (total_events > 0) {
      # Initial CoE scaling: adjust all log_mu proportionally (weighted sums
      # so the scaling is consistent with total_events).
      decomp_init <- .hzr_multiphase_cumhaz(
        time, theta_start, phases, covariate_counts, x_list,
        per_phase = TRUE
      )
      sumcz_init <- sum(weights * decomp_init$total)
      if (sumcz_init > 0 && is.finite(sumcz_init)) {
        init_factor <- log(total_events / sumcz_init)
        if (is.finite(init_factor)) {
          for (nm in names(phases)) {
            theta_start[log_mu_positions[[nm]]] <-
              theta_start[log_mu_positions[[nm]]] + init_factor
          }
        }
      }

      # Select fixmu phase (largest weighted cumhaz contributor after scaling)
      fixmu_phase <- .hzr_select_fixmu_phase(
        theta_start, time, status, phases, covariate_counts, x_list,
        weights = weights
      )
      fixmu_pos <- log_mu_positions[[fixmu_phase]]

      # Apply precise CoE adjustment to the fixmu phase
      theta_start <- .hzr_conserve_events(
        theta_start, fixmu_phase, fixmu_pos,
        time, status, phases, covariate_counts, x_list, total_events,
        weights = weights
      )

      # Wrap logl and gradient: apply CoE before each evaluation
      logl_fn_pre_coe <- logl_fn
      logl_fn <- function(theta, time, status, time_lower,
                          time_upper, x, weights = NULL, ...) {
        theta <- .hzr_conserve_events(
          theta, fixmu_phase, fixmu_pos,
          time, status, phases, covariate_counts, x_list, total_events,
          weights = weights
        )
        logl_fn_pre_coe(theta, time, status, time_lower,
                        time_upper, x, weights = weights, ...)
      }

      gradient_fn_pre_coe <- gradient_fn
      gradient_fn <- function(theta, time, status, time_lower,
                              time_upper, x, weights = NULL, ...) {
        theta <- .hzr_conserve_events(
          theta, fixmu_phase, fixmu_pos,
          time, status, phases, covariate_counts, x_list, total_events,
          weights = weights
        )
        gradient_fn_pre_coe(theta, time, status, time_lower,
                            time_upper, x, weights = weights, ...)
      }
    } else {
      use_conserve <- FALSE
    }
  } else {
    use_conserve <- FALSE
  }

  # --- Fixed-parameter masking ------------------------------------------------
  # Build a logical mask: TRUE = free (optimized), FALSE = fixed (held constant)
  free_mask <- .hzr_phase_free_mask(phases, covariate_counts)

  # If CoE is active, also fix the fixmu phase's log_mu
  if (use_conserve && !is.null(fixmu_pos)) {
    free_mask[fixmu_pos] <- FALSE
  }

  any_fixed <- !all(free_mask)
  theta_fixed <- theta_start  # full vector; fixed entries stay at these values

  # When some parameters are fixed, wrap logl/gradient to operate on the
  # reduced (free-only) parameter vector.
  if (any_fixed) {
    free_idx <- which(free_mask)

    # Expand reduced theta to full theta
    expand_theta <- function(theta_free) {
      theta_full <- theta_fixed
      theta_full[free_idx] <- theta_free
      theta_full
    }

    logl_fn_full <- logl_fn
    logl_fn <- function(theta, time, status, time_lower, time_upper, x,
                        weights = NULL, ...) {
      logl_fn_full(expand_theta(theta), time, status, time_lower,
                   time_upper, x, weights = weights, ...)
    }

    gradient_fn_full <- gradient_fn
    gradient_fn <- function(theta, time, status, time_lower, time_upper, x,
                            weights = NULL, ...) {
      grad_full <- gradient_fn_full(expand_theta(theta), time, status,
                                     time_lower, time_upper, x,
                                     weights = weights, ...)
      grad_full[free_idx]
    }

    theta_start_optim <- theta_start[free_idx]
  } else {
    theta_start_optim <- theta_start
  }

  # --- Multi-start optimization -----------------------------------------------
  n_starts <- if (!is.null(control$n_starts)) control$n_starts else 5L
  control$n_starts <- NULL  # remove before passing to optim

  best_result <- NULL
  best_value <- -Inf

  # When shapes are fixed the free-parameter count is small (typically just the

  # mu intercepts).  BFGS can struggle on this reduced problem because the
  # likelihood surface is poorly scaled across phases (e.g. the late-phase mu
  # may be ~1e-8).  Strategy: run Nelder-Mead first (derivative-free, robust to
  # scaling) to find a good basin, then polish with BFGS for accurate SE.
  # Nelder-Mead requires >= 2 parameters; use Brent for 1D
  use_nelder_mead_warmup <- any_fixed && length(theta_start_optim) >= 2L &&
    length(theta_start_optim) <= 10L

  for (start_i in seq_len(n_starts)) {
    if (start_i == 1L) {
      theta_try <- theta_start_optim
    } else {
      # Deterministic perturbation seeded from start index for reproducibility
      theta_try <- theta_start_optim +
        stats::rnorm(length(theta_start_optim), sd = 0.5)
    }

    # Nelder-Mead warm-up when shapes are fixed
    if (use_nelder_mead_warmup) {
      # Negative log-likelihood for minimization (matches .hzr_optim_generic)
      nm_objective <- function(theta) {
        if (!all(is.finite(theta))) return(1e10)
        ll <- -logl_fn(
          theta = theta, time = time, status = status,
          time_lower = time_lower, time_upper = time_upper,
          x = x, weights = weights, return_gradient = FALSE
        )
        if (!is.finite(ll)) return(1e10)
        ll
      }
      nm_result <- tryCatch(
        stats::optim(
          par = theta_try, fn = nm_objective,
          method = "Nelder-Mead",
          control = list(maxit = 5000L, reltol = 1e-10)
        ),
        error = function(e) NULL
      )
      if (!is.null(nm_result) && is.finite(nm_result$value)) {
        theta_try <- nm_result$par
      }
    }

    result <- tryCatch(
      .hzr_optim_generic(
        logl_fn     = logl_fn,
        gradient_fn = gradient_fn,
        time        = time, status = status,
        time_lower  = time_lower, time_upper = time_upper,
        x           = x,
        theta_start = theta_try,
        weights     = weights,
        control     = control,
        use_bounds  = FALSE
      ),
      error = function(e) NULL
    )

    if (!is.null(result) && is.finite(result$value) && result$value > best_value) {
      best_value <- result$value
      best_result <- result
    }
  }

  if (is.null(best_result)) {
    stop("Multiphase optimization failed to converge on any start.", call. = FALSE)
  }

  # Expand optimized free params back to full theta vector
  if (any_fixed) {
    theta_full <- theta_fixed
    theta_full[free_idx] <- best_result$par
    best_result$par <- theta_full

    # Final CoE adjustment: solve fixmu log_mu at the optimum
    if (use_conserve && !is.null(fixmu_pos)) {
      best_result$par <- .hzr_conserve_events(
        best_result$par, fixmu_phase, fixmu_pos,
        time, status, phases, covariate_counts, x_list, total_events,
        weights = weights
      )
    }

    # Expand vcov to full dimension (NA for fixed params -- not estimated)
    if (!is.null(best_result$vcov) && is.matrix(best_result$vcov)) {
      p_full <- length(theta_fixed)
      vcov_full <- matrix(NA_real_, p_full, p_full)
      vcov_full[free_idx, free_idx] <- best_result$vcov
      best_result$vcov <- vcov_full
    }

    best_result$fixed_mask <- !free_mask
  }

  # Restore parameter names
  names(best_result$par) <- theta_names

  # Store phase metadata for downstream use (predict, summary)
  best_result$phases <- phases
  best_result$covariate_counts <- covariate_counts
  best_result$x_list <- x_list

  best_result
}


# ============================================================================
# G3 phase derivatives (central finite differences)
# ============================================================================

#' Finite-difference derivatives of G3 phase Phi and phi w.r.t. shape params
#'
#' Computes the G3 cumulative intensity, its time derivative, and their
#' partial derivatives with respect to log_tau, gamma, alpha, and eta using
#' central finite differences.
#'
#' @param time Numeric vector of positive times.
#' @param tau Positive scalar scale parameter.
#' @param gamma Positive scalar time exponent.
#' @param alpha Non-negative scalar shape parameter.
#' @param eta Positive scalar outer exponent.
#' @param h Relative step size for finite differences (default 1e-5).
#' @return Named list with Phi, phi, and 8 derivative vectors.
#' @keywords internal
.hzr_g3_phase_derivatives <- function(time, tau, gamma, alpha, eta,
                                       h = 1e-5) {

  # Base evaluation
  d0 <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                          alpha = alpha, eta = eta)
  Phi0 <- d0$G3
  phi0 <- d0$g3

  # Central differences for log_tau: tau * d/d(tau) = d/d(log(tau))
  eps_tau <- max(abs(tau) * h, 1e-10)
  if (tau - eps_tau > 0) {
    d_plus  <- hzr_decompos_g3(time, tau = tau + eps_tau, gamma = gamma,
                                 alpha = alpha, eta = eta)
    d_minus <- hzr_decompos_g3(time, tau = tau - eps_tau, gamma = gamma,
                                 alpha = alpha, eta = eta)
    dPhi_dtau <- (d_plus$G3 - d_minus$G3) / (2 * eps_tau)
    dphi_dtau <- (d_plus$g3 - d_minus$g3) / (2 * eps_tau)
  } else {
    d_plus  <- hzr_decompos_g3(time, tau = tau + eps_tau, gamma = gamma,
                                 alpha = alpha, eta = eta)
    dPhi_dtau <- (d_plus$G3 - Phi0) / eps_tau
    dphi_dtau <- (d_plus$g3 - phi0) / eps_tau
  }
  # Chain rule: d/d(log_tau) = tau * d/d(tau)
  dPhi_dlog_tau <- tau * dPhi_dtau
  dphi_dlog_tau <- tau * dphi_dtau

  # Central differences for gamma (must stay positive)
  eps_g <- max(abs(gamma) * h, 1e-10)
  if (gamma - eps_g > 0) {
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma + eps_g,
                                 alpha = alpha, eta = eta)
    d_minus <- hzr_decompos_g3(time, tau = tau, gamma = gamma - eps_g,
                                 alpha = alpha, eta = eta)
    dPhi_dgamma <- (d_plus$G3 - d_minus$G3) / (2 * eps_g)
    dphi_dgamma <- (d_plus$g3 - d_minus$g3) / (2 * eps_g)
  } else {
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma + eps_g,
                                 alpha = alpha, eta = eta)
    dPhi_dgamma <- (d_plus$G3 - Phi0) / eps_g
    dphi_dgamma <- (d_plus$g3 - phi0) / eps_g
  }

  # Central differences for alpha
  if (alpha > h) {
    eps_a <- max(abs(alpha) * h, 1e-10)
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha + eps_a, eta = eta)
    d_minus <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha - eps_a, eta = eta)
    dPhi_dalpha <- (d_plus$G3 - d_minus$G3) / (2 * eps_a)
    dphi_dalpha <- (d_plus$g3 - d_minus$g3) / (2 * eps_a)
  } else {
    # alpha near 0: use forward difference
    eps_a <- h
    d_plus <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                alpha = alpha + eps_a, eta = eta)
    dPhi_dalpha <- (d_plus$G3 - Phi0) / eps_a
    dphi_dalpha <- (d_plus$g3 - phi0) / eps_a
  }

  # Central differences for eta (must stay positive)
  eps_e <- max(abs(eta) * h, 1e-10)
  if (eta - eps_e > 0) {
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha, eta = eta + eps_e)
    d_minus <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha, eta = eta - eps_e)
    dPhi_deta <- (d_plus$G3 - d_minus$G3) / (2 * eps_e)
    dphi_deta <- (d_plus$g3 - d_minus$g3) / (2 * eps_e)
  } else {
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha, eta = eta + eps_e)
    dPhi_deta <- (d_plus$G3 - Phi0) / eps_e
    dphi_deta <- (d_plus$g3 - phi0) / eps_e
  }

  list(
    Phi = Phi0,
    phi = phi0,
    dPhi_dlog_tau = dPhi_dlog_tau,
    dPhi_dgamma   = dPhi_dgamma,
    dPhi_dalpha   = dPhi_dalpha,
    dPhi_deta     = dPhi_deta,
    dphi_dlog_tau = dphi_dlog_tau,
    dphi_dgamma   = dphi_dgamma,
    dphi_dalpha   = dphi_dalpha,
    dphi_deta     = dphi_deta
  )
}


# ============================================================================
# Back-transformation helpers
# ============================================================================

#' Transform multiphase theta from internal to user-facing scale
#'
#' Converts log_mu -> mu, log_t_half -> t_half; nu, m, beta pass through.
#'
#' @param theta Full parameter vector (internal scale).
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of per-phase covariate counts.
#' @return Named numeric vector on the reporting scale.
#' @keywords internal
.hzr_multiphase_theta_user <- function(theta, phases, covariate_counts) {
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  values <- numeric(0)
  nms    <- character(0)

  for (nm in names(phases)) {
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    if (phases[[nm]]$type == "constant") {
      values <- c(values, exp(pars$log_mu))
      nms    <- c(nms, paste0(nm, ".mu"))
    } else if (phases[[nm]]$type == "g3") {
      values <- c(values, exp(pars$log_mu), exp(pars$log_tau),
                  pars$gamma, pars$alpha, pars$eta)
      nms    <- c(nms, paste0(nm, ".mu"), paste0(nm, ".tau"),
                  paste0(nm, ".gamma"), paste0(nm, ".alpha"), paste0(nm, ".eta"))
    } else {
      values <- c(values, exp(pars$log_mu), exp(pars$log_t_half),
                  pars$nu, pars$m)
      nms    <- c(nms, paste0(nm, ".mu"), paste0(nm, ".t_half"),
                  paste0(nm, ".nu"), paste0(nm, ".m"))
    }

    if (length(pars$beta) > 0) {
      values <- c(values, pars$beta)
      # Try to derive covariate names from the theta_split names
      n_skip <- switch(phases[[nm]]$type, constant = 1L, g3 = 5L, 4L)
      raw_names <- names(theta_split[[nm]])
      if (!is.null(raw_names) && length(raw_names) > n_skip) {
        beta_labels <- raw_names[(n_skip + 1L):length(raw_names)]
        # Strip phase prefix if present
        beta_labels <- sub(paste0("^", nm, "\\."), "", beta_labels)
      } else {
        beta_labels <- paste0("x", seq_along(pars$beta))
      }
      nms <- c(nms, paste0(nm, ".", beta_labels))
    }
  }

  names(values) <- nms
  values
}

# predict(type = "hazard") for multiphase models: the instantaneous additive
# hazard h(t|x) = sum_j mu_j(x) * phi_j'(t). Single-distribution "hazard"
# (exp(eta)) is unchanged; this covers the multiphase path added so the
# instantaneous hazard has a public predict() route (previously internal only).

.ph_fit <- function(formula = survival::Surv(time, status) ~ 1, data, ...) {
  set.seed(1)
  hazard(formula, data = data, dist = "multiphase",
         phases = list(
           early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                                fixed = "shapes", ...),
           constant = hzr_phase("constant")),
         fit = TRUE, control = list(n_starts = 1, maxit = 500, conserve = TRUE))
}

.ph_data <- function(n = 300, seed = 3) {
  set.seed(seed)
  x <- rnorm(n)
  t <- rexp(n, 0.3) + 0.01
  cens <- runif(n, 0.2, 6)
  data.frame(time = pmin(t, cens), status = as.integer(t <= cens), x = x)
}

test_that("multiphase predict(type = 'hazard') returns the additive hazard", {
  d <- .ph_data()
  fit <- .ph_fit(data = d)
  grid <- data.frame(time = seq(0.05, 5, length.out = 25))

  haz <- predict(fit, newdata = grid, type = "hazard")
  ref <- TemporalHazard:::.hzr_multiphase_hazard(
    grid$time, fit$fit$theta, fit$fit$phases,
    fit$fit$covariate_counts, fit$fit$x_list)

  expect_equal(unname(haz), unname(ref), tolerance = 1e-10)
  expect_true(all(haz >= 0))
  expect_equal(length(haz), nrow(grid))
})

test_that("multiphase hazard requires time and rejects linear_predictor", {
  d <- .ph_data()
  fit <- .ph_fit(data = d)
  expect_error(predict(fit, newdata = data.frame(z = 1), type = "hazard"),
               "time")
  expect_error(predict(fit, type = "linear_predictor"),
               "linear_predictor")
})

test_that("multiphase hazard supports covariate newdata", {
  d <- .ph_data()
  set.seed(1)
  fit <- hazard(survival::Surv(time, status) ~ 1, data = d, dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                           fixed = "shapes", formula = ~ x),
      constant = hzr_phase("constant")),
    fit = TRUE, control = list(n_starts = 1, maxit = 500, conserve = TRUE))
  h0 <- predict(fit, newdata = data.frame(time = 1, x = 0), type = "hazard")
  h1 <- predict(fit, newdata = data.frame(time = 1, x = 1), type = "hazard")
  expect_true(is.finite(h0) && is.finite(h1))
  expect_true(h0 > 0 && h1 > 0)
  expect_false(isTRUE(all.equal(unname(h0), unname(h1))))  # covariate matters
})

test_that("multiphase hazard se.fit returns delta-method limits", {
  skip_if_not_installed("numDeriv")
  d <- .ph_data()
  fit <- .ph_fit(data = d)
  grid <- data.frame(time = c(0.1, 0.5, 1, 3))

  out <- predict(fit, newdata = grid, type = "hazard", se.fit = TRUE)
  expect_s3_class(out, "data.frame")
  expect_named(out, c("fit", "se.fit", "lower", "upper"))
  # Point estimate matches the non-se path.
  pt <- predict(fit, newdata = grid, type = "hazard")
  expect_equal(out$fit, unname(pt), tolerance = 1e-8)
  expect_true(all(is.finite(out$se.fit)) && all(out$se.fit > 0))
  expect_true(all(out$lower <= out$fit + 1e-9) &&
              all(out$upper >= out$fit - 1e-9))
  expect_true(all(out$lower >= 0))  # log-scale CL keeps hazard positive
})

test_that("multiphase hazard rejects decompose = TRUE", {
  d <- .ph_data()
  fit <- .ph_fit(data = d)
  expect_error(
    predict(fit, newdata = data.frame(time = 1), type = "hazard",
            decompose = TRUE),
    "decompose")
})

# test-multiphase-likelihood.R — Tests for multiphase likelihood engine
#
# Tests theta splitting, cumhaz/hazard computation, log-likelihood
# evaluation at known parameters, and optimizer convergence on synthetic data.

# ============================================================================
# Helpers
# ============================================================================

# Simple synthetic data: exponential-ish with known rate
make_test_data <- function(n = 200, seed = 42) {
  set.seed(seed)
  time   <- rexp(n, rate = 0.3) + 0.01
  status <- rbinom(n, 1, prob = 0.7)
  age    <- rnorm(n, mean = 60, sd = 10)
  data.frame(time = time, status = status, age = age)
}


# ============================================================================
# .hzr_split_theta()
# ============================================================================

test_that(".hzr_split_theta partitions correctly for 2-phase model", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    late  = hzr_phase("hazard")
  ))
  cov_counts <- c(early = 0L, late = 0L)

  # early: log_mu + log_t_half + nu + m = 4

  # late:  log_mu + log_t_half + nu + m = 4
  theta <- 1:8
  split <- .hzr_split_theta(theta, phases, cov_counts)

  expect_equal(split$early, 1:4)
  expect_equal(split$late, 5:8)
})

test_that(".hzr_split_theta handles constant phase (1 param)", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    bg    = hzr_phase("constant"),
    late  = hzr_phase("hazard")
  ))
  cov_counts <- c(early = 0L, bg = 0L, late = 0L)

  # early: 4, bg: 1 (log_mu only), late: 4 = total 9
  theta <- 1:9
  split <- .hzr_split_theta(theta, phases, cov_counts)

  expect_equal(split$early, 1:4)
  expect_equal(split$bg, 5)
  expect_equal(split$late, 6:9)
})

test_that(".hzr_split_theta includes covariates", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    late  = hzr_phase("hazard")
  ))
  cov_counts <- c(early = 2L, late = 1L)

  # early: 4 + 2 = 6, late: 4 + 1 = 5, total = 11
  theta <- seq(0.1, 1.1, by = 0.1)
  split <- .hzr_split_theta(theta, phases, cov_counts)

  expect_equal(length(split$early), 6)
  expect_equal(length(split$late), 5)
})


# ============================================================================
# .hzr_unpack_phase_theta()
# ============================================================================

test_that("unpack_phase_theta works for cdf phase", {
  phase <- hzr_phase("cdf")
  theta_j <- c(-2.3, log(2), 1.5, 0.3)  # log_mu, log_t_half, nu, m
  pars <- .hzr_unpack_phase_theta(theta_j, phase)

  expect_equal(pars$log_mu, -2.3)
  expect_equal(pars$log_t_half, log(2))
  expect_equal(pars$nu, 1.5)
  expect_equal(pars$m, 0.3)
  expect_equal(length(pars$beta), 0)
})

test_that("unpack_phase_theta works for constant phase", {
  phase <- hzr_phase("constant")
  theta_j <- c(-1.5, 0.02)  # log_mu, beta_1
  pars <- .hzr_unpack_phase_theta(theta_j, phase)

  expect_equal(pars$log_mu, -1.5)
  expect_true(is.na(pars$log_t_half))
  expect_true(is.na(pars$nu))
  expect_true(is.na(pars$m))
  expect_equal(pars$beta, 0.02)
})


# ============================================================================
# .hzr_multiphase_cumhaz() — known-parameter evaluation
# ============================================================================

test_that("single constant phase reduces to mu * t", {
  phases <- .hzr_validate_phases(list(bg = hzr_phase("constant")))
  cov_counts <- c(bg = 0L)
  x_list <- list(bg = NULL)

  # log_mu = log(0.5) => mu = 0.5
  theta <- log(0.5)
  time <- c(1, 2, 5)

  cumhaz <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list)
  expect_equal(cumhaz, 0.5 * time, tolerance = 1e-12)
})

test_that("single cdf phase produces mu * G(t)", {
  phases <- .hzr_validate_phases(list(early = hzr_phase("cdf")))
  cov_counts <- c(early = 0L)
  x_list <- list(early = NULL)

  # log_mu = 0 => mu = 1; log_t_half = log(3); nu = 2; m = 0
  theta <- c(0, log(3), 2, 0)
  time <- seq(0.5, 10, by = 0.5)

  cumhaz <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list)
  expected_G <- hzr_decompos(time, t_half = 3, nu = 2, m = 0)$G
  expect_equal(cumhaz, expected_G, tolerance = 1e-10)
})

test_that("two-phase additive: constant + cdf", {
  phases <- .hzr_validate_phases(list(
    bg    = hzr_phase("constant"),
    early = hzr_phase("cdf")
  ))
  cov_counts <- c(bg = 0L, early = 0L)
  x_list <- list(bg = NULL, early = NULL)

  # bg: log_mu = log(0.1) => mu_bg = 0.1
  # early: log_mu = log(0.5), log_t_half = log(2), nu = 1, m = 0
  theta <- c(log(0.1), log(0.5), log(2), 1, 0)
  time <- c(1, 5, 10)

  cumhaz <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list)
  G <- hzr_decompos(time, t_half = 2, nu = 1, m = 0)$G
  expected <- 0.1 * time + 0.5 * G
  expect_equal(cumhaz, expected, tolerance = 1e-10)
})

test_that("per_phase = TRUE returns decomposed contributions", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    late  = hzr_phase("hazard")
  ))
  cov_counts <- c(early = 0L, late = 0L)
  x_list <- list(early = NULL, late = NULL)

  theta <- c(log(0.5), log(2), 2, 0,   # early
             log(0.3), log(5), 1, 0)    # late

  time <- c(1, 3, 7)
  result <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list,
                                     per_phase = TRUE)

  expect_true(is.list(result))
  expect_true("total" %in% names(result))
  expect_true("early" %in% names(result))
  expect_true("late" %in% names(result))
  expect_equal(result$total, result$early + result$late, tolerance = 1e-12)
})


# ============================================================================
# .hzr_multiphase_hazard()
# ============================================================================

test_that("constant phase hazard is mu * 1 = mu", {
  phases <- .hzr_validate_phases(list(bg = hzr_phase("constant")))
  cov_counts <- c(bg = 0L)
  x_list <- list(bg = NULL)

  theta <- log(0.5)
  time <- c(1, 2, 5)

  haz <- .hzr_multiphase_hazard(time, theta, phases, cov_counts, x_list)
  expect_equal(haz, rep(0.5, 3), tolerance = 1e-12)
})


# ============================================================================
# .hzr_logl_multiphase() — basic properties
# ============================================================================

test_that("logl is finite for valid parameters", {
  dat <- make_test_data(n = 50)
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf", t_half = 2, nu = 1, m = 0)
  ))
  cov_counts <- c(early = 0L)
  x_list <- list(early = NULL)

  theta <- c(log(0.1), log(2), 1, 0)

  ll <- .hzr_logl_multiphase(
    theta = theta, time = dat$time, status = dat$status,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )

  expect_true(is.finite(ll))
  expect_true(ll < 0)  # log-likelihood should be negative
})

test_that("logl returns -Inf for m < 0 and nu < 0", {
  dat <- make_test_data(n = 50)
  phases <- .hzr_validate_phases(list(
    bad = hzr_phase("cdf", t_half = 2, nu = 1, m = 0)
  ))
  cov_counts <- c(bad = 0L)
  x_list <- list(bad = NULL)

  # Force invalid: nu = -1 and m = -1 in the theta vector
  theta <- c(log(0.1), log(2), -1, -1)

  ll <- .hzr_logl_multiphase(
    theta = theta, time = dat$time, status = dat$status,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )

  expect_equal(ll, -Inf)
})

test_that("logl is higher at better parameters (constant-rate sanity check)", {
  # Generate data from a known constant-rate model: H(t) = lambda * t
  set.seed(99)
  true_lambda <- 0.3
  n <- 200
  time <- rexp(n, rate = true_lambda) + 0.01
  status <- rep(1L, n)  # all events

  phases <- .hzr_validate_phases(list(bg = hzr_phase("constant")))
  cov_counts <- c(bg = 0L)
  x_list <- list(bg = NULL)

  # True parameter
  ll_true <- .hzr_logl_multiphase(
    theta = log(true_lambda), time = time, status = status,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )

  # Wrong parameter (much higher rate)
  ll_wrong <- .hzr_logl_multiphase(
    theta = log(5.0), time = time, status = status,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )

  expect_true(ll_true > ll_wrong)
})


# ============================================================================
# .hzr_multiphase_theta_user() — back-transformation
# ============================================================================

test_that("theta_user back-transforms correctly", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    bg    = hzr_phase("constant")
  ))
  cov_counts <- c(early = 0L, bg = 0L)

  # internal: [log(0.5), log(3), 2, 0.1, log(0.2)]
  theta_int <- c(log(0.5), log(3), 2, 0.1, log(0.2))
  theta_usr <- .hzr_multiphase_theta_user(theta_int, phases, cov_counts)

  expect_equal(unname(theta_usr[1]), 0.5, tolerance = 1e-10)  # mu early
  expect_equal(unname(theta_usr[2]), 3.0, tolerance = 1e-10)  # t_half early
  expect_equal(unname(theta_usr[3]), 2.0)                     # nu early
  expect_equal(unname(theta_usr[4]), 0.1)                     # m early
  expect_equal(unname(theta_usr[5]), 0.2, tolerance = 1e-10)  # mu bg
})


# ============================================================================
# .hzr_optim_multiphase() — convergence on simple data
# ============================================================================

test_that("optimizer recovers constant rate from exponential data", {
  skip_on_cran()  # optimization test — may be slow

  set.seed(123)
  true_lambda <- 0.4
  n <- 300
  dat <- data.frame(
    time   = rexp(n, rate = true_lambda) + 0.01,
    status = rep(1L, n)
  )

  phases <- list(bg = hzr_phase("constant"))

  result <- .hzr_optim_multiphase(
    time = dat$time, status = dat$status,
    phases = phases,
    control = list(n_starts = 3, maxit = 500)
  )

  expect_equal(result$convergence, 0)

  # Recover mu ~ true_lambda (on internal scale, par is log_mu)
  mu_hat <- unname(exp(result$par[1]))
  expect_equal(mu_hat, true_lambda, tolerance = 0.15)
})

test_that("optimizer handles two phases (cdf + constant)", {
  skip_on_cran()

  # Simulate: H(t) = 0.3 * G(t; t_half=2, nu=1, m=0) + 0.1 * t
  set.seed(456)
  n <- 400
  t_grid <- seq(0.01, 15, length.out = n)

  # Compute survival under the model and simulate events
  G <- hzr_decompos(t_grid, t_half = 2, nu = 1, m = 0)$G
  cumhaz_true <- 0.3 * G + 0.1 * t_grid
  surv_true <- exp(-cumhaz_true)

  # Simple simulation: event if uniform < 1 - S(t)
  set.seed(789)
  u <- runif(n)
  status <- as.integer(u > surv_true)

  dat <- data.frame(time = t_grid, status = status)

  phases <- list(
    early = hzr_phase("cdf", t_half = 2, nu = 1, m = 0),
    bg    = hzr_phase("constant")
  )

  result <- suppressWarnings(
    .hzr_optim_multiphase(
      time = dat$time, status = dat$status,
      phases = phases,
      control = list(n_starts = 3, maxit = 1000)
    )
  )

  expect_true(is.finite(result$value))
  expect_equal(result$convergence, 0)

  # At minimum, the log-likelihood should be better than starting values
  theta_start <- unlist(lapply(names(.hzr_validate_phases(phases)), function(nm) {
    .hzr_phase_start(.hzr_validate_phases(phases)[[nm]])
  }))
  ll_start <- .hzr_logl_multiphase(
    theta = theta_start, time = dat$time, status = dat$status,
    phases = .hzr_validate_phases(phases),
    covariate_counts = c(early = 0L, bg = 0L),
    x_list = list(early = NULL, bg = NULL)
  )
  expect_true(result$value >= ll_start)
})


# ============================================================================
# Regression: 3+ covariate phase must not emit vector-recycling warnings
# ============================================================================

test_that("3-covariate phase does not emit mu_j * phi_j recycling warning", {
  # Reproduces the issue where model.matrix() silently drops NA rows in a
  # phase-specific formula, leaving x_list[[nm]] shorter than time and
  # triggering "longer object length is not a multiple of shorter object
  # length" on every mu_j * phi_j multiplication inside
  # .hzr_multiphase_cumhaz() / .hzr_multiphase_hazard().  The avc dataset
  # has 5 NAs in inc_surg, so a 3-covariate phase referencing it would
  # produce a 305-row design matrix against a 310-row time vector.
  skip_if_not(
    tryCatch({
      data("avc", package = "TemporalHazard", envir = environment())
      exists("avc", inherits = FALSE)
    }, error = function(e) FALSE),
    "avc dataset not available"
  )

  expect_no_warning(
    fit <- hazard(
      survival::Surv(int_dead, dead) ~ 1, data = avc,
      dist = "multiphase",
      phases = list(
        early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                              fixed = "shapes",
                              formula = ~ age + mal + inc_surg),
        constant = hzr_phase("constant")
      ),
      fit = TRUE,
      control = list(n_starts = 1L, maxit = 10L)
    )
  )
})

# test-multiphase-parity.R — C binary parity and round-trip tests for multiphase
#
# Tests the R multiphase implementation against:
#   1. C HAZARD binary reference output (KUL CABG dataset)
#   2. Synthetic round-trip recovery
#
# The C reference model (hz.deadp.KUL) has ALL shapes fixed and only
# estimates 3 log(mu) parameters.  Our R implementation estimates shapes
# too, so parity tests focus on:
#   - Log-likelihood evaluation at C parameters (exact match)
#   - Full fit convergence with informed starting values
#   - Predict consistency

# ============================================================================
# Helpers
# ============================================================================

load_kul_csv <- function() {
  csv_path <- system.file("extdata", "cabgkul.csv", package = "TemporalHazard")
  if (!nzchar(csv_path) || !file.exists(csv_path)) {
    csv_path <- file.path("inst", "extdata", "cabgkul.csv")
  }
  if (!file.exists(csv_path)) return(NULL)
  utils::read.csv(csv_path)
}

load_synthetic_fixture <- function() {
  fixture_file <- system.file("fixtures", "mp_synthetic_3phase.rds",
                              package = "TemporalHazard")
  if (!nzchar(fixture_file) || !file.exists(fixture_file)) {
    fixture_file <- file.path("inst", "fixtures", "mp_synthetic_3phase.rds")
  }
  if (!file.exists(fixture_file)) return(NULL)
  readRDS(fixture_file)
}

# Starting theta on internal scale, matching SAS PARMS from hz.deadp.KUL.sas:
#   MUE=0.02, THALF=0.2, NU=1, M=1, MUC=0.0008, MUL=1E-9, TAU=1, GAMMA=3, ALPHA=1, ETA=1
# Internal layout: [log_mu, log_t_half, nu, m] per cdf/hazard phase,
#                  [log_mu] for constant,
#                  [log_mu, log_tau, gamma, alpha, eta] for g3 phase
kul_theta_start <- c(
  log(0.02),     # early.log_mu
  log(0.2),      # early.log_t_half
  1,             # early.nu
  1,             # early.m
  log(0.0008),   # constant.log_mu
  log(1e-9),     # late.log_mu
  log(1),        # late.log_tau
  3,             # late.gamma
  1,             # late.alpha
  1              # late.eta
)

kul_phases <- function() {
  list(
    early    = hzr_phase("cdf",  t_half = 0.2, nu = 1, m = 1),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",   tau = 1, gamma = 3, alpha = 1, eta = 1)
  )
}


# ============================================================================
# C binary parity: log-likelihood at reference parameters
# ============================================================================

test_that("R log-likelihood at C reference parameters matches C output", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  time   <- dat$int_dead
  status <- dat$dead

  phases <- kul_phases()
  covariate_counts <- c(early = 0L, constant = 0L, late = 0L)
  x_list <- list(early = NULL, constant = NULL, late = NULL)

  # C converged parameters on internal scale
  theta_c <- c(
    -3.77955,     # early.log_mu  (E0)
    log(0.2),     # early.log_t_half
    1,            # early.nu
    1,            # early.m
    -7.2258,      # constant.log_mu (C0)
    -16.6578,     # late.log_mu (L0)
    log(1),       # late.log_tau
    3,            # late.gamma
    1,            # late.alpha
    1             # late.eta
  )

  logl_r <- .hzr_logl_multiphase(
    theta = theta_c,
    time = time,
    status = status,
    phases = phases,
    covariate_counts = covariate_counts,
    x_list = x_list
  )

  expect_true(is.finite(logl_r))

  # C reference log-likelihood: -3740.52
  # R now uses G3 decomposition for the late phase, matching C.
  expect_equal(
    logl_r,
    -3740.52,
    tolerance = 0.01,
    label = "R logl at C params matches C reference"
  )
})


test_that("R cumulative hazard decomposition is consistent at C parameters", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  phases <- kul_phases()
  covariate_counts <- c(early = 0L, constant = 0L, late = 0L)
  x_list <- list(early = NULL, constant = NULL, late = NULL)

  theta_c <- c(
    -3.77955, log(0.2), 1, 1,
    -7.2258,
    -16.6578, log(1), 3, 1, 1
  )

  t_grid <- c(0.1, 0.5, 1, 2, 5, 10, 20, 50, 100, 200)

  decomp <- .hzr_multiphase_cumhaz(
    time = t_grid,
    theta = theta_c,
    phases = phases,
    covariate_counts = covariate_counts,
    x_list = x_list,
    per_phase = TRUE
  )

  # All contributions should be non-negative
  expect_true(all(decomp$early >= 0))
  expect_true(all(decomp$constant >= 0))
  expect_true(all(decomp$late >= 0))

  # Total should equal sum of components
  expect_equal(
    decomp$total,
    decomp$early + decomp$constant + decomp$late,
    tolerance = 1e-12
  )

  # Survival = exp(-H) should be in [0, 1]
  surv <- exp(-decomp$total)
  expect_true(all(surv >= 0 & surv <= 1))

  # Early contribution should saturate (CDF bounded by 1 * mu_early)
  mu_early <- exp(-3.77955)
  expect_true(all(decomp$early <= mu_early + 1e-10))

  # Constant grows linearly with time
  mu_const <- exp(-7.2258)
  expect_equal(decomp$constant, mu_const * t_grid, tolerance = 1e-10)
})


test_that("Conservation of events at C reference parameters", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  time   <- dat$int_dead
  status <- dat$dead

  phases <- kul_phases()
  covariate_counts <- c(early = 0L, constant = 0L, late = 0L)
  x_list <- list(early = NULL, constant = NULL, late = NULL)

  theta_c <- c(
    -3.77955, log(0.2), 1, 1,
    -7.2258,
    -16.6578, log(1), 3, 1, 1
  )

  cumhaz <- .hzr_multiphase_cumhaz(
    time = time,
    theta = theta_c,
    phases = phases,
    covariate_counts = covariate_counts,
    x_list = x_list
  )

  # C reference reports: "Number of events conserved = 544.9993"
  conserved <- sum(1 - exp(-cumhaz))
  expect_equal(conserved, 545, tolerance = 2)
})


# ============================================================================
# C binary parity: full model fit on KUL data
# ============================================================================

test_that("Full multiphase fit on KUL data converges", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  # Provide starting theta from SAS PARMS so the optimizer begins in the

  # right basin.  Without this, the default mu_start = 0.01 for all phases
  # is far from the late-phase mu ≈ 5.8e-8 (log_mu ≈ -16.7).
  fit <- hazard(
    time   = dat$int_dead,
    status = dat$dead,
    dist   = "multiphase",
    theta  = kul_theta_start,
    phases = kul_phases(),
    fit = TRUE,
    control = list(n_starts = 10, maxit = 2000, reltol = 1e-10)
  )

  expect_s3_class(fit, "hazard")
  expect_true(is.finite(fit$fit$objective))

  # With 10 free parameters (shapes free), the optimizer can improve
  # beyond the shapes-fixed solution.  Should reach at least C's -3740.52.
  expect_true(fit$fit$objective >= -3745,
              label = paste("R logl", round(fit$fit$objective, 2),
                            ">= -3745"))

  # Phase names should be preserved
  expect_equal(names(fit$spec$phases), c("early", "constant", "late"))
  expect_equal(length(fit$fit$theta), 10)  # 4 + 1 + 5
})

# ============================================================================
# Shape-fixed fit: matches C binary workflow (only mu estimated)
# ============================================================================

test_that("KUL fit with fixed shapes: mechanism works correctly", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  # C binary fixes all shapes and only estimates 3 mu parameters.
  # Replicate this by using fixed = "shapes" on each non-constant phase.
  phases_fixed <- list(
    early    = hzr_phase("cdf",  t_half = 0.2, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",   tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  )

  # --- Verify phases carry fixed attribute ---
  expect_equal(phases_fixed$early$fixed, c("t_half", "nu", "m"))
  expect_equal(phases_fixed$late$fixed, c("tau", "gamma", "alpha", "eta"))

  # Verify free_mask: 10 params total, 3 free (the 3 log_mu)
  cov_cnt <- c(early = 0L, constant = 0L, late = 0L)
  direct_mask <- .hzr_phase_free_mask(phases_fixed, cov_cnt)
  expect_equal(sum(direct_mask), 3L)
  expect_equal(direct_mask,
               c(TRUE, FALSE, FALSE, FALSE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE))

  # Starting values
  theta_start <- c(
    log(0.02),  log(0.2), 1, 1,
    log(0.0008),
    log(1e-7),  log(1),   3, 1, 1
  )

  fit <- hazard(
    time   = dat$int_dead,
    status = dat$dead,
    dist   = "multiphase",
    theta  = theta_start,
    phases = phases_fixed,
    fit    = TRUE,
    control = list(n_starts = 5, maxit = 2000, reltol = 1e-12)
  )

  expect_s3_class(fit, "hazard")
  expect_true(is.finite(fit$fit$objective))

  # Verify the optimizer used fixed-parameter masking
  expect_true(!is.null(fit$fit$fixed_mask),
              label = "fixed_mask should be set when shapes are fixed")

  # Shape-fixed fit with G3 late phase should match C's -3740.52
  expect_equal(fit$fit$objective, -3740.52, tolerance = 0.05,
               label = "R logl (shapes fixed) matches C reference")

  # Fixed shape parameters MUST stay at their starting values
  expect_equal(unname(fit$fit$theta[2]), log(0.2))   # early.log_t_half
  expect_equal(unname(fit$fit$theta[3]), 1)           # early.nu
  expect_equal(unname(fit$fit$theta[4]), 1)           # early.m
  expect_equal(unname(fit$fit$theta[7]), log(1))      # late.log_tau
  expect_equal(unname(fit$fit$theta[8]), 3)           # late.gamma
  expect_equal(unname(fit$fit$theta[9]), 1)           # late.alpha
  expect_equal(unname(fit$fit$theta[10]), 1)          # late.eta

  # Full theta should still be length 10 (shapes present but fixed)
  expect_equal(length(fit$fit$theta), 10)
})


test_that("KUL fit mu estimates are in neighborhood of C reference", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  fit <- hazard(
    time = dat$int_dead,
    status = dat$dead,
    dist = "multiphase",
    theta = kul_theta_start,
    phases = kul_phases(),
    fit = TRUE,
    control = list(n_starts = 10, maxit = 2000, reltol = 1e-10)
  )

  theta <- fit$fit$theta

  # Internal layout: early=[log_mu, log_t_half, nu, m], constant=[log_mu],
  #                  late=[log_mu, log_tau, gamma, alpha, eta]
  log_mu_early <- theta[1]
  log_mu_const <- theta[5]
  log_mu_late  <- theta[6]

  # C reference: E0=-3.77955, C0=-7.2258, L0=-16.6578
  # With shapes free, mu may shift to compensate — use loose tolerance.
  # The early and constant phases are well-identified; late is harder.
  # G3 late phase has 4 shape params (tau, gamma, alpha, eta), creating
  # a wider mu-shape tradeoff surface than G1's 3 params, so late needs
  # extra tolerance.  Fixed-shape parity test validates exact logl match.
  expect_true(
    abs(log_mu_early - (-3.77955)) < 3,
    label = paste(
      "early log_mu", round(log_mu_early, 3),
      "within 3 of C ref -3.780"
    )
  )
  expect_true(
    abs(log_mu_const - (-7.2258)) < 3,
    label = paste(
      "constant log_mu", round(log_mu_const, 3),
      "within 3 of C ref -7.226"
    )
  )
  # Late log_mu has wide mu-shape tradeoff with 4 free G3 params.
  # Validate via log-likelihood rather than point estimate.
  # Free shapes should match or improve on C reference (which fixed shapes).
  expect_true(
    fit$fit$objective >= -3750,
    label = paste(
      "free-shape log-lik", round(fit$fit$objective, 2),
      "should be >= -3750 (C ref with fixed shapes: -3740.52)"
    )
  )
})

test_that("predict works on KUL multiphase fit", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  fit <- hazard(
    time   = dat$int_dead,
    status = dat$dead,
    dist   = "multiphase",
    theta  = kul_theta_start,
    phases = kul_phases(),
    fit = TRUE,
    control = list(n_starts = 10, maxit = 2000, reltol = 1e-10)
  )

  # Predict at landmark times (months)
  nd <- data.frame(time = c(0.5, 1, 6, 12, 24, 60, 120, 200))
  surv <- predict(fit, newdata = nd, type = "survival")

  expect_equal(length(surv), 8)
  expect_true(all(surv >= 0 & surv <= 1))
  # Survival should be monotone decreasing
  expect_true(all(diff(surv) <= 0))

  # Decomposed cumulative hazard
  decomp <- predict(fit, newdata = nd, type = "cumulative_hazard",
                    decompose = TRUE)
  expect_true(is.data.frame(decomp))
  expect_true(all(c("time", "total", "early", "constant", "late") %in% names(decomp)))
  expect_equal(decomp$total, decomp$early + decomp$constant + decomp$late,
               tolerance = 1e-10)
})


# ============================================================================
# Synthetic golden fixture: round-trip recovery
# ============================================================================

test_that("Synthetic 3-phase fixture round-trip: parameters recovered", {
  skip_on_cran()

  fixture <- load_synthetic_fixture()
  skip_if(is.null(fixture), "Synthetic 3-phase fixture not available")

  # Skip if fixture was generated with divergent optimizer (sanity check)
  skip_if(is.null(fixture$fit), "Fixture has no fit results")
  skip_if(!is.finite(fixture$fit$logl), "Fixture fit did not converge")
  skip_if(
    any(abs(fixture$fit$theta) > 100),
    "Fixture theta unreasonable; regenerate fixtures (data-raw/golden_fixtures.R)"
  )

  # Refit using the same phase specs AND the fixture's converged theta as start
  refit <- hazard(
    time   = fixture$data$time,
    status = fixture$data$status,
    dist   = "multiphase",
    theta  = fixture$fit$theta,
    phases = list(
      early    = hzr_phase("cdf",
                           t_half = fixture$phases$early$t_half,
                           nu = fixture$phases$early$nu,
                           m = fixture$phases$early$m),
      constant = hzr_phase("constant"),
      late     = hzr_phase("hazard",
                           t_half = fixture$phases$late$t_half,
                           nu = fixture$phases$late$nu,
                           m = fixture$phases$late$m)
    ),
    fit = TRUE,
    # This round-trip check validates recovery from a stored converged theta.
    # Keep optimization deterministic and disable CoE reparameterization,
    # which can move theta along a near-flat ridge while preserving objective.
    control = list(n_starts = 1, conserve = FALSE, maxit = 1000, reltol = 1e-8)
  )

  expect_true(!is.null(refit$fit))

  # Parameter estimates should match fixture within tolerance
  expect_equal(refit$fit$theta, fixture$fit$theta, tolerance = 1e-3)

  # Log-likelihood should match
  expect_equal(refit$fit$objective, fixture$fit$logl, tolerance = 1e-2)
})


# ============================================================================
# Two-phase parity: early + constant (simpler case)
# ============================================================================

test_that("Two-phase (cdf + constant) round-trip on simulated data", {
  skip_on_cran()

  set.seed(123)
  n <- 300

  # True model: early CDF + constant
  mu_e <- 0.1
  t_half_e <- 1
  nu_e <- 1.5
  m_e <- 0
  mu_c <- 0.005

  t_grid <- seq(0.01, 30, length.out = 2000)
  d <- hzr_decompos(t_grid, t_half = t_half_e, nu = nu_e, m = m_e)
  H_true <- mu_e * d$G + mu_c * t_grid
  S_true <- exp(-H_true)

  u <- runif(n)
  time <- approx(S_true, t_grid, xout = u, rule = 2)$y
  cens_time <- runif(n, 0, quantile(time, 0.75))
  status <- as.integer(time <= cens_time)
  time <- pmin(time, cens_time)
  keep <- time > 0
  time <- time[keep]
  status <- status[keep]

  # Provide reasonable starting theta:
  #   early: [log(mu_e), log(t_half_e), nu_e, m_e] = [log(0.1), log(1), 1.5, 0]
  #   bg:    [log(mu_c)] = [log(0.005)]
  theta_start <- c(log(mu_e), log(t_half_e), nu_e, m_e, log(mu_c))

  fit <- hazard(
    time = time,
    status = status,
    dist = "multiphase",
    theta = theta_start,
    phases = list(
      early = hzr_phase("cdf", t_half = t_half_e, nu = nu_e, m = m_e),
      bg = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )

  expect_true(is.finite(fit$fit$objective))

  # Recovered mu_early should be in the right ballpark
  mu_early_hat <- exp(fit$fit$theta[1])
  expect_true(
    abs(log(mu_early_hat / mu_e)) < 1.5,
    label = paste(
      "early mu_hat", round(mu_early_hat, 4),
      "within 1.5 log-units of truth", mu_e
    )
  )

  # Predictions should be valid
  nd <- data.frame(time = c(0.5, 2, 10))
  surv <- predict(fit, newdata = nd, type = "survival")
  expect_true(all(surv >= 0 & surv <= 1))
  expect_true(all(diff(surv) <= 0))
})


# ============================================================================
# Standard error comparison (C reference, profile likelihood)
# ============================================================================

test_that("R standard errors at C params are consistent with C reference", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")
  skip_if_not_installed("numDeriv")

  phases <- kul_phases()
  covariate_counts <- c(early = 0L, constant = 0L, late = 0L)
  x_list <- list(early = NULL, constant = NULL, late = NULL)

  # C reference converged theta (shapes fixed at exact values)
  theta_c <- c(-3.77955, log(0.2), 1, 1,
               -7.2258,
               -16.6578, log(1), 3, 1, 1)

  # Profile likelihood: only vary the 3 log_mu parameters (indices 1, 5, 6)
  # holding shapes fixed — matches what the C binary did.
  mu_idx <- c(1L, 5L, 6L)

  profile_logl <- function(mu_theta) {
    theta_full <- theta_c
    theta_full[mu_idx] <- mu_theta
    .hzr_logl_multiphase(
      theta = theta_full,
      time = dat$int_dead,
      status = dat$dead,
      phases = phases,
      covariate_counts = covariate_counts,
      x_list = x_list
    )
  }

  # Compute the early-phase SE via profile Hessian.  The late-phase
  # Hessian element is not numerically resolvable with finite differences
  # because mu_late ≈ 5.8e-8 contributes negligibly to the likelihood at
  # most observation times — the curvature signal is below double-precision
  # noise.  The C binary used analytic derivatives and had no difficulty.
  # TODO: add analytic Hessian; then test all 3 SEs.

  # Test the early-phase SE only (well-conditioned)
  early_logl <- function(log_mu_e) {
    theta_full <- theta_c
    theta_full[1] <- log_mu_e
    .hzr_logl_multiphase(
      theta = theta_full,
      time = dat$int_dead,
      status = dat$dead,
      phases = phases,
      covariate_counts = covariate_counts,
      x_list = x_list
    )
  }

  h <- 1e-4
  ll_p <- early_logl(theta_c[1] + h)
  ll_m <- early_logl(theta_c[1] - h)
  ll_0 <- early_logl(theta_c[1])
  d2 <- (ll_p + ll_m - 2 * ll_0) / h^2

  skip_if(
    !is.finite(d2) || d2 >= 0,
    "Early-phase Hessian not negative definite"
  )

  se_early <- sqrt(1 / (-d2))

  # C reference: SE(E0) = 0.09381214
  expect_equal(
    se_early,
    0.09381214,
    tolerance = 0.01,
    label = "Early phase SE matches C reference"
  )
})

# test-multiphase-likelihood.R — Tests for multiphase likelihood engine
#
# Tests theta splitting, cumhaz/hazard computation, log-likelihood
# evaluation at known parameters, and optimizer convergence on synthetic data.

# ============================================================================
# Helpers
# ============================================================================

# Simple synthetic data: exponential-ish with known rate
make_test_data <- function(n = 200, seed = 42) {
  set.seed(seed)
  time   <- rexp(n, rate = 0.3) + 0.01
  status <- rbinom(n, 1, prob = 0.7)
  age    <- rnorm(n, mean = 60, sd = 10)
  data.frame(time = time, status = status, age = age)
}


# ============================================================================
# .hzr_split_theta()
# ============================================================================

test_that(".hzr_split_theta partitions correctly for 2-phase model", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    late  = hzr_phase("hazard")
  ))
  cov_counts <- c(early = 0L, late = 0L)

  # early: log_mu + log_t_half + nu + m = 4

  # late:  log_mu + log_t_half + nu + m = 4
  theta <- 1:8
  split <- .hzr_split_theta(theta, phases, cov_counts)

  expect_equal(split$early, 1:4)
  expect_equal(split$late, 5:8)
})

test_that(".hzr_split_theta handles constant phase (1 param)", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    bg    = hzr_phase("constant"),
    late  = hzr_phase("hazard")
  ))
  cov_counts <- c(early = 0L, bg = 0L, late = 0L)

  # early: 4, bg: 1 (log_mu only), late: 4 = total 9
  theta <- 1:9
  split <- .hzr_split_theta(theta, phases, cov_counts)

  expect_equal(split$early, 1:4)
  expect_equal(split$bg, 5)
  expect_equal(split$late, 6:9)
})

test_that(".hzr_split_theta includes covariates", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    late  = hzr_phase("hazard")
  ))
  cov_counts <- c(early = 2L, late = 1L)

  # early: 4 + 2 = 6, late: 4 + 1 = 5, total = 11
  theta <- seq(0.1, 1.1, by = 0.1)
  split <- .hzr_split_theta(theta, phases, cov_counts)

  expect_equal(length(split$early), 6)
  expect_equal(length(split$late), 5)
})


# ============================================================================
# .hzr_unpack_phase_theta()
# ============================================================================

test_that("unpack_phase_theta works for cdf phase", {
  phase <- hzr_phase("cdf")
  theta_j <- c(-2.3, log(2), 1.5, 0.3)  # log_mu, log_t_half, nu, m
  pars <- .hzr_unpack_phase_theta(theta_j, phase)

  expect_equal(pars$log_mu, -2.3)
  expect_equal(pars$log_t_half, log(2))
  expect_equal(pars$nu, 1.5)
  expect_equal(pars$m, 0.3)
  expect_equal(length(pars$beta), 0)
})

test_that("unpack_phase_theta works for constant phase", {
  phase <- hzr_phase("constant")
  theta_j <- c(-1.5, 0.02)  # log_mu, beta_1
  pars <- .hzr_unpack_phase_theta(theta_j, phase)

  expect_equal(pars$log_mu, -1.5)
  expect_true(is.na(pars$log_t_half))
  expect_true(is.na(pars$nu))
  expect_true(is.na(pars$m))
  expect_equal(pars$beta, 0.02)
})


# ============================================================================
# .hzr_multiphase_cumhaz() — known-parameter evaluation
# ============================================================================

test_that("single constant phase reduces to mu * t", {
  phases <- .hzr_validate_phases(list(bg = hzr_phase("constant")))
  cov_counts <- c(bg = 0L)
  x_list <- list(bg = NULL)

  # log_mu = log(0.5) => mu = 0.5
  theta <- log(0.5)
  time <- c(1, 2, 5)

  cumhaz <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list)
  expect_equal(cumhaz, 0.5 * time, tolerance = 1e-12)
})

test_that("single cdf phase produces mu * G(t)", {
  phases <- .hzr_validate_phases(list(early = hzr_phase("cdf")))
  cov_counts <- c(early = 0L)
  x_list <- list(early = NULL)

  # log_mu = 0 => mu = 1; log_t_half = log(3); nu = 2; m = 0
  theta <- c(0, log(3), 2, 0)
  time <- seq(0.5, 10, by = 0.5)

  cumhaz <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list)
  expected_G <- hzr_decompos(time, t_half = 3, nu = 2, m = 0)$G
  expect_equal(cumhaz, expected_G, tolerance = 1e-10)
})

test_that("two-phase additive: constant + cdf", {
  phases <- .hzr_validate_phases(list(
    bg    = hzr_phase("constant"),
    early = hzr_phase("cdf")
  ))
  cov_counts <- c(bg = 0L, early = 0L)
  x_list <- list(bg = NULL, early = NULL)

  # bg: log_mu = log(0.1) => mu_bg = 0.1
  # early: log_mu = log(0.5), log_t_half = log(2), nu = 1, m = 0
  theta <- c(log(0.1), log(0.5), log(2), 1, 0)
  time <- c(1, 5, 10)

  cumhaz <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list)
  G <- hzr_decompos(time, t_half = 2, nu = 1, m = 0)$G
  expected <- 0.1 * time + 0.5 * G
  expect_equal(cumhaz, expected, tolerance = 1e-10)
})

test_that("per_phase = TRUE returns decomposed contributions", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    late  = hzr_phase("hazard")
  ))
  cov_counts <- c(early = 0L, late = 0L)
  x_list <- list(early = NULL, late = NULL)

  theta <- c(log(0.5), log(2), 2, 0,   # early
             log(0.3), log(5), 1, 0)    # late

  time <- c(1, 3, 7)
  result <- .hzr_multiphase_cumhaz(time, theta, phases, cov_counts, x_list,
                                     per_phase = TRUE)

  expect_true(is.list(result))
  expect_true("total" %in% names(result))
  expect_true("early" %in% names(result))
  expect_true("late" %in% names(result))
  expect_equal(result$total, result$early + result$late, tolerance = 1e-12)
})


# ============================================================================
# .hzr_multiphase_hazard()
# ============================================================================

test_that("constant phase hazard is mu * 1 = mu", {
  phases <- .hzr_validate_phases(list(bg = hzr_phase("constant")))
  cov_counts <- c(bg = 0L)
  x_list <- list(bg = NULL)

  theta <- log(0.5)
  time <- c(1, 2, 5)

  haz <- .hzr_multiphase_hazard(time, theta, phases, cov_counts, x_list)
  expect_equal(haz, rep(0.5, 3), tolerance = 1e-12)
})


# ============================================================================
# .hzr_logl_multiphase() — basic properties
# ============================================================================

test_that("logl is finite for valid parameters", {
  dat <- make_test_data(n = 50)
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf", t_half = 2, nu = 1, m = 0)
  ))
  cov_counts <- c(early = 0L)
  x_list <- list(early = NULL)

  theta <- c(log(0.1), log(2), 1, 0)

  ll <- .hzr_logl_multiphase(
    theta = theta, time = dat$time, status = dat$status,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )

  expect_true(is.finite(ll))
  expect_true(ll < 0)  # log-likelihood should be negative
})

test_that("logl returns -Inf for m < 0 and nu < 0", {
  dat <- make_test_data(n = 50)
  phases <- .hzr_validate_phases(list(
    bad = hzr_phase("cdf", t_half = 2, nu = 1, m = 0)
  ))
  cov_counts <- c(bad = 0L)
  x_list <- list(bad = NULL)

  # Force invalid: nu = -1 and m = -1 in the theta vector
  theta <- c(log(0.1), log(2), -1, -1)

  ll <- .hzr_logl_multiphase(
    theta = theta, time = dat$time, status = dat$status,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )

  expect_equal(ll, -Inf)
})

test_that("logl is higher at better parameters (constant-rate sanity check)", {
  # Generate data from a known constant-rate model: H(t) = lambda * t
  set.seed(99)
  true_lambda <- 0.3
  n <- 200
  time <- rexp(n, rate = true_lambda) + 0.01
  status <- rep(1L, n)  # all events

  phases <- .hzr_validate_phases(list(bg = hzr_phase("constant")))
  cov_counts <- c(bg = 0L)
  x_list <- list(bg = NULL)

  # True parameter
  ll_true <- .hzr_logl_multiphase(
    theta = log(true_lambda), time = time, status = status,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )

  # Wrong parameter (much higher rate)
  ll_wrong <- .hzr_logl_multiphase(
    theta = log(5.0), time = time, status = status,
    phases = phases, covariate_counts = cov_counts, x_list = x_list
  )

  expect_true(ll_true > ll_wrong)
})


# ============================================================================
# .hzr_multiphase_theta_user() — back-transformation
# ============================================================================

test_that("theta_user back-transforms correctly", {
  phases <- .hzr_validate_phases(list(
    early = hzr_phase("cdf"),
    bg    = hzr_phase("constant")
  ))
  cov_counts <- c(early = 0L, bg = 0L)

  # internal: [log(0.5), log(3), 2, 0.1, log(0.2)]
  theta_int <- c(log(0.5), log(3), 2, 0.1, log(0.2))
  theta_usr <- .hzr_multiphase_theta_user(theta_int, phases, cov_counts)

  expect_equal(unname(theta_usr[1]), 0.5, tolerance = 1e-10)  # mu early
  expect_equal(unname(theta_usr[2]), 3.0, tolerance = 1e-10)  # t_half early
  expect_equal(unname(theta_usr[3]), 2.0)                     # nu early
  expect_equal(unname(theta_usr[4]), 0.1)                     # m early
  expect_equal(unname(theta_usr[5]), 0.2, tolerance = 1e-10)  # mu bg
})


# ============================================================================
# .hzr_optim_multiphase() — convergence on simple data
# ============================================================================

test_that("optimizer recovers constant rate from exponential data", {
  skip_on_cran()  # optimization test — may be slow

  set.seed(123)
  true_lambda <- 0.4
  n <- 300
  dat <- data.frame(
    time   = rexp(n, rate = true_lambda) + 0.01,
    status = rep(1L, n)
  )

  phases <- list(bg = hzr_phase("constant"))

  result <- .hzr_optim_multiphase(
    time = dat$time, status = dat$status,
    phases = phases,
    control = list(n_starts = 3, maxit = 500)
  )

  expect_equal(result$convergence, 0)

  # Recover mu ~ true_lambda (on internal scale, par is log_mu)
  mu_hat <- unname(exp(result$par[1]))
  expect_equal(mu_hat, true_lambda, tolerance = 0.15)
})

test_that("optimizer handles two phases (cdf + constant)", {
  skip_on_cran()

  # Simulate: H(t) = 0.3 * G(t; t_half=2, nu=1, m=0) + 0.1 * t
  set.seed(456)
  n <- 400
  t_grid <- seq(0.01, 15, length.out = n)

  # Compute survival under the model and simulate events
  G <- hzr_decompos(t_grid, t_half = 2, nu = 1, m = 0)$G
  cumhaz_true <- 0.3 * G + 0.1 * t_grid
  surv_true <- exp(-cumhaz_true)

  # Simple simulation: event if uniform < 1 - S(t)
  set.seed(789)
  u <- runif(n)
  status <- as.integer(u > surv_true)

  dat <- data.frame(time = t_grid, status = status)

  phases <- list(
    early = hzr_phase("cdf", t_half = 2, nu = 1, m = 0),
    bg    = hzr_phase("constant")
  )

  result <- suppressWarnings(
    .hzr_optim_multiphase(
      time = dat$time, status = dat$status,
      phases = phases,
      control = list(n_starts = 3, maxit = 1000)
    )
  )

  expect_true(is.finite(result$value))
  expect_equal(result$convergence, 0)

  # At minimum, the log-likelihood should be better than starting values
  theta_start <- unlist(lapply(names(.hzr_validate_phases(phases)), function(nm) {
    .hzr_phase_start(.hzr_validate_phases(phases)[[nm]])
  }))
  ll_start <- .hzr_logl_multiphase(
    theta = theta_start, time = dat$time, status = dat$status,
    phases = .hzr_validate_phases(phases),
    covariate_counts = c(early = 0L, bg = 0L),
    x_list = list(early = NULL, bg = NULL)
  )
  expect_true(result$value >= ll_start)
})


# ============================================================================
# Regression: 3+ covariate phase must not emit vector-recycling warnings
# ============================================================================

test_that("3-covariate phase does not emit mu_j * phi_j recycling warning", {
  # Reproduces the issue where model.matrix() silently drops NA rows in a
  # phase-specific formula, leaving x_list[[nm]] shorter than time and
  # triggering "longer object length is not a multiple of shorter object
  # length" on every mu_j * phi_j multiplication inside
  # .hzr_multiphase_cumhaz() / .hzr_multiphase_hazard().  The avc dataset
  # has 5 NAs in inc_surg, so a 3-covariate phase referencing it would
  # produce a 305-row design matrix against a 310-row time vector.
  skip_if_not(
    tryCatch({
      data("avc", package = "TemporalHazard", envir = environment())
      exists("avc", inherits = FALSE)
    }, error = function(e) FALSE),
    "avc dataset not available"
  )

  w <- testthat::capture_warnings(
    fit <- hazard(
      survival::Surv(int_dead, dead) ~ 1, data = avc,
      dist = "multiphase",
      phases = list(
        early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                              fixed = "shapes",
                              formula = ~ age + mal + inc_surg),
        constant = hzr_phase("constant")
      ),
      fit = TRUE,
      control = list(n_starts = 1L, maxit = 10L)
    )
  )
  # The bug under test produced a vector-recycling warning; assert it is absent.
  # (An incidental non-PD-Hessian warning at maxit = 10 is unrelated and allowed.)
  expect_false(any(grepl("not a multiple|longer object length", w)))
})

# tests/testthat/test-conservation-of-events.R
# Tests for the Conservation of Events (CoE) theorem implementation

test_that("CoE is active by default for multiphase models", {
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  expect_true(isTRUE(fit_coe$fit$converged) || is.finite(fit_coe$fit$objective))
  expect_true(is.finite(fit_coe$fit$objective))
})

test_that("CoE can be disabled via control$conserve = FALSE", {
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  fit_no_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = FALSE)
  )
  expect_true(is.finite(fit_no_coe$fit$objective))
})

test_that("CoE improves conservation ratio toward 1.0", {
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")

  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )

  # Check conservation: predicted events should be close to observed
  gof <- hzr_gof(fit_coe)
  s <- attr(gof, "summary")
  ratio <- s$total_expected / s$total_observed

  # With CoE, the conservation ratio should be very close to 1.0
  expect_true(
    abs(ratio - 1.0) < 0.15,
    label = paste("conservation ratio", round(ratio, 4),
                  "should be within 0.15 of 1.0")
  )
})

test_that("CoE CABGKUL log-likelihood matches C reference", {
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 5, maxit = 1000)
  )

  # C reference with fixed shapes: log-lik = -3740.52
  # CoE should help reach this or better
  expect_true(
    fit_coe$fit$objective >= -3745,
    label = paste("CoE log-lik", round(fit_coe$fit$objective, 2),
                  "should be >= -3745 (C ref: -3740.52)")
  )
})

test_that("CoE works with 2-phase model (early + constant)", {
  set.seed(42)
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = avc,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  expect_true(isTRUE(fit$fit$converged) || is.finite(fit$fit$objective))
})

test_that(".hzr_log_mu_positions returns correct positions", {
  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1)
  )
  cov_counts <- c(early = 0L, constant = 0L, late = 0L)
  pos <- TemporalHazard:::.hzr_log_mu_positions(phases, cov_counts)

  # early: log_mu at 1, then 3 shapes = 4 params
  # constant: log_mu at 5, no shapes = 1 param
  # late: log_mu at 6, then 4 G3 shapes = 5 params
  expect_equal(pos[["early"]], 1L)
  expect_equal(pos[["constant"]], 5L)
  expect_equal(pos[["late"]], 6L)
})

test_that(".hzr_conserve_events adjusts theta correctly", {
  data(cabgkul, package = "TemporalHazard")
  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1),
    constant = hzr_phase("constant")
  )
  cov_counts <- c(early = 0L, constant = 0L)
  # Arbitrary starting theta: [log_mu_e, log_thalf, nu, m, log_mu_c]
  theta <- c(-3, log(0.2), 1, 1, -5)

  theta_adj <- TemporalHazard:::.hzr_conserve_events(
    theta, fixmu_phase = "constant", fixmu_pos = 5L,
    time = cabgkul$int_dead, status = cabgkul$dead,
    phases = phases, covariate_counts = cov_counts,
    x_list = list(early = NULL, constant = NULL),
    total_events = sum(cabgkul$dead)
  )

  # The constant phase log_mu should have changed
  expect_false(theta_adj[5] == theta[5])
  # Other params should be unchanged
  expect_equal(theta_adj[1:4], theta[1:4])
})

# ===========================================================================
# CoE parity: reduced-dim CoE and full-dim optimization find same optimum
# ===========================================================================
#
# CoE is a dimension-reduction technique -- one phase's log_mu is solved
# analytically at each iteration instead of numerically optimized.  The
# log-likelihood function is unchanged, so the MLE should be identical
# (up to optimizer tolerance) whether CoE is on or off.

test_that("CoE and non-CoE find the same optimum on right-censored data", {
  skip_on_cran()
  set.seed(101)
  data(cabgkul, package = "TemporalHazard")
  small <- cabgkul[1:60, ]

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )
  fit_no_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = FALSE)
  )

  expect_equal(fit_coe$fit$objective, fit_no_coe$fit$objective,
               tolerance = 1e-2)
})

# ===========================================================================
# CoE with interval-censored data auto-disables cleanly
# ===========================================================================

test_that("CoE auto-disables (no error) when any observation is interval-censored", {
  # Per likelihood-multiphase.R the CoE guard is:
  #   coe_supported_data <- all(status %in% c(0, 1))
  # Interval-censored data (status == 2) therefore falls through to the
  # full-dimensional optimizer.  This test asserts the fit runs to a
  # finite log-likelihood.  It does not claim the Turner conservation
  # property holds for interval data -- that extension is tracked as
  # part of Phase 4e.
  skip_on_cran()
  set.seed(13)
  n <- 30
  lo <- runif(n, 0.1, 1.5)
  hi <- lo + runif(n, 0.1, 0.5)
  df <- data.frame(lo = lo, hi = hi)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  fit <- hazard(
    survival::Surv(lo, hi, type = "interval2") ~ 1,
    data = df, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 2, maxit = 300)
  )
  expect_true(is.finite(fit$fit$objective))
})

# ===========================================================================
# CoE with non-unit weights (wired up in v0.9.6, Phase 4e)
# ===========================================================================
#
# Prior to 0.9.6, `.hzr_conserve_events()` summed per-phase cumhaz without
# applying row weights, so Turner's adjustment was on a mismatched scale
# vs the (weighted) observed event count.  CoE therefore auto-disabled
# when any weight != 1.  As of 0.9.6, weights are threaded into
# `.hzr_conserve_events()` / `.hzr_select_fixmu_phase()` and CoE stays on
# for weighted fits.  Duplication parity is the acceptance criterion.

test_that("weighted multiphase fit matches the row-duplicated fit (CoE off)", {
  skip_on_cran()
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  small <- cabgkul[1:40, ]
  w <- sample(1:2, nrow(small), replace = TRUE)
  idx <- rep(seq_len(nrow(small)), times = w)
  dup <- small[idx, ]

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  # Force CoE off on both sides so the comparison isolates weighted LL
  # equivalence rather than the CoE vs non-CoE dimension trick.
  fit_w <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    weights = w, fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = FALSE)
  )
  fit_dup <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = dup, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = FALSE)
  )

  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-3)
  expect_equal(unname(coef(fit_w)), unname(coef(fit_dup)), tolerance = 1e-2)
})

test_that("weighted multiphase fit matches the row-duplicated fit (CoE on)", {
  skip_on_cran()
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  small <- cabgkul[1:40, ]
  w <- sample(1:2, nrow(small), replace = TRUE)
  idx <- rep(seq_len(nrow(small)), times = w)
  dup <- small[idx, ]

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  # CoE enabled on both sides: the weighted fit reduces the dimension on
  # D, the unweighted fit reduces it on expand(D, w).  The optimum (MLE
  # and log-likelihood) must match because CoE is a reparameterisation
  # of the same surface.
  fit_w <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    weights = w, fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = TRUE)
  )
  fit_dup <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = dup, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = TRUE)
  )

  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-3)
  expect_equal(unname(coef(fit_w)), unname(coef(fit_dup)), tolerance = 1e-2)
})

test_that("CoE default (conserve=TRUE) with weights reaches the same optimum as conserve=FALSE", {
  # CoE reduces dimension; it does not change the log-likelihood surface.
  # Both paths must land at the same MLE log-likelihood.
  skip_on_cran()
  set.seed(77)
  data(cabgkul, package = "TemporalHazard")
  small <- cabgkul[1:40, ]
  w <- sample(1:2, nrow(small), replace = TRUE)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  fit_default <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    weights = w, fit = TRUE,
    control = list(n_starts = 2, maxit = 300)  # conserve default TRUE
  )
  fit_explicit_off <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    weights = w, fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = FALSE)
  )

  expect_equal(fit_default$fit$objective, fit_explicit_off$fit$objective,
               tolerance = 1e-3)
})

# test-multiphase-gradient.R — Validate analytic gradient against numerical
#
# Tests that .hzr_gradient_multiphase() returns gradients consistent with
# central-difference numerical gradients of .hzr_logl_multiphase().
# This validates the semi-analytic gradient (chain-rule for mu/beta,
# central-diff for shape params via .hzr_phase_derivatives()).

# ============================================================================
# Helpers
# ============================================================================

load_kul_csv <- function() {
  csv_path <- system.file("extdata", "cabgkul.csv", package = "TemporalHazard")
  if (!nzchar(csv_path) || !file.exists(csv_path)) {
    csv_path <- file.path("inst", "extdata", "cabgkul.csv")
  }
  if (!file.exists(csv_path)) return(NULL)
  utils::read.csv(csv_path)
}

kul_phases <- function() {
  list(
    early    = hzr_phase("cdf",  t_half = 0.2, nu = 1, m = 1),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",   tau = 1, gamma = 3, alpha = 1, eta = 1)
  )
}

# Central-difference numerical gradient (tight step, reference implementation)
numerical_gradient <- function(theta, time, status, phases,
                                covariate_counts, x_list,
                                time_lower = NULL, time_upper = NULL) {
  p <- length(theta)
  grad <- numeric(p)
  eps_rel <- (.Machine$double.eps)^(1 / 3)

  for (i in seq_len(p)) {
    h_i <- eps_rel * max(abs(theta[i]), 1)

    theta_plus <- theta
    theta_plus[i] <- theta_plus[i] + h_i
    theta_minus <- theta
    theta_minus[i] <- theta_minus[i] - h_i

    ll_plus <- .hzr_logl_multiphase(
      theta_plus, time, status,
      time_lower = time_lower, time_upper = time_upper,
      phases = phases, covariate_counts = covariate_counts, x_list = x_list
    )
    ll_minus <- .hzr_logl_multiphase(
      theta_minus, time, status,
      time_lower = time_lower, time_upper = time_upper,
      phases = phases, covariate_counts = covariate_counts, x_list = x_list
    )

    if (is.finite(ll_plus) && is.finite(ll_minus)) {
      grad[i] <- (ll_plus - ll_minus) / (2 * h_i)
    } else if (is.finite(ll_plus)) {
      ll0 <- .hzr_logl_multiphase(
        theta, time, status,
        time_lower = time_lower, time_upper = time_upper,
        phases = phases, covariate_counts = covariate_counts, x_list = x_list
      )
      grad[i] <- (ll_plus - ll0) / h_i
    }
  }

  grad
}


# ============================================================================
# Phase derivative unit tests
# ============================================================================

test_that(".hzr_phase_derivatives returns correct structure", {
  t_grid <- seq(0.5, 10, by = 0.5)

  pd <- .hzr_phase_derivatives(
    t_grid,
    t_half = 2,
    nu = 1.5,
    m = 0.8,
    type = "cdf"
  )
  expect_named(
    pd,
    c(
      "Phi", "phi", "dPhi_dlog_thalf", "dPhi_dnu", "dPhi_dm",
      "dphi_dlog_thalf", "dphi_dnu", "dphi_dm"
    )
  )
  expect_length(pd$Phi, length(t_grid))
  expect_true(all(is.finite(pd$Phi)))
  expect_true(all(is.finite(pd$dPhi_dlog_thalf)))
})

test_that(".hzr_phase_derivatives for constant phase returns zeros", {
  t_grid <- seq(0.5, 5, by = 0.5)
  pd <- .hzr_phase_derivatives(t_grid, type = "constant")

  expect_equal(pd$Phi, t_grid)
  expect_equal(pd$phi, rep(1, length(t_grid)))
  expect_equal(pd$dPhi_dlog_thalf, rep(0, length(t_grid)))
  expect_equal(pd$dPhi_dnu, rep(0, length(t_grid)))
  expect_equal(pd$dPhi_dm, rep(0, length(t_grid)))
})

test_that(".hzr_phase_derivatives matches numerical for cdf phase (case 1)", {
  t_grid <- seq(0.5, 8, by = 0.5)
  t_half <- 3
  nu <- 2
  m <- 1

  pd <- .hzr_phase_derivatives(
    t_grid,
    t_half = t_half,
    nu = nu,
    m = m,
    type = "cdf"
  )

  # Numerical check for dPhi/d(log_t_half)
  h <- 1e-5
  Phi_plus  <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half * exp(h),
    nu = nu,
    m = m,
    type = "cdf"
  )
  Phi_minus <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half * exp(-h),
    nu = nu,
    m = m,
    type = "cdf"
  )
  num_dPhi_dlog_thalf <- (Phi_plus - Phi_minus) / (2 * h)
  expect_equal(pd$dPhi_dlog_thalf, num_dPhi_dlog_thalf, tolerance = 1e-4)

  # Numerical check for dPhi/d(nu)
  Phi_plus  <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half,
    nu = nu + h,
    m = m,
    type = "cdf"
  )
  Phi_minus <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half,
    nu = nu - h,
    m = m,
    type = "cdf"
  )
  num_dPhi_dnu <- (Phi_plus - Phi_minus) / (2 * h)
  expect_equal(pd$dPhi_dnu, num_dPhi_dnu, tolerance = 1e-4)
})

test_that(".hzr_phase_derivatives matches numerical for hazard phase", {
  t_grid <- seq(0.5, 8, by = 0.5)
  t_half <- 5
  nu <- 1
  m <- 0

  pd <- .hzr_phase_derivatives(
    t_grid,
    t_half = t_half,
    nu = nu,
    m = m,
    type = "hazard"
  )

  h <- 1e-5
  Phi_plus  <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half * exp(h),
    nu = nu,
    m = m,
    type = "hazard"
  )
  Phi_minus <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half * exp(-h),
    nu = nu,
    m = m,
    type = "hazard"
  )
  num_dPhi_dlog_thalf <- (Phi_plus - Phi_minus) / (2 * h)
  expect_equal(pd$dPhi_dlog_thalf, num_dPhi_dlog_thalf, tolerance = 1e-4)
})

test_that(".hzr_phase_derivatives works for all 6 decomposition cases", {
  t_grid <- seq(0.5, 5, by = 0.5)
  cases <- list(
    list(t_half = 3, nu = 2,  m = 1),    # Case 1: m>0, nu>0
    list(t_half = 3, nu = 2,  m = 0),    # Case 1L: m=0, nu>0
    list(t_half = 3, nu = 2,  m = -0.5), # Case 2: m<0, nu>0
    list(t_half = 3, nu = 0,  m = -0.5), # Case 2L: m<0, nu=0
    list(t_half = 3, nu = -1, m = 1),    # Case 3: m>0, nu<0
    list(t_half = 3, nu = -1, m = 0)     # Case 3L: m=0, nu<0
  )

  for (i in seq_along(cases)) {
    cc <- cases[[i]]
    for (type in c("cdf", "hazard")) {
      pd <- .hzr_phase_derivatives(
        t_grid,
        t_half = cc$t_half,
        nu = cc$nu,
        m = cc$m,
        type = type
      )
      expect_true(
        all(is.finite(pd$Phi)),
        label = paste("Case", i, type, "Phi finite")
      )
      expect_true(
        all(is.finite(pd$dPhi_dlog_thalf)),
        label = paste("Case", i, type, "dPhi/dlog_thalf finite")
      )
      expect_true(
        all(is.finite(pd$dPhi_dnu)),
        label = paste("Case", i, type, "dPhi/dnu finite")
      )
      expect_true(
        all(is.finite(pd$dPhi_dm)),
        label = paste("Case", i, type, "dPhi/dm finite")
      )
    }
  }
})


# ============================================================================
# Full gradient tests on KUL dataset
# ============================================================================

test_that("Analytic gradient matches numerical at C reference parameters", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  phases <- kul_phases()
  covariate_counts <- c(early = 0L, constant = 0L, late = 0L)
  x_list <- list(early = NULL, constant = NULL, late = NULL)

  # C converged parameters on internal scale
  theta_c <- c(
    -3.77955,   # early.log_mu  (mu = 0.02268)
    log(0.2),   # early.log_t_half
    1,           # early.nu
    1,           # early.m
    -7.2258,     # constant.log_mu  (mu = 0.0007269)
    -16.6578,    # late.log_mu  (mu = 5.837e-08)
    log(1),      # late.log_tau
    3,           # late.gamma
    1,           # late.alpha
    1            # late.eta
  )

  grad_analytic <- .hzr_gradient_multiphase(
    theta_c, dat$int_dead, dat$dead,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  grad_numeric <- numerical_gradient(
    theta_c, dat$int_dead, dat$dead,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  # Both should be finite

  expect_true(all(is.finite(grad_analytic)),
              label = "Analytic gradient all finite")
  expect_true(all(is.finite(grad_numeric)),
              label = "Numerical gradient all finite")

  # Relative error should be small for each component
  # Allow looser tolerance for shape params (central-diff vs central-diff)
  denom <- pmax(abs(grad_numeric), 1e-6)
  rel_err <- abs(grad_analytic - grad_numeric) / denom

  # log_mu and constant parameters should match tightly
  expect_true(rel_err[1] < 0.01, label = "early.log_mu gradient < 1% error")
  expect_true(rel_err[5] < 0.01, label = "constant.log_mu gradient < 1% error")
  expect_true(rel_err[6] < 0.01, label = "late.log_mu gradient < 1% error")

  # Shape params may have larger relative error near zero gradients
  # but absolute agreement should be reasonable
  abs_err <- abs(grad_analytic - grad_numeric)
  expect_true(all(abs_err < 1),
              label = paste("Max absolute gradient error:",
                            round(max(abs_err), 4)))
})

test_that("Analytic gradient matches numerical at SAS starting values", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  phases <- kul_phases()
  covariate_counts <- c(early = 0L, constant = 0L, late = 0L)
  x_list <- list(early = NULL, constant = NULL, late = NULL)

  # SAS PARMS starting values
  theta_start <- c(
    log(0.02), log(0.2), 1, 1,
    log(0.0008),
    log(1e-9), log(1), 3, 1, 1
  )

  grad_analytic <- .hzr_gradient_multiphase(
    theta_start, dat$int_dead, dat$dead,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  grad_numeric <- numerical_gradient(
    theta_start, dat$int_dead, dat$dead,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  expect_true(all(is.finite(grad_analytic)))
  expect_true(all(is.finite(grad_numeric)))

  # At the starting point the gradients should be non-trivial and match
  denom <- pmax(abs(grad_numeric), 1e-6)
  rel_err <- abs(grad_analytic - grad_numeric) / denom

  # mu params should match well
  expect_true(rel_err[1] < 0.05, label = "early.log_mu gradient match")
  expect_true(rel_err[5] < 0.05, label = "constant.log_mu gradient match")
  expect_true(rel_err[6] < 0.05, label = "late.log_mu gradient match")
})


# ============================================================================
# Gradient on synthetic data with covariates
# ============================================================================

test_that("Analytic gradient works with covariates", {
  skip_on_cran()

  set.seed(42)
  n <- 200
  time   <- rexp(n, rate = 0.3) + 0.01
  status <- sample(0:1, n, replace = TRUE, prob = c(0.3, 0.7))
  x_mat  <- matrix(rnorm(n * 2), ncol = 2, dimnames = list(NULL, c("age", "sex")))

  phases <- list(
    early = hzr_phase("cdf",    t_half = 1, nu = 1, m = 0),
    late  = hzr_phase("hazard", t_half = 5, nu = 1, m = 0)
  )
  covariate_counts <- c(early = 2L, late = 2L)
  x_list <- list(early = x_mat, late = x_mat)

  # theta: [log_mu, log_t_half, nu, m, beta1, beta2] x 2 phases
  theta <- c(
    log(0.1), log(1), 1, 0, 0.5, -0.3,   # early
    log(0.01), log(5), 1, 0, 0.1, 0.2     # late
  )

  grad_analytic <- .hzr_gradient_multiphase(
    theta, time, status,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  grad_numeric <- numerical_gradient(
    theta, time, status,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  expect_true(all(is.finite(grad_analytic)))
  expect_true(all(is.finite(grad_numeric)))

  # Check beta parameters specifically (indices 5,6 and 11,12)
  beta_idx <- c(5, 6, 11, 12)
  denom <- pmax(abs(grad_numeric[beta_idx]), 1e-6)
  rel_err_beta <- abs(grad_analytic[beta_idx] - grad_numeric[beta_idx]) / denom
  expect_true(all(rel_err_beta < 0.05),
              label = paste("Beta gradient max rel error:",
                            round(max(rel_err_beta), 4)))
})


# ============================================================================
# Simple 2-phase model: gradient at known parameters
# ============================================================================

test_that("Gradient is near zero at converged fit", {
  skip_on_cran()

  set.seed(42)
  n <- 100
  time   <- rexp(n, rate = 0.5) + 0.01
  status <- sample(0:1, n, replace = TRUE, prob = c(0.2, 0.8))

  phases <- list(
    early = hzr_phase("cdf",    t_half = 1, nu = 1.5, m = 0),
    const = hzr_phase("constant")
  )

  fit <- tryCatch(
    hazard(time = time, status = status, dist = "multiphase",
           phases = phases, fit = TRUE,
           control = list(n_starts = 3, maxit = 500)),
    error = function(e) NULL
  )
  skip_if(is.null(fit), "Fit did not converge")
  skip_if(!fit$fit$converged, "Fit did not converge")

  covariate_counts <- c(early = 0L, const = 0L)
  x_list <- list(early = NULL, const = NULL)

  grad_at_mle <- .hzr_gradient_multiphase(
    fit$fit$theta, time, status,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  # Gradient should be reasonably small at the MLE.  With a small sample

  # (n=100) and rough BFGS convergence the per-observation score components
  # don't cancel perfectly, so we use a generous threshold.
  max_abs_grad <- max(abs(grad_at_mle))
  expect_true(max_abs_grad < 500,
              label = paste("Max |gradient| at MLE:", round(max_abs_grad, 2),
                            "should be < 500"))
})

# tests/testthat/test-conservation-of-events.R
# Tests for the Conservation of Events (CoE) theorem implementation

test_that("CoE is active by default for multiphase models", {
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  expect_true(isTRUE(fit_coe$fit$converged) || is.finite(fit_coe$fit$objective))
  expect_true(is.finite(fit_coe$fit$objective))
})

test_that("CoE can be disabled via control$conserve = FALSE", {
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  fit_no_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = FALSE)
  )
  expect_true(is.finite(fit_no_coe$fit$objective))
})

test_that("CoE improves conservation ratio toward 1.0", {
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")

  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )

  # Check conservation: predicted events should be close to observed
  gof <- hzr_gof(fit_coe)
  s <- attr(gof, "summary")
  ratio <- s$total_expected / s$total_observed

  # With CoE, the conservation ratio should be very close to 1.0
  expect_true(
    abs(ratio - 1.0) < 0.15,
    label = paste("conservation ratio", round(ratio, 4),
                  "should be within 0.15 of 1.0")
  )
})

test_that("CoE CABGKUL log-likelihood matches C reference", {
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 5, maxit = 1000)
  )

  # C reference with fixed shapes: log-lik = -3740.52
  # CoE should help reach this or better
  expect_true(
    fit_coe$fit$objective >= -3745,
    label = paste("CoE log-lik", round(fit_coe$fit$objective, 2),
                  "should be >= -3745 (C ref: -3740.52)")
  )
})

test_that("CoE works with 2-phase model (early + constant)", {
  set.seed(42)
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = avc,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  expect_true(isTRUE(fit$fit$converged) || is.finite(fit$fit$objective))
})

test_that(".hzr_log_mu_positions returns correct positions", {
  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1)
  )
  cov_counts <- c(early = 0L, constant = 0L, late = 0L)
  pos <- TemporalHazard:::.hzr_log_mu_positions(phases, cov_counts)

  # early: log_mu at 1, then 3 shapes = 4 params
  # constant: log_mu at 5, no shapes = 1 param
  # late: log_mu at 6, then 4 G3 shapes = 5 params
  expect_equal(pos[["early"]], 1L)
  expect_equal(pos[["constant"]], 5L)
  expect_equal(pos[["late"]], 6L)
})

test_that(".hzr_conserve_events adjusts theta correctly", {
  data(cabgkul, package = "TemporalHazard")
  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1),
    constant = hzr_phase("constant")
  )
  cov_counts <- c(early = 0L, constant = 0L)
  # Arbitrary starting theta: [log_mu_e, log_thalf, nu, m, log_mu_c]
  theta <- c(-3, log(0.2), 1, 1, -5)

  theta_adj <- TemporalHazard:::.hzr_conserve_events(
    theta, fixmu_phase = "constant", fixmu_pos = 5L,
    time = cabgkul$int_dead, status = cabgkul$dead,
    phases = phases, covariate_counts = cov_counts,
    x_list = list(early = NULL, constant = NULL),
    total_events = sum(cabgkul$dead)
  )

  # The constant phase log_mu should have changed
  expect_false(theta_adj[5] == theta[5])
  # Other params should be unchanged
  expect_equal(theta_adj[1:4], theta[1:4])
})

# ===========================================================================
# CoE parity: reduced-dim CoE and full-dim optimization find same optimum
# ===========================================================================
#
# CoE is a dimension-reduction technique -- one phase's log_mu is solved
# analytically at each iteration instead of numerically optimized.  The
# log-likelihood function is unchanged, so the MLE should be identical
# (up to optimizer tolerance) whether CoE is on or off.

test_that("CoE and non-CoE find the same optimum on right-censored data", {
  skip_on_cran()
  set.seed(101)
  data(cabgkul, package = "TemporalHazard")
  small <- cabgkul[1:60, ]

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )
  fit_no_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = FALSE)
  )

  expect_equal(fit_coe$fit$objective, fit_no_coe$fit$objective,
               tolerance = 1e-2)
})

# ===========================================================================
# CoE with interval-censored data auto-disables cleanly
# ===========================================================================

test_that("CoE auto-disables (no error) when any observation is interval-censored", {
  # Per likelihood-multiphase.R the CoE guard is:
  #   coe_supported_data <- all(status %in% c(0, 1))
  # Interval-censored data (status == 2) therefore falls through to the
  # full-dimensional optimizer.  This test asserts the fit runs to a
  # finite log-likelihood.  It does not claim the Turner conservation
  # property holds for interval data -- that extension is tracked as
  # part of Phase 4e.
  skip_on_cran()
  set.seed(13)
  n <- 30
  lo <- runif(n, 0.1, 1.5)
  hi <- lo + runif(n, 0.1, 0.5)
  df <- data.frame(lo = lo, hi = hi)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  fit <- hazard(
    survival::Surv(lo, hi, type = "interval2") ~ 1,
    data = df, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 2, maxit = 300)
  )
  expect_true(is.finite(fit$fit$objective))
})

# ===========================================================================
# CoE with non-unit weights (wired up in v0.9.6, Phase 4e)
# ===========================================================================
#
# Prior to 0.9.6, `.hzr_conserve_events()` summed per-phase cumhaz without
# applying row weights, so Turner's adjustment was on a mismatched scale
# vs the (weighted) observed event count.  CoE therefore auto-disabled
# when any weight != 1.  As of 0.9.6, weights are threaded into
# `.hzr_conserve_events()` / `.hzr_select_fixmu_phase()` and CoE stays on
# for weighted fits.  Duplication parity is the acceptance criterion.

test_that("weighted multiphase fit matches the row-duplicated fit (CoE off)", {
  skip_on_cran()
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  small <- cabgkul[1:40, ]
  w <- sample(1:2, nrow(small), replace = TRUE)
  idx <- rep(seq_len(nrow(small)), times = w)
  dup <- small[idx, ]

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  # Force CoE off on both sides so the comparison isolates weighted LL
  # equivalence rather than the CoE vs non-CoE dimension trick.
  fit_w <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    weights = w, fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = FALSE)
  )
  fit_dup <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = dup, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = FALSE)
  )

  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-3)
  expect_equal(unname(coef(fit_w)), unname(coef(fit_dup)), tolerance = 1e-2)
})

test_that("weighted multiphase fit matches the row-duplicated fit (CoE on)", {
  skip_on_cran()
  set.seed(42)
  data(cabgkul, package = "TemporalHazard")
  small <- cabgkul[1:40, ]
  w <- sample(1:2, nrow(small), replace = TRUE)
  idx <- rep(seq_len(nrow(small)), times = w)
  dup <- small[idx, ]

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  # CoE enabled on both sides: the weighted fit reduces the dimension on
  # D, the unweighted fit reduces it on expand(D, w).  The optimum (MLE
  # and log-likelihood) must match because CoE is a reparameterisation
  # of the same surface.
  fit_w <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    weights = w, fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = TRUE)
  )
  fit_dup <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = dup, dist = "multiphase", phases = phases,
    fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = TRUE)
  )

  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-3)
  expect_equal(unname(coef(fit_w)), unname(coef(fit_dup)), tolerance = 1e-2)
})

test_that("CoE default (conserve=TRUE) with weights reaches the same optimum as conserve=FALSE", {
  # CoE reduces dimension; it does not change the log-likelihood surface.
  # Both paths must land at the same MLE log-likelihood.
  skip_on_cran()
  set.seed(77)
  data(cabgkul, package = "TemporalHazard")
  small <- cabgkul[1:40, ]
  w <- sample(1:2, nrow(small), replace = TRUE)

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                           fixed = "shapes"),
    constant = hzr_phase("constant")
  )

  fit_default <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    weights = w, fit = TRUE,
    control = list(n_starts = 2, maxit = 300)  # conserve default TRUE
  )
  fit_explicit_off <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = small, dist = "multiphase", phases = phases,
    weights = w, fit = TRUE,
    control = list(n_starts = 2, maxit = 300, conserve = FALSE)
  )

  expect_equal(fit_default$fit$objective, fit_explicit_off$fit$objective,
               tolerance = 1e-3)
})


# ===========================================================================
# 4-phase CoE algebra (Phase 7d)
# ===========================================================================
#
# .hzr_select_fixmu_phase() and .hzr_conserve_events() must work correctly
# for N = 4 phases.  The key regression: when a G3 phase has a large
# unnormalized cumhaz at starting theta, the old which.max() selection
# picked that phase, causing CoE to pin it away from its true near-zero MLE.

test_that(".hzr_log_mu_positions returns correct positions for 4 phases", {
  phases4 <- list(
    early1   = hzr_phase("cdf", t_half = 0.1, nu = 1, m = 1),
    early2   = hzr_phase("cdf", t_half = 2.0, nu = 1, m = 1),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1)
  )
  cov_counts <- c(early1 = 0L, early2 = 0L, constant = 0L, late = 0L)
  pos <- TemporalHazard:::.hzr_log_mu_positions(phases4, cov_counts)

  # early1: log_mu at 1, then 3 shapes = 4 params → early2 starts at 5
  # early2: log_mu at 5, then 3 shapes = 4 params → constant starts at 9
  # constant: log_mu at 9, no shapes = 1 param → late starts at 10
  # late: log_mu at 10, then 4 G3 shapes = 5 params
  expect_equal(pos[["early1"]],   1L)
  expect_equal(pos[["early2"]],   5L)
  expect_equal(pos[["constant"]], 9L)
  expect_equal(pos[["late"]],    10L)
})

test_that(".hzr_select_fixmu_phase excludes extreme-outlier phases for N=4", {
  # With G3 starting values tau=1, gamma=3, alpha=1, eta=1 the late phase
  # cumhaz is orders of magnitude larger than the other phases at typical
  # starting theta.  The fixed selection should skip it and pick among the
  # non-outlier phases.
  data(cabgkul, package = "TemporalHazard")
  phases4 <- list(
    early1   = hzr_phase("cdf", t_half = 0.1, nu = 1, m = 1, fixed = "shapes"),
    early2   = hzr_phase("cdf", t_half = 2.0, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  )
  cov_counts <- c(early1 = 0L, early2 = 0L, constant = 0L, late = 0L)
  x_list <- list(early1 = NULL, early2 = NULL, constant = NULL, late = NULL)
  theta0 <- c(-3, log(0.1), 1, 1,   # early1
               -3, log(2.0), 1, 1,   # early2
               -5,                   # constant
               -3, log(1), 3, 1, 1)  # late

  fixmu <- TemporalHazard:::.hzr_select_fixmu_phase(
    theta0, cabgkul$int_dead, cabgkul$dead,
    phases4, cov_counts, x_list
  )
  # G3 late phase should be excluded as an outlier; constant or one of the
  # CDF phases should be selected instead
  expect_false(fixmu == "late",
               label = paste("fixmu should not be 'late'; got:", fixmu))
})

test_that(".hzr_conserve_events satisfies CoE identity for 4 phases", {
  data(cabgkul, package = "TemporalHazard")
  # Use 4 CDF/constant phases (no G3) to avoid the shape-value scale issue:
  # G3 with tau=1 generates enormous base cumhaz at long follow-up times even
  # with log_mu = -20, making jevent < 0 and causing a valid guard bail-out.
  phases4 <- list(
    early1   = hzr_phase("cdf", t_half = 0.1, nu = 1, m = 1),
    early2   = hzr_phase("cdf", t_half = 2.0, nu = 1, m = 1),
    constant = hzr_phase("constant"),
    late     = hzr_phase("cdf", t_half = 20, nu = 1, m = 1)
  )
  cov_counts <- c(early1 = 0L, early2 = 0L, constant = 0L, late = 0L)
  x_list <- list(early1 = NULL, early2 = NULL, constant = NULL, late = NULL)
  # Positions: early1=1 (4 params), early2=5 (4), constant=9 (1), late=10 (4)
  # Use small log_mu values so total cumhaz < total_events — the CoE
  # adjustment then has room to increase the constant phase log_mu upward.
  # exp(-6.5) * sum(cabgkul$int_dead) ≈ 503 < 545 = total_events.
  theta0 <- c(-10, log(0.1), 1, 1,   # early1  (negligible contribution)
               -10, log(2.0), 1, 1,   # early2  (negligible contribution)
               -6.5,                   # constant (log_mu at pos 9, will be adjusted)
               -10, log(20), 1, 1)     # late    (negligible contribution)
  total_events <- sum(cabgkul$dead)

  theta_adj <- TemporalHazard:::.hzr_conserve_events(
    theta0, fixmu_phase = "constant", fixmu_pos = 9L,
    time = cabgkul$int_dead, status = cabgkul$dead,
    phases = phases4, covariate_counts = cov_counts,
    x_list = x_list, total_events = total_events
  )

  # Only the constant phase log_mu should have changed
  expect_false(theta_adj[9] == theta0[9])
  expect_equal(theta_adj[-9], theta0[-9])

  # After adjustment, total predicted events should equal observed events
  decomp <- TemporalHazard:::.hzr_multiphase_cumhaz(
    cabgkul$int_dead, theta_adj, phases4, cov_counts, x_list
  )
  expect_equal(sum(decomp), total_events, tolerance = 1e-6)
})

test_that("4-phase CoE fit converges and conservation ratio is 1.0", {
  data(cabgkul, package = "TemporalHazard")
  phases4 <- list(
    early1   = hzr_phase("cdf", t_half = 0.1, nu = 1, m = 1, fixed = "shapes"),
    early2   = hzr_phase("cdf", t_half = 2.0, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  )
  set.seed(42)
  fit4 <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data    = cabgkul,
    dist    = "multiphase",
    phases  = phases4,
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )

  expect_true(is.finite(fit4$fit$objective))
  gof <- hzr_gof(fit4)
  s   <- attr(gof, "summary")
  ratio <- unname(s$total_expected / s$total_observed)
  expect_equal(ratio, 1.0, tolerance = 1e-4)
})

test_that("4-phase CoE and non-CoE find the same optimum", {
  skip_on_cran()
  data(cabgkul, package = "TemporalHazard")
  phases4 <- list(
    early1   = hzr_phase("cdf", t_half = 0.1, nu = 1, m = 1, fixed = "shapes"),
    early2   = hzr_phase("cdf", t_half = 2.0, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  )
  set.seed(42)
  fit_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data    = cabgkul,
    dist    = "multiphase",
    phases  = phases4,
    fit     = TRUE,
    control = list(n_starts = 5, maxit = 500, conserve = TRUE)
  )
  fit_no_coe <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data    = cabgkul,
    dist    = "multiphase",
    phases  = phases4,
    fit     = TRUE,
    control = list(n_starts = 5, maxit = 500, conserve = FALSE)
  )
  expect_equal(fit_coe$fit$objective, fit_no_coe$fit$objective, tolerance = 0.1)
})

test_that("4-phase predict(decompose=TRUE) returns 4 per-phase columns", {
  data(cabgkul, package = "TemporalHazard")
  phases4 <- list(
    early1   = hzr_phase("cdf", t_half = 0.1, nu = 1, m = 1, fixed = "shapes"),
    early2   = hzr_phase("cdf", t_half = 2.0, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  )
  set.seed(42)
  fit4 <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data    = cabgkul,
    dist    = "multiphase",
    phases  = phases4,
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500)
  )
  nd <- data.frame(time = c(6, 24, 60, 120))
  d  <- predict(fit4, newdata = nd, type = "cumulative_hazard",
                decompose = TRUE)

  expect_true(is.data.frame(d) || is.matrix(d))
  expect_true("total"   %in% colnames(d))
  expect_true("early1"  %in% colnames(d))
  expect_true("early2"  %in% colnames(d))
  expect_true("constant" %in% colnames(d))
  expect_true("late"    %in% colnames(d))
  # Total must equal sum of per-phase contributions
  phase_sum <- d[, "early1"] + d[, "early2"] + d[, "constant"] + d[, "late"]
  expect_equal(unname(d[, "total"]), unname(phase_sum), tolerance = 1e-10)
})

test_that("weighted multiphase fit matches duplicated-row fit, covariates, CoE on", {
  skip_on_cran()
  set.seed(43)
  n      <- 40
  t      <- runif(n, 0.05, 3)
  status <- rbinom(n, 1, 0.4)
  x      <- rnorm(n)
  w      <- sample(1:3, n, replace = TRUE)
  df     <- data.frame(time = t, status = status, x = x)
  df_exp <- df[rep(seq_len(n), times = w), , drop = FALSE]

  mk_fit <- function(data, weights) {
    hazard(
      survival::Surv(time, status) ~ 1,
      data = data, dist = "multiphase",
      phases = list(
        early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                             fixed = "shapes", formula = ~ x),
        constant = hzr_phase("constant", formula = ~ x)
      ),
      weights = weights, fit = TRUE,
      control = list(conserve = TRUE, n_starts = 1L, maxit = 500L)
    )
  }

  fit_w   <- mk_fit(df,     w)
  fit_dup <- mk_fit(df_exp, NULL)

  expect_equal(coef(fit_w), coef(fit_dup), tolerance = 1e-3)
  expect_equal(fit_w$fit$objective, fit_dup$fit$objective, tolerance = 1e-4)
})

# Interval-censoring under the multiphase additive model (roadmap 7c).
#
# The multiphase likelihood has a working interval-/left-censored code path
# (`.hzr_logl_multiphase()`, status %in% c(-1, 2)) but no real-data or SAS
# parity fixture, so it was previously untested in isolation.  These are
# R-only Tier-1 self-consistency invariants: every assertion follows from the
# mathematics of the additive cumulative hazard
# `H(t) = sum_j mu_j(x) Phi_j(t)` and needs no external reference.  They mirror
# the single-distribution checks in test-interval-censoring-weibull.R, anchored
# to the independently-exercised `.hzr_multiphase_cumhaz()` evaluator.

# Shared two-phase spec (early CDF + constant), shapes fixed so the only free
# parameters are the two intercepts -- keeps the manual cumulative hazard exact.
mp_ic_phases <- function() {
  list(
    early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1, fixed = "shapes"),
    constant = hzr_phase("constant")
  )
}
mp_ic_cc    <- c(early = 0L, constant = 0L)
mp_ic_xl    <- list(early = NULL, constant = NULL)
# theta layout: [log_mu_early, log_t_half, nu, m, log_mu_constant]
mp_ic_theta <- c(-1.0, log(0.3), 1, 1, -1.5)

# Convenience: the model's cumulative hazard at the shared theta.
mp_ic_H <- function(t) {
  TemporalHazard:::.hzr_multiphase_cumhaz(
    t, mp_ic_theta, mp_ic_phases(), mp_ic_cc, mp_ic_xl)
}

mp_ic_ll <- function(time, status, time_lower = NULL, time_upper = NULL,
                     weights = NULL) {
  TemporalHazard:::.hzr_logl_multiphase(
    theta = mp_ic_theta, time = time, status = status,
    time_lower = time_lower, time_upper = time_upper,
    phases = mp_ic_phases(), covariate_counts = mp_ic_cc,
    x_list = mp_ic_xl, weights = weights)
}

# ---------------------------------------------------------------------------

test_that("multiphase log-likelihood handles all four censoring statuses", {
  # One row of each: exact event, right-censored, left-censored, interval.
  time       <- c(1.0, 2.0, 1.5, 0.8)
  status     <- c(1L, 0L, -1L, 2L)
  time_lower <- c(1.0, 2.0, 0.0, 0.6)
  time_upper <- c(1.0, 2.0, 1.5, 1.4)

  ll <- mp_ic_ll(time, status, time_lower, time_upper)
  expect_true(is.finite(ll))
})

test_that("multiphase interval contribution equals log(S(lower) - S(upper))", {
  l <- 0.9
  u <- 1.7
  ll <- mp_ic_ll(time = 1.0, status = 2L, time_lower = l, time_upper = u)

  # -H(l) + log(1 - exp(-(H(u) - H(l))))  ==  log(S(l) - S(u)).
  manual_form  <- -mp_ic_H(l) + hzr_log1mexp(mp_ic_H(u) - mp_ic_H(l))
  manual_surv  <- log(exp(-mp_ic_H(l)) - exp(-mp_ic_H(u)))

  expect_equal(unname(ll), unname(manual_form), tolerance = 1e-10)
  expect_equal(unname(ll), unname(manual_surv), tolerance = 1e-10)
})

test_that("multiphase left-censored contribution equals log(1 - exp(-H(u)))", {
  u  <- 1.3
  ll <- mp_ic_ll(time = u, status = -1L)
  manual <- hzr_log1mexp(mp_ic_H(u))
  expect_equal(unname(ll), unname(manual), tolerance = 1e-10)
})

test_that("multiphase right-censored contribution is -(H(stop) - H(start))", {
  # Plain right-censored (no entry time): -H(stop).
  ll_plain <- mp_ic_ll(time = c(0.7, 1.2, 2.0), status = c(0L, 0L, 0L))
  expect_equal(unname(ll_plain), -sum(mp_ic_H(c(0.7, 1.2, 2.0))),
               tolerance = 1e-10)

  # Left-truncated (counting-process start): -(H(stop) - H(start)).
  stop_t  <- c(1.5, 2.2)
  start_t <- c(0.3, 0.8)
  ll_lt <- mp_ic_ll(time = stop_t, status = c(0L, 0L), time_lower = start_t)
  expect_equal(unname(ll_lt),
               -sum(mp_ic_H(stop_t) - mp_ic_H(start_t)),
               tolerance = 1e-10)
})

test_that("multiphase interval bounds are validated (lower > upper -> -Inf)", {
  # Specifically -Inf (an infeasible interval), not merely non-finite: a
  # likelihood of +Inf would be a bug, so assert the exact value.
  expect_equal(
    mp_ic_ll(time = 1.0, status = 2L, time_lower = 2.0, time_upper = 1.0),
    -Inf)
})

test_that("interval-censored multiphase: integer weights match row duplication", {
  # The interval branch of the weighted sum must scale by the row weight,
  # i.e. a weight-w interval row equals w identical interval rows.  Covers the
  # interval x weights x multiphase combination (the weights tests use only
  # event/right-censored multiphase rows).
  set.seed(91)
  n <- 24
  status <- rep(2L, n)
  lo <- runif(n, 0.1, 1.5)
  up <- lo + runif(n, 0.1, 1.0)
  w  <- sample(1:3, n, replace = TRUE)
  idx <- rep(seq_len(n), times = w)

  ll_w   <- mp_ic_ll(time = lo, status = status,
                     time_lower = lo, time_upper = up, weights = w)
  ll_dup <- mp_ic_ll(time = lo[idx], status = status[idx],
                     time_lower = lo[idx], time_upper = up[idx],
                     weights = rep(1, sum(w)))
  expect_equal(ll_w, ll_dup, tolerance = 1e-10)
})

test_that("multiphase fit converges on an interval-censored sample", {
  skip_on_cran()
  set.seed(92)
  n <- 200
  true_t <- rexp(n, rate = 0.6) + 0.02
  # Interval-censor everyone into 0.5-wide windows around the true time.
  brk        <- ceiling(true_t / 0.5)
  time_lower <- (brk - 1L) * 0.5
  time_upper <- brk * 0.5
  status     <- rep(2L, n)

  fit <- suppressWarnings(hazard(
    time = time_lower, status = status,
    time_lower = time_lower, time_upper = time_upper,
    dist = "multiphase", phases = mp_ic_phases(),
    fit = TRUE, control = list(n_starts = 1L, maxit = 500L, conserve = FALSE)
  ))

  # The optimizer must actually converge on interval-censored multiphase data
  # (not merely return a logical flag).
  expect_true(isTRUE(fit$fit$converged))
  expect_true(is.finite(fit$fit$objective))
  # Two intercepts estimated (early + constant), shapes held fixed.
  expect_length(fit$fit$theta, 5L)
})

# Tier-1 correctness invariants (see inst/dev/CORRECTNESS-STRATEGY.md).
#
# These are R-only, single-engine checks: properties every fitted model must
# satisfy by the mathematics, with no SAS reference and no captured fixtures.
# They are the cheap workhorse of the correctness regime -- they caught (or
# would have caught) the multiphase vcov, CoE left-truncation, deciles-method,
# and CoE final-adjustment bugs.
#
# Covered here: distributional sanity (S in [0,1] and monotone; H monotone and
# >= 0; S = exp(-H)), predict-type internal consistency, multiphase log-
# likelihood self-consistency (LL at returned theta == reported objective), and
# the Conservation-of-Events identity. Future expansion (documented, not yet
# implemented): stationarity (score ~ 0 at the MLE) and analytic-vs-numDeriv
# gradient agreement -- the latter is already exercised per-distribution by the
# analytic-Hessian tests.

# ---------------------------------------------------------------------------
# Synthetic data generators spanning the feature space.
# ---------------------------------------------------------------------------

# Right-censored survival data with one covariate. Deterministic given `seed`.
.inv_sim <- function(n = 300, rate = 0.3, cens_max = 6, seed = 1) {
  set.seed(seed)
  x <- rnorm(n)
  t <- rexp(n, rate = rate) + 0.01
  cens <- runif(n, 0.2, cens_max)
  data.frame(time = pmin(t, cens), status = as.integer(t <= cens), x = x)
}

# Left-truncated (counting-process) two-phase-ish data. Deterministic.
.inv_sim_lt <- function(n = 400, seed = 7) {
  set.seed(seed)
  u <- runif(n)
  tev <- ifelse(u < 0.5, rexp(n, 3), rexp(n, 0.4))
  cens <- runif(n, 0.2, 6)
  stop <- pmin(tev, cens)
  data.frame(start = pmin(runif(n, 0, 1), 0.8 * stop),
             stop = stop, event = as.integer(tev <= cens))
}

# ---------------------------------------------------------------------------
# Model matrix: a list of fitted models spanning distributions, covariates,
# conservation, left-truncation and weights. Each entry carries the newdata
# grid (a fixed covariate profile over a time grid) for the distributional
# checks, and the raw data for the CoE / self-consistency checks.
# ---------------------------------------------------------------------------
.inv_models <- function() {
  df  <- .inv_sim()
  dlt <- .inv_sim_lt()
  set.seed(11)
  df$w <- sample(1:3, nrow(df), replace = TRUE)  # integer weights
  tg  <- seq(0.05, 5, length.out = 40)

  mk <- function(name, fit, grid, data = NULL, multiphase = FALSE,
                 conserve = FALSE, time = NULL, status = NULL,
                 time_lower = NULL, weights = NULL) {
    list(name = name, fit = fit, grid = grid, data = data,
         multiphase = multiphase, conserve = conserve,
         time = time, status = status, time_lower = time_lower,
         weights = weights)
  }

  models <- list()

  # --- Single-distribution families (intercept + one covariate) ------------
  models[[length(models) + 1L]] <- mk("weibull",
    hazard(survival::Surv(time, status) ~ x, data = df,
           theta = c(0.3, 1, 0), dist = "weibull", fit = TRUE),
    grid = data.frame(time = tg, x = 0))
  models[[length(models) + 1L]] <- mk("exponential",
    hazard(survival::Surv(time, status) ~ x, data = df,
           theta = c(log(0.3), 0), dist = "exponential", fit = TRUE),
    grid = data.frame(time = tg, x = 0))
  models[[length(models) + 1L]] <- mk("loglogistic",
    hazard(survival::Surv(time, status) ~ x, data = df,
           theta = c(0, 0.2, 0), dist = "loglogistic", fit = TRUE),
    grid = data.frame(time = tg, x = 0))
  models[[length(models) + 1L]] <- mk("lognormal",
    hazard(survival::Surv(time, status) ~ x, data = df,
           theta = c(1, 0, 0), dist = "lognormal", fit = TRUE),
    grid = data.frame(time = tg, x = 0))

  # --- Multiphase: conserve on/off, covariates, truncation, weights --------
  two_phase <- function(formula, data, conserve) {
    set.seed(101)
    hazard(formula, data = data, dist = "multiphase",
           phases = list(
             early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                                  fixed = "shapes"),
             constant = hzr_phase("constant")),
           fit = TRUE,
           control = list(n_starts = 1, maxit = 500, conserve = conserve))
  }

  models[[length(models) + 1L]] <- mk("multiphase-coe",
    two_phase(survival::Surv(time, status) ~ 1, df, TRUE),
    grid = data.frame(time = tg), data = df, multiphase = TRUE, conserve = TRUE,
    time = df$time, status = df$status)

  models[[length(models) + 1L]] <- mk("multiphase-free",
    two_phase(survival::Surv(time, status) ~ 1, df, FALSE),
    grid = data.frame(time = tg), data = df, multiphase = TRUE, conserve = FALSE,
    time = df$time, status = df$status)

  set.seed(101)
  mp_cov <- hazard(survival::Surv(time, status) ~ 1, data = df,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                           fixed = "shapes", formula = ~ x),
      constant = hzr_phase("constant")),
    fit = TRUE, control = list(n_starts = 1, maxit = 500, conserve = TRUE))
  models[[length(models) + 1L]] <- mk("multiphase-covariate",
    mp_cov, grid = data.frame(time = tg, x = 0), data = df,
    multiphase = TRUE, conserve = TRUE, time = df$time, status = df$status)

  models[[length(models) + 1L]] <- mk("multiphase-truncated",
    two_phase(survival::Surv(start, stop, event) ~ 1, dlt, TRUE),
    grid = data.frame(time = tg), data = dlt, multiphase = TRUE,
    conserve = TRUE, time = dlt$stop, status = dlt$event,
    time_lower = dlt$start)

  set.seed(101)
  mp_w <- hazard(survival::Surv(time, status) ~ 1, data = df,
    dist = "multiphase", weights = df$w,
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.3, nu = 1, m = 1,
                           fixed = "shapes"),
      constant = hzr_phase("constant")),
    fit = TRUE, control = list(n_starts = 1, maxit = 500, conserve = TRUE))
  models[[length(models) + 1L]] <- mk("multiphase-weighted",
    mp_w, grid = data.frame(time = tg), data = df, multiphase = TRUE,
    conserve = TRUE, time = df$time, status = df$status, weights = df$w)

  models
}

# Memoized accessor: the model matrix is fit once per test-file run and reused
# across the invariant test_that() blocks (otherwise .inv_models() refits the
# whole matrix once per block).
.inv_models_cache <- new.env(parent = emptyenv())
.inv_models_cached <- function() {
  if (is.null(.inv_models_cache$models)) {
    .inv_models_cache$models <- .inv_models()
  }
  .inv_models_cache$models
}

# ---------------------------------------------------------------------------
# Invariant assertions.
# ---------------------------------------------------------------------------

# S(t) in [0,1] and non-increasing; H(t) >= 0 and non-decreasing; S = exp(-H)
# (which also pins predict("survival") and predict("cumulative_hazard") to be
# mutually consistent).
.inv_assert_distribution <- function(m) {
  S <- unname(predict(m$fit, newdata = m$grid, type = "survival"))
  H <- unname(predict(m$fit, newdata = m$grid, type = "cumulative_hazard"))
  testthat::expect_true(all(is.finite(S)) && all(is.finite(H)),
                        label = paste0(m$name, ": finite S/H"))
  testthat::expect_true(all(S >= -1e-9 & S <= 1 + 1e-9),
                        label = paste0(m$name, ": S in [0,1]"))
  testthat::expect_true(all(diff(S) <= 1e-9),
                        label = paste0(m$name, ": S non-increasing"))
  testthat::expect_true(all(H >= -1e-9),
                        label = paste0(m$name, ": H >= 0"))
  testthat::expect_true(all(diff(H) >= -1e-9),
                        label = paste0(m$name, ": H non-decreasing"))
  testthat::expect_equal(S, exp(-H), tolerance = 1e-6,
                         label = paste0(m$name, ": S = exp(-H)"))
}

# LL recomputed at the returned coefficients equals the reported objective.
.inv_assert_self_consistent <- function(m) {
  w <- if (is.null(m$weights)) rep(1, length(m$time)) else m$weights
  ll <- TemporalHazard:::.hzr_logl_multiphase(
    m$fit$fit$theta, m$time, m$status, time_lower = m$time_lower,
    phases = m$fit$fit$phases, covariate_counts = m$fit$fit$covariate_counts,
    x_list = m$fit$fit$x_list, weights = w)
  testthat::expect_equal(ll, m$fit$fit$objective, tolerance = 1e-6,
                         label = paste0(m$name, ": LL(theta) == objective"))
}

# Conservation of Events: sum of (weighted) observed events equals sum of
# (weighted) predicted cumulative hazard over follow-up, H(stop) - H(start).
.inv_assert_coe <- function(m) {
  th <- m$fit$fit$theta
  ph <- m$fit$fit$phases
  cc <- m$fit$fit$covariate_counts
  xl <- m$fit$fit$x_list
  w  <- if (is.null(m$weights)) rep(1, length(m$time)) else m$weights
  H_stop  <- TemporalHazard:::.hzr_multiphase_cumhaz(m$time, th, ph, cc, xl)
  H_start <- if (is.null(m$time_lower)) 0 else
    TemporalHazard:::.hzr_multiphase_cumhaz(m$time_lower, th, ph, cc, xl)
  pred <- sum(w * (H_stop - H_start))
  obs  <- sum(w * m$status)
  # Relative tolerance with an absolute floor so a zero-event group does not
  # demand exact floating-point equality.
  testthat::expect_equal(pred, obs, tolerance = max(1e-6, 1e-3 * obs),
                         label = paste0(m$name, ": CoE sum H == sum events"))
}

# test-multiphase-gradient.R — Validate analytic gradient against numerical
#
# Tests that .hzr_gradient_multiphase() returns gradients consistent with
# central-difference numerical gradients of .hzr_logl_multiphase().
# This validates the semi-analytic gradient (chain-rule for mu/beta,
# central-diff for shape params via .hzr_phase_derivatives()).

# ============================================================================
# Helpers
# ============================================================================

load_kul_csv <- function() {
  csv_path <- system.file("extdata", "cabgkul.csv", package = "TemporalHazard")
  if (!nzchar(csv_path) || !file.exists(csv_path)) {
    csv_path <- file.path("inst", "extdata", "cabgkul.csv")
  }
  if (!file.exists(csv_path)) return(NULL)
  utils::read.csv(csv_path)
}

kul_phases <- function() {
  list(
    early    = hzr_phase("cdf",  t_half = 0.2, nu = 1, m = 1),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",   tau = 1, gamma = 3, alpha = 1, eta = 1)
  )
}

# Central-difference numerical gradient (tight step, reference implementation)
numerical_gradient <- function(theta, time, status, phases,
                                covariate_counts, x_list,
                                time_lower = NULL, time_upper = NULL) {
  p <- length(theta)
  grad <- numeric(p)
  eps_rel <- (.Machine$double.eps)^(1 / 3)

  for (i in seq_len(p)) {
    h_i <- eps_rel * max(abs(theta[i]), 1)

    theta_plus <- theta
    theta_plus[i] <- theta_plus[i] + h_i
    theta_minus <- theta
    theta_minus[i] <- theta_minus[i] - h_i

    ll_plus <- .hzr_logl_multiphase(
      theta_plus, time, status,
      time_lower = time_lower, time_upper = time_upper,
      phases = phases, covariate_counts = covariate_counts, x_list = x_list
    )
    ll_minus <- .hzr_logl_multiphase(
      theta_minus, time, status,
      time_lower = time_lower, time_upper = time_upper,
      phases = phases, covariate_counts = covariate_counts, x_list = x_list
    )

    if (is.finite(ll_plus) && is.finite(ll_minus)) {
      grad[i] <- (ll_plus - ll_minus) / (2 * h_i)
    } else if (is.finite(ll_plus)) {
      ll0 <- .hzr_logl_multiphase(
        theta, time, status,
        time_lower = time_lower, time_upper = time_upper,
        phases = phases, covariate_counts = covariate_counts, x_list = x_list
      )
      grad[i] <- (ll_plus - ll0) / h_i
    }
  }

  grad
}


# ============================================================================
# Phase derivative unit tests
# ============================================================================

test_that(".hzr_phase_derivatives returns correct structure", {
  t_grid <- seq(0.5, 10, by = 0.5)

  pd <- .hzr_phase_derivatives(
    t_grid,
    t_half = 2,
    nu = 1.5,
    m = 0.8,
    type = "cdf"
  )
  expect_named(
    pd,
    c(
      "Phi", "phi", "dPhi_dlog_thalf", "dPhi_dnu", "dPhi_dm",
      "dphi_dlog_thalf", "dphi_dnu", "dphi_dm"
    )
  )
  expect_length(pd$Phi, length(t_grid))
  expect_true(all(is.finite(pd$Phi)))
  expect_true(all(is.finite(pd$dPhi_dlog_thalf)))
})

test_that(".hzr_phase_derivatives for constant phase returns zeros", {
  t_grid <- seq(0.5, 5, by = 0.5)
  pd <- .hzr_phase_derivatives(t_grid, type = "constant")

  expect_equal(pd$Phi, t_grid)
  expect_equal(pd$phi, rep(1, length(t_grid)))
  expect_equal(pd$dPhi_dlog_thalf, rep(0, length(t_grid)))
  expect_equal(pd$dPhi_dnu, rep(0, length(t_grid)))
  expect_equal(pd$dPhi_dm, rep(0, length(t_grid)))
})

test_that(".hzr_phase_derivatives matches numerical for cdf phase (case 1)", {
  t_grid <- seq(0.5, 8, by = 0.5)
  t_half <- 3
  nu <- 2
  m <- 1

  pd <- .hzr_phase_derivatives(
    t_grid,
    t_half = t_half,
    nu = nu,
    m = m,
    type = "cdf"
  )

  # Numerical check for dPhi/d(log_t_half)
  h <- 1e-5
  Phi_plus  <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half * exp(h),
    nu = nu,
    m = m,
    type = "cdf"
  )
  Phi_minus <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half * exp(-h),
    nu = nu,
    m = m,
    type = "cdf"
  )
  num_dPhi_dlog_thalf <- (Phi_plus - Phi_minus) / (2 * h)
  expect_equal(pd$dPhi_dlog_thalf, num_dPhi_dlog_thalf, tolerance = 1e-4)

  # Numerical check for dPhi/d(nu)
  Phi_plus  <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half,
    nu = nu + h,
    m = m,
    type = "cdf"
  )
  Phi_minus <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half,
    nu = nu - h,
    m = m,
    type = "cdf"
  )
  num_dPhi_dnu <- (Phi_plus - Phi_minus) / (2 * h)
  expect_equal(pd$dPhi_dnu, num_dPhi_dnu, tolerance = 1e-4)
})

test_that(".hzr_phase_derivatives matches numerical for hazard phase", {
  t_grid <- seq(0.5, 8, by = 0.5)
  t_half <- 5
  nu <- 1
  m <- 0

  pd <- .hzr_phase_derivatives(
    t_grid,
    t_half = t_half,
    nu = nu,
    m = m,
    type = "hazard"
  )

  h <- 1e-5
  Phi_plus  <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half * exp(h),
    nu = nu,
    m = m,
    type = "hazard"
  )
  Phi_minus <- hzr_phase_cumhaz(
    t_grid,
    t_half = t_half * exp(-h),
    nu = nu,
    m = m,
    type = "hazard"
  )
  num_dPhi_dlog_thalf <- (Phi_plus - Phi_minus) / (2 * h)
  expect_equal(pd$dPhi_dlog_thalf, num_dPhi_dlog_thalf, tolerance = 1e-4)
})

test_that(".hzr_phase_derivatives works for all 6 decomposition cases", {
  t_grid <- seq(0.5, 5, by = 0.5)
  cases <- list(
    list(t_half = 3, nu = 2,  m = 1),    # Case 1: m>0, nu>0
    list(t_half = 3, nu = 2,  m = 0),    # Case 1L: m=0, nu>0
    list(t_half = 3, nu = 2,  m = -0.5), # Case 2: m<0, nu>0
    list(t_half = 3, nu = 0,  m = -0.5), # Case 2L: m<0, nu=0
    list(t_half = 3, nu = -1, m = 1),    # Case 3: m>0, nu<0
    list(t_half = 3, nu = -1, m = 0)     # Case 3L: m=0, nu<0
  )

  for (i in seq_along(cases)) {
    cc <- cases[[i]]
    for (type in c("cdf", "hazard")) {
      pd <- .hzr_phase_derivatives(
        t_grid,
        t_half = cc$t_half,
        nu = cc$nu,
        m = cc$m,
        type = type
      )
      expect_true(
        all(is.finite(pd$Phi)),
        label = paste("Case", i, type, "Phi finite")
      )
      expect_true(
        all(is.finite(pd$dPhi_dlog_thalf)),
        label = paste("Case", i, type, "dPhi/dlog_thalf finite")
      )
      expect_true(
        all(is.finite(pd$dPhi_dnu)),
        label = paste("Case", i, type, "dPhi/dnu finite")
      )
      expect_true(
        all(is.finite(pd$dPhi_dm)),
        label = paste("Case", i, type, "dPhi/dm finite")
      )
    }
  }
})


# ============================================================================
# Full gradient tests on KUL dataset
# ============================================================================

test_that("Analytic gradient matches numerical at C reference parameters", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  phases <- kul_phases()
  covariate_counts <- c(early = 0L, constant = 0L, late = 0L)
  x_list <- list(early = NULL, constant = NULL, late = NULL)

  # C converged parameters on internal scale
  theta_c <- c(
    -3.77955,   # early.log_mu  (mu = 0.02268)
    log(0.2),   # early.log_t_half
    1,           # early.nu
    1,           # early.m
    -7.2258,     # constant.log_mu  (mu = 0.0007269)
    -16.6578,    # late.log_mu  (mu = 5.837e-08)
    log(1),      # late.log_tau
    3,           # late.gamma
    1,           # late.alpha
    1            # late.eta
  )

  grad_analytic <- .hzr_gradient_multiphase(
    theta_c, dat$int_dead, dat$dead,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  grad_numeric <- numerical_gradient(
    theta_c, dat$int_dead, dat$dead,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  # Both should be finite

  expect_true(all(is.finite(grad_analytic)),
              label = "Analytic gradient all finite")
  expect_true(all(is.finite(grad_numeric)),
              label = "Numerical gradient all finite")

  # Relative error should be small for each component
  # Allow looser tolerance for shape params (central-diff vs central-diff)
  denom <- pmax(abs(grad_numeric), 1e-6)
  rel_err <- abs(grad_analytic - grad_numeric) / denom

  # log_mu and constant parameters should match tightly
  expect_true(rel_err[1] < 0.01, label = "early.log_mu gradient < 1% error")
  expect_true(rel_err[5] < 0.01, label = "constant.log_mu gradient < 1% error")
  expect_true(rel_err[6] < 0.01, label = "late.log_mu gradient < 1% error")

  # Shape params may have larger relative error near zero gradients
  # but absolute agreement should be reasonable
  abs_err <- abs(grad_analytic - grad_numeric)
  expect_true(all(abs_err < 1),
              label = paste("Max absolute gradient error:",
                            round(max(abs_err), 4)))
})

test_that("Analytic gradient matches numerical at SAS starting values", {
  skip_on_cran()

  dat <- load_kul_csv()
  skip_if(is.null(dat), "KUL dataset not available")

  phases <- kul_phases()
  covariate_counts <- c(early = 0L, constant = 0L, late = 0L)
  x_list <- list(early = NULL, constant = NULL, late = NULL)

  # SAS PARMS starting values
  theta_start <- c(
    log(0.02), log(0.2), 1, 1,
    log(0.0008),
    log(1e-9), log(1), 3, 1, 1
  )

  grad_analytic <- .hzr_gradient_multiphase(
    theta_start, dat$int_dead, dat$dead,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  grad_numeric <- numerical_gradient(
    theta_start, dat$int_dead, dat$dead,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  expect_true(all(is.finite(grad_analytic)))
  expect_true(all(is.finite(grad_numeric)))

  # At the starting point the gradients should be non-trivial and match
  denom <- pmax(abs(grad_numeric), 1e-6)
  rel_err <- abs(grad_analytic - grad_numeric) / denom

  # mu params should match well
  expect_true(rel_err[1] < 0.05, label = "early.log_mu gradient match")
  expect_true(rel_err[5] < 0.05, label = "constant.log_mu gradient match")
  expect_true(rel_err[6] < 0.05, label = "late.log_mu gradient match")
})


# ============================================================================
# Gradient on synthetic data with covariates
# ============================================================================

test_that("Analytic gradient works with covariates", {
  skip_on_cran()

  set.seed(42)
  n <- 200
  time   <- rexp(n, rate = 0.3) + 0.01
  status <- sample(0:1, n, replace = TRUE, prob = c(0.3, 0.7))
  x_mat  <- matrix(rnorm(n * 2), ncol = 2, dimnames = list(NULL, c("age", "sex")))

  phases <- list(
    early = hzr_phase("cdf",    t_half = 1, nu = 1, m = 0),
    late  = hzr_phase("hazard", t_half = 5, nu = 1, m = 0)
  )
  covariate_counts <- c(early = 2L, late = 2L)
  x_list <- list(early = x_mat, late = x_mat)

  # theta: [log_mu, log_t_half, nu, m, beta1, beta2] x 2 phases
  theta <- c(
    log(0.1), log(1), 1, 0, 0.5, -0.3,   # early
    log(0.01), log(5), 1, 0, 0.1, 0.2     # late
  )

  grad_analytic <- .hzr_gradient_multiphase(
    theta, time, status,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  grad_numeric <- numerical_gradient(
    theta, time, status,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  expect_true(all(is.finite(grad_analytic)))
  expect_true(all(is.finite(grad_numeric)))

  # Check beta parameters specifically (indices 5,6 and 11,12)
  beta_idx <- c(5, 6, 11, 12)
  denom <- pmax(abs(grad_numeric[beta_idx]), 1e-6)
  rel_err_beta <- abs(grad_analytic[beta_idx] - grad_numeric[beta_idx]) / denom
  expect_true(all(rel_err_beta < 0.05),
              label = paste("Beta gradient max rel error:",
                            round(max(rel_err_beta), 4)))
})


# ============================================================================
# Simple 2-phase model: gradient at known parameters
# ============================================================================

test_that("Gradient is near zero at converged fit", {
  skip_on_cran()

  set.seed(42)
  n <- 100
  time   <- rexp(n, rate = 0.5) + 0.01
  status <- sample(0:1, n, replace = TRUE, prob = c(0.2, 0.8))

  phases <- list(
    early = hzr_phase("cdf",    t_half = 1, nu = 1.5, m = 0),
    const = hzr_phase("constant")
  )

  fit <- tryCatch(
    suppressWarnings(hazard(time = time, status = status, dist = "multiphase",
           phases = phases, fit = TRUE,
           control = list(n_starts = 3, maxit = 500))),
    error = function(e) NULL
  )
  skip_if(is.null(fit), "Fit did not converge")
  skip_if(!fit$fit$converged, "Fit did not converge")

  covariate_counts <- c(early = 0L, const = 0L)
  x_list <- list(early = NULL, const = NULL)

  grad_at_mle <- .hzr_gradient_multiphase(
    fit$fit$theta, time, status,
    phases = phases, covariate_counts = covariate_counts, x_list = x_list
  )

  # Gradient should be reasonably small at the MLE.  With a small sample

  # (n=100) and rough BFGS convergence the per-observation score components
  # don't cancel perfectly, so we use a generous threshold.
  max_abs_grad <- max(abs(grad_at_mle))
  expect_true(max_abs_grad < 500,
              label = paste("Max |gradient| at MLE:", round(max_abs_grad, 2),
                            "should be < 500"))
})

#' Parse Surv() formula for hazard modeling
#'
#' Extracts time, status, time_lower, time_upper, and predictors from a formula
#' of the form `Surv(time, status) ~ x1 + x2 + ...`.
#' Supports right-censored, left-censored, interval-censored, and
#' counting-process (start-stop) data.
#'
#' For counting-process (start-stop) data, use `Surv(start, stop, event)`.
#' The start times are returned as `time_lower` and stop times as `time`,
#' enabling the likelihood to compute `H(stop) - H(start)` per epoch.
#'
#' @param formula A formula object with Surv() on the LHS.
#' @param data A data frame containing variables referenced in the formula.
#' @return A list with elements: time, status, time_lower, time_upper, x
#' @keywords internal
.hzr_parse_formula <- function(formula, data) {
  if (!is.data.frame(data)) {
    stop("'data' must be a data frame.", call. = FALSE)
  }

  if (!inherits(formula, "formula")) {
    stop("'formula' must be a formula object.", call. = FALSE)
  }

  # Parse the LHS (should be Surv(...))
  lhs <- formula[[2L]]
  rhs <- formula[[3L]]

  # Extract Surv() call
  if (!is.call(lhs) || !grepl("Surv$", deparse(lhs[[1L]]))) {
    stop("Formula LHS must be a Surv() call.", call. = FALSE)
  }

  # Evaluate the entire Surv() call in the context of data
  # Make sure Surv is available in the evaluation environment
  env_with_surv <- list2env(c(as.list(data), list(Surv = survival::Surv)))
  surv_obj <- eval(lhs, envir = env_with_surv)

  # Surv() returns a Surv object; extract the matrix and attributes
  if (!inherits(surv_obj, "Surv")) {
    stop("Formula LHS must return a Surv object.", call. = FALSE)
  }

  surv_type <- attr(surv_obj, "type")
  surv_mat <- unclass(surv_obj)

  if (surv_type == "right") {
    # Format: [time, status]
    time <- surv_mat[, 1L]
    status <- surv_mat[, 2L]
    time_lower <- NULL
    time_upper <- NULL
  } else if (surv_type == "left") {
    # Format: [time, status]
    time <- surv_mat[, 1L]
    status <- surv_mat[, 2L]
    time_lower <- NULL
    time_upper <- surv_mat[, 1L]
  } else if (surv_type == "interval") {
    # Format: [time1, time2, status]
    time_lower <- surv_mat[, 1L]
    time <- surv_mat[, 1L]
    time_upper <- surv_mat[, 2L]
    status <- surv_mat[, 3L]
  } else if (surv_type == "counting") {
    # Start-stop (counting process) format: Surv(start, stop, event)
    # Used for repeating events / epoch-decomposed longitudinal data.
    # Each epoch contributes H(stop) - H(start) to the likelihood.
    time_lower <- surv_mat[, 1L]  # entry (start) time
    time <- surv_mat[, 2L]        # exit (stop) time
    time_upper <- NULL
    status <- surv_mat[, 3L]
  } else {
    stop("Unsupported Surv() type: ", surv_type, call. = FALSE)
  }

  # Parse RHS (predictors)
  x <- NULL
  if (!is.null(rhs)) {
    # Reconstruct as a formula for model.matrix()
    rhs_formula <- formula(paste("~", deparse(rhs)))
    tryCatch({
      x <- stats::model.matrix(rhs_formula, data = data)
      # Remove intercept column if present
      if (ncol(x) > 0 && colnames(x)[1L] == "(Intercept)") {
        x <- x[, -1L, drop = FALSE]
      }
      if (ncol(x) == 0) {
        x <- NULL
      }
    }, error = function(e) {
      stop("Failed to parse formula RHS: ", e$message, call. = FALSE)
    })
  }

  list(
    time = time,
    status = status,
    time_lower = time_lower,
    time_upper = time_upper,
    x = x,
    surv_type = surv_type
  )
}

# tests/testthat/test-formula-helpers.R
# Unit tests for .hzr_parse_formula() covering the four Surv() types it
# dispatches on (right, left, interval, counting-process) and its error
# paths.  These are the only tests that exercise R/formula-helpers.R
# directly -- integration tests elsewhere exercise it implicitly via
# hazard(), so a bug in the parser would only surface as odd downstream
# behaviour without these.

parse <- function(...) TemporalHazard:::.hzr_parse_formula(...)

# ---------------------------------------------------------------------------
# Right-censored: Surv(time, status)
# ---------------------------------------------------------------------------

test_that("right-censored Surv parses into time + status only", {
  df <- data.frame(
    t = c(1.5, 2.0, 3.25),
    d = c(1, 0, 1),
    x = c(0.1, 0.2, 0.3)
  )
  out <- parse(survival::Surv(t, d) ~ x, data = df)

  expect_equal(out$time,   df$t)
  expect_equal(out$status, df$d)
  expect_null(out$time_lower)
  expect_null(out$time_upper)
  expect_equal(as.numeric(out$x), df$x)
  expect_equal(colnames(out$x), "x")
})

# ---------------------------------------------------------------------------
# Left-censored: Surv(time, status, type = "left")
# ---------------------------------------------------------------------------

test_that("left-censored Surv sets time_upper but not time_lower", {
  df <- data.frame(
    t = c(1, 2, 3),
    d = c(1, 0, 1)
  )
  out <- parse(
    survival::Surv(t, d, type = "left") ~ 1,
    data = df
  )

  expect_equal(out$time,       df$t)
  expect_equal(out$status,     df$d)
  expect_null(out$time_lower)
  expect_equal(out$time_upper, df$t)
  expect_null(out$x)
})

# ---------------------------------------------------------------------------
# Interval-censored: Surv(time1, time2, type = "interval2")
# ---------------------------------------------------------------------------

test_that("interval-censored Surv extracts time_lower + time_upper + status", {
  # All rows interval-censored so Surv doesn't overwrite any bound.
  df <- data.frame(
    lo = c(1.0, 2.0, 3.0),
    hi = c(1.5, 2.8, 3.5)
  )
  out <- parse(
    survival::Surv(lo, hi, type = "interval2") ~ 1,
    data = df
  )

  # interval status codes coming out of Surv are 0/1/2/3; the parser
  # keeps them as-is, just routes lower/upper columns.
  expect_length(out$time, 3)
  expect_equal(out$time_lower, df$lo)
  expect_equal(out$time_upper, df$hi)
  expect_true(all(out$status %in% c(0, 1, 2, 3)))
})

# ---------------------------------------------------------------------------
# Counting-process (start-stop): Surv(start, stop, event)
# ---------------------------------------------------------------------------

test_that("counting-process Surv maps start->time_lower and stop->time", {
  df <- data.frame(
    start = c(0.0, 1.5, 0.0),
    stop  = c(1.5, 2.5, 3.0),
    event = c(0,   1,   1),
    age   = c(50,  50,  62)
  )
  out <- parse(survival::Surv(start, stop, event) ~ age, data = df)

  expect_equal(out$time_lower, df$start)
  expect_equal(out$time,       df$stop)
  expect_equal(out$status,     df$event)
  expect_null(out$time_upper)
  expect_equal(as.numeric(out$x), df$age)
})

# ---------------------------------------------------------------------------
# Predictor handling
# ---------------------------------------------------------------------------

test_that("intercept column is dropped from the design matrix", {
  df <- data.frame(t = c(1, 2, 3), d = c(1, 0, 1), a = c(10, 20, 30))
  out <- parse(survival::Surv(t, d) ~ a, data = df)
  expect_false("(Intercept)" %in% colnames(out$x))
  expect_equal(colnames(out$x), "a")
})

test_that("intercept-only RHS returns NULL design matrix", {
  df <- data.frame(t = c(1, 2, 3), d = c(1, 0, 1))
  out <- parse(survival::Surv(t, d) ~ 1, data = df)
  expect_null(out$x)
})

test_that("factor predictors expand to dummy columns", {
  df <- data.frame(
    t = c(1, 2, 3, 4),
    d = c(1, 0, 1, 0),
    g = factor(c("a", "b", "a", "c"))
  )
  out <- parse(survival::Surv(t, d) ~ g, data = df)
  # Factor with 3 levels -> 2 dummies (baseline absorbed)
  expect_equal(ncol(out$x), 2L)
  expect_true(all(grepl("^g", colnames(out$x))))
})

test_that("multi-term RHS preserves all predictors in order", {
  df <- data.frame(
    t = 1:4, d = c(1, 0, 1, 0),
    a = c(0.1, 0.2, 0.3, 0.4),
    b = c(1, 2, 3, 4),
    c = c(10, 20, 30, 40)
  )
  out <- parse(survival::Surv(t, d) ~ a + b + c, data = df)
  expect_equal(colnames(out$x), c("a", "b", "c"))
})

# ---------------------------------------------------------------------------
# Error paths
# ---------------------------------------------------------------------------

test_that("non-data.frame data is rejected", {
  expect_error(
    parse(survival::Surv(t, d) ~ 1, data = list(t = 1:3, d = c(1, 0, 1))),
    "data frame"
  )
})

test_that("non-formula input is rejected", {
  df <- data.frame(t = 1:3, d = c(1, 0, 1))
  expect_error(parse("Surv(t, d) ~ 1", data = df), "formula")
})

test_that("LHS without Surv() is rejected", {
  df <- data.frame(t = 1:3, d = c(1, 0, 1))
  expect_error(parse(t ~ d, data = df), "Surv\\(\\) call")
})

test_that("unknown predictor in RHS raises an informative error", {
  df <- data.frame(t = 1:3, d = c(1, 0, 1))
  expect_error(
    parse(survival::Surv(t, d) ~ nonexistent, data = df),
    "Failed to parse formula RHS"
  )
})

# argument_mapping.R -- SAS HAZARD -> R argument translation table
#
# PURPOSE
# -------
# During the SAS-to-R migration, analysts may need to know how legacy HAZARD
# procedure statements map to hazard() arguments.  hzr_argument_mapping()
# exposes the mapping as a data frame so it can be consulted programmatically
# or rendered in vignettes.
#
# The internal table .hzr_argument_mapping_table drives the exported function.
# Add rows as new arguments are implemented; update implementation_status from
# "planned" to "implemented" when the feature lands.

#' Legacy HAZARD to TemporalHazard argument mapping
#'
#' Returns a formal mapping table that defines how legacy SAS HAZARD/C-style
#' inputs map to `hazard(...)` arguments in this package.
#'
#' @param include_planned Logical; if `FALSE`, only rows marked as implemented
#'   are returned.
#'
#' @return A data frame with one row per mapping rule.
#' @seealso [hazard()] for the target arguments, [hzr_phase()] for phase
#'   construction, and [hzr_decompos()]/[hzr_decompos_g3()] for the
#'   parameter-level early/late translations.
#' @examples
#' hzr_argument_mapping()
#' hzr_argument_mapping(include_planned = FALSE)
#' @export
hzr_argument_mapping <- function(include_planned = TRUE) {
  map <- .hzr_argument_mapping_table
  if (!isTRUE(include_planned)) {
    map <- map[map$implementation_status == "implemented", , drop = FALSE]
  }
  rownames(map) <- NULL
  map
}

.hzr_argument_mapping_table <- data.frame(
  sas_statement = c(
    # --- Core hazard() arguments -----------------------------------------------
    "HAZARD", "HAZARD", "HAZARD", "HAZARD", "HAZARD", "HAZARD", "HAZARD",
    "TIME", "EVENT", "PARMS", "DIST",
    # --- Multiphase (phases / hzr_phase) arguments -----------------------------
    "HAZARD", "HAZARD",
    "G1", "G1", "G1", "G1",
    "G2",
    "G3", "G3", "G3", "G3"
  ),
  legacy_input = c(
    "TIME variable", "EVENT/censor variable", "X covariate block",
    "initial parameters", "baseline distribution", "control options",
    "additional legacy options", "t", "status", "theta0", "dist",
    # Multiphase
    "phases (3-phase structure)", "MU_1, MU_2, MU_3",
    "THALF / RHO (early)", "NU (early)", "M (early)", "DELTA (early)",
    "G2 constant phase",
    "TAU (late)", "GAMMA (late)", "ALPHA (late)", "ETA (late)"
  ),
  c_concept = c(
    "obs time array", "event indicator array", "design matrix",
    "parameter vector", "phase distribution selector", "optimizer/control struct",
    "misc legacy switches", "time vector", "event vector", "starting coef", "dist selector",
    # Multiphase
    "3-phase Early/Const/Late", "per-phase scale factors",
    "early half-life", "early time exponent", "early shape", "early time transform",
    "constant hazard rate phase",
    "late G3 scale", "late G3 time exponent", "late G3 shape", "late G3 outer exponent"
  ),
  r_parameter = c(
    "time", "status", "x", "theta", "dist", "control", "...",
    "time", "status", "theta", "dist",
    # Multiphase
    "phases (list of hzr_phase())", "mu (via exp(log_mu) in theta)",
    "hzr_phase(t_half=)", "hzr_phase(nu=)", "hzr_phase(m=)", "(absorbed by decompos)",
    "hzr_phase('constant')",
    "hzr_phase('g3', tau=)", "hzr_phase('g3', gamma=)", "hzr_phase('g3', alpha=)", "hzr_phase('g3', eta=)"
  ),
  required = c(
    TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE, FALSE, FALSE,
    # Multiphase
    FALSE, FALSE,
    FALSE, FALSE, FALSE, FALSE,
    FALSE,
    FALSE, FALSE, FALSE, FALSE
  ),
  expected_type = c(
    "numeric vector", "numeric/logical vector", "numeric matrix or data.frame",
    "numeric vector", "character scalar", "named list", "named arguments",
    "numeric vector", "numeric/logical vector", "numeric vector", "character scalar",
    # Multiphase
    "list of hzr_phase objects", "numeric (per-phase)",
    "positive scalar", "numeric scalar", "numeric scalar", "numeric scalar",
    "hzr_phase('constant')",
    "positive scalar", "numeric scalar", "numeric scalar", "numeric scalar"
  ),
  transform_rule = c(
    "pass through as numeric", "coerce to numeric 0/1", "data.frame -> data.matrix",
    "length must equal ncol(x) when x is present", "normalized lower-case label",
    "stored in spec$control", "stored in legacy_args for parity",
    "pass through", "coerce to numeric", "map PARMS/INITIAL to theta", "map DIST= to dist",
    # Multiphase
    "list(early=hzr_phase('cdf',...), constant=hzr_phase('constant'), late=hzr_phase('g3',...))",
    "exp(alpha_j) in internal parameterization; estimated on log scale",
    "maps directly to hzr_phase(t_half=) starting value",
    "maps directly to hzr_phase(nu=) starting value",
    "maps directly to hzr_phase(m=) starting value",
    "time transform B(t) = (exp(delta*t)-1)/delta absorbed into decompos shape",
    "hzr_phase('constant') with no shape parameters",
    "maps directly to hzr_phase('g3', tau=) for late phase",
    "maps directly to hzr_phase('g3', gamma=) for late phase",
    "maps directly to hzr_phase('g3', alpha=) for late phase",
    "maps directly to hzr_phase('g3', eta=) for late phase"
  ),
  implementation_status = c(
    "implemented", "implemented", "implemented", "implemented", "implemented",
    "implemented", "implemented", "implemented", "implemented", "planned", "implemented",
    # Multiphase
    "implemented", "implemented",
    "implemented", "implemented", "implemented", "implemented",
    "implemented",
    "implemented", "implemented", "implemented", "implemented"
  ),
  notes = c(
    "Core observation time input.",
    "Event indicator currently retained as numeric in object$data$status.",
    "Future versions will support richer design encoding helpers.",
    "Used by predict.hazard as coefficient vector.",
    "Current default is 'weibull'; more options planned.",
    "Control list is stored and reserved for optimizer parity.",
    "Supports legacy-style pass-through options during migration.",
    "Canonical SAS migration uses TIME= mapping.",
    "Canonical SAS migration uses EVENT= mapping.",
    "SAS PARMS syntax parser not yet implemented.",
    "SAS DIST keyword maps directly to dist.",
    # Multiphase
    "Use dist='multiphase' with phases argument. N-phase generalization of legacy 3-phase model.",
    "Each phase has its own scale mu_j(x) = exp(alpha_j + x*beta_j). Starting value via hzr_phase().",
    "Half-life: time at which G(t_half) = 0.5. Same concept as SAS RHO/THALF.",
    "Time exponent controlling rate dynamics. Same parameter name as SAS early NU.",
    "Shape exponent controlling distributional form. Same parameter name as SAS early M.",
    "The C DELTA controlled B(t) = (exp(delta*t)-1)/delta. This transform is absorbed by decompos().",
    "Flat background rate. No shape parameters estimated. SAS G2 equivalent.",
    "Late-phase G3 scale parameter. Maps directly to hzr_phase('g3', tau=).",
    "Late-phase G3 time exponent. Maps directly to hzr_phase('g3', gamma=).",
    "Late-phase G3 shape parameter. alpha=0 gives exponential case. Maps directly to hzr_phase('g3', alpha=).",
    "Late-phase G3 outer exponent. Maps directly to hzr_phase('g3', eta=).")
  ,
  stringsAsFactors = FALSE
)

#' @importFrom stats optim
#' @keywords internal
NULL

# likelihood-loglogistic.R -- Log-logistic parametric hazard likelihood, gradient, and optimizer
#
# MODEL
# -----
# Log-logistic proportional-odds model (commonly used in actuarial work):
#
#   h(t | x)  = alpha beta t^(beta-1) exp(eta) / (1 + alpha t^beta exp(eta))   hazard
#   H(t | x)  = log(1 + alpha t^beta exp(eta))               cumulative hazard
#   S(t | x)  = 1 / (1 + alpha t^beta exp(eta))              survival
#
# where alpha > 0 (scale), beta > 0 (shape), eta = x beta (linear predictor).
#
# When beta = 1, the hazard function is monotone decreasing.  When beta > 1 it
# has a single interior maximum (unimodal), which makes the log-logistic
# useful for modelling failure rates that rise then fall over time.
#
# THETA LAYOUT
# ------------
#   theta[1]   = log(alpha)   (unconstrained; alpha recovered via exp())
#   theta[2]   = log(beta)   (unconstrained; beta recovered via exp())
#   theta[3:p] = beta_coef   (covariate coefficients; unrestricted)
#
# KEY GRADIENT NOTE
# -----------------
# Let pw_i = term_i / (1 + term_i) where term_i = alpha t_i^beta exp(eta_i)
# and let w_i = (1 + delta_i) * pw_i  (events get weight 2, censored get 1).
#
#   dL/d(log alpha) = sum(delta) - sum(w)
#   dL/d(log beta) = sum(delta) + beta * [sum(delta * log t) - sum(w * log t)]
#   dL/dbeta_j     = t(X) %*% (delta - w)
#
# The (1+delta) weighting arises because events contribute both log h(t) and
# log S(t), each of which depends on log(1 + term).  Censored observations
# contribute only log S(t).
#
# FUNCTIONS
# ---------
#   .hzr_logl_loglogistic()     -- log-likelihood (optionally returning gradient)
#   .hzr_gradient_loglogistic() -- analytical score vector
#   .hzr_optim_loglogistic()    -- unconstrained BFGS wrapper

# Log-Logistic Parametric Hazard Likelihood
#
# Evaluate the log-likelihood and its derivatives for log-logistic hazard models.
# The log-logistic distribution is more flexible than exponential with separate
# scale (alpha) and shape (beta) parameters.

#' Log-likelihood for log-logistic hazard with covariates
#'
#' Computes the negative log-likelihood for right-censored data under the log-logistic
#' parametric hazard model with optional linear-predictor covariates.
#'
#' @param theta Vector of parameters:
#'   theta\[1\] = log(alpha) where alpha > 0 is the scale parameter
#'   theta\[2\] = log(beta) where beta > 0 is the shape parameter
#'   theta\[3:length\]: Covariate coefficients (linear on log-scale of cumulative hazard)
#'
#' @param time Numeric vector of follow-up times (n)
#' @param status Numeric vector of event indicators: 1 = event, 0 = censored (n)
#' @param time_lower Optional numeric lower bound vector for interval-censored rows.
#'   Defaults to time if NULL.
#' @param time_upper Optional numeric upper bound vector for left/interval-censored rows.
#'   Defaults to time if NULL.
#' @param x Design matrix of covariates (n x p_coef); NULL for no covariates
#' @param return_gradient Logical; if TRUE, attach gradient vector as attribute
#'
#' @return Scalar log-likelihood value. If return_gradient = TRUE, gradient vector
#' is attached as attribute \code{"gradient"}.
#'
#' @details
#' The log-logistic hazard model is parameterized as:
#'
#' \deqn{h(t | x) = \frac{\alpha \beta t^{\beta - 1}}{1 + \alpha t^{\beta} \exp(\eta)}}
#'
#' where:
#' - \eqn{\alpha > 0} is scale parameter
#' - \eqn{\beta > 0} is shape parameter
#' - \eqn{\eta = x \beta} is covariate effect (linear on log-scale)
#'
#' The survival function is:
#'
#' \deqn{S(t | x) = \frac{1}{1 + \alpha t^{\beta} \exp(\eta)}}
#'
#' The cumulative hazard is:
#'
#' \deqn{H(t | x) = \log(1 + \alpha t^{\beta} \exp(\eta))}
#'
#' The log-likelihood for right-censored data is:
#'
#' \deqn{\ell(\theta) = \sum_{\delta_i = 1} [\log(\alpha \beta) + (\beta - 1)
#'   \log(t_i) - \log(1 + \alpha t_i^{\beta} \exp(\eta_i))] + \sum_{i}
#'   \log(1 + \alpha t_i^{\beta} \exp(\eta_i))}
#'
#' Reparameterization: \u03b8\[1\] = log(\u03b1), \u03b8\[2\] = log(\u03b2) avoids constrained optimization.
#'
#' Mixed censoring status coding:
#' - 1: exact event at time
#' - 0: right-censored at time
#' - -1: left-censored with upper bound time_upper \(or time\)
#' - 2: interval-censored in the interval \(time_lower, time_upper\)
#'
#' @noRd
.hzr_logl_loglogistic <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    return_gradient = FALSE) {

  n <- length(time)

  # Default unit weights
  if (is.null(weights)) weights <- rep(1, n)

  # Extract parameters
  log_alpha <- theta[1]
  log_beta <- theta[2]
  alpha <- exp(log_alpha)  # Always positive
  beta <- exp(log_beta)    # Always positive

  # Covariate coefficients (if any)
  if (!is.null(x)) {
    if (is.null(attr(x, "dimnames")[[2]])) {
      colnames(x) <- paste0("beta_", seq_len(ncol(x)))
    }
    beta_coef <- theta[3:length(theta)]
    eta <- as.numeric(x %*% beta_coef)  # n-vector
  } else {
    eta <- rep(0, n)
  }

  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  if (any(status == 1 & time <= 0)) {
    return(Inf)
  }
  if (any(status %in% c(-1, 2) & upper <= 0)) {
    return(Inf)
  }
  if (any(status == 2) && any(lower[status == 2] >= upper[status == 2])) {
    return(Inf)
  }

  # Build term values for event/lower/upper times. Using log-scale first keeps
  # overflow checks centralized before exponentiation.
  log_term <- log_alpha + beta * log(time) + eta
  log_term_l <- log_alpha + beta * log(lower) + eta
  log_term_u <- log_alpha + beta * log(upper) + eta

  # Check for numerical overflow
  max_log_term <- max(c(log_term, log_term_l, log_term_u), na.rm = TRUE)
  if (max_log_term > 100) {
    return(Inf)
  }

  term_event <- exp(log_term)
  term_lower <- exp(log_term_l)
  term_upper <- exp(log_term_u)

  idx_event <- status == 1
  idx_right <- status == 0
  idx_left <- status == -1
  idx_interval <- status == 2

  # Exact event: w * log f = w * [log h + log S]
  # f(t|x) = alpha beta t^(beta-1) exp(eta) / (1 + alpha t^beta exp(eta))^2
  # log f  = log alpha + log beta + (beta-1) log t + eta - 2 log(1 + term)
  ll_event <- if (any(idx_event)) {
    sum(
      weights[idx_event] *
        (log_alpha + log_beta + (beta - 1) * log(time[idx_event]) +
           eta[idx_event] - 2 * log1p(term_event[idx_event]))
    )
  } else {
    0
  }

  # Right-censored: w * log S = -w * log(1 + term)
  ll_right <- if (any(idx_right)) {
    -sum(weights[idx_right] * log1p(term_event[idx_right]))
  } else {
    0
  }

  # Left-censored: w * log F = w * log(term/(1+term))
  ll_left <- if (any(idx_left)) {
    sum(weights[idx_left] *
          (log_term_u[idx_left] - log1p(term_upper[idx_left])))
  } else {
    0
  }

  # Interval-censored: w * log(F(u) - F(l)).
  # We compute on probability scale and then log, with positivity guard.
  ll_interval <- if (any(idx_interval)) {
    p_u <- term_upper[idx_interval] / (1 + term_upper[idx_interval])
    p_l <- term_lower[idx_interval] / (1 + term_lower[idx_interval])
    diff_p <- p_u - p_l
    if (any(diff_p <= 0)) return(Inf)
    sum(weights[idx_interval] * log(diff_p))
  } else {
    0
  }

  logl <- ll_event + ll_right + ll_left + ll_interval

  if (!is.finite(logl)) {
    return(Inf)
  }

  # If gradient requested, compute score vector
  if (return_gradient) {
    grad <- .hzr_gradient_loglogistic(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      eta = eta, alpha = alpha, beta = beta,
      log_alpha = log_alpha, log_beta = log_beta, term = term_event
    )
    attr(logl, "gradient") <- grad
  }

  logl
}

#' Score vector (gradient of log-likelihood) for log-logistic
#'
#' Computes the score vector of the log-logistic log-likelihood w.r.t. all parameters.
#'
#' The log-likelihood is:
#'   \eqn{L = \sum(\delta_i * [\log(\alpha) + \log(\beta) + (\beta-1)
#'   *\log(t_i)]) - \sum(\log(1 + \alpha*t_i^\beta*\exp(\eta_i)))}
#'
#' Let p_i = alpha*t_i^beta*exp(eta_i) / (1 + alpha*t_i^beta*exp(eta_i)) = probability of event at t_i
#'
#' Derivatives (using chain rule and reparameterization):
#' dL/d(log alpha) = sum(delta_i) - alpha * sum(t_i^beta * exp(eta_i) / (1 + alpha*t_i^beta*exp(eta_i)))
#'             = sum(delta_i) - sum(alpha * t_i^beta * exp(eta_i) / (1 + term))
#'
#' dL/d(log beta) = sum(delta_i) + sum(delta_i * log(t_i))
#'                  - beta * sum(alpha*t_i^beta*log(t_i)*exp(eta_i)
#'                                 /(1 + alpha*t_i^beta*exp(eta_i)))
#'                = sum(delta_i) + sum(delta_i * log(t_i))
#'                  - beta * sum(t_i^beta*log(t_i)
#'                                 /(1 + 1/(alpha*t_i^beta*exp(eta_i))))
#'
#' dL/dbeta_j = sum(delta_i * x_ij) - sum(alpha*t_i^beta*exp(eta_i)*x_ij / (1 + alpha*t_i^beta*exp(eta_i)))
#'         = t(X) %*% (delta - p)
#'
#' @noRd
.hzr_gradient_loglogistic <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    eta = NULL,
    alpha = NULL,
    beta = NULL,
    log_alpha = NULL,
    log_beta = NULL,
    term = NULL) {

  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)

  # Default unit weights -- keep gradient consistent with the LL when weights
  # are omitted.
  if (is.null(weights)) weights <- rep(1, n)

  # Mixed censoring uses numerical gradient for robustness.
  if (any(status %in% c(-1, 2))) {
    return(.hzr_numeric_grad_loglogistic(theta, time, status, time_lower,
                                          time_upper, x, weights = weights))
  }

  # Recompute if not provided
  if (is.null(term) || is.null(alpha) || is.null(beta)) {
    log_alpha <- theta[1]
    log_beta <- theta[2]
    alpha <- exp(log_alpha)
    beta <- exp(log_beta)

    if (!is.null(x)) {
      beta_coef <- theta[3:length(theta)]
      eta <- as.numeric(x %*% beta_coef)
    } else {
      eta <- rep(0, n)
    }

    term <- alpha * (time ^ beta) * exp(eta)
  }

  # pw_i = term_i / (1 + term_i), wm_i = weights_i * (1 + delta_i) * pw_i
  # (wm is the weighted Mills building block used in every derivative term.)
  pw <- term / (1 + term)
  w_status <- weights * status
  wm <- weights * (1 + status) * pw

  # dL/d(log alpha) = sum(w * delta) - sum(wm)
  grad[1] <- sum(w_status) - sum(wm)

  # dL/d(log beta) = sum(w * delta) + beta * [sum(w * delta * log t) - sum(wm * log t)]
  log_t <- log(time)
  grad[2] <- sum(w_status) +
             beta * (sum(w_status * log_t) - sum(wm * log_t))

  # dL/dbeta_j = t(X) %*% (w * delta - wm)
  if (p > 2 && !is.null(x)) {
    residual <- w_status - wm
    grad[3:p] <- as.numeric(crossprod(x, residual))
  }

  grad
}

#' @noRd
.hzr_optim_loglogistic <- function(
    time, status, time_lower = NULL, time_upper = NULL,
    x = NULL, theta_start, weights = NULL, control = list()) {
  .hzr_optim_generic(
    logl_fn = .hzr_logl_loglogistic,
    gradient_fn = .hzr_gradient_loglogistic,
    time = time, status = status,
    time_lower = time_lower, time_upper = time_upper,
    x = x, theta_start = theta_start, weights = weights,
    control = control, use_bounds = FALSE
  )
}

.hzr_numeric_grad_loglogistic <- function(theta, time, status,
                                            time_lower = NULL, time_upper = NULL,
                                            x = NULL, weights = NULL) {
  # Reuse log-likelihood as scalar objective and differentiate numerically.
  objective <- function(par) {
    .hzr_logl_loglogistic(
      theta = par,
      time = time,
      status = status,
      time_lower = time_lower,
      time_upper = time_upper,
      x = x,
      weights = weights,
      return_gradient = FALSE
    )
  }

  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(as.numeric(numDeriv::grad(objective, theta)))
  }

  eps <- 1e-6
  grad <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tm <- theta
    tp[i] <- tp[i] + eps
    tm[i] <- tm[i] - eps
    grad[i] <- (objective(tp) - objective(tm)) / (2 * eps)
  }
  grad
}

library(testthat)

test_that("Log-logistic likelihood computation (univariate)", {
  # Simple univariate case
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 1, 0, 1, 0)

  # θ = [log(α), log(β)]
  # log(α) = 0 => α = 1
  # log(β) = 0 => β = 1 (exponential limit)
  log_alpha <- 0
  log_beta <- 0
  theta <- c(log_alpha, log_beta)

  logl <- .hzr_logl_loglogistic(theta, time, status, x = NULL, return_gradient = FALSE)

  # Check that likelihood is finite
  expect_true(is.finite(logl))
  # Log-likelihood should be negative (typical for parametric models)
  expect_true(logl < 0)
})

test_that("Log-logistic likelihood with covariates (multivariate)", {
  set.seed(123)
  n <- 20
  time <- rexp(n, rate = 0.5)
  status <- rbinom(n, size = 1, prob = 0.7)
  x <- cbind(runif(n), rnorm(n))

  log_alpha <- 0.5
  log_beta <- 0.3
  beta_coef <- c(0.2, -0.3)
  theta <- c(log_alpha, log_beta, beta_coef)

  logl <- .hzr_logl_loglogistic(theta, time, status, x = x, return_gradient = FALSE)

  expect_true(is.finite(logl))
  expect_true(logl < 0)
})

test_that("Log-logistic gradient computation against numerical derivative", {
  set.seed(456)
  time <- c(0.5, 1.0, 1.5, 2.0, 2.5)
  status <- c(1, 1, 0, 1, 0)

  log_alpha <- 0.2
  log_beta <- -0.1
  theta <- c(log_alpha, log_beta)

  # Analytical gradient
  logl <- .hzr_logl_loglogistic(theta, time, status, x = NULL, return_gradient = TRUE)
  grad_analytical <- attr(logl, "gradient")

  # Numerical gradient
  eps <- 1e-4
  grad_numerical <- numeric(length(theta))
  for (i in seq_along(theta)) {
    theta_plus <- theta
    theta_plus[i] <- theta_plus[i] + eps
    logl_plus <- .hzr_logl_loglogistic(theta_plus, time, status, x = NULL, return_gradient = FALSE)

    theta_minus <- theta
    theta_minus[i] <- theta_minus[i] - eps
    logl_minus <- .hzr_logl_loglogistic(theta_minus, time, status, x = NULL, return_gradient = FALSE)

    grad_numerical[i] <- (logl_plus - logl_minus) / (2 * eps)
  }

  # Compare (within numerical tolerance)
  expect_equal(grad_analytical, grad_numerical, tolerance = 1e-4)
})

test_that("Log-logistic gradient with covariates", {
  set.seed(789)
  time <- c(0.5, 1.0, 1.5, 2.0, 2.5)
  status <- c(1, 1, 0, 1, 0)
  x <- cbind(c(1, 0, 1, 0, 1), c(0.5, 1.5, -0.5, 2.0, 1.0))
  colnames(x) <- c("x1", "x2")

  theta <- c(log_alpha = 0.3, log_beta = 0.1, beta_1 = 0.2, beta_2 = -0.1)

  # Analytical gradient
  logl <- .hzr_logl_loglogistic(theta, time, status, x = x, return_gradient = TRUE)
  grad_analytical <- attr(logl, "gradient")

  # Numerical gradient
  eps <- 1e-4
  grad_numerical <- numeric(length(theta))
  for (i in seq_along(theta)) {
    theta_plus <- theta
    theta_plus[i] <- theta_plus[i] + eps
    logl_plus <- .hzr_logl_loglogistic(theta_plus, time, status, x = x, return_gradient = FALSE)

    theta_minus <- theta
    theta_minus[i] <- theta_minus[i] - eps
    logl_minus <- .hzr_logl_loglogistic(theta_minus, time, status, x = x, return_gradient = FALSE)

    grad_numerical[i] <- (logl_plus - logl_minus) / (2 * eps)
  }

  expect_equal(grad_analytical, grad_numerical, tolerance = 1e-4)
})

test_that("Log-logistic optimization converges (univariate)", {
  set.seed(111)
  n <- 30
  # Simulate from log-logistic with α=1, β=1.5
  alpha_true <- 1.0
  beta_true <- 1.5
  u <- runif(n)
  time <- (u / (1 - u)) ^ (1 / beta_true) / alpha_true
  # Apply censoring
  status <- rbinom(n, size = 1, prob = 0.8)

  # Starting values
  theta_start <- c(log(0.5), log(1.0))

  # Fit
  fit <- .hzr_optim_loglogistic(time, status, x = NULL, theta_start = theta_start)

  expect_true(fit$convergence == 0, info = paste("Message:", fit$message))
  expect_true(is.finite(fit$value))
  expect_equal(length(fit$par), 2)

  # Check that estimates are in reasonable range
  expect_true(fit$par[1] > log(0.1) && fit$par[1] < log(10))
  expect_true(fit$par[2] > log(0.5) && fit$par[2] < log(5))
})

test_that("Log-logistic optimization converges (multivariate)", {
  set.seed(222)
  n <- 40

  # True parameters
  alpha_true <- 1.0
  beta_true <- 1.2
  beta_coef_true <- c(0.3, -0.2)

  # Generate covariates
  x <- cbind(rnorm(n), rnorm(n))
  eta <- x %*% beta_coef_true

  # Generate baseline times from log-logistic
  u <- runif(n)
  baseline_time <- (u / (1 - u)) ^ (1 / beta_true) / alpha_true

  # Apply covariate effect
  time <- baseline_time * exp(-eta / beta_true)  # Approximate adjustment

  # Apply censoring
  status <- rbinom(n, size = 1, prob = 0.75)

  # Starting values
  theta_start <- c(log(0.8), log(1.0), 0.1, -0.1)

  # Fit
  fit <- .hzr_optim_loglogistic(time, status, x = x, theta_start = theta_start)

  expect_true(fit$convergence == 0, info = paste("Message:", fit$message))
  expect_true(is.finite(fit$value))
  expect_equal(length(fit$par), 4)
})

test_that("Log-logistic survival predictions monotonicity", {
  set.seed(333)
  time <- seq(0.1, 5, by = 0.5)
  theta <- c(log(1.0), log(1.5))  # log-logistic params

  # Predict survival at increasing times
  fit_obj <- list(
    spec = list(dist = "loglogistic"),
    fit = list(theta = theta),
    data = list(time = time, status = rep(1, length(time)), x = NULL)
  )
  class(fit_obj) <- "hazard"

  survival <- predict(fit_obj, newdata = data.frame(time = time), type = "survival")

  # Survival should be decreasing with time
  expect_equal(survival, sort(survival, decreasing = TRUE), tolerance = 1e-10)
})

test_that("Log-logistic cumulative hazard predictions monotonicity", {
  set.seed(444)
  time <- seq(0.1, 5, by = 0.5)
  theta <- c(log(1.0), log(1.5))

  fit_obj <- list(
    spec = list(dist = "loglogistic"),
    fit = list(theta = theta),
    data = list(time = time, status = rep(1, length(time)), x = NULL)
  )
  class(fit_obj) <- "hazard"

  cumhaz <- predict(fit_obj, newdata = data.frame(time = time), type = "cumulative_hazard")

  # Cumulative hazard should be increasing with time
  expect_equal(cumhaz, sort(cumhaz, decreasing = FALSE), tolerance = 1e-10)
})

test_that("Log-logistic survival and cumulative hazard relationship", {
  set.seed(555)
  time <- c(0.5, 1.0, 1.5, 2.0, 2.5, 3.0)
  theta <- c(log(1.2), log(1.8))

  fit_obj <- list(
    spec = list(dist = "loglogistic"),
    fit = list(theta = theta),
    data = list(time = time, status = rep(1, length(time)), x = NULL)
  )
  class(fit_obj) <- "hazard"

  survival <- predict(fit_obj, newdata = data.frame(time = time), type = "survival")
  cumhaz <- predict(fit_obj, newdata = data.frame(time = time), type = "cumulative_hazard")

  # S(t) = exp(-H(t))
  expected_survival <- exp(-cumhaz)
  expect_equal(survival, expected_survival, tolerance = 1e-10)
})

test_that("Log-logistic covariate effect on survival", {
  set.seed(666)
  time <- c(1.0, 2.0, 3.0)

  # With covariates
  x_high <- cbind(c(1, 1, 1))  # Positive covariate effect
  x_low <- cbind(c(-1, -1, -1))  # Negative covariate effect

  theta <- c(log(1.0), log(1.5), 0.5)  # Positive beta coefficient

  fit_obj <- list(
    spec = list(dist = "loglogistic"),
    fit = list(theta = theta),
    data = list(time = time, status = rep(1, length(time)), x = NULL)
  )
  class(fit_obj) <- "hazard"

  s_high <- predict(fit_obj, newdata = data.frame(time = time, V1 = c(1, 1, 1)), type = "survival")
  s_low <- predict(fit_obj, newdata = data.frame(time = time, V1 = c(-1, -1, -1)), type = "survival")

  # Higher covariate value should decrease survival (increase hazard) with positive beta
  expect_true(all(s_high < s_low))
})

test_that("Log-logistic hazard() function fitting (univariate)", {
  set.seed(777)
  n <- 40
  alpha_true <- 0.9
  beta_true <- 1.3
  u <- runif(n)
  time <- (u / (1 - u)) ^ (1 / beta_true) / alpha_true
  status <- rbinom(n, size = 1, prob = 0.8)

  theta_start <- c(log(0.7), log(1.0))

  # Fit via hazard() function
  fit <- hazard(
    time = time,
    status = status,
    theta = theta_start,
    dist = "loglogistic",
    fit = TRUE
  )

  expect_true(fit$fit$converged)
  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$fit$theta), 2)
})

test_that("Log-logistic hazard() function fitting (multivariate)", {
  set.seed(888)
  n <- 50

  # True parameters
  alpha_true <- 1.1
  beta_true <- 1.4
  beta_coef_true <- c(0.25, -0.15)

  x <- cbind(rnorm(n), rnorm(n))
  eta <- x %*% beta_coef_true

  # Generate times from log-logistic
  u <- runif(n)
  baseline_time <- (u / (1 - u)) ^ (1 / beta_true) / alpha_true
  time <- baseline_time * exp(-eta / beta_true)
  status <- rbinom(n, size = 1, prob = 0.75)

  theta_start <- c(log(0.8), log(1.0), 0.1, -0.1)

  fit <- hazard(
    time = time,
    status = status,
    x = x,
    theta = theta_start,
    dist = "loglogistic",
    fit = TRUE
  )

  expect_true(fit$fit$converged)
  expect_equal(length(fit$fit$theta), 4)
})

test_that("Log-logistic predict() linear_predictor", {
  theta <- c(log(1.0), log(1.5), 0.3, -0.2)
  fit_obj <- list(
    spec = list(dist = "loglogistic"),
    fit = list(theta = theta),
    data = list(time = 1:5, status = rep(1, 5), x = cbind(V1 = 1:5, V2 = 5:1))
  )
  class(fit_obj) <- "hazard"

  newdata <- data.frame(V1 = c(1, 0, -1), V2 = c(0, 1, -1))
  pred <- predict(fit_obj, newdata = newdata, type = "linear_predictor")

  # Should match x %*% beta_coef
  expected <- as.numeric(as.matrix(newdata) %*% c(0.3, -0.2))
  expect_equal(pred, expected)
})

test_that("Log-logistic predict() hazard", {
  theta <- c(log(1.0), log(1.5), 0.2, -0.1)
  fit_obj <- list(
    spec = list(dist = "loglogistic"),
    fit = list(theta = theta),
    data = list(time = 1:5, status = rep(1, 5), x = cbind(V1 = 1:5, V2 = 5:1))
  )
  class(fit_obj) <- "hazard"

  newdata <- data.frame(V1 = c(1, 0, 1), V2 = c(0, 1, -1))
  pred <- predict(fit_obj, newdata = newdata, type = "hazard")

  # Hazard should be exp(eta) where eta = x %*% beta
  x_mat <- as.matrix(newdata)
  beta <- theta[3:4]
  eta <- as.numeric(x_mat %*% beta)
  expected <- exp(eta)

  expect_equal(pred, expected)
})

test_that("Log-logistic parameter bounds are reasonable", {
  set.seed(999)
  n <- 35
  time <- rexp(n, rate = 0.4)
  status <- rbinom(n, size = 1, prob = 0.8)

  theta_start <- c(log(0.5), log(0.8))

  fit <- .hzr_optim_loglogistic(time, status, x = NULL, theta_start = theta_start)

  # Parameters should be finite and not absurdly large/small
  expect_true(all(is.finite(fit$par)))
  expect_true(all(fit$par > -10 & fit$par < 10))
})

test_that("Log-logistic error handling: invalid theta length", {
  time <- c(1, 2, 3)
  status <- c(1, 1, 0)

  # Too few parameters for covariates
  theta_bad <- c(log(1.0))  # Missing log(beta)

  expect_error(
    hazard(time = time, status = status, x = NULL, theta = theta_bad, dist = "loglogistic", fit = FALSE),
    NA  # Should allow creation without fitting
  )
})

test_that("Log-logistic mathematical relationship with α=β=1 (exponential limit)", {
  # When α=1 and β=1, log-logistic should approximate exponential
  time <- seq(0.1, 3, by = 0.5)
  theta_ll <- c(log(1.0), log(1.0))  # log-logistic with α=β=1
  theta_exp <- c(log(1.0))  # exponential with λ=1

  # Log-logistic: S(t) = 1 / (1 + t) when α=β=1
  # This is NOT the same as exponential S(t) = exp(-t)
  # So this test just confirms both are well-defined

  fit_ll <- list(
    spec = list(dist = "loglogistic"),
    fit = list(theta = theta_ll),
    data = list(time = time, status = rep(1, length(time)), x = NULL)
  )
  class(fit_ll) <- "hazard"

  s_ll <- predict(fit_ll, newdata = data.frame(time = time), type = "survival")

  # S(t) = 1 / (1 + t)
  expected_s <- 1 / (1 + time)
  expect_equal(s_ll, expected_s, tolerance = 1e-10)
})

test_that("Log-logistic negative log-likelihood at boundary", {
  time <- c(0.1, 0.5, 1.0, 2.0, 5.0)
  status <- c(1, 1, 0, 1, 0)

  # Very small α and β (extreme parameter space)
  theta_small <- c(log(1e-3), log(1e-1))
  logl_small <- .hzr_logl_loglogistic(theta_small, time, status)

  # Very large α and β
  theta_large <- c(log(1e2), log(1e1))
  logl_large <- .hzr_logl_loglogistic(theta_large, time, status)

  # Both should be finite
  expect_true(is.finite(logl_small))
  expect_true(is.finite(logl_large))
})

#' @importFrom stats optim
#' @keywords internal
NULL

# optimizer.R -- Generic parametric hazard optimizer
#
# Consolidates the common optimization pattern shared across all distributions.
# Each distribution provides its own log-likelihood and gradient functions;
# this module handles the objective wrapping, optim() call, and post-fit
# Hessian/vcov computation.

#' Generic optimizer for parametric hazard likelihoods
#'
#' Maximises a log-likelihood by minimising its negation via \code{stats::optim}.
#' Handles constrained (L-BFGS-B) and unconstrained (BFGS) optimization,
#' post-fit Hessian computation, and vcov extraction.
#'
#' @param logl_fn Function(theta, time, status, time_lower, time_upper, x, ...)
#'   returning the scalar log-likelihood.  Must accept \code{return_gradient}
#'   argument (not used here directly, but the function signature must allow it).
#' @param gradient_fn Function(theta, time, status, time_lower, time_upper, x, ...)
#'   returning the score vector (gradient of the log-likelihood, NOT negated).
#' @param time Numeric vector of follow-up times.
#' @param status Numeric vector of event/censoring indicators.
#' @param time_lower Optional lower bounds for interval censoring.
#' @param time_upper Optional upper bounds for left/interval censoring.
#' @param x Optional design matrix.
#' @param theta_start Starting parameter vector.
#' @param control List of control options (maxit, reltol, abstol).
#' @param use_bounds Logical; if TRUE, use L-BFGS-B with lower bounds on
#'   constrained parameters.  Used for Weibull where mu/nu are on the natural
#'   scale.
#' @param lower_bounds Numeric vector of lower bounds for L-BFGS-B, the same
#'   length as \code{theta_start}.  Only used when \code{use_bounds = TRUE}.
#'   Defaults to \code{rep(1e-6, length(theta_start))} when NULL, but callers
#'   should supply distribution-specific bounds (e.g. \code{c(1e-6, 1e-6,
#'   rep(-Inf, p_cov))} for Weibull so that covariate betas are unconstrained).
#'
#' @return List with par, value (log-likelihood), convergence, counts, message,
#'   hessian, vcov.
#' @noRd
.hzr_optim_generic <- function(
    logl_fn,
    gradient_fn,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    theta_start,
    weights = NULL,
    control = list(),
    use_bounds = FALSE,
    lower_bounds = NULL) {

  control <- utils::modifyList(
    list(maxit = 1000, reltol = 1e-5, abstol = 1e-6),
    control
  )

  # Resolve lower bounds: default to 1e-6 on all params only when the caller
  # has not supplied explicit bounds.  Callers like .hzr_optim_weibull pass
  # c(1e-6, 1e-6, -Inf, ...) so that shape params are constrained but betas
  # are free.
  if (use_bounds && is.null(lower_bounds)) {
    lower_bounds <- rep(1e-6, length(theta_start))
  }

  # Negative log-likelihood for minimization
  objective <- function(theta) {
    if (!all(is.finite(theta))) return(1e10)
    # Only penalise infeasible shape params (those with finite lower bounds > -Inf).
    if (use_bounds && any(theta[is.finite(lower_bounds)] <= lower_bounds[is.finite(lower_bounds)])) return(1e10)

    ll <- -logl_fn(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights, return_gradient = FALSE
    )
    if (!is.finite(ll)) return(1e10)
    ll
  }

  # Negative score vector for minimization
  gradient <- function(theta) {
    if (!all(is.finite(theta))) return(rep(0, length(theta)))
    if (use_bounds && any(
      theta[is.finite(lower_bounds)] <= lower_bounds[is.finite(lower_bounds)]
    )) {
      return(rep(0, length(theta)))
    }

    grad <- tryCatch(
      gradient_fn(
        theta = theta, time = time, status = status,
        time_lower = time_lower, time_upper = time_upper,
        x = x, weights = weights
      ),
      error = function(e) rep(0, length(theta))
    )

    grad[!is.finite(grad)] <- 0
    -grad
  }

  # Dispatch to constrained or unconstrained optimizer
  if (use_bounds) {
    result <- optim(
      par = theta_start, fn = objective, gr = gradient,
      method = "L-BFGS-B",
      lower = lower_bounds,
      upper = rep(Inf, length(theta_start)),
      control = list(
        maxit = control$maxit,
        factr = 1 / control$reltol,
        pgtol = control$abstol
      ),
      hessian = FALSE
    )
  } else {
    result <- optim(
      par = theta_start, fn = objective, gr = gradient,
      method = "BFGS",
      control = list(maxit = control$maxit, reltol = control$reltol),
      hessian = FALSE
    )
  }

  # Post-fit Hessian for standard errors
  hess_result <- tryCatch(
    {
      if (requireNamespace("numDeriv", quietly = TRUE)) {
        numDeriv::hessian(objective, result$par)
      } else {
        NA
      }
    },
    error = function(e) NA
  )

  vcov <- if (!anyNA(hess_result)) {
    tryCatch(
      solve(hess_result),
      error = function(e) {
        warning("Hessian not invertible; standard errors unavailable")
        NA
      }
    )
  } else {
    NA
  }

  list(
    par = result$par,
    value = -result$value,
    convergence = result$convergence,
    counts = result$counts,
    message = result$message,
    hessian = hess_result,
    vcov = vcov
  )
}

test_that("well-conditioned PD Hessian inverts cleanly with diagnostics", {
  H <- matrix(c(4, 1, 1, 3), 2, 2)
  res <- expect_silent(.hzr_safe_solve(H))
  expect_equal(res$vcov, solve(H), tolerance = 1e-10)
  expect_true(res$pd)
  expect_true(is.finite(res$rcond) && res$rcond > 1e-3)
})

test_that("asymmetric input is symmetrized before inversion", {
  H_sym  <- matrix(c(4, 1, 1, 3), 2, 2)
  H_asym <- matrix(c(4, 0, 2, 3), 2, 2)  # (H+t(H))/2 == H_sym
  expect_equal(.hzr_safe_solve(H_asym)$vcov, solve(H_sym), tolerance = 1e-10)
})

test_that("ill-conditioned Hessian warns by name but still returns numbers", {
  H <- matrix(c(1, 1, 1, 1 + 1e-12), 2, 2)
  expect_warning(res <- .hzr_safe_solve(H), "ill-conditioned")
  expect_true(is.matrix(res$vcov))
  expect_true(is.finite(res$rcond))
})

test_that("non-PD Hessian falls back to solve and reports pd = FALSE", {
  # Indefinite (eigenvalues +/-): chol() fails, solve() succeeds.
  H <- matrix(c(1, 2, 2, 1), 2, 2)
  res <- suppressWarnings(.hzr_safe_solve(H))
  expect_false(res$pd)
  expect_true(is.matrix(res$vcov))
})

test_that("non-positive variance diagonals are NA'd with a warning", {
  # Indefinite matrix whose inverse has a negative diagonal entry. A non-PD
  # Hessian emits both the positive-definiteness and the non-positive-variance
  # warning, so capture all warnings rather than matching a single one.
  H <- matrix(c(1, 2, 2, 1), 2, 2)
  w <- testthat::capture_warnings(res <- .hzr_safe_solve(H))
  expect_true(any(grepl("[Nn]on-positive variance", w)))
  expect_true(any(is.na(diag(res$vcov))))
})

test_that("non-PD Hessian with positive inverse-variances warns about positive-definiteness", {
  # Indefinite (eigenvalues +1, -3) but the inverse diagonal is +1/3 on both
  # entries, so the non-positive-variance guard does NOT fire -- this isolates
  # the positive-definiteness warning on its own.
  H <- matrix(c(-1, 2, 2, -1), 2, 2)
  expect_warning(res <- .hzr_safe_solve(H), "positive-definite")
  expect_false(res$pd)
  expect_true(all(diag(res$vcov) > 0))
})

test_that("non-finite Hessian returns NA vcov with a warning", {
  H <- matrix(c(1, NA, NA, 1), 2, 2)
  expect_warning(res <- .hzr_safe_solve(H), "non-finite")
  expect_true(all(is.na(res$vcov)))
  expect_true(is.na(res$pd))
})

test_that("scalar / non-matrix input is handled as non-finite", {
  expect_warning(res <- .hzr_safe_solve(NA), "non-finite")
  expect_true(all(is.na(res$vcov)))
})

test_that("explicit tol drives the ill-conditioned threshold", {
  H <- matrix(c(4, 1, 1, 3), 2, 2)        # well-conditioned (rcond ~ 0.4)
  expect_silent(.hzr_safe_solve(H, tol = 1e-8))      # below rcond: no warning
  expect_warning(.hzr_safe_solve(H, tol = 0.5), "ill-conditioned")  # above rcond: warns
})

test_that(".hzr_optim_generic uses a supplied analytic hessian_fn", {
  set.seed(1)
  time <- rexp(100, rate = 0.5)
  status <- rep(1L, 100)
  marker <- matrix(42, 1, 1)
  res <- .hzr_optim_generic(
    logl_fn = .hzr_logl_exponential, gradient_fn = .hzr_gradient_exponential,
    time = time, status = status, x = NULL, theta_start = c(log_rate = 0),
    control = list(maxit = 200, reltol = 1e-8, abstol = 1e-8),
    hessian_fn = function(par) marker
  )
  expect_equal(unname(res$hessian), unname(marker))
  expect_equal(unname(res$vcov), solve(marker), tolerance = 1e-10)
})

test_that(".hzr_optim_generic falls back to numDeriv when hessian_fn returns NULL", {
  skip_if_not_installed("numDeriv")
  set.seed(1)
  time <- rexp(100, rate = 0.5)
  status <- rep(1L, 100)
  res <- .hzr_optim_generic(
    logl_fn = .hzr_logl_exponential, gradient_fn = .hzr_gradient_exponential,
    time = time, status = status, x = NULL, theta_start = c(log_rate = 0),
    control = list(maxit = 200, reltol = 1e-8, abstol = 1e-8),
    hessian_fn = function(par) NULL
  )
  expect_true(is.matrix(res$hessian) && is.finite(res$hessian[1, 1]))
  expect_true(isTRUE(res$pd))
})

test_that(".hzr_optim_generic warns and falls back on a non-conformant hessian_fn", {
  skip_if_not_installed("numDeriv")
  set.seed(1)
  time <- rexp(100, rate = 0.5)
  status <- rep(1L, 100)
  # Wrong dimension (2x2 for a 1-parameter fit) -> warn + numerical fallback.
  expect_warning(
    res <- .hzr_optim_generic(
      logl_fn = .hzr_logl_exponential, gradient_fn = .hzr_gradient_exponential,
      time = time, status = status, x = NULL, theta_start = c(log_rate = 0),
      control = list(maxit = 200, reltol = 1e-8, abstol = 1e-8),
      hessian_fn = function(par) matrix(1, 2, 2)
    ),
    "non-conformant"
  )
  expect_true(is.matrix(res$hessian) && all(dim(res$hessian) == 1L))
  expect_true(isTRUE(res$pd))
})

test_that(".hzr_optim_generic returns rcond and pd diagnostics", {
  skip_if_not_installed("numDeriv")
  set.seed(1)
  n <- 200L
  time <- rexp(n, rate = 0.5)
  status <- rep(1L, n)
  res <- .hzr_optim_generic(
    logl_fn = .hzr_logl_exponential,
    gradient_fn = .hzr_gradient_exponential,
    time = time, status = status,
    x = NULL, theta_start = c(log_rate = 0),
    control = list(maxit = 200, reltol = 1e-8, abstol = 1e-8),
    use_bounds = FALSE
  )
  expect_true(is.finite(res$rcond))
  expect_true(isTRUE(res$pd))
  expect_true(is.matrix(res$vcov) && all(is.finite(diag(res$vcov))))
})

test_that("singular Hessian (chol and solve both fail) warns 'not invertible'", {
  # Rank-deficient PSD matrix: chol() fails (not PD) and solve() fails
  # (singular), so the helper takes the not-invertible path.
  H <- matrix(c(1, 1, 1, 1), 2, 2)
  w <- testthat::capture_warnings(res <- .hzr_safe_solve(H))
  expect_true(any(grepl("not invertible", w)))
  expect_true(all(is.na(res$vcov)))
  expect_true(is.na(res$pd))
})

# stepwise-step.R -- Per-step drivers for hzr_stepwise()
#
# Step 8.4-8.5 of STEPWISE-DESIGN.md.  One forward step or one backward
# step, driven by the unified `.hzr_candidate_score()` wrapper.  Higher
# level control (two-way loop, MOVE cap, force_in, max_steps) lives in
# the hzr_stepwise() driver to be built in sec.8.6.

# ---------------------------------------------------------------------------
# Candidate enumeration
# ---------------------------------------------------------------------------

#' Normalise a user-supplied scope into a flat list of (var, phase) pairs
#'
#' Drops variables already in the model, variables in `force_out`, and
#' candidates whose phase is not present in a multiphase fit.
#'
#' @param fit The current fit.
#' @param scope One of: NULL (data-driven; infer from `data`), a
#'   one-sided formula (single-dist), or a named list of one-sided
#'   formulas keyed by phase (multiphase).
#' @param data Data frame to derive candidates from when `scope = NULL`.
#' @param force_out Character vector of variable names to exclude.
#'
#' @return A list of `list(var = <chr>, phase = <chr or NULL>)`.
#'
#' @keywords internal
#' @noRd
.hzr_stepwise_candidates <- function(fit, scope = NULL, data,
                                      force_out = character()) {
  dist <- fit$spec$dist

  if (dist == "multiphase") {
    phase_names <- names(fit$spec$phases)
    current_per_phase <- .hzr_scope_current_vars(fit)

    if (is.null(scope)) {
      # Default: all data-frame vars (excluding Surv components and
      # force_out) are candidates for every phase.
      lhs_vars <- all.vars(stats::as.formula(
        deparse(fit$call$formula)
      )[[2L]])
      data_vars <- setdiff(colnames(data), c(lhs_vars, force_out))
      scope <- setNames(
        lapply(phase_names, function(p) {
          rhs_syms <- data_vars
          # Return as a one-sided formula for symmetry with the
          # user-supplied case.
          if (length(rhs_syms) == 0L) return(NULL)
          stats::as.formula(paste("~", paste(rhs_syms, collapse = " + ")))
        }),
        phase_names
      )
    } else {
      if (!is.list(scope) || is.null(names(scope))) {
        stop("`scope` for multiphase fits must be a named list keyed by phase.",
             call. = FALSE)
      }
      unknown <- setdiff(names(scope), phase_names)
      if (length(unknown) > 0L) {
        stop("Unknown phase(s) in scope: ",
             paste(sQuote(unknown), collapse = ", "),
             call. = FALSE)
      }
    }

    candidates <- list()
    for (p in names(scope)) {
      sc <- scope[[p]]
      if (is.null(sc)) next
      terms_p <- .hzr_formula_rhs_terms(sc)
      eligible <- setdiff(terms_p, c(current_per_phase[[p]], force_out))
      for (v in eligible) {
        candidates[[length(candidates) + 1L]] <-
          list(var = v, phase = p)
      }
    }
    return(candidates)
  }

  # Single-distribution
  if (is.null(scope)) {
    f <- fit$call$formula
    if (!is.null(f) && !inherits(f, "formula")) {
      f <- stats::as.formula(deparse(f))
    }
    lhs_vars <- if (is.null(f)) character() else all.vars(f[[2L]])
    data_vars <- setdiff(colnames(data), c(lhs_vars, force_out))
  } else {
    if (inherits(scope, "formula")) {
      data_vars <- .hzr_formula_rhs_terms(scope)
    } else if (is.character(scope)) {
      data_vars <- scope
    } else {
      stop("`scope` must be NULL, a one-sided formula, or a character vector.",
           call. = FALSE)
    }
    data_vars <- setdiff(data_vars, force_out)
  }

  current_vars <- .hzr_scope_current_vars(fit)
  eligible <- setdiff(data_vars, current_vars)
  lapply(eligible, function(v) list(var = v, phase = NULL))
}


# ---------------------------------------------------------------------------
# Forward step
# ---------------------------------------------------------------------------

#' Execute one forward step of stepwise selection
#'
#' Refits the current model with each candidate (variable, phase)
#' appended, scores each via `.hzr_candidate_score(mode = "entry")`,
#' and accepts the best if its score clears the entry threshold.
#'
#' Divergent candidate refits emit a `warning()` naming the failing
#' `(variable, phase)` pair and are excluded from the selection, per
#' the sec.2 Q5 decision in STEPWISE-DESIGN.md.
#'
#' @param current Fitted `hazard` object that is the starting point.
#' @param scope Scope specification (see `.hzr_stepwise_candidates`).
#' @param data Data frame for refits.
#' @param criterion Either `"wald"` or `"aic"`.
#' @param slentry Entry threshold for the Wald criterion (ignored when
#'   `criterion = "aic"`; the entry rule there is dAIC < 0).
#' @param force_out Character vector of variables that may never be
#'   considered as candidates.
#' @param ... Forwarded to `.hzr_refit_with_scope()` (and thence to
#'   `hazard()`), e.g. `control = list(...)`.
#'
#' @return A list with:
#' \describe{
#'   \item{accepted}{Logical; did any candidate enter this step?}
#'   \item{fit}{If accepted, the new fit.  Otherwise `current` echoed.}
#'   \item{variable}{Variable that entered, or `NA_character_`.}
#'   \item{phase}{Phase entered, or `NA_character_` (single-dist).}
#'   \item{score}{Winning score (p or dAIC), or `NA_real_`.}
#'   \item{p_value}{Winning p-value.}
#'   \item{delta_aic}{Winning dAIC.}
#'   \item{stat, df}{Wald statistic / df of the winner.}
#'   \item{all_scores}{Tibble-like data frame of every candidate
#'     considered and its score.}
#'   \item{refit_failures}{Character vector of `"var@phase"` tokens for
#'     candidates whose refit diverged.}
#' }
#'
#' @keywords internal
#' @noRd
.hzr_stepwise_forward_step <- function(current, scope = NULL, data,
                                        criterion = c("wald", "aic"),
                                        slentry   = 0.30,
                                        force_out = character(),
                                        ...) {
  criterion <- match.arg(criterion)
  if (!inherits(current, "hazard")) {
    stop("`current` must be a fitted `hazard` object.", call. = FALSE)
  }

  cands <- .hzr_stepwise_candidates(current, scope = scope, data = data,
                                     force_out = force_out)

  null_result <- function() {
    list(
      accepted  = FALSE,
      fit       = current,
      variable  = NA_character_,
      phase     = NA_character_,
      score     = NA_real_,
      p_value   = NA_real_,
      delta_aic = NA_real_,
      stat      = NA_real_,
      df        = NA_integer_,
      all_scores = data.frame(
        variable  = character(),
        phase     = character(),
        score     = numeric(),
        p_value   = numeric(),
        delta_aic = numeric(),
        stat      = numeric(),
        df        = integer(),
        stringsAsFactors = FALSE
      ),
      refit_failures = character()
    )
  }

  if (length(cands) == 0L) {
    return(null_result())
  }

  rows     <- vector("list", length(cands))
  failures <- character()

  for (i in seq_along(cands)) {
    cand <- cands[[i]]
    candidate_fit <- tryCatch(
      .hzr_refit_with_scope(
        current, action = "add",
        var = cand$var, phase = cand$phase,
        data = data, ...
      ),
      error = function(e) e
    )

    failure_token <- if (is.null(cand$phase)) {
      cand$var
    } else {
      paste0(cand$var, "@", cand$phase)
    }

    if (inherits(candidate_fit, "error") ||
          isFALSE(candidate_fit$fit$converged)) {
      warning("Stepwise forward: candidate refit failed for ",
              failure_token, ".", call. = FALSE)
      failures <- c(failures, failure_token)
      rows[[i]] <- data.frame(
        variable  = cand$var,
        phase     = cand$phase %||% NA_character_,
        score     = NA_real_,
        p_value   = NA_real_,
        delta_aic = NA_real_,
        stat      = NA_real_,
        df        = NA_integer_,
        stringsAsFactors = FALSE
      )
      next
    }

    # Coefficient name(s) of the newly-entered variable in the candidate
    # fit.  For single-dist this is `var` directly; for multiphase the
    # coef is phase-prefixed.
    coef_name <- .hzr_candidate_coef_name(candidate_fit, cand$var,
                                           cand$phase)

    s <- .hzr_candidate_score(
      criterion = criterion, mode = "entry",
      current = current, candidate = candidate_fit,
      names = coef_name
    )

    rows[[i]] <- data.frame(
      variable  = cand$var,
      phase     = cand$phase %||% NA_character_,
      score     = s$score,
      p_value   = s$p_value,
      delta_aic = s$delta_aic,
      stat      = s$stat,
      df        = s$df,
      stringsAsFactors = FALSE
    )
    # Cache the fit on the row so we can recover it for the winner
    attr(rows[[i]], "fit") <- candidate_fit
  }

  all_scores <- do.call(rbind, rows)
  # Strip the per-row fit attributes from the combined frame but keep
  # them in a parallel list keyed by row for winner lookup.
  candidate_fits <- lapply(rows, function(r) attr(r, "fit"))

  valid <- which(!is.na(all_scores$score))
  if (length(valid) == 0L) {
    out <- null_result()
    out$all_scores <- all_scores
    out$refit_failures <- failures
    return(out)
  }

  best_idx <- valid[which.min(all_scores$score[valid])]
  best     <- all_scores[best_idx, ]

  threshold_met <- if (criterion == "wald") {
    best$score < slentry
  } else {
    best$score < 0
  }

  if (!threshold_met) {
    out <- null_result()
    out$all_scores <- all_scores
    out$refit_failures <- failures
    return(out)
  }

  list(
    accepted  = TRUE,
    fit       = candidate_fits[[best_idx]],
    variable  = best$variable,
    phase     = best$phase,
    score     = best$score,
    p_value   = best$p_value,
    delta_aic = best$delta_aic,
    stat      = best$stat,
    df        = best$df,
    all_scores = all_scores,
    refit_failures = failures
  )
}


# ---------------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# Backward step
# ---------------------------------------------------------------------------

#' Enumerate the currently-in-model (variable, phase) pairs eligible
#' for a drop test.
#'
#' Force-in variables are *included* in the returned list so their
#' scores appear in the trace (sec.2 Q4 of STEPWISE-DESIGN.md); the
#' forward/backward driver filters them out of the argmax pool via the
#' `force_in` flag on each row.
#'
#' @keywords internal
#' @noRd
.hzr_stepwise_drop_candidates <- function(fit) {
  dist <- fit$spec$dist
  if (dist == "multiphase") {
    per_phase <- .hzr_scope_current_vars(fit)
    out <- list()
    for (p in names(per_phase)) {
      for (v in per_phase[[p]]) {
        out[[length(out) + 1L]] <- list(var = v, phase = p)
      }
    }
    return(out)
  }
  vars <- .hzr_scope_current_vars(fit)
  lapply(vars, function(v) list(var = v, phase = NULL))
}


#' Execute one backward step of stepwise selection
#'
#' For each currently-in-model variable, scores drop via
#' `.hzr_candidate_score(mode = "drop")`.  Force-in variables are
#' scored and reported but excluded from the drop decision.
#'
#' Unlike the forward step there is no per-candidate refit: the Wald
#' statistic and its AIC approximation both operate on the current
#' model's vcov.  A single refit fires only after the drop decision
#' via `.hzr_refit_with_scope()`.
#'
#' @param current Fitted `hazard` object.
#' @param data Data frame used to rebuild the dropped model.
#' @param criterion Either `"wald"` or `"aic"`.
#' @param slstay Retention threshold for the Wald criterion (ignored
#'   when `criterion = "aic"`; the drop rule there is dAIC_drop < 0).
#' @param force_in Character vector of variables that may never be
#'   dropped.
#' @param ... Forwarded to `.hzr_refit_with_scope()` for the post-drop
#'   refit.
#'
#' @return A list with the same top-level shape as
#'   `.hzr_stepwise_forward_step()`, except:
#'   * `all_scores` gains a logical `force_in` column.
#'   * The action this represents is a drop, so `accepted = TRUE` means
#'     the variable was removed from the model.
#'
#' @keywords internal
#' @noRd
.hzr_stepwise_backward_step <- function(current, data,
                                         criterion = c("wald", "aic"),
                                         slstay    = 0.20,
                                         force_in  = character(),
                                         ...) {
  criterion <- match.arg(criterion)
  if (!inherits(current, "hazard")) {
    stop("`current` must be a fitted `hazard` object.", call. = FALSE)
  }

  cands <- .hzr_stepwise_drop_candidates(current)

  empty_scores <- data.frame(
    variable  = character(),
    phase     = character(),
    force_in  = logical(),
    score     = numeric(),
    p_value   = numeric(),
    delta_aic = numeric(),
    stat      = numeric(),
    df        = integer(),
    stringsAsFactors = FALSE
  )

  null_result <- function(all_scores = empty_scores) {
    list(
      accepted  = FALSE,
      fit       = current,
      variable  = NA_character_,
      phase     = NA_character_,
      score     = NA_real_,
      p_value   = NA_real_,
      delta_aic = NA_real_,
      stat      = NA_real_,
      df        = NA_integer_,
      all_scores     = all_scores,
      refit_failures = character()
    )
  }

  if (length(cands) == 0L) {
    return(null_result())
  }

  rows <- vector("list", length(cands))
  for (i in seq_along(cands)) {
    cand <- cands[[i]]
    coef_name <- .hzr_candidate_coef_name(current, cand$var, cand$phase)

    s <- .hzr_candidate_score(
      criterion = criterion, mode = "drop",
      current = current, names = coef_name
    )

    rows[[i]] <- data.frame(
      variable  = cand$var,
      phase     = cand$phase %||% NA_character_,
      force_in  = cand$var %in% force_in,
      score     = s$score,
      p_value   = s$p_value,
      delta_aic = s$delta_aic,
      stat      = s$stat,
      df        = s$df,
      stringsAsFactors = FALSE
    )
  }
  all_scores <- do.call(rbind, rows)

  eligible <- which(!all_scores$force_in & !is.na(all_scores$score))
  if (length(eligible) == 0L) {
    return(null_result(all_scores))
  }

  best_idx <- eligible[which.min(all_scores$score[eligible])]
  best     <- all_scores[best_idx, ]

  threshold_met <- if (criterion == "wald") {
    best$score < (1 - slstay)          # i.e. p > slstay
  } else {
    best$score < 0                     # dAIC_drop < 0
  }

  if (!threshold_met) {
    return(null_result(all_scores))
  }

  refitted <- tryCatch(
    .hzr_refit_with_scope(
      current, action = "drop",
      var = best$variable,
      phase = if (is.na(best$phase)) NULL else best$phase,
      data = data, ...
    ),
    error = function(e) e
  )

  failure_token <- if (is.na(best$phase)) {
    best$variable
  } else {
    paste0(best$variable, "@", best$phase)
  }

  if (inherits(refitted, "error") || isFALSE(refitted$fit$converged)) {
    warning("Stepwise backward: post-drop refit failed for ",
            failure_token, ".", call. = FALSE)
    out <- null_result(all_scores)
    out$refit_failures <- failure_token
    return(out)
  }

  list(
    accepted  = TRUE,
    fit       = refitted,
    variable  = best$variable,
    phase     = best$phase,
    score     = best$score,
    p_value   = best$p_value,
    delta_aic = best$delta_aic,
    stat      = best$stat,
    df        = best$df,
    all_scores     = all_scores,
    refit_failures = character()
  )
}


#' Name under which a newly-entered variable appears in coef(fit)
#'
#' Canonical naming differs between fit kinds:
#'   multiphase  -- phase-prefixed formula names (e.g. `"early.age"`).
#'   single-dist -- positional `"betaN"` from `.hzr_parameter_names()`,
#'     where N is the column index of `var` in `colnames(fit$data$x)`.
#'
#' This matches the naming `summary.hazard()` prints and the canonical
#' name `.hzr_wald_p()` uses for coefficient lookup.
#'
#' Stepwise v1 is scoped to main-effect terms only.  Multi-column
#' expansions (factors with > 2 levels, splines, interactions)
#' trigger an error so the caller can rebuild their formula with the
#' expansion baked in rather than silently scoring just one coefficient.
#'
#' @keywords internal
#' @noRd
.hzr_candidate_coef_name <- function(fit, var, phase) {
  if (fit$spec$dist == "multiphase") {
    target <- paste0(phase, ".", var)
    coef_names <- names(stats::coef(fit))
    if (!is.null(coef_names) && !target %in% coef_names) {
      expanded <- grep(
        paste0("^", .hzr_regex_escape(target)),
        coef_names, value = TRUE
      )
      if (length(expanded) > 1L) {
        stop(
          "Variable ", sQuote(var), " in phase ", sQuote(phase),
          " expands to multiple coefficients (",
          paste(sQuote(expanded), collapse = ", "),
          ").  Stepwise v1 supports main-effect terms only; ",
          "rebuild your candidate as pre-expanded main effects and retry.",
          call. = FALSE
        )
      }
    }
    return(target)
  }
  xcols <- colnames(fit$data$x)
  idx <- which(xcols == var)
  if (length(idx) == 0L) {
    # Check for a multi-column expansion masquerading as one variable.
    prefix_hits <- grep(paste0("^", .hzr_regex_escape(var)), xcols)
    if (length(prefix_hits) > 1L) {
      stop(
        "Variable ", sQuote(var),
        " expands to multiple coefficients (",
        paste(sQuote(xcols[prefix_hits]), collapse = ", "),
        ").  Stepwise v1 supports main-effect terms only; ",
        "rebuild your candidate as an already-expanded set of ",
        "main effects (e.g. a numeric contrast matrix) and retry.",
        call. = FALSE
      )
    }
    stop("Variable ", sQuote(var),
         " not found in the design matrix.",
         call. = FALSE)
  }
  if (length(idx) > 1L) {
    stop("Variable ", sQuote(var),
         " matches multiple design-matrix columns -- ",
         "stepwise v1 supports main-effect terms only.",
         call. = FALSE)
  }
  paste0("beta", idx)
}


#' Escape a character vector for literal use in a regular expression
#'
#' @keywords internal
#' @noRd
.hzr_regex_escape <- function(x) {
  gsub("([.\\\\|*+?^$(){}\\[\\]])", "\\\\\\1", x, perl = TRUE)
}


#' %||% -- NULL-coalesce for lazy defaults
#' @keywords internal
#' @noRd
`%||%` <- function(a, b) if (is.null(a)) b else a

# `.fit_stepwise_base()` lives in helper-stepwise.R (auto-sourced by
# testthat).

# .hzr_refit_with_scope -----------------------------------------------------

test_that("refit adds a covariate and grows theta", {
  obj <- .fit_stepwise_base()
  expect_length(obj$fit$fit$theta, 2L)

  refit <- .hzr_refit_with_scope(
    obj$fit, action = "add", var = "x1",
    data = obj$data
  )
  expect_s3_class(refit, "hazard")
  expect_length(refit$fit$theta, 3L)
  expect_true(refit$fit$converged)
  # The signal covariate should have a non-zero estimate
  last_coef <- refit$fit$theta[length(refit$fit$theta)]
  expect_gt(abs(last_coef), 0.1)
})

test_that("refit drops a covariate and shrinks theta", {
  obj <- .fit_stepwise_base()
  with_x <- .hzr_refit_with_scope(obj$fit, action = "add", var = "x1",
                                    data = obj$data)
  back <- .hzr_refit_with_scope(with_x, action = "drop", var = "x1",
                                 data = obj$data)
  expect_length(back$fit$theta, 2L)
  expect_true(back$fit$converged)
})

test_that("refit errors without a formula-based base fit", {
  fit <- .fit_weibull(n = 100L, seed = 1L)
  expect_error(
    .hzr_refit_with_scope(fit, action = "add", var = "x1",
                           data = data.frame()),
    "formula interface"
  )
})

test_that("refit rejects missing `phase` on multiphase fits", {
  data(avc)
  set.seed(1)
  fit <- hazard(
    Surv(int_dead, dead) ~ 1, data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2L, maxit = 500L)
  )
  expect_error(
    .hzr_refit_with_scope(fit, action = "add", var = "age",
                           data = avc),
    "`phase` is required"
  )
})

test_that("refit adds a var to a specific phase for multiphase", {
  data(avc)
  set.seed(1)
  fit <- hazard(
    Surv(int_dead, dead) ~ 1, data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2L, maxit = 500L)
  )
  refit <- .hzr_refit_with_scope(
    fit, action = "add", var = "age", phase = "early",
    data = avc, control = list(n_starts = 2L, maxit = 500L)
  )
  expect_s3_class(refit, "hazard")
  expect_true("early.age" %in% names(coef(refit)))
  expect_false("constant.age" %in% names(coef(refit)))
})


# .hzr_stepwise_candidates --------------------------------------------------

test_that("candidates excludes already-in-model vars and force_out", {
  obj <- .fit_stepwise_base()
  c1 <- .hzr_stepwise_candidates(obj$fit, scope = NULL,
                                   data = obj$data)
  vars <- vapply(c1, `[[`, character(1L), "var")
  expect_setequal(vars, c("x1", "x2", "x3"))

  with_x1 <- .hzr_refit_with_scope(obj$fit, action = "add", var = "x1",
                                     data = obj$data)
  c2 <- .hzr_stepwise_candidates(with_x1, scope = NULL,
                                   data = obj$data,
                                   force_out = "x3")
  vars2 <- vapply(c2, `[[`, character(1L), "var")
  expect_setequal(vars2, "x2")
})

test_that("candidates for multiphase emits (var, phase) pairs", {
  data(avc)
  set.seed(1)
  fit <- hazard(
    Surv(int_dead, dead) ~ 1, data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2L, maxit = 500L)
  )
  scope <- list(
    early    = ~ age + mal,
    constant = ~ age
  )
  cands <- .hzr_stepwise_candidates(fit, scope = scope, data = avc)
  pairs <- vapply(cands, function(c) paste0(c$var, "@", c$phase),
                  character(1L))
  expect_setequal(pairs, c("age@early", "mal@early", "age@constant"))
})


# .hzr_stepwise_forward_step ------------------------------------------------

test_that("forward step enters the strongest candidate under Wald", {
  obj <- .fit_stepwise_base()
  step <- .hzr_stepwise_forward_step(
    obj$fit, scope = ~ x1 + x2 + x3, data = obj$data,
    criterion = "wald", slentry = 0.30
  )
  expect_true(step$accepted)
  expect_identical(step$variable, "x1")
  expect_lt(step$score, 0.30)
  expect_lt(step$p_value, 0.05)  # signal should be highly significant
  # all_scores should list every candidate that was tried
  expect_true(nrow(step$all_scores) >= 3L)
})

test_that("forward step returns unchanged when no candidate clears threshold", {
  obj <- .fit_stepwise_base(signal_beta = 0)  # pure noise
  step <- .hzr_stepwise_forward_step(
    obj$fit, scope = ~ x1 + x2 + x3, data = obj$data,
    criterion = "wald", slentry = 0.01   # strict threshold
  )
  expect_false(step$accepted)
  expect_identical(step$fit, obj$fit)
})

test_that("forward step under AIC uses ΔAIC rule and accepts when negative", {
  obj <- .fit_stepwise_base()
  step <- .hzr_stepwise_forward_step(
    obj$fit, scope = ~ x1 + x2 + x3, data = obj$data,
    criterion = "aic"
  )
  expect_true(step$accepted)
  expect_identical(step$variable, "x1")
  expect_lt(step$score, 0)        # ΔAIC improvement
  expect_equal(step$score, step$delta_aic)
})

test_that("forward step respects force_out", {
  obj <- .fit_stepwise_base()
  step <- .hzr_stepwise_forward_step(
    obj$fit, scope = NULL, data = obj$data,
    criterion = "wald", slentry = 0.30,
    force_out = "x1"   # the signal is locked out
  )
  expect_true(step$accepted || !step$accepted)  # may or may not accept,
  # but x1 must not be the chosen variable:
  expect_false(isTRUE(step$variable == "x1"))
})

test_that("forward step reports no candidates when scope is empty", {
  obj <- .fit_stepwise_base()
  step <- .hzr_stepwise_forward_step(
    obj$fit, scope = character(), data = obj$data,
    criterion = "wald"
  )
  expect_false(step$accepted)
  expect_identical(nrow(step$all_scores), 0L)
})

# `.fit_overfitted()` lives in helper-stepwise.R (auto-sourced by
# testthat).

# .hzr_stepwise_drop_candidates --------------------------------------------

test_that("drop_candidates enumerates all in-model vars for single-dist", {
  obj <- .fit_overfitted()
  cands <- .hzr_stepwise_drop_candidates(obj$fit)
  vars  <- vapply(cands, `[[`, character(1L), "var")
  expect_setequal(vars, c("x1", "x2", "x3"))
  # Every candidate has phase = NULL
  expect_true(all(vapply(cands, function(c) is.null(c$phase), logical(1L))))
})

test_that("drop_candidates emits (var, phase) pairs for multiphase", {
  data(avc)
  set.seed(1)
  fit <- hazard(
    Surv(int_dead, dead) ~ 1, data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes", formula = ~ age),
      constant = hzr_phase("constant", formula = ~ age + mal)
    ),
    fit = TRUE,
    control = list(n_starts = 2L, maxit = 500L)
  )
  cands <- .hzr_stepwise_drop_candidates(fit)
  pairs <- vapply(cands, function(c) paste0(c$var, "@", c$phase),
                  character(1L))
  expect_setequal(pairs, c("age@early", "age@constant", "mal@constant"))
})


# .hzr_stepwise_backward_step ----------------------------------------------

test_that("backward step drops the weakest noise variable under Wald", {
  obj <- .fit_overfitted()
  step <- .hzr_stepwise_backward_step(
    obj$fit, data = obj$data,
    criterion = "wald", slstay = 0.20
  )
  expect_true(step$accepted)
  # The dropped var must be one of the noise covariates
  expect_true(step$variable %in% c("x2", "x3"))
  # Refit should lack that coefficient
  expect_false(step$variable %in% colnames(step$fit$data$x))
  # all_scores must cover every in-model var, including the strong one
  expect_setequal(step$all_scores$variable, c("x1", "x2", "x3"))
})

test_that("backward step keeps every variable when they're all strong", {
  # High-strength signal on every covariate -> nothing clears drop
  # threshold.
  set.seed(77L)
  n <- 500L
  df <- data.frame(
    time   = rexp(n, rate = 1),
    status = rep(1L, n),
    x1     = rnorm(n),
    x2     = rnorm(n)
  )
  df$time <- df$time * exp(-0.8 * df$x1 - 0.7 * df$x2)

  fit <- hazard(
    Surv(time, status) ~ x1 + x2, data = df,
    theta = c(0.5, 1.0, 0, 0),
    dist = "weibull", fit = TRUE
  )

  step <- .hzr_stepwise_backward_step(
    fit, data = df, criterion = "wald", slstay = 0.20
  )
  expect_false(step$accepted)
  expect_identical(step$fit, fit)
  expect_true(all(step$all_scores$p_value < 0.05))
})

test_that("backward step under AIC uses ΔAIC rule and the W - 2*df shortcut", {
  obj <- .fit_overfitted()
  step <- .hzr_stepwise_backward_step(
    obj$fit, data = obj$data, criterion = "aic"
  )
  # Noise vars have z^2 ~ 1 on average, so z^2 - 2 < 0 -> drop.
  expect_true(step$accepted)
  expect_lt(step$score, 0)
  expect_equal(step$score, step$delta_aic)
})

test_that("force_in variables are scored but not dropped", {
  obj <- .fit_overfitted()
  step <- .hzr_stepwise_backward_step(
    obj$fit, data = obj$data, criterion = "wald", slstay = 0.20,
    force_in = c("x2", "x3")
  )
  # Only x1 is droppable, but x1 is the strong signal -> no drop.
  expect_false(step$accepted)
  # Every in-model var appears in all_scores with force_in flagged
  # correctly.
  scores <- step$all_scores
  expect_true(scores$force_in[scores$variable == "x2"])
  expect_true(scores$force_in[scores$variable == "x3"])
  expect_false(scores$force_in[scores$variable == "x1"])
})

test_that("force_in can lock out every candidate entirely", {
  obj <- .fit_overfitted()
  step <- .hzr_stepwise_backward_step(
    obj$fit, data = obj$data, criterion = "wald", slstay = 0.20,
    force_in = c("x1", "x2", "x3")
  )
  expect_false(step$accepted)
  # All rows flagged as force_in
  expect_true(all(step$all_scores$force_in))
})

test_that("backward step returns unchanged when there are no covariates", {
  set.seed(1)
  n <- 100L
  df <- data.frame(time = rexp(n), status = rep(1L, n))
  fit <- hazard(
    Surv(time, status) ~ 1, data = df,
    theta = c(0.5, 1.0), dist = "weibull", fit = TRUE
  )
  step <- .hzr_stepwise_backward_step(
    fit, data = df, criterion = "wald", slstay = 0.20
  )
  expect_false(step$accepted)
  expect_identical(step$fit, fit)
  expect_identical(nrow(step$all_scores), 0L)
})

test_that("backward step works on multiphase with phase-specific drop", {
  # Note: avoid 3+ covariates per phase — there's a pre-existing
  # recycling warning in .hzr_multiphase_cumhaz() triggered at that
  # threshold (flagged as a separate spawned task).
  data(avc)
  set.seed(1)
  fit <- hazard(
    Surv(int_dead, dead) ~ 1, data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes",
                            formula = ~ age + mal),
      constant = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 3L, maxit = 1000L)
  )

  step <- .hzr_stepwise_backward_step(
    fit, data = avc, criterion = "wald", slstay = 0.20,
    control = list(n_starts = 2L, maxit = 500L)
  )

  # If a drop happens, it's from the early phase (the only phase with
  # covariates), and the refit should be multiphase and converged.
  if (step$accepted) {
    expect_identical(step$phase, "early")
    expect_s3_class(step$fit, "hazard")
    expect_identical(step$fit$spec$dist, "multiphase")
    expect_true(step$fit$fit$converged)
  }
  # Regardless, every (var, phase) pair in the early phase must be
  # represented in all_scores.
  early_vars <- step$all_scores$variable[step$all_scores$phase == "early"]
  expect_setequal(early_vars, c("age", "mal"))
})

# Broader integration tests for hzr_stepwise() — Step 8.7 /
# STEPWISE-DESIGN.md §6.1.  These exercise end-to-end scenarios that
# individual-helper tests do not cover: phase-specific selection,
# warm-start refit efficiency, and the candidate-failure warning path.


# Phase-specific entry -----------------------------------------------------

test_that("phase-specific candidates are enumerated distinctly per phase", {
  # The core requirement the design doc calls out is that a single
  # variable can be considered for entry into each phase independently.
  # Test the mechanism directly: after adding `x` to one phase, the
  # remaining candidates for the other phase still include `x`.
  data(avc)
  set.seed(1L)
  fit <- hazard(
    Surv(int_dead, dead) ~ 1, data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2L, maxit = 500L)
  )

  # Scope advertises `age` for both phases.
  scope <- list(early = ~ age, constant = ~ age)
  before <- .hzr_stepwise_candidates(fit, scope = scope, data = avc)
  pairs_before <- vapply(before,
                         function(c) paste0(c$var, "@", c$phase),
                         character(1L))
  expect_setequal(pairs_before, c("age@early", "age@constant"))

  # Add age into the early phase and re-enumerate.
  with_early <- .hzr_refit_with_scope(
    fit, action = "add", var = "age", phase = "early",
    data = avc, control = list(n_starts = 2L, maxit = 500L)
  )
  after <- .hzr_stepwise_candidates(with_early, scope = scope,
                                      data = avc)
  pairs_after <- vapply(after,
                        function(c) paste0(c$var, "@", c$phase),
                        character(1L))
  # age@constant should still be a candidate; age@early must not be.
  expect_true("age@constant" %in% pairs_after)
  expect_false("age@early" %in% pairs_after)
})


# Warm-start refit efficiency ----------------------------------------------

test_that("warm-started refits converge in far fewer iterations than cold starts", {
  # Fit a baseline model with two covariates, then add a third via the
  # refit wrapper. Compare optim counts to a cold hazard() call at the
  # same target.
  set.seed(17L)
  n <- 300L
  df <- data.frame(
    time   = rexp(n, 1),
    status = rep(1L, n),
    x1     = rnorm(n),
    x2     = rnorm(n),
    x3     = rnorm(n)
  )
  df$time <- df$time * exp(-0.8 * df$x1 - 0.4 * df$x2)

  base <- hazard(
    Surv(time, status) ~ x1 + x2, data = df,
    theta = c(0.5, 1.0, 0, 0), dist = "weibull", fit = TRUE
  )

  warm <- .hzr_refit_with_scope(
    base, action = "add", var = "x3", data = df
  )

  # Cold fit to the same target — starts every beta at 0 and shape
  # params at (0.5, 1.0) rather than the MLE.
  cold <- hazard(
    Surv(time, status) ~ x1 + x2 + x3, data = df,
    theta = c(0.5, 1.0, 0, 0, 0), dist = "weibull", fit = TRUE
  )

  expect_true(warm$fit$converged)
  # Warm and cold both reach the same MLE (essential correctness
  # guarantee).
  expect_equal(warm$fit$objective, cold$fit$objective, tolerance = 1e-3)

  # Warm should use no more function evaluations than cold.  The
  # design-doc promise is "typically one BFGS pass (~0.1 s)"; on
  # trivial single-distribution problems the gap can be small because
  # cold-start is already near the MLE (weibull starts at mu = 0.5,
  # nu = 1 which fits rexp(·, 1) data well).  Regressions in warm-
  # start correctness would show up as a strict increase here.
  warm_iters <- warm$fit$counts["function"]
  cold_iters <- cold$fit$counts["function"]
  expect_lte(warm_iters, cold_iters)
})


# Candidate-failure warning -------------------------------------------------

test_that("multi-level factor candidate errors with a main-effects note", {
  # A 3-level factor expands to 2 columns in the design matrix — v1
  # scope is main effects only, so stepwise should error cleanly
  # rather than silently score one contrast.
  set.seed(223L)
  n <- 200L
  df <- data.frame(
    time   = rexp(n),
    status = rep(1L, n),
    x1     = rnorm(n),
    f      = factor(sample(letters[1:3], n, replace = TRUE))
  )
  df$time <- df$time * exp(-0.8 * df$x1)

  # Fit with the factor already in the model so we can test the
  # single-dist guard path directly via .hzr_candidate_coef_name().
  fit <- hazard(
    Surv(time, status) ~ x1 + f, data = df,
    theta = c(0.5, 1.0, 0, 0, 0),
    dist = "weibull", fit = TRUE
  )

  expect_error(
    TemporalHazard:::.hzr_candidate_coef_name(fit, "f", NULL),
    "expands to multiple coefficients"
  )
})

test_that("one bad candidate does not prevent others from entering", {
  # Set up a scope where one candidate is broken (via a nonsense name
  # triggering the `var not in data` branch, which surfaces as an
  # error inside tryCatch) while a real one succeeds. The forward step
  # should warn about the broken candidate and still accept the good
  # one.
  obj <- .fit_driver_base(seed = 444L)
  df2 <- obj$data
  # Introduce a candidate that refers to a column *not* in `data` —
  # hazard() will error inside the candidate refit's tryCatch.
  expect_warning(
    step <- .hzr_stepwise_forward_step(
      obj$fit,
      scope = c("x1", "nonexistent"),
      data = df2,
      criterion = "wald", slentry = 0.30
    ),
    "candidate refit failed for nonexistent"
  )
  expect_true(step$accepted)
  expect_identical(step$variable, "x1")
  expect_identical(step$refit_failures, "nonexistent")
})

#' @importFrom stats optim setNames
#' @keywords internal
NULL

# likelihood-weibull.R -- Weibull parametric hazard likelihood, gradient, and optimizer
#
# MODEL
# -----
# Proportional-hazards (PH) parameterization:
#
#   H(t | x)  = (mu t)^nu exp(eta)              cumulative hazard
#   h(t | x)  = nu mu^nu t^(nu-1) exp(eta)      hazard = dH/dt = (nu/t) H
#   S(t | x)  = exp(-H(t | x))              survival
#
# where mu > 0 (scale), nu > 0 (shape), eta = x beta (linear predictor).
#
# THETA LAYOUT
# ------------
#   theta[1] = mu   (must be > 0; enforced by L-BFGS-B lower bound)
#   theta[2] = nu   (must be > 0; enforced by L-BFGS-B lower bound)
#   theta[3:p] = beta (covariate coefficients; unrestricted)
#
# Note: unlike the other distributions, Weibull optimises on the natural scale
# and uses L-BFGS-B with lower = 1e-6 instead of log-reparameterisation.
# This preserves direct interpretability of the mu/nu estimates.
#
# FUNCTIONS
# ---------
#   .hzr_logl_weibull()     -- log-likelihood (optionally returning gradient)
#   .hzr_gradient_weibull() -- analytical score vector
#   .hzr_optim_weibull()    -- L-BFGS-B wrapper with post-fit Hessian SE

# Weibull Parametric Hazard Likelihood
#
# Evaluate the log-likelihood and its derivatives for Weibull hazard models.
# This is the core mathematical engine for model fitting.

#' Log-likelihood for Weibull hazard with covariates
#'
#' Computes the negative log-likelihood for a sample under the Weibull
#' parametric hazard model with optional linear-predictor covariates.
#'
#' @param theta Vector of parameters:
#'   theta\[1:n_shape\]: Shape/scale parameters (distribution-specific)
#'   theta\[n_shape+1:length\]: Covariate coefficients (linear on log-hazard scale)
#'   where n_shape = number of shape parameters (e.g., 2 for Weibull scale/shape)
#'
#' @param time Numeric vector of follow-up times (n)
#' @param status Numeric vector of event indicators: 1 = event, 0 = censored (n)
#' @param time_lower Optional numeric lower bound vector for interval-censored rows.
#'   Defaults to time if NULL.
#' @param time_upper Optional numeric upper bound vector for left/interval-censored rows.
#'   Defaults to time if NULL.
#' @param x Design matrix of covariates (n x p_coef); NULL for no covariates
#' @param dist_name Character name of baseline distribution ("weibull", etc.)
#' @param return_gradient Logical; if TRUE, attach gradient vector as attribute
#' @param return_hessian Logical; if TRUE, attach Hessian matrix as attribute
#'
#' @return Scalar log-likelihood value. If return_gradient = TRUE, gradient vector
#' is attached as attribute \code{"gradient"}. If return_hessian = TRUE, Hessian
#' matrix is attached as \code{"hessian"}.
#'
#' @details
#' The Weibull hazard model is parameterized via its cumulative hazard
#' \eqn{H(t | x) = (\mu t)^{\nu} \exp(\eta)}, so the hazard is its exact
#' derivative:
#'
#' \deqn{h(t | x) = \nu \mu^{\nu} t^{\nu-1} \exp(\eta) = (\nu / t)\, H(t | x)}
#'
#' where:
#' - \eqn{\mu > 0} is scale (MU)
#' - \eqn{\nu > 0} is shape (NU)
#' - \eqn{\eta = x \beta} is log-relative-hazard (covariates)
#'
#' The log-likelihood for right-censored data is:
#'
#' \deqn{\ell(\theta) = \sum_{i: \delta_i = 1} \log h(t_i | x_i)
#'   - \sum_i S(t_i | x_i)}
#'
#' where S is the survival function (1 - CDF).
#'
#' Mixed censoring status coding:
#' - 1: exact event at time
#' - 0: right-censored at time
#' - -1: left-censored with upper bound time_upper \(or time\)
#' - 2: interval-censored in the interval \(time_lower, time_upper\)
#'
#' @noRd
.hzr_logl_weibull <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    dist_name = "weibull",
    return_gradient = FALSE,
    return_hessian = FALSE) {

  # Shape parameter count for Weibull
  n_shape <- 2  # mu, nu (scale, shape)
  n <- length(time)

  # Default unit weights
  if (is.null(weights)) weights <- rep(1, n)

  # Extract parameters
  mu <- theta[1]    # scale > 0
  nu <- theta[2]    # shape > 0

  # Covariate coefficients (if any)
  if (!is.null(x)) {
    if (is.null(attr(x, "dimnames")[[2]])) {
      colnames(x) <- paste0("beta_", seq_len(ncol(x)))
    }
    beta <- theta[3:length(theta)]
    eta <- as.numeric(x %*% beta)  # n-vector
  } else {
    eta <- rep(0, n)
  }

  # Stability checks
  if (mu <= 0 || nu <= 0) {
    return(Inf)  # Infeasible: return large penalty
  }

  # Normalize censoring bounds to lower/upper vectors for unified formulas.
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  # Exact events require strictly positive time because log(h(t)) is used.
  # Right-censored observations can be at time 0 (valid: log S(0) = 0).
  if (any(status == 1 & time <= 0)) {
    return(Inf)
  }
  if (any(status %in% c(-1, 2)) && any(upper <= 0)) {
    return(Inf)
  }
  if (any(status == 2) && any(lower[status == 2] >= upper[status == 2])) {
    return(Inf)
  }

  # Event-time hazard/cumulative-hazard (exact events only).
  # Hazard is the exact derivative of the cumulative hazard H = (mu t)^nu e^eta:
  #   h = dH/dt = nu * mu^nu * t^(nu-1) * e^eta = (nu / t) * H
  # (Form A, matching the C/SAS HAZARD reference where HF = MU * dG/dt.)
  haz_event <- nu * mu ^ nu * (time ^ (nu - 1)) * exp(eta)
  cumhaz_event <- (mu * time) ^ nu * exp(eta)

  # Lower/upper cumulative hazards for censoring contributions.
  cumhaz_lower <- (mu * lower) ^ nu * exp(eta)
  cumhaz_upper <- (mu * upper) ^ nu * exp(eta)

  # Counting-process entry-time cumulative hazard (H(start)) for
  # event/right-censored rows.  Only rows where time_lower < time and
  # status is 0 or 1 (genuine epoch entries) get a non-zero start.
  # Interval-censored rows (status == 2) always use start = 0 because
  # their contribution goes through cumhaz_lower/cumhaz_upper, not H(start).
  # This prevents time_lower being mistakenly interpreted as an epoch start
  # when it was supplied as the interval lower bound for status-2 rows.
  start_vec <- rep(0, n)
  if (!is.null(time_lower)) {
    epoch_idx <- status %in% c(0L, 1L) & time_lower < time
    start_vec[epoch_idx] <- time_lower[epoch_idx]
  }
  cumhaz_start <- (mu * start_vec) ^ nu * exp(eta)

  idx_event <- status == 1
  idx_right <- status == 0
  idx_left <- status == -1
  idx_interval <- status == 2

  # Exact events: w * [log h(stop) - (H(stop) - H(start))]
  ll_event <- if (any(idx_event)) {
    sum(weights[idx_event] *
          (log(haz_event[idx_event]) -
             (cumhaz_event[idx_event] - cumhaz_start[idx_event])))
  } else {
    0
  }

  # Right-censored: w * [-(H(stop) - H(start))]
  ll_right <- if (any(idx_right)) {
    -sum(weights[idx_right] *
           (cumhaz_event[idx_right] - cumhaz_start[idx_right]))
  } else {
    0
  }

  # Left-censored: w * [log(1 - exp(-H(u)))]
  ll_left <- if (any(idx_left)) {
    sum(weights[idx_left] * hzr_log1mexp(cumhaz_upper[idx_left]))
  } else {
    0
  }

  # Interval-censored: w * [-H(l) + log(1 - exp(-(H(u)-H(l))))]
  ll_interval <- if (any(idx_interval)) {
    delta_h <- cumhaz_upper[idx_interval] - cumhaz_lower[idx_interval]
    sum(weights[idx_interval] *
          (-cumhaz_lower[idx_interval] + hzr_log1mexp(delta_h)))
  } else {
    0
  }

  logl <- ll_event + ll_right + ll_left + ll_interval

  if (!is.finite(logl)) {
    return(Inf)
  }

  # If gradient requested, compute score vector.
  if (return_gradient) {
    grad <- .hzr_gradient_weibull(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      eta = eta, cumhaz = cumhaz_event, haz = haz_event,
      mu = mu, nu = nu, n_shape = n_shape
    )
    attr(logl, "gradient") <- grad
  }

  # If Hessian requested (stub for M2)
  if (return_hessian) {
    attr(logl, "hessian") <- matrix(NA, length(theta), length(theta))
  }

  logl
}

#' Score vector (gradient of log-likelihood)
#'
#' Computes the score vector (gradient) of the Weibull log-likelihood w.r.t. all parameters.
#'
#' @param time_lower Optional lower bounds for interval-censored rows.
#' @param time_upper Optional upper bounds for left/interval-censored rows.
#'
#' The log-likelihood is:
#'   L = sum(delta_i * log h(t_i)) - sum(H(t_i))
#'
#' where h(t) = nu * mu^nu * t^(nu-1) * exp(eta) and H(t) = (mu*t)^nu * exp(eta)
#' (so log h = log(nu) + nu*log(mu) + (nu-1)*log(t) + eta). Derivatives are:
#'
#' dL/dmu  = (nu / mu) * sum(delta_i) - (nu / mu) * sum(H(t_i))
#' dL/dnu  = sum(delta_i / nu) + sum(delta_i) * log(mu) + sum(delta_i * log(t_i))
#'           - sum(log(mu * t_i) * H(t_i))
#' dL/dbeta_j = sum(delta_i * x_ij) - sum(H(t_i) * x_ij)  = t(X) %*% (delta - H)
#'
#' @noRd
.hzr_gradient_weibull <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    eta = NULL,
    cumhaz = NULL,
    haz = NULL,
    mu = NULL,
    nu = NULL,
    n_shape = 2) {

  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)

  # Default unit weights -- keep gradient consistent with the LL when weights
  # are omitted.
  if (is.null(weights)) weights <- rep(1, n)

  # Interval/left censoring currently uses a robust numerical gradient fallback.
  # The closed-form derivatives for these terms are deferred to a later pass.
  if (any(status %in% c(-1, 2))) {
    return(.hzr_numeric_grad_weibull(theta, time, status, time_lower,
                                       time_upper, x, weights = weights))
  }

  # Sanity check: if eta, cumhaz, haz not provided, compute them
  if (is.null(eta) || is.null(cumhaz) || is.null(haz)) {
    # Need to recompute from theta
    mu <- theta[1]
    nu <- theta[2]

    if (!is.null(x)) {
      beta <- theta[3:length(theta)]
      eta <- as.numeric(x %*% beta)
    } else {
      eta <- rep(0, n)
    }

    haz <- nu * mu ^ nu * (time ^ (nu - 1)) * exp(eta)
    cumhaz <- (mu * time) ^ nu * exp(eta)
  }

  # Counting-process entry-time cumulative hazard; H(start) = 0 except
  # for genuine epoch rows (status 0/1 with time_lower < time).
  start_vec <- rep(0, n)
  if (!is.null(time_lower)) {
    epoch_idx <- status %in% c(0L, 1L) & time_lower < time
    start_vec[epoch_idx] <- time_lower[epoch_idx]
  }
  cumhaz_start <- (mu * start_vec) ^ nu * exp(eta)

  # Weighted building blocks: each per-row contribution below is the
  # unweighted form scaled by `weights`, and the cumulative-hazard term is
  # the epoch difference `H(stop) - H(start)` (degenerates to `H(stop)` when
  # start = 0).
  w_status <- weights * status
  w_cumhaz_net <- weights * (cumhaz - cumhaz_start)

  # ===== Gradient w.r.t. mu (scale parameter) =====
  # log h = log(nu) + nu*log(mu) + (nu-1)*log(t) + eta, so the event term
  # contributes d log h/dmu = nu/mu.  dH(t)/dmu = (nu / mu) * H(t), so the
  # cumulative term subtracts -(nu/mu) * (H(stop) - H(start)).
  grad[1] <- (nu / mu) * (sum(w_status) - sum(w_cumhaz_net))

  # ===== Gradient w.r.t. nu (shape parameter) =====
  # dH(t)/dnu = log(mu*t) * H(t).  When start = 0, H(start) = 0 and the
  # log(mu*start) term is undefined; `ifelse` picks the well-defined 0
  # limit there.
  # log h = log(nu) + nu*log(mu) + (nu-1)*log(t) + eta, so the event term
  # contributes d log h/dnu = 1/nu + log(mu) + log(t); the log(mu) piece is
  # constant across rows.
  # Guard log(time): a legal right-censored time = 0 row has w_status = 0 and
  # cumhaz = 0, but an unguarded log(0) = -Inf turns those into 0 * -Inf = NaN,
  # poisoning the entire summed nu (shape) gradient component (start is already
  # guarded by the ifelse below).
  log_t <- log(pmax(time, .Machine$double.xmin))
  log_mu_start <- ifelse(start_vec > 0, log(mu * start_vec), 0)
  d_nu_start <- weights * log_mu_start * cumhaz_start
  grad[2] <- sum(w_status) / nu +
             sum(w_status) * log(mu) +
             sum(w_status * log_t) -
             (sum((log(mu) + log_t) * weights * cumhaz) - sum(d_nu_start))

  # ===== Gradient w.r.t. beta (covariate coefficients) =====
  if (n_shape < p && !is.null(x)) {
    # dL/dbeta = t(X) %*% (w * (delta - (H(stop) - H(start))))
    residual <- w_status - w_cumhaz_net
    grad[3:p] <- as.numeric(crossprod(x, residual))
  }

  grad
}

#' Analytic Hessian of the Weibull negative log-likelihood (internal scale)
#'
#' Returns the Hessian of the objective (negative log-likelihood) on the internal
#' \code{phi = (alpha, psi, beta)} parameterization that \code{.hzr_optim_weibull}
#' optimizes, where \code{g = exp(psi)} and \code{H(t) = exp(alpha + eta) t^g}.
#' This matches \code{numDeriv::hessian(objective)} so it can be used directly by
#' \code{.hzr_optim_generic()}; the downstream delta-method transform to the
#' natural \code{(mu, nu, beta)} scale is applied by the caller.  Coverage mirrors
#' the analytic gradient: event + right-censored rows only; returns \code{NULL}
#' for any left/interval-censored row (\code{status \%in\% c(-1, 2)}) to request
#' the numerical-Hessian fallback.
#'
#' @noRd
.hzr_hessian_weibull_internal <- function(
    theta, time, status,
    time_lower = NULL, time_upper = NULL, x = NULL, weights = NULL) {

  n <- length(time)
  if (is.null(weights)) weights <- rep(1, n)

  # Coverage contract: closed form is event + right censored only.
  if (any(status %in% c(-1, 2))) {
    return(NULL)
  }

  alpha <- theta[1]
  psi <- theta[2]
  g <- exp(psi)
  p <- length(theta)
  p_cov <- p - 2L
  # Fail loud on inconsistent theta/x: a silent has_cov = FALSE would zero the
  # covariate rows/cols and yield a conformant-but-wrong vcov.
  if (!is.null(x)) x <- as.matrix(x)
  n_x <- if (is.null(x)) 0L else NCOL(x)
  if (n_x != p_cov) {
    stop(".hzr_hessian_weibull_internal(): ncol(x) (", n_x, ") must equal the ",
         "number of covariate parameters in theta (", p_cov, ").", call. = FALSE)
  }
  has_cov <- p_cov > 0L
  beta <- if (has_cov) theta[3:p] else numeric(0)
  eta <- if (has_cov) as.numeric(x %*% beta) else rep(0, n)

  # Cumulative hazards and counting-process entry-time (epochs only).
  base <- exp(alpha + eta)
  cumhaz <- base * (time ^ g)
  start_vec <- rep(0, n)
  if (!is.null(time_lower)) {
    epoch_idx <- status %in% c(0L, 1L) & time_lower < time
    start_vec[epoch_idx] <- time_lower[epoch_idx]
  }
  cumhaz_start <- base * (start_vec ^ g)

  log_t <- log(pmax(time, .Machine$double.xmin))
  log_ts <- ifelse(start_vec > 0, log(start_vec), 0)

  w <- weights
  delta <- as.numeric(status == 1)
  net <- cumhaz - cumhaz_start                       # A_i

  # Weighted aggregate building blocks.
  wA   <- w * net
  wD   <- w * (cumhaz * log_t - cumhaz_start * log_ts)
  # psi,psi scalar.
  hpp <- -g * (sum(w * delta * log_t) - sum(w * cumhaz * log_t) +
                 sum(w * cumhaz_start * log_ts)) +
          g^2 * (sum(w * cumhaz * log_t^2) - sum(w * cumhaz_start * log_ts^2))

  # Design augmented with the alpha intercept column.
  x_tilde <- if (has_cov) cbind(1, x) else matrix(1, nrow = n, ncol = 1L)

  hq <- crossprod(x_tilde, wA * x_tilde)             # (alpha,beta) block
  v  <- g * as.numeric(crossprod(x_tilde, wD))        # psi cross (alpha,beta)

  # Assemble p x p with order (alpha = 1, psi = 2, beta = 3..p).
  idx_q <- if (has_cov) c(1L, 3:p) else 1L
  hess <- matrix(0, p, p)
  hess[idx_q, idx_q] <- hq
  hess[2L, idx_q] <- v
  hess[idx_q, 2L] <- v
  hess[2L, 2L] <- hpp
  dimnames(hess) <- list(names(theta), names(theta))
  hess
}

#' @noRd
.hzr_optim_weibull <- function(
    time, status, time_lower = NULL, time_upper = NULL,
    x = NULL, theta_start, weights = NULL, control = list()) {

  # -- Internal reparameterisation --------------------------------------------
  # The original hazard C code parameterises the cumulative hazard as
  #
  #     H(t | x) = exp(alpha + Xbeta) * t^gamma
  #
  # where alpha is a free intercept, gamma = exp(psi) is the shape, and beta enters
  # the same log-hazard-rate space as alpha.  This decouples scale from shape
  # and avoids the degenerate ridge that appears when covariates with large
  # means (e.g. age ~ 62) interact with the (log mu, log nu) parameterisation.
  #
  # We optimise over phi = (alpha, psi, beta_1, ...) -- all unconstrained -- then
  # back-transform to the user-facing (mu, nu, beta) scale:
  #
  #     nu = exp(psi)      mu = exp(alpha / nu)
  #
  # and apply the delta method for the variance-covariance matrix.
  # --------------------------------------------------------------------------

  n_shape <- 2L
  p <- length(theta_start)
  n <- length(time)

  # -- Convert user theta (mu, nu, beta) -> internal (alpha, psi, beta) ------
  mu_start    <- unname(theta_start[1])
  nu_start    <- unname(theta_start[2])
  alpha_start <- nu_start * log(mu_start)   # alpha = nu * log(mu)
  psi_start   <- log(nu_start)              # psi = log(nu)
  beta_start  <- if (p > n_shape) unname(theta_start[(n_shape + 1):p]) else numeric(0)
  phi_start   <- c(alpha_start, psi_start, beta_start)

  # -- Internal likelihood: H(t|x) = exp(alpha + Xbeta) * t^exp(psi) ----------------
  # `weights` is declared as a formal (with a default of NULL) so that when
  # .hzr_optim_generic forwards weights via ..., it matches this formal
  # rather than landing in ... -- which would collide with the explicit
  # `weights =` on the delegated call below.
  .outer_w <- if (is.null(weights)) rep(1, n) else weights

  logl_internal <- function(theta, time, status, time_lower, time_upper,
                            x, weights = NULL, ...) {
    w_use <- if (is.null(weights)) .outer_w else weights
    alpha <- theta[1]
    g     <- exp(theta[2])   # nu = shape
    beta  <- if (p > n_shape) theta[(n_shape + 1):p] else numeric(0)
    eta   <- if (!is.null(x) && length(beta) > 0) {
      as.numeric(x %*% beta)
    } else {
      rep(0, n)
    }

    # Left / interval censoring: delegate to full Weibull likelihood via
    # the natural-scale function, converting params on the fly.
    if (any(status %in% c(-1, 2))) {
      mu <- exp(alpha / g)
      return(.hzr_logl_weibull(c(mu, g, beta), time, status,
                               time_lower, time_upper, x,
                               weights = w_use, ...))
    }

    log_t <- log(pmax(time, .Machine$double.xmin))
    H     <- exp(alpha + eta) * (time ^ g)

    # Counting-process entry-time H(start); zero except for genuine epochs.
    start_vec <- rep(0, n)
    if (!is.null(time_lower)) {
      epoch_idx <- status %in% c(0L, 1L) & time_lower < time
      start_vec[epoch_idx] <- time_lower[epoch_idx]
    }
    H_start   <- exp(alpha + eta) * (start_vec ^ g)

    idx_event <- status == 1
    idx_right <- status == 0

    ll_event <- if (any(idx_event)) {
      sum(w_use[idx_event] * (alpha + eta[idx_event] + theta[2] +
          (g - 1) * log_t[idx_event] -
          (H[idx_event] - H_start[idx_event])))
    } else {
      0
    }

    ll_right <- if (any(idx_right)) {
      -sum(w_use[idx_right] * (H[idx_right] - H_start[idx_right]))
    } else {
      0
    }

    logl <- ll_event + ll_right
    if (!is.finite(logl)) return(Inf)
    logl
  }

  # -- Internal gradient -----------------------------------------------------
  # `weights` is captured from the enclosing optim frame via `.outer_w`
  # (same trick used by logl_internal above) so the analytic gradient stays
  # consistent with the weighted LL whether or not .hzr_optim_generic
  # forwards the argument via ...
  grad_internal <- function(theta, time, status, time_lower, time_upper, x,
                            weights = NULL, ...) {
    w_use <- if (is.null(weights)) .outer_w else weights
    alpha <- theta[1]
    psi   <- theta[2]
    g     <- exp(psi)
    beta  <- if (p > n_shape) theta[(n_shape + 1):p] else numeric(0)
    eta   <- if (!is.null(x) && length(beta) > 0) as.numeric(x %*% beta) else rep(0, n)

    # Left / interval censoring -> numerical gradient fallback (already weighted
    # because logl_internal propagates weights via the closure).
    if (any(status %in% c(-1, 2))) {
      obj <- function(th) {
        logl_internal(th, time, status, time_lower, time_upper, x,
                      weights = w_use)
      }
      if (requireNamespace("numDeriv", quietly = TRUE))
        return(as.numeric(numDeriv::grad(obj, theta)))
      eps <- 1e-6
      grd <- numeric(p)
      for (i in seq_along(theta)) {
        tp <- tm <- theta
        tp[i] <- tp[i] + eps
        tm[i] <- tm[i] - eps
        grd[i] <- (obj(tp) - obj(tm)) / (2 * eps)
      }
      return(grd)
    }

    log_t <- log(pmax(time, .Machine$double.xmin))
    H     <- exp(alpha + eta) * (time ^ g)

    # H(start) for genuine epoch rows; zero otherwise.
    # `log_t_start` is only used where start > 0 (guarded below).
    start_vec <- rep(0, n)
    if (!is.null(time_lower)) {
      epoch_idx <- status %in% c(0L, 1L) & time_lower < time
      start_vec[epoch_idx] <- time_lower[epoch_idx]
    }
    H_start   <- exp(alpha + eta) * (start_vec ^ g)
    log_t_start <- ifelse(start_vec > 0, log(pmax(start_vec, .Machine$double.xmin)), 0)

    grad  <- numeric(p)

    # Weighted building blocks.  Every term below is the unweighted form with
    # `status` -> `w * status` and `H` -> `w * (H(stop) - H(start))`.
    w_status   <- w_use * status
    w_H_net    <- w_use * (H - H_start)
    w_H        <- w_use * H
    w_H_start  <- w_use * H_start

    # dL/dalpha = sum(w * delta) - sum(w * (H(stop) - H(start)))
    grad[1] <- sum(w_status) - sum(w_H_net)

    # dL/dpsi = sum(w * delta) + g * sum(w * delta * log(stop))
    #          - g * [sum(w * H(stop) * log(stop)) - sum(w * H(start) * log(start))]
    grad[2] <- sum(w_status) +
               g * sum(w_status * log_t) -
               g * (sum(w_H * log_t) - sum(w_H_start * log_t_start))

    # dL/dbeta_j = X_j' (w * (delta - (H(stop) - H(start))))
    if (p > n_shape && !is.null(x)) {
      grad[(n_shape + 1):p] <- as.numeric(crossprod(x, w_status - w_H_net))
    }

    grad
  }

  # -- Analytic Hessian closure (internal scale) --------------------------------
  hessian_fn <- function(par) {
    .hzr_hessian_weibull_internal(
      theta = par, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights
    )
  }

  # -- Optimise (all unconstrained) ------------------------------------------
  result <- .hzr_optim_generic(
    logl_fn     = logl_internal,
    gradient_fn = grad_internal,
    time        = time, status = status,
    time_lower  = time_lower, time_upper = time_upper,
    x           = x,
    theta_start = phi_start,
    weights     = weights,
    control     = control,
    use_bounds  = FALSE,
    hessian_fn  = hessian_fn
  )

  # -- Back-transform to natural scale (mu, nu, beta) ----------------------------
  alpha_hat <- result$par[1]
  psi_hat   <- result$par[2]
  nu_hat    <- exp(psi_hat)
  mu_hat    <- exp(alpha_hat / nu_hat)
  beta_hat  <- if (p > n_shape) result$par[(n_shape + 1):p] else numeric(0)

  result$par <- setNames(c(mu_hat, nu_hat, beta_hat), names(theta_start))

  # Delta method: Cov(mu, nu, beta) = J * Cov(alpha, psi, beta) * J^T
  #
  #   dmu/dalpha = mu / nu             dmu/dpsi = -mu alpha / nu
  #   dnu/dalpha = 0                 dnu/dpsi = nu
  #   dbeta/dalpha = 0                 dbeta/dpsi = 0         dbeta/dbeta = I
  if (is.matrix(result$vcov) && !anyNA(result$vcov)) {
    J <- diag(p)
    J[1, 1] <- mu_hat / nu_hat
    J[1, 2] <- -mu_hat * alpha_hat / nu_hat
    J[2, 1] <- 0
    J[2, 2] <- nu_hat
    result$vcov <- J %*% result$vcov %*% t(J)
  }

  result
}

.hzr_numeric_grad_weibull <- function(theta, time, status, time_lower = NULL,
                                       time_upper = NULL, x = NULL,
                                       weights = NULL) {
  objective <- function(par) {
    .hzr_logl_weibull(
      theta = par,
      time = time,
      status = status,
      time_lower = time_lower,
      time_upper = time_upper,
      x = x,
      weights = weights,
      return_gradient = FALSE
    )
  }

  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(as.numeric(numDeriv::grad(objective, theta)))
  }

  # Fallback central differences if numDeriv is unavailable.
  eps <- 1e-6
  grad <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tm <- theta
    tp[i] <- tp[i] + eps
    tm[i] <- tm[i] - eps
    grad[i] <- (objective(tp) - objective(tm)) / (2 * eps)
  }
  grad
}

# test-weibull-hessian.R -- numDeriv cross-check for .hzr_hessian_weibull_internal()
#
# Reconstruct the optimizer's internal objective (negative log-likelihood on the
# (alpha, psi, beta) scale). Post-PR#58 the natural-scale .hzr_logl_weibull
# equals logl_internal for event+right data, so we can map phi -> (mu, nu, beta)
# and reuse it.
.wb_obj_internal <- function(phi, time, status, time_lower = NULL,
                             time_upper = NULL, x = NULL, weights = NULL) {
  alpha <- phi[1]
  g <- exp(phi[2])
  beta <- if (length(phi) > 2) phi[3:length(phi)] else numeric(0)
  mu <- exp(alpha / g)
  theta_nat <- c(mu, g, beta)
  -.hzr_logl_weibull(theta_nat, time, status, time_lower, time_upper,
                     x = x, weights = weights)
}

test_that(".hzr_hessian_weibull_internal matches numDeriv (no covariates)", {
  skip_if_not_installed("numDeriv")
  set.seed(21)
  n <- 300
  time <- rexp(n, 0.6) + 0.01
  status <- rbinom(n, 1, 0.75)
  phi <- c(alpha = 0.4, psi = log(1.5))
  obj <- function(p) .wb_obj_internal(p, time, status)
  H_an <- .hzr_hessian_weibull_internal(phi, time, status)
  H_nd <- numDeriv::hessian(obj, phi)
  expect_equal(unname(H_an), unname(H_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_weibull_internal matches numDeriv (covariates + weights)", {
  skip_if_not_installed("numDeriv")
  set.seed(22)
  n <- 400
  x <- cbind(a = rnorm(n), b = rbinom(n, 1, 0.4))
  eta <- as.numeric(x %*% c(0.3, -0.4))
  time <- rweibull(n, shape = 1.4, scale = exp(-eta / 1.4)) + 0.01
  status <- rbinom(n, 1, 0.8)
  w <- runif(n, 0.5, 2)
  phi <- c(alpha = 0.2, psi = log(1.4), 0.3, -0.4)
  obj <- function(p) .wb_obj_internal(p, time, status, x = x, weights = w)
  H_an <- .hzr_hessian_weibull_internal(phi, time, status, x = x, weights = w)
  H_nd <- numDeriv::hessian(obj, phi)
  expect_equal(unname(H_an), unname(H_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_weibull_internal matches numDeriv (counting-process start)", {
  skip_if_not_installed("numDeriv")
  set.seed(23)
  n <- 250
  start <- runif(n, 0, 0.5)
  time <- start + rexp(n, 0.7) + 0.05
  status <- rbinom(n, 1, 0.7)
  phi <- c(alpha = 0.3, psi = log(1.2))
  obj <- function(p) .wb_obj_internal(p, time, status, time_lower = start)
  H_an <- .hzr_hessian_weibull_internal(phi, time, status, time_lower = start)
  H_nd <- numDeriv::hessian(obj, phi)
  expect_equal(unname(H_an), unname(H_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_weibull_internal returns NULL for left/interval censoring", {
  set.seed(24)
  n <- 40
  time <- rexp(n, 0.5) + 0.01
  status <- rep(1L, n)
  status[1:5] <- 2L
  expect_null(.hzr_hessian_weibull_internal(
    c(alpha = 0, psi = 0), time, status,
    time_lower = time, time_upper = time + 0.5))
  status2 <- rep(1L, n)
  status2[1:5] <- -1L
  expect_null(.hzr_hessian_weibull_internal(
    c(alpha = 0, psi = 0), time, status2, time_upper = time + 0.5))
})

test_that(".hzr_hessian_weibull_internal handles right-censored time = 0 rows", {
  skip_if_not_installed("numDeriv")
  set.seed(27)
  n <- 60
  time <- c(0, rexp(n - 1, 0.6) + 0.01)      # row 1: right-censored at time 0
  status <- c(0L, rbinom(n - 1, 1, 0.7))
  phi <- c(alpha = 0.3, psi = log(1.3))
  H_an <- .hzr_hessian_weibull_internal(phi, time, status)
  expect_true(all(is.finite(H_an)))
  obj <- function(p) .wb_obj_internal(p, time, status)
  H_nd <- numDeriv::hessian(obj, phi)
  expect_equal(unname(H_an), unname(H_nd), tolerance = 1e-4)
})

test_that("weibull fit vcov uses the analytic Hessian (matches numDeriv+delta)", {
  skip_if_not_installed("numDeriv")
  set.seed(25)
  n <- 500
  z <- rnorm(n)
  time <- rweibull(n, shape = 1.5, scale = exp(-0.5 * z / 1.5)) + 0.01
  status <- rbinom(n, 1, 0.85)
  fit <- hazard(
    survival::Surv(time, status) ~ z,
    data = data.frame(time, status, z),
    dist = "weibull", theta = c(mu = 1, nu = 1, z = 0), fit = TRUE
  )
  # Reference: numDeriv internal-scale Hessian -> invert -> delta-method J.
  th <- fit$fit$theta              # natural (mu, nu, beta)
  mu <- th[1]
  nu <- th[2]
  beta <- th[-(1:2)]
  alpha <- unname(nu * log(mu))
  psi <- unname(log(nu))
  phi <- c(alpha, psi, unname(beta))
  obj <- function(p) .wb_obj_internal(p, time, status, x = cbind(z))
  v_int <- solve(numDeriv::hessian(obj, phi))
  pdim <- length(phi)
  jmat <- diag(pdim)
  jmat[1, 1] <- mu / nu
  jmat[1, 2] <- -mu * alpha / nu
  jmat[2, 2] <- nu
  v_ref <- jmat %*% v_int %*% t(jmat)
  expect_equal(unname(fit$fit$vcov), unname(v_ref), tolerance = 1e-3)
  expect_true(isTRUE(fit$fit$pd))
})

test_that("weibull SEs are invariant to covariate rescaling", {
  set.seed(26)
  n <- 600
  z <- rnorm(n)
  time <- rweibull(n, shape = 1.3, scale = exp(-0.4 * z / 1.3)) + 0.01
  status <- rbinom(n, 1, 0.9)
  f1 <- hazard(survival::Surv(time, status) ~ z,
               data = data.frame(time, status, z = z),
               dist = "weibull", theta = c(mu = 1, nu = 1, z = 0), fit = TRUE)
  f2 <- hazard(survival::Surv(time, status) ~ z,
               data = data.frame(time, status, z = z / 100),
               dist = "weibull", theta = c(mu = 1, nu = 1, z = 0), fit = TRUE)
  se1 <- sqrt(diag(f1$fit$vcov))
  se2 <- sqrt(diag(f2$fit$vcov))
  # mu and nu SEs are invariant to covariate rescaling.
  expect_equal(unname(se1[1]), unname(se2[1]), tolerance = 1e-2)
  expect_equal(unname(se1[2]), unname(se2[2]), tolerance = 1e-2)
})

test_that(".hzr_hessian_weibull_internal errors on inconsistent theta/x", {
  set.seed(62)
  n <- 30
  time <- rexp(n, 0.5) + 0.01
  status <- rbinom(n, 1, 0.7)
  expect_error(
    .hzr_hessian_weibull_internal(c(alpha = 0, psi = 0, z = 0.3), time, status, x = NULL),
    "ncol")
  x2 <- cbind(a = rnorm(n), b = rnorm(n))
  expect_error(
    .hzr_hessian_weibull_internal(c(alpha = 0, psi = 0, z = 0.3), time, status, x = x2),
    "ncol")
})

#' AVC: Atrioventricular Canal Repair
#'
#' Survival data for 310 patients who underwent repair of atrioventricular
#' septal defects (congenital heart disease) at the Cleveland Clinic between
#' 1977 and 1993. Exhibits two identifiable hazard phases: an early
#' post-operative risk and a constant late phase.
#'
#' @format A data frame with 310 rows and 11 variables:
#' \describe{
#'   \item{study}{Patient identifier}
#'   \item{status}{NYHA functional class (1--4)}
#'   \item{inc_surg}{Surgical grade of AV valve incompetence}
#'   \item{opmos}{Date of operation (months since January 1967)}
#'   \item{age}{Age at repair (months)}
#'   \item{mal}{Malalignment indicator (0/1)}
#'   \item{com_iv}{Interventricular communication indicator (0/1)}
#'   \item{orifice}{Associated cardiac anomaly indicator (0/1)}
#'   \item{dead}{Death indicator (1 = dead, 0 = censored)}
#'   \item{int_dead}{Follow-up interval to death or last contact (months)}
#'   \item{op_age}{Interaction term: opmos x age}
#' }
#'
#' @source Blackstone, Naftel, and Turner (1986)
#'   \doi{10.1080/01621459.1986.10478314}. Cleveland Clinic Foundation.
#'
#' @examples
#' data(avc)
#' avc <- na.omit(avc)
#'
#' # Kaplan-Meier survival
#' km <- survival::survfit(survival::Surv(int_dead, dead) ~ 1, data = avc)
#' plot(km, xlab = "Months after AVC repair", ylab = "Survival",
#'      main = "AVC: Kaplan-Meier survival estimate")
#'
#' \donttest{
#' # Two-phase hazard fit (early CDF + constant -- what AVC supports)
#' fit <- hazard(
#'   survival::Surv(int_dead, dead) ~ 1, data = avc,
#'   dist = "multiphase",
#'   phases = list(
#'     early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1),
#'     constant = hzr_phase("constant")
#'   ),
#'   fit = TRUE, control = list(n_starts = 5, maxit = 1000)
#' )
#' summary(fit)
#' }
#'
#' @seealso \code{vignette("fitting-hazard-models")},
#'   \code{vignette("prediction-visualization")}
#' @family datasets
"avc"

#' CABGKUL: Primary Isolated Coronary Artery Bypass Grafting (KU Leuven)
#'
#' Survival data for 5,880 patients who underwent primary isolated CABG
#' at KU Leuven, Belgium, between 1971 and July 1987. The simplest dataset
#' structure (intercept-only, right-censored) with large sample size
#' exercising all three temporal hazard phases.
#'
#' @format A data frame with 5880 rows and 2 variables:
#' \describe{
#'   \item{int_dead}{Follow-up interval to death or last contact (months)}
#'   \item{dead}{Death indicator (1 = dead, 0 = censored)}
#' }
#'
#' @source KU Leuven cardiac surgery registry. Primary benchmark dataset for
#'   C binary parity testing.
#'
#' @examples
#' data(cabgkul)
#'
#' # Kaplan-Meier survival
#' km <- survival::survfit(survival::Surv(int_dead, dead) ~ 1, data = cabgkul)
#' plot(km, xlab = "Months after CABG", ylab = "Survival",
#'      main = "CABGKUL: Kaplan-Meier survival (n = 5,880)")
#'
#' \donttest{
#' # Single-phase Weibull fit with parametric overlay
#' fit <- hazard(survival::Surv(int_dead, dead) ~ 1, data = cabgkul,
#'               dist = "weibull", theta = c(mu = 0.10, nu = 1.0), fit = TRUE)
#' t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.9, length.out = 200)
#' surv   <- predict(fit, newdata = data.frame(time = t_grid),
#'                   type = "survival")
#' plot(km, xlab = "Months after CABG", ylab = "Survival",
#'      main = "CABGKUL: Weibull vs. Kaplan-Meier")
#' lines(t_grid, surv, col = "blue", lwd = 2)
#' legend("bottomleft", c("KM", "Weibull"), col = c("black", "blue"),
#'        lty = 1, lwd = c(1, 2))
#' }
#'
#' @seealso \code{vignette("fitting-hazard-models")}
#' @family datasets
"cabgkul"

# `cabgkul` is referenced by name (lazy-loaded dataset) in the package's
# own parity tests; declare it as a known global so static checkers do not
# flag a missing binding.
utils::globalVariables("cabgkul")

#' OMC: Open Mitral Commissurotomy
#'
#' Data for 339 patients who underwent open mitral commissurotomy at the
#' University of Alabama Birmingham. Contains repeated thromboembolic events
#' (up to 3 per patient) with left censoring, exercising the interval
#' censoring likelihood.
#'
#' @format A data frame with 339 rows and 7 variables:
#' \describe{
#'   \item{study}{Patient identifier}
#'   \item{te1}{Indicator for first thromboembolic event}
#'   \item{te2}{Indicator for second thromboembolic event}
#'   \item{te3}{Indicator for third thromboembolic event}
#'   \item{int_dead}{Follow-up interval to death or last contact (months)}
#'   \item{dead}{Death indicator (1 = dead, 0 = censored)}
#'   \item{opdjul}{Operation date (Julian)}
#' }
#'
#' @source University of Alabama Birmingham cardiac surgery registry.
#' @family datasets
"omc"

#' TGA: Transposition of the Great Arteries
#'
#' Survival data for 470 patients who underwent the arterial switch operation
#' for transposition of the great arteries at Boston Children's Hospital and
#' Children's Hospital of Philadelphia. Used for sensitivity analysis and
#' internal validation examples.
#'
#' @format A data frame with 470 rows and 14 variables:
#' \describe{
#'   \item{study}{Patient identifier}
#'   \item{simple}{Simple TGA indicator (0/1)}
#'   \item{dextroin}{D-looped transposition indicator (0/1)}
#'   \item{ca_1rl2c}{Coronary artery pattern indicator}
#'   \item{hyaaproc}{Hybrid approach procedure indicator (0/1)}
#'   \item{no_tca}{No total circulatory arrest indicator (0/1)}
#'   \item{tca_time}{Total circulatory arrest time (minutes)}
#'   \item{age_days}{Age at operation (days)}
#'   \item{arciopyr}{Aortic cross-clamp time per year}
#'   \item{dead}{Death indicator (1 = dead, 0 = censored)}
#'   \item{int_dead}{Follow-up interval to death or last contact (months)}
#'   \item{source}{Source institution (BCH or CHOP)}
#'   \item{ca1_2_l}{Coronary artery configuration (1/2/L)}
#'   \item{opyear}{Year of operation}
#' }
#'
#' @source Boston Children's Hospital and Children's Hospital of Philadelphia.
#' @family datasets
"tga"

#' Valves: Primary Heart Valve Replacement
#'
#' Data for 1,533 patients who underwent primary heart valve replacement.
#' The largest multivariable example dataset with multiple endpoints
#' including death, prosthetic valve endocarditis (PVE), bioprosthesis
#' degeneration, and reoperation.
#'
#' @format A data frame with 1533 rows and 19 variables:
#' \describe{
#'   \item{age_cop}{Age at operation (years)}
#'   \item{nyha}{NYHA functional class (1--4)}
#'   \item{mitral}{Mitral valve position indicator (0/1)}
#'   \item{double_}{Double valve replacement indicator (0/1)}
#'   \item{ao_pinc}{Aortic position, incompetence (0/1)}
#'   \item{black}{Black race indicator (0/1)}
#'   \item{i_path}{Ischemic pathology indicator (0/1)}
#'   \item{nve}{Native valve endocarditis indicator (0/1)}
#'   \item{mechvalv}{Mechanical valve indicator (0/1)}
#'   \item{male}{Male sex indicator (0/1)}
#'   \item{int_dead}{Follow-up interval to death or last contact (months)}
#'   \item{dead}{Death indicator (1 = dead, 0 = censored)}
#'   \item{int_pve}{Follow-up interval to PVE or last contact (months)}
#'   \item{pve}{PVE indicator (1 = PVE, 0 = censored)}
#'   \item{bio}{Bioprosthesis indicator (0/1)}
#'   \item{int_rdg}{Follow-up interval to degeneration or last contact (months)}
#'   \item{reop_dg}{Reoperation for degeneration indicator (0/1)}
#'   \item{int_reop}{Follow-up interval to reoperation or last contact (months)}
#'   \item{reop}{Reoperation indicator (0/1)}
#' }
#'
#' @source Cleveland Clinic Foundation heart valve replacement registry.
#'
#' @examples
#' data(valves)
#' valves_cc <- na.omit(valves)
#'
#' # Kaplan-Meier for two endpoints
#' km_death <- survival::survfit(
#'   survival::Surv(int_dead, dead) ~ 1, data = valves_cc)
#' km_pve <- survival::survfit(
#'   survival::Surv(int_pve, pve) ~ 1, data = valves_cc)
#'
#' plot(km_death, xlab = "Months after valve replacement", ylab = "Survival",
#'      main = "Valves: Death and PVE endpoints")
#' lines(km_pve, col = "red")
#' legend("bottomleft", c("Death", "PVE"), col = c("black", "red"), lty = 1)
#'
#' @seealso \code{vignette("fitting-hazard-models")},
#'   \code{vignette("prediction-visualization")}
#' @family datasets
"valves"

#' @importFrom stats dnorm pnorm optim
#' @keywords internal
NULL

# likelihood-lognormal.R -- Log-normal parametric hazard likelihood, gradient, and optimizer
#
# MODEL
# -----
# Accelerated Failure Time (AFT) log-normal model:
#
#   log(T_i) ~ Normal(eta_i, sigma^2)   where eta_i = mu + x_i beta
#
#   z_i       = (log(t_i) - eta_i) / sigma          standardised residual
#   S(t | x)  = Phi(-z_i)                        survival  (normal CDF)
#   f(t | x)  = phi(z_i) / (sigma t_i)              density
#   h(t | x)  = phi(z_i) / (sigma t_i Phi(-z_i))     hazard
#   H(t | x)  = -log Phi(-z_i)                  cumulative hazard
#
# where mu in R (location), sigma > 0 (scale), eta_i = mu + x_i beta (AFT predictor).
#
# AFT vs PH DISTINCTION
# ---------------------
# Unlike Weibull / exponential / log-logistic (PH models), covariates in the
# AFT parameterisation *shift the log-time distribution*.  The linear predictor
# adds to mu (location), not to log-hazard.  Consequently:
#
#   * predict() "linear_predictor" returns beta (same interface as PH)
#   * predict() "survival" returns Phi(-z) directly (NOT exp(-H))
#   * predict() "cumulative_hazard" returns -log Phi(-z)
#
# See the EXCEPTION note in hazard_api.R's predict() section banner.
#
# THETA LAYOUT
# ------------
#   theta[1]   = mu         (location; unrestricted)
#   theta[2]   = log(sigma)   (unconstrained; sigma recovered via exp())
#   theta[3:p] = beta        (covariate AFT coefficients; unrestricted)
#
# GRADIENT (let w_i = phi(z_i)/Phi(-z_i), the inverse Mills ratio)
# ---------------------------------------------------------------
#   dL/dmu          = (1/sigma) * [sum(delta z) + sum((1-delta) w)]
#   dL/d(log sigma)    = sum(delta(z^2-1)) + sum((1-delta) w z)
#   dL/dbeta_j        = (1/sigma) * sum([delta z + (1-delta) w] * x_j)
#
# FUNCTIONS
# ---------
#   .hzr_logl_lognormal()     -- log-likelihood (optionally returning gradient)
#   .hzr_gradient_lognormal() -- analytical score vector
#   .hzr_optim_lognormal()    -- unconstrained BFGS wrapper

# Log-Normal Parametric Hazard Likelihood
#
# Evaluate the log-likelihood and its derivatives for log-normal hazard models.
# The log-normal is an AFT (accelerated failure time) model with separate
# location (mu) and scale (sigma) parameters.

#' Log-likelihood for log-normal hazard with covariates
#'
#' Computes the log-likelihood for right-censored data under the log-normal
#' parametric hazard model with optional linear-predictor covariates.
#'
#' @param theta Vector of parameters:
#'   theta\[1\] = mu (location, unrestricted)
#'   theta\[2\] = log(sigma) where sigma > 0 is the scale parameter
#'   theta\[3:length\]: Covariate coefficients (AFT parameterization: add to location)
#'
#' @param time Numeric vector of follow-up times (n)
#' @param status Numeric vector of event indicators: 1 = event, 0 = censored (n)
#' @param time_lower Optional numeric lower bound vector for interval-censored rows.
#'   Defaults to time if NULL.
#' @param time_upper Optional numeric upper bound vector for left/interval-censored rows.
#'   Defaults to time if NULL.
#' @param x Design matrix of covariates (n x p_coef); NULL for no covariates
#' @param return_gradient Logical; if TRUE, attach gradient vector as attribute
#'
#' @return Scalar log-likelihood value. If return_gradient = TRUE, gradient vector
#' is attached as attribute \code{"gradient"}.
#'
#' @details
#' The log-normal hazard model uses AFT parameterization:
#'
#' z_i = (log(t_i) - eta_i) / sigma, where eta_i = mu + x_i beta
#'
#' Survival function:
#' \deqn{S(t | x) = \Phi(-z_i)}
#'
#' Density:
#' \deqn{f(t | x) = \phi(z_i) / (\sigma t_i)}
#'
#' Hazard:
#' \deqn{h(t | x) = \phi(z_i) / (\sigma t_i \Phi(-z_i))}
#'
#' The log-likelihood for right-censored data is:
#' \deqn{\ell(\theta) = \sum_{\delta_i=1} [\log \phi(z_i) - \log \sigma - \log t_i]
#'   + \sum_{\delta_i=0} \log \Phi(-z_i)}
#'
#' Reparameterization: \u03b8\[1\] = \u03bc, \u03b8\[2\] = log(\u03c3) avoids constraints.
#'
#' Mixed censoring status coding:
#' - 1: exact event at time
#' - 0: right-censored at time
#' - -1: left-censored with upper bound time_upper \(or time\)
#' - 2: interval-censored in the interval \(time_lower, time_upper\)
#'
#' @noRd
.hzr_logl_lognormal <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    return_gradient = FALSE) {

  n <- length(time)

  # Default unit weights
  if (is.null(weights)) weights <- rep(1, n)

  # Extract parameters
  mu <- theta[1]          # Location (unrestricted)
  log_sigma <- theta[2]   # Log-scale (unrestricted)
  sigma <- exp(log_sigma) # Always positive

  # Covariate coefficients (if any) -- AFT parameterization: add to location
  if (!is.null(x)) {
    if (is.null(attr(x, "dimnames")[[2]])) {
      colnames(x) <- paste0("beta_", seq_len(ncol(x)))
    }
    beta_coef <- theta[3:length(theta)]
    eta <- mu + as.numeric(x %*% beta_coef)  # Total linear predictor (location)
  } else {
    eta <- rep(mu, n)
  }

  # Normalize censoring bounds to lower/upper vectors for unified formulas.
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  if (any(status %in% c(1, 0) & time <= 0)) return(Inf)
  if (any(status == -1 & upper <= 0)) return(Inf)
  if (any(status == 2 & (lower <= 0 | upper <= 0))) return(Inf)
  if (any(status == 2) && any(lower[status == 2] >= upper[status == 2])) return(Inf)

  # Event-time terms are based on z at observed `time`.
  log_t <- log(time)
  z <- (log_t - eta) / sigma
  log_phi_z <- dnorm(z, log = TRUE)
  log_surv <- pnorm(-z, log.p = TRUE)

  idx_event <- status == 1
  idx_right <- status == 0
  idx_left <- status == -1
  idx_interval <- status == 2

  # Event: w * log f(t)
  ll_event <- if (any(idx_event)) {
    sum(weights[idx_event] *
          (log_phi_z[idx_event] - log_sigma - log_t[idx_event]))
  } else {
    0
  }

  # Right-censored: w * log S(t)
  ll_right <- if (any(idx_right)) {
    sum(weights[idx_right] * log_surv[idx_right])
  } else {
    0
  }

  # Left-censored: w * log F(u)
  ll_left <- if (any(idx_left)) {
    z_u <- (log(upper[idx_left]) - eta[idx_left]) / sigma
    sum(weights[idx_left] * pnorm(z_u, log.p = TRUE))
  } else {
    0
  }

  # Interval-censored: w * log(F(u) - F(l)) with positivity guard.
  ll_interval <- if (any(idx_interval)) {
    z_l <- (log(lower[idx_interval]) - eta[idx_interval]) / sigma
    z_u <- (log(upper[idx_interval]) - eta[idx_interval]) / sigma
    f_l <- pnorm(z_l)
    f_u <- pnorm(z_u)
    diff_f <- f_u - f_l
    if (any(diff_f <= 0)) return(Inf)
    sum(weights[idx_interval] * log(diff_f))
  } else {
    0
  }

  logl <- ll_event + ll_right + ll_left + ll_interval

  if (!is.finite(logl)) {
    return(Inf)
  }

  if (return_gradient) {
    grad <- .hzr_gradient_lognormal(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      eta = eta, sigma = sigma, log_sigma = log_sigma,
      z = z, log_phi_z = log_phi_z, log_surv = log_surv
    )
    attr(logl, "gradient") <- grad
  }

  logl
}

#' Score vector (gradient of log-likelihood) for log-normal
#'
#' Computes the score vector of the log-normal log-likelihood w.r.t. all parameters.
#'
#' Let z_i = (log(t_i) - eta_i) / sigma, where eta_i = mu + x_i beta
#' Let w_i = phi(z_i) / Phi(-z_i) (inverse Mills ratio)
#'
#' Derivatives:
#' \eqn{dL/d\mu = (1/\sigma) * [\sum(\delta_i * z_i) + \sum((1 - \delta_i) * w_i)]}
#' \eqn{dL/d(\log \sigma) = \sum(\delta_i * (z_i^2 - 1)) + \sum((1 - \delta_i) * w_i * z_i)}
#' \eqn{dL/d\beta_j = (1/\sigma) * \sum([\delta_i * z_i + (1 - \delta_i) * w_i] * x_{ij})}
#'
#' @noRd
.hzr_gradient_lognormal <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    eta = NULL,
    sigma = NULL,
    log_sigma = NULL,
    z = NULL,
    log_phi_z = NULL,
    log_surv = NULL) {

  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)

  # Default unit weights -- keep gradient consistent with the LL when weights
  # are omitted.
  if (is.null(weights)) weights <- rep(1, n)

  # Mixed-censoring score uses numerical differentiation for robustness.
  if (any(status %in% c(-1, 2))) {
    return(.hzr_numeric_grad_lognormal(theta, time, status, time_lower,
                                        time_upper, x, weights = weights))
  }

  # Recompute if not provided
  if (is.null(z) || is.null(sigma)) {
    mu <- theta[1]
    log_sigma <- theta[2]
    sigma <- exp(log_sigma)

    if (!is.null(x)) {
      beta_coef <- theta[3:length(theta)]
      eta <- mu + as.numeric(x %*% beta_coef)
    } else {
      eta <- rep(mu, n)
    }

    log_t <- log(time)
    z <- (log_t - eta) / sigma
    log_phi_z <- dnorm(z, log = TRUE)
    log_surv <- pnorm(-z, log.p = TRUE)
  }

  # Inverse Mills ratio: mills_i = phi(z_i) / Phi(-z_i)
  # Computed in log scale for numerical stability, then exponentiating.
  log_mills <- log_phi_z - log_surv
  mills <- exp(log_mills)  # Safe: log_surv can be -Inf when Phi(-z) -> 0 (z -> +Inf)
  # Clamp to prevent Inf * 0 issues when z is very large.
  mills <- pmin(mills, 1e6)

  censored <- 1 - status  # (1 - delta_i)

  # Weighted per-row building blocks. Every term below is the unweighted form
  # with `delta`, `(1-delta)*mills` etc. multiplied by `weights`.
  w_status <- weights * status
  w_censmills <- weights * censored * mills

  # ===== Gradient w.r.t. mu =====
  # dL/dmu = (1/sigma) * sum(w * [delta * z + (1-delta) * mills])
  grad[1] <- (sum(w_status * z) + sum(w_censmills)) / sigma

  # ===== Gradient w.r.t. log(sigma) =====
  # dL/d(log sigma) = sum(w * delta * (z^2 - 1)) + sum(w * (1-delta) * mills * z)
  grad[2] <- sum(w_status * (z^2 - 1)) + sum(w_censmills * z)

  # ===== Gradient w.r.t. covariate beta =====
  # dL/dbeta_j = (1/sigma) * sum(w * [delta * z + (1-delta) * mills] * x_ij)
  if (p > 2 && !is.null(x)) {
    score_i <- (w_status * z + w_censmills) / sigma
    grad[3:p] <- as.numeric(crossprod(x, score_i))
  }

  grad
}

#' @noRd
.hzr_optim_lognormal <- function(
    time, status, time_lower = NULL, time_upper = NULL,
    x = NULL, theta_start, weights = NULL, control = list()) {
  .hzr_optim_generic(
    logl_fn = .hzr_logl_lognormal,
    gradient_fn = .hzr_gradient_lognormal,
    time = time, status = status,
    time_lower = time_lower, time_upper = time_upper,
    x = x, theta_start = theta_start, weights = weights,
    control = control, use_bounds = FALSE
  )
}

.hzr_numeric_grad_lognormal <- function(theta, time, status,
                                          time_lower = NULL, time_upper = NULL,
                                          x = NULL, weights = NULL) {
  # Reuse log-likelihood as scalar objective and differentiate numerically.
  objective <- function(par) {
    .hzr_logl_lognormal(
      theta = par,
      time = time,
      status = status,
      time_lower = time_lower,
      time_upper = time_upper,
      x = x,
      weights = weights,
      return_gradient = FALSE
    )
  }

  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(as.numeric(numDeriv::grad(objective, theta)))
  }

  eps <- 1e-6
  grad <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tm <- theta
    tp[i] <- tp[i] + eps
    tm[i] <- tm[i] - eps
    grad[i] <- (objective(tp) - objective(tm)) / (2 * eps)
  }
  grad
}

## test-lognormal-dist.R — Unit tests for log-normal (AFT) parametric hazard model
##
## Coverage:
##   - Log-likelihood finite and correctly signed
##   - Analytical gradient matches numerical derivative (univariate + multivariate)
##   - Optimizer convergence (univariate + multivariate)
##   - predict() all four types: linear_predictor, hazard, survival, cumulative_hazard
##   - Survival monotone decreasing; cumhaz monotone increasing
##   - S(t) = exp(-H(t)) relationship
##   - Covariate direction of effect
##   - Mathematical identity: at true params, z ~ Normal(0,1)
##   - Boundary / edge cases (extreme sigma, all events, all censored)

library(testthat)

# ---------------------------------------------------------------------------
# Likelihood
# ---------------------------------------------------------------------------

test_that("log-normal likelihood is finite and negative (univariate)", {
  time   <- c(0.5, 1.0, 2.0, 3.0, 5.0)
  status <- c(1,   1,   0,   1,   0)
  theta  <- c(mu = 0.5, log_sigma = 0.0)  # mu=0.5, sigma=1

  logl <- .hzr_logl_lognormal(theta, time, status)

  expect_true(is.finite(logl))
  expect_true(logl < 0)
})

test_that("log-normal likelihood is finite with covariates (multivariate)", {
  set.seed(101)
  n      <- 25
  time   <- rlnorm(n, meanlog = 1, sdlog = 0.8)
  status <- rbinom(n, 1, 0.75)
  x      <- cbind(rnorm(n), rbinom(n, 1, 0.5))
  theta  <- c(mu = 1.0, log_sigma = log(0.8), beta1 = 0.2, beta2 = -0.3)

  logl <- .hzr_logl_lognormal(theta, time, status, x = x)

  expect_true(is.finite(logl))
  expect_true(logl < 0)
})

test_that("log-normal likelihood increases near the true parameters", {
  set.seed(202)
  n      <- 200
  mu_true    <- 1.2
  sigma_true <- 0.9
  time   <- rlnorm(n, meanlog = mu_true, sdlog = sigma_true)
  status <- rbinom(n, 1, 0.8)

  theta_true <- c(mu_true, log(sigma_true))
  theta_off  <- c(0.0, 0.0)

  logl_true <- .hzr_logl_lognormal(theta_true, time, status)
  logl_off  <- .hzr_logl_lognormal(theta_off,  time, status)

  # MLE should (in expectation) be better than a far-off value
  expect_true(logl_true > logl_off)
})

# ---------------------------------------------------------------------------
# Gradient — numerical verification
# ---------------------------------------------------------------------------

test_that("log-normal gradient matches numerical derivative (univariate)", {
  time   <- c(0.5, 1.0, 1.5, 2.0, 3.0)
  status <- c(1,   1,   0,   1,   0)
  theta  <- c(mu = 0.3, log_sigma = -0.2)
  eps    <- 1e-5

  logl_with_grad <- .hzr_logl_lognormal(
    theta,
    time,
    status,
    return_gradient = TRUE
  )
  grad_analytical <- attr(logl_with_grad, "gradient")

  grad_numerical <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tp[i] <- tp[i] + eps
    tm <- theta
    tm[i] <- tm[i] - eps
    grad_numerical[i] <- (
      .hzr_logl_lognormal(tp, time, status) -
      .hzr_logl_lognormal(tm, time, status)
    ) / (2 * eps)
  }

  expect_equal(grad_analytical, grad_numerical, tolerance = 1e-4)
})

test_that("log-normal gradient matches numerical derivative (multivariate)", {
  set.seed(303)
  time   <- c(0.4, 0.9, 1.4, 2.1, 3.0)
  status <- c(1,   0,   1,   1,   0)
  x      <- cbind(c(1, 0, 1, 0, 1), c(0.5, -0.5, 1.0, -1.0, 0.0))
  colnames(x) <- c("x1", "x2")
  theta  <- c(mu = 0.5, log_sigma = -0.1, beta1 = 0.3, beta2 = -0.2)
  eps    <- 1e-5

  logl_with_grad <- .hzr_logl_lognormal(
    theta,
    time,
    status,
    x = x,
    return_gradient = TRUE
  )
  grad_analytical <- attr(logl_with_grad, "gradient")

  grad_numerical <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tp[i] <- tp[i] + eps
    tm <- theta
    tm[i] <- tm[i] - eps
    grad_numerical[i] <- (
      .hzr_logl_lognormal(tp, time, status, x = x) -
      .hzr_logl_lognormal(tm, time, status, x = x)
    ) / (2 * eps)
  }

  expect_equal(grad_analytical, grad_numerical, tolerance = 1e-4)
})

# ---------------------------------------------------------------------------
# Optimizer
# ---------------------------------------------------------------------------

test_that("log-normal optimizer converges (univariate)", {
  set.seed(404)
  n          <- 60
  mu_true    <- 1.0
  sigma_true <- 0.8
  time       <- rlnorm(n, meanlog = mu_true, sdlog = sigma_true)
  cens       <- runif(n, 0, quantile(time, 0.8))
  status     <- as.integer(time <= cens)
  time       <- pmin(time, cens)

  fit <- .hzr_optim_lognormal(time, status, x = NULL,
                               theta_start = c(0.5, 0.0))

  expect_equal(fit$convergence, 0L, info = fit$message)
  expect_true(is.finite(fit$value))
  expect_equal(length(fit$par), 2L)

  # Recovered mu should be in a plausible neighbourhood
  expect_true(abs(fit$par[1] - mu_true) < 0.5)
  # Recovered sigma (on natural scale) should be positive and reasonable
  expect_true(exp(fit$par[2]) > 0.3 && exp(fit$par[2]) < 2.5)
})

test_that("log-normal optimizer converges (multivariate)", {
  set.seed(505)
  n <- 80
  mu_true    <- 0.8
  sigma_true <- 0.7
  beta_true  <- c(0.4, -0.3)

  x   <- cbind(rnorm(n), rnorm(n))
  eta <- mu_true + x %*% beta_true         # AFT: eta shifts log(T)
  time   <- rlnorm(n, meanlog = eta, sdlog = sigma_true)
  cens   <- runif(n, 0, quantile(time, 0.75))
  status <- as.integer(time <= cens)
  time   <- pmin(time, cens)

  fit <- .hzr_optim_lognormal(time, status, x = x,
                               theta_start = c(0.5, 0.0, 0.0, 0.0))

  expect_equal(fit$convergence, 0L, info = fit$message)
  expect_equal(length(fit$par), 4L)
  expect_true(is.finite(fit$value))
})

test_that("hazard() dispatches lognormal and converges", {
  set.seed(606)
  n      <- 60
  time   <- rlnorm(n, meanlog = 1, sdlog = 0.8)
  cens   <- runif(n, 0, quantile(time, 0.8))
  status <- as.integer(time <= cens)
  time   <- pmin(time, cens)

  fit <- hazard(time = time, status = status,
                theta = c(mu = 1.0, log_sigma = 0.0),
                dist = "lognormal", fit = TRUE)

  expect_s3_class(fit, "hazard")
  expect_true(fit$fit$converged)
  expect_true(is.finite(fit$fit$objective))
})

# ---------------------------------------------------------------------------
# predict() — all four types
# ---------------------------------------------------------------------------

.make_ln_obj <- function(theta, n = 5) {
  obj <- list(
    spec = list(dist = "lognormal"),
    fit  = list(theta = theta),
    data = list(time = seq(0.5, 2.5, length.out = n),
                status = rep(1L, n), x = NULL)
  )
  class(obj) <- "hazard"
  obj
}

test_that("predict() survival is in (0,1) and finite", {
  fit_obj <- .make_ln_obj(c(mu = 0.5, log_sigma = log(0.8)))
  time    <- seq(0.2, 4.0, by = 0.4)
  surv    <- predict(fit_obj, newdata = data.frame(time = time), type = "survival")

  expect_equal(length(surv), length(time))
  expect_true(all(surv > 0 & surv < 1))
  expect_true(all(is.finite(surv)))
})

test_that("predict() survival is monotone decreasing", {
  fit_obj <- .make_ln_obj(c(mu = 0.5, log_sigma = log(0.8)))
  time    <- seq(0.1, 5.0, by = 0.3)
  surv    <- predict(fit_obj, newdata = data.frame(time = time), type = "survival")

  expect_equal(surv, sort(surv, decreasing = TRUE), tolerance = 1e-10)
})

test_that("predict() cumulative_hazard is monotone increasing", {
  fit_obj <- .make_ln_obj(c(mu = 0.5, log_sigma = log(0.8)))
  time    <- seq(0.1, 5.0, by = 0.3)
  cumhaz  <- predict(fit_obj, newdata = data.frame(time = time),
                     type = "cumulative_hazard")

  expect_equal(cumhaz, sort(cumhaz), tolerance = 1e-10)
})

test_that("predict() survival = exp(-cumhaz) identity holds", {
  fit_obj <- .make_ln_obj(c(mu = 0.8, log_sigma = log(0.7)))
  time    <- c(0.5, 1.0, 1.5, 2.0, 3.0, 4.0)
  surv    <- predict(
    fit_obj,
    newdata = data.frame(time = time),
    type = "survival"
  )
  cumhaz  <- predict(
    fit_obj,
    newdata = data.frame(time = time),
    type = "cumulative_hazard"
  )

  expect_equal(surv, exp(-cumhaz), tolerance = 1e-10)
})

test_that("predict() linear_predictor returns beta dot x (AFT covariate effect)", {
  theta   <- c(mu = 1.0, log_sigma = 0.0, beta1 = 0.4, beta2 = -0.2)
  fit_obj <- list(
    spec = list(dist = "lognormal"),
    fit  = list(theta = theta),
    data = list(time = 1:3, status = rep(1L, 3),
                x = cbind(V1 = 1:3, V2 = 3:1))
  )
  class(fit_obj) <- "hazard"

  x_new  <- data.frame(V1 = c(1, 0, -1), V2 = c(0, 1, -1))
  lp     <- predict(fit_obj, newdata = x_new, type = "linear_predictor")

  expected <- as.numeric(as.matrix(x_new) %*% c(0.4, -0.2))
  expect_equal(lp, expected)
})

test_that("predict() hazard returns exp(linear_predictor)", {
  theta   <- c(mu = 1.0, log_sigma = 0.0, beta1 = 0.3, beta2 = -0.1)
  fit_obj <- list(
    spec = list(dist = "lognormal"),
    fit  = list(theta = theta),
    data = list(time = 1:3, status = rep(1L, 3),
                x = cbind(V1 = 1:3, V2 = 3:1))
  )
  class(fit_obj) <- "hazard"

  x_new <- data.frame(V1 = c(1, 2), V2 = c(-1, 0))
  h     <- predict(fit_obj, newdata = x_new, type = "hazard")
  lp    <- predict(fit_obj, newdata = x_new, type = "linear_predictor")

  expect_equal(h, exp(lp))
})

# ---------------------------------------------------------------------------
# Covariate effects
# ---------------------------------------------------------------------------

test_that("positive AFT covariate shifts survival upward", {
  # Larger x → larger eta → larger log(T) → later failure → higher survival
  theta   <- c(mu = 0.5, log_sigma = log(0.7), beta1 = 0.5)
  fit_obj <- list(
    spec = list(dist = "lognormal"),
    fit  = list(theta = theta),
    data = list(time = 1:3, status = rep(1L, 3),
                x = cbind(V1 = 1:3))
  )
  class(fit_obj) <- "hazard"

  t_eval <- c(1.0, 2.0, 3.0)
  s_high <- predict(fit_obj,
                    newdata = data.frame(time = t_eval, V1 = rep(2, 3)),
                    type = "survival")
  s_low  <- predict(fit_obj,
                    newdata = data.frame(time = t_eval, V1 = rep(-2, 3)),
                    type = "survival")

  # Higher covariate value → longer survival → higher S(t)
  expect_true(all(s_high > s_low))
})

# ---------------------------------------------------------------------------
# Mathematical identity: standardised residuals
# ---------------------------------------------------------------------------

test_that("standardised residuals are approximately N(0,1) at true parameters", {
  set.seed(707)
  n          <- 500
  mu_true    <- 1.3
  sigma_true <- 0.6
  time       <- rlnorm(n, meanlog = mu_true, sdlog = sigma_true)
  # No censoring so all z are observed
  status <- rep(1L, n)

  z <- (log(time) - mu_true) / sigma_true

  # Mean and SD of z should be close to 0 and 1
  expect_equal(mean(z), 0, tolerance = 0.15)
  expect_equal(sd(z), 1, tolerance = 0.15)
})

# ---------------------------------------------------------------------------
# Edge cases
# ---------------------------------------------------------------------------

test_that("log-normal likelihood handles extreme sigma gracefully", {
  time   <- c(0.5, 1.0, 2.0, 4.0)
  status <- c(1,   1,   0,   1)

  # Very small sigma (near-constant log-time)
  theta_tight <- c(0.0, log(0.01))
  logl_tight  <- .hzr_logl_lognormal(theta_tight, time, status)
  expect_true(is.finite(logl_tight))

  # Large sigma (very spread out)
  theta_wide <- c(0.0, log(5.0))
  logl_wide  <- .hzr_logl_lognormal(theta_wide, time, status)
  expect_true(is.finite(logl_wide))
})

test_that("log-normal optimizer produces finite parameters with high censoring", {
  set.seed(808)
  n      <- 40
  time   <- rlnorm(n, meanlog = 0.5, sdlog = 0.9)
  # ~60% censored
  cens   <- runif(n, 0, quantile(time, 0.5))
  status <- as.integer(time <= cens)
  time   <- pmin(time, cens)

  fit <- .hzr_optim_lognormal(time, status, x = NULL,
                               theta_start = c(0.0, 0.0))

  expect_true(all(is.finite(fit$par)))
  expect_true(is.finite(fit$value))
})

# candidate-score.R -- Unified criterion wrapper for stepwise selection.
#
# Step 8.2 of STEPWISE-DESIGN.md.  Wraps the Wald helper (Phase 4b sec.8.1)
# and an AIC path behind a single signature so the forward and backward
# drivers can share structure.
#
# Score semantics
# ---------------
# Lower `score` is always "better" in the candidate's favour:
#
#   criterion="wald", mode="entry"  ->  p_value(candidate for new coef)
#   criterion="wald", mode="drop"   ->  1 - p_value(current)  (high p -> easy drop)
#   criterion="aic",  mode="entry"  ->  AIC(candidate) - AIC(current)
#   criterion="aic",  mode="drop"   ->  DeltaAIC_drop ~ W - 2*df  (Wald->LR approx)
#
# For forward selection: argmin(score) picks the best candidate and the
# threshold test is `score < slentry` (Wald) or `score < 0` (AIC).
# For backward selection: argmin(score) picks the variable most worth
# dropping and the threshold test is `score < (1 - slstay)` (Wald) or
# `score < 0` (AIC).
#
# The list always carries both the raw `p_value` and `delta_aic` fields
# so the selection trace can show whichever is most informative
# regardless of the active criterion.

#' AIC for a fitted hazard model
#'
#' @param fit A fitted `hazard` object (`fit = TRUE`).
#' @return Numeric scalar AIC, or `NA_real_` if the log-likelihood is
#'   not available.
#'
#' @keywords internal
#' @noRd
.hzr_aic <- function(fit) {
  if (!inherits(fit, "hazard")) {
    stop("`fit` must be a `hazard` object.", call. = FALSE)
  }
  loglik <- fit$fit$objective
  if (is.null(loglik) || !is.finite(loglik)) {
    return(NA_real_)
  }

  # Count free parameters.  `fixed_mask` is TRUE for fixed entries
  # (see likelihood-multiphase.R: `fixed_mask <- !free_mask`), so free
  # parameters are the `!fixed_mask` count.  Fall back to the full
  # theta length when the mask is absent or length-mismatched.
  mask <- fit$fit$fixed_mask
  k <- if (!is.null(mask) && length(mask) == length(fit$fit$theta)) {
    sum(!as.logical(mask))
  } else {
    length(fit$fit$theta)
  }

  -2 * loglik + 2 * k
}


#' Score a stepwise candidate under Wald or AIC criterion
#'
#' Produces a unified "smaller is better" score for either adding new
#' coefficients to a model (`mode = "entry"`) or dropping coefficients
#' from the current model (`mode = "drop"`), under a Wald or AIC
#' criterion.
#'
#' @param criterion Either `"wald"` or `"aic"`.
#' @param mode Either `"entry"` or `"drop"`.
#' @param current Fitted `hazard` object that is the starting point for
#'   the step.  Always required (AIC baseline; Wald target for drops).
#' @param candidate Fitted `hazard` object containing the proposed new
#'   coefficient(s).  Required when `mode = "entry"`; ignored otherwise.
#' @param names Character vector of coefficient names being tested.
#'   Must exist in the target fit (`candidate` for entries, `current`
#'   for drops).
#'
#' @return A list with:
#' \describe{
#'   \item{score}{Numeric, smaller is better (see file header).}
#'   \item{criterion}{Echoed input.}
#'   \item{mode}{Echoed input.}
#'   \item{stat}{Wald z (scalar) or chi^2 (joint).}
#'   \item{df}{Integer degrees of freedom.}
#'   \item{p_value}{Always populated when computable.}
#'   \item{delta_aic}{Always populated when computable.}
#'   \item{names}{Echoed input.}
#' }
#'
#' @keywords internal
#' @noRd
.hzr_candidate_score <- function(criterion = c("wald", "aic"),
                                  mode      = c("entry", "drop"),
                                  current,
                                  candidate = NULL,
                                  names) {
  criterion <- match.arg(criterion)
  mode      <- match.arg(mode)

  if (!inherits(current, "hazard")) {
    stop("`current` must be a fitted `hazard` object.", call. = FALSE)
  }
  if (mode == "entry") {
    if (is.null(candidate) || !inherits(candidate, "hazard")) {
      stop("`candidate` must be a fitted `hazard` object when mode = 'entry'.",
           call. = FALSE)
    }
    test_fit <- candidate
  } else {
    test_fit <- current
  }

  wald <- .hzr_wald_p(test_fit, names)

  # Wald -> LR approximation:  W = z^2 when df = 1;
  # otherwise W is already the chi^2 statistic.
  W <- if (wald$df == 1L) {
    if (is.finite(wald$stat)) wald$stat^2 else NA_real_
  } else {
    if (is.finite(wald$stat)) wald$stat else NA_real_
  }

  # AIC components
  if (criterion == "aic" && mode == "entry") {
    aic_cur <- .hzr_aic(current)
    aic_can <- .hzr_aic(candidate)
    delta   <- if (is.finite(aic_cur) && is.finite(aic_can)) {
      aic_can - aic_cur
    } else {
      NA_real_
    }
  } else if (criterion == "aic" && mode == "drop") {
    delta <- if (is.finite(W)) W - 2 * wald$df else NA_real_
  } else if (mode == "drop") {
    # Wald + drop: compute DeltaAIC approx anyway so the trace can show it.
    delta <- if (is.finite(W)) W - 2 * wald$df else NA_real_
  } else {
    # Wald + entry: no DeltaAIC available without refitting;  leave as NA.
    delta <- NA_real_
  }

  score <- if (criterion == "wald") {
    if (mode == "entry") {
      wald$p_value           # smaller p = stronger entry case
    } else {
      if (is.na(wald$p_value)) NA_real_ else 1 - wald$p_value
    }                        # larger p = easier drop, so 1-p is small
  } else {
    delta                    # AIC: smaller DeltaAIC = better outcome
  }

  list(
    score     = score,
    criterion = criterion,
    mode      = mode,
    stat      = wald$stat,
    df        = wald$df,
    p_value   = wald$p_value,
    delta_aic = delta,
    names     = names
  )
}

# `.fit_weibull()` is defined in `helper-stepwise.R` and auto-sourced
# by testthat before any test-*.R files run.

# .hzr_aic -----------------------------------------------------------------

test_that(".hzr_aic matches -2*logLik + 2k for a single-distribution fit", {
  fit <- .fit_weibull()
  k <- length(fit$fit$theta)
  expected <- -2 * fit$fit$objective + 2 * k
  expect_equal(.hzr_aic(fit), expected)
})

test_that(".hzr_aic returns NA when logLik is not available", {
  fit <- .fit_weibull()
  fit$fit$objective <- NULL
  expect_true(is.na(.hzr_aic(fit)))
})

test_that(".hzr_aic counts only FREE parameters when fixed_mask is set", {
  # Multiphase fits expose fixed_mask where TRUE == FIXED (see
  # .hzr_optim_multiphase).  Simulate a fit with 8 theta entries of
  # which 3 are held fixed; k should be 5, not 8 (and not 3).
  fake <- structure(
    list(
      fit = list(
        theta      = setNames(seq_len(8), paste0("p", 1:8)),
        objective  = -100,
        fixed_mask = c(FALSE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE)
      )
    ),
    class = "hazard"
  )
  # 5 free params → AIC = -2*(-100) + 2*5 = 210
  expect_equal(.hzr_aic(fake), 210)
})

# Wald / entry -------------------------------------------------------------

test_that("Wald entry score equals candidate's p-value", {
  current <- hazard(
    time = rexp(200L, 0.5), status = rep(1L, 200L),
    theta = c(0.5, 1.0), dist = "weibull", fit = TRUE
  )
  candidate <- .fit_weibull(n = 200L, betas = 0.3, seed = 42L)

  covariate_row <- setdiff(
    rownames(summary(candidate)$coefficients), c("mu", "nu"))

  s <- .hzr_candidate_score(
    criterion = "wald", mode = "entry",
    current = current, candidate = candidate,
    names = covariate_row
  )

  wald <- .hzr_wald_p(candidate, covariate_row)
  expect_equal(s$score, wald$p_value)
  expect_equal(s$p_value, wald$p_value)
  expect_identical(s$criterion, "wald")
  expect_identical(s$mode, "entry")
  expect_true(is.na(s$delta_aic))   # no AIC produced in Wald-entry
})

# Wald / drop --------------------------------------------------------------

test_that("Wald drop score equals 1 - p_value (larger p => smaller score)", {
  fit <- .fit_weibull()
  covariate_row <- setdiff(
    rownames(summary(fit)$coefficients), c("mu", "nu"))

  s <- .hzr_candidate_score(
    criterion = "wald", mode = "drop",
    current = fit, names = covariate_row
  )

  wald <- .hzr_wald_p(fit, covariate_row)
  expect_equal(s$score, 1 - wald$p_value)
  expect_equal(s$p_value, wald$p_value)
  # ΔAIC is computed as a bonus for Wald-drop; check the formula
  expect_equal(s$delta_aic, wald$stat^2 - 2 * wald$df)
})

# AIC / entry --------------------------------------------------------------

test_that("AIC entry score equals AIC(candidate) - AIC(current)", {
  current <- hazard(
    time = rexp(200L, 0.5), status = rep(1L, 200L),
    theta = c(0.5, 1.0), dist = "weibull", fit = TRUE
  )
  candidate <- .fit_weibull(n = 200L, betas = 0.3, seed = 42L)
  covariate_row <- setdiff(
    rownames(summary(candidate)$coefficients), c("mu", "nu"))

  s <- .hzr_candidate_score(
    criterion = "aic", mode = "entry",
    current = current, candidate = candidate,
    names = covariate_row
  )

  expected_delta <- .hzr_aic(candidate) - .hzr_aic(current)
  expect_equal(s$score, expected_delta)
  expect_equal(s$delta_aic, expected_delta)
  expect_identical(s$criterion, "aic")
  # p_value is still populated for the candidate's new coef
  expect_true(is.finite(s$p_value))
})

# AIC / drop ---------------------------------------------------------------

test_that("AIC drop score uses Wald->LR approximation W - 2*df", {
  fit <- .fit_weibull()
  covariate_row <- setdiff(
    rownames(summary(fit)$coefficients), c("mu", "nu"))

  s <- .hzr_candidate_score(
    criterion = "aic", mode = "drop",
    current = fit, names = covariate_row
  )

  wald <- .hzr_wald_p(fit, covariate_row)
  expect_equal(s$score, wald$stat^2 - 2 * wald$df)
  expect_equal(s$delta_aic, s$score)
})

test_that("AIC drop: scalar case matches z^2 - 2 directly", {
  fit <- .fit_weibull(n = 500L, betas = 0.5)   # strong effect → large z
  covariate_row <- setdiff(
    rownames(summary(fit)$coefficients), c("mu", "nu"))
  wald <- .hzr_wald_p(fit, covariate_row)
  expect_equal(wald$df, 1L)

  s <- .hzr_candidate_score(
    criterion = "aic", mode = "drop",
    current = fit, names = covariate_row
  )
  expect_equal(s$score, wald$stat^2 - 2)
  # Strong effect should make dropping costly (score > 0)
  expect_gt(s$score, 0)
})

test_that("AIC drop: weak effect yields negative score (drop improves AIC)", {
  set.seed(123)
  n <- 300L
  x <- rnorm(n)
  # No effect: beta = 0
  time <- rexp(n, rate = 1)
  status <- rep(1L, n)
  fit <- hazard(
    time   = time,
    status = status,
    x      = matrix(x, ncol = 1L, dimnames = list(NULL, "x1")),
    theta  = c(0.5, 1.0, 0),
    dist   = "weibull",
    fit    = TRUE
  )

  covariate_row <- setdiff(
    rownames(summary(fit)$coefficients), c("mu", "nu"))

  s <- .hzr_candidate_score(
    criterion = "aic", mode = "drop",
    current = fit, names = covariate_row
  )
  # With no true effect and n = 300, z^2 << 2 on average; ΔAIC < 0.
  expect_lt(s$score, 0)
})

# Multi-df joint test ------------------------------------------------------

test_that("joint (df = 2) Wald drop uses chi^2 directly, not z^2", {
  fit <- .fit_weibull(n = 300L, betas = c(0.4, -0.3), seed = 7L)
  covariate_rows <- setdiff(
    rownames(summary(fit)$coefficients), c("mu", "nu"))
  expect_length(covariate_rows, 2L)

  s <- .hzr_candidate_score(
    criterion = "aic", mode = "drop",
    current = fit, names = covariate_rows
  )

  wald <- .hzr_wald_p(fit, covariate_rows)
  expect_equal(wald$df, 2L)
  # df > 1 path: W = stat (already chi^2), not stat^2
  expect_equal(s$score, wald$stat - 2 * 2)
})

# Error surface ------------------------------------------------------------

test_that("entry mode requires a candidate", {
  fit <- .fit_weibull()
  expect_error(
    .hzr_candidate_score(
      criterion = "wald", mode = "entry",
      current = fit, names = "beta1"
    ),
    "candidate"
  )
})

test_that("non-hazard current is rejected", {
  expect_error(
    .hzr_candidate_score(
      criterion = "wald", mode = "drop",
      current = list(), names = "x1"
    ),
    "`current` must be a fitted `hazard` object"
  )
})

test_that("unknown names propagate the Wald helper's error", {
  fit <- .fit_weibull()
  expect_error(
    .hzr_candidate_score(
      criterion = "wald", mode = "drop",
      current = fit, names = "nonexistent"
    ),
    "Unknown coefficient name"
  )
})

# NA propagation -----------------------------------------------------------

test_that("non-invertible vcov yields NA score without erroring", {
  fit <- .fit_weibull()
  fit$fit$vcov <- NULL    # simulate non-invertible Hessian
  covariate_row <- setdiff(
    rownames(summary(fit)$coefficients), c("mu", "nu"))

  s <- .hzr_candidate_score(
    criterion = "wald", mode = "drop",
    current = fit, names = covariate_row
  )
  expect_true(is.na(s$score))
  expect_true(is.na(s$p_value))
  expect_true(is.na(s$delta_aic))
})

#' @keywords internal
NULL

# hessian-invert.R -- Numerically stable Hessian inversion with diagnostics.

# Reciprocal-condition warning threshold (~1.5e-8). Shared by .hzr_safe_solve()
# and the summary() diagnostic note so they never drift apart.
.hzr_rcond_tol <- .Machine$double.eps^0.5

#' Stable Hessian inversion with conditioning diagnostics
#'
#' Inverts a negative-log-likelihood Hessian into a variance-covariance
#' matrix, hardening against the ill-conditioning that arises at high
#' parameter counts.  Symmetrizes the input, checks the reciprocal condition
#' number, inverts via Cholesky (with a \code{solve()} fallback for non-PD
#' Hessians), and guards non-positive variances.  Emits a named warning for
#' every degenerate path.
#'
#' @param H Square numeric Hessian of the negative log-likelihood.
#' @param tol Reciprocal-condition warning threshold.
#' @return A list with:
#'   \code{vcov} (the variance-covariance matrix, or \code{NA} on failure;
#'   diagonals with non-positive variance, and their rows/cols, set to
#'   \code{NA}); \code{rcond} (reciprocal condition number of the symmetrized
#'   Hessian, \code{NA} if unavailable); \code{pd} (\code{TRUE} if the Hessian
#'   was positive-definite, \code{FALSE} if inverted via fallback, \code{NA}
#'   if not invertible).
#' @noRd
.hzr_safe_solve <- function(H, tol = .hzr_rcond_tol) {
  # (1) Non-finite / non-matrix guard
  if (is.null(H) || !is.matrix(H) || anyNA(H) || any(!is.finite(H))) {
    warning("Hessian contains non-finite entries; standard errors unavailable")
    return(list(vcov = NA, rcond = NA_real_, pd = NA))
  }

  # (2) Symmetrize (numDeriv Hessians are only symmetric to Richardson tol)
  H <- (H + t(H)) / 2

  # (3) Conditioning check
  rc <- tryCatch(rcond(H), error = function(e) NA_real_)
  if (is.na(rc)) {
    warning("Hessian conditioning could not be assessed; standard errors may be unreliable")
  } else if (rc < tol) {
    warning(sprintf(
      "Hessian is ill-conditioned (rcond = %.3g); standard errors may be unreliable",
      rc
    ))
  }

  # (4) Stable inversion: Cholesky (PD) with solve() fallback (non-PD)
  ch <- tryCatch(chol(H), error = function(e) NULL)
  if (!is.null(ch)) {
    pd <- TRUE
    vcov <- chol2inv(ch)
  } else {
    pd <- FALSE
    vcov <- tryCatch(solve(H), error = function(e) NULL)
    if (is.null(vcov)) {
      warning("Hessian not invertible; standard errors unavailable")
      return(list(vcov = NA, rcond = rc, pd = NA))
    }
    warning("Hessian is not positive-definite at the optimum; standard errors may be unreliable")
  }
  dimnames(vcov) <- dimnames(H)  # chol2inv() drops names

  # (5) Non-positive-variance guard
  d <- diag(vcov)
  bad <- !is.finite(d) | d <= 0
  if (any(bad)) {
    warning("Non-positive variance estimates; the optimum may not be a proper maximum")
    vcov[bad, ] <- NA_real_
    vcov[, bad] <- NA_real_
  }

  list(vcov = vcov, rcond = rc, pd = pd)
}

# Shared fixtures for Wald / candidate-score / stepwise tests.  testthat
# auto-sources any helper-*.R file before running test-*.R files.

.fit_weibull <- function(n = 200L, betas = 0.3, seed = 42L) {
  set.seed(seed)
  X <- matrix(rnorm(n * length(betas)), ncol = length(betas))
  colnames(X) <- paste0("x", seq_len(ncol(X)))
  lp <- as.numeric(X %*% betas)
  time <- rexp(n, rate = exp(lp))
  status <- rep(1L, n)
  hazard(
    time   = time,
    status = status,
    x      = X,
    theta  = c(0.5, 1.0, rep(0, ncol(X))),
    dist   = "weibull",
    fit    = TRUE
  )
}

# Used by the driver and integration tests: intercept-only Weibull
# base fit with a strong signal (x1) plus three noise covariates in
# the data frame, ready for forward selection.
.fit_driver_base <- function(n = 500L, seed = 911L, signal_beta = 0.9) {
  set.seed(seed)
  df <- data.frame(
    time   = rexp(n, rate = 1),
    status = rep(1L, n),
    x1     = rnorm(n),
    x2     = rnorm(n),
    x3     = rnorm(n),
    x4     = rnorm(n)
  )
  df$time <- df$time * exp(-signal_beta * df$x1)

  fit <- hazard(
    Surv(time, status) ~ 1,
    data = df,
    theta = c(0.5, 1.0),
    dist = "weibull",
    fit = TRUE
  )
  list(fit = fit, data = df)
}

# Overfitted Weibull including three covariates (one signal, two
# noise) — the starting point for backward-selection tests.
.fit_overfitted <- function(n = 400L, seed = 101L, signal_beta = 0.8) {
  set.seed(seed)
  df <- data.frame(
    time   = rexp(n, rate = 1),
    status = rep(1L, n),
    x1     = rnorm(n),
    x2     = rnorm(n),
    x3     = rnorm(n)
  )
  df$time <- df$time * exp(-signal_beta * df$x1)

  fit <- hazard(
    Surv(time, status) ~ x1 + x2 + x3,
    data = df,
    theta = c(0.5, 1.0, 0, 0, 0),
    dist = "weibull",
    fit = TRUE
  )
  list(fit = fit, data = df)
}

# Intercept-only Weibull used by the forward-step unit tests.
.fit_stepwise_base <- function(n = 400L, seed = 101L, signal_beta = 0.8) {
  set.seed(seed)
  df <- data.frame(
    time   = rexp(n, rate = 1),
    status = rep(1L, n),
    x1     = rnorm(n),
    x2     = rnorm(n),
    x3     = rnorm(n)
  )
  df$time <- df$time * exp(-signal_beta * df$x1)

  fit <- hazard(
    Surv(time, status) ~ 1,
    data = df,
    theta = c(0.5, 1.0),
    dist = "weibull",
    fit = TRUE
  )
  list(fit = fit, data = df)
}

# likelihood-multiphase.R -- Multiphase additive hazard likelihood and optimizer
#
# REFERENCES
# ----------
# Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
# hazard into phases, each incorporating a separate stream of concomitant
# information. J Am Stat Assoc. 1986;81(395):615-624.
#
# Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
# Probability of atrial fibrillation after ablation: Using a parametric
# nonlinear temporal decomposition mixed effects model. Stat Methods Med Res.
# 2018;27(1):126-141.
#
# MODEL
# -----
# Additive cumulative hazard across J phases:
#
#   H(t | x) = sum_{j=1}^{J}  mu_j(x) * Phi_j(t; t_half_j, nu_j, m_j)
#
# where:
#   mu_j(x) = exp(alpha_j + x_j * beta_j)  -- phase-specific log-linear scale
#   Phi_j(t) depends on phase type:
#     "cdf"      -> G(t)             bounded [0, 1]   (early risk)
#     "hazard"   -> -log(1 - G(t))   monotone incr.   (late risk)
#     "constant" -> t                flat rate         (background)
#
# Total hazard:
#   h(t | x) = sum_j  mu_j(x) * phi_j(t)
#
# where phi_j = dPhi_j/dt.
#
# Survival:
#   S(t | x) = exp(-H(t | x))
#
# THETA LAYOUT (internal / estimation scale)
# ------------------------------------------
# For each phase j, the sub-vector is:
#   [log_mu_j, log_t_half_j, nu_j, m_j, beta_j_1, ..., beta_j_pj]
#   (constant phases omit log_t_half, nu, m)
#
# The full theta is the concatenation of all phase sub-vectors.
#
# FUNCTIONS
# ---------
#   .hzr_logl_multiphase()     -- log-likelihood
#   .hzr_optim_multiphase()    -- optimizer using .hzr_optim_generic()
#   .hzr_multiphase_cumhaz()   -- compute total H(t|x) from theta + phases
#   .hzr_multiphase_hazard()   -- compute total h(t|x) from theta + phases
#   .hzr_split_theta()         -- split theta into per-phase sub-vectors


# ============================================================================
# Theta vector manipulation
# ============================================================================

#' Split the full theta vector into per-phase sub-vectors
#'
#' @param theta Numeric vector -- full parameter vector (internal scale).
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector -- number of covariates per phase.
#' @return Named list of numeric vectors, one per phase.
#' @keywords internal
.hzr_split_theta <- function(theta, phases, covariate_counts) {
  result <- vector("list", length(phases))
  names(result) <- names(phases)
  pos <- 1L

  for (nm in names(phases)) {
    np <- .hzr_phase_n_params(phases[[nm]], n_covariates = covariate_counts[[nm]])
    result[[nm]] <- theta[pos:(pos + np - 1L)]
    pos <- pos + np
  }

  result
}


#' Extract parameters from a phase sub-vector (internal scale)
#'
#' @param theta_j Numeric sub-vector for one phase.
#' @param phase An `hzr_phase` object.
#' @return Named list with `log_mu`, `log_t_half`, `nu`, `m`, `beta`.
#' @keywords internal
.hzr_unpack_phase_theta <- function(theta_j, phase) {
  pos <- 1L
  log_mu <- theta_j[pos]
  pos <- pos + 1L

  if (phase$type == "g3") {
    log_tau <- theta_j[pos]
    pos <- pos + 1L
    gamma_  <- theta_j[pos]
    pos <- pos + 1L
    alpha_  <- theta_j[pos]
    pos <- pos + 1L
    eta_    <- theta_j[pos]
    pos <- pos + 1L
    log_t_half <- NA_real_
    nu <- NA_real_
    m <- NA_real_
  } else if (phase$type != "constant") {
    log_t_half <- theta_j[pos]
    pos <- pos + 1L
    nu <- theta_j[pos]
    pos <- pos + 1L
    m <- theta_j[pos]
    pos <- pos + 1L
    log_tau <- NA_real_
    gamma_ <- NA_real_
    alpha_ <- NA_real_
    eta_ <- NA_real_
  } else {
    log_t_half <- NA_real_
    nu <- NA_real_
    m <- NA_real_
    log_tau <- NA_real_
    gamma_ <- NA_real_
    alpha_ <- NA_real_
    eta_ <- NA_real_
  }

  beta <- if (pos <= length(theta_j)) theta_j[pos:length(theta_j)] else numeric(0)

  list(log_mu = log_mu, log_t_half = log_t_half, nu = nu, m = m,
       log_tau = log_tau, gamma = gamma_, alpha = alpha_, eta = eta_,
       beta = beta)
}


# ============================================================================
# Cumulative hazard and hazard computations
# ============================================================================

#' Compute total cumulative hazard H(t|x) for multiphase model
#'
#' @param time Numeric vector of times (n).
#' @param theta Full parameter vector (internal scale).
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of covariate counts per phase.
#' @param x_list Named list of design matrices, one per phase. Each element is
#'   an n x p_j matrix or NULL.
#' @param per_phase Logical; if TRUE return a list with total and per-phase
#'   contributions.
#' @return If `per_phase = FALSE`: numeric vector of length n (total H(t|x)).
#'   If `per_phase = TRUE`: named list with `$total` and one element per phase.
#' @keywords internal
.hzr_multiphase_cumhaz <- function(time, theta, phases, covariate_counts,
                                    x_list, per_phase = FALSE) {
  n <- length(time)
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  total <- rep(0, n)
  phase_contributions <- if (per_phase) vector("list", length(phases)) else NULL

  for (i in seq_along(phases)) {
    nm <- names(phases)[i]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    # mu_j(x) = exp(log_mu + x_j * beta_j)
    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      eta_j <- pars$log_mu + as.numeric(x_list[[nm]] %*% pars$beta)
    } else {
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)

    # Phi_j(t)
    if (phases[[nm]]$type == "constant") {
      phi_j <- time
    } else if (phases[[nm]]$type == "g3") {
      tau_j <- exp(pars$log_tau)
      d3 <- hzr_decompos_g3(time, tau = tau_j, gamma = pars$gamma,
                              alpha = pars$alpha, eta = pars$eta)
      phi_j <- d3$G3
    } else {
      t_half_j <- exp(pars$log_t_half)
      # Guard against invalid decomposition parameters
      if (pars$m < 0 && pars$nu < 0) return(rep(Inf, n))
      phi_j <- hzr_phase_cumhaz(time, t_half = t_half_j, nu = pars$nu,
                                  m = pars$m, type = phases[[nm]]$type)
    }

    contrib <- mu_j * phi_j
    total <- total + contrib

    if (per_phase) phase_contributions[[i]] <- contrib
  }

  if (per_phase) {
    names(phase_contributions) <- names(phases)
    phase_contributions$total <- total
    return(phase_contributions)
  }

  total
}


#' Compute total instantaneous hazard h(t|x) for multiphase model
#'
#' @inheritParams .hzr_multiphase_cumhaz
#' @return Numeric vector of length n.
#' @keywords internal
.hzr_multiphase_hazard <- function(time, theta, phases, covariate_counts,
                                    x_list) {
  n <- length(time)
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  total <- rep(0, n)

  for (i in seq_along(phases)) {
    nm <- names(phases)[i]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      eta_j <- pars$log_mu + as.numeric(x_list[[nm]] %*% pars$beta)
    } else {
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)

    if (phases[[nm]]$type == "constant") {
      dphi_j <- rep(1, n)
    } else if (phases[[nm]]$type == "g3") {
      tau_j <- exp(pars$log_tau)
      d3 <- hzr_decompos_g3(time, tau = tau_j, gamma = pars$gamma,
                              alpha = pars$alpha, eta = pars$eta)
      dphi_j <- d3$g3
    } else {
      t_half_j <- exp(pars$log_t_half)
      if (pars$m < 0 && pars$nu < 0) return(rep(Inf, n))
      dphi_j <- hzr_phase_hazard(time, t_half = t_half_j, nu = pars$nu,
                                   m = pars$m, type = phases[[nm]]$type)
    }

    total <- total + mu_j * dphi_j
  }

  total
}


# ============================================================================
# Conservation of Events (CoE) -- Turner's theorem
# ============================================================================
#
# For the additive hazard model H(t|x) = Sigma_j mu_j(x) * Phi_j(t),
# the MLE constraint (score equation for mu) implies:
#
#   Sigma_i E(i) = Sigma_i H(t_i | x_i)
#
# i.e., total observed events equals total predicted cumulative hazard.
#
# This allows one phase's log_mu to be solved analytically at each
# optimizer iteration, reducing the optimization dimension by 1.
#
# Reference: Turner ME Jr., as implemented in C HAZARD setcoe.c / consrv.c.

#' Find the position of each phase's log_mu in the full theta vector
#'
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of per-phase covariate counts.
#' @return Named integer vector: position of log_mu for each phase.
#' @keywords internal
.hzr_log_mu_positions <- function(phases, covariate_counts) {
  positions <- setNames(integer(length(phases)), names(phases))
  pos <- 1L
  for (nm in names(phases)) {
    positions[[nm]] <- pos
    np <- .hzr_phase_n_params(phases[[nm]],
                               n_covariates = covariate_counts[[nm]])
    pos <- pos + np
  }
  positions
}


#' Select the phase whose log_mu will be solved by conservation
#'
#' Chooses the phase contributing the largest share of total cumulative
#' hazard at the current theta, matching the C HAZARD SETCOE strategy.
#'
#' When one phase dominates by an extreme margin (>10x the median contribution)
#' it is typically due to poorly-scaled shape parameters at the starting theta
#' rather than a genuine signal that the phase should absorb all events.
#' Selecting such a phase as the fixmu phase traps the optimizer: CoE keeps
#' rescaling that phase's log_mu upward while the optimizer tries to drive it
#' to near-zero.  We therefore exclude extreme outliers and select among the
#' remaining phases.  If all phases are outliers (or only one phase exists)
#' the largest is used as a fallback.
#'
#' @param theta Full parameter vector (internal scale).
#' @param time Numeric vector of follow-up times.
#' @param status Numeric event indicator.
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector.
#' @param x_list Named list of per-phase design matrices.
#' @param weights Optional numeric vector of row weights (length n). Defaults to
#'   unit weights. Applied when summing per-phase cumhaz so that selection
#'   happens on the same scale as the (weighted) observed event count.
#' @param time_lower Optional numeric vector of counting-process entry (start)
#'   times. When supplied, phases are ranked by entry-time cumulative hazard
#'   `H(stop) - H(start)`, the scale on which events are conserved. `NULL` (the
#'   default) means no truncation, i.e. `H(start) = 0`.
#' @return Character: name of the phase to fix.
#' @keywords internal
.hzr_select_fixmu_phase <- function(theta, time, status,
                                     phases, covariate_counts, x_list,
                                     weights = NULL, time_lower = NULL) {
  decomp <- .hzr_multiphase_cumhaz(time, theta, phases,
                                     covariate_counts, x_list,
                                     per_phase = TRUE)
  # Left truncation (counting-process entry times): rank phases by the cumulative
  # hazard accrued over follow-up, H(stop) - H(start), the same scale on which
  # events are conserved. See `.hzr_conserve_events`.
  if (!is.null(time_lower)) {
    decomp_start <- .hzr_multiphase_cumhaz(time_lower, theta, phases,
                                            covariate_counts, x_list,
                                            per_phase = TRUE)
    for (nm in names(phases)) {
      decomp[[nm]] <- decomp[[nm]] - decomp_start[[nm]]
    }
  }
  if (is.null(weights)) weights <- rep(1, length(time))
  phase_sums <- vapply(names(phases), function(nm) {
    sum(weights * decomp[[nm]])
  }, numeric(1))

  # Exclude extreme outliers: a phase whose contribution exceeds 10x the
  # median is likely dominated by large shape-parameter values at the starting
  # theta rather than reflecting the true relative importance of the phase.
  # Fixing such a phase traps the optimizer when the true MLE has that phase
  # near zero.  Among the remaining (non-outlier) phases, pick the largest.
  if (length(phase_sums) > 1L) {
    finite_sums <- phase_sums[is.finite(phase_sums)]
    if (length(finite_sums) >= 1L) {
      med <- stats::median(finite_sums, na.rm = TRUE)
      if (is.finite(med) && med > 0) {
        non_outlier <- phase_sums[is.finite(phase_sums) & phase_sums <= 10 * med]
        if (length(non_outlier) >= 1L) {
          return(names(which.max(non_outlier)))
        }
      }
    }
  }

  names(which.max(phase_sums))
}


#' Apply the Conservation of Events adjustment to one phase's log_mu
#'
#' Given the current theta vector, analytically solve the fixmu phase's
#' log_mu so that total predicted events = total observed events.
#'
#' This is called BEFORE each likelihood evaluation inside the optimizer,
#' matching the C HAZARD `CONSRV` entry point.
#'
#' @param theta Full parameter vector (internal scale).
#' @param fixmu_phase Character: name of the phase whose log_mu is solved.
#' @param fixmu_pos Integer: position of that log_mu in theta.
#' @param time Numeric vector of follow-up times.
#' @param status Numeric event indicator.
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector.
#' @param x_list Named list of per-phase design matrices.
#' @param total_events Numeric: sum of observed events (precomputed, weighted
#'   when `weights` is supplied).
#' @param weights Optional numeric vector of row weights (length n). Defaults to
#'   unit weights. Applied when summing per-phase cumhaz so Turner's adjustment
#'   is computed on the same scale as `total_events`.
#' @param time_lower Optional numeric vector of counting-process entry (start)
#'   times. When supplied, conservation is enforced on the entry-time scale --
#'   `Sum E = Sum [H(stop) - H(start)]` -- by subtracting the entry-time
#'   cumulative hazard, matching the multiphase likelihood (and C HAZARD
#'   `setcoe` under `LCENSOR`/`STARTTME`). `NULL` (the default) means no
#'   truncation, i.e. `H(start) = 0`.
#' @return Updated theta vector with fixmu phase's log_mu adjusted.
#' @keywords internal
.hzr_conserve_events <- function(theta, fixmu_phase, fixmu_pos,
                                  time, status,
                                  phases, covariate_counts, x_list,
                                  total_events, weights = NULL,
                                  time_lower = NULL) {
  # Compute per-phase cumulative hazard contributions
  decomp <- .hzr_multiphase_cumhaz(time, theta, phases,
                                     covariate_counts, x_list,
                                     per_phase = TRUE)

  # Left truncation: subtract the per-phase entry-time cumulative hazard so the
  # conserved quantity is H(stop) - H(start), the same constraint the score
  # equation for mu satisfies. Without this, CoE conserves Sum H(stop) and the
  # fixmu phase absorbs the spurious Sum H(start), biasing its intercept.
  if (!is.null(time_lower)) {
    decomp_start <- .hzr_multiphase_cumhaz(time_lower, theta, phases,
                                            covariate_counts, x_list,
                                            per_phase = TRUE)
    decomp$total <- decomp$total - decomp_start$total
    decomp[[fixmu_phase]] <- decomp[[fixmu_phase]] - decomp_start[[fixmu_phase]]
  }

  if (is.null(weights)) weights <- rep(1, length(time))

  # Total predicted events (weighted sum of cumhaz across all observations)
  sumcz <- sum(weights * decomp$total)

  # Weighted contribution from the fixmu phase alone
  sumcj <- sum(weights * decomp[[fixmu_phase]])

  if (sumcj <= 0 || !is.finite(sumcj)) return(theta)

  # Discrepancy: how many events are unaccounted for
  devent <- total_events - sumcz

  # Events the fixmu phase should absorb
  jevent <- sumcj + devent

  if (jevent <= 0 || !is.finite(jevent)) return(theta)

  # Multiplicative adjustment in log scale
  lfactor <- log(jevent / sumcj)

  if (!is.finite(lfactor)) return(theta)

  theta[fixmu_pos] <- theta[fixmu_pos] + lfactor
  theta
}


# ============================================================================
# Log-likelihood
# ============================================================================

#' Log-likelihood for multiphase additive hazard model
#'
#' @param theta Full parameter vector (internal scale).
#' @param time Numeric vector of follow-up times (n).
#' @param status Numeric event indicator: 1 = event, 0 = right-censored,
#'   -1 = left-censored, 2 = interval-censored.
#' @param time_lower Optional lower bounds for interval censoring.
#' @param time_upper Optional upper bounds for left/interval censoring.
#' @param x Design matrix (unused directly; kept for interface compatibility).
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of per-phase covariate counts.
#' @param x_list Named list of per-phase design matrices.
#' @param return_gradient Logical (ignored; gradient via separate function).
#' @param return_hessian Logical (ignored).
#' @param ... Ignored.
#' @return Scalar log-likelihood. Returns -Inf for infeasible parameters.
#' @keywords internal
.hzr_logl_multiphase <- function(theta, time, status,
                                  time_lower = NULL, time_upper = NULL,
                                  x = NULL, weights = NULL,
                                  phases, covariate_counts, x_list,
                                  return_gradient = FALSE,
                                  return_hessian = FALSE, ...) {

  n <- length(time)
  if (is.null(weights)) weights <- rep(1, n)

  # Feasibility: check parameter constraints

  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  for (nm in names(phases)) {
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])
    if (phases[[nm]]$type %in% c("cdf", "hazard")) {
      if (pars$m < 0 && pars$nu < 0) return(-Inf)
    }
    if (phases[[nm]]$type == "g3") {
      # G3 requires gamma > 0, alpha >= 0, eta > 0
      if (pars$gamma <= 0 || pars$eta <= 0 || pars$alpha < 0) return(-Inf)
    }
  }

  # Validate event times
  if (any(status == 1 & time <= 0)) return(-Inf)

  # Normalize censoring bounds
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  # Cumulative hazard at event/censoring times
  cumhaz <- .hzr_multiphase_cumhaz(time, theta, phases, covariate_counts, x_list)
  if (any(!is.finite(cumhaz))) return(-Inf)

  # Counting-process entry-time cumulative hazard; H(start) = 0 when no
  # `time_lower` supplied (plain right-censored data).  For status in
  # {0, 1}, the contribution becomes H(stop) - H(start); for status == 2,
  # this value is unused (interval-censoring handled separately below).
  if (is.null(time_lower)) {
    cumhaz_start <- rep(0, n)
  } else {
    cumhaz_start <- .hzr_multiphase_cumhaz(time_lower, theta, phases,
                                             covariate_counts, x_list)
    if (any(!is.finite(cumhaz_start))) return(-Inf)
  }

  # Instantaneous hazard at event times (only needed for exact events)
  idx_event <- status == 1

  logl <- 0

  # Exact events: w * [log h(stop) - (H(stop) - H(start))]
  if (any(idx_event)) {
    haz <- .hzr_multiphase_hazard(time, theta, phases, covariate_counts, x_list)
    if (any(!is.finite(haz[idx_event])) || any(haz[idx_event] <= 0)) return(-Inf)
    logl <- logl + sum(weights[idx_event] *
                         (log(haz[idx_event]) -
                            (cumhaz[idx_event] - cumhaz_start[idx_event])))
  }

  # Right-censored: w * [-(H(stop) - H(start))]
  idx_right <- status == 0
  if (any(idx_right)) {
    logl <- logl - sum(weights[idx_right] *
                         (cumhaz[idx_right] - cumhaz_start[idx_right]))
  }

  # Left-censored: w * [log(1 - exp(-H(u)))]
  idx_left <- status == -1
  if (any(idx_left)) {
    cumhaz_upper <- .hzr_multiphase_cumhaz(upper, theta, phases,
                                             covariate_counts, x_list)
    logl <- logl + sum(weights[idx_left] *
                         hzr_log1mexp(cumhaz_upper[idx_left]))
  }

  # Interval-censored: w * [-H(l) + log(1 - exp(-(H(u) - H(l))))]
  idx_interval <- status == 2
  if (any(idx_interval)) {
    cumhaz_lower <- .hzr_multiphase_cumhaz(lower, theta, phases,
                                             covariate_counts, x_list)
    cumhaz_upper_iv <- .hzr_multiphase_cumhaz(upper, theta, phases,
                                                covariate_counts, x_list)
    delta_h <- cumhaz_upper_iv[idx_interval] - cumhaz_lower[idx_interval]
    logl <- logl + sum(weights[idx_interval] *
                         (-cumhaz_lower[idx_interval] +
                            hzr_log1mexp(delta_h)))
  }

  if (!is.finite(logl)) return(-Inf)

  logl
}


# ============================================================================
# Analytic gradient
# ============================================================================

#' Gradient of the multiphase log-likelihood
#'
#' Computes the score vector \eqn{d\ell / d\theta} using analytic chain-rule
#' formulas for `log_mu` and `beta` parameters, and central-difference
#' derivatives (via `.hzr_phase_derivatives()`) for shape parameters
#' (`log_t_half`, `nu`, `m`).
#'
#' @inheritParams .hzr_logl_multiphase
#' @return Numeric vector of length `length(theta)` -- the gradient.
#'   Returns a zero vector if any component is non-finite (guards optimizer).
#' @keywords internal
.hzr_gradient_multiphase <- function(theta, time, status,
                                      time_lower = NULL, time_upper = NULL,
                                      x = NULL, weights = NULL,
                                      phases, covariate_counts, x_list,
                                      ...) {
  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)
  if (is.null(weights)) weights <- rep(1, n)

  # Feasibility check
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  for (nm in names(phases)) {
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])
    if (phases[[nm]]$type %in% c("cdf", "hazard")) {
      if (pars$m < 0 && pars$nu < 0) return(grad)
    }
    if (phases[[nm]]$type == "g3") {
      if (pars$gamma <= 0 || pars$eta <= 0 || pars$alpha < 0) return(grad)
    }
  }

  # Normalize censoring bounds
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  # Observation type masks
  idx_event    <- which(status == 1)
  idx_right    <- which(status == 0)
  idx_left     <- which(status == -1)

  idx_interval <- which(status == 2)

  has_left     <- length(idx_left) > 0
  has_interval <- length(idx_interval) > 0

  # -- Per-phase quantities --------------------------------------------------
  # We need, for each phase j and each observation i:
  #   mu_j(x_i), Phi_j(t_i), phi_j(t_i), and shape derivatives of Phi and phi
  #
  # Then the per-observation gradient contributions are assembled from:
  #   H(t) = sum_j mu_j * Phi_j    (cumulative hazard)
  #   h(t) = sum_j mu_j * phi_j    (instantaneous hazard)

  # Pre-compute total H(t) and h(t) at observation times (and censoring bounds)
  H_t <- rep(0, n)      # total cumulative hazard at time[i]
  h_t <- rep(0, n)      # total instantaneous hazard at time[i]

  # Storage for per-phase intermediate quantities
  n_phases <- length(phases)
  phase_mu         <- vector("list", n_phases)  # mu_j(x_i) for each i
  phase_Phi        <- vector("list", n_phases)  # Phi_j(t_i) for each i
  phase_phi        <- vector("list", n_phases)  # phi_j(t_i) for each i
  phase_deriv      <- vector("list", n_phases)  # derivative list at time
  phase_Phi_start  <- vector("list", n_phases)  # Phi_j(start_i), if needed
  phase_deriv_start <- vector("list", n_phases) # derivative list at start
  # Whether to actually compute start-time quantities for the gradient.
  # Without any counting-process rows (time_lower = NULL or all zeros with
  # status in {0, 1}) these additions are identically zero and we can skip.
  # Only genuine epoch rows (status 0/1 with time_lower < time) get a
  # non-zero start.  Interval-censored rows (status 2) are handled through
  # their own lower/upper cumhaz terms and must not contribute to H(start).
  need_start <- !is.null(time_lower) &&
                any(time_lower > 0 & time_lower < time & status %in% c(0L, 1L))
  start_vec <- if (need_start) {
    sv <- rep(0, n)
    epoch_idx <- status %in% c(0L, 1L) & time_lower < time
    sv[epoch_idx] <- time_lower[epoch_idx]
    sv
  } else {
    NULL
  }

  for (j in seq_along(phases)) {
    nm <- names(phases)[j]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    # mu_j(x_i) = exp(log_mu + x * beta)
    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      eta_j <- pars$log_mu + as.numeric(x_list[[nm]] %*% pars$beta)
    } else {
      eta_j <- rep(pars$log_mu, n)
    }
    mu_j <- exp(eta_j)
    phase_mu[[j]] <- mu_j

    # Phi_j, phi_j, and shape derivatives at observation time
    if (phases[[nm]]$type == "constant") {
      phase_Phi[[j]] <- time
      phase_phi[[j]] <- rep(1, n)
      phase_deriv[[j]] <- NULL  # no shape params for constant
    } else if (phases[[nm]]$type == "g3") {
      tau_j <- exp(pars$log_tau)
      d3 <- hzr_decompos_g3(time, tau = tau_j, gamma = pars$gamma,
                              alpha = pars$alpha, eta = pars$eta)
      phase_Phi[[j]] <- d3$G3
      phase_phi[[j]] <- d3$g3
      # G3 shape derivatives via finite differences (for now)
      phase_deriv[[j]] <- .hzr_g3_phase_derivatives(
        time, tau = tau_j, gamma = pars$gamma,
        alpha = pars$alpha, eta = pars$eta)
    } else {
      t_half_j <- exp(pars$log_t_half)
      pd <- .hzr_phase_derivatives(time, t_half = t_half_j, nu = pars$nu,
                                    m = pars$m, type = phases[[nm]]$type)
      phase_Phi[[j]]   <- pd$Phi
      phase_phi[[j]]   <- pd$phi
      phase_deriv[[j]] <- pd
    }

    # Matching quantities at start_vec -- only Phi_j and its shape
    # derivatives matter for the H(start) gradient contribution.
    if (need_start) {
      if (phases[[nm]]$type == "constant") {
        phase_Phi_start[[j]] <- start_vec
        phase_deriv_start[[j]] <- NULL
      } else if (phases[[nm]]$type == "g3") {
        tau_j <- exp(pars$log_tau)
        d3s <- hzr_decompos_g3(start_vec, tau = tau_j, gamma = pars$gamma,
                                 alpha = pars$alpha, eta = pars$eta)
        phase_Phi_start[[j]] <- d3s$G3
        phase_deriv_start[[j]] <- .hzr_g3_phase_derivatives(
          start_vec, tau = tau_j, gamma = pars$gamma,
          alpha = pars$alpha, eta = pars$eta)
      } else {
        t_half_j <- exp(pars$log_t_half)
        pds <- .hzr_phase_derivatives(start_vec, t_half = t_half_j,
                                        nu = pars$nu, m = pars$m,
                                        type = phases[[nm]]$type)
        phase_Phi_start[[j]]   <- pds$Phi
        phase_deriv_start[[j]] <- pds
      }
    }

    H_t <- H_t + mu_j * phase_Phi[[j]]
    h_t <- h_t + mu_j * phase_phi[[j]]
  }

  # Guard: if total hazard or cumhaz is non-finite, return zero gradient
  if (any(!is.finite(H_t)) || any(!is.finite(h_t))) return(grad)

  # -- Per-observation weight vectors ----------------------------------------
  # dLogl/dH(t_i) depends on observation type:
  #   event:         d/dH [ log h(t) - H(t) ] = -1        (H part)
  #   right-cens:    d/dH [ -H(t) ]            = -1
  #   left-cens:     d/dH [ log(1-exp(-H(u))) ] = exp(-H) / (1 - exp(-H))
  #   interval-cens: more complex (handled separately)
  #
  # For events, there's also the d/d[theta] of log h(t):
  #   d(log h) / d(theta_j) = (1/h) * dh/d(theta_j)

  # Weight for the cumulative hazard part at `time[i]`: w_H_i = dLogl/dH(t_i),
  # scaled by Fisher weights so the score matches the weighted log-likelihood.
  w_H <- numeric(n)
  w_H[idx_event] <- -weights[idx_event]
  w_H[idx_right] <- -weights[idx_right]

  if (has_left) {
    H_upper <- .hzr_multiphase_cumhaz(upper, theta, phases, covariate_counts,
                                       x_list)
    # d/dH log(1 - exp(-H)) = exp(-H) / (1 - exp(-H))
    exp_neg_H_u <- exp(-H_upper[idx_left])
    one_minus   <- pmax(1 - exp_neg_H_u, .Machine$double.xmin)
    w_H[idx_left] <- weights[idx_left] * (exp_neg_H_u / one_minus)
    # Left-censored uses H(upper), not H(time), so we handle below
  }

  # H(start) contribution for counting-process rows (status in {0, 1}).
  # Per-row contribution is `+H(start_i)` inside the log-likelihood, so
  # the derivative w.r.t. theta picks up `+weights_i * dH(start)/dtheta`.
  has_start <- !is.null(time_lower) && any(time_lower > 0 & status %in% c(0L, 1L))
  w_H_start <- numeric(n)
  if (has_start) {
    w_H_start[idx_event] <- weights[idx_event]
    w_H_start[idx_right] <- weights[idx_right]
  }

  # Weight for the instantaneous hazard part (events only): w_i / h(t_i).
  # This multiplies dh/d(theta_j).
  inv_h <- numeric(n)
  if (length(idx_event) > 0) {
    h_event <- h_t[idx_event]
    h_event <- pmax(h_event, .Machine$double.xmin)
    inv_h[idx_event] <- weights[idx_event] / h_event
  }

  # -- Assemble gradient per phase -------------------------------------------
  pos <- 1L  # position in the full theta vector

  for (j in seq_along(phases)) {
    nm <- names(phases)[j]
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])
    mu_j  <- phase_mu[[j]]
    Phi_j <- phase_Phi[[j]]
    phi_j <- phase_phi[[j]]
    Phi_j_start <- phase_Phi_start[[j]]  # NULL when need_start = FALSE

    # -- d(logl) / d(log_mu_j) ------------------------------------------
    # From H part: w_H_i * dH/d(log_mu) = w_H_i * mu_j * Phi_j
    # From h part (events): inv_h_i * dh/d(log_mu) = inv_h_i * mu_j * phi_j
    dlogl_dlog_mu <- sum(w_H * mu_j * Phi_j) +
                     sum(inv_h[idx_event] * mu_j[idx_event] * phi_j[idx_event])

    # Counting-process entry-time contribution: +H(start) inside the LL,
    # so its derivative picks up +w_H_start * dH(start)/d(log_mu).
    if (need_start) {
      dlogl_dlog_mu <- dlogl_dlog_mu +
        sum(w_H_start * mu_j * Phi_j_start)
    }

    # Left-censored: uses H(upper), need separate Phi_j evaluation there
    if (has_left) {
      if (phases[[nm]]$type == "constant") {
        Phi_j_upper <- upper[idx_left]
      } else if (phases[[nm]]$type == "g3") {
        tau_j <- exp(pars$log_tau)
        d3_upper <- hzr_decompos_g3(upper[idx_left], tau = tau_j,
                                      gamma = pars$gamma, alpha = pars$alpha,
                                      eta = pars$eta)
        Phi_j_upper <- d3_upper$G3
      } else {
        t_half_j <- exp(pars$log_t_half)
        Phi_j_upper <- hzr_phase_cumhaz(upper[idx_left], t_half = t_half_j,
                                          nu = pars$nu, m = pars$m,
                                          type = phases[[nm]]$type)
      }
      dlogl_dlog_mu <- dlogl_dlog_mu +
        sum(w_H[idx_left] * mu_j[idx_left] * Phi_j_upper)
      # Subtract the w_H * mu * Phi at time[i] that we already added above
      # (for left-censored, the H contribution is at upper, not time)
      dlogl_dlog_mu <- dlogl_dlog_mu -
        sum(w_H[idx_left] * mu_j[idx_left] * Phi_j[idx_left])
    }

    # Interval-censored: use numerical fallback for these observations
    # (handled at end of loop via finite-difference correction)

    grad[pos] <- dlogl_dlog_mu
    pos <- pos + 1L

    # -- Shape parameters (non-constant phases only) ---------------------
    if (phases[[nm]]$type %in% c("cdf", "hazard")) {
      pd <- phase_deriv[[j]]

      # Compute shape derivatives at upper[idx_left] for left-censoring
      pd_upper <- NULL
      if (has_left) {
        t_half_j <- exp(pars$log_t_half)
        pd_upper <- .hzr_phase_derivatives(upper[idx_left],
          t_half = t_half_j, nu = pars$nu, m = pars$m,
          type = phases[[nm]]$type)
      }

      pd_start <- if (need_start) phase_deriv_start[[j]] else NULL

      # For each shape param s in {log_t_half, nu, m}:
      shape_derivs <- list(
        list(dPhi = pd$dPhi_dlog_thalf, dphi = pd$dphi_dlog_thalf,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dlog_thalf,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dlog_thalf),
        list(dPhi = pd$dPhi_dnu,        dphi = pd$dphi_dnu,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dnu,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dnu),
        list(dPhi = pd$dPhi_dm,         dphi = pd$dphi_dm,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dm,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dm)
      )

      for (s in seq_along(shape_derivs)) {
        dPhi_ds <- shape_derivs[[s]]$dPhi
        dphi_ds <- shape_derivs[[s]]$dphi

        dlogl_ds <- sum(w_H * mu_j * dPhi_ds) +
                    sum(inv_h[idx_event] * mu_j[idx_event] * dphi_ds[idx_event])

        # Counting-process start contribution: +w_H_start * mu * dPhi(start)
        if (need_start && !is.null(shape_derivs[[s]]$dPhi_start)) {
          dlogl_ds <- dlogl_ds +
            sum(w_H_start * mu_j * shape_derivs[[s]]$dPhi_start)
        }

        # Left-censoring correction: swap dPhi at time for dPhi at upper
        if (has_left && !is.null(shape_derivs[[s]]$dPhi_upper)) {
          dlogl_ds <- dlogl_ds +
            sum(w_H[idx_left] * mu_j[idx_left] * shape_derivs[[s]]$dPhi_upper) -
            sum(w_H[idx_left] * mu_j[idx_left] * dPhi_ds[idx_left])
        }

        grad[pos] <- dlogl_ds
        pos <- pos + 1L
      }
    } else if (phases[[nm]]$type == "g3") {
      pd <- phase_deriv[[j]]

      # Compute shape derivatives at upper[idx_left] for left-censoring
      pd_upper <- NULL
      if (has_left) {
        tau_j <- exp(pars$log_tau)
        pd_upper <- .hzr_g3_phase_derivatives(upper[idx_left],
          tau = tau_j, gamma = pars$gamma,
          alpha = pars$alpha, eta = pars$eta)
      }

      pd_start <- if (need_start) phase_deriv_start[[j]] else NULL

      # For each shape param s in {log_tau, gamma, alpha, eta}:
      shape_derivs <- list(
        list(dPhi = pd$dPhi_dlog_tau, dphi = pd$dphi_dlog_tau,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dlog_tau,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dlog_tau),
        list(dPhi = pd$dPhi_dgamma,   dphi = pd$dphi_dgamma,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dgamma,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dgamma),
        list(dPhi = pd$dPhi_dalpha,   dphi = pd$dphi_dalpha,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_dalpha,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_dalpha),
        list(dPhi = pd$dPhi_deta,     dphi = pd$dphi_deta,
             dPhi_upper = if (!is.null(pd_upper)) pd_upper$dPhi_deta,
             dPhi_start = if (!is.null(pd_start)) pd_start$dPhi_deta)
      )

      for (s in seq_along(shape_derivs)) {
        dPhi_ds <- shape_derivs[[s]]$dPhi
        dphi_ds <- shape_derivs[[s]]$dphi

        dlogl_ds <- sum(w_H * mu_j * dPhi_ds) +
                    sum(inv_h[idx_event] * mu_j[idx_event] * dphi_ds[idx_event])

        # Counting-process start contribution
        if (need_start && !is.null(shape_derivs[[s]]$dPhi_start)) {
          dlogl_ds <- dlogl_ds +
            sum(w_H_start * mu_j * shape_derivs[[s]]$dPhi_start)
        }

        # Left-censoring correction: swap dPhi at time for dPhi at upper
        if (has_left && !is.null(shape_derivs[[s]]$dPhi_upper)) {
          dlogl_ds <- dlogl_ds +
            sum(w_H[idx_left] * mu_j[idx_left] * shape_derivs[[s]]$dPhi_upper) -
            sum(w_H[idx_left] * mu_j[idx_left] * dPhi_ds[idx_left])
        }

        grad[pos] <- dlogl_ds
        pos <- pos + 1L
      }
    }

    # -- Covariate coefficients beta_jk ----------------------------------
    # dH/d(beta_jk) = x_ik * mu_j_i * Phi_j_i
    # dh/d(beta_jk) = x_ik * mu_j_i * phi_j_i
    if (length(pars$beta) > 0 && !is.null(x_list[[nm]])) {
      x_j <- x_list[[nm]]
      for (k in seq_along(pars$beta)) {
        x_k <- x_j[, k]
        dlogl_dbeta <- sum(w_H * x_k * mu_j * Phi_j) +
                       sum(inv_h[idx_event] * x_k[idx_event] *
                           mu_j[idx_event] * phi_j[idx_event])

        # Counting-process start contribution: +w_H_start * x_k * mu_j * Phi_start
        if (need_start) {
          dlogl_dbeta <- dlogl_dbeta +
            sum(w_H_start * x_k * mu_j * Phi_j_start)
        }

        if (has_left) {
          dlogl_dbeta <- dlogl_dbeta +
            sum(w_H[idx_left] * x_k[idx_left] *
                mu_j[idx_left] * Phi_j_upper) -
            sum(w_H[idx_left] * x_k[idx_left] *
                mu_j[idx_left] * Phi_j[idx_left])
        }

        grad[pos] <- dlogl_dbeta
        pos <- pos + 1L
      }
    } else {
      pos <- pos + covariate_counts[[nm]]
    }
  }

  # -- Interval-censored fallback: add correction via finite difference ---
  # Interval-censored observations are uncommon; their gradient contribution
  # is added via one-sided numerical difference on the weighted
  # log-likelihood of just those observations.
  if (has_interval) {
    w_iv <- weights[idx_interval]
    logl_iv <- function(th) {
      cumhaz_l <- .hzr_multiphase_cumhaz(lower, th, phases, covariate_counts,
                                          x_list)
      cumhaz_u <- .hzr_multiphase_cumhaz(upper, th, phases, covariate_counts,
                                          x_list)
      delta <- cumhaz_u[idx_interval] - cumhaz_l[idx_interval]
      sum(w_iv *
            (-cumhaz_l[idx_interval] + hzr_log1mexp(delta)))
    }
    eps_rel <- sqrt(.Machine$double.eps)
    ll0_iv <- logl_iv(theta)
    for (i in seq_len(p)) {
      h_i <- eps_rel * max(abs(theta[i]), 1)
      theta_p <- theta
      theta_p[i] <- theta_p[i] + h_i
      grad[i] <- grad[i] + (logl_iv(theta_p) - ll0_iv) / h_i
    }
  }

  # Safety: zero out non-finite entries
  grad[!is.finite(grad)] <- 0

  grad
}


# ============================================================================
# Optimizer
# ============================================================================

#' Fit a multiphase additive hazard model via maximum likelihood
#'
#' Assembles starting values from phase specifications, resolves per-phase
#' design matrices, and delegates to `.hzr_optim_generic()`.
#'
#' @param time Numeric vector of follow-up times.
#' @param status Numeric event indicator vector.
#' @param time_lower Optional lower bounds for interval censoring.
#' @param time_upper Optional upper bounds for left/interval censoring.
#' @param x Global design matrix (n x p) or NULL.
#' @param theta_start Starting parameter vector (full internal scale).
#'   If NULL, assembled automatically from phase specs.
#' @param control Named list of control options.
#' @param phases Named list of validated `hzr_phase` objects.
#' @param formula_global The global formula (used when phases have no
#'   phase-specific formula).
#' @param data Data frame containing covariates (needed for phase-specific
#'   formula evaluation).
#' @return List with par (internal scale), value, convergence, vcov, etc.
#' @keywords internal
.hzr_optim_multiphase <- function(time, status,
                                   time_lower = NULL, time_upper = NULL,
                                   x = NULL,
                                   theta_start = NULL,
                                   weights = NULL,
                                   control = list(),
                                   phases,
                                   formula_global = NULL,
                                   data = NULL) {

  if (is.null(weights)) weights <- rep(1, length(time))

  phases <- .hzr_validate_phases(phases)

  # --- Resolve per-phase design matrices and covariate counts ----------------
  # Build phase-specific design matrices with na.action = na.pass so that
  # rows with NA covariates are preserved (not silently dropped).  We then
  # align all observation-aligned inputs (time, status, weights, ...) by
  # removing any row that has NA in any phase's design matrix.  Without this
  # alignment, model.matrix()'s default na.omit shortens x_j relative to
  # time, and mu_j * phi_j emits a vector-recycling warning on every
  # likelihood evaluation.
  x_list <- vector("list", length(phases))
  names(x_list) <- names(phases)
  covariate_counts <- setNames(integer(length(phases)), names(phases))

  for (nm in names(phases)) {
    ph <- phases[[nm]]
    if (!is.null(ph$formula) && !is.null(data)) {
      # Phase-specific formula: build design matrix from data.  We route
      # through model.frame(na.action = na.pass) explicitly because
      # model.matrix() consults getOption("na.action") when it constructs
      # the frame internally, which defaults to na.omit and silently drops
      # rows with NA covariates.
      mf_j <- stats::model.frame(ph$formula, data = data,
                                   na.action = stats::na.pass)
      x_j <- stats::model.matrix(ph$formula, data = mf_j)[, -1L,
                                                           drop = FALSE]
      x_list[[nm]] <- x_j
      covariate_counts[[nm]] <- ncol(x_j)
    } else if (!is.null(x)) {
      # Inherit global design matrix
      x_list[[nm]] <- x
      covariate_counts[[nm]] <- ncol(x)
    } else {
      x_list[[nm]] <- NULL
      covariate_counts[[nm]] <- 0L
    }
  }

  # Align rows: drop any observation with an NA in any phase-specific design
  # matrix (or a row-count mismatch relative to time).
  n_time <- length(time)
  drop_rows <- logical(n_time)
  for (nm in names(phases)) {
    xm <- x_list[[nm]]
    if (is.null(xm) || ncol(xm) == 0L) next
    if (nrow(xm) != n_time) {
      stop("Design matrix for phase '", nm, "' has ", nrow(xm),
           " rows but length(time) is ", n_time,
           ". Check that 'data' matches 'time'/'status'.", call. = FALSE)
    }
    drop_rows <- drop_rows |
      apply(xm, 1L, function(row) any(is.na(row)))
  }
  if (any(drop_rows)) {
    keep <- !drop_rows
    time   <- time[keep]
    status <- status[keep]
    if (!is.null(time_lower)) time_lower <- time_lower[keep]
    if (!is.null(time_upper)) time_upper <- time_upper[keep]
    if (!is.null(weights))    weights    <- weights[keep]
    if (!is.null(x))          x          <- x[keep, , drop = FALSE]
    for (nm in names(phases)) {
      if (!is.null(x_list[[nm]])) {
        x_list[[nm]] <- x_list[[nm]][keep, , drop = FALSE]
      }
    }
  }

  # --- Assemble starting values if not provided ------------------------------
  if (is.null(theta_start)) {
    theta_start <- unlist(lapply(names(phases), function(nm) {
      .hzr_phase_start(phases[[nm]], n_covariates = covariate_counts[[nm]])
    }))
  }

  # Build parameter names for the full theta vector
  theta_names <- unlist(lapply(names(phases), function(nm) {
    cov_names <- if (covariate_counts[[nm]] > 0 && !is.null(x_list[[nm]])) {
      colnames(x_list[[nm]])
    } else {
      character(0)
    }
    # Ensure we have names even if colnames are NULL
    if (covariate_counts[[nm]] > 0 && length(cov_names) == 0) {
      cov_names <- paste0("x", seq_len(covariate_counts[[nm]]))
    }
    .hzr_phase_theta_names(phases[[nm]], nm, cov_names)
  }))

  names(theta_start) <- theta_names

  # --- Likelihood wrapper matching .hzr_optim_generic() signature -----------
  # NOTE: `weights` is an explicit formal so that .hzr_optim_generic can pass
  # it through without colliding with a closure capture via `...`
  # (which would trigger "formal argument 'weights' matched by multiple
  # actual arguments").
  logl_fn <- function(theta, time, status, time_lower, time_upper, x,
                      weights = NULL, ...) {
    .hzr_logl_multiphase(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights,
      phases = phases, covariate_counts = covariate_counts,
      x_list = x_list, ...
    )
  }

  # --- Analytic gradient (semi-analytic: chain-rule for mu/beta, central ----
  # differences for shape parameters via .hzr_phase_derivatives()).
  # Falls back to numerical gradient if analytic returns all zeros.
  #
  # IMPORTANT: Save a reference to the ORIGINAL (unwrapped) logl_fn for use

  # inside the gradient function's numerical fallback.  After this point,
  # logl_fn will be re-bound by CoE and fixed-mask wrappers, but the gradient
  # function always receives a FULL theta vector from its own wrapper chain,
  # so it must call the unwrapped version that also expects full theta.
  logl_fn_unwrapped <- logl_fn

  gradient_fn <- function(theta, time, status, time_lower, time_upper, x,
                          weights = NULL, ...) {
    grad <- .hzr_gradient_multiphase(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      phases = phases, covariate_counts = covariate_counts, x_list = x_list
    )

    # Fallback: if gradient is all zero (e.g. at infeasible point), try
    # numerical gradient of the *weighted* LL to keep optimizer moving.
    if (all(grad == 0)) {
      eps_rel <- sqrt(.Machine$double.eps)
      p <- length(theta)
      ll0 <- logl_fn_unwrapped(theta, time, status, time_lower,
                                time_upper, x, weights = weights, ...)
      for (i in seq_len(p)) {
        h_i <- eps_rel * max(abs(theta[i]), 1)
        theta_plus <- theta
        theta_plus[i] <- theta_plus[i] + h_i
        ll_plus <- logl_fn_unwrapped(theta_plus, time, status,
                                      time_lower, time_upper, x,
                                      weights = weights, ...)
        grad[i] <- (ll_plus - ll0) / h_i
      }
    }

    grad
  }

  # --- Conservation of Events (CoE) setup -------------------------------------
  use_conserve <- if (!is.null(control$conserve)) control$conserve else TRUE
  control$conserve <- NULL  # remove before passing to optim

  fixmu_phase <- NULL
  fixmu_pos <- NULL
  total_events <- NULL
  log_mu_positions <- .hzr_log_mu_positions(phases, covariate_counts)

  # CoE is only valid for right-censored + exact-event data.  Interval and
  # left censoring require a different event-counting formulation.
  coe_supported_data <- all(status %in% c(0, 1))

  if (use_conserve && coe_supported_data && length(phases) >= 2L) {
    total_events <- sum(weights[status == 1])
    if (total_events > 0) {
      # Initial CoE scaling: adjust all log_mu proportionally (weighted sums
      # so the scaling is consistent with total_events).
      decomp_init <- .hzr_multiphase_cumhaz(
        time, theta_start, phases, covariate_counts, x_list,
        per_phase = TRUE
      )
      # Entry-time scale under left truncation: conserve H(stop) - H(start).
      init_total <- decomp_init$total
      if (!is.null(time_lower)) {
        decomp_init_start <- .hzr_multiphase_cumhaz(
          time_lower, theta_start, phases, covariate_counts, x_list,
          per_phase = TRUE
        )
        init_total <- init_total - decomp_init_start$total
      }
      sumcz_init <- sum(weights * init_total)
      if (sumcz_init > 0 && is.finite(sumcz_init)) {
        init_factor <- log(total_events / sumcz_init)
        if (is.finite(init_factor)) {
          for (nm in names(phases)) {
            theta_start[log_mu_positions[[nm]]] <-
              theta_start[log_mu_positions[[nm]]] + init_factor
          }
        }
      }

      # Select fixmu phase (largest weighted cumhaz contributor after scaling)
      fixmu_phase <- .hzr_select_fixmu_phase(
        theta_start, time, status, phases, covariate_counts, x_list,
        weights = weights, time_lower = time_lower
      )
      fixmu_pos <- log_mu_positions[[fixmu_phase]]

      # Apply precise CoE adjustment to the fixmu phase
      theta_start <- .hzr_conserve_events(
        theta_start, fixmu_phase, fixmu_pos,
        time, status, phases, covariate_counts, x_list, total_events,
        weights = weights, time_lower = time_lower
      )

      # Wrap logl and gradient: apply CoE before each evaluation
      logl_fn_pre_coe <- logl_fn
      logl_fn <- function(theta, time, status, time_lower,
                          time_upper, x, weights = NULL, ...) {
        theta <- .hzr_conserve_events(
          theta, fixmu_phase, fixmu_pos,
          time, status, phases, covariate_counts, x_list, total_events,
          weights = weights, time_lower = time_lower
        )
        logl_fn_pre_coe(theta, time, status, time_lower,
                        time_upper, x, weights = weights, ...)
      }

      gradient_fn_pre_coe <- gradient_fn
      gradient_fn <- function(theta, time, status, time_lower,
                              time_upper, x, weights = NULL, ...) {
        theta <- .hzr_conserve_events(
          theta, fixmu_phase, fixmu_pos,
          time, status, phases, covariate_counts, x_list, total_events,
          weights = weights, time_lower = time_lower
        )
        gradient_fn_pre_coe(theta, time, status, time_lower,
                            time_upper, x, weights = weights, ...)
      }
    } else {
      use_conserve <- FALSE
    }
  } else {
    use_conserve <- FALSE
  }

  # --- Fixed-parameter masking ------------------------------------------------
  # Build a logical mask: TRUE = free (optimized), FALSE = fixed (held constant)
  free_mask <- .hzr_phase_free_mask(phases, covariate_counts)

  # If CoE is active, also fix the fixmu phase's log_mu
  if (use_conserve && !is.null(fixmu_pos)) {
    free_mask[fixmu_pos] <- FALSE
  }

  any_fixed <- !all(free_mask)
  theta_fixed <- theta_start  # full vector; fixed entries stay at these values

  # When some parameters are fixed, wrap logl/gradient to operate on the
  # reduced (free-only) parameter vector.
  if (any_fixed) {
    free_idx <- which(free_mask)
    free_idx_eff <- free_idx

    # Expand reduced theta to full theta
    expand_theta <- function(theta_free) {
      theta_full <- theta_fixed
      theta_full[free_idx] <- theta_free
      theta_full
    }

    logl_fn_full <- logl_fn
    logl_fn <- function(theta, time, status, time_lower, time_upper, x,
                        weights = NULL, ...) {
      logl_fn_full(expand_theta(theta), time, status, time_lower,
                   time_upper, x, weights = weights, ...)
    }

    gradient_fn_full <- gradient_fn
    gradient_fn <- function(theta, time, status, time_lower, time_upper, x,
                            weights = NULL, ...) {
      grad_full <- gradient_fn_full(expand_theta(theta), time, status,
                                     time_lower, time_upper, x,
                                     weights = weights, ...)
      grad_full[free_idx]
    }

    theta_start_optim <- theta_start[free_idx]
  } else {
    free_idx_eff <- seq_along(theta_start)
    theta_start_optim <- theta_start
  }

  # --- Multi-start optimization -----------------------------------------------
  n_starts <- if (!is.null(control$n_starts)) control$n_starts else 5L
  control$n_starts <- NULL  # remove before passing to optim

  best_result <- NULL
  best_value <- -Inf

  # When shapes are fixed the free-parameter count is small (typically just the

  # mu intercepts).  BFGS can struggle on this reduced problem because the
  # likelihood surface is poorly scaled across phases (e.g. the late-phase mu
  # may be ~1e-8).  Strategy: run Nelder-Mead first (derivative-free, robust to
  # scaling) to find a good basin, then polish with BFGS for accurate SE.
  # Nelder-Mead requires >= 2 parameters; use Brent for 1D
  use_nelder_mead_warmup <- any_fixed && length(theta_start_optim) >= 2L &&
    length(theta_start_optim) <= 10L

  # Analytic Hessian closure -- covers event + right-censored rows.
  # Returns NULL for left/interval-censored data to trigger numDeriv fallback.
  # Defined OUTSIDE the n_starts loop: it is identical across all starts and
  # captures only stable (pre-loop) variables.
  hessian_fn_mp <- function(theta_free) {
    # theta_free is on the REDUCED (free-parameter) scale; expand to full.
    theta_full <- if (any_fixed) {
      th <- theta_fixed
      th[free_idx_eff] <- theta_free
      th
    } else {
      theta_free
    }
    H_full <- .hzr_hessian_multiphase(
      theta_full, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights,
      phases = phases, covariate_counts = covariate_counts, x_list = x_list
    )
    if (is.null(H_full)) return(NULL)
    # Restrict to the free-parameter block that the optimizer sees
    if (any_fixed) H_full[free_idx_eff, free_idx_eff] else H_full
  }

  for (start_i in seq_len(n_starts)) {
    if (start_i == 1L) {
      theta_try <- theta_start_optim
    } else {
      # Random perturbation of the starting values, drawn from the ambient RNG
      # stream (not internally seeded). Call set.seed() before fitting for
      # reproducible multi-start results.
      theta_try <- theta_start_optim +
        stats::rnorm(length(theta_start_optim), sd = 0.5)
    }

    # Nelder-Mead warm-up when shapes are fixed
    if (use_nelder_mead_warmup) {
      # Negative log-likelihood for minimization (matches .hzr_optim_generic)
      nm_objective <- function(theta) {
        if (!all(is.finite(theta))) return(1e10)
        ll <- -logl_fn(
          theta = theta, time = time, status = status,
          time_lower = time_lower, time_upper = time_upper,
          x = x, weights = weights, return_gradient = FALSE
        )
        if (!is.finite(ll)) return(1e10)
        ll
      }
      nm_result <- tryCatch(
        stats::optim(
          par = theta_try, fn = nm_objective,
          method = "Nelder-Mead",
          control = list(maxit = 5000L, reltol = 1e-10)
        ),
        error = function(e) NULL
      )
      if (!is.null(nm_result) && is.finite(nm_result$value)) {
        theta_try <- nm_result$par
      }
    }

    result <- tryCatch(
      .hzr_optim_generic(
        logl_fn     = logl_fn,
        gradient_fn = gradient_fn,
        time        = time, status = status,
        time_lower  = time_lower, time_upper = time_upper,
        x           = x,
        theta_start = theta_try,
        weights     = weights,
        control     = control,
        use_bounds  = FALSE,
        hessian_fn  = hessian_fn_mp
      ),
      error = function(e) NULL
    )

    if (!is.null(result) && is.finite(result$value) && result$value > best_value) {
      best_value <- result$value
      best_result <- result
    }
  }

  if (is.null(best_result)) {
    stop("Multiphase optimization failed to converge on any start.", call. = FALSE)
  }

  # Expand optimized free params back to full theta vector
  if (any_fixed) {
    theta_full <- theta_fixed
    theta_full[free_idx] <- best_result$par
    best_result$par <- theta_full

    # Final CoE adjustment: solve fixmu log_mu at the optimum. Must pass
    # time_lower so the conserved log_mu in the returned coefficients is solved
    # on the same entry-time scale the objective was optimized on; otherwise a
    # left-truncated fit returns a conserved mu inconsistent with its objective.
    if (use_conserve && !is.null(fixmu_pos)) {
      best_result$par <- .hzr_conserve_events(
        best_result$par, fixmu_phase, fixmu_pos,
        time, status, phases, covariate_counts, x_list, total_events,
        weights = weights, time_lower = time_lower
      )
    }

    # Expand vcov to full dimension (NA for fixed params -- not estimated)
    if (!is.null(best_result$vcov) && is.matrix(best_result$vcov)) {
      p_full <- length(theta_fixed)
      vcov_full <- matrix(NA_real_, p_full, p_full)
      vcov_full[free_idx, free_idx] <- best_result$vcov
      best_result$vcov <- vcov_full
    }

    best_result$fixed_mask <- !free_mask

    # --- Full-information vcov for Conservation-of-Events fits ---------------
    # CoE removes the conserved phase's log_mu from the *search* (its score
    # equation IS the CoE constraint), so the optimizer's Hessian leaves that
    # parameter with no variance. But at the optimum the CoE solution is the
    # unconstrained MLE, so the reported *uncertainty* should be the full-
    # information vcov -- exactly what an all-mu-free (conserve = FALSE) fit
    # gives at the same point. Recompute the vcov from the unconstrained-
    # objective Hessian over the free set with the conserved log_mu restored,
    # so the conserved phase (and anything depending on it, e.g. se(H) and
    # survival CLs) gets a proper standard error.
    if (use_conserve && !is.null(fixmu_pos)) {
      recomputed <- FALSE
      free_unc <- free_mask
      free_unc[fixmu_pos] <- TRUE
      idx_unc <- which(free_unc)
      base_theta <- best_result$par
      neg_ll_unc <- function(th_free) {
        th <- base_theta
        th[idx_unc] <- th_free
        -logl_fn_pre_coe(th, time, status, time_lower, time_upper, x,
                         weights = weights, return_gradient = FALSE)
      }
      # Analytic Hessian attempt -- no numDeriv dependency
      H_unc <- tryCatch(
        .hzr_hessian_multiphase(
          base_theta, time = time, status = status,
          time_lower = time_lower, time_upper = time_upper,
          x = x, weights = weights,
          phases = phases, covariate_counts = covariate_counts,
          x_list = x_list
        ),
        error = function(e) NULL
      )
      if (is.matrix(H_unc)) {
        # Restrict to the unconstrained free set
        H_unc <- H_unc[idx_unc, idx_unc]
      }
      # numDeriv fallback only when analytic declines (left/interval rows)
      if (is.null(H_unc) && requireNamespace("numDeriv", quietly = TRUE)) {
        H_unc <- tryCatch(
          numDeriv::hessian(neg_ll_unc, base_theta[idx_unc]),
          error = function(e) NULL
        )
      }
      if (is.matrix(H_unc)) {
        inv_unc <- .hzr_safe_solve(H_unc)
        if (is.matrix(inv_unc$vcov)) {
          p_full <- length(base_theta)
          vcov_full <- matrix(NA_real_, p_full, p_full)
          vcov_full[idx_unc, idx_unc] <- inv_unc$vcov
          best_result$vcov  <- vcov_full
          best_result$rcond <- inv_unc$rcond
          best_result$pd    <- inv_unc$pd
          # The conserved log_mu now carries a variance; it is fixed only for
          # the search, not for inference.
          best_result$fixed_mask <- !free_unc
          recomputed <- TRUE
        }
      }
      if (!recomputed) {
        # Fall back to the reduced (search-only) vcov, in which the conserved
        # log_mu has no variance. Warn loudly so the understated uncertainty is
        # visible rather than silent.
        warning(
          "Could not compute the full-information variance for the ",
          "Conservation-of-Events-conserved phase 'log_mu' (numDeriv ",
          "unavailable or the Hessian was not invertible). Its standard error ",
          "stays NA and downstream standard errors / confidence limits for ",
          "that phase may be understated.", call. = FALSE)
      }
    }
  }

  # Restore parameter names
  names(best_result$par) <- theta_names

  # Store phase metadata for downstream use (predict, summary)
  best_result$phases <- phases
  best_result$covariate_counts <- covariate_counts
  best_result$x_list <- x_list

  best_result
}


# ============================================================================
# G3 phase derivatives (central finite differences)
# ============================================================================

#' Finite-difference derivatives of G3 phase Phi and phi w.r.t. shape params
#'
#' Computes the G3 cumulative intensity, its time derivative, and their
#' partial derivatives with respect to log_tau, gamma, alpha, and eta using
#' central finite differences.
#'
#' @param time Numeric vector of positive times.
#' @param tau Positive scalar scale parameter.
#' @param gamma Positive scalar time exponent.
#' @param alpha Non-negative scalar shape parameter.
#' @param eta Positive scalar outer exponent.
#' @param h Relative step size for finite differences (default 1e-5).
#' @return Named list with Phi, phi, and 8 derivative vectors.
#' @keywords internal
.hzr_g3_phase_derivatives <- function(time, tau, gamma, alpha, eta,
                                       h = 1e-5) {

  # Base evaluation
  d0 <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                          alpha = alpha, eta = eta)
  Phi0 <- d0$G3
  phi0 <- d0$g3

  # Central differences for log_tau: tau * d/d(tau) = d/d(log(tau))
  eps_tau <- max(abs(tau) * h, 1e-10)
  if (tau - eps_tau > 0) {
    d_plus  <- hzr_decompos_g3(time, tau = tau + eps_tau, gamma = gamma,
                                 alpha = alpha, eta = eta)
    d_minus <- hzr_decompos_g3(time, tau = tau - eps_tau, gamma = gamma,
                                 alpha = alpha, eta = eta)
    dPhi_dtau <- (d_plus$G3 - d_minus$G3) / (2 * eps_tau)
    dphi_dtau <- (d_plus$g3 - d_minus$g3) / (2 * eps_tau)
  } else {
    d_plus  <- hzr_decompos_g3(time, tau = tau + eps_tau, gamma = gamma,
                                 alpha = alpha, eta = eta)
    dPhi_dtau <- (d_plus$G3 - Phi0) / eps_tau
    dphi_dtau <- (d_plus$g3 - phi0) / eps_tau
  }
  # Chain rule: d/d(log_tau) = tau * d/d(tau)
  dPhi_dlog_tau <- tau * dPhi_dtau
  dphi_dlog_tau <- tau * dphi_dtau

  # Central differences for gamma (must stay positive)
  eps_g <- max(abs(gamma) * h, 1e-10)
  if (gamma - eps_g > 0) {
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma + eps_g,
                                 alpha = alpha, eta = eta)
    d_minus <- hzr_decompos_g3(time, tau = tau, gamma = gamma - eps_g,
                                 alpha = alpha, eta = eta)
    dPhi_dgamma <- (d_plus$G3 - d_minus$G3) / (2 * eps_g)
    dphi_dgamma <- (d_plus$g3 - d_minus$g3) / (2 * eps_g)
  } else {
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma + eps_g,
                                 alpha = alpha, eta = eta)
    dPhi_dgamma <- (d_plus$G3 - Phi0) / eps_g
    dphi_dgamma <- (d_plus$g3 - phi0) / eps_g
  }

  # Central differences for alpha
  if (alpha > h) {
    eps_a <- max(abs(alpha) * h, 1e-10)
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha + eps_a, eta = eta)
    d_minus <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha - eps_a, eta = eta)
    dPhi_dalpha <- (d_plus$G3 - d_minus$G3) / (2 * eps_a)
    dphi_dalpha <- (d_plus$g3 - d_minus$g3) / (2 * eps_a)
  } else {
    # alpha near 0: use forward difference
    eps_a <- h
    d_plus <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                alpha = alpha + eps_a, eta = eta)
    dPhi_dalpha <- (d_plus$G3 - Phi0) / eps_a
    dphi_dalpha <- (d_plus$g3 - phi0) / eps_a
  }

  # Central differences for eta (must stay positive)
  eps_e <- max(abs(eta) * h, 1e-10)
  if (eta - eps_e > 0) {
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha, eta = eta + eps_e)
    d_minus <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha, eta = eta - eps_e)
    dPhi_deta <- (d_plus$G3 - d_minus$G3) / (2 * eps_e)
    dphi_deta <- (d_plus$g3 - d_minus$g3) / (2 * eps_e)
  } else {
    d_plus  <- hzr_decompos_g3(time, tau = tau, gamma = gamma,
                                 alpha = alpha, eta = eta + eps_e)
    dPhi_deta <- (d_plus$G3 - Phi0) / eps_e
    dphi_deta <- (d_plus$g3 - phi0) / eps_e
  }

  list(
    Phi = Phi0,
    phi = phi0,
    dPhi_dlog_tau = dPhi_dlog_tau,
    dPhi_dgamma   = dPhi_dgamma,
    dPhi_dalpha   = dPhi_dalpha,
    dPhi_deta     = dPhi_deta,
    dphi_dlog_tau = dphi_dlog_tau,
    dphi_dgamma   = dphi_dgamma,
    dphi_dalpha   = dphi_dalpha,
    dphi_deta     = dphi_deta
  )
}


# ============================================================================
# Back-transformation helpers
# ============================================================================

#' Transform multiphase theta from internal to user-facing scale
#'
#' Converts log_mu -> mu, log_t_half -> t_half; nu, m, beta pass through.
#'
#' @param theta Full parameter vector (internal scale).
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of per-phase covariate counts.
#' @return Named numeric vector on the reporting scale.
#' @keywords internal
.hzr_multiphase_theta_user <- function(theta, phases, covariate_counts) {
  theta_split <- .hzr_split_theta(theta, phases, covariate_counts)
  values <- numeric(0)
  nms    <- character(0)

  for (nm in names(phases)) {
    pars <- .hzr_unpack_phase_theta(theta_split[[nm]], phases[[nm]])

    if (phases[[nm]]$type == "constant") {
      values <- c(values, exp(pars$log_mu))
      nms    <- c(nms, paste0(nm, ".mu"))
    } else if (phases[[nm]]$type == "g3") {
      values <- c(values, exp(pars$log_mu), exp(pars$log_tau),
                  pars$gamma, pars$alpha, pars$eta)
      nms    <- c(nms, paste0(nm, ".mu"), paste0(nm, ".tau"),
                  paste0(nm, ".gamma"), paste0(nm, ".alpha"), paste0(nm, ".eta"))
    } else {
      values <- c(values, exp(pars$log_mu), exp(pars$log_t_half),
                  pars$nu, pars$m)
      nms    <- c(nms, paste0(nm, ".mu"), paste0(nm, ".t_half"),
                  paste0(nm, ".nu"), paste0(nm, ".m"))
    }

    if (length(pars$beta) > 0) {
      values <- c(values, pars$beta)
      # Try to derive covariate names from the theta_split names
      n_skip <- switch(phases[[nm]]$type, constant = 1L, g3 = 5L, 4L)
      raw_names <- names(theta_split[[nm]])
      if (!is.null(raw_names) && length(raw_names) > n_skip) {
        beta_labels <- raw_names[(n_skip + 1L):length(raw_names)]
        # Strip phase prefix if present
        beta_labels <- sub(paste0("^", nm, "\\."), "", beta_labels)
      } else {
        beta_labels <- paste0("x", seq_along(pars$beta))
      }
      nms <- c(nms, paste0(nm, ".", beta_labels))
    }
  }

  names(values) <- nms
  values
}

# Shared parser for SAS HAZARD .lst capture files.  testthat auto-sources
# this helper before running test-sas-parity.R.
#
# Reference fixtures live at:
#   ~/Documents/GitHub/hazard/examples/*.lst   (paired with *.sas)
#
# Capture vintage: v4.4 macOS (2026-04-27).  Final values refresh when
# v4.4.6 captures land — the parser itself is version-stable.
#
# Encoding notes (verified across all 18 fixtures):
#   - CRLF line endings  ->  strip \r
#   - SAS form-feed page breaks  ->  convert \f to \n
#   - "Non-ISO extended-ASCII" classification causes grep -c to silently
#     return 0 without -a; R's readLines handles this correctly when the
#     normalized text is split on \n.
#
# Public API:
#   .hzr_parse_sas_lst(path) -> list(fits = list(<one per Final Results:>))
#
#   Each fit element:
#     loglik        scalar, final "Log likelihood = N" before "Final Results:"
#     n_obs         integer
#     n_events      integer
#     n_censored    integer
#     params        data.frame(phase, name, label, estimate, se, z, p)
#                   from "Parameter Estimate Summary"
#     natural       data.frame(phase, name, fixed, estimate)
#                   from "Estimates for Model Parameters"
#     vcov          numeric matrix (NULL if NOCOV)
#     correlation   numeric matrix (NULL if NOCOR)
#     events_conserved  scalar or NA

.hzr_read_lst <- function(path) {
  raw <- readLines(path, warn = FALSE, encoding = "latin1")
  # Convert form-feed page breaks to newlines, then re-split.
  joined <- paste(raw, collapse = "\n")
  joined <- gsub("\f", "\n", joined, fixed = TRUE)
  joined <- gsub("\r", "",  joined, fixed = TRUE)
  strsplit(joined, "\n", fixed = TRUE)[[1]]
}

.hzr_extract_loglik <- function(lines) {
  m <- regmatches(
    lines,
    regexec("Log likelihood\\s*=\\s*(-?[0-9.Ee+-]+)", lines)
  )
  vals <- vapply(
    m,
    function(x) if (length(x) >= 2) as.numeric(x[2]) else NA_real_,
    numeric(1)
  )
  vals[!is.na(vals)]
}

.hzr_extract_obs_counts <- function(lines) {
  # "There are  5880 observations available for analysis with:"
  # "          545 events"
  # "         5335 Right Censored Observations"
  obs_line <- grep("observations available for analysis", lines, value = TRUE)[1]
  ev_line  <- grep("\\bevents\\s*$",                       lines, value = TRUE)[1]
  rc_line  <- grep("Right Censored Observations",          lines, value = TRUE)[1]

  pull <- function(pat, s) {
    if (is.na(s)) return(NA_integer_)
    m <- regmatches(s, regexec(pat, s))[[1]]
    if (length(m) >= 2) as.integer(m[2]) else NA_integer_
  }

  list(
    n_obs      = pull("There are\\s+([0-9]+)\\s+observations",          obs_line),
    n_events   = pull("([0-9]+)\\s+events",                              ev_line),
    n_censored = pull("([0-9]+)\\s+Right Censored Observations",         rc_line)
  )
}

.hzr_extract_events_conserved <- function(lines) {
  s <- grep("Number of events conserved", lines, value = TRUE)
  if (!length(s)) return(NA_real_)
  m <- regmatches(s[1], regexec("=\\s*(-?[0-9.Ee+-]+)", s[1]))[[1]]
  if (length(m) >= 2) as.numeric(m[2]) else NA_real_
}

# Parameter Estimate Summary table.  Layout:
#
#   Phase      Parameter   Label              Estimate    Std error      Z        Prob>|Z|
#   ---------- ...
#   Early:     E0                              -3.77955    0.09381214   -40.288   <.0001
#              ---------------- ...
#   Constant:  C0                              -7.2258     0.09312647   -77.591   <.0001
#
# - Phase label appears only on the first row of each phase; subsequent
#   rows in the same phase have a blank Phase column.
# - Inner "---" separator rows divide phases.
# - Outer "===" / long "---" rule bounds the table.
.hzr_extract_param_summary <- function(lines) {
  start <- grep("Parameter Estimate Summary", lines)
  if (!length(start)) return(NULL)
  start <- start[1]

  # Find the header row (starts with "Phase").
  hdr <- which(grepl("^\\s*Phase\\s+Parameter", lines))
  hdr <- hdr[hdr > start][1]
  if (is.na(hdr)) return(NULL)

  # Body starts after the rule line under the header.
  body_start <- hdr + 2L

  # Body ends at the next blank-rule (long dashes) followed by blank lines
  # before "Estimates for Model Parameters".
  end_marker <- grep("Estimates for Model Parameters", lines)
  end_marker <- end_marker[end_marker > hdr][1]
  if (is.na(end_marker)) return(NULL)

  block <- lines[body_start:(end_marker - 1L)]
  block <- block[nzchar(trimws(block))]
  # Drop separator rows (only dashes / spaces).
  block <- block[!grepl("^[\\s-]+$", block, perl = TRUE)]

  current_phase <- NA_character_
  rows <- list()

  for (ln in block) {
    # Phase label appears at the very start of the line, ending in ":".
    m_phase <- regmatches(ln, regexec("^\\s+(Early|Constant|Late):", ln))[[1]]
    if (length(m_phase) >= 2) {
      current_phase <- m_phase[2]
      ln <- sub("^\\s*(Early|Constant|Late):\\s*", "", ln)
    } else {
      # Continuation row — strip leading whitespace.
      ln <- sub("^\\s+", "", ln)
    }

    # Tail of each row: estimate, se, z, p.  Use a trailing-fields regex
    # so we capture them regardless of label column width.
    # Trailing fields: estimate, se, z, p.  p-value can be "<.0001" or a
    # decimal like "0.0032" / ".0032".
    tail_re <- "(-?[0-9.Ee+-]+)\\s+(-?[0-9.Ee+-]+)\\s+(-?[0-9.Ee+-]+)\\s+(<\\.?[0-9]+|[0-9.]+)\\s*$"
    m_tail <- regmatches(ln, regexec(tail_re, ln))[[1]]
    if (length(m_tail) < 5) next

    # Head: parameter name and optional label.  Strip the matched tail.
    head <- sub(tail_re, "", ln)
    head <- trimws(head)
    # First token is the parameter name; remainder is the label.
    head_split <- regmatches(head, regexec("^(\\S+)\\s*(.*)$", head))[[1]]
    name  <- if (length(head_split) >= 2) head_split[2] else NA_character_
    label <- if (length(head_split) >= 3) trimws(head_split[3]) else ""

    p_str <- m_tail[5]
    p_val <- if (grepl("^<", p_str)) {
      as.numeric(sub("^<", "", p_str))
    } else {
      as.numeric(p_str)
    }

    rows[[length(rows) + 1L]] <- data.frame(
      phase    = current_phase,
      name     = name,
      label    = label,
      estimate = as.numeric(m_tail[2]),
      se       = as.numeric(m_tail[3]),
      z        = as.numeric(m_tail[4]),
      p        = p_val,
      stringsAsFactors = FALSE
    )
  }

  if (!length(rows)) return(NULL)
  do.call(rbind, rows)
}

# Estimates for Model Parameters table — natural-scale values, all params
# (free + fixed).  Layout:
#
#   Phase       Parameter     Fixed?     Estimate
#   Early:      DELTA         Yes                  0
#               THALF         Yes                0.2
#                (RHO                            0.2)
#               ...
#               MUE                       0.02283304
.hzr_extract_natural <- function(lines) {
  start <- grep("Estimates for Model Parameters", lines)
  if (!length(start)) return(NULL)
  start <- start[1]

  hdr <- which(grepl("^\\s*Phase\\s+Parameter\\s+Fixed\\?", lines))
  hdr <- hdr[hdr > start][1]
  if (is.na(hdr)) return(NULL)

  body_start <- hdr + 2L

  end_marker <- grep("Asymptotic Variance-Covariance Matrix|Asymptotic Correlation Matrix",
                     lines)
  end_marker <- end_marker[end_marker > hdr][1]
  end_idx <- if (is.na(end_marker)) length(lines) else end_marker - 1L

  block <- lines[body_start:end_idx]
  block <- block[nzchar(trimws(block))]
  block <- block[!grepl("^[\\s-]+$", block, perl = TRUE)]

  current_phase <- NA_character_
  rows <- list()
  for (ln in block) {
    # Skip RHO informational rows: " (RHO                            0.2)"
    if (grepl("^\\s*\\(RHO", ln)) next

    # Natural-scale table is deeply indented (~40+ leading spaces); accept
    # arbitrary leading whitespace before the phase label.
    m_phase <- regmatches(ln, regexec("^\\s+(Early|Constant|Late):", ln))[[1]]
    if (length(m_phase) >= 2) {
      current_phase <- m_phase[2]
      ln <- sub("^\\s*(Early|Constant|Late):\\s*", "", ln)
    } else {
      ln <- sub("^\\s+", "", ln)
    }

    # Tail: optional "Yes"/"No", then numeric estimate.
    tail_re <- "^(\\S+)\\s+(?:(Yes|No)\\s+)?(-?[0-9.Ee+-]+)\\s*$"
    m <- regmatches(ln, regexec(tail_re, ln, perl = TRUE))[[1]]
    if (length(m) < 4) next

    rows[[length(rows) + 1L]] <- data.frame(
      phase    = current_phase,
      name     = m[2],
      fixed    = identical(m[3], "Yes"),
      estimate = as.numeric(m[4]),
      stringsAsFactors = FALSE
    )
  }
  if (!length(rows)) return(NULL)
  do.call(rbind, rows)
}

# Symmetric matrix parser shared by vcov and correlation blocks.
#
# Wide matrices (>~8 params) wrap both the header row and each data row
# across multiple lines.  Pages are also broken by form-feed inserting a
# title block, so we collect column names from any line of header-shape
# until a separator rule appears, and we accumulate numerics from
# continuation lines until the next labelled row or section.
.hzr_extract_matrix <- function(lines, marker) {
  start <- grep(marker, lines)
  if (!length(start)) return(NULL)
  start <- start[1]

  # Walk forward to collect the (possibly multi-line) header.  Header lines
  # contain only parameter names (identifiers) separated by 2+ spaces;
  # they terminate at the rule line of dashes.
  i <- start + 1L
  col_names <- character()
  saw_header <- FALSE
  while (i <= length(lines)) {
    ln <- lines[i]
    if (grepl("^\\s*-{10,}", ln)) {
      if (saw_header) break
    } else if (grepl("^\\s+[A-Za-z_][A-Za-z0-9_]*(\\s{2,}[A-Za-z_][A-Za-z0-9_]*)+\\s*$", ln)) {
      col_names <- c(col_names, strsplit(trimws(ln), "\\s{2,}")[[1]])
      saw_header <- TRUE
    } else if (saw_header && !nzchar(trimws(ln))) {
      # blank line inside header — keep going
    } else if (saw_header) {
      break
    }
    i <- i + 1L
  }
  if (!length(col_names)) return(NULL)
  k <- length(col_names)

  body <- lines[i:length(lines)]
  stop_pat <- "Asymptotic Correlation Matrix|Final Results|Initial Summary|Estimates for Model"
  if (!grepl("Correlation", marker)) {
    # vcov block: stop at the correlation block.
  } else {
    stop_pat <- "Final Results|Initial Summary|Estimates for Model"
  }
  stop_idx <- grep(stop_pat, body)
  if (length(stop_idx)) body <- body[seq_len(stop_idx[1] - 1L)]

  M <- matrix(NA_real_, k, k)
  # Use unique row/col names so duplicate parameters (e.g. STATUS appearing
  # in both Early and Constant phases) get distinct slots.  Track phase
  # context in the body so callers can resolve "Early:STATUS" vs
  # "Constant:STATUS" via the rownames suffix.
  rownames(M) <- make.unique(col_names, sep = "__")
  colnames(M) <- rownames(M)

  # The body presents rows in the same order as header columns (verified
  # across all in-scope fixtures).  Track a single row cursor that
  # advances each time we see a new labelled row — this is robust to
  # duplicate parameter names because position, not name, drives the
  # routing.  Phase delimiters in the body act as sanity check only.
  row_cursor <- 0L
  pending <- list()
  current_row  <- NA_integer_
  current_name <- NA_character_
  current_phase <- NA_character_
  current_vals <- numeric()
  flush <- function() {
    if (!is.na(current_row) && length(current_vals)) {
      pending[[length(pending) + 1L]] <<- list(
        row = current_row, vals = current_vals,
        name = current_name, phase = current_phase
      )
    }
    current_row  <<- NA_integer_
    current_name <<- NA_character_
    current_vals <<- numeric()
  }

  for (ln in body) {
    # Skip rule lines.
    if (grepl("^\\s*-{10,}", ln)) next
    # Phase-label rows update phase context; don't reset row tracking.
    m_phase <- regmatches(ln, regexec("^\\s+(Early|Constant|Late):\\s*$", ln))[[1]]
    if (length(m_phase) >= 2) {
      current_phase <- m_phase[2]
      next
    }
    # Skip page-break title lines from form-feed inserts.
    if (!nzchar(trimws(ln))) next

    # New labelled row: leading identifier followed by numerics.
    toks <- regmatches(ln, regexec("^\\s+([A-Za-z_][A-Za-z0-9_]*)\\s+(.+)$", ln))[[1]]
    # Only treat as a real row if the identifier is one of the matrix's
    # parameter names.  Form-feed page breaks inject title-block lines
    # like "Primary Valve Operations ..." or "Multivariable Analysis ..."
    # that start with an identifier but aren't rows; they must not
    # advance the cursor or be parsed as data.
    is_real_row <- length(toks) >= 3 &&
      !grepl("^[+-]?[0-9.]", toks[2]) &&
      toks[2] %in% col_names
    if (is_real_row) {
      flush()
      rn <- toks[2]
      # Cursor-based routing handles duplicate parameter names (e.g.
      # STATUS or AGE_COP appearing in both Early and Constant phases).
      # Sanity-check that col_names[cursor] matches; otherwise pick the
      # next at-or-after-cursor match to keep position-based routing.
      row_cursor <- row_cursor + 1L
      if (row_cursor <= k && identical(col_names[row_cursor], rn)) {
        current_row <- row_cursor
      } else {
        candidates <- which(col_names == rn)
        forward <- candidates[candidates >= row_cursor]
        current_row <- if (length(forward)) forward[1] else
                       if (length(candidates)) candidates[1] else NA_integer_
        if (!is.na(current_row)) row_cursor <- current_row
      }
      current_name <- rn
      if (!is.na(current_row)) {
        rest <- toks[3]
        nums <- suppressWarnings(as.numeric(
          regmatches(rest, gregexpr("-?[0-9.]+(?:[Ee][+-]?[0-9]+)?", rest, perl = TRUE))[[1]]
        ))
        current_vals <- nums[!is.na(nums)]
      }
    } else if (!is.na(current_row) && length(toks) < 3) {
      # Continuation line: pure numerics, no leading identifier.  Title
      # lines (Primary, Multivariable, ...) have identifiers and we
      # explicitly skip them here by requiring the leading-identifier
      # regex to have *not* matched.
      nums <- suppressWarnings(as.numeric(
        regmatches(ln, gregexpr("-?[0-9.]+(?:[Ee][+-]?[0-9]+)?", ln, perl = TRUE))[[1]]
      ))
      current_vals <- c(current_vals, nums[!is.na(nums)])
    }
  }
  flush()

  for (p in pending) {
    n <- min(length(p$vals), k)
    if (n > 0L) M[p$row, seq_len(n)] <- p$vals[seq_len(n)]
  }

  for (i2 in seq_len(k)) for (j2 in seq_len(k)) {
    if (is.na(M[i2, j2]) && !is.na(M[j2, i2])) M[i2, j2] <- M[j2, i2]
  }
  M
}

# Split the file into per-fit blocks on "Initial Summary:" anchors.
.hzr_split_fits <- function(lines) {
  anchors <- grep("Initial Summary:", lines)
  if (!length(anchors)) return(list(lines))
  ends <- c(tail(anchors, -1L) - 1L, length(lines))
  mapply(function(s, e) lines[s:e], anchors, ends, SIMPLIFY = FALSE)
}

.hzr_parse_sas_lst <- function(path) {
  lines <- .hzr_read_lst(path)
  fits  <- .hzr_split_fits(lines)
  parsed <- lapply(fits, function(block) {
    obs <- .hzr_extract_obs_counts(block)
    ll  <- .hzr_extract_loglik(block)
    list(
      loglik            = if (length(ll)) ll[length(ll)] else NA_real_,
      loglik_trace      = ll,
      n_obs             = obs$n_obs,
      n_events          = obs$n_events,
      n_censored        = obs$n_censored,
      events_conserved  = .hzr_extract_events_conserved(block),
      params            = .hzr_extract_param_summary(block),
      natural           = .hzr_extract_natural(block),
      vcov              = .hzr_extract_matrix(block, "Asymptotic Variance-Covariance Matrix"),
      correlation       = .hzr_extract_matrix(block, "Asymptotic Correlation Matrix")
    )
  })
  list(fits = parsed)
}

# Default discovery: ~/Documents/GitHub/hazard/examples/ if it exists,
# else NULL.  Skip tests when fixtures are unavailable.
.hzr_sas_fixture_dir <- function() {
  env <- Sys.getenv("HAZARD_EXAMPLES_DIR", "")
  if (nzchar(env) && dir.exists(env)) return(env)
  default <- path.expand("~/Documents/GitHub/hazard/examples")
  if (dir.exists(default)) return(default)
  NA_character_
}

# ---------------------------------------------------------------------------
# OMC (PRIMISOL) dataset derivation
# ---------------------------------------------------------------------------
# The shipped `omc` R dataset has only the columns extracted at the source
# (study, te1, te2, te3, int_dead, dead, opdjul).  The OMC parity fixtures
# derive INT_TE / TE / STARTTME / CENSORED / NOPREVTE / MORBID via a
# multi-observation-per-patient DATA step that also needs te1djul/te2djul/
# te3djul, reop flags, fupdjul, and grade columns.  We reproduce that
# derivation here from the raw flat file.
#
# .hzr_omc_raw_path()     -- locate raw data file; returns NA_character_ if absent
# .hzr_derive_primisol()  -- parse + expand rows; returns list(te, te_mod, tm)

.hzr_omc_raw_path <- function() {
  env <- Sys.getenv("HAZARD_OMC_RAW", "")
  if (nzchar(env) && file.exists(env)) return(env)
  fixture_dir <- .hzr_sas_fixture_dir()
  if (!is.na(fixture_dir)) {
    cand <- file.path(fixture_dir, "data", "omc")
    if (file.exists(cand)) return(cand)
  }
  NA_character_
}

# Parse the SAS fixed-width raw file into a one-row-per-patient data frame.
# Column positions exactly match the SAS INPUT statement in hz.te123.OMC.sas.
.hzr_read_omc_raw <- function(path) {
  lines <- readLines(path, warn = FALSE)

  fld <- function(s, e = s) {
    v <- trimws(substr(lines, s, e))
    v[v == "."] <- NA_character_
    v
  }
  num <- function(s, e = s) suppressWarnings(as.numeric(fld(s, e)))
  chr <- function(s, e = s) fld(s, e)

  data.frame(
    study    = chr(1, 10),
    te1      = num(12),
    te2      = num(14),
    te3      = num(16),
    te1djul  = num(18, 25),
    te2djul  = num(27, 33),
    te3djul  = num(35, 41),
    avp_rp1  = chr(43),
    avp_rp2  = chr(45),
    avp_rp3  = chr(47),
    mvp_rp1  = chr(49),
    mvp_rp2  = chr(51),
    mvp_rp3  = chr(53),
    tvp_rp1  = chr(55),
    tvp_rp2  = chr(57),
    tvp_rp3  = chr(59),
    rp1djul  = num(61, 67),
    rp2djul  = num(69, 75),
    rp3djul  = num(77, 83),
    int_dead = num(111, 120),
    opdjul   = num(122, 128),
    dead     = num(130),
    fupdjul  = num(132, 138),
    te1grade = num(140),
    te2grade = num(142),
    te3grade = num(144),
    stringsAsFactors = FALSE
  )
}

# Reproduce the SAS PRIMISOL DATA step from hz.te123.OMC.sas and
# hz.tm123.OMC.sas.  Returns a list with three data frames:
#
#   te      -- for hz.te123.OMC fit 1 (left-truncated intervals, NOPREVTE)
#   te_mod  -- for hz.te123.OMC fit 2 (INT_TE adjusted to relative time,
#               + NOTE2 = NOPREVTE^2, NOTEE = exp(NOPREVTE))
#   tm      -- for hz.tm123.OMC    (same intervals, MORBID severity weight)
.hzr_derive_primisol <- function(raw_path) {
  d <- .hzr_read_omc_raw(raw_path)

  # IF TE1=1 AND TE1DJUL=. THEN DELETE
  d <- d[!(d$te1 == 1 & is.na(d$te1djul)), ]

  # Replace missing TE indicators with 0
  d$te1[is.na(d$te1)] <- 0L
  d$te2[is.na(d$te2)] <- 0L
  d$te3[is.na(d$te3)] <- 0L

  # Valve replacement: RP3 checked first, then RP2 overwrites, then RP1
  # overwrites (SAS IF statements execute in order 3->2->1; RP1 takes
  # precedence because it runs last and overwrites any earlier assignment).
  is_r <- function(x) !is.na(x) & trimws(x) == "R"
  n    <- nrow(d)
  REPL    <- rep(0L, n)
  REPLDJUL <- d$fupdjul   # default: censor at last follow-up

  for (rp_num in c(3L, 2L, 1L)) {
    avp <- is_r(d[[paste0("avp_rp", rp_num)]])
    mvp <- is_r(d[[paste0("mvp_rp", rp_num)]])
    tvp <- is_r(d[[paste0("tvp_rp", rp_num)]])
    idx <- avp | mvp | tvp
    REPL[idx]     <- 1L
    REPLDJUL[idx] <- d[[paste0("rp", rp_num, "djul")]][idx]
  }
  d$REPL    <- REPL
  d$REPLDJUL <- REPLDJUL

  # Censor TEs that occurred at or after valve replacement
  for (j in 1:3) {
    te_col   <- paste0("te",  j)
    djul_col <- paste0("te", j, "djul")
    mask <- d$REPL == 1L & d[[te_col]] == 1L &
            !is.na(d[[djul_col]]) & d[[djul_col]] >= d$REPLDJUL
    d[[te_col]][mask] <- 0L
  }

  # INT_REPL: time to valve replacement (or censoring if no replacement)
  d$INT_REPL <- d$int_dead
  repl_idx <- d$REPL == 1L
  d$INT_REPL[repl_idx] <-
    (d$REPLDJUL[repl_idx] - d$opdjul[repl_idx]) * 12 / 365.2425

  # -- Multi-row expansion -------------------------------------------------
  rows_te <- vector("list", n * 4L)   # pre-allocate generously
  rows_tm <- vector("list", n * 4L)
  nr_te <- 0L
  nr_tm <- 0L

  append_te <- function(study, te1, te2, te3, int_te, starttme, censored, te, noprevte) {
    nr_te <<- nr_te + 1L
    rows_te[[nr_te]] <<- data.frame(
      study = study, te1 = te1, te2 = te2, te3 = te3,
      int_te = int_te, starttme = starttme, censored = censored,
      te = te, noprevte = noprevte, stringsAsFactors = FALSE
    )
  }
  append_tm <- function(study, te1, te2, te3, int_te, starttme, censored, te, morbid) {
    nr_tm <<- nr_tm + 1L
    rows_tm[[nr_tm]] <<- data.frame(
      study = study, te1 = te1, te2 = te2, te3 = te3,
      int_te = int_te, starttme = starttme, censored = censored,
      te = te, morbid = morbid, stringsAsFactors = FALSE
    )
  }

  for (i in seq_len(nrow(d))) {
    r       <- d[i, ]
    study   <- trimws(r$study)
    te1_val <- r$te1
    te2_val <- r$te2
    te3_val <- r$te3

    # ---- Censored summary row (always output) ----------------------------
    # STARTTME advances through any events that preceded the censored interval.
    starttme <- 0
    noprevte <- 0L

    if (!is.na(te1_val) && te1_val == 1L) {
      starttme <- 12 * (r$te1djul - r$opdjul) / 365.2425
      noprevte <- 1L
    }
    # Special hospital-acquired TEs for two patients
    if (study == "039") starttme <- (3 / 24) * 12 / 365.2425
    if (study == "096") starttme <- (2 / 24) * 12 / 365.2425
    if (!is.na(te2_val) && te2_val == 1L) {
      starttme <- 12 * (r$te2djul - r$opdjul) / 365.2425
      noprevte <- 2L
    }
    if (!is.na(te3_val) && te3_val == 1L) {
      starttme <- 12 * (r$te3djul - r$opdjul) / 365.2425
      noprevte <- 3L
    }

    append_te(study, te1_val, te2_val, te3_val, r$INT_REPL, starttme, 1L, 0L, noprevte)
    append_tm(study, te1_val, te2_val, te3_val, r$INT_REPL, starttme, 1L, 0L, 0)

    # ---- TE1 event row ---------------------------------------------------
    if (!is.na(te1_val) && te1_val == 1L) {
      ite <- 12 * (r$te1djul - r$opdjul) / 365.2425
      if (study == "039") ite <- (3 / 24) * 12 / 365.2425
      if (study == "096") ite <- (2 / 24) * 12 / 365.2425
      append_te(study, te1_val, te2_val, te3_val, ite, 0, 0L, 1L, 0L)
      m1 <- if (is.na(r$te1grade)) 1.70 else r$te1grade
      append_tm(study, te1_val, te2_val, te3_val, ite, 0, 0L, 1L, m1)
    }

    # ---- TE2 event row ---------------------------------------------------
    if (!is.na(te2_val) && te2_val == 1L) {
      s2  <- 12 * (r$te1djul - r$opdjul) / 365.2425
      if (study == "039") s2 <- (3 / 24) * 12 / 365.2425
      if (study == "096") s2 <- (2 / 24) * 12 / 365.2425
      ite <- 12 * (r$te2djul - r$opdjul) / 365.2425
      append_te(study, te1_val, te2_val, te3_val, ite, s2, 0L, 1L, 1L)
      m2 <- if (is.na(r$te2grade)) 2.17 else r$te2grade
      append_tm(study, te1_val, te2_val, te3_val, ite, s2, 0L, 1L, m2)
    }

    # ---- TE3 event row ---------------------------------------------------
    if (!is.na(te3_val) && te3_val == 1L) {
      s2  <- 12 * (r$te2djul - r$opdjul) / 365.2425
      ite <- 12 * (r$te3djul - r$opdjul) / 365.2425
      append_te(study, te1_val, te2_val, te3_val, ite, s2, 0L, 1L, 2L)
      m3 <- if (is.na(r$te3grade)) 4.00 else r$te3grade
      append_tm(study, te1_val, te2_val, te3_val, ite, s2, 0L, 1L, m3)
    }
  }

  te <- do.call(rbind, rows_te[seq_len(nr_te)])
  tm <- do.call(rbind, rows_tm[seq_len(nr_tm)])

  # Drop zero/negative-duration rows (e.g. study 173: died same day as TE,
  # so the trailing censored row has STARTTME == INT_TE).  SAS HAZARD drops
  # these automatically when LCENSOR STARTTME is in effect.
  te <- te[te$int_te > te$starttme, ]
  tm <- tm[tm$int_te > tm$starttme, ]

  # hz.te123.OMC fit 2: modulated renewal — INT_TE relative to prior event
  te_mod <- te
  te_mod$int_te <- te_mod$int_te - te_mod$starttme
  te_mod$note2  <- te_mod$noprevte^2
  te_mod$notee  <- exp(te_mod$noprevte)
  # Drop any zero-duration modulated rows (relative interval would be 0)
  te_mod <- te_mod[te_mod$int_te > 0, ]

  list(te = te, te_mod = te_mod, tm = tm)
}

# SAS HAZARD parity tests
#
# Compare TemporalHazard::hazard() output to reference .lst captures from
# ~/Documents/GitHub/hazard/examples/.  Tests skip on CRAN and skip when
# the fixtures directory is unavailable.
#
# Reference parser lives in helper-sas-parity.R.
# Inventory + gap log: inst/dev/PRE-CRAN-PARITY-INVENTORY.md

skip_if_no_sas_fixtures <- function() {
  dir <- .hzr_sas_fixture_dir() # nolint: object_usage_linter.
  testthat::skip_if(is.na(dir), "SAS HAZARD fixture directory not available")
  dir
}

skip_if_no_omc_raw <- function() {
  p <- .hzr_omc_raw_path() # nolint: object_usage_linter.
  testthat::skip_if(is.na(p), "OMC raw data file not available")
  p
}

test_that("hz.deadp.KUL: 3-phase fixed-shape Weibull-tail matches SAS LL/MLEs", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()

  ref <- .hzr_parse_sas_lst(file.path(dir, "hz.deadp.KUL.lst"))$fits[[1]]
  expect_equal(ref$loglik,           -3740.52,  tolerance = 1e-2)
  expect_equal(ref$n_obs,            5880L)
  expect_equal(ref$n_events,         545L)

  data(cabgkul, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = FALSE)
  )

  # Final log-likelihood matches SAS to print precision.
  expect_equal(fit$fit$objective, ref$loglik, tolerance = 1e-2,
               label = "R log-likelihood vs SAS reference")

  # Free parameters on the optim scale (log_mu for each phase).  The full
  # theta vector includes fixed shape params; mask down to the 3 free.
  free <- !fit$fit$fixed_mask
  theta_free <- fit$fit$theta[free]
  V_free     <- fit$fit$vcov[free, free, drop = FALSE]
  expect_equal(length(theta_free), 3L)

  # Natural-scale MUE / MUC / MUL match.
  nat <- ref$natural
  mue_ref <- nat$estimate[nat$name == "MUE"]
  muc_ref <- nat$estimate[nat$name == "MUC"]
  mul_ref <- nat$estimate[nat$name == "MUL"]
  expect_equal(unname(exp(theta_free[1])), mue_ref, tolerance = 1e-3,
               label = "MUE (early) natural-scale")
  expect_equal(unname(exp(theta_free[2])), muc_ref, tolerance = 1e-3,
               label = "MUC (constant) natural-scale")
  expect_equal(unname(exp(theta_free[3])), mul_ref, tolerance = 1e-3,
               label = "MUL (late) natural-scale")

  # Std errors from the param-summary table (log-mu scale).
  params_ref <- ref$params
  se_e0 <- params_ref$se[params_ref$name == "E0"]
  se_c0 <- params_ref$se[params_ref$name == "C0"]
  se_l0 <- params_ref$se[params_ref$name == "L0"]
  expect_equal(sqrt(V_free[1, 1]), se_e0, tolerance = 5e-3,
               label = "SE(E0) vs SAS")
  expect_equal(sqrt(V_free[2, 2]), se_c0, tolerance = 5e-3,
               label = "SE(C0) vs SAS")
  expect_equal(sqrt(V_free[3, 3]), se_l0, tolerance = 5e-3,
               label = "SE(L0) vs SAS")

  # Off-diagonal vcov sanity (covariance vs SAS Asymptotic V-cov Matrix).
  expect_equal(V_free[1, 2], ref$vcov["E0", "C0"], tolerance = 5e-3,
               label = "cov(E0, C0)")
  expect_equal(V_free[2, 3], ref$vcov["C0", "L0"], tolerance = 5e-3,
               label = "cov(C0, L0)")
  expect_equal(V_free[1, 3], ref$vcov["E0", "L0"], tolerance = 5e-3,
               label = "cov(E0, L0)")
})

test_that("hz.death.AVC: 2-phase free-shape Early matches SAS LL/MLEs", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()

  ref <- .hzr_parse_sas_lst(file.path(dir, "hz.death.AVC.lst"))$fits[[1]]
  expect_equal(ref$loglik,    -210.501, tolerance = 1e-2)
  expect_equal(ref$n_obs,     310L)
  expect_equal(ref$n_events,  70L)

  data(avc, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = avc,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.1512, nu = 1.44, m = 1,
                            fixed = "m"),
      constant = hzr_phase("constant")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )

  expect_equal(fit$fit$objective, ref$loglik, tolerance = 1e-2,
               label = "R log-likelihood vs SAS reference")

  # Natural-scale parameters from "Estimates for Model Parameters" table.
  nat <- ref$natural
  thalf_ref <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  nu_ref    <- nat$estimate[nat$name == "NU"    & nat$phase == "Early"]
  mue_ref   <- nat$estimate[nat$name == "MUE"]
  muc_ref   <- nat$estimate[nat$name == "MUC"]

  th <- fit$fit$theta
  expect_equal(unname(exp(th["early.log_t_half"])), thalf_ref, tolerance = 1e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th["early.nu"]),              nu_ref,    tolerance = 1e-3,
               label = "NU (Early) natural-scale")
  expect_equal(unname(exp(th["early.log_mu"])),     mue_ref,   tolerance = 1e-3,
               label = "MUE natural-scale")
  expect_equal(unname(exp(th["constant.log_mu"])),  muc_ref,   tolerance = 1e-3,
               label = "MUC natural-scale (CoE-constrained)")

  # SE comparison: SAS reports log-scale SEs for E2=log_t_half and E0=log_mu;
  # E3=NU is on natural scale.  We compare log-scale SEs where R also
  # parameterizes on log scale.  CoE constraint zeroes C0 SE in R; SAS
  # still reports one — note as gap, don't assert.
  V <- fit$fit$vcov
  idx <- c(log_t_half = which(names(th) == "early.log_t_half"),
           log_mu     = which(names(th) == "early.log_mu"))

  se_e2_ref <- ref$params$se[ref$params$name == "E2"]
  expect_equal(sqrt(V[idx["log_t_half"], idx["log_t_half"]]), se_e2_ref,
               tolerance = 5e-3, label = "SE(log_t_half / E2)")
  # NOTE: SE(log_mu / E0) disagrees with SAS (R ~0.059 vs SAS 0.133).
  # SAS reports asymptotic vcov projected onto the CoE conservation
  # manifold; R returns the raw inverse Hessian on the free-parameter
  # submatrix.  Logged as P2 gap #11 in PRE-CRAN-PARITY-INVENTORY.md.
})

# ---------------------------------------------------------------------------
# hm.deadp.VALVES: nu = 0, m < 0 parametric-form correctness check
# ---------------------------------------------------------------------------
# The inventory P1 gap #5 originally claimed a ~17-unit LL gap at nu=0, m<0.
# Investigation showed R's null model matches SAS exactly (LL = -1864.76),
# confirming that hzr_decompos() Case 2L (m<0, nu=0) is correctly
# implemented.  The previously observed "gap" was a spurious comparison
# between R's covariate-model LL and the SAS stepwise null LL.
#
# The final covariate model (SELECTION SLS=0.1 MAXSTEPS=5, 7 free params)
# belongs to the stepwise harness (gap #3); R finds a better optimum
# (-1786) than SAS (-1804.78) because the stepwise initialization puts
# SAS in a different basin.  That is not a bug.
#
# This test verifies the parametric-form correctness only (null model).
test_that("hm.deadp.VALVES: null model nu=0 m<0 LL matches SAS initial (Case 2L check)", {
  testthat::skip_on_cran()
  skip_if_no_sas_fixtures()

  # SAS Initial Summary LL (no covariates, before stepwise selection):
  # shapes THALF=0.6781774 FIXTHALF, NU=0 FIXNU, M=-2.15596 FIXM; free: MUE, MUC.
  data(valves, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = valves,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.6781774, nu = 0, m = -2.15596,
                             fixed = c("t_half", "nu", "m")),
      constant = hzr_phase("constant")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = FALSE)
  )

  # SAS initial LL (from the first "Log likelihood = " before any step):
  # grep confirmed: -1864.76.
  expect_equal(fit$fit$objective, -1864.76, tolerance = 1e-2,
               label = "R null LL vs SAS initial (Case 2L: m<0, nu=0)")

  # Natural-scale mu values should be positive and finite.
  th <- fit$fit$theta
  expect_true(is.finite(th["early.log_mu"]),    label = "MUE finite")
  expect_true(is.finite(th["constant.log_mu"]), label = "MUC finite")

  # NOTE: the final stepwise covariate model (LL=-1804.78 with NYHA, I_PATH,
  # AGE_COP, BLACK) is tested via hzr_stepwise() in the stepwise harness
  # (gap #3).  R finds a better optimum (-1786) for that model because the
  # multi-start optimizer avoids the basin SAS's stepwise procedure falls
  # into.  This is not a bug; both optimize the same likelihood.
})

# ---------------------------------------------------------------------------
# hz.te123.OMC: 2-phase (Early CDF + Late Weibull) repeated TE events
# ---------------------------------------------------------------------------
# Two fits:
#   Fit 1 -- left-truncated (LCENSOR STARTTME), no covariates, CONSERVE
#   Fit 2 -- modulated renewal (INT_TE = INT_TE - STARTTME), late-phase
#            covariates NOPREVTE + NOTEE, CONSERVE
#
# Data derivation: PRIMISOL DATA step reproduced in .hzr_derive_primisol();
# raw fixed-width file from ~/Documents/GitHub/hazard/examples/data/omc.
#
# Known gaps:
#   P1 #6 (fit 1): CoE for early+late 2-phase differs from SAS CONSERVE.
#   SAS keeps all 5 mus free under Lagrange constraint; R fixes late.log_mu
#   via closed-form CoE, yielding LL offset ~0.14 and MUE shifted.
#   Shape params (THALF, NU, ETA) still match well; asserted below.
#   MUE NOT asserted (affected by P1 #6 gap).
test_that("hz.te123.OMC fit 1: left-truncated 2-phase shape params match SAS", {
  testthat::skip_on_cran()
  dir  <- skip_if_no_sas_fixtures()
  skip_if_no_omc_raw()

  ref  <- .hzr_parse_sas_lst(file.path(dir, "hz.te123.OMC.lst"))$fits[[1]]
  expect_equal(ref$loglik,   -322.226, tolerance = 1e-2)
  expect_equal(ref$n_obs,    382L)
  expect_equal(ref$n_events, 44L)

  raw      <- .hzr_omc_raw_path()
  primisol <- .hzr_derive_primisol(raw)

  # Verify the R derivation reproduces SAS observation counts.
  expect_equal(nrow(primisol$te), 382L,   label = "PRIMISOL n_obs")
  expect_equal(sum(primisol$te$te), 44L,  label = "PRIMISOL n_events")

  fit <- hazard(
    survival::Surv(starttme, int_te, te) ~ 1,
    data   = primisol$te,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 0.06, nu = -1.74, m = 0, fixed = "m"),
      late  = hzr_phase("g3",  tau = 1, gamma = 1, alpha = 1, eta = 1.32,
                         fixed = c("tau", "gamma", "alpha"))
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )

  # LL: P1 #6 gap produces ~0.14 offset; assert within 0.2.
  expect_equal(fit$fit$objective, ref$loglik, tolerance = 0.2,
               label = "R LL vs SAS (P1 #6 gap: CoE implementation differs)")

  # Shape params from "Estimates for Model Parameters" natural-scale table.
  nat       <- ref$natural
  thalf_ref <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  nu_ref    <- nat$estimate[nat$name == "NU"    & nat$phase == "Early"]
  eta_ref   <- nat$estimate[nat$name == "ETA"   & nat$phase == "Late"]

  th <- fit$fit$theta
  expect_equal(unname(exp(th["early.log_t_half"])), thalf_ref, tolerance = 5e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th["early.nu"]),              nu_ref,    tolerance = 5e-3,
               label = "NU (Early) natural-scale")
  expect_equal(unname(th["late.eta"]),              eta_ref,   tolerance = 5e-3,
               label = "ETA (Late) natural-scale")

  # NOTE: MUE NOT asserted — affected by P1 #6 CoE gap.
  # R: MUE ~ 0.01601; SAS: MUE = 0.01740.
})

test_that("hz.te123.OMC fit 2: modulated renewal + late covariates matches SAS LL/MLEs", {
  testthat::skip_on_cran()
  dir  <- skip_if_no_sas_fixtures()
  skip_if_no_omc_raw()

  ref  <- .hzr_parse_sas_lst(file.path(dir, "hz.te123.OMC.lst"))$fits[[2]]
  expect_equal(ref$loglik,   -311.597, tolerance = 1e-2)
  expect_equal(ref$n_obs,    382L)
  expect_equal(ref$n_events, 44L)

  raw      <- .hzr_omc_raw_path()
  primisol <- .hzr_derive_primisol(raw)

  # Fit 2: modulated renewal.  INT_TE = INT_TE - STARTTME (relative time);
  # no LCENSOR; late phase has covariates NOPREVTE and NOTEE.
  fit <- hazard(
    survival::Surv(int_te, te) ~ 1,
    data   = primisol$te_mod,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 0.053, nu = -1.738, m = 0, fixed = "m"),
      late  = hzr_phase("g3",  tau = 1, gamma = 1, alpha = 1, eta = 1,
                         fixed = "shapes",
                         formula = ~ noprevte + notee)
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )

  expect_equal(fit$fit$objective, ref$loglik, tolerance = 1e-2,
               label = "R LL vs SAS (fit 2)")

  nat       <- ref$natural
  thalf_ref <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  nu_ref    <- nat$estimate[nat$name == "NU"    & nat$phase == "Early"]
  mue_ref   <- nat$estimate[nat$name == "MUE"]
  mul_ref   <- nat$estimate[nat$name == "MUL"]

  th <- fit$fit$theta
  expect_equal(unname(exp(th["early.log_t_half"])), thalf_ref, tolerance = 1e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th["early.nu"]),              nu_ref,    tolerance = 1e-3,
               label = "NU (Early) natural-scale")
  expect_equal(unname(exp(th["early.log_mu"])),     mue_ref,   tolerance = 1e-3,
               label = "MUE (Early) natural-scale")
  expect_equal(unname(exp(th["late.log_mu"])),      mul_ref,   tolerance = 1e-3,
               label = "MUL (Late) natural-scale")

  # Late-phase covariate coefficients — compare to SAS Parameter Estimate Summary.
  params_ref <- ref$params
  np_ref  <- params_ref$estimate[params_ref$name == "NOPREVTE"]
  ne_ref  <- params_ref$estimate[params_ref$name == "NOTEE"]
  expect_equal(unname(th["late.noprevte"]), np_ref, tolerance = 1e-3,
               label = "NOPREVTE coefficient")
  expect_equal(unname(th["late.notee"]),    ne_ref, tolerance = 1e-3,
               label = "NOTEE coefficient")

  # SEs from the parameter-estimate summary table (log-scale where applicable).
  V <- fit$fit$vcov
  param_idx <- function(nm) which(names(th) == nm)
  se_e2_ref  <- params_ref$se[params_ref$name == "E2"]
  se_np_ref  <- params_ref$se[params_ref$name == "NOPREVTE"]
  se_ne_ref  <- params_ref$se[params_ref$name == "NOTEE"]
  expect_equal(sqrt(V[param_idx("early.log_t_half"), param_idx("early.log_t_half")]),
               se_e2_ref, tolerance = 5e-3, label = "SE(log_t_half / E2)")
  expect_equal(sqrt(V[param_idx("late.noprevte"), param_idx("late.noprevte")]),
               se_np_ref, tolerance = 5e-2, label = "SE(NOPREVTE)")
  expect_equal(sqrt(V[param_idx("late.notee"),    param_idx("late.notee")]),
               se_ne_ref, tolerance = 5e-2, label = "SE(NOTEE)")
})

# ---------------------------------------------------------------------------
# hz.tm123.OMC: 2-phase (Early CDF + Late Weibull) permanent morbidity
# ---------------------------------------------------------------------------
# Uses WEIGHT MORBID: each TE event is weighted by severity grade.
# SAS WEIGHT applies the weight to the event log-hazard term only;
# censored intervals contribute to the survival function with weight = 1.
# R: pass weights = ifelse(te==1, morbid, 1) to reproduce this semantics.
#
# Known gaps:
#   Same P1 #6 gap as fit 1 above (CoE for early+late, MUE shifted ~10%).
#   LL offset ~0.43.  THALF and ETA assert within 5e-3.
test_that("hz.tm123.OMC: morbidity-weighted 2-phase shape params match SAS", {
  testthat::skip_on_cran()
  dir  <- skip_if_no_sas_fixtures()
  skip_if_no_omc_raw()

  ref  <- .hzr_parse_sas_lst(file.path(dir, "hz.tm123.OMC.lst"))$fits[[1]]
  expect_equal(ref$loglik,   -581.528, tolerance = 1e-2)
  expect_equal(ref$n_obs,    382L)
  expect_equal(ref$n_events, 44L)

  raw      <- .hzr_omc_raw_path()
  primisol <- .hzr_derive_primisol(raw)

  # WEIGHT MORBID: apply severity weight to event rows; censored rows use
  # weight = 1 so they contribute normally to the survival function.
  wt <- ifelse(primisol$tm$te == 1L, primisol$tm$morbid, 1)

  fit <- hazard(
    survival::Surv(starttme, int_te, te) ~ 1,
    data    = primisol$tm,
    dist    = "multiphase",
    phases  = list(
      early = hzr_phase("cdf", t_half = 0.0109, nu = 1, m = 1e-6,
                         fixed = c("nu", "m")),
      late  = hzr_phase("g3",  tau = 1, gamma = 1, alpha = 1, eta = 1.4112,
                         fixed = c("tau", "gamma", "alpha"))
    ),
    weights = wt,
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )

  # LL: P1 #6 gap produces ~0.43 offset; assert within 0.5.
  expect_equal(fit$fit$objective, ref$loglik, tolerance = 0.5,
               label = "R LL vs SAS (P1 #6 gap: CoE implementation differs)")

  nat       <- ref$natural
  thalf_ref <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  eta_ref   <- nat$estimate[nat$name == "ETA"   & nat$phase == "Late"]

  th <- fit$fit$theta
  expect_equal(unname(exp(th["early.log_t_half"])), thalf_ref, tolerance = 5e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th["late.eta"]),              eta_ref,   tolerance = 5e-3,
               label = "ETA (Late) natural-scale")

  # NOTE: MUE NOT asserted — P1 #6 CoE gap shifts MUE ~10% (R 0.01643 vs SAS 0.01828).
  # NOTE: NU=1 FIXNU (fixed); M=1e-6 FIXM (fixed) — not compared.
})

#' @importFrom utils write.csv read.table
#' @keywords internal
NULL

# parity-helpers.R -- Helpers for cross-validating R output against the legacy C binary
#
# STATUS: Stub / scaffold (M1 infrastructure, awaiting C binary integration)
#
# PURPOSE
# -------
# These functions wrap the compiled HAZARD C binary so the test suite can invoke
# it programmatically, capture its output, and compare parameter estimates with
# those produced by the R implementation.  This cross-validation is the primary
# correctness signal during the SAS->R migration.
#
# CURRENT STATE
# -------------
# .hzr_get_hazard_binary() -- locates binary via env var / option / package fallback
# .hzr_run_hazard_binary() -- writes working files, builds a .sas control file,
#                             and invokes the legacy executable as:
#                             hazard.exe < prefix.sas > prefix.lst 2>&1
# .hzr_parse_hazard_output() -- parses parameter tables from the .lst output.
#
# ENABLING C PARITY TESTS
# -----------------------
# 1. Prefer external path via TEMPORAL_HAZARD_BIN or options(temporal_hazard.binary=...)
#    for local acceptance testing.
# 2. Optionally place the compiled binary at inst/bin/hazard (chmod +x) for local use.
# 3. Complete .hzr_run_hazard_binary(): actual system() call + output parser.
# 4. Update test-parity-core.R to call .hzr_run_hazard_binary() instead of
#    loading .rds fixtures where true C-binary comparison is desired.
#
# Until the binary is wired up, parity tests use synthetic R-generated fixtures
# produced by golden_fixtures.R.

# Parity Testing Helpers
#
# Functions to generate golden fixtures from the C reference hazard binary
# and validate R implementation against them.

#' Get path to compiled hazard binary
#'
#' @noRd
.hzr_get_hazard_binary <- function(binary = c("hazard", "hazpred")) {
  binary <- match.arg(binary)
  env_var <- if (identical(binary, "hazpred")) {
    "TEMPORAL_HAZPRED_BIN"
  } else {
    "TEMPORAL_HAZARD_BIN"
  }
  option_name <- if (identical(binary, "hazpred")) {
    "temporal_hazard.hazpred_binary"
  } else {
    "temporal_hazard.binary"
  }

  external_path <- Sys.getenv(env_var, unset = "")
  if (!nzchar(external_path)) {
    external_path <- getOption(option_name, default = "")
  }

  if (nzchar(external_path)) {
    if (!file.exists(external_path)) {
      stop(
        "Binary path from ", env_var, "/options(", option_name, "=...) does not exist: ",
        external_path,
        call. = FALSE
      )
    }
    external_path
  } else {
    # Fallback to package-local binary in inst/bin.
    # Return NA if not available (allows graceful skipping in tests)
    local_candidates <- c(binary, paste0(binary, ".exe"))
    for (candidate in local_candidates) {
      bin_path <- system.file("bin", candidate, package = "TemporalHazard")
      if (nzchar(bin_path) && file.exists(bin_path)) {
        return(bin_path)
      }
    }
    NA
  }
}

#' Sanitise variable names for legacy SAS control files
#'
#' @param x Character vector of candidate names
#'
#' @return Uppercase names limited to SAS-friendly identifier characters
#'
#' @noRd
.hzr_as_legacy_names <- function(x) {
  if (length(x) == 0L) {
    character()
  } else {
    x <- toupper(gsub("[^A-Za-z0-9_]", "_", x))
    x <- gsub("^[^A-Za-z_]+", "X", x)
    make.unique(x, sep = "_")
  }
}

#' Infer how many leading theta elements are baseline shape parameters
#'
#' @param dist Distribution name
#' @param control Optional control list
#'
#' @return Integer scalar
#'
#' @noRd
.hzr_shape_parameter_count <- function(dist, control = list()) {
  if (!is.null(control$shape_param_count)) {
    as.integer(control$shape_param_count)
  } else {
    switch(
      dist,
      weibull = 2L,
      exponential = 1L,
      loglogistic = 2L,
      lognormal = 2L,
      0L
    )
  }
}

#' Split theta into baseline and covariate components for legacy output
#'
#' @param theta Numeric parameter vector
#' @param x_names Legacy covariate names used in the data extract
#' @param dist Distribution name
#' @param control Optional control list
#'
#' @return List with entries baseline and covariates
#'
#' @noRd
.hzr_split_theta_for_legacy <- function(theta, x_names, dist, control = list()) {
  if (is.null(theta) || !length(theta)) {
    list(baseline = numeric(), covariates = numeric())
  } else {
    theta_names <- names(theta)
    if (is.null(theta_names)) {
      theta_names <- rep("", length(theta))
    }
    theta_names_upper <- .hzr_as_legacy_names(
      ifelse(nzchar(theta_names), theta_names, paste0("TH", seq_along(theta)))
    )
    x_names_upper <- .hzr_as_legacy_names(x_names)

    matched_covariates <- theta_names_upper %in% x_names_upper
    if (any(matched_covariates)) {
      baseline <- theta[!matched_covariates]
      covariates <- theta[matched_covariates]
      names(baseline) <- theta_names_upper[!matched_covariates]
      names(covariates) <- theta_names_upper[matched_covariates]
      list(baseline = baseline, covariates = covariates)
    } else {
      shape_count <- .hzr_shape_parameter_count(dist = dist, control = control)
      shape_count <- min(shape_count, length(theta))
      baseline <- theta[seq_len(shape_count)]
      covariates <- if (length(theta) > shape_count) {
        theta[seq.int(shape_count + 1L, length(theta))]
      } else {
        numeric()
      }

      names(baseline) <- theta_names_upper[seq_len(shape_count)]
      if (length(covariates)) {
        covariate_names <- if (length(x_names_upper) >= length(covariates)) {
          x_names_upper[seq_along(covariates)]
        } else {
          theta_names_upper[seq.int(shape_count + 1L, length(theta))]
        }
        names(covariates) <- covariate_names
      }

      list(baseline = baseline, covariates = covariates)
    }
  }
}

#' Format a named numeric vector as SAS assignments
#'
#' @param values Named numeric vector
#'
#' @return Length-one character scalar
#'
#' @noRd
.hzr_format_legacy_assignments <- function(values) {
  if (!length(values)) {
    ""
  } else {
    parts <- paste0(
      names(values), "=",
      format(unname(values), digits = 10, scientific = TRUE, trim = TRUE)
    )
    paste(parts, collapse = ", ")
  }
}

#' Build PROC HAZARD option tokens from control settings
#'
#' @param control Optional control list
#'
#' @return Character vector of PROC option tokens
#'
#' @noRd
.hzr_format_proc_options <- function(control = list()) {
  options <- character()

  if (isTRUE(control$nocov)) {
    options <- c(options, "NOCOV")
  }
  if (isTRUE(control$nocor)) {
    options <- c(options, "NOCOR")
  }
  if (isTRUE(control$p)) {
    options <- c(options, "P")
  }
  if (isTRUE(control$conserve)) {
    options <- c(options, "CONSERVE")
  }
  if (isTRUE(control$quasi)) {
    options <- c(options, "QUASI")
  }
  if (!is.null(control$condition)) {
    options <- c(options, paste0("CONDITION=", control$condition))
  }
  if (!is.null(control$outhaz)) {
    options <- c(options, paste0("OUTHAZ=", .hzr_as_legacy_names(control$outhaz)))
  }

  options
}

#' Build a %HAZARD control file
#'
#' @param data_name Logical SAS dataset token to embed in PROC HAZARD
#' @param data_file Path to the temporary flat data file
#' @param time_var Legacy time variable name
#' @param status_var Legacy event variable name
#' @param x_names Legacy covariate names
#' @param dist Distribution name
#' @param theta Optional starting parameter vector
#' @param control Optional control list
#'
#' @return Character vector of SAS/control lines
#'
#' @noRd
.hzr_build_hazard_sas_input <- function(
    data_name,
    data_file,
    time_var,
    status_var,
    x_names = character(),
    dist,
    theta = NULL,
    control = list()) {

  sas_lines <- control$sas_lines %||% NULL
  if (!is.null(sas_lines)) {
    return(as.character(sas_lines))
  }

  proc_options <- .hzr_format_proc_options(control)
  proc_line <- paste(
    "PROC HAZARD",
    paste0("DATA=", data_name),
    paste(proc_options, collapse = " ")
  )
  proc_line <- trimws(gsub("[[:space:]]+", " ", proc_line))
  proc_line <- paste0(proc_line, ";")

  theta_parts <- .hzr_split_theta_for_legacy(theta = theta, x_names = x_names, dist = dist, control = control)
  parms_line <- if (length(theta_parts$baseline)) {
    paste0("     PARMS ", .hzr_format_legacy_assignments(theta_parts$baseline), ";")
  } else {
    NULL
  }

  fix_tokens <- control$fix %||% character()
  if (length(fix_tokens) && !is.null(parms_line)) {
    fix_tokens <- paste0("FIX", .hzr_as_legacy_names(fix_tokens))
    parms_line <- paste0(sub(";$", "", parms_line), " ", paste(fix_tokens, collapse = " "), ";")
  }

  phase_assignments <- control$phase_assignments %||% NULL
  covariate_lines <- character()
  if (length(theta_parts$covariates)) {
    if (is.null(phase_assignments)) {
      covariate_lines <- paste0(
        "     EARLY ",
        .hzr_format_legacy_assignments(theta_parts$covariates),
        ";"
      )
    } else if (is.list(phase_assignments)) {
      for (phase_name in names(phase_assignments)) {
        vars <- .hzr_as_legacy_names(unlist(phase_assignments[[phase_name]], use.names = FALSE))
        values <- theta_parts$covariates[names(theta_parts$covariates) %in% vars]
        if (length(values)) {
          covariate_lines <- c(
            covariate_lines,
            paste0("     ", toupper(phase_name), " ", .hzr_format_legacy_assignments(values), ";")
          )
        }
      }
    }
  }

  c(
    "%HAZARD(",
    paste("* TEMPORAL_HAZARD_DATA_FILE:", normalizePath(data_file, winslash = "/", mustWork = FALSE), ";"),
    paste("* DIST:", toupper(dist), ";"),
    proc_line,
    paste0("     TIME ", time_var, ";"),
    paste0("     EVENT ", status_var, ";"),
    parms_line,
    covariate_lines,
    ");"
  )
}

#' Build a %HAZPRED control file
#'
#' @param data_name Logical SAS dataset token to embed in PROC HAZARD
#' @param input_data_name Optional IN= dataset token for predictions
#' @param output_data_name Optional OUT= dataset token for predictions
#' @param time_var Legacy time variable name
#' @param control Optional control list
#'
#' @return Character vector of SAS/control lines
#'
#' @noRd
.hzr_build_hazpred_sas_input <- function(
    data_name,
    input_data_name,
    output_data_name,
    time_var,
    control = list()) {

  sas_lines <- control$sas_lines %||% NULL
  if (!is.null(sas_lines)) {
    return(as.character(sas_lines))
  }

  predict_types <- control$predict_types
  if (is.null(predict_types)) {
    predict_types <- c("SURVIVAL", "HAZARD")
  }
  predict_types <- toupper(predict_types)
  proc_parts <- c(
    "PROC HAZARD",
    paste0("DATA=", data_name)
  )
  if (!is.null(input_data_name)) {
    proc_parts <- c(proc_parts, paste0("IN=", input_data_name))
  }
  if (!is.null(output_data_name)) {
    proc_parts <- c(proc_parts, paste0("OUT=", output_data_name))
  }

  c(
    "%HAZPRED(",
    paste0(trimws(gsub("[[:space:]]+", " ", paste(proc_parts, collapse = " "))), ";"),
    paste0("     TIME ", time_var, ";"),
    paste0("     PREDICT ", paste(predict_types, collapse = " "), ";"),
    ");"
  )
}

#' Execute legacy binary using stdin/stdout redirection
#'
#' @param bin_path Path to executable
#' @param sas_file Path to control input file
#' @param lst_file Path to listing output file
#'
#' @return List with command string, status code, stdout text, and output lines
#'
#' @noRd
.hzr_run_legacy_binary <- function(bin_path, sas_file, lst_file) {
  command <- paste(
    shQuote(bin_path),
    "<",
    shQuote(sas_file),
    ">",
    shQuote(lst_file),
    "2>&1"
  )

  status_code <- system(command, intern = FALSE, ignore.stdout = FALSE, ignore.stderr = FALSE, wait = TRUE)
  output_text <- if (file.exists(lst_file)) readLines(lst_file, warn = FALSE) else character()

  list(
    call_string = command,
    status_code = unname(as.integer(status_code)),
    stdout = paste(output_text, collapse = "\n"),
    output_text = output_text
  )
}

#' Run hazard C binary on input data
#'
#' Invokes the compiled \code{hazard} executable with prepared input and parses output.
#' The legacy executable is called using shell redirection as
#' \code{hazard.exe < prefix.sas > prefix.lst 2>&1}.
#'
#' @param time Numeric vector of follow-up times
#' @param status Numeric vector of event indicators (0/1)
#' @param x Optional design matrix (univariable if NULL; multivar if provided)
#' @param theta Starting parameter vector (names must match C code expectations)
#' @param dist Distribution name ("weibull", etc.)
#' @param control Optional control list (nocov, nocor, condition, etc.)
#' @param data_name Character name for temporary data file prefix
#' @param binary Which legacy executable to run: \code{"hazard"} or \code{"hazpred"}
#'
#' @return List with:
#'   - \code{call_string}: Full command-line invocation for reproducibility
#'   - \code{stdout}: Raw standard output from hazard binary
#'   - \code{parsed}: Parsed results (data frame with estimates, SE, z, p-values)
#'   - \code{info}: Metadata (df, condition, convergence flags)
#'
#' @noRd
.hzr_run_hazard_binary <- function(
    time,
    status,
    x = NULL,
    theta = NULL,
    dist = "weibull",
    control = list(),
    data_name = "golden_test",
    binary = c("hazard", "hazpred")) {

  binary <- match.arg(binary)
  bin_path <- .hzr_get_hazard_binary(binary = binary)
  if (is.na(bin_path)) {
    return(list(
      call_string = NA_character_,
      stdout = "",
      parsed = .hzr_parse_hazard_output(character(), theta = theta),
      info = list(
        dist = dist,
        n = length(time),
        events = sum(status),
        binary = binary,
        binary_path = NA_character_,
        status_code = NA_integer_,
        sas_file = NA_character_,
        lst_file = NA_character_
      )
    ))
  }

  temp_dir <- tempdir()

  prefix <- file.path(temp_dir, data_name)
  data_file <- file.path(temp_dir, paste0(data_name, ".csv"))
  sas_file <- file.path(temp_dir, paste0(data_name, ".sas"))
  lst_file <- file.path(temp_dir, paste0(data_name, ".lst"))

  time_var <- .hzr_as_legacy_names(control$time_var %||% "TIME")
  status_var <- .hzr_as_legacy_names(control$status_var %||% "STATUS")
  dataset_name <- .hzr_as_legacy_names(control$data_name %||% toupper(data_name))

  df <- data.frame(
    time,
    as.integer(status)
  )
  names(df) <- c(time_var, status_var)

  if (!is.null(x)) {
    if (is.vector(x)) {
      x_names <- .hzr_as_legacy_names(colnames(as.matrix(x)) %||% "X1")
      df[[x_names[1L]]] <- x
    } else {
      x_names <- .hzr_as_legacy_names(colnames(x) %||% paste0("X", seq_len(ncol(x))))
      for (i in seq_len(ncol(x))) {
        df[[x_names[i]]] <- x[, i]
      }
    }
  } else {
    x_names <- character()
  }

  # Persist a simple flat data extract alongside the control program.
  write.csv(df, data_file, row.names = FALSE)

  if (!is.null(control$sas_file)) {
    sas_file <- control$sas_file
  } else {
    sas_lines <- if (identical(binary, "hazpred")) {
      .hzr_build_hazpred_sas_input(
        data_name = dataset_name,
        input_data_name = .hzr_as_legacy_names(control$input_data_name %||% dataset_name),
        output_data_name = .hzr_as_legacy_names(control$output_data_name %||% paste0(dataset_name, "_OUT")),
        time_var = time_var,
        control = control
      )
    } else {
      .hzr_build_hazard_sas_input(
        data_name = dataset_name,
        data_file = data_file,
        time_var = time_var,
        status_var = status_var,
        x_names = x_names,
        dist = dist,
        theta = theta,
        control = control
      )
    }
    writeLines(sas_lines, sas_file, useBytes = TRUE)
  }

  exec_result <- .hzr_run_legacy_binary(
    bin_path = bin_path,
    sas_file = sas_file,
    lst_file = lst_file
  )

  parsed <- if (!is.na(exec_result$status_code) && exec_result$status_code == 0L) {
    .hzr_parse_hazard_output(exec_result$output_text, theta = theta)
  } else {
    structure(
      data.frame(
        parameter = character(),
        estimate = numeric(),
        std_err = numeric(),
        z_stat = numeric(),
        p_value = numeric(),
        stringsAsFactors = FALSE
      ),
      output_text = paste(exec_result$output_text, collapse = "\n"),
      parse_status = "binary_failed"
    )
  }

  list(
    call_string = exec_result$call_string,
    stdout = exec_result$stdout,
    parsed = parsed,
    info = list(
      dist = dist,
      n = length(time),
      events = sum(status),
      binary = binary,
      binary_path = bin_path,
      status_code = exec_result$status_code,
      prefix = prefix,
      data_file = data_file,
      sas_file = sas_file,
      lst_file = lst_file
    )
  )
}

#' Parse hazard binary output
#'
#' Parses tabular output from the C hazard binary into a structured data frame.
#' Expects columns: parameter, estimate, StdErr (or SE), z-value (or z), p-value (or pval).
#' The parser is flexible and case-insensitive for column matching.
#'
#' @param output_text Character vector (lines) from binary output file
#' @param theta Optional parameter vector for context/validation
#'
#' @return Data frame with columns: parameter, estimate, std_err, z_stat, p_value
#' @keywords internal
.hzr_parse_hazard_output <- function(output_text, theta = NULL) {
  if (length(output_text) == 0 || all(nchar(trimws(output_text)) == 0)) {
    # Empty output: return empty data frame with proper structure
    return(structure(
      data.frame(
        parameter = character(),
        estimate = numeric(),
        std_err = numeric(),
        z_stat = numeric(),
        p_value = numeric(),
        stringsAsFactors = FALSE
      ),
      output_text = "",
      parse_status = "empty"
    ))
  }

  # Join text into single string and parse as table
  text <- paste(output_text, collapse = "\n")
  text_trimmed <- trimws(text)

  # Try to read as whitespace-separated table (skip comment lines)
  tryCatch(
    {
      lines <- strsplit(text_trimmed, "\n")[[1]]
      # Find header line (contains keywords like 'Parameter', 'Estimate', etc.)
      header_idx <- grep("(?i)(parameter|estimate|std|err|z[-_]?stat|p[-_]?val)", lines, perl = TRUE)[1]

      if (is.na(header_idx)) {
        warning("Could not identify header line in hazard output", call. = FALSE)
        return(structure(
          data.frame(
            parameter = character(),
            estimate = numeric(),
            std_err = numeric(),
            z_stat = numeric(),
            p_value = numeric(),
            stringsAsFactors = FALSE
          ),
          output_text = text,
          parse_status = "no_header"
        ))
      }

      # Read the table from R data (skip header line)
      data_start <- header_idx + 1
      data_lines <- lines[data_start:length(lines)]
      data_text <- paste(data_lines, collapse = "\n")

      # Parse as fixed-format or space-separated table
      df <- read.table(
        text = data_text,
        header = FALSE,
        colClasses = "character",
        fill = TRUE,
        na.strings = c("NA", ".", "NaN", "Inf"),
        stringsAsFactors = FALSE
      )

      if (nrow(df) == 0) {
        return(structure(
          data.frame(
            parameter = character(),
            estimate = numeric(),
            std_err = numeric(),
            z_stat = numeric(),
            p_value = numeric(),
            stringsAsFactors = FALSE
          ),
          output_text = text,
          parse_status = "empty_table"
        ))
      }

      # Map columns based on position and header keywords
      # Standard format: Parameter, Estimate, StdErr, Z, P
      # This is flexible and should handle variations

      # Assume first column is parameter names, next 4 are numeric
      param_col <- df[[1]]
      est_col <- suppressWarnings(as.numeric(df[[2]]))
      se_col <- suppressWarnings(as.numeric(df[[3]]))
      z_col <- suppressWarnings(as.numeric(df[[4]]))
      p_col <- suppressWarnings(as.numeric(df[[5]]))

      result <- data.frame(
        parameter = as.character(param_col),
        estimate = est_col,
        std_err = se_col,
        z_stat = z_col,
        p_value = p_col,
        stringsAsFactors = FALSE
      )

      structure(
        result,
        output_text = text,
        parse_status = "success"
      )
    },
    error = function(e) {
      warning("Failed to parse hazard binary output: ", e$message, call. = FALSE)
      structure(
        data.frame(
          parameter = character(),
          estimate = numeric(),
          std_err = numeric(),
          z_stat = numeric(),
          p_value = numeric(),
          stringsAsFactors = FALSE
        ),
        output_text = text,
        parse_status = paste("error:", e$message)
      )
    }
  )
}

#' Generate golden fixture for univariable Weibull model
#'
#' Runs the C hazard binary on a dataset and stores results as test fixture.
#'
#' @param fixture_name Character identifying the fixture (e.g., "avc_death_hz")
#' @param time Numeric vector of follow-up times
#' @param status Numeric vector of event indicators
#' @param x Optional design matrix for multivariable models
#' @param theta Optional starting parameters
#' @param control Optional control list
#' @param output_dir Directory to store the fixture. Required; the function
#'   never writes to a default path. Pass \code{tempdir()} for a throwaway
#'   run or an explicit project path (e.g. \code{"inst/fixtures"}) to persist.
#'
#' @return Invisibly returns the parsed output of the reference binary.
#'
#' @noRd
.hzr_generate_golden_fixture <- function(
    fixture_name,
    time,
    status,
    x = NULL,
    theta = NULL,
    control = list(),
    output_dir) {

  stopifnot(
    "`output_dir` must be a single non-empty path" =
      is.character(output_dir) && length(output_dir) == 1L && nzchar(output_dir)
  )
  dir.create(output_dir, showWarnings = FALSE, recursive = TRUE)

  # Run binary
  result <- .hzr_run_hazard_binary(
    time = time,
    status = status,
    x = x,
    theta = theta,
    control = control,
    data_name = fixture_name
  )

  # Save fixture as RDS for reproducible testing
  fixture_file <- file.path(output_dir, paste0(fixture_name, ".rds"))
  saveRDS(result, fixture_file)

  message("Generated golden fixture: ", fixture_file)

  invisible(result$parsed)
}

# C Binary Parity Tests
#
# These tests compare R package results against the C hazard binary.
# They are optional and will skip gracefully if the binary is not available.
# To enable: set environment variable TEMPORAL_HAZARD_BIN=/path/to/hazard
# or use options(temporal_hazard.binary="/path/to/hazard")

test_that("hazard binary can be located (if available)", {
  bin <- TemporalHazard:::.hzr_get_hazard_binary()
  if (is.na(bin)) {
    skip("C hazard binary not available")
  }
  expect_true(file.exists(bin))
  expect_match(bin, "hazard")
})

test_that("legacy command uses stdin/stdout redirection", {
  # Skip on Windows where /bin/cat does not exist
  skip_if(
    .Platform$OS.type != "unix",
    "Unix shell tools not available (Windows)"
  )

  temp_dir <- tempdir()
  sas_file <- file.path(temp_dir, "test-prefix.sas")
  lst_file <- file.path(temp_dir, "test-prefix.lst")
  on.exit(unlink(c(sas_file, lst_file)), add = TRUE)
  writeLines("Parameter Estimate StdErr Z.value P.value", sas_file)

  result <- TemporalHazard:::.hzr_run_legacy_binary(
    bin_path = "/bin/cat",
    sas_file = sas_file,
    lst_file = lst_file
  )

  expect_match(result$call_string, "<")
  expect_match(result$call_string, ">")
  expect_match(result$call_string, "2>&1", fixed = TRUE)
  expect_equal(result$status_code, 0)
  expect_true(file.exists(lst_file))
})

test_that("build_hazard_sas_input emits legacy %HAZARD block", {
  sas_lines <- TemporalHazard:::.hzr_build_hazard_sas_input(
    data_name = "GOLDEN_TEST",
    data_file = "/tmp/golden_test.csv",
    time_var = "INT_DEAD",
    status_var = "DEAD",
    x_names = c("AGE", "STATUS"),
    dist = "weibull",
    theta = c(MUE = 0.2, NU = 1.4, AGE = -0.03, STATUS = 0.5),
    control = list(nocov = TRUE, nocor = TRUE, condition = 14, fix = "M")
  )

  sas_text <- paste(sas_lines, collapse = "\n")
  expect_match(sas_text, "%HAZARD\\(")
  expect_match(sas_text, "PROC HAZARD DATA=GOLDEN_TEST NOCOV NOCOR CONDITION=14;")
  expect_match(sas_text, "TIME INT_DEAD;")
  expect_match(sas_text, "EVENT DEAD;")
  expect_match(sas_text, "PARMS MUE=2.0e-01, NU=1.4e\\+00 FIXM;")
  expect_match(sas_text, "EARLY AGE=-3e-02, STATUS=5e-01;")
})

test_that("build_hazpred_sas_input emits legacy %HAZPRED block", {
  sas_lines <- TemporalHazard:::.hzr_build_hazpred_sas_input(
    data_name = "OUTEST",
    input_data_name = "AVCS",
    output_data_name = "HPOUT",
    time_var = "INT_DEAD",
    control = list(predict_types = c("survival", "hazard"))
  )

  sas_text <- paste(sas_lines, collapse = "\n")
  expect_match(sas_text, "%HAZPRED\\(")
  expect_match(sas_text, "PROC HAZARD DATA=OUTEST IN=AVCS OUT=HPOUT")
  expect_match(sas_text, "TIME INT_DEAD;")
  expect_match(sas_text, "PREDICT SURVIVAL HAZARD;")
})

test_that("parse_hazard_output handles empty output gracefully", {
  result <- TemporalHazard:::.hzr_parse_hazard_output(character())
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 0)
  expect_named(result, c("parameter", "estimate", "std_err", "z_stat", "p_value"))
})

test_that("parse_hazard_output parses standard parameter table", {
  # Simulate standard statistical output format
  output <- c(
    "Parameter    Estimate      StdErr      Z.value        P.value",
    "shape        1.234567      0.054321    22.725000      0.000000",
    "scale        10.987654     0.321098    34.225000      0.000000"
  )

  result <- TemporalHazard:::.hzr_parse_hazard_output(output)
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_equal(result$parameter, c("shape", "scale"))
  expect_true(all(!is.na(result$estimate)))
  expect_true(all(!is.na(result$std_err)))
})

test_that("run_hazard_binary integration test (if binary available)", {
  bin <- TemporalHazard:::.hzr_get_hazard_binary()
  if (is.na(bin)) {
    skip("C hazard binary not available; skipping integration test")
  }

  # Use a simple golden fixture dataset
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 1, 0, 1, 1)
  x <- data.frame(x1 = c(0, 1, 0, 1, 0))

  # Run binary fit
  result <- TemporalHazard:::.hzr_run_hazard_binary(
    time = time,
    status = status,
    x = x,
    dist = "weibull"
  )

  # Verify output structure (function returns list with parsed component)
  expect_type(result, "list")
  expect_named(result, c("call_string", "stdout", "parsed", "info"))
  expect_match(result$call_string, "<")
  expect_match(result$call_string, ">")
  expect_match(result$call_string, "\\.sas")
  expect_match(result$call_string, "\\.lst")

  parsed <- result$parsed
  expect_s3_class(parsed, "data.frame")
  expect_named(parsed, c("parameter", "estimate", "std_err", "z_stat", "p_value"))

  if (!is.na(result$info$status_code) && result$info$status_code != 0) {
    skip(sprintf("Legacy binary returned non-zero status %d", result$info$status_code))
  }

  if (nrow(parsed) > 0) {
    # If binary produced output, verify reasonable values
    expect_true(all(!is.na(parsed$estimate)))
    expect_true(all(parsed$std_err > 0 | is.na(parsed$std_err)))
  }
})

#' @importFrom stats optim
#' @keywords internal
NULL

# likelihood-loglogistic.R -- Log-logistic parametric hazard likelihood, gradient, and optimizer
#
# MODEL
# -----
# Log-logistic proportional-odds model (commonly used in actuarial work):
#
#   h(t | x)  = alpha beta t^(beta-1) exp(eta) / (1 + alpha t^beta exp(eta))   hazard
#   H(t | x)  = log(1 + alpha t^beta exp(eta))               cumulative hazard
#   S(t | x)  = 1 / (1 + alpha t^beta exp(eta))              survival
#
# where alpha > 0 (scale), beta > 0 (shape), eta = x beta (linear predictor).
#
# When beta = 1, the hazard function is monotone decreasing.  When beta > 1 it
# has a single interior maximum (unimodal), which makes the log-logistic
# useful for modelling failure rates that rise then fall over time.
#
# THETA LAYOUT
# ------------
#   theta[1]   = log(alpha)   (unconstrained; alpha recovered via exp())
#   theta[2]   = log(beta)   (unconstrained; beta recovered via exp())
#   theta[3:p] = beta_coef   (covariate coefficients; unrestricted)
#
# KEY GRADIENT NOTE
# -----------------
# Let pw_i = term_i / (1 + term_i) where term_i = alpha t_i^beta exp(eta_i)
# and let w_i = (1 + delta_i) * pw_i  (events get weight 2, censored get 1).
#
#   dL/d(log alpha) = sum(delta) - sum(w)
#   dL/d(log beta) = sum(delta) + beta * [sum(delta * log t) - sum(w * log t)]
#   dL/dbeta_j     = t(X) %*% (delta - w)
#
# The (1+delta) weighting arises because events contribute both log h(t) and
# log S(t), each of which depends on log(1 + term).  Censored observations
# contribute only log S(t).
#
# FUNCTIONS
# ---------
#   .hzr_logl_loglogistic()     -- log-likelihood (optionally returning gradient)
#   .hzr_gradient_loglogistic() -- analytical score vector
#   .hzr_optim_loglogistic()    -- unconstrained BFGS wrapper

# Log-Logistic Parametric Hazard Likelihood
#
# Evaluate the log-likelihood and its derivatives for log-logistic hazard models.
# The log-logistic distribution is more flexible than exponential with separate
# scale (alpha) and shape (beta) parameters.

#' Log-likelihood for log-logistic hazard with covariates
#'
#' Computes the negative log-likelihood for right-censored data under the log-logistic
#' parametric hazard model with optional linear-predictor covariates.
#'
#' @param theta Vector of parameters:
#'   theta\[1\] = log(alpha) where alpha > 0 is the scale parameter
#'   theta\[2\] = log(beta) where beta > 0 is the shape parameter
#'   theta\[3:length\]: Covariate coefficients (linear on log-scale of cumulative hazard)
#'
#' @param time Numeric vector of follow-up times (n)
#' @param status Numeric vector of event indicators: 1 = event, 0 = censored (n)
#' @param time_lower Optional numeric lower bound vector for interval-censored rows.
#'   Defaults to time if NULL.
#' @param time_upper Optional numeric upper bound vector for left/interval-censored rows.
#'   Defaults to time if NULL.
#' @param x Design matrix of covariates (n x p_coef); NULL for no covariates
#' @param return_gradient Logical; if TRUE, attach gradient vector as attribute
#'
#' @return Scalar log-likelihood value. If return_gradient = TRUE, gradient vector
#' is attached as attribute \code{"gradient"}.
#'
#' @details
#' The log-logistic hazard model is parameterized as:
#'
#' \deqn{h(t | x) = \frac{\alpha \beta t^{\beta - 1}}{1 + \alpha t^{\beta} \exp(\eta)}}
#'
#' where:
#' - \eqn{\alpha > 0} is scale parameter
#' - \eqn{\beta > 0} is shape parameter
#' - \eqn{\eta = x \beta} is covariate effect (linear on log-scale)
#'
#' The survival function is:
#'
#' \deqn{S(t | x) = \frac{1}{1 + \alpha t^{\beta} \exp(\eta)}}
#'
#' The cumulative hazard is:
#'
#' \deqn{H(t | x) = \log(1 + \alpha t^{\beta} \exp(\eta))}
#'
#' The log-likelihood for right-censored data is:
#'
#' \deqn{\ell(\theta) = \sum_{\delta_i = 1} [\log(\alpha \beta) + (\beta - 1)
#'   \log(t_i) - \log(1 + \alpha t_i^{\beta} \exp(\eta_i))] + \sum_{i}
#'   \log(1 + \alpha t_i^{\beta} \exp(\eta_i))}
#'
#' Reparameterization: \u03b8\[1\] = log(\u03b1), \u03b8\[2\] = log(\u03b2) avoids constrained optimization.
#'
#' Mixed censoring status coding:
#' - 1: exact event at time
#' - 0: right-censored at time
#' - -1: left-censored with upper bound time_upper \(or time\)
#' - 2: interval-censored in the interval \(time_lower, time_upper\)
#'
#' @noRd
.hzr_logl_loglogistic <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    return_gradient = FALSE) {

  n <- length(time)

  # Default unit weights
  if (is.null(weights)) weights <- rep(1, n)

  # Extract parameters
  log_alpha <- theta[1]
  log_beta <- theta[2]
  alpha <- exp(log_alpha)  # Always positive
  beta <- exp(log_beta)    # Always positive

  # Covariate coefficients (if any)
  if (!is.null(x)) {
    if (is.null(attr(x, "dimnames")[[2]])) {
      colnames(x) <- paste0("beta_", seq_len(ncol(x)))
    }
    beta_coef <- theta[3:length(theta)]
    eta <- as.numeric(x %*% beta_coef)  # n-vector
  } else {
    eta <- rep(0, n)
  }

  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  if (any(status == 1 & time <= 0)) {
    return(Inf)
  }
  if (any(status %in% c(-1, 2) & upper <= 0)) {
    return(Inf)
  }
  if (any(status == 2) && any(lower[status == 2] >= upper[status == 2])) {
    return(Inf)
  }

  # Build term values for event/lower/upper times. Using log-scale first keeps
  # overflow checks centralized before exponentiation.
  log_term <- log_alpha + beta * log(time) + eta
  log_term_l <- log_alpha + beta * log(lower) + eta
  log_term_u <- log_alpha + beta * log(upper) + eta

  # Check for numerical overflow
  max_log_term <- max(c(log_term, log_term_l, log_term_u), na.rm = TRUE)
  if (max_log_term > 100) {
    return(Inf)
  }

  term_event <- exp(log_term)
  term_lower <- exp(log_term_l)
  term_upper <- exp(log_term_u)

  idx_event <- status == 1
  idx_right <- status == 0
  idx_left <- status == -1
  idx_interval <- status == 2

  # Exact event: w * log f = w * [log h + log S]
  # f(t|x) = alpha beta t^(beta-1) exp(eta) / (1 + alpha t^beta exp(eta))^2
  # log f  = log alpha + log beta + (beta-1) log t + eta - 2 log(1 + term)
  ll_event <- if (any(idx_event)) {
    sum(
      weights[idx_event] *
        (log_alpha + log_beta + (beta - 1) * log(time[idx_event]) +
           eta[idx_event] - 2 * log1p(term_event[idx_event]))
    )
  } else {
    0
  }

  # Right-censored: w * log S = -w * log(1 + term)
  ll_right <- if (any(idx_right)) {
    -sum(weights[idx_right] * log1p(term_event[idx_right]))
  } else {
    0
  }

  # Left-censored: w * log F = w * log(term/(1+term))
  ll_left <- if (any(idx_left)) {
    sum(weights[idx_left] *
          (log_term_u[idx_left] - log1p(term_upper[idx_left])))
  } else {
    0
  }

  # Interval-censored: w * log(F(u) - F(l)).
  # We compute on probability scale and then log, with positivity guard.
  ll_interval <- if (any(idx_interval)) {
    p_u <- term_upper[idx_interval] / (1 + term_upper[idx_interval])
    p_l <- term_lower[idx_interval] / (1 + term_lower[idx_interval])
    diff_p <- p_u - p_l
    if (any(diff_p <= 0)) return(Inf)
    sum(weights[idx_interval] * log(diff_p))
  } else {
    0
  }

  logl <- ll_event + ll_right + ll_left + ll_interval

  if (!is.finite(logl)) {
    return(Inf)
  }

  # If gradient requested, compute score vector
  if (return_gradient) {
    grad <- .hzr_gradient_loglogistic(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      eta = eta, alpha = alpha, beta = beta,
      log_alpha = log_alpha, log_beta = log_beta, term = term_event
    )
    attr(logl, "gradient") <- grad
  }

  logl
}

#' Score vector (gradient of log-likelihood) for log-logistic
#'
#' Computes the score vector of the log-logistic log-likelihood w.r.t. all parameters.
#'
#' The log-likelihood is:
#'   \eqn{L = \sum(\delta_i * [\log(\alpha) + \log(\beta) + (\beta-1)
#'   *\log(t_i)]) - \sum(\log(1 + \alpha*t_i^\beta*\exp(\eta_i)))}
#'
#' Let p_i = alpha*t_i^beta*exp(eta_i) / (1 + alpha*t_i^beta*exp(eta_i)) = probability of event at t_i
#'
#' Derivatives (using chain rule and reparameterization):
#' dL/d(log alpha) = sum(delta_i) - alpha * sum(t_i^beta * exp(eta_i) / (1 + alpha*t_i^beta*exp(eta_i)))
#'             = sum(delta_i) - sum(alpha * t_i^beta * exp(eta_i) / (1 + term))
#'
#' dL/d(log beta) = sum(delta_i) + sum(delta_i * log(t_i))
#'                  - beta * sum(alpha*t_i^beta*log(t_i)*exp(eta_i)
#'                                 /(1 + alpha*t_i^beta*exp(eta_i)))
#'                = sum(delta_i) + sum(delta_i * log(t_i))
#'                  - beta * sum(t_i^beta*log(t_i)
#'                                 /(1 + 1/(alpha*t_i^beta*exp(eta_i))))
#'
#' dL/dbeta_j = sum(delta_i * x_ij) - sum(alpha*t_i^beta*exp(eta_i)*x_ij / (1 + alpha*t_i^beta*exp(eta_i)))
#'         = t(X) %*% (delta - p)
#'
#' @noRd
.hzr_gradient_loglogistic <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    eta = NULL,
    alpha = NULL,
    beta = NULL,
    log_alpha = NULL,
    log_beta = NULL,
    term = NULL) {

  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)

  # Default unit weights -- keep gradient consistent with the LL when weights
  # are omitted.
  if (is.null(weights)) weights <- rep(1, n)

  # Mixed censoring uses numerical gradient for robustness.
  if (any(status %in% c(-1, 2))) {
    return(.hzr_numeric_grad_loglogistic(theta, time, status, time_lower,
                                          time_upper, x, weights = weights))
  }

  # Recompute if not provided
  if (is.null(term) || is.null(alpha) || is.null(beta)) {
    log_alpha <- theta[1]
    log_beta <- theta[2]
    alpha <- exp(log_alpha)
    beta <- exp(log_beta)

    if (!is.null(x)) {
      beta_coef <- theta[3:length(theta)]
      eta <- as.numeric(x %*% beta_coef)
    } else {
      eta <- rep(0, n)
    }

    term <- alpha * (time ^ beta) * exp(eta)
  }

  # pw_i = term_i / (1 + term_i), wm_i = weights_i * (1 + delta_i) * pw_i
  # (wm is the weighted Mills building block used in every derivative term.)
  pw <- term / (1 + term)
  w_status <- weights * status
  wm <- weights * (1 + status) * pw

  # dL/d(log alpha) = sum(w * delta) - sum(wm)
  grad[1] <- sum(w_status) - sum(wm)

  # dL/d(log beta) = sum(w * delta) + beta * [sum(w * delta * log t) - sum(wm * log t)]
  # Guard log(time): a legal right-censored time = 0 row has w_status = 0 and
  # wm = 0, but an unguarded log(0) = -Inf turns those into 0 * -Inf = NaN, which
  # would poison the entire summed log_beta gradient component.
  log_t <- log(pmax(time, .Machine$double.xmin))
  grad[2] <- sum(w_status) +
             beta * (sum(w_status * log_t) - sum(wm * log_t))

  # dL/dbeta_j = t(X) %*% (w * delta - wm)
  if (p > 2 && !is.null(x)) {
    residual <- w_status - wm
    grad[3:p] <- as.numeric(crossprod(x, residual))
  }

  grad
}

#' Analytic Hessian of the log-logistic negative log-likelihood
#'
#' Returns the Hessian of the objective (negative log-likelihood) on the internal
#' \code{theta = (log alpha, log beta, beta_coef)} scale, matching
#' \code{numDeriv::hessian(objective)} so it can be used directly by
#' \code{.hzr_optim_generic()}.  Coverage mirrors the analytic gradient: event +
#' right-censored rows only; returns \code{NULL} for any left/interval-censored
#' row (\code{status \%in\% c(-1, 2)}) to request the numerical-Hessian fallback.
#'
#' With \code{term = alpha t^beta e^eta}, \code{p = term/(1+term)},
#' \code{u = (1, beta*log t, x)} and \code{D = w (1 + delta) p (1 - p)}, the
#' objective Hessian is \code{U' diag(D) U} plus a scalar
#' \code{beta * sum(w * log t * ((1+delta) p - delta))} added at the (log beta,
#' log beta) entry.
#'
#' @noRd
.hzr_hessian_loglogistic <- function(
    theta, time, status,
    time_lower = NULL, time_upper = NULL, x = NULL, weights = NULL) {

  n <- length(time)
  if (is.null(weights)) weights <- rep(1, n)

  # Coverage contract: closed form is event + right censored only.
  if (any(status %in% c(-1, 2))) {
    return(NULL)
  }

  log_alpha <- theta[1]
  log_beta <- theta[2]
  beta <- exp(log_beta)
  p <- length(theta)
  p_cov <- p - 2L
  # Fail loud on inconsistent theta/x: a silent has_cov = FALSE would zero the
  # covariate rows/cols and yield a conformant-but-wrong vcov.
  if (!is.null(x)) x <- as.matrix(x)
  n_x <- if (is.null(x)) 0L else NCOL(x)
  if (n_x != p_cov) {
    stop(".hzr_hessian_loglogistic(): ncol(x) (", n_x, ") must equal the number ",
         "of covariate parameters in theta (", p_cov, ").", call. = FALSE)
  }
  has_cov <- p_cov > 0L
  beta_coef <- if (has_cov) theta[3:p] else numeric(0)
  eta <- if (has_cov) as.numeric(x %*% beta_coef) else rep(0, n)

  log_t <- log(pmax(time, .Machine$double.xmin))
  term <- exp(log_alpha + beta * log_t + eta)
  pr <- term / (1 + term)
  delta <- as.numeric(status == 1)

  # Per-row design u = (1, beta*log t, x); diagonal weight D = w(1+delta)p(1-p).
  d_wt <- weights * (1 + delta) * pr * (1 - pr)
  u <- cbind(1, beta * log_t)
  if (has_cov) u <- cbind(u, x)

  hess <- crossprod(u, d_wt * u)
  hess[2L, 2L] <- hess[2L, 2L] +
    beta * sum(weights * log_t * ((1 + delta) * pr - delta))
  dimnames(hess) <- list(names(theta), names(theta))
  hess
}

#' @noRd
.hzr_optim_loglogistic <- function(
    time, status, time_lower = NULL, time_upper = NULL,
    x = NULL, theta_start, weights = NULL, control = list()) {
  hessian_fn <- function(par) {
    .hzr_hessian_loglogistic(
      theta = par, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights
    )
  }
  .hzr_optim_generic(
    logl_fn = .hzr_logl_loglogistic,
    gradient_fn = .hzr_gradient_loglogistic,
    time = time, status = status,
    time_lower = time_lower, time_upper = time_upper,
    x = x, theta_start = theta_start, weights = weights,
    control = control, use_bounds = FALSE,
    hessian_fn = hessian_fn
  )
}

.hzr_numeric_grad_loglogistic <- function(theta, time, status,
                                            time_lower = NULL, time_upper = NULL,
                                            x = NULL, weights = NULL) {
  # Reuse log-likelihood as scalar objective and differentiate numerically.
  objective <- function(par) {
    .hzr_logl_loglogistic(
      theta = par,
      time = time,
      status = status,
      time_lower = time_lower,
      time_upper = time_upper,
      x = x,
      weights = weights,
      return_gradient = FALSE
    )
  }

  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(as.numeric(numDeriv::grad(objective, theta)))
  }

  eps <- 1e-6
  grad <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tm <- theta
    tp[i] <- tp[i] + eps
    tm[i] <- tm[i] - eps
    grad[i] <- (objective(tp) - objective(tm)) / (2 * eps)
  }
  grad
}

test_that(".hzr_hessian_loglogistic matches numDeriv (no covariates)", {
  skip_if_not_installed("numDeriv")
  set.seed(31)
  n <- 300
  time <- rexp(n, 0.6) + 0.01
  status <- rbinom(n, 1, 0.75)
  theta <- c(log_alpha = log(0.5), log_beta = log(1.4))
  obj <- function(th) -.hzr_logl_loglogistic(th, time, status)
  h_an <- .hzr_hessian_loglogistic(theta, time, status)
  h_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(h_an), unname(h_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_loglogistic matches numDeriv (covariates + weights)", {
  skip_if_not_installed("numDeriv")
  set.seed(32)
  n <- 400
  x <- cbind(a = rnorm(n), b = rbinom(n, 1, 0.4))
  eta <- as.numeric(x %*% c(0.3, -0.4))
  time <- rexp(n, rate = exp(-eta) * 0.5) + 0.01
  status <- rbinom(n, 1, 0.8)
  w <- runif(n, 0.5, 2)
  theta <- c(log_alpha = log(0.5), log_beta = log(1.3), 0.3, -0.4)
  obj <- function(th) -.hzr_logl_loglogistic(th, time, status, x = x, weights = w)
  h_an <- .hzr_hessian_loglogistic(theta, time, status, x = x, weights = w)
  h_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(h_an), unname(h_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_loglogistic handles right-censored time = 0 rows", {
  skip_if_not_installed("numDeriv")
  set.seed(33)
  n <- 60
  time <- c(0, rexp(n - 1, 0.6) + 0.01)
  status <- c(0L, rbinom(n - 1, 1, 0.7))
  theta <- c(log_alpha = log(0.6), log_beta = log(1.2))
  h_an <- .hzr_hessian_loglogistic(theta, time, status)
  expect_true(all(is.finite(h_an)))
  obj <- function(th) -.hzr_logl_loglogistic(th, time, status)
  h_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(h_an), unname(h_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_loglogistic returns NULL for left/interval censoring", {
  set.seed(34)
  n <- 40
  time <- rexp(n, 0.5) + 0.01
  status <- rep(1L, n)
  status[1:5] <- 2L
  expect_null(.hzr_hessian_loglogistic(
    c(log_alpha = 0, log_beta = 0), time, status,
    time_lower = time, time_upper = time + 0.5))
  status2 <- rep(1L, n)
  status2[1:5] <- -1L
  expect_null(.hzr_hessian_loglogistic(
    c(log_alpha = 0, log_beta = 0), time, status2, time_upper = time + 0.5))
})

test_that("loglogistic fit vcov uses the analytic Hessian (matches numDeriv)", {
  skip_if_not_installed("numDeriv")
  set.seed(35)
  n <- 500
  z <- rnorm(n)
  time <- rexp(n, rate = exp(-0.5 * z) * 0.4) + 0.01
  status <- rbinom(n, 1, 0.85)
  fit <- hazard(
    survival::Surv(time, status) ~ z,
    data = data.frame(time, status, z),
    dist = "loglogistic",
    theta = c(log_alpha = 0, log_beta = 0, z = 0), fit = TRUE
  )
  obj <- function(th) -.hzr_logl_loglogistic(th, time, status, x = cbind(z))
  v_nd <- solve(numDeriv::hessian(obj, fit$fit$theta))
  expect_equal(unname(fit$fit$vcov), unname(v_nd), tolerance = 1e-4)
  expect_true(isTRUE(fit$fit$pd))
})

test_that("loglogistic SEs are invariant to covariate rescaling", {
  set.seed(36)
  n <- 600
  z <- rnorm(n)
  time <- rexp(n, rate = exp(-0.4 * z) * 0.5) + 0.01
  status <- rbinom(n, 1, 0.9)
  f1 <- hazard(survival::Surv(time, status) ~ z,
               data = data.frame(time, status, z = z),
               dist = "loglogistic",
               theta = c(log_alpha = 0, log_beta = 0, z = 0), fit = TRUE)
  f2 <- hazard(survival::Surv(time, status) ~ z,
               data = data.frame(time, status, z = z / 100),
               dist = "loglogistic",
               theta = c(log_alpha = 0, log_beta = 0, z = 0), fit = TRUE)
  se1 <- sqrt(diag(f1$fit$vcov))
  se2 <- sqrt(diag(f2$fit$vcov))
  expect_equal(unname(se1[1]), unname(se2[1]), tolerance = 1e-2)
  expect_equal(unname(se1[2]), unname(se2[2]), tolerance = 1e-2)
})

test_that(".hzr_logl_loglogistic gradient is finite for right-censored time = 0 rows", {
  skip_if_not_installed("numDeriv")
  set.seed(37)
  n <- 60
  time <- c(0, rexp(n - 1, 0.6) + 0.01)
  status <- c(0L, rbinom(n - 1, 1, 0.7))
  theta <- c(log_alpha = log(0.6), log_beta = log(1.2))
  ll <- .hzr_logl_loglogistic(theta, time, status, return_gradient = TRUE)
  g_an <- attr(ll, "gradient")
  expect_true(all(is.finite(g_an)))
  obj <- function(th) .hzr_logl_loglogistic(th, time, status)
  g_nd <- numDeriv::grad(obj, theta)
  expect_equal(g_an, g_nd, tolerance = 1e-5)
})

test_that(".hzr_hessian_loglogistic errors on inconsistent theta/x", {
  set.seed(63)
  n <- 30
  time <- rexp(n, 0.5) + 0.01
  status <- rbinom(n, 1, 0.7)
  expect_error(
    .hzr_hessian_loglogistic(c(log_alpha = 0, log_beta = 0, z = 0.3), time, status, x = NULL),
    "ncol")
  x2 <- cbind(a = rnorm(n), b = rnorm(n))
  expect_error(
    .hzr_hessian_loglogistic(c(log_alpha = 0, log_beta = 0, z = 0.3), time, status, x = x2),
    "ncol")
})

#' @importFrom stats optim pnorm
#' @importFrom survival Surv
#' @keywords internal
NULL

# hazard_api.R -- Primary user-facing API for TemporalHazard
#
# This file contains the main entry points:
#   hazard()         -- construct and optionally fit a parametric hazard model
#   predict.hazard() -- generate predictions from a fitted hazard object
#   print.hazard()   -- compact S3 print method
#   coef.hazard()    -- extract fitted parameter vector
#   vcov.hazard()    -- extract variance-covariance matrix
#
# DISTRIBUTION PARAMETERIZATION CONVENTIONS
# -----------------------------------------
# Each distribution stores a flattened theta vector.  Shape/scale parameters
# always come first; covariate coefficients (beta) follow:
#
#   Weibull:     theta = [mu, nu, beta_1, beta_2, ...]      mu>0, nu>0
#   Exponential: theta = [log(lambda), beta_1, beta_2, ...]      lambda>0 via exp()
#   Loglogistic: theta = [log(alpha), log(beta), beta_1, ...]   alpha>0, beta>0 via exp()
#   Lognormal:   theta = [mu, log(sigma), beta_1, beta_2, ...]   sigma>0 via exp(); AFT model
#
# ADDING A NEW DISTRIBUTION
# -------------------------
# 1. Create R/likelihood-<dist>.R with .hzr_logl_<dist>(),
#    .hzr_gradient_<dist>(), and .hzr_optim_<dist>().
# 2. Add an else-if branch in hazard() below under "Distribution dispatch".
# 3. Add an else-if branch in predict.hazard() under "Dispatch by distribution".
# 4. Update the supported-distributions guard in predict.hazard().
# 5. Update the coefficient-extraction block in predict.hazard() for
#    linear_predictor / hazard prediction types.
# 6. Write tests in tests/testthat/test-<dist>-dist.R.
# 7. Generate a golden fixture via data-raw/golden_fixtures.R
#    (.hzr_create_<dist>_golden_fixture()).

#' Build and optionally fit a hazard model
#'
#' Creates a `hazard` object and optionally fits it via maximum likelihood.
#' This mirrors the argument-oriented workflow of the legacy HAZARD C/SAS
#' implementation: supply starting values in `theta` and the function will
#' optimize to produce fitted estimates.
#'
#' @param time Numeric follow-up time vector.
#' @param status Numeric or logical event indicator vector.
#' @param time_lower Optional numeric lower bound vector for censoring intervals.
#'   Used when `status == 2` (interval-censored); defaults to `time` if NULL.
#' @param time_upper Optional numeric upper bound vector for censoring intervals.
#'   Used when `status %in% c(-1, 2)`; defaults to `time` if NULL.
#' @param x Optional design matrix (or data frame coercible to matrix).
#' @param formula Optional formula of the form `Surv(time, status) ~ predictors`.
#'   When provided, overrides direct time/status/x arguments and extracts from data.
#'   Example: `hazard(Surv(time, status) ~ x1 + x2, data = df, dist = "weibull", fit = TRUE)`.
#' @param data Optional data frame containing variables referenced in formula.
#' @param time_windows Optional numeric vector of strictly positive cut points for
#'   piecewise time-varying coefficients. When provided, each predictor column in
#'   `x` is expanded into one column per time window so each window gets its own
#'   coefficient.
#' @param theta Optional numeric coefficient vector (starting values for optimization).
#' @param dist Character baseline distribution label (default "weibull").
#'   Use `"multiphase"` for N-phase additive hazard models (requires `phases`).
#' @param phases Optional named list of [hzr_phase()] objects specifying the
#'   phases for a multiphase model (`dist = "multiphase"`).  See Examples.
#' @param fit Logical; if TRUE, fit the model via maximum likelihood (default FALSE).
#' @param weights Optional numeric vector of observation weights (non-negative).
#'   Each observation's log-likelihood contribution is multiplied by its weight.
#'   Use for severity-weighted repeated events. Default `NULL` (unit weights).
#'   Implements the SAS `WEIGHT` statement.
#' @param control Named list of control options (see Details).
#' @param ... Additional named arguments retained for parity with legacy calling
#'   conventions.
#'
#' @details
#' Control parameters:
#' - `maxit`: Maximum iterations (default 1000)
#' - `reltol`: Relative parameter change tolerance (default 1e-5)
#' - `abstol`: Absolute gradient norm tolerance (default 1e-6)
#' - `method`: Optimization method: "bfgs" or "nm" (default "bfgs")
#' - `condition`: Condition number control (default 14)
#' - `nocov`, `nocor`: Suppress covariance/correlation output (legacy; no-op in M2)
#'
#' Censoring status coding:
#' - 1: Exact event at time
#' - 0: Right-censored at time
#' - -1: Left-censored with upper bound at time_upper \(or time\)
#' - 2: Interval-censored in the interval \(time_lower, time_upper\)
#'
#' Time-varying coefficients:
#' - If `time_windows` is supplied, predictors are expanded to piecewise window
#'   interactions so each window has its own coefficient vector.
#' - This is implemented as design-matrix expansion, so the existing likelihood
#'   engines remain unchanged.
#'
#' @examples
#' # -- Univariable Weibull ----------------------------------------------
#' set.seed(1)
#' time   <- rexp(50, rate = 0.3)
#' status <- sample(0:1, 50, replace = TRUE, prob = c(0.3, 0.7))
#' fit <- hazard(time = time, status = status,
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' summary(fit)
#'
#' # -- Formula interface with covariates --------------------------------
#' set.seed(1001)
#' n   <- 180
#' dat <- data.frame(
#'   time   = rexp(n, rate = 0.35) + 0.05,
#'   status = rbinom(n, size = 1, prob = 0.6),
#'   age    = rnorm(n, mean = 62, sd = 11),
#'   nyha   = sample(1:4, n, replace = TRUE),
#'   shock  = rbinom(n, size = 1, prob = 0.18)
#' )
#'
#' fit2 <- hazard(
#'   survival::Surv(time, status) ~ age + nyha + shock,
#'   data    = dat,
#'   theta   = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
#'   dist    = "weibull",
#'   fit     = TRUE,
#'   control = list(maxit = 300)
#' )
#' summary(fit2)
#'
#' \donttest{
#' # -- Parametric survival with Kaplan-Meier overlay -----------------
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'
#'   # Parametric curve on a fine grid at median covariate profile
#'   t_grid   <- seq(0.05, max(dat$time), length.out = 80)
#'   curve_df <- data.frame(
#'     time = t_grid, age = median(dat$age), nyha = 2, shock = 0
#'   )
#'   curve_df$survival <- predict(fit2, newdata = curve_df,
#'                                type = "survival") * 100
#'
#'   # Kaplan-Meier empirical overlay
#'   km    <- survival::survfit(survival::Surv(time, status) ~ 1, data = dat)
#'   km_df <- data.frame(time = km$time, survival = km$surv * 100)
#'
#'   ggplot() +
#'     geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier")) +
#'     geom_line(data = curve_df, aes(time, survival,
#'                                    colour = "Parametric (Weibull)")) +
#'     scale_colour_manual(
#'       values = c("Parametric (Weibull)" = "#0072B2",
#'                  "Kaplan-Meier"         = "#D55E00")
#'     ) +
#'     scale_y_continuous(limits = c(0, 100)) +
#'     labs(x = "Months after surgery", y = "Freedom from death (%)",
#'          colour = NULL) +
#'     theme_minimal() +
#'     theme(legend.position = "bottom")
#' }
#' }
#'
#' \donttest{
#' # -- Multiphase model (two phases) ---------------------------------
#' fit_mp <- hazard(
#'   survival::Surv(time, status) ~ 1,
#'   data   = dat,
#'   dist   = "multiphase",
#'   phases = list(
#'     early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                        fixed = "shapes"),
#'     late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
#'                        fixed = "shapes")
#'   ),
#'   fit     = TRUE,
#'   control = list(n_starts = 5, maxit = 1000)
#' )
#' summary(fit_mp)
#'
#' # -- Per-phase decomposed cumulative hazard ------------------------
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   t_grid <- seq(0.01, max(dat$time), length.out = 100)
#'   decomp <- predict(fit_mp, newdata = data.frame(time = t_grid),
#'                     type = "cumulative_hazard", decompose = TRUE)
#'
#'   df_long <- data.frame(
#'     time = rep(decomp$time, 3),
#'     cumhaz = c(decomp$total, decomp$early, decomp$late),
#'     component = rep(c("Total", "Early (cdf)", "Late (cdf)"),
#'                     each = nrow(decomp))
#'   )
#'   df_long$component <- factor(df_long$component,
#'     levels = c("Total", "Early (cdf)", "Late (cdf)"))
#'
#'   ggplot2::ggplot(df_long,
#'     ggplot2::aes(x = time, y = cumhaz, colour = component,
#'                  linewidth = component)) +
#'     ggplot2::geom_line() +
#'     ggplot2::scale_colour_manual(values = c(
#'       "Total" = "black", "Early (cdf)" = "#0072B2",
#'       "Late (cdf)" = "#D55E00"
#'     )) +
#'     ggplot2::scale_linewidth_manual(values = c(
#'       "Total" = 1.2, "Early (cdf)" = 0.6, "Late (cdf)" = 0.6
#'     )) +
#'     ggplot2::labs(
#'       x = "Time", y = "Cumulative hazard H(t)",
#'       colour = NULL, linewidth = NULL,
#'       title = "Multiphase decomposition: early + late"
#'     ) +
#'     ggplot2::theme_minimal() +
#'     ggplot2::theme(legend.position = "bottom")
#' }
#' }
#'
#' @seealso
#' [predict.hazard()] for survival/cumulative-hazard predictions,
#' [summary.hazard()] for model summaries,
#' [hzr_phase()] for specifying multiphase temporal shapes.
#'
#' Vignettes with worked examples:
#' \code{vignette("fitting-hazard-models")} --- single-phase through multiphase fitting,
#' \code{vignette("prediction-visualization")} --- prediction types and decomposed hazard plots,
#' \code{vignette("inference-diagnostics")} --- bootstrap CIs and model diagnostics.
#'
#' @references
#' Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
#' hazard into phases, each incorporating a separate stream of concomitant
#' information. *J Am Stat Assoc.* 1986;81(395):615--624.
#' \doi{10.1080/01621459.1986.10478314}
#'
#' Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
#' Probability of atrial fibrillation after ablation: Using a parametric
#' nonlinear temporal decomposition mixed effects model. *Stat Methods Med Res.*
#' 2018;27(1):126--141. \doi{10.1177/0962280215623583}
#'
#' @return An object of class \code{hazard}, a named list with components:
#'   \code{call} (the matched call),
#'   \code{spec} (model specification: \code{dist}, \code{control},
#'   \code{time_windows}, \code{phases}),
#'   \code{data} (input data: \code{time}, \code{status}, \code{x},
#'   \code{weights}, etc.),
#'   \code{fit} (optimisation results: \code{theta}, \code{objective},
#'   \code{converged}, \code{se}, \code{vcov}, \code{counts}, \code{message};
#'   all \code{NULL} when \code{fit = FALSE}),
#'   and \code{engine} (implementation tag, \code{"native-r-m2"}).
#' @export
hazard <- function(formula = NULL,
                   data = NULL,
                   time = NULL,
                   status = NULL,
                   time_lower = NULL,
                   time_upper = NULL,
                   x = NULL,
                   time_windows = NULL,
                   theta = NULL,
                   dist = "weibull",
                   phases = NULL,
                   fit = FALSE,
                   weights = NULL,
                   control = list(),
                   ...) {
  # Formula dispatch: if formula is provided, parse it and extract time/status/x from data
  if (!is.null(formula)) {
    if (is.null(data)) {
      stop("'data' is required when 'formula' is provided.", call. = FALSE)
    }
    parsed <- .hzr_parse_formula(formula = formula, data = data)
    time <- parsed$time
    status <- parsed$status
    time_lower <- parsed$time_lower
    time_upper <- parsed$time_upper
    x <- parsed$x

  }

  # After formula dispatch, require time and status
  if (is.null(time) || is.null(status)) {
    stop("'time' and 'status' are required (either directly or via 'formula').", call. = FALSE)
  }
  if (!is.numeric(time) || any(!is.finite(time)) || any(time < 0)) {
    stop("'time' must be a numeric vector of finite non-negative values.", call. = FALSE)
  }

  n <- length(time)
  if (length(status) != n) {
    stop("'status' must have the same length as 'time'.", call. = FALSE)
  }

  # Convert Surv object status to numeric if needed (after formula parsing)
  if (inherits(status, "Surv")) {
    status <- unclass(status)[, 2L]
  }

  # Optional censoring bounds:
  # - status = 1 (event): uses observed event time in `time`
  # - status = 0 (right-censored): censoring time in `time` (or `time_lower`)
  # - status = -1 (left-censored): upper bound in `time` (or `time_upper`)
  # - status = 2 (interval-censored): [time_lower, time_upper] required
  if (!is.null(time_lower)) {
    if (!is.numeric(time_lower) || length(time_lower) != n || any(!is.finite(time_lower)) || any(time_lower < 0)) {
      stop("'time_lower' must be a numeric vector of finite non-negative values matching length(time).", call. = FALSE)
    }
  }

  if (!is.null(time_upper)) {
    if (!is.numeric(time_upper) || length(time_upper) != n || any(!is.finite(time_upper)) || any(time_upper < 0)) {
      stop("'time_upper' must be a numeric vector of finite non-negative values matching length(time).", call. = FALSE)
    }
  }

  if (!is.null(x)) {
    x <- .hzr_as_design_matrix(x, n = n)
  }

  if (!is.null(time_windows)) {
    # We use cut points to define window-specific coefficient blocks.
    if (!is.numeric(time_windows) || any(!is.finite(time_windows)) || anyDuplicated(time_windows)) {
      stop("'time_windows' must be a numeric vector of unique finite cut points.", call. = FALSE)
    }
    time_windows <- sort(time_windows)
    if (any(time_windows <= 0)) {
      stop("'time_windows' cut points must be strictly positive.", call. = FALSE)
    }
    if (is.null(x)) {
      stop("'time_windows' requires predictor matrix 'x'.", call. = FALSE)
    }
  }

  x_fit <- x
  if (!is.null(time_windows) && !is.null(x)) {
    # Expand X -> [X_w1 | X_w2 | ...] where each row is active only in its window.
    x_fit <- .hzr_expand_time_varying_design(x = x, time = time, time_windows = time_windows)
  }

  if (!is.null(theta)) {
    if (!is.numeric(theta) || any(!is.finite(theta))) {
      stop("'theta' must be a finite numeric vector when provided.", call. = FALSE)
    }

    # If x exists, theta must include coefficients for all variates
    # theta = [shape parms ... | covariate coefficients ...]
    # For now, assume theta length determines whether we expect x
    required_coef <- if (is.null(x_fit)) 0L else ncol(x_fit)
    if (!is.null(x_fit) && length(theta) < required_coef) {
      stop("'theta' length must be >= number of required coefficients (", required_coef, ").", call. = FALSE)
    }
  }

  # --- Validate and normalize weights ----------------------------------------
  n_obs <- length(time)
  if (!is.null(weights)) {
    if (!is.numeric(weights) || length(weights) != n_obs) {
      stop("'weights' must be a numeric vector of length ", n_obs, ".",
           call. = FALSE)
    }
    if (any(weights < 0) || any(!is.finite(weights))) {
      stop("'weights' must be non-negative and finite.", call. = FALSE)
    }
  }

  if (!is.character(dist) || length(dist) != 1 || !nzchar(dist)) {
    stop("'dist' must be a non-empty character scalar.", call. = FALSE)
  }

  if (!is.list(control)) {
    stop("'control' must be a list.", call. = FALSE)
  }

  # Multiphase validation
  if (dist == "multiphase") {
    if (is.null(phases)) {
      stop("'phases' is required when dist = 'multiphase'. ",
           "Supply a list of hzr_phase() specifications.", call. = FALSE)
    }
    phases <- .hzr_validate_phases(phases)
  } else if (!is.null(phases)) {
    warning("'phases' is ignored when dist != 'multiphase'.")
    phases <- NULL
  }

  if (any(status %in% c(-1, 2))) {
    if (is.null(time_upper)) {
      warning("'time_upper' not provided; using 'time' as upper bound for left/interval-censored rows.")
    }
    if (any(status == 2) && is.null(time_lower)) {
      warning("'time_lower' not provided; using 'time' as lower bound for interval-censored rows.")
    }
  }

  # fit_state holds the result of optimization (or just starting values if fit=FALSE).
  # Fields:
  #   theta     -- parameter vector (starting values before fit; MLE estimates after)
  #   converged -- TRUE if optimizer reported convergence (code 0); NA if not fitted
  #   objective -- log-likelihood at theta; NA_real_ if not fitted
  #   se        -- standard errors (sqrt of vcov diagonal); NULL / NA if unavailable
  #   gradient  -- score vector at solution (populated by some optimizers); NULL otherwise
  #   vcov      -- variance-covariance matrix; NA if Hessian not invertible
  #   counts    -- c(fn, gr) evaluation counts from optim()
  #   message   -- convergence message string from optim()
  fit_state <- list(
    theta = theta,
    converged = NA,
    objective = NA_real_,
    se = NULL,
    gradient = NULL
  )

  .hzr_run_fit_safely <- function(expr) {
    withCallingHandlers(
      expr,
      warning = function(w) {
        msg <- conditionMessage(w)
        if (grepl("NaNs produced", msg, fixed = TRUE) ||
            grepl("Hessian not invertible; standard errors unavailable", msg, fixed = TRUE)) {
          invokeRestart("muffleWarning")
        }
      }
    )
  }

  .hzr_safe_se_from_vcov <- function(vcov_mat) {
    if (is.null(vcov_mat) || !is.matrix(vcov_mat) || anyNA(vcov_mat)) {
      return(NA)
    }
    d <- diag(vcov_mat)
    # Guard against small negative/invalid variances from numerical Hessians.
    if (!all(is.finite(d)) || any(d < 0)) {
      return(rep(NA_real_, length(d)))
    }
    sqrt(d)
  }

  # Distribution dispatch -- select the distribution-specific optimizer and fit.
  if (fit && dist == "multiphase") {
    # Multiphase: theta_start is optional (assembled from phase specs if NULL)
    optim_result <- .hzr_run_fit_safely(.hzr_optim_multiphase(
      time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x_fit, theta_start = theta, weights = weights,
      control = control,
      phases = phases, formula_global = formula, data = data
    ))

    fit_state$theta <- optim_result$par
    fit_state$objective <- optim_result$value
    fit_state$converged <- (optim_result$convergence == 0)
    fit_state$se <- .hzr_safe_se_from_vcov(optim_result$vcov)
    fit_state$vcov <- optim_result$vcov
    fit_state$counts <- optim_result$counts
    fit_state$message <- optim_result$message
    # Store optimizer metadata for predict/summary
    fit_state$phases <- optim_result$phases
    fit_state$covariate_counts <- optim_result$covariate_counts
    fit_state$x_list <- optim_result$x_list
    fit_state$fixed_mask <- optim_result$fixed_mask

  } else if (fit && !is.null(theta)) {
    optim_fn <- switch(
      dist,
      weibull = .hzr_optim_weibull,
      exponential = .hzr_optim_exponential,
      loglogistic = .hzr_optim_loglogistic,
      lognormal = .hzr_optim_lognormal,
      stop("Distribution '", dist, "' not yet supported for fitting.", call. = FALSE)
    )

    optim_result <- .hzr_run_fit_safely(optim_fn(
      time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x_fit, theta_start = theta, weights = weights,
      control = control
    ))

    fit_state$theta <- optim_result$par
    fit_state$objective <- optim_result$value
    fit_state$converged <- (optim_result$convergence == 0)
    fit_state$se <- .hzr_safe_se_from_vcov(optim_result$vcov)
    fit_state$vcov <- optim_result$vcov
    fit_state$counts <- optim_result$counts
    fit_state$message <- optim_result$message
  }

  # Assemble the hazard S3 object.
  # $call       -- captured call for reproducibility / print
  # $spec       -- model specification (dist, control)
  # $data       -- raw data stored for default predict() / refit
  # $fit        -- optimisation results (see fit_state fields above)
  # $legacy_args -- pass-through ... args for SAS-migration parity
  # $engine     -- implementation tag ("native-r-m2")
  obj <- list(
    call = match.call(),
    spec = list(dist = dist, control = control, time_windows = time_windows,
                phases = phases),
    data = list(
      time = time,
      time_lower = time_lower,
      time_upper = time_upper,
      status = as.numeric(status),
      x = x,
      weights = weights
    ),
    fit = fit_state,
    legacy_args = list(...),
    engine = "native-r-m2"
  )

  class(obj) <- "hazard"
  obj
}

#' Predict from a hazard model object
#'
#' Produces prediction outputs from a `hazard` object. Supports multiple prediction
#' types including linear predictor, hazard, survival probability, and cumulative hazard.
#'
#' @param object A `hazard` object.
#' @param newdata Optional matrix or data frame of predictors. For types requiring
#'   time (e.g., "survival", "cumulative_hazard"), newdata should include a `time`
#'   column, or time will be taken from the fitted object's data.
#' @param type Prediction type:
#'   - `"linear_predictor"`: Linear predictor eta = x*beta (not available for multiphase)
#'   - `"hazard"`: Hazard scale exp(eta) (not available for multiphase)
#'   - `"survival"`: Survival probability S(t|x) = exp(-H(t|x))
#'   - `"cumulative_hazard"`: Cumulative hazard H(t|x) at event times
#' @param decompose Logical; if `TRUE` and the model is multiphase, return a
#'   data frame with per-phase cumulative hazard contributions alongside the
#'   total.  Ignored for single-distribution models.  Default `FALSE`.
#' @param se.fit Logical; if `TRUE`, compute delta-method standard errors and
#'   confidence limits for each prediction.  The return value becomes a data
#'   frame with columns `fit`, `se.fit`, `lower`, `upper`.  Default `FALSE`.
#'   CLs are computed on the log-hazard / log-cumhaz scale and on the
#'   log(-log(survival)) scale so lower/upper stay inside the valid range
#'   of each prediction type; `linear_predictor` uses symmetric natural-scale
#'   CLs.  Not compatible with `decompose = TRUE`.
#' @param level Numeric confidence level in `(0, 1)`; default `0.95`.
#'   Only used when `se.fit = TRUE`.
#' @param ... Unused; included for S3 compatibility.
#'
#' @details
#' For Weibull models with survival or cumulative_hazard predictions:
#' - Cumulative hazard: H(t|x) = (mu*t)^nu * exp(eta)
#' - Survival: S(t|x) = exp(-H(t|x))
#'
#' Time values must be positive and finite. If newdata contains a `time` column,
#' it will be used; otherwise, the time vector from the fitted object is used.
#' For models fit with `time_windows`, predictions for `type = "linear_predictor"`
#' or `"hazard"` also require time values (via `newdata$time` or fitted-time fallback)
#' so window-specific coefficients can be selected.
#'
#' @return Numeric vector of predictions.
#' @examples
#' # -- Basic predictions ------------------------------------------------
#' set.seed(1)
#' fit <- hazard(time = rexp(50, 0.3), status = rep(1L, 50),
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' predict(fit, type = "survival")
#' predict(fit, newdata = data.frame(time = c(1, 2, 5)),
#'         type = "cumulative_hazard")
#'
#' # -- Patient-specific survival curves ---------------------------------
#' set.seed(1001)
#' n   <- 180
#' dat <- data.frame(
#'   time   = rexp(n, rate = 0.35) + 0.05,
#'   status = rbinom(n, size = 1, prob = 0.6),
#'   age    = rnorm(n, mean = 62, sd = 11),
#'   nyha   = sample(1:4, n, replace = TRUE),
#'   shock  = rbinom(n, size = 1, prob = 0.18)
#' )
#' fit2 <- hazard(
#'   survival::Surv(time, status) ~ age + nyha + shock,
#'   data  = dat,
#'   theta = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
#'   dist  = "weibull", fit = TRUE
#' )
#'
#' new_patients <- data.frame(
#'   time = c(0.5, 1.5, 3.0),
#'   age  = c(50, 65, 75),
#'   nyha = c(1, 3, 4),
#'   shock = c(0, 0, 1)
#' )
#' # Compute predictions from the clean covariate frame before adding columns
#' surv   <- predict(fit2, newdata = new_patients, type = "survival")
#' cumhaz <- predict(fit2, newdata = new_patients, type = "cumulative_hazard")
#' new_patients$survival          <- surv
#' new_patients$cumulative_hazard <- cumhaz
#' new_patients
#'
#' \donttest{
#' # -- Grouped survival curves ---------------------------------------
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'
#'   t_grid <- seq(0.05, max(dat$time), length.out = 80)
#'   profiles <- data.frame(
#'     label = c("Low risk (age 50, NYHA I)",
#'               "High risk (age 75, NYHA IV)"),
#'     age   = c(50, 75),
#'     nyha  = c(1, 4),
#'     shock = c(0, 1)
#'   )
#'
#'   curve_list <- lapply(seq_len(nrow(profiles)), function(i) {
#'     nd <- data.frame(
#'       time  = t_grid,
#'       age   = profiles$age[i],
#'       nyha  = profiles$nyha[i],
#'       shock = profiles$shock[i]
#'     )
#'     nd$survival <- predict(fit2, newdata = nd, type = "survival") * 100
#'     nd$profile  <- profiles$label[i]
#'     nd
#'   })
#'   curve_df <- do.call(rbind, curve_list)
#'
#'   ggplot(curve_df, aes(time, survival, colour = profile)) +
#'     geom_line() +
#'     scale_y_continuous(limits = c(0, 100)) +
#'     labs(x = "Months after surgery",
#'          y = "Freedom from death (%)",
#'          title = "Predicted survival by risk profile",
#'          colour = NULL) +
#'     theme_minimal()
#' }
#' }
#'
#' \donttest{
#' # -- Multiphase predictions with decomposition --------------------
#' set.seed(42)
#' n   <- 200
#' dat <- data.frame(
#'   time   = rexp(n, rate = 0.25) + 0.01,
#'   status = rbinom(n, size = 1, prob = 0.65)
#' )
#' fit_mp <- hazard(
#'   survival::Surv(time, status) ~ 1,
#'   data   = dat,
#'   dist   = "multiphase",
#'   phases = list(
#'     early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                        fixed = "shapes"),
#'     late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
#'                        fixed = "shapes")
#'   ),
#'   fit     = TRUE,
#'   control = list(n_starts = 5, maxit = 1000)
#' )
#'
#' t_grid <- seq(0.01, max(dat$time) * 0.9, length.out = 100)
#' nd     <- data.frame(time = t_grid)
#'
#' # Overall survival
#' predict(fit_mp, newdata = nd, type = "survival")
#'
#' # Per-phase decomposed cumulative hazard
#' decomp <- predict(fit_mp, newdata = nd,
#'                   type = "cumulative_hazard", decompose = TRUE)
#' head(decomp)
#' }
#'
#' @seealso
#' [hazard()] for model fitting,
#' [summary.hazard()] for model summaries,
#' [hzr_phase()] for multiphase temporal shapes.
#'
#' \code{vignette("prediction-visualization")} for detailed prediction
#' workflows including decomposed hazard plots and patient-specific curves.
#' @export
predict.hazard <- function(object, newdata = NULL,
                           type = c("hazard", "linear_predictor",
                                    "survival", "cumulative_hazard"),
                           decompose = FALSE,
                           se.fit = FALSE, level = 0.95, ...) {
  type <- match.arg(type)
  theta <- object$fit$theta
  time_windows <- object$spec$time_windows

  if (is.null(theta)) {
    stop("No coefficients ('theta') are available in 'object'.", call. = FALSE)
  }

  if (!is.logical(se.fit) || length(se.fit) != 1L || is.na(se.fit)) {
    stop("'se.fit' must be TRUE or FALSE.", call. = FALSE)
  }
  if (se.fit) {
    if (!is.numeric(level) || length(level) != 1L ||
          is.na(level) || level <= 0 || level >= 1) {
      stop("'level' must be a single number in (0, 1).", call. = FALSE)
    }
    if (decompose) {
      stop("'se.fit = TRUE' with 'decompose = TRUE' is not supported. ",
           "Request point predictions first (decompose = TRUE, ",
           "se.fit = FALSE), then compute CLs separately on the total.",
           call. = FALSE)
    }
  }

  # -----------------------------------------------------------------------
  # Predictions that do NOT need time (linear_predictor, hazard)
  # -----------------------------------------------------------------------
  # These predictions work purely through the linear predictor eta = X beta.
  # Shape parameters are stripped from theta; only covariate beta's are used.
  # hazard returns exp(eta), which is the relative-hazard multiplier (not the
  # baseline conditional hazard, which would require time + distribution).
  if (type %in% c("hazard", "linear_predictor")) {
    if (object$spec$dist == "multiphase") {
      stop("Prediction type '", type, "' is not supported for multiphase models. ",
           "Use 'survival' or 'cumulative_hazard' instead.", call. = FALSE)
    }
    n_pred <- NULL
    pred_time <- NULL
    if (is.null(newdata)) {
      x <- object$data$x
      pred_time <- object$data$time
    } else {
      newdata <- as.data.frame(newdata)
      n_pred <- nrow(newdata)
      if ("time" %in% names(newdata)) {
        pred_time <- newdata$time
      }
      # Remove time column if present (not needed for hazard/linear_predictor)
      newdata <- newdata[, names(newdata) != "time", drop = FALSE]
      if (ncol(newdata) > 0) {
        x <- as.matrix(newdata)
      } else {
        x <- NULL
      }
    }

    if (!is.null(time_windows)) {
      if (is.null(x)) {
        stop("Time-varying coefficients require predictors for '", type, "' predictions.", call. = FALSE)
      }
      if (is.null(pred_time)) {
        stop(
          "Time-varying coefficients require a 'time' column in newdata for '",
          type,
          "' predictions.",
          call. = FALSE
        )
      }
      # Apply the same piecewise expansion used at fit time.
      x <- .hzr_expand_time_varying_design(x = x, time = pred_time, time_windows = time_windows)
    }

    # Extract covariate coefficients by stripping shape parameters from theta.
    n_shape <- .hzr_shape_parameter_count(object$spec$dist)

    if (is.null(x)) {
      if (length(theta) <= n_shape) {
        # Univariable model: no covariates, eta = 0
        if (is.null(n_pred)) n_pred <- length(object$data$time)
        eta <- rep(0, n_pred)
      } else {
        stop("Predictors are required either in the fitted object or via 'newdata'.", call. = FALSE)
      }
    } else {
      beta <- if (length(theta) > n_shape) theta[(n_shape + 1):length(theta)] else theta
      if (ncol(x) != length(beta)) {
        stop("Number of predictor columns (", ncol(x),
             ") must match number of covariate coefficients (", length(beta), ").",
             call. = FALSE)
      }
      eta <- as.numeric(x %*% beta)
    }

    if (type == "linear_predictor") {
      if (!se.fit) return(eta)
      diff_fn <- function(th) {
        if (is.null(x)) return(rep(0, length(eta)))
        n_shape <- .hzr_shape_parameter_count(object$spec$dist)
        beta_cand <- if (length(th) > n_shape) th[(n_shape + 1):length(th)] else th
        as.numeric(x %*% beta_cand)
      }
      return(.hzr_predict_with_se(object = object, type = "linear_predictor",
                                    time = NULL, x = x, level = level,
                                    diff_fn = diff_fn))
    }
    if (!se.fit) return(exp(eta))
    diff_fn <- function(th) {
      if (is.null(x)) return(rep(1, length(eta)))
      n_shape <- .hzr_shape_parameter_count(object$spec$dist)
      beta_cand <- if (length(th) > n_shape) th[(n_shape + 1):length(th)] else th
      exp(as.numeric(x %*% beta_cand))
    }
    return(.hzr_predict_with_se(object = object, type = "hazard",
                                  time = NULL, x = x, level = level,
                                  diff_fn = diff_fn))
  }

  # -----------------------------------------------------------------------
  # Predictions that DO need time (survival, cumulative_hazard)
  # -----------------------------------------------------------------------
  # Each distribution computes H(t|x) in its own way; all then share the
  # same final return statements:
  #   cumulative_hazard -> return(cumhaz)
  #   survival          -> return(exp(-cumhaz))
  #
  # EXCEPTION: log-normal uses a direct Phi(-z) formula for survival and
  # returns early inside its branch, bypassing the shared return below.
  # This is because the log-normal is an AFT model where H(t) = -log Phi(-z)
  # is numerically better computed directly from the normal CDF rather than
  # via exp(-H).
  if (type %in% c("survival", "cumulative_hazard")) {
    supported <- c("weibull", "exponential", "loglogistic", "lognormal", "multiphase")
    if (!object$spec$dist %in% supported) {
      stop("Prediction type '", type, "' is only supported for ",
           paste(supported, collapse = ", "), " models.", call. = FALSE)
    }

    # --- Multiphase prediction (early return) ---------------------------------
    if (object$spec$dist == "multiphase") {
      # Extract time from newdata or fitted data
      if (!is.null(newdata)) {
        newdata <- as.data.frame(newdata)
        if (!"time" %in% names(newdata)) {
          stop("'newdata' must contain a 'time' column for '", type, "' predictions.",
               call. = FALSE)
        }
        pred_time <- newdata$time
      } else {
        pred_time <- object$data$time
      }

      if (!is.numeric(pred_time) || any(!is.finite(pred_time)) || any(pred_time < 0)) {
        stop("'time' must be a numeric vector of finite non-negative values.", call. = FALSE)
      }

      # Recover phase metadata from fit
      phases <- object$fit$phases
      if (is.null(phases)) phases <- object$spec$phases
      cov_counts <- object$fit$covariate_counts
      x_list <- object$fit$x_list

      # If newdata has covariates, rebuild per-phase design matrices
      if (!is.null(newdata)) {
        nd_covs <- newdata[, names(newdata) != "time", drop = FALSE]
        for (nm in names(phases)) {
          ph <- phases[[nm]]
          if (!is.null(ph$formula) && ncol(nd_covs) > 0) {
            x_list[[nm]] <- stats::model.matrix(ph$formula, data = newdata)[, -1L, drop = FALSE]
          } else if (cov_counts[[nm]] > 0 && ncol(nd_covs) > 0) {
            x_list[[nm]] <- as.matrix(nd_covs)
          } else {
            x_list[[nm]] <- NULL
          }
        }
      }

      if (se.fit) {
        diff_fn <- function(th) {
          .hzr_multiphase_cumhaz(pred_time, th, phases, cov_counts, x_list)
        }
        return(.hzr_predict_with_se(
          object = object, type = type, time = pred_time,
          x_list = x_list, cov_counts = cov_counts, phases = phases,
          level = level, diff_fn = diff_fn
        ))
      }

      result <- .hzr_multiphase_cumhaz(
        pred_time, theta, phases, cov_counts, x_list,
        per_phase = decompose
      )

      if (decompose) {
        # Return a data frame with time, total, and per-phase columns
        out <- data.frame(time = pred_time, total = result$total)
        for (nm in names(phases)) {
          out[[nm]] <- result[[nm]]
        }
        if (type == "survival") {
          out$total <- exp(-out$total)
          for (nm in names(phases)) {
            # Per-phase survival contribution is not additive; provide cumhaz
            # columns as-is and only transform total.
          }
        }
        return(out)
      }

      cumhaz <- result
      if (type == "cumulative_hazard") return(cumhaz)
      return(exp(-cumhaz))
    }

    # --- Standard single-distribution prediction ------------------------------
    # Extract time from newdata or use fitted data
    if (!is.null(newdata)) {
      newdata <- as.data.frame(newdata)
      if ("time" %in% names(newdata)) {
        time <- newdata$time
        x <- as.matrix(newdata[, names(newdata) != "time", drop = FALSE])
      } else {
        stop("'newdata' must contain a 'time' column for '", type, "' predictions.", call. = FALSE)
      }
    } else {
      time <- object$data$time
      x <- object$data$x
    }

    if (!is.null(time_windows) && !is.null(x) && ncol(x) > 0) {
      # Keep prediction design matrix consistent with training-time expansion.
      x <- .hzr_expand_time_varying_design(x = x, time = time, time_windows = time_windows)
    }

    if (!is.numeric(time) || any(!is.finite(time)) || any(time < 0)) {
      stop("'time' must be a numeric vector of finite non-negative values.", call. = FALSE)
    }

    # Extract shape parameters and covariate coefficients
    n_shape <- .hzr_shape_parameter_count(object$spec$dist)

    if (!is.null(x) && ncol(x) > 0) {
      if (length(theta) < ncol(x) + n_shape) {
        stop("Number of parameters insufficient for predictor columns.", call. = FALSE)
      }
      beta_coef <- theta[(n_shape + 1):length(theta)]
      eta <- as.numeric(x %*% beta_coef)
    } else {
      eta <- rep(0, length(time))
    }

    # Dispatch cumulative hazard computation by distribution.
    # Log-normal is an AFT model, so H is derived from the standardised
    # residual z = (log t - mu - x beta) / sigma.  For se.fit we define a
    # closure `cumhaz_of` that computes H for any candidate theta -- this
    # is the delta-method target for both "cumulative_hazard" and
    # "survival" predictions.
    dist_lbl <- object$spec$dist
    has_cov <- !is.null(x) && ncol(x) > 0

    # Preserve the pre-0.9.8 stop() behaviour on an ill-conditioned MLE.
    # The closures below return NA on negative shape parameters so numeric
    # jacobian perturbations stay robust, but we want a clean error at the
    # point estimate itself.
    if (dist_lbl == "weibull" && (theta[1] <= 0 || theta[2] <= 0)) {
      stop("Weibull shape parameters (mu, nu) must be positive.", call. = FALSE)
    }

    cumhaz_of <- if (dist_lbl == "weibull") {
      function(th) {
        if (th[1] <= 0 || th[2] <= 0) return(rep(NA_real_, length(time)))
        beta_cand <- if (length(th) > 2) th[3:length(th)] else numeric(0)
        eta_cand <- if (has_cov) as.numeric(x %*% beta_cand) else rep(0, length(time))
        (th[1] * time) ^ th[2] * exp(eta_cand)
      }
    } else if (dist_lbl == "exponential") {
      function(th) {
        beta_cand <- if (length(th) > 1) th[2:length(th)] else numeric(0)
        eta_cand <- if (has_cov) as.numeric(x %*% beta_cand) else rep(0, length(time))
        exp(th[1]) * time * exp(eta_cand)
      }
    } else if (dist_lbl == "loglogistic") {
      function(th) {
        beta_cand <- if (length(th) > 2) th[3:length(th)] else numeric(0)
        eta_cand <- if (has_cov) as.numeric(x %*% beta_cand) else rep(0, length(time))
        log(1 + exp(th[1]) * (time ^ exp(th[2])) * exp(eta_cand))
      }
    } else if (dist_lbl == "lognormal") {
      function(th) {
        beta_cand <- if (length(th) > 2) th[3:length(th)] else numeric(0)
        # AFT: covariates shift the location.
        eta_aft <- if (has_cov) th[1] + as.numeric(x %*% beta_cand) else rep(th[1], length(time))
        z <- (log(time) - eta_aft) / exp(th[2])
        -pnorm(-z, log.p = TRUE)
      }
    } else {
      stop("Unknown distribution '", dist_lbl, "'.", call. = FALSE)
    }

    if (se.fit) {
      return(.hzr_predict_with_se(
        object = object, type = type, time = time, x = x,
        level = level, diff_fn = cumhaz_of
      ))
    }

    cumhaz <- cumhaz_of(theta)
    if (type == "cumulative_hazard") return(cumhaz)
    return(exp(-cumhaz))
  }

  stop("Unknown prediction type: '", type, "'.", call. = FALSE)
}


#' Print method for fitted hazard models
#'
#' Compact one-block summary of a fitted `hazard` object: sample size,
#' number of predictors, distribution, theta vector, and log-likelihood.
#' S3 dispatch only -- users call `print(fit)` rather than invoking this
#' directly.
#'
#' @param x A `hazard` object returned by [hazard()].
#' @param ... Additional arguments (ignored).
#' @return `x`, invisibly.
#' @keywords internal
#' @export
print.hazard <- function(x, ...) {
  n <- length(x$data$time)
  p <- if (is.null(x$data$x)) 0L else ncol(x$data$x)
  cat("hazard object\n")
  cat("  observations:", n, "\n")
  cat("  predictors:  ", p, "\n")
  cat("  dist:        ", x$spec$dist, "\n")

  if (x$spec$dist == "multiphase" && !is.null(x$spec$phases)) {
    ph <- x$spec$phases
    cat("  phases:      ", length(ph),
        " (", paste(names(ph), collapse = ", "), ")\n", sep = "")
  }

  cat("  engine:      ", x$engine, "\n")
  if (!anyNA(x$fit$objective)) {
    cat("  log-lik:     ", format(x$fit$objective, digits = 6), "\n")
    cat("  converged:   ", x$fit$converged, "\n")
  }
  invisible(x)
}

#' Summarize a hazard model
#'
#' Returns a compact summary of a `hazard` object, including model metadata,
#' fit diagnostics, and coefficient-level statistics when available.
#'
#' @param object A `hazard` object.
#' @param ... Unused; for S3 compatibility.
#' @return An object of class `summary.hazard`.
#' @examples
#' # -- Single-phase Weibull summary ------------------------------------
#' fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' summary(fit)
#'
#' \donttest{
#' # -- Multiphase model summary ----------------------------------------
#' set.seed(42)
#' n   <- 200
#' dat <- data.frame(
#'   time   = rexp(n, rate = 0.25) + 0.01,
#'   status = rbinom(n, size = 1, prob = 0.65)
#' )
#' fit_mp <- hazard(
#'   survival::Surv(time, status) ~ 1,
#'   data   = dat,
#'   dist   = "multiphase",
#'   phases = list(
#'     early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                        fixed = "shapes"),
#'     late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
#'                        fixed = "shapes")
#'   ),
#'   fit     = TRUE,
#'   control = list(n_starts = 5, maxit = 1000)
#' )
#' summary(fit_mp)
#' }
#'
#' @seealso
#' [hazard()] for model fitting, [predict.hazard()] for predictions.
#'
#' \code{vignette("fitting-hazard-models")} for fitting workflows,
#' \code{vignette("inference-diagnostics")} for bootstrap CIs and diagnostics.
#' @export
summary.hazard <- function(object, ...) {
  n <- length(object$data$time)
  p <- if (is.null(object$data$x)) 0L else ncol(object$data$x)
  theta <- object$fit$theta
  vcov_mat <- object$fit$vcov

  coef_table <- NULL
  if (!is.null(theta)) {
    # For multiphase, use the theta names directly (they're already informative)
    if (object$spec$dist == "multiphase") {
      coef_names <- names(theta)
      if (is.null(coef_names)) coef_names <- paste0("param_", seq_along(theta))
    } else {
      coef_names <- .hzr_parameter_names(theta = theta, dist = object$spec$dist, p = p)
    }

    std_error <- rep(NA_real_, length(theta))
    z_stat <- rep(NA_real_, length(theta))
    p_value <- rep(NA_real_, length(theta))

    if (!is.null(vcov_mat) && is.matrix(vcov_mat)) {
      d <- diag(vcov_mat)
      std_error <- sqrt(d)            # NA for fixed params, finite for free
      valid <- is.finite(std_error) & std_error > 0
      z_stat[valid] <- theta[valid] / std_error[valid]
      p_value[valid] <- 2 * pnorm(-abs(z_stat[valid]))
    }

    coef_table <- data.frame(
      estimate = unname(theta),
      std_error = std_error,
      z_stat = z_stat,
      p_value = p_value,
      row.names = coef_names,
      check.names = FALSE
    )
  }

  out <- list(
    call = object$call,
    n = n,
    p = p,
    dist = object$spec$dist,
    engine = object$engine,
    converged = object$fit$converged,
    log_lik = object$fit$objective,
    counts = object$fit$counts,
    message = object$fit$message,
    coefficients = coef_table,
    has_vcov = !is.null(vcov_mat) && is.matrix(vcov_mat),
    phases = object$spec$phases
  )

  class(out) <- "summary.hazard"
  out
}

#' Print method for hazard summary objects
#'
#' Formatted console display of [summary.hazard()] output: distribution,
#' phase list (for multiphase), coefficient table with standard errors,
#' and log-likelihood.  S3 dispatch only -- users call `print(summary(fit))`
#' rather than invoking this directly.
#'
#' @param x A `summary.hazard` object returned by [summary.hazard()].
#' @param ... Additional arguments (ignored).
#' @return `x`, invisibly.
#' @keywords internal
#' @export
print.summary.hazard <- function(x, ...) {
  if (x$dist == "multiphase" && !is.null(x$phases)) {
    cat("Multiphase hazard model (", length(x$phases), " phases)\n", sep = "")
  } else {
    cat("hazard model summary\n")
  }
  cat("  observations:", x$n, "\n")
  cat("  predictors:  ", x$p, "\n")
  cat("  dist:        ", x$dist, "\n")

  if (!is.null(x$phases)) {
    for (i in seq_along(x$phases)) {
      nm <- names(x$phases)[i]
      ph <- x$phases[[i]]
      label <- switch(ph$type,
        cdf      = paste0("cdf (", nm, " risk)"),
        hazard   = "hazard (late risk)",
        constant = "constant (flat rate)",
        g3       = "g3 (late risk)")
      cat("  phase ", i, ":      ", nm, " - ", label, "\n", sep = "")
    }
  }

  cat("  engine:      ", x$engine, "\n")

  if (!is.null(x$converged) && !is.na(x$converged)) {
    cat("  converged:   ", x$converged, "\n")
  }
  if (!is.null(x$log_lik) && !is.na(x$log_lik)) {
    cat("  log-lik:     ", format(x$log_lik, digits = 6), "\n")
  }
  if (!is.null(x$counts)) {
    fn_count <- x$counts[["function"]] %||% x$counts[["fn"]] %||% NA_integer_
    gr_count <- x$counts[["gradient"]] %||% x$counts[["gr"]] %||% NA_integer_
    if (!is.na(fn_count) || !is.na(gr_count)) {
      cat("  evaluations: ", "fn=", fn_count, ", gr=", gr_count, "\n", sep = "")
    }
  }
  if (!is.null(x$message) && nzchar(x$message)) {
    cat("  message:     ", x$message, "\n")
  }

  if (!is.null(x$coefficients)) {
    if (!is.null(x$phases)) {
      # Group coefficients by phase for readable output
      cat("\nCoefficients (internal scale):\n")
      for (nm in names(x$phases)) {
        prefix <- paste0("^", nm, "\\.")
        rows <- grep(prefix, rownames(x$coefficients))
        if (length(rows) > 0) {
          ph <- x$phases[[nm]]
          label <- switch(ph$type,
            cdf = "cdf", hazard = "hazard", constant = "constant",
            g3 = "g3")
          cat("\n  Phase: ", nm, " (", label, ")\n", sep = "")
          sub_table <- x$coefficients[rows, , drop = FALSE]
          # Strip phase prefix from row names for cleaner display
          rownames(sub_table) <- sub(prefix, "  ", rownames(sub_table))
          print(sub_table)
        }
      }
    } else {
      cat("\nCoefficients:\n")
      print(x$coefficients)
    }
  }

  invisible(x)
}

#' Extract coefficients from hazard model
#'
#' @param object A `hazard` object.
#' @param ... Unused; for S3 compatibility.
#' @examples
#' fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' coef(fit)
#' @return A named numeric vector of fitted parameter estimates, or \code{NULL}
#'   if the model has not been fitted (\code{fit = FALSE}).
#' @export
coef.hazard <- function(object, ...) {
  if (is.null(object$fit$theta)) {
    return(NULL)
  }
  object$fit$theta
}

#' Extract variance-covariance matrix from hazard model
#'
#' Returns the estimated variance-covariance matrix of the fitted coefficients.
#'
#' @param object A `hazard` object.
#' @param ... Unused; for S3 compatibility.
#' @examples
#' fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' vcov(fit)
#' @return A numeric matrix containing the estimated variance-covariance matrix
#'   of the fitted coefficients, or \code{NA} if the model has not been fitted
#'   or the covariance matrix is unavailable.
#' @export
vcov.hazard <- function(object, ...) {
  if (is.null(object$fit$vcov) || anyNA(object$fit$vcov)) {
    return(NA)
  }
  object$fit$vcov
}

.hzr_parameter_names <- function(theta, dist, p) {
  theta_names <- names(theta)
  if (!is.null(theta_names) && all(nzchar(theta_names))) {
    return(theta_names)
  }

  if (!is.null(p) && p > 0L && length(theta) == p) {
    return(paste0("beta", seq_along(theta)))
  }

  base_names <- switch(
    dist,
    weibull = c("mu", "nu"),
    exponential = c("log_lambda"),
    loglogistic = c("log_alpha", "log_beta"),
    lognormal = c("mu", "log_sigma"),
    character()
  )

  n_theta <- length(theta)
  n_base <- min(length(base_names), n_theta)
  out <- character(n_theta)

  if (n_base > 0) {
    out[seq_len(n_base)] <- base_names[seq_len(n_base)]
  }
  if (n_theta > n_base) {
    out[seq.int(n_base + 1L, n_theta)] <- paste0("beta", seq_len(n_theta - n_base))
  }
  out
}

#' Coerce x to a validated numeric design matrix
#'
#' Accepts data.frame or numeric matrix input; validates dimensions and
#' finiteness.  Called by hazard() to normalise the x argument before any
#' downstream use.
#'
#' @param x A numeric matrix or data frame with numeric columns.
#' @param n Expected row count (length of time/status vectors); checked if non-NULL.
#' @return A numeric matrix with column names preserved (or added if absent).
#' @keywords internal
#'
.hzr_as_design_matrix <- function(x, n = NULL) {
  if (is.data.frame(x)) {
    x <- data.matrix(x)
  }

  if (!is.matrix(x) || !is.numeric(x)) {
    stop("Predictor input must be a numeric matrix or coercible data frame.", call. = FALSE)
  }

  if (any(!is.finite(x))) {
    stop("Predictor matrix contains non-finite values.", call. = FALSE)
  }

  if (!is.null(n) && nrow(x) != n) {
    stop("Predictor rows must match the length of 'time'.", call. = FALSE)
  }

  x
}

#' Expand predictors for piecewise time-varying coefficients
#'
#' Builds a block design matrix with one predictor block per time window. For each
#' observation, only the block corresponding to that row's time window is active;
#' all other blocks are zero.
#'
#' Example with two predictors and one cut point:
#' - input columns: `x1`, `x2`
#' - windows: `(-Inf, c1]`, `(c1, Inf)`
#' - output columns: `x1_w1`, `x2_w1`, `x1_w2`, `x2_w2`
#'
#' @param x Numeric predictor matrix/data.frame.
#' @param time Numeric time vector used to assign rows to windows.
#' @param time_windows Numeric vector of window cut points.
#' @return Expanded numeric matrix with window-specific column names.
#' @noRd
.hzr_expand_time_varying_design <- function(x, time, time_windows) {
  if (is.data.frame(x)) {
    x <- data.matrix(x)
  }

  if (!is.matrix(x) || !is.numeric(x)) {
    stop("Time-varying design requires numeric matrix/data.frame predictors.", call. = FALSE)
  }

  if (!is.numeric(time) || length(time) != nrow(x) || any(!is.finite(time)) || any(time < 0)) {
    stop("'time' must be finite, non-negative, and match predictor rows for time-varying expansion.", call. = FALSE)
  }

  if (!is.numeric(time_windows) || length(time_windows) < 1) {
    stop("'time_windows' must contain at least one cut point.", call. = FALSE)
  }

  time_windows <- sort(unique(time_windows))
  # Bin index k means observation time falls in window k.
  bins <- cut(time, breaks = c(-Inf, time_windows, Inf), right = TRUE, include.lowest = TRUE, labels = FALSE)
  n_bins <- length(time_windows) + 1L

  base_names <- colnames(x)
  if (is.null(base_names)) {
    base_names <- paste0("x", seq_len(ncol(x)))
  }

  out <- matrix(0, nrow = nrow(x), ncol = ncol(x) * n_bins)
  col_ptr <- 1L
  out_names <- character(ncol(out))

  for (k in seq_len(n_bins)) {
    idx <- bins == k
    # Populate only the active window block for rows in this bin.
    block <- matrix(0, nrow = nrow(x), ncol = ncol(x))
    if (any(idx)) {
      block[idx, ] <- x[idx, , drop = FALSE]
    }
    cols <- col_ptr:(col_ptr + ncol(x) - 1L)
    out[, cols] <- block
    out_names[cols] <- paste0(base_names, "_w", k)
    col_ptr <- col_ptr + ncol(x)
  }

  colnames(out) <- out_names
  out
}

# tests/testthat/test-phase-specific-covariates.R
#
# Phase 7d: correctness tests for phase-specific covariates (hzr_phase(formula=)).
#
# hzr_phase(formula = ~ ...) lets a covariate modulate the scale of a single
# phase, so the same covariate can carry different effects in different phases.
# The shipped smoke test only checks that such a fit runs; it does not check
# that the coefficients are correct or that a phase-specific covariate stays
# confined to its phase.
#
# These are simulation-based recovery tests -- the honest substitute for a SAS
# parity fixture when no reference output exists. They guard against this
# package's recurring "accepts the formal but never applies it" failure mode
# (seen with weights, counting-process times, and CoE weights) by confirming a
# covariate entered into a specific phase (a) recovers its true coefficient and
# (b) affects only that phase.

# ---------------------------------------------------------------------------
# Simulation helper: additive two-phase model (CDF early + constant), with
# optional phase-specific covariate effects on each phase's scale.
#   H(t | x) = mu_e(x) * Phi_early(t) + mu_c(x) * t
# Event times are drawn by inverting the additive cumulative hazard on a grid.
# ---------------------------------------------------------------------------
.sim_phase_cov <- function(n, beta_early = NULL, beta_const = NULL,
                           log_mu_e = -1.5, log_mu_c = -2.5,
                           t_half = 0.5, tmax = 80, seed = 1) {
  set.seed(seed)
  x      <- rnorm(n)
  tgrid  <- seq(0.001, tmax, length.out = 4000)
  Phi_e  <- hzr_phase_cumhaz(tgrid, t_half = t_half, nu = 1, m = 1, type = "cdf")
  U      <- -log(runif(n))
  sim_t  <- numeric(n)
  status <- integer(n)
  for (i in seq_len(n)) {
    mu_e <- exp(log_mu_e + if (!is.null(beta_early)) beta_early * x[i] else 0)
    mu_c <- exp(log_mu_c + if (!is.null(beta_const)) beta_const * x[i] else 0)
    H    <- mu_e * Phi_e + mu_c * tgrid
    # First grid index where the accumulated hazard reaches the target U[i];
    # NA only when the event never occurs within the grid (right-censored).
    j    <- match(TRUE, H >= U[i])
    if (is.na(j)) {
      sim_t[i]  <- max(tgrid)
      status[i] <- 0L            # right-censored: event beyond the grid
    } else {
      sim_t[i]  <- tgrid[j]
      status[i] <- 1L            # event observed (even if on the last point)
    }
  }
  data.frame(
    int_dead = sim_t,
    dead     = status,
    x        = x
  )
}

# ---------------------------------------------------------------------------
# Structural tests (fast, always run): a phase-specific covariate produces a
# coefficient for its own phase only.
# ---------------------------------------------------------------------------

test_that("a covariate in one phase produces a beta for that phase only", {
  df <- .sim_phase_cov(n = 200, beta_early = 0.8, seed = 3)

  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = df,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes", formula = ~ x),
      constant = hzr_phase("constant")
    ),
    fit     = TRUE,
    control = list(n_starts = 1L, maxit = 50L)
  )

  nm <- names(coef(fit))
  expect_true("early.x" %in% nm)
  expect_false("constant.x" %in% nm)
  # covariate_counts reflects the per-phase covariate placement
  expect_equal(unname(fit$fit$covariate_counts[["early"]]), 1L)
  expect_equal(unname(fit$fit$covariate_counts[["constant"]]), 0L)
})

test_that("the same covariate can enter two phases independently", {
  df <- .sim_phase_cov(n = 200, beta_early = 0.8, beta_const = -0.4, seed = 4)

  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = df,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes", formula = ~ x),
      constant = hzr_phase("constant", formula = ~ x)
    ),
    fit     = TRUE,
    control = list(n_starts = 1L, maxit = 50L)
  )

  nm <- names(coef(fit))
  expect_true(all(c("early.x", "constant.x") %in% nm))
  expect_equal(unname(fit$fit$covariate_counts[["early"]]), 1L)
  expect_equal(unname(fit$fit$covariate_counts[["constant"]]), 1L)
})

# ---------------------------------------------------------------------------
# Recovery tests (heavier simulations; skip on CRAN): the fitted coefficients
# match the data-generating truth.
# ---------------------------------------------------------------------------

test_that("phase-specific covariate in the early phase is recovered", {
  skip_on_cran()
  beta_early <- 0.8
  log_mu_e   <- -1.5
  log_mu_c   <- -3.0
  df <- .sim_phase_cov(n = 3000, beta_early = beta_early,
                       log_mu_e = log_mu_e, log_mu_c = log_mu_c, seed = 7)

  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = df,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes", formula = ~ x),
      constant = hzr_phase("constant")
    ),
    fit     = TRUE,
    control = list(n_starts = 3L, maxit = 1000L)
  )
  co <- coef(fit)

  expect_equal(unname(co[["early.x"]]),        beta_early, tolerance = 0.12)
  expect_equal(unname(co[["early.log_mu"]]),   log_mu_e,   tolerance = 0.15)
  expect_equal(unname(co[["constant.log_mu"]]), log_mu_c,  tolerance = 0.20)
})

test_that("the same covariate recovers opposite-sign effects across phases", {
  skip_on_cran()
  beta_early <- 1.0
  beta_const <- -0.6
  log_mu_e   <- -1.5
  log_mu_c   <- -2.5
  df <- .sim_phase_cov(n = 5000, beta_early = beta_early, beta_const = beta_const,
                       log_mu_e = log_mu_e, log_mu_c = log_mu_c, seed = 11)

  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = df,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes", formula = ~ x),
      constant = hzr_phase("constant", formula = ~ x)
    ),
    fit     = TRUE,
    control = list(n_starts = 3L, maxit = 1000L)
  )
  co <- coef(fit)

  # The defining feature: one covariate, two different (here opposite-sign)
  # phase-specific effects, both recovered.
  expect_equal(unname(co[["early.x"]]),    beta_early, tolerance = 0.12)
  expect_equal(unname(co[["constant.x"]]), beta_const, tolerance = 0.12)
  expect_true(co[["early.x"]] > 0)
  expect_true(co[["constant.x"]] < 0)
})

test_that("a covariate confined to one phase does not leak into the other", {
  skip_on_cran()
  # Truth: x affects ONLY the early phase. A model that also offers x to the
  # constant phase should estimate the constant-phase effect near zero.
  df <- .sim_phase_cov(n = 5000, beta_early = 1.0, beta_const = NULL,
                       log_mu_e = -1.5, log_mu_c = -2.5, seed = 13)

  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = df,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes", formula = ~ x),
      constant = hzr_phase("constant", formula = ~ x)
    ),
    fit     = TRUE,
    control = list(n_starts = 3L, maxit = 1000L)
  )
  co <- coef(fit)

  expect_equal(unname(co[["early.x"]]),    1.0, tolerance = 0.15)
  expect_equal(unname(co[["constant.x"]]), 0.0, tolerance = 0.15)
})

# ---------------------------------------------------------------------------
# Regression test: formula preprocessor parse-tree walk (not string regex)
# ---------------------------------------------------------------------------

test_that("formula with base-R function name matching phase name is not wrongly stripped", {
  # Regression for: the old string-regex `grepl("\\blog\\s*\\(", rhs_txt)`
  # would fire on `~ log(age)` when a phase happened to be named "log".
  # The parse-tree walk in .hzr_formula_has_phase_scope() must NOT strip a
  # formula whose only use of `log` is as a standard R function, not as a
  # phase-scoped call — even when the phase list contains no phase named "log".
  # Here we test the helper directly with a contrived phase name.
  df <- .sim_phase_cov(n = 200, beta_early = 0.5, seed = 99)
  df$log_age <- log(df$x + 2)  # a predictor that uses log()

  phases <- list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes", formula = ~ log_age),
    constant = hzr_phase("constant")
  )
  # The global formula uses log() as a base-R function on the RHS.
  # Phase names are "early" and "constant" — neither is "log".
  # The preprocessor must leave the formula intact (no stripping to ~ 1).
  # If it wrongly strips, model.matrix() gets `~ 1` and the fit will lack
  # the log_age covariate, causing covariate_counts[["early"]] == 0.
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data = df, dist = "multiphase", phases = phases,
    fit = TRUE, control = list(n_starts = 1L)
  )
  # Covariate was routed via hzr_phase(formula = ~ log_age), not the global
  # formula, so covariate_counts should still be 1 regardless.
  expect_equal(unname(fit$fit$covariate_counts[["early"]]), 1L,
               label = "log_age covariate routed correctly via phase formula")

  # Now test the helper directly: a formula ~ log(age) with phases named
  # "early"/"constant" must NOT trigger phase-scope detection.
  rhs_no_scope <- quote(log(age) + offset(z))
  expect_false(
    .hzr_formula_has_phase_scope(rhs_no_scope, c("early", "constant")),
    label = "log() call not mistaken for phase scope"
  )

  # And a genuine phase-scoped formula MUST trigger it.
  rhs_with_scope <- quote(early(age + x) + constant(z))
  expect_true(
    .hzr_formula_has_phase_scope(rhs_with_scope, c("early", "constant")),
    label = "early() call correctly detected as phase scope"
  )
})

test_that("Univariable model recovers estimated parameters reproducibly", {
  # Load the golden fixture
  fixture_file <- system.file("fixtures", "hz_univariate.rds", package = "TemporalHazard")
  fixture <- readRDS(fixture_file)

  # Skip if fixture doesn't exist
  skip_if_not(file.exists(fixture_file))

  # Re-fit the same model
  refit <- hazard(
    time = fixture$data$time,
    status = fixture$data$status,
    x = NULL,
    theta = c(mu = 0.3, nu = 1.0),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 500, reltol = 1e-6)
  )

  # Model should have fit results
  expect_true(!is.null(refit$fit))
  expect_true("theta" %in% names(refit$fit))

  # Parameter estimates should match fixture closely (within tolerance)
  # Allow slightly higher tolerance due to optimization variations
  expect_equal(refit$fit$theta, fixture$fit$theta, tolerance = 1e-3)

  # Log-likelihood should be close
  expect_equal(refit$fit$objective, fixture$fit$logl, tolerance = 1e-2)
})

test_that("Multivariable model with covariates recovers parameters", {
  fixture_file <- system.file("fixtures", "hm_multivariate.rds", package = "TemporalHazard")
  fixture <- readRDS(fixture_file)

  skip_if_not(file.exists(fixture_file))

  # Re-fit
  refit <- hazard(
    time = fixture$data$time,
    status = fixture$data$status,
    x = fixture$data$x,
    theta = c(mu = 0.4, nu = 1.0, beta_X1 = -0.2, beta_X2 = 0.3),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 500, reltol = 1e-6)
  )

  # Model should have fit results
  expect_true(!is.null(refit$fit))
  expect_true("theta" %in% names(refit$fit))

  # Parameter estimates within tolerance
  expect_equal(refit$fit$theta, fixture$fit$theta, tolerance = 1e-3)

  # Log-likelihood should be close
  expect_equal(refit$fit$objective, fixture$fit$logl, tolerance = 1e-2)
})

test_that("Edge case: small sample with many covariates", {
  fixture_file <- system.file("fixtures", "hm_edge_case.rds", package = "TemporalHazard")
  fixture <- readRDS(fixture_file)

  skip_if_not(file.exists(fixture_file))

  # Re-fit
  refit <- hazard(
    time = fixture$data$time,
    status = fixture$data$status,
    x = fixture$data$x,
    theta = c(mu = 0.5, nu = 1.2, beta_X1 = 0, beta_X2 = 0, beta_X3 = 0),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 300, reltol = 1e-5)
  )

  # Check convergence flag (may not always converge in 300 iters)
  expect_type(refit$fit, "list")
  expect_true("theta" %in% names(refit$fit))
  expect_true(all(is.finite(refit$fit$theta)))
  expect_true(is.finite(refit$fit$objective))

  # This fixture is intentionally ill-conditioned (small n, high leverage),
  # so assert objective parity rather than exact parameter recovery.
  expect_equal(refit$fit$objective, fixture$fit$logl, tolerance = 5)
})

test_that("Univariable predictions are computable", {
  fixture_file <- system.file("fixtures", "hz_univariate.rds", package = "TemporalHazard")
  fixture <- readRDS(fixture_file)

  skip_if_not(file.exists(fixture_file))

  # Refit model
  refit <- hazard(
    time = fixture$data$time,
    status = fixture$data$status,
    x = NULL,
    theta = c(mu = 0.3, nu = 1.0),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 500, reltol = 1e-6)
  )

  # Predictions should be computable with time data
  pred1 <- predict(refit, type = "linear_predictor",
                   newdata = data.frame(time = fixture$data$time[1:5]))
  pred2 <- predict(refit, type = "hazard",
                   newdata = data.frame(time = fixture$data$time[1:5]))

  # Should return numeric vectors
  expect_true(is.numeric(pred1))
  expect_true(is.numeric(pred2))
  expect_true(length(pred1) ==  5)
  expect_true(length(pred2) == 5)
})

test_that("Convergence codes are reasonable", {
  fixture_file <- system.file("fixtures", "hz_univariate.rds", package = "TemporalHazard")
  fixture <- readRDS(fixture_file)

  skip_if_not(file.exists(fixture_file))

  # Refit
  refit <- hazard(
    time = fixture$data$time,
    status = fixture$data$status,
    x = NULL,
    theta = c(mu = 0.3, nu = 1.0),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 500, reltol = 1e-6)
  )

  # Convergence code should be logical or integer (TRUE or FALSE for success)
  expect_true(is.logical(refit$fit$converged) || is.integer(refit$fit$converged))
})

library(testthat)

test_that("time-varying design expansion dimensions and names are correct", {
  x <- cbind(x1 = c(1, 2, 3, 4), x2 = c(2, 2, 2, 2))
  time <- c(0.5, 1.5, 2.5, 3.5)
  windows <- c(1, 2)

  x_tv <- .hzr_expand_time_varying_design(x, time, windows)

  expect_equal(dim(x_tv), c(4, 6))
  expect_equal(colnames(x_tv), c("x1_w1", "x2_w1", "x1_w2", "x2_w2", "x1_w3", "x2_w3"))

  expect_equal(unname(x_tv[1, ]), c(1, 2, 0, 0, 0, 0))
  expect_equal(unname(x_tv[2, ]), c(0, 0, 2, 2, 0, 0))
  expect_equal(unname(x_tv[3, ]), c(0, 0, 0, 0, 3, 2))
})

test_that("predict linear_predictor applies window-specific coefficients", {
  time <- c(0.5, 1.5, 2.5)
  x <- cbind(x1 = c(1, 1, 1))

  # exponential: theta = [log_lambda, beta_w1, beta_w2, beta_w3]
  fit <- hazard(
    time = time,
    status = c(1, 1, 1),
    x = x,
    time_windows = c(1, 2),
    theta = c(log(0.2), 0.0, 1.0, 2.0),
    dist = "exponential",
    fit = FALSE
  )

  pred <- predict(
    fit,
    newdata = data.frame(time = time, x1 = c(1, 1, 1)),
    type = "linear_predictor"
  )

  expect_equal(pred, c(0, 1, 2), tolerance = 1e-12)
})

test_that("predict hazard and cumulative_hazard remain coherent with time-varying coefficients", {
  time <- c(0.5, 1.5, 2.5)
  x <- cbind(x1 = c(1, 1, 1))

  fit <- hazard(
    time = time,
    status = c(1, 1, 1),
    x = x,
    time_windows = c(1, 2),
    theta = c(log(0.3), 0.0, 0.5, 1.0),
    dist = "exponential",
    fit = FALSE
  )

  lp <- predict(fit, newdata = data.frame(time = time, x1 = 1), type = "linear_predictor")
  hz_mult <- predict(fit, newdata = data.frame(time = time, x1 = 1), type = "hazard")
  ch <- predict(fit, newdata = data.frame(time = time, x1 = 1), type = "cumulative_hazard")
  s <- predict(fit, newdata = data.frame(time = time, x1 = 1), type = "survival")

  expect_equal(hz_mult, exp(lp), tolerance = 1e-12)

  lambda <- 0.3
  expect_equal(ch, lambda * time * exp(lp), tolerance = 1e-10)
  expect_equal(s, exp(-ch), tolerance = 1e-12)
})

test_that("time-varying predictor requires time at prediction for newdata", {
  fit <- hazard(
    time = c(1, 2, 3),
    status = c(1, 1, 1),
    x = cbind(x1 = c(1, 1, 1)),
    time_windows = c(1.5),
    theta = c(log(0.2), 0.1, 0.2),
    dist = "exponential",
    fit = FALSE
  )

  expect_error(
    predict(fit, newdata = data.frame(x1 = c(1, 1)), type = "linear_predictor"),
    "require a 'time' column"
  )
})

test_that("time-varying fit runs for Weibull and keeps expanded coefficient count", {
  set.seed(1001)
  n <- 80
  time <- rexp(n, rate = 0.4) + 0.05
  status <- rbinom(n, 1, 0.7)
  x <- cbind(x1 = rnorm(n))

  fit <- hazard(
    time = time,
    status = status,
    x = x,
    time_windows = c(1.5, 3.0),
    # Weibull: [mu, nu, beta_w1, beta_w2, beta_w3]
    theta = c(0.5, 1.1, 0.0, 0.0, 0.0),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 200)
  )

  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$fit$theta), 5)
})

test_that("theta length validation reflects expanded time-varying design", {
  expect_error(
    hazard(
      time = c(1, 2, 3),
      status = c(1, 0, 1),
      x = cbind(x1 = c(1, 2, 3)),
      time_windows = c(1.5, 2.5),
      # needs at least 3 coefficients for expanded design (plus shape, depending dist)
      theta = c(0.1, 0.2),
      dist = "exponential",
      fit = FALSE
    ),
    "required coefficients"
  )
})

# test-multiphase-api.R — End-to-end tests for multiphase hazard via hazard()
#
# Tests construction, fitting, predict, summary, and print for dist="multiphase".

# ============================================================================
# Helpers
# ============================================================================

make_multiphase_data <- function(n = 200, seed = 42) {
  set.seed(seed)
  data.frame(
    time   = rexp(n, rate = 0.3) + 0.01,
    status = rbinom(n, 1, prob = 0.7),
    age    = rnorm(n, mean = 60, sd = 10),
    nyha   = sample(1:4, n, replace = TRUE)
  )
}


# ============================================================================
# hazard() construction with phases
# ============================================================================

test_that("hazard() accepts dist='multiphase' with phases", {
  dat <- make_multiphase_data(n = 50)

  fit <- hazard(
    survival::Surv(time, status) ~ age,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE
  )

  expect_s3_class(fit, "hazard")
  expect_equal(fit$spec$dist, "multiphase")
  expect_true(!is.null(fit$spec$phases))
  expect_equal(length(fit$spec$phases), 2)
  expect_equal(names(fit$spec$phases), c("early", "bg"))
})

test_that("hazard() errors when dist='multiphase' without phases", {
  dat <- make_multiphase_data(n = 50)
  expect_error(
    hazard(survival::Surv(time, status) ~ age, data = dat,
           dist = "multiphase", fit = TRUE),
    "phases.*required"
  )
})

test_that("hazard() warns when phases given with non-multiphase dist", {
  dat <- make_multiphase_data(n = 50)
  expect_warning(
    hazard(survival::Surv(time, status) ~ age, data = dat,
           dist = "weibull", theta = c(0.3, 1, 0),
           phases = list(hzr_phase("cdf")),
           fit = FALSE),
    "phases.*ignored"
  )
})


# ============================================================================
# Single-phase (constant) multiphase — should recover exponential-like fit
# ============================================================================

test_that("single constant phase fits and converges", {
  skip_on_cran()

  set.seed(99)
  dat <- data.frame(
    time   = rexp(200, rate = 0.3) + 0.01,
    status = rep(1L, 200)
  )

  fit <- hazard(
    time = dat$time, status = dat$status,
    dist = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE
  )

  expect_true(fit$fit$converged)
  expect_true(is.finite(fit$fit$objective))

  # theta should have bg.log_mu
  expect_true("bg.log_mu" %in% names(fit$fit$theta))
})


# ============================================================================
# Two-phase fit
# ============================================================================

test_that("two-phase (cdf + constant) fits and stores phases", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 150)

  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  expect_s3_class(fit, "hazard")
  expect_true(is.finite(fit$fit$objective))

  # Phases metadata should be stored
  expect_true(!is.null(fit$fit$phases))
  expect_equal(names(fit$fit$phases), c("early", "bg"))
  expect_true(!is.null(fit$fit$covariate_counts))
})


# ============================================================================
# predict.hazard() for multiphase
# ============================================================================

test_that("predict survival and cumhaz for multiphase model", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 100)

  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  # Predict on training data
  surv <- predict(fit, type = "survival")
  cumhaz <- predict(fit, type = "cumulative_hazard")

  expect_true(all(is.finite(surv)))
  expect_true(all(surv >= 0 & surv <= 1))
  expect_true(all(is.finite(cumhaz)))
  expect_true(all(cumhaz >= 0))
  expect_equal(surv, exp(-cumhaz), tolerance = 1e-10)
})

test_that("predict with newdata for multiphase model", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 100)

  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  nd <- data.frame(time = c(0.5, 1.0, 2.0, 5.0))
  surv <- predict(fit, newdata = nd, type = "survival")

  expect_equal(length(surv), 4)
  expect_true(all(surv >= 0 & surv <= 1))
  # Survival should be monotone decreasing
  expect_true(all(diff(surv) <= 0))
})

test_that("predict decompose=TRUE returns data frame", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 100)

  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  nd <- data.frame(time = c(0.5, 1.0, 2.0))
  decomp <- predict(fit, newdata = nd, type = "cumulative_hazard",
                    decompose = TRUE)

  expect_true(is.data.frame(decomp))
  expect_true("time" %in% names(decomp))
  expect_true("total" %in% names(decomp))
  expect_true("early" %in% names(decomp))
  expect_true("bg" %in% names(decomp))

  # Total should equal sum of components
  expect_equal(decomp$total, decomp$early + decomp$bg, tolerance = 1e-10)
})

test_that("predict linear_predictor errors for multiphase", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 50)
  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE,
    control = list(n_starts = 1, maxit = 300)
  )

  expect_error(predict(fit, type = "linear_predictor"), "not supported.*multiphase")
})


# ============================================================================
# print and summary
# ============================================================================

test_that("print.hazard shows phase info for multiphase", {
  dat <- make_multiphase_data(n = 50)
  fit <- hazard(
    time = dat$time, status = dat$status,
    dist = "multiphase",
    phases = list(
      early = hzr_phase("cdf"),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 1, maxit = 300)
  )

  out <- capture.output(print(fit))
  expect_true(any(grepl("multiphase", out)))
  expect_true(any(grepl("phases.*2", out)))
})

test_that("summary.hazard shows per-phase coefficients", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 100)
  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  s <- summary(fit)
  expect_s3_class(s, "summary.hazard")
  expect_equal(s$dist, "multiphase")
  expect_true(!is.null(s$phases))
  expect_true(!is.null(s$coefficients))

  # Should have rows for both phases
  rn <- rownames(s$coefficients)
  expect_true(any(grepl("^early\\.", rn)))
  expect_true(any(grepl("^bg\\.", rn)))

  # Print should work without error
  out <- capture.output(print(s))
  expect_true(any(grepl("Multiphase", out)))
  expect_true(any(grepl("Phase.*early", out)))
  expect_true(any(grepl("Phase.*bg", out)))
})


# ============================================================================
# coef and vcov
# ============================================================================

test_that("coef returns named theta for multiphase", {
  dat <- make_multiphase_data(n = 50)
  fit <- hazard(
    time = dat$time, status = dat$status,
    dist = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE,
    control = list(n_starts = 1, maxit = 300)
  )

  cf <- coef(fit)
  expect_true(!is.null(cf))
  expect_true(!is.null(names(cf)))
  expect_true("bg.log_mu" %in% names(cf))
})


# ============================================================================
# Three-phase classic pattern
# ============================================================================

test_that("three-phase (cdf + constant + cdf) constructs and fits", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 200)

  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0),
      constant = hzr_phase("constant"),
      late     = hzr_phase("cdf",      t_half = 5,   nu = 1, m = 0)
    ),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  expect_s3_class(fit, "hazard")
  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$spec$phases), 3)

  # Predictions should work
  nd <- data.frame(time = c(0.5, 2, 8))
  surv <- predict(fit, newdata = nd, type = "survival")
  expect_equal(length(surv), 3)
  expect_true(all(surv >= 0 & surv <= 1))
})

test_that("hazard() builds a hazard object", {
  x <- matrix(c(1, 0, 0, 1), ncol = 2)
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = x,
    theta = c(0.2, -0.1),
    dist = "weibull",
    maxit = 50
  )

  expect_s3_class(fit, "hazard")
  expect_equal(fit$spec$dist, "weibull")
  expect_equal(fit$legacy_args$maxit, 50)
})

test_that("hazard() validates core dimensions", {
  expect_error(
    hazard(time = c(1, 2), status = 1),
    "same length"
  )

  expect_error(
    hazard(time = c(1, 2), status = c(1, 0), x = matrix(1, nrow = 3, ncol = 1)),
    "rows must match"
  )
})

test_that("predict.hazard returns linear predictor and hazard scale", {
  x <- matrix(c(1, 0, 0, 1), ncol = 2)
  fit <- hazard(time = c(1, 2), status = c(1, 0), x = x, theta = c(0.3, -0.2))

  eta <- predict(fit, type = "linear_predictor")
  hz <- predict(fit, type = "hazard")

  expect_equal(eta, c(0.3, -0.2), tolerance = 1e-12)
  expect_equal(hz, exp(eta), tolerance = 1e-12)
})

test_that("predict.hazard accepts newdata", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = matrix(c(1, 2, 3, 4), ncol = 2),
    theta = c(0.5, 0.25)
  )

  newdata <- matrix(c(2, 1, 0, 1), ncol = 2)
  eta <- predict(fit, newdata = newdata, type = "linear_predictor")

  expect_equal(eta, as.numeric(newdata %*% c(0.5, 0.25)), tolerance = 1e-12)
})

test_that("predict.hazard errors when theta is missing", {
  fit <- hazard(time = c(1, 2), status = c(1, 0), x = matrix(c(1, 0), ncol = 1))
  expect_error(predict(fit), "No coefficients")
})

test_that("summary.hazard returns model summary metadata", {
  fit <- hazard(
    time = c(1, 2, 3),
    status = c(1, 0, 1),
    x = matrix(c(1, 0, 0, 1, 1, 1), ncol = 2),
    theta = c(0.3, -0.2),
    dist = "weibull"
  )

  s <- summary(fit)

  expect_s3_class(s, "summary.hazard")
  expect_equal(s$n, 3)
  expect_equal(s$p, 2)
  expect_equal(s$dist, "weibull")
  expect_equal(rownames(s$coefficients), c("beta1", "beta2"))
  expect_equal(s$coefficients$estimate, c(0.3, -0.2), tolerance = 1e-12)
})

test_that("summary.hazard includes standard errors when vcov is available", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = matrix(c(1, 0), ncol = 1),
    theta = c(0.4),
    dist = "exponential"
  )
  fit$fit$vcov <- matrix(0.25, nrow = 1, ncol = 1)

  s <- summary(fit)

  expect_true(s$has_vcov)
  expect_equal(s$coefficients$std_error, 0.5, tolerance = 1e-12)
  expect_true(is.finite(s$coefficients$z_stat))
  expect_true(is.finite(s$coefficients$p_value))
})

test_that("optimizer produces valid vcov and SEs for all distributions", {
  skip_if_not_installed("numDeriv")

  set.seed(42)
  n <- 200
  time <- rexp(n, 0.5)
  cens <- runif(n, 0, quantile(time, 0.8))
  status <- as.integer(time <= cens)
  time <- pmin(time, cens)

  # Weibull
  fit_w <- hazard(time = time, status = status,
                  theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
  expect_true(is.matrix(fit_w$fit$vcov))
  expect_true(all(diag(fit_w$fit$vcov) > 0))
  expect_true(all(is.finite(fit_w$fit$se)))
  expect_equal(fit_w$fit$se, sqrt(diag(fit_w$fit$vcov)))

  # Exponential
  fit_e <- hazard(time = time, status = status,
                  theta = c(log(0.3)), dist = "exponential", fit = TRUE)
  expect_true(is.matrix(fit_e$fit$vcov))
  expect_true(all(diag(fit_e$fit$vcov) > 0))
  expect_true(all(is.finite(fit_e$fit$se)))

  # Log-logistic
  fit_ll <- hazard(time = time, status = status,
                   theta = c(0, 0.2), dist = "loglogistic", fit = TRUE)
  expect_true(is.matrix(fit_ll$fit$vcov))
  expect_true(all(diag(fit_ll$fit$vcov) > 0))
  expect_true(all(is.finite(fit_ll$fit$se)))

  # Log-normal
  fit_ln <- hazard(time = time, status = status,
                   theta = c(1, 0), dist = "lognormal", fit = TRUE)
  expect_true(is.matrix(fit_ln$fit$vcov))
  expect_true(all(diag(fit_ln$fit$vcov) > 0))
  expect_true(all(is.finite(fit_ln$fit$se)))
})

test_that("summary shows finite z-stats and p-values from fitted model", {
  skip_if_not_installed("numDeriv")

  set.seed(99)
  n <- 150
  x <- matrix(rnorm(n), ncol = 1)
  time <- rexp(n, exp(0.5 * x))
  status <- rep(1L, n)

  fit <- hazard(time = time, status = status, x = x,
                theta = c(0.3, 1.0, 0), dist = "weibull", fit = TRUE)
  s <- summary(fit)

  expect_true(s$has_vcov)
  expect_true(all(is.finite(s$coefficients$std_error)))
  expect_true(all(is.finite(s$coefficients$z_stat)))
  expect_true(all(is.finite(s$coefficients$p_value)))
  expect_true(all(s$coefficients$p_value >= 0 & s$coefficients$p_value <= 1))
})

test_that("print.summary.hazard prints without error", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = matrix(c(1, 0), ncol = 1),
    theta = c(0.4),
    dist = "exponential"
  )

  expect_output(print(summary(fit)), "hazard model summary")
})

test_that("hazard() accepts formula interface with Surv()", {
  df <- data.frame(
    time = c(1, 2, 3),
    status = c(1, 0, 1),
    x1 = c(0, 1, 0),
    x2 = c(1, 1, 0)
  )

  fit <- hazard(
    Surv(time, status) ~ x1 + x2,
    data = df,
    theta = c(0.3, -0.2),
    dist = "weibull"
  )

  expect_s3_class(fit, "hazard")
  expect_equal(fit$spec$dist, "weibull")
  expect_equal(length(fit$fit$theta), 2)
})

test_that("formula interface extracts predictors correctly", {
  df <- data.frame(
    time = c(1, 2, 3),
    status = c(1, 0, 1),
    x1 = c(0.5, 1.5, 2.5),
    x2 = c(1, 2, 3)
  )

  fit <- hazard(
    Surv(time, status) ~ x1 + x2,
    data = df,
    theta = c(-0.5, 0.3, -0.1),  # log(lambda), beta1, beta2 for exponential
    dist = "exponential"
  )

  pred <- predict(fit, type = "linear_predictor")
  expected_eta <- c(0.5 * 0.3 + 1 * (-0.1), 1.5 * 0.3 + 2 * (-0.1), 2.5 * 0.3 + 3 * (-0.1))
  expect_equal(pred, expected_eta, tolerance = 1e-12)
})

test_that("formula interface requires data when formula is provided", {
  expect_error(
    hazard(Surv(c(1, 2), c(1, 0)) ~ x, data = NULL),
    "'data' is required"
  )
})

test_that("formula interface works without intercept", {
  df <- data.frame(
    time = c(1, 2, 3),
    status = c(1, 0, 1),
    x = c(1, 0, 1)
  )

  fit <- hazard(
    Surv(time, status) ~ x - 1,
    data = df,
    theta = 0.4,
    dist = "exponential"
  )

  expect_equal(ncol(fit$data$x), 1)
  expect_equal(as.numeric(fit$data$x[, 1]), c(1, 0, 1))
})

# test-multiphase-api.R — End-to-end tests for multiphase hazard via hazard()
#
# Tests construction, fitting, predict, summary, and print for dist="multiphase".

# ============================================================================
# Helpers
# ============================================================================

make_multiphase_data <- function(n = 200, seed = 42) {
  set.seed(seed)
  data.frame(
    time   = rexp(n, rate = 0.3) + 0.01,
    status = rbinom(n, 1, prob = 0.7),
    age    = rnorm(n, mean = 60, sd = 10),
    nyha   = sample(1:4, n, replace = TRUE)
  )
}


# ============================================================================
# hazard() construction with phases
# ============================================================================

test_that("hazard() accepts dist='multiphase' with phases", {
  dat <- make_multiphase_data(n = 50)

  fit <- suppressWarnings(hazard(
    survival::Surv(time, status) ~ age,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE
  ))

  expect_s3_class(fit, "hazard")
  expect_equal(fit$spec$dist, "multiphase")
  expect_true(!is.null(fit$spec$phases))
  expect_equal(length(fit$spec$phases), 2)
  expect_equal(names(fit$spec$phases), c("early", "bg"))
})

test_that("hazard() errors when dist='multiphase' without phases", {
  dat <- make_multiphase_data(n = 50)
  expect_error(
    hazard(survival::Surv(time, status) ~ age, data = dat,
           dist = "multiphase", fit = TRUE),
    "phases.*required"
  )
})

test_that("hazard() warns when phases given with non-multiphase dist", {
  dat <- make_multiphase_data(n = 50)
  expect_warning(
    hazard(survival::Surv(time, status) ~ age, data = dat,
           dist = "weibull", theta = c(0.3, 1, 0),
           phases = list(hzr_phase("cdf")),
           fit = FALSE),
    "phases.*ignored"
  )
})


# ============================================================================
# Single-phase (constant) multiphase — should recover exponential-like fit
# ============================================================================

test_that("single constant phase fits and converges", {
  skip_on_cran()

  set.seed(99)
  dat <- data.frame(
    time   = rexp(200, rate = 0.3) + 0.01,
    status = rep(1L, 200)
  )

  fit <- hazard(
    time = dat$time, status = dat$status,
    dist = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE
  )

  expect_true(fit$fit$converged)
  expect_true(is.finite(fit$fit$objective))

  # theta should have bg.log_mu
  expect_true("bg.log_mu" %in% names(fit$fit$theta))
})


# ============================================================================
# Two-phase fit
# ============================================================================

test_that("two-phase (cdf + constant) fits and stores phases", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 150)

  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  expect_s3_class(fit, "hazard")
  expect_true(is.finite(fit$fit$objective))

  # Phases metadata should be stored
  expect_true(!is.null(fit$fit$phases))
  expect_equal(names(fit$fit$phases), c("early", "bg"))
  expect_true(!is.null(fit$fit$covariate_counts))
})


# ============================================================================
# predict.hazard() for multiphase
# ============================================================================

test_that("predict survival and cumhaz for multiphase model", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 100)

  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  # Predict on training data
  surv <- predict(fit, type = "survival")
  cumhaz <- predict(fit, type = "cumulative_hazard")

  expect_true(all(is.finite(surv)))
  expect_true(all(surv >= 0 & surv <= 1))
  expect_true(all(is.finite(cumhaz)))
  expect_true(all(cumhaz >= 0))
  expect_equal(surv, exp(-cumhaz), tolerance = 1e-10)
})

test_that("predict with newdata for multiphase model", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 100)

  fit <- hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  )

  nd <- data.frame(time = c(0.5, 1.0, 2.0, 5.0))
  surv <- predict(fit, newdata = nd, type = "survival")

  expect_equal(length(surv), 4)
  expect_true(all(surv >= 0 & surv <= 1))
  # Survival should be monotone decreasing
  expect_true(all(diff(surv) <= 0))
})

test_that("predict decompose=TRUE returns data frame", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 100)

  fit <- suppressWarnings(hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  ))

  nd <- data.frame(time = c(0.5, 1.0, 2.0))
  decomp <- predict(fit, newdata = nd, type = "cumulative_hazard",
                    decompose = TRUE)

  expect_true(is.data.frame(decomp))
  expect_true("time" %in% names(decomp))
  expect_true("total" %in% names(decomp))
  expect_true("early" %in% names(decomp))
  expect_true("bg" %in% names(decomp))

  # Total should equal sum of components
  expect_equal(decomp$total, decomp$early + decomp$bg, tolerance = 1e-10)
})

test_that("predict linear_predictor errors for multiphase", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 50)
  fit <- suppressWarnings(hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE,
    control = list(n_starts = 1, maxit = 300)
  ))

  expect_error(predict(fit, type = "linear_predictor"), "not supported.*multiphase")
})


# ============================================================================
# print and summary
# ============================================================================

test_that("print.hazard shows phase info for multiphase", {
  dat <- make_multiphase_data(n = 50)
  fit <- suppressWarnings(hazard(
    time = dat$time, status = dat$status,
    dist = "multiphase",
    phases = list(
      early = hzr_phase("cdf"),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 1, maxit = 300)
  ))

  out <- capture.output(print(fit))
  expect_true(any(grepl("multiphase", out)))
  expect_true(any(grepl("phases.*2", out)))
})

test_that("summary.hazard shows per-phase coefficients", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 100)
  fit <- suppressWarnings(hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
      bg    = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  ))

  s <- summary(fit)
  expect_s3_class(s, "summary.hazard")
  expect_equal(s$dist, "multiphase")
  expect_true(!is.null(s$phases))
  expect_true(!is.null(s$coefficients))

  # Should have rows for both phases
  rn <- rownames(s$coefficients)
  expect_true(any(grepl("^early\\.", rn)))
  expect_true(any(grepl("^bg\\.", rn)))

  # Print should work without error
  out <- capture.output(print(s))
  expect_true(any(grepl("Multiphase", out)))
  expect_true(any(grepl("Phase.*early", out)))
  expect_true(any(grepl("Phase.*bg", out)))
})


# ============================================================================
# coef and vcov
# ============================================================================

test_that("coef returns named theta for multiphase", {
  dat <- make_multiphase_data(n = 50)
  fit <- suppressWarnings(hazard(
    time = dat$time, status = dat$status,
    dist = "multiphase",
    phases = list(bg = hzr_phase("constant")),
    fit = TRUE,
    control = list(n_starts = 1, maxit = 300)
  ))

  cf <- coef(fit)
  expect_true(!is.null(cf))
  expect_true(!is.null(names(cf)))
  expect_true("bg.log_mu" %in% names(cf))
})


# ============================================================================
# Three-phase classic pattern
# ============================================================================

test_that("three-phase (cdf + constant + cdf) constructs and fits", {
  skip_on_cran()

  dat <- make_multiphase_data(n = 200)

  fit <- suppressWarnings(hazard(
    survival::Surv(time, status) ~ 1,
    data   = dat,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0),
      constant = hzr_phase("constant"),
      late     = hzr_phase("cdf",      t_half = 5,   nu = 1, m = 0)
    ),
    fit = TRUE,
    control = list(n_starts = 2, maxit = 500)
  ))

  expect_s3_class(fit, "hazard")
  expect_true(is.finite(fit$fit$objective))
  expect_equal(length(fit$spec$phases), 3)

  # Predictions should work
  nd <- data.frame(time = c(0.5, 2, 8))
  surv <- predict(fit, newdata = nd, type = "survival")
  expect_equal(length(surv), 3)
  expect_true(all(surv >= 0 & surv <= 1))
})

test_that("fit object carries rcond and pd diagnostics (weibull)", {
  set.seed(2)
  n <- 250L
  dat <- data.frame(t = rweibull(n, shape = 1.4, scale = 3),
                    d = rep(1L, n))
  fit <- hazard(survival::Surv(t, d) ~ 1, data = dat, dist = "weibull",
                theta = c(mu = 1, nu = 1), fit = TRUE)
  expect_true(is.finite(fit$fit$rcond))
  expect_true(isTRUE(fit$fit$pd))
})

# NOTE: A more comprehensive version of this test (13-param, CoE, scale-invariance)
# lives in test-multiphase-hessian.R (Task 4). This simpler 2-param fit
# remains useful for the Layer-1 diagnostics contract.
test_that("fit object carries rcond and pd diagnostics (multiphase)", {
  skip_on_cran()
  set.seed(101)
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1, data = avc, dist = "multiphase",
    phases = list(early = hzr_phase("cdf", t_half = 0.15, nu = 1.4, m = 1,
                                    fixed = "m"),
                  constant = hzr_phase("constant")),
    fit = TRUE, control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )
  expect_true(is.finite(fit$fit$rcond))
  expect_false(is.null(fit$fit$pd))
})

test_that("summary prints no Hessian note for a clean fit", {
  set.seed(3)
  dat <- data.frame(t = rweibull(250, 1.4, 3), d = rep(1L, 250))
  fit <- hazard(survival::Surv(t, d) ~ 1, data = dat, dist = "weibull",
                fit = TRUE, theta = c(mu = 1, nu = 1))
  out <- capture.output(print(summary(fit)))
  expect_false(any(grepl("ill-conditioned|not positive-definite", out)))
})

test_that("summary prints a note when the fit is flagged ill-conditioned", {
  s <- summary(structure(
    list(spec = list(dist = "weibull", phases = NULL),
         engine = "test",
         fit = list(theta = c(a = 1), vcov = matrix(1, 1, 1),
                    converged = TRUE, objective = -1,
                    rcond = 1e-12, pd = TRUE),
         data = list(time = 1:5, x = NULL),
         call = quote(hazard())),
    class = "hazard"))
  out <- capture.output(print(s))
  expect_true(any(grepl("ill-conditioned", out)))
})

test_that("summary prints a note when the fit is not positive-definite", {
  s <- summary(structure(
    list(spec = list(dist = "weibull", phases = NULL),
         engine = "test",
         fit = list(theta = c(a = 1), vcov = matrix(1, 1, 1),
                    converged = TRUE, objective = -1,
                    rcond = 0.5, pd = FALSE),
         data = list(time = 1:5, x = NULL),
         call = quote(hazard())),
    class = "hazard"))
  out <- capture.output(print(s))
  expect_true(any(grepl("not positive-definite", out)))
  expect_false(any(grepl("ill-conditioned", out)))  # rcond=0.5 is healthy
})

test_that("13-parameter multiphase deciles fit is stable (anchor)", {
  skip_on_cran()
  set.seed(102)
  data(avc, package = "TemporalHazard")
  avc <- na.omit(avc)

  # Continuous covariates in `avc` span wildly different scales (age up to
  # ~286, op_age into the thousands). Unscaled, the multiphase Hessian picks
  # up non-finite entries and collapses to < 12 identifiable directions, so
  # scale them as any analyst fitting this model would. This is what makes
  # the high-dimensional inversion path a *meaningful* (not degenerate) test.
  for (cl in c("age", "opmos", "op_age", "inc_surg")) {
    avc[[cl]] <- as.numeric(scale(avc[[cl]]))
  }

  # A high-dimensional multiphase fit with covariates in both phases,
  # exercising the 12+-parameter inversion path named in DEVELOPMENT-PLAN
  # §7c ("hm.death.AVC.deciles"). Six covariates per phase plus the phase
  # shape parameters drive the free-parameter count past 12.
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1, data = avc, dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf",
                           formula = ~ age + mal + inc_surg + opmos +
                             op_age + status,
                           t_half = 0.15, nu = 1.4, m = 1, fixed = "m"),
      constant = hzr_phase("constant",
                           formula = ~ age + mal + inc_surg + opmos +
                             op_age + status)
    ),
    fit = TRUE, control = list(n_starts = 3, maxit = 800, conserve = TRUE)
  )

  expect_true(fit$fit$converged)

  free <- which(is.finite(diag(fit$fit$vcov)))
  expect_gte(length(free), 12L)               # genuinely high-dimensional
  se <- sqrt(diag(fit$fit$vcov)[free])
  expect_true(all(is.finite(se)))             # no NaN/NA SEs on free params
  expect_true(all(se > 0))                    # positive variances
  expect_true(is.finite(fit$fit$rcond))       # conditioning was measured
})

test_that("summary reports when standard errors are unavailable", {
  # A fitted object whose Hessian could not be inverted: vcov is NA (scalar),
  # so has_vcov is FALSE while converged is TRUE.
  s <- summary(structure(
    list(spec = list(dist = "weibull", phases = NULL),
         engine = "test",
         fit = list(theta = c(a = 1), vcov = NA,
                    converged = TRUE, objective = -1,
                    rcond = NA_real_, pd = NA),
         data = list(time = 1:5, x = NULL),
         call = quote(hazard())),
    class = "hazard"))
  out <- capture.output(print(s))
  expect_true(any(grepl("standard errors unavailable", out)))
})

test_that("clean fit reports no standard-errors-unavailable note", {
  set.seed(7)
  dat <- data.frame(t = rweibull(250, 1.4, 3), d = rep(1L, 250))
  fit <- hazard(survival::Surv(t, d) ~ 1, data = dat, dist = "weibull",
                fit = TRUE, theta = c(mu = 1, nu = 1))
  out <- capture.output(print(summary(fit)))
  expect_false(any(grepl("standard errors unavailable", out)))
})

test_that("hazard() builds a hazard object", {
  x <- matrix(c(1, 0, 0, 1), ncol = 2)
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = x,
    theta = c(0.2, -0.1),
    dist = "weibull",
    maxit = 50
  )

  expect_s3_class(fit, "hazard")
  expect_equal(fit$spec$dist, "weibull")
  expect_equal(fit$legacy_args$maxit, 50)
})

test_that("hazard() validates core dimensions", {
  expect_error(
    hazard(time = c(1, 2), status = 1),
    "same length"
  )

  expect_error(
    hazard(time = c(1, 2), status = c(1, 0), x = matrix(1, nrow = 3, ncol = 1)),
    "rows must match"
  )
})

test_that("predict.hazard returns linear predictor and hazard scale", {
  x <- matrix(c(1, 0, 0, 1), ncol = 2)
  fit <- hazard(time = c(1, 2), status = c(1, 0), x = x, theta = c(0.3, -0.2))

  eta <- predict(fit, type = "linear_predictor")
  hz <- predict(fit, type = "hazard")

  expect_equal(eta, c(0.3, -0.2), tolerance = 1e-12)
  expect_equal(hz, exp(eta), tolerance = 1e-12)
})

test_that("predict.hazard accepts newdata", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = matrix(c(1, 2, 3, 4), ncol = 2),
    theta = c(0.5, 0.25)
  )

  newdata <- matrix(c(2, 1, 0, 1), ncol = 2)
  eta <- predict(fit, newdata = newdata, type = "linear_predictor")

  expect_equal(eta, as.numeric(newdata %*% c(0.5, 0.25)), tolerance = 1e-12)
})

test_that("predict.hazard errors when theta is missing", {
  fit <- hazard(time = c(1, 2), status = c(1, 0), x = matrix(c(1, 0), ncol = 1))
  expect_error(predict(fit), "No coefficients")
})

test_that("summary.hazard returns model summary metadata", {
  fit <- hazard(
    time = c(1, 2, 3),
    status = c(1, 0, 1),
    x = matrix(c(1, 0, 0, 1, 1, 1), ncol = 2),
    theta = c(0.3, -0.2),
    dist = "weibull"
  )

  s <- summary(fit)

  expect_s3_class(s, "summary.hazard")
  expect_equal(s$n, 3)
  expect_equal(s$p, 2)
  expect_equal(s$dist, "weibull")
  expect_equal(rownames(s$coefficients), c("beta1", "beta2"))
  expect_equal(s$coefficients$estimate, c(0.3, -0.2), tolerance = 1e-12)
})

test_that("summary.hazard includes standard errors when vcov is available", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = matrix(c(1, 0), ncol = 1),
    theta = c(0.4),
    dist = "exponential"
  )
  fit$fit$vcov <- matrix(0.25, nrow = 1, ncol = 1)

  s <- summary(fit)

  expect_true(s$has_vcov)
  expect_equal(s$coefficients$std_error, 0.5, tolerance = 1e-12)
  expect_true(is.finite(s$coefficients$z_stat))
  expect_true(is.finite(s$coefficients$p_value))
})

test_that("optimizer produces valid vcov and SEs for all distributions", {
  skip_if_not_installed("numDeriv")

  set.seed(42)
  n <- 200
  time <- rexp(n, 0.5)
  cens <- runif(n, 0, quantile(time, 0.8))
  status <- as.integer(time <= cens)
  time <- pmin(time, cens)

  # Weibull
  fit_w <- hazard(time = time, status = status,
                  theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
  expect_true(is.matrix(fit_w$fit$vcov))
  expect_true(all(diag(fit_w$fit$vcov) > 0))
  expect_true(all(is.finite(fit_w$fit$se)))
  expect_equal(fit_w$fit$se, sqrt(diag(fit_w$fit$vcov)))

  # Exponential
  fit_e <- hazard(time = time, status = status,
                  theta = c(log(0.3)), dist = "exponential", fit = TRUE)
  expect_true(is.matrix(fit_e$fit$vcov))
  expect_true(all(diag(fit_e$fit$vcov) > 0))
  expect_true(all(is.finite(fit_e$fit$se)))

  # Log-logistic
  fit_ll <- hazard(time = time, status = status,
                   theta = c(0, 0.2), dist = "loglogistic", fit = TRUE)
  expect_true(is.matrix(fit_ll$fit$vcov))
  expect_true(all(diag(fit_ll$fit$vcov) > 0))
  expect_true(all(is.finite(fit_ll$fit$se)))

  # Log-normal
  fit_ln <- hazard(time = time, status = status,
                   theta = c(1, 0), dist = "lognormal", fit = TRUE)
  expect_true(is.matrix(fit_ln$fit$vcov))
  expect_true(all(diag(fit_ln$fit$vcov) > 0))
  expect_true(all(is.finite(fit_ln$fit$se)))
})

test_that("vcov.hazard returns a named matrix and preserves NA rows", {
  # A multiphase fit legitimately has NA variance rows: parameters held fixed
  # (e.g. early shapes) and the Conservation-of-Events-conserved phase log_mu
  # carry no Hessian-based variance. The old contract returned a scalar NA for
  # the whole matrix whenever any cell was NA, making vcov() unusable for
  # multiphase models and discarding the finite free-parameter block.
  theta <- c(early.log_mu = -1.5, early.log_t_half = log(0.5),
             early.nu = 1, early.m = 1, early.x = 0.8,
             constant.log_mu = -3.0, constant.x = -0.4)
  V <- matrix(NA_real_, 7, 7)
  free <- c(1L, 5L, 7L)               # early.log_mu, early.x, constant.x
  V[free, free] <- matrix(c(0.0101, -0.0065, 0.0002,
                            -0.0065, 0.0072, -0.0004,
                            0.0002, -0.0004, 0.0012), 3, 3)
  fit <- structure(list(fit = list(theta = theta, vcov = V)), class = "hazard")

  Vout <- vcov(fit)
  expect_true(is.matrix(Vout))                       # not scalar NA
  expect_equal(dim(Vout), c(7L, 7L))
  expect_equal(rownames(Vout), names(theta))         # named for alignment
  expect_equal(colnames(Vout), names(theta))
  expect_true(anyNA(Vout))                           # NA rows preserved
  expect_true(all(is.na(diag(Vout)[c(2, 3, 4, 6)]))) # fixed/conserved -> NA
  expect_true(all(is.finite(diag(Vout)[free])))      # free block intact
})

test_that("vcov.hazard distinguishes a covariate shared across phases by name", {
  theta <- c(early.log_mu = -1.5, early.x = 0.8,
             constant.log_mu = -3.0, constant.x = -0.4)
  V <- diag(c(0.01, 0.02, NA_real_, 0.03))
  fit <- structure(list(fit = list(theta = theta, vcov = V)), class = "hazard")

  Vout <- vcov(fit)
  expect_setequal(rownames(Vout), c("early.log_mu", "early.x",
                                    "constant.log_mu", "constant.x"))
  # The two phase-specific x coefficients must resolve to distinct slots.
  expect_equal(Vout["early.x", "early.x"], 0.02)
  expect_equal(Vout["constant.x", "constant.x"], 0.03)
})

test_that("vcov.hazard returns scalar NA only when the matrix is truly absent", {
  fit_null <- structure(list(fit = list(theta = c(a = 1), vcov = NULL)),
                        class = "hazard")
  expect_true(is.na(vcov(fit_null)) && !is.matrix(vcov(fit_null)))

  fit_scalar <- structure(list(fit = list(theta = c(a = 1), vcov = NA)),
                          class = "hazard")
  expect_true(is.na(vcov(fit_scalar)) && !is.matrix(vcov(fit_scalar)))
})

test_that("vcov.hazard names single-phase (weibull) coefficients", {
  skip_if_not_installed("numDeriv")
  set.seed(7)
  n <- 200
  time <- rexp(n, 0.5)
  status <- rep(1L, n)
  fit <- hazard(time = time, status = status,
                theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
  Vout <- vcov(fit)
  expect_true(is.matrix(Vout))
  expect_false(is.null(rownames(Vout)))
  expect_equal(rownames(Vout), colnames(Vout))
})

test_that("summary shows finite z-stats and p-values from fitted model", {
  skip_if_not_installed("numDeriv")

  set.seed(99)
  n <- 150
  x <- matrix(rnorm(n), ncol = 1)
  time <- rexp(n, exp(0.5 * x))
  status <- rep(1L, n)

  fit <- hazard(time = time, status = status, x = x,
                theta = c(0.3, 1.0, 0), dist = "weibull", fit = TRUE)
  s <- summary(fit)

  expect_true(s$has_vcov)
  expect_true(all(is.finite(s$coefficients$std_error)))
  expect_true(all(is.finite(s$coefficients$z_stat)))
  expect_true(all(is.finite(s$coefficients$p_value)))
  expect_true(all(s$coefficients$p_value >= 0 & s$coefficients$p_value <= 1))
})

test_that("print.summary.hazard prints without error", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = matrix(c(1, 0), ncol = 1),
    theta = c(0.4),
    dist = "exponential"
  )

  expect_output(print(summary(fit)), "hazard model summary")
})

test_that("hazard() accepts formula interface with Surv()", {
  df <- data.frame(
    time = c(1, 2, 3),
    status = c(1, 0, 1),
    x1 = c(0, 1, 0),
    x2 = c(1, 1, 0)
  )

  fit <- hazard(
    Surv(time, status) ~ x1 + x2,
    data = df,
    theta = c(0.3, -0.2),
    dist = "weibull"
  )

  expect_s3_class(fit, "hazard")
  expect_equal(fit$spec$dist, "weibull")
  expect_equal(length(fit$fit$theta), 2)
})

test_that("formula interface extracts predictors correctly", {
  df <- data.frame(
    time = c(1, 2, 3),
    status = c(1, 0, 1),
    x1 = c(0.5, 1.5, 2.5),
    x2 = c(1, 2, 3)
  )

  fit <- hazard(
    Surv(time, status) ~ x1 + x2,
    data = df,
    theta = c(-0.5, 0.3, -0.1),  # log(lambda), beta1, beta2 for exponential
    dist = "exponential"
  )

  pred <- predict(fit, type = "linear_predictor")
  expected_eta <- c(0.5 * 0.3 + 1 * (-0.1), 1.5 * 0.3 + 2 * (-0.1), 2.5 * 0.3 + 3 * (-0.1))
  expect_equal(pred, expected_eta, tolerance = 1e-12)
})

test_that("formula interface requires data when formula is provided", {
  expect_error(
    hazard(Surv(c(1, 2), c(1, 0)) ~ x, data = NULL),
    "'data' is required"
  )
})

test_that("formula interface works without intercept", {
  df <- data.frame(
    time = c(1, 2, 3),
    status = c(1, 0, 1),
    x = c(1, 0, 1)
  )

  fit <- hazard(
    Surv(time, status) ~ x - 1,
    data = df,
    theta = 0.4,
    dist = "exponential"
  )

  expect_equal(ncol(fit$data$x), 1)
  expect_equal(as.numeric(fit$data$x[, 1]), c(1, 0, 1))
})

test_that("predict() supports survival type", {
  # Create a simple fitted model
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(mu = 0.5, nu = 1.2),
    dist = "weibull",
    fit = FALSE
  )

  # Predict survival at original times
  surv <- predict(fit, type = "survival")

  expect_true(is.numeric(surv))
  expect_equal(length(surv), length(time))
  expect_true(all(surv > 0 & surv <= 1))  # Survival probability in (0, 1]
})

test_that("predict() supports cumulative_hazard type", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(mu = 0.5, nu = 1.2),
    dist = "weibull",
    fit = FALSE
  )

  # Predict cumulative hazard at original times
  cumhaz <- predict(fit, type = "cumulative_hazard")

  expect_true(is.numeric(cumhaz))
  expect_equal(length(cumhaz), length(time))
  expect_true(all(cumhaz > 0))  # Cumulative hazard positive
})

test_that("survival and cumulative hazard are related correctly", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(mu = 0.5, nu = 1.2),
    dist = "weibull",
    fit = FALSE
  )

  surv <- predict(fit, type = "survival")
  cumhaz <- predict(fit, type = "cumulative_hazard")

  # S(t) = exp(-H(t))
  # So cumhaz = -log(surv)
  expected_cumhaz <- -log(surv)

  expect_equal(cumhaz, expected_cumhaz, tolerance = 1e-12)
})

test_that("predict() works with covariates for survival", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)
  x <- matrix(c(0.5, 1.2, -0.3, 0.8, 1.5), ncol = 1)

  fit <- hazard(
    time = time,
    status = status,
    x = x,
    theta = c(mu = 0.5, nu = 1.2, beta = 0.3),
    dist = "weibull",
    fit = FALSE
  )

  # Predict survival at original times and data
  newdata <- data.frame(time = time, X1 = x[, 1])
  surv <- predict(fit, newdata = newdata, type = "survival")

  expect_true(is.numeric(surv))
  expect_equal(length(surv), length(time))
  expect_true(all(surv > 0 & surv <= 1))
})

test_that("predict() handles newdata time column correctly", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(mu = 0.5, nu = 1.2),
    dist = "weibull",
    fit = FALSE
  )

  # Predict at different times
  new_time <- c(0.5, 1.0, 1.5, 2.0, 2.5)
  newdata <- data.frame(time = new_time)

  surv_new <- predict(fit, newdata = newdata, type = "survival")

  expect_equal(length(surv_new), length(new_time))
  expect_true(all(surv_new > 0 & surv_new <= 1))
})

test_that("predict() validates time is positive", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(mu = 0.5, nu = 1.2),
    dist = "weibull",
    fit = FALSE
  )

  # Negative time should error
  newdata <- data.frame(time = c(1, -1, 2))
  expect_error(predict(fit, newdata = newdata, type = "survival"))
})

test_that("predict() errors if survival requested on non-Weibull/exponential", {
  time <- c(1, 2, 3)
  status <- c(1, 0, 1)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(0.5, 0.3),
    dist = "gompertz",  # Not yet supported
    fit = FALSE
  )

  expect_error(predict(fit, type = "survival"), "only supported for")
})

test_that("predict() linear_predictor and hazard still work unchanged", {
  time <- c(1, 2, 3, 4, 5)
  status <- c(1, 0, 1, 1, 0)
  x <- matrix(c(0.5, 1.2, -0.3, 0.8, 1.5), ncol = 1)

  # theta must have: shape params (2 for Weibull) + covariate coefs (1 for 1 covariate)
  fit <- hazard(
    time = time,
    status = status,
    x = x,
    theta = c(0.5, 0.3, 0.1),  # mu, nu, beta
    dist = "weibull",
    fit = FALSE
  )

  newdata <- data.frame(X1 = c(0.2, 0.5, 0.8))

  eta <- predict(fit, newdata = newdata, type = "linear_predictor")
  hz <- predict(fit, newdata = newdata, type = "hazard")

  # Check relationship: hz = exp(eta)
  expect_equal(hz, exp(eta), tolerance = 1e-12)
})

test_that("predict() works on fitted model with survival type", {
  time <- c(1, 2, 3, 4, 5, 6)
  status <- c(1, 0, 1, 1, 0, 1)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(mu = 0.3, nu = 1.0),
    dist = "weibull",
    fit = TRUE,
    control = list(maxit = 100)
  )

  # Check if fit object has theta (may fail to converge)
  if (is.null(fit$fit$theta)) {
    skip("Model did not converge")
  }

  # Predict survival from fitted model
  surv <- predict(fit, type = "survival")

  expect_true(is.numeric(surv))
  expect_equal(length(surv), length(time))
  expect_true(all(surv > 0 & surv <= 1))
})

test_that("monotonicity: survival decreases with time", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = NULL,
    theta = c(mu = 0.5, nu = 1.5),
    dist = "weibull",
    fit = FALSE
  )

  # Predict survival across increasing times
  new_times <- seq(0.1, 5, by = 0.5)
  newdata <- data.frame(time = new_times)
  surv <- predict(fit, newdata = newdata, type = "survival")

  # Survival should be monotonically decreasing
  expect_true(all(diff(surv) < 0))
})

test_that("monotonicity: cumulative hazard increases with time", {
  fit <- hazard(
    time = c(1, 2),
    status = c(1, 0),
    x = NULL,
    theta = c(mu = 0.5, nu = 1.5),
    dist = "weibull",
    fit = FALSE
  )

  # Predict cumulative hazard across increasing times
  new_times <- seq(0.1, 5, by = 0.5)
  newdata <- data.frame(time = new_times)
  cumhaz <- predict(fit, newdata = newdata, type = "cumulative_hazard")

  # Cumulative hazard should be monotonically increasing
  expect_true(all(diff(cumhaz) > 0))
})

test_that("covariate effects on survival are correct", {
  time <- c(1, 2, 3)
  status <- c(1, 0, 1)

  fit <- hazard(
    time = time,
    status = status,
    x = NULL,
    theta = c(mu = 0.5, nu = 1.2, beta = 0.5),  # 3 parameters
    dist = "weibull",
    fit = FALSE
  )

  # Compare survival with and without covariate effect
  t_eval <- 2.0

  # No covariate effect
  newdata_no_x <- data.frame(time = t_eval)
  # surv without covariate = exp(-(0.5 * 2)^1.2)

  # With covariate effect (x=1): exp(eta) where eta = 0.5*1 = 0.5
  newdata_x1 <- data.frame(time = t_eval, X1 = 1)
  surv_x1 <- predict(fit, newdata = newdata_x1, type = "survival")

  # Higher covariate effect should reduce survival (positive beta increases hazard)
  # So with positive covariate, survival should be lower
  expect_true(is.numeric(surv_x1))
  expect_true(surv_x1 > 0 && surv_x1 < 1)
})

#' @importFrom stats optim
#' @keywords internal
NULL

# optimizer.R -- Generic parametric hazard optimizer
#
# Consolidates the common optimization pattern shared across all distributions.
# Each distribution provides its own log-likelihood and gradient functions;
# this module handles the objective wrapping, optim() call, and post-fit
# Hessian/vcov computation.

#' Generic optimizer for parametric hazard likelihoods
#'
#' Maximises a log-likelihood by minimising its negation via \code{stats::optim}.
#' Handles constrained (L-BFGS-B) and unconstrained (BFGS) optimization,
#' post-fit Hessian computation, and vcov extraction.
#'
#' @param logl_fn Function(theta, time, status, time_lower, time_upper, x, ...)
#'   returning the scalar log-likelihood.  Must accept \code{return_gradient}
#'   argument (not used here directly, but the function signature must allow it).
#' @param gradient_fn Function(theta, time, status, time_lower, time_upper, x, ...)
#'   returning the score vector (gradient of the log-likelihood, NOT negated).
#' @param time Numeric vector of follow-up times.
#' @param status Numeric vector of event/censoring indicators.
#' @param time_lower Optional lower bounds for interval censoring.
#' @param time_upper Optional upper bounds for left/interval censoring.
#' @param x Optional design matrix.
#' @param theta_start Starting parameter vector.
#' @param control List of control options (maxit, reltol, abstol).
#' @param use_bounds Logical; if TRUE, use L-BFGS-B with lower bounds on
#'   constrained parameters.  Used for Weibull where mu/nu are on the natural
#'   scale.
#' @param lower_bounds Numeric vector of lower bounds for L-BFGS-B, the same
#'   length as \code{theta_start}.  Only used when \code{use_bounds = TRUE}.
#'   Defaults to \code{rep(1e-6, length(theta_start))} when NULL, but callers
#'   should supply distribution-specific bounds (e.g. \code{c(1e-6, 1e-6,
#'   rep(-Inf, p_cov))} for Weibull so that covariate betas are unconstrained).
#' @param hessian_fn Optional function(theta) returning the Hessian of the
#'   negative log-likelihood at \code{theta} (same scale as
#'   \code{numDeriv::hessian(objective)}).  When it returns a conformant square
#'   matrix (dimension equal to the number of parameters), that matrix is used
#'   for standard errors.  The numerical Hessian is used instead when
#'   \code{hessian_fn} is \code{NULL} (the default), when the function returns
#'   \code{NULL} (e.g. a censoring branch it does not cover analytically), or
#'   when it errors.  A non-NULL, non-conformant return raises a warning and
#'   also falls back to the numerical Hessian.
#'
#' @return List with par, value (log-likelihood), convergence, counts, message,
#'   hessian, vcov.
#' @noRd
.hzr_optim_generic <- function(
    logl_fn,
    gradient_fn,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    theta_start,
    weights = NULL,
    control = list(),
    use_bounds = FALSE,
    lower_bounds = NULL,
    hessian_fn = NULL) {

  control <- utils::modifyList(
    list(maxit = 1000, reltol = 1e-5, abstol = 1e-6),
    control
  )

  # Resolve lower bounds: default to 1e-6 on all params only when the caller
  # has not supplied explicit bounds.  Callers like .hzr_optim_weibull pass
  # c(1e-6, 1e-6, -Inf, ...) so that shape params are constrained but betas
  # are free.
  if (use_bounds && is.null(lower_bounds)) {
    lower_bounds <- rep(1e-6, length(theta_start))
  }

  # Negative log-likelihood for minimization
  objective <- function(theta) {
    if (!all(is.finite(theta))) return(1e10)
    # Only penalise infeasible shape params (those with finite lower bounds > -Inf).
    if (use_bounds && any(theta[is.finite(lower_bounds)] <= lower_bounds[is.finite(lower_bounds)])) return(1e10)

    ll <- -logl_fn(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights, return_gradient = FALSE
    )
    if (!is.finite(ll)) return(1e10)
    ll
  }

  # Negative score vector for minimization
  gradient <- function(theta) {
    if (!all(is.finite(theta))) return(rep(0, length(theta)))
    if (use_bounds && any(
      theta[is.finite(lower_bounds)] <= lower_bounds[is.finite(lower_bounds)]
    )) {
      return(rep(0, length(theta)))
    }

    grad <- tryCatch(
      gradient_fn(
        theta = theta, time = time, status = status,
        time_lower = time_lower, time_upper = time_upper,
        x = x, weights = weights
      ),
      error = function(e) rep(0, length(theta))
    )

    grad[!is.finite(grad)] <- 0
    -grad
  }

  # Dispatch to constrained or unconstrained optimizer
  if (use_bounds) {
    result <- optim(
      par = theta_start, fn = objective, gr = gradient,
      method = "L-BFGS-B",
      lower = lower_bounds,
      upper = rep(Inf, length(theta_start)),
      control = list(
        maxit = control$maxit,
        factr = 1 / control$reltol,
        pgtol = control$abstol
      ),
      hessian = FALSE
    )
  } else {
    result <- optim(
      par = theta_start, fn = objective, gr = gradient,
      method = "BFGS",
      control = list(maxit = control$maxit, reltol = control$reltol),
      hessian = FALSE
    )
  }

  # Post-fit Hessian for standard errors.  Prefer the caller's analytic Hessian
  # (on the objective / negative-log-likelihood scale) when supplied; otherwise,
  # or when it declines by returning NULL, fall back to a numerical Hessian.
  hess_result <- NULL
  if (!is.null(hessian_fn)) {
    hess_result <- tryCatch(hessian_fn(result$par), error = function(e) NULL)
    # A non-NULL hook result must be a square matrix matching the parameter
    # dimension; a misbehaving hook should not silently produce a wrong vcov.
    p_dim <- length(result$par)
    if (!is.null(hess_result) &&
        !(is.matrix(hess_result) && all(dim(hess_result) == p_dim))) {
      warning("hessian_fn returned a non-conformant result; using numerical Hessian")
      hess_result <- NULL
    }
  }
  if (is.null(hess_result)) {
    hess_result <- tryCatch(
      {
        if (requireNamespace("numDeriv", quietly = TRUE)) {
          numDeriv::hessian(objective, result$par)
        } else {
          NULL
        }
      },
      error = function(e) NULL
    )
  }

  # Hardened inversion + conditioning diagnostics (Layer 1).
  inv <- if (is.matrix(hess_result)) {
    .hzr_safe_solve(hess_result)
  } else {
    list(vcov = NA, rcond = NA_real_, pd = NA)
  }

  list(
    par = result$par,
    value = -result$value,
    convergence = result$convergence,
    counts = result$counts,
    message = result$message,
    hessian = hess_result,
    vcov = inv$vcov,
    rcond = inv$rcond,
    pd = inv$pd
  )
}

# stepwise-refit.R -- Scope-mutating refit wrapper for stepwise selection.
#
# Step 8.4 of STEPWISE-DESIGN.md (shared with the forward-step driver).
# Given a current fit plus a single scope mutation (add/drop a variable,
# optionally scoped to a phase), rebuild the hazard() call with the
# updated formula / phases list and return the refitted object.
#
# Design note on warm-start
# -------------------------
# Single-distribution fits: hazard() requires an explicit `theta`
# starting vector (the `fit && !is.null(theta)` guard in hazard_api.R).
# We warm-start by re-using the current theta, appending a zero for the
# newly-added beta or dropping the row for the removed one. This lands
# the optimizer near the MLE and typically converges in a handful of
# BFGS iterations.
#
# Multiphase fits: theta = NULL lets hazard() reassemble starting values
# from the phase specs. Warm-starting the full multiphase vector is
# intricate (per-phase layout with optional fixed shapes) and deferred
# to a future optimisation; the Conservation-of-Events adjustment and
# multi-start loop make re-initialisation cheap enough for v1.
#
# Scope for v1: **main effects only**. A term like a multi-level
# factor or a spline that expands to several coefficients would break
# the one-zero-per-add / one-index-per-drop theta layout. The upstream
# guard in .hzr_candidate_coef_name() errors on such terms before the
# refit fires, so callers via hzr_stepwise() never reach this function
# with an expanding term. Anyone calling .hzr_refit_with_scope()
# directly should pre-expand factors and splines into explicit main
# effects.

#' Refit a hazard model with a single scope mutation applied
#'
#' @param current A fitted `hazard` object that was built via the
#'   `formula` / `data` interface.
#' @param action Either `"add"` or `"drop"`.
#' @param var Character scalar naming the variable to add or drop.
#' @param phase For multiphase models, character scalar naming the
#'   phase whose scope changes. Ignored (must be NULL) otherwise.
#' @param data Data frame the original fit was built on. Passed to
#'   `hazard()` for the refit.
#' @param ... Additional named args forwarded to `hazard()` (for
#'   example `control = ...`). `time_windows` and `weights` are pulled
#'   from `current$data` automatically; passing them via `...` will
#'   override.
#'
#' @return A new fitted `hazard` object with `$converged` possibly
#'   FALSE if the refit failed to converge.
#'
#' @keywords internal
#' @noRd
.hzr_refit_with_scope <- function(current, action = c("add", "drop"),
                                   var, phase = NULL, data, ...) {
  action <- match.arg(action)
  if (!inherits(current, "hazard")) {
    stop("`current` must be a fitted `hazard` object.", call. = FALSE)
  }
  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }
  if (!is.character(var) || length(var) != 1L || !nzchar(var)) {
    stop("`var` must be a non-empty character scalar.", call. = FALSE)
  }

  dist <- current$spec$dist
  user_args <- list(...)

  # Reuse the original fit's weights / time_windows unless user overrides
  default_weights <- current$data$weights
  default_windows <- current$spec$time_windows

  weights <- if ("weights" %in% names(user_args)) {
    user_args$weights
  } else {
    default_weights
  }
  time_windows <- if ("time_windows" %in% names(user_args)) {
    user_args$time_windows
  } else {
    default_windows
  }

  extra_args <- user_args[!names(user_args) %in%
                            c("weights", "time_windows")]

  # Recover the original formula.  match.call() stores the formula arg
  # as an unevaluated `call`; coerce to a real formula.
  raw_formula <- current$call$formula
  if (is.null(raw_formula)) {
    stop("`current` was not built via the formula interface; refit ",
         "requires a formula-based base fit.", call. = FALSE)
  }
  current_formula <- if (inherits(raw_formula, "formula")) {
    raw_formula
  } else {
    stats::as.formula(deparse(raw_formula))
  }

  if (dist == "multiphase") {
    if (is.null(phase)) {
      stop("`phase` is required when `current` is a multiphase model.",
           call. = FALSE)
    }
    if (!phase %in% names(current$spec$phases)) {
      stop("Unknown phase: ", sQuote(phase), ".  Available: ",
           paste(sQuote(names(current$spec$phases)), collapse = ", "),
           call. = FALSE)
    }

    new_phases <- current$spec$phases
    new_phases[[phase]] <- .hzr_phase_update_formula(
      new_phases[[phase]], action = action, var = var
    )

    do.call(hazard, c(
      list(
        formula      = current_formula,
        data         = data,
        dist         = "multiphase",
        phases       = new_phases,
        weights      = weights,
        time_windows = time_windows,
        fit          = TRUE
      ),
      extra_args
    ))
  } else {
    # Single-distribution path: mutate the global formula, warm-start
    # theta by inserting / dropping the relevant beta slot.
    new_formula <- .hzr_formula_update(current_formula, action, var)

    n_shape <- .hzr_shape_parameter_count(dist, control = current$spec$control)
    theta_old <- current$fit$theta
    if (is.null(theta_old)) {
      stop("`current` has no fitted theta; refit requires a fitted model.",
           call. = FALSE)
    }

    current_vars <- .hzr_scope_current_vars(current)
    if (action == "add") {
      # Warm-start with an extra zero for the new beta (appended last,
      # matching formula-term ordering).
      if (var %in% current_vars) {
        # Already present; just rebuild theta as-is
        theta_start <- theta_old
      } else {
        theta_start <- c(theta_old, 0)
      }
    } else {
      if (!var %in% current_vars) {
        theta_start <- theta_old
      } else {
        # Drop position: position in beta slot = match index within
        # current_vars, shifted by n_shape.
        drop_idx <- match(var, current_vars) + n_shape
        theta_start <- theta_old[-drop_idx]
      }
    }

    do.call(hazard, c(
      list(
        formula      = new_formula,
        data         = data,
        dist         = dist,
        theta        = theta_start,
        weights      = weights,
        time_windows = time_windows,
        fit          = TRUE
      ),
      extra_args
    ))
  }
}

# decomposition.R -- Generalized temporal decomposition
#
# PURPOSE
# -------
# The unified parametric family decompos(t; t_half, nu, m) generates all
# temporal phase shapes used in multiphase hazard models.  It produces three
# quantities: G(t) (CDF), g(t) (density -- "early" pattern), and h(t) =
# g(t)/(1-G(t)) (hazard -- "late" pattern).
#
# Originally introduced by Blackstone, Naftel, and Turner (1986, JASA 81:615)
# and extended to longitudinal mixed-effects settings by Rajeswaran et al.
# (2018, Stat Methods Med Res 27:126).  Ported here from the mixhazard
# package with enhanced numerical stability and integrated phase-level
# helpers for the additive cumulative hazard model:
#
#   H(t | x) = sum_j  mu_j(x) * Phi_j(t; t_half_j, nu_j, m_j)
#
# PARAMETER MAPPING FROM SAS/C HAZARD
# ------------------------------------
# SAS Early (G1): DELTA, RHO/THALF, NU, M  -->  t_half, nu, m
# SAS Late  (G3): TAU, GAMMA, ALPHA, ETA   -->  t_half, nu, m
# SAS Const (G2): (none)                   -->  type = "constant"
#
# The 4-parameter C/SAS parameterizations collapse onto this 3-parameter
# family.  The C DELTA parameter controlled a time transformation
# B(t) = (exp(delta*t) - 1)/delta that is absorbed by the shape.

# ============================================================================
# hzr_decompos -- core engine
# ============================================================================

#' Generalized temporal decomposition
#'
#' Computes the cumulative distribution \eqn{G(t)}, density \eqn{g(t)}, and
#' hazard \eqn{h(t) = g(t)/(1 - G(t))} for the parametric family defined by
#' half-life, time exponent, and shape.  This single function generates all
#' temporal phase shapes used in multiphase hazard models.
#'
#' @section Parameter mapping from SAS/C HAZARD:
#'
#' The original C code used separate parameterizations for early (DELTA,
#' RHO/THALF, NU, M) and late (TAU, GAMMA, ALPHA, ETA) phases.  Both
#' collapse onto the three parameters here.  See
#' [hzr_argument_mapping()] for the full translation table.
#'
#' @section Valid parameter combinations:
#'
#' Six cases are defined by the signs of `nu` and `m`:
#'
#' \tabular{lll}{
#'   **Case** \tab **Sign** \tab **Behavior** \cr
#'   1     \tab m > 0, nu > 0 \tab Standard sigmoidal \cr
#'   1L    \tab m = 0, nu > 0 \tab Exponential-like (Weibull CDF) \cr
#'   2     \tab m < 0, nu > 0 \tab Heavy-tailed \cr
#'   2L    \tab m < 0, nu = 0 \tab Exponential decay \cr
#'   3     \tab m > 0, nu < 0 \tab Bounded cumulative \cr
#'   3L    \tab m = 0, nu < 0 \tab Bounded exponential \cr
#' }
#'
#' The combination m < 0 **and** nu < 0 is undefined and raises an error.
#'
#' @param time Numeric vector of times (must be > 0).
#' @param t_half Half-life: time at which \eqn{G(t_{1/2}) = 0.5}.
#'   Must be > 0.
#' @param nu Time exponent controlling rate dynamics.
#'   SAS early: `NU`.  SAS late: relates to `GAMMA`/`ETA`.
#' @param m Shape exponent controlling the distributional form.
#'   SAS early: `M`.  SAS late: relates to `GAMMA`/`ALPHA`.
#'
#' @return A named list with three numeric vectors, each the same length
#'   as `time`:
#' \describe{
#'   \item{G}{Cumulative distribution \eqn{G(t) \in [0, 1]}.}
#'   \item{g}{Density \eqn{g(t) = dG/dt \ge 0}.  The "early" phase
#'     temporal pattern.}
#'   \item{h}{Hazard \eqn{h(t) = g(t)/(1 - G(t)) \ge 0}.  The "late"
#'     phase temporal pattern.}
#' }
#'
#' @references
#' Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
#' hazard into phases, each incorporating a separate stream of concomitant
#' information. *J Am Stat Assoc.* 1986;81(395):615--624.
#' \doi{10.1080/01621459.1986.10478314}
#'
#' Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
#' Probability of atrial fibrillation after ablation: Using a parametric
#' nonlinear temporal decomposition mixed effects model. *Stat Methods Med Res.*
#' 2018;27(1):126--141. \doi{10.1177/0962280215623583}
#'
#' @examples
#' t_grid <- seq(0.1, 10, by = 0.1)
#'
#' # Case 1: standard sigmoidal (m > 0, nu > 0)
#' d1 <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 1)
#' plot(t_grid, d1$G, type = "l", main = "CDF (m=1, nu=2)")
#'
#' # Case 1L: Weibull-like (m = 0, nu > 0)
#' d1L <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 0)
#'
#' # Case 2: heavy-tailed (m < 0, nu > 0)
#' d2 <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = -1)
#'
#' @seealso [hzr_phase_cumhaz()] for the phase-level cumulative hazard
#'   contribution, [hzr_argument_mapping()] for SAS/C parameter mapping,
#'   [hzr_phase()] for specifying phases in [hazard()] models.
#'
#' \code{vignette("mf-mathematical-foundations")} for the full derivation.
#'
#' @export
hzr_decompos <- function(time, t_half, nu, m) {

  # --- Input validation ------------------------------------------------------
  stopifnot(
    "time must be numeric"   = is.numeric(time),
    "t_half must be a positive scalar" =
      is.numeric(t_half) && length(t_half) == 1L && t_half > 0,
    "nu must be a numeric scalar" =
      is.numeric(nu) && length(nu) == 1L && is.finite(nu),
    "m must be a numeric scalar" =
      is.numeric(m) && length(m) == 1L && is.finite(m)
  )

  if (m < 0 && nu < 0) {
    stop("Decomposition undefined when both m < 0 and nu < 0 (m = ",
         m, ", nu = ", nu, ").", call. = FALSE)
  }

  # Clamp time away from zero to avoid division-by-zero / 0^negative

  time <- pmax(time, .Machine$double.xmin)

  # Pre-compute recurring exponent terms
  if (m != 0) mm1 <- -(1 / m) - 1
  if (nu != 0) num1 <- -(1 / nu) - 1

  # --- Case dispatch ---------------------------------------------------------
  if (m > 0 && nu > 0) {
    # Case 1: standard sigmoidal
    rho   <- nu * t_half * (((2^m - 1) / m)^nu)
    bt    <- nu * time / rho
    btnu  <- 1 + m * bt^(-1 / nu)
    G     <- btnu^(-1 / m)
    g     <- (btnu^mm1) * (bt^num1) / rho

  } else if (m == 0 && nu > 0) {
    # Case 1L: Weibull-like (m -> 0 limit)
    rho   <- nu * t_half * (log(2)^nu)
    bt    <- nu * time / rho
    btnu  <- bt^(-1 / nu)
    G     <- exp(-btnu)
    g     <- G * (bt^num1) / rho

  } else if (m < 0 && nu > 0) {
    # Case 2: heavy-tailed
    rho   <- nu * t_half / ((1 - 2^m)^(-nu) - 1)
    bt    <- 1 + nu * time / rho
    btnu  <- 1 - bt^(-1 / nu)
    G     <- btnu^(-1 / m)
    g     <- -(btnu^mm1) * (bt^num1) / (m * rho)

  } else if (m < 0 && nu == 0) {
    # Case 2L: exponential decay (nu -> 0 limit)
    rho   <- -t_half / log(1 - 2^m)
    bt    <- exp(-time / rho)
    btm   <- 1 - bt
    G     <- btm^(-1 / m)
    g     <- -(btm^mm1) * bt / (m * rho)

  } else if (m > 0 && nu < 0) {
    # Case 3: bounded cumulative
    rho   <- -nu * t_half * ((2^m - 1)^nu)
    bt    <- -nu * time / rho
    btnu  <- 1 + m * bt^(-1 / nu)
    G     <- 1 - btnu^(-1 / m)
    g     <- (btnu^mm1) * (bt^num1) / rho

  } else if (m == 0 && nu < 0) {
    # Case 3L: bounded exponential (m -> 0 limit)
    rho   <- -nu * t_half * (log(2)^nu)
    bt    <- -nu * time / rho
    btnu  <- bt^(-1 / nu)
    G     <- 1 - exp(-btnu)
    g     <- exp(-btnu) * (bt^num1) / rho
  }

  # --- Hazard from density and CDF ------------------------------------------
  # h(t) = g(t) / (1 - G(t)), with guard against G(t) = 1
  one_minus_G <- pmax(1 - G, .Machine$double.xmin)
  h <- g / one_minus_G

  list(G = G, g = g, h = h)
}


# ============================================================================
# hzr_decompos_g3 -- Late-phase (G3) engine
# ============================================================================

#' Late-phase (G3) temporal decomposition
#'
#' Computes the cumulative intensity \eqn{G_3(t)} and its derivative
#' \eqn{g_3(t) = dG_3/dt} for the late-phase parametric family used in the
#' original Blackstone C/SAS HAZARD code.  Unlike [hzr_decompos()] (which
#' computes the early-phase G1 -- a bounded CDF), this function can produce
#' **unbounded** values, making it suitable for modelling increasing late risk.
#'
#' @section Mathematical form:
#'
#' When \eqn{\alpha > 0}:
#' \deqn{G_3(t) = \bigl(\bigl((t/\tau)^\gamma + 1\bigr)^{1/\alpha}
#'                       - 1\bigr)^\eta}
#'
#' When \eqn{\alpha = 0} (limiting exponential case):
#' \deqn{G_3(t) = \bigl(\exp\bigl((t/\tau)^\gamma\bigr) - 1\bigr)^\eta}
#'
#' @section Parameter mapping from SAS/C HAZARD:
#'
#' | SAS name | R argument | Role |
#' |----------|-----------|------|
#' | TAU      | `tau`     | Scale (time at which \eqn{(t/\tau) = 1}) |
#' | GAMMA    | `gamma`   | Power exponent on \eqn{t/\tau} |
#' | ALPHA    | `alpha`   | Shape (0 = exponential limiting case) |
#' | ETA      | `eta`     | Outer power exponent |
#'
#' @param time Numeric vector of times (must be > 0).
#' @param tau Positive scalar scale parameter.
#' @param gamma Positive scalar time exponent.
#' @param alpha Non-negative scalar shape parameter (0 selects limiting case).
#' @param eta Positive scalar outer exponent.
#'
#' @return A named list with two numeric vectors, each the same length
#'   as `time`:
#' \describe{
#'   \item{G3}{Cumulative intensity \eqn{G_3(t) \ge 0} (may exceed 1).}
#'   \item{g3}{Derivative \eqn{g_3(t) = dG_3/dt \ge 0}.}
#' }
#'
#' @references
#' Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
#' hazard into phases, each incorporating a separate stream of concomitant
#' information. *J Am Stat Assoc.* 1986;81(395):615--624.
#' \doi{10.1080/01621459.1986.10478314}
#'
#' @examples
#' t_grid <- seq(0.1, 10, by = 0.1)
#'
#' # Weibull-like: alpha = 1 gives G3(t) = (t/tau)^(gamma*eta)
#' d <- hzr_decompos_g3(t_grid, tau = 1, gamma = 3, alpha = 1, eta = 1)
#' plot(t_grid, d$G3, type = "l", main = "G3: power law (gamma=3)")
#'
#' # General case with alpha > 0
#' d2 <- hzr_decompos_g3(t_grid, tau = 2, gamma = 2, alpha = 0.5, eta = 1)
#'
#' @seealso [hzr_decompos()] for the early-phase (G1) decomposition,
#'   [hzr_phase_cumhaz()] for phase-level cumulative hazard helpers.
#'
#' @export
hzr_decompos_g3 <- function(time, tau, gamma, alpha, eta) {

  # --- Input validation ------------------------------------------------------
  # Soft checks: return Inf/0 for infeasible params (optimizer needs graceful

  # failure) rather than hard stops.
  if (!is.numeric(time)) stop("time must be numeric")
  if (!is.numeric(tau) || length(tau) != 1L) stop("tau must be a scalar")
  if (!is.numeric(gamma) || length(gamma) != 1L) stop("gamma must be a scalar")
  if (!is.numeric(alpha) || length(alpha) != 1L) stop("alpha must be a scalar")

  if (!is.numeric(eta) || length(eta) != 1L) stop("eta must be a scalar")

  # Return Inf/NaN for infeasible parameters (optimizer will see -Inf logl)
  if (tau <= 0 || gamma <= 0 || eta <= 0 || alpha < 0 || !is.finite(alpha)) {
    return(list(G3 = rep(Inf, length(time)), g3 = rep(NaN, length(time))))
  }

  # Clamp time away from zero
  time <- pmax(time, .Machine$double.xmin)

  # --- Common terms ----------------------------------------------------------
  # Work on log scale for numerical stability, mirroring the C implementation
  ln_t_tau  <- log(time / tau)                    # ln(T/tau)
  ln_t_tau_g <- gamma * ln_t_tau                   # gamma * ln(T/tau)

  # --- Case dispatch: alpha > 0 vs alpha = 0 --------------------------------
  if (alpha > 0) {
    # G3(t) = (((t/tau)^gamma + 1)^(1/alpha) - 1)^eta
    #
    # On log scale:
    #   tGamma = ln((t/tau)^gamma + 1) = ln(exp(ln_t_tau_g) + 1) = log1p(exp(ln_t_tau_g))
    #   tEta   = ln(exp(tGamma/alpha) - 1) = ln(expm1(tGamma/alpha))
    #   lnG3   = eta * tEta

    tGamma <- .log1pexp(ln_t_tau_g)   # ln((t/tau)^gamma + 1)
    inner  <- tGamma / alpha           # ln((t/tau)^gamma + 1) / alpha
    tEta   <- .log_expm1(inner)        # ln(exp(inner) - 1)
    lnG3   <- eta * tEta

    G3 <- exp(lnG3)

    # g3(t) = dG3/dt
    # lnScale = ln(gamma) + ln(eta) - ln(tau) - ln(alpha)
    # lnSG3 = lnScale + (eta-1)*tEta + ((1-alpha)/alpha)*tGamma + (gamma-1)*ln(t/tau)
    lnScale <- log(gamma) + log(eta) - log(tau) - log(alpha)
    lnSG3   <- lnScale + (eta - 1) * tEta +
               ((1 - alpha) / alpha) * tGamma +
               (gamma - 1) * ln_t_tau

    g3 <- exp(lnSG3)

  } else {
    # alpha = 0 limiting case:
    # G3(t) = (exp((t/tau)^gamma) - 1)^eta
    #
    # tGamma = ln(exp(exp(ln_t_tau_g)) - 1)
    #        = ln(exp((t/tau)^gamma) - 1)
    #        = log(expm1((t/tau)^gamma))
    # lnG3 = eta * tGamma

    t_tau_g <- exp(ln_t_tau_g)         # (t/tau)^gamma
    tGamma  <- .log_expm1(t_tau_g)     # ln(exp((t/tau)^gamma) - 1)
    lnG3    <- eta * tGamma

    G3 <- exp(lnG3)

    # lnScale = ln(gamma) + ln(eta) - ln(tau)
    # lnSG3 = lnScale + (eta-1)*tGamma + (t/tau)^gamma + (gamma-1)*ln(t/tau)
    lnScale <- log(gamma) + log(eta) - log(tau)
    lnSG3   <- lnScale + (eta - 1) * tGamma +
               t_tau_g +
               (gamma - 1) * ln_t_tau

    g3 <- exp(lnSG3)
  }

  list(G3 = G3, g3 = g3)
}


# --- Numerically stable helpers for G3 decomposition -----------------------

#' Compute log(1 + exp(x)) with overflow/underflow protection
#' @param x Numeric vector.
#' @return Numeric vector: log(1 + exp(x)).
#' @keywords internal
.log1pexp <- function(x) {
  # For large x: log(1+exp(x)) ~ x
  # For moderate x: use log1p(exp(x))
  # For very negative x: log(1+exp(x)) ~ exp(x)
  out <- x
  big  <- x > 35
  mid  <- !big & x > -35
  small <- x <= -35

  out[big]   <- x[big]
  out[mid]   <- log1p(exp(x[mid]))
  out[small] <- exp(x[small])

  out
}


#' Compute log(exp(x) - 1) with numerical stability
#' @param x Numeric vector (must be > 0 for exp(x) > 1).
#' @return Numeric vector: log(exp(x) - 1).
#' @keywords internal
.log_expm1 <- function(x) {
  # For large x: log(exp(x) - 1) ~ x
  # For small positive x: use log(expm1(x)) for accuracy
  out <- x
  big <- x > 35
  ok  <- !big & x > 1e-10
  tiny <- x <= 1e-10

  out[big] <- x[big]
  out[ok]  <- log(expm1(x[ok]))
  # For very small x: expm1(x) ~ x, so log(expm1(x)) ~ log(x)
  out[tiny] <- log(pmax(x[tiny], .Machine$double.xmin))

  out
}


# ============================================================================
# Phase-level helpers for the additive cumulative hazard model
# ============================================================================

#' Cumulative hazard contribution from a single phase
#'
#' Computes \eqn{\Phi_j(t)} for one phase in the additive model
#' \eqn{H(t|x) = \sum_j \mu_j(x) \Phi_j(t)}.
#'
#' @param time Numeric vector of times (> 0).
#' @param t_half Half-life parameter (> 0).
#' @param nu Time exponent.
#' @param m Shape parameter.
#' @param type Phase type: `"cdf"` (early -- uses \eqn{G(t)}),
#'   `"hazard"` (late -- uses cumulative hazard from \eqn{h(t)}), or
#'   `"constant"` (flat rate -- \eqn{\Phi = t}).
#'
#' @return Numeric vector of cumulative hazard contributions \eqn{\Phi(t)},
#'   same length as `time`.
#'
#' @details
#' - `"cdf"`: \eqn{\Phi(t) = G(t)}.  Bounded \eqn{[0, 1]}.  Models early
#'   risk that resolves over time.
#' - `"hazard"`: \eqn{\Phi(t) = -\log(1 - G(t))}.  Monotone increasing.
#'   Models late or aging risk.  This is the cumulative hazard derived from
#'   the hazard function \eqn{h(t)}, since
#'   \eqn{\int_0^t h(s)\,ds = -\log(1 - G(t))}.
#' - `"constant"`: \eqn{\Phi(t) = t}.  Ignores `t_half`, `nu`, `m`.
#'   Equivalent to exponential (constant hazard rate).
#'
#' @examples
#' t_grid <- seq(0.1, 10, by = 0.1)
#' phi_early <- hzr_phase_cumhaz(t_grid, t_half = 2, nu = 2, m = 0,
#'                                type = "cdf")
#' phi_late  <- hzr_phase_cumhaz(t_grid, t_half = 5, nu = 1, m = 0,
#'                                type = "hazard")
#' phi_const <- hzr_phase_cumhaz(t_grid, type = "constant")
#'
#' @seealso [hzr_decompos()] for the underlying parametric family,
#'   [hzr_phase_hazard()] for the instantaneous hazard contribution.
#'
#' @export
hzr_phase_cumhaz <- function(time, t_half = 1, nu = 1, m = 0,
                              type = c("cdf", "hazard", "constant")) {
  type <- match.arg(type)

  if (type == "constant") {
    return(time)
  }

  d <- hzr_decompos(time, t_half = t_half, nu = nu, m = m)

  switch(type,
    cdf    = d$G,
    hazard = {
      # Cumulative hazard from h(t): integral_0^t h(s)ds = -log(1 - G(t))
      one_minus_G <- pmax(1 - d$G, .Machine$double.xmin)
      -log(one_minus_G)
    }
  )
}


# ============================================================================
# Phase-level derivatives for analytic gradient
# ============================================================================

#' Derivatives of phase cumulative and instantaneous hazard w.r.t. shape params
#'
#' Computes \eqn{\Phi_j(t)}, \eqn{\phi_j(t)}, and their derivatives with
#' respect to `t_half`, `nu`, and `m` using central finite differences on
#' [hzr_decompos()].  The `log_t_half` derivative is obtained via the chain
#' rule: \eqn{d\Phi/d(\log t_{1/2}) = t_{1/2} \cdot d\Phi/dt_{1/2}}.
#'
#' @param time Numeric vector of positive times.
#' @param t_half Positive scalar half-life.
#' @param nu Numeric scalar time exponent.
#' @param m Numeric scalar shape exponent.
#' @param type Phase type: `"cdf"`, `"hazard"`, or `"constant"`.
#'
#' @return Named list:
#' \describe{
#'   \item{Phi}{Cumulative hazard contribution \eqn{\Phi(t)}.}
#'   \item{phi}{Instantaneous hazard contribution \eqn{\phi(t) = d\Phi/dt}.}
#'   \item{dPhi_dlog_thalf}{\eqn{d\Phi / d(\log t_{1/2})}.}
#'   \item{dPhi_dnu}{\eqn{d\Phi / d\nu}.}
#'   \item{dPhi_dm}{\eqn{d\Phi / dm}.}
#'   \item{dphi_dlog_thalf}{\eqn{d\phi / d(\log t_{1/2})}.}
#'   \item{dphi_dnu}{\eqn{d\phi / d\nu}.}
#'   \item{dphi_dm}{\eqn{d\phi / dm}.}
#' }
#' @keywords internal
.hzr_phase_derivatives <- function(time, t_half, nu, m,
                                    type = c("cdf", "hazard", "constant")) {
  type <- match.arg(type)
  n <- length(time)

  # Constant phases: Phi = t, phi = 1, no shape dependence

  if (type == "constant") {
    zeros <- rep(0, n)
    return(list(
      Phi = time,
      phi = rep(1, n),
      dPhi_dlog_thalf = zeros,
      dPhi_dnu        = zeros,
      dPhi_dm         = zeros,
      dphi_dlog_thalf = zeros,
      dphi_dnu        = zeros,
      dphi_dm         = zeros
    ))
  }

  # Base evaluation
  d0 <- hzr_decompos(time, t_half = t_half, nu = nu, m = m)

  # Extract Phi and phi from decomposition for the given phase type
  extract <- function(d, tp) {
    if (tp == "cdf") {
      Phi <- d$G
      phi <- d$g
    } else {
      # "hazard": Phi = -log(1 - G), phi = h
      one_minus_G <- pmax(1 - d$G, .Machine$double.xmin)
      Phi <- -log(one_minus_G)
      phi <- d$h
    }
    list(Phi = Phi, phi = phi)
  }

  base <- extract(d0, type)

  # Central-difference derivatives w.r.t. shape parameters
  eps_rel <- (.Machine$double.eps) ^ (1 / 3)  # optimal for central diff

  # Helper: perturbed decomposition (returns NULL if invalid params)
  perturb_decompos <- function(th, n_val, m_val) {
    if (m_val < 0 && n_val < 0) return(NULL)
    if (th <= 0) return(NULL)
    tryCatch(
      hzr_decompos(time, t_half = th, nu = n_val, m = m_val),
      error = function(e) NULL
    )
  }

  # Derivative w.r.t. t_half (then multiply by t_half for log_t_half chain rule)
  h_th <- eps_rel * max(abs(t_half), 1e-4)
  d_plus  <- perturb_decompos(t_half + h_th, nu, m)
  d_minus <- perturb_decompos(t_half - h_th, nu, m)
  if (!is.null(d_plus) && !is.null(d_minus)) {
    e_plus  <- extract(d_plus, type)
    e_minus <- extract(d_minus, type)
    dPhi_dt_half <- (e_plus$Phi - e_minus$Phi) / (2 * h_th)
    dphi_dt_half <- (e_plus$phi - e_minus$phi) / (2 * h_th)
  } else {
    # One-sided fallback
    if (!is.null(d_plus)) {
      e_plus <- extract(d_plus, type)
      dPhi_dt_half <- (e_plus$Phi - base$Phi) / h_th
      dphi_dt_half <- (e_plus$phi - base$phi) / h_th
    } else {
      dPhi_dt_half <- rep(0, n)
      dphi_dt_half <- rep(0, n)
    }
  }
  # Chain rule: d/d(log_t_half) = t_half * d/d(t_half)
  dPhi_dlog_thalf <- t_half * dPhi_dt_half
  dphi_dlog_thalf <- t_half * dphi_dt_half

  # Derivative w.r.t. nu
  h_nu <- eps_rel * max(abs(nu), 1)
  d_plus  <- perturb_decompos(t_half, nu + h_nu, m)
  d_minus <- perturb_decompos(t_half, nu - h_nu, m)
  if (!is.null(d_plus) && !is.null(d_minus)) {
    e_plus  <- extract(d_plus, type)
    e_minus <- extract(d_minus, type)
    dPhi_dnu <- (e_plus$Phi - e_minus$Phi) / (2 * h_nu)
    dphi_dnu <- (e_plus$phi - e_minus$phi) / (2 * h_nu)
  } else if (!is.null(d_plus)) {
    e_plus <- extract(d_plus, type)
    dPhi_dnu <- (e_plus$Phi - base$Phi) / h_nu
    dphi_dnu <- (e_plus$phi - base$phi) / h_nu
  } else if (!is.null(d_minus)) {
    e_minus <- extract(d_minus, type)
    dPhi_dnu <- (base$Phi - e_minus$Phi) / h_nu
    dphi_dnu <- (base$phi - e_minus$phi) / h_nu
  } else {
    dPhi_dnu <- rep(0, n)
    dphi_dnu <- rep(0, n)
  }

  # Derivative w.r.t. m
  h_m <- eps_rel * max(abs(m), 1)
  d_plus  <- perturb_decompos(t_half, nu, m + h_m)
  d_minus <- perturb_decompos(t_half, nu, m - h_m)
  if (!is.null(d_plus) && !is.null(d_minus)) {
    e_plus  <- extract(d_plus, type)
    e_minus <- extract(d_minus, type)
    dPhi_dm <- (e_plus$Phi - e_minus$Phi) / (2 * h_m)
    dphi_dm <- (e_plus$phi - e_minus$phi) / (2 * h_m)
  } else if (!is.null(d_plus)) {
    e_plus <- extract(d_plus, type)
    dPhi_dm <- (e_plus$Phi - base$Phi) / h_m
    dphi_dm <- (e_plus$phi - base$phi) / h_m
  } else if (!is.null(d_minus)) {
    e_minus <- extract(d_minus, type)
    dPhi_dm <- (base$Phi - e_minus$Phi) / h_m
    dphi_dm <- (base$phi - e_minus$phi) / h_m
  } else {
    dPhi_dm <- rep(0, n)
    dphi_dm <- rep(0, n)
  }

  list(
    Phi             = base$Phi,
    phi             = base$phi,
    dPhi_dlog_thalf = dPhi_dlog_thalf,
    dPhi_dnu        = dPhi_dnu,
    dPhi_dm         = dPhi_dm,
    dphi_dlog_thalf = dphi_dlog_thalf,
    dphi_dnu        = dphi_dnu,
    dphi_dm         = dphi_dm
  )
}


#' Instantaneous hazard contribution from a single phase
#'
#' Computes \eqn{\phi_j(t) = d\Phi_j/dt} for one phase -- the derivative of
#' the cumulative hazard contribution returned by [hzr_phase_cumhaz()].
#'
#' @inheritParams hzr_phase_cumhaz
#'
#' @return Numeric vector of instantaneous hazard contributions \eqn{\phi(t)},
#'   same length as `time`.
#'
#' @details
#' - `"cdf"`: \eqn{\phi(t) = g(t)} (density).
#' - `"hazard"`: \eqn{\phi(t) = h(t) = g(t)/(1-G(t))}.
#' - `"constant"`: \eqn{\phi(t) = 1}.
#'
#' @examples
#' t_grid <- seq(0.1, 10, by = 0.1)
#' phi_early <- hzr_phase_hazard(t_grid, t_half = 2, nu = 2, m = 0,
#'                                type = "cdf")
#' phi_late  <- hzr_phase_hazard(t_grid, t_half = 5, nu = 1, m = 0,
#'                                type = "hazard")
#'
#' @seealso [hzr_decompos()] for the underlying parametric family,
#'   [hzr_phase_cumhaz()] for the cumulative version.
#'
#' @export
hzr_phase_hazard <- function(time, t_half = 1, nu = 1, m = 0,
                              type = c("cdf", "hazard", "constant")) {
  type <- match.arg(type)

  if (type == "constant") {
    return(rep(1, length(time)))
  }

  d <- hzr_decompos(time, t_half = t_half, nu = nu, m = m)

  switch(type,
    cdf    = d$g,
    hazard = d$h
  )
}

# tests/testthat/test-decompos-boundary.R
#
# Phase 7d: boundary and limiting-case behavior of hzr_decompos().
#
# hzr_decompos() dispatches on the signs of (m, nu) into six branches plus two
# explicit error guards.  The branches include three "limiting" forms (m -> 0
# for nu > 0 and nu < 0, and nu -> 0 for m < 0) that must agree with the
# general branches as the parameter approaches the boundary.  These tests:
#   (1) confirm the previously-unhandled (nu == 0, m >= 0) combination now fails
#       loud instead of raising the cryptic "object 'G' not found";
#   (2) lock in continuity of the verified limiting branches;
#   (3) check internal consistency (g == dG/dt) and CDF sanity (monotone, [0,1]);
#   (4) record the known Case 3 (m > 0, nu < 0) <-> Case 3L discontinuity as a
#       skipped test pending validation against the C HAZARD G1 reference.

# ---------------------------------------------------------------------------
# (1) Unhandled-combination guard
# ---------------------------------------------------------------------------

test_that("nu == 0 with m >= 0 fails loud (not 'object G not found')", {
  # Previously these fell through every dispatch branch, leaving G unassigned
  # and raising a cryptic internal error.
  expect_error(
    hzr_decompos(c(1, 2), t_half = 3, nu = 0, m = 1),
    "Decomposition undefined for nu = 0 with m = 1"
  )
  expect_error(
    hzr_decompos(c(1, 2), t_half = 3, nu = 0, m = 0),
    "Decomposition undefined for nu = 0 with m = 0"
  )
})

test_that("m < 0 with nu < 0 still errors (existing guard)", {
  expect_error(
    hzr_decompos(c(1, 2), t_half = 3, nu = -1, m = -1),
    "Decomposition undefined when both m < 0 and nu < 0"
  )
})

# ---------------------------------------------------------------------------
# (2) Continuity of the verified limiting branches
# ---------------------------------------------------------------------------

test_that("Case 1 (m>0) limits to Case 1L (m=0) as m -> 0, nu > 0", {
  t <- c(0.3, 1, 2.5, 7)
  g_general <- hzr_decompos(t, t_half = 3, nu = 2, m = 1e-7)$G
  g_limit   <- hzr_decompos(t, t_half = 3, nu = 2, m = 0)$G
  expect_equal(g_general, g_limit, tolerance = 1e-5)
})

test_that("Case 2 (m<0) limits to Case 1L (m=0) as m -> 0, nu > 0", {
  t <- c(0.3, 1, 2.5, 7)
  g_general <- hzr_decompos(t, t_half = 3, nu = 2, m = -1e-7)$G
  g_limit   <- hzr_decompos(t, t_half = 3, nu = 2, m = 0)$G
  expect_equal(g_general, g_limit, tolerance = 1e-5)
})

test_that("Case 2 (nu>0) limits to Case 2L (nu=0) as nu -> 0, m < 0", {
  t <- c(0.3, 1, 2.5, 7)
  g_general <- hzr_decompos(t, t_half = 3, nu = 1e-7, m = -1)$G
  g_limit   <- hzr_decompos(t, t_half = 3, nu = 0,    m = -1)$G
  expect_equal(g_general, g_limit, tolerance = 1e-5)
})

# ---------------------------------------------------------------------------
# (3) Internal consistency (g == dG/dt) and CDF sanity
# ---------------------------------------------------------------------------

test_that("g equals the numerical derivative of G across the used branches", {
  check_deriv <- function(t_half, nu, m) {
    t0 <- 2.0
    h  <- 1e-6
    d  <- hzr_decompos(t0, t_half = t_half, nu = nu, m = m)
    dp <- hzr_decompos(t0 + h, t_half = t_half, nu = nu, m = m)
    dm <- hzr_decompos(t0 - h, t_half = t_half, nu = nu, m = m)
    num_g <- (dp$G - dm$G) / (2 * h)
    expect_equal(d$g, num_g, tolerance = 1e-4)
  }
  check_deriv(3,  2,  1)   # Case 1
  check_deriv(3,  2,  0)   # Case 1L
  check_deriv(3,  2, -1)   # Case 2
  check_deriv(3,  0, -1)   # Case 2L
})

test_that("G is a valid CDF (monotone, within [0, 1]) for the early-phase cases", {
  t <- seq(0.01, 30, length.out = 200)
  for (pars in list(c(nu = 2, m = 1), c(nu = 2, m = 0), c(nu = 2, m = -1))) {
    G <- hzr_decompos(t, t_half = 3, nu = pars[["nu"]], m = pars[["m"]])$G
    expect_true(all(G >= 0 & G <= 1),
                label = paste("G in [0,1] for nu=", pars[["nu"]], "m=", pars[["m"]]))
    expect_true(all(diff(G) >= -1e-9),
                label = paste("G monotone for nu=", pars[["nu"]], "m=", pars[["m"]]))
  }
})

# ---------------------------------------------------------------------------
# (4) Extreme t_half stability
# ---------------------------------------------------------------------------

test_that("extreme t_half values produce finite, valid G (nu > 0)", {
  t <- c(0.5, 1, 2, 5, 20)
  for (th in c(1e-6, 1e-3, 1e3, 1e6)) {
    G <- hzr_decompos(t, t_half = th, nu = 2, m = 1)$G
    expect_true(all(is.finite(G)), label = paste("finite G at t_half =", th))
    expect_true(all(G >= 0 & G <= 1), label = paste("G in [0,1] at t_half =", th))
    expect_true(all(diff(G) >= -1e-9), label = paste("G monotone at t_half =", th))
  }
})

# ---------------------------------------------------------------------------
# (5) Case 3 (m>0, nu<0) continuity and C parity
# ---------------------------------------------------------------------------
# Case 3's rho previously used a bare (2^m - 1)^nu (no /m divisor), leaving a
# spurious factor of m on the bt^(-1/nu) term. That diverged from the C G1
# evaluator (g1flag = 5) by up to ~0.2 and broke continuity with the m -> 0
# limit (Case 3L). The /m form matches C exactly and restores continuity.

test_that("Case 3 (m>0, nu<0) limits to Case 3L (m=0) as m -> 0", {
  t <- c(0.3, 1, 2.5, 7)
  g_general <- hzr_decompos(t, t_half = 3, nu = -2, m = 1e-7)$G
  g_limit   <- hzr_decompos(t, t_half = 3, nu = -2, m = 0)$G
  expect_equal(g_general, g_limit, tolerance = 1e-5)
})

test_that("Case 3 (m>0, nu<0) matches the C HAZARD g1flag=5 G1 evaluator", {
  # Direct port of hzd_set_rho() case 5 + hzd_ln_G1_and_SG1() case 5:
  #   rho_C = t_half * (2^m - 1)^nu;  G1 = 1 - ((t/rho_C)^(-1/nu) + 1)^(-1/m)
  c_g1flag5 <- function(time, t_half, nu, m) {
    rho_c <- t_half * (2^m - 1)^nu
    bt    <- time / rho_c
    1 - (bt^(-1 / nu) + 1)^(-1 / m)
  }
  t <- c(0.3, 1, 2.5, 7, 15)
  for (pars in list(c(nu = -2, m = 0.5), c(nu = -1, m = 2), c(nu = -3, m = 1))) {
    nu <- pars[["nu"]]
    m  <- pars[["m"]]
    expect_equal(
      hzr_decompos(t, t_half = 3, nu = nu, m = m)$G,
      c_g1flag5(t, t_half = 3, nu = nu, m = m),
      tolerance = 1e-10,
      label = paste("Case 3 vs C g1flag=5 for nu =", nu, "m =", m)
    )
  }
})

# stepwise.R -- user-facing driver for Phase 4b stepwise covariate
# selection.  Step 8.6 of STEPWISE-DESIGN.md.
#
# The driver combines .hzr_stepwise_forward_step() and
# .hzr_stepwise_backward_step() into a two-way loop with:
#
#   * SAS-style MOVE oscillation guard: when a variable hits
#     `max_move` entries/exits, it is frozen (added to both the
#     force_in and force_out sets used internally).
#   * `max_steps` hard cap that warns on hit.
#   * `force_in` / `force_out` user-supplied constraints.
#   * Optional console trace mirroring the sec.5 output spec.
#
# The returned object inherits from both `hzr_stepwise` and `hazard`
# so it can be passed to `predict()`, `summary()`, `coef()`, etc.
# through the same S3 infrastructure as a plain fit.

#' Stepwise covariate selection for a parametric hazard model
#'
#' Run forward, backward, or two-way stepwise selection on an existing
#' `hazard` fit using Wald p-values or AIC deltas as the entry /
#' retention criterion.  Phase-specific entry is supported for
#' multiphase models: a covariate can enter one phase and not another.
#'
#' @param fit A fitted `hazard` object built via the
#'   `formula = Surv(...) ~ predictors, data = df` interface.
#' @param scope Candidate set.  `NULL` (default) uses every data-frame
#'   column not already in the model for every phase.  For
#'   single-distribution fits, pass a one-sided formula
#'   (`~ age + nyha`) or a character vector of names.  For multiphase
#'   fits, pass a named list of one-sided formulas keyed by phase.
#' @param data Data frame the base fit was built on.  Required for
#'   refits.
#' @param direction One of `"both"` (default), `"forward"`,
#'   `"backward"`.
#' @param criterion One of `"wald"` (default) or `"aic"`.  SAS-style
#'   p-value thresholds apply to Wald; AIC uses `DeltaAIC < 0` uniformly.
#' @param slentry Entry p-value threshold for the Wald criterion.
#'   Default `0.30` matches SAS `SLENTRY`.
#' @param slstay Retention p-value threshold for the Wald criterion.
#'   Default `0.20` matches SAS `SLSTAY`.
#' @param max_steps Hard cap on total accepted actions.  Emits a
#'   `warning()` if hit.  Default `50`.
#' @param max_move Per-variable oscillation cap.  When a variable has
#'   entered + exited more than `max_move` times it is frozen for the
#'   remainder of the run.  Default `4`.
#' @param force_in Character vector of variables that must remain in
#'   the model.  Such variables are still scored and reported in the
#'   selection trace, but are never dropped.
#' @param force_out Character vector of variables that may never be
#'   considered as candidates.
#' @param trace Logical; print step-by-step progress to the console.
#'   Default `TRUE`.
#' @param ... Passed to the underlying `hazard()` refits (e.g.
#'   `control = list(n_starts = 3)`).
#'
#' @return An object of class `c("hzr_stepwise", "hazard")` -- the
#'   final fit augmented with:
#'   \describe{
#'     \item{\code{steps}}{Data frame with one row per accepted /
#'       frozen action; see Details.}
#'     \item{\code{scope}}{Record of the candidate scope, plus
#'       `force_in`, `force_out`, and the frozen set.}
#'     \item{\code{criteria}}{Named list of the threshold / direction
#'       settings actually applied.}
#'     \item{\code{trace_msg}}{Character vector of the trace lines,
#'       captured regardless of the `trace` flag.}
#'     \item{\code{elapsed}}{`difftime` from start to finish.}
#'     \item{\code{final_call}}{The call that produced this result.}
#'   }
#'
#' @details
#' The `steps` data frame has columns:
#'
#' \describe{
#'   \item{\code{step_num}}{Integer sequence starting at 1.}
#'   \item{\code{action}}{`"enter"`, `"drop"`, or `"frozen"`.}
#'   \item{\code{variable}}{Variable affected.}
#'   \item{\code{phase}}{Phase name (multiphase) or `NA_character_`.}
#'   \item{\code{criterion}}{`"wald"` or `"aic"`.}
#'   \item{\code{score}}{Winning score used for the decision.}
#'   \item{\code{stat}, \code{df}}{Wald statistic and degrees of
#'     freedom.}
#'   \item{\code{p_value}, \code{delta_aic}}{Always populated when
#'     computable, regardless of the active criterion.}
#'   \item{\code{logLik}, \code{aic}, \code{n_coef}}{Goodness-of-fit
#'     diagnostics of the model *after* this step.}
#' }
#'
#' @examples
#' data(avc)
#' avc <- na.omit(avc)
#' base <- hazard(survival::Surv(int_dead, dead) ~ age,
#'                data = avc, dist = "weibull", fit = TRUE)
#' \donttest{
#' sw <- hzr_stepwise(base, scope = ~ age + nyha,
#'                    data = avc, direction = "forward",
#'                    control = list(n_starts = 1))
#' print(sw)
#' }
#'
#' @export
hzr_stepwise <- function(fit,
                         scope     = NULL,
                         data,
                         direction = c("both", "forward", "backward"),
                         criterion = c("wald", "aic"),
                         slentry   = 0.30,
                         slstay    = 0.20,
                         max_steps = 50L,
                         max_move  = 4L,
                         force_in  = character(),
                         force_out = character(),
                         trace     = TRUE,
                         ...) {
  direction <- match.arg(direction)
  criterion <- match.arg(criterion)

  if (!inherits(fit, "hazard")) {
    stop("`fit` must be a `hazard` object.", call. = FALSE)
  }
  if (missing(data) || !is.data.frame(data)) {
    stop("`data` must be a data frame (typically the frame used for the base fit).",
         call. = FALSE)
  }

  ts_start <- Sys.time()
  call <- match.call()

  extra_args <- list(...)

  steps     <- list()
  trace_msg <- character()

  emit <- function(msg) {
    trace_msg[[length(trace_msg) + 1L]] <<- msg
    if (isTRUE(trace)) cat(msg, "\n", sep = "")
  }

  # Header line (mirrors design sec.5).
  header <- if (criterion == "wald") {
    sprintf(
      "Stepwise selection (direction = %s, criterion = wald, slentry = %.2f, slstay = %.2f)",
      direction, slentry, slstay
    )
  } else {
    sprintf(
      "Stepwise selection (direction = %s, criterion = aic)",
      direction
    )
  }
  emit(header)
  emit("")

  # Move counter: per-variable tally of entries + exits.  Use a named
  # list rather than a named integer vector -- `lst[[missing]]` returns
  # NULL, whereas `vec[[missing]]` errors with "subscript out of bounds".
  move_counts <- list()
  frozen      <- character()

  current <- fit
  step_no <- 0L
  stopped_by_max_steps <- FALSE

  record_step <- function(action, out) {
    step_no <<- step_no + 1L
    row <- data.frame(
      step_num  = step_no,
      action    = action,
      variable  = out$variable,
      phase     = out$phase,
      criterion = criterion,
      score     = out$score,
      stat      = out$stat,
      df        = out$df,
      p_value   = out$p_value,
      delta_aic = out$delta_aic,
      logLik    = current$fit$objective   %||% NA_real_,
      aic       = .hzr_aic(current),
      n_coef    = length(current$fit$theta),
      stringsAsFactors = FALSE
    )
    steps[[length(steps) + 1L]] <<- row

    score_fmt <- if (criterion == "wald") {
      sprintf("p = %.3f", out$p_value)
    } else {
      sprintf("\u0394AIC = %+.2f", out$delta_aic)
    }
    phase_txt <- if (is.na(out$phase)) {
      ""
    } else if (action == "enter") {
      paste0("  into  ", out$phase)
    } else {
      paste0("  from  ", out$phase)
    }
    emit(sprintf(
      "Step %d: %-6s %s%s   (%s)",
      step_no, toupper(action), out$variable, phase_txt, score_fmt
    ))
  }

  record_freeze <- function(var, phase_hint = NA_character_) {
    step_no <<- step_no + 1L
    row <- data.frame(
      step_num  = step_no,
      action    = "frozen",
      variable  = var,
      phase     = phase_hint,
      criterion = criterion,
      score     = NA_real_,
      stat      = NA_real_,
      df        = NA_integer_,
      p_value   = NA_real_,
      delta_aic = NA_real_,
      logLik    = current$fit$objective   %||% NA_real_,
      aic       = .hzr_aic(current),
      n_coef    = length(current$fit$theta),
      stringsAsFactors = FALSE
    )
    steps[[length(steps) + 1L]] <<- row
    emit(sprintf(
      "Step %d: FROZEN %s   (exceeded max_move = %d; OSCILLATING)",
      step_no, var, max_move
    ))
  }

  bump_move <- function(var) {
    move_counts[[var]] <<- (move_counts[[var]] %||% 0L) + 1L
    if (move_counts[[var]] > max_move && !var %in% frozen) {
      frozen <<- c(frozen, var)
      record_freeze(var)
    }
  }

  # Main loop
  repeat {
    if (step_no >= max_steps) {
      warning("Stepwise selection hit max_steps = ", max_steps,
              "; stopping early.", call. = FALSE)
      stopped_by_max_steps <- TRUE
      break
    }

    add_happened  <- FALSE
    drop_happened <- FALSE

    effective_force_out <- unique(c(force_out, frozen))
    effective_force_in  <- unique(c(force_in,  frozen))

    if (direction %in% c("forward", "both")) {
      fwd <- do.call(.hzr_stepwise_forward_step, c(list(
        current   = current,
        scope     = scope,
        data      = data,
        criterion = criterion,
        slentry   = slentry,
        force_out = effective_force_out
      ), extra_args))

      if (fwd$accepted) {
        current <- fwd$fit
        record_step("enter", fwd)
        bump_move(fwd$variable)
        add_happened <- TRUE
      }
    }

    if (direction %in% c("backward", "both")) {
      bwd <- do.call(.hzr_stepwise_backward_step, c(list(
        current   = current,
        data      = data,
        criterion = criterion,
        slstay    = slstay,
        force_in  = effective_force_in
      ), extra_args))

      if (bwd$accepted) {
        current <- bwd$fit
        record_step("drop", bwd)
        bump_move(bwd$variable)
        drop_happened <- TRUE
      }
    }

    if (!add_happened && !drop_happened) {
      emit(sprintf("(no further action after %d step%s)",
                   step_no, if (step_no == 1L) "" else "s"))
      break
    }
  }

  elapsed <- difftime(Sys.time(), ts_start, units = "secs")

  emit("")
  emit(sprintf("Final model: %d covariate%s, logLik = %.2f, AIC = %.2f",
               max(0L, length(current$fit$theta) -
                     .hzr_stepwise_shape_count(current)),
               if (length(current$fit$theta) -
                     .hzr_stepwise_shape_count(current) == 1L) "" else "s",
               current$fit$objective %||% NA_real_,
               .hzr_aic(current)))

  steps_df <- if (length(steps) == 0L) {
    data.frame(
      step_num = integer(), action = character(),
      variable = character(), phase = character(),
      criterion = character(), score = numeric(),
      stat = numeric(), df = integer(),
      p_value = numeric(), delta_aic = numeric(),
      logLik = numeric(), aic = numeric(), n_coef = integer(),
      stringsAsFactors = FALSE
    )
  } else {
    do.call(rbind, steps)
  }

  result <- current
  result$steps      <- steps_df
  result$scope      <- list(
    candidates = scope,
    force_in   = force_in,
    force_out  = force_out,
    frozen     = frozen
  )
  result$criteria   <- list(
    direction = direction,
    criterion = criterion,
    slentry   = slentry,
    slstay    = slstay,
    max_steps = max_steps,
    max_move  = max_move,
    hit_max_steps = stopped_by_max_steps
  )
  result$trace_msg  <- trace_msg
  result$elapsed    <- elapsed
  result$final_call <- call

  class(result) <- unique(c("hzr_stepwise", class(result)))
  result
}


#' @rdname hzr_stepwise
#' @param x An `hzr_stepwise` object.
#' @param ... Unused.
#' @return `print.hzr_stepwise` returns `x` invisibly.
#' @export
print.hzr_stepwise <- function(x, ...) {
  cat(paste(x$trace_msg, collapse = "\n"), "\n", sep = "")
  invisible(x)
}


#' @rdname hzr_stepwise
#' @param object An `hzr_stepwise` object.
#' @return `summary.hzr_stepwise` returns a `summary.hzr_stepwise` object
#'   (extends `summary.hazard`) with `$stepwise_steps` and `$stepwise_trace`
#'   appended.
#' @export
summary.hzr_stepwise <- function(object, ...) {
  # Strip the stepwise class so NextMethod dispatches cleanly to
  # summary.hazard.
  class(object) <- setdiff(class(object), "hzr_stepwise")
  out <- NextMethod()
  out$stepwise_steps <- object$steps
  out$stepwise_trace <- object$trace_msg
  class(out) <- unique(c("summary.hzr_stepwise", class(out)))
  out
}


#' @rdname hzr_stepwise
#' @return `print.summary.hzr_stepwise` returns `x` invisibly.
#' @export
print.summary.hzr_stepwise <- function(x, ...) {
  if (!is.null(x$stepwise_trace)) {
    cat(paste(x$stepwise_trace, collapse = "\n"), "\n\n", sep = "")
  }
  class(x) <- setdiff(class(x), "summary.hzr_stepwise")
  NextMethod()
}


#' @rdname hzr_stepwise
#' @return `as.data.frame.hzr_stepwise` returns the `$steps` data frame.
#' @export
as.data.frame.hzr_stepwise <- function(x, ...) {
  x$steps
}


#' Extract the captured console trace from an `hzr_stepwise` fit
#'
#' Every run of [hzr_stepwise()] records the header, per-step lines,
#' and final summary regardless of the `trace` flag.  This accessor
#' returns the full character vector for display or logging.
#'
#' @param fit An `hzr_stepwise` object.
#' @return Character vector, one element per console line.
#' @examples
#' data(avc)
#' avc <- na.omit(avc)
#' base <- hazard(survival::Surv(int_dead, dead) ~ age,
#'                data = avc, dist = "weibull", fit = TRUE)
#' \donttest{
#' sw <- hzr_stepwise(base, scope = ~ age + nyha,
#'                    data = avc, direction = "forward",
#'                    control = list(n_starts = 1))
#' cat(stepwise_trace(sw), sep = "\n")
#' }
#' @export
stepwise_trace <- function(fit) {
  if (!inherits(fit, "hzr_stepwise")) {
    stop("`fit` must be an `hzr_stepwise` object.", call. = FALSE)
  }
  fit$trace_msg
}


# Local shape-count helper that tolerates multiphase as well as single
# distributions (the shared `.hzr_shape_parameter_count` in
# parity-helpers only handles the latter).
.hzr_stepwise_shape_count <- function(fit) {
  if (fit$spec$dist == "multiphase") {
    # For multiphase, "coefficient count" is sum of betas across phases
    # -- i.e. theta length minus all non-beta slots.  Easiest path:
    # count columns in x_list.
    x_list <- fit$fit$x_list
    if (is.null(x_list)) return(length(fit$fit$theta))
    total_betas <- sum(vapply(x_list, function(m) {
      if (is.null(m)) 0L else ncol(m)
    }, integer(1L)))
    length(fit$fit$theta) - total_betas
  } else {
    .hzr_shape_parameter_count(fit$spec$dist,
                                control = fit$spec$control)
  }
}

# `.fit_driver_base()` lives in helper-stepwise.R (auto-sourced by
# testthat) so it can be shared with the integration tests.

# Return class and shape ----------------------------------------------------

test_that("hzr_stepwise returns an `hzr_stepwise`-classed `hazard` object", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1 + x2 + x3 + x4, data = obj$data,
    direction = "forward", trace = FALSE
  )
  expect_s3_class(res, "hzr_stepwise")
  expect_s3_class(res, "hazard")
  # Still a usable hazard fit
  expect_s3_class(summary(res), "summary.hzr_stepwise")
  expect_true(is.data.frame(res$steps))
  expect_true(is.character(res$trace_msg))
})


# Forward-only happy path ---------------------------------------------------

test_that("forward selection picks the strong signal and ignores noise", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1 + x2 + x3 + x4, data = obj$data,
    direction = "forward", criterion = "wald",
    slentry = 0.05, trace = FALSE
  )
  # x1 should be the only (or first) variable to enter
  entered <- res$steps$variable[res$steps$action == "enter"]
  expect_true("x1" %in% entered)
  # Scores for entered vars all clear slentry
  expect_true(all(res$steps$p_value[res$steps$action == "enter"] < 0.05,
                  na.rm = TRUE))
})


# Backward-only drops noise -------------------------------------------------

test_that("backward selection drops weak covariates from an overfit model", {
  obj <- .fit_driver_base()
  overfit <- hzr_stepwise(
    obj$fit, scope = ~ x1 + x2 + x3 + x4, data = obj$data,
    direction = "forward", criterion = "wald",
    slentry = 0.99, trace = FALSE   # admit everyone
  )
  # Now strip back
  trimmed <- hzr_stepwise(
    overfit, data = obj$data,
    direction = "backward", criterion = "wald",
    slstay = 0.20, trace = FALSE
  )
  dropped <- trimmed$steps$variable[trimmed$steps$action == "drop"]
  expect_true(any(c("x2", "x3", "x4") %in% dropped))
  expect_false("x1" %in% dropped)
})


# Two-way convergence -------------------------------------------------------

test_that("direction = both converges to a stable model", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1 + x2 + x3 + x4, data = obj$data,
    direction = "both", criterion = "wald",
    slentry = 0.30, slstay = 0.20, trace = FALSE
  )
  # Final model contains x1 and no strictly-dropped var
  final_vars <- colnames(res$data$x)
  expect_true("x1" %in% final_vars)
  # The elapsed field is populated
  expect_s3_class(res$elapsed, "difftime")
})


# AIC criterion -------------------------------------------------------------

test_that("criterion = aic uses ΔAIC and records it in the trace", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1 + x2 + x3 + x4, data = obj$data,
    direction = "forward", criterion = "aic", trace = FALSE
  )
  expect_equal(unique(res$steps$criterion), "aic")
  enters <- res$steps[res$steps$action == "enter", ]
  expect_true(all(enters$delta_aic < 0, na.rm = TRUE))
  # ΔAIC formatting appears in the trace for each entry
  expect_true(any(grepl("AIC", res$trace_msg)))
})


# force_out ----------------------------------------------------------------

test_that("force_out keeps a variable from ever entering", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1 + x2 + x3 + x4, data = obj$data,
    direction = "forward", criterion = "wald",
    slentry = 0.99,                    # admit everyone else
    force_out = "x1",
    trace = FALSE
  )
  expect_false("x1" %in% res$steps$variable[res$steps$action == "enter"])
})


# force_in -----------------------------------------------------------------

test_that("force_in variables stay in even with high p-value", {
  # Start from an overfit model including x2 (pure noise) and force it in
  obj <- .fit_driver_base(signal_beta = 0)
  full <- hazard(
    Surv(time, status) ~ x2,
    data = obj$data,
    theta = c(0.5, 1.0, 0),
    dist = "weibull", fit = TRUE
  )
  res <- hzr_stepwise(
    full, scope = NULL, data = obj$data,
    direction = "backward", criterion = "wald",
    slstay = 0.01,          # very strict — would normally drop x2
    force_in = "x2", trace = FALSE
  )
  # x2 never drops
  expect_false("x2" %in% res$steps$variable[res$steps$action == "drop"])
  # x2 still in final model
  expect_true("x2" %in% colnames(res$data$x))
})


# MOVE oscillation guard ----------------------------------------------------

test_that("max_move = 0 freezes a variable after its first entry", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1 + x2 + x3 + x4, data = obj$data,
    direction = "forward", criterion = "wald",
    slentry = 0.99, max_move = 0L, trace = FALSE
  )
  # The first accepted entry triggers a freeze immediately.
  expect_true("frozen" %in% res$steps$action)
  # The frozen variable is in the scope record
  expect_true(length(res$scope$frozen) >= 1L)
})


# max_steps cap -------------------------------------------------------------

test_that("hitting max_steps warns and stops", {
  obj <- .fit_driver_base()
  expect_warning(
    res <- hzr_stepwise(
      obj$fit, scope = ~ x1 + x2 + x3 + x4, data = obj$data,
      direction = "forward", criterion = "wald",
      slentry = 0.99, max_steps = 1L, trace = FALSE
    ),
    "max_steps"
  )
  expect_true(res$criteria$hit_max_steps)
  expect_true(nrow(res$steps) >= 1L)
})


# Trace output --------------------------------------------------------------

test_that("trace captures header, per-step lines, and a final summary", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1, data = obj$data,
    direction = "forward", criterion = "wald", trace = FALSE
  )
  # Header mentions the criterion
  expect_match(res$trace_msg[1L], "criterion = wald")
  # At least one step line
  expect_true(any(grepl("Step 1", res$trace_msg)))
  # Final line mentions AIC
  expect_true(any(grepl("AIC", res$trace_msg)))
})


# Print + summary methods ---------------------------------------------------

test_that("print.hzr_stepwise emits the captured trace", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1, data = obj$data,
    direction = "forward", criterion = "wald", trace = FALSE
  )
  out <- capture.output(print(res))
  expect_true(any(grepl("Stepwise selection", out)))
  expect_true(any(grepl("Step 1", out)))
})

test_that("summary.hzr_stepwise dispatches to summary.hazard on the final fit", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1, data = obj$data,
    direction = "forward", criterion = "wald", trace = FALSE
  )
  s <- summary(res)
  expect_s3_class(s, "summary.hzr_stepwise")
  expect_s3_class(s, "summary.hazard")
  # Final fit's coef table is accessible
  expect_true(is.data.frame(s$coefficients))
})


# Input validation ---------------------------------------------------------

test_that("hzr_stepwise rejects non-hazard fit", {
  expect_error(
    hzr_stepwise(list(), data = data.frame()),
    "must be a `hazard` object"
  )
})

test_that("hzr_stepwise requires `data`", {
  obj <- .fit_driver_base()
  expect_error(hzr_stepwise(obj$fit), "`data` must be a data frame")
  expect_error(hzr_stepwise(obj$fit, data = "not a df"),
               "`data` must be a data frame")
})


# Polish helpers -----------------------------------------------------------

test_that("as.data.frame returns the $steps trace", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1 + x2, data = obj$data,
    direction = "forward", criterion = "wald", trace = FALSE
  )
  df <- as.data.frame(res)
  expect_s3_class(df, "data.frame")
  expect_identical(df, res$steps)
  expect_true(all(c("step_num", "action", "variable", "phase",
                    "score", "p_value", "delta_aic") %in% names(df)))
})

test_that("stepwise_trace returns the captured console lines", {
  obj <- .fit_driver_base()
  res <- hzr_stepwise(
    obj$fit, scope = ~ x1, data = obj$data,
    direction = "forward", criterion = "wald", trace = FALSE
  )
  tr <- stepwise_trace(res)
  expect_identical(tr, res$trace_msg)
  expect_true(any(grepl("Stepwise selection", tr)))
})

test_that("stepwise_trace rejects non-hzr_stepwise input", {
  expect_error(stepwise_trace(list()),
               "must be an `hzr_stepwise` object")
})

# stepwise-formula.R -- Formula and phase-scope plumbing for stepwise
# covariate selection.
#
# Step 8.3 of STEPWISE-DESIGN.md.  Pure helpers that mutate formulas /
# phase lists / coefficient scopes without touching the optimizer.  The
# actual refit driver that combines these into a new hazard() call
# lands in sec.8.4 alongside the forward/backward step implementations.
#
# Two entry points matter to downstream code:
#
#   .hzr_formula_update(formula, action, var)
#     Add or drop a variable from a one-sided or two-sided formula.
#
#   .hzr_phase_update_formula(phase, action, var)
#     Same mutation, but on the `formula` slot of an `hzr_phase` object;
#     returns an updated `hzr_phase` so the multiphase driver can
#     rebuild the phases list by name.
#
# Both are symmetric under add/drop and idempotent w.r.t. already-present
# or already-absent variables.  Variables currently in scope come from:
#
#   .hzr_scope_current_vars(fit, phase = NULL)
#     Lists the RHS term labels for the global formula (single-dist) or
#     the named phase's formula (multiphase).

# ---------------------------------------------------------------------------
# Formula helpers
# ---------------------------------------------------------------------------

#' Extract RHS term labels from a formula
#'
#' Handles both one-sided (`~ x + y`) and two-sided (`Surv(time, status) ~
#' x + y`) formulas.  Returns an empty character vector for `~ 1`.
#'
#' @keywords internal
#' @noRd
.hzr_formula_rhs_terms <- function(formula) {
  if (is.null(formula)) return(character())
  if (!inherits(formula, "formula")) {
    stop("formula must be a `formula` object.", call. = FALSE)
  }
  # terms() needs a formula with no empty LHS/RHS; ~ 1 has one intercept term.
  tt <- tryCatch(stats::terms(formula),
                 error = function(e) NULL)
  if (is.null(tt)) return(character())
  attr(tt, "term.labels")
}


#' Add or drop a variable from a formula's RHS
#'
#' @param formula Existing formula.  One-sided (`~ x`) or two-sided
#'   (`Surv(time, status) ~ x`) -- the LHS is preserved verbatim.
#' @param action Either `"add"` or `"drop"`.
#' @param var Character scalar naming the variable to add / drop.
#'
#' @return A new formula with the requested change.  Idempotent: adding
#'   a variable that is already present is a no-op; dropping one that
#'   is not present is a no-op.  Dropping the last term leaves `~ 1`.
#'
#' @keywords internal
#' @noRd
.hzr_formula_update <- function(formula, action = c("add", "drop"), var) {
  action <- match.arg(action)
  if (!inherits(formula, "formula")) {
    stop("`formula` must be a `formula` object.", call. = FALSE)
  }
  if (!is.character(var) || length(var) != 1L || !nzchar(var)) {
    stop("`var` must be a non-empty character scalar.", call. = FALSE)
  }

  current_terms <- .hzr_formula_rhs_terms(formula)

  if (action == "add") {
    if (var %in% current_terms) return(formula)
    new_terms <- c(current_terms, var)
  } else {
    if (!var %in% current_terms) return(formula)
    new_terms <- setdiff(current_terms, var)
  }

  rhs <- if (length(new_terms) == 0L) "1" else paste(new_terms, collapse = " + ")

  is_two_sided <- length(formula) == 3L
  lhs <- if (is_two_sided) deparse(formula[[2L]]) else NULL
  env <- environment(formula)

  new_text <- if (is_two_sided) {
    paste(lhs, "~", rhs)
  } else {
    paste("~", rhs)
  }

  stats::as.formula(new_text, env = env)
}


# ---------------------------------------------------------------------------
# Phase-level helpers (multiphase models)
# ---------------------------------------------------------------------------

#' Add or drop a variable from a phase's formula
#'
#' @param phase An `hzr_phase` object.  Its `formula` slot may be NULL
#'   (no phase-specific covariates) -- in the add case a fresh
#'   `~ var` formula is created.
#' @param action Either `"add"` or `"drop"`.
#' @param var Character scalar.
#'
#' @return An updated `hzr_phase` object with the new formula.
#'
#' @keywords internal
#' @noRd
.hzr_phase_update_formula <- function(phase, action = c("add", "drop"), var) {
  action <- match.arg(action)
  if (!inherits(phase, "hzr_phase")) {
    stop("`phase` must be an `hzr_phase` object.", call. = FALSE)
  }
  if (!is.character(var) || length(var) != 1L || !nzchar(var)) {
    stop("`var` must be a non-empty character scalar.", call. = FALSE)
  }

  f <- phase$formula
  if (is.null(f)) {
    if (action == "drop") {
      return(phase)   # nothing to drop
    }
    # Create a fresh one-sided formula in the current calling env so the
    # variable can be resolved later by data-frame column lookup.
    phase$formula <- stats::as.formula(
      paste("~", var),
      env = parent.frame()
    )
    return(phase)
  }

  phase$formula <- .hzr_formula_update(f, action, var)

  # If dropping emptied the RHS (`~ 1`), null the slot out for
  # consistency with hzr_phase(formula = NULL) construction.
  if (action == "drop" &&
        identical(.hzr_formula_rhs_terms(phase$formula), character())) {
    phase$formula <- NULL
  }

  phase
}


# ---------------------------------------------------------------------------
# Scope inspection
# ---------------------------------------------------------------------------

#' List variables currently in a hazard model's scope
#'
#' For single-distribution models returns the RHS term labels of the
#' global formula recorded in `fit$call$formula`.  For multiphase models
#' returns the term labels of the named phase's formula; pass
#' `phase = NULL` to get a named list keyed by phase.
#'
#' @param fit A fitted `hazard` object.
#' @param phase Character scalar phase name (multiphase only), or NULL
#'   to return all phases.
#'
#' @return Character vector of variable names, or a named list of such
#'   vectors.
#'
#' @keywords internal
#' @noRd
.hzr_scope_current_vars <- function(fit, phase = NULL) {
  if (!inherits(fit, "hazard")) {
    stop("`fit` must be a `hazard` object.", call. = FALSE)
  }

  if (fit$spec$dist != "multiphase") {
    if (!is.null(phase)) {
      stop("`phase` is only meaningful for multiphase models.", call. = FALSE)
    }
    f <- fit$call$formula
    if (is.null(f)) {
      # Non-formula fit (time/x interface); infer from design matrix names.
      xcols <- colnames(fit$data$x)
      return(if (is.null(xcols)) character() else xcols)
    }
    # `match.call()` captures `formula` as an unevaluated `call` rather
    # than a real `formula` object; coerce before term extraction.
    if (!inherits(f, "formula")) {
      f <- stats::as.formula(deparse(f))
    }
    return(.hzr_formula_rhs_terms(f))
  }

  # Multiphase
  per_phase <- lapply(fit$spec$phases, function(ph) {
    if (is.null(ph$formula)) character() else .hzr_formula_rhs_terms(ph$formula)
  })

  if (is.null(phase)) return(per_phase)

  if (!is.character(phase) || length(phase) != 1L) {
    stop("`phase` must be a single character scalar.", call. = FALSE)
  }
  if (!phase %in% names(per_phase)) {
    stop("Unknown phase: ", sQuote(phase), ".  Available: ",
         paste(sQuote(names(per_phase)), collapse = ", "),
         call. = FALSE)
  }
  per_phase[[phase]]
}

# .hzr_formula_rhs_terms ----------------------------------------------------

test_that("rhs_terms extracts labels from common formula shapes", {
  expect_identical(.hzr_formula_rhs_terms(~ 1), character())
  expect_identical(.hzr_formula_rhs_terms(~ x), "x")
  expect_identical(.hzr_formula_rhs_terms(~ x + y), c("x", "y"))
  expect_identical(.hzr_formula_rhs_terms(Surv(time, status) ~ 1),
                   character())
  expect_identical(.hzr_formula_rhs_terms(Surv(time, status) ~ x + y),
                   c("x", "y"))
  expect_identical(.hzr_formula_rhs_terms(NULL), character())
})

# .hzr_formula_update -------------------------------------------------------

test_that("formula_update adds a variable to a one-sided formula", {
  f <- ~ x
  out <- .hzr_formula_update(f, "add", "y")
  expect_identical(.hzr_formula_rhs_terms(out), c("x", "y"))
})

test_that("formula_update adds to intercept-only formula", {
  out <- .hzr_formula_update(~ 1, "add", "x")
  expect_identical(.hzr_formula_rhs_terms(out), "x")
})

test_that("formula_update drops an existing variable", {
  out <- .hzr_formula_update(~ x + y + z, "drop", "y")
  expect_identical(.hzr_formula_rhs_terms(out), c("x", "z"))
})

test_that("dropping the only term leaves ~ 1", {
  out <- .hzr_formula_update(~ x, "drop", "x")
  expect_identical(.hzr_formula_rhs_terms(out), character())
  expect_identical(deparse(out), "~1")
})

test_that("adding a duplicate is a no-op", {
  f <- ~ x + y
  out <- .hzr_formula_update(f, "add", "x")
  expect_identical(.hzr_formula_rhs_terms(out), c("x", "y"))
})

test_that("dropping a missing variable is a no-op", {
  f <- ~ x + y
  out <- .hzr_formula_update(f, "drop", "z")
  expect_identical(.hzr_formula_rhs_terms(out), c("x", "y"))
})

test_that("two-sided formula preserves LHS across updates", {
  f <- Surv(time, status) ~ x + y
  added <- .hzr_formula_update(f, "add", "z")
  expect_identical(deparse(added[[2L]]), "Surv(time, status)")
  expect_identical(.hzr_formula_rhs_terms(added), c("x", "y", "z"))

  dropped <- .hzr_formula_update(f, "drop", "x")
  expect_identical(deparse(dropped[[2L]]), "Surv(time, status)")
  expect_identical(.hzr_formula_rhs_terms(dropped), "y")
})

test_that("formula environment is preserved across updates", {
  env <- new.env()
  f <- stats::as.formula("~ x", env = env)
  out <- .hzr_formula_update(f, "add", "y")
  expect_identical(environment(out), env)
})

test_that("invalid inputs are rejected", {
  expect_error(.hzr_formula_update("not a formula", "add", "x"),
               "must be a `formula` object")
  expect_error(.hzr_formula_update(~ 1, "add", character()),
               "non-empty character scalar")
  expect_error(.hzr_formula_update(~ 1, "add", ""),
               "non-empty character scalar")
  expect_error(.hzr_formula_update(~ 1, "add", c("x", "y")),
               "non-empty character scalar")
})

# .hzr_phase_update_formula -------------------------------------------------

test_that("phase_update_formula adds to an existing phase formula", {
  ph <- hzr_phase("cdf", t_half = 1, nu = 1, m = 1, formula = ~ age)
  out <- .hzr_phase_update_formula(ph, "add", "shock")
  expect_s3_class(out, "hzr_phase")
  expect_identical(.hzr_formula_rhs_terms(out$formula), c("age", "shock"))
})

test_that("phase_update_formula creates a formula when the phase had none", {
  ph <- hzr_phase("constant")
  expect_null(ph$formula)
  out <- .hzr_phase_update_formula(ph, "add", "age")
  expect_s3_class(out, "hzr_phase")
  expect_identical(.hzr_formula_rhs_terms(out$formula), "age")
})

test_that("phase_update_formula drops the only term, nulling the slot", {
  ph <- hzr_phase("cdf", t_half = 1, nu = 1, m = 1, formula = ~ age)
  out <- .hzr_phase_update_formula(ph, "drop", "age")
  expect_null(out$formula)
})

test_that("phase_update_formula on NULL-formula drop is a no-op", {
  ph <- hzr_phase("constant")
  out <- .hzr_phase_update_formula(ph, "drop", "age")
  expect_null(out$formula)
})

test_that("phase_update_formula preserves other phase attributes", {
  ph <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 1,
                  fixed = "shapes", formula = ~ age)
  out <- .hzr_phase_update_formula(ph, "add", "shock")
  expect_identical(out$type, "cdf")
  expect_identical(out$t_half, 0.5)
  expect_identical(out$nu, 2)
  expect_identical(out$m, 1)
  expect_identical(out$fixed, c("t_half", "nu", "m"))
})

test_that("phase_update_formula rejects non-phase input", {
  expect_error(.hzr_phase_update_formula(list(), "add", "x"),
               "must be an `hzr_phase` object")
})

# .hzr_scope_current_vars ---------------------------------------------------

test_that("scope_current_vars lists vars for a single-distribution formula fit", {
  set.seed(1)
  n <- 100
  df <- data.frame(time = rexp(n), status = rep(1L, n),
                   age = rnorm(n), nyha = rnorm(n))
  fit <- hazard(
    Surv(time, status) ~ age + nyha,
    data = df,
    theta = c(0.5, 1.0, 0, 0),
    dist = "weibull",
    fit = TRUE
  )
  expect_identical(.hzr_scope_current_vars(fit), c("age", "nyha"))
})

test_that("scope_current_vars falls back to colnames(x) for non-formula fits", {
  fit <- .fit_weibull(n = 100L, betas = 0.3, seed = 1L)
  expect_identical(.hzr_scope_current_vars(fit), "x1")
})

test_that("scope_current_vars returns per-phase list for multiphase", {
  data(avc)
  set.seed(1)
  fit <- hazard(
    Surv(int_dead, dead) ~ 1,
    data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes", formula = ~ age),
      constant = hzr_phase("constant", formula = ~ age + mal)
    ),
    fit = TRUE,
    control = list(n_starts = 2L, maxit = 500L)
  )

  per_phase <- .hzr_scope_current_vars(fit)
  expect_named(per_phase, c("early", "constant"))
  expect_identical(per_phase$early, "age")
  expect_identical(per_phase$constant, c("age", "mal"))

  expect_identical(.hzr_scope_current_vars(fit, phase = "early"), "age")
  expect_identical(.hzr_scope_current_vars(fit, phase = "constant"),
                   c("age", "mal"))
})

test_that("scope_current_vars rejects a bad phase name for multiphase", {
  data(avc)
  set.seed(1)
  fit <- hazard(
    Surv(int_dead, dead) ~ 1, data = avc,
    dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant")
    ),
    fit = TRUE,
    control = list(n_starts = 2L, maxit = 500L)
  )
  expect_error(.hzr_scope_current_vars(fit, phase = "late"),
               "Unknown phase")
})

test_that("scope_current_vars rejects `phase` on single-distribution fits", {
  fit <- .fit_weibull(n = 100L, seed = 1L)
  expect_error(.hzr_scope_current_vars(fit, phase = "early"),
               "only meaningful for multiphase")
})

test_that("scope_current_vars rejects non-hazard input", {
  expect_error(.hzr_scope_current_vars(list()),
               "must be a `hazard` object")
})

# phase-spec.R -- Phase specification for multiphase hazard models
#
# PURPOSE
# -------
# Defines the hzr_phase() constructor that specifies one phase in an
# N-phase additive cumulative hazard model:
#
#   H(t | x) = sum_j  mu_j(x) * Phi_j(t; t_half_j, nu_j, m_j)
#
# Each phase is one term in the sum.  The constructor stores:
#   - type:    "cdf" (early risk), "hazard" (late risk), or "constant"
#   - t_half, nu, m: starting values for the decompos shape parameters
#   - formula: optional one-sided formula for phase-specific covariates
#
# The helpers extract metadata needed during likelihood construction:
#   - .hzr_phase_n_shape():     number of shape parameters (3 or 0)
#   - .hzr_phase_theta_names(): named labels for the parameter sub-vector
#
# SAS/C BRIDGE
# ------------
# SAS users specify Early/Constant/Late phases via DIST keywords.  The
# hzr_phase() constructor maps directly:
#   SAS Early (G1) -> hzr_phase("cdf",      t_half, nu, m)
#   SAS Const (G2) -> hzr_phase("constant")
#   SAS Late  (G3) -> hzr_phase("g3",       tau, gamma, alpha, eta)
#
# The "cdf" and "hazard" types use the G1 decomposition (bounded [0,1]).
# The "g3" type uses the G3 decomposition (unbounded power-law), matching
# the C/SAS HAZARD implementation for late-phase rising hazards.
# The "hazard" type (-log(1-G(t))) is available for alternative models.

# ============================================================================
# Constructor
# ============================================================================

#' Specify a single hazard phase
#'
#' Creates an `hzr_phase` object describing one term in a multiphase additive
#' cumulative hazard model.  Pass a list of these to the `phases` argument of
#' [hazard()] when `dist = "multiphase"`.
#'
#' @section Phase types:
#'
#' \describe{
#'   \item{`"cdf"`}{Early risk that resolves over time.
#'     \eqn{\Phi(t) = G(t)}, bounded \eqn{[0, 1]}.
#'     SAS equivalent: Early / G1 phase.}
#'   \item{`"hazard"`}{Late or aging risk that accumulates.
#'     \eqn{\Phi(t) = -\log(1 - G(t))}, monotone increasing.
#'     SAS equivalent: Late / G3 phase.}
#'   \item{`"constant"`}{Flat background hazard rate.
#'     \eqn{\Phi(t) = t}. No shape parameters are estimated.
#'     SAS equivalent: Constant / G2 phase.}
#' }
#'
#' @param type Character; phase type: `"cdf"`, `"hazard"`, `"g3"`, or
#'   `"constant"`.
#' @param t_half Positive scalar; initial half-life (time at which
#'   \eqn{G(t_{1/2}) = 0.5}).  Used for `"cdf"` and `"hazard"` phases.
#'   SAS early: `THALF`/`RHO`.
#' @param nu Numeric scalar; initial time exponent.  Used for `"cdf"` and
#'   `"hazard"` phases.  SAS early: `NU`.
#' @param m Numeric scalar; initial shape exponent.  Used for `"cdf"` and
#'   `"hazard"` phases.  SAS early: `M`.
#' @param tau Positive scalar; scale parameter for `"g3"` phases.
#'   SAS late: `TAU`.
#' @param gamma Positive scalar; time exponent for `"g3"` phases.
#'   SAS late: `GAMMA`.
#' @param alpha Non-negative scalar; shape parameter for `"g3"` phases.
#'   When `alpha > 0`, the generic G3 formula is used; `alpha = 0` gives the
#'   exponential limiting case.  SAS late: `ALPHA`.
#' @param eta Positive scalar; outer exponent for `"g3"` phases.
#'   SAS late: `ETA`.
#' @param formula Optional one-sided formula (e.g. `~ age + nyha`) for
#'   phase-specific covariates.  When `NULL` (default), the phase inherits
#'   the global formula from [hazard()].
#' @param fixed Character vector naming shape parameters to hold fixed during
#'   optimization.  Valid names for `"cdf"`/`"hazard"`: `"t_half"`, `"nu"`,
#'   `"m"`, or `"shapes"` (shorthand for all three).  Valid names for `"g3"`:
#'   `"tau"`, `"gamma"`, `"alpha"`, `"eta"`, or `"shapes"` (shorthand for all
#'   four).  Fixed parameters are held at their starting values; only `mu`
#'   (and covariates) are estimated.  Ignored for `"constant"` phases.
#'   This mirrors the SAS/C HAZARD workflow where shapes are typically fixed
#'   and only scale parameters are estimated.
#'
#' @return An S3 object of class `"hzr_phase"` with elements:
#' \describe{
#'   \item{type}{Phase type string.}
#'   \item{t_half}{Initial half-life (cdf/hazard phases).}
#'   \item{nu}{Initial time exponent (cdf/hazard phases).}
#'   \item{m}{Initial shape exponent (cdf/hazard phases).}
#'   \item{tau}{Scale parameter (g3 phases).}
#'   \item{gamma}{Time exponent (g3 phases).}
#'   \item{alpha}{Shape parameter (g3 phases).}
#'   \item{eta}{Outer exponent (g3 phases).}
#'   \item{formula}{Phase-specific formula or `NULL`.}
#'   \item{fixed}{Character vector of fixed parameter names (may be empty).}
#' }
#'
#' @examples
#' # Classic 3-phase Blackstone pattern
#' early <- hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0)
#' const <- hzr_phase("constant")
#' late  <- hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1)
#'
#' # Fix all shapes (C/SAS-style: only estimate mu)
#' early_fixed <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                           fixed = "shapes")
#' late_fixed  <- hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
#'                           fixed = "shapes")
#'
#' # Fix only some parameters
#' early_partial <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                             fixed = c("nu", "m"))
#'
#' # Phase with specific covariates
#' early_cov <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                         formula = ~ age + shock)
#'
#' # Use in hazard():
#' # hazard(Surv(time, status) ~ age, data = dat,
#' #        dist = "multiphase",
#' #        phases = list(early = early, constant = const, late = late))
#'
#' @seealso [hazard()] for fitting multiphase models,
#'   [hzr_decompos()] for the underlying parametric family,
#'   [hzr_phase_cumhaz()] and [hzr_phase_hazard()] for computing
#'   \eqn{\Phi(t)} and \eqn{\phi(t)} from these specifications.
#'
#' \code{vignette("fitting-hazard-models")} for multiphase fitting examples,
#' \code{vignette("mf-mathematical-foundations")} for the mathematical framework.
#'
#' @export
hzr_phase <- function(type = c("cdf", "hazard", "constant", "g3"),
                      t_half = 1, nu = 1, m = 0,
                      tau = 1, gamma = 1, alpha = 1, eta = 1,
                      formula = NULL,
                      fixed = character(0)) {

  type <- match.arg(type)

  # --- Validate shape parameters based on type ------------------------------
  if (type == "g3") {
    # G3 late-phase decomposition: 4 parameters (tau, gamma, alpha, eta)
    stopifnot(
      "tau must be a positive scalar" =
        is.numeric(tau) && length(tau) == 1L && tau > 0,
      "gamma must be a positive scalar" =
        is.numeric(gamma) && length(gamma) == 1L && gamma > 0,
      "alpha must be a non-negative scalar" =
        is.numeric(alpha) && length(alpha) == 1L && alpha >= 0 && is.finite(alpha),
      "eta must be a positive scalar" =
        is.numeric(eta) && length(eta) == 1L && eta > 0
    )
  } else if (type != "constant") {
    # G1 early-phase decomposition: 3 parameters (t_half, nu, m)
    stopifnot(
      "t_half must be a positive scalar" =
        is.numeric(t_half) && length(t_half) == 1L &&
        is.finite(t_half) && t_half > 0,
      "nu must be a finite numeric scalar" =
        is.numeric(nu) && length(nu) == 1L && is.finite(nu),
      "m must be a finite numeric scalar" =
        is.numeric(m) && length(m) == 1L && is.finite(m)
    )

    if (m < 0 && nu < 0) {
      stop("Decomposition undefined when both m < 0 and nu < 0 (m = ",
           m, ", nu = ", nu, ").  See ?hzr_decompos for valid cases.",
           call. = FALSE)
    }
  }

  # --- Validate formula if supplied -----------------------------------------
  if (!is.null(formula)) {
    if (!inherits(formula, "formula")) {
      stop("formula must be a one-sided formula (e.g. ~ age + nyha) or NULL.",
           call. = FALSE)
    }
    # Must be one-sided (no LHS)
    if (length(formula) == 3L) {
      stop("Phase formula must be one-sided (e.g. ~ age + nyha), ",
           "not two-sided (response ~ predictors).", call. = FALSE)
    }
  }

  # --- Validate and normalize fixed parameters ------------------------------
  if (length(fixed) > 0 && type == "constant") {
    warning("fixed is ignored for constant phases.", call. = FALSE)
    fixed <- character(0)
  }
  if (length(fixed) > 0) {
    if (type == "g3") {
      # G3: expand "shapes" to tau, gamma, alpha, eta
      if ("shapes" %in% fixed) {
        fixed <- union(setdiff(fixed, "shapes"), c("tau", "gamma", "alpha", "eta"))
      }
      valid_fixed <- c("tau", "gamma", "alpha", "eta")
    } else {
      # G1: expand "shapes" to t_half, nu, m
      if ("shapes" %in% fixed) {
        fixed <- union(setdiff(fixed, "shapes"), c("t_half", "nu", "m"))
      }
      valid_fixed <- c("t_half", "nu", "m")
    }
    bad <- setdiff(fixed, valid_fixed)
    if (length(bad) > 0) {
      stop("Invalid fixed parameter names: ",
           paste(bad, collapse = ", "),
           ". Valid names: ", paste(valid_fixed, collapse = ", "),
           ", or 'shapes' for all.", call. = FALSE)
    }
    fixed <- unique(fixed)
  }

  if (type == "g3") {
    obj <- list(
      type    = type,
      tau     = tau,
      gamma   = gamma,
      alpha   = alpha,
      eta     = eta,
      formula = formula,
      fixed   = fixed
    )
  } else {
    obj <- list(
      type    = type,
      t_half  = if (type == "constant") NA_real_ else t_half,
      nu      = if (type == "constant") NA_real_ else nu,
      m       = if (type == "constant") NA_real_ else m,
      formula = formula,
      fixed   = fixed
    )
  }

  structure(obj, class = "hzr_phase")
}


# ============================================================================
# S3 methods
# ============================================================================

#' @rdname hzr_phase
#' @param x An `hzr_phase` object (for `print.hzr_phase()`).
#' @param ... Additional arguments (ignored).
#' @export
print.hzr_phase <- function(x, ...) {
  label <- switch(x$type,
    cdf      = "cdf (early risk)",
    hazard   = "hazard (late risk)",
    constant = "constant (flat rate)",
    g3       = "g3 (late phase)"
  )
  cat("<hzr_phase>", label, "\n")

  if (x$type == "g3") {
    cat("  tau =", format(x$tau, digits = 4),
        " gamma =", format(x$gamma, digits = 4),
        " alpha =", format(x$alpha, digits = 4),
        " eta =", format(x$eta, digits = 4), "\n")
  } else if (x$type != "constant") {
    cat("  t_half =", format(x$t_half, digits = 4),
        " nu =", format(x$nu, digits = 4),
        " m =", format(x$m, digits = 4), "\n")
  }

  if (x$type != "constant" && length(x$fixed) > 0) {
    cat("  fixed:", paste(x$fixed, collapse = ", "), "\n")
  }

  if (!is.null(x$formula)) {
    cat("  covariates:", deparse(x$formula), "\n")
  }

  invisible(x)
}


# ============================================================================
# Type-checking helper
# ============================================================================

#' Test if an object is an hzr_phase
#'
#' @param x Object to test.
#' @return Logical scalar.
#'
#' @examples
#' is_hzr_phase(hzr_phase("cdf"))
#' is_hzr_phase("not a phase")
#'
#' @export
is_hzr_phase <- function(x) {
  inherits(x, "hzr_phase")
}


# ============================================================================
# Internal helpers for likelihood construction
# ============================================================================

#' Number of shape parameters for a phase
#'
#' Returns 3 (t_half, nu, m) for `"cdf"` and `"hazard"` phases, 0 for
#' `"constant"`.
#'
#' @param phase An `hzr_phase` object.
#' @return Integer: 3 or 0.
#' @keywords internal
.hzr_phase_n_shape <- function(phase) {
  stopifnot(is_hzr_phase(phase))
  switch(phase$type,
    constant = 0L,
    g3       = 4L,
    3L  # cdf, hazard
  )
}


#' Generate parameter names for a phase's sub-vector
#'
#' Builds the named labels for the parameter block that belongs to a single
#' phase in the full theta vector.  The block layout is:
#'
#' \itemize{
#'   \item For `"cdf"`/`"hazard"`: `[log_mu, log_t_half, nu, m, beta_1, ..., beta_p]`
#'   \item For `"constant"`:       `[log_mu, beta_1, ..., beta_p]`
#' }
#'
#' @param phase An `hzr_phase` object.
#' @param phase_name Character label for the phase (e.g. `"early"`, `"phase_1"`).
#' @param covariate_names Character vector of covariate column names that
#'   this phase uses.  Can be length 0 if no covariates.
#' @return Character vector of parameter names.
#' @keywords internal
.hzr_phase_theta_names <- function(phase, phase_name, covariate_names = character(0)) {
  stopifnot(is_hzr_phase(phase))

  # Intercept (always present)
  names_out <- paste0(phase_name, ".log_mu")

  # Shape parameters
  if (phase$type == "g3") {
    names_out <- c(names_out,
      paste0(phase_name, ".log_tau"),
      paste0(phase_name, ".gamma"),
      paste0(phase_name, ".alpha"),
      paste0(phase_name, ".eta")
    )
  } else if (phase$type != "constant") {
    names_out <- c(names_out,
      paste0(phase_name, ".log_t_half"),
      paste0(phase_name, ".nu"),
      paste0(phase_name, ".m")
    )
  }

  # Covariate coefficients
  if (length(covariate_names) > 0L) {
    names_out <- c(names_out,
      paste0(phase_name, ".", covariate_names)
    )
  }

  names_out
}


#' Total number of parameters for a phase
#'
#' Returns the count of free parameters in the theta sub-vector for one phase:
#' 1 (log_mu) + n_shape + n_covariates.
#'
#' @param phase An `hzr_phase` object.
#' @param n_covariates Integer; number of covariate columns this phase uses.
#' @return Integer.
#' @keywords internal
.hzr_phase_n_params <- function(phase, n_covariates = 0L) {
  stopifnot(is_hzr_phase(phase))
  1L + .hzr_phase_n_shape(phase) + as.integer(n_covariates)
}


#' Extract starting values from a phase specification
#'
#' Returns initial theta sub-vector on the estimation (internal) scale:
#' log(mu), log(t_half), nu, m, followed by zeros for covariate coefficients.
#'
#' @param phase An `hzr_phase` object.
#' @param n_covariates Integer; number of covariate columns.
#' @param mu_start Numeric scalar; initial scale parameter (default 0.1).
#' @return Named numeric vector of starting values.
#' @keywords internal
.hzr_phase_start <- function(phase, n_covariates = 0L, mu_start = 0.1) {
  stopifnot(is_hzr_phase(phase))
  stopifnot(
    "mu_start must be a positive scalar" =
      is.numeric(mu_start) && length(mu_start) == 1L && mu_start > 0
  )

  vals <- log(mu_start)  # log_mu

  if (phase$type == "g3") {
    vals <- c(vals, log(phase$tau), phase$gamma, phase$alpha, phase$eta)
  } else if (phase$type != "constant") {
    vals <- c(vals, log(phase$t_half), phase$nu, phase$m)
  }

  # Covariate betas initialized to zero
  if (n_covariates > 0L) {
    vals <- c(vals, rep(0, n_covariates))
  }

  vals
}


#' Build a logical mask of free (non-fixed) parameters in the full theta vector
#'
#' Returns a logical vector the same length as the full theta where `TRUE`
#' means the parameter is free (to be optimized) and `FALSE` means it is
#' held fixed at its starting value.
#'
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of per-phase covariate counts.
#' @return Logical vector of length `sum(.hzr_phase_n_params(...))`.
#' @keywords internal
.hzr_phase_free_mask <- function(phases, covariate_counts) {
  mask <- logical(0)

  for (nm in names(phases)) {
    ph <- phases[[nm]]
    n_cov <- covariate_counts[[nm]]

    # log_mu is always free
    mask <- c(mask, TRUE)

    # Shape parameters
    if (ph$type == "g3") {
      fixed <- if (is.null(ph$fixed)) character(0) else ph$fixed
      mask <- c(mask,
        !("tau"   %in% fixed),  # log_tau
        !("gamma" %in% fixed),  # gamma
        !("alpha" %in% fixed),  # alpha
        !("eta"   %in% fixed)   # eta
      )
    } else if (ph$type != "constant") {
      fixed <- if (is.null(ph$fixed)) character(0) else ph$fixed
      mask <- c(mask,
        !("t_half" %in% fixed),  # log_t_half
        !("nu"     %in% fixed),  # nu
        !("m"      %in% fixed)   # m
      )
    }

    # Covariate betas are always free
    if (n_cov > 0L) {
      mask <- c(mask, rep(TRUE, n_cov))
    }
  }

  mask
}


#' Validate a list of phase specifications
#'
#' Checks that `phases` is a non-empty named list of `hzr_phase` objects.
#' Auto-names unnamed phases as `phase_1`, `phase_2`, etc.
#'
#' @param phases A list of `hzr_phase` objects.
#' @return The validated (and possibly auto-named) list, invisibly.
#' @keywords internal
.hzr_validate_phases <- function(phases) {
  # Catch bare hzr_phase passed instead of list(hzr_phase(...))
  if (is_hzr_phase(phases)) {
    stop("phases must be a non-empty list of hzr_phase objects, ",
         "not a single hzr_phase.  Wrap it: list(hzr_phase(...)).", call. = FALSE)
  }
  if (!is.list(phases) || length(phases) == 0L) {
    stop("phases must be a non-empty list of hzr_phase objects.", call. = FALSE)
  }

  # Check each element

  for (i in seq_along(phases)) {
    if (!is_hzr_phase(phases[[i]])) {
      stop("phases[[", i, "]] is not an hzr_phase object. ",
           "Use hzr_phase() to create phase specifications.", call. = FALSE)
    }
  }

  # Auto-name unnamed phases
  nms <- names(phases)
  if (is.null(nms)) {
    nms <- character(length(phases))
  }
  empty <- nms == "" | is.na(nms)
  if (any(empty)) {
    nms[empty] <- paste0("phase_", which(empty))
  }

  # Check for duplicate names
  if (anyDuplicated(nms)) {
    stop("Phase names must be unique. Duplicates found: ",
         paste(nms[duplicated(nms)], collapse = ", "), call. = FALSE)
  }

  names(phases) <- nms
  invisible(phases)
}

# test-phase-spec.R — Tests for hzr_phase() and internal helpers
#
# Validates construction, validation, printing, parameter name generation,
# starting value extraction, and phase list validation.

# ============================================================================
# hzr_phase() construction
# ============================================================================

test_that("hzr_phase('cdf') creates valid object with defaults", {
  p <- hzr_phase("cdf")
  expect_s3_class(p, "hzr_phase")
  expect_equal(p$type, "cdf")
  expect_equal(p$t_half, 1)
  expect_equal(p$nu, 1)
  expect_equal(p$m, 0)
  expect_null(p$formula)
})

test_that("hzr_phase('hazard') stores custom starting values", {
  p <- hzr_phase("hazard", t_half = 5, nu = 0.8, m = -0.5)
  expect_equal(p$type, "hazard")
  expect_equal(p$t_half, 5)
  expect_equal(p$nu, 0.8)
  expect_equal(p$m, -0.5)
})

test_that("hzr_phase('constant') stores NA for shape parameters", {
  p <- hzr_phase("constant")
  expect_equal(p$type, "constant")
  expect_true(is.na(p$t_half))
  expect_true(is.na(p$nu))
  expect_true(is.na(p$m))
})

test_that("hzr_phase('constant') ignores supplied shape values", {
  p <- hzr_phase("constant", t_half = 99, nu = 99, m = 99)
  expect_true(is.na(p$t_half))
  expect_true(is.na(p$nu))
  expect_true(is.na(p$m))
})

test_that("hzr_phase stores one-sided formula", {
  p <- hzr_phase("cdf", formula = ~ age + nyha)
  expect_true(inherits(p$formula, "formula"))
})

test_that("type is matched by match.arg", {
  # "c" is ambiguous (matches both "cdf" and "constant"), so use longer prefixes
  expect_equal(hzr_phase("cd")$type, "cdf")
  expect_equal(hzr_phase("h")$type, "hazard")
  expect_equal(hzr_phase("con")$type, "constant")
})


# ============================================================================
# Input validation
# ============================================================================

test_that("hzr_phase rejects invalid type", {
  expect_error(hzr_phase("unknown"))
})

test_that("hzr_phase rejects non-positive t_half", {
  expect_error(hzr_phase("cdf", t_half = 0), "t_half must be a positive")
  expect_error(hzr_phase("cdf", t_half = -1), "t_half must be a positive")
})

test_that("hzr_phase rejects non-finite shape parameters", {
  expect_error(hzr_phase("cdf", nu = Inf), "nu must be a finite")
  expect_error(hzr_phase("cdf", m = NA), "m must be a finite")
  expect_error(hzr_phase("cdf", t_half = NaN), "t_half must be a positive")
})

test_that("hzr_phase rejects m < 0 AND nu < 0", {
  expect_error(hzr_phase("cdf", nu = -1, m = -1),
               "both m < 0 and nu < 0")
})

test_that("hzr_phase rejects two-sided formula", {
  expect_error(hzr_phase("cdf", formula = y ~ x),
               "one-sided")
})

test_that("hzr_phase rejects non-formula formula argument", {
  expect_error(hzr_phase("cdf", formula = "not a formula"),
               "formula must be a one-sided formula")
})


# ============================================================================
# is_hzr_phase()
# ============================================================================

test_that("is_hzr_phase returns TRUE for phases", {
  expect_true(is_hzr_phase(hzr_phase("cdf")))
  expect_true(is_hzr_phase(hzr_phase("constant")))
})

test_that("is_hzr_phase returns FALSE for non-phases", {
  expect_false(is_hzr_phase("cdf"))
  expect_false(is_hzr_phase(list(type = "cdf")))
  expect_false(is_hzr_phase(42))
  expect_false(is_hzr_phase(NULL))
})


# ============================================================================
# print.hzr_phase()
# ============================================================================

test_that("print.hzr_phase produces expected output for cdf", {
  p <- hzr_phase("cdf", t_half = 2.5, nu = 1.5, m = 0.3)
  out <- capture.output(print(p))
  expect_true(any(grepl("hzr_phase.*cdf.*early", out)))
  expect_true(any(grepl("t_half", out)))
})

test_that("print.hzr_phase produces expected output for constant", {
  p <- hzr_phase("constant")
  out <- capture.output(print(p))
  expect_true(any(grepl("constant.*flat", out)))
  # No shape line for constant
  expect_false(any(grepl("t_half", out)))
})

test_that("print.hzr_phase shows formula when present", {
  p <- hzr_phase("hazard", formula = ~ age + shock)
  out <- capture.output(print(p))
  expect_true(any(grepl("covariates.*age.*shock", out)))
})


# ============================================================================
# .hzr_phase_n_shape()
# ============================================================================

test_that(".hzr_phase_n_shape returns 3 for cdf and hazard", {
  expect_equal(.hzr_phase_n_shape(hzr_phase("cdf")), 3L)
  expect_equal(.hzr_phase_n_shape(hzr_phase("hazard")), 3L)
})

test_that(".hzr_phase_n_shape returns 0 for constant", {
  expect_equal(.hzr_phase_n_shape(hzr_phase("constant")), 0L)
})


# ============================================================================
# .hzr_phase_theta_names()
# ============================================================================

test_that("theta names for cdf phase without covariates", {
  p <- hzr_phase("cdf")
  nms <- .hzr_phase_theta_names(p, "early")
  expect_equal(nms, c("early.log_mu", "early.log_t_half",
                       "early.nu", "early.m"))
})

test_that("theta names for hazard phase with covariates", {
  p <- hzr_phase("hazard")
  nms <- .hzr_phase_theta_names(p, "late", c("age", "nyha"))
  expect_equal(nms, c("late.log_mu", "late.log_t_half",
                       "late.nu", "late.m",
                       "late.age", "late.nyha"))
})

test_that("theta names for constant phase", {
  p <- hzr_phase("constant")
  nms <- .hzr_phase_theta_names(p, "const")
  expect_equal(nms, "const.log_mu")
})

test_that("theta names for constant phase with covariates", {
  p <- hzr_phase("constant")
  nms <- .hzr_phase_theta_names(p, "const", c("age"))
  expect_equal(nms, c("const.log_mu", "const.age"))
})


# ============================================================================
# .hzr_phase_n_params()
# ============================================================================

test_that("n_params correct for cdf without covariates", {
  expect_equal(.hzr_phase_n_params(hzr_phase("cdf")), 4L)  # mu + 3 shape
})

test_that("n_params correct for cdf with 2 covariates", {
  expect_equal(.hzr_phase_n_params(hzr_phase("cdf"), n_covariates = 2), 6L)
})

test_that("n_params correct for constant without covariates", {
  expect_equal(.hzr_phase_n_params(hzr_phase("constant")), 1L)  # mu only
})

test_that("n_params correct for constant with 3 covariates", {
  expect_equal(.hzr_phase_n_params(hzr_phase("constant"), n_covariates = 3), 4L)
})


# ============================================================================
# .hzr_phase_start()
# ============================================================================

test_that("start values for cdf phase", {
  p <- hzr_phase("cdf", t_half = 2, nu = 1.5, m = 0.3)
  sv <- .hzr_phase_start(p)
  expect_equal(length(sv), 4L)
  expect_equal(sv[1], log(0.1))         # log_mu default
  expect_equal(sv[2], log(2))           # log_t_half
  expect_equal(sv[3], 1.5)             # nu
  expect_equal(sv[4], 0.3)             # m
})

test_that("start values for constant phase", {
  p <- hzr_phase("constant")
  sv <- .hzr_phase_start(p)
  expect_equal(length(sv), 1L)
  expect_equal(sv[1], log(0.1))
})

test_that("start values include zero betas for covariates", {
  p <- hzr_phase("hazard", t_half = 5, nu = 1, m = 0)
  sv <- .hzr_phase_start(p, n_covariates = 3)
  expect_equal(length(sv), 7L)  # log_mu + 3 shape + 3 betas
  expect_equal(sv[5:7], c(0, 0, 0))
})

test_that("start values respect mu_start", {
  p <- hzr_phase("cdf")
  sv <- .hzr_phase_start(p, mu_start = 0.5)
  expect_equal(sv[1], log(0.5))
})

test_that(".hzr_phase_start rejects non-positive mu_start", {
  expect_error(.hzr_phase_start(hzr_phase("cdf"), mu_start = 0),
               "mu_start must be a positive")
  expect_error(.hzr_phase_start(hzr_phase("cdf"), mu_start = -1),
               "mu_start must be a positive")
})


# ============================================================================
# .hzr_validate_phases()
# ============================================================================

test_that("validate_phases accepts named list of phases", {
  phases <- list(
    early    = hzr_phase("cdf"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("hazard")
  )
  result <- .hzr_validate_phases(phases)
  expect_equal(names(result), c("early", "constant", "late"))
})

test_that("validate_phases auto-names unnamed phases", {
  phases <- list(hzr_phase("cdf"), hzr_phase("constant"))
  result <- .hzr_validate_phases(phases)
  expect_equal(names(result), c("phase_1", "phase_2"))
})

test_that("validate_phases auto-names partially named phases", {
  phases <- list(early = hzr_phase("cdf"), hzr_phase("constant"))
  result <- .hzr_validate_phases(phases)
  expect_equal(names(result), c("early", "phase_2"))
})

test_that("validate_phases rejects empty list", {
  expect_error(.hzr_validate_phases(list()),
               "non-empty list")
})

test_that("validate_phases rejects non-list", {
  expect_error(.hzr_validate_phases("not a list"),
               "non-empty list")
})

test_that("validate_phases rejects bare hzr_phase (not wrapped in list)", {
  expect_error(.hzr_validate_phases(hzr_phase("cdf")),
               "not a single hzr_phase")
})

test_that("validate_phases rejects non-phase elements", {
  expect_error(.hzr_validate_phases(list(hzr_phase("cdf"), "bad")),
               "not an hzr_phase")
})

test_that("validate_phases rejects duplicate names", {
  phases <- list(early = hzr_phase("cdf"), early = hzr_phase("hazard"))
  expect_error(.hzr_validate_phases(phases),
               "unique")
})


# ============================================================================
# Round-trip: start values match theta names in length
# ============================================================================

test_that("start values and theta names have matching length", {
  cases <- list(
    list(phase = hzr_phase("cdf"),      name = "early", ncov = 0L),
    list(phase = hzr_phase("cdf"),      name = "early", ncov = 3L),
    list(phase = hzr_phase("hazard"),   name = "late",  ncov = 2L),
    list(phase = hzr_phase("constant"), name = "const", ncov = 0L),
    list(phase = hzr_phase("constant"), name = "const", ncov = 1L)
  )

  for (cc in cases) {
    sv  <- .hzr_phase_start(cc$phase, n_covariates = cc$ncov)
    nms <- .hzr_phase_theta_names(cc$phase, cc$name,
                                   if (cc$ncov > 0) paste0("x", seq_len(cc$ncov))
                                   else character(0))
    np  <- .hzr_phase_n_params(cc$phase, n_covariates = cc$ncov)

    expect_equal(length(sv), length(nms),
                 label = paste("start/names length for", cc$name,
                               "ncov=", cc$ncov))
    expect_equal(length(sv), np,
                 label = paste("start/n_params for", cc$name,
                               "ncov=", cc$ncov))
  }
})


# ============================================================================
# Fixed parameter support
# ============================================================================

test_that("hzr_phase stores fixed parameter names", {
  ph <- hzr_phase("cdf", t_half = 1, nu = 2, m = 0, fixed = "shapes")
  expect_equal(sort(ph$fixed), c("m", "nu", "t_half"))

  ph2 <- hzr_phase("hazard", t_half = 5, nu = 1, m = 0, fixed = c("nu", "m"))
  expect_equal(sort(ph2$fixed), c("m", "nu"))

  ph3 <- hzr_phase("cdf", t_half = 1, nu = 1, m = 0)
  expect_equal(ph3$fixed, character(0))
})

test_that("hzr_phase rejects invalid fixed names", {
  expect_error(
    hzr_phase("cdf", t_half = 1, nu = 1, m = 0, fixed = "mu"),
    "Invalid fixed"
  )
  expect_error(
    hzr_phase("cdf", t_half = 1, nu = 1, m = 0, fixed = "log_mu"),
    "Invalid fixed"
  )
})

test_that("fixed is ignored for constant phases with warning", {
  expect_warning(
    ph <- hzr_phase("constant", fixed = "shapes"),
    "ignored for constant"
  )
  expect_equal(ph$fixed, character(0))
})

test_that(".hzr_phase_free_mask works correctly", {
  phases <- list(
    early    = hzr_phase("cdf", t_half = 1, nu = 1, m = 0,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("hazard", t_half = 5, nu = 1, m = 0,
                          fixed = c("nu", "m"))
  )
  cov_counts <- c(early = 0L, constant = 0L, late = 2L)

  mask <- .hzr_phase_free_mask(phases, cov_counts)

  # early: log_mu=T, log_t_half=F, nu=F, m=F
  # constant: log_mu=T
  # late: log_mu=T, log_t_half=T, nu=F, m=F, beta1=T, beta2=T
  expected <- c(TRUE, FALSE, FALSE, FALSE,   # early
                TRUE,                          # constant
                TRUE, TRUE, FALSE, FALSE, TRUE, TRUE)  # late
  expect_equal(mask, expected)
  expect_equal(sum(mask), 6)  # 6 free params
  expect_equal(sum(!mask), 5) # 5 fixed params
})

test_that(".hzr_phase_free_mask is all TRUE when nothing fixed", {
  phases <- list(
    early = hzr_phase("cdf", t_half = 1, nu = 1, m = 0),
    const = hzr_phase("constant")
  )
  cov_counts <- c(early = 0L, const = 0L)
  mask <- .hzr_phase_free_mask(phases, cov_counts)
  expect_true(all(mask))
  expect_equal(length(mask), 5)  # 4 early + 1 const
})

#' @importFrom stats dnorm pnorm optim
#' @keywords internal
NULL

# likelihood-lognormal.R -- Log-normal parametric hazard likelihood, gradient, and optimizer
#
# MODEL
# -----
# Accelerated Failure Time (AFT) log-normal model:
#
#   log(T_i) ~ Normal(eta_i, sigma^2)   where eta_i = mu + x_i beta
#
#   z_i       = (log(t_i) - eta_i) / sigma          standardised residual
#   S(t | x)  = Phi(-z_i)                        survival  (normal CDF)
#   f(t | x)  = phi(z_i) / (sigma t_i)              density
#   h(t | x)  = phi(z_i) / (sigma t_i Phi(-z_i))     hazard
#   H(t | x)  = -log Phi(-z_i)                  cumulative hazard
#
# where mu in R (location), sigma > 0 (scale), eta_i = mu + x_i beta (AFT predictor).
#
# AFT vs non-AFT DISTINCTION
# --------------------------
# Unlike the non-AFT families -- Weibull / exponential (proportional hazards)
# and log-logistic (proportional odds), where covariates act on the hazard or
# odds scale -- the AFT parameterisation has covariates *shift the log-time
# distribution*.  The linear predictor adds to mu (location), not to log-hazard.
# Consequently:
#
#   * predict() "linear_predictor" returns beta (same interface as PH)
#   * predict() "survival" returns Phi(-z) directly (NOT exp(-H))
#   * predict() "cumulative_hazard" returns -log Phi(-z)
#
# See the EXCEPTION note in hazard_api.R's predict() section banner.
#
# THETA LAYOUT
# ------------
#   theta[1]   = mu         (location; unrestricted)
#   theta[2]   = log(sigma)   (unconstrained; sigma recovered via exp())
#   theta[3:p] = beta        (covariate AFT coefficients; unrestricted)
#
# GRADIENT (let w_i = phi(z_i)/Phi(-z_i), the inverse Mills ratio)
# ---------------------------------------------------------------
#   dL/dmu          = (1/sigma) * [sum(delta z) + sum((1-delta) w)]
#   dL/d(log sigma)    = sum(delta(z^2-1)) + sum((1-delta) w z)
#   dL/dbeta_j        = (1/sigma) * sum([delta z + (1-delta) w] * x_j)
#
# FUNCTIONS
# ---------
#   .hzr_logl_lognormal()     -- log-likelihood (optionally returning gradient)
#   .hzr_gradient_lognormal() -- analytical score vector
#   .hzr_optim_lognormal()    -- unconstrained BFGS wrapper

# Log-Normal Parametric Hazard Likelihood
#
# Evaluate the log-likelihood and its derivatives for log-normal hazard models.
# The log-normal is an AFT (accelerated failure time) model with separate
# location (mu) and scale (sigma) parameters.

#' Log-likelihood for log-normal hazard with covariates
#'
#' Computes the log-likelihood for right-censored data under the log-normal
#' parametric hazard model with optional linear-predictor covariates.
#'
#' @param theta Vector of parameters:
#'   theta\[1\] = mu (location, unrestricted)
#'   theta\[2\] = log(sigma) where sigma > 0 is the scale parameter
#'   theta\[3:length\]: Covariate coefficients (AFT parameterization: add to location)
#'
#' @param time Numeric vector of follow-up times (n)
#' @param status Numeric vector of event indicators: 1 = event, 0 = censored (n)
#' @param time_lower Optional numeric lower bound vector for interval-censored rows.
#'   Defaults to time if NULL.
#' @param time_upper Optional numeric upper bound vector for left/interval-censored rows.
#'   Defaults to time if NULL.
#' @param x Design matrix of covariates (n x p_coef); NULL for no covariates
#' @param return_gradient Logical; if TRUE, attach gradient vector as attribute
#'
#' @return Scalar log-likelihood value. If return_gradient = TRUE, gradient vector
#' is attached as attribute \code{"gradient"}.
#'
#' @details
#' The log-normal hazard model uses AFT parameterization:
#'
#' z_i = (log(t_i) - eta_i) / sigma, where eta_i = mu + x_i beta
#'
#' Survival function:
#' \deqn{S(t | x) = \Phi(-z_i)}
#'
#' Density:
#' \deqn{f(t | x) = \phi(z_i) / (\sigma t_i)}
#'
#' Hazard:
#' \deqn{h(t | x) = \phi(z_i) / (\sigma t_i \Phi(-z_i))}
#'
#' The log-likelihood for right-censored data is:
#' \deqn{\ell(\theta) = \sum_{\delta_i=1} [\log \phi(z_i) - \log \sigma - \log t_i]
#'   + \sum_{\delta_i=0} \log \Phi(-z_i)}
#'
#' Reparameterization: \u03b8\[1\] = \u03bc, \u03b8\[2\] = log(\u03c3) avoids constraints.
#'
#' Mixed censoring status coding:
#' - 1: exact event at time
#' - 0: right-censored at time
#' - -1: left-censored with upper bound time_upper \(or time\)
#' - 2: interval-censored in the interval \(time_lower, time_upper\)
#'
#' @noRd
.hzr_logl_lognormal <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    return_gradient = FALSE) {

  n <- length(time)

  # Default unit weights
  if (is.null(weights)) weights <- rep(1, n)

  # Extract parameters
  mu <- theta[1]          # Location (unrestricted)
  log_sigma <- theta[2]   # Log-scale (unrestricted)
  sigma <- exp(log_sigma) # Always positive

  # Covariate coefficients (if any) -- AFT parameterization: add to location
  if (!is.null(x)) {
    if (is.null(attr(x, "dimnames")[[2]])) {
      colnames(x) <- paste0("beta_", seq_len(ncol(x)))
    }
    beta_coef <- theta[3:length(theta)]
    eta <- mu + as.numeric(x %*% beta_coef)  # Total linear predictor (location)
  } else {
    eta <- rep(mu, n)
  }

  # Normalize censoring bounds to lower/upper vectors for unified formulas.
  lower <- if (is.null(time_lower)) time else time_lower
  upper <- if (is.null(time_upper)) time else time_upper

  if (any(status %in% c(1, 0) & time <= 0)) return(Inf)
  if (any(status == -1 & upper <= 0)) return(Inf)
  if (any(status == 2 & (lower <= 0 | upper <= 0))) return(Inf)
  if (any(status == 2) && any(lower[status == 2] >= upper[status == 2])) return(Inf)

  # Event-time terms are based on z at observed `time`.
  log_t <- log(time)
  z <- (log_t - eta) / sigma
  log_phi_z <- dnorm(z, log = TRUE)
  log_surv <- pnorm(-z, log.p = TRUE)

  idx_event <- status == 1
  idx_right <- status == 0
  idx_left <- status == -1
  idx_interval <- status == 2

  # Event: w * log f(t)
  ll_event <- if (any(idx_event)) {
    sum(weights[idx_event] *
          (log_phi_z[idx_event] - log_sigma - log_t[idx_event]))
  } else {
    0
  }

  # Right-censored: w * log S(t)
  ll_right <- if (any(idx_right)) {
    sum(weights[idx_right] * log_surv[idx_right])
  } else {
    0
  }

  # Left-censored: w * log F(u)
  ll_left <- if (any(idx_left)) {
    z_u <- (log(upper[idx_left]) - eta[idx_left]) / sigma
    sum(weights[idx_left] * pnorm(z_u, log.p = TRUE))
  } else {
    0
  }

  # Interval-censored: w * log(F(u) - F(l)) with positivity guard.
  ll_interval <- if (any(idx_interval)) {
    z_l <- (log(lower[idx_interval]) - eta[idx_interval]) / sigma
    z_u <- (log(upper[idx_interval]) - eta[idx_interval]) / sigma
    f_l <- pnorm(z_l)
    f_u <- pnorm(z_u)
    diff_f <- f_u - f_l
    if (any(diff_f <= 0)) return(Inf)
    sum(weights[idx_interval] * log(diff_f))
  } else {
    0
  }

  logl <- ll_event + ll_right + ll_left + ll_interval

  if (!is.finite(logl)) {
    return(Inf)
  }

  if (return_gradient) {
    grad <- .hzr_gradient_lognormal(
      theta = theta, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper, x = x,
      weights = weights,
      eta = eta, sigma = sigma, log_sigma = log_sigma,
      z = z, log_phi_z = log_phi_z, log_surv = log_surv
    )
    attr(logl, "gradient") <- grad
  }

  logl
}

#' Score vector (gradient of log-likelihood) for log-normal
#'
#' Computes the score vector of the log-normal log-likelihood w.r.t. all parameters.
#'
#' Let z_i = (log(t_i) - eta_i) / sigma, where eta_i = mu + x_i beta
#' Let w_i = phi(z_i) / Phi(-z_i) (inverse Mills ratio)
#'
#' Derivatives:
#' \eqn{dL/d\mu = (1/\sigma) * [\sum(\delta_i * z_i) + \sum((1 - \delta_i) * w_i)]}
#' \eqn{dL/d(\log \sigma) = \sum(\delta_i * (z_i^2 - 1)) + \sum((1 - \delta_i) * w_i * z_i)}
#' \eqn{dL/d\beta_j = (1/\sigma) * \sum([\delta_i * z_i + (1 - \delta_i) * w_i] * x_{ij})}
#'
#' @noRd
.hzr_gradient_lognormal <- function(
    theta,
    time,
    status,
    time_lower = NULL,
    time_upper = NULL,
    x = NULL,
    weights = NULL,
    eta = NULL,
    sigma = NULL,
    log_sigma = NULL,
    z = NULL,
    log_phi_z = NULL,
    log_surv = NULL) {

  n <- length(time)
  p <- length(theta)
  grad <- numeric(p)

  # Default unit weights -- keep gradient consistent with the LL when weights
  # are omitted.
  if (is.null(weights)) weights <- rep(1, n)

  # Mixed-censoring score uses numerical differentiation for robustness.
  if (any(status %in% c(-1, 2))) {
    return(.hzr_numeric_grad_lognormal(theta, time, status, time_lower,
                                        time_upper, x, weights = weights))
  }

  # Recompute if not provided
  if (is.null(z) || is.null(sigma)) {
    mu <- theta[1]
    log_sigma <- theta[2]
    sigma <- exp(log_sigma)

    if (!is.null(x)) {
      beta_coef <- theta[3:length(theta)]
      eta <- mu + as.numeric(x %*% beta_coef)
    } else {
      eta <- rep(mu, n)
    }

    log_t <- log(time)
    z <- (log_t - eta) / sigma
    log_phi_z <- dnorm(z, log = TRUE)
    log_surv <- pnorm(-z, log.p = TRUE)
  }

  # Inverse Mills ratio: mills_i = phi(z_i) / Phi(-z_i)
  # Computed in log scale for numerical stability, then exponentiating.
  log_mills <- log_phi_z - log_surv
  mills <- exp(log_mills)  # Safe: log_surv can be -Inf when Phi(-z) -> 0 (z -> +Inf)
  # Clamp to prevent Inf * 0 issues when z is very large.
  mills <- pmin(mills, 1e6)

  censored <- 1 - status  # (1 - delta_i)

  # Weighted per-row building blocks. Every term below is the unweighted form
  # with `delta`, `(1-delta)*mills` etc. multiplied by `weights`.
  w_status <- weights * status
  w_censmills <- weights * censored * mills

  # ===== Gradient w.r.t. mu =====
  # dL/dmu = (1/sigma) * sum(w * [delta * z + (1-delta) * mills])
  grad[1] <- (sum(w_status * z) + sum(w_censmills)) / sigma

  # ===== Gradient w.r.t. log(sigma) =====
  # dL/d(log sigma) = sum(w * delta * (z^2 - 1)) + sum(w * (1-delta) * mills * z)
  grad[2] <- sum(w_status * (z^2 - 1)) + sum(w_censmills * z)

  # ===== Gradient w.r.t. covariate beta =====
  # dL/dbeta_j = (1/sigma) * sum(w * [delta * z + (1-delta) * mills] * x_ij)
  if (p > 2 && !is.null(x)) {
    score_i <- (w_status * z + w_censmills) / sigma
    grad[3:p] <- as.numeric(crossprod(x, score_i))
  }

  grad
}

#' Analytic Hessian of the log-normal negative log-likelihood
#'
#' Returns the Hessian of the objective (negative log-likelihood) on the internal
#' \code{theta = (mu, log_sigma, beta_coef)} scale, matching
#' \code{numDeriv::hessian(objective)} so it can be used directly by
#' \code{.hzr_optim_generic()}.  Coverage mirrors the analytic gradient: event +
#' right-censored rows only; returns \code{NULL} for any left/interval-censored
#' row (\code{status \%in\% c(-1, 2)}) to request the numerical-Hessian fallback.
#'
#' With \code{z = (log t - eta)/sigma}, \code{eta = mu + x beta_coef}, and the
#' inverse Mills ratio \code{m = phi(z)/Phi(-z)} (\code{dm/dz = m(m - z)}), the
#' objective Hessian assembles per-row coefficients A (eta,eta), B (eta,log_sigma),
#' C (log_sigma,log_sigma) through the design \code{Xtilde = [1 | X]} for the
#' \code{(mu, beta_coef)} block.
#'
#' @noRd
.hzr_hessian_lognormal <- function(
    theta, time, status,
    time_lower = NULL, time_upper = NULL, x = NULL, weights = NULL) {

  n <- length(time)
  if (is.null(weights)) weights <- rep(1, n)

  # Coverage contract: closed form is event + right censored only.
  if (any(status %in% c(-1, 2))) {
    return(NULL)
  }

  mu <- theta[1]
  log_sigma <- theta[2]
  sigma <- exp(log_sigma)
  p <- length(theta)
  p_cov <- p - 2L
  # Fail loud on inconsistent theta/x: a silent has_cov = FALSE would zero the
  # covariate rows/cols and yield a conformant-but-wrong vcov the optimizer's
  # dimension check cannot catch.
  if (!is.null(x)) x <- as.matrix(x)
  n_x <- if (is.null(x)) 0L else NCOL(x)
  if (n_x != p_cov) {
    stop(".hzr_hessian_lognormal(): ncol(x) (", n_x, ") must equal the number ",
         "of covariate parameters in theta (", p_cov, ").", call. = FALSE)
  }
  has_cov <- p_cov > 0L
  beta_coef <- if (has_cov) theta[3:p] else numeric(0)
  eta <- if (has_cov) mu + as.numeric(x %*% beta_coef) else rep(mu, n)

  z <- (log(time) - eta) / sigma
  # Inverse Mills ratio on the log scale, clamped like the analytic gradient.
  m <- pmin(exp(dnorm(z, log = TRUE) - pnorm(-z, log.p = TRUE)), 1e6)

  delta <- as.numeric(status == 1)
  cens <- 1 - delta

  # Per-row objective Hessian coefficients (negation of the LL second derivs).
  a_coef <- (delta + cens * m * (m - z)) / sigma^2                 # (eta, eta)
  b_coef <- (delta * 2 * z + cens * m * (z * (m - z) + 1)) / sigma # (eta, log_sigma)
  c_coef <- delta * 2 * z^2 + cens * m * z * (z * (m - z) + 1)     # (log_sigma)^2

  x_tilde <- if (has_cov) cbind(1, x) else matrix(1, nrow = n, ncol = 1L)

  hqq <- crossprod(x_tilde, (weights * a_coef) * x_tilde)
  v <- as.numeric(crossprod(x_tilde, weights * b_coef))
  hss <- sum(weights * c_coef)

  # Assemble p x p with order (mu = 1, log_sigma = 2, beta_coef = 3..p).
  idx_q <- if (has_cov) c(1L, 3:p) else 1L
  hess <- matrix(0, p, p)
  hess[idx_q, idx_q] <- hqq
  hess[2L, idx_q] <- v
  hess[idx_q, 2L] <- v
  hess[2L, 2L] <- hss
  dimnames(hess) <- list(names(theta), names(theta))
  hess
}

#' @noRd
.hzr_optim_lognormal <- function(
    time, status, time_lower = NULL, time_upper = NULL,
    x = NULL, theta_start, weights = NULL, control = list()) {
  hessian_fn <- function(par) {
    .hzr_hessian_lognormal(
      theta = par, time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x, weights = weights
    )
  }
  .hzr_optim_generic(
    logl_fn = .hzr_logl_lognormal,
    gradient_fn = .hzr_gradient_lognormal,
    time = time, status = status,
    time_lower = time_lower, time_upper = time_upper,
    x = x, theta_start = theta_start, weights = weights,
    control = control, use_bounds = FALSE,
    hessian_fn = hessian_fn
  )
}

.hzr_numeric_grad_lognormal <- function(theta, time, status,
                                          time_lower = NULL, time_upper = NULL,
                                          x = NULL, weights = NULL) {
  # Reuse log-likelihood as scalar objective and differentiate numerically.
  objective <- function(par) {
    .hzr_logl_lognormal(
      theta = par,
      time = time,
      status = status,
      time_lower = time_lower,
      time_upper = time_upper,
      x = x,
      weights = weights,
      return_gradient = FALSE
    )
  }

  if (requireNamespace("numDeriv", quietly = TRUE)) {
    return(as.numeric(numDeriv::grad(objective, theta)))
  }

  eps <- 1e-6
  grad <- numeric(length(theta))
  for (i in seq_along(theta)) {
    tp <- theta
    tm <- theta
    tp[i] <- tp[i] + eps
    tm[i] <- tm[i] - eps
    grad[i] <- (objective(tp) - objective(tm)) / (2 * eps)
  }
  grad
}

test_that(".hzr_hessian_lognormal matches numDeriv (no covariates)", {
  skip_if_not_installed("numDeriv")
  set.seed(51)
  n <- 300
  time <- exp(rnorm(n, 0.2, 0.8))
  status <- rbinom(n, 1, 0.75)
  theta <- c(mu = 0.2, log_sigma = log(0.8))
  obj <- function(th) -.hzr_logl_lognormal(th, time, status)
  h_an <- .hzr_hessian_lognormal(theta, time, status)
  h_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(h_an), unname(h_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_lognormal matches numDeriv (covariates + weights)", {
  skip_if_not_installed("numDeriv")
  set.seed(52)
  n <- 400
  x <- cbind(a = rnorm(n), b = rbinom(n, 1, 0.4))
  eta <- 0.2 + as.numeric(x %*% c(0.3, -0.4))
  time <- exp(rnorm(n, eta, 0.7))
  status <- rbinom(n, 1, 0.8)
  w <- runif(n, 0.5, 2)
  theta <- c(mu = 0.2, log_sigma = log(0.7), 0.3, -0.4)
  obj <- function(th) -.hzr_logl_lognormal(th, time, status, x = x, weights = w)
  h_an <- .hzr_hessian_lognormal(theta, time, status, x = x, weights = w)
  h_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(h_an), unname(h_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_lognormal matches numDeriv (heavy right-censoring)", {
  skip_if_not_installed("numDeriv")
  set.seed(53)
  n <- 300
  time <- exp(rnorm(n, 0.0, 0.6))
  status <- rbinom(n, 1, 0.3)
  theta <- c(mu = 0.1, log_sigma = log(0.6))
  obj <- function(th) -.hzr_logl_lognormal(th, time, status)
  h_an <- .hzr_hessian_lognormal(theta, time, status)
  h_nd <- numDeriv::hessian(obj, theta)
  expect_equal(unname(h_an), unname(h_nd), tolerance = 1e-4)
})

test_that(".hzr_hessian_lognormal returns NULL for left/interval censoring", {
  set.seed(54)
  n <- 40
  time <- exp(rnorm(n, 0, 0.5))
  status <- rep(1L, n)
  status[1:5] <- 2L
  expect_null(.hzr_hessian_lognormal(
    c(mu = 0, log_sigma = 0), time, status,
    time_lower = time, time_upper = time + 0.5))
  status2 <- rep(1L, n)
  status2[1:5] <- -1L
  expect_null(.hzr_hessian_lognormal(
    c(mu = 0, log_sigma = 0), time, status2, time_upper = time + 0.5))
})

test_that("lognormal fit vcov uses the analytic Hessian (matches numDeriv)", {
  skip_if_not_installed("numDeriv")
  set.seed(55)
  n <- 500
  z <- rnorm(n)
  time <- exp(rnorm(n, 0.2 + 0.5 * z, 0.7))
  status <- rbinom(n, 1, 0.85)
  fit <- hazard(
    survival::Surv(time, status) ~ z,
    data = data.frame(time, status, z),
    dist = "lognormal",
    theta = c(mu = 0, log_sigma = 0, z = 0), fit = TRUE
  )
  obj <- function(th) -.hzr_logl_lognormal(th, time, status, x = cbind(z))
  v_nd <- solve(numDeriv::hessian(obj, fit$fit$theta))
  expect_equal(unname(fit$fit$vcov), unname(v_nd), tolerance = 1e-4)
  expect_true(isTRUE(fit$fit$pd))
})

test_that("lognormal SEs are invariant to covariate rescaling", {
  set.seed(56)
  n <- 600
  z <- rnorm(n)
  time <- exp(rnorm(n, 0.1 + 0.4 * z, 0.6))
  status <- rbinom(n, 1, 0.9)
  f1 <- hazard(survival::Surv(time, status) ~ z,
               data = data.frame(time, status, z = z),
               dist = "lognormal",
               theta = c(mu = 0, log_sigma = 0, z = 0), fit = TRUE)
  f2 <- hazard(survival::Surv(time, status) ~ z,
               data = data.frame(time, status, z = z / 100),
               dist = "lognormal",
               theta = c(mu = 0, log_sigma = 0, z = 0), fit = TRUE)
  se1 <- sqrt(diag(f1$fit$vcov))
  se2 <- sqrt(diag(f2$fit$vcov))
  expect_equal(unname(se1[1]), unname(se2[1]), tolerance = 1e-2)
  expect_equal(unname(se1[2]), unname(se2[2]), tolerance = 1e-2)
})

test_that(".hzr_hessian_lognormal errors on inconsistent theta/x", {
  set.seed(57)
  n <- 30
  time <- exp(rnorm(n, 0, 0.5))
  status <- rbinom(n, 1, 0.7)
  # theta declares one covariate but x is NULL.
  expect_error(
    .hzr_hessian_lognormal(c(mu = 0, log_sigma = 0, z = 0.3), time, status, x = NULL),
    "ncol")
  # x supplied with the wrong number of columns.
  x2 <- cbind(a = rnorm(n), b = rnorm(n))
  expect_error(
    .hzr_hessian_lognormal(c(mu = 0, log_sigma = 0, z = 0.3), time, status, x = x2),
    "ncol")
})

# Shared parser for SAS HAZARD .lst capture files.  testthat auto-sources
# this helper before running test-sas-parity.R.
#
# Reference fixtures live at:
#   ~/Documents/GitHub/hazard/examples/*.lst   (paired with *.sas)
#
# Capture vintage: v4.4 macOS (2026-04-27).  Final values refresh when
# v4.4.6 captures land — the parser itself is version-stable.
#
# Encoding notes (verified across all 18 fixtures):
#   - CRLF line endings  ->  strip \r
#   - SAS form-feed page breaks  ->  convert \f to \n
#   - "Non-ISO extended-ASCII" classification causes grep -c to silently
#     return 0 without -a; R's readLines handles this correctly when the
#     normalized text is split on \n.
#
# Public API:
#   .hzr_parse_sas_lst(path) -> list(fits = list(<one per Final Results:>))
#
#   Each fit element:
#     loglik        scalar, final "Log likelihood = N" before "Final Results:"
#     n_obs         integer
#     n_events      integer
#     n_censored    integer
#     params        data.frame(phase, name, label, estimate, se, z, p)
#                   from "Parameter Estimate Summary"
#     natural       data.frame(phase, name, fixed, estimate)
#                   from "Estimates for Model Parameters"
#     vcov          numeric matrix (NULL if NOCOV)
#     correlation   numeric matrix (NULL if NOCOR)
#     events_conserved  scalar or NA

.hzr_read_lst <- function(path) {
  raw <- readLines(path, warn = FALSE, encoding = "latin1")
  # Convert form-feed page breaks to newlines, then re-split.
  joined <- paste(raw, collapse = "\n")
  joined <- gsub("\f", "\n", joined, fixed = TRUE)
  joined <- gsub("\r", "",  joined, fixed = TRUE)
  strsplit(joined, "\n", fixed = TRUE)[[1]]
}

.hzr_extract_loglik <- function(lines) {
  m <- regmatches(
    lines,
    regexec("Log likelihood\\s*=\\s*(-?[0-9.Ee+-]+)", lines)
  )
  vals <- vapply(
    m,
    function(x) if (length(x) >= 2) as.numeric(x[2]) else NA_real_,
    numeric(1)
  )
  vals[!is.na(vals)]
}

.hzr_extract_obs_counts <- function(lines) {
  # "There are  5880 observations available for analysis with:"
  # "          545 events"
  # "         5335 Right Censored Observations"
  obs_line <- grep("observations available for analysis", lines, value = TRUE)[1]
  ev_line  <- grep("\\bevents\\s*$",                       lines, value = TRUE)[1]
  rc_line  <- grep("Right Censored Observations",          lines, value = TRUE)[1]

  pull <- function(pat, s) {
    if (is.na(s)) return(NA_integer_)
    m <- regmatches(s, regexec(pat, s))[[1]]
    if (length(m) >= 2) as.integer(m[2]) else NA_integer_
  }

  list(
    n_obs      = pull("There are\\s+([0-9]+)\\s+observations",          obs_line),
    n_events   = pull("([0-9]+)\\s+events",                              ev_line),
    n_censored = pull("([0-9]+)\\s+Right Censored Observations",         rc_line)
  )
}

.hzr_extract_events_conserved <- function(lines) {
  s <- grep("Number of events conserved", lines, value = TRUE)
  if (!length(s)) return(NA_real_)
  m <- regmatches(s[1], regexec("=\\s*(-?[0-9.Ee+-]+)", s[1]))[[1]]
  if (length(m) >= 2) as.numeric(m[2]) else NA_real_
}

# Parameter Estimate Summary table.  Layout:
#
#   Phase      Parameter   Label              Estimate    Std error      Z        Prob>|Z|
#   ---------- ...
#   Early:     E0                              -3.77955    0.09381214   -40.288   <.0001
#              ---------------- ...
#   Constant:  C0                              -7.2258     0.09312647   -77.591   <.0001
#
# - Phase label appears only on the first row of each phase; subsequent
#   rows in the same phase have a blank Phase column.
# - Inner "---" separator rows divide phases.
# - Outer "===" / long "---" rule bounds the table.
.hzr_extract_param_summary <- function(lines) {
  start <- grep("Parameter Estimate Summary", lines)
  if (!length(start)) return(NULL)
  start <- start[1]

  # Find the header row (starts with "Phase").
  hdr <- which(grepl("^\\s*Phase\\s+Parameter", lines))
  hdr <- hdr[hdr > start][1]
  if (is.na(hdr)) return(NULL)

  # Body starts after the rule line under the header.
  body_start <- hdr + 2L

  # Body ends at the next blank-rule (long dashes) followed by blank lines
  # before "Estimates for Model Parameters".
  end_marker <- grep("Estimates for Model Parameters", lines)
  end_marker <- end_marker[end_marker > hdr][1]
  if (is.na(end_marker)) return(NULL)

  block <- lines[body_start:(end_marker - 1L)]
  block <- block[nzchar(trimws(block))]
  # Drop separator rows (only dashes / spaces).
  block <- block[!grepl("^[\\s-]+$", block, perl = TRUE)]

  current_phase <- NA_character_
  rows <- list()

  for (ln in block) {
    # Phase label appears at the very start of the line, ending in ":".
    m_phase <- regmatches(ln, regexec("^\\s+(Early|Constant|Late):", ln))[[1]]
    if (length(m_phase) >= 2) {
      current_phase <- m_phase[2]
      ln <- sub("^\\s*(Early|Constant|Late):\\s*", "", ln)
    } else {
      # Continuation row — strip leading whitespace.
      ln <- sub("^\\s+", "", ln)
    }

    # Tail of each row: estimate, se, z, p.  Use a trailing-fields regex
    # so we capture them regardless of label column width.
    # Trailing fields: estimate, se, z, p.  p-value can be "<.0001" or a
    # decimal like "0.0032" / ".0032".
    tail_re <- "(-?[0-9.Ee+-]+)\\s+(-?[0-9.Ee+-]+)\\s+(-?[0-9.Ee+-]+)\\s+(<\\.?[0-9]+|[0-9.]+)\\s*$"
    m_tail <- regmatches(ln, regexec(tail_re, ln))[[1]]
    if (length(m_tail) < 5) next

    # Head: parameter name and optional label.  Strip the matched tail.
    head <- sub(tail_re, "", ln)
    head <- trimws(head)
    # First token is the parameter name; remainder is the label.
    head_split <- regmatches(head, regexec("^(\\S+)\\s*(.*)$", head))[[1]]
    name  <- if (length(head_split) >= 2) head_split[2] else NA_character_
    label <- if (length(head_split) >= 3) trimws(head_split[3]) else ""

    p_str <- m_tail[5]
    p_val <- if (grepl("^<", p_str)) {
      as.numeric(sub("^<", "", p_str))
    } else {
      as.numeric(p_str)
    }

    rows[[length(rows) + 1L]] <- data.frame(
      phase    = current_phase,
      name     = name,
      label    = label,
      estimate = as.numeric(m_tail[2]),
      se       = as.numeric(m_tail[3]),
      z        = as.numeric(m_tail[4]),
      p        = p_val,
      stringsAsFactors = FALSE
    )
  }

  if (!length(rows)) return(NULL)
  do.call(rbind, rows)
}

# Estimates for Model Parameters table — natural-scale values, all params
# (free + fixed).  Layout:
#
#   Phase       Parameter     Fixed?     Estimate
#   Early:      DELTA         Yes                  0
#               THALF         Yes                0.2
#                (RHO                            0.2)
#               ...
#               MUE                       0.02283304
.hzr_extract_natural <- function(lines) {
  start <- grep("Estimates for Model Parameters", lines)
  if (!length(start)) return(NULL)
  start <- start[1]

  hdr <- which(grepl("^\\s*Phase\\s+Parameter\\s+Fixed\\?", lines))
  hdr <- hdr[hdr > start][1]
  if (is.na(hdr)) return(NULL)

  body_start <- hdr + 2L

  end_marker <- grep("Asymptotic Variance-Covariance Matrix|Asymptotic Correlation Matrix",
                     lines)
  end_marker <- end_marker[end_marker > hdr][1]
  end_idx <- if (is.na(end_marker)) length(lines) else end_marker - 1L

  block <- lines[body_start:end_idx]
  block <- block[nzchar(trimws(block))]
  block <- block[!grepl("^[\\s-]+$", block, perl = TRUE)]

  current_phase <- NA_character_
  rows <- list()
  for (ln in block) {
    # Skip RHO informational rows: " (RHO                            0.2)"
    if (grepl("^\\s*\\(RHO", ln)) next

    # Natural-scale table is deeply indented (~40+ leading spaces); accept
    # arbitrary leading whitespace before the phase label.
    m_phase <- regmatches(ln, regexec("^\\s+(Early|Constant|Late):", ln))[[1]]
    if (length(m_phase) >= 2) {
      current_phase <- m_phase[2]
      ln <- sub("^\\s*(Early|Constant|Late):\\s*", "", ln)
    } else {
      ln <- sub("^\\s+", "", ln)
    }

    # Tail: optional "Yes"/"No", then numeric estimate.
    tail_re <- "^(\\S+)\\s+(?:(Yes|No)\\s+)?(-?[0-9.Ee+-]+)\\s*$"
    m <- regmatches(ln, regexec(tail_re, ln, perl = TRUE))[[1]]
    if (length(m) < 4) next

    rows[[length(rows) + 1L]] <- data.frame(
      phase    = current_phase,
      name     = m[2],
      fixed    = identical(m[3], "Yes"),
      estimate = as.numeric(m[4]),
      stringsAsFactors = FALSE
    )
  }
  if (!length(rows)) return(NULL)
  do.call(rbind, rows)
}

# Symmetric matrix parser shared by vcov and correlation blocks.
#
# Wide matrices (>~8 params) wrap both the header row and each data row
# across multiple lines.  Pages are also broken by form-feed inserting a
# title block, so we collect column names from any line of header-shape
# until a separator rule appears, and we accumulate numerics from
# continuation lines until the next labelled row or section.
.hzr_extract_matrix <- function(lines, marker) {
  start <- grep(marker, lines)
  if (!length(start)) return(NULL)
  start <- start[1]

  # Walk forward to collect the (possibly multi-line) header.  Header lines
  # contain only parameter names (identifiers) separated by 2+ spaces;
  # they terminate at the rule line of dashes.
  i <- start + 1L
  col_names <- character()
  saw_header <- FALSE
  while (i <= length(lines)) {
    ln <- lines[i]
    if (grepl("^\\s*-{10,}", ln)) {
      if (saw_header) break
    } else if (grepl("^\\s+[A-Za-z_][A-Za-z0-9_]*(\\s{2,}[A-Za-z_][A-Za-z0-9_]*)+\\s*$", ln)) {
      col_names <- c(col_names, strsplit(trimws(ln), "\\s{2,}")[[1]])
      saw_header <- TRUE
    } else if (saw_header && !nzchar(trimws(ln))) {
      # blank line inside header — keep going
    } else if (saw_header) {
      break
    }
    i <- i + 1L
  }
  if (!length(col_names)) return(NULL)
  k <- length(col_names)

  body <- lines[i:length(lines)]
  stop_pat <- "Asymptotic Correlation Matrix|Final Results|Initial Summary|Estimates for Model"
  if (!grepl("Correlation", marker)) {
    # vcov block: stop at the correlation block.
  } else {
    stop_pat <- "Final Results|Initial Summary|Estimates for Model"
  }
  stop_idx <- grep(stop_pat, body)
  if (length(stop_idx)) body <- body[seq_len(stop_idx[1] - 1L)]

  M <- matrix(NA_real_, k, k)
  # Use unique row/col names so duplicate parameters (e.g. STATUS appearing
  # in both Early and Constant phases) get distinct slots.  Track phase
  # context in the body so callers can resolve "Early:STATUS" vs
  # "Constant:STATUS" via the rownames suffix.
  rownames(M) <- make.unique(col_names, sep = "__")
  colnames(M) <- rownames(M)

  # The body presents rows in the same order as header columns (verified
  # across all in-scope fixtures).  Track a single row cursor that
  # advances each time we see a new labelled row — this is robust to
  # duplicate parameter names because position, not name, drives the
  # routing.  Phase delimiters in the body act as sanity check only.
  row_cursor <- 0L
  pending <- list()
  current_row  <- NA_integer_
  current_name <- NA_character_
  current_phase <- NA_character_
  current_vals <- numeric()
  flush <- function() {
    if (!is.na(current_row) && length(current_vals)) {
      pending[[length(pending) + 1L]] <<- list(
        row = current_row, vals = current_vals,
        name = current_name, phase = current_phase
      )
    }
    current_row  <<- NA_integer_
    current_name <<- NA_character_
    current_vals <<- numeric()
  }

  for (ln in body) {
    # Skip rule lines.
    if (grepl("^\\s*-{10,}", ln)) next
    # Phase-label rows update phase context; don't reset row tracking.
    m_phase <- regmatches(ln, regexec("^\\s+(Early|Constant|Late):\\s*$", ln))[[1]]
    if (length(m_phase) >= 2) {
      current_phase <- m_phase[2]
      next
    }
    # Skip page-break title lines from form-feed inserts.
    if (!nzchar(trimws(ln))) next

    # New labelled row: leading identifier followed by numerics.
    toks <- regmatches(ln, regexec("^\\s+([A-Za-z_][A-Za-z0-9_]*)\\s+(.+)$", ln))[[1]]
    # Only treat as a real row if the identifier is one of the matrix's
    # parameter names.  Form-feed page breaks inject title-block lines
    # like "Primary Valve Operations ..." or "Multivariable Analysis ..."
    # that start with an identifier but aren't rows; they must not
    # advance the cursor or be parsed as data.
    is_real_row <- length(toks) >= 3 &&
      !grepl("^[+-]?[0-9.]", toks[2]) &&
      toks[2] %in% col_names
    if (is_real_row) {
      flush()
      rn <- toks[2]
      # Cursor-based routing handles duplicate parameter names (e.g.
      # STATUS or AGE_COP appearing in both Early and Constant phases).
      # Sanity-check that col_names[cursor] matches; otherwise pick the
      # next at-or-after-cursor match to keep position-based routing.
      row_cursor <- row_cursor + 1L
      if (row_cursor <= k && identical(col_names[row_cursor], rn)) {
        current_row <- row_cursor
      } else {
        candidates <- which(col_names == rn)
        forward <- candidates[candidates >= row_cursor]
        current_row <- if (length(forward)) forward[1] else
                       if (length(candidates)) candidates[1] else NA_integer_
        if (!is.na(current_row)) row_cursor <- current_row
      }
      current_name <- rn
      if (!is.na(current_row)) {
        rest <- toks[3]
        nums <- suppressWarnings(as.numeric(
          regmatches(rest, gregexpr("-?[0-9.]+(?:[Ee][+-]?[0-9]+)?", rest, perl = TRUE))[[1]]
        ))
        current_vals <- nums[!is.na(nums)]
      }
    } else if (!is.na(current_row) && length(toks) < 3) {
      # Continuation line: pure numerics, no leading identifier.  Title
      # lines (Primary, Multivariable, ...) have identifiers and we
      # explicitly skip them here by requiring the leading-identifier
      # regex to have *not* matched.
      nums <- suppressWarnings(as.numeric(
        regmatches(ln, gregexpr("-?[0-9.]+(?:[Ee][+-]?[0-9]+)?", ln, perl = TRUE))[[1]]
      ))
      current_vals <- c(current_vals, nums[!is.na(nums)])
    }
  }
  flush()

  for (p in pending) {
    n <- min(length(p$vals), k)
    if (n > 0L) M[p$row, seq_len(n)] <- p$vals[seq_len(n)]
  }

  for (i2 in seq_len(k)) for (j2 in seq_len(k)) {
    if (is.na(M[i2, j2]) && !is.na(M[j2, i2])) M[i2, j2] <- M[j2, i2]
  }
  M
}

# Split the file into per-fit blocks on "Initial Summary:" anchors.
.hzr_split_fits <- function(lines) {
  anchors <- grep("Initial Summary:", lines)
  if (!length(anchors)) return(list(lines))
  ends <- c(tail(anchors, -1L) - 1L, length(lines))
  mapply(function(s, e) lines[s:e], anchors, ends, SIMPLIFY = FALSE)
}

.hzr_parse_sas_lst <- function(path) {
  lines <- .hzr_read_lst(path)
  fits  <- .hzr_split_fits(lines)
  parsed <- lapply(fits, function(block) {
    obs <- .hzr_extract_obs_counts(block)
    ll  <- .hzr_extract_loglik(block)
    list(
      loglik            = if (length(ll)) ll[length(ll)] else NA_real_,
      loglik_trace      = ll,
      n_obs             = obs$n_obs,
      n_events          = obs$n_events,
      n_censored        = obs$n_censored,
      events_conserved  = .hzr_extract_events_conserved(block),
      params            = .hzr_extract_param_summary(block),
      natural           = .hzr_extract_natural(block),
      vcov              = .hzr_extract_matrix(block, "Asymptotic Variance-Covariance Matrix"),
      correlation       = .hzr_extract_matrix(block, "Asymptotic Correlation Matrix")
    )
  })
  list(fits = parsed)
}

# Parse the %DECILES goodness-of-fit table printed by the SAS deciles.hazard
# macro. The block is introduced by a spaced-out title "D E C I L E ..." and
# has one row per group: the leading token is the _DECILE_ index ("." for the
# overall row, then 0..groups-1), followed by numeric columns whose first four
# are CASES, EXPECTED (sum of cumulative hazard), CUM_HAZ (mean), ACTUAL
# (observed events). Returns a data frame with columns decile/cases/expected/
# actual (NA decile = overall row), or NULL if no decile block is present.
.hzr_parse_sas_deciles <- function(path) {
  lines <- .hzr_read_lst(path)
  start <- grep("D E C I L E   A N A L Y S I S", lines)
  if (!length(start)) return(NULL)
  body <- lines[start[1]:length(lines)]

  rows <- list()
  started <- FALSE
  num_only <- "^[0-9eE+.[:space:]-]+$"
  for (ln in body) {
    if (!nzchar(trimws(ln))) next
    m <- regmatches(ln, regexec("^[[:space:]]*([.]|[0-9]+)[[:space:]]+(.+)$", ln))[[1]]
    is_row <- length(m) >= 3 && grepl(num_only, m[3])
    if (is_row) {
      nums <- suppressWarnings(as.numeric(strsplit(trimws(m[3]),
                                                   "[[:space:]]+")[[1]]))
      nums <- nums[!is.na(nums)]
      if (length(nums) >= 4) {
        rows[[length(rows) + 1L]] <- data.frame(
          decile   = if (m[2] == ".") NA_integer_ else as.integer(m[2]),
          cases    = nums[1],
          expected = nums[2],
          actual   = nums[4]
        )
        started <- TRUE
        next
      }
    }
    # A non-blank, non-data line after the table has started ends the block.
    if (started) break
  }
  if (!length(rows)) return(NULL)
  do.call(rbind, rows)
}

# Parse a SAS %KAPLAN / %NELSONT life-table block. These are printed by the
# kaplan.sas / nelsont.sas actuarial macros as a wide table with a header line
#   Obs INT_DEAD NUMBER CENSORED DEAD CUM_SURV SE_EXACT CL_LOWER ... PROPLIFE
# (KAPLAN) or a slightly different column set (NELSONT adds NUMRISK, drops
# SE_EXACT). Header-driven so it tolerates either layout. `which` selects the
# block: "kaplan" (first block with an SE_EXACT column), "nelson" (block with a
# NUMRISK column), or a `_CATG=<x>` stratum marker like "catg0"/"catg1".
# Returns a data frame named by the header columns (leading Obs dropped, "."
# -> NA), or NULL if the block is absent.
.hzr_parse_sas_lifetable <- function(path, which = c("kaplan", "nelson",
                                                     "catg0", "catg1")) {
  which <- match.arg(which)
  lines <- .hzr_read_lst(path)
  headers <- grep("^\\s*Obs\\s+INT_DEAD\\s+NUMBER", lines)
  if (!length(headers)) return(NULL)

  pick <- NULL
  if (which == "nelson") {
    pick <- headers[grepl("NUMRISK", lines[headers])][1]
  } else if (which == "kaplan") {
    # First KAPLAN-style header (has SE_EXACT, no NUMRISK).
    cand <- headers[grepl("SE_EXACT", lines[headers]) &
                      !grepl("NUMRISK", lines[headers])]
    pick <- cand[1]
  } else {
    # Stratum: the header following the matching _CATG=<n> rule line.
    n <- sub("catg", "", which)
    catg <- grep(paste0("_CATG=", n, "\\b"), lines)
    if (length(catg)) {
      after <- headers[headers > catg[1]]
      pick <- after[1]
    }
  }
  if (is.null(pick) || is.na(pick)) return(NULL)

  cols <- strsplit(trimws(lines[pick]), "\\s+")[[1]]
  rows <- list()
  for (ln in lines[(pick + 1L):length(lines)]) {
    if (grepl("^\\s*-{5,}", ln)) break          # summary rule ends the block
    if (!nzchar(trimws(ln))) next
    toks <- strsplit(trimws(ln), "\\s+")[[1]]
    # Data rows lead with the integer Obs counter.
    if (!grepl("^[0-9]+$", toks[1])) {
      if (length(rows)) break else next
    }
    if (length(toks) < length(cols)) next
    vals <- toks[seq_along(cols)]
    vals[vals == "."] <- NA
    rows[[length(rows) + 1L]] <- as.numeric(vals)
  }
  if (!length(rows)) return(NULL)
  df <- as.data.frame(do.call(rbind, rows))
  names(df) <- cols
  df[["Obs"]] <- NULL
  df
}

# Parse the HAZPRED "digital nomogram" prediction table printed by jobs like
# hp.death.AVC.sas: a PROC PRINT with header
#   Obs YEARS MONTHS _SURVIV _CLLSURV _CLUSURV _HAZARD _CLLHAZ _CLUHAZ
# Returns a data frame (Obs dropped) with the leading underscores stripped from
# the column names: YEARS, MONTHS, SURVIV, CLLSURV, CLUSURV, HAZARD, CLLHAZ,
# CLUHAZ. NULL if the table is absent.
.hzr_parse_sas_nomogram <- function(path) {
  lines <- .hzr_read_lst(path)
  h <- grep("YEARS[[:space:]]+MONTHS[[:space:]]+_SURVIV", lines)
  if (!length(h)) return(NULL)
  cols <- c("YEARS", "MONTHS", "SURVIV", "CLLSURV", "CLUSURV",
            "HAZARD", "CLLHAZ", "CLUHAZ")
  rows <- list()
  for (ln in lines[(h[1] + 1L):length(lines)]) {
    if (!nzchar(trimws(ln))) next
    toks <- strsplit(trimws(ln), "[[:space:]]+")[[1]]
    if (!grepl("^[0-9]+$", toks[1])) {
      if (length(rows)) break else next
    }
    if (length(toks) < 9L) next
    rows[[length(rows) + 1L]] <- as.numeric(toks[2:9])
  }
  if (!length(rows)) return(NULL)
  df <- as.data.frame(do.call(rbind, rows))
  names(df) <- cols
  df
}

# Parse the SAS %HAZBOOT bootstrap output printed by bs.death.AVC.sas: one
# per-phase coefficient table (Early/Constant/Late) where each row is one
# bootstrap resample's *selected* model -- a "." marks a covariate that was
# NOT selected in that resample.  Header looks like
#   Obs  E0  AGE STATUS COM_IV OPMOS OP_AGE RESAMPL ORIFICE MAL INC_SURG
# (intercept E0/C0/L0 identifies the phase: early/constant/late).
#
# Returns a named list with one data frame per present phase (covariate
# coefficients, NA where unselected; plus the RESAMPL id), and a
# `selection_freq` attribute: per-phase named integer vector counting, over
# the resamples, how often each covariate was selected.  NULL if no bootstrap
# table is present.  A phase whose covariates are all unselected (e.g. the
# placeholder Late table of a 2-phase fit) is dropped.
.hzr_parse_sas_bootstrap <- function(path) {
  lines <- .hzr_read_lst(path)
  phase_for <- c(E0 = "early", C0 = "constant", L0 = "late")

  parse_phase <- function(intercept) {
    h <- grep(paste0("Obs[[:space:]]+", intercept, "\\b.*RESAMPL"), lines)
    if (!length(h)) return(NULL)
    cols <- strsplit(trimws(lines[h[1]]), "[[:space:]]+")[[1]]
    rows <- list()
    for (ln in lines[(h[1] + 1L):length(lines)]) {
      if (!nzchar(trimws(ln))) next        # tolerate blank/pagination rows
      toks <- strsplit(trimws(ln), "[[:space:]]+")[[1]]
      if (!grepl("^[0-9]+$", toks[1])) {
        if (length(rows)) break else next
      }
      if (length(toks) < length(cols)) next
      vals <- toks[seq_along(cols)]
      vals[vals == "."] <- NA
      rows[[length(rows) + 1L]] <- suppressWarnings(as.numeric(vals))
    }
    if (!length(rows)) return(NULL)
    df <- as.data.frame(do.call(rbind, rows))
    names(df) <- cols
    df[["Obs"]] <- NULL
    df
  }

  out <- list()
  freq <- list()
  for (ic in names(phase_for)) {
    df <- parse_phase(ic)
    if (is.null(df)) next
    covs <- setdiff(names(df), c(ic, "RESAMPL"))
    f <- vapply(df[covs], function(x) sum(!is.na(x)), integer(1))
    if (sum(f) == 0L) next                     # drop empty placeholder phase
    out[[phase_for[[ic]]]] <- df
    freq[[phase_for[[ic]]]] <- f
  }
  if (!length(out)) return(NULL)
  attr(out, "selection_freq") <- freq
  out
}

# Parse the multivariable HAZPRED "digital nomogram" printed by the
# patient-specific prediction jobs (hp.death.AVC.hm1 / hm2).  Unlike the null
# model nomogram above, these PROC PRINT tables carry the full covariate
# profile on every row and are printed BY a grouping variable (MAL or COM_IV),
# so each row is a self-contained (time + covariates -> predictions) record:
#   Obs MONTHS YEARS OPYEAR OPMOS AGE COM_IV MAL INC_SURG ORIFICE STATUS
#       _SURVIV _CLLSURV _CLUSURV _HAZARD _CLLHAZ _CLUHAZ
# Header-driven (reads column names from the printed header), so it tolerates
# layout differences between fixtures.  Parsing is scoped to the data block
# immediately following each matching header line: within a block, data rows
# lead with the integer Obs counter; the block ends at the first non-data line
# after data have started (so unrelated `Obs ...` tables elsewhere in the
# listing are never ingested).  Page breaks / BY-groups repeat the header, and
# all their blocks are concatenated.  Returns a data frame with the leading "_"
# stripped from the prediction columns and Obs dropped; NULL if absent.
.hzr_parse_sas_nomogram_mv <- function(path) {
  lines <- .hzr_read_lst(path)
  h <- grep("MONTHS[[:space:]].*_SURVIV", lines)
  if (!length(h)) return(NULL)
  cols <- strsplit(trimws(lines[h[1]]), "[[:space:]]+")[[1]]
  ncol <- length(cols)
  rows <- list()
  for (hi in h) {
    if (hi >= length(lines)) next
    started <- FALSE
    for (ln in lines[(hi + 1L):length(lines)]) {
      if (!nzchar(trimws(ln))) next                 # tolerate blank spacer rows
      toks <- strsplit(trimws(ln), "[[:space:]]+")[[1]]
      if (!grepl("^[0-9]+$", toks[1])) {            # data rows lead with Obs
        if (started) break else next                # block ends after data
      }
      if (length(toks) < ncol) next
      vals <- toks[seq_len(ncol)]
      vals[vals == "."] <- NA
      rows[[length(rows) + 1L]] <- suppressWarnings(as.numeric(vals))
      started <- TRUE
    }
  }
  if (!length(rows)) return(NULL)
  df <- as.data.frame(do.call(rbind, rows))
  names(df) <- sub("^_", "", cols)
  df[["Obs"]] <- NULL
  df
}

# Fit the saved multivariable both-phase "HMDEATH" model used by the AVC
# patient-specific prediction fixtures (hm.death.AVC final model; consumed by
# hp.death.AVC.hm1 / hm2).  Early phase carries 6 covariates, Constant phase 3
# (STATUS appears in both).  Deterministic warm start from the .sas PARMS /
# coefficient statements (documented starting values, not the converged
# answer).  Returns the fitted hazard object.
.hzr_fit_avc_hmdeath <- function() {
  # Load via an explicit env + `$` so object_usage_linter sees a binding
  # (a bare `avc` from data() reads as an undefined global under lint_package).
  e <- new.env()
  utils::data("avc", package = "TemporalHazard", envir = e)
  d <- e$avc
  # SAS PROC STANDARD ... REPLACE fills missing INC_SURG with the column mean.
  d$inc_surg[is.na(d$inc_surg)] <- mean(d$inc_surg, na.rm = TRUE)
  start_thalf <- 0.1905077
  start_nu    <- 1.437416
  theta0 <- c(
    log(0.3504743), log(start_thalf), start_nu, 1,
    -0.03205774, 1.336675, 0.6872028, -0.01963377, 0.0002086689, 0.5169533,
    log(4.391673e-07), 1.375285, 3.11765, 1.054988
  )
  hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = d,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = start_thalf, nu = start_nu, m = 1,
                           fixed = "m",
                           formula = ~ age + com_iv + mal + opmos + op_age + status),
      constant = hzr_phase("constant", formula = ~ inc_surg + orifice + status)
    ),
    theta   = theta0,
    fit     = TRUE,
    control = list(n_starts = 1, maxit = 2000, conserve = TRUE)
  )
}

# Parse the stratified "Predict number of deaths" observed-vs-expected table
# printed by hs.death.AVC.hm1.sas: one row per stratum (PROC SUMMARY BY COM_IV)
# with the printed header
#   Obs STATUS INC_SURG OPMOS AGE MAL COM_IV ORIFICE DEAD INT_DEAD PROB
#       CUM_HAZ DEAD PEXPECT EXPECTED ACTUAL
# (note the duplicated DEAD column, so we read by position).  Returns a data
# frame with one row per stratum: com_iv, pexpect (Sum 1 - S), expected
# (Sum cumulative hazard), actual (observed deaths).  NULL if absent.
.hzr_parse_sas_calibration <- function(path) {
  lines <- .hzr_read_lst(path)
  h <- grep("COM_IV.*PEXPECT.*EXPECTED.*ACTUAL", lines)
  if (!length(h)) return(NULL)
  rows <- list()
  for (ln in lines[(h[1] + 1L):length(lines)]) {
    if (!nzchar(trimws(ln))) next                  # SAS double-spaces the rows
    toks <- strsplit(trimws(ln), "[[:space:]]+")[[1]]
    if (!grepl("^[0-9]+$", toks[1])) {
      if (length(rows)) break else next
    }
    if (length(toks) < 16L) next
    v <- suppressWarnings(as.numeric(toks[1:16]))
    rows[[length(rows) + 1L]] <- data.frame(
      com_iv   = v[7],
      pexpect  = v[14],
      expected = v[15],
      actual   = v[16]
    )
  }
  if (!length(rows)) return(NULL)
  do.call(rbind, rows)
}

# Parse the per-stratum mean-survival "digital" table printed by
# hs.death.AVC.hm1.sas: blocks delimited by "Interventricular communication=<g>"
# rule lines, each with header  Obs YEARS NSURVIV MSURVIV MCLLSURV MCLUSURV
# (NSURVIV = stratum size; MSURVIV/MCLLSURV/MCLUSURV = mean of per-subject
# survival and its CL across the stratum).  Returns a long data frame with a
# leading `com_iv` column; NULL if absent.
.hzr_parse_sas_strata_survival <- function(path) {
  lines <- .hzr_read_lst(path)
  rules <- grep("Interventricular communication=([0-9])", lines)
  if (!length(rules)) return(NULL)
  out <- list()
  for (r in rules) {
    g <- as.integer(sub(".*communication=([0-9]).*", "\\1", lines[r]))
    hdr <- grep("Obs[[:space:]]+YEARS[[:space:]]+NSURVIV", lines)
    hdr <- hdr[hdr > r][1]
    if (is.na(hdr)) next
    started <- FALSE
    for (ln in lines[(hdr + 1L):length(lines)]) {
      if (!nzchar(trimws(ln))) next                # tolerate blank spacer rows
      toks <- strsplit(trimws(ln), "[[:space:]]+")[[1]]
      if (!grepl("^[0-9]+$", toks[1])) {
        if (started) break else next
      }
      if (length(toks) < 6L) next
      started <- TRUE
      v <- suppressWarnings(as.numeric(toks[2:6]))
      out[[length(out) + 1L]] <- data.frame(
        com_iv = g, YEARS = v[1], NSURVIV = v[2],
        MSURVIV = v[3], MCLLSURV = v[4], MCLUSURV = v[5]
      )
    }
  }
  if (!length(out)) return(NULL)
  do.call(rbind, out)
}

# Default discovery: ~/Documents/GitHub/hazard/examples/ if it exists,
# else NULL.  Skip tests when fixtures are unavailable.
.hzr_sas_fixture_dir <- function() {
  env <- Sys.getenv("HAZARD_EXAMPLES_DIR", "")
  if (nzchar(env) && dir.exists(env)) return(env)
  default <- path.expand("~/Documents/GitHub/hazard/examples")
  if (dir.exists(default)) return(default)
  NA_character_
}

# ---------------------------------------------------------------------------
# OMC (PRIMISOL) dataset derivation
# ---------------------------------------------------------------------------
# The shipped `omc` R dataset has only the columns extracted at the source
# (study, te1, te2, te3, int_dead, dead, opdjul).  The OMC parity fixtures
# derive INT_TE / TE / STARTTME / CENSORED / NOPREVTE / MORBID via a
# multi-observation-per-patient DATA step that also needs te1djul/te2djul/
# te3djul, reop flags, fupdjul, and grade columns.  We reproduce that
# derivation here from the raw flat file.
#
# .hzr_omc_raw_path()     -- locate raw data file; returns NA_character_ if absent
# .hzr_derive_primisol()  -- parse + expand rows; returns list(te, te_mod, tm)

.hzr_omc_raw_path <- function() {
  env <- Sys.getenv("HAZARD_OMC_RAW", "")
  if (nzchar(env) && file.exists(env)) return(env)
  fixture_dir <- .hzr_sas_fixture_dir()
  if (!is.na(fixture_dir)) {
    cand <- file.path(fixture_dir, "data", "omc")
    if (file.exists(cand)) return(cand)
  }
  NA_character_
}

# Parse the SAS fixed-width raw file into a one-row-per-patient data frame.
# Column positions exactly match the SAS INPUT statement in hz.te123.OMC.sas.
.hzr_read_omc_raw <- function(path) {
  lines <- readLines(path, warn = FALSE)

  fld <- function(s, e = s) {
    v <- trimws(substr(lines, s, e))
    v[v == "."] <- NA_character_
    v
  }
  num <- function(s, e = s) suppressWarnings(as.numeric(fld(s, e)))
  chr <- function(s, e = s) fld(s, e)

  data.frame(
    study    = chr(1, 10),
    te1      = num(12),
    te2      = num(14),
    te3      = num(16),
    te1djul  = num(18, 25),
    te2djul  = num(27, 33),
    te3djul  = num(35, 41),
    avp_rp1  = chr(43),
    avp_rp2  = chr(45),
    avp_rp3  = chr(47),
    mvp_rp1  = chr(49),
    mvp_rp2  = chr(51),
    mvp_rp3  = chr(53),
    tvp_rp1  = chr(55),
    tvp_rp2  = chr(57),
    tvp_rp3  = chr(59),
    rp1djul  = num(61, 67),
    rp2djul  = num(69, 75),
    rp3djul  = num(77, 83),
    int_dead = num(111, 120),
    opdjul   = num(122, 128),
    dead     = num(130),
    fupdjul  = num(132, 138),
    te1grade = num(140),
    te2grade = num(142),
    te3grade = num(144),
    stringsAsFactors = FALSE
  )
}

# Reproduce the SAS PRIMISOL DATA step from hz.te123.OMC.sas and
# hz.tm123.OMC.sas.  Returns a list with three data frames:
#
#   te      -- for hz.te123.OMC fit 1 (left-truncated intervals, NOPREVTE)
#   te_mod  -- for hz.te123.OMC fit 2 (INT_TE adjusted to relative time,
#               + NOTE2 = NOPREVTE^2, NOTEE = exp(NOPREVTE))
#   tm      -- for hz.tm123.OMC    (same intervals, MORBID severity weight)
.hzr_derive_primisol <- function(raw_path) {
  d <- .hzr_read_omc_raw(raw_path)

  # IF TE1=1 AND TE1DJUL=. THEN DELETE
  d <- d[!(d$te1 == 1 & is.na(d$te1djul)), ]

  # Replace missing TE indicators with 0
  d$te1[is.na(d$te1)] <- 0L
  d$te2[is.na(d$te2)] <- 0L
  d$te3[is.na(d$te3)] <- 0L

  # Valve replacement: RP3 checked first, then RP2 overwrites, then RP1
  # overwrites (SAS IF statements execute in order 3->2->1; RP1 takes
  # precedence because it runs last and overwrites any earlier assignment).
  is_r <- function(x) !is.na(x) & trimws(x) == "R"
  n    <- nrow(d)
  REPL    <- rep(0L, n)
  REPLDJUL <- d$fupdjul   # default: censor at last follow-up

  for (rp_num in c(3L, 2L, 1L)) {
    avp <- is_r(d[[paste0("avp_rp", rp_num)]])
    mvp <- is_r(d[[paste0("mvp_rp", rp_num)]])
    tvp <- is_r(d[[paste0("tvp_rp", rp_num)]])
    idx <- avp | mvp | tvp
    REPL[idx]     <- 1L
    REPLDJUL[idx] <- d[[paste0("rp", rp_num, "djul")]][idx]
  }
  d$REPL    <- REPL
  d$REPLDJUL <- REPLDJUL

  # Censor TEs that occurred at or after valve replacement
  for (j in 1:3) {
    te_col   <- paste0("te",  j)
    djul_col <- paste0("te", j, "djul")
    mask <- d$REPL == 1L & d[[te_col]] == 1L &
            !is.na(d[[djul_col]]) & d[[djul_col]] >= d$REPLDJUL
    d[[te_col]][mask] <- 0L
  }

  # INT_REPL: time to valve replacement (or censoring if no replacement)
  d$INT_REPL <- d$int_dead
  repl_idx <- d$REPL == 1L
  d$INT_REPL[repl_idx] <-
    (d$REPLDJUL[repl_idx] - d$opdjul[repl_idx]) * 12 / 365.2425

  # -- Multi-row expansion -------------------------------------------------
  rows_te <- vector("list", n * 4L)   # pre-allocate generously
  rows_tm <- vector("list", n * 4L)
  nr_te <- 0L
  nr_tm <- 0L

  append_te <- function(study, te1, te2, te3, int_te, starttme, censored, te, noprevte) {
    nr_te <<- nr_te + 1L
    rows_te[[nr_te]] <<- data.frame(
      study = study, te1 = te1, te2 = te2, te3 = te3,
      int_te = int_te, starttme = starttme, censored = censored,
      te = te, noprevte = noprevte, stringsAsFactors = FALSE
    )
  }
  append_tm <- function(study, te1, te2, te3, int_te, starttme, censored, te, morbid) {
    nr_tm <<- nr_tm + 1L
    rows_tm[[nr_tm]] <<- data.frame(
      study = study, te1 = te1, te2 = te2, te3 = te3,
      int_te = int_te, starttme = starttme, censored = censored,
      te = te, morbid = morbid, stringsAsFactors = FALSE
    )
  }

  for (i in seq_len(nrow(d))) {
    r       <- d[i, ]
    study   <- trimws(r$study)
    te1_val <- r$te1
    te2_val <- r$te2
    te3_val <- r$te3

    # ---- Censored summary row (always output) ----------------------------
    # STARTTME advances through any events that preceded the censored interval.
    starttme <- 0
    noprevte <- 0L

    if (!is.na(te1_val) && te1_val == 1L) {
      starttme <- 12 * (r$te1djul - r$opdjul) / 365.2425
      noprevte <- 1L
    }
    # Special hospital-acquired TEs for two patients
    if (study == "039") starttme <- (3 / 24) * 12 / 365.2425
    if (study == "096") starttme <- (2 / 24) * 12 / 365.2425
    if (!is.na(te2_val) && te2_val == 1L) {
      starttme <- 12 * (r$te2djul - r$opdjul) / 365.2425
      noprevte <- 2L
    }
    if (!is.na(te3_val) && te3_val == 1L) {
      starttme <- 12 * (r$te3djul - r$opdjul) / 365.2425
      noprevte <- 3L
    }

    append_te(study, te1_val, te2_val, te3_val, r$INT_REPL, starttme, 1L, 0L, noprevte)
    append_tm(study, te1_val, te2_val, te3_val, r$INT_REPL, starttme, 1L, 0L, 0)

    # ---- TE1 event row ---------------------------------------------------
    if (!is.na(te1_val) && te1_val == 1L) {
      ite <- 12 * (r$te1djul - r$opdjul) / 365.2425
      if (study == "039") ite <- (3 / 24) * 12 / 365.2425
      if (study == "096") ite <- (2 / 24) * 12 / 365.2425
      append_te(study, te1_val, te2_val, te3_val, ite, 0, 0L, 1L, 0L)
      m1 <- if (is.na(r$te1grade)) 1.70 else r$te1grade
      append_tm(study, te1_val, te2_val, te3_val, ite, 0, 0L, 1L, m1)
    }

    # ---- TE2 event row ---------------------------------------------------
    if (!is.na(te2_val) && te2_val == 1L) {
      s2  <- 12 * (r$te1djul - r$opdjul) / 365.2425
      if (study == "039") s2 <- (3 / 24) * 12 / 365.2425
      if (study == "096") s2 <- (2 / 24) * 12 / 365.2425
      ite <- 12 * (r$te2djul - r$opdjul) / 365.2425
      append_te(study, te1_val, te2_val, te3_val, ite, s2, 0L, 1L, 1L)
      m2 <- if (is.na(r$te2grade)) 2.17 else r$te2grade
      append_tm(study, te1_val, te2_val, te3_val, ite, s2, 0L, 1L, m2)
    }

    # ---- TE3 event row ---------------------------------------------------
    if (!is.na(te3_val) && te3_val == 1L) {
      s2  <- 12 * (r$te2djul - r$opdjul) / 365.2425
      ite <- 12 * (r$te3djul - r$opdjul) / 365.2425
      append_te(study, te1_val, te2_val, te3_val, ite, s2, 0L, 1L, 2L)
      m3 <- if (is.na(r$te3grade)) 4.00 else r$te3grade
      append_tm(study, te1_val, te2_val, te3_val, ite, s2, 0L, 1L, m3)
    }
  }

  te <- do.call(rbind, rows_te[seq_len(nr_te)])
  tm <- do.call(rbind, rows_tm[seq_len(nr_tm)])

  # Drop zero/negative-duration rows (e.g. study 173: died same day as TE,
  # so the trailing censored row has STARTTME == INT_TE).  SAS HAZARD drops
  # these automatically when LCENSOR STARTTME is in effect.
  te <- te[te$int_te > te$starttme, ]
  tm <- tm[tm$int_te > tm$starttme, ]

  # hz.te123.OMC fit 2: modulated renewal — INT_TE relative to prior event
  te_mod <- te
  te_mod$int_te <- te_mod$int_te - te_mod$starttme
  te_mod$note2  <- te_mod$noprevte^2
  te_mod$notee  <- exp(te_mod$noprevte)
  # Drop any zero-duration modulated rows (relative interval would be 0)
  te_mod <- te_mod[te_mod$int_te > 0, ]

  list(te = te, te_mod = te_mod, tm = tm)
}

# SAS HAZARD parity tests
#
# Compare TemporalHazard::hazard() output to reference .lst captures from
# ~/Documents/GitHub/hazard/examples/.  Tests skip on CRAN and skip when
# the fixtures directory is unavailable.
#
# Reference parser lives in helper-sas-parity.R.
# Inventory + gap log: inst/dev/PRE-CRAN-PARITY-INVENTORY.md

skip_if_no_sas_fixtures <- function() {
  dir <- .hzr_sas_fixture_dir() # nolint: object_usage_linter.
  testthat::skip_if(is.na(dir), "SAS HAZARD fixture directory not available")
  dir
}

skip_if_no_omc_raw <- function() {
  p <- .hzr_omc_raw_path() # nolint: object_usage_linter.
  testthat::skip_if(is.na(p), "OMC raw data file not available")
  p
}

test_that("hz.deadp.KUL: 3-phase fixed-shape Weibull-tail matches SAS LL/MLEs", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  set.seed(101)  # deterministic multi-start; independent of test order

  ref <- .hzr_parse_sas_lst(file.path(dir, "hz.deadp.KUL.lst"))$fits[[1]]
  expect_equal(ref$loglik,           -3740.52,  tolerance = 1e-2)
  expect_equal(ref$n_obs,            5880L)
  expect_equal(ref$n_events,         545L)

  data(cabgkul, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = cabgkul,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                            fixed = "shapes"),
      constant = hzr_phase("constant"),
      late     = hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                            fixed = "shapes")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = FALSE)
  )

  # Final log-likelihood matches SAS to print precision.
  expect_equal(fit$fit$objective, ref$loglik, tolerance = 1e-2,
               label = "R log-likelihood vs SAS reference")

  # Free parameters on the optim scale (log_mu for each phase).  The full
  # theta vector includes fixed shape params; mask down to the 3 free.
  free <- !fit$fit$fixed_mask
  theta_free <- fit$fit$theta[free]
  V_free     <- fit$fit$vcov[free, free, drop = FALSE]
  expect_equal(length(theta_free), 3L)

  # Natural-scale MUE / MUC / MUL match.
  nat <- ref$natural
  mue_ref <- nat$estimate[nat$name == "MUE"]
  muc_ref <- nat$estimate[nat$name == "MUC"]
  mul_ref <- nat$estimate[nat$name == "MUL"]
  expect_equal(unname(exp(theta_free[1])), mue_ref, tolerance = 1e-3,
               label = "MUE (early) natural-scale")
  expect_equal(unname(exp(theta_free[2])), muc_ref, tolerance = 1e-3,
               label = "MUC (constant) natural-scale")
  expect_equal(unname(exp(theta_free[3])), mul_ref, tolerance = 1e-3,
               label = "MUL (late) natural-scale")

  # Std errors from the param-summary table (log-mu scale).
  params_ref <- ref$params
  se_e0 <- params_ref$se[params_ref$name == "E0"]
  se_c0 <- params_ref$se[params_ref$name == "C0"]
  se_l0 <- params_ref$se[params_ref$name == "L0"]
  expect_equal(sqrt(V_free[1, 1]), se_e0, tolerance = 5e-3,
               label = "SE(E0) vs SAS")
  expect_equal(sqrt(V_free[2, 2]), se_c0, tolerance = 5e-3,
               label = "SE(C0) vs SAS")
  expect_equal(sqrt(V_free[3, 3]), se_l0, tolerance = 5e-3,
               label = "SE(L0) vs SAS")

  # Off-diagonal vcov sanity (covariance vs SAS Asymptotic V-cov Matrix).
  expect_equal(V_free[1, 2], ref$vcov["E0", "C0"], tolerance = 5e-3,
               label = "cov(E0, C0)")
  expect_equal(V_free[2, 3], ref$vcov["C0", "L0"], tolerance = 5e-3,
               label = "cov(C0, L0)")
  expect_equal(V_free[1, 3], ref$vcov["E0", "L0"], tolerance = 5e-3,
               label = "cov(E0, L0)")
})

test_that("hz.death.AVC: 2-phase free-shape Early matches SAS LL/MLEs", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  set.seed(102)  # deterministic multi-start; independent of test order

  ref <- .hzr_parse_sas_lst(file.path(dir, "hz.death.AVC.lst"))$fits[[1]]
  expect_equal(ref$loglik,    -210.501, tolerance = 1e-2)
  expect_equal(ref$n_obs,     310L)
  expect_equal(ref$n_events,  70L)

  data(avc, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = avc,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.1512, nu = 1.44, m = 1,
                            fixed = "m"),
      constant = hzr_phase("constant")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )

  expect_equal(fit$fit$objective, ref$loglik, tolerance = 1e-2,
               label = "R log-likelihood vs SAS reference")

  # Natural-scale parameters from "Estimates for Model Parameters" table.
  nat <- ref$natural
  thalf_ref <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  nu_ref    <- nat$estimate[nat$name == "NU"    & nat$phase == "Early"]
  mue_ref   <- nat$estimate[nat$name == "MUE"]
  muc_ref   <- nat$estimate[nat$name == "MUC"]

  th <- fit$fit$theta
  expect_equal(unname(exp(th["early.log_t_half"])), thalf_ref, tolerance = 1e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th["early.nu"]),              nu_ref,    tolerance = 1e-3,
               label = "NU (Early) natural-scale")
  expect_equal(unname(exp(th["early.log_mu"])),     mue_ref,   tolerance = 1e-3,
               label = "MUE natural-scale")
  expect_equal(unname(exp(th["constant.log_mu"])),  muc_ref,   tolerance = 1e-3,
               label = "MUC natural-scale (CoE-constrained)")

  # SE comparison (log-mu / log-t_half scale). With the full-information vcov
  # for CoE fits, all three log-scale SEs match SAS -- including E0, the
  # CoE-conserved early log_mu, whose variance was previously dropped (the old
  # ~0.059-vs-0.133 gap; now resolved).
  se <- sqrt(diag(fit$fit$vcov))
  names(se) <- names(th)
  se_ref <- function(nm) ref$params$se[ref$params$name == nm]
  expect_equal(unname(se[["early.log_t_half"]]), se_ref("E2"), tolerance = 5e-3,
               label = "SE(log_t_half / E2)")
  expect_equal(unname(se[["early.log_mu"]]),     se_ref("E0"), tolerance = 5e-3,
               label = "SE(early log_mu / E0) -- conserved phase, full-info vcov")
  expect_equal(unname(se[["constant.log_mu"]]),  se_ref("C0"), tolerance = 5e-3,
               label = "SE(constant log_mu / C0)")
})

# ---------------------------------------------------------------------------
# hm.deadp.VALVES: nu = 0, m < 0 parametric-form correctness check
# ---------------------------------------------------------------------------
# The inventory P1 gap #5 originally claimed a ~17-unit LL gap at nu=0, m<0.
# Investigation showed R's null model matches SAS exactly (LL = -1864.76),
# confirming that hzr_decompos() Case 2L (m<0, nu=0) is correctly
# implemented.  The previously observed "gap" was a spurious comparison
# between R's covariate-model LL and the SAS stepwise null LL.
#
# The final covariate model (SELECTION SLS=0.1 MAXSTEPS=5, 7 free params)
# belongs to the stepwise harness (gap #3); R finds a better optimum
# (-1786) than SAS (-1804.78) because the stepwise initialization puts
# SAS in a different basin.  That is not a bug.
#
# This test verifies the parametric-form correctness only (null model).
test_that("hm.deadp.VALVES: null model nu=0 m<0 LL matches SAS initial (Case 2L check)", {
  testthat::skip_on_cran()
  skip_if_no_sas_fixtures()
  set.seed(103)  # deterministic multi-start; independent of test order

  # SAS Initial Summary LL (no covariates, before stepwise selection):
  # shapes THALF=0.6781774 FIXTHALF, NU=0 FIXNU, M=-2.15596 FIXM; free: MUE, MUC.
  data(valves, package = "TemporalHazard")
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = valves,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.6781774, nu = 0, m = -2.15596,
                             fixed = c("t_half", "nu", "m")),
      constant = hzr_phase("constant")
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = FALSE)
  )

  # SAS initial LL (from the first "Log likelihood = " before any step):
  # grep confirmed: -1864.76.
  expect_equal(fit$fit$objective, -1864.76, tolerance = 1e-2,
               label = "R null LL vs SAS initial (Case 2L: m<0, nu=0)")

  # Natural-scale mu values should be positive and finite.
  th <- fit$fit$theta
  expect_true(is.finite(th["early.log_mu"]),    label = "MUE finite")
  expect_true(is.finite(th["constant.log_mu"]), label = "MUC finite")

  # NOTE: the final stepwise covariate model (LL=-1804.78 with NYHA, I_PATH,
  # AGE_COP, BLACK) is tested via hzr_stepwise() in the stepwise harness
  # (gap #3).  R finds a better optimum (-1786) for that model because the
  # multi-start optimizer avoids the basin SAS's stepwise procedure falls
  # into.  This is not a bug; both optimize the same likelihood.
})

# ---------------------------------------------------------------------------
# hm.death.patient: 2-phase Early(CDF, free THALF/NU, M fixed) + Constant,
# multivariable in BOTH phases with a covariate (AGE_COP) shared across phases.
# Primary VALVES cohort, per-patient death. CONSERVE on, no truncation.
# ---------------------------------------------------------------------------
# This exercises the case the gap list flagged as P1 #1 (a covariate name
# appearing in both Early and Constant). The `.lst` parser routes the duplicate
# AGE_COP labels to distinct slots, and `vcov()` now returns a named matrix, so
# parity can be checked by name.
#
# The fit is warm-started from the SAS estimates. Blind multi-start lands in a
# different (shrunk-beta) basin here -- a 12-parameter CoE model with widely
# scaled covariates is hard for BFGS to optimize from beta = 0 -- so this test
# validates likelihood/model-spec parity (R agrees the SAS point is the MLE and
# reports the same log-likelihood there), not blind optimizer convergence.
test_that("hm.death.patient: 2-phase both-phase covariate model matches SAS", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()

  ref <- .hzr_parse_sas_lst(file.path(dir, "hm.death.patient.lst"))$fits[[1]]
  expect_equal(ref$loglik,   -1792.07, tolerance = 1e-2)
  expect_equal(ref$n_obs,    1533L)
  expect_equal(ref$n_events, 338L)

  # Reference estimates, pulled from the parsed fixture (no hard-coded numbers).
  nat <- ref$natural
  mue <- nat$estimate[nat$name == "MUE"]
  muc <- nat$estimate[nat$name == "MUC"]
  thalf <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  nu    <- nat$estimate[nat$name == "NU"    & nat$phase == "Early"]
  # name first so vapply(names(cov), par_beta, ..., phase = "Early") reads
  # unambiguously: the positional covariate name fills `name`, `phase` is named.
  par_beta <- function(name, phase) {
    ref$params$estimate[ref$params$phase == phase & ref$params$name == name]
  }
  # SAS covariate labels paired with the R `valves` column names (DOUBLE maps
  # to `double_` because `double` is a reserved word in R).
  early_cov <- c(AGE_COP = "age_cop", NYHA = "nyha", MV_NYHA = "mv_nyha",
                 DOUBLE = "double_", AO_PINC = "ao_pinc")
  const_cov <- c(AGE_COP = "age_cop", BLACK = "black", I_PATH = "i_path")
  beta_early <- vapply(names(early_cov), par_beta, numeric(1), phase = "Early")
  beta_const <- vapply(names(const_cov), par_beta, numeric(1), phase = "Constant")

  data(valves, package = "TemporalHazard")
  d <- valves
  # SAS PROC STANDARD ... REPLACE fills missing NYHA with the variable mean.
  d$nyha[is.na(d$nyha)] <- mean(d$nyha, na.rm = TRUE)
  d$mv_nyha <- d$mitral * d$nyha

  # Internal-scale warm start: log_mu, log_t_half, nu, m (fixed), early betas,
  # then constant log_mu, constant betas -- in phase/formula order.
  theta0 <- c(log(mue), log(thalf), nu, 1, unname(beta_early),
              log(muc), unname(beta_const))

  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = d,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = thalf, nu = nu, m = 1, fixed = "m",
                           formula = ~ age_cop + nyha + mv_nyha + double_ + ao_pinc),
      constant = hzr_phase("constant", formula = ~ age_cop + black + i_path)
    ),
    theta   = theta0,
    fit     = TRUE,
    control = list(n_starts = 1, maxit = 2000, conserve = TRUE)
  )

  expect_true(isTRUE(fit$fit$converged))
  # R agrees the SAS estimates are the MLE: log-likelihood matches to print
  # precision and the fit does not drift away from the seed.
  expect_equal(fit$fit$objective, ref$loglik, tolerance = 1e-2,
               label = "R log-likelihood vs SAS reference")

  th <- fit$fit$theta
  expect_equal(unname(exp(th[["early.log_mu"]])),      mue,   tolerance = 1e-4,
               label = "MUE (Early) natural-scale")
  expect_equal(unname(exp(th[["early.log_t_half"]])),  thalf, tolerance = 5e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th[["early.nu"]]),               nu,    tolerance = 5e-3,
               label = "NU (Early) natural-scale")
  expect_equal(unname(exp(th[["constant.log_mu"]])),   muc,   tolerance = 1e-4,
               label = "MUC (Constant) natural-scale")

  # Covariate coefficients (same scale in R and SAS), including AGE_COP which
  # enters both phases and must resolve to distinct, name-addressable slots.
  for (nm in names(early_cov)) {
    expect_equal(unname(th[[paste0("early.", early_cov[[nm]])]]),
                 unname(beta_early[[nm]]), tolerance = 5e-3,
                 label = paste0("early.", nm, " coefficient"))
  }
  for (nm in names(const_cov)) {
    expect_equal(unname(th[[paste0("constant.", const_cov[[nm]])]]),
                 unname(beta_const[[nm]]), tolerance = 5e-3,
                 label = paste0("constant.", nm, " coefficient"))
  }

  # vcov() is name-addressable for the shared covariate (P1 #1 fix): the two
  # AGE_COP coefficients occupy distinct, finite-variance slots.
  V <- vcov(fit)
  expect_true(is.matrix(V))
  expect_true(all(c("early.age_cop", "constant.age_cop") %in% rownames(V)))
})

# ---------------------------------------------------------------------------
# hm.death.AVC.deciles: 2-phase Early(CDF, free THALF/NU, M fixed) + Constant,
# 6 Early covariates + 3 Constant covariates, with STATUS shared across both
# phases. Plus a %DECILES goodness-of-fit table that hzr_deciles() reproduces.
# ---------------------------------------------------------------------------
# The fit starts from the SAS job's documented PARMS starting values (NOT the
# converged .lst estimates), with n_starts = 1: a deterministic, non-circular
# convergence test (start where SAS started, check R reaches where SAS
# finished). Blind multi-start lands in a different basin for this 12-parameter
# CoE model (extreme MUC ~ 4.4e-7 constant phase), so we seed deterministically.
#
# The %DECILES goodness-of-fit table IS captured in the .lst (under the spaced
# title "D E C I L E   A N A L Y S I S"); .hzr_parse_sas_deciles() reads it and
# we assert full CASES/EXPECTED/ACTUAL parity against hzr_deciles().
test_that("hm.death.AVC.deciles: 2-phase multivariable model matches SAS", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()

  # Seed for determinism. Every multi-start parity test in this file now seeds
  # itself, so the previous save/restore RNG-neutral workaround is unnecessary:
  # tests no longer depend on the RNG state left by whatever ran before them.
  set.seed(107)

  ref <- .hzr_parse_sas_lst(file.path(dir, "hm.death.AVC.deciles.lst"))$fits[[1]]
  expect_equal(ref$loglik,   -160.408, tolerance = 1e-2)
  expect_equal(ref$n_obs,    310L)
  expect_equal(ref$n_events, 70L)

  nat <- ref$natural
  mue <- nat$estimate[nat$name == "MUE"]
  muc <- nat$estimate[nat$name == "MUC"]
  thalf <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  nu    <- nat$estimate[nat$name == "NU"    & nat$phase == "Early"]
  # name first so vapply(names(cov), par_beta, ..., phase = "Early") reads
  # unambiguously: the positional covariate name fills `name`, `phase` is named.
  par_beta <- function(name, phase) {
    ref$params$estimate[ref$params$phase == phase & ref$params$name == name]
  }
  # SAS covariate labels -> R `avc` column names (lowercase, identical here).
  early_cov <- c(AGE = "age", COM_IV = "com_iv", MAL = "mal",
                 OPMOS = "opmos", OP_AGE = "op_age", STATUS = "status")
  const_cov <- c(INC_SURG = "inc_surg", ORIFICE = "orifice", STATUS = "status")
  beta_early <- vapply(names(early_cov), par_beta, numeric(1), phase = "Early")
  beta_const <- vapply(names(const_cov), par_beta, numeric(1), phase = "Constant")

  data(avc, package = "TemporalHazard")
  d <- avc
  # SAS PROC STANDARD ... REPLACE fills missing INC_SURG with the variable mean.
  d$inc_surg[is.na(d$inc_surg)] <- mean(d$inc_surg, na.rm = TRUE)

  # Deterministic start from the SAS job's PARMS statement (the documented
  # starting values, not the converged answer). Order: early log_mu, log_t_half,
  # nu, m(fixed), early betas; then constant log_mu, constant betas.
  start_thalf <- 0.1905077
  start_nu    <- 1.437416
  theta0 <- c(
    log(0.3504743), log(start_thalf), start_nu, 1,
    -0.03205774, 1.336675, 0.6872028, -0.01963377, 0.0002086689, 0.5169533,
    log(4.391673e-07), 1.375285, 3.11765, 1.054988
  )

  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data   = d,
    dist   = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = start_thalf, nu = start_nu, m = 1,
                           fixed = "m",
                           formula = ~ age + com_iv + mal + opmos + op_age + status),
      constant = hzr_phase("constant", formula = ~ inc_surg + orifice + status)
    ),
    theta   = theta0,
    fit     = TRUE,
    control = list(n_starts = 1, maxit = 2000, conserve = TRUE)
  )

  expect_true(isTRUE(fit$fit$converged))
  expect_equal(fit$fit$objective, ref$loglik, tolerance = 1e-2,
               label = "R log-likelihood vs SAS reference")

  th <- fit$fit$theta
  # Tolerances are looser than the previous (circular) warm-start-from-the-
  # answer version: starting from SAS's PARMS, R converges to its own optimum,
  # which agrees with SAS's reported MLEs to ~1e-4 on a flat likelihood.
  expect_equal(unname(exp(th[["early.log_mu"]])),     mue,   tolerance = 1e-3,
               label = "MUE (Early) natural-scale")
  expect_equal(unname(exp(th[["early.log_t_half"]])), thalf, tolerance = 5e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th[["early.nu"]]),              nu,    tolerance = 5e-3,
               label = "NU (Early) natural-scale")
  # MUC ~ 4.4e-7: the constant phase is barely identified here (near-singular
  # Hessian), so compare on the log scale with a relative tolerance rather than
  # an absolute one on a tiny number.
  expect_equal(unname(th[["constant.log_mu"]]),       log(muc), tolerance = 5e-3,
               label = "log MUC (Constant) internal-scale")

  for (nm in names(early_cov)) {
    expect_equal(unname(th[[paste0("early.", early_cov[[nm]])]]),
                 unname(beta_early[[nm]]), tolerance = 5e-3,
                 label = paste0("early.", nm, " coefficient"))
  }
  for (nm in names(const_cov)) {
    expect_equal(unname(th[[paste0("constant.", const_cov[[nm]])]]),
                 unname(beta_const[[nm]]), tolerance = 5e-3,
                 label = paste0("constant.", nm, " coefficient"))
  }

  # STATUS enters both phases and must resolve to distinct named vcov slots.
  V <- vcov(fit)
  expect_true(is.matrix(V))
  expect_true(all(c("early.status", "constant.status") %in% rownames(V)))

  # %DECILES goodness-of-fit parity. SAS GROUPS=5 at TIME=120 over the same
  # cohort. hzr_deciles() labels group 1 = lowest risk, the reverse of SAS
  # _DECILE_ 0 = highest risk, so align by reversing R's group order.
  sas_dec <- .hzr_parse_sas_deciles(file.path(dir, "hm.death.AVC.deciles.lst"))
  expect_false(is.null(sas_dec),
               label = "SAS decile table parsed from the .lst")
  expect_equal(nrow(sas_dec), 6L)                        # overall row + 5 groups
  sas_grp <- sas_dec[!is.na(sas_dec$decile), ]
  sas_grp <- sas_grp[order(sas_grp$decile), ]           # deciles 0..4

  dec <- hzr_deciles(fit, time = 120, groups = 5)
  expect_s3_class(dec, "hzr_deciles")
  r <- dec[order(-dec$group), ]                         # group 5..1 == decile 0..4

  expect_equal(r$n, sas_grp$cases,
               label = "decile group sizes (CASES) vs SAS")
  expect_equal(r$events, sas_grp$actual,
               label = "observed events per decile (ACTUAL) vs SAS")
  expect_equal(r$expected, sas_grp$expected, tolerance = 1e-2,
               label = "expected events per decile vs SAS")

  # Conservation of events: total expected == total observed == SAS total (70).
  ov <- attr(dec, "overall")
  expect_equal(ov$total_expected, ov$total_events, tolerance = 1e-3)
  expect_equal(ov$total_events, 70L)
})

# ---------------------------------------------------------------------------
# hm.death.AVC fit 1: phase-aware forward stepwise selection (DOCUMENTED GAP)
# ---------------------------------------------------------------------------
# This fixture runs SAS's `SELECTION SLE=0.2 SLS=0.1` with shaping parameters
# fixed during selection, offering the same candidate variables to BOTH the
# Early and Constant phases with per-variable flags (/I include, /S start,
# /E exclude).  SAS converges in 10 steps to Early{AGE,STATUS,COM_IV,MAL,
# OPMOS,OP_AGE} + Constant{STATUS,ORIFICE,INC_SURG}, log-likelihood -160.64
# (hm.death.AVC.lst step trace, lines 104-650).  That FINAL selected model is
# the saved "HMDEATH" fit, whose estimates and predictions are already covered
# by the `hm.death.AVC.deciles`, `hp.death.AVC.hm1`, and `hp.death.AVC.hm2`
# parity tests -- so nothing about the converged model is left unverified here.
#
# What is NOT a parity target is the SELECTION PATH.  Investigation (2026-06-06)
# confirmed R's hzr_stepwise() cannot reproduce SAS's path or final set on this
# fixture, for fundamental (not fixable-by-tuning) reasons:
#   * SAS uses APPROXIMATE variances during selection ("variances are
#     approximate because shaping parameter covariances are ignored", lst:101);
#     R uses the full Hessian, which here is near-singular with non-positive
#     variance estimates (rcond ~1e-12), so the Wald statistics that drive the
#     enter/drop decisions are not comparable between engines.
#   * SAS's /I and /S flags are PHASE-LEVEL (force MAL into Early only); R's
#     force_in is variable-level (phase-blind), so the constraint sets differ.
#   * R oscillates at p ~ slstay (= 0.1) and freezes; seeded with the SAS /S
#     start set it lands at LL -185.8, intercept-only base already -182.8,
#     both far worse than SAS's -160.64 -- a different optimizer basin, the
#     same phenomenon already documented for hm.deadp.VALVES above.
#
# This block is therefore a REGRESSION GUARD for the multiphase phase-aware
# stepwise code path on a real fixture (it must run end-to-end and return a
# well-formed result), plus a recorded SAS reference -- NOT a parity assertion.
# See inst/dev/FIXTURE-GAP-LIST.md (Group A, hm.death.AVC fit 1).
test_that("hm.death.AVC stepwise: multiphase selection runs (SAS path is a documented gap)", {
  testthat::skip_on_cran()
  skip_if_no_sas_fixtures()
  set.seed(110)

  # Recorded SAS reference (hm.death.AVC.lst): final stepwise model.
  sas_final_vars <- list(
    early    = c("age", "status", "com_iv", "mal", "opmos", "op_age"),
    constant = c("status", "orifice", "inc_surg")
  )
  expect_length(sas_final_vars$early,    6L)
  expect_length(sas_final_vars$constant, 3L)

  data(avc, package = "TemporalHazard")
  d <- avc
  d$inc_surg[is.na(d$inc_surg)] <- mean(d$inc_surg, na.rm = TRUE)

  # Base model seeds the SAS /S (start) + /I (include) variables in their
  # phases; shapes fixed, as SAS fixes them during selection.
  base <- hazard(
    survival::Surv(int_dead, dead) ~ 1, data = d, dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.1511909, nu = 1.438631, m = 1,
                           fixed = "shapes", formula = ~ com_iv + mal),
      constant = hzr_phase("constant",
                           formula = ~ orifice + mal + op_age + inc_surg)),
    fit = TRUE, control = list(n_starts = 1, conserve = TRUE))

  sw <- hzr_stepwise(
    base,
    scope = list(early    = ~ age + status + orifice + inc_surg + opmos + op_age,
                 constant = ~ age + status + opmos),
    data = d, direction = "both", criterion = "wald",
    slentry = 0.2, slstay = 0.1,
    force_in = c("mal", "inc_surg"),
    control = list(n_starts = 1, conserve = TRUE), trace = FALSE)

  # Regression guard only: the phase-aware multiphase stepwise path completes
  # and returns a well-formed result on real data.  We deliberately do NOT
  # assert anything about WHICH variables R selects or the attained
  # log-likelihood relative to SAS -- the selection path is a documented gap
  # (see the block comment above), and asserting divergence would perversely
  # fail if hzr_stepwise() later improved toward SAS.
  expect_s3_class(sw, "hzr_stepwise")
  expect_true(is.data.frame(sw$steps))
  expect_true(all(c("action", "variable", "phase") %in% names(sw$steps)))
  expect_true(is.finite(sw$fit$objective))
})

# ---------------------------------------------------------------------------
# ac.death.AVC: actuarial life tables -- Kaplan-Meier (%KAPLAN) and
# Nelson-Aalen (%NELSONT), overall and KM stratified by COM_IV.
# ---------------------------------------------------------------------------
# No model fit: hzr_kaplan()/hzr_nelson() are non-parametric, deterministic.
# SAS prints times to ~3 decimals, so rows are aligned by index (both tables
# are sorted by time with the same event times). Tolerances reflect SAS print
# rounding (~5e-6 in practice).
test_that("ac.death.AVC: Kaplan-Meier and Nelson-Aalen life tables match SAS", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  lst <- file.path(dir, "ac.death.AVC.lst")
  # An older local fixture set may predate this capture; skip cleanly rather
  # than hard-error in readLines() if this specific .lst is absent.
  testthat::skip_if_not(file.exists(lst), "ac.death.AVC.lst fixture not present")

  data(avc, package = "TemporalHazard")

  # --- Kaplan-Meier, overall (%KAPLAN _CATG=ALL) ---------------------------
  km <- hzr_kaplan(avc$int_dead, avc$dead)
  sk <- .hzr_parse_sas_lifetable(lst, "kaplan")
  expect_false(is.null(sk), label = "KAPLAN life table parsed")
  expect_equal(nrow(km), nrow(sk))
  expect_equal(km$n_risk,  sk$NUMBER)
  expect_equal(km$n_event, sk$DEAD)
  expect_equal(km$survival, sk$CUM_SURV, tolerance = 1e-4,
               label = "KM survival vs SAS CUM_SURV")
  expect_equal(km$std_err, sk$SE_EXACT, tolerance = 1e-4,
               label = "KM standard error vs SAS SE_EXACT")
  # CUM_HAZ = -log(CUM_SURV); the final all-events row is S=0 (NA in SAS).
  ok <- !is.na(sk$CUM_HAZ)
  expect_equal(km$cumhaz[ok], sk$CUM_HAZ[ok], tolerance = 1e-4,
               label = "KM cumulative hazard (-log S) vs SAS CUM_HAZ")

  # --- Nelson-Aalen, overall (%NELSONT _CATG=ALL) --------------------------
  ne <- hzr_nelson(avc$int_dead, avc$dead)
  sn <- .hzr_parse_sas_lifetable(lst, "nelson")
  expect_false(is.null(sn), label = "NELSONT life table parsed")
  expect_equal(nrow(ne), nrow(sn))
  ok_n <- !is.na(sn$CUM_HAZ)
  expect_equal(ne$cumhaz[ok_n], sn$CUM_HAZ[ok_n], tolerance = 1e-4,
               label = "Nelson-Aalen cumulative hazard vs SAS CUM_HAZ")

  # --- Kaplan-Meier stratified by COM_IV (%KAPLAN STRATIFY=COM_IV) ----------
  for (g in 0:1) {
    sub <- avc[avc$com_iv == g, ]
    kg  <- hzr_kaplan(sub$int_dead, sub$dead)
    sg  <- .hzr_parse_sas_lifetable(lst, paste0("catg", g))
    expect_false(is.null(sg), label = paste0("COM_IV=", g, " stratum parsed"))
    expect_equal(nrow(kg), nrow(sg))
    expect_equal(kg$n_risk, sg$NUMBER,
                 label = paste0("COM_IV=", g, " n at risk"))
    expect_equal(kg$survival, sg$CUM_SURV, tolerance = 1e-4,
                 label = paste0("COM_IV=", g, " KM survival"))
  }
})

# ---------------------------------------------------------------------------
# hp.death.AVC: HAZPRED predictions from the saved hz.death.AVC fit.
# ---------------------------------------------------------------------------
# %HAZPRED predicts survival and hazard at a "digital nomogram" of time points
# from a saved model (EXAMPLES.HZDEATH = the hz.death.AVC 2-phase Early+Constant
# fit). We refit that model deterministically from its .sas PARMS and predict at
# the same exact times (reconstructed from the SAS DO loop; the printed MONTHS
# are rounded to 3 decimals, so predicting at the rounded values would inflate
# the error). Survival and the instantaneous multiphase hazard both go through
# the public predict() path (type = "survival" / type = "hazard"), with their
# HAZPRED confidence limits also asserted: survival via conf.type = "logit"
# (HAZPRED's logit transform; R's default is complementary-log-log) and hazard
# via the log scale (which already matches HAZPRED). The full-information vcov
# for CoE fits supplies the se(H) basis that makes the CLs match to ~1e-5.
test_that("hp.death.AVC: HAZPRED survival/hazard nomogram matches SAS", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  lst <- file.path(dir, "hp.death.AVC.lst")
  testthat::skip_if_not(file.exists(lst), "hp.death.AVC.lst fixture not present")

  nom <- .hzr_parse_sas_nomogram(lst)
  expect_false(is.null(nom), label = "HAZPRED nomogram parsed")

  # Exact prediction times from hp.death.AVC.sas:
  #   DO MONTHS = 1*DTY..7*DTY, 14*DTY, 30*DTY, 1,2,3,6,12,18, 24 TO 180 BY 12
  # with DTY = 12 / 365.2425 (one day, in months).
  dty <- 12 / 365.2425
  months <- c((1:7) * dty, 14 * dty, 30 * dty, 1, 2, 3, 6, 12, 18,
              seq(24, 180, by = 12))
  expect_equal(length(months), nrow(nom))

  # Saved model = hz.death.AVC fit; refit deterministically from its .sas PARMS.
  data(avc, package = "TemporalHazard")
  theta0 <- c(log(0.2361727), log(0.1512095), 1.438652, 1, log(0.0005437099))
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1,
    data    = avc,
    dist    = "multiphase",
    phases  = list(
      early    = hzr_phase("cdf", t_half = 0.1512095, nu = 1.438652, m = 1,
                           fixed = "m"),
      constant = hzr_phase("constant")
    ),
    theta   = theta0,
    fit     = TRUE,
    control = list(n_starts = 1, maxit = 2000, conserve = TRUE)
  )
  expect_equal(fit$fit$objective, -210.501, tolerance = 1e-2,
               label = "saved-model LL vs SAS")

  # Survival predictions (public API).
  surv <- predict(fit, newdata = data.frame(time = months), type = "survival")
  expect_equal(unname(surv), nom$SURVIV, tolerance = 1e-4,
               label = "HAZPRED _SURVIV nomogram")

  # SAS HAZPRED uses a 1-SD level (CLEVEL = 0.68268948 -> z = 1).
  lvl <- 2 * stats::pnorm(1) - 1

  # Survival confidence limits: conf.type = "logit" reproduces HAZPRED's
  # logit-scale CLs (the default "log-log" intentionally does not).
  scl <- predict(fit, newdata = data.frame(time = months), type = "survival",
                 se.fit = TRUE, level = lvl, conf.type = "logit")
  expect_equal(scl$lower, nom$CLLSURV, tolerance = 1e-4,
               label = "HAZPRED _CLLSURV (logit CL)")
  expect_equal(scl$upper, nom$CLUSURV, tolerance = 1e-4,
               label = "HAZPRED _CLUSURV (logit CL)")

  # Instantaneous multiphase hazard via the public predict() path.
  haz <- predict(fit, newdata = data.frame(time = months), type = "hazard")
  expect_equal(unname(haz), nom$HAZARD, tolerance = 1e-3,
               label = "HAZPRED _HAZARD nomogram")

  # Hazard confidence limits (log scale -- HAZPRED and R agree here).
  hcl <- predict(fit, newdata = data.frame(time = months), type = "hazard",
                 se.fit = TRUE, level = lvl)
  expect_equal(hcl$lower, nom$CLLHAZ, tolerance = 1e-4,
               label = "HAZPRED _CLLHAZ")
  expect_equal(hcl$upper, nom$CLUHAZ, tolerance = 1e-4,
               label = "HAZPRED _CLUHAZ")
})

# ---------------------------------------------------------------------------
# bs.death.AVC: %HAZBOOT bootstrap + per-resample stepwise (DOCUMENTED GAP)
# ---------------------------------------------------------------------------
# SAS %HAZBOOT(RESAMPL=5, SLE=0.12, SLS=0.1) draws bootstrap resamples and runs
# a *fresh stepwise variable selection* on each, reporting one selected model
# per resample (bs.death.AVC.lst: a "." marks a covariate not selected in that
# resample).  The headline result is a variable-selection FREQUENCY per phase
# (e.g. Early STATUS/COM_IV selected in 5/5 resamples, OP_AGE 3/5, MAL 2/5).
#
# R has no equivalent procedure.  hzr_bootstrap() resamples and refits a FIXED
# model (covariate-stability bootstrap); it has no embedded-selection mode, and
# building one would call hzr_stepwise() per resample -- which already diverges
# from SAS on this exact AVC model (see the hm.death.AVC stepwise documented
# gap).  Combined with %HAZBOOT's intrinsic non-determinism (SEED=-1) and the
# tiny RESAMPL=5, selection-frequency parity is not a meaningful target today.
#
# This block therefore (1) captures and asserts the SAS reference frequencies
# in parseable form -- so the parity test is half-written for if/when R gains a
# bootstrap-with-selection capability -- and (2) regression-guards that R's
# (fixed-model) hzr_bootstrap() runs end-to-end on this dataset.  It does NOT
# assert R-vs-SAS frequency parity.  See inst/dev/FIXTURE-GAP-LIST.md.
test_that("bs.death.AVC: SAS bootstrap selection frequencies parse (parity is a documented gap)", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  lst <- file.path(dir, "bs.death.AVC.lst")
  testthat::skip_if_not(file.exists(lst), "bs.death.AVC.lst fixture not present")

  boot <- .hzr_parse_sas_bootstrap(lst)
  expect_false(is.null(boot), label = "SAS %HAZBOOT tables parsed")
  expect_true(all(c("early", "constant") %in% names(boot)))
  expect_equal(nrow(boot$early), 5L)          # RESAMPL = 5

  # SAS reference selection frequencies (out of 5 resamples).  Names are the
  # SAS column labels (upper case), preserved as parsed.
  freq <- attr(boot, "selection_freq")
  expect_equal(freq$early[["STATUS"]], 5L)
  expect_equal(freq$early[["COM_IV"]], 5L)
  expect_equal(freq$early[["AGE"]],    4L)
  expect_equal(freq$early[["OPMOS"]],  4L)
  expect_equal(freq$early[["OP_AGE"]], 3L)
  expect_equal(freq$early[["MAL"]],    2L)
  expect_equal(freq$constant[["OPMOS"]],   5L)
  expect_equal(freq$constant[["ORIFICE"]], 3L)
})

test_that("bs.death.AVC: R fixed-model bootstrap runs on the AVC cohort", {
  testthat::skip_on_cran()
  # No SAS .lst needed here -- this exercises R's bootstrap path on the shipped
  # `avc` dataset, so it must run in public CI (do NOT gate on fixtures).
  set.seed(111)

  data(avc, package = "TemporalHazard")
  # Null 2-phase AVC model (matches hz.death.AVC), bootstrapped.  This exercises
  # R's bootstrap path on the fixture cohort; it is NOT the %HAZBOOT procedure.
  fit <- hazard(
    survival::Surv(int_dead, dead) ~ 1, data = avc, dist = "multiphase",
    phases = list(
      early    = hzr_phase("cdf", t_half = 0.1512, nu = 1.44, m = 1, fixed = "m"),
      constant = hzr_phase("constant")),
    fit = TRUE, control = list(n_starts = 1, conserve = TRUE))

  bs <- hzr_bootstrap(fit, n_boot = 20L, seed = 111L)
  expect_s3_class(bs, "hzr_bootstrap")
  expect_gt(bs$n_success, 0L)
})

# ---------------------------------------------------------------------------
# hp.death.AVC.hm1 / hm2: patient-specific HAZPRED predictions from the saved
# multivariable both-phase model (hm.death.AVC final fit, "HMDEATH").
# ---------------------------------------------------------------------------
# Both jobs predict survival + instantaneous hazard (with logit survival CLs
# and log hazard CLs at the 1-SD level) for two covariate profiles printed as
# a BY-group "digital nomogram".  Each printed row carries its own covariate
# vector, so newdata is built directly from the parsed table -- the test is
# agnostic to the specific profiles (hm1 varies MAL 0/1; hm2 varies COM_IV 0/1
# across dates of repair).
#
# Tolerances are looser than the null-model nomogram (hp.death.AVC): this is a
# 9-coefficient, near-singular fit (rcond ~5e-12; see hm.death.AVC.deciles),
# so R converges to its own optimum a few e-4 from SAS's reported MLEs, which
# propagates to ~5e-4 in survival and (amplified at the steep early-phase
# times) ~8e-3 in the instantaneous hazard.  SAS 1-SD level: CLEVEL = z = 1.
.hzr_check_hmdeath_nomogram <- function(df, fit) {
  lvl <- 2 * stats::pnorm(1) - 1
  nd <- data.frame(
    time    = df$MONTHS,
    age     = df$AGE,
    com_iv  = df$COM_IV,
    mal     = df$MAL,
    opmos   = df$OPMOS,
    op_age  = df$OPMOS * df$AGE,
    status  = df$STATUS,
    inc_surg = df$INC_SURG,
    orifice = df$ORIFICE
  )

  surv <- predict(fit, newdata = nd, type = "survival")
  testthat::expect_equal(unname(surv), df$SURVIV, tolerance = 5e-4,
               label = "HAZPRED _SURVIV (multivariable nomogram)")
  scl <- predict(fit, newdata = nd, type = "survival", se.fit = TRUE,
                 level = lvl, conf.type = "logit")
  testthat::expect_equal(scl$lower, df$CLLSURV, tolerance = 5e-4,
               label = "HAZPRED _CLLSURV (logit CL)")
  testthat::expect_equal(scl$upper, df$CLUSURV, tolerance = 5e-4,
               label = "HAZPRED _CLUSURV (logit CL)")

  haz <- predict(fit, newdata = nd, type = "hazard")
  testthat::expect_equal(unname(haz), df$HAZARD, tolerance = 8e-3,
               label = "HAZPRED _HAZARD (multivariable nomogram)")
  hcl <- predict(fit, newdata = nd, type = "hazard", se.fit = TRUE, level = lvl)
  testthat::expect_equal(hcl$lower, df$CLLHAZ, tolerance = 8e-3,
               label = "HAZPRED _CLLHAZ")
  testthat::expect_equal(hcl$upper, df$CLUHAZ, tolerance = 8e-3,
               label = "HAZPRED _CLUHAZ")
}

test_that("hp.death.AVC.hm1: patient-specific predictions (MAL 0/1) match SAS", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  lst <- file.path(dir, "hp.death.AVC.hm1.lst")
  testthat::skip_if_not(file.exists(lst), "hp.death.AVC.hm1.lst fixture not present")

  df <- .hzr_parse_sas_nomogram_mv(lst)
  expect_false(is.null(df), label = "hm1 multivariable nomogram parsed")
  expect_setequal(unique(df$MAL), c(0, 1))     # two covariate profiles

  set.seed(108)
  fit <- .hzr_fit_avc_hmdeath()
  expect_equal(fit$fit$objective, -160.408, tolerance = 1e-2,
               label = "HMDEATH saved-model LL vs SAS")
  .hzr_check_hmdeath_nomogram(df, fit)
})

test_that("hp.death.AVC.hm2: predictions by date of repair (COM_IV 0/1) match SAS", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  lst <- file.path(dir, "hp.death.AVC.hm2.lst")
  testthat::skip_if_not(file.exists(lst), "hp.death.AVC.hm2.lst fixture not present")

  df <- .hzr_parse_sas_nomogram_mv(lst)
  expect_false(is.null(df), label = "hm2 multivariable nomogram parsed")
  expect_setequal(unique(df$COM_IV), c(0, 1))  # complete vs partial canal

  set.seed(109)
  fit <- .hzr_fit_avc_hmdeath()
  expect_equal(fit$fit$objective, -160.408, tolerance = 1e-2,
               label = "HMDEATH saved-model LL vs SAS")
  .hzr_check_hmdeath_nomogram(df, fit)
})

# ---------------------------------------------------------------------------
# hs.death.AVC.hm1: stratified observed-vs-expected calibration + mean survival
# ---------------------------------------------------------------------------
# Patient-specific HAZPRED predictions from the saved HMDEATH model, summarized
# BY type of AV canal (COM_IV).  Two SAS tables are reproduced:
#   1. "Predict number of deaths" -- per stratum, EXPECTED = sum of predicted
#      cumulative hazard at each subject's own follow-up, PEXPECT = sum of
#      predicted death probability (1 - S), ACTUAL = observed deaths.  Totals
#      conserve events (14.76 + 55.24 = 70 = ACTUAL total).
#   2. Per-stratum mean survival at a digital time grid (MSURVIV = mean over the
#      stratum of each subject's predicted survival; NSURVIV = stratum size).
# This is the population-averaged / calibration counterpart to the per-patient
# hp.death.AVC.hm1/hm2 nomograms.  Same fit; tolerances reflect the same
# near-singular 9-coefficient model (sums over 154/156 subjects, so absolute
# tolerances are scaled up accordingly).
test_that("hs.death.AVC.hm1: stratified observed-vs-expected calibration matches SAS", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  lst <- file.path(dir, "hs.death.AVC.hm1.lst")
  testthat::skip_if_not(file.exists(lst), "hs.death.AVC.hm1.lst fixture not present")

  cal <- .hzr_parse_sas_calibration(lst)
  expect_false(is.null(cal), label = "SAS observed-vs-expected table parsed")
  expect_equal(nrow(cal), 2L)                       # two COM_IV strata
  cal <- cal[order(cal$com_iv), ]

  set.seed(112)
  fit <- .hzr_fit_avc_hmdeath()
  expect_equal(fit$fit$objective, -160.408, tolerance = 1e-2)

  data(avc, package = "TemporalHazard")
  d <- avc
  d$inc_surg[is.na(d$inc_surg)] <- mean(d$inc_surg, na.rm = TRUE)
  nd <- data.frame(
    time = d$int_dead, age = d$age, com_iv = d$com_iv, mal = d$mal,
    opmos = d$opmos, op_age = d$opmos * d$age, status = d$status,
    inc_surg = d$inc_surg, orifice = d$orifice)
  ch <- predict(fit, newdata = nd, type = "cumulative_hazard")
  sv <- predict(fit, newdata = nd, type = "survival")

  for (k in seq_len(nrow(cal))) {
    g <- cal$com_iv[k]
    i <- d$com_iv == g
    expect_equal(sum(ch[i]),       cal$expected[k], tolerance = 5e-3,
                 label = paste0("EXPECTED (sum cumhaz), COM_IV=", g))
    expect_equal(sum(1 - sv[i]),   cal$pexpect[k],  tolerance = 5e-3,
                 label = paste0("PEXPECT (sum 1-S), COM_IV=", g))
    expect_equal(sum(d$dead[i]),   cal$actual[k],
                 label = paste0("ACTUAL (observed deaths), COM_IV=", g))
  }
  # Conservation of events across strata: total expected == total observed.
  expect_equal(sum(ch), sum(cal$actual), tolerance = 1e-2,
               label = "total expected == total observed (CoE)")
})

test_that("hs.death.AVC.hm1: per-stratum mean survival curve matches SAS", {
  testthat::skip_on_cran()
  dir <- skip_if_no_sas_fixtures()
  lst <- file.path(dir, "hs.death.AVC.hm1.lst")
  testthat::skip_if_not(file.exists(lst), "hs.death.AVC.hm1.lst fixture not present")

  ms <- .hzr_parse_sas_strata_survival(lst)
  expect_false(is.null(ms), label = "SAS per-stratum mean-survival table parsed")
  expect_setequal(unique(ms$com_iv), c(0, 1))

  set.seed(113)
  fit <- .hzr_fit_avc_hmdeath()

  data(avc, package = "TemporalHazard")
  d <- avc
  d$inc_surg[is.na(d$inc_surg)] <- mean(d$inc_surg, na.rm = TRUE)

  # NSURVIV (stratum size) must match the cohort.
  for (g in c(0, 1)) {
    n_sas <- ms$NSURVIV[ms$com_iv == g][1]
    expect_equal(sum(d$com_iv == g), n_sas,
                 label = paste0("stratum size NSURVIV, COM_IV=", g))
  }

  # MSURVIV = mean over the stratum of each subject's predicted survival at the
  # grid time (MONTHS = YEARS * 12).  Compare row by row.
  r_msurviv <- numeric(nrow(ms))
  for (k in seq_len(nrow(ms))) {
    g <- ms$com_iv[k]
    i <- which(d$com_iv == g)
    nd <- data.frame(
      time = rep(ms$YEARS[k] * 12, length(i)),
      age = d$age[i], com_iv = d$com_iv[i], mal = d$mal[i], opmos = d$opmos[i],
      op_age = d$opmos[i] * d$age[i], status = d$status[i],
      inc_surg = d$inc_surg[i], orifice = d$orifice[i])
    r_msurviv[k] <- mean(predict(fit, newdata = nd, type = "survival"))
  }
  expect_equal(r_msurviv, ms$MSURVIV, tolerance = 5e-4,
               label = "per-stratum mean survival (MSURVIV) vs SAS")
})

# ---------------------------------------------------------------------------
# hz.te123.OMC: 2-phase (Early CDF + Late Weibull) repeated TE events
# ---------------------------------------------------------------------------
# Two fits:
#   Fit 1 -- left-truncated (LCENSOR STARTTME), no covariates, CONSERVE
#   Fit 2 -- modulated renewal (INT_TE = INT_TE - STARTTME), late-phase
#            covariates NOPREVTE + NOTEE, CONSERVE
#
# Data derivation: PRIMISOL DATA step reproduced in .hzr_derive_primisol();
# raw fixed-width file from ~/Documents/GitHub/hazard/examples/data/omc.
#
# Known gaps:
#   P1 #6 (fit 1): CoE for early+late 2-phase differs from SAS CONSERVE.
#   SAS keeps all 5 mus free under Lagrange constraint; R fixes late.log_mu
#   via closed-form CoE, yielding LL offset ~0.14 and MUE shifted.
#   Shape params (THALF, NU, ETA) still match well; asserted below.
#   MUE NOT asserted (affected by P1 #6 gap).
test_that("hz.te123.OMC fit 1: left-truncated 2-phase shape params match SAS", {
  testthat::skip_on_cran()
  dir  <- skip_if_no_sas_fixtures()
  skip_if_no_omc_raw()
  set.seed(104)  # deterministic multi-start; independent of test order

  ref  <- .hzr_parse_sas_lst(file.path(dir, "hz.te123.OMC.lst"))$fits[[1]]
  expect_equal(ref$loglik,   -322.226, tolerance = 1e-2)
  expect_equal(ref$n_obs,    382L)
  expect_equal(ref$n_events, 44L)

  raw      <- .hzr_omc_raw_path()
  primisol <- .hzr_derive_primisol(raw)

  # Verify the R derivation reproduces SAS observation counts.
  expect_equal(nrow(primisol$te), 382L,   label = "PRIMISOL n_obs")
  expect_equal(sum(primisol$te$te), 44L,  label = "PRIMISOL n_events")

  fit <- hazard(
    survival::Surv(starttme, int_te, te) ~ 1,
    data   = primisol$te,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 0.06, nu = -1.74, m = 0, fixed = "m"),
      late  = hzr_phase("g3",  tau = 1, gamma = 1, alpha = 1, eta = 1.32,
                         fixed = c("tau", "gamma", "alpha"))
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )

  # LL: P1 #6 gap produces ~0.14 offset; assert within 0.2.
  expect_equal(fit$fit$objective, ref$loglik, tolerance = 0.2,
               label = "R LL vs SAS (P1 #6 gap: CoE implementation differs)")

  # Shape params from "Estimates for Model Parameters" natural-scale table.
  nat       <- ref$natural
  thalf_ref <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  nu_ref    <- nat$estimate[nat$name == "NU"    & nat$phase == "Early"]
  eta_ref   <- nat$estimate[nat$name == "ETA"   & nat$phase == "Late"]

  th <- fit$fit$theta
  expect_equal(unname(exp(th["early.log_t_half"])), thalf_ref, tolerance = 5e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th["early.nu"]),              nu_ref,    tolerance = 5e-3,
               label = "NU (Early) natural-scale")
  expect_equal(unname(th["late.eta"]),              eta_ref,   tolerance = 5e-3,
               label = "ETA (Late) natural-scale")

  # NOTE: MUE NOT asserted — affected by P1 #6 CoE gap.
  # R: MUE ~ 0.01601; SAS: MUE = 0.01740.
})

test_that("hz.te123.OMC fit 2: modulated renewal + late covariates matches SAS LL/MLEs", {
  testthat::skip_on_cran()
  dir  <- skip_if_no_sas_fixtures()
  skip_if_no_omc_raw()
  set.seed(105)  # deterministic multi-start; independent of test order

  ref  <- .hzr_parse_sas_lst(file.path(dir, "hz.te123.OMC.lst"))$fits[[2]]
  expect_equal(ref$loglik,   -311.597, tolerance = 1e-2)
  expect_equal(ref$n_obs,    382L)
  expect_equal(ref$n_events, 44L)

  raw      <- .hzr_omc_raw_path()
  primisol <- .hzr_derive_primisol(raw)

  # Fit 2: modulated renewal.  INT_TE = INT_TE - STARTTME (relative time);
  # no LCENSOR; late phase has covariates NOPREVTE and NOTEE.
  fit <- hazard(
    survival::Surv(int_te, te) ~ 1,
    data   = primisol$te_mod,
    dist   = "multiphase",
    phases = list(
      early = hzr_phase("cdf", t_half = 0.053, nu = -1.738, m = 0, fixed = "m"),
      late  = hzr_phase("g3",  tau = 1, gamma = 1, alpha = 1, eta = 1,
                         fixed = "shapes",
                         formula = ~ noprevte + notee)
    ),
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )

  expect_equal(fit$fit$objective, ref$loglik, tolerance = 1e-2,
               label = "R LL vs SAS (fit 2)")

  nat       <- ref$natural
  thalf_ref <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  nu_ref    <- nat$estimate[nat$name == "NU"    & nat$phase == "Early"]
  mue_ref   <- nat$estimate[nat$name == "MUE"]
  mul_ref   <- nat$estimate[nat$name == "MUL"]

  th <- fit$fit$theta
  expect_equal(unname(exp(th["early.log_t_half"])), thalf_ref, tolerance = 1e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th["early.nu"]),              nu_ref,    tolerance = 1e-3,
               label = "NU (Early) natural-scale")
  expect_equal(unname(exp(th["early.log_mu"])),     mue_ref,   tolerance = 1e-3,
               label = "MUE (Early) natural-scale")
  expect_equal(unname(exp(th["late.log_mu"])),      mul_ref,   tolerance = 1e-3,
               label = "MUL (Late) natural-scale")

  # Late-phase covariate coefficients — compare to SAS Parameter Estimate Summary.
  params_ref <- ref$params
  np_ref  <- params_ref$estimate[params_ref$name == "NOPREVTE"]
  ne_ref  <- params_ref$estimate[params_ref$name == "NOTEE"]
  expect_equal(unname(th["late.noprevte"]), np_ref, tolerance = 1e-3,
               label = "NOPREVTE coefficient")
  expect_equal(unname(th["late.notee"]),    ne_ref, tolerance = 1e-3,
               label = "NOTEE coefficient")

  # SEs from the parameter-estimate summary table (log-scale where applicable).
  V <- fit$fit$vcov
  param_idx <- function(nm) which(names(th) == nm)
  se_e2_ref  <- params_ref$se[params_ref$name == "E2"]
  se_np_ref  <- params_ref$se[params_ref$name == "NOPREVTE"]
  se_ne_ref  <- params_ref$se[params_ref$name == "NOTEE"]
  expect_equal(sqrt(V[param_idx("early.log_t_half"), param_idx("early.log_t_half")]),
               se_e2_ref, tolerance = 5e-3, label = "SE(log_t_half / E2)")
  expect_equal(sqrt(V[param_idx("late.noprevte"), param_idx("late.noprevte")]),
               se_np_ref, tolerance = 5e-2, label = "SE(NOPREVTE)")
  expect_equal(sqrt(V[param_idx("late.notee"),    param_idx("late.notee")]),
               se_ne_ref, tolerance = 5e-2, label = "SE(NOTEE)")
})

# ---------------------------------------------------------------------------
# hz.tm123.OMC: 2-phase (Early CDF + Late Weibull) permanent morbidity
# ---------------------------------------------------------------------------
# Uses WEIGHT MORBID: each TE event is weighted by severity grade.
# SAS WEIGHT applies the weight to the event log-hazard term only;
# censored intervals contribute to the survival function with weight = 1.
# R: pass weights = ifelse(te==1, morbid, 1) to reproduce this semantics.
#
# Known gaps:
#   Same P1 #6 gap as fit 1 above (CoE for early+late, MUE shifted ~10%).
#   LL offset ~0.43.  THALF and ETA assert within 5e-3.
test_that("hz.tm123.OMC: morbidity-weighted 2-phase shape params match SAS", {
  testthat::skip_on_cran()
  dir  <- skip_if_no_sas_fixtures()
  skip_if_no_omc_raw()
  set.seed(106)  # deterministic multi-start; independent of test order

  ref  <- .hzr_parse_sas_lst(file.path(dir, "hz.tm123.OMC.lst"))$fits[[1]]
  expect_equal(ref$loglik,   -581.528, tolerance = 1e-2)
  expect_equal(ref$n_obs,    382L)
  expect_equal(ref$n_events, 44L)

  raw      <- .hzr_omc_raw_path()
  primisol <- .hzr_derive_primisol(raw)

  # WEIGHT MORBID: apply severity weight to event rows; censored rows use
  # weight = 1 so they contribute normally to the survival function.
  wt <- ifelse(primisol$tm$te == 1L, primisol$tm$morbid, 1)

  fit <- hazard(
    survival::Surv(starttme, int_te, te) ~ 1,
    data    = primisol$tm,
    dist    = "multiphase",
    phases  = list(
      early = hzr_phase("cdf", t_half = 0.0109, nu = 1, m = 1e-6,
                         fixed = c("nu", "m")),
      late  = hzr_phase("g3",  tau = 1, gamma = 1, alpha = 1, eta = 1.4112,
                         fixed = c("tau", "gamma", "alpha"))
    ),
    weights = wt,
    fit     = TRUE,
    control = list(n_starts = 3, maxit = 500, conserve = TRUE)
  )

  # LL: P1 #6 gap produces ~0.43 offset; assert within 0.5.
  expect_equal(fit$fit$objective, ref$loglik, tolerance = 0.5,
               label = "R LL vs SAS (P1 #6 gap: CoE implementation differs)")

  nat       <- ref$natural
  thalf_ref <- nat$estimate[nat$name == "THALF" & nat$phase == "Early"]
  eta_ref   <- nat$estimate[nat$name == "ETA"   & nat$phase == "Late"]

  th <- fit$fit$theta
  expect_equal(unname(exp(th["early.log_t_half"])), thalf_ref, tolerance = 5e-3,
               label = "THALF (Early) natural-scale")
  expect_equal(unname(th["late.eta"]),              eta_ref,   tolerance = 5e-3,
               label = "ETA (Late) natural-scale")

  # NOTE: MUE NOT asserted — P1 #6 CoE gap shifts MUE ~10% (R 0.01643 vs SAS 0.01828).
  # NOTE: NU=1 FIXNU (fixed); M=1e-6 FIXM (fixed) — not compared.
})

#' @importFrom stats optim pnorm
#' @importFrom survival Surv
#' @keywords internal
NULL

# hazard_api.R -- Primary user-facing API for TemporalHazard
#
# This file contains the main entry points:
#   hazard()         -- construct and optionally fit a parametric hazard model
#   predict.hazard() -- generate predictions from a fitted hazard object
#   print.hazard()   -- compact S3 print method
#   coef.hazard()    -- extract fitted parameter vector
#   vcov.hazard()    -- extract variance-covariance matrix
#
# DISTRIBUTION PARAMETERIZATION CONVENTIONS
# -----------------------------------------
# Each distribution stores a flattened theta vector.  Shape/scale parameters
# always come first; covariate coefficients (beta) follow:
#
#   Weibull:     theta = [mu, nu, beta_1, beta_2, ...]      mu>0, nu>0
#   Exponential: theta = [log(lambda), beta_1, beta_2, ...]      lambda>0 via exp()
#   Loglogistic: theta = [log(alpha), log(beta), beta_1, ...]   alpha>0, beta>0 via exp()
#   Lognormal:   theta = [mu, log(sigma), beta_1, beta_2, ...]   sigma>0 via exp(); AFT model
#
# ADDING A NEW DISTRIBUTION
# -------------------------
# 1. Create R/likelihood-<dist>.R with .hzr_logl_<dist>(),
#    .hzr_gradient_<dist>(), and .hzr_optim_<dist>().
# 2. Add an else-if branch in hazard() below under "Distribution dispatch".
# 3. Add an else-if branch in predict.hazard() under "Dispatch by distribution".
# 4. Update the supported-distributions guard in predict.hazard().
# 5. Update the coefficient-extraction block in predict.hazard() for
#    linear_predictor / hazard prediction types.
# 6. Write tests in tests/testthat/test-<dist>-dist.R.
# 7. Generate a golden fixture via data-raw/golden_fixtures.R
#    (.hzr_create_<dist>_golden_fixture()).

#' Build and optionally fit a hazard model
#'
#' Creates a `hazard` object and optionally fits it via maximum likelihood.
#' This mirrors the argument-oriented workflow of the legacy HAZARD C/SAS
#' implementation: supply starting values in `theta` and the function will
#' optimize to produce fitted estimates.
#'
#' @section The fitted model:
#'
#' For `dist = "multiphase"` the cumulative hazard, instantaneous hazard, and
#' survival are
#'
#' \deqn{H(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \, \Phi_j(t),
#'       \qquad
#'       h(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \, \varphi_j(t),
#'       \qquad
#'       S(t \mid \mathbf{x}) = \exp\!\bigl(-H(t \mid \mathbf{x})\bigr)}
#'
#' where \eqn{\mu_j(\mathbf{x}) = \exp(\alpha_j + \mathbf{x}_j^\top
#' \boldsymbol{\beta}_j)} and the temporal shapes \eqn{\Phi_j}, \eqn{\varphi_j}
#' are set by each phase's `type` (see [hzr_phase()]).  The
#' proportional-hazards single-phase families (`"weibull"`, `"exponential"`)
#' are the special case \eqn{J = 1}, with covariates acting multiplicatively on
#' one temporal shape.  The `"loglogistic"` (proportional-odds) and
#' `"lognormal"` (accelerated-failure-time) families place covariates
#' differently --- on the odds of failure and the log-time location,
#' respectively --- so they are separate parameterizations, not special cases
#' of this additive form.  Parameters are estimated
#' on an unconstrained internal scale (e.g. \eqn{\log\mu}, \eqn{\log t_{1/2}})
#' and transformed back for reporting; see
#' `vignette("mf-mathematical-foundations")`.
#'
#' @section Baseline distributions:
#'
#' The `dist` argument selects the parametric form of the baseline hazard.  The
#' four single-distribution families differ in the *shape* the
#' hazard traces over follow-up; choose by what the risk is expected to do over
#' time.  `"multiphase"` is the general additive model that lets several such
#' shapes coexist.
#'
#' \describe{
#'   \item{`"weibull"` --- monotone rising or falling hazard (default)}{The
#'     workhorse parametric model: \eqn{H(t \mid \mathbf{x}) = (\mu t)^\nu
#'     \exp(\eta)}, with hazard \eqn{h \propto t^{\nu - 1}}.  The single shape
#'     \eqn{\nu} makes risk increase over time (\eqn{\nu > 1}), decrease
#'     (\eqn{\nu < 1}), or stay flat (\eqn{\nu = 1}).  Use it as the default when
#'     a single monotone trend describes the hazard.}
#'   \item{`"exponential"` --- constant hazard}{The memoryless special case
#'     \eqn{\nu = 1}: a time-invariant baseline rate, \eqn{H(t \mid \mathbf{x}) =
#'     \mu t \exp(\eta)}.  Use it when the event rate does not change with
#'     follow-up time (the constant background risk also appears as the
#'     `"constant"` phase in a multiphase model).}
#'   \item{`"loglogistic"` --- unimodal (rise-then-fall) hazard}{A log-logistic
#'     proportional-odds form (covariates act multiplicatively on the odds of
#'     failure, \eqn{\exp(\eta)}, not as an AFT time shift) whose hazard rises to
#'     a single peak and then declines when the shape exceeds 1 (and is monotone
#'     decreasing otherwise), with heavier tails than the log-normal.  Use it
#'     when risk climbs to an early peak and then eases off.}
#'   \item{`"lognormal"` --- early-peaking, resolving hazard}{An
#'     accelerated-failure-time form in which \eqn{\log} time is Gaussian; the
#'     hazard rises to an early peak and then decays toward zero.  Use it for
#'     risk that is concentrated early and resolves over time.}
#'   \item{`"multiphase"` --- additive N-phase hazard}{Sums several phase shapes
#'     into one model, \eqn{H = \sum_j \mu_j(\mathbf{x}) \Phi_j(t)}, so the
#'     overall hazard can fall, level off, and rise again within one fit.
#'     Requires `phases`; see [hzr_phase()] for the available phase shapes.  This
#'     is the form that reproduces the classic C/SAS HAZARD models.}
#' }
#'
#' @param time Numeric follow-up time vector.
#' @param status Numeric or logical event indicator vector.
#' @param time_lower Optional numeric lower bound vector for censoring intervals.
#'   Used when `status == 2` (interval-censored); defaults to `time` if NULL.
#' @param time_upper Optional numeric upper bound vector for censoring intervals.
#'   Used when `status %in% c(-1, 2)`; defaults to `time` if NULL.
#' @param x Optional design matrix (or data frame coercible to matrix).
#' @param formula Optional formula of the form `Surv(time, status) ~ predictors`.
#'   When provided, overrides direct time/status/x arguments and extracts from data.
#'   Example: `hazard(Surv(time, status) ~ x1 + x2, data = df, dist = "weibull", fit = TRUE)`.
#' @param data Optional data frame containing variables referenced in formula.
#' @param time_windows Optional numeric vector of strictly positive cut points for
#'   piecewise time-varying coefficients. When provided, each predictor column in
#'   `x` is expanded into one column per time window so each window gets its own
#'   coefficient.
#' @param theta Optional numeric coefficient vector (starting values for optimization).
#' @param dist Character baseline distribution label (default `"weibull"`).
#'   One of `"weibull"`, `"exponential"`, `"loglogistic"`, `"lognormal"`, or
#'   `"multiphase"`.  The single-distribution families differ in the *shape* the
#'   hazard traces over time; `"multiphase"` builds an additive N-phase hazard
#'   and requires `phases`.  See the **Baseline distributions** section for what
#'   each means and when to use it.
#' @param phases Optional named list of [hzr_phase()] objects specifying the
#'   phases for a multiphase model (`dist = "multiphase"`).  See Examples.
#' @param fit Logical; if TRUE, fit the model via maximum likelihood (default FALSE).
#' @param weights Optional numeric vector of observation weights (non-negative).
#'   Each observation's log-likelihood contribution is multiplied by its weight.
#'   Use for severity-weighted repeated events. Default `NULL` (unit weights).
#'   Implements the SAS `WEIGHT` statement.
#' @param control Named list of control options (see Details).
#' @param ... Additional named arguments retained for parity with legacy calling
#'   conventions.
#'
#' @details
#' Control parameters:
#' - `maxit`: Maximum iterations (default 1000)
#' - `n_starts`: Number of optimization starts for multiphase fits (default 5).
#'   Each start after the first adds random noise to the initial values, drawn
#'   from the ambient RNG stream; call `set.seed()` before fitting for
#'   reproducible results.
#' - `reltol`: Relative parameter change tolerance (default 1e-5)
#' - `abstol`: Absolute gradient norm tolerance (default 1e-6)
#' - `method`: Optimization method: "bfgs" or "nm" (default "bfgs")
#' - `condition`: Condition number control (default 14)
#' - `nocov`, `nocor`: Suppress covariance/correlation output (legacy; no-op in M2)
#'
#' Censoring status coding:
#' - 1: Exact event at time
#' - 0: Right-censored at time
#' - -1: Left-censored with upper bound at time_upper \(or time\)
#' - 2: Interval-censored in the interval \(time_lower, time_upper\)
#'
#' Time-varying coefficients:
#' - If `time_windows` is supplied, predictors are expanded to piecewise window
#'   interactions so each window has its own coefficient vector.
#' - This is implemented as design-matrix expansion, so the existing likelihood
#'   engines remain unchanged.
#'
#' @examples
#' # -- Univariable Weibull ----------------------------------------------
#' set.seed(1)
#' time   <- rexp(50, rate = 0.3)
#' status <- sample(0:1, 50, replace = TRUE, prob = c(0.3, 0.7))
#' fit <- hazard(time = time, status = status,
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' summary(fit)
#'
#' # -- Formula interface with covariates --------------------------------
#' set.seed(1001)
#' n   <- 180
#' dat <- data.frame(
#'   time   = rexp(n, rate = 0.35) + 0.05,
#'   status = rbinom(n, size = 1, prob = 0.6),
#'   age    = rnorm(n, mean = 62, sd = 11),
#'   nyha   = sample(1:4, n, replace = TRUE),
#'   shock  = rbinom(n, size = 1, prob = 0.18)
#' )
#'
#' fit2 <- hazard(
#'   survival::Surv(time, status) ~ age + nyha + shock,
#'   data    = dat,
#'   theta   = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
#'   dist    = "weibull",
#'   fit     = TRUE,
#'   control = list(maxit = 300)
#' )
#' summary(fit2)
#'
#' \donttest{
#' # -- Parametric survival with Kaplan-Meier overlay -----------------
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'
#'   # Parametric curve on a fine grid at median covariate profile
#'   t_grid   <- seq(0.05, max(dat$time), length.out = 80)
#'   curve_df <- data.frame(
#'     time = t_grid, age = median(dat$age), nyha = 2, shock = 0
#'   )
#'   curve_df$survival <- predict(fit2, newdata = curve_df,
#'                                type = "survival") * 100
#'
#'   # Kaplan-Meier empirical overlay
#'   km    <- survival::survfit(survival::Surv(time, status) ~ 1, data = dat)
#'   km_df <- data.frame(time = km$time, survival = km$surv * 100)
#'
#'   ggplot() +
#'     geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier")) +
#'     geom_line(data = curve_df, aes(time, survival,
#'                                    colour = "Parametric (Weibull)")) +
#'     scale_colour_manual(
#'       values = c("Parametric (Weibull)" = "#0072B2",
#'                  "Kaplan-Meier"         = "#D55E00")
#'     ) +
#'     scale_y_continuous(limits = c(0, 100)) +
#'     labs(x = "Months after surgery", y = "Freedom from death (%)",
#'          colour = NULL) +
#'     theme_minimal() +
#'     theme(legend.position = "bottom")
#' }
#' }
#'
#' \donttest{
#' # -- Multiphase model (two phases) ---------------------------------
#' fit_mp <- hazard(
#'   survival::Surv(time, status) ~ 1,
#'   data   = dat,
#'   dist   = "multiphase",
#'   phases = list(
#'     early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                        fixed = "shapes"),
#'     late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
#'                        fixed = "shapes")
#'   ),
#'   fit     = TRUE,
#'   control = list(n_starts = 5, maxit = 1000)
#' )
#' summary(fit_mp)
#'
#' # -- Per-phase decomposed cumulative hazard ------------------------
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   t_grid <- seq(0.01, max(dat$time), length.out = 100)
#'   decomp <- predict(fit_mp, newdata = data.frame(time = t_grid),
#'                     type = "cumulative_hazard", decompose = TRUE)
#'
#'   df_long <- data.frame(
#'     time = rep(decomp$time, 3),
#'     cumhaz = c(decomp$total, decomp$early, decomp$late),
#'     component = rep(c("Total", "Early (cdf)", "Late (cdf)"),
#'                     each = nrow(decomp))
#'   )
#'   df_long$component <- factor(df_long$component,
#'     levels = c("Total", "Early (cdf)", "Late (cdf)"))
#'
#'   ggplot2::ggplot(df_long,
#'     ggplot2::aes(x = time, y = cumhaz, colour = component,
#'                  linewidth = component)) +
#'     ggplot2::geom_line() +
#'     ggplot2::scale_colour_manual(values = c(
#'       "Total" = "black", "Early (cdf)" = "#0072B2",
#'       "Late (cdf)" = "#D55E00"
#'     )) +
#'     ggplot2::scale_linewidth_manual(values = c(
#'       "Total" = 1.2, "Early (cdf)" = 0.6, "Late (cdf)" = 0.6
#'     )) +
#'     ggplot2::labs(
#'       x = "Time", y = "Cumulative hazard H(t)",
#'       colour = NULL, linewidth = NULL,
#'       title = "Multiphase decomposition: early + late"
#'     ) +
#'     ggplot2::theme_minimal() +
#'     ggplot2::theme(legend.position = "bottom")
#' }
#' }
#'
#' @seealso
#' [predict.hazard()] for survival/cumulative-hazard predictions,
#' [summary.hazard()] for model summaries,
#' [hzr_phase()] for specifying multiphase temporal shapes.
#'
#' Vignettes with worked examples:
#' \code{vignette("fitting-hazard-models")} --- single-phase through multiphase fitting,
#' \code{vignette("prediction-visualization")} --- prediction types and decomposed hazard plots,
#' \code{vignette("inference-diagnostics")} --- bootstrap CIs and model diagnostics.
#'
#' @references
#' Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
#' hazard into phases, each incorporating a separate stream of concomitant
#' information. *J Am Stat Assoc.* 1986;81(395):615--624.
#' \doi{10.1080/01621459.1986.10478314}
#'
#' Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
#' Probability of atrial fibrillation after ablation: Using a parametric
#' nonlinear temporal decomposition mixed effects model. *Stat Methods Med Res.*
#' 2018;27(1):126--141. \doi{10.1177/0962280215623583}
#'
#' @return An object of class \code{hazard}, a named list with components:
#'   \code{call} (the matched call),
#'   \code{spec} (model specification: \code{dist}, \code{control},
#'   \code{time_windows}, \code{phases}),
#'   \code{data} (input data: \code{time}, \code{status}, \code{x},
#'   \code{weights}, etc.),
#'   \code{fit} (optimisation results: \code{theta}, \code{objective},
#'   \code{converged}, \code{se}, \code{vcov}, \code{counts}, \code{message};
#'   all \code{NULL} when \code{fit = FALSE}),
#'   and \code{engine} (implementation tag, \code{"native-r-m2"}).
#' @export
hazard <- function(formula = NULL,
                   data = NULL,
                   time = NULL,
                   status = NULL,
                   time_lower = NULL,
                   time_upper = NULL,
                   x = NULL,
                   time_windows = NULL,
                   theta = NULL,
                   dist = "weibull",
                   phases = NULL,
                   fit = FALSE,
                   weights = NULL,
                   control = list(),
                   ...) {
  # Formula dispatch: if formula is provided, parse it and extract time/status/x from data
  if (!is.null(formula)) {
    if (is.null(data)) {
      stop("'data' is required when 'formula' is provided.", call. = FALSE)
    }

    # For multiphase models, the formula RHS may contain phase-scoped terms of
    # the form `phase_name(var1 + var2)`.  These are not valid R expressions
    # so model.matrix() would fail.  Strip them here: replace the RHS with `1`
    # (no global predictors) so that .hzr_parse_formula only extracts
    # time/status from the LHS.  Covariate routing is handled per-phase via
    # hzr_phase(formula = ...) and resolved inside .hzr_optim_multiphase().
    formula_for_parse <- formula
    if (!is.null(phases) && length(formula) >= 3L) {
      # Check if RHS contains phase-scoped calls of the form `phase_name(...)`.
      # Use a parse-tree walk (not string regex) to avoid false positives when
      # a phase name coincides with a base-R function (e.g., "log", "exp").
      if (.hzr_formula_has_phase_scope(formula[[3L]], names(phases))) {
        formula_for_parse <- stats::reformulate("1", response = formula[[2L]])
      }
    }

    parsed <- .hzr_parse_formula(formula = formula_for_parse, data = data)
    time <- parsed$time
    status <- parsed$status
    time_lower <- parsed$time_lower
    time_upper <- parsed$time_upper
    x <- parsed$x

  }

  # After formula dispatch, require time and status
  if (is.null(time) || is.null(status)) {
    stop("'time' and 'status' are required (either directly or via 'formula').", call. = FALSE)
  }
  if (!is.numeric(time) || any(!is.finite(time)) || any(time < 0)) {
    stop("'time' must be a numeric vector of finite non-negative values.", call. = FALSE)
  }

  n <- length(time)
  if (length(status) != n) {
    stop("'status' must have the same length as 'time'.", call. = FALSE)
  }

  # Convert Surv object status to numeric if needed (after formula parsing)
  if (inherits(status, "Surv")) {
    status <- unclass(status)[, 2L]
  }

  # Optional censoring bounds:
  # - status = 1 (event): uses observed event time in `time`
  # - status = 0 (right-censored): censoring time in `time` (or `time_lower`)
  # - status = -1 (left-censored): upper bound in `time` (or `time_upper`)
  # - status = 2 (interval-censored): [time_lower, time_upper] required
  if (!is.null(time_lower)) {
    if (!is.numeric(time_lower) || length(time_lower) != n || any(!is.finite(time_lower)) || any(time_lower < 0)) {
      stop("'time_lower' must be a numeric vector of finite non-negative values matching length(time).", call. = FALSE)
    }
  }

  if (!is.null(time_upper)) {
    if (!is.numeric(time_upper) || length(time_upper) != n || any(!is.finite(time_upper)) || any(time_upper < 0)) {
      stop("'time_upper' must be a numeric vector of finite non-negative values matching length(time).", call. = FALSE)
    }
  }

  if (!is.null(x)) {
    x <- .hzr_as_design_matrix(x, n = n)
  }

  if (!is.null(time_windows)) {
    # We use cut points to define window-specific coefficient blocks.
    if (!is.numeric(time_windows) || any(!is.finite(time_windows)) || anyDuplicated(time_windows)) {
      stop("'time_windows' must be a numeric vector of unique finite cut points.", call. = FALSE)
    }
    time_windows <- sort(time_windows)
    if (any(time_windows <= 0)) {
      stop("'time_windows' cut points must be strictly positive.", call. = FALSE)
    }
    if (is.null(x)) {
      stop("'time_windows' requires predictor matrix 'x'.", call. = FALSE)
    }
  }

  x_fit <- x
  if (!is.null(time_windows) && !is.null(x)) {
    # Expand X -> [X_w1 | X_w2 | ...] where each row is active only in its window.
    x_fit <- .hzr_expand_time_varying_design(x = x, time = time, time_windows = time_windows)
  }

  if (!is.null(theta)) {
    if (!is.numeric(theta) || any(!is.finite(theta))) {
      stop("'theta' must be a finite numeric vector when provided.", call. = FALSE)
    }

    # If x exists, theta must include coefficients for all variates
    # theta = [shape parms ... | covariate coefficients ...]
    # For now, assume theta length determines whether we expect x
    required_coef <- if (is.null(x_fit)) 0L else ncol(x_fit)
    if (!is.null(x_fit) && length(theta) < required_coef) {
      stop("'theta' length must be >= number of required coefficients (", required_coef, ").", call. = FALSE)
    }
  }

  # --- Validate and normalize weights ----------------------------------------
  n_obs <- length(time)
  if (!is.null(weights)) {
    if (!is.numeric(weights) || length(weights) != n_obs) {
      stop("'weights' must be a numeric vector of length ", n_obs, ".",
           call. = FALSE)
    }
    if (any(weights < 0) || any(!is.finite(weights))) {
      stop("'weights' must be non-negative and finite.", call. = FALSE)
    }
  }

  if (!is.character(dist) || length(dist) != 1 || !nzchar(dist)) {
    stop("'dist' must be a non-empty character scalar.", call. = FALSE)
  }

  if (!is.list(control)) {
    stop("'control' must be a list.", call. = FALSE)
  }

  # Multiphase validation
  if (dist == "multiphase") {
    if (is.null(phases)) {
      stop("'phases' is required when dist = 'multiphase'. ",
           "Supply a list of hzr_phase() specifications.", call. = FALSE)
    }
    phases <- .hzr_validate_phases(phases)
  } else if (!is.null(phases)) {
    warning("'phases' is ignored when dist != 'multiphase'.")
    phases <- NULL
  }

  if (any(status %in% c(-1, 2))) {
    if (is.null(time_upper)) {
      warning("'time_upper' not provided; using 'time' as upper bound for left/interval-censored rows.")
    }
    if (any(status == 2) && is.null(time_lower)) {
      warning("'time_lower' not provided; using 'time' as lower bound for interval-censored rows.")
    }
  }

  # fit_state holds the result of optimization (or just starting values if fit=FALSE).
  # Fields:
  #   theta     -- parameter vector (starting values before fit; MLE estimates after)
  #   converged -- TRUE if optimizer reported convergence (code 0); NA if not fitted
  #   objective -- log-likelihood at theta; NA_real_ if not fitted
  #   se        -- standard errors (sqrt of vcov diagonal); NULL / NA if unavailable
  #   gradient  -- score vector at solution (populated by some optimizers); NULL otherwise
  #   vcov      -- variance-covariance matrix; NA if Hessian not invertible
  #   counts    -- c(fn, gr) evaluation counts from optim()
  #   message   -- convergence message string from optim()
  fit_state <- list(
    theta = theta,
    converged = NA,
    objective = NA_real_,
    se = NULL,
    gradient = NULL
  )

  .hzr_run_fit_safely <- function(expr) {
    withCallingHandlers(
      expr,
      warning = function(w) {
        msg <- conditionMessage(w)
        # Muffle only benign numerical noise from optim().  The hardened-inversion
        # diagnostics (ill-conditioned / not positive-definite / not invertible /
        # non-finite) are deliberately allowed to surface so a fit that cannot
        # produce reliable standard errors is never silent.
        if (grepl("NaNs produced", msg, fixed = TRUE)) {
          invokeRestart("muffleWarning")
        }
      }
    )
  }

  .hzr_safe_se_from_vcov <- function(vcov_mat) {
    if (is.null(vcov_mat) || !is.matrix(vcov_mat) || anyNA(vcov_mat)) {
      return(NA)
    }
    d <- diag(vcov_mat)
    # Guard against small negative/invalid variances from numerical Hessians.
    if (!all(is.finite(d)) || any(d < 0)) {
      return(rep(NA_real_, length(d)))
    }
    sqrt(d)
  }

  # Distribution dispatch -- select the distribution-specific optimizer and fit.
  if (fit && dist == "multiphase") {
    # Multiphase: theta_start is optional (assembled from phase specs if NULL)
    optim_result <- .hzr_run_fit_safely(.hzr_optim_multiphase(
      time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x_fit, theta_start = theta, weights = weights,
      control = control,
      phases = phases, formula_global = formula, data = data
    ))

    fit_state$theta <- optim_result$par
    fit_state$par   <- optim_result$par   # alias for downstream use
    fit_state$objective <- optim_result$value
    fit_state$converged <- (optim_result$convergence == 0)
    fit_state$se <- .hzr_safe_se_from_vcov(optim_result$vcov)
    fit_state$vcov <- optim_result$vcov
    fit_state$rcond <- optim_result$rcond
    fit_state$pd <- optim_result$pd
    fit_state$counts <- optim_result$counts
    fit_state$message <- optim_result$message
    # Store optimizer metadata for predict/summary
    fit_state$phases <- optim_result$phases
    fit_state$covariate_counts <- optim_result$covariate_counts
    fit_state$x_list <- optim_result$x_list
    fit_state$fixed_mask <- optim_result$fixed_mask

  } else if (fit && !is.null(theta)) {
    optim_fn <- switch(
      dist,
      weibull = .hzr_optim_weibull,
      exponential = .hzr_optim_exponential,
      loglogistic = .hzr_optim_loglogistic,
      lognormal = .hzr_optim_lognormal,
      stop("Distribution '", dist, "' not yet supported for fitting.", call. = FALSE)
    )

    optim_result <- .hzr_run_fit_safely(optim_fn(
      time = time, status = status,
      time_lower = time_lower, time_upper = time_upper,
      x = x_fit, theta_start = theta, weights = weights,
      control = control
    ))

    fit_state$theta <- optim_result$par
    fit_state$par   <- optim_result$par
    fit_state$objective <- optim_result$value
    fit_state$converged <- (optim_result$convergence == 0)
    fit_state$se <- .hzr_safe_se_from_vcov(optim_result$vcov)
    fit_state$vcov <- optim_result$vcov
    fit_state$rcond <- optim_result$rcond
    fit_state$pd <- optim_result$pd
    fit_state$counts <- optim_result$counts
    fit_state$message <- optim_result$message
  }

  # Assemble the hazard S3 object.
  # $call       -- captured call for reproducibility / print
  # $spec       -- model specification (dist, control)
  # $data       -- raw data stored for default predict() / refit
  # $fit        -- optimisation results (see fit_state fields above)
  # $legacy_args -- pass-through ... args for SAS-migration parity
  # $engine     -- implementation tag ("native-r-m2")
  obj <- list(
    call = match.call(),
    spec = list(dist = dist, control = control, time_windows = time_windows,
                phases = phases),
    data = list(
      time = time,
      time_lower = time_lower,
      time_upper = time_upper,
      status = as.numeric(status),
      x = x,
      weights = weights,
      # The evaluated `data` argument as passed to hazard() (formula path; NULL
      # when called with raw vectors). This is the user's data frame, not a
      # model.frame() result. Stored so refit-based tooling such as
      # hzr_bootstrap() can re-run the original call without a caller-frame
      # lookup of the `data` symbol, which fails when that symbol has gone out
      # of scope.
      frame = data
    ),
    fit = fit_state,
    legacy_args = list(...),
    engine = "native-r-m2"
  )

  class(obj) <- "hazard"
  obj
}

#' Predict from a hazard model object
#'
#' Produces prediction outputs from a `hazard` object. Supports multiple prediction
#' types including linear predictor, hazard, survival probability, and cumulative hazard.
#'
#' @param object A `hazard` object.
#' @param newdata Optional matrix or data frame of predictors. For types requiring
#'   time (e.g., "survival", "cumulative_hazard"), newdata should include a `time`
#'   column, or time will be taken from the fitted object's data.
#' @param type Prediction type:
#'   - `"linear_predictor"`: Linear predictor eta = x*beta (not available for multiphase)
#'   - `"hazard"`: Instantaneous hazard. Single-distribution models return the
#'     hazard scale exp(eta); multiphase models return the additive hazard
#'     h(t|x) = sum_j mu_j(x) phi_j'(t) and so require time values (like
#'     `"survival"`/`"cumulative_hazard"`). `decompose` is not supported for
#'     `"hazard"`.
#'   - `"survival"`: Survival probability S(t|x) = exp(-H(t|x))
#'   - `"cumulative_hazard"`: Cumulative hazard H(t|x) at event times
#' @param decompose Logical; if `TRUE` and the model is multiphase, return a
#'   data frame with per-phase cumulative hazard contributions alongside the
#'   total.  Ignored for single-distribution models.  Default `FALSE`.
#' @param se.fit Logical; if `TRUE`, compute delta-method standard errors and
#'   confidence limits for each prediction.  The return value becomes a data
#'   frame with columns `fit`, `se.fit`, `lower`, `upper`.  Default `FALSE`.
#'   CLs are computed on the log-hazard / log-cumhaz scale and on the
#'   log(-log(survival)) scale so lower/upper stay inside the valid range
#'   of each prediction type; `linear_predictor` uses symmetric natural-scale
#'   CLs.
#'   For multiphase models, `se.fit = TRUE` combines with `decompose = TRUE`
#'   when `type = "cumulative_hazard"`: the result is a long data frame with one
#'   row per prediction time and component (`component` in `"total"` plus each
#'   phase name) and columns `fit`, `se.fit`, `lower`, `upper`. Per-phase CLs
#'   use only that phase's parameters, so they do not sum to the total CL.
#'   The combination is not available for `type = "survival"` (per-phase
#'   survival is not additive).
#' @param level Numeric confidence level in `(0, 1)`; default `0.95`.
#'   Only used when `se.fit = TRUE`.
#' @param conf.type Transform for `type = "survival"` confidence limits when
#'   `se.fit = TRUE`: `"log-log"` (default) builds them on `log(-log S)` (the
#'   `survival::survfit` standard); `"logit"` builds them on `logit(1 - S)`,
#'   reproducing SAS HAZARD's `HAZPRED` survival limits. Other types are
#'   unaffected (hazard/cumulative-hazard use a log scale that already matches
#'   HAZPRED). Only used when `se.fit = TRUE`.
#' @param ... Unused; included for S3 compatibility.
#'
#' @details
#' For Weibull models with survival or cumulative_hazard predictions:
#' - Cumulative hazard: H(t|x) = (mu*t)^nu * exp(eta)
#' - Survival: S(t|x) = exp(-H(t|x))
#'
#' Time values must be positive and finite. If newdata contains a `time` column,
#' it will be used; otherwise, the time vector from the fitted object is used.
#' For models fit with `time_windows`, predictions for `type = "linear_predictor"`
#' or `"hazard"` also require time values (via `newdata$time` or fitted-time fallback)
#' so window-specific coefficients can be selected.
#'
#' @return When `se.fit = FALSE` (default), a numeric vector of predictions.
#'   When `se.fit = TRUE`, a data frame with columns `fit`, `se.fit`, `lower`,
#'   `upper` (delta-method point estimate, standard error, and confidence
#'   limits at `level`). For multiphase `type = "cumulative_hazard"` with
#'   `decompose = TRUE`, a long data frame (`time`, `component`, `fit`,
#'   `se.fit`, `lower`, `upper`); with `decompose = TRUE` and `se.fit = FALSE`,
#'   a wide data frame of per-phase contributions.
#' @examples
#' # -- Basic predictions ------------------------------------------------
#' set.seed(1)
#' fit <- hazard(time = rexp(50, 0.3), status = rep(1L, 50),
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' predict(fit, type = "survival")
#' predict(fit, newdata = data.frame(time = c(1, 2, 5)),
#'         type = "cumulative_hazard")
#'
#' # -- Patient-specific survival curves ---------------------------------
#' set.seed(1001)
#' n   <- 180
#' dat <- data.frame(
#'   time   = rexp(n, rate = 0.35) + 0.05,
#'   status = rbinom(n, size = 1, prob = 0.6),
#'   age    = rnorm(n, mean = 62, sd = 11),
#'   nyha   = sample(1:4, n, replace = TRUE),
#'   shock  = rbinom(n, size = 1, prob = 0.18)
#' )
#' fit2 <- hazard(
#'   survival::Surv(time, status) ~ age + nyha + shock,
#'   data  = dat,
#'   theta = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
#'   dist  = "weibull", fit = TRUE
#' )
#'
#' new_patients <- data.frame(
#'   time = c(0.5, 1.5, 3.0),
#'   age  = c(50, 65, 75),
#'   nyha = c(1, 3, 4),
#'   shock = c(0, 0, 1)
#' )
#' # Compute predictions from the clean covariate frame before adding columns
#' surv   <- predict(fit2, newdata = new_patients, type = "survival")
#' cumhaz <- predict(fit2, newdata = new_patients, type = "cumulative_hazard")
#' new_patients$survival          <- surv
#' new_patients$cumulative_hazard <- cumhaz
#' new_patients
#'
#' \donttest{
#' # -- Grouped survival curves ---------------------------------------
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'
#'   t_grid <- seq(0.05, max(dat$time), length.out = 80)
#'   profiles <- data.frame(
#'     label = c("Low risk (age 50, NYHA I)",
#'               "High risk (age 75, NYHA IV)"),
#'     age   = c(50, 75),
#'     nyha  = c(1, 4),
#'     shock = c(0, 1)
#'   )
#'
#'   curve_list <- lapply(seq_len(nrow(profiles)), function(i) {
#'     nd <- data.frame(
#'       time  = t_grid,
#'       age   = profiles$age[i],
#'       nyha  = profiles$nyha[i],
#'       shock = profiles$shock[i]
#'     )
#'     nd$survival <- predict(fit2, newdata = nd, type = "survival") * 100
#'     nd$profile  <- profiles$label[i]
#'     nd
#'   })
#'   curve_df <- do.call(rbind, curve_list)
#'
#'   ggplot(curve_df, aes(time, survival, colour = profile)) +
#'     geom_line() +
#'     scale_y_continuous(limits = c(0, 100)) +
#'     labs(x = "Months after surgery",
#'          y = "Freedom from death (%)",
#'          title = "Predicted survival by risk profile",
#'          colour = NULL) +
#'     theme_minimal()
#' }
#' }
#'
#' \donttest{
#' # -- Multiphase predictions with decomposition --------------------
#' set.seed(42)
#' n   <- 200
#' dat <- data.frame(
#'   time   = rexp(n, rate = 0.25) + 0.01,
#'   status = rbinom(n, size = 1, prob = 0.65)
#' )
#' fit_mp <- hazard(
#'   survival::Surv(time, status) ~ 1,
#'   data   = dat,
#'   dist   = "multiphase",
#'   phases = list(
#'     early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                        fixed = "shapes"),
#'     late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
#'                        fixed = "shapes")
#'   ),
#'   fit     = TRUE,
#'   control = list(n_starts = 5, maxit = 1000)
#' )
#'
#' t_grid <- seq(0.01, max(dat$time) * 0.9, length.out = 100)
#' nd     <- data.frame(time = t_grid)
#'
#' # Overall survival
#' predict(fit_mp, newdata = nd, type = "survival")
#'
#' # Per-phase decomposed cumulative hazard
#' decomp <- predict(fit_mp, newdata = nd,
#'                   type = "cumulative_hazard", decompose = TRUE)
#' head(decomp)
#' }
#'
#' @seealso
#' [hazard()] for model fitting,
#' [summary.hazard()] for model summaries,
#' [hzr_phase()] for multiphase temporal shapes.
#'
#' \code{vignette("prediction-visualization")} for detailed prediction
#' workflows including decomposed hazard plots and patient-specific curves.
#' @export
predict.hazard <- function(object, newdata = NULL,
                           type = c("hazard", "linear_predictor",
                                    "survival", "cumulative_hazard"),
                           decompose = FALSE,
                           se.fit = FALSE, level = 0.95,
                           conf.type = c("log-log", "logit"), ...) {
  type <- match.arg(type)
  # `conf.type` is validated lazily inside the se.fit survival path (it only
  # affects survival CLs); validating here would error on an otherwise-ignored
  # value (e.g. se.fit = FALSE, or a non-survival type).
  theta <- object$fit$theta
  time_windows <- object$spec$time_windows

  if (is.null(theta)) {
    stop("No coefficients ('theta') are available in 'object'.", call. = FALSE)
  }

  if (!is.logical(se.fit) || length(se.fit) != 1L || is.na(se.fit)) {
    stop("'se.fit' must be TRUE or FALSE.", call. = FALSE)
  }
  if (se.fit) {
    if (!is.numeric(level) || length(level) != 1L ||
          is.na(level) || level <= 0 || level >= 1) {
      stop("'level' must be a single number in (0, 1).", call. = FALSE)
    }
    if (decompose && object$spec$dist == "multiphase" &&
          type != "cumulative_hazard") {
      stop("'se.fit = TRUE' with 'decompose = TRUE' is only supported for ",
           "type = \"cumulative_hazard\" (per-phase survival is not additive). ",
           "Got type = \"", type, "\".", call. = FALSE)
    }
  }

  # -----------------------------------------------------------------------
  # Predictions that do NOT need time (linear_predictor, hazard)
  # -----------------------------------------------------------------------
  # These predictions work purely through the linear predictor eta = X beta.
  # Shape parameters are stripped from theta; only covariate beta's are used.
  # hazard returns exp(eta), which is the relative-hazard multiplier (not the
  # baseline conditional hazard, which would require time + distribution).
  # Multiphase `hazard` is the instantaneous additive hazard h(t) = sum_j
  # mu_j(x) * phi_j'(t); it is time-based, so it is handled in the time-based
  # branch below alongside survival / cumulative_hazard. Only the eta-based
  # types (single-dist hazard = exp(eta); linear_predictor) are handled here.
  if (type %in% c("hazard", "linear_predictor") &&
      !(type == "hazard" && object$spec$dist == "multiphase")) {
    if (object$spec$dist == "multiphase") {
      stop("Prediction type 'linear_predictor' is not supported for multiphase ",
           "models. Use 'survival', 'cumulative_hazard', or 'hazard' instead.",
           call. = FALSE)
    }
    n_pred <- NULL
    pred_time <- NULL
    if (is.null(newdata)) {
      x <- object$data$x
      pred_time <- object$data$time
    } else {
      newdata <- as.data.frame(newdata)
      n_pred <- nrow(newdata)
      if ("time" %in% names(newdata)) {
        pred_time <- newdata$time
      }
      # Remove time column if present (not needed for hazard/linear_predictor)
      newdata <- newdata[, names(newdata) != "time", drop = FALSE]
      if (ncol(newdata) > 0) {
        x <- as.matrix(newdata)
      } else {
        x <- NULL
      }
    }

    if (!is.null(time_windows)) {
      if (is.null(x)) {
        stop("Time-varying coefficients require predictors for '", type, "' predictions.", call. = FALSE)
      }
      if (is.null(pred_time)) {
        stop(
          "Time-varying coefficients require a 'time' column in newdata for '",
          type,
          "' predictions.",
          call. = FALSE
        )
      }
      # Apply the same piecewise expansion used at fit time.
      x <- .hzr_expand_time_varying_design(x = x, time = pred_time, time_windows = time_windows)
    }

    # Extract covariate coefficients by stripping shape parameters from theta.
    n_shape <- .hzr_shape_parameter_count(object$spec$dist)

    if (is.null(x)) {
      if (length(theta) <= n_shape) {
        # Univariable model: no covariates, eta = 0
        if (is.null(n_pred)) n_pred <- length(object$data$time)
        eta <- rep(0, n_pred)
      } else {
        stop("Predictors are required either in the fitted object or via 'newdata'.", call. = FALSE)
      }
    } else {
      beta <- if (length(theta) > n_shape) theta[(n_shape + 1):length(theta)] else theta
      if (ncol(x) != length(beta)) {
        stop("Number of predictor columns (", ncol(x),
             ") must match number of covariate coefficients (", length(beta), ").",
             call. = FALSE)
      }
      eta <- as.numeric(x %*% beta)
    }

    if (type == "linear_predictor") {
      if (!se.fit) return(eta)
      diff_fn <- function(th) {
        if (is.null(x)) return(rep(0, length(eta)))
        n_shape <- .hzr_shape_parameter_count(object$spec$dist)
        beta_cand <- if (length(th) > n_shape) th[(n_shape + 1):length(th)] else th
        as.numeric(x %*% beta_cand)
      }
      return(.hzr_predict_with_se(object = object, type = "linear_predictor",
                                    time = NULL, x = x, level = level,
                                    diff_fn = diff_fn))
    }
    if (!se.fit) return(exp(eta))
    diff_fn <- function(th) {
      if (is.null(x)) return(rep(1, length(eta)))
      n_shape <- .hzr_shape_parameter_count(object$spec$dist)
      beta_cand <- if (length(th) > n_shape) th[(n_shape + 1):length(th)] else th
      exp(as.numeric(x %*% beta_cand))
    }
    return(.hzr_predict_with_se(object = object, type = "hazard",
                                  time = NULL, x = x, level = level,
                                  diff_fn = diff_fn))
  }

  # -----------------------------------------------------------------------
  # Predictions that DO need time (survival, cumulative_hazard)
  # -----------------------------------------------------------------------
  # Each distribution computes H(t|x) in its own way; all then share the
  # same final return statements:
  #   cumulative_hazard -> return(cumhaz)
  #   survival          -> return(exp(-cumhaz))
  #
  # EXCEPTION: log-normal uses a direct Phi(-z) formula for survival and
  # returns early inside its branch, bypassing the shared return below.
  # This is because the log-normal is an AFT model where H(t) = -log Phi(-z)
  # is numerically better computed directly from the normal CDF rather than
  # via exp(-H).
  # `hazard` is included here only for multiphase (single-dist hazard returned
  # above via exp(eta)); it is the instantaneous additive hazard h(t).
  if (type %in% c("survival", "cumulative_hazard", "hazard")) {
    supported <- c("weibull", "exponential", "loglogistic", "lognormal", "multiphase")
    if (!object$spec$dist %in% supported) {
      stop("Prediction type '", type, "' is only supported for ",
           paste(supported, collapse = ", "), " models.", call. = FALSE)
    }

    # --- Multiphase prediction (early return) ---------------------------------
    if (object$spec$dist == "multiphase") {
      # Extract time from newdata or fitted data
      if (!is.null(newdata)) {
        newdata <- as.data.frame(newdata)
        if (!"time" %in% names(newdata)) {
          stop("'newdata' must contain a 'time' column for '", type, "' predictions.",
               call. = FALSE)
        }
        pred_time <- newdata$time
      } else {
        pred_time <- object$data$time
      }

      if (!is.numeric(pred_time) || any(!is.finite(pred_time)) || any(pred_time < 0)) {
        stop("'time' must be a numeric vector of finite non-negative values.", call. = FALSE)
      }

      # Recover phase metadata from fit
      phases <- object$fit$phases
      if (is.null(phases)) phases <- object$spec$phases
      cov_counts <- object$fit$covariate_counts
      x_list <- object$fit$x_list

      # If newdata has covariates, rebuild per-phase design matrices
      if (!is.null(newdata)) {
        nd_covs <- newdata[, names(newdata) != "time", drop = FALSE]
        for (nm in names(phases)) {
          ph <- phases[[nm]]
          if (!is.null(ph$formula) && ncol(nd_covs) > 0) {
            x_list[[nm]] <- stats::model.matrix(ph$formula, data = newdata)[, -1L, drop = FALSE]
          } else if (cov_counts[[nm]] > 0 && ncol(nd_covs) > 0) {
            x_list[[nm]] <- as.matrix(nd_covs)
          } else {
            x_list[[nm]] <- NULL
          }
        }
      }

      # Instantaneous additive hazard is not phase-decomposable through this
      # path (the internal hazard evaluator returns the total only).
      if (type == "hazard" && decompose) {
        stop("decompose = TRUE is not supported for type = 'hazard'.",
             call. = FALSE)
      }

      if (se.fit) {
        if (decompose) {
          return(.hzr_predict_with_se_decomposed( # nolint: object_usage_linter.
            object = object, time = pred_time, x_list = x_list,
            cov_counts = cov_counts, phases = phases, level = level
          ))
        }
        diff_fn <- if (type == "hazard") {
          function(th) {
            .hzr_multiphase_hazard(pred_time, th, phases, cov_counts, x_list)
          }
        } else {
          function(th) {
            .hzr_multiphase_cumhaz(pred_time, th, phases, cov_counts, x_list)
          }
        }
        return(.hzr_predict_with_se(
          object = object, type = type, time = pred_time,
          x_list = x_list, cov_counts = cov_counts, phases = phases,
          level = level, diff_fn = diff_fn, conf_type = conf.type
        ))
      }

      if (type == "hazard") {
        return(.hzr_multiphase_hazard(pred_time, theta, phases, cov_counts,
                                      x_list))
      }

      result <- .hzr_multiphase_cumhaz(
        pred_time, theta, phases, cov_counts, x_list,
        per_phase = decompose
      )

      if (decompose) {
        # Return a data frame with time, total, and per-phase columns
        out <- data.frame(time = pred_time, total = result$total)
        for (nm in names(phases)) {
          out[[nm]] <- result[[nm]]
        }
        if (type == "survival") {
          out$total <- exp(-out$total)
          for (nm in names(phases)) {
            # Per-phase survival contribution is not additive; provide cumhaz
            # columns as-is and only transform total.
          }
        }
        return(out)
      }

      cumhaz <- result
      if (type == "cumulative_hazard") return(cumhaz)
      return(exp(-cumhaz))
    }

    # --- Standard single-distribution prediction ------------------------------
    # Extract time from newdata or use fitted data
    if (!is.null(newdata)) {
      newdata <- as.data.frame(newdata)
      if ("time" %in% names(newdata)) {
        time <- newdata$time
        x <- as.matrix(newdata[, names(newdata) != "time", drop = FALSE])
      } else {
        stop("'newdata' must contain a 'time' column for '", type, "' predictions.", call. = FALSE)
      }
    } else {
      time <- object$data$time
      x <- object$data$x
    }

    if (!is.null(time_windows) && !is.null(x) && ncol(x) > 0) {
      # Keep prediction design matrix consistent with training-time expansion.
      x <- .hzr_expand_time_varying_design(x = x, time = time, time_windows = time_windows)
    }

    if (!is.numeric(time) || any(!is.finite(time)) || any(time < 0)) {
      stop("'time' must be a numeric vector of finite non-negative values.", call. = FALSE)
    }

    # Extract shape parameters and covariate coefficients
    n_shape <- .hzr_shape_parameter_count(object$spec$dist)

    if (!is.null(x) && ncol(x) > 0) {
      if (length(theta) < ncol(x) + n_shape) {
        stop("Number of parameters insufficient for predictor columns.", call. = FALSE)
      }
      beta_coef <- theta[(n_shape + 1):length(theta)]
      eta <- as.numeric(x %*% beta_coef)
    } else {
      eta <- rep(0, length(time))
    }

    # Dispatch cumulative hazard computation by distribution.
    # Log-normal is an AFT model, so H is derived from the standardised
    # residual z = (log t - mu - x beta) / sigma.  For se.fit we define a
    # closure `cumhaz_of` that computes H for any candidate theta -- this
    # is the delta-method target for both "cumulative_hazard" and
    # "survival" predictions.
    dist_lbl <- object$spec$dist
    has_cov <- !is.null(x) && ncol(x) > 0

    # Preserve the pre-0.9.8 stop() behaviour on an ill-conditioned MLE.
    # The closures below return NA on negative shape parameters so numeric
    # jacobian perturbations stay robust, but we want a clean error at the
    # point estimate itself.
    if (dist_lbl == "weibull" && (theta[1] <= 0 || theta[2] <= 0)) {
      stop("Weibull shape parameters (mu, nu) must be positive.", call. = FALSE)
    }

    cumhaz_of <- if (dist_lbl == "weibull") {
      function(th) {
        if (th[1] <= 0 || th[2] <= 0) return(rep(NA_real_, length(time)))
        beta_cand <- if (length(th) > 2) th[3:length(th)] else numeric(0)
        eta_cand <- if (has_cov) as.numeric(x %*% beta_cand) else rep(0, length(time))
        (th[1] * time) ^ th[2] * exp(eta_cand)
      }
    } else if (dist_lbl == "exponential") {
      function(th) {
        beta_cand <- if (length(th) > 1) th[2:length(th)] else numeric(0)
        eta_cand <- if (has_cov) as.numeric(x %*% beta_cand) else rep(0, length(time))
        exp(th[1]) * time * exp(eta_cand)
      }
    } else if (dist_lbl == "loglogistic") {
      function(th) {
        beta_cand <- if (length(th) > 2) th[3:length(th)] else numeric(0)
        eta_cand <- if (has_cov) as.numeric(x %*% beta_cand) else rep(0, length(time))
        log(1 + exp(th[1]) * (time ^ exp(th[2])) * exp(eta_cand))
      }
    } else if (dist_lbl == "lognormal") {
      function(th) {
        beta_cand <- if (length(th) > 2) th[3:length(th)] else numeric(0)
        # AFT: covariates shift the location.
        eta_aft <- if (has_cov) th[1] + as.numeric(x %*% beta_cand) else rep(th[1], length(time))
        z <- (log(time) - eta_aft) / exp(th[2])
        -pnorm(-z, log.p = TRUE)
      }
    } else {
      stop("Unknown distribution '", dist_lbl, "'.", call. = FALSE)
    }

    if (se.fit) {
      return(.hzr_predict_with_se(
        object = object, type = type, time = time, x = x,
        level = level, diff_fn = cumhaz_of, conf_type = conf.type
      ))
    }

    cumhaz <- cumhaz_of(theta)
    if (type == "cumulative_hazard") return(cumhaz)
    return(exp(-cumhaz))
  }

  stop("Unknown prediction type: '", type, "'.", call. = FALSE)
}


#' Print method for fitted hazard models
#'
#' Compact one-block summary of a fitted `hazard` object: sample size,
#' number of predictors, distribution, theta vector, and log-likelihood.
#' S3 dispatch only -- users call `print(fit)` rather than invoking this
#' directly.
#'
#' @param x A `hazard` object returned by [hazard()].
#' @param ... Additional arguments (ignored).
#' @return `x`, invisibly.
#' @keywords internal
#' @export
print.hazard <- function(x, ...) {
  n <- length(x$data$time)
  p <- if (is.null(x$data$x)) 0L else ncol(x$data$x)
  cat("hazard object\n")
  cat("  observations:", n, "\n")
  cat("  predictors:  ", p, "\n")
  cat("  dist:        ", x$spec$dist, "\n")

  if (x$spec$dist == "multiphase" && !is.null(x$spec$phases)) {
    ph <- x$spec$phases
    cat("  phases:      ", length(ph),
        " (", paste(names(ph), collapse = ", "), ")\n", sep = "")
  }

  cat("  engine:      ", x$engine, "\n")
  if (!anyNA(x$fit$objective)) {
    cat("  log-lik:     ", format(x$fit$objective, digits = 6), "\n")
    cat("  converged:   ", x$fit$converged, "\n")
  }
  invisible(x)
}

#' Summarize a hazard model
#'
#' Returns a compact summary of a `hazard` object, including model metadata,
#' fit diagnostics, and coefficient-level statistics when available.
#'
#' @param object A `hazard` object.
#' @param ... Unused; for S3 compatibility.
#' @return An object of class `summary.hazard`.
#' @examples
#' # -- Single-phase Weibull summary ------------------------------------
#' fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' summary(fit)
#'
#' \donttest{
#' # -- Multiphase model summary ----------------------------------------
#' set.seed(42)
#' n   <- 200
#' dat <- data.frame(
#'   time   = rexp(n, rate = 0.25) + 0.01,
#'   status = rbinom(n, size = 1, prob = 0.65)
#' )
#' fit_mp <- hazard(
#'   survival::Surv(time, status) ~ 1,
#'   data   = dat,
#'   dist   = "multiphase",
#'   phases = list(
#'     early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                        fixed = "shapes"),
#'     late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
#'                        fixed = "shapes")
#'   ),
#'   fit     = TRUE,
#'   control = list(n_starts = 5, maxit = 1000)
#' )
#' summary(fit_mp)
#' }
#'
#' @seealso
#' [hazard()] for model fitting, [predict.hazard()] for predictions.
#'
#' \code{vignette("fitting-hazard-models")} for fitting workflows,
#' \code{vignette("inference-diagnostics")} for bootstrap CIs and diagnostics.
#' @export
summary.hazard <- function(object, ...) {
  n <- length(object$data$time)
  p <- if (is.null(object$data$x)) 0L else ncol(object$data$x)
  theta <- object$fit$theta
  vcov_mat <- object$fit$vcov

  coef_table <- NULL
  if (!is.null(theta)) {
    # For multiphase, use the theta names directly (they're already informative)
    if (object$spec$dist == "multiphase") {
      coef_names <- names(theta)
      if (is.null(coef_names)) coef_names <- paste0("param_", seq_along(theta))
    } else {
      coef_names <- .hzr_parameter_names(theta = theta, dist = object$spec$dist, p = p)
    }

    std_error <- rep(NA_real_, length(theta))
    z_stat <- rep(NA_real_, length(theta))
    p_value <- rep(NA_real_, length(theta))

    if (!is.null(vcov_mat) && is.matrix(vcov_mat)) {
      d <- diag(vcov_mat)
      std_error <- sqrt(d)            # NA for fixed params, finite for free
      valid <- is.finite(std_error) & std_error > 0
      z_stat[valid] <- theta[valid] / std_error[valid]
      p_value[valid] <- 2 * pnorm(-abs(z_stat[valid]))
    }

    coef_table <- data.frame(
      estimate = unname(theta),
      std_error = std_error,
      z_stat = z_stat,
      p_value = p_value,
      row.names = coef_names,
      check.names = FALSE
    )
  }

  out <- list(
    call = object$call,
    n = n,
    p = p,
    dist = object$spec$dist,
    engine = object$engine,
    converged = object$fit$converged,
    log_lik = object$fit$objective,
    counts = object$fit$counts,
    message = object$fit$message,
    coefficients = coef_table,
    has_vcov = !is.null(vcov_mat) && is.matrix(vcov_mat),
    rcond = object$fit$rcond,
    pd = object$fit$pd,
    phases = object$spec$phases
  )

  class(out) <- "summary.hazard"
  out
}

#' Print method for hazard summary objects
#'
#' Formatted console display of [summary.hazard()] output: distribution,
#' phase list (for multiphase), coefficient table with standard errors,
#' and log-likelihood.  When the post-fit Hessian is ill-conditioned or not
#' positive-definite, a note warns that the standard errors may be unreliable;
#' when the Hessian could not be inverted at all, a note reports that standard
#' errors are unavailable.  S3 dispatch only -- users call `print(summary(fit))`
#' rather than invoking this directly.
#'
#' @param x A `summary.hazard` object returned by [summary.hazard()].
#' @param ... Additional arguments (ignored).
#' @return `x`, invisibly.
#' @keywords internal
#' @export
print.summary.hazard <- function(x, ...) {
  if (x$dist == "multiphase" && !is.null(x$phases)) {
    cat("Multiphase hazard model (", length(x$phases), " phases)\n", sep = "")
  } else {
    cat("hazard model summary\n")
  }
  cat("  observations:", x$n, "\n")
  cat("  predictors:  ", x$p, "\n")
  cat("  dist:        ", x$dist, "\n")

  if (!is.null(x$phases)) {
    for (i in seq_along(x$phases)) {
      nm <- names(x$phases)[i]
      ph <- x$phases[[i]]
      label <- switch(ph$type,
        cdf      = paste0("cdf (", nm, " risk)"),
        hazard   = "hazard (late risk)",
        constant = "constant (flat rate)",
        g3       = "g3 (late risk)")
      cat("  phase ", i, ":      ", nm, " - ", label, "\n", sep = "")
    }
  }

  cat("  engine:      ", x$engine, "\n")

  if (!is.null(x$converged) && !is.na(x$converged)) {
    cat("  converged:   ", x$converged, "\n")
  }
  if (!is.null(x$log_lik) && !is.na(x$log_lik)) {
    cat("  log-lik:     ", format(x$log_lik, digits = 6), "\n")
  }
  if (!is.null(x$rcond) && !is.na(x$rcond) && x$rcond < .hzr_rcond_tol) {
    cat("  Note: Hessian ill-conditioned (rcond = ",
        format(x$rcond, digits = 3),
        "); standard errors may be unreliable.\n", sep = "")
  }
  if (!is.null(x$pd) && !is.na(x$pd) && !isTRUE(x$pd)) {
    cat("  Note: Hessian not positive-definite at the optimum; ",
        "standard errors may be unreliable.\n", sep = "")
  }
  if (!is.null(x$converged) && !is.na(x$converged) && isFALSE(x$has_vcov)) {
    cat("  Note: standard errors unavailable; the Hessian could not be ",
        "inverted.\n", sep = "")
  }
  if (!is.null(x$counts)) {
    fn_count <- x$counts[["function"]] %||% x$counts[["fn"]] %||% NA_integer_
    gr_count <- x$counts[["gradient"]] %||% x$counts[["gr"]] %||% NA_integer_
    if (!is.na(fn_count) || !is.na(gr_count)) {
      cat("  evaluations: ", "fn=", fn_count, ", gr=", gr_count, "\n", sep = "")
    }
  }
  if (!is.null(x$message) && nzchar(x$message)) {
    cat("  message:     ", x$message, "\n")
  }

  if (!is.null(x$coefficients)) {
    if (!is.null(x$phases)) {
      # Group coefficients by phase for readable output
      cat("\nCoefficients (internal scale):\n")
      for (nm in names(x$phases)) {
        prefix <- paste0("^", nm, "\\.")
        rows <- grep(prefix, rownames(x$coefficients))
        if (length(rows) > 0) {
          ph <- x$phases[[nm]]
          label <- switch(ph$type,
            cdf = "cdf", hazard = "hazard", constant = "constant",
            g3 = "g3")
          cat("\n  Phase: ", nm, " (", label, ")\n", sep = "")
          sub_table <- x$coefficients[rows, , drop = FALSE]
          # Strip phase prefix from row names for cleaner display
          rownames(sub_table) <- sub(prefix, "  ", rownames(sub_table))
          print(sub_table)
        }
      }
    } else {
      cat("\nCoefficients:\n")
      print(x$coefficients)
    }
  }

  invisible(x)
}

#' Extract coefficients from hazard model
#'
#' @param object A `hazard` object.
#' @param ... Unused; for S3 compatibility.
#' @examples
#' fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' coef(fit)
#' @return A named numeric vector of fitted parameter estimates, or \code{NULL}
#'   if the model has not been fitted (\code{fit = FALSE}).
#' @export
coef.hazard <- function(object, ...) {
  if (is.null(object$fit$theta)) {
    return(NULL)
  }
  object$fit$theta
}

#' Extract variance-covariance matrix from hazard model
#'
#' Returns the estimated variance-covariance matrix of the fitted coefficients.
#'
#' @param object A `hazard` object.
#' @param ... Unused; for S3 compatibility.
#' @examples
#' fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
#'               theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
#' vcov(fit)
#' @return A numeric matrix containing the estimated variance-covariance matrix
#'   of the fitted coefficients, with rows and columns named by the coefficient
#'   labels (phase-prefixed for multiphase models, e.g. \code{early.x}). Rows
#'   and columns for parameters held fixed (e.g. fixed shape parameters) are
#'   \code{NA} because they carry no variance; the finite free-parameter block
#'   is still usable. For Conservation-of-Events fits the conserved phase
#'   \code{log_mu} \emph{normally} carries a variance: it is removed from the
#'   optimizer search but the vcov is the full-information matrix at the optimum
#'   (the CoE solution is the unconstrained MLE). That recomputation requires
#'   \pkg{numDeriv} and an invertible Hessian; if either is unavailable the fit
#'   emits a warning and the conserved \code{log_mu} stays \code{NA} (the rest
#'   of the matrix is unaffected). Returns a scalar \code{NA} only when the
#'   model has not been fitted or no covariance matrix is available.
#' @export
vcov.hazard <- function(object, ...) {
  v <- object$fit$vcov
  if (is.null(v) || !is.matrix(v)) {
    return(NA)
  }
  # Label rows/cols with the coefficient names so callers can align the matrix
  # by name. This matters for multiphase models where the same covariate can
  # enter more than one phase (e.g. early.x vs constant.x): without names the
  # two coefficients are indistinguishable. NA variance rows are retained
  # rather than collapsing the whole matrix to a scalar NA -- a multiphase fit
  # legitimately has NA rows for parameters held fixed (e.g. early shapes),
  # which carry no Hessian-based variance. (The CoE-conserved log_mu is
  # normally NOT NA -- it gets the full-information variance -- but stays NA if
  # that recomputation was unavailable; the fit warns in that case.)
  # The finite free-parameter block is still usable.
  theta <- object$fit$theta
  nm <- names(theta)
  if (is.null(nm) || !all(nzchar(nm))) {
    p <- if (is.null(object$data$x)) 0L else ncol(object$data$x)
    nm <- .hzr_parameter_names(theta = theta, dist = object$spec$dist, p = p)
  }
  if (!is.null(nm) && length(nm) == nrow(v)) {
    dimnames(v) <- list(nm, nm)
  }
  v
}

.hzr_parameter_names <- function(theta, dist, p) {
  theta_names <- names(theta)
  if (!is.null(theta_names) && all(nzchar(theta_names))) {
    return(theta_names)
  }

  if (!is.null(p) && p > 0L && length(theta) == p) {
    return(paste0("beta", seq_along(theta)))
  }

  base_names <- switch(
    dist,
    weibull = c("mu", "nu"),
    exponential = c("log_lambda"),
    loglogistic = c("log_alpha", "log_beta"),
    lognormal = c("mu", "log_sigma"),
    character()
  )

  n_theta <- length(theta)
  n_base <- min(length(base_names), n_theta)
  out <- character(n_theta)

  if (n_base > 0) {
    out[seq_len(n_base)] <- base_names[seq_len(n_base)]
  }
  if (n_theta > n_base) {
    out[seq.int(n_base + 1L, n_theta)] <- paste0("beta", seq_len(n_theta - n_base))
  }
  out
}

#' Coerce x to a validated numeric design matrix
#'
#' Accepts data.frame or numeric matrix input; validates dimensions and
#' finiteness.  Called by hazard() to normalise the x argument before any
#' downstream use.
#'
#' @param x A numeric matrix or data frame with numeric columns.
#' @param n Expected row count (length of time/status vectors); checked if non-NULL.
#' @return A numeric matrix with column names preserved (or added if absent).
#' @keywords internal
#'
.hzr_as_design_matrix <- function(x, n = NULL) {
  if (is.data.frame(x)) {
    x <- data.matrix(x)
  }

  if (!is.matrix(x) || !is.numeric(x)) {
    stop("Predictor input must be a numeric matrix or coercible data frame.", call. = FALSE)
  }

  if (any(!is.finite(x))) {
    stop("Predictor matrix contains non-finite values.", call. = FALSE)
  }

  if (!is.null(n) && nrow(x) != n) {
    stop("Predictor rows must match the length of 'time'.", call. = FALSE)
  }

  x
}

#' Expand predictors for piecewise time-varying coefficients
#'
#' Builds a block design matrix with one predictor block per time window. For each
#' observation, only the block corresponding to that row's time window is active;
#' all other blocks are zero.
#'
#' Example with two predictors and one cut point:
#' - input columns: `x1`, `x2`
#' - windows: `(-Inf, c1]`, `(c1, Inf)`
#' - output columns: `x1_w1`, `x2_w1`, `x1_w2`, `x2_w2`
#'
#' @param x Numeric predictor matrix/data.frame.
#' @param time Numeric time vector used to assign rows to windows.
#' @param time_windows Numeric vector of window cut points.
#' @return Expanded numeric matrix with window-specific column names.
#' @noRd
.hzr_expand_time_varying_design <- function(x, time, time_windows) {
  if (is.data.frame(x)) {
    x <- data.matrix(x)
  }

  if (!is.matrix(x) || !is.numeric(x)) {
    stop("Time-varying design requires numeric matrix/data.frame predictors.", call. = FALSE)
  }

  if (!is.numeric(time) || length(time) != nrow(x) || any(!is.finite(time)) || any(time < 0)) {
    stop("'time' must be finite, non-negative, and match predictor rows for time-varying expansion.", call. = FALSE)
  }

  if (!is.numeric(time_windows) || length(time_windows) < 1) {
    stop("'time_windows' must contain at least one cut point.", call. = FALSE)
  }

  time_windows <- sort(unique(time_windows))
  # Bin index k means observation time falls in window k.
  bins <- cut(time, breaks = c(-Inf, time_windows, Inf), right = TRUE, include.lowest = TRUE, labels = FALSE)
  n_bins <- length(time_windows) + 1L

  base_names <- colnames(x)
  if (is.null(base_names)) {
    base_names <- paste0("x", seq_len(ncol(x)))
  }

  out <- matrix(0, nrow = nrow(x), ncol = ncol(x) * n_bins)
  col_ptr <- 1L
  out_names <- character(ncol(out))

  for (k in seq_len(n_bins)) {
    idx <- bins == k
    # Populate only the active window block for rows in this bin.
    block <- matrix(0, nrow = nrow(x), ncol = ncol(x))
    if (any(idx)) {
      block[idx, ] <- x[idx, , drop = FALSE]
    }
    cols <- col_ptr:(col_ptr + ncol(x) - 1L)
    out[, cols] <- block
    out_names[cols] <- paste0(base_names, "_w", k)
    col_ptr <- col_ptr + ncol(x)
  }

  colnames(out) <- out_names
  out
}

# decomposition.R -- Generalized temporal decomposition
#
# PURPOSE
# -------
# The unified parametric family decompos(t; t_half, nu, m) generates all
# temporal phase shapes used in multiphase hazard models.  It produces three
# quantities: G(t) (CDF), g(t) (density -- "early" pattern), and h(t) =
# g(t)/(1-G(t)) (hazard -- "late" pattern).
#
# Originally introduced by Blackstone, Naftel, and Turner (1986, JASA 81:615)
# and extended to longitudinal mixed-effects settings by Rajeswaran et al.
# (2018, Stat Methods Med Res 27:126).  Ported here from the mixhazard
# package with enhanced numerical stability and integrated phase-level
# helpers for the additive cumulative hazard model:
#
#   H(t | x) = sum_j  mu_j(x) * Phi_j(t; t_half_j, nu_j, m_j)
#
# PARAMETER MAPPING FROM SAS/C HAZARD
# ------------------------------------
# SAS Early (G1): DELTA, RHO/THALF, NU, M  -->  t_half, nu, m
# SAS Late  (G3): TAU, GAMMA, ALPHA, ETA   -->  t_half, nu, m
# SAS Const (G2): (none)                   -->  type = "constant"
#
# The 4-parameter C/SAS parameterizations collapse onto this 3-parameter
# family.  The C DELTA parameter controlled a time transformation
# B(t) = (exp(delta*t) - 1)/delta that is absorbed by the shape.

# ============================================================================
# hzr_decompos -- core engine
# ============================================================================

#' Generalized temporal decomposition
#'
#' Computes the cumulative distribution \eqn{G(t)}, density \eqn{g(t)}, and
#' hazard \eqn{h(t) = g(t)/(1 - G(t))} for the parametric family defined by
#' half-life, time exponent, and shape.  This single function generates all
#' temporal phase shapes used in multiphase hazard models.
#'
#' @section Parameter mapping from SAS/C HAZARD:
#'
#' The original C code used separate parameterizations for early (DELTA,
#' RHO/THALF, NU, M) and late (TAU, GAMMA, ALPHA, ETA) phases.  Both
#' collapse onto the three parameters here.  See
#' [hzr_argument_mapping()] for the full translation table.
#'
#' @section Valid parameter combinations:
#'
#' Six cases are defined by the signs of `nu` and `m`:
#'
#' \tabular{lll}{
#'   **Case** \tab **Sign** \tab **Behavior** \cr
#'   1     \tab m > 0, nu > 0 \tab Standard sigmoidal \cr
#'   1L    \tab m = 0, nu > 0 \tab Exponential-like (Weibull CDF) \cr
#'   2     \tab m < 0, nu > 0 \tab Heavy-tailed \cr
#'   2L    \tab m < 0, nu = 0 \tab Exponential decay \cr
#'   3     \tab m > 0, nu < 0 \tab Bounded cumulative \cr
#'   3L    \tab m = 0, nu < 0 \tab Bounded exponential \cr
#' }
#'
#' The combination m < 0 **and** nu < 0 is undefined and raises an error.
#' `nu = 0` is supported only with m < 0 (Case 2L, the exponential-decay
#' limit); `nu = 0` with m >= 0 has no usable limiting form and raises an
#' error.
#'
#' @section Mathematical form:
#'
#' The construction fixes a rate \eqn{\rho} so that \eqn{G(t_{1/2}) = 0.5}
#' exactly.  For the base case (\eqn{m > 0,\ \nu > 0}):
#'
#' \deqn{\rho = \nu \, t_{1/2} \left(\frac{2^m - 1}{m}\right)^{\!\nu},
#'       \qquad b(t) = \frac{\nu t}{\rho}}
#'
#' The CDF and density are then
#'
#' \deqn{G(t) = \bigl(1 + m \, b(t)^{-1/\nu}\bigr)^{-1/m}, \qquad
#'       g(t) = \bigl(1 + m \, b(t)^{-1/\nu}\bigr)^{-1/m - 1}
#'              \, b(t)^{-1/\nu - 1} / \rho}
#'
#' and the hazard is \eqn{h(t) = g(t) / (1 - G(t))}.  The remaining five cases
#' in the table above arise as limits (\eqn{m \to 0}, \eqn{\nu \to 0}) or sign
#' reflections of this base form; the implementation dispatches to the
#' appropriate branch after inspecting the signs of `nu` and `m`.  See
#' `vignette("mf-mathematical-foundations")` for the full derivation of every
#' case.
#'
#' @param time Numeric vector of times (must be > 0).
#' @param t_half Half-life: time at which \eqn{G(t_{1/2}) = 0.5}.
#'   Must be > 0.
#' @param nu Time exponent controlling rate dynamics.
#'   SAS early: `NU`.  SAS late: relates to `GAMMA`/`ETA`.
#' @param m Shape exponent controlling the distributional form.
#'   SAS early: `M`.  SAS late: relates to `GAMMA`/`ALPHA`.
#'
#' @return A named list with three numeric vectors, each the same length
#'   as `time`:
#' \describe{
#'   \item{G}{Cumulative distribution \eqn{G(t) \in [0, 1]}.}
#'   \item{g}{Density \eqn{g(t) = dG/dt \ge 0}.  The "early" phase
#'     temporal pattern.}
#'   \item{h}{Hazard \eqn{h(t) = g(t)/(1 - G(t)) \ge 0}.  The "late"
#'     phase temporal pattern.}
#' }
#'
#' @references
#' Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
#' hazard into phases, each incorporating a separate stream of concomitant
#' information. *J Am Stat Assoc.* 1986;81(395):615--624.
#' \doi{10.1080/01621459.1986.10478314}
#'
#' Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
#' Probability of atrial fibrillation after ablation: Using a parametric
#' nonlinear temporal decomposition mixed effects model. *Stat Methods Med Res.*
#' 2018;27(1):126--141. \doi{10.1177/0962280215623583}
#'
#' @examples
#' t_grid <- seq(0.1, 10, by = 0.1)
#'
#' # Case 1: standard sigmoidal (m > 0, nu > 0)
#' d1 <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 1)
#' plot(t_grid, d1$G, type = "l", main = "CDF (m=1, nu=2)")
#'
#' # Case 1L: Weibull-like (m = 0, nu > 0)
#' d1L <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 0)
#'
#' # Case 2: heavy-tailed (m < 0, nu > 0)
#' d2 <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = -1)
#'
#' @seealso [hzr_phase_cumhaz()] for the phase-level cumulative hazard
#'   contribution, [hzr_argument_mapping()] for SAS/C parameter mapping,
#'   [hzr_phase()] for specifying phases in [hazard()] models.
#'
#' \code{vignette("mf-mathematical-foundations")} for the full derivation.
#'
#' @export
hzr_decompos <- function(time, t_half, nu, m) {

  # --- Input validation ------------------------------------------------------
  stopifnot(
    "time must be numeric"   = is.numeric(time),
    "t_half must be a positive scalar" =
      is.numeric(t_half) && length(t_half) == 1L && t_half > 0,
    "nu must be a numeric scalar" =
      is.numeric(nu) && length(nu) == 1L && is.finite(nu),
    "m must be a numeric scalar" =
      is.numeric(m) && length(m) == 1L && is.finite(m)
  )

  if (m < 0 && nu < 0) {
    stop("Decomposition undefined when both m < 0 and nu < 0 (m = ",
         m, ", nu = ", nu, ").", call. = FALSE)
  }

  # Clamp time away from zero to avoid division-by-zero / 0^negative

  time <- pmax(time, .Machine$double.xmin)

  # Pre-compute recurring exponent terms
  if (m != 0) mm1 <- -(1 / m) - 1
  if (nu != 0) num1 <- -(1 / nu) - 1

  # --- Case dispatch ---------------------------------------------------------
  if (m > 0 && nu > 0) {
    # Case 1: standard sigmoidal
    rho   <- nu * t_half * (((2^m - 1) / m)^nu)
    bt    <- nu * time / rho
    btnu  <- 1 + m * bt^(-1 / nu)
    G     <- btnu^(-1 / m)
    g     <- (btnu^mm1) * (bt^num1) / rho

  } else if (m == 0 && nu > 0) {
    # Case 1L: Weibull-like (m -> 0 limit)
    rho   <- nu * t_half * (log(2)^nu)
    bt    <- nu * time / rho
    btnu  <- bt^(-1 / nu)
    G     <- exp(-btnu)
    g     <- G * (bt^num1) / rho

  } else if (m < 0 && nu > 0) {
    # Case 2: heavy-tailed
    rho   <- nu * t_half / ((1 - 2^m)^(-nu) - 1)
    bt    <- 1 + nu * time / rho
    btnu  <- 1 - bt^(-1 / nu)
    G     <- btnu^(-1 / m)
    g     <- -(btnu^mm1) * (bt^num1) / (m * rho)

  } else if (m < 0 && nu == 0) {
    # Case 2L: exponential decay (nu -> 0 limit)
    rho   <- -t_half / log(1 - 2^m)
    bt    <- exp(-time / rho)
    btm   <- 1 - bt
    G     <- btm^(-1 / m)
    g     <- -(btm^mm1) * bt / (m * rho)

  } else if (m > 0 && nu < 0) {
    # Case 3: bounded cumulative (C HAZARD g1flag = 5, "Mixed Generic")
    # rho uses the (2^m - 1)/m form (matching Case 1), NOT a bare (2^m - 1).
    # Without the /m divisor the CDF carried a spurious factor of m on the
    # bt^(-1/nu) term, diverging from the C G1 evaluator by up to ~0.2 and
    # breaking continuity with the m -> 0 limit (Case 3L).  With /m the m
    # factors cancel and G reduces to 1 - (bt^(-1/nu) + 1)^(-1/m), matching
    # C exactly (verified against src/common/hzd_ln_G1_and_SG1.c case 5).
    rho   <- -nu * t_half * (((2^m - 1) / m)^nu)
    bt    <- -nu * time / rho
    btnu  <- 1 + m * bt^(-1 / nu)
    G     <- 1 - btnu^(-1 / m)
    g     <- (btnu^mm1) * (bt^num1) / rho

  } else if (m == 0 && nu < 0) {
    # Case 3L: bounded exponential (m -> 0 limit)
    rho   <- -nu * t_half * (log(2)^nu)
    bt    <- -nu * time / rho
    btnu  <- bt^(-1 / nu)
    G     <- 1 - exp(-btnu)
    g     <- exp(-btnu) * (bt^num1) / rho

  } else {
    # Remaining combination: nu == 0 with m >= 0.  The nu -> 0 limit is only
    # defined for m < 0 (Case 2L, exponential decay); for m >= 0 the limit is
    # degenerate (G collapses to a step function), so there is no usable form.
    # Fail loud rather than leaving G/g unassigned (which would raise the
    # cryptic "object 'G' not found").
    stop("Decomposition undefined for nu = ", nu, " with m = ", m, ". ",
         "The nu -> 0 limit is implemented only for m < 0; ",
         "supply a nonzero nu when m >= 0.", call. = FALSE)
  }

  # --- Hazard from density and CDF ------------------------------------------
  # h(t) = g(t) / (1 - G(t)), with guard against G(t) = 1
  one_minus_G <- pmax(1 - G, .Machine$double.xmin)
  h <- g / one_minus_G

  list(G = G, g = g, h = h)
}


# ============================================================================
# hzr_decompos_g3 -- Late-phase (G3) engine
# ============================================================================

#' Late-phase (G3) temporal decomposition
#'
#' Computes the cumulative intensity \eqn{G_3(t)} and its derivative
#' \eqn{g_3(t) = dG_3/dt} for the late-phase parametric family used in the
#' original Blackstone C/SAS HAZARD code.  Unlike [hzr_decompos()] (which
#' computes the early-phase G1 -- a bounded CDF), this function can produce
#' **unbounded** values, making it suitable for modelling increasing late risk.
#'
#' @section Mathematical form:
#'
#' When \eqn{\alpha > 0}:
#' \deqn{G_3(t) = \bigl(\bigl((t/\tau)^\gamma + 1\bigr)^{1/\alpha}
#'                       - 1\bigr)^\eta}
#'
#' When \eqn{\alpha = 0} (limiting exponential case):
#' \deqn{G_3(t) = \bigl(\exp\bigl((t/\tau)^\gamma\bigr) - 1\bigr)^\eta}
#'
#' @section Parameter mapping from SAS/C HAZARD:
#'
#' | SAS name | R argument | Role |
#' |----------|-----------|------|
#' | TAU      | `tau`     | Scale (time at which \eqn{(t/\tau) = 1}) |
#' | GAMMA    | `gamma`   | Power exponent on \eqn{t/\tau} |
#' | ALPHA    | `alpha`   | Shape (0 = exponential limiting case) |
#' | ETA      | `eta`     | Outer power exponent |
#'
#' @param time Numeric vector of times (must be > 0).
#' @param tau Positive scalar scale parameter.
#' @param gamma Positive scalar time exponent.
#' @param alpha Non-negative scalar shape parameter (0 selects limiting case).
#' @param eta Positive scalar outer exponent.
#'
#' @return A named list with two numeric vectors, each the same length
#'   as `time`:
#' \describe{
#'   \item{G3}{Cumulative intensity \eqn{G_3(t) \ge 0} (may exceed 1).}
#'   \item{g3}{Derivative \eqn{g_3(t) = dG_3/dt \ge 0}.}
#' }
#'
#' @references
#' Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
#' hazard into phases, each incorporating a separate stream of concomitant
#' information. *J Am Stat Assoc.* 1986;81(395):615--624.
#' \doi{10.1080/01621459.1986.10478314}
#'
#' @examples
#' t_grid <- seq(0.1, 10, by = 0.1)
#'
#' # Weibull-like: alpha = 1 gives G3(t) = (t/tau)^(gamma*eta)
#' d <- hzr_decompos_g3(t_grid, tau = 1, gamma = 3, alpha = 1, eta = 1)
#' plot(t_grid, d$G3, type = "l", main = "G3: power law (gamma=3)")
#'
#' # General case with alpha > 0
#' d2 <- hzr_decompos_g3(t_grid, tau = 2, gamma = 2, alpha = 0.5, eta = 1)
#'
#' @seealso [hzr_decompos()] for the early-phase (G1) decomposition,
#'   [hzr_phase_cumhaz()] for phase-level cumulative hazard helpers.
#'
#' @export
hzr_decompos_g3 <- function(time, tau, gamma, alpha, eta) {

  # --- Input validation ------------------------------------------------------
  # Soft checks: return Inf/0 for infeasible params (optimizer needs graceful

  # failure) rather than hard stops.
  if (!is.numeric(time)) stop("time must be numeric")
  if (!is.numeric(tau) || length(tau) != 1L) stop("tau must be a scalar")
  if (!is.numeric(gamma) || length(gamma) != 1L) stop("gamma must be a scalar")
  if (!is.numeric(alpha) || length(alpha) != 1L) stop("alpha must be a scalar")

  if (!is.numeric(eta) || length(eta) != 1L) stop("eta must be a scalar")

  # Return Inf/NaN for infeasible parameters (optimizer will see -Inf logl)
  if (tau <= 0 || gamma <= 0 || eta <= 0 || alpha < 0 || !is.finite(alpha)) {
    return(list(G3 = rep(Inf, length(time)), g3 = rep(NaN, length(time))))
  }

  # Clamp time away from zero
  time <- pmax(time, .Machine$double.xmin)

  # --- Common terms ----------------------------------------------------------
  # Work on log scale for numerical stability, mirroring the C implementation
  ln_t_tau  <- log(time / tau)                    # ln(T/tau)
  ln_t_tau_g <- gamma * ln_t_tau                   # gamma * ln(T/tau)

  # --- Case dispatch: alpha > 0 vs alpha = 0 --------------------------------
  if (alpha > 0) {
    # G3(t) = (((t/tau)^gamma + 1)^(1/alpha) - 1)^eta
    #
    # On log scale:
    #   tGamma = ln((t/tau)^gamma + 1) = ln(exp(ln_t_tau_g) + 1) = log1p(exp(ln_t_tau_g))
    #   tEta   = ln(exp(tGamma/alpha) - 1) = ln(expm1(tGamma/alpha))
    #   lnG3   = eta * tEta

    tGamma <- .log1pexp(ln_t_tau_g)   # ln((t/tau)^gamma + 1)
    inner  <- tGamma / alpha           # ln((t/tau)^gamma + 1) / alpha
    tEta   <- .log_expm1(inner)        # ln(exp(inner) - 1)
    lnG3   <- eta * tEta

    G3 <- exp(lnG3)

    # g3(t) = dG3/dt
    # lnScale = ln(gamma) + ln(eta) - ln(tau) - ln(alpha)
    # lnSG3 = lnScale + (eta-1)*tEta + ((1-alpha)/alpha)*tGamma + (gamma-1)*ln(t/tau)
    lnScale <- log(gamma) + log(eta) - log(tau) - log(alpha)
    lnSG3   <- lnScale + (eta - 1) * tEta +
               ((1 - alpha) / alpha) * tGamma +
               (gamma - 1) * ln_t_tau

    g3 <- exp(lnSG3)

  } else {
    # alpha = 0 limiting case:
    # G3(t) = (exp((t/tau)^gamma) - 1)^eta
    #
    # tGamma = ln(exp(exp(ln_t_tau_g)) - 1)
    #        = ln(exp((t/tau)^gamma) - 1)
    #        = log(expm1((t/tau)^gamma))
    # lnG3 = eta * tGamma

    t_tau_g <- exp(ln_t_tau_g)         # (t/tau)^gamma
    tGamma  <- .log_expm1(t_tau_g)     # ln(exp((t/tau)^gamma) - 1)
    lnG3    <- eta * tGamma

    G3 <- exp(lnG3)

    # lnScale = ln(gamma) + ln(eta) - ln(tau)
    # lnSG3 = lnScale + (eta-1)*tGamma + (t/tau)^gamma + (gamma-1)*ln(t/tau)
    lnScale <- log(gamma) + log(eta) - log(tau)
    lnSG3   <- lnScale + (eta - 1) * tGamma +
               t_tau_g +
               (gamma - 1) * ln_t_tau

    g3 <- exp(lnSG3)
  }

  list(G3 = G3, g3 = g3)
}


# --- Numerically stable helpers for G3 decomposition -----------------------

#' Compute log(1 + exp(x)) with overflow/underflow protection
#' @param x Numeric vector.
#' @return Numeric vector: log(1 + exp(x)).
#' @keywords internal
.log1pexp <- function(x) {
  # For large x: log(1+exp(x)) ~ x
  # For moderate x: use log1p(exp(x))
  # For very negative x: log(1+exp(x)) ~ exp(x)
  out <- x
  big  <- x > 35
  mid  <- !big & x > -35
  small <- x <= -35

  out[big]   <- x[big]
  out[mid]   <- log1p(exp(x[mid]))
  out[small] <- exp(x[small])

  out
}


#' Compute log(exp(x) - 1) with numerical stability
#' @param x Numeric vector (must be > 0 for exp(x) > 1).
#' @return Numeric vector: log(exp(x) - 1).
#' @keywords internal
.log_expm1 <- function(x) {
  # For large x: log(exp(x) - 1) ~ x
  # For small positive x: use log(expm1(x)) for accuracy
  out <- x
  big <- x > 35
  ok  <- !big & x > 1e-10
  tiny <- x <= 1e-10

  out[big] <- x[big]
  out[ok]  <- log(expm1(x[ok]))
  # For very small x: expm1(x) ~ x, so log(expm1(x)) ~ log(x)
  out[tiny] <- log(pmax(x[tiny], .Machine$double.xmin))

  out
}


# ============================================================================
# Phase-level helpers for the additive cumulative hazard model
# ============================================================================

#' Cumulative hazard contribution from a single phase
#'
#' Computes \eqn{\Phi_j(t)} for one phase in the additive model
#' \eqn{H(t|x) = \sum_j \mu_j(x) \Phi_j(t)}.
#'
#' @param time Numeric vector of times (> 0).
#' @param t_half Half-life parameter (> 0).
#' @param nu Time exponent.
#' @param m Shape parameter.
#' @param type Phase type: `"cdf"` (early -- uses \eqn{G(t)}),
#'   `"hazard"` (late -- uses cumulative hazard from \eqn{h(t)}), or
#'   `"constant"` (flat rate -- \eqn{\Phi = t}).
#'
#' @return Numeric vector of cumulative hazard contributions \eqn{\Phi(t)},
#'   same length as `time`.
#'
#' @details
#' - `"cdf"`: \eqn{\Phi(t) = G(t)}.  Bounded \eqn{[0, 1]}.  Models early
#'   risk that resolves over time.
#' - `"hazard"`: \eqn{\Phi(t) = -\log(1 - G(t))}.  Monotone increasing.
#'   Models late or aging risk.  This is the cumulative hazard derived from
#'   the hazard function \eqn{h(t)}, since
#'   \eqn{\int_0^t h(s)\,ds = -\log(1 - G(t))}.
#' - `"constant"`: \eqn{\Phi(t) = t}.  Ignores `t_half`, `nu`, `m`.
#'   Equivalent to exponential (constant hazard rate).
#'
#' @examples
#' t_grid <- seq(0.1, 10, by = 0.1)
#' phi_early <- hzr_phase_cumhaz(t_grid, t_half = 2, nu = 2, m = 0,
#'                                type = "cdf")
#' phi_late  <- hzr_phase_cumhaz(t_grid, t_half = 5, nu = 1, m = 0,
#'                                type = "hazard")
#' phi_const <- hzr_phase_cumhaz(t_grid, type = "constant")
#'
#' @seealso [hzr_decompos()] for the underlying parametric family,
#'   [hzr_phase_hazard()] for the instantaneous hazard contribution.
#'
#' @export
hzr_phase_cumhaz <- function(time, t_half = 1, nu = 1, m = 0,
                              type = c("cdf", "hazard", "constant")) {
  type <- match.arg(type)

  if (type == "constant") {
    return(time)
  }

  d <- hzr_decompos(time, t_half = t_half, nu = nu, m = m)

  switch(type,
    cdf    = d$G,
    hazard = {
      # Cumulative hazard from h(t): integral_0^t h(s)ds = -log(1 - G(t))
      one_minus_G <- pmax(1 - d$G, .Machine$double.xmin)
      -log(one_minus_G)
    }
  )
}


# ============================================================================
# Phase-level derivatives for analytic gradient
# ============================================================================

#' Derivatives of phase cumulative and instantaneous hazard w.r.t. shape params
#'
#' Computes \eqn{\Phi_j(t)}, \eqn{\phi_j(t)}, and their derivatives with
#' respect to `t_half`, `nu`, and `m` using central finite differences on
#' [hzr_decompos()].  The `log_t_half` derivative is obtained via the chain
#' rule: \eqn{d\Phi/d(\log t_{1/2}) = t_{1/2} \cdot d\Phi/dt_{1/2}}.
#'
#' @param time Numeric vector of positive times.
#' @param t_half Positive scalar half-life.
#' @param nu Numeric scalar time exponent.
#' @param m Numeric scalar shape exponent.
#' @param type Phase type: `"cdf"`, `"hazard"`, or `"constant"`.
#'
#' @return Named list:
#' \describe{
#'   \item{Phi}{Cumulative hazard contribution \eqn{\Phi(t)}.}
#'   \item{phi}{Instantaneous hazard contribution \eqn{\phi(t) = d\Phi/dt}.}
#'   \item{dPhi_dlog_thalf}{\eqn{d\Phi / d(\log t_{1/2})}.}
#'   \item{dPhi_dnu}{\eqn{d\Phi / d\nu}.}
#'   \item{dPhi_dm}{\eqn{d\Phi / dm}.}
#'   \item{dphi_dlog_thalf}{\eqn{d\phi / d(\log t_{1/2})}.}
#'   \item{dphi_dnu}{\eqn{d\phi / d\nu}.}
#'   \item{dphi_dm}{\eqn{d\phi / dm}.}
#' }
#' @keywords internal
.hzr_phase_derivatives <- function(time, t_half, nu, m,
                                    type = c("cdf", "hazard", "constant")) {
  type <- match.arg(type)
  n <- length(time)

  # Constant phases: Phi = t, phi = 1, no shape dependence

  if (type == "constant") {
    zeros <- rep(0, n)
    return(list(
      Phi = time,
      phi = rep(1, n),
      dPhi_dlog_thalf = zeros,
      dPhi_dnu        = zeros,
      dPhi_dm         = zeros,
      dphi_dlog_thalf = zeros,
      dphi_dnu        = zeros,
      dphi_dm         = zeros
    ))
  }

  # Base evaluation
  d0 <- hzr_decompos(time, t_half = t_half, nu = nu, m = m)

  # Extract Phi and phi from decomposition for the given phase type
  extract <- function(d, tp) {
    if (tp == "cdf") {
      Phi <- d$G
      phi <- d$g
    } else {
      # "hazard": Phi = -log(1 - G), phi = h
      one_minus_G <- pmax(1 - d$G, .Machine$double.xmin)
      Phi <- -log(one_minus_G)
      phi <- d$h
    }
    list(Phi = Phi, phi = phi)
  }

  base <- extract(d0, type)

  # Central-difference derivatives w.r.t. shape parameters
  eps_rel <- (.Machine$double.eps) ^ (1 / 3)  # optimal for central diff

  # Helper: perturbed decomposition (returns NULL if invalid params)
  perturb_decompos <- function(th, n_val, m_val) {
    if (m_val < 0 && n_val < 0) return(NULL)
    if (th <= 0) return(NULL)
    tryCatch(
      hzr_decompos(time, t_half = th, nu = n_val, m = m_val),
      error = function(e) NULL
    )
  }

  # Derivative w.r.t. t_half (then multiply by t_half for log_t_half chain rule)
  h_th <- eps_rel * max(abs(t_half), 1e-4)
  d_plus  <- perturb_decompos(t_half + h_th, nu, m)
  d_minus <- perturb_decompos(t_half - h_th, nu, m)
  if (!is.null(d_plus) && !is.null(d_minus)) {
    e_plus  <- extract(d_plus, type)
    e_minus <- extract(d_minus, type)
    dPhi_dt_half <- (e_plus$Phi - e_minus$Phi) / (2 * h_th)
    dphi_dt_half <- (e_plus$phi - e_minus$phi) / (2 * h_th)
  } else {
    # One-sided fallback
    if (!is.null(d_plus)) {
      e_plus <- extract(d_plus, type)
      dPhi_dt_half <- (e_plus$Phi - base$Phi) / h_th
      dphi_dt_half <- (e_plus$phi - base$phi) / h_th
    } else {
      dPhi_dt_half <- rep(0, n)
      dphi_dt_half <- rep(0, n)
    }
  }
  # Chain rule: d/d(log_t_half) = t_half * d/d(t_half)
  dPhi_dlog_thalf <- t_half * dPhi_dt_half
  dphi_dlog_thalf <- t_half * dphi_dt_half

  # Derivative w.r.t. nu
  h_nu <- eps_rel * max(abs(nu), 1)
  d_plus  <- perturb_decompos(t_half, nu + h_nu, m)
  d_minus <- perturb_decompos(t_half, nu - h_nu, m)
  if (!is.null(d_plus) && !is.null(d_minus)) {
    e_plus  <- extract(d_plus, type)
    e_minus <- extract(d_minus, type)
    dPhi_dnu <- (e_plus$Phi - e_minus$Phi) / (2 * h_nu)
    dphi_dnu <- (e_plus$phi - e_minus$phi) / (2 * h_nu)
  } else if (!is.null(d_plus)) {
    e_plus <- extract(d_plus, type)
    dPhi_dnu <- (e_plus$Phi - base$Phi) / h_nu
    dphi_dnu <- (e_plus$phi - base$phi) / h_nu
  } else if (!is.null(d_minus)) {
    e_minus <- extract(d_minus, type)
    dPhi_dnu <- (base$Phi - e_minus$Phi) / h_nu
    dphi_dnu <- (base$phi - e_minus$phi) / h_nu
  } else {
    dPhi_dnu <- rep(0, n)
    dphi_dnu <- rep(0, n)
  }

  # Derivative w.r.t. m
  h_m <- eps_rel * max(abs(m), 1)
  d_plus  <- perturb_decompos(t_half, nu, m + h_m)
  d_minus <- perturb_decompos(t_half, nu, m - h_m)
  if (!is.null(d_plus) && !is.null(d_minus)) {
    e_plus  <- extract(d_plus, type)
    e_minus <- extract(d_minus, type)
    dPhi_dm <- (e_plus$Phi - e_minus$Phi) / (2 * h_m)
    dphi_dm <- (e_plus$phi - e_minus$phi) / (2 * h_m)
  } else if (!is.null(d_plus)) {
    e_plus <- extract(d_plus, type)
    dPhi_dm <- (e_plus$Phi - base$Phi) / h_m
    dphi_dm <- (e_plus$phi - base$phi) / h_m
  } else if (!is.null(d_minus)) {
    e_minus <- extract(d_minus, type)
    dPhi_dm <- (base$Phi - e_minus$Phi) / h_m
    dphi_dm <- (base$phi - e_minus$phi) / h_m
  } else {
    dPhi_dm <- rep(0, n)
    dphi_dm <- rep(0, n)
  }

  list(
    Phi             = base$Phi,
    phi             = base$phi,
    dPhi_dlog_thalf = dPhi_dlog_thalf,
    dPhi_dnu        = dPhi_dnu,
    dPhi_dm         = dPhi_dm,
    dphi_dlog_thalf = dphi_dlog_thalf,
    dphi_dnu        = dphi_dnu,
    dphi_dm         = dphi_dm
  )
}


#' Instantaneous hazard contribution from a single phase
#'
#' Computes \eqn{\phi_j(t) = d\Phi_j/dt} for one phase -- the derivative of
#' the cumulative hazard contribution returned by [hzr_phase_cumhaz()].
#'
#' @inheritParams hzr_phase_cumhaz
#'
#' @return Numeric vector of instantaneous hazard contributions \eqn{\phi(t)},
#'   same length as `time`.
#'
#' @details
#' - `"cdf"`: \eqn{\phi(t) = g(t)} (density).
#' - `"hazard"`: \eqn{\phi(t) = h(t) = g(t)/(1-G(t))}.
#' - `"constant"`: \eqn{\phi(t) = 1}.
#'
#' @examples
#' t_grid <- seq(0.1, 10, by = 0.1)
#' phi_early <- hzr_phase_hazard(t_grid, t_half = 2, nu = 2, m = 0,
#'                                type = "cdf")
#' phi_late  <- hzr_phase_hazard(t_grid, t_half = 5, nu = 1, m = 0,
#'                                type = "hazard")
#'
#' @seealso [hzr_decompos()] for the underlying parametric family,
#'   [hzr_phase_cumhaz()] for the cumulative version.
#'
#' @export
hzr_phase_hazard <- function(time, t_half = 1, nu = 1, m = 0,
                              type = c("cdf", "hazard", "constant")) {
  type <- match.arg(type)

  if (type == "constant") {
    return(rep(1, length(time)))
  }

  d <- hzr_decompos(time, t_half = t_half, nu = nu, m = m)

  switch(type,
    cdf    = d$g,
    hazard = d$h
  )
}

# stepwise.R -- user-facing driver for Phase 4b stepwise covariate
# selection.  Step 8.6 of STEPWISE-DESIGN.md.
#
# The driver combines .hzr_stepwise_forward_step() and
# .hzr_stepwise_backward_step() into a two-way loop with:
#
#   * SAS-style MOVE oscillation guard: when a variable hits
#     `max_move` entries/exits, it is frozen (added to both the
#     force_in and force_out sets used internally).
#   * `max_steps` hard cap that warns on hit.
#   * `force_in` / `force_out` user-supplied constraints.
#   * Optional console trace mirroring the sec.5 output spec.
#
# The returned object inherits from both `hzr_stepwise` and `hazard`
# so it can be passed to `predict()`, `summary()`, `coef()`, etc.
# through the same S3 infrastructure as a plain fit.

#' Stepwise covariate selection for a parametric hazard model
#'
#' Run forward, backward, or two-way stepwise selection on an existing
#' `hazard` fit using Wald p-values or AIC deltas as the entry /
#' retention criterion.  Phase-specific entry is supported for
#' multiphase models: a covariate can enter one phase and not another.
#'
#' @section Selection direction and criterion:
#'
#' Two arguments shape the search.  `direction` decides which moves are
#' allowed at each step; `criterion` decides how a candidate move is scored
#' and whether it is accepted.
#'
#' \describe{
#'   \item{`direction = "forward"`}{Start from the base model and only
#'     *add* variables --- the best eligible candidate enters each step
#'     until none clears the entry rule.  Variables never leave once in.}
#'   \item{`direction = "backward"`}{Start from the full candidate model and
#'     only *drop* variables --- the weakest term leaves each step until all
#'     survivors clear the retention rule.}
#'   \item{`direction = "both"` (default)}{Two-way stepwise: after each
#'     entry, already-selected variables are re-tested and may be dropped.
#'     This is the SAS `SELECTION = STEPWISE` strategy.  `max_move` caps how
#'     often a single variable may oscillate before it is frozen.}
#' }
#'
#' \describe{
#'   \item{`criterion = "wald"` (default)}{Accept moves on SAS-style
#'     significance thresholds, using the Wald \eqn{\chi^2} of the affected
#'     coefficient(s): a candidate enters if its p-value is below `slentry`, and
#'     a term is dropped if its p-value rises above `slstay`.  Entry candidates
#'     are scored from a refit that adds the candidate (so its new coefficient
#'     can be tested); drop candidates are scored from the *current* model's
#'     Wald p-values without a per-candidate refit, and a single refit is run
#'     only after a drop is chosen.  Note this differs algorithmically from
#'     C/SAS HAZARD, which selects on a *score* (Q) statistic evaluated without
#'     refitting; the two can take different step paths even when they converge
#'     to a similar final model.}
#'   \item{`criterion = "aic"`}{Accept any move with
#'     \eqn{\Delta\mathrm{AIC} < 0} (a strictly better penalised fit), ignoring
#'     `slentry` / `slstay`.  Entry candidates use the actual
#'     \eqn{\Delta\mathrm{AIC}} from the candidate refit; drop candidates use a
#'     Wald-to-likelihood-ratio approximation,
#'     \eqn{\Delta\mathrm{AIC} \approx W - 2\,\mathrm{df}}, computed from the
#'     current model without a per-candidate refit (the chosen drop is refit
#'     afterwards).  Use this for a non-significance-based,
#'     information-criterion search.}
#' }
#'
#' @param fit A fitted `hazard` object built via the
#'   `formula = Surv(...) ~ predictors, data = df` interface.
#' @param scope Candidate set.  `NULL` (default) uses every data-frame
#'   column not already in the model for every phase.  For
#'   single-distribution fits, pass a one-sided formula
#'   (`~ age + nyha`) or a character vector of names.  For multiphase
#'   fits, pass a named list of one-sided formulas keyed by phase.
#' @param data Data frame the base fit was built on.  Required for
#'   refits.
#' @param direction Search strategy --- one of `"both"` (default),
#'   `"forward"`, or `"backward"`.  Controls whether variables may only
#'   enter, only leave, or both.  See the **Selection direction and
#'   criterion** section.
#' @param criterion Entry / retention rule --- one of `"wald"` (default)
#'   or `"aic"`.  `"wald"` applies SAS-style p-value thresholds
#'   (`slentry` / `slstay`); `"aic"` adds or drops whenever it lowers the
#'   AIC.  See the **Selection direction and criterion** section.
#' @param slentry Entry p-value threshold for the Wald criterion.
#'   Default `0.30` matches SAS `SLENTRY`.
#' @param slstay Retention p-value threshold for the Wald criterion.
#'   Default `0.20` matches SAS `SLSTAY`.
#' @param max_steps Hard cap on total accepted actions.  Emits a
#'   `warning()` if hit.  Default `50`.
#' @param max_move Per-variable oscillation cap.  When a variable has
#'   entered + exited more than `max_move` times it is frozen for the
#'   remainder of the run.  Default `4`.
#' @param force_in Character vector of variables that must remain in
#'   the model.  Such variables are still scored and reported in the
#'   selection trace, but are never dropped.
#' @param force_out Character vector of variables that may never be
#'   considered as candidates.
#' @param trace Logical; print step-by-step progress to the console.
#'   Default `TRUE`.
#' @param ... Passed to the underlying `hazard()` refits (e.g.
#'   `control = list(n_starts = 3)`).
#'
#' @return An object of class `c("hzr_stepwise", "hazard")` -- the
#'   final fit augmented with:
#'   \describe{
#'     \item{\code{steps}}{Data frame with one row per accepted /
#'       frozen action; see Details.}
#'     \item{\code{scope}}{Record of the candidate scope, plus
#'       `force_in`, `force_out`, and the frozen set.}
#'     \item{\code{criteria}}{Named list of the threshold / direction
#'       settings actually applied.}
#'     \item{\code{trace_msg}}{Character vector of the trace lines,
#'       captured regardless of the `trace` flag.}
#'     \item{\code{elapsed}}{`difftime` from start to finish.}
#'     \item{\code{final_call}}{The call that produced this result.}
#'   }
#'
#' @details
#' The `steps` data frame has columns:
#'
#' \describe{
#'   \item{\code{step_num}}{Integer sequence starting at 1.}
#'   \item{\code{action}}{`"enter"`, `"drop"`, or `"frozen"`.}
#'   \item{\code{variable}}{Variable affected.}
#'   \item{\code{phase}}{Phase name (multiphase) or `NA_character_`.}
#'   \item{\code{criterion}}{`"wald"` or `"aic"`.}
#'   \item{\code{score}}{Winning score used for the decision.}
#'   \item{\code{stat}, \code{df}}{Wald statistic and degrees of
#'     freedom.}
#'   \item{\code{p_value}, \code{delta_aic}}{Always populated when
#'     computable, regardless of the active criterion.}
#'   \item{\code{logLik}, \code{aic}, \code{n_coef}}{Goodness-of-fit
#'     diagnostics of the model *after* this step.}
#' }
#'
#' @examples
#' data(avc)
#' avc <- na.omit(avc)
#' base <- hazard(survival::Surv(int_dead, dead) ~ age,
#'                data = avc, dist = "weibull", fit = TRUE)
#' \donttest{
#' sw <- hzr_stepwise(base, scope = ~ age + nyha,
#'                    data = avc, direction = "forward",
#'                    control = list(n_starts = 1))
#' print(sw)
#' }
#'
#' @seealso [hazard()] for the base model and [hzr_phase()] for multiphase
#'   scopes; [stepwise_trace()] to retrieve the captured selection log.
#' @export
hzr_stepwise <- function(fit,
                         scope     = NULL,
                         data,
                         direction = c("both", "forward", "backward"),
                         criterion = c("wald", "aic"),
                         slentry   = 0.30,
                         slstay    = 0.20,
                         max_steps = 50L,
                         max_move  = 4L,
                         force_in  = character(),
                         force_out = character(),
                         trace     = TRUE,
                         ...) {
  direction <- match.arg(direction)
  criterion <- match.arg(criterion)

  if (!inherits(fit, "hazard")) {
    stop("`fit` must be a `hazard` object.", call. = FALSE)
  }
  if (missing(data) || !is.data.frame(data)) {
    stop("`data` must be a data frame (typically the frame used for the base fit).",
         call. = FALSE)
  }

  ts_start <- Sys.time()
  call <- match.call()

  extra_args <- list(...)

  steps     <- list()
  trace_msg <- character()

  emit <- function(msg) {
    trace_msg[[length(trace_msg) + 1L]] <<- msg
    if (isTRUE(trace)) cat(msg, "\n", sep = "")
  }

  # Header line (mirrors design sec.5).
  header <- if (criterion == "wald") {
    sprintf(
      "Stepwise selection (direction = %s, criterion = wald, slentry = %.2f, slstay = %.2f)",
      direction, slentry, slstay
    )
  } else {
    sprintf(
      "Stepwise selection (direction = %s, criterion = aic)",
      direction
    )
  }
  emit(header)
  emit("")

  # Move counter: per-variable tally of entries + exits.  Use a named
  # list rather than a named integer vector -- `lst[[missing]]` returns
  # NULL, whereas `vec[[missing]]` errors with "subscript out of bounds".
  move_counts <- list()
  frozen      <- character()

  current <- fit
  step_no <- 0L
  stopped_by_max_steps <- FALSE

  record_step <- function(action, out) {
    step_no <<- step_no + 1L
    row <- data.frame(
      step_num  = step_no,
      action    = action,
      variable  = out$variable,
      phase     = out$phase,
      criterion = criterion,
      score     = out$score,
      stat      = out$stat,
      df        = out$df,
      p_value   = out$p_value,
      delta_aic = out$delta_aic,
      logLik    = current$fit$objective   %||% NA_real_,
      aic       = .hzr_aic(current),
      n_coef    = length(current$fit$theta),
      stringsAsFactors = FALSE
    )
    steps[[length(steps) + 1L]] <<- row

    score_fmt <- if (criterion == "wald") {
      sprintf("p = %.3f", out$p_value)
    } else {
      sprintf("\u0394AIC = %+.2f", out$delta_aic)
    }
    phase_txt <- if (is.na(out$phase)) {
      ""
    } else if (action == "enter") {
      paste0("  into  ", out$phase)
    } else {
      paste0("  from  ", out$phase)
    }
    emit(sprintf(
      "Step %d: %-6s %s%s   (%s)",
      step_no, toupper(action), out$variable, phase_txt, score_fmt
    ))
  }

  record_freeze <- function(var, phase_hint = NA_character_) {
    step_no <<- step_no + 1L
    row <- data.frame(
      step_num  = step_no,
      action    = "frozen",
      variable  = var,
      phase     = phase_hint,
      criterion = criterion,
      score     = NA_real_,
      stat      = NA_real_,
      df        = NA_integer_,
      p_value   = NA_real_,
      delta_aic = NA_real_,
      logLik    = current$fit$objective   %||% NA_real_,
      aic       = .hzr_aic(current),
      n_coef    = length(current$fit$theta),
      stringsAsFactors = FALSE
    )
    steps[[length(steps) + 1L]] <<- row
    emit(sprintf(
      "Step %d: FROZEN %s   (exceeded max_move = %d; OSCILLATING)",
      step_no, var, max_move
    ))
  }

  bump_move <- function(var) {
    move_counts[[var]] <<- (move_counts[[var]] %||% 0L) + 1L
    if (move_counts[[var]] > max_move && !var %in% frozen) {
      frozen <<- c(frozen, var)
      record_freeze(var)
    }
  }

  # Main loop
  repeat {
    if (step_no >= max_steps) {
      warning("Stepwise selection hit max_steps = ", max_steps,
              "; stopping early.", call. = FALSE)
      stopped_by_max_steps <- TRUE
      break
    }

    add_happened  <- FALSE
    drop_happened <- FALSE

    effective_force_out <- unique(c(force_out, frozen))
    effective_force_in  <- unique(c(force_in,  frozen))

    if (direction %in% c("forward", "both")) {
      fwd <- do.call(.hzr_stepwise_forward_step, c(list(
        current   = current,
        scope     = scope,
        data      = data,
        criterion = criterion,
        slentry   = slentry,
        force_out = effective_force_out
      ), extra_args))

      if (fwd$accepted) {
        current <- fwd$fit
        record_step("enter", fwd)
        bump_move(fwd$variable)
        add_happened <- TRUE
      }
    }

    if (direction %in% c("backward", "both")) {
      bwd <- do.call(.hzr_stepwise_backward_step, c(list(
        current   = current,
        data      = data,
        criterion = criterion,
        slstay    = slstay,
        force_in  = effective_force_in
      ), extra_args))

      if (bwd$accepted) {
        current <- bwd$fit
        record_step("drop", bwd)
        bump_move(bwd$variable)
        drop_happened <- TRUE
      }
    }

    if (!add_happened && !drop_happened) {
      emit(sprintf("(no further action after %d step%s)",
                   step_no, if (step_no == 1L) "" else "s"))
      break
    }
  }

  elapsed <- difftime(Sys.time(), ts_start, units = "secs")

  emit("")
  emit(sprintf("Final model: %d covariate%s, logLik = %.2f, AIC = %.2f",
               max(0L, length(current$fit$theta) -
                     .hzr_stepwise_shape_count(current)),
               if (length(current$fit$theta) -
                     .hzr_stepwise_shape_count(current) == 1L) "" else "s",
               current$fit$objective %||% NA_real_,
               .hzr_aic(current)))

  steps_df <- if (length(steps) == 0L) {
    data.frame(
      step_num = integer(), action = character(),
      variable = character(), phase = character(),
      criterion = character(), score = numeric(),
      stat = numeric(), df = integer(),
      p_value = numeric(), delta_aic = numeric(),
      logLik = numeric(), aic = numeric(), n_coef = integer(),
      stringsAsFactors = FALSE
    )
  } else {
    do.call(rbind, steps)
  }

  result <- current
  result$steps      <- steps_df
  result$scope      <- list(
    candidates = scope,
    force_in   = force_in,
    force_out  = force_out,
    frozen     = frozen
  )
  result$criteria   <- list(
    direction = direction,
    criterion = criterion,
    slentry   = slentry,
    slstay    = slstay,
    max_steps = max_steps,
    max_move  = max_move,
    hit_max_steps = stopped_by_max_steps
  )
  result$trace_msg  <- trace_msg
  result$elapsed    <- elapsed
  result$final_call <- call

  class(result) <- unique(c("hzr_stepwise", class(result)))
  result
}


#' @rdname hzr_stepwise
#' @param x An `hzr_stepwise` object.
#' @param ... Unused.
#' @return `print.hzr_stepwise` returns `x` invisibly.
#' @export
print.hzr_stepwise <- function(x, ...) {
  cat(paste(x$trace_msg, collapse = "\n"), "\n", sep = "")
  invisible(x)
}


#' @rdname hzr_stepwise
#' @param object An `hzr_stepwise` object.
#' @return `summary.hzr_stepwise` returns a `summary.hzr_stepwise` object
#'   (extends `summary.hazard`) with `$stepwise_steps` and `$stepwise_trace`
#'   appended.
#' @export
summary.hzr_stepwise <- function(object, ...) {
  # Strip the stepwise class so NextMethod dispatches cleanly to
  # summary.hazard.
  class(object) <- setdiff(class(object), "hzr_stepwise")
  out <- NextMethod()
  out$stepwise_steps <- object$steps
  out$stepwise_trace <- object$trace_msg
  class(out) <- unique(c("summary.hzr_stepwise", class(out)))
  out
}


#' @rdname hzr_stepwise
#' @return `print.summary.hzr_stepwise` returns `x` invisibly.
#' @export
print.summary.hzr_stepwise <- function(x, ...) {
  if (!is.null(x$stepwise_trace)) {
    cat(paste(x$stepwise_trace, collapse = "\n"), "\n\n", sep = "")
  }
  class(x) <- setdiff(class(x), "summary.hzr_stepwise")
  NextMethod()
}


#' @rdname hzr_stepwise
#' @return `as.data.frame.hzr_stepwise` returns the `$steps` data frame.
#' @export
as.data.frame.hzr_stepwise <- function(x, ...) {
  x$steps
}


#' Extract the captured console trace from an `hzr_stepwise` fit
#'
#' Every run of [hzr_stepwise()] records the header, per-step lines,
#' and final summary regardless of the `trace` flag.  This accessor
#' returns the full character vector for display or logging.
#'
#' @param fit An `hzr_stepwise` object.
#' @return Character vector, one element per console line.
#' @seealso [hzr_stepwise()], which produces the object this accessor reads.
#' @examples
#' data(avc)
#' avc <- na.omit(avc)
#' base <- hazard(survival::Surv(int_dead, dead) ~ age,
#'                data = avc, dist = "weibull", fit = TRUE)
#' \donttest{
#' sw <- hzr_stepwise(base, scope = ~ age + nyha,
#'                    data = avc, direction = "forward",
#'                    control = list(n_starts = 1))
#' cat(stepwise_trace(sw), sep = "\n")
#' }
#' @export
stepwise_trace <- function(fit) {
  if (!inherits(fit, "hzr_stepwise")) {
    stop("`fit` must be an `hzr_stepwise` object.", call. = FALSE)
  }
  fit$trace_msg
}


# Local shape-count helper that tolerates multiphase as well as single
# distributions (the shared `.hzr_shape_parameter_count` in
# parity-helpers only handles the latter).
.hzr_stepwise_shape_count <- function(fit) {
  if (fit$spec$dist == "multiphase") {
    # For multiphase, "coefficient count" is sum of betas across phases
    # -- i.e. theta length minus all non-beta slots.  Easiest path:
    # count columns in x_list.
    x_list <- fit$fit$x_list
    if (is.null(x_list)) return(length(fit$fit$theta))
    total_betas <- sum(vapply(x_list, function(m) {
      if (is.null(m)) 0L else ncol(m)
    }, integer(1L)))
    length(fit$fit$theta) - total_betas
  } else {
    .hzr_shape_parameter_count(fit$spec$dist,
                                control = fit$spec$control)
  }
}

# TemporalHazard-package.R -- Package-level overview help page (?TemporalHazard)
#
# Provides the front-door documentation: the core multiphase model, the phase
# vocabulary, the SAS/C HAZARD bridge, and a map of the main entry points.
# Modelled on the overview pages of randomForestSRC and ggRandomForests.

#' TemporalHazard: Temporal Parametric Hazard Modeling
#'
#' Native R implementation of the multiphase parametric hazard model of
#' Blackstone, Naftel, and Turner (1986), with a focus on behavioral parity,
#' transparent numerics, and reproducible validation against the original
#' \sQuote{C}/\sQuote{SAS} HAZARD program.  The package fits time-varying hazards
#' as an additive sum of parametric *phases* --- early, constant, and late risk
#' streams --- each carrying its own covariate effects.
#'
#' @section The multiphase model:
#'
#' The total cumulative hazard decomposes additively across \eqn{J} phases:
#'
#' \deqn{H(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \,
#'       \Phi_j(t)}
#'
#' where \eqn{\mu_j(\mathbf{x}) = \exp(\alpha_j + \mathbf{x}_j^\top
#' \boldsymbol{\beta}_j)} is the phase-specific log-linear scale (intercept plus
#' covariate effects) and \eqn{\Phi_j(t)} is the temporal shape contributed by
#' phase \eqn{j}, with its own parameters depending on the phase `type` (see the
#' phase vocabulary below).  The instantaneous hazard and survival follow
#' directly:
#'
#' \deqn{h(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \,
#'       \varphi_j(t), \qquad
#'       S(t \mid \mathbf{x}) = \exp\!\bigl(-H(t \mid \mathbf{x})\bigr)}
#'
#' with \eqn{\varphi_j = d\Phi_j/dt}.  Each phase is specified with
#' [hzr_phase()] and the model is fit by maximum likelihood with [hazard()].
#'
#' @section Phase vocabulary:
#'
#' \tabular{llll}{
#'   **Phase type** \tab **\eqn{\Phi_j(t)}** \tab **Domain** \tab **Use** \cr
#'   `"cdf"` \tab \eqn{G(t)} \tab \eqn{[0, 1]} \tab Early risk that resolves over time \cr
#'   `"hazard"` \tab \eqn{-\log(1 - G(t))} \tab \eqn{[0, \infty)} \tab Late or aging risk that accumulates \cr
#'   `"g3"` \tab \eqn{G_3(t)} \tab \eqn{[0, \infty)} \tab Unbounded late risk (original C/SAS late phase) \cr
#'   `"constant"` \tab \eqn{t} \tab \eqn{[0, \infty)} \tab Flat background rate (no shape parameters) \cr
#' }
#'
#' Here \eqn{G(t)} is the generalized temporal decomposition CDF computed by
#' [hzr_decompos()], and \eqn{G_3(t)} is the unbounded late-phase intensity from
#' [hzr_decompos_g3()].  See `vignette("mf-mathematical-foundations")` for the
#' full derivation.
#'
#' @section SAS/C HAZARD bridge:
#'
#' The classic three-phase HAZARD model maps directly onto `hzr_phase()` calls:
#'
#' \tabular{lll}{
#'   **HAZARD phase** \tab **Role** \tab **R equivalent** \cr
#'   G1 (early)    \tab Early resolving risk \tab `hzr_phase("cdf", ...)` \cr
#'   G2 (constant) \tab Flat background rate \tab `hzr_phase("constant")` \cr
#'   G3 (late)     \tab Rising late risk     \tab `hzr_phase("g3", ...)` \cr
#' }
#'
#' TemporalHazard generalizes the fixed three-phase structure to \eqn{N} phases
#' of any type.  [hzr_argument_mapping()] gives the full parameter translation
#' table between the SAS/C parameterization and the R arguments.
#'
#' @section Main entry points:
#'
#' \describe{
#'   \item{Model fitting}{[hazard()] --- build and fit single- or multiphase
#'     models; [hzr_phase()] --- specify one phase; [hzr_stepwise()] --- forward,
#'     backward, or bidirectional covariate selection.}
#'   \item{Prediction}{`predict()` on a fitted `hazard` object --- survival,
#'     cumulative hazard, and per-phase decomposed hazard; `summary()` ---
#'     coefficient tables with Wald inference.}
#'   \item{Parametric family}{[hzr_decompos()] --- the early-phase (G1)
#'     decomposition \eqn{G(t)}, \eqn{g(t)}, \eqn{h(t)}; [hzr_decompos_g3()] ---
#'     the late-phase (G3) intensity; [hzr_phase_cumhaz()] and
#'     [hzr_phase_hazard()] --- per-phase \eqn{\Phi(t)} and \eqn{\varphi(t)}.}
#'   \item{Diagnostics}{[hzr_kaplan()], [hzr_nelson()] --- nonparametric
#'     references; [hzr_gof()] --- goodness of fit; [hzr_calibrate()],
#'     [hzr_deciles()] --- calibration; [hzr_bootstrap()] --- resampling CIs;
#'     [hzr_competing_risks()] --- cumulative incidence.}
#' }
#'
#' @section Vignettes:
#'
#' \describe{
#'   \item{`vignette("getting-started")`}{First fit, end to end.}
#'   \item{`vignette("mf-mathematical-foundations")`}{The decomposition family
#'     and multiphase model, with derivations.}
#'   \item{`vignette("fitting-hazard-models")`}{Single-phase through multiphase
#'     fitting.}
#'   \item{`vignette("prediction-visualization")`}{Prediction types and
#'     decomposed-hazard plots.}
#'   \item{`vignette("inference-diagnostics")`}{Bootstrap CIs and diagnostics.}
#'   \item{`vignette("sas-to-r-migration")`}{Translating SAS/C HAZARD code.}
#' }
#'
#' @references
#' Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
#' hazard into phases, each incorporating a separate stream of concomitant
#' information. *J Am Stat Assoc.* 1986;81(395):615--624.
#' \doi{10.1080/01621459.1986.10478314}
#'
#' Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
#' Probability of atrial fibrillation after ablation: Using a parametric
#' nonlinear temporal decomposition mixed effects model. *Stat Methods Med Res.*
#' 2018;27(1):126--141. \doi{10.1177/0962280215623583}
#'
#' @keywords internal
"_PACKAGE"

# stepwise-formula.R -- Formula and phase-scope plumbing for stepwise
# covariate selection.
#
# Step 8.3 of STEPWISE-DESIGN.md.  Pure helpers that mutate formulas /
# phase lists / coefficient scopes without touching the optimizer.  The
# actual refit driver that combines these into a new hazard() call
# lands in sec.8.4 alongside the forward/backward step implementations.
#
# Two entry points matter to downstream code:
#
#   .hzr_formula_update(formula, action, var)
#     Add or drop a variable from a one-sided or two-sided formula.
#
#   .hzr_phase_update_formula(phase, action, var)
#     Same mutation, but on the `formula` slot of an `hzr_phase` object;
#     returns an updated `hzr_phase` so the multiphase driver can
#     rebuild the phases list by name.
#
# Both are symmetric under add/drop and idempotent w.r.t. already-present
# or already-absent variables.  Variables currently in scope come from:
#
#   .hzr_scope_current_vars(fit, phase = NULL)
#     Lists the RHS term labels for the global formula (single-dist) or
#     the named phase's formula (multiphase).

# ---------------------------------------------------------------------------
# Formula helpers
# ---------------------------------------------------------------------------

#' Extract RHS term labels from a formula
#'
#' Handles both one-sided (`~ x + y`) and two-sided (`Surv(time, status) ~
#' x + y`) formulas.  Returns an empty character vector for `~ 1`.
#'
#' @keywords internal
#' @noRd
.hzr_formula_rhs_terms <- function(formula) {
  if (is.null(formula)) return(character())
  if (!inherits(formula, "formula")) {
    stop("formula must be a `formula` object.", call. = FALSE)
  }
  # terms() needs a formula with no empty LHS/RHS; ~ 1 has one intercept term.
  tt <- tryCatch(stats::terms(formula),
                 error = function(e) NULL)
  if (is.null(tt)) return(character())
  attr(tt, "term.labels")
}


#' Add or drop a variable from a formula's RHS
#'
#' @param formula Existing formula.  One-sided (`~ x`) or two-sided
#'   (`Surv(time, status) ~ x`) -- the LHS is preserved verbatim.
#' @param action Either `"add"` or `"drop"`.
#' @param var Character scalar naming the variable to add / drop.
#'
#' @return A new formula with the requested change.  Idempotent: adding
#'   a variable that is already present is a no-op; dropping one that
#'   is not present is a no-op.  Dropping the last term leaves `~ 1`.
#'
#' @keywords internal
#' @noRd
.hzr_formula_update <- function(formula, action = c("add", "drop"), var) {
  action <- match.arg(action)
  if (!inherits(formula, "formula")) {
    stop("`formula` must be a `formula` object.", call. = FALSE)
  }
  if (!is.character(var) || length(var) != 1L || !nzchar(var)) {
    stop("`var` must be a non-empty character scalar.", call. = FALSE)
  }

  current_terms <- .hzr_formula_rhs_terms(formula)

  if (action == "add") {
    if (var %in% current_terms) return(formula)
    new_terms <- c(current_terms, var)
  } else {
    if (!var %in% current_terms) return(formula)
    new_terms <- setdiff(current_terms, var)
  }

  rhs <- if (length(new_terms) == 0L) "1" else paste(new_terms, collapse = " + ")

  is_two_sided <- length(formula) == 3L
  lhs <- if (is_two_sided) deparse(formula[[2L]]) else NULL
  env <- environment(formula)

  new_text <- if (is_two_sided) {
    paste(lhs, "~", rhs)
  } else {
    paste("~", rhs)
  }

  stats::as.formula(new_text, env = env)
}


# ---------------------------------------------------------------------------
# Phase-level helpers (multiphase models)
# ---------------------------------------------------------------------------

#' Add or drop a variable from a phase's formula
#'
#' @param phase An `hzr_phase` object.  Its `formula` slot may be NULL
#'   (no phase-specific covariates) -- in the add case a fresh
#'   `~ var` formula is created.
#' @param action Either `"add"` or `"drop"`.
#' @param var Character scalar.
#'
#' @return An updated `hzr_phase` object with the new formula.
#'
#' @keywords internal
#' @noRd
.hzr_phase_update_formula <- function(phase, action = c("add", "drop"), var) {
  action <- match.arg(action)
  if (!inherits(phase, "hzr_phase")) {
    stop("`phase` must be an `hzr_phase` object.", call. = FALSE)
  }
  if (!is.character(var) || length(var) != 1L || !nzchar(var)) {
    stop("`var` must be a non-empty character scalar.", call. = FALSE)
  }

  f <- phase$formula
  if (is.null(f)) {
    if (action == "drop") {
      return(phase)   # nothing to drop
    }
    # Create a fresh one-sided formula in the current calling env so the
    # variable can be resolved later by data-frame column lookup.
    phase$formula <- stats::as.formula(
      paste("~", var),
      env = parent.frame()
    )
    return(phase)
  }

  phase$formula <- .hzr_formula_update(f, action, var)

  # If dropping emptied the RHS (`~ 1`), null the slot out for
  # consistency with hzr_phase(formula = NULL) construction.
  if (action == "drop" &&
        identical(.hzr_formula_rhs_terms(phase$formula), character())) {
    phase$formula <- NULL
  }

  phase
}


# ---------------------------------------------------------------------------
# Scope inspection
# ---------------------------------------------------------------------------

#' List variables currently in a hazard model's scope
#'
#' For single-distribution models returns the RHS term labels of the
#' global formula recorded in `fit$call$formula`.  For multiphase models
#' returns the term labels of the named phase's formula; pass
#' `phase = NULL` to get a named list keyed by phase.
#'
#' @param fit A fitted `hazard` object.
#' @param phase Character scalar phase name (multiphase only), or NULL
#'   to return all phases.
#'
#' @return Character vector of variable names, or a named list of such
#'   vectors.
#'
#' @keywords internal
#' @noRd
.hzr_scope_current_vars <- function(fit, phase = NULL) {
  if (!inherits(fit, "hazard")) {
    stop("`fit` must be a `hazard` object.", call. = FALSE)
  }

  if (fit$spec$dist != "multiphase") {
    if (!is.null(phase)) {
      stop("`phase` is only meaningful for multiphase models.", call. = FALSE)
    }
    f <- fit$call$formula
    if (is.null(f)) {
      # Non-formula fit (time/x interface); infer from design matrix names.
      xcols <- colnames(fit$data$x)
      return(if (is.null(xcols)) character() else xcols)
    }
    # `match.call()` captures `formula` as an unevaluated `call` rather
    # than a real `formula` object; coerce before term extraction.
    if (!inherits(f, "formula")) {
      f <- stats::as.formula(deparse(f))
    }
    return(.hzr_formula_rhs_terms(f))
  }

  # Multiphase
  per_phase <- lapply(fit$spec$phases, function(ph) {
    if (is.null(ph$formula)) character() else .hzr_formula_rhs_terms(ph$formula)
  })

  if (is.null(phase)) return(per_phase)

  if (!is.character(phase) || length(phase) != 1L) {
    stop("`phase` must be a single character scalar.", call. = FALSE)
  }
  if (!phase %in% names(per_phase)) {
    stop("Unknown phase: ", sQuote(phase), ".  Available: ",
         paste(sQuote(names(per_phase)), collapse = ", "),
         call. = FALSE)
  }
  per_phase[[phase]]
}

# ---------------------------------------------------------------------------
# Phase-scope detection helper
# ---------------------------------------------------------------------------
#' Detect phase-scoped calls in a formula RHS parse tree
#'
#' Walks the call tree of \code{rhs} (the RHS of a formula, i.e.
#' \code{formula[[3L]]}) and returns \code{TRUE} if any call node has a
#' function symbol that exactly matches one of \code{phase_names}.
#'
#' This is stricter than a string-regex approach: a phase named \code{"log"}
#' will NOT produce a false positive when the formula contains \code{log(age)},
#' because \code{log} would also appear in \code{phase_names} only if the user
#' deliberately named a phase \code{"log"}.  Conversely, the function only
#' fires when the call head is an exact match to a known phase name -- standard
#' R functions that happen to share names with phases do not trigger the check
#' unless those names are actually phase names.
#'
#' @param rhs  A language object (the RHS of a formula, typically
#'   \code{formula[[3L]]}).
#' @param phase_names  Character vector of phase names to look for.
#' @return \code{TRUE} if any call in the tree has its function head in
#'   \code{phase_names}; \code{FALSE} otherwise.
#' @noRd
.hzr_formula_has_phase_scope <- function(rhs, phase_names) {
  if (is.null(rhs) || length(phase_names) == 0L) return(FALSE)
  # Walk recursively: check current node, then recurse into sub-expressions.
  .walk <- function(node) {
    if (is.call(node)) {
      # The head of the call (node[[1L]]) is the function symbol or expression.
      head <- node[[1L]]
      if (is.symbol(head) && as.character(head) %in% phase_names) {
        return(TRUE)
      }
      # Recurse into all arguments (node[[2L]], node[[3L]], ...) and the head.
      for (i in seq_along(node)) {
        if (.walk(node[[i]])) return(TRUE)
      }
    }
    FALSE
  }
  .walk(rhs)
}

# phase-spec.R -- Phase specification for multiphase hazard models
#
# PURPOSE
# -------
# Defines the hzr_phase() constructor that specifies one phase in an
# N-phase additive cumulative hazard model:
#
#   H(t | x) = sum_j  mu_j(x) * Phi_j(t; t_half_j, nu_j, m_j)
#
# Each phase is one term in the sum.  The constructor stores:
#   - type:    "cdf" (early risk), "hazard" (late risk), or "constant"
#   - t_half, nu, m: starting values for the decompos shape parameters
#   - formula: optional one-sided formula for phase-specific covariates
#
# The helpers extract metadata needed during likelihood construction:
#   - .hzr_phase_n_shape():     number of shape parameters (3 or 0)
#   - .hzr_phase_theta_names(): named labels for the parameter sub-vector
#
# SAS/C BRIDGE
# ------------
# SAS users specify Early/Constant/Late phases via DIST keywords.  The
# hzr_phase() constructor maps directly:
#   SAS Early (G1) -> hzr_phase("cdf",      t_half, nu, m)
#   SAS Const (G2) -> hzr_phase("constant")
#   SAS Late  (G3) -> hzr_phase("g3",       tau, gamma, alpha, eta)
#
# The "cdf" and "hazard" types use the G1 decomposition (bounded [0,1]).
# The "g3" type uses the G3 decomposition (unbounded power-law), matching
# the C/SAS HAZARD implementation for late-phase rising hazards.
# The "hazard" type (-log(1-G(t))) is available for alternative models.

# ============================================================================
# Constructor
# ============================================================================

#' Specify a single hazard phase
#'
#' Creates an `hzr_phase` object describing one term in a multiphase additive
#' cumulative hazard model.  Pass a list of these to the `phases` argument of
#' [hazard()] when `dist = "multiphase"`.
#'
#' @section Role in the multiphase model:
#'
#' Each phase is one term \eqn{j} in the additive cumulative hazard
#'
#' \deqn{H(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \,
#'       \Phi_j(t)}
#'
#' where \eqn{\mu_j(\mathbf{x}) = \exp(\alpha_j + \mathbf{x}_j^\top
#' \boldsymbol{\beta}_j)} is the phase-specific log-linear scale and
#' \eqn{\Phi_j(t)} is the temporal shape selected by `type` (below).  The
#' `t_half`/`nu`/`m` (or g3 `tau`/`gamma`/`alpha`/`eta`) arguments set the
#' starting values for that shape; `formula` attaches the covariates
#' \eqn{\mathbf{x}_j} that enter \eqn{\mu_j}.
#'
#' @section Phase types:
#'
#' The `type` argument chooses the temporal shape \eqn{\Phi_j(t)} for the phase.
#' Each captures a qualitatively different pattern of risk over time; a typical
#' clinical model combines an *early*, a *constant*, and a *late* phase so that
#' the total hazard can fall, level off, and rise again.
#'
#' \describe{
#'   \item{`"cdf"` --- early, resolving risk}{Named for the **c**umulative
#'     **d**istribution **f**unction: the phase contributes \eqn{\Phi(t) = G(t)},
#'     the bounded CDF of the temporal decomposition (\eqn{0} at \eqn{t = 0},
#'     rising to a ceiling of \eqn{1}).  Because it saturates, the *hazard* it
#'     adds, \eqn{\mu\,g(t)}, peaks early and then decays toward zero --- the
#'     signature of a one-time insult that patients either succumb to or survive
#'     past, e.g. peri-operative mortality.  Shape set by `t_half`, `nu`, `m`.
#'     SAS/C equivalent: the Early (G1) phase.}
#'   \item{`"hazard"` --- accumulating aging risk (G1 family)}{Named because the
#'     phase contributes a **cumulative hazard** built from the same G1 family:
#'     \eqn{\Phi(t) = -\log(1 - G(t))}, which is unbounded and monotone
#'     increasing.  Its hazard \eqn{\mu\,h(t)} rises without leveling off, so it
#'     models risk that grows as subjects age.  This is an alternative late-risk
#'     form derived from G1; for the original SAS/C late phase prefer `"g3"`.
#'     Shape set by `t_half`, `nu`, `m`.}
#'   \item{`"g3"` --- late, rising risk (original C/SAS late phase)}{Named for
#'     the **G3** (third) decomposition family used by the original HAZARD
#'     program for the late phase.  It contributes \eqn{\Phi(t) = G_3(t)} from
#'     [hzr_decompos_g3()], an unbounded intensity with its own four-parameter
#'     shape (`tau`, `gamma`, `alpha`, `eta`) that is more flexible than the
#'     G1-derived `"hazard"` form for capturing accelerating late mortality
#'     (e.g. structural valve deterioration years after surgery).  Use this when
#'     reproducing classic three-phase HAZARD models.  SAS/C equivalent: the
#'     Late (G3) phase.}
#'   \item{`"constant"` --- flat background rate}{A time-invariant hazard:
#'     \eqn{\Phi(t) = t}, so the added hazard \eqn{\mu} is constant (the
#'     exponential model).  It represents the steady, ongoing risk present at all
#'     follow-up times, independent of how long ago the time origin was.  Takes
#'     no shape parameters --- only its scale \eqn{\mu} (and any covariates) is
#'     estimated.  SAS/C equivalent: the Constant (G2) phase.}
#' }
#'
#' The shape derivative \eqn{\varphi_j = d\Phi_j/dt} (which forms the
#' instantaneous hazard contribution \eqn{\mu_j\,\varphi_j(t)}) is \eqn{g(t)} for
#' `"cdf"`, \eqn{h(t)} for `"hazard"`, \eqn{g_3(t)} for `"g3"`, and \eqn{1} for
#' `"constant"`.
#'
#' @param type Character; the phase's temporal shape --- one of `"cdf"`
#'   (early resolving risk), `"hazard"` (accumulating G1 aging risk), `"g3"`
#'   (late rising risk, the original C/SAS late phase), or `"constant"` (flat
#'   background rate).  See the **Phase types** section for what each means and
#'   when to use it.
#' @param t_half Positive scalar; initial half-life (time at which
#'   \eqn{G(t_{1/2}) = 0.5}).  Used for `"cdf"` and `"hazard"` phases.
#'   SAS early: `THALF`/`RHO`.
#' @param nu Numeric scalar; initial time exponent.  Used for `"cdf"` and
#'   `"hazard"` phases.  SAS early: `NU`.
#' @param m Numeric scalar; initial shape exponent.  Used for `"cdf"` and
#'   `"hazard"` phases.  SAS early: `M`.
#' @param tau Positive scalar; scale parameter for `"g3"` phases.
#'   SAS late: `TAU`.
#' @param gamma Positive scalar; time exponent for `"g3"` phases.
#'   SAS late: `GAMMA`.
#' @param alpha Non-negative scalar; shape parameter for `"g3"` phases.
#'   When `alpha > 0`, the generic G3 formula is used; `alpha = 0` gives the
#'   exponential limiting case.  SAS late: `ALPHA`.
#' @param eta Positive scalar; outer exponent for `"g3"` phases.
#'   SAS late: `ETA`.
#' @param formula Optional one-sided formula (e.g. `~ age + nyha`) for
#'   phase-specific covariates.  When `NULL` (default), the phase inherits
#'   the global formula from [hazard()].
#' @param fixed Character vector naming shape parameters to hold fixed during
#'   optimization.  Valid names for `"cdf"`/`"hazard"`: `"t_half"`, `"nu"`,
#'   `"m"`, or `"shapes"` (shorthand for all three).  Valid names for `"g3"`:
#'   `"tau"`, `"gamma"`, `"alpha"`, `"eta"`, or `"shapes"` (shorthand for all
#'   four).  Fixed parameters are held at their starting values; only `mu`
#'   (and covariates) are estimated.  Ignored for `"constant"` phases.
#'   This mirrors the SAS/C HAZARD workflow where shapes are typically fixed
#'   and only scale parameters are estimated.
#'
#' @return An S3 object of class `"hzr_phase"` with elements:
#' \describe{
#'   \item{type}{Phase type string.}
#'   \item{t_half}{Initial half-life (cdf/hazard phases).}
#'   \item{nu}{Initial time exponent (cdf/hazard phases).}
#'   \item{m}{Initial shape exponent (cdf/hazard phases).}
#'   \item{tau}{Scale parameter (g3 phases).}
#'   \item{gamma}{Time exponent (g3 phases).}
#'   \item{alpha}{Shape parameter (g3 phases).}
#'   \item{eta}{Outer exponent (g3 phases).}
#'   \item{formula}{Phase-specific formula or `NULL`.}
#'   \item{fixed}{Character vector of fixed parameter names (may be empty).}
#' }
#'
#' @examples
#' # Classic 3-phase Blackstone pattern
#' early <- hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0)
#' const <- hzr_phase("constant")
#' late  <- hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1)
#'
#' # Fix all shapes (C/SAS-style: only estimate mu)
#' early_fixed <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                           fixed = "shapes")
#' late_fixed  <- hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
#'                           fixed = "shapes")
#'
#' # Fix only some parameters
#' early_partial <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                             fixed = c("nu", "m"))
#'
#' # Phase with specific covariates
#' early_cov <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
#'                         formula = ~ age + shock)
#'
#' # Use in hazard():
#' # hazard(Surv(time, status) ~ age, data = dat,
#' #        dist = "multiphase",
#' #        phases = list(early = early, constant = const, late = late))
#'
#' @seealso [hazard()] for fitting multiphase models,
#'   [hzr_decompos()] for the underlying parametric family,
#'   [hzr_phase_cumhaz()] and [hzr_phase_hazard()] for computing
#'   \eqn{\Phi(t)} and \eqn{\phi(t)} from these specifications.
#'
#' \code{vignette("fitting-hazard-models")} for multiphase fitting examples,
#' \code{vignette("mf-mathematical-foundations")} for the mathematical framework.
#'
#' @export
hzr_phase <- function(type = c("cdf", "hazard", "constant", "g3"),
                      t_half = 1, nu = 1, m = 0,
                      tau = 1, gamma = 1, alpha = 1, eta = 1,
                      formula = NULL,
                      fixed = character(0)) {

  type <- match.arg(type)

  # --- Validate shape parameters based on type ------------------------------
  if (type == "g3") {
    # G3 late-phase decomposition: 4 parameters (tau, gamma, alpha, eta)
    stopifnot(
      "tau must be a positive scalar" =
        is.numeric(tau) && length(tau) == 1L && tau > 0,
      "gamma must be a positive scalar" =
        is.numeric(gamma) && length(gamma) == 1L && gamma > 0,
      "alpha must be a non-negative scalar" =
        is.numeric(alpha) && length(alpha) == 1L && alpha >= 0 && is.finite(alpha),
      "eta must be a positive scalar" =
        is.numeric(eta) && length(eta) == 1L && eta > 0
    )
  } else if (type != "constant") {
    # G1 early-phase decomposition: 3 parameters (t_half, nu, m)
    stopifnot(
      "t_half must be a positive scalar" =
        is.numeric(t_half) && length(t_half) == 1L &&
        is.finite(t_half) && t_half > 0,
      "nu must be a finite numeric scalar" =
        is.numeric(nu) && length(nu) == 1L && is.finite(nu),
      "m must be a finite numeric scalar" =
        is.numeric(m) && length(m) == 1L && is.finite(m)
    )

    if (m < 0 && nu < 0) {
      stop("Decomposition undefined when both m < 0 and nu < 0 (m = ",
           m, ", nu = ", nu, ").  See ?hzr_decompos for valid cases.",
           call. = FALSE)
    }
  }

  # --- Validate formula if supplied -----------------------------------------
  if (!is.null(formula)) {
    if (!inherits(formula, "formula")) {
      stop("formula must be a one-sided formula (e.g. ~ age + nyha) or NULL.",
           call. = FALSE)
    }
    # Must be one-sided (no LHS)
    if (length(formula) == 3L) {
      stop("Phase formula must be one-sided (e.g. ~ age + nyha), ",
           "not two-sided (response ~ predictors).", call. = FALSE)
    }
  }

  # --- Validate and normalize fixed parameters ------------------------------
  if (length(fixed) > 0 && type == "constant") {
    warning("fixed is ignored for constant phases.", call. = FALSE)
    fixed <- character(0)
  }
  if (length(fixed) > 0) {
    if (type == "g3") {
      # G3: expand "shapes" to tau, gamma, alpha, eta
      if ("shapes" %in% fixed) {
        fixed <- union(setdiff(fixed, "shapes"), c("tau", "gamma", "alpha", "eta"))
      }
      valid_fixed <- c("tau", "gamma", "alpha", "eta")
    } else {
      # G1: expand "shapes" to t_half, nu, m
      if ("shapes" %in% fixed) {
        fixed <- union(setdiff(fixed, "shapes"), c("t_half", "nu", "m"))
      }
      valid_fixed <- c("t_half", "nu", "m")
    }
    bad <- setdiff(fixed, valid_fixed)
    if (length(bad) > 0) {
      stop("Invalid fixed parameter names: ",
           paste(bad, collapse = ", "),
           ". Valid names: ", paste(valid_fixed, collapse = ", "),
           ", or 'shapes' for all.", call. = FALSE)
    }
    fixed <- unique(fixed)
  }

  if (type == "g3") {
    obj <- list(
      type    = type,
      tau     = tau,
      gamma   = gamma,
      alpha   = alpha,
      eta     = eta,
      formula = formula,
      fixed   = fixed
    )
  } else {
    obj <- list(
      type    = type,
      t_half  = if (type == "constant") NA_real_ else t_half,
      nu      = if (type == "constant") NA_real_ else nu,
      m       = if (type == "constant") NA_real_ else m,
      formula = formula,
      fixed   = fixed
    )
  }

  structure(obj, class = "hzr_phase")
}


# ============================================================================
# S3 methods
# ============================================================================

#' @rdname hzr_phase
#' @param x An `hzr_phase` object (for `print.hzr_phase()`).
#' @param ... Additional arguments (ignored).
#' @export
print.hzr_phase <- function(x, ...) {
  label <- switch(x$type,
    cdf      = "cdf (early risk)",
    hazard   = "hazard (late risk)",
    constant = "constant (flat rate)",
    g3       = "g3 (late phase)"
  )
  cat("<hzr_phase>", label, "\n")

  if (x$type == "g3") {
    cat("  tau =", format(x$tau, digits = 4),
        " gamma =", format(x$gamma, digits = 4),
        " alpha =", format(x$alpha, digits = 4),
        " eta =", format(x$eta, digits = 4), "\n")
  } else if (x$type != "constant") {
    cat("  t_half =", format(x$t_half, digits = 4),
        " nu =", format(x$nu, digits = 4),
        " m =", format(x$m, digits = 4), "\n")
  }

  if (x$type != "constant" && length(x$fixed) > 0) {
    cat("  fixed:", paste(x$fixed, collapse = ", "), "\n")
  }

  if (!is.null(x$formula)) {
    cat("  covariates:", deparse(x$formula), "\n")
  }

  invisible(x)
}


# ============================================================================
# Type-checking helper
# ============================================================================

#' Test if an object is an hzr_phase
#'
#' @param x Object to test.
#' @return Logical scalar.
#'
#' @examples
#' is_hzr_phase(hzr_phase("cdf"))
#' is_hzr_phase("not a phase")
#'
#' @export
is_hzr_phase <- function(x) {
  inherits(x, "hzr_phase")
}


# ============================================================================
# Internal helpers for likelihood construction
# ============================================================================

#' Number of shape parameters for a phase
#'
#' Returns 3 (t_half, nu, m) for `"cdf"` and `"hazard"` phases, 0 for
#' `"constant"`.
#'
#' @param phase An `hzr_phase` object.
#' @return Integer: 3 or 0.
#' @keywords internal
.hzr_phase_n_shape <- function(phase) {
  stopifnot(is_hzr_phase(phase))
  switch(phase$type,
    constant = 0L,
    g3       = 4L,
    3L  # cdf, hazard
  )
}


#' Generate parameter names for a phase's sub-vector
#'
#' Builds the named labels for the parameter block that belongs to a single
#' phase in the full theta vector.  The block layout is:
#'
#' \itemize{
#'   \item For `"cdf"`/`"hazard"`: `[log_mu, log_t_half, nu, m, beta_1, ..., beta_p]`
#'   \item For `"constant"`:       `[log_mu, beta_1, ..., beta_p]`
#' }
#'
#' @param phase An `hzr_phase` object.
#' @param phase_name Character label for the phase (e.g. `"early"`, `"phase_1"`).
#' @param covariate_names Character vector of covariate column names that
#'   this phase uses.  Can be length 0 if no covariates.
#' @return Character vector of parameter names.
#' @keywords internal
.hzr_phase_theta_names <- function(phase, phase_name, covariate_names = character(0)) {
  stopifnot(is_hzr_phase(phase))

  # Intercept (always present)
  names_out <- paste0(phase_name, ".log_mu")

  # Shape parameters
  if (phase$type == "g3") {
    names_out <- c(names_out,
      paste0(phase_name, ".log_tau"),
      paste0(phase_name, ".gamma"),
      paste0(phase_name, ".alpha"),
      paste0(phase_name, ".eta")
    )
  } else if (phase$type != "constant") {
    names_out <- c(names_out,
      paste0(phase_name, ".log_t_half"),
      paste0(phase_name, ".nu"),
      paste0(phase_name, ".m")
    )
  }

  # Covariate coefficients
  if (length(covariate_names) > 0L) {
    names_out <- c(names_out,
      paste0(phase_name, ".", covariate_names)
    )
  }

  names_out
}


#' Total number of parameters for a phase
#'
#' Returns the count of free parameters in the theta sub-vector for one phase:
#' 1 (log_mu) + n_shape + n_covariates.
#'
#' @param phase An `hzr_phase` object.
#' @param n_covariates Integer; number of covariate columns this phase uses.
#' @return Integer.
#' @keywords internal
.hzr_phase_n_params <- function(phase, n_covariates = 0L) {
  stopifnot(is_hzr_phase(phase))
  1L + .hzr_phase_n_shape(phase) + as.integer(n_covariates)
}


#' Extract starting values from a phase specification
#'
#' Returns initial theta sub-vector on the estimation (internal) scale:
#' log(mu), log(t_half), nu, m, followed by zeros for covariate coefficients.
#'
#' @param phase An `hzr_phase` object.
#' @param n_covariates Integer; number of covariate columns.
#' @param mu_start Numeric scalar; initial scale parameter (default 0.1).
#' @return Named numeric vector of starting values.
#' @keywords internal
.hzr_phase_start <- function(phase, n_covariates = 0L, mu_start = 0.1) {
  stopifnot(is_hzr_phase(phase))
  stopifnot(
    "mu_start must be a positive scalar" =
      is.numeric(mu_start) && length(mu_start) == 1L && mu_start > 0
  )

  vals <- log(mu_start)  # log_mu

  if (phase$type == "g3") {
    vals <- c(vals, log(phase$tau), phase$gamma, phase$alpha, phase$eta)
  } else if (phase$type != "constant") {
    vals <- c(vals, log(phase$t_half), phase$nu, phase$m)
  }

  # Covariate betas initialized to zero
  if (n_covariates > 0L) {
    vals <- c(vals, rep(0, n_covariates))
  }

  vals
}


#' Build a logical mask of free (non-fixed) parameters in the full theta vector
#'
#' Returns a logical vector the same length as the full theta where `TRUE`
#' means the parameter is free (to be optimized) and `FALSE` means it is
#' held fixed at its starting value.
#'
#' @param phases Named list of validated `hzr_phase` objects.
#' @param covariate_counts Named integer vector of per-phase covariate counts.
#' @return Logical vector of length `sum(.hzr_phase_n_params(...))`.
#' @keywords internal
.hzr_phase_free_mask <- function(phases, covariate_counts) {
  mask <- logical(0)

  for (nm in names(phases)) {
    ph <- phases[[nm]]
    n_cov <- covariate_counts[[nm]]

    # log_mu is always free
    mask <- c(mask, TRUE)

    # Shape parameters
    if (ph$type == "g3") {
      fixed <- if (is.null(ph$fixed)) character(0) else ph$fixed
      mask <- c(mask,
        !("tau"   %in% fixed),  # log_tau
        !("gamma" %in% fixed),  # gamma
        !("alpha" %in% fixed),  # alpha
        !("eta"   %in% fixed)   # eta
      )
    } else if (ph$type != "constant") {
      fixed <- if (is.null(ph$fixed)) character(0) else ph$fixed
      mask <- c(mask,
        !("t_half" %in% fixed),  # log_t_half
        !("nu"     %in% fixed),  # nu
        !("m"      %in% fixed)   # m
      )
    }

    # Covariate betas are always free
    if (n_cov > 0L) {
      mask <- c(mask, rep(TRUE, n_cov))
    }
  }

  mask
}


#' Validate a list of phase specifications
#'
#' Checks that `phases` is a non-empty named list of `hzr_phase` objects.
#' Auto-names unnamed phases as `phase_1`, `phase_2`, etc.
#'
#' @param phases A list of `hzr_phase` objects.
#' @return The validated (and possibly auto-named) list, invisibly.
#' @keywords internal
.hzr_validate_phases <- function(phases) {
  # Catch bare hzr_phase passed instead of list(hzr_phase(...))
  if (is_hzr_phase(phases)) {
    stop("phases must be a non-empty list of hzr_phase objects, ",
         "not a single hzr_phase.  Wrap it: list(hzr_phase(...)).", call. = FALSE)
  }
  if (!is.list(phases) || length(phases) == 0L) {
    stop("phases must be a non-empty list of hzr_phase objects.", call. = FALSE)
  }

  # Check each element

  for (i in seq_along(phases)) {
    if (!is_hzr_phase(phases[[i]])) {
      stop("phases[[", i, "]] is not an hzr_phase object. ",
           "Use hzr_phase() to create phase specifications.", call. = FALSE)
    }
  }

  # Auto-name unnamed phases
  nms <- names(phases)
  if (is.null(nms)) {
    nms <- character(length(phases))
  }
  empty <- nms == "" | is.na(nms)
  if (any(empty)) {
    nms[empty] <- paste0("phase_", which(empty))
  }

  # Check for duplicate names
  if (anyDuplicated(nms)) {
    stop("Phase names must be unique. Duplicates found: ",
         paste(nms[duplicated(nms)], collapse = ", "), call. = FALSE)
  }

  names(phases) <- nms
  invisible(phases)
}

# Tier-1 correctness invariants -- see inst/dev/CORRECTNESS-STRATEGY.md and
# helper-invariants.R. Single-engine, dataset-driven property tests: no SAS
# reference, no captured fixtures. Run over a model matrix spanning the four
# single distributions plus multiphase (conserve on/off, covariates,
# left-truncation, weights).

test_that("predicted survival and cumulative hazard are distributionally sane", {
  skip_on_cran()
  skip_if_not_installed("survival")
  for (m in .inv_models_cached()) {
    .inv_assert_distribution(m)
  }
})

test_that("multiphase log-likelihood is self-consistent with returned coefficients", {
  skip_on_cran()
  skip_if_not_installed("survival")
  for (m in .inv_models_cached()) {
    if (isTRUE(m$multiphase)) .inv_assert_self_consistent(m)
  }
})

test_that("Conservation of Events identity holds for conserve = TRUE multiphase fits", {
  skip_on_cran()
  skip_if_not_installed("survival")
  for (m in .inv_models_cached()) {
    if (isTRUE(m$multiphase) && isTRUE(m$conserve)) .inv_assert_coe(m)
  }
})

#!/bin/bash
# Run the AVC fractional-weight SAS capture.
#
# Usage:  cd to this directory, then ./run.sh
#
# Set HAZAPPS and HZ_MACROS before calling if the auto-discovery below does
# not find the CCF HAZARD install (HZ_MACROS is used instead of MACROS to
# avoid clobbering an ambient $MACROS pointing at the general SAS library):
#   HAZAPPS=/path/to/hazard/bin  HZ_MACROS=/path/to/hazard/macros  ./run.sh

set -euo pipefail

DIR="$(cd "$(dirname "$0")" && pwd)"
cd "$DIR"

# ---------------------------------------------------------------------------
# Discover HAZAPPS (directory containing hazard.exe / hazard)
# $HAZARD is the CCF-standard env var name on lri-sas-p-* servers
# Prefer ~/hazard/bin (local build) over the system install
# ---------------------------------------------------------------------------
if [ -z "${HAZAPPS:-}" ]; then
  for d in \
      "$HOME/hazard/bin" \
      "${HAZARD:-}" \
      /programs/apps/sas/hazard \
      /opt/hazard/bin \
      /opt/hazard \
      "$HOME/hazard"; do
    [ -z "$d" ] && continue
    if [ -f "$d/hazard.exe" ] || [ -f "$d/hazard" ]; then
      HAZAPPS="$d"
      break
    fi
  done
fi

if [ -z "${HAZAPPS:-}" ]; then
  echo "ERROR: Cannot find the HAZARD binary."
  echo "  Set HAZAPPS=/path/to/dir/containing/hazard.exe before running."
  exit 1
fi
export HAZAPPS

# ---------------------------------------------------------------------------
# Discover MACROS (directory containing hazard.sas for SASAUTOS)
# Always verify -- the ambient $MACROS env var may point to the general
# SAS macro library, not the HAZARD-specific macros.
# ---------------------------------------------------------------------------
# Honor a caller-supplied HZ_MACROS first; otherwise auto-discover.
_hz_macros=""
for d in \
    "${HZ_MACROS:-}" \
    "$HAZAPPS" \
    "$HAZAPPS/macros" \
    "$(cd "$HAZAPPS/.." 2>/dev/null && pwd)/macros" \
    /programs/apps/sas/hazard \
    /opt/hazard/macros \
    "$HOME/hazard/macros"; do
  [ -z "$d" ] && continue
  if [ -f "$d/hazard.sas" ]; then
    _hz_macros="$(cd "$d" && pwd)"
    break
  fi
done

if [ -z "$_hz_macros" ]; then
  echo "ERROR: Cannot find hazard.sas (SASAUTOS macros directory)."
  echo "  Looked in: $HAZAPPS, $HAZAPPS/macros, and common paths."
  echo "  Set HZ_MACROS=/path/to/dir/containing/hazard.sas before running."
  exit 1
fi
export HZ_MACROS="$_hz_macros"   # use HZ_MACROS to avoid clobbering ambient $MACROS

echo "HAZAPPS=$HAZAPPS"
echo "HZ_MACROS=$HZ_MACROS"
echo ""

mkdir -p capture

# SAS Foundation overrides $TMPDIR internally to /saswork/, breaking the
# XPORT handoff to the C binary.  Pass HZ_TMPDIR so the SAS script can call
# OPTIONS SET=TMPDIR before %HAZARD runs.  Use $HOME/tmp (short path) because
# the HAZARD C binary likely has a fixed-size path buffer (~64-80 chars).
mkdir -p "$HOME/tmp"
export HZ_TMPDIR="$HOME/tmp"

echo "Running SAS..."
# Run SAS; capture exit code without letting set -e abort before we report
sas_rc=0
sas avc-weighted.sas || sas_rc=$?

if [ $sas_rc -ne 0 ]; then
  echo "WARNING: SAS exited with code $sas_rc -- check avc-weighted.log for errors."
fi

# Copy the SAS listing to capture/ -- this is the raw output parsed by R
if [ -f avc-weighted.lst ]; then
  cp avc-weighted.lst capture/
  echo "Listing captured: capture/avc-weighted.lst"
else
  echo "WARNING: avc-weighted.lst not found -- SAS may not have run."
fi

# Always show last few lines of the log so errors are visible
if [ -f avc-weighted.log ]; then
  echo ""
  echo "--- last 20 lines of avc-weighted.log ---"
  tail -20 avc-weighted.log
fi

echo ""
echo "Files in capture/:"
ls -lh capture/ 2>/dev/null || echo "(empty)"

# Stepwise Fixture Schema

Each fixture `.rds` file deserialises to a named list with four
components: `meta`, `steps`, `final`, and `scope`.  All fields are
required unless marked optional.

```r
list(
  meta = list(
    dataset       = "cabgkul",          # name of the data frame used
    dist          = "weibull",          # or "exponential", "multiphase", ...
    criterion     = "wald",             # or "aic"
    direction     = "forward",          # or "backward", "both"
    slentry       = 0.30,               # entry threshold (criterion = wald)
    slstay        = 0.20,               # retention threshold (criterion = wald)
    sas_version   = "9.4",
    captured_on   = "2026-04-17",       # ISO-8601
    proc_hazard   = "PROC HAZARD ...",  # exact SAS invocation for audit
    notes         = NULL                # optional free text
  ),

  scope = list(
    # For single-distribution models: character vector of candidate
    # names.  For multiphase: named list keyed by phase.
    candidates = c("age", "mal", "shock", "nyha", "inc_surg"),
    force_in   = character(),
    force_out  = character()
  ),

  steps = data.frame(
    step_num  = integer(),     # 1..N
    action    = character(),   # "enter" | "drop"
    variable  = character(),
    phase     = character(),   # NA_character_ for single-dist
    stat      = numeric(),     # Wald chi-square
    df        = integer(),     # degrees of freedom (usually 1)
    p_value   = numeric(),     # two-sided
    stringsAsFactors = FALSE
  ),

  final = list(
    coef = data.frame(
      variable  = character(),  # same order as final model's print
      phase     = character(),  # NA_character_ for single-dist
      estimate  = numeric(),
      std_error = numeric(),
      z_stat    = numeric(),
      p_value   = numeric(),
      stringsAsFactors = FALSE
    ),
    logLik     = numeric(1L),   # at termination
    aic        = numeric(1L),   # optional; computed if absent
    iterations = integer(1L)    # SAS's iteration count (reported)
  )
)
```

## Notes

- SAS `PROC HAZARD` reports Wald chi-square; we store it directly
  rather than converting to z.  The R helper squares z for df = 1 to
  compare.
- For multiphase scenarios, `phase` must contain the SAS phase name
  as it appears in the output (e.g. `"EARLY"`, `"LATE"`, `"CONSTANT"`)
  — the R-side test lowercases and matches to `names(fit$spec$phases)`.
- `final$coef` must be in the same order as the R fit reports it when
  all entered variables are present.  The comparator sorts by name
  before checking, so ordering is for readability only.
- Tolerances live in `test-stepwise-parity.R`; see
  `README.md` for the current values.

---
title: "Complete Clinical Analysis Walkthrough"
subtitle: "From Kaplan-Meier baseline to validated multivariable model"
author: "John Ehrlinger"
date: last-modified
format:
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    number-sections: true
vignette: >
  %\VignetteIndexEntry{Complete Clinical Analysis Walkthrough}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE)
has_ggplot2 <- requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5,
  eval = has_pkg
)
```

The previous vignettes covered each piece of the analytical workflow
in isolation — how to fit a model, how to predict from it, how to
validate it. This vignette runs all of those pieces together on a
single dataset, in the order you'd actually run them in a real
clinical analysis. The point isn't to introduce new functions; it's
to show how the pieces chain into a disciplined sequence that turns
raw covariates into a fitted, validated, defensible hazard model.

The sequence mirrors the one in the original SAS HAZARD system, which
codified what cardiothoracic-surgery biostatisticians had been doing
informally for decades:

1. **Nonparametric baseline** — Kaplan-Meier life table to see what
   the data is saying before any model intervenes.
2. **Shape fitting** — start with a single-distribution Weibull,
   build up to a multiphase decomposition when the KM curve
   demands it.
3. **Variable screening** — univariable association tests and
   calibration plots to decide which covariates enter the model and
   in what functional form.
4. **Multivariable model** — covariates layered onto a hazard shape
   you already trust.
5. **Prediction** — patient-specific risk profiles for clinical
   reporting and decision support.
6. **Validation** — decile-of-risk calibration to verify the model
   is honest across the risk spectrum, not just on average.

This corresponds to the SAS programs `ac.*` → `hz.*` → `lg.*` →
`hm.*` → `hp.*` → `hs.*`. If you're familiar with the SAS workflow,
the section structure should feel familiar; if not, treat this as
the canonical analytical sequence to follow on any new dataset.

We use the **AVC** dataset (310 patients, atrioventricular canal
repair) which has rich covariates and two identifiable hazard
phases — fewer phases than the 3-phase CABG example you've seen in
other vignettes, but with the covariate complexity needed to
exercise the screening, multivariable-fit, and validation steps.

## Data preparation

Load the package and the AVC dataset, drop incomplete rows so the
design matrix is rectangular for the multivariable fits to come, and
inspect the resulting column types and ranges. The `na.omit()` step
is conservative — losing rows is a real cost — but for a walkthrough
we want every later fit to use the same patient set so the comparisons
are apples-to-apples.

```{r}
#| label: setup
library(TemporalHazard)
if (has_ggplot2) library(ggplot2)

data(avc)
avc <- na.omit(avc)
str(avc)
```

The AVC dataset contains `r if (exists("avc", inherits = TRUE)) nrow(avc) else 310` patients after removing incomplete
cases.  Key covariates include age at repair (`age`, in months), NYHA
functional class (`status`), presence of malalignment (`mal`), and
interventricular communication (`com_iv`).

## Step 1: Nonparametric baseline

Before fitting any parametric model, establish the empirical survival curve
using the Kaplan-Meier estimator.  This is the benchmark against which all
parametric fits will be compared.

`hzr_kaplan()` wraps `survival::survfit()` and adds logit-transformed exact
confidence limits that respect the `[0, 1]` boundary --- more accurate in
the tails than the default Greenwood intervals.  This matches the SAS
`kaplan.sas` macro output structure.

```{r}
#| label: km-life-table
km <- hzr_kaplan(time = avc$int_dead, status = avc$dead)
head(km)
```

The returned data frame is the life table: one row per event time with
`n_risk`, `n_event`, `survival`, logit-transformed 95% CLs
(`cl_lower` / `cl_upper`), and a KM-based cumulative hazard
(`-log(survival)` --- use `hzr_nelson()` if you need the Nelson-Aalen
estimator instead).  Plotting just requires the survival column:

```{r}
#| label: fig-km-baseline
#| fig-cap: "Kaplan-Meier survival estimate for AVC patients (n = 310)"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
km_df <- data.frame(
  time     = km$time,
  survival = km$survival * 100,
  lower    = km$cl_lower * 100,
  upper    = km$cl_upper * 100
)

ggplot(km_df, aes(time, survival)) +
  geom_step(colour = "#D55E00", linewidth = 0.8) +
  geom_ribbon(aes(ymin = lower, ymax = upper),
              stat = "identity", alpha = 0.15, fill = "#D55E00") +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()
```

The KM curve shows a sharp early drop (operative mortality) followed by a
roughly constant attrition rate.  This two-phase pattern suggests an early
CDF phase plus a constant phase --- no obvious late rising hazard.

## Step 2: Shape fitting --- simple to complex

### 2a. Single-phase Weibull

The discipline here is to start with the simplest parametric model
and earn any added complexity. A single-distribution Weibull
intercept-only fit gives us a smooth two-parameter curve to compare
against the KM baseline. If the Weibull tracks the KM step function
reasonably well, we may be done; if it can't, the gap tells us
exactly where a richer model needs to go.

```{r}
#| label: weibull-fit
fit_weib <- hazard(
  survival::Surv(int_dead, dead) ~ 1,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.05, nu = 0.5),
  fit   = TRUE
)
summary(fit_weib)
```

The Weibull forces a monotone hazard shape.  Let's overlay it on the
Kaplan-Meier to see where it fits well and where it doesn't.

```{r}
#| label: fig-weibull-overlay
#| fig-cap: "Single Weibull vs. Kaplan-Meier"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
t_grid <- seq(0.01, max(avc$int_dead) * 0.9, length.out = 200)
surv_weib <- predict(fit_weib,
                     newdata = data.frame(time = t_grid),
                     type = "survival") * 100

ggplot() +
  geom_step(data = km_df, aes(time, survival),
            colour = "#D55E00", linewidth = 0.6) +
  geom_line(data = data.frame(time = t_grid, survival = surv_weib),
            aes(time, survival), colour = "#0072B2", linewidth = 0.8) +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()
```

The single Weibull typically misses the sharp early drop --- it compromises
between the early and late time frames.

### 2b. Two-phase model (early CDF + constant)

The KM curve drops steeply in the first few months — operative
mortality — and then settles into a roughly linear decline. The
Weibull tried to capture both with one shape and ended up
compromising. Splitting the hazard into two phases lets each
mechanism have its own parameterization: an `"cdf"` early phase that
saturates as operative risk resolves, plus a `"constant"` phase that
carries the steady background attrition. Two phases is what AVC
actually needs; CABG with longer follow-up would need a third
late-rising phase to handle graft deterioration, but the AVC
follow-up window doesn't extend far enough to identify one.

```{r}
#| label: multiphase-fit
fit_mp <- hazard(
  survival::Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)
summary(fit_mp)
```

Note the use of `fixed = "shapes"` --- we fix the temporal shape parameters
and only estimate the scale (log_mu) for each phase.  This matches the
standard HAZARD workflow: shapes are set from clinical knowledge or
preliminary exploration, then scales and covariates are estimated.

```{r}
#| label: fig-multiphase-overlay
#| fig-cap: "Two-phase parametric model vs. Kaplan-Meier"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
surv_mp <- predict(fit_mp,
                   newdata = data.frame(time = t_grid),
                   type = "survival") * 100

ggplot() +
  geom_step(data = km_df, aes(time, survival),
            colour = "#D55E00", linewidth = 0.6) +
  geom_line(data = data.frame(time = t_grid, survival = surv_mp),
            aes(time, survival), colour = "#0072B2", linewidth = 0.8) +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()
```

### 2c. Decomposed hazard

The overlay plot shows the *total* fit tracks KM well, but it
doesn't show whether each phase is doing the job we asked of it.
Decomposing the cumulative hazard into per-phase contributions is
the diagnostic that answers that question: the early phase should
account for most of the steep early drop, the constant phase should
carry the rest, and neither phase should be doing implausible work
in the wrong time window.

```{r}
#| label: fig-decomposed
#| fig-cap: "Phase decomposition of cumulative hazard"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
decomp <- predict(fit_mp,
                  newdata = data.frame(time = t_grid),
                  type = "cumulative_hazard",
                  decompose = TRUE)

decomp_df <- data.frame(
  time  = t_grid,
  total = decomp[, "total"],
  early = decomp[, "early"],
  const = decomp[, "constant"]
)

ggplot(decomp_df, aes(x = time)) +
  geom_line(aes(y = total), linewidth = 0.9, colour = "black") +
  geom_line(aes(y = early), linewidth = 0.7, colour = "#E69F00",
            linetype = "dashed") +
  geom_line(aes(y = const), linewidth = 0.7, colour = "#56B4E9",
            linetype = "dashed") +
  labs(x = "Months after AVC repair", y = "Cumulative hazard") +
  theme_minimal()
```

The early phase contributes most of its risk in the first few months, then
levels off.  The constant phase accumulates linearly over time.

## Step 3: Variable screening

Before entering covariates into the hazard model, screen them for
association with the outcome.  This step corresponds to the `lg.*` programs
in the SAS workflow.

### 3a. Univariable logistic screening

The cheapest screening tool we have is a univariable logistic
regression of the event indicator on each covariate. It throws away
the time-to-event structure but it answers a binary question
quickly: does this covariate have *any* association with mortality
at all? Covariates whose univariable p-value is huge will not
suddenly become significant in the multivariable hazard model
either — those can be deprioritized. Covariates with small
univariable p-values deserve closer functional-form inspection
before they enter the formula.

```{r}
#| label: screening
covariates <- c("age", "status", "mal", "com_iv", "orifice",
                "inc_surg", "opmos")

screen_results <- data.frame(
  variable = covariates,
  coef     = numeric(length(covariates)),
  p_value  = numeric(length(covariates))
)

for (i in seq_along(covariates)) {
  v <- covariates[i]
  fml <- as.formula(paste("dead ~", v))
  lg <- glm(fml, data = avc, family = binomial)
  s <- summary(lg)$coefficients
  if (nrow(s) >= 2) {
    screen_results$coef[i] <- s[2, 1]
    screen_results$p_value[i] <- s[2, 4]
  }
}

screen_results <- screen_results[order(screen_results$p_value), ]
screen_results$significant <- ifelse(screen_results$p_value < 0.05,
                                      "*", "")
screen_results
```

### 3b. Functional form assessment

For continuous covariates that appear significant, examine whether the
relationship with outcome is monotone on the logit scale.  `hzr_calibrate()`
implements the SAS `logit.sas` / `logitgr.sas` macros: bin the continuous
covariate into quantile groups, compute the observed event rate per bin,
and transform it to the logit scale.

```{r}
#| label: age-calibrate
cal_age <- hzr_calibrate(x = avc$age, event = avc$dead,
                          groups = 10, link = "logit")
cal_age
```

Plot the logit-transformed event rate against the bin centre (`mean`
column).  A straight line means the covariate can enter the model
untransformed; monotone-but-curved shapes suggest a log or square-root
transform; U-shapes call for a quadratic term.

```{r}
#| label: fig-calibrate-age
#| fig-cap: "Decile calibration: age vs. logit(P(death))"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(cal_age, aes(mean, link_value)) +
  geom_point(size = 3, colour = "#0072B2") +
  geom_line(colour = "#0072B2", alpha = 0.4) +
  geom_smooth(method = "lm", formula = y ~ x, se = FALSE,
              colour = "#D55E00", linetype = "dashed") +
  labs(x = "Age at repair (months, decile midpoint)",
       y = "logit(P(death))") +
  theme_minimal()
```

The blue points are the observed decile logits; the dashed red line is a
linear reference.  Deviations from linearity flag where a transform is
warranted.

## Step 4: Multivariable model

### 4a. Manual specification

We've already established the two-phase shape and screened the
covariates; the multivariable model is the synthesis. Drop the
covariates that passed screening onto the right-hand side of the
formula and refit. The shape parameters stay fixed (we've earned
that decision from §2); the optimizer estimates the two phase
scales and one coefficient per covariate, jointly. This is the
working model you'd report from this analysis.

```{r}
#| label: multivariable-fit
fit_mv <- hazard(
  survival::Surv(int_dead, dead) ~ age + status + mal + com_iv,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)
summary(fit_mv)
```

The coefficient table shows phase-specific covariate effects.  A positive
coefficient means higher hazard (worse prognosis).  Interpretation:
covariates scale the phase-specific hazard multiplicatively via
exp(x * beta).

### 4b. Automated stepwise selection

The manual approach works when screening has already narrowed the
candidate pool, but with a larger pool it helps to let the model
choose.  `hzr_stepwise()` runs forward, backward, or two-way
selection against an existing `hazard` fit, scoring candidates with
Wald p-values or delta-AIC.  Defaults match SAS `PROC HAZARD`
(`SLENTRY = 0.30`, `SLSTAY = 0.20`).

We start from a baseline with no covariates, offer the same screened
candidate pool, and let the algorithm decide.  For multiphase models
the scope is a named list of one-sided formulas keyed by phase, so
each phase can advertise its own candidate set; stepping operates
per `(variable, phase)` pair, letting a covariate enter one phase
and not another.  Single-distribution models accept either a flat
one-sided formula or a character vector of names.

```{r}
#| label: stepwise-forward
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE)
base_mp <- hazard(
  survival::Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 3, maxit = 500)
)

fit_step <- hzr_stepwise(
  base_mp,
  scope     = list(
    early    = ~ age + status + mal + com_iv,
    constant = ~ age + status + mal + com_iv
  ),
  data      = avc,
  direction = "both",
  criterion = "wald",
  trace     = FALSE,
  control   = list(n_starts = 2, maxit = 500)
)
fit_step
```

The `$steps` table records every accepted action with its Wald
statistic and p-value:

```{r}
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE)
fit_step$steps[, c("step_num", "action", "variable", "phase",
                   "p_value", "aic")]
```

The final model is the fit at the top of the object, reachable
through the usual hazard accessors:

```{r}
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE)
logLik_manual <- fit_mv$fit$objective
logLik_step   <- fit_step$fit$objective
c(manual = logLik_manual, stepwise = logLik_step,
  aic_manual = 2 * length(fit_mv$fit$theta) - 2 * logLik_manual,
  aic_step   = 2 * length(fit_step$fit$theta) - 2 * logLik_step)
```

When the screening and stepwise agree on the same covariate set the
log-likelihoods match exactly; when stepwise selects a leaner model
AIC drops.  For an AIC-driven run use `criterion = "aic"`; for a
forward-only sweep `direction = "forward"`.  See `?hzr_stepwise` for
the full argument surface including `force_in`, `force_out`, and the
`max_move` oscillation guard.

## Step 5: Prediction

### 5a. Baseline survival curve

Generate the survival curve for a reference patient (median age, NYHA
class 1, no malalignment, no interventricular communication).

`predict(..., se.fit = TRUE)` adds a delta-method standard error and 95%
confidence band around every prediction (log(-log) scale for survival so
the band is guaranteed to lie in `[0, 1]`).  This is the parametric
analogue of the KM confidence limits from Step 1 --- a patient-specific
uncertainty estimate the nonparametric curve cannot provide.

```{r}
#| label: fig-prediction
#| fig-cap: "Reference patient survival with 95% delta-method CI and KM overlay"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ref_patient <- data.frame(
  time   = t_grid,
  age    = median(avc$age),
  status = 1,
  mal    = 0,
  com_iv = 0
)

surv_ref <- predict(fit_mv, newdata = ref_patient,
                    type = "survival", se.fit = TRUE)

ref_df <- data.frame(
  time     = t_grid,
  survival = surv_ref$fit * 100,
  lower    = surv_ref$lower * 100,
  upper    = surv_ref$upper * 100
)

ggplot() +
  geom_step(data = km_df, aes(time, survival),
            colour = "#D55E00", linewidth = 0.6, alpha = 0.7) +
  geom_ribbon(data = ref_df, aes(time, ymin = lower, ymax = upper),
              fill = "#0072B2", alpha = 0.15) +
  geom_line(data = ref_df, aes(time, survival),
            colour = "#0072B2", linewidth = 0.8) +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()
```

### 5b. Sensitivity analysis --- risk factor comparison

Coefficient tables tell you about effect *direction and magnitude*
on the log-hazard scale. They don't directly tell a clinician what
to expect at the bedside. Sensitivity analysis closes that gap by
scoring two clinically meaningful profiles — a low-risk patient
drawn from the favorable end of each covariate, and a high-risk
patient drawn from the unfavorable end — through the fitted model
and plotting the resulting survival curves with delta-method
confidence bands. The vertical and horizontal gaps between the two
curves are the model's clinical-impact statement, in the units
(survival probability and time) that the audience actually
recognizes.

```{r}
#| label: fig-sensitivity
#| fig-cap: "Survival by risk profile: low-risk (blue) vs. high-risk (red)"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
low_risk <- data.frame(
  time   = t_grid,
  age    = quantile(avc$age, 0.25),
  status = 1,
  mal    = 0,
  com_iv = 0
)

high_risk <- data.frame(
  time   = t_grid,
  age    = quantile(avc$age, 0.90),
  status = 4,
  mal    = 1,
  com_iv = 1
)

surv_lo <- predict(fit_mv, newdata = low_risk,
                   type = "survival", se.fit = TRUE)
surv_hi <- predict(fit_mv, newdata = high_risk,
                   type = "survival", se.fit = TRUE)

sens_df <- data.frame(
  time     = rep(t_grid, 2),
  survival = c(surv_lo$fit, surv_hi$fit) * 100,
  lower    = c(surv_lo$lower, surv_hi$lower) * 100,
  upper    = c(surv_lo$upper, surv_hi$upper) * 100,
  profile  = rep(c("Low risk", "High risk"), each = length(t_grid))
)

ggplot(sens_df, aes(time, survival, colour = profile, fill = profile)) +
  geom_ribbon(aes(ymin = lower, ymax = upper),
              alpha = 0.15, colour = NA) +
  geom_line(linewidth = 0.8) +
  scale_colour_manual(values = c("Low risk" = "#0072B2",
                                  "High risk" = "#D55E00")) +
  scale_fill_manual(values = c("Low risk" = "#0072B2",
                                "High risk" = "#D55E00")) +
  labs(x = "Months after AVC repair", y = "Survival (%)",
       colour = NULL, fill = NULL) +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

Non-overlapping confidence bands between the two profiles indicate the
risk-factor combination is well-identified; overlap around the bands'
centre-of-mass flags where the model can't distinguish the profiles with
the available sample size.

## Step 6: Validation --- decile-of-risk calibration

Partition patients into deciles by predicted risk and compare observed
vs. expected event rates.  Good calibration means the two track each
other.  This implements the workflow of the SAS `deciles.hazard.sas`
macro.

```{r}
#| label: fig-deciles
#| fig-cap: "Observed vs. expected event rates by risk decile"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
cal <- hzr_deciles(fit_mv, time = max(avc$int_dead))
print(cal)
```

```{r}
#| label: fig-decile-plot
#| fig-cap: "Decile calibration: observed (bars) vs. expected (points) event rates"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(cal, aes(x = group)) +
  geom_col(aes(y = observed_rate), fill = "#56B4E9", alpha = 0.7) +
  geom_point(aes(y = expected_rate), colour = "#D55E00", size = 3) +
  geom_line(aes(y = expected_rate), colour = "#D55E00", linewidth = 0.5) +
  scale_x_continuous(breaks = seq_len(nrow(cal))) +
  labs(x = "Risk decile (1 = lowest)", y = "Event rate",
       caption = paste("Overall chi-sq =",
                        round(attr(cal, "overall")$chi_sq, 2),
                        ", p =",
                        format.pval(attr(cal, "overall")$p_value,
                                     digits = 3))) +
  theme_minimal()
```

A non-significant overall chi-square statistic (p > 0.05) indicates
adequate calibration --- the model's predictions are consistent with
observed event rates across the risk spectrum.

### Conservation of events check

The conservation-of-events principle states that a well-fitting model
should predict the same total number of events as actually observed.
`hzr_gof()` tracks this cumulatively over time --- the residual
(expected minus observed) should stay near zero.

```{r}
#| label: gof-summary
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE)
gof <- hzr_gof(fit_mv)
print(gof)
```

```{r}
#| label: fig-gof-survival
#| fig-cap: "Parametric (blue) vs. Kaplan-Meier (orange) survival"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(gof, aes(x = time)) +
  geom_step(aes(y = km_surv * 100), colour = "#D55E00", linewidth = 0.6) +
  geom_line(aes(y = par_surv * 100), colour = "#0072B2", linewidth = 0.8) +
  labs(x = "Months after AVC repair", y = "Survival (%)") +
  coord_cartesian(ylim = c(0, 100)) +
  theme_minimal()
```

```{r}
#| label: fig-gof-events
#| fig-cap: "Cumulative observed (orange) vs. expected (blue) events"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(gof, aes(x = time)) +
  geom_line(aes(y = cum_observed), colour = "#D55E00", linewidth = 0.8) +
  geom_line(aes(y = cum_expected), colour = "#0072B2",
            linewidth = 0.8, linetype = "dashed") +
  labs(x = "Months after AVC repair", y = "Cumulative events") +
  theme_minimal()
```

```{r}
#| label: fig-gof-residual
#| fig-cap: "Conservation-of-events residual (expected minus observed) over time"
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
ggplot(gof, aes(x = time, y = residual)) +
  geom_line(linewidth = 0.7, colour = "grey30") +
  geom_hline(yintercept = 0, linetype = "dashed", colour = "grey60") +
  labs(x = "Months after AVC repair",
       y = "Residual (expected - observed)") +
  theme_minimal()
```

A residual that hovers near zero across the full time range indicates the
model conserves events well.  Persistent positive residual means the model
overpredicts risk; persistent negative means it underpredicts.

## Why not Cox regression?

For this kind of analysis the temporal parametric approach buys you
several things that Cox proportional hazards does not:

| Feature | Cox (`coxph`) | TemporalHazard |
|:---|:---:|:---:|
| Proportional hazards required | Yes | No |
| Baseline hazard specified | No | Yes (parametric) |
| Multiple hazard phases | No | Yes (additive) |
| Patient-specific survival prediction | Approximate | Exact |
| Smooth extrapolation beyond data | No | Yes |
| Conservation of events check | No | Yes (`hzr_gof()`) |
| Interval censoring | Not standard | Supported |

Many clinical outcomes show **non-proportional
hazards**: the risk factors that dominate early mortality (e.g., surgical
complexity) differ from those driving late attrition (e.g., age, comorbidity).
The multiphase model captures this through phase-specific
covariate effects.

## Summary

The complete analytical workflow follows a disciplined sequence:

1. **Kaplan-Meier baseline** --- establish the empirical survival pattern
2. **Shape fitting** --- match the temporal shape, simple to complex
3. **Variable screening** --- logistic screening, LOESS functional form
4. **Multivariable model** --- enter covariates with fixed shapes
5. **Prediction** --- reference curves, sensitivity analysis
6. **Validation** --- decile calibration, conservation-of-events check

This sequence ensures that the temporal shape is established before
covariates are introduced, and that the final model is validated against
the observed data.  See `vignette("getting-started")` for the minimal
API workflow and `vignette("mf-mathematical-foundations")` for the
mathematical details.

---
title: "Inference & Diagnostics"
subtitle: "Exploratory screening, bootstrap confidence intervals, and sensitivity analysis"
format:
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    number-sections: true
vignette: >
  %\VignetteIndexEntry{Inference & Diagnostics}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)
```

```{r}
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)
```

Fitting a hazard model is the middle of the workflow, not the end.
Before you fit, you screen the covariates so you know what
transformations the data actually supports. After you fit, you
quantify uncertainty (bootstrap CIs, Wald intervals), check whether
the model is honest to the data (goodness-of-fit overlays,
decile-of-risk calibration), and probe its behavior under different
covariate scenarios (sensitivity analysis). This vignette walks the
diagnostic and inferential tools the package ships for each of those
steps.

These pieces correspond directly to steps in the classic SAS HAZARD
workflow — the `lg.*` logistic-screening macros, the `bs.*` bootstrap
macros, the `hs.*` hazard-sensitivity macros — re-implemented as
first-class R functions (`hzr_calibrate()`, `hzr_kaplan()`,
`hzr_nelson()`, `hzr_bootstrap()`, `hzr_gof()`, `hzr_deciles()`). If
you've worked through a SAS HAZARD analysis you already know the
shape of what follows; if not, treat this vignette as the standard
diagnostic pass you'd run on any fitted hazard model before trusting
its conclusions.


## Exploratory covariate screening

Before fitting the parametric hazard model, screen each candidate
covariate with a simple logistic regression against the event
indicator. This is much cheaper than fitting the full hazard model
and answers two distinct questions: which covariates carry signal at
all (rough significance triage), and what *functional form* each
covariate enters with — linear, log, polynomial, or something
non-monotone that needs binning. The screening step is what stops you
from blindly throwing every column at `hazard()` and then trying to
debug a model that has shape mis-specification baked in.

```{r}
#| label: explore-data
data(avc)
avc <- na.omit(avc)

# Candidate covariates
candidates <- c("age", "status", "mal", "com_iv", "inc_surg", "orifice")
```

Fit a univariable logistic regression for each candidate against the
death indicator. The p-values that come out are not the final word on
whether a covariate matters — they ignore the joint structure of the
hazard model and they treat the event time as a binary outcome — but
they do tell you which covariates have no chance of mattering, and
which deserve closer inspection.

```{r}
#| label: explore-logit
logit_fits <- lapply(candidates, function(var) {
  fmla <- as.formula(paste("dead ~", var))
  glm(fmla, data = avc, family = binomial())
})
names(logit_fits) <- candidates

# Which covariates are significant univariably?
p_values <- vapply(logit_fits, function(f) {
  coef(summary(f))[2, "Pr(>|z|)"]
}, numeric(1))

data.frame(covariate = names(p_values),
           p_value   = round(p_values, 4),
           row.names = NULL)
```

```{r}
#| label: fig-explore-age
#| fig-cap: "Exploratory: age vs. mortality with LOESS smooth"
#| fig-width: 7
#| fig-height: 4
ggplot(avc, aes(age, dead)) +
  geom_jitter(height = 0.05, width = 0, alpha = 0.3, size = 1) +
  geom_smooth(method = "loess", se = TRUE, colour = "#0072B2",
              linewidth = 0.8) +
  labs(x = "Age at operation (months)", y = "Death (0/1)") +
  theme_minimal()
```

The LOESS smooth reveals the *shape* of the covariate-mortality
relationship in a non-parametric way. A roughly linear smooth means
the covariate enters the hazard model on its natural scale; a U or
inverted-U shape means a polynomial or a non-monotone transform; a
plateau-then-rise means a threshold effect that may need binning.
This is the call you make *before* writing the formula for
`hazard()`, not after staring at a converged-but-misspecified model.


### Quantile calibration with `hzr_calibrate()`

`hzr_calibrate()` bins a continuous covariate into quantile groups and
applies the logit / Gompertz / Cox link transform, so you can eyeball
whether the covariate-outcome relationship is linear on the chosen link
scale.  It corresponds to the SAS `logit.sas` / `logitgr.sas` macros.

```{r}
#| label: calibrate-age
age_cal <- hzr_calibrate(
  x      = avc$age,
  event  = avc$dead,
  groups = 10,
  link   = "logit"
)
print(age_cal)
```

A roughly linear trend across groups means the logit link is
appropriate; strong curvature flags the need for a transform (for
example, polynomial or log).

## Nonparametric baselines

A parametric fit has to be compared against *something*. The natural
something is the nonparametric estimate of the same quantity from the
same data — Kaplan-Meier for survival, Nelson-Aalen for cumulative
hazard. The package's `hzr_kaplan()` and `hzr_nelson()` are
convenience wrappers that return richer output than `survival::survfit()`:
`hzr_kaplan()` ships KM with logit-transformed confidence limits
(more accurate in the tails than the standard Greenwood limits, which
can stray outside $[0, 1]$), interval hazard rates, density estimates,
and a restricted-mean-survival-time scalar. `hzr_nelson()` returns the
Wayne Nelson cumulative hazard estimator with log-normal confidence
limits — the right reference when you care about the integrated
intensity rather than the survival probability.

```{r}
#| label: km-baseline
km <- hzr_kaplan(time = avc$int_dead, status = avc$dead, conf_level = 0.95)
print(km)
```

```{r}
#| label: nelson-baseline
nel <- hzr_nelson(time = avc$int_dead, event = avc$dead, conf_level = 0.95)
print(nel)
```


## Bootstrap confidence intervals

The delta-method CIs `predict(se.fit = TRUE)` returns are fast and
asymptotically correct, but they lean on the Hessian-derived
parameter vcov — which can mislead when the likelihood surface is
poorly behaved near the MLE (boundary fits, near-degenerate
parameter combinations, small samples). The package flags these
cases for you: each fit carries `rcond` and `pd` diagnostics, and
`summary()` prints a note (with a matching warning at fit time) when
the Hessian is ill-conditioned or not positive-definite — a strong
signal to prefer the bootstrap. Bootstrap resampling is the
robust alternative: refit the model on each resampled dataset,
collect the resulting parameter and prediction values, and read the
empirical 2.5th / 97.5th percentiles as the 95% CI. It's
computationally heavier but it doesn't assume the likelihood is
locally quadratic, and it picks up the right tail behavior even
when delta-method theory creaks.

`hzr_bootstrap()` wraps the resample-and-refit loop and corresponds to
the SAS `bootstrap.hazard.sas` and `bootstrap.summary.sas` macros. We
fit a base model first; bootstrap replicates will resample from the
same data and refit using these starting values.

```{r}
#| label: fit-base
# Fit the base model that bootstrapping will use
fit <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 4)),
  fit   = TRUE,
  control = list(maxit = 500)
)

# Prediction grid at a median-risk profile
t_grid <- seq(0.01, max(avc$int_dead) * 0.9, length.out = 100)
base_nd <- data.frame(time = t_grid, age = median(avc$age),
                      status = 2, mal = 0, com_iv = 0)

surv_point <- predict(fit, newdata = base_nd, type = "survival")
```

`hzr_bootstrap()` returns a long data frame of per-replicate
coefficient estimates (`$replicates`) and a parameter-level summary
(mean, SD, 95% percentile CI) in `$summary`. We use `n_boot = 30`
here purely so the vignette renders in reasonable time — for a real
analysis you'd want 200 or more replicates. The `set.seed(42)` call
before `hzr_bootstrap()` makes the resample sequence reproducible.

```{r}
#| label: bootstrap
set.seed(42)
boot <- hzr_bootstrap(fit, n_boot = 30)  # kept small for vignette build time
print(boot)
```

The parameter-level CIs above come directly from the bootstrap
distribution.  Prediction-level CIs (on the survival curve or hazard
profile) require pivoting `boot$replicates` to wide format and
evaluating the parametric survival formula once per replicate.  That
workflow is out of scope for this section, but the building blocks are
all in the returned object.


## Goodness-of-fit overlay

The Kaplan-Meier overlay in the prediction-visualization vignette is
the *visual* goodness-of-fit check. `hzr_gof()` is the *quantitative*
complement: it tabulates parametric predictions side by side with the
nonparametric KM estimate at each event time, and computes the
Conservation-of-Events observed-vs-expected ratio across the whole
follow-up window. The ratio compresses everything the model is doing
into a single number, which makes it the right summary statistic to
report alongside coefficient tables. (`hzr_gof()` corresponds to the
SAS `hazplot.sas` macro.)

```{r}
#| label: gof-summary
gof <- hzr_gof(fit)
print(gof)
```

The observed vs. expected ratio (printed at the bottom) is the
Conservation-of-Events check: a well-specified model recovers the event
count exactly, so a ratio close to 1 is what we want.  Ratios well
above or below 1 flag either a misspecified shape or a covariate that
isn't really linear on the log-hazard scale.


## Decile-of-risk calibration

The conservation-of-events ratio above tells you whether the model is
calibrated *on average*. That's necessary but not sufficient — a
model can hit the right total event count while systematically
over-predicting risk for one part of the cohort and under-predicting
for another, with the errors cancelling in aggregate.
`hzr_deciles()` is the check that catches that failure mode: it
partitions patients into deciles of predicted risk at a chosen time
point and compares observed with expected event counts within each
decile, returning a chi-square goodness-of-fit test. A
well-calibrated model has observed-vs-expected close in every decile;
a model that's globally calibrated but locally biased will show large
residuals in the extreme deciles. (Corresponds to the SAS
`deciles.hazard.sas` macro.)

```{r}
#| label: deciles
deciles <- hzr_deciles(fit, time = 120, groups = 10)
print(deciles)
```

The chi-square test statistic and p-value summarise whether the
observed decile-by-decile event counts are consistent with what the
parametric model predicts.  A good visual check is to plot the two
columns side by side:

```{r}
#| label: fig-deciles
#| fig-cap: "Observed vs. expected mortality by decile of risk"
#| fig-width: 7
#| fig-height: 4.5
decile_df <- as.data.frame(deciles)
cal_long <- rbind(
  data.frame(group = decile_df$group,
             rate  = decile_df$observed_rate,
             Series = "Observed"),
  data.frame(group = decile_df$group,
             rate  = decile_df$expected_rate,
             Series = "Expected")
)
ggplot(cal_long, aes(group, rate, fill = Series)) +
  geom_col(position = position_dodge(width = 0.7), alpha = 0.8) +
  scale_fill_manual(values = c("Observed" = "#56B4E9",
                                "Expected" = "#E69F00")) +
  scale_x_continuous(breaks = seq_len(nrow(decile_df))) +
  scale_y_continuous(labels = scales::percent_format(accuracy = 1)) +
  labs(x = "Risk decile (1 = lowest)", y = "Mortality rate", fill = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```


## Sensitivity analysis

Coefficient estimates tell you the *direction and magnitude* of a
covariate's effect on the hazard scale, but they don't directly answer
the clinical question of *what survival difference this implies* for a
real patient. Sensitivity analysis closes that gap. You define two
or more covariate profiles — typically a reference profile and one or
more high-risk profiles — score each through `predict()`, and plot
the resulting survival curves on shared axes. The horizontal and
vertical gaps between curves translate the model's coefficients into
the clinically meaningful currency of survival probability differences
and median-survival shifts. It's also a stress test: if your high-risk
profile produces only a small survival gap, either the risk factors
aren't doing much in this cohort, or the model is missing the
mechanism that makes them matter.

```{r}
#| label: sensitivity
# Define reference and high-risk profiles
ref_profile <- data.frame(
  age    = median(avc$age),
  status = 2,
  mal    = 0,
  com_iv = 0
)

high_risk <- data.frame(
  age    = quantile(avc$age, 0.90),
  status = 4,
  mal    = 1,
  com_iv = 1
)
```

```{r}
#| label: fig-sensitivity
#| fig-cap: "Sensitivity analysis: reference vs. high-risk profile"
#| fig-width: 7
#| fig-height: 4.5
sens_curves <- do.call(rbind, lapply(
  list("Reference" = ref_profile, "High risk" = high_risk),
  function(p) {
    nd <- p[rep(1, length(t_grid)), ]
    nd$time <- t_grid
    data.frame(time = t_grid,
               survival = predict(fit, newdata = nd, type = "survival") * 100,
               Profile = deparse(substitute(p)))
  }
))
# Fix profile labels
sens_curves$Profile <- rep(c("Reference", "High risk"),
                           each = length(t_grid))
sens_curves$Profile <- factor(sens_curves$Profile,
                              levels = c("Reference", "High risk"))

ggplot(sens_curves, aes(time, survival, colour = Profile)) +
  geom_line(linewidth = 0.9) +
  scale_colour_manual(values = c("Reference" = "#009E73",
                                 "High risk" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

The gap between the curves quantifies the clinical impact of the risk factors.
The sensitivity analysis is most informative when combined with bootstrap
confidence intervals on each curve, showing whether the difference is
statistically meaningful.


## Analysis workflow summary

The pieces in this vignette aren't independent diagnostics — they
chain together into a single analytical workflow that goes from raw
covariates to a defensible, uncertainty-quantified, well-calibrated
fitted model. The complete sequence, with the SAS HAZARD
correspondences each piece descends from:

1. **Exploratory screening** (`glm`, `hzr_calibrate()`) --- identify
   covariate transformations and functional forms
2. **Nonparametric baselines** (`hzr_kaplan()`, `hzr_nelson()`) ---
   reference KM / Nelson cumulative hazard estimators
3. **Fit hazard model** (`hazard()`) --- parametric shape + covariates
4. **Predict & visualize** (`predict()`) --- survival, hazard, risk profiles
5. **Goodness-of-fit overlay** (`hzr_gof()`) --- parametric vs. KM +
   Conservation-of-Events check
6. **Bootstrap CIs** (`hzr_bootstrap()`) --- uncertainty quantification
   via resampling
7. **Sensitivity analysis** (`predict()` on covariate profiles) ---
   compare scenarios across risk factors
8. **Decile calibration** (`hzr_deciles()`) --- chi-square observed
   vs. expected by risk decile

See `vignette("getting-started")` for the minimal workflow and
`vignette("fitting-hazard-models")` for the model-fitting details.

---
title: "Prediction & Visualization"
subtitle: "Survival curves, phase decomposition, and risk profiles"
format:
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    number-sections: true
vignette: >
  %\VignetteIndexEntry{Prediction & Visualization}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)
```

```{r}
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)
```

A fitted hazard model is only useful insofar as you can extract
answers from it. This vignette covers the predict-and-visualize half
of the workflow: pulling survival probabilities, cumulative hazards,
and patient-level risk scores out of a fitted model, attaching
delta-method uncertainty to those predictions, and plotting them in
the formats that come up in actual clinical analyses.

The structure mirrors how a real analysis unfolds. We start with the
Kaplan-Meier baseline (the nonparametric reference every parametric
fit gets judged against), enumerate the prediction types the model
exposes, plot the parametric survival curve against KM as a
goodness-of-fit check, layer on confidence bands, then move to the
multiphase decomposition and patient-specific risk profiles where the
covariate machinery pays off. The last section reuses the workflow
across multiple endpoints on the same cohort. If you haven't seen the
fitting workflow yet, `vignette("fitting-hazard-models")` is the
prerequisite.


## Kaplan-Meier baseline

The Kaplan-Meier curve is what the data itself says about survival
before any parametric model intervenes. It makes no assumption about
the shape of the hazard, handles censoring honestly, and gives you a
nonparametric step function that's the gold-standard reference for any
follow-up fit. Every parametric prediction that follows gets compared
back to this curve — if a model's smooth prediction can't track the KM
step function, the model is missing something the data has been
telling you all along.

```{r}
#| label: km-baseline
data(avc)
avc <- na.omit(avc)
km <- survfit(Surv(int_dead, dead) ~ 1, data = avc)
```

```{r}
#| label: fig-km
#| fig-cap: "Kaplan-Meier survival estimate: death after AVC repair"
#| fig-width: 7
#| fig-height: 4
km_df <- data.frame(time = km$time, survival = km$surv * 100)

ggplot(km_df, aes(time, survival)) +
  geom_step(linewidth = 0.6) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after repair", y = "Freedom from death (%)") +
  theme_minimal()
```


## Prediction types

The `predict()` method exposes four quantities through its `type=`
argument, each useful for a different downstream question:

- `"linear_predictor"` — $x^\top \beta$, the covariate-driven
  log-hazard shift. Use it to rank patients on relative risk without
  pinning down absolute survival probabilities.
- `"hazard"` — $\exp(x^\top \beta)$, the multiplicative hazard ratio
  relative to the baseline. Use it when you want hazard ratios for
  reporting or for further calculation.
- `"cumulative_hazard"` — $H(t \mid x)$, the integrated hazard up to
  time $t$ for each row. Use it when you need the raw integrated
  intensity (decomposing it by phase, for example, or computing
  derived quantities like the expected number of events).
- `"survival"` — $S(t \mid x) = \exp(-H(t \mid x))$, the probability
  of surviving past time $t$. This is what clinicians and patients
  actually want to see; it's also what we plot for diagnostics.

The two predictions we extract below are `"survival"` and
`"cumulative_hazard"` — survival for the clinical communication, and
the cumulative hazard so we can sanity-check the relationship
$S = \exp(-H)$ holds row-by-row. We fit a multivariable Weibull on the
AVC death endpoint first.

```{r}
#| label: fit-mv
fit <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 4)),
  fit   = TRUE,
  control = list(maxit = 500)
)
```

Pick a representative patient — here a median-aged, mid-status,
no-malalignment, no-grade-IV-complication profile — and evaluate the
prediction types over a fine time grid. The result is a data frame
with one row per time point, ready for plotting or downstream
analysis.

```{r}
#| label: pred-types
t_grid <- seq(0.01, max(avc$int_dead) * 0.95, length.out = 200)
profile <- data.frame(
  time   = t_grid,
  age    = median(avc$age),
  status = 2,
  mal    = 0,
  com_iv = 0
)

surv   <- predict(fit, newdata = profile, type = "survival")
cumhaz <- predict(fit, newdata = profile, type = "cumulative_hazard")

profile$survival          <- surv
profile$cumulative_hazard <- cumhaz

head(profile[, c("time", "survival", "cumulative_hazard")])
```


## Parametric survival with KM overlay

The fundamental diagnostic for any parametric survival fit is whether
its predictions track the Kaplan-Meier step function. Plot the
median-profile survival curve from the previous chunk on top of the
KM estimate from the raw cohort. Where the parametric curve hugs the
steps, the model is faithful to the data; where it drifts, it's
imposing a shape the data doesn't support. This isn't a hypothesis
test, it's an eyeball check — and it's the single most informative
thing you can do with a fitted model before trusting its
predictions.

```{r}
#| label: fig-surv-overlay
#| fig-cap: "Weibull parametric survival vs. Kaplan-Meier (AVC death)"
#| fig-width: 7
#| fig-height: 4.5
ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_line(data = profile,
            aes(time, survival * 100, colour = "Parametric (Weibull)"),
            linewidth = 1) +
  scale_colour_manual(
    values = c("Parametric (Weibull)" = "#0072B2",
               "Kaplan-Meier"         = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

## Confidence limits on predictions

A point estimate without an uncertainty band tells you what the model
thinks, not how confident it is. For patient-level survival
predictions that distinction matters: a survival probability of 0.85
with a tight band around it is a fundamentally different number than
a survival probability of 0.85 with a 0.4–0.97 band around it, even
though both round to "85%". Delta-method confidence limits let you
plot the second case honestly instead of pretending it's the first.

As of v0.9.8, `predict.hazard()` accepts `se.fit = TRUE` (default
`FALSE`) and `level = 0.95` to return delta-method standard errors
and confidence limits alongside the point estimate. The return value
changes shape: a plain numeric vector with `se.fit = FALSE`, a data
frame with columns `fit`, `se.fit`, `lower`, and `upper` with
`se.fit = TRUE`.

```{r}
#| label: pred-with-se
# Build a clean newdata frame (the earlier chunk appended result
# columns to `profile`, which would confuse predict()'s column count).
profile_ci <- data.frame(
  time   = t_grid,
  age    = median(avc$age),
  status = 2,
  mal    = 0,
  com_iv = 0
)
surv_ci <- predict(fit, newdata = profile_ci,
                   type = "survival", se.fit = TRUE, level = 0.95)
head(surv_ci)
```

Confidence limits use SAS-matched transformations chosen so the
interval respects the natural range of each quantity: log-scale for
`hazard` and `cumulative_hazard` (keeps the lower bound positive),
and `log(-log(S))` for `survival` (keeps the interval inside
$[0, 1]$). The linear predictor uses symmetric natural-scale CLs.
The point isn't notational fussiness — it's that a symmetric CI on
survival probability would frequently dip below 0 or rise above 1,
giving nonsense values that the transformed form avoids by construction.

```{r}
#| label: fig-surv-ci
#| fig-cap: "Parametric survival with 95% delta-method confidence band"
#| fig-width: 7
#| fig-height: 4.5
ci_df <- data.frame(
  time     = profile_ci$time,
  survival = surv_ci$fit * 100,
  lower    = surv_ci$lower * 100,
  upper    = surv_ci$upper * 100
)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_ribbon(data = ci_df,
              aes(time, ymin = lower, ymax = upper),
              fill = "#0072B2", alpha = 0.15) +
  geom_line(data = ci_df,
            aes(time, survival, colour = "Parametric + 95% CI"),
            linewidth = 1) +
  scale_colour_manual(
    values = c("Parametric + 95% CI" = "#0072B2",
               "Kaplan-Meier"        = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

Implementation detail worth knowing: Weibull and multiphase use a
closed-form Jacobian for the delta-method calculation. Exponential,
log-logistic, and log-normal fall back to `numDeriv::jacobian()` on a
per-call cumulative-hazard closure — identical results, just slightly
slower because the Jacobian is computed numerically per prediction
point. The user-facing API is the same regardless of distribution.

## Decomposed multiphase hazard

The multiphase model's whole point is that the total hazard is a sum
of phase contributions. `predict()` with `decompose = TRUE` and
`type = "cumulative_hazard"` exposes those contributions row by row,
returning a data frame with one column per phase plus a `total`
column. Numerically differentiating each column gives you the
instantaneous hazard rate for each phase — which is the diagnostic
that tells you whether the multiphase model is doing what you asked
of it. The early phase should dominate near $t = 0$ and fall off, the
constant phase should be a flat floor, and the late phase should sit
near zero early and rise after a lag.

```{r}
#| label: fit-mp
data(cabgkul)

fit_mp <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = cabgkul,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",  tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)
```

```{r}
#| label: fig-decomposed-hazard
#| fig-cap: "Additive phase decomposition: total hazard rate (solid) = early + constant + late (dashed)"
#| fig-width: 7
#| fig-height: 4.5
t_mp <- seq(0.01, max(cabgkul$int_dead) * 0.95, length.out = 200)
nd   <- data.frame(time = t_mp)

decomp <- predict(fit_mp, newdata = nd, type = "cumulative_hazard",
                  decompose = TRUE)

# Numerical differentiation: h(t) ≈ ΔH(t) / Δt
num_hazard <- function(cumhaz, time) {
  dt <- diff(time)
  dH <- diff(cumhaz)
  c(dH[1] / dt[1], dH / dt)
}

h_long <- rbind(
  data.frame(time = t_mp, hazard = num_hazard(decomp$early, t_mp),
             Phase = "Early"),
  data.frame(time = t_mp, hazard = num_hazard(decomp$constant, t_mp),
             Phase = "Constant"),
  data.frame(time = t_mp, hazard = num_hazard(decomp$late, t_mp),
             Phase = "Late"),
  data.frame(time = t_mp, hazard = num_hazard(decomp$total, t_mp),
             Phase = "Total")
)
h_long$Phase <- factor(h_long$Phase,
                       levels = c("Total", "Early", "Constant", "Late"))

ggplot(h_long, aes(time, hazard, colour = Phase, linetype = Phase)) +
  geom_line(aes(linewidth = Phase)) +
  scale_colour_manual(values = c(Total = "#222222", Early = "#E69F00",
                                 Constant = "#56B4E9", Late = "#CC79A7")) +
  scale_linetype_manual(values = c(Total = "solid", Early = "dashed",
                                   Constant = "dashed", Late = "dashed")) +
  scale_linewidth_manual(values = c(Total = 1.3, Early = 0.7,
                                    Constant = 0.7, Late = 0.7)) +
  labs(x = "Months after CABG", y = "Hazard rate",
       colour = "Phase", linetype = "Phase", linewidth = "Phase") +
  theme_minimal() +
  theme(legend.position = "bottom")
```

Each phase is doing its job. The early (orange) phase captures the
steep post-operative risk that peaks within the first months. The
constant (blue) phase represents the ongoing background mortality
that persists once patients are past the operative window. The late
(pink) phase captures the gradually increasing risk of late
attrition — graft failure, comorbidity progression, the aging cohort.
Crucially, none of these shapes was specified by hand; the optimizer
landed on them given the data and the phase-type choices we made up
front. If a phase looked wrong here (a constant phase dominating
where we expected an early peak, or a late phase that never rose)
that would be a signal to revisit either the starting values, the
shape parameters, or the data itself.


## Multiphase survival with KM overlay

The decomposed-hazard plot above tells us the model has the right
*shape* per phase. To check that the *total* fit also tracks the
data, we collapse back to the overall survival curve and overlay it
on the KM estimate — same diagnostic as the single-Weibull case
earlier in this vignette, but now against a model that has the
structural flexibility to follow the cohort's actual mortality
pattern.

```{r}
#| label: fig-mp-surv
#| fig-cap: "Multiphase parametric survival vs. Kaplan-Meier"
#| fig-width: 7
#| fig-height: 4.5
surv_mp <- predict(fit_mp, newdata = nd, type = "survival") * 100

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_line(data = data.frame(time = t_grid, survival = surv_mp),
            aes(time, survival, colour = "Multiphase (3-phase)"),
            linewidth = 1) +
  scale_colour_manual(
    values = c("Multiphase (3-phase)" = "#0072B2",
               "Kaplan-Meier"         = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```


## Patient-specific risk profiles

This is the payoff of having covariates in the model. A
population-level curve is fine for cohort-level reporting, but the
question a clinician usually wants answered is: *what does the model
predict for this particular patient, given their risk factors?*
Holding the fitted model fixed and varying the covariate profile
generates patient-specific survival curves you can show side by side.
Below we score three plausible profiles drawn from the cohort's own
quantiles — a low-risk patient (young, mild status, no malalignment,
no grade-IV complications), a median patient, and a high-risk patient
(older, severe status, with both adverse anatomical and
post-operative findings).

```{r}
#| label: fig-risk-profiles
#| fig-cap: "Predicted survival by risk profile"
#| fig-width: 7
#| fig-height: 4.5
profiles <- list(
  "Low risk"  = data.frame(age = quantile(avc$age, 0.25),
                            status = 1, mal = 0, com_iv = 0),
  "Median"    = data.frame(age = median(avc$age),
                            status = 2, mal = 0, com_iv = 0),
  "High risk" = data.frame(age = quantile(avc$age, 0.90),
                            status = 4, mal = 1, com_iv = 1)
)

curves <- do.call(rbind, lapply(names(profiles), function(nm) {
  nd <- profiles[[nm]][rep(1, length(t_grid)), ]
  nd$time <- t_grid
  data.frame(time = t_grid,
             survival = predict(fit, newdata = nd, type = "survival") * 100,
             Profile = nm)
}))
curves$Profile <- factor(curves$Profile,
                         levels = c("Low risk", "Median", "High risk"))

ggplot(curves, aes(time, survival, colour = Profile)) +
  geom_line(linewidth = 0.9) +
  scale_colour_manual(values = c("Low risk" = "#009E73",
                                 "Median"   = "#0072B2",
                                 "High risk" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

The gap between the three curves is the model's *prognostic
discrimination*: a wider spread means the covariates are doing real
work, separating patients with different actual risk levels. A
collapsed plot (all three curves on top of each other) means the
covariate effects are too weak to distinguish patients meaningfully,
even if they're statistically significant.


## Multi-endpoint visualization: valves

When a cohort has multiple clinical endpoints — death, valve
endocarditis, reoperation — putting them on the same survival axis
gives an immediate comparative picture of which event the cohort is
most at risk for, and over what time horizon. The `valves` dataset
has separate event-time pairs for each endpoint, so we can plot all
of them as Kaplan-Meier curves on shared axes without needing a
parametric fit at all. (You'd parametrize them in the next step, one
endpoint per `hazard()` call, as we did in
`vignette("fitting-hazard-models")`.)

```{r}
#| label: fig-valves-endpoints
#| fig-cap: "Freedom from death and PVE after valve replacement"
#| fig-width: 7
#| fig-height: 4.5
data(valves)
valves <- na.omit(valves)

km_death <- survfit(Surv(int_dead, dead) ~ 1, data = valves)
km_pve   <- survfit(Surv(int_pve, pve) ~ 1, data = valves)

ep_df <- rbind(
  data.frame(time = km_death$time, survival = km_death$surv * 100,
             Endpoint = "Death"),
  data.frame(time = km_pve$time, survival = km_pve$surv * 100,
             Endpoint = "PVE")
)

ggplot(ep_df, aes(time, survival, colour = Endpoint)) +
  geom_step(linewidth = 0.7) +
  scale_y_continuous(limits = c(0, 100)) +
  scale_colour_manual(values = c("Death" = "#D55E00", "PVE" = "#0072B2")) +
  labs(x = "Months after valve replacement",
       y = "Freedom from event (%)", colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

---
title: "Getting Started with TemporalHazard"
subtitle: "Build, fit, and predict with temporal parametric hazard models"
format:
  html:
    code-fold: false
vignette: >
  %\VignetteIndexEntry{Getting Started with TemporalHazard}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg    <- requireNamespace("TemporalHazard", quietly = TRUE)
has_ggplot2 <- requireNamespace("ggplot2",       quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = has_pkg
)
```

TemporalHazard fits *parametric hazard models* to time-to-event data.
If you've worked with `survival::coxph()` or Kaplan-Meier curves before
you already have most of the vocabulary; this is a quick refresh of the
bits that matter for what follows.

A **hazard** is the instantaneous risk of the event at time $t$, given
the subject has survived until $t$. A Kaplan-Meier curve reflects it
implicitly: KM gives you survival probabilities, and the rate at which
those probabilities drop is (minus) the hazard. A **parametric** hazard
model writes the hazard as a smooth function of $t$ with a small number
of parameters, plus covariate effects. The payoff over nonparametric
KM is a smooth curve you can extrapolate, patient-specific risk
predictions, and the ability to compare fitted hazard *shapes* across
groups or models.

This vignette walks the minimal workflow. We start with a
single-distribution Weibull fit on simulated data — fit, summary,
predict — then move to the multiphase fit on CABGKUL, the canonical
three-phase cardiac-surgery example.

## A first Weibull fit

The simulated data below has 180 patients with three plausible risk
covariates — age, NYHA functional class, and cardiogenic shock. We fit
a single Weibull hazard: a scale parameter (`mu`) and a shape exponent
(`nu`) that lets the hazard accelerate when `nu > 1`, decelerate when
`nu < 1`, or stay flat at `nu = 1`. The Weibull is the natural starting
point for a single-distribution fit because it covers all three
monotone shapes with just two parameters.

The `hazard()` interface takes a survival formula
(`Surv(time, status)` on the left, covariates on the right), a
starting-value vector (`theta`), and a distribution name. Passing
`fit = TRUE` runs the optimizer and returns a fitted model; passing
`fit = FALSE` returns the model specification without fitting, which is
useful for sanity-checking the formula and starting values before
committing to the optimizer.

```{r}
library(TemporalHazard)
library(survival)

set.seed(1001)
n <- 180
dat <- data.frame(
  time = rexp(n, rate = 0.35) + 0.05,
  status = rbinom(n, size = 1, prob = 0.6),
  age = rnorm(n, mean = 62, sd = 11),
  nyha = sample(1:4, n, replace = TRUE),
  shock = rbinom(n, size = 1, prob = 0.18)
)

fit <- TemporalHazard::hazard(
  Surv(time, status) ~ age + nyha + shock,
  data = dat,
  theta = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
  dist = "weibull",
  fit = TRUE,
  control = list(maxit = 300)
)

summary(fit)
```

## Prediction workflow

A fitted hazard model lets you score new patients without refitting.
The `predict()` method takes a `newdata` frame and a `type` argument
that selects which quantity to compute:

- `"linear_predictor"` — the covariate-driven log-hazard-ratio for
  each row, $x^\top \beta$.
- `"hazard"` — the hazard multiplier, $\exp(x^\top \beta)$.
- `"survival"` — the probability of surviving past each row's `time`,
  $S(t \mid x)$.
- `"cumulative_hazard"` — the integrated hazard up to each row's
  `time`, $H(t \mid x) = -\log S(t \mid x)$.

The three rows below are made-up patients spanning the covariate range
in the training data: a younger, low-NYHA, no-shock patient at an
early time; a middle case at the median; and an older, high-NYHA,
shock patient further out. We compute all four prediction types for
each.

```{r}
new_patients <- data.frame(
  time = c(0.5, 1.5, 3.0),
  age = c(50, 65, 75),
  nyha = c(1, 3, 4),
  shock = c(0, 0, 1)
)

pred_input <- new_patients

new_patients$linear_predictor <- predict(fit, newdata = pred_input, type = "linear_predictor")
new_patients$hazard_multiplier <- predict(fit, newdata = pred_input, type = "hazard")
new_patients$survival <- predict(fit, newdata = pred_input, type = "survival")
new_patients$cumulative_hazard <- predict(fit, newdata = pred_input, type = "cumulative_hazard")

new_patients
```

## Visualizing predicted survival

The summary table and the patient-level scores tell us the fit
*converged* and what it *predicts*, but not whether the predicted shape
matches the data. For that we plot the model. Below we build a smooth
survival curve over a fine time grid for a median-profile patient and
overlay the Kaplan-Meier nonparametric estimate from the raw data on
the same axes. If the parametric curve hugs the KM step function the
Weibull shape is honest to the data; visible drift in either direction
flags a model the optimizer fit happily but that doesn't actually
describe the cohort.

```{r}
#| label: fig-survival
#| fig-cap: "Parametric Weibull survival curve with Kaplan-Meier overlay"
#| fig-width: 7
#| fig-height: 5
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
library(ggplot2)

# Parametric curve on a fine grid
t_grid <- seq(0.05, max(dat$time), length.out = 80)
curve_df <- data.frame(
  time  = t_grid,
  age   = median(dat$age),
  nyha  = 2,
  shock = 0
)
curve_df$survival <- predict(fit, newdata = curve_df, type = "survival") * 100

# Kaplan-Meier empirical overlay
km <- survival::survfit(survival::Surv(time, status) ~ 1, data = dat)
km_df <- data.frame(time = km$time, survival = km$surv * 100)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier")) +
  geom_line(data = curve_df, aes(time, survival,
                                  colour = "Parametric (Weibull)")) +
  scale_colour_manual(
    values = c("Parametric (Weibull)" = "#0072B2",
               "Kaplan-Meier"         = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after surgery", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

## Multiphase models

The Weibull fit above describes the hazard with one smooth, monotone
curve — it can rise, fall, or stay flat over time, but it cannot change
direction. That's fine for some processes (light bulbs failing, parts
wearing out). It breaks down quickly for clinical data.

Consider CABG (coronary artery bypass graft) surgery. In the days right
after the operation the risk of death is at its highest: the patient is
in the ICU, exposed to operative complications, infections, bleeding.
If they survive that window the risk drops to a low, roughly flat
background rate: they recover, go home, live their lives. Years later
the risk rises again as the grafts begin to deteriorate and the cohort
ages. Three distinct regimes, each with its own clinical mechanism.

No single Weibull (or log-normal, or any other monotone distribution)
can fit all three at once. The optimizer would have to compromise:
picking a shape that's mediocre everywhere instead of right in any one
regime. You'd predict too few early deaths, the wrong slope in the
middle, and too few late deaths.

The **multiphase decomposition** is the workaround. Instead of asking
one distribution to do everything, we describe the total cumulative
hazard as a *sum* of phase-specific contributions, each with its own
temporal shape:

$$H(t \mid x) = \sum_{j=1}^{J} \mu_j(x) \cdot \Phi_j(t)$$

Each $\Phi_j(t)$ is a phase-specific *temporal shape*: a unit-scaled
curve that says *how* phase $j$'s hazard accumulates over time. Each
$\mu_j(x)$ is the phase-specific *scale*: how much cumulative hazard
belongs to phase $j$, possibly modulated by covariates. The phases
overlap and their hazards add. There's no switching, no thresholds, no
piecewise joins; at any time $t$ the instantaneous hazard $h(t) = dH/dt$
is the sum of the per-phase contributions $\mu_j \cdot \phi_j(t)$, where
$\phi_j = d\Phi_j/dt$.

### Why multiple phases?

Because the number of phases is the modeling lever that lets you match
the data's actual structure. Some processes are well-described by two
phases (early failure + steady-state). Most cardiac-surgery cohorts
need three: early operative risk + constant background + late
deterioration. A small handful — acute aortic dissection is the
classic — need four (operative + subacute + constant + late). The
framework is general; the *count* is a modeling choice driven by
clinical knowledge and the shape of the Kaplan-Meier cumulative hazard
curve.

For the CABGKUL fit below we use three phases. That's not a property of
the package; it's a property of this dataset.

Each phase is specified with `hzr_phase()`. The first argument picks
the shape function: `"cdf"` for a saturating curve bounded between 0
and 1 (the SAS "early" / G1 shape), `"constant"` for a flat hazard
plateau (SAS "G2"), `"g3"` for the polynomial late-rising shape from
the SAS "late" library. Remaining arguments set the shape's free
parameters, or fix them with `fixed = "shapes"`. The Phase types
section below has the full menu.

For this fit we use the textbook three-phase decomposition: a
saturating early peak, a constant background, and a polynomial late
rise. We fix the shapes so we can compare the fitted scales against
the published reference; in your own work you would usually estimate
them.

```{r}
#| label: multiphase-fit
# CABGKUL is the benchmark dataset for 3-phase decomposition (n = 5,880)
data(cabgkul)

fit_mp <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = cabgkul,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.2, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3",  tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)

summary(fit_mp)
```

The summary marks most rows `NA` in the `std_error`, `z_stat`, and
`p_value` columns. Two mechanisms produce these, both deliberate:

- The seven shape parameters carry `fixed = "shapes"` and act as
  user-supplied constants (`t_half`, `nu`, `m` on the early phase; `tau`,
  `gamma`, `alpha`, `eta` on the late phase). The optimizer never moves
  them, so they have no estimated variance to report.
- One of the three `log_mu` scale parameters is also `NA`. By default the
  multiphase optimizer enforces Conservation of Events (Turner's theorem):
  one phase's `log_mu` is solved analytically at every iteration so that
  total predicted events equal total observed. That phase is not a free
  parameter at the MLE, so its inverse-Hessian variance is degenerate, and
  the table marks it `NA` rather than print a misleading number. Pass
  `control = list(conserve = FALSE)` to disable Conservation of Events and
  estimate all three scale parameters freely.

The two remaining `log_mu` rows are the optimizer-estimated free
parameters; their Wald statistics reflect the two-parameter fit
dimension that remains after the Conservation-of-Events constraint
absorbs the third `log_mu`.

### Decomposed hazard visualization

The MLE numbers in the summary tell us *what* the model committed to,
but not *how* the three phases fit together over time. For that we
need the per-phase contributions. The `decompose = TRUE` argument on
`predict()` returns one cumulative-hazard column per phase plus the
total; we numerically differentiate each one (a coarse first-difference
is fine here, since we just want to look) to get an instantaneous
hazard rate.

The plot that follows is the diagnostic that matters: it shows whether
the model carved up the timeline the way we expected. The early phase
should dominate near $t = 0$ and die off. The constant phase should be
a flat floor. The late phase should be near zero early and rise after
a lag. If any of those shapes looks wrong, the fix is in the starting
values or in the choice of shape function, not in the data.

```{r}
#| label: fig-hazard-phases
#| fig-cap: "Additive phase decomposition: total hazard (solid) = early + constant + late (dashed)"
#| fig-width: 7
#| fig-height: 4.5
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.95, length.out = 200)
nd     <- data.frame(time = t_grid)

# decompose = TRUE returns per-phase cumulative hazard columns
decomp <- predict(fit_mp, newdata = nd, type = "cumulative_hazard",
                  decompose = TRUE)

# Numerical differentiation: h(t) ≈ ΔH(t) / Δt
num_hazard <- function(cumhaz, time) {
  dt <- diff(time)
  dH <- diff(cumhaz)
  c(dH[1] / dt[1], dH / dt)
}

h_long <- rbind(
  data.frame(time = t_grid, hazard = num_hazard(decomp$early, t_grid),
             Phase = "Early"),
  data.frame(time = t_grid, hazard = num_hazard(decomp$constant, t_grid),
             Phase = "Constant"),
  data.frame(time = t_grid, hazard = num_hazard(decomp$late, t_grid),
             Phase = "Late"),
  data.frame(time = t_grid, hazard = num_hazard(decomp$total, t_grid),
             Phase = "Total")
)
h_long$Phase <- factor(h_long$Phase,
                       levels = c("Total", "Early", "Constant", "Late"))

ggplot(h_long, aes(time, hazard, colour = Phase, linetype = Phase)) +
  geom_line(aes(linewidth = Phase)) +
  scale_colour_manual(values = c(Total = "#222222", Early = "#E69F00",
                                 Constant = "#56B4E9", Late = "#CC79A7")) +
  scale_linetype_manual(values = c(Total = "solid", Early = "dashed",
                                   Constant = "dashed", Late = "dashed")) +
  scale_linewidth_manual(values = c(Total = 1.3, Early = 0.7,
                                    Constant = 0.7, Late = 0.7)) +
  labs(x = "Months after CABG", y = "Hazard rate",
       colour = "Phase", linetype = "Phase", linewidth = "Phase") +
  theme_minimal() +
  theme(legend.position = "bottom")
```

### Multiphase survival with Kaplan-Meier overlay

The decomposition plot tells us the model has the right *shape*; the
overlay plot tells us it has the right *level*. Plotting the
parametric survival curve against the Kaplan-Meier estimate on the
same axis is the single most useful diagnostic for a hazard fit. KM
is the gold-standard nonparametric reference; if the parametric curve
drifts away from it, something is wrong with the model that more
iterations won't fix.

```{r}
#| label: fig-mp-survival
#| fig-cap: "Multiphase parametric survival vs. Kaplan-Meier"
#| fig-width: 7
#| fig-height: 4.5
#| eval: !expr requireNamespace("TemporalHazard", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
surv_df <- data.frame(
  time     = t_grid,
  survival = predict(fit_mp, newdata = nd, type = "survival") * 100
)

km    <- survfit(Surv(int_dead, dead) ~ 1, data = cabgkul)
km_df <- data.frame(time = km$time, survival = km$surv * 100)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.6) +
  geom_line(data = surv_df, aes(time, survival, colour = "Multiphase model"),
            linewidth = 1.1) +
  scale_colour_manual(
    values = c("Multiphase model" = "#0072B2", "Kaplan-Meier" = "#D55E00")
  ) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after CABG", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

### Phase types

Picking the right phase shape is the single biggest modeling decision
in a multiphase fit. The package ships the canonical shapes from the
SAS C HAZARD library:

- **`"cdf"`** — a saturating curve $\Phi(t) \in [0, 1]$, parameterized
  by `t_half` (the time at which $G(t_{1/2}) = 0.5$), `nu` (a time
  exponent), and `m` (a shape exponent). This is the SAS "early" or G1
  shape. Use it for the early phase, or for any phase whose accumulated
  hazard saturates rather than growing without bound.
- **`"constant"`** — $\Phi(t) = t$. The phase's hazard rate is flat;
  there are no shape parameters to estimate, only the scale $\mu$. This
  is the SAS G2 shape. Use it for a background plateau.
- **`"g3"`** — the SAS "late" shape, parameterized by `tau` (a
  time-scale), `gamma` (a time exponent), `alpha` (a shape parameter;
  $\alpha = 0$ gives an exponential limit), and `eta` (an outer
  exponent). Use it for a late phase that is near-zero early and
  accelerates over time — graft deterioration over years is the
  archetype.

See `?hzr_phase` for the additional `"hazard"` shape and the full
parameterization of each.

In practice you choose phase shapes by looking at the Kaplan-Meier
cumulative hazard plot, identifying which regimes are present, and
matching each regime to the shape family whose behavior fits. Then
you fit, look at the decomposition plot, and adjust starting values
or fix shapes if a phase doesn't land where you expected.

```{r}
#| eval: false
hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 1)   # Early risk (bounded)
hzr_phase("constant")                                   # Flat background rate
hzr_phase("cdf",      t_half = 10,  nu = 1, m = 0)    # Late risk (CDF-based)
hzr_phase("g3",       tau = 1, gamma = 3, alpha = 1,   # Late risk (G3 power law)
                       eta = 1)
```

The `"g3"` type uses the four-parameter G3 decomposition from the original
C/SAS HAZARD program, providing unbounded power-law growth for late-phase
hazards. See `vignette("mf-mathematical-foundations")` for the full
mathematical treatment.

## Numerical helpers

The package ships a small set of numerical helpers used internally to
keep the likelihood evaluations stable across the wide range of
arguments an optimizer encounters. They're exported so you can reach
for them in custom code or when debugging a fit.

The most common one is `hzr_log1pexp(x)`, which computes
$\log(1 + e^x)$ in a numerically stable way. The naive `log(1 + exp(x))`
overflows once $x$ exceeds about 700 (because `exp(x)` itself
overflows); the helper switches to the asymptotic $x + \log(1 + e^{-x})$
for large $x$, which is exact to floating-point precision. The package
uses it inside every log-likelihood expression that involves a
log-survival term, so the same evaluation can run on $x = -50$ and
$x = 5000$ without producing `Inf` or `NaN`.

```{r}
TemporalHazard::hzr_log1pexp(c(-2, 0, 2))
```

---
title: "Migrating from SAS HAZARD to TemporalHazard"
format: html
vignette: >
  %\VignetteIndexEntry{Migrating from SAS HAZARD to TemporalHazard}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = has_pkg
)
```

```{r}
#| label: setup
library(TemporalHazard)
```

## Overview

If you've been running multiphase hazard analyses in SAS HAZARD —
the macro suite that wraps the C HAZARD binary written by Blackstone,
Naftel, and Turner at UAB — this vignette is the bridge to running
the same analyses in R. Every SAS `HAZARD` statement, every common
macro option, and every output field maps to something in
`TemporalHazard`; this document spells out the correspondences and
flags the small handful of features the R port doesn't yet cover.

The mapping is faithful, not literal. SAS HAZARD is a step-driven
DSL: you write a `PROC HAZARD` block with separate `TIME`, `EVENT`,
`PARMS`, `EARLY`, `CONSTANT`, and `SELECTION` statements, and the
macro assembles them into a binary call. R is function-driven: you
write a single `hazard()` call with named arguments. The conceptual
pieces are identical; the syntactic ergonomics differ. Use this
vignette to translate an existing SAS analysis, or as a reference
when comparing the package's output against a SAS HAZARD reference
fit for parity testing.

The full formal argument mapping table is available programmatically
and ships with the package — handy when you want to grep for a
specific SAS parameter name and find its R equivalent without
scrolling through the prose below:

```{r}
#| label: mapping-table
knitr::kable(
  TemporalHazard::hzr_argument_mapping(),
  caption = "Formal argument map: SAS HAZARD/C → hazard()",
  col.names = c(
    "SAS Statement", "Legacy Input", "C Concept",
    "R Parameter", "Required", "Expected Type",
    "Transform Rule", "Status", "Notes"
  )
)
```

---

## Statement-by-statement mapping

A SAS HAZARD analysis is a stack of statements inside a `PROC HAZARD`
block. We walk through them in roughly the order they appear in a
typical analysis script, showing the SAS form and the corresponding
R call.

### `PROC HAZARD DATA=`

The procedure-level statement that names the dataset and sets
global options. The `NOCOV` / `NOCOR` flags suppress covariance and
correlation output; `CONDITION=` is a tolerance switch for the
convergence check.

```sas
PROC HAZARD DATA=AVCS NOCOV NOCOR CONDITION=14;
```

Maps to `hazard()` `control` list:

```{r}
#| eval: false
fit <- hazard(
  ...,
  control = list(
    nocov      = TRUE,   # suppress covariance output
    nocor      = TRUE,   # suppress correlation output
    condition  = 14      # CONDITION= switch
  )
)
```

Additional `PROC HAZARD` options with no direct R equivalent yet are passed
through `...` as named arguments and stored in `fit$legacy_args`.

---

### `TIME`

Names the follow-up time variable. In SAS HAZARD this is a separate
statement; in R it's the first argument to `Surv()` inside the
formula.

```sas
TIME INT_DEAD;
```

Maps directly to `time`:

```{r}
#| eval: false
fit <- hazard(
  time   = avcs$INT_DEAD,
  ...
)
```

- Must be a non-negative numeric vector in the same time unit as the original
  analysis (months, in the AVC example).
- Missing values are not permitted.

---

### `EVENT`

Names the event-indicator variable. Like `TIME`, this is a separate
statement in SAS HAZARD but enters the R formula through `Surv()`.

```sas
EVENT DEAD;
```

Maps to `status`:

```{r}
#| eval: false
fit <- hazard(
  status = avcs$DEAD,   # 1 = event, 0 = censored
  ...
)
```

- `TemporalHazard` coerces `status` to `numeric`; logical vectors are also accepted.
- The HAZARD convention uses `1` = occurred, `0` = censored/alive at last contact.

---

### `PARMS`

Supplies starting values for the optimizer and flags which
parameters to hold fixed (`FIXM`, `FIXMU`, etc.). The starting
values matter — for the multiphase optimizer in particular, a poor
starting point can park the fit at a local minimum well away from
the global MLE.

```sas
PARMS MUE=0.3504743 THALF=0.1905077 NU=1.437416 M=1 FIXM
      MUC=4.391673E-07;
```

Maps to `theta` (coefficient/parameter vector) and `control` (fix flags):

```{r}
#| eval: false
fit <- hazard(
  theta   = c(MUE = 0.3504743, THALF = 0.1905077, NU = 1.437416,
              M = 1,           MUC   = 4.391673e-07),
  control = list(
    fix = c("M")   # FIXM → freeze M during optimization
  ),
  ...
)
```

> **Note:** Full SAS `PARMS` syntax is not mirrored one-for-one in the public API.
> In the current package, supply the parameter vector directly via `theta` and
> record fixed-parameter intent in `control$fix`. The legacy parity helpers do
> generate SAS-style control text when comparing against the historical binaries.

---

### `EARLY` and `CONSTANT` covariate blocks

These statements assign covariates to specific phases. In SAS HAZARD
each block lists the covariates that affect that phase and their
starting coefficients. Covariates can appear in multiple blocks with
different starting values — same column, different phase-specific
effect.

```sas
EARLY  AGE=-0.03205774, COM_IV=1.336675, MAL=0.6872028,
       OPMOS=-0.01963377, OP_AGE=0.0002086689, STATUS=0.5169533;
CONSTANT INC_SURG=1.375285, ORIFICE=3.11765, STATUS=1.054988;
```

In HAZARD, `EARLY` and `CONSTANT` define phase-specific covariate coefficients.
In `TemporalHazard` these are unified into a single design matrix `x` and
coefficient vector `theta` during M1. Phase assignment will be formalised in
M2 when the multi-phase likelihood is implemented.

**Current convention (M1):** combine all covariates into `x` and supply the
corresponding starting coefficients in `theta`.

```{r}
#| eval: false
# Build design matrix from the AVC data set
X <- data.matrix(avcs[, c("AGE", "COM_IV", "MAL", "OPMOS", "OP_AGE",
                           "STATUS", "INC_SURG", "ORIFICE")])

# Starting values from SAS EARLY + CONSTANT blocks combined
theta_start <- c(
  AGE      = -0.03205774,
  COM_IV   =  1.336675,
  MAL      =  0.6872028,
  OPMOS    = -0.01963377,
  OP_AGE   =  0.0002086689,
  STATUS   =  0.5169533,   # EARLY phase coefficient
  INC_SURG =  1.375285,
  ORIFICE  =  3.11765
)

fit <- hazard(
  time   = avcs$INT_DEAD,
  status = avcs$DEAD,
  x      = X,
  theta  = theta_start,
  dist   = "weibull"
)
```

---

### `SELECTION`

Triggers stepwise covariate selection inside the hazard fit. `SLE`
is the significance level for entry, `SLS` for staying. SAS HAZARD
embedded this in the same `PROC HAZARD` call; in R the stepwise
loop is a separate `hzr_stepwise()` function that wraps a fitted
`hazard()` object.

```sas
SELECTION SLE=0.2 SLS=0.1;
```

`TemporalHazard` now ships `hzr_stepwise()`, which implements forward,
backward, and two-way stepwise selection with SAS-style SLENTRY /
SLSTAY thresholds, phase-specific entry for multiphase models, and a
MOVE oscillation guard.  FAST-screening (Lawless-Singhal approximate
Wald updates) is not yet implemented; every candidate currently gets a
full refit.  See `?hzr_stepwise` for the full option list.

---

## SAS macro equivalents

The legacy SAS distribution ships a suite of macros for nonparametric
baselines, goodness-of-fit diagnostics, bootstrap inference, and
variable calibration that sit alongside `PROC HAZARD`.  Each has an R
equivalent in `TemporalHazard`:

| SAS macro | R function | Purpose |
|:---|:---|:---|
| `kaplan.sas` | `hzr_kaplan()` | KM survival with logit-transformed exact CL |
| `nelsonl.sas` / `nelsont.sas` | `hzr_nelson()` | Nelson cumulative hazard with lognormal CL |
| `hazplot.sas` | `hzr_gof()` | Parametric vs. KM overlay + CoE goodness-of-fit |
| `deciles.hazard.sas` | `hzr_deciles()` | Decile-of-risk calibration + chi-square GOF |
| `logit.sas` / `logitgr.sas` | `hzr_calibrate()` | Quantile-group calibration (logit / Gompertz / Cox link) |
| `bootstrap.hazard.sas` | `hzr_bootstrap()` | Bootstrap resampling + summary |
| `markov.sas` | `hzr_competing_risks()` | Aalen-Johansen competing-risks incidence |

Minimal illustrations on the AVC dataset follow.  The
`vignette("inference-diagnostics")` walkthrough builds these into a
full analysis workflow.

```{r}
#| label: macro-demos-setup
library(survival)
data(avc)
avc <- na.omit(avc)

# Base parametric fit for downstream diagnostics
fit <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv,
  data = avc, dist = "weibull",
  theta = c(mu = 0.2, nu = 1, rep(0, 4)),
  fit = TRUE, control = list(maxit = 500)
)
```

```{r}
#| label: kaplan-demo
km <- hzr_kaplan(time = avc$int_dead, status = avc$dead)
head(km, 4)
```

```{r}
#| label: nelson-demo
nel <- hzr_nelson(time = avc$int_dead, event = avc$dead)
head(nel, 4)
```

```{r}
#| label: gof-demo
head(hzr_gof(fit), 4)
```

```{r}
#| label: deciles-demo
hzr_deciles(fit, time = 120, groups = 10)
```

```{r}
#| label: calibrate-demo
hzr_calibrate(avc$age, avc$dead, groups = 10, link = "logit")
```

```{r}
#| label: bootstrap-demo
set.seed(1)
hzr_bootstrap(fit, n_boot = 20)  # small for vignette build
```

```{r}
#| label: competing-demo
data(valves)
# Synthesize a competing-risks indicator for illustration.
valves$ev <- with(valves, ifelse(dead == 1, 1L,
                                   ifelse(pve == 1,  2L,
                                   ifelse(reop == 1, 3L, 0L))))
head(hzr_competing_risks(valves$int_dead, valves$ev), 4)
```

---

## Full worked example: AVC death after repair

Statement-by-statement mapping is fine for reference, but the full
gestalt only lands when you see a complete SAS HAZARD analysis and
its R translation side by side. The example below is the final
multivariable model from `examples/hm.death.AVC.sas` in the
reference C repository — death after atrioventricular canal repair,
two-phase model with covariates assigned to specific phases. It's
the canonical "this is what a real SAS HAZARD analysis looks like"
specimen.

### SAS (original)

The original SAS code, exactly as it appears in the reference
distribution. Notice how the statements stack inside a single
`%HAZARD()` macro call.

```sas
%HAZARD(
PROC HAZARD DATA=AVCS P CONSERVE OUTHAZ=OUTEST CONDITION=14 QUASI;
     TIME INT_DEAD;
     EVENT DEAD;
     PARMS MUE=0.3504743 THALF=0.1905077 NU=1.437416 M=1 FIXM
           MUC=4.391673E-07;
     EARLY   AGE=-0.03205774, COM_IV=1.336675,  MAL=0.6872028,
             OPMOS=-0.01963377, OP_AGE=0.0002086689, STATUS=0.5169533;
     CONSTANT INC_SURG=1.375285, ORIFICE=3.11765, STATUS=1.054988;
);
```

### R equivalent (current runnable translation pattern)

The same model in `TemporalHazard`. Every SAS statement above has a
corresponding R argument here: `TIME` and `EVENT` collapse into
`Surv(int_dead, dead)` on the left of the formula; the global
covariate list goes on the right; the `PARMS` starting values
become the `theta` vector; phase-specific assignments use the
`formula` argument inside each `hzr_phase()`; `FIXM` becomes
`fixed = "shapes"` (or a subset of shape names). The line-by-line
correspondence is the point — once you've translated one analysis
this way, the pattern carries to every other.

```{r}
#| label: avc-example
#| eval: false
# Assumed: avcs is a data.frame read from the AVC flat file
# (same variables as the SAS DATA step)

avcs <- avcs |>
  transform(
    LN_AGE   = log(AGE),
    LN_OPMOS = log(OPMOS),
    LN_INC   = ifelse(is.na(INC_SURG), NA, log(INC_SURG + 1)),
    LN_NYHA  = log(STATUS)
  )

# Replace missing INC_SURG with column mean (mirrors PROC STANDARD REPLACE)
avcs$INC_SURG[is.na(avcs$INC_SURG)] <- mean(avcs$INC_SURG, na.rm = TRUE)

X <- data.matrix(avcs[, c("AGE", "COM_IV", "MAL", "OPMOS", "OP_AGE",
                           "STATUS", "INC_SURG", "ORIFICE")])

fit <- hazard(
  time    = avcs$INT_DEAD,
  status  = avcs$DEAD,
  x       = X,
  theta   = c(
    # Hazard shape parameters (from PARMS)
    MUE   = 0.3504743,
    THALF = 0.1905077,
    NU    = 1.437416,
    M     = 1,
    MUC   = 4.391673e-07,
    # Covariate coefficients (from EARLY + CONSTANT blocks)
    AGE      = -0.03205774,
    COM_IV   =  1.336675,
    MAL      =  0.6872028,
    OPMOS    = -0.01963377,
    OP_AGE   =  0.0002086689,
    STATUS   =  0.5169533,
    INC_SURG =  1.375285,
    ORIFICE  =  3.11765
  ),
  dist    = "weibull",
  control = list(
    condition = 14,
    quasi     = TRUE,
    conserve  = TRUE,
    fix       = c("M")   # FIXM
  )
)

fit
```

---

## Data preparation differences

| SAS HAZARD                             | TemporalHazard R                                     |
|----------------------------------------|---------------------------------------------------|
| `PROC STANDARD REPLACE` for missing    | Replace with `mean(..., na.rm = TRUE)` manually  |
| Log transforms in `DATA` step          | `transform()` or `dplyr::mutate()`                |
| Variable labels via `LABEL`            | Column names of `x` matrix carry through          |
| `FILENAME` / `INFILE` / `INPUT`       | `read.table()`, `read.csv()`, or `haven::read_sas()` |
| SAS formats (date, currency, etc.)     | Standard R numeric; apply `as.Date()` if needed  |

---

## Output object

`hazard()` returns a list of class `hazard`:

| Slot                  | Contents                                                     |
|-----------------------|--------------------------------------------------------------|
| `$call`               | Unevaluated `match.call()` for reproducibility              |
| `$spec$dist`          | Baseline distribution label                                 |
| `$spec$control`       | Control options passed at construction                      |
| `$data$time`          | Follow-up time vector                                       |
| `$data$status`        | Event indicator (numeric)                                   |
| `$data$x`             | Design matrix                                               |
| `$fit$theta`          | Coefficient vector (starting values at M1; fitted at M2+)  |
| `$fit$converged`      | `NA` at M1; `TRUE`/`FALSE` from M2 optimizer               |
| `$fit$objective`      | Log-likelihood at convergence (M2+)                        |
| `$legacy_args`        | Named pass-through arguments for parity                     |

---

## Prediction

In SAS HAZARD the `P` (predict / print) option on the `PROC HAZARD`
line writes predicted survival, hazard, and cumulative-hazard tables
to the output dataset. In R the same predictions come from
`predict.hazard()` on the fitted object, with the requested quantity
chosen via the `type=` argument. `predict.hazard()` currently
supports four output types — `"linear_predictor"`, `"hazard"`,
`"survival"`, and `"cumulative_hazard"` — covering every quantity
the SAS `P` option produces:

```{r}
#| eval: false
#| label: predict-example
# Linear predictor (X %*% theta)
eta <- predict(fit, type = "linear_predictor")

# Hazard scale (exp(eta))
hz  <- predict(fit, type = "hazard")
```

The code chunk above shows only `"linear_predictor"` and `"hazard"`;
`"survival"` and `"cumulative_hazard"` follow the same pattern. For
multiphase fits pass `decompose = TRUE` to get per-phase cumulative
hazard contributions instead of just the total.

---

## Known limitations vs. SAS HAZARD

A SAS HAZARD veteran migrating to `TemporalHazard` should be aware of
the following scope limits as of v0.9.8.  Detailed status per feature is
tracked in `inst/dev/SAS-PARITY-GAP-ANALYSIS.md` and
`inst/dev/DEVELOPMENT-PLAN.md`.

### Stepwise variable selection (`SELECTION` statement)

- **Supported:** `hzr_stepwise()` implements forward / backward /
  two-way stepwise with SAS-style SLENTRY / SLSTAY thresholds,
  per-phase entry for multiphase, and a MOVE oscillation guard.
- **Not yet supported:** FAST-screening (Lawless-Singhal approximate
  Wald updates).  Every candidate currently gets a full refit, which
  is slower but always correct.

### Per-phase time-varying windows

- **Supported:** `time_windows` applies one piecewise-constant cut-point
  set across all covariates.
- **Not yet supported:** distinct `EARLY`/`LATE` window sets per phase.
  Workaround: expand the design matrix manually before calling
  `hazard()`.

### Output datasets (`OUTEST=`, `OUTVCOV=`)

- The R equivalents are `coef(fit)` and `vcov(fit)` in memory; there
  is no automatic dataset-export mode.  `saveRDS()` them yourself if
  you need an on-disk artefact.

### Density / quantile prediction types

- `predict.hazard()` covers `hazard`, `cumulative_hazard`, `survival`,
  and `linear_predictor`.  Density and quantile (median survival)
  types are not wired up; derive them from `cumulative_hazard` and
  `survival` predictions if needed.

### Previously listed gaps that have since been closed

For users migrating from older TemporalHazard versions or reading
older SAS parity notes:

- **`weights` on all distributions** — shipped v0.9.6.  Weibull,
  exponential, log-logistic, log-normal, and multiphase all honour
  observation weights end-to-end (LL + analytic gradient + multiphase
  Conservation of Events).
- **`Surv(start, stop, event)` with `start > 0`** — shipped v0.9.7.
  Counting-process rows contribute `H(stop) - H(start)` for Weibull
  and multiphase.  The previous `hazard()` guard against non-zero
  starts is gone.
- **Delta-method prediction confidence limits** — shipped v0.9.8.
  Use `predict(..., se.fit = TRUE, level = 0.95)` to get a data frame
  with `fit`, `se.fit`, `lower`, and `upper` per row.  Closed-form
  Jacobian for Weibull and multiphase, `numDeriv::jacobian` fallback
  for exponential / log-logistic / log-normal.

---

## Rcpp acceleration note

`TemporalHazard` is currently pure R. If profiling against large real datasets
turns up bottlenecks in the likelihood kernel or the optimizer inner loop, those
specific functions will be re-implemented with Rcpp. The public interface
(`hazard()`, `predict.hazard()`, etc.) will not change.


## References {.unnumbered}

Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. *J Am Stat Assoc.* 1986;81(395):615–624.
doi: [10.1080/01621459.1986.10478314](https://doi.org/10.1080/01621459.1986.10478314)

---
title: "Mathematical Foundations of TemporalHazard"
subtitle: "Generalized decomposition, multiphase additive hazard, censoring, and time-varying covariates"
format:
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    number-sections: true
vignette: >
  %\VignetteIndexEntry{Mathematical Foundations of TemporalHazard}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE)
has_ggplot <- requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = has_pkg,
  fig.width = 7,
  fig.height = 4.5
)
```

```{r}
#| label: setup
library(TemporalHazard)
```

This vignette lays out the mathematical foundation for the models in
TemporalHazard. The other vignettes treat the package as a black box
you call `hazard()` on; this one opens the box. If you've used the
package and want to know *what* it's computing, *why* the
parameterization is what it is, and *how* the multiphase
decomposition relates to standard parametric survival families, this
is the place.

The four pieces, in order: the generalized temporal decomposition
family (Blackstone, Naftel, and Turner 1986) that every phase is
built from; the additive multiphase hazard model that composes
phases into a full hazard; the maximum-likelihood objective under
mixed censoring (right, left, interval, counting-process), which is
what the optimizer minimizes; and extensions for phase-specific and
time-varying covariates. The same decomposition framework carries
over to longitudinal mixed-effects settings (Rajeswaran et al.
2018), though this vignette stays in the survival regime.

For users migrating from the legacy SAS/C HAZARD program, each
section notes how the R parameterization relates to the original
parameter names. For the full translation table, see
`vignette("sas-to-r-migration")` or call `hzr_argument_mapping()`.


# Generalized Temporal Decomposition {#sec-decomposition}

## The parametric family

Every temporal phase in TemporalHazard is built from a single parametric
family, `decompos(t; t_half, nu, m)`, introduced by Blackstone, Naftel, and
Turner (1986) that produces three linked quantities from just three parameters:

- $G(t)$ — cumulative distribution function (CDF), bounded $[0, 1]$
- $g(t) = dG/dt$ — probability density function
- $h(t) = g(t) / (1 - G(t))$ — hazard function

The three parameters control the shape of the distribution:

| Parameter | Meaning | Constraint |
|-----------|---------|------------|
| `t_half`  | Half-life: time at which $G(t_{1/2}) = 0.5$ | $> 0$ |
| `nu`      | Time exponent — controls rate dynamics | any finite value |
| `m`       | Shape exponent — controls distributional form | any finite value |

::: {.callout-note}
## SAS/C parameter bridge

The original C code used different parameter names per phase:

- **Early phase (G1):** `DELTA`, `RHO`/`THALF`, `NU`, `M` $\to$ `t_half`, `nu`, `m`
- **Late phase (G3):** `TAU`, `GAMMA`, `ALPHA`, `ETA` $\to$ `t_half`, `nu`, `m`

Both collapse onto the same 3-parameter decomposition family.  The C `DELTA`
parameter controlled a time transformation $B(t) = (\exp(\delta t) - 1)/\delta$
that is absorbed into the shape.
:::

In R, `hzr_decompos()` computes all three quantities:

```{r}
#| label: basic-decompos
t_grid <- seq(0.01, 10, length.out = 200)

# Standard sigmoidal (Case 1: m > 0, nu > 0)
d <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 1)

# Verify the half-life property: G(t_half) = 0.5
d_half <- hzr_decompos(3, t_half = 3, nu = 2, m = 1)
cat("G(t_half) =", round(d_half$G, 6), "\n")

# Verify the identity: h(t) = g(t) / (1 - G(t))
h_check <- d$g / (1 - d$G)
cat("Max |h - g/(1-G)| =", max(abs(d$h - h_check)), "\n")
```


## The six valid cases

Different sign combinations of $\nu$ and $m$ pick out different
asymptotic behaviors of `decompos()`, and the package treats each as
a separate case for numerical stability. The combination $m < 0$
**and** $\nu < 0$ produces an integrand with no finite normalization
and is rejected at the input-validation step. The remaining six are
the catalog every phase in the package selects from:

```{r}
#| label: six-cases-table
#| echo: false
cases <- data.frame(
  Case = c("1", "1L", "2", "2L", "3", "3L"),
  m     = c("> 0", "= 0", "< 0", "< 0", "> 0", "= 0"),
  nu    = c("> 0", "> 0", "> 0", "= 0", "< 0", "< 0"),
  Behavior = c(
    "Standard sigmoidal",
    "Weibull-like (exponential limit as m -> 0)",
    "Heavy-tailed",
    "Exponential decay",
    "Bounded cumulative",
    "Bounded exponential"
  )
)
knitr::kable(cases, caption = "Six valid parameter sign combinations")
```

The following plot shows the CDF $G(t)$, density $g(t)$, and hazard $h(t)$
for each case, all with `t_half = 3`:

```{r}
#| label: fig-six-cases
#| fig-cap: "G(t), g(t), and h(t) for all six valid decomposition cases"
#| fig-height: 8
if (has_ggplot) {
  library(ggplot2)

  params <- list(
    "Case 1: m=1, nu=2"     = list(nu = 2,   m = 1),
    "Case 1L: m=0, nu=2"    = list(nu = 2,   m = 0),
    "Case 2: m=-0.5, nu=2"  = list(nu = 2,   m = -0.5),
    "Case 2L: m=-0.5, nu=0" = list(nu = 0,   m = -0.5),
    "Case 3: m=1, nu=-0.5"  = list(nu = -0.5, m = 1),
    "Case 3L: m=0, nu=-0.5" = list(nu = -0.5, m = 0)
  )

  t_grid <- seq(0.01, 10, length.out = 200)
  rows <- list()

  for (nm in names(params)) {
    p <- params[[nm]]
    d <- hzr_decompos(t_grid, t_half = 3, nu = p$nu, m = p$m)
    rows <- c(rows, list(data.frame(
      time = rep(t_grid, 3),
      value = c(d$G, d$g, d$h),
      quantity = rep(c("G(t) — CDF", "g(t) — density", "h(t) — hazard"),
                     each = length(t_grid)),
      case = nm
    )))
  }

  df <- do.call(rbind, rows)

  # Cap hazard display at reasonable values for readability
  df$value[df$quantity == "h(t) — hazard" & df$value > 5] <- NA

  ggplot(df, aes(x = time, y = value)) +
    geom_line(linewidth = 0.6) +
    facet_grid(quantity ~ case, scales = "free_y") +
    labs(x = "Time", y = NULL) +
    theme_minimal(base_size = 10) +
    theme(strip.text = element_text(size = 7))
}
```


## Derivation sketch

The construction begins with a rate function $\rho$ chosen so that
$G(t_{1/2}) = 0.5$ exactly.  For Case 1 ($m > 0, \nu > 0$):

$$
\rho = \nu \, t_{1/2} \left(\frac{2^m - 1}{m}\right)^{\!\nu}
$$

Then, with the dimensionless time $b(t) = \nu t / \rho$:

$$
G(t) = \left(1 + m \, b(t)^{-1/\nu}\right)^{-1/m}
$$

$$
g(t) = \left(1 + m \, b(t)^{-1/\nu}\right)^{-1/m - 1}
       \cdot b(t)^{-1/\nu - 1} / \rho
$$

The other five cases arise as limits ($m \to 0$, $\nu \to 0$) or sign
reflections of this base form.  The implementation in `hzr_decompos()`
dispatches to the appropriate branch after checking the signs of `nu` and `m`.


# Additive Multiphase Hazard Model {#sec-multiphase}

## Model specification

The total cumulative hazard decomposes additively across $J$ phases:

$$
H(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \cdot \Phi_j(t;\, t_{1/2,j},\, \nu_j,\, m_j)
$$

where:

- $\mu_j(\mathbf{x}) = \exp(\alpha_j + \mathbf{x}_j \boldsymbol{\beta}_j)$ is the phase-specific log-linear scale with covariates
- $\Phi_j(t)$ is the temporal shape, determined by the phase type

The three phase types correspond to different uses of the decomposition output:

| Phase type | $\Phi_j(t)$ | Domain | Interpretation |
|------------|-------------|--------|----------------|
| `"cdf"`    | $G(t)$ | $[0, 1]$ | Early risk that resolves over time |
| `"hazard"` | $-\log(1 - G(t))$ | $[0, \infty)$ | Late or aging risk that accumulates |
| `"constant"` | $t$ | $[0, \infty)$ | Flat background rate (no shape parameters) |

::: {.callout-note}
## SAS/C bridge

The classic three-phase HAZARD model used:

- **G1** (early) $\to$ `hzr_phase("cdf", ...)`
- **G2** (constant) $\to$ `hzr_phase("constant")`
- **G3** (late) $\to$ `hzr_phase("hazard", ...)`

TemporalHazard generalizes this to $N$ phases of any type.
:::


## Derived quantities

From the cumulative hazard, the instantaneous hazard and survival are:

$$
h(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \cdot \varphi_j(t)
\qquad\text{where } \varphi_j = d\Phi_j/dt
$$

$$
S(t \mid \mathbf{x}) = \exp\!\bigl(-H(t \mid \mathbf{x})\bigr)
$$

The derivative $\varphi_j(t)$ depends on phase type:

- `"cdf"`: $\varphi(t) = g(t)$ (the density)
- `"hazard"`: $\varphi(t) = h(t) = g(t)/(1 - G(t))$
- `"constant"`: $\varphi(t) = 1$


## Constructing phases in R

The math above is exposed in R through `hzr_phase()`, which builds
one phase at a time, and `hazard()`, which composes phases into a
full model. The same `(t_half, nu, m)` triple from `decompos()`
parameterizes the `"cdf"` and `"hazard"` shapes; the `"g3"` shape
exposes its own `(tau, gamma, alpha, eta)` parameters from the SAS
"late" library; and `"constant"` has no shape parameters at all. A
canonical three-phase model uses the `"cdf"` shape for the early
phase, `"constant"` for the background, and either a delayed
`"cdf"` or `"g3"` for the late phase.

```{r}
#| label: phase-construction
# Classic three-phase pattern
early <- hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0)
const <- hzr_phase("constant")
late  <- hzr_phase("hazard",   t_half = 5,   nu = 1, m = 0)

print(early)
print(const)
print(late)
```

The cumulative hazard contribution $\Phi(t)$ and instantaneous hazard
$\varphi(t)$ for each phase can be computed directly:

```{r}
#| label: fig-phase-shapes
#| fig-cap: "Phi(t) and phi(t) for each phase type"
#| fig-height: 5
if (has_ggplot) {
  t_grid <- seq(0.01, 12, length.out = 200)

  phi_early <- hzr_phase_cumhaz(t_grid, t_half = 0.5, nu = 2, m = 0,
                                 type = "cdf")
  phi_late  <- hzr_phase_cumhaz(t_grid, t_half = 5, nu = 1, m = 0,
                                 type = "hazard")
  phi_const <- hzr_phase_cumhaz(t_grid, type = "constant")

  dphi_early <- hzr_phase_hazard(t_grid, t_half = 0.5, nu = 2, m = 0,
                                  type = "cdf")
  dphi_late  <- hzr_phase_hazard(t_grid, t_half = 5, nu = 1, m = 0,
                                  type = "hazard")
  dphi_const <- hzr_phase_hazard(t_grid, type = "constant")

  df <- data.frame(
    time = rep(t_grid, 6),
    value = c(phi_early, phi_late, phi_const,
              dphi_early, dphi_late, dphi_const),
    phase = rep(rep(c("cdf (early)", "hazard (late)", "constant"),
                    each = length(t_grid)), 2),
    quantity = rep(c("Phi(t) — cumulative", "phi(t) — instantaneous"),
                   each = 3 * length(t_grid))
  )

  ggplot(df, aes(x = time, y = value, colour = phase)) +
    geom_line(linewidth = 0.8) +
    facet_wrap(~ quantity, scales = "free_y", ncol = 1) +
    scale_colour_manual(values = c(
      "cdf (early)" = "#0072B2",
      "hazard (late)" = "#D55E00",
      "constant" = "#009E73"
    )) +
    labs(x = "Time", y = NULL, colour = "Phase type") +
    theme_minimal(base_size = 11) +
    theme(legend.position = "bottom")
}
```


## Additive composition example

To build intuition, here is how three phases combine into a total hazard.
The key insight is that phases contribute additively to the **cumulative
hazard** $H(t)$, not to the survival directly.

```{r}
#| label: fig-additive-demo
#| fig-cap: "Three-phase additive cumulative hazard: total = early + constant + late"
#| fig-height: 4
if (has_ggplot) {
  t_grid <- seq(0.01, 15, length.out = 300)

  mu_early <- 0.3
  mu_const <- 0.05
  mu_late  <- 0.2

  H_early <- mu_early * hzr_phase_cumhaz(t_grid, t_half = 0.5, nu = 2,
                                           m = 0, type = "cdf")
  H_const <- mu_const * hzr_phase_cumhaz(t_grid, type = "constant")
  H_late  <- mu_late  * hzr_phase_cumhaz(t_grid, t_half = 5, nu = 1,
                                           m = 0, type = "hazard")
  H_total <- H_early + H_const + H_late

  df <- data.frame(
    time = rep(t_grid, 4),
    cumhaz = c(H_early, H_const, H_late, H_total),
    component = rep(c("Early (cdf)", "Constant", "Late (hazard)", "Total"),
                    each = length(t_grid))
  )
  df$component <- factor(df$component,
    levels = c("Total", "Early (cdf)", "Constant", "Late (hazard)"))

  ggplot(df, aes(x = time, y = cumhaz, colour = component,
                 linewidth = component)) +
    geom_line() +
    scale_colour_manual(values = c(
      "Total" = "black", "Early (cdf)" = "#0072B2",
      "Constant" = "#009E73", "Late (hazard)" = "#D55E00"
    )) +
    scale_linewidth_manual(values = c(
      "Total" = 1.2, "Early (cdf)" = 0.6,
      "Constant" = 0.6, "Late (hazard)" = 0.6
    )) +
    labs(x = "Time", y = "Cumulative hazard H(t)",
         colour = NULL, linewidth = NULL) +
    theme_minimal(base_size = 11) +
    theme(legend.position = "bottom")
}
```


# Maximum Likelihood Estimation {#sec-likelihood}

## Log-likelihood under mixed censoring

TemporalHazard supports four censoring types in a single dataset, coded
by the `status` indicator:

| Code | Type | Contribution to log-likelihood |
|------|------|-------------------------------|
| $\delta_i = 1$ | Exact event | $\log h(t_i \mid \mathbf{x}_i) - H(t_i \mid \mathbf{x}_i)$ |
| $\delta_i = 0$ | Right-censored | $-H(t_i \mid \mathbf{x}_i)$ |
| $\delta_i = -1$ | Left-censored | $\log\bigl(1 - \exp(-H(u_i \mid \mathbf{x}_i))\bigr)$ |
| $\delta_i = 2$ | Interval-censored | $-H(l_i \mid \mathbf{x}_i) + \log\bigl(1 - \exp(-(H(u_i) - H(l_i)))\bigr)$ |

where $l_i$ and $u_i$ are the lower and upper bounds of the censoring
interval.

The full log-likelihood is:

$$
\ell(\boldsymbol{\theta}) = \sum_{i:\,\delta_i=1}
  \Bigl[\log h(t_i \mid \mathbf{x}_i) - H(t_i \mid \mathbf{x}_i)\Bigr]
  - \sum_{i:\,\delta_i=0} H(t_i \mid \mathbf{x}_i)
$$
$$
  + \sum_{i:\,\delta_i=-1}
    \log\!\bigl(1 - e^{-H(u_i \mid \mathbf{x}_i)}\bigr)
  + \sum_{i:\,\delta_i=2}
    \Bigl[-H(l_i \mid \mathbf{x}_i)
    + \log\!\bigl(1 - e^{-(H(u_i) - H(l_i))}\bigr)\Bigr]
$$

The $\log(1 - e^{-x})$ terms are computed using the numerically stable
primitive `hzr_log1mexp()`, which avoids catastrophic cancellation when $x$
is near zero.

```{r}
#| label: log1mexp-demo
# Naive log(1 - exp(-x)) fails near zero
x_small <- 1e-15
cat("Naive:  ", log(1 - exp(-x_small)), "\n")
cat("Stable: ", hzr_log1mexp(x_small), "\n")
```


## Internal parameterization

During optimization, shape parameters are reparameterized for unconstrained
search.  For each phase $j$, the internal parameter sub-vector is:

| Parameter | Internal (estimation) | User-facing (reported) |
|-----------|----------------------|----------------------|
| Scale     | $\log \mu_j$ | $\mu_j = \exp(\log \mu_j)$ |
| Half-life | $\log t_{1/2,j}$ | $t_{1/2,j} = \exp(\log t_{1/2,j})$ |
| Time exponent | $\nu_j$ (unconstrained) | $\nu_j$ |
| Shape     | $m_j$ (unconstrained) | $m_j$ |
| Covariates | $\beta_{j,1}, \ldots, \beta_{j,p_j}$ | same |

Constant phases omit `log_t_half`, `nu`, and `m`, contributing only one
shape-free parameter (`log_mu`).  The full $\boldsymbol{\theta}$ is the
concatenation of all phase sub-vectors.


## Optimization strategy

The optimizer uses `stats::optim()` with BFGS on the unconstrained internal
scale, wrapped by `.hzr_optim_generic()`.  Key features:

- **Multi-start**: the optimizer runs from $k$ random perturbations around
  the user-supplied starting values (default $k = 5$, controlled by
  `control$n_starts`).  The run with the highest log-likelihood is kept.
- **Feasibility guard**: any parameter vector where $m < 0$ **and**
  $\nu < 0$ for the same phase returns $-\infty$ immediately.
- **Post-fit Hessian**: the numerical Hessian of the negative
  log-likelihood at the solution is inverted to produce the
  variance-covariance matrix $\hat{V}$, with standard errors
  $\sqrt{\text{diag}(\hat{V})}$.  The inversion is hardened against the
  ill-conditioning that arises at high parameter counts: the Hessian is
  symmetrized, its reciprocal condition number is checked, and it is
  inverted via a Cholesky factorization, with a general `solve()` fallback
  when the Hessian is not positive-definite.  Non-positive variance
  estimates are flagged rather than silently returned as `NaN` standard
  errors.  Each fit carries `rcond` (reciprocal condition number) and `pd`
  (positive-definiteness) diagnostics; `summary()` prints a note, and a
  warning is raised, when a fit is ill-conditioned or not positive-definite.


# Covariates and Phase-Specific Formulas {#sec-covariates}

## Global covariates

By default, every phase shares the same covariate vector from the model
formula.  Each phase gets its own coefficient vector $\boldsymbol{\beta}_j$:

$$
\mu_j(\mathbf{x}) = \exp\!\bigl(\alpha_j + \mathbf{x}\,\boldsymbol{\beta}_j\bigr)
$$

So the same predictors can have different effects on early vs. late risk,
which is a core feature of multiphase models.

```{r}
#| label: global-covariates
#| eval: false
# Both phases use age and nyha from the global formula
hazard(
  survival::Surv(time, status) ~ age + nyha,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf",    t_half = 0.5, nu = 2, m = 0),
    late  = hzr_phase("hazard", t_half = 5,   nu = 1, m = 0)
  ),
  fit = TRUE
)
```


## Phase-specific covariates

The global formula above lets every covariate act on every phase
through a shared shift $x^\top \beta$. That's the right structure
when a covariate plausibly affects the entire hazard trajectory. But
clinical reality often pulls the other way: surgical risk factors
(grade IV complications, intra-operative blood loss) belong in the
early phase but say nothing about late attrition; chronic
comorbidities (NYHA class progression, late-onset diabetes) belong
in the late phase but were irrelevant during the operative window.
Phase-specific covariates let you build that structure directly.

When a phase carries its own one-sided formula, the phase gets its
own design matrix from the data, and only those covariates appear
in that phase's $\boldsymbol{\beta}_j$. Other phases ignore them
entirely.

```{r}
#| label: phase-specific-covariates
#| eval: false
# Early risk depends on surgical factors; late risk on chronic conditions
hazard(
  survival::Surv(time, status) ~ age + nyha + shock,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf",    t_half = 0.5, nu = 2, m = 0,
                      formula = ~ age + shock),
    late  = hzr_phase("hazard", t_half = 5,   nu = 1, m = 0,
                      formula = ~ age + nyha)
  ),
  fit = TRUE
)
```

When a phase has a `formula`, it gets its own design matrix built from the
data, and only those covariates appear in its $\boldsymbol{\beta}_j$.  The
parameter vector is shorter for phases with fewer covariates, which can
improve identifiability and convergence.


## Time-varying coefficients

For single-distribution models, TemporalHazard supports piecewise
time-varying coefficients via the `time_windows` argument.  This expands the
design matrix into window-specific blocks:

$$
\eta_i = \sum_{k=1}^{K} \mathbf{x}_i \boldsymbol{\beta}_k \cdot
\mathbf{1}(t_i \in W_k)
$$

where $W_1, \ldots, W_K$ are the time windows defined by the cut points.
Each predictor gets a separate coefficient in each window, allowing effects
to change over follow-up time.

```{r}
#| label: time-varying-demo
#| eval: false
# Two windows: [0, 2] and (2, infinity)
hazard(
  survival::Surv(time, status) ~ age + nyha,
  data         = dat,
  theta        = c(mu = 0.25, nu = 1.1,
                   age_w1 = 0, nyha_w1 = 0,
                   age_w2 = 0, nyha_w2 = 0),
  dist         = "weibull",
  time_windows = 2,
  fit          = TRUE
)
```

::: {.callout-tip}
## Multiphase vs. time-varying coefficients

For multiphase models, the phase structure itself captures time-varying
effects: early phases dominate at small $t$ and late phases at large $t$.
The `time_windows` mechanism is a complementary approach for
single-distribution models.
:::


# Identifiability and Practical Considerations {#sec-identifiability}

## Parameter identifiability

Multiphase models with many free parameters can have flat or multi-modal
likelihood surfaces.  Practical guidelines:

1. **Fix shape parameters when possible.**  If clinical knowledge suggests a
   specific temporal pattern (e.g. early mortality follows a Weibull shape
   with $m = 0$), fix `m` in the `hzr_phase()` starting values and inspect
   whether the optimizer moves it.

2. **Start from the SAS/C estimates.**  If legacy results are available,
   translate them using `hzr_argument_mapping()` and supply as starting values.

3. **Use multi-start optimization.**  The default `control$n_starts = 5`
   helps escape local optima, but complex models may benefit from more starts.

4. **Phase-specific covariates improve identifiability.**  Restricting each
   phase to clinically relevant predictors reduces the parameter count and
   sharpens the likelihood surface.


## Numerical stability

The decomposition engine applies several guards:

- Time is clamped above `.Machine$double.xmin` to avoid $0^{\text{negative}}$
- $1 - G(t)$ is clamped above `.Machine$double.xmin` before computing the
  hazard $h(t) = g(t)/(1 - G(t))$
- Left- and interval-censored contributions use `hzr_log1mexp()` to avoid
  $\log(0)$ when $H(t)$ is very small

Together these keep the gradients finite throughout the optimization, even in
regions of parameter space far from the optimum.


# Summary of Key Functions {#sec-api}

| Function | Purpose |
|----------|---------|
| `hzr_decompos(t, t_half, nu, m)` | Core decomposition: returns $G$, $g$, $h$ |
| `hzr_phase_cumhaz(t, ..., type)` | Phase cumulative hazard $\Phi(t)$ |
| `hzr_phase_hazard(t, ..., type)` | Phase instantaneous hazard $\varphi(t)$ |
| `hzr_phase(type, t_half, nu, m, formula)` | Construct a phase specification |
| `hazard(..., dist = "multiphase", phases = ...)` | Fit a multiphase model |
| `predict(fit, type, decompose = TRUE)` | Per-phase decomposed predictions |
| `hzr_argument_mapping()` | SAS/C $\to$ R parameter translation table |
| `hzr_log1mexp(x)` | Stable $\log(1 - e^{-x})$ |

For a worked clinical example, see `vignette("getting-started")`.  For
migration from SAS HAZARD, see `vignette("sas-to-r-migration")`.


# References {.unnumbered}

Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. *J Am Stat Assoc.* 1986;81(395):615–624.
doi: [10.1080/01621459.1986.10478314](https://doi.org/10.1080/01621459.1986.10478314)

Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
Probability of atrial fibrillation after ablation: Using a parametric
nonlinear temporal decomposition mixed effects model. *Stat Methods Med Res.*
2018;27(1):126–141.
doi: [10.1177/0962280215623583](https://doi.org/10.1177/0962280215623583)

---
title: "Package Architecture"
subtitle: "Design, golden fixtures, and dataset catalog for TemporalHazard"
author: "John Ehrlinger"
date: last-modified
format:
  html:
    toc: true
    toc-depth: 3
    number-sections: true
    code-fold: true
    code-summary: "Show code"
vignette: >
  %\VignetteIndexEntry{Package Architecture}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE)
if (has_pkg) library(TemporalHazard)
knitr::opts_chunk$set(echo = TRUE, collapse = TRUE, comment = "#>",
                      eval = has_pkg)
```

# Overview

TemporalHazard is a pure-R reimplementation of the C/SAS HAZARD procedure
originally developed at the University of Alabama at Birmingham (UAB) for
multi-phase parametric hazard modeling (Blackstone, Naftel, and Turner 1986).
The SAS/C code and this R package are currently developed and maintained at
The Cleveland Clinic Foundation, and the R code was wholly developed at The
Cleveland Clinic Foundation. The package provides a
unified framework for fitting additive hazard models with an arbitrary
number of temporal phases, each governed by the three-parameter
`decompos(t; t_half, nu, m)` family. The generalized temporal
decomposition extends naturally to longitudinal mixed-effects settings
(Rajeswaran et al. 2018).

This vignette documents the internal architecture: how source files are
organized, how functions compose into the fitting pipeline, how golden
fixtures ensure regression-free development, and what reference datasets
ship with the package.


# Source file organization

The R source lives in `R/` and follows a layered design. Lower layers
know nothing about higher layers; control flows downward from the API
through distribution-specific optimizers to shared numerical primitives.

```{r file-layout, echo=FALSE}
layout <- data.frame(
  Layer = c(
    rep("User API", 2),
    rep("Multiphase engine", 3),
    rep("Single-phase likelihoods", 4),
    rep("Shared infrastructure", 5)
  ),
  File = c(
    "hazard_api.R", "argument_mapping.R",
    "likelihood-multiphase.R", "phase-spec.R", "decomposition.R",
    "likelihood-weibull.R", "likelihood-exponential.R",
    "likelihood-loglogistic.R", "likelihood-lognormal.R",
    "optimizer.R", "math_primitives.R", "formula-helpers.R",
    "golden_fixtures.R", "parity-helpers.R"
  ),
  Purpose = c(
    "hazard(), predict.hazard(), print/summary/coef/vcov S3 methods",
    "SAS HAZARD -> R parameter translation table",
    "Additive N-phase likelihood, cumhaz/hazard evaluators, multi-start optimizer",
    "hzr_phase() constructor, validators, starting-value assembly",
    "Unified decompos(t; t_half, nu, m) parametric family",
    "Weibull PH likelihood, gradient, optimizer",
    "Exponential (constant hazard) likelihood",
    "Log-logistic (proportional odds) likelihood",
    "Log-normal (AFT) likelihood",
    "Generic L-BFGS-B/BFGS optimizer with Hessian-based vcov",
    "Numerically stable log1pexp, log1mexp, clamp_prob",
    "Surv() formula parsing for right/left/interval censoring",
    "Synthetic fixture generators (.rds reference outputs)",
    "Stubs for cross-validating against C HAZARD binary"
  )
)
knitr::kable(layout, caption = "R source files by architectural layer")
```


# Function call graph

The primary user entry point is `hazard()`, which dispatches to
distribution-specific optimizers. The diagram below shows the call flow
for a multiphase fit.

```
Call graph for a multiphase hazard fit
--------------------------------------

hazard(dist='multiphase')
+-- .hzr_optim_multiphase()
|   +-- Resolve per-phase design matrices
|   +-- .hzr_phase_start() per phase
|   +-- .hzr_optim_generic()
|       +-- stats::optim(method='BFGS')
|           +-- .hzr_logl_multiphase()
|               +-- .hzr_split_theta()
|               +-- .hzr_multiphase_cumhaz()
|               |   +-- hzr_phase_cumhaz() / hzr_decompos_g3()
|               +-- .hzr_multiphase_hazard()
|                   +-- hzr_phase_hazard() / hzr_decompos_g3()
+-- predict.hazard()
    +-- .hzr_multiphase_cumhaz()
    +-- .hzr_multiphase_hazard()
```

For single-phase distributions (Weibull, exponential, log-logistic,
log-normal), `hazard()` dispatches to the corresponding
`.hzr_optim_<dist>()` function, which calls `.hzr_optim_generic()` with
the distribution-specific log-likelihood and gradient.


## Key internal functions

### Theta vector layout

The multiphase model concatenates per-phase sub-vectors into a single
flat `theta` on the **internal (estimation) scale**:

```{r theta-layout, echo=FALSE}
theta_layout <- data.frame(
  `Phase type` = c("cdf / hazard", "g3", "constant"),
  `Sub-vector` = c(
    "[log_mu, log_t_half, nu, m, beta_1, ..., beta_p]",
    "[log_mu, log_tau, gamma, alpha, eta, beta_1, ..., beta_p]",
    "[log_mu, beta_1, ..., beta_p]"
  ),
  `Count` = c("4 + p", "5 + p", "1 + p")
)
knitr::kable(theta_layout, caption = "Internal theta layout per phase")
```

`.hzr_split_theta()` partitions the concatenated vector into a named
list of per-phase sub-vectors. `.hzr_unpack_phase_theta()` extracts
named parameters (`log_mu`, `log_t_half`, `nu`, `m`, `beta`) from each
sub-vector.

### Phase specification

`hzr_phase(type, t_half, nu, m, formula)` creates a lightweight S3
object that stores the phase type, initial shape parameters, and an
optional phase-specific covariate formula. The constructor validates
inputs and returns `NA` for shape parameters of constant phases.

A list of `hzr_phase` objects is validated by `.hzr_validate_phases()`,
which auto-names unnamed phases ("phase_1", "phase_2", ...) and catches
the common mistake of passing a bare `hzr_phase` instead of a list.

### Decomposition engine

`hzr_decompos(time, t_half, nu, m)` is the mathematical core. It
computes `G(t)` (CDF), `g(t)` (density), and `h(t)` (hazard) for the
three-parameter family across six valid sign combinations of `nu` and
`m`. The phase-level helpers `hzr_phase_cumhaz()` and
`hzr_phase_hazard()` wrap `hzr_decompos()` and apply the phase type
mapping:

| Phase type | $\Phi(t)$ | $\phi(t)$ |
|:-----------|:----------|:----------|
| `"cdf"`    | $G_1(t)$    | $g_1(t)$    |
| `"hazard"` | $-\log(1-G_1(t))$ | $h_1(t)$ |
| `"g3"`     | $G_3(t; \tau, \gamma, \alpha, \eta)$ | $g_3(t)$ |
| `"constant"` | $t$     | $1$       |

### Multi-start optimization

`.hzr_optim_multiphase()` runs a multi-start strategy: the first start
uses the assembled starting values (from `theta` or `hzr_phase` specs);
subsequent starts perturb randomly (sd = 0.5). The best log-likelihood
across all starts is kept. This helps the optimizer escape the shallow local
optima that are common in multiphase models.

The optimizer delegates to `.hzr_optim_generic()`, which wraps
`stats::optim()` with method `"BFGS"` (unconstrained; all scale
parameters are log-transformed). The Hessian is computed numerically
at the converged point for variance-covariance estimation.


# Golden fixture system

Golden fixtures are pre-fitted model results saved as `.rds` files in
`inst/fixtures/`. They serve as **regression anchors**: each test run
refits the model on the same data and compares estimates to the stored
values. This catches regressions when the likelihood, gradient, or
optimizer changes.

## Fixture format

Every fixture is a named list with a consistent structure:

```r
list(
  description = "Human-readable description",
  data = list(time, status, [x], n, events),
  seed = 42,
  true_params = list(...),   # simulation truth (synthetic fixtures)
  fit = list(
    theta     = <named numeric>,  # converged parameter vector
    logl      = <scalar>,         # log-likelihood at convergence
    converged = <logical>,        # convergence flag
    vcov      = <matrix>          # asymptotic variance-covariance
  ),
  timestamp = <POSIXct>
)
```

For C-reference fixtures (e.g., `mp_c_reference_kul.rds`), the `fit`
element is replaced by `c_reference` containing the C binary output:
parameter estimates, standard errors, variance-covariance matrix, and
the C log-likelihood.

## Current fixtures

```{r fixtures, echo=FALSE}
fixtures <- data.frame(
  File = c(
    "hz_univariate.rds",
    "hm_multivariate.rds",
    "hm_edge_case.rds",
    "hz_loglogistic.rds",
    "hz_lognormal.rds",
    "mp_synthetic_3phase.rds",
    "mp_c_reference_kul.rds"
  ),
  Distribution = c(
    "Weibull", "Weibull", "Weibull",
    "Log-logistic", "Log-normal",
    "Multiphase (3)", "Multiphase (3)"
  ),
  Description = c(
    "Univariable shape estimation (n=100)",
    "2 covariates (n=100)",
    "Edge case: n=20, 3 covariates",
    "Univariable (n=80)",
    "Univariable (n=80)",
    "Synthetic early CDF + constant + late hazard (n=500)",
    "KUL CABG dataset + C HAZARD binary reference output (n=5880)"
  ),
  Source = c(
    rep("Synthetic (seed=42)", 5),
    "Synthetic (seed=42)",
    "Clinical + C binary"
  )
)
knitr::kable(fixtures, caption = "Golden fixture inventory")
```

## Regenerating fixtures

Fixtures must be regenerated whenever the model parameterization changes.
The generators are maintainer-only helpers kept in `data-raw/` (outside the
installed package). Run them interactively from the package source tree
after `devtools::load_all()` and sourcing the generator script:

```r
devtools::load_all()
source("data-raw/golden_fixtures.R")

# Each generator requires an explicit output directory (no default path
# is written). Use the project source dir to persist, or tempdir() to test.
dir <- "inst/fixtures"

# Single-phase distributions
.hzr_create_synthetic_golden_fixtures(dir, seed = 42L)   # Weibull variants
.hzr_create_loglogistic_golden_fixture(dir, seed = 42L)  # Log-logistic
.hzr_create_lognormal_golden_fixture(dir, seed = 42L)    # Log-normal

# Multiphase
.hzr_create_multiphase_golden_fixture(dir, seed = 42L)   # Synthetic 3-phase
.hzr_create_c_reference_kul_fixture(dir)                  # C binary reference
```

Pass an explicit `seed` for reproducibility; the global RNG state is saved
and restored, so the caller's stream is unaffected. With the default
(`seed = NULL`) no seed is set. Commit the updated `.rds` files alongside
any code changes.


# Testing strategy

The test suite (`tests/testthat/`) is organized into four tiers:

```{r test-tiers, echo=FALSE}
tiers <- data.frame(
  Tier = c(
    "Unit tests",
    "Distribution tests",
    "Integration tests",
    "Parity tests"
  ),
  Files = c(
    "test-math-primitives, test-decomposition, test-phase-spec, test-argument-mapping",
    paste("test-gradient-weibull, test-exponential-dist,",
          "test-loglogistic-dist, test-lognormal-dist, test-multiphase-gradient"),
    paste("test-hazard-api, test-predict-types, test-interval-censoring-*,",
          "test-time-varying-*, test-multiphase-likelihood, test-multiphase-api"),
    "test-parity-core, test-parity-edge-cases, test-parity-c-binary, test-multiphase-parity"
  ),
  Purpose = c(
    "Verify individual functions in isolation",
    "Likelihood, gradient, and optimizer for each distribution",
    "End-to-end: hazard() -> predict() -> summary() pipeline, censoring types, multiphase wiring",
    "Golden fixture round-trip, C binary cross-validation"
  )
)
knitr::kable(tiers, caption = "Test suite tiers")
```

### Multiphase parity tests

The multiphase parity tests (`test-multiphase-parity.R`) validate
against the C HAZARD binary output for the KUL CABG dataset:

1. **Likelihood evaluation** ---Evaluates the R log-likelihood at the C
   converged parameters and asserts it matches the C output (-3740.52).

2. **Decomposition consistency** ---Verifies phase additivity and CDF
   saturation at the C reference parameter values.

3. **Conservation of events** ---Checks that the model-implied expected
   events ($\sum [1 - \exp(-H(t_i))]$) matches the observed event count
   (545), as reported by the C binary (544.9993).

4. **Profile standard errors** ---Computes a numerical Hessian varying
   only the 3 log(mu) parameters (shapes held fixed), matching the C
   binary's estimation strategy, and compares standard errors.

5. **Full fit convergence** ---Fits the R multiphase optimizer on the
   full dataset with informed starting values and checks that the
   log-likelihood meets or exceeds the C reference.


# Dataset catalog {#sec-datasets}

TemporalHazard ships five clinical reference datasets in
`inst/extdata/`, converted from the original C/SAS HAZARD test data.
These datasets are used in vignette examples and parity testing.

```{r datasets, echo=FALSE}
datasets <- data.frame(
  File = c("avc.csv", "cabgkul.csv", "omc.csv", "tga.csv", "valves.csv"),
  Study = c(
    "Atrioventricular canal repair",
    "Coronary artery bypass grafting (KU Leuven)",
    "Open mitral commissurotomy",
    "Transposition of great arteries (arterial switch)",
    "Primary valve replacement"
  ),
  n = c(310, 5880, 339, 470, 1533),
  Events = c("70 deaths", "545 deaths", "thromboembolic events (repeated)", "deaths", "deaths, PVE, reoperation"),
  Covariates = c(
    "NYHA, age, anatomy, era",
    "None (intercept only)",
    "TE events (repeated measures)",
    "anatomy, coronary pattern, era",
    "age, NYHA, valve position, pathology, race"
  ),
  `SAS origin` = c(
    "hz.death.AVC, hm.death.AVC",
    "hz.deadp.KUL",
    "hz.te123.OMC",
    "hs.dthar.TGA",
    "hm.deadp.VALVES"
  )
)
knitr::kable(datasets, caption = "Reference datasets (lazy-loaded via `data()`)")
```


## Loading datasets

```{r load-demo}
# Datasets are lazy-loaded with the package — just reference them directly.
# Raw CSVs are also available in inst/extdata/ for advanced use:
#   read.csv(system.file("extdata", "cabgkul.csv", package = "TemporalHazard"))

data(cabgkul)
str(cabgkul)

data(avc)
str(avc)
```


## Dataset details

### AVC (atrioventricular canal repair)

The AVC dataset contains 310 patients who underwent repair of
atrioventricular septal defects. It is the primary example dataset used
throughout the hazard modeling examples and has the richest set of
covariates for multivariable analysis.

```{r avc-detail, echo=FALSE}
avc_vars <- data.frame(
  Variable = c("study", "status", "inc_surg", "opmos", "age", "mal",
               "com_iv", "orifice", "dead", "int_dead", "op_age"),
  Label = c("Study number", "NYHA functional class (I-V)", "Surgical grade of AV valve incompetence",
            "Date of operation (months since Jan 1967)", "Age (months) at repair",
            "Important associated cardiac anomaly", "Interventricular communication",
            "Accessory left AV valve orifice", "Death (event indicator)",
            "Follow-up interval (months)", "Interaction: opmos x age"),
  Type = c("character", rep("numeric", 10))
)
knitr::kable(avc_vars, caption = "AVC dataset variables")
```

### CABG/KUL (coronary artery bypass grafting)

The KUL dataset is a large series of 5880 primary isolated CABG patients
from KU Leuven (1971--July 1987). It serves as the primary benchmark for
C binary parity testing because it has the simplest structure
(intercept-only, right-censored) combined with a large sample size that
exercises all three temporal phases.

The original SAS analysis (`hz.deadp.KUL.sas`) also includes
return-of-angina and reintervention endpoints (recorded in the original
fixed-width file with 6 columns), but only the death endpoint
(`int_dead`, `dead`) is extracted for parity testing.

### OMC (open mitral commissurotomy)

The OMC dataset contains 339 patients and is unique in the collection
because it involves **repeated thromboembolic events** (up to 3 per
patient) with **left censoring**. The SAS analysis transforms the
dataset into a repeated-events format using STARTTME and CENSORED
indicators, exercising the interval censoring likelihood.

### TGA (transposition of great arteries)

The TGA dataset contains 470 patients who underwent the arterial switch
operation. It includes derived variables (logarithmic and inverse
transforms of clinical measurements) and is primarily used for
sensitivity analysis and internal validation examples.

### Valves (primary valve replacement)

The VALVES dataset is the largest multivariable example with 1533
patients and multiple endpoints (death, prosthetic valve endocarditis,
bioprosthesis degeneration, reoperation). The SAS analysis
(`hm.deadp.VALVES.sas`) fits a multivariable 3-phase model with 10
covariates across multiple event types.


# SAS/C parameter mapping

The original C/SAS HAZARD procedure uses a different parameterization
than TemporalHazard. The `hzr_argument_mapping()` function documents
the full translation.

```{r mapping}
mapping <- hzr_argument_mapping()
knitr::kable(
  mapping[, c("legacy_input", "r_parameter", "implementation_status", "notes")],
  caption = "SAS HAZARD to R parameter mapping (excerpt)"
)
```


## Early phase (G1) mapping

The SAS early phase uses four parameters: DELTA, RHO (or THALF), NU, M.
These collapse onto the three-parameter decompos family:

- **DELTA** ---Time transformation `B(t) = (exp(delta*t) - 1)/delta`.
  When DELTA = 0 (the common case), `B(t) = t` and the transformation
  is absorbed. Non-zero DELTA is not currently supported.
- **RHO / THALF** ---Scale parameter. `RHO = NU * THALF * ((2^M - 1)/M)^NU`.
  Maps directly to `t_half` in `hzr_phase()`.
- **NU** ---Time exponent. Maps directly.
- **M** ---Shape exponent. Maps directly.

## Late phase (G3) mapping

The SAS late phase uses the G3 decomposition with four parameters, now
directly supported via `hzr_phase("g3", ...)`:

- **TAU** --> `tau` (scale parameter)
- **GAMMA** --> `gamma` (time exponent)
- **ALPHA** --> `alpha` (shape parameter; `alpha = 0` gives exponential case)
- **ETA** --> `eta` (outer exponent)

The G3 formula (for `alpha > 0`) is:
$G_3(t) = \left(\left((t/\tau)^\gamma + 1\right)^{1/\alpha} - 1\right)^\eta$

Unlike G1, G3 is unbounded ---it can grow without limit, making it
suitable for late-phase rising hazards. For the KUL benchmark with
`gamma = 3, alpha = 1, eta = 1`, this simplifies to $G_3(t) = t^3$.


# Version history

The package follows semantic versioning with a prerelease qualifier
during active development:

- **v0.1.0** ---Single-phase engine: Weibull, exponential, log-logistic,
  log-normal distributions with formula interface, predict, and golden
  fixture testing.
- **v0.9.0** ---Multiphase engine: N-phase additive cumulative hazard,
  `hzr_phase()` specification, decomposition engine, C binary parity
  tests, dataset catalog.
- **v0.9.1** (current) ---Vignette suite, roxygen multiphase examples,
  CI workflow fixes (load_pkgload), SAS missing-value handling
  (`na.strings`), print.hazard phase labels, and README refresh.


# References {.unnumbered}

Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. *J Am Stat Assoc.* 1986;81(395):615-624.
doi: [10.1080/01621459.1986.10478314](https://doi.org/10.1080/01621459.1986.10478314)

Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
Probability of atrial fibrillation after ablation: Using a parametric
nonlinear temporal decomposition mixed effects model. *Stat Methods Med Res.*
2018;27(1):126-141.
doi: [10.1177/0962280215623583](https://doi.org/10.1177/0962280215623583)

---
title: "Fitting Hazard Models"
subtitle: "From intercept-only to multiphase additive hazard"
format:
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    number-sections: true
vignette: >
  %\VignetteIndexEntry{Fitting Hazard Models}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)
```

```{r}
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)
```

This vignette walks the core model-fitting workflow from the inside
out: intercept-only fits to establish the baseline hazard shape, then
covariates on top, then the multiphase decomposition, then
multi-endpoint analyses on the same cohort. Every example uses a
clinical dataset shipped with the package. If you haven't seen the
basics — what a parametric hazard model is, why we use the
`Surv(time, status)` formula — start with `vignette("getting-started")`
first; this vignette assumes that context.

The progression matters. Single-distribution intercept-only fits tell
you whether the baseline hazard shape is monotone (Weibull territory)
or has structure that demands a multiphase decomposition.
Multivariable fits add covariate effects on top of a shape you already
trust. Multiphase fits split the baseline shape into clinically
interpretable phases. Multi-endpoint analyses reuse all the above for
separate clinical outcomes — death, reoperation, infection — on the
same patient cohort.


## Intercept-only model: CABG survival (KU Leuven)

The `cabgkul` dataset contains 5,880 patients who underwent primary isolated
coronary artery bypass grafting at KU Leuven between 1971 and 1987.  With only
two columns --- follow-up time and death indicator --- it is the simplest
starting point.

```{r}
#| label: kul-data
data(cabgkul)
str(cabgkul)
```

Fit an intercept-only Weibull. With no covariates on the right-hand
side of the formula the model estimates only the baseline hazard
shape — the scale `mu` and exponent `nu` of a Weibull curve fit to all
5,880 patients pooled. This is the right starting point for any new
dataset: before asking which covariates matter, ask whether a single
monotone hazard even fits the population-level pattern.

```{r}
#| label: kul-weibull
fit_kul <- hazard(
  Surv(int_dead, dead) ~ 1,
  data  = cabgkul,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0),
  fit   = TRUE
)

fit_kul
```

The summary tells us where the optimizer landed; the picture tells us
whether that landing point matches the data. Plot the fitted survival
curve on a fine time grid and overlay the Kaplan-Meier step function
from the raw cohort.

```{r}
#| label: fig-kul-km
#| fig-cap: "Weibull parametric survival vs. Kaplan-Meier (CABG, KU Leuven)"
#| fig-width: 7
#| fig-height: 4.5
t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.9, length.out = 200)
nd     <- data.frame(time = t_grid)
surv   <- predict(fit_kul, newdata = nd, type = "survival") * 100

km     <- survfit(Surv(int_dead, dead) ~ 1, data = cabgkul)
km_df  <- data.frame(time = km$time, survival = km$surv * 100)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_line(data = data.frame(time = t_grid, survival = surv),
            aes(time, survival, colour = "Weibull"), linewidth = 1) +
  scale_colour_manual(values = c("Weibull" = "#0072B2",
                                 "Kaplan-Meier" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after CABG", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

A single Weibull captures the broad trend but misses the distinct early
operative risk and late attrition that the KM curve reveals.  This motivates
the multiphase approach below.


## Multivariable model: AVC repair

The `avc` dataset has 310 patients who underwent atrioventricular canal
repair, with 9 candidate covariates spanning patient demographics (age,
NYHA status), anatomical features (malalignment, orifice morphology),
intra-operative grading (`inc_surg` = surgical grade of AV valve
incompetence), and post-operative complications (`com_iv` = grade IV
complications). We drop incomplete rows so the design matrix is
rectangular, then look at the column types and ranges.

```{r}
#| label: avc-data
data(avc)
avc <- na.omit(avc)
str(avc)
```

Now we put covariates on the right-hand side of the formula and refit.
The `theta` vector grows: two Weibull shape parameters (`mu`, `nu`) plus
six covariate coefficients (`beta1`..`beta6`), each starting at zero.
The optimizer estimates a log-hazard-ratio for every covariate jointly
with the Weibull shape — so the shape and the covariate effects are
identified from the same likelihood, not sequentially.

```{r}
#| label: avc-mv
fit_avc <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv + inc_surg + orifice,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 6)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_avc
```

Each coefficient is a log-hazard-ratio: positive means higher risk,
negative means lower, zero means no effect. The large positive
coefficients on `mal` (anatomical malalignment) and `com_iv`
(grade IV post-operative complications) flag these as the dominant
risk markers in this cohort. The standard errors and Wald
z-statistics in the summary tell you which effects are well
identified and which are noise — a coefficient with a z-statistic
near zero contributes essentially nothing the data can defend.


## Multiphase model: additive hazard decomposition

The single-Weibull fits above gave us a curve that's mediocre
everywhere instead of right anywhere. That's a structural limitation
of a monotone parametric shape, not something more iterations will
fix. The Blackstone–Naftel–Turner framework's key idea is to split
the hazard into a *sum* of phase-specific contributions, each with its
own temporal shape and its own scale:

$$H(t \mid x) = \sum_{j=1}^{J} \mu_j(x) \cdot \Phi_j(t)$$

Each $\Phi_j(t)$ is a phase-specific unit-scaled curve (early-peaking
saturating, flat constant, late-rising polynomial) and each $\mu_j(x)$
is the phase-specific scale, possibly modulated by covariates. The
phases overlap and add — no switching, no thresholds — so the total
instantaneous hazard at any $t$ is the sum of the per-phase rates.
See `vignette("getting-started")` for the longer-form motivation; what
follows here is the practical workflow for *fitting* one.

For AVC we'll use two phases — an early phase to absorb the
operative-window mortality, and a constant phase for the background
rate. AVC patients don't have a clear late-deterioration regime over
this follow-up window, so a third (g3) phase would be unidentified.
We fix the shape parameters and estimate only the scales, matching
the workflow you'd run against a SAS HAZARD reference fit.

```{r}
#| label: avc-mp
fit_mp <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)

summary(fit_mp)
```

The diagnostic that matters is whether the multiphase fit actually
out-performs the single Weibull against the data. Plot both parametric
curves against the same Kaplan-Meier reference so we can see, by eye,
where each model is honest and where each is reaching.

```{r}
#| label: fig-avc-compare
#| fig-cap: "Single-phase Weibull vs. multiphase model against Kaplan-Meier (AVC)"
#| fig-width: 7
#| fig-height: 4.5
t_grid <- seq(0.01, max(avc$int_dead) * 0.95, length.out = 200)
nd     <- data.frame(time = t_grid)

km_avc <- survfit(Surv(int_dead, dead) ~ 1, data = avc)
km_df  <- data.frame(time = km_avc$time, survival = km_avc$surv * 100)

fit_wb <- hazard(
  Surv(int_dead, dead) ~ 1, data = avc, dist = "weibull",
  theta = c(mu = 0.20, nu = 1.0), fit = TRUE
)

surv_wb <- predict(fit_wb, newdata = nd, type = "survival") * 100
surv_mp <- predict(fit_mp, newdata = nd, type = "survival") * 100

plot_df <- rbind(
  data.frame(time = t_grid, survival = surv_wb, Model = "Single Weibull"),
  data.frame(time = t_grid, survival = surv_mp, Model = "Multiphase (2-phase)")
)

ggplot() +
  geom_step(data = km_df, aes(time, survival), colour = "grey50",
            linewidth = 0.5) +
  geom_line(data = plot_df, aes(time, survival, colour = Model),
            linewidth = 1) +
  scale_colour_manual(values = c("Single Weibull" = "#E69F00",
                                 "Multiphase (2-phase)" = "#0072B2")) +
  scale_y_continuous(limits = c(0, 100)) +
  annotate("text", x = max(t_grid) * 0.6, y = 95, label = "KM (grey)",
           size = 3, colour = "grey50") +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

The multiphase model tracks the KM curve much more closely than the
single Weibull, especially across the steep early-mortality window —
which is exactly where the single Weibull was forced to compromise.
The constant phase then carries the slow post-recovery attrition. The
point isn't that multiphase always wins; it's that *when the data has
phase structure*, fitting that structure explicitly is strictly more
honest than averaging it away into one monotone curve.


## Multi-endpoint models: heart valve replacement

The `valves` dataset (1,533 patients) has multiple time-to-event endpoints ---
death, prosthetic valve endocarditis (PVE), and reoperation --- each with its
own follow-up time and event indicator.  The same `hazard()` call fits each
endpoint independently:

Start with the death endpoint. We use age at operation, NYHA class,
and mechanical-valve indicator as covariates — the clinically
canonical set for survival after valve replacement.

```{r}
#| label: valves-death
data(valves)
valves <- na.omit(valves)

fit_death <- hazard(
  Surv(int_dead, dead) ~ age_cop + nyha + mechvalv,
  data  = valves,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0, rep(0, 3)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_death
```

Switch endpoints. Same data, same package, but now we model time to
prosthetic valve endocarditis instead of death. The covariate list
shifts to match the clinical question: `nve` (native-valve endocarditis
history) replaces `nyha` because functional class is less informative
for infection risk than prior endocarditis exposure. The fit returns
its own MLE, coefficients, and standard errors completely independent
of the death model.

```{r}
#| label: valves-pve
fit_pve <- hazard(
  Surv(int_pve, pve) ~ age_cop + nve + mechvalv,
  data  = valves,
  dist  = "weibull",
  theta = c(mu = 0.02, nu = 1.0, rep(0, 3)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_pve
```

Each endpoint gets its own model with its own covariates, but the
hazard model structure — temporal shape plus covariate effects — stays
the same whatever the clinical endpoint. Repeating the workflow for a
third endpoint (reoperation, for example) is mechanical: swap the
`Surv(...)` columns, swap the covariates, refit. The advantage over
running three separate analyses in different tools is that the
predictions, diagnostics, and uncertainty quantification all come from
the same package — there's no risk of subtle differences in censoring
handling or estimator choice between endpoints.


## Interval and left censoring

Right censoring is by far the most common censoring type in clinical
survival data — a patient is still alive at last follow-up, so all we
know is that their event time exceeds the observed window. Two other
types arise frequently enough to warrant explicit handling.

**Interval censoring** occurs when the event is known to have happened
*between* two observation times, but not at a specific time.  The
canonical example is clinic-visit data: a patient is confirmed alive at
month 12 and found dead at month 24, so the event occurred somewhere in
$(12, 24)$.  A naive analysis treats the event as right-censored at 12
or exactly observed at 24; both introduce bias.

**Left censoring** is the mirror: the event occurred *before* the first
observation time (time_upper), so it was already established at the
moment of first contact — $T \leq t_{\text{upper}}$.

### Status codes

The package encodes censoring type in the `status` vector:

| Status | Type | Interpretation |
|-------:|------|----------------|
| `1` | Exact event | Observed at `time` |
| `0` | Right-censored | Event did not occur by `time` |
| `2` | Interval-censored | Event occurred in `(time_lower, time_upper)` |
| `-1` | Left-censored | Event occurred before `time` (upper bound) |

For interval-censored rows, supply both `time_lower` and `time_upper`
directly to `hazard()`.  The formula interface handles `Surv(time, status)`
for right-censored data and `Surv(start, stop, event)` for counting-process
(repeating-event) data; interval censoring is passed via the direct arguments.

### Mixed censoring example

We simulate 400 cardiac surgery patients observed at semi-annual clinic
visits (every 6 months, 4--8 scheduled visits per patient).  If the
event falls between two consecutive visits the observation is
interval-censored; if it falls after the last visit the patient is
right-censored.

```{r}
#| label: interval-simulate
set.seed(101)
n         <- 400
visit_gap <- 6   # months between visits

# True event times: Weibull mu = 0.025, nu = 1.0 (median ~28 months)
true_time <- rexp(n, rate = 0.025)  # nu=1 → exponential

n_visits  <- sample(4:8, n, replace = TRUE)

status     <- integer(n)
time       <- numeric(n)
time_lower <- numeric(n)
time_upper <- numeric(n)

for (i in seq_len(n)) {
  visits <- seq(visit_gap, n_visits[i] * visit_gap, by = visit_gap)
  t_ev   <- true_time[i]
  brk    <- findInterval(t_ev, c(0, visits))  # which interval contains t_ev

  if (brk > length(visits)) {
    # Event after last visit: right-censored.
    # time_lower = 0: with H(time_lower) = H(0) = 0, the right-censored
    # contribution reduces to -(H(time) - H(0)) = -H(time), which is correct.
    status[i]     <- 0L
    time[i]       <- max(visits)
    time_lower[i] <- 0
    time_upper[i] <- max(visits)
  } else {
    # Event within the visit window: interval-censored in (lower, upper)
    lower_bound   <- if (brk == 1L) 0 else visits[brk - 1L]
    upper_bound   <- visits[brk]
    status[i]     <- 2L
    time[i]       <- lower_bound
    time_lower[i] <- lower_bound
    time_upper[i] <- upper_bound
  }
}

table(status)
```

::: {.callout-note}
**`time_lower` as a counting-process entry time.**  In the Weibull and
multiphase likelihoods, `time_lower` doubles as the *entry* (left-truncation)
time for right-censored and exact-event rows (`status %in% c(0, 1)`): when
`0 < time_lower < time`, the row contributes H(stop) − H(start), the
counting-process form used for epoch-decomposed repeated events.  For an
ordinary right-censored or event row with no entry time, leave `time_lower`
at `0` or omit it.  Setting `time_lower = time` is treated as *no entry* —
`time_lower` acts as an entry time only when strictly less than `time`, so
it no longer zeroes the row's contribution.  The exponential, log-logistic,
and log-normal likelihoods do not use `time_lower` as an entry time and are
unaffected.
:::

Fit a Weibull model using both censoring types:

```{r}
#| label: interval-fit
fit_ic <- hazard(
  time       = time,
  status     = status,
  time_lower = time_lower,
  time_upper = time_upper,
  dist  = "weibull",
  theta = c(mu = 0.025, nu = 1.0),
  fit   = TRUE
)

fit_ic
```

### Interval-censored vs naive right-censored

A naive analyst who doesn't have visit-bracket data would record the death
at the *discovery* visit (`time_upper`) rather than as an interval.
This overstates the event time — the patient appears to have survived
longer than they did — biasing the estimated hazard shape.

```{r}
#| label: interval-vs-naive
# Naive: treat each interval-censored row as an exact event at time_upper
# (the visit when the death was first recorded)
fit_naive <- hazard(
  time   = ifelse(status == 2L, time_upper, time),
  status = ifelse(status == 2L, 1L,         status),
  dist  = "weibull",
  theta = c(mu = 0.025, nu = 1.0),
  fit   = TRUE
)

# Compare MLEs; true parameters are mu = 0.025, nu = 1.0
rbind(
  interval_censored = round(coef(fit_ic)[1:2],   4),
  naive_exact_upper = round(coef(fit_naive)[1:2], 4),
  truth             = c(mu = 0.025, nu = 1.0)
)
```

The interval-censored fit recovers both parameters accurately.  The naive
fit estimates $\mu$ comparably but introduces a substantial positive bias
in $\nu$ — it sees events consistently appearing at visit times (every 6
months) and infers a sharper, more periodic hazard shape that doesn't
match the underlying exponential structure.


## Convergence troubleshooting

Multiphase models have more parameters than single-distribution fits, and the
likelihood surface can be flat or multimodal when the data doesn't strongly
identify all of them. Most convergence problems trace back to one of three
causes: poor starting values, too many free parameters for the data at hand,
or a model that asks for a phase the data doesn't contain. This section shows
how to diagnose each and what to do about it.

### Reading the KM cumulative hazard for starting values

For a Weibull model, the relationship between the starting values and the
data is direct: because $H(t) = (\mu t)^\nu$, taking logs gives
$\log H(t) = \nu \log t + \nu \log \mu$. Plot $\log H(t)$ (the Nelson-Aalen
cumulative hazard on the log scale) against $\log t$ and fit a straight line;
the slope is $\hat\nu$ and the intercept is $\hat\nu \log \hat\mu$.

```{r}
#| label: starting-values
nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)

# Drop the zero-time boundary point before log-transforming
nel_clean <- nel[nel$time > 0 & nel$cumhaz > 0, ]
log_t <- log(nel_clean$time)
log_H <- log(nel_clean$cumhaz)

lm_fit  <- lm(log_H ~ log_t)
nu_hat  <- unname(coef(lm_fit)[2])
mu_hat  <- exp(unname(coef(lm_fit)[1]) / nu_hat)
c(mu = round(mu_hat, 4), nu = round(nu_hat, 4))
```

Those values are the Weibull starting point the data itself suggests. They
won't be the MLEs --- the log-log line uses every event time equally, whereas
the MLE weights by the likelihood --- but they land the optimizer in a sensible
neighbourhood and prevent false-convergence to a degenerate solution.

For multiphase models the reading is qualitative. Plot the KM cumulative hazard
on the natural scale and look for kinks: a steep-then-flattening shape in the
first few months signals an early phase whose `t_half` should sit in that early
window; a steady linear rise afterwards signals a constant phase; a late
upward curve beyond the flat region signals a late phase. Set `t_half` to
roughly the time at which the early kink levels off, and set the constant
phase scale mu to approximate the slope of the linear mid-section.

```{r}
#| label: fig-cumhaz-shape
#| fig-cap: "Nelson-Aalen cumulative hazard for CABGKUL — three-phase shape visible as two kinks"
#| fig-width: 7
#| fig-height: 4
ggplot(data.frame(time = nel$time, cumhaz = nel$cumhaz),
       aes(time, cumhaz)) +
  geom_step(colour = "#D55E00", linewidth = 0.7) +
  labs(x = "Months after CABG", y = "Cumulative hazard") +
  theme_minimal()
```

The early steep rise, the mid-range roughly-linear section, and the late
upward acceleration map directly onto the three phases in the CABGKUL model.

### When to fix shape parameters

`fixed = "shapes"` locks the temporal shape parameters (e.g., `t_half`,
`nu`, `m` for a CDF phase; `tau`, `gamma`, `alpha`, `eta` for a g3 phase)
and lets the optimizer estimate only the scale (`log_mu`) for each phase.
This cuts the parameter count substantially and is almost always what you
want in two situations:

- **You have a reference fit.** If a SAS HAZARD run already produced shape
  estimates, replicate those shapes exactly and re-estimate only the scale.
  This is the standard parity workflow.
- **Your sample is small relative to the number of phases.** A rough guide
  is 50 or more events per free shape parameter. Below that threshold the
  shapes are weakly identified and the optimizer wanders.

Estimate shapes freely (`fixed = "none"`, the default) only when you are
exploring a new dataset for the first time and have no prior on the temporal
structure — and even then, start with `fixed = "shapes"` and release the
shapes one at a time if the fixed-shapes fit shows systematic misfit.

### Signs of overparameterization

When a model has more phases than the data supports, the symptoms are
recognizable:

| Symptom | What it means |
|---------|--------------|
| `fit$fit$converged == FALSE` | Optimizer hit `maxit` without satisfying the gradient tolerance |
| `vcov()` returns `NA` | Hessian is singular — parameters are not jointly identified |
| A phase scale (`log_mu`) at a boundary value | That phase is contributing essentially zero hazard; it isn't needed |
| Enormous standard errors on one or more parameters | Flat likelihood in that direction — weak identification |
| Two phases with nearly identical shapes | One can be collapsed into the other |

The AVC dataset has a clear two-phase structure (early CDF plus constant).
Adding a third late-rising phase asks the data for a pattern it doesn't
contain over this follow-up window:

```{r}
#| label: overparameterized
set.seed(42)
fit_3ph <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant"),
    late     = hzr_phase("g3", tau = 5, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")
  ),
  fit = TRUE, control = list(n_starts = 5, maxit = 1000)
)

# Scale magnitudes: exp(log_mu) ≈ 0 for any phase the data doesn't support
log_mu_idx <- grep("log_mu", names(coef(fit_3ph)))
round(exp(coef(fit_3ph)[log_mu_idx]), 6)
```

The constant and late phase scales are both near zero — the optimizer
found no evidence in the AVC data for either of those temporal shapes over
this follow-up window. The early phase is absorbing all the identifiable
structure. Any phase whose fitted scale is $\mu \lesssim 10^{-4}$ is not
contributing meaningful hazard and is a candidate for removal. The
right diagnostic question is not "did the AIC improve?" but "does this
phase represent real clinical biology?" — late deterioration after AVC
repair requires long follow-up to observe; this dataset doesn't have it.

When shapes are also free (`fixed = "none"`), a redundant phase can make
the Hessian rank-deficient and `vcov()` will return `NA` (the Hessian
inversion failed). Fix by adding `fixed = "shapes"` to each phase to
reduce the free-parameter count, then release shapes one at a time if the
fixed-shapes fit shows systematic misfit.

### Optimizer options

Three `control` arguments address most remaining convergence problems:

**`n_starts`** (default 5 for multiphase; single-distribution models run
one optimizer call and ignore this parameter) runs the optimizer from
`n_starts` randomly jittered copies of the initial `theta`, then keeps the
best solution. The default of 5 is sufficient for two-phase models; increase
to 8–10 for three or more phases where the likelihood surface has more local
minima.

```{r}
#| label: nstarts
set.seed(42)
fit_robust <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 10, maxit = 1000)
)
fit_robust$fit$converged
```

**Optimizer strategy (automatic).** The package always uses BFGS as the
primary optimizer. For fixed-shape multiphase models with 2–10 free
parameters, a Nelder-Mead pass runs first on each start to find a good
basin, then BFGS polishes the solution. This warm-up is transparent —
it happens automatically when the conditions are met and there is no
user-facing control to switch it on or off.

**`maxit`** (default 1000) sets the BFGS iteration cap. Hitting `maxit`
without converging usually means either the starting values are far from
the optimum (fix with better starts or `n_starts`) or the model is
overparameterized (fix by fixing shapes or dropping a phase). Raising
`maxit` beyond 2000 rarely helps if the optimizer is genuinely stuck.


## Phase types reference

You've now seen each phase type in use: a `"cdf"` early phase for AVC
operative mortality, a `"constant"` phase for AVC background rate, and
the implicit single shape of every Weibull fit. The package supports
three phase types in total, summarized here for quick reference:

| Type | Description | Typical use |
|------|-------------|-------------|
| `"cdf"` | Sigmoidal CDF shape (parameterized by `t_half`, `nu`, `m`) | Early or late phases with transient risk |
| `"constant"` | Flat hazard (no temporal shape parameters) | Ongoing background risk |
| `"g3"` | Late-phase G3 parameterization (4 parameters: `tau`, `gamma`, `alpha`, `eta`) | Late-rising risk matching C/SAS G3 output |

The `"cdf"` type covers the widest range of shapes: setting `t_half`
small (e.g., 0.5) creates an early-peaking phase; setting it large
(e.g., 10) creates a late-rising phase. The `"constant"` phase needs no
shape parameters. The `"g3"` shape is the explicit late-rising parameterization
that matches the SAS HAZARD "late" library; use it when you need
parity against a C/SAS reference fit, or when the late rise has a
clear lag-then-accelerate pattern that a delayed `"cdf"` doesn't
capture cleanly. See `vignette("mf-mathematical-foundations")` for the
full mathematical treatment of each, including the parameter
identifiability constraints.

---
title: "Mathematical Foundations of TemporalHazard"
subtitle: "Generalized decomposition, multiphase additive hazard, censoring, and time-varying covariates"
format:
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    number-sections: true
vignette: >
  %\VignetteIndexEntry{Mathematical Foundations of TemporalHazard}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE)
has_ggplot <- requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = has_pkg,
  fig.width = 7,
  fig.height = 4.5
)
```

```{r}
#| label: setup
library(TemporalHazard)
```

This vignette lays out the mathematical foundation for the models in
TemporalHazard. The other vignettes treat the package as a black box
you call `hazard()` on; this one opens the box. If you've used the
package and want to know *what* it's computing, *why* the
parameterization is what it is, and *how* the multiphase
decomposition relates to standard parametric survival families, this
is the place.

The four pieces, in order: the generalized temporal decomposition
family (Blackstone, Naftel, and Turner 1986) that every phase is
built from; the additive multiphase hazard model that composes
phases into a full hazard; the maximum-likelihood objective under
mixed censoring (right, left, interval, counting-process), which is
what the optimizer minimizes; and extensions for phase-specific and
time-varying covariates. The same decomposition framework carries
over to longitudinal mixed-effects settings (Rajeswaran et al.
2018), though this vignette stays in the survival regime.

For users migrating from the legacy SAS/C HAZARD program, each
section notes how the R parameterization relates to the original
parameter names. For the full translation table, see
`vignette("sas-to-r-migration")` or call `hzr_argument_mapping()`.


# Generalized Temporal Decomposition {#sec-decomposition}

## The parametric family

Every temporal phase in TemporalHazard is built from a single parametric
family, `decompos(t; t_half, nu, m)`, introduced by Blackstone, Naftel, and
Turner (1986) that produces three linked quantities from just three parameters:

- $G(t)$ — cumulative distribution function (CDF), bounded $[0, 1]$
- $g(t) = dG/dt$ — probability density function
- $h(t) = g(t) / (1 - G(t))$ — hazard function

The three parameters control the shape of the distribution:

| Parameter | Meaning | Constraint |
|-----------|---------|------------|
| `t_half`  | Half-life: time at which $G(t_{1/2}) = 0.5$ | $> 0$ |
| `nu`      | Time exponent — controls rate dynamics | any finite value |
| `m`       | Shape exponent — controls distributional form | any finite value |

::: {.callout-note}
## SAS/C parameter bridge

The original C code used different parameter names per phase:

- **Early phase (G1):** `DELTA`, `RHO`/`THALF`, `NU`, `M` $\to$ `t_half`, `nu`, `m`
- **Late phase (G3):** `TAU`, `GAMMA`, `ALPHA`, `ETA` $\to$ `t_half`, `nu`, `m`

Both collapse onto the same 3-parameter decomposition family.  The C `DELTA`
parameter controlled a time transformation $B(t) = (\exp(\delta t) - 1)/\delta$
that is absorbed into the shape.
:::

In R, `hzr_decompos()` computes all three quantities:

```{r}
#| label: basic-decompos
t_grid <- seq(0.01, 10, length.out = 200)

# Standard sigmoidal (Case 1: m > 0, nu > 0)
d <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 1)

# Verify the half-life property: G(t_half) = 0.5
d_half <- hzr_decompos(3, t_half = 3, nu = 2, m = 1)
cat("G(t_half) =", round(d_half$G, 6), "\n")

# Verify the identity: h(t) = g(t) / (1 - G(t))
h_check <- d$g / (1 - d$G)
cat("Max |h - g/(1-G)| =", max(abs(d$h - h_check)), "\n")
```


## The six valid cases

Different sign combinations of $\nu$ and $m$ pick out different
asymptotic behaviors of `decompos()`, and the package treats each as
a separate case for numerical stability. The combination $m < 0$
**and** $\nu < 0$ produces an integrand with no finite normalization
and is rejected at the input-validation step. The remaining six are
the catalog every phase in the package selects from:

```{r}
#| label: six-cases-table
#| echo: false
cases <- data.frame(
  Case = c("1", "1L", "2", "2L", "3", "3L"),
  m     = c("> 0", "= 0", "< 0", "< 0", "> 0", "= 0"),
  nu    = c("> 0", "> 0", "> 0", "= 0", "< 0", "< 0"),
  Behavior = c(
    "Standard sigmoidal",
    "Weibull-like (exponential limit as m -> 0)",
    "Heavy-tailed",
    "Exponential decay",
    "Bounded cumulative",
    "Bounded exponential"
  )
)
knitr::kable(cases, caption = "Six valid parameter sign combinations")
```

The following plot shows the CDF $G(t)$, density $g(t)$, and hazard $h(t)$
for each case, all with `t_half = 3`:

```{r}
#| label: fig-six-cases
#| fig-cap: "G(t), g(t), and h(t) for all six valid decomposition cases"
#| fig-height: 8
if (has_ggplot) {
  library(ggplot2)

  params <- list(
    "Case 1: m=1, nu=2"     = list(nu = 2,   m = 1),
    "Case 1L: m=0, nu=2"    = list(nu = 2,   m = 0),
    "Case 2: m=-0.5, nu=2"  = list(nu = 2,   m = -0.5),
    "Case 2L: m=-0.5, nu=0" = list(nu = 0,   m = -0.5),
    "Case 3: m=1, nu=-0.5"  = list(nu = -0.5, m = 1),
    "Case 3L: m=0, nu=-0.5" = list(nu = -0.5, m = 0)
  )

  t_grid <- seq(0.01, 10, length.out = 200)
  rows <- list()

  for (nm in names(params)) {
    p <- params[[nm]]
    d <- hzr_decompos(t_grid, t_half = 3, nu = p$nu, m = p$m)
    rows <- c(rows, list(data.frame(
      time = rep(t_grid, 3),
      value = c(d$G, d$g, d$h),
      quantity = rep(c("G(t) — CDF", "g(t) — density", "h(t) — hazard"),
                     each = length(t_grid)),
      case = nm
    )))
  }

  df <- do.call(rbind, rows)

  # Cap hazard display at reasonable values for readability
  df$value[df$quantity == "h(t) — hazard" & df$value > 5] <- NA

  ggplot(df, aes(x = time, y = value)) +
    geom_line(linewidth = 0.6) +
    facet_grid(quantity ~ case, scales = "free_y") +
    labs(x = "Time", y = NULL) +
    theme_minimal(base_size = 10) +
    theme(strip.text = element_text(size = 7))
}
```


## Derivation sketch

The construction begins with a rate function $\rho$ chosen so that
$G(t_{1/2}) = 0.5$ exactly.  For Case 1 ($m > 0, \nu > 0$):

$$
\rho = \nu \, t_{1/2} \left(\frac{2^m - 1}{m}\right)^{\!\nu}
$$

Then, with the dimensionless time $b(t) = \nu t / \rho$:

$$
G(t) = \left(1 + m \, b(t)^{-1/\nu}\right)^{-1/m}
$$

$$
g(t) = \left(1 + m \, b(t)^{-1/\nu}\right)^{-1/m - 1}
       \cdot b(t)^{-1/\nu - 1} / \rho
$$

The other five cases arise as limits ($m \to 0$, $\nu \to 0$) or sign
reflections of this base form.  The implementation in `hzr_decompos()`
dispatches to the appropriate branch after checking the signs of `nu` and `m`.


# Additive Multiphase Hazard Model {#sec-multiphase}

## Model specification

The total cumulative hazard decomposes additively across $J$ phases:

$$
H(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \cdot \Phi_j(t;\, t_{1/2,j},\, \nu_j,\, m_j)
$$

where:

- $\mu_j(\mathbf{x}) = \exp(\alpha_j + \mathbf{x}_j \boldsymbol{\beta}_j)$ is the phase-specific log-linear scale with covariates
- $\Phi_j(t)$ is the temporal shape, determined by the phase type

The three phase types correspond to different uses of the decomposition output:

| Phase type | $\Phi_j(t)$ | Domain | Interpretation |
|------------|-------------|--------|----------------|
| `"cdf"`    | $G(t)$ | $[0, 1]$ | Early risk that resolves over time |
| `"hazard"` | $-\log(1 - G(t))$ | $[0, \infty)$ | Late or aging risk that accumulates |
| `"constant"` | $t$ | $[0, \infty)$ | Flat background rate (no shape parameters) |

::: {.callout-note}
## SAS/C bridge

The classic three-phase HAZARD model used:

- **G1** (early) $\to$ `hzr_phase("cdf", ...)`
- **G2** (constant) $\to$ `hzr_phase("constant")`
- **G3** (late) $\to$ `hzr_phase("hazard", ...)`

TemporalHazard generalizes this to $N$ phases of any type.
:::


## Derived quantities

From the cumulative hazard, the instantaneous hazard and survival are:

$$
h(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \cdot \varphi_j(t)
\qquad\text{where } \varphi_j = d\Phi_j/dt
$$

$$
S(t \mid \mathbf{x}) = \exp\!\bigl(-H(t \mid \mathbf{x})\bigr)
$$

The derivative $\varphi_j(t)$ depends on phase type:

- `"cdf"`: $\varphi(t) = g(t)$ (the density)
- `"hazard"`: $\varphi(t) = h(t) = g(t)/(1 - G(t))$
- `"constant"`: $\varphi(t) = 1$


## Constructing phases in R

The math above is exposed in R through `hzr_phase()`, which builds
one phase at a time, and `hazard()`, which composes phases into a
full model. The same `(t_half, nu, m)` triple from `decompos()`
parameterizes the `"cdf"` and `"hazard"` shapes; the `"g3"` shape
exposes its own `(tau, gamma, alpha, eta)` parameters from the SAS
"late" library; and `"constant"` has no shape parameters at all. A
canonical three-phase model uses the `"cdf"` shape for the early
phase, `"constant"` for the background, and either a delayed
`"cdf"` or `"g3"` for the late phase.

```{r}
#| label: phase-construction
# Classic three-phase pattern
early <- hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0)
const <- hzr_phase("constant")
late  <- hzr_phase("hazard",   t_half = 5,   nu = 1, m = 0)

print(early)
print(const)
print(late)
```

The cumulative hazard contribution $\Phi(t)$ and instantaneous hazard
$\varphi(t)$ for each phase can be computed directly:

```{r}
#| label: fig-phase-shapes
#| fig-cap: "Phi(t) and phi(t) for each phase type"
#| fig-height: 5
if (has_ggplot) {
  t_grid <- seq(0.01, 12, length.out = 200)

  phi_early <- hzr_phase_cumhaz(t_grid, t_half = 0.5, nu = 2, m = 0,
                                 type = "cdf")
  phi_late  <- hzr_phase_cumhaz(t_grid, t_half = 5, nu = 1, m = 0,
                                 type = "hazard")
  phi_const <- hzr_phase_cumhaz(t_grid, type = "constant")

  dphi_early <- hzr_phase_hazard(t_grid, t_half = 0.5, nu = 2, m = 0,
                                  type = "cdf")
  dphi_late  <- hzr_phase_hazard(t_grid, t_half = 5, nu = 1, m = 0,
                                  type = "hazard")
  dphi_const <- hzr_phase_hazard(t_grid, type = "constant")

  df <- data.frame(
    time = rep(t_grid, 6),
    value = c(phi_early, phi_late, phi_const,
              dphi_early, dphi_late, dphi_const),
    phase = rep(rep(c("cdf (early)", "hazard (late)", "constant"),
                    each = length(t_grid)), 2),
    quantity = rep(c("Phi(t) — cumulative", "phi(t) — instantaneous"),
                   each = 3 * length(t_grid))
  )

  ggplot(df, aes(x = time, y = value, colour = phase)) +
    geom_line(linewidth = 0.8) +
    facet_wrap(~ quantity, scales = "free_y", ncol = 1) +
    scale_colour_manual(values = c(
      "cdf (early)" = "#0072B2",
      "hazard (late)" = "#D55E00",
      "constant" = "#009E73"
    )) +
    labs(x = "Time", y = NULL, colour = "Phase type") +
    theme_minimal(base_size = 11) +
    theme(legend.position = "bottom")
}
```


## Additive composition example

To build intuition, here is how three phases combine into a total hazard.
The key insight is that phases contribute additively to the **cumulative
hazard** $H(t)$, not to the survival directly.

```{r}
#| label: fig-additive-demo
#| fig-cap: "Three-phase additive cumulative hazard: total = early + constant + late"
#| fig-height: 4
if (has_ggplot) {
  t_grid <- seq(0.01, 15, length.out = 300)

  mu_early <- 0.3
  mu_const <- 0.05
  mu_late  <- 0.2

  H_early <- mu_early * hzr_phase_cumhaz(t_grid, t_half = 0.5, nu = 2,
                                           m = 0, type = "cdf")
  H_const <- mu_const * hzr_phase_cumhaz(t_grid, type = "constant")
  H_late  <- mu_late  * hzr_phase_cumhaz(t_grid, t_half = 5, nu = 1,
                                           m = 0, type = "hazard")
  H_total <- H_early + H_const + H_late

  df <- data.frame(
    time = rep(t_grid, 4),
    cumhaz = c(H_early, H_const, H_late, H_total),
    component = rep(c("Early (cdf)", "Constant", "Late (hazard)", "Total"),
                    each = length(t_grid))
  )
  df$component <- factor(df$component,
    levels = c("Total", "Early (cdf)", "Constant", "Late (hazard)"))

  ggplot(df, aes(x = time, y = cumhaz, colour = component,
                 linewidth = component)) +
    geom_line() +
    scale_colour_manual(values = c(
      "Total" = "black", "Early (cdf)" = "#0072B2",
      "Constant" = "#009E73", "Late (hazard)" = "#D55E00"
    )) +
    scale_linewidth_manual(values = c(
      "Total" = 1.2, "Early (cdf)" = 0.6,
      "Constant" = 0.6, "Late (hazard)" = 0.6
    )) +
    labs(x = "Time", y = "Cumulative hazard H(t)",
         colour = NULL, linewidth = NULL) +
    theme_minimal(base_size = 11) +
    theme(legend.position = "bottom")
}
```


# Maximum Likelihood Estimation {#sec-likelihood}

## Log-likelihood under mixed censoring

TemporalHazard supports four censoring types in a single dataset, coded
by the `status` indicator:

| Code | Type | Contribution to log-likelihood |
|------|------|-------------------------------|
| $\delta_i = 1$ | Exact event | $\log h(t_i \mid \mathbf{x}_i) - H(t_i \mid \mathbf{x}_i)$ |
| $\delta_i = 0$ | Right-censored | $-H(t_i \mid \mathbf{x}_i)$ |
| $\delta_i = -1$ | Left-censored | $\log\bigl(1 - \exp(-H(u_i \mid \mathbf{x}_i))\bigr)$ |
| $\delta_i = 2$ | Interval-censored | $-H(l_i \mid \mathbf{x}_i) + \log\bigl(1 - \exp(-(H(u_i) - H(l_i)))\bigr)$ |

where $l_i$ and $u_i$ are the lower and upper bounds of the censoring
interval.

The full log-likelihood is:

$$
\ell(\boldsymbol{\theta}) = \sum_{i:\,\delta_i=1}
  \Bigl[\log h(t_i \mid \mathbf{x}_i) - H(t_i \mid \mathbf{x}_i)\Bigr]
  - \sum_{i:\,\delta_i=0} H(t_i \mid \mathbf{x}_i)
$$
$$
  + \sum_{i:\,\delta_i=-1}
    \log\!\bigl(1 - e^{-H(u_i \mid \mathbf{x}_i)}\bigr)
  + \sum_{i:\,\delta_i=2}
    \Bigl[-H(l_i \mid \mathbf{x}_i)
    + \log\!\bigl(1 - e^{-(H(u_i) - H(l_i))}\bigr)\Bigr]
$$

The $\log(1 - e^{-x})$ terms are computed using the numerically stable
primitive `hzr_log1mexp()`, which avoids catastrophic cancellation when $x$
is near zero.

```{r}
#| label: log1mexp-demo
# Naive log(1 - exp(-x)) fails near zero
x_small <- 1e-15
cat("Naive:  ", log(1 - exp(-x_small)), "\n")
cat("Stable: ", hzr_log1mexp(x_small), "\n")
```


## Internal parameterization

During optimization, shape parameters are reparameterized for unconstrained
search.  For each phase $j$, the internal parameter sub-vector is:

| Parameter | Internal (estimation) | User-facing (reported) |
|-----------|----------------------|----------------------|
| Scale     | $\log \mu_j$ | $\mu_j = \exp(\log \mu_j)$ |
| Half-life | $\log t_{1/2,j}$ | $t_{1/2,j} = \exp(\log t_{1/2,j})$ |
| Time exponent | $\nu_j$ (unconstrained) | $\nu_j$ |
| Shape     | $m_j$ (unconstrained) | $m_j$ |
| Covariates | $\beta_{j,1}, \ldots, \beta_{j,p_j}$ | same |

Constant phases omit `log_t_half`, `nu`, and `m`, contributing only one
shape-free parameter (`log_mu`).  The full $\boldsymbol{\theta}$ is the
concatenation of all phase sub-vectors.


## Optimization strategy

The optimizer uses `stats::optim()` with BFGS on the unconstrained internal
scale, wrapped by `.hzr_optim_generic()`.  Key features:

- **Multi-start**: the optimizer runs from $k$ random perturbations around
  the user-supplied starting values (default $k = 5$, controlled by
  `control$n_starts`).  The run with the highest log-likelihood is kept.
- **Feasibility guard**: any parameter vector where $m < 0$ **and**
  $\nu < 0$ for the same phase returns $-\infty$ immediately.
- **Post-fit Hessian**: the numerical Hessian at the solution is inverted
  to produce the variance-covariance matrix.  Standard errors are
  $\sqrt{\text{diag}(\hat{V})}$.


# Covariates and Phase-Specific Formulas {#sec-covariates}

## Global covariates

By default, every phase shares the same covariate vector from the model
formula.  Each phase gets its own coefficient vector $\boldsymbol{\beta}_j$:

$$
\mu_j(\mathbf{x}) = \exp\!\bigl(\alpha_j + \mathbf{x}\,\boldsymbol{\beta}_j\bigr)
$$

So the same predictors can have different effects on early vs. late risk,
which is a core feature of multiphase models.

```{r}
#| label: global-covariates
#| eval: false
# Both phases use age and nyha from the global formula
hazard(
  survival::Surv(time, status) ~ age + nyha,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf",    t_half = 0.5, nu = 2, m = 0),
    late  = hzr_phase("hazard", t_half = 5,   nu = 1, m = 0)
  ),
  fit = TRUE
)
```


## Phase-specific covariates

The global formula above lets every covariate act on every phase
through a shared shift $x^\top \beta$. That's the right structure
when a covariate plausibly affects the entire hazard trajectory. But
clinical reality often pulls the other way: surgical risk factors
(grade IV complications, intra-operative blood loss) belong in the
early phase but say nothing about late attrition; chronic
comorbidities (NYHA class progression, late-onset diabetes) belong
in the late phase but were irrelevant during the operative window.
Phase-specific covariates let you build that structure directly.

When a phase carries its own one-sided formula, the phase gets its
own design matrix from the data, and only those covariates appear
in that phase's $\boldsymbol{\beta}_j$. Other phases ignore them
entirely.

```{r}
#| label: phase-specific-covariates
#| eval: false
# Early risk depends on surgical factors; late risk on chronic conditions
hazard(
  survival::Surv(time, status) ~ age + nyha + shock,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf",    t_half = 0.5, nu = 2, m = 0,
                      formula = ~ age + shock),
    late  = hzr_phase("hazard", t_half = 5,   nu = 1, m = 0,
                      formula = ~ age + nyha)
  ),
  fit = TRUE
)
```

When a phase has a `formula`, it gets its own design matrix built from the
data, and only those covariates appear in its $\boldsymbol{\beta}_j$.  The
parameter vector is shorter for phases with fewer covariates, which can
improve identifiability and convergence.


## Time-varying coefficients

For single-distribution models, TemporalHazard supports piecewise
time-varying coefficients via the `time_windows` argument.  This expands the
design matrix into window-specific blocks:

$$
\eta_i = \sum_{k=1}^{K} \mathbf{x}_i \boldsymbol{\beta}_k \cdot
\mathbf{1}(t_i \in W_k)
$$

where $W_1, \ldots, W_K$ are the time windows defined by the cut points.
Each predictor gets a separate coefficient in each window, allowing effects
to change over follow-up time.

```{r}
#| label: time-varying-demo
#| eval: false
# Two windows: [0, 2] and (2, infinity)
hazard(
  survival::Surv(time, status) ~ age + nyha,
  data         = dat,
  theta        = c(mu = 0.25, nu = 1.1,
                   age_w1 = 0, nyha_w1 = 0,
                   age_w2 = 0, nyha_w2 = 0),
  dist         = "weibull",
  time_windows = 2,
  fit          = TRUE
)
```

::: {.callout-tip}
## Multiphase vs. time-varying coefficients

For multiphase models, the phase structure itself captures time-varying
effects: early phases dominate at small $t$ and late phases at large $t$.
The `time_windows` mechanism is a complementary approach for
single-distribution models.
:::


# Identifiability and Practical Considerations {#sec-identifiability}

## Parameter identifiability

Multiphase models with many free parameters can have flat or multi-modal
likelihood surfaces.  Practical guidelines:

1. **Fix shape parameters when possible.**  If clinical knowledge suggests a
   specific temporal pattern (e.g. early mortality follows a Weibull shape
   with $m = 0$), fix `m` in the `hzr_phase()` starting values and inspect
   whether the optimizer moves it.

2. **Start from the SAS/C estimates.**  If legacy results are available,
   translate them using `hzr_argument_mapping()` and supply as starting values.

3. **Use multi-start optimization.**  The default `control$n_starts = 5`
   helps escape local optima, but complex models may benefit from more starts.

4. **Phase-specific covariates improve identifiability.**  Restricting each
   phase to clinically relevant predictors reduces the parameter count and
   sharpens the likelihood surface.


## Numerical stability

The decomposition engine applies several guards:

- Time is clamped above `.Machine$double.xmin` to avoid $0^{\text{negative}}$
- $1 - G(t)$ is clamped above `.Machine$double.xmin` before computing the
  hazard $h(t) = g(t)/(1 - G(t))$
- Left- and interval-censored contributions use `hzr_log1mexp()` to avoid
  $\log(0)$ when $H(t)$ is very small

Together these keep the gradients finite throughout the optimization, even in
regions of parameter space far from the optimum.


# Summary of Key Functions {#sec-api}

| Function | Purpose |
|----------|---------|
| `hzr_decompos(t, t_half, nu, m)` | Core decomposition: returns $G$, $g$, $h$ |
| `hzr_phase_cumhaz(t, ..., type)` | Phase cumulative hazard $\Phi(t)$ |
| `hzr_phase_hazard(t, ..., type)` | Phase instantaneous hazard $\varphi(t)$ |
| `hzr_phase(type, t_half, nu, m, formula)` | Construct a phase specification |
| `hazard(..., dist = "multiphase", phases = ...)` | Fit a multiphase model |
| `predict(fit, type, decompose = TRUE)` | Per-phase decomposed predictions |
| `hzr_argument_mapping()` | SAS/C $\to$ R parameter translation table |
| `hzr_log1mexp(x)` | Stable $\log(1 - e^{-x})$ |

For a worked clinical example, see `vignette("getting-started")`.  For
migration from SAS HAZARD, see `vignette("sas-to-r-migration")`.


# References {.unnumbered}

Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. *J Am Stat Assoc.* 1986;81(395):615–624.
doi: [10.1080/01621459.1986.10478314](https://doi.org/10.1080/01621459.1986.10478314)

Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
Probability of atrial fibrillation after ablation: Using a parametric
nonlinear temporal decomposition mixed effects model. *Stat Methods Med Res.*
2018;27(1):126–141.
doi: [10.1177/0962280215623583](https://doi.org/10.1177/0962280215623583)

---
title: "Fitting Hazard Models"
subtitle: "From intercept-only to multiphase additive hazard"
format:
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    number-sections: true
vignette: >
  %\VignetteIndexEntry{Fitting Hazard Models}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)
```

```{r}
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)
```

This vignette walks the core model-fitting workflow from the inside
out: intercept-only fits to establish the baseline hazard shape, then
covariates on top, then the multiphase decomposition, then
multi-endpoint analyses on the same cohort. Every example uses a
clinical dataset shipped with the package. If you haven't seen the
basics — what a parametric hazard model is, why we use the
`Surv(time, status)` formula — start with `vignette("getting-started")`
first; this vignette assumes that context.

The progression matters. Single-distribution intercept-only fits tell
you whether the baseline hazard shape is monotone (Weibull territory)
or has structure that demands a multiphase decomposition.
Multivariable fits add covariate effects on top of a shape you already
trust. Multiphase fits split the baseline shape into clinically
interpretable phases. Multi-endpoint analyses reuse all the above for
separate clinical outcomes — death, reoperation, infection — on the
same patient cohort.


## Intercept-only model: CABG survival (KU Leuven)

The `cabgkul` dataset contains 5,880 patients who underwent primary isolated
coronary artery bypass grafting at KU Leuven between 1971 and 1987.  With only
two columns --- follow-up time and death indicator --- it is the simplest
starting point.

```{r}
#| label: kul-data
data(cabgkul)
str(cabgkul)
```

Fit an intercept-only Weibull. With no covariates on the right-hand
side of the formula the model estimates only the baseline hazard
shape — the scale `mu` and exponent `nu` of a Weibull curve fit to all
5,880 patients pooled. This is the right starting point for any new
dataset: before asking which covariates matter, ask whether a single
monotone hazard even fits the population-level pattern.

```{r}
#| label: kul-weibull
fit_kul <- hazard(
  Surv(int_dead, dead) ~ 1,
  data  = cabgkul,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0),
  fit   = TRUE
)

fit_kul
```

The summary tells us where the optimizer landed; the picture tells us
whether that landing point matches the data. Plot the fitted survival
curve on a fine time grid and overlay the Kaplan-Meier step function
from the raw cohort.

```{r}
#| label: fig-kul-km
#| fig-cap: "Weibull parametric survival vs. Kaplan-Meier (CABG, KU Leuven)"
#| fig-width: 7
#| fig-height: 4.5
t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.9, length.out = 200)
nd     <- data.frame(time = t_grid)
surv   <- predict(fit_kul, newdata = nd, type = "survival") * 100

km     <- survfit(Surv(int_dead, dead) ~ 1, data = cabgkul)
km_df  <- data.frame(time = km$time, survival = km$surv * 100)

ggplot() +
  geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier"),
            linewidth = 0.5) +
  geom_line(data = data.frame(time = t_grid, survival = surv),
            aes(time, survival, colour = "Weibull"), linewidth = 1) +
  scale_colour_manual(values = c("Weibull" = "#0072B2",
                                 "Kaplan-Meier" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after CABG", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

A single Weibull captures the broad trend but misses the distinct early
operative risk and late attrition that the KM curve reveals.  This motivates
the multiphase approach below.


## Multivariable model: AVC repair

The `avc` dataset has 310 patients who underwent atrioventricular canal
repair, with 9 candidate covariates spanning patient demographics (age,
NYHA status), anatomical features (malalignment, orifice morphology),
intra-operative grading (`inc_surg` = surgical grade of AV valve
incompetence), and post-operative complications (`com_iv` = grade IV
complications). We drop incomplete rows so the design matrix is
rectangular, then look at the column types and ranges.

```{r}
#| label: avc-data
data(avc)
avc <- na.omit(avc)
str(avc)
```

Now we put covariates on the right-hand side of the formula and refit.
The `theta` vector grows: two Weibull shape parameters (`mu`, `nu`) plus
six covariate coefficients (`beta1`..`beta6`), each starting at zero.
The optimizer estimates a log-hazard-ratio for every covariate jointly
with the Weibull shape — so the shape and the covariate effects are
identified from the same likelihood, not sequentially.

```{r}
#| label: avc-mv
fit_avc <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv + inc_surg + orifice,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 6)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_avc
```

Each coefficient is a log-hazard-ratio: positive means higher risk,
negative means lower, zero means no effect. The large positive
coefficients on `mal` (anatomical malalignment) and `com_iv`
(grade IV post-operative complications) flag these as the dominant
risk markers in this cohort. The standard errors and Wald
z-statistics in the summary tell you which effects are well
identified and which are noise — a coefficient with a z-statistic
near zero contributes essentially nothing the data can defend.


## Multiphase model: additive hazard decomposition

The single-Weibull fits above gave us a curve that's mediocre
everywhere instead of right anywhere. That's a structural limitation
of a monotone parametric shape, not something more iterations will
fix. The Blackstone–Naftel–Turner framework's key idea is to split
the hazard into a *sum* of phase-specific contributions, each with its
own temporal shape and its own scale:

$$H(t \mid x) = \sum_{j=1}^{J} \mu_j(x) \cdot \Phi_j(t)$$

Each $\Phi_j(t)$ is a phase-specific unit-scaled curve (early-peaking
saturating, flat constant, late-rising polynomial) and each $\mu_j(x)$
is the phase-specific scale, possibly modulated by covariates. The
phases overlap and add — no switching, no thresholds — so the total
instantaneous hazard at any $t$ is the sum of the per-phase rates.
See `vignette("getting-started")` for the longer-form motivation; what
follows here is the practical workflow for *fitting* one.

For AVC we'll use two phases — an early phase to absorb the
operative-window mortality, and a constant phase for the background
rate. AVC patients don't have a clear late-deterioration regime over
this follow-up window, so a third (g3) phase would be unidentified.
We fix the shape parameters and estimate only the scales, matching
the workflow you'd run against a SAS HAZARD reference fit.

```{r}
#| label: avc-mp
fit_mp <- hazard(
  Surv(int_dead, dead) ~ 1,
  data   = avc,
  dist   = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1,
                          fixed = "shapes"),
    constant = hzr_phase("constant")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)

summary(fit_mp)
```

The diagnostic that matters is whether the multiphase fit actually
out-performs the single Weibull against the data. Plot both parametric
curves against the same Kaplan-Meier reference so we can see, by eye,
where each model is honest and where each is reaching.

```{r}
#| label: fig-avc-compare
#| fig-cap: "Single-phase Weibull vs. multiphase model against Kaplan-Meier (AVC)"
#| fig-width: 7
#| fig-height: 4.5
t_grid <- seq(0.01, max(avc$int_dead) * 0.95, length.out = 200)
nd     <- data.frame(time = t_grid)

km_avc <- survfit(Surv(int_dead, dead) ~ 1, data = avc)
km_df  <- data.frame(time = km_avc$time, survival = km_avc$surv * 100)

fit_wb <- hazard(
  Surv(int_dead, dead) ~ 1, data = avc, dist = "weibull",
  theta = c(mu = 0.20, nu = 1.0), fit = TRUE
)

surv_wb <- predict(fit_wb, newdata = nd, type = "survival") * 100
surv_mp <- predict(fit_mp, newdata = nd, type = "survival") * 100

plot_df <- rbind(
  data.frame(time = t_grid, survival = surv_wb, Model = "Single Weibull"),
  data.frame(time = t_grid, survival = surv_mp, Model = "Multiphase (2-phase)")
)

ggplot() +
  geom_step(data = km_df, aes(time, survival), colour = "grey50",
            linewidth = 0.5) +
  geom_line(data = plot_df, aes(time, survival, colour = Model),
            linewidth = 1) +
  scale_colour_manual(values = c("Single Weibull" = "#E69F00",
                                 "Multiphase (2-phase)" = "#0072B2")) +
  scale_y_continuous(limits = c(0, 100)) +
  annotate("text", x = max(t_grid) * 0.6, y = 95, label = "KM (grey)",
           size = 3, colour = "grey50") +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

The multiphase model tracks the KM curve much more closely than the
single Weibull, especially across the steep early-mortality window —
which is exactly where the single Weibull was forced to compromise.
The constant phase then carries the slow post-recovery attrition. The
point isn't that multiphase always wins; it's that *when the data has
phase structure*, fitting that structure explicitly is strictly more
honest than averaging it away into one monotone curve.


## Multi-endpoint models: heart valve replacement

The `valves` dataset (1,533 patients) has multiple time-to-event endpoints ---
death, prosthetic valve endocarditis (PVE), and reoperation --- each with its
own follow-up time and event indicator.  The same `hazard()` call fits each
endpoint independently:

Start with the death endpoint. We use age at operation, NYHA class,
and mechanical-valve indicator as covariates — the clinically
canonical set for survival after valve replacement.

```{r}
#| label: valves-death
data(valves)
valves <- na.omit(valves)

fit_death <- hazard(
  Surv(int_dead, dead) ~ age_cop + nyha + mechvalv,
  data  = valves,
  dist  = "weibull",
  theta = c(mu = 0.10, nu = 1.0, rep(0, 3)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_death
```

Switch endpoints. Same data, same package, but now we model time to
prosthetic valve endocarditis instead of death. The covariate list
shifts to match the clinical question: `nve` (native-valve endocarditis
history) replaces `nyha` because functional class is less informative
for infection risk than prior endocarditis exposure. The fit returns
its own MLE, coefficients, and standard errors completely independent
of the death model.

```{r}
#| label: valves-pve
fit_pve <- hazard(
  Surv(int_pve, pve) ~ age_cop + nve + mechvalv,
  data  = valves,
  dist  = "weibull",
  theta = c(mu = 0.02, nu = 1.0, rep(0, 3)),
  fit   = TRUE,
  control = list(maxit = 500)
)

fit_pve
```

Each endpoint gets its own model with its own covariates, but the
hazard model structure — temporal shape plus covariate effects — stays
the same whatever the clinical endpoint. Repeating the workflow for a
third endpoint (reoperation, for example) is mechanical: swap the
`Surv(...)` columns, swap the covariates, refit. The advantage over
running three separate analyses in different tools is that the
predictions, diagnostics, and uncertainty quantification all come from
the same package — there's no risk of subtle differences in censoring
handling or estimator choice between endpoints.


## Phase types reference

You've now seen each phase type in use: a `"cdf"` early phase for AVC
operative mortality, a `"constant"` phase for AVC background rate, and
the implicit single shape of every Weibull fit. The package supports
three phase types in total, summarized here for quick reference:

| Type | Description | Typical use |
|------|-------------|-------------|
| `"cdf"` | Sigmoidal CDF shape (parameterized by `t_half`, `nu`, `m`) | Early or late phases with transient risk |
| `"constant"` | Flat hazard (no temporal shape parameters) | Ongoing background risk |
| `"g3"` | Late-phase G3 parameterization (4 parameters: `tau`, `gamma`, `alpha`, `eta`) | Late-rising risk matching C/SAS G3 output |

The `"cdf"` type covers the widest range of shapes: setting `t_half`
small (e.g., 0.5) creates an early-peaking phase; setting it large
(e.g., 10) creates a late-rising phase. The `"constant"` phase needs no
shape parameters. The `"g3"` shape is the explicit late-rising parameterization
that matches the SAS HAZARD "late" library; use it when you need
parity against a C/SAS reference fit, or when the late rise has a
clear lag-then-accelerate pattern that a delayed `"cdf"` doesn't
capture cleanly. See `vignette("mf-mathematical-foundations")` for the
full mathematical treatment of each, including the parameter
identifiability constraints.

---
title: "Inference & Diagnostics"
subtitle: "Exploratory screening, bootstrap confidence intervals, and sensitivity analysis"
format:
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    number-sections: true
vignette: >
  %\VignetteIndexEntry{Inference & Diagnostics}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| include: false
has_pkg <- requireNamespace("TemporalHazard", quietly = TRUE) &&
  requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = has_pkg
)
```

```{r}
#| label: setup
library(TemporalHazard)
library(survival)
library(ggplot2)
```

Fitting a hazard model is the middle of the workflow, not the end.
Before you fit, you screen the covariates so you know what
transformations the data actually supports. After you fit, you
quantify uncertainty (bootstrap CIs, Wald intervals), check whether
the model is honest to the data (goodness-of-fit overlays,
decile-of-risk calibration), and probe its behavior under different
covariate scenarios (sensitivity analysis). This vignette walks the
diagnostic and inferential tools the package ships for each of those
steps.

These pieces correspond directly to steps in the classic SAS HAZARD
workflow — the `lg.*` logistic-screening macros, the `bs.*` bootstrap
macros, the `hs.*` hazard-sensitivity macros — re-implemented as
first-class R functions (`hzr_calibrate()`, `hzr_kaplan()`,
`hzr_nelson()`, `hzr_bootstrap()`, `hzr_gof()`, `hzr_deciles()`). If
you've worked through a SAS HAZARD analysis you already know the
shape of what follows; if not, treat this vignette as the standard
diagnostic pass you'd run on any fitted hazard model before trusting
its conclusions.


## Exploratory covariate screening

Before fitting the parametric hazard model, screen each candidate
covariate with a simple logistic regression against the event
indicator. This is much cheaper than fitting the full hazard model
and answers two distinct questions: which covariates carry signal at
all (rough significance triage), and what *functional form* each
covariate enters with — linear, log, polynomial, or something
non-monotone that needs binning. The screening step is what stops you
from blindly throwing every column at `hazard()` and then trying to
debug a model that has shape mis-specification baked in.

```{r}
#| label: explore-data
data(avc)
avc <- na.omit(avc)

# Candidate covariates
candidates <- c("age", "status", "mal", "com_iv", "inc_surg", "orifice")
```

Fit a univariable logistic regression for each candidate against the
death indicator. The p-values that come out are not the final word on
whether a covariate matters — they ignore the joint structure of the
hazard model and they treat the event time as a binary outcome — but
they do tell you which covariates have no chance of mattering, and
which deserve closer inspection.

```{r}
#| label: explore-logit
logit_fits <- lapply(candidates, function(var) {
  fmla <- as.formula(paste("dead ~", var))
  glm(fmla, data = avc, family = binomial())
})
names(logit_fits) <- candidates

# Which covariates are significant univariably?
p_values <- vapply(logit_fits, function(f) {
  coef(summary(f))[2, "Pr(>|z|)"]
}, numeric(1))

data.frame(covariate = names(p_values),
           p_value   = round(p_values, 4),
           row.names = NULL)
```

```{r}
#| label: fig-explore-age
#| fig-cap: "Exploratory: age vs. mortality with LOESS smooth"
#| fig-width: 7
#| fig-height: 4
ggplot(avc, aes(age, dead)) +
  geom_jitter(height = 0.05, width = 0, alpha = 0.3, size = 1) +
  geom_smooth(method = "loess", se = TRUE, colour = "#0072B2",
              linewidth = 0.8) +
  labs(x = "Age at operation (months)", y = "Death (0/1)") +
  theme_minimal()
```

The LOESS smooth reveals the *shape* of the covariate-mortality
relationship in a non-parametric way. A roughly linear smooth means
the covariate enters the hazard model on its natural scale; a U or
inverted-U shape means a polynomial or a non-monotone transform; a
plateau-then-rise means a threshold effect that may need binning.
This is the call you make *before* writing the formula for
`hazard()`, not after staring at a converged-but-misspecified model.


### Quantile calibration with `hzr_calibrate()`

`hzr_calibrate()` bins a continuous covariate into quantile groups and
applies the logit / Gompertz / Cox link transform, so you can eyeball
whether the covariate-outcome relationship is linear on the chosen link
scale.  It corresponds to the SAS `logit.sas` / `logitgr.sas` macros.

```{r}
#| label: calibrate-age
age_cal <- hzr_calibrate(
  x      = avc$age,
  event  = avc$dead,
  groups = 10,
  link   = "logit"
)
print(age_cal)
```

A roughly linear trend across groups means the logit link is
appropriate; strong curvature flags the need for a transform (for
example, polynomial or log).

## Nonparametric baselines

A parametric fit has to be compared against *something*. The natural
something is the nonparametric estimate of the same quantity from the
same data — Kaplan-Meier for survival, Nelson-Aalen for cumulative
hazard. The package's `hzr_kaplan()` and `hzr_nelson()` are
convenience wrappers that return richer output than `survival::survfit()`:
`hzr_kaplan()` ships KM with logit-transformed confidence limits
(more accurate in the tails than the standard Greenwood limits, which
can stray outside $[0, 1]$), interval hazard rates, density estimates,
and a restricted-mean-survival-time scalar. `hzr_nelson()` returns the
Wayne Nelson cumulative hazard estimator with log-normal confidence
limits — the right reference when you care about the integrated
intensity rather than the survival probability.

```{r}
#| label: km-baseline
km <- hzr_kaplan(time = avc$int_dead, status = avc$dead, conf_level = 0.95)
print(km)
```

```{r}
#| label: nelson-baseline
nel <- hzr_nelson(time = avc$int_dead, event = avc$dead, conf_level = 0.95)
print(nel)
```


## Bootstrap confidence intervals

The delta-method CIs `predict(se.fit = TRUE)` returns are fast and
asymptotically correct, but they lean on the Hessian-derived
parameter vcov — which can mislead when the likelihood surface is
poorly behaved near the MLE (boundary fits, near-degenerate
parameter combinations, small samples). Bootstrap resampling is the
robust alternative: refit the model on each resampled dataset,
collect the resulting parameter and prediction values, and read the
empirical 2.5th / 97.5th percentiles as the 95% CI. It's
computationally heavier but it doesn't assume the likelihood is
locally quadratic, and it picks up the right tail behavior even
when delta-method theory creaks.

`hzr_bootstrap()` wraps the resample-and-refit loop and corresponds to
the SAS `bootstrap.hazard.sas` and `bootstrap.summary.sas` macros. We
fit a base model first; bootstrap replicates will resample from the
same data and refit using these starting values.

```{r}
#| label: fit-base
# Fit the base model that bootstrapping will use
fit <- hazard(
  Surv(int_dead, dead) ~ age + status + mal + com_iv,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.20, nu = 1.0, rep(0, 4)),
  fit   = TRUE,
  control = list(maxit = 500)
)

# Prediction grid at a median-risk profile
t_grid <- seq(0.01, max(avc$int_dead) * 0.9, length.out = 100)
base_nd <- data.frame(time = t_grid, age = median(avc$age),
                      status = 2, mal = 0, com_iv = 0)

surv_point <- predict(fit, newdata = base_nd, type = "survival")
```

`hzr_bootstrap()` returns a long data frame of per-replicate
coefficient estimates (`$replicates`) and a parameter-level summary
(mean, SD, 95% percentile CI) in `$summary`. We use `n_boot = 30`
here purely so the vignette renders in reasonable time — for a real
analysis you'd want 200 or more replicates. The `set.seed(42)` call
before `hzr_bootstrap()` makes the resample sequence reproducible.

```{r}
#| label: bootstrap
set.seed(42)
boot <- hzr_bootstrap(fit, n_boot = 30)  # kept small for vignette build time
print(boot)
```

The parameter-level CIs above come directly from the bootstrap
distribution.  Prediction-level CIs (on the survival curve or hazard
profile) require pivoting `boot$replicates` to wide format and
evaluating the parametric survival formula once per replicate.  That
workflow is out of scope for this section, but the building blocks are
all in the returned object.


## Goodness-of-fit overlay

The Kaplan-Meier overlay in the prediction-visualization vignette is
the *visual* goodness-of-fit check. `hzr_gof()` is the *quantitative*
complement: it tabulates parametric predictions side by side with the
nonparametric KM estimate at each event time, and computes the
Conservation-of-Events observed-vs-expected ratio across the whole
follow-up window. The ratio compresses everything the model is doing
into a single number, which makes it the right summary statistic to
report alongside coefficient tables. (`hzr_gof()` corresponds to the
SAS `hazplot.sas` macro.)

```{r}
#| label: gof-summary
gof <- hzr_gof(fit)
print(gof)
```

The observed vs. expected ratio (printed at the bottom) is the
Conservation-of-Events check: a well-specified model recovers the event
count exactly, so a ratio close to 1 is what we want.  Ratios well
above or below 1 flag either a misspecified shape or a covariate that
isn't really linear on the log-hazard scale.


## Decile-of-risk calibration

The conservation-of-events ratio above tells you whether the model is
calibrated *on average*. That's necessary but not sufficient — a
model can hit the right total event count while systematically
over-predicting risk for one part of the cohort and under-predicting
for another, with the errors cancelling in aggregate.
`hzr_deciles()` is the check that catches that failure mode: it
partitions patients into deciles of predicted risk at a chosen time
point and compares observed with expected event counts within each
decile, returning a chi-square goodness-of-fit test. A
well-calibrated model has observed-vs-expected close in every decile;
a model that's globally calibrated but locally biased will show large
residuals in the extreme deciles. (Corresponds to the SAS
`deciles.hazard.sas` macro.)

```{r}
#| label: deciles
deciles <- hzr_deciles(fit, time = 120, groups = 10)
print(deciles)
```

The chi-square test statistic and p-value summarise whether the
observed decile-by-decile event counts are consistent with what the
parametric model predicts.  A good visual check is to plot the two
columns side by side:

```{r}
#| label: fig-deciles
#| fig-cap: "Observed vs. expected mortality by decile of risk"
#| fig-width: 7
#| fig-height: 4.5
decile_df <- as.data.frame(deciles)
cal_long <- rbind(
  data.frame(group = decile_df$group,
             rate  = decile_df$observed_rate,
             Series = "Observed"),
  data.frame(group = decile_df$group,
             rate  = decile_df$expected_rate,
             Series = "Expected")
)
ggplot(cal_long, aes(group, rate, fill = Series)) +
  geom_col(position = position_dodge(width = 0.7), alpha = 0.8) +
  scale_fill_manual(values = c("Observed" = "#56B4E9",
                                "Expected" = "#E69F00")) +
  scale_x_continuous(breaks = seq_len(nrow(decile_df))) +
  scale_y_continuous(labels = scales::percent_format(accuracy = 1)) +
  labs(x = "Risk decile (1 = lowest)", y = "Mortality rate", fill = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```


## Sensitivity analysis

Coefficient estimates tell you the *direction and magnitude* of a
covariate's effect on the hazard scale, but they don't directly answer
the clinical question of *what survival difference this implies* for a
real patient. Sensitivity analysis closes that gap. You define two
or more covariate profiles — typically a reference profile and one or
more high-risk profiles — score each through `predict()`, and plot
the resulting survival curves on shared axes. The horizontal and
vertical gaps between curves translate the model's coefficients into
the clinically meaningful currency of survival probability differences
and median-survival shifts. It's also a stress test: if your high-risk
profile produces only a small survival gap, either the risk factors
aren't doing much in this cohort, or the model is missing the
mechanism that makes them matter.

```{r}
#| label: sensitivity
# Define reference and high-risk profiles
ref_profile <- data.frame(
  age    = median(avc$age),
  status = 2,
  mal    = 0,
  com_iv = 0
)

high_risk <- data.frame(
  age    = quantile(avc$age, 0.90),
  status = 4,
  mal    = 1,
  com_iv = 1
)
```

```{r}
#| label: fig-sensitivity
#| fig-cap: "Sensitivity analysis: reference vs. high-risk profile"
#| fig-width: 7
#| fig-height: 4.5
sens_curves <- do.call(rbind, lapply(
  list("Reference" = ref_profile, "High risk" = high_risk),
  function(p) {
    nd <- p[rep(1, length(t_grid)), ]
    nd$time <- t_grid
    data.frame(time = t_grid,
               survival = predict(fit, newdata = nd, type = "survival") * 100,
               Profile = deparse(substitute(p)))
  }
))
# Fix profile labels
sens_curves$Profile <- rep(c("Reference", "High risk"),
                           each = length(t_grid))
sens_curves$Profile <- factor(sens_curves$Profile,
                              levels = c("Reference", "High risk"))

ggplot(sens_curves, aes(time, survival, colour = Profile)) +
  geom_line(linewidth = 0.9) +
  scale_colour_manual(values = c("Reference" = "#009E73",
                                 "High risk" = "#D55E00")) +
  scale_y_continuous(limits = c(0, 100)) +
  labs(x = "Months after AVC repair", y = "Freedom from death (%)",
       colour = NULL) +
  theme_minimal() +
  theme(legend.position = "bottom")
```

The gap between the curves quantifies the clinical impact of the risk factors.
The sensitivity analysis is most informative when combined with bootstrap
confidence intervals on each curve, showing whether the difference is
statistically meaningful.


## Analysis workflow summary

The pieces in this vignette aren't independent diagnostics — they
chain together into a single analytical workflow that goes from raw
covariates to a defensible, uncertainty-quantified, well-calibrated
fitted model. The complete sequence, with the SAS HAZARD
correspondences each piece descends from:

1. **Exploratory screening** (`glm`, `hzr_calibrate()`) --- identify
   covariate transformations and functional forms
2. **Nonparametric baselines** (`hzr_kaplan()`, `hzr_nelson()`) ---
   reference KM / Nelson cumulative hazard estimators
3. **Fit hazard model** (`hazard()`) --- parametric shape + covariates
4. **Predict & visualize** (`predict()`) --- survival, hazard, risk profiles
5. **Goodness-of-fit overlay** (`hzr_gof()`) --- parametric vs. KM +
   Conservation-of-Events check
6. **Bootstrap CIs** (`hzr_bootstrap()`) --- uncertainty quantification
   via resampling
7. **Sensitivity analysis** (`predict()` on covariate profiles) ---
   compare scenarios across risk factors
8. **Decile calibration** (`hzr_deciles()`) --- chi-square observed
   vs. expected by risk decile

See `vignette("getting-started")` for the minimal workflow and
`vignette("fitting-hazard-models")` for the model-fitting details.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_primitives.R
\name{hzr_clamp_prob}
\alias{hzr_clamp_prob}
\title{Clamp probabilities away from 0 and 1}
\usage{
hzr_clamp_prob(p, eps = 1e-12)
}
\arguments{
\item{p}{Numeric vector of probabilities.}

\item{eps}{Small positive tolerance.}
}
\value{
Numeric vector bounded to \verb{[eps, 1 - eps]}.
}
\description{
Clamp probabilities away from 0 and 1
}
\examples{
hzr_clamp_prob(c(0, 0.5, 1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_primitives.R
\name{hzr_clamp_prob}
\alias{hzr_clamp_prob}
\title{Clamp probabilities away from 0 and 1}
\usage{
hzr_clamp_prob(p, eps = 1e-12)
}
\arguments{
\item{p}{Numeric vector of probabilities.}

\item{eps}{Small positive tolerance in \eqn{(0, 0.5)}; default \code{1e-12}.}
}
\value{
Numeric vector bounded to \verb{[eps, 1 - eps]}.
}
\description{
Bounds a probability vector to \eqn{[\epsilon,\, 1 - \epsilon]} so that
downstream \eqn{\log p} or \eqn{\log(1 - p)} evaluations stay finite.  Used
throughout the likelihood and prediction code to guard survival and CDF
values against exact 0 or 1.
}
\examples{
hzr_clamp_prob(c(0, 0.5, 1))
}
\seealso{
\code{\link[=hzr_log1pexp]{hzr_log1pexp()}} and \code{\link[=hzr_log1mexp]{hzr_log1mexp()}} for the companion
stable-logarithm primitives.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_cl_from_se}
\alias{.hzr_predict_cl_from_se}
\title{Apply a scale transform + back-transform for CLs}
\usage{
.hzr_predict_cl_from_se(fit, se_nat, level, scale)
}
\arguments{
\item{fit}{Point estimate (numeric vector length n).}

\item{se_nat}{SE on the natural scale (numeric vector length n).}

\item{level}{Confidence level.}

\item{scale}{One of "natural", "log", "loglog_survival".}
}
\value{
data.frame with columns fit, se.fit, lower, upper.
}
\description{
Apply a scale transform + back-transform for CLs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{.hzr_validate_phases}
\alias{.hzr_validate_phases}
\title{Validate a list of phase specifications}
\usage{
.hzr_validate_phases(phases)
}
\arguments{
\item{phases}{A list of \code{hzr_phase} objects.}
}
\value{
The validated (and possibly auto-named) list, invisibly.
}
\description{
Checks that \code{phases} is a non-empty named list of \code{hzr_phase} objects.
Auto-names unnamed phases as \code{phase_1}, \code{phase_2}, etc.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{hazard}
\alias{hazard}
\title{Build and optionally fit a hazard model}
\usage{
hazard(
  formula = NULL,
  data = NULL,
  time = NULL,
  status = NULL,
  time_lower = NULL,
  time_upper = NULL,
  x = NULL,
  time_windows = NULL,
  theta = NULL,
  dist = "weibull",
  phases = NULL,
  fit = FALSE,
  weights = NULL,
  control = list(),
  ...
)
}
\arguments{
\item{formula}{Optional formula of the form \code{Surv(time, status) ~ predictors}.
When provided, overrides direct time/status/x arguments and extracts from data.
Example: \code{hazard(Surv(time, status) ~ x1 + x2, data = df, dist = "weibull", fit = TRUE)}.}

\item{data}{Optional data frame containing variables referenced in formula.}

\item{time}{Numeric follow-up time vector.}

\item{status}{Numeric or logical event indicator vector.}

\item{time_lower}{Optional numeric lower bound vector for censoring intervals.
Used when \code{status == 2} (interval-censored); defaults to \code{time} if NULL.}

\item{time_upper}{Optional numeric upper bound vector for censoring intervals.
Used when \code{status \%in\% c(-1, 2)}; defaults to \code{time} if NULL.}

\item{x}{Optional design matrix (or data frame coercible to matrix).}

\item{time_windows}{Optional numeric vector of strictly positive cut points for
piecewise time-varying coefficients. When provided, each predictor column in
\code{x} is expanded into one column per time window so each window gets its own
coefficient.}

\item{theta}{Optional numeric coefficient vector (starting values for optimization).}

\item{dist}{Character baseline distribution label (default "weibull").
Use \code{"multiphase"} for N-phase additive hazard models (requires \code{phases}).}

\item{phases}{Optional named list of \code{\link[=hzr_phase]{hzr_phase()}} objects specifying the
phases for a multiphase model (\code{dist = "multiphase"}).  See Examples.}

\item{fit}{Logical; if TRUE, fit the model via maximum likelihood (default FALSE).}

\item{weights}{Optional numeric vector of observation weights (non-negative).
Each observation's log-likelihood contribution is multiplied by its weight.
Use for severity-weighted repeated events. Default \code{NULL} (unit weights).
Implements the SAS \code{WEIGHT} statement.}

\item{control}{Named list of control options (see Details).}

\item{...}{Additional named arguments retained for parity with legacy calling
conventions.}
}
\value{
An object of class \code{hazard}, a named list with components:
\code{call} (the matched call),
\code{spec} (model specification: \code{dist}, \code{control},
\code{time_windows}, \code{phases}),
\code{data} (input data: \code{time}, \code{status}, \code{x},
\code{weights}, etc.),
\code{fit} (optimisation results: \code{theta}, \code{objective},
\code{converged}, \code{se}, \code{vcov}, \code{counts}, \code{message};
all \code{NULL} when \code{fit = FALSE}),
and \code{engine} (implementation tag, \code{"native-r-m2"}).
}
\description{
Creates a \code{hazard} object and optionally fits it via maximum likelihood.
This mirrors the argument-oriented workflow of the legacy HAZARD C/SAS
implementation: supply starting values in \code{theta} and the function will
optimize to produce fitted estimates.
}
\details{
Control parameters:
\itemize{
\item \code{maxit}: Maximum iterations (default 1000)
\item \code{reltol}: Relative parameter change tolerance (default 1e-5)
\item \code{abstol}: Absolute gradient norm tolerance (default 1e-6)
\item \code{method}: Optimization method: "bfgs" or "nm" (default "bfgs")
\item \code{condition}: Condition number control (default 14)
\item \code{nocov}, \code{nocor}: Suppress covariance/correlation output (legacy; no-op in M2)
}

Censoring status coding:
\itemize{
\item 1: Exact event at time
\item 0: Right-censored at time
\item -1: Left-censored with upper bound at time_upper \(or time\)
\item 2: Interval-censored in the interval \(time_lower, time_upper\)
}

Time-varying coefficients:
\itemize{
\item If \code{time_windows} is supplied, predictors are expanded to piecewise window
interactions so each window has its own coefficient vector.
\item This is implemented as design-matrix expansion, so the existing likelihood
engines remain unchanged.
}
}
\examples{
# -- Univariable Weibull ----------------------------------------------
set.seed(1)
time   <- rexp(50, rate = 0.3)
status <- sample(0:1, 50, replace = TRUE, prob = c(0.3, 0.7))
fit <- hazard(time = time, status = status,
              theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
summary(fit)

# -- Formula interface with covariates --------------------------------
set.seed(1001)
n   <- 180
dat <- data.frame(
  time   = rexp(n, rate = 0.35) + 0.05,
  status = rbinom(n, size = 1, prob = 0.6),
  age    = rnorm(n, mean = 62, sd = 11),
  nyha   = sample(1:4, n, replace = TRUE),
  shock  = rbinom(n, size = 1, prob = 0.18)
)

fit2 <- hazard(
  survival::Surv(time, status) ~ age + nyha + shock,
  data    = dat,
  theta   = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
  dist    = "weibull",
  fit     = TRUE,
  control = list(maxit = 300)
)
summary(fit2)

\donttest{
# -- Parametric survival with Kaplan-Meier overlay -----------------
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)

  # Parametric curve on a fine grid at median covariate profile
  t_grid   <- seq(0.05, max(dat$time), length.out = 80)
  curve_df <- data.frame(
    time = t_grid, age = median(dat$age), nyha = 2, shock = 0
  )
  curve_df$survival <- predict(fit2, newdata = curve_df,
                               type = "survival") * 100

  # Kaplan-Meier empirical overlay
  km    <- survival::survfit(survival::Surv(time, status) ~ 1, data = dat)
  km_df <- data.frame(time = km$time, survival = km$surv * 100)

  ggplot() +
    geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier")) +
    geom_line(data = curve_df, aes(time, survival,
                                   colour = "Parametric (Weibull)")) +
    scale_colour_manual(
      values = c("Parametric (Weibull)" = "#0072B2",
                 "Kaplan-Meier"         = "#D55E00")
    ) +
    scale_y_continuous(limits = c(0, 100)) +
    labs(x = "Months after surgery", y = "Freedom from death (\%)",
         colour = NULL) +
    theme_minimal() +
    theme(legend.position = "bottom")
}
}

\donttest{
# -- Multiphase model (two phases) ---------------------------------
fit_mp <- hazard(
  survival::Surv(time, status) ~ 1,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                       fixed = "shapes"),
    late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
                       fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)
summary(fit_mp)

# -- Per-phase decomposed cumulative hazard ------------------------
if (requireNamespace("ggplot2", quietly = TRUE)) {
  t_grid <- seq(0.01, max(dat$time), length.out = 100)
  decomp <- predict(fit_mp, newdata = data.frame(time = t_grid),
                    type = "cumulative_hazard", decompose = TRUE)

  df_long <- data.frame(
    time = rep(decomp$time, 3),
    cumhaz = c(decomp$total, decomp$early, decomp$late),
    component = rep(c("Total", "Early (cdf)", "Late (cdf)"),
                    each = nrow(decomp))
  )
  df_long$component <- factor(df_long$component,
    levels = c("Total", "Early (cdf)", "Late (cdf)"))

  ggplot2::ggplot(df_long,
    ggplot2::aes(x = time, y = cumhaz, colour = component,
                 linewidth = component)) +
    ggplot2::geom_line() +
    ggplot2::scale_colour_manual(values = c(
      "Total" = "black", "Early (cdf)" = "#0072B2",
      "Late (cdf)" = "#D55E00"
    )) +
    ggplot2::scale_linewidth_manual(values = c(
      "Total" = 1.2, "Early (cdf)" = 0.6, "Late (cdf)" = 0.6
    )) +
    ggplot2::labs(
      x = "Time", y = "Cumulative hazard H(t)",
      colour = NULL, linewidth = NULL,
      title = "Multiphase decomposition: early + late"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(legend.position = "bottom")
}
}

}
\references{
Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. \emph{J Am Stat Assoc.} 1986;81(395):615--624.
\doi{10.1080/01621459.1986.10478314}

Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
Probability of atrial fibrillation after ablation: Using a parametric
nonlinear temporal decomposition mixed effects model. \emph{Stat Methods Med Res.}
2018;27(1):126--141. \doi{10.1177/0962280215623583}
}
\seealso{
\code{\link[=predict.hazard]{predict.hazard()}} for survival/cumulative-hazard predictions,
\code{\link[=summary.hazard]{summary.hazard()}} for model summaries,
\code{\link[=hzr_phase]{hzr_phase()}} for specifying multiphase temporal shapes.

Vignettes with worked examples:
\code{vignette("fitting-hazard-models")} --- single-phase through multiphase fitting,
\code{vignette("prediction-visualization")} --- prediction types and decomposed hazard plots,
\code{vignette("inference-diagnostics")} --- bootstrap CIs and model diagnostics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{hazard}
\alias{hazard}
\title{Build and optionally fit a hazard model}
\usage{
hazard(
  formula = NULL,
  data = NULL,
  time = NULL,
  status = NULL,
  time_lower = NULL,
  time_upper = NULL,
  x = NULL,
  time_windows = NULL,
  theta = NULL,
  dist = "weibull",
  phases = NULL,
  fit = FALSE,
  weights = NULL,
  control = list(),
  ...
)
}
\arguments{
\item{formula}{Optional formula of the form \code{Surv(time, status) ~ predictors}.
When provided, overrides direct time/status/x arguments and extracts from data.
Example: \code{hazard(Surv(time, status) ~ x1 + x2, data = df, dist = "weibull", fit = TRUE)}.}

\item{data}{Optional data frame containing variables referenced in formula.}

\item{time}{Numeric follow-up time vector.}

\item{status}{Numeric or logical event indicator vector.}

\item{time_lower}{Optional numeric lower bound vector for censoring intervals.
Used when \code{status == 2} (interval-censored); defaults to \code{time} if NULL.}

\item{time_upper}{Optional numeric upper bound vector for censoring intervals.
Used when \code{status \%in\% c(-1, 2)}; defaults to \code{time} if NULL.}

\item{x}{Optional design matrix (or data frame coercible to matrix).}

\item{time_windows}{Optional numeric vector of strictly positive cut points for
piecewise time-varying coefficients. When provided, each predictor column in
\code{x} is expanded into one column per time window so each window gets its own
coefficient.}

\item{theta}{Optional numeric coefficient vector (starting values for optimization).}

\item{dist}{Character baseline distribution label (default \code{"weibull"}).
One of \code{"weibull"}, \code{"exponential"}, \code{"loglogistic"}, \code{"lognormal"}, or
\code{"multiphase"}.  The single-distribution families differ in the \emph{shape} the
hazard traces over time; \code{"multiphase"} builds an additive N-phase hazard
and requires \code{phases}.  See the \strong{Baseline distributions} section for what
each means and when to use it.}

\item{phases}{Optional named list of \code{\link[=hzr_phase]{hzr_phase()}} objects specifying the
phases for a multiphase model (\code{dist = "multiphase"}).  See Examples.}

\item{fit}{Logical; if TRUE, fit the model via maximum likelihood (default FALSE).}

\item{weights}{Optional numeric vector of observation weights (non-negative).
Each observation's log-likelihood contribution is multiplied by its weight.
Use for severity-weighted repeated events. Default \code{NULL} (unit weights).
Implements the SAS \code{WEIGHT} statement.}

\item{control}{Named list of control options (see Details).}

\item{...}{Additional named arguments retained for parity with legacy calling
conventions.}
}
\value{
An object of class \code{hazard}, a named list with components:
\code{call} (the matched call),
\code{spec} (model specification: \code{dist}, \code{control},
\code{time_windows}, \code{phases}),
\code{data} (input data: \code{time}, \code{status}, \code{x},
\code{weights}, etc.),
\code{fit} (optimisation results: \code{theta}, \code{objective},
\code{converged}, \code{se}, \code{vcov}, \code{counts}, \code{message};
all \code{NULL} when \code{fit = FALSE}),
and \code{engine} (implementation tag, \code{"native-r-m2"}).
}
\description{
Creates a \code{hazard} object and optionally fits it via maximum likelihood.
This mirrors the argument-oriented workflow of the legacy HAZARD C/SAS
implementation: supply starting values in \code{theta} and the function will
optimize to produce fitted estimates.
}
\details{
Control parameters:
\itemize{
\item \code{maxit}: Maximum iterations (default 1000)
\item \code{n_starts}: Number of optimization starts for multiphase fits (default 5).
Each start after the first adds random noise to the initial values, drawn
from the ambient RNG stream; call \code{set.seed()} before fitting for
reproducible results.
\item \code{reltol}: Relative parameter change tolerance (default 1e-5)
\item \code{abstol}: Absolute gradient norm tolerance (default 1e-6)
\item \code{method}: Optimization method: "bfgs" or "nm" (default "bfgs")
\item \code{condition}: Condition number control (default 14)
\item \code{nocov}, \code{nocor}: Suppress covariance/correlation output (legacy; no-op in M2)
}

Censoring status coding:
\itemize{
\item 1: Exact event at time
\item 0: Right-censored at time
\item -1: Left-censored with upper bound at time_upper \(or time\)
\item 2: Interval-censored in the interval \(time_lower, time_upper\)
}

Time-varying coefficients:
\itemize{
\item If \code{time_windows} is supplied, predictors are expanded to piecewise window
interactions so each window has its own coefficient vector.
\item This is implemented as design-matrix expansion, so the existing likelihood
engines remain unchanged.
}
}
\section{The fitted model}{


For \code{dist = "multiphase"} the cumulative hazard, instantaneous hazard, and
survival are

\deqn{H(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \, \Phi_j(t),
      \qquad
      h(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \, \varphi_j(t),
      \qquad
      S(t \mid \mathbf{x}) = \exp\!\bigl(-H(t \mid \mathbf{x})\bigr)}

where \eqn{\mu_j(\mathbf{x}) = \exp(\alpha_j + \mathbf{x}_j^\top
\boldsymbol{\beta}_j)} and the temporal shapes \eqn{\Phi_j}, \eqn{\varphi_j}
are set by each phase's \code{type} (see \code{\link[=hzr_phase]{hzr_phase()}}).  The
proportional-hazards single-phase families (\code{"weibull"}, \code{"exponential"})
are the special case \eqn{J = 1}, with covariates acting multiplicatively on
one temporal shape.  The \code{"loglogistic"} (proportional-odds) and
\code{"lognormal"} (accelerated-failure-time) families place covariates
differently --- on the odds of failure and the log-time location,
respectively --- so they are separate parameterizations, not special cases
of this additive form.  Parameters are estimated
on an unconstrained internal scale (e.g. \eqn{\log\mu}, \eqn{\log t_{1/2}})
and transformed back for reporting; see
\code{vignette("mf-mathematical-foundations")}.
}

\section{Baseline distributions}{


The \code{dist} argument selects the parametric form of the baseline hazard.  The
four single-distribution families differ in the \emph{shape} the
hazard traces over follow-up; choose by what the risk is expected to do over
time.  \code{"multiphase"} is the general additive model that lets several such
shapes coexist.

\describe{
\item{\code{"weibull"} --- monotone rising or falling hazard (default)}{The
workhorse parametric model: \eqn{H(t \mid \mathbf{x}) = (\mu t)^\nu
    \exp(\eta)}, with hazard \eqn{h \propto t^{\nu - 1}}.  The single shape
\eqn{\nu} makes risk increase over time (\eqn{\nu > 1}), decrease
(\eqn{\nu < 1}), or stay flat (\eqn{\nu = 1}).  Use it as the default when
a single monotone trend describes the hazard.}
\item{\code{"exponential"} --- constant hazard}{The memoryless special case
\eqn{\nu = 1}: a time-invariant baseline rate, \eqn{H(t \mid \mathbf{x}) =
    \mu t \exp(\eta)}.  Use it when the event rate does not change with
follow-up time (the constant background risk also appears as the
\code{"constant"} phase in a multiphase model).}
\item{\code{"loglogistic"} --- unimodal (rise-then-fall) hazard}{A log-logistic
proportional-odds form (covariates act multiplicatively on the odds of
failure, \eqn{\exp(\eta)}, not as an AFT time shift) whose hazard rises to
a single peak and then declines when the shape exceeds 1 (and is monotone
decreasing otherwise), with heavier tails than the log-normal.  Use it
when risk climbs to an early peak and then eases off.}
\item{\code{"lognormal"} --- early-peaking, resolving hazard}{An
accelerated-failure-time form in which \eqn{\log} time is Gaussian; the
hazard rises to an early peak and then decays toward zero.  Use it for
risk that is concentrated early and resolves over time.}
\item{\code{"multiphase"} --- additive N-phase hazard}{Sums several phase shapes
into one model, \eqn{H = \sum_j \mu_j(\mathbf{x}) \Phi_j(t)}, so the
overall hazard can fall, level off, and rise again within one fit.
Requires \code{phases}; see \code{\link[=hzr_phase]{hzr_phase()}} for the available phase shapes.  This
is the form that reproduces the classic C/SAS HAZARD models.}
}
}

\examples{
# -- Univariable Weibull ----------------------------------------------
set.seed(1)
time   <- rexp(50, rate = 0.3)
status <- sample(0:1, 50, replace = TRUE, prob = c(0.3, 0.7))
fit <- hazard(time = time, status = status,
              theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
summary(fit)

# -- Formula interface with covariates --------------------------------
set.seed(1001)
n   <- 180
dat <- data.frame(
  time   = rexp(n, rate = 0.35) + 0.05,
  status = rbinom(n, size = 1, prob = 0.6),
  age    = rnorm(n, mean = 62, sd = 11),
  nyha   = sample(1:4, n, replace = TRUE),
  shock  = rbinom(n, size = 1, prob = 0.18)
)

fit2 <- hazard(
  survival::Surv(time, status) ~ age + nyha + shock,
  data    = dat,
  theta   = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
  dist    = "weibull",
  fit     = TRUE,
  control = list(maxit = 300)
)
summary(fit2)

\donttest{
# -- Parametric survival with Kaplan-Meier overlay -----------------
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)

  # Parametric curve on a fine grid at median covariate profile
  t_grid   <- seq(0.05, max(dat$time), length.out = 80)
  curve_df <- data.frame(
    time = t_grid, age = median(dat$age), nyha = 2, shock = 0
  )
  curve_df$survival <- predict(fit2, newdata = curve_df,
                               type = "survival") * 100

  # Kaplan-Meier empirical overlay
  km    <- survival::survfit(survival::Surv(time, status) ~ 1, data = dat)
  km_df <- data.frame(time = km$time, survival = km$surv * 100)

  ggplot() +
    geom_step(data = km_df, aes(time, survival, colour = "Kaplan-Meier")) +
    geom_line(data = curve_df, aes(time, survival,
                                   colour = "Parametric (Weibull)")) +
    scale_colour_manual(
      values = c("Parametric (Weibull)" = "#0072B2",
                 "Kaplan-Meier"         = "#D55E00")
    ) +
    scale_y_continuous(limits = c(0, 100)) +
    labs(x = "Months after surgery", y = "Freedom from death (\%)",
         colour = NULL) +
    theme_minimal() +
    theme(legend.position = "bottom")
}
}

\donttest{
# -- Multiphase model (two phases) ---------------------------------
fit_mp <- hazard(
  survival::Surv(time, status) ~ 1,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                       fixed = "shapes"),
    late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
                       fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)
summary(fit_mp)

# -- Per-phase decomposed cumulative hazard ------------------------
if (requireNamespace("ggplot2", quietly = TRUE)) {
  t_grid <- seq(0.01, max(dat$time), length.out = 100)
  decomp <- predict(fit_mp, newdata = data.frame(time = t_grid),
                    type = "cumulative_hazard", decompose = TRUE)

  df_long <- data.frame(
    time = rep(decomp$time, 3),
    cumhaz = c(decomp$total, decomp$early, decomp$late),
    component = rep(c("Total", "Early (cdf)", "Late (cdf)"),
                    each = nrow(decomp))
  )
  df_long$component <- factor(df_long$component,
    levels = c("Total", "Early (cdf)", "Late (cdf)"))

  ggplot2::ggplot(df_long,
    ggplot2::aes(x = time, y = cumhaz, colour = component,
                 linewidth = component)) +
    ggplot2::geom_line() +
    ggplot2::scale_colour_manual(values = c(
      "Total" = "black", "Early (cdf)" = "#0072B2",
      "Late (cdf)" = "#D55E00"
    )) +
    ggplot2::scale_linewidth_manual(values = c(
      "Total" = 1.2, "Early (cdf)" = 0.6, "Late (cdf)" = 0.6
    )) +
    ggplot2::labs(
      x = "Time", y = "Cumulative hazard H(t)",
      colour = NULL, linewidth = NULL,
      title = "Multiphase decomposition: early + late"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(legend.position = "bottom")
}
}

}
\references{
Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. \emph{J Am Stat Assoc.} 1986;81(395):615--624.
\doi{10.1080/01621459.1986.10478314}

Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
Probability of atrial fibrillation after ablation: Using a parametric
nonlinear temporal decomposition mixed effects model. \emph{Stat Methods Med Res.}
2018;27(1):126--141. \doi{10.1177/0962280215623583}
}
\seealso{
\code{\link[=predict.hazard]{predict.hazard()}} for survival/cumulative-hazard predictions,
\code{\link[=summary.hazard]{summary.hazard()}} for model summaries,
\code{\link[=hzr_phase]{hzr_phase()}} for specifying multiphase temporal shapes.

Vignettes with worked examples:
\code{vignette("fitting-hazard-models")} --- single-phase through multiphase fitting,
\code{vignette("prediction-visualization")} --- prediction types and decomposed hazard plots,
\code{vignette("inference-diagnostics")} --- bootstrap CIs and model diagnostics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_deciles}
\alias{hzr_deciles}
\title{Decile-of-risk calibration}
\usage{
hzr_deciles(object, time, groups = 10L, status = NULL, event_time = NULL)
}
\arguments{
\item{object}{A fitted \code{hazard} object (with \code{fit = TRUE}).}

\item{time}{Numeric scalar: the time point at which to evaluate
predicted survival / cumulative hazard.  For example, \code{time = 12}
evaluates 12-month predictions.}

\item{groups}{Integer: number of risk groups (default 10 for deciles).}

\item{status}{Optional numeric vector of event indicators (1 = event,
0 = censored).
If \code{NULL} (default), extracted from the fitted object's stored data.}

\item{event_time}{Optional numeric vector of observed event/censoring
times.  If \code{NULL} (default), extracted from the fitted object.}
}
\value{
A data frame with one row per risk group and columns:
\describe{
\item{group}{Integer group label (1 = lowest risk).}
\item{n}{Number of observations in the group.}
\item{events}{Observed event count by the requested horizon
(event indicator = 1 and event time <= \code{time}).}
\item{expected}{Expected event count by the requested horizon,
computed as the sum of individual event probabilities
(\eqn{1 - S(time)}).}
\item{observed_rate}{Observed event rate (events / n).}
\item{expected_rate}{Expected event rate (expected / n).}
\item{chi_sq}{Chi-square contribution: (events - expected)^2 /
expected.}
\item{p_value}{Upper-tail p-value from the chi-square test for
this group (1 df).}
\item{mean_survival}{Mean predicted survival probability in the
group.}
\item{mean_cumhaz}{Mean predicted cumulative hazard in the group.}
}

An attribute \code{"overall"} is attached with the overall chi-square
statistic, degrees of freedom, and p-value.
}
\description{
Partition observations into groups (default 10) by predicted risk and
compare observed vs. expected event counts in each group.  Good
calibration means the two track each other across the risk spectrum.
}
\details{
This implements the workflow of the SAS \code{deciles.hazard.sas} macro.
Patients are ranked by predicted cumulative hazard at a specified time
point, grouped into quantile bins, and each bin is tested with a
chi-square goodness-of-fit statistic. Subjects censored before the
requested horizon are excluded from the observed-vs-expected comparison.
}
\examples{
\donttest{
data(avc)
avc <- na.omit(avc)
fit <- hazard(
  survival::Surv(int_dead, dead) ~ age + mal,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
  fit   = TRUE
)
cal <- hzr_deciles(fit, time = 120)
print(cal)
}

}
\seealso{
\code{\link[=predict.hazard]{predict.hazard()}} for the prediction types used internally.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_deciles}
\alias{hzr_deciles}
\title{Decile-of-risk calibration}
\usage{
hzr_deciles(object, time, groups = 10L, status = NULL, event_time = NULL)
}
\arguments{
\item{object}{A fitted \code{hazard} object (with \code{fit = TRUE}).}

\item{time}{Numeric scalar: the horizon at which predicted survival is used
to \strong{rank subjects into risk groups} (e.g. \code{time = 12} ranks by 12-month
predicted survival). It does not restrict the event/expected counts, which
are accumulated over each subject's full follow-up.}

\item{groups}{Integer: number of risk groups (default 10 for deciles).}

\item{status}{Optional numeric vector of event indicators (1 = event,
0 = censored).
If \code{NULL} (default), extracted from the fitted object's stored data.}

\item{event_time}{Optional numeric vector of observed event/censoring
times.  If \code{NULL} (default), extracted from the fitted object.}
}
\value{
A data frame with one row per risk group and columns:
\describe{
\item{group}{Integer group label (1 = lowest risk, ranked by predicted
survival at \code{time}).}
\item{n}{Number of observations in the group.}
\item{events}{Observed event count in the group (all events over
follow-up).}
\item{expected}{Expected event count: the sum of each subject's predicted
cumulative hazard at its own follow-up time.}
\item{observed_rate}{Observed event rate (events / n).}
\item{expected_rate}{Expected event rate (expected / n).}
\item{chi_sq}{Chi-square contribution: (events - expected)^2 /
expected.}
\item{p_value}{Upper-tail p-value from the chi-square test for
this group (1 df).}
\item{mean_survival}{Mean predicted survival probability at the horizon
in the group.}
\item{mean_cumhaz}{Mean predicted cumulative hazard at follow-up in the
group.}
}

An attribute \code{"overall"} is attached with the overall chi-square
statistic, degrees of freedom, and p-value.
}
\description{
Partition observations into groups (default 10) by predicted risk and
compare observed vs. expected event counts in each group.  Good
calibration means the two track each other across the risk spectrum.
}
\details{
This reproduces the SAS \code{deciles.hazard.sas} macro. \strong{All} subjects are
ranked by predicted survival at the horizon \code{time} and split into equal-sized
risk groups. Within each group the \strong{expected} event count is the sum of
each subject's predicted cumulative hazard at its \emph{own} follow-up time, and
the \strong{observed} count is its number of events; under conservation of events
the group totals sum to the total observed events. The horizon therefore only
stratifies subjects into risk groups -- it does not restrict or exclude any
subject, and the expected/observed totals are independent of it.
}
\examples{
\donttest{
data(avc)
avc <- na.omit(avc)
fit <- hazard(
  survival::Surv(int_dead, dead) ~ age + mal,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
  fit   = TRUE
)
cal <- hzr_deciles(fit, time = 120)
print(cal)
}

}
\seealso{
\code{\link[=predict.hazard]{predict.hazard()}} for the prediction types used internally.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_optim_multiphase}
\alias{.hzr_optim_multiphase}
\title{Fit a multiphase additive hazard model via maximum likelihood}
\usage{
.hzr_optim_multiphase(
  time,
  status,
  time_lower = NULL,
  time_upper = NULL,
  x = NULL,
  theta_start = NULL,
  weights = NULL,
  control = list(),
  phases,
  formula_global = NULL,
  data = NULL
)
}
\arguments{
\item{time}{Numeric vector of follow-up times.}

\item{status}{Numeric event indicator vector.}

\item{time_lower}{Optional lower bounds for interval censoring.}

\item{time_upper}{Optional upper bounds for left/interval censoring.}

\item{x}{Global design matrix (n x p) or NULL.}

\item{theta_start}{Starting parameter vector (full internal scale).
If NULL, assembled automatically from phase specs.}

\item{control}{Named list of control options.}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{formula_global}{The global formula (used when phases have no
phase-specific formula).}

\item{data}{Data frame containing covariates (needed for phase-specific
formula evaluation).}
}
\value{
List with par (internal scale), value, convergence, vcov, etc.
}
\description{
Assembles starting values from phase specifications, resolves per-phase
design matrices, and delegates to \code{.hzr_optim_generic()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stepwise.R
\name{stepwise_trace}
\alias{stepwise_trace}
\title{Extract the captured console trace from an \code{hzr_stepwise} fit}
\usage{
stepwise_trace(fit)
}
\arguments{
\item{fit}{An \code{hzr_stepwise} object.}
}
\value{
Character vector, one element per console line.
}
\description{
Every run of \code{\link[=hzr_stepwise]{hzr_stepwise()}} records the header, per-step lines,
and final summary regardless of the \code{trace} flag.  This accessor
returns the full character vector for display or logging.
}
\examples{
data(avc)
avc <- na.omit(avc)
base <- hazard(survival::Surv(int_dead, dead) ~ age,
               data = avc, dist = "weibull", fit = TRUE)
\donttest{
sw <- hzr_stepwise(base, scope = ~ age + nyha,
                   data = avc, direction = "forward",
                   control = list(n_starts = 1))
cat(stepwise_trace(sw), sep = "\n")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{print.hazard}
\alias{print.hazard}
\title{Print method for fitted hazard models}
\usage{
\method{print}{hazard}(x, ...)
}
\arguments{
\item{x}{A \code{hazard} object returned by \code{\link[=hazard]{hazard()}}.}

\item{...}{Additional arguments (ignored).}
}
\value{
\code{x}, invisibly.
}
\description{
Compact one-block summary of a fitted \code{hazard} object: sample size,
number of predictors, distribution, theta vector, and log-likelihood.
S3 dispatch only -- users call \code{print(fit)} rather than invoking this
directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stepwise.R
\name{stepwise_trace}
\alias{stepwise_trace}
\title{Extract the captured console trace from an \code{hzr_stepwise} fit}
\usage{
stepwise_trace(fit)
}
\arguments{
\item{fit}{An \code{hzr_stepwise} object.}
}
\value{
Character vector, one element per console line.
}
\description{
Every run of \code{\link[=hzr_stepwise]{hzr_stepwise()}} records the header, per-step lines,
and final summary regardless of the \code{trace} flag.  This accessor
returns the full character vector for display or logging.
}
\examples{
data(avc)
avc <- na.omit(avc)
base <- hazard(survival::Surv(int_dead, dead) ~ age,
               data = avc, dist = "weibull", fit = TRUE)
\donttest{
sw <- hzr_stepwise(base, scope = ~ age + nyha,
                   data = avc, direction = "forward",
                   control = list(n_starts = 1))
cat(stepwise_trace(sw), sep = "\n")
}
}
\seealso{
\code{\link[=hzr_stepwise]{hzr_stepwise()}}, which produces the object this accessor reads.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{print.hzr_gof}
\alias{print.hzr_gof}
\title{Print method for hzr_gof}
\usage{
\method{print}{hzr_gof}(x, digits = 3, ...)
}
\arguments{
\item{x}{An \code{hzr_gof} object.}

\item{digits}{Number of decimal places for formatting.}

\item{...}{Additional arguments (ignored).}
}
\value{
The object \code{x} of class \code{c("hzr_gof", "data.frame")},
invisibly. The data frame has one row per time point and columns:
\code{time}, \code{n_risk}, \code{n_event}, \code{n_censor},
\code{km_surv} (Kaplan-Meier survival), \code{km_cumhaz},
\code{par_surv} (parametric survival), \code{par_cumhaz},
\code{cum_observed} (cumulative observed events),
\code{cum_expected} (cumulative expected events from model),
\code{residual} (cum_expected - cum_observed).
Multiphase models additionally include \code{par_cumhaz_<phase>} columns
for per-phase cumulative hazard contributions.
A \code{"summary"} attribute contains scalar diagnostics:
\code{total_observed}, \code{total_expected}, \code{final_residual},
\code{dist}, \code{n}.
}
\description{
Print method for hzr_gof
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomposition.R
\name{.log1pexp}
\alias{.log1pexp}
\title{Compute log(1 + exp(x)) with overflow/underflow protection}
\usage{
.log1pexp(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Numeric vector: log(1 + exp(x)).
}
\description{
Compute log(1 + exp(x)) with overflow/underflow protection
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_with_se}
\alias{.hzr_predict_with_se}
\title{Compute point predictions + delta-method CLs}
\usage{
.hzr_predict_with_se(
  object,
  type,
  time = NULL,
  x = NULL,
  x_list = NULL,
  cov_counts = NULL,
  phases = NULL,
  level = 0.95,
  diff_fn
)
}
\arguments{
\item{object}{Fitted \code{hazard} object.}

\item{type}{One of "hazard", "linear_predictor", "survival",
"cumulative_hazard".}

\item{time}{Numeric prediction times or NULL (for type = "hazard" /
"linear_predictor").}

\item{x}{Design matrix (single-distribution) or NULL.  Ignored for
multiphase; use \code{x_list} instead.}

\item{x_list}{Named list of per-phase design matrices (multiphase only).}

\item{cov_counts}{Named integer covariate counts (multiphase only).}

\item{phases}{Named list of \code{hzr_phase} objects (multiphase only).}

\item{level}{Confidence level (default 0.95).}

\item{diff_fn}{Required function(theta) -> numeric vector of length n
returning the delta-method target (H, exp(eta), or eta depending on
\code{type}).  Used for the point estimate AND for the numeric jacobian
fallback in exp / loglogistic / lognormal.}
}
\value{
data.frame with columns \code{fit}, \code{se.fit}, \code{lower}, \code{upper}.
}
\description{
Dispatched from predict.hazard() when \code{se.fit = TRUE}.
}
\details{
\subsection{DELTA-METHOD TARGET}{

The quantity we actually differentiate depends on the prediction type and
the CL scale the type uses:

type = "cumulative_hazard" -> target = H, log-scale CL
type = "survival"          -> target = H, log-log-survival CL
(final \code{fit} is exp(-H))
type = "hazard"            -> target = exp(eta) (single-dist only),
log-scale CL
type = "linear_predictor"  -> target = eta, natural-scale CL

The caller supplies \code{diff_fn(theta)} that returns the target vector of
length n.  For Weibull and multiphase we build J analytically; for
exp / loglogistic / lognormal we fall through to a numeric jacobian of
\code{diff_fn}.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_with_se}
\alias{.hzr_predict_with_se}
\title{Compute point predictions + delta-method CLs}
\usage{
.hzr_predict_with_se(
  object,
  type,
  time = NULL,
  x = NULL,
  x_list = NULL,
  cov_counts = NULL,
  phases = NULL,
  level = 0.95,
  diff_fn,
  conf_type = c("log-log", "logit")
)
}
\arguments{
\item{object}{Fitted \code{hazard} object.}

\item{type}{One of "hazard", "linear_predictor", "survival",
"cumulative_hazard".}

\item{time}{Numeric prediction times or NULL (for type = "hazard" /
"linear_predictor").}

\item{x}{Design matrix (single-distribution) or NULL.  Ignored for
multiphase; use \code{x_list} instead.}

\item{x_list}{Named list of per-phase design matrices (multiphase only).}

\item{cov_counts}{Named integer covariate counts (multiphase only).}

\item{phases}{Named list of \code{hzr_phase} objects (multiphase only).}

\item{level}{Confidence level (default 0.95).}

\item{diff_fn}{Required function(theta) -> numeric vector of length n
returning the delta-method target (H, exp(eta), or eta depending on
\code{type}).  Used for the point estimate AND for the numeric jacobian
fallback in exp / loglogistic / lognormal.}

\item{conf_type}{Survival CL transform: \code{"log-log"} (default, on
\verb{log(-log S)}) or \code{"logit"} (on \code{logit(1 - S)}, HAZPRED-compatible).
Only consulted when \code{type == "survival"}.}
}
\value{
data.frame with columns \code{fit}, \code{se.fit}, \code{lower}, \code{upper}.
}
\description{
Dispatched from predict.hazard() when \code{se.fit = TRUE}.
}
\details{
\subsection{DELTA-METHOD TARGET}{

The quantity we actually differentiate depends on the prediction type and
the CL scale the type uses:

type = "cumulative_hazard" -> target = H, log-scale CL
type = "survival"          -> target = H, log-log-survival CL
(final \code{fit} is exp(-H))
type = "hazard"            -> target = exp(eta) (single-dist only),
log-scale CL
type = "linear_predictor"  -> target = eta, natural-scale CL

The caller supplies \code{diff_fn(theta)} that returns the target vector of
length n.  For Weibull and multiphase we build J analytically; for
exp / loglogistic / lognormal we fall through to a numeric jacobian of
\code{diff_fn}.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{summary.hazard}
\alias{summary.hazard}
\title{Summarize a hazard model}
\usage{
\method{summary}{hazard}(object, ...)
}
\arguments{
\item{object}{A \code{hazard} object.}

\item{...}{Unused; for S3 compatibility.}
}
\value{
An object of class \code{summary.hazard}.
}
\description{
Returns a compact summary of a \code{hazard} object, including model metadata,
fit diagnostics, and coefficient-level statistics when available.
}
\examples{
# -- Single-phase Weibull summary ------------------------------------
fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
              theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
summary(fit)

\donttest{
# -- Multiphase model summary ----------------------------------------
set.seed(42)
n   <- 200
dat <- data.frame(
  time   = rexp(n, rate = 0.25) + 0.01,
  status = rbinom(n, size = 1, prob = 0.65)
)
fit_mp <- hazard(
  survival::Surv(time, status) ~ 1,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                       fixed = "shapes"),
    late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
                       fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)
summary(fit_mp)
}

}
\seealso{
\code{\link[=hazard]{hazard()}} for model fitting, \code{\link[=predict.hazard]{predict.hazard()}} for predictions.

\code{vignette("fitting-hazard-models")} for fitting workflows,
\code{vignette("inference-diagnostics")} for bootstrap CIs and diagnostics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomposition.R
\name{.hzr_phase_derivatives}
\alias{.hzr_phase_derivatives}
\title{Derivatives of phase cumulative and instantaneous hazard w.r.t. shape params}
\usage{
.hzr_phase_derivatives(
  time,
  t_half,
  nu,
  m,
  type = c("cdf", "hazard", "constant")
)
}
\arguments{
\item{time}{Numeric vector of positive times.}

\item{t_half}{Positive scalar half-life.}

\item{nu}{Numeric scalar time exponent.}

\item{m}{Numeric scalar shape exponent.}

\item{type}{Phase type: \code{"cdf"}, \code{"hazard"}, or \code{"constant"}.}
}
\value{
Named list:
\describe{
\item{Phi}{Cumulative hazard contribution \eqn{\Phi(t)}.}
\item{phi}{Instantaneous hazard contribution \eqn{\phi(t) = d\Phi/dt}.}
\item{dPhi_dlog_thalf}{\eqn{d\Phi / d(\log t_{1/2})}.}
\item{dPhi_dnu}{\eqn{d\Phi / d\nu}.}
\item{dPhi_dm}{\eqn{d\Phi / dm}.}
\item{dphi_dlog_thalf}{\eqn{d\phi / d(\log t_{1/2})}.}
\item{dphi_dnu}{\eqn{d\phi / d\nu}.}
\item{dphi_dm}{\eqn{d\phi / dm}.}
}
}
\description{
Computes \eqn{\Phi_j(t)}, \eqn{\phi_j(t)}, and their derivatives with
respect to \code{t_half}, \code{nu}, and \code{m} using central finite differences on
\code{\link[=hzr_decompos]{hzr_decompos()}}.  The \code{log_t_half} derivative is obtained via the chain
rule: \eqn{d\Phi/d(\log t_{1/2}) = t_{1/2} \cdot d\Phi/dt_{1/2}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_competing_risks}
\alias{hzr_competing_risks}
\alias{print.hzr_competing_risks}
\title{Competing risks cumulative incidence}
\usage{
hzr_competing_risks(time, event)

\method{print}{hzr_competing_risks}(x, digits = 4, ...)
}
\arguments{
\item{time}{Numeric vector of follow-up times.}

\item{event}{Integer vector of event type indicators:
0 = censored, 1 = event type 1, 2 = event type 2, etc.}

\item{x}{An \code{hzr_competing_risks} object.}

\item{digits}{Number of decimal places for formatting.}

\item{...}{Additional arguments (ignored).}
}
\value{
A data frame with one row per unique event time and columns:
\describe{
\item{time}{Event time.}
\item{n_risk}{Number at risk.}
\item{n_event_1, n_event_2, ...}{Events of each type at this time.}
\item{n_censor}{Number censored at this time.}
\item{surv}{Overall event-free survival (freedom from all events).}
\item{incid_1, incid_2, ...}{Cumulative incidence for each event type.}
\item{se_surv}{Standard error of overall survival.}
\item{se_1, se_2, ...}{Standard error of each cumulative incidence.}
}
}
\description{
Compute cumulative incidence functions for multiple competing event
types using the Aalen-Johansen estimator with Greenwood variance.
This is the R equivalent of the SAS \code{markov.sas} macro.
}
\details{
Unlike the naive 1 - KM estimator (which overestimates incidence when
competing risks exist), this provides the correct marginal cumulative
incidence for each event type.
}
\examples{
data(valves)
valves_cc <- na.omit(valves)
# Combine death and PVE into a competing risks event variable
# 0 = censored, 1 = death, 2 = PVE
event_cr <- ifelse(valves_cc$dead == 1, 1L,
                   ifelse(valves_cc$pve == 1, 2L, 0L))
time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
cr <- hzr_competing_risks(time_cr, event_cr)
head(cr)

}
\seealso{
\code{\link[=hzr_kaplan]{hzr_kaplan()}} for single-event survival estimation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_competing_risks}
\alias{hzr_competing_risks}
\alias{print.hzr_competing_risks}
\title{Competing risks cumulative incidence}
\usage{
hzr_competing_risks(time, event)

\method{print}{hzr_competing_risks}(x, digits = 4, ...)
}
\arguments{
\item{time}{Numeric vector of follow-up times.}

\item{event}{Integer vector of event type indicators:
0 = censored, 1 = event type 1, 2 = event type 2, etc.}

\item{x}{An \code{hzr_competing_risks} object.}

\item{digits}{Number of decimal places for formatting.}

\item{...}{Additional arguments (ignored).}
}
\value{
A data frame with one row per unique event time and columns:
\describe{
\item{time}{Event time.}
\item{n_risk}{Number at risk.}
\item{n_event_1, n_event_2, ...}{Events of each type at this time.}
\item{n_censor}{Number censored at this time.}
\item{surv}{Overall event-free survival (freedom from all events).}
\item{incid_1, incid_2, ...}{Cumulative incidence for each event type.}
\item{se_surv}{Standard error of overall survival.}
\item{se_1, se_2, ...}{Standard error of each cumulative incidence.}
}
}
\description{
Compute cumulative incidence functions for multiple competing event
types using the Aalen-Johansen estimator with Greenwood variance.
This is the R equivalent of the SAS \code{markov.sas} macro.
}
\details{
Unlike the naive 1 - KM estimator (which overestimates incidence when
competing risks exist), this provides the correct marginal cumulative
incidence for each event type.
}
\note{
This estimator is \strong{unweighted}: every observation contributes a
unit count to the at-risk and event tallies. There is no \code{weights}
argument, so case or inverse-probability weights are not yet supported
for competing-risks incidence (unlike \code{\link[=hazard]{hazard()}}, which accepts
\code{weights}). Pre-expand weighted rows to individual records if an
approximate weighted estimate is needed.
}
\examples{
data(valves)
valves_cc <- na.omit(valves)
# Combine death and PVE into a competing risks event variable
# 0 = censored, 1 = death, 2 = PVE
event_cr <- ifelse(valves_cc$dead == 1, 1L,
                   ifelse(valves_cc$pve == 1, 2L, 0L))
time_cr <- pmin(valves_cc$int_dead, valves_cc$int_pve)
cr <- hzr_competing_risks(time_cr, event_cr)
head(cr)

}
\references{
Aalen O, Johansen S (1978). An empirical transition matrix for
non-homogeneous Markov chains based on censored observations.
\emph{Scand J Statist} 5(3):141--150.

Kalbfleisch JD, Prentice RL (1980). \emph{The Statistical Analysis of Failure
Time Data.} Wiley, New York.
}
\seealso{
\code{\link[=hzr_kaplan]{hzr_kaplan()}} for single-event survival estimation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{.hzr_as_design_matrix}
\alias{.hzr_as_design_matrix}
\title{Coerce x to a validated numeric design matrix}
\usage{
.hzr_as_design_matrix(x, n = NULL)
}
\arguments{
\item{x}{A numeric matrix or data frame with numeric columns.}

\item{n}{Expected row count (length of time/status vectors); checked if non-NULL.}
}
\value{
A numeric matrix with column names preserved (or added if absent).
}
\description{
Accepts data.frame or numeric matrix input; validates dimensions and
finiteness.  Called by hazard() to normalise the x argument before any
downstream use.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parity-helpers.R
\name{.hzr_parse_hazard_output}
\alias{.hzr_parse_hazard_output}
\title{Parse hazard binary output}
\usage{
.hzr_parse_hazard_output(output_text, theta = NULL)
}
\arguments{
\item{output_text}{Character vector (lines) from binary output file}

\item{theta}{Optional parameter vector for context/validation}
}
\value{
Data frame with columns: parameter, estimate, std_err, z_stat, p_value
}
\description{
Parses tabular output from the C hazard binary into a structured data frame.
Expects columns: parameter, estimate, StdErr (or SE), z-value (or z), p-value (or pval).
The parser is flexible and case-insensitive for column matching.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomposition.R
\name{hzr_decompos_g3}
\alias{hzr_decompos_g3}
\title{Late-phase (G3) temporal decomposition}
\usage{
hzr_decompos_g3(time, tau, gamma, alpha, eta)
}
\arguments{
\item{time}{Numeric vector of times (must be > 0).}

\item{tau}{Positive scalar scale parameter.}

\item{gamma}{Positive scalar time exponent.}

\item{alpha}{Non-negative scalar shape parameter (0 selects limiting case).}

\item{eta}{Positive scalar outer exponent.}
}
\value{
A named list with two numeric vectors, each the same length
as \code{time}:
\describe{
\item{G3}{Cumulative intensity \eqn{G_3(t) \ge 0} (may exceed 1).}
\item{g3}{Derivative \eqn{g_3(t) = dG_3/dt \ge 0}.}
}
}
\description{
Computes the cumulative intensity \eqn{G_3(t)} and its derivative
\eqn{g_3(t) = dG_3/dt} for the late-phase parametric family used in the
original Blackstone C/SAS HAZARD code.  Unlike \code{\link[=hzr_decompos]{hzr_decompos()}} (which
computes the early-phase G1 -- a bounded CDF), this function can produce
\strong{unbounded} values, making it suitable for modelling increasing late risk.
}
\section{Mathematical form}{


When \eqn{\alpha > 0}:
\deqn{G_3(t) = \bigl(\bigl((t/\tau)^\gamma + 1\bigr)^{1/\alpha}
                      - 1\bigr)^\eta}

When \eqn{\alpha = 0} (limiting exponential case):
\deqn{G_3(t) = \bigl(\exp\bigl((t/\tau)^\gamma\bigr) - 1\bigr)^\eta}
}

\section{Parameter mapping from SAS/C HAZARD}{
\tabular{lll}{
   SAS name \tab R argument \tab Role \cr
   TAU \tab \code{tau} \tab Scale (time at which \eqn{(t/\tau) = 1}) \cr
   GAMMA \tab \code{gamma} \tab Power exponent on \eqn{t/\tau} \cr
   ALPHA \tab \code{alpha} \tab Shape (0 = exponential limiting case) \cr
   ETA \tab \code{eta} \tab Outer power exponent \cr
}
}

\examples{
t_grid <- seq(0.1, 10, by = 0.1)

# Weibull-like: alpha = 1 gives G3(t) = (t/tau)^(gamma*eta)
d <- hzr_decompos_g3(t_grid, tau = 1, gamma = 3, alpha = 1, eta = 1)
plot(t_grid, d$G3, type = "l", main = "G3: power law (gamma=3)")

# General case with alpha > 0
d2 <- hzr_decompos_g3(t_grid, tau = 2, gamma = 2, alpha = 0.5, eta = 1)

}
\references{
Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. \emph{J Am Stat Assoc.} 1986;81(395):615--624.
\doi{10.1080/01621459.1986.10478314}
}
\seealso{
\code{\link[=hzr_decompos]{hzr_decompos()}} for the early-phase (G1) decomposition,
\code{\link[=hzr_phase_cumhaz]{hzr_phase_cumhaz()}} for phase-level cumulative hazard helpers.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_nelson}
\alias{hzr_nelson}
\alias{print.hzr_nelson}
\title{Wayne Nelson cumulative hazard estimator with lognormal confidence limits}
\usage{
hzr_nelson(time, event, weight = NULL, conf_level = 0.95)

\method{print}{hzr_nelson}(x, digits = 4, ...)
}
\arguments{
\item{time}{Numeric vector of follow-up times.}

\item{event}{Numeric event indicator (1 = event, 0 = censored).}

\item{weight}{Optional numeric vector of event weights (default 1).
Weights are applied only to events (censored observations contribute
zero weight). Use for severity-weighted repeated events.}

\item{conf_level}{Confidence level for the interval (default 0.95).}

\item{x}{An \code{hzr_nelson} object.}

\item{digits}{Number of decimal places for formatting.}

\item{...}{Additional arguments (ignored).}
}
\value{
A data frame with one row per unique event time and columns:
\describe{
\item{time}{Event time.}
\item{n_risk}{Number at risk.}
\item{n_event}{Number of events at this time.}
\item{weight_sum}{Sum of event weights at this time.}
\item{cumhaz}{Nelson cumulative hazard estimate.}
\item{std_err}{Standard error.}
\item{cl_lower}{Lower lognormal confidence limit.}
\item{cl_upper}{Upper lognormal confidence limit.}
\item{hazard}{Interval hazard rate.}
\item{cum_events}{Cumulative (weighted) event count.}
}
}
\description{
Compute the Nelson-Aalen cumulative hazard estimate with lognormal
confidence limits. Supports weighted events for severity-adjusted
analyses of repeated/recurrent events.
This is the R equivalent of the SAS \code{nelsonl.sas} macro.
}
\details{
Unlike \code{survival::survfit()} which uses the Breslow estimator with
Greenwood variance, this function uses the Wayne Nelson estimator
with lognormal confidence limits that are always non-negative.
}
\examples{
data(cabgkul)
nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)
head(nel)

}
\seealso{
\code{\link[=hzr_kaplan]{hzr_kaplan()}} for survival estimation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_nelson}
\alias{hzr_nelson}
\alias{print.hzr_nelson}
\title{Wayne Nelson cumulative hazard estimator with lognormal confidence limits}
\usage{
hzr_nelson(time, event, weight = NULL, conf_level = 0.95)

\method{print}{hzr_nelson}(x, digits = 4, ...)
}
\arguments{
\item{time}{Numeric vector of follow-up times.}

\item{event}{Numeric event indicator (1 = event, 0 = censored).}

\item{weight}{Optional numeric vector of event weights (default 1).
Weights are applied only to events (censored observations contribute
zero weight). Use for severity-weighted repeated events.}

\item{conf_level}{Confidence level for the interval (default 0.95).}

\item{x}{An \code{hzr_nelson} object.}

\item{digits}{Number of decimal places for formatting.}

\item{...}{Additional arguments (ignored).}
}
\value{
A data frame with one row per unique event time and columns:
\describe{
\item{time}{Event time.}
\item{n_risk}{Number at risk.}
\item{n_event}{Number of events at this time.}
\item{weight_sum}{Sum of event weights at this time.}
\item{cumhaz}{Nelson cumulative hazard estimate.}
\item{std_err}{Standard error.}
\item{cl_lower}{Lower lognormal confidence limit.}
\item{cl_upper}{Upper lognormal confidence limit.}
\item{hazard}{Interval hazard rate.}
\item{cum_events}{Cumulative (weighted) event count.}
}
}
\description{
Compute the Nelson-Aalen cumulative hazard estimate with lognormal
confidence limits. Supports weighted events for severity-adjusted
analyses of repeated/recurrent events.
This is the R equivalent of the SAS \code{nelsonl.sas} macro.
}
\details{
Unlike \code{survival::survfit()} which uses the Breslow estimator with
Greenwood variance, this function uses the Wayne Nelson estimator
with lognormal confidence limits that are always non-negative.
}
\examples{
data(cabgkul)
nel <- hzr_nelson(cabgkul$int_dead, cabgkul$dead)
head(nel)

}
\references{
Nelson W (1972). Theory and applications of hazard plotting for censored
failure data. \emph{Technometrics} 14(4):945--966.
\doi{10.1080/00401706.1972.10488991}

Aalen O (1978). Nonparametric inference for a family of counting processes.
\emph{Ann Statist} 6(4):701--726. \doi{10.1214/aos/1176344247}
}
\seealso{
\code{\link[=hzr_kaplan]{hzr_kaplan()}} for survival estimation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_multiphase_theta_user}
\alias{.hzr_multiphase_theta_user}
\title{Transform multiphase theta from internal to user-facing scale}
\usage{
.hzr_multiphase_theta_user(theta, phases, covariate_counts)
}
\arguments{
\item{theta}{Full parameter vector (internal scale).}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector of per-phase covariate counts.}
}
\value{
Named numeric vector on the reporting scale.
}
\description{
Converts log_mu -> mu, log_t_half -> t_half; nu, m, beta pass through.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_bootstrap}
\alias{hzr_bootstrap}
\alias{print.hzr_bootstrap}
\title{Bootstrap resampling for hazard model coefficients}
\usage{
hzr_bootstrap(
  object,
  n_boot = 200L,
  fraction = 1,
  seed = NULL,
  verbose = FALSE
)

\method{print}{hzr_bootstrap}(x, digits = 4, ...)
}
\arguments{
\item{object}{A fitted \code{hazard} object (with \code{fit = TRUE}).}

\item{n_boot}{Integer: number of bootstrap replicates (default 200).}

\item{fraction}{Numeric in (0, 1]: fraction of data to sample per
replicate (default 1.0 for full bootstrap; < 1 for bagging).}

\item{seed}{Optional integer random seed for reproducibility. When
supplied, \code{set.seed(seed)} is called at function entry, jumping the
global RNG to the seeded state; it is not restored on exit. Pass
\code{NULL} (the default) to skip the \code{set.seed()} call and start from
the caller's current RNG state. Note that the bootstrap consumes
random numbers either way, so the global RNG state will advance
during the call -- \code{seed = NULL} avoids the \emph{reset} at entry, not
the advance during resampling.}

\item{verbose}{Logical; if \code{TRUE}, print progress every 50 replicates.}

\item{x}{An \code{hzr_bootstrap} object.}

\item{digits}{Number of decimal places for formatting.}

\item{...}{Additional arguments (ignored).}
}
\value{
A list with class \code{"hzr_bootstrap"} containing:
\describe{
\item{replicates}{Data frame with columns \code{replicate}, \code{parameter},
and \code{estimate} -- one row per parameter per successful replicate.}
\item{summary}{Data frame with columns \code{parameter}, \code{n}, \code{pct},
\code{mean}, \code{sd}, \code{min}, \code{max}, \code{ci_lower}, \code{ci_upper} -- one row per
parameter.}
\item{n_success}{Number of successfully converged replicates.}
\item{n_failed}{Number of replicates that failed to converge.}
}
}
\description{
Resample data with replacement, refit the hazard model on each
replicate, and accumulate coefficient distributions. Returns a tidy
data frame of per-replicate estimates with summary statistics.
This is the R equivalent of the SAS \code{bootstrap.hazard.sas} macro.
}
\examples{
\donttest{
data(avc)
avc <- na.omit(avc)
fit <- hazard(
  survival::Surv(int_dead, dead) ~ age + mal,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.01, nu = 0.5, 0, 0),
  fit   = TRUE
)
bs <- hzr_bootstrap(fit, n_boot = 50, seed = 123)
print(bs)
}

}
\seealso{
\code{\link[=hazard]{hazard()}} for model fitting, \code{\link[=vcov.hazard]{vcov.hazard()}} for
Hessian-based standard errors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{avc}
\alias{avc}
\title{AVC: Atrioventricular Canal Repair}
\format{
A data frame with 310 rows and 11 variables:
\describe{
\item{study}{Patient identifier}
\item{status}{NYHA functional class (1--4)}
\item{inc_surg}{Surgical grade of AV valve incompetence}
\item{opmos}{Date of operation (months since January 1967)}
\item{age}{Age at repair (months)}
\item{mal}{Malalignment indicator (0/1)}
\item{com_iv}{Interventricular communication indicator (0/1)}
\item{orifice}{Associated cardiac anomaly indicator (0/1)}
\item{dead}{Death indicator (1 = dead, 0 = censored)}
\item{int_dead}{Follow-up interval to death or last contact (months)}
\item{op_age}{Interaction term: opmos x age}
}
}
\source{
Blackstone, Naftel, and Turner (1986)
\doi{10.1080/01621459.1986.10478314}. Cleveland Clinic Foundation.
}
\usage{
avc
}
\description{
Survival data for 310 patients who underwent repair of atrioventricular
septal defects (congenital heart disease) at the Cleveland Clinic between
1977 and 1993. Exhibits two identifiable hazard phases: an early
post-operative risk and a constant late phase.
}
\examples{
data(avc)
avc <- na.omit(avc)

# Kaplan-Meier survival
km <- survival::survfit(survival::Surv(int_dead, dead) ~ 1, data = avc)
plot(km, xlab = "Months after AVC repair", ylab = "Survival",
     main = "AVC: Kaplan-Meier survival estimate")

\donttest{
# Two-phase hazard fit (early CDF + constant -- what AVC supports)
fit <- hazard(
  survival::Surv(int_dead, dead) ~ 1, data = avc,
  dist = "multiphase",
  phases = list(
    early    = hzr_phase("cdf", t_half = 0.5, nu = 1, m = 1),
    constant = hzr_phase("constant")
  ),
  fit = TRUE, control = list(n_starts = 5, maxit = 1000)
)
summary(fit)
}

}
\seealso{
\code{vignette("fitting-hazard-models")},
\code{vignette("prediction-visualization")}

Other datasets:
\code{\link{cabgkul}},
\code{\link{omc}},
\code{\link{tga}},
\code{\link{valves}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_calibrate}
\alias{hzr_calibrate}
\title{Calibrate a continuous variable against an outcome}
\usage{
hzr_calibrate(
  x,
  event,
  groups = 10L,
  by = NULL,
  link = c("logit", "gompertz", "cox"),
  time = NULL
)
}
\arguments{
\item{x}{Numeric vector: the continuous covariate to calibrate.}

\item{event}{Numeric vector: event indicator (1 = event, 0 = no event).}

\item{groups}{Integer: number of quantile bins (default 10).}

\item{by}{Optional factor or character vector for stratified calibration
(SAS \code{logitgr.sas} functionality). If provided, calibration is
computed within each stratum. Default \code{NULL} (no stratification).}

\item{link}{Character: transform to apply to event probabilities.
One of \code{"logit"} (default), \code{"gompertz"} (complementary log-log),
or \code{"cox"}.}

\item{time}{Optional numeric vector: follow-up time, required when
\code{link = "cox"}. The Cox link computes
\eqn{\log(\text{events} / \sum \text{time})} (constant hazard rate).}
}
\value{
A data frame with one row per group (or per group-by-stratum
combination) and columns:
\describe{
\item{group}{Integer group label.}
\item{by}{Stratum level (only present when \code{by} is provided).}
\item{n}{Number of observations in the group.}
\item{events}{Number of events.}
\item{mean}{Mean of \code{x} within the group.}
\item{min}{Minimum of \code{x} within the group.}
\item{max}{Maximum of \code{x} within the group.}
\item{prob}{Event probability (events / n), or for Cox link:
events / sum(time).}
\item{link_value}{Transformed probability on the chosen link scale.}
}
}
\description{
Group a continuous covariate into quantile bins, compute the event
probability (or hazard rate) per bin, and apply a link transform
(logit, Gompertz, or Cox).
This is the R equivalent of the SAS \code{logit.sas} and \code{logitgr.sas}
macros.
}
\details{
Use this function before model entry to assess whether the covariate
relationship with the outcome is approximately linear on the link
scale. If the transformed probabilities are roughly linear against
the group means, the covariate can enter the model untransformed.
Curvature suggests a transformation (log, quadratic) may improve fit.
}
\examples{
data(avc)
avc <- na.omit(avc)

# Logit calibration of age
cal <- hzr_calibrate(avc$age, avc$dead, groups = 10)
print(cal)

\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  ggplot(cal, aes(mean, link_value)) +
    geom_point(size = 3) +
    geom_smooth(method = "lm", formula = y ~ x, se = FALSE,
                linetype = "dashed") +
    labs(x = "Age at repair (months)", y = "Logit(P(death))") +
    theme_minimal()
}
}

}
\seealso{
\code{\link[=hzr_deciles]{hzr_deciles()}} for model-based calibration after fitting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{.hzr_phase_n_params}
\alias{.hzr_phase_n_params}
\title{Total number of parameters for a phase}
\usage{
.hzr_phase_n_params(phase, n_covariates = 0L)
}
\arguments{
\item{phase}{An \code{hzr_phase} object.}

\item{n_covariates}{Integer; number of covariate columns this phase uses.}
}
\value{
Integer.
}
\description{
Returns the count of free parameters in the theta sub-vector for one phase:
1 (log_mu) + n_shape + n_covariates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{.hzr_phase_start}
\alias{.hzr_phase_start}
\title{Extract starting values from a phase specification}
\usage{
.hzr_phase_start(phase, n_covariates = 0L, mu_start = 0.1)
}
\arguments{
\item{phase}{An \code{hzr_phase} object.}

\item{n_covariates}{Integer; number of covariate columns.}

\item{mu_start}{Numeric scalar; initial scale parameter (default 0.1).}
}
\value{
Named numeric vector of starting values.
}
\description{
Returns initial theta sub-vector on the estimation (internal) scale:
log(mu), log(t_half), nu, m, followed by zeros for covariate coefficients.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{print.summary.hazard}
\alias{print.summary.hazard}
\title{Print method for hazard summary objects}
\usage{
\method{print}{summary.hazard}(x, ...)
}
\arguments{
\item{x}{A \code{summary.hazard} object returned by \code{\link[=summary.hazard]{summary.hazard()}}.}

\item{...}{Additional arguments (ignored).}
}
\value{
\code{x}, invisibly.
}
\description{
Formatted console display of \code{\link[=summary.hazard]{summary.hazard()}} output: distribution,
phase list (for multiphase), coefficient table with standard errors,
and log-likelihood.  S3 dispatch only -- users call \code{print(summary(fit))}
rather than invoking this directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{print.summary.hazard}
\alias{print.summary.hazard}
\title{Print method for hazard summary objects}
\usage{
\method{print}{summary.hazard}(x, ...)
}
\arguments{
\item{x}{A \code{summary.hazard} object returned by \code{\link[=summary.hazard]{summary.hazard()}}.}

\item{...}{Additional arguments (ignored).}
}
\value{
\code{x}, invisibly.
}
\description{
Formatted console display of \code{\link[=summary.hazard]{summary.hazard()}} output: distribution,
phase list (for multiphase), coefficient table with standard errors,
and log-likelihood.  When the post-fit Hessian is ill-conditioned or not
positive-definite, a note warns that the standard errors may be unreliable;
when the Hessian could not be inverted at all, a note reports that standard
errors are unavailable.  S3 dispatch only -- users call \code{print(summary(fit))}
rather than invoking this directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/argument_mapping.R
\name{hzr_argument_mapping}
\alias{hzr_argument_mapping}
\title{Legacy HAZARD to TemporalHazard argument mapping}
\usage{
hzr_argument_mapping(include_planned = TRUE)
}
\arguments{
\item{include_planned}{Logical; if \code{FALSE}, only rows marked as implemented
are returned.}
}
\value{
A data frame with one row per mapping rule.
}
\description{
Returns a formal mapping table that defines how legacy SAS HAZARD/C-style
inputs map to \code{hazard(...)} arguments in this package.
}
\examples{
hzr_argument_mapping()
hzr_argument_mapping(include_planned = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/argument_mapping.R
\name{hzr_argument_mapping}
\alias{hzr_argument_mapping}
\title{Legacy HAZARD to TemporalHazard argument mapping}
\usage{
hzr_argument_mapping(include_planned = TRUE)
}
\arguments{
\item{include_planned}{Logical; if \code{FALSE}, only rows marked as implemented
are returned.}
}
\value{
A data frame with one row per mapping rule.
}
\description{
Returns a formal mapping table that defines how legacy SAS HAZARD/C-style
inputs map to \code{hazard(...)} arguments in this package.
}
\examples{
hzr_argument_mapping()
hzr_argument_mapping(include_planned = FALSE)
}
\seealso{
\code{\link[=hazard]{hazard()}} for the target arguments, \code{\link[=hzr_phase]{hzr_phase()}} for phase
construction, and \code{\link[=hzr_decompos]{hzr_decompos()}}/\code{\link[=hzr_decompos_g3]{hzr_decompos_g3()}} for the
parameter-level early/late translations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_se_from_jacobian}
\alias{.hzr_predict_se_from_jacobian}
\title{Per-row standard errors via the delta method}
\usage{
.hzr_predict_se_from_jacobian(J, vcov)
}
\arguments{
\item{J}{Jacobian (n x p).}

\item{vcov}{p x p variance-covariance matrix.}
}
\value{
Numeric vector of length n.
}
\description{
Per-row standard errors via the delta method
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_kaplan}
\alias{hzr_kaplan}
\title{Kaplan-Meier survival with exact logit confidence limits}
\usage{
hzr_kaplan(time, status, conf_level = 0.95, event_only = TRUE)
}
\arguments{
\item{time}{Numeric vector of follow-up times.}

\item{status}{Numeric event indicator (1 = event, 0 = censored).}

\item{conf_level}{Confidence level for the interval (default 0.95).
The SAS default of 0.68268948 corresponds to a 1-SD interval.}

\item{event_only}{Logical; if \code{TRUE} (default), only return rows at
event times (where \code{n_event > 0}).
If \code{FALSE}, return rows at all times reported by
\code{survival::survfit()} (events and censoring times).}
}
\value{
A data frame with one row per time point and columns:
\describe{
\item{time}{Event/censoring time.}
\item{n_risk}{Number at risk at start of interval.}
\item{n_event}{Number of events at this time.}
\item{n_censor}{Number censored at this time.}
\item{survival}{Kaplan-Meier survival estimate.}
\item{std_err}{Standard error of survival (Greenwood).}
\item{cl_lower}{Lower confidence limit (logit-transformed).}
\item{cl_upper}{Upper confidence limit (logit-transformed).}
\item{cumhaz}{Cumulative hazard \eqn{= -\log(S)}.}
\item{hazard}{Interval hazard rate
\eqn{= \log(S_{t-1} / S_t) / \Delta t}.}
\item{density}{Probability density estimate
\eqn{= (S_{t-1} - S_t) / \Delta t}.}
\item{life}{Restricted mean survival time (area under curve to
this time).}
}
}
\description{
Compute the product-limit (Kaplan-Meier) survival estimate with
logit-transformed confidence limits that respect the \eqn{[0, 1]}
boundary.
This is the R equivalent of the SAS \code{kaplan.sas} macro.
}
\details{
The standard Greenwood confidence interval can exceed \eqn{[0, 1]} in the
tails. The logit-transformed interval avoids this by working on the
log-odds scale:

\deqn{
  \text{CL}_{\text{lower}} = S / \bigl(S + (1-S)\,
  \exp(z_\alpha\,\text{SI})\bigr)
}
\deqn{
  \text{CL}_{\text{upper}} = S / \bigl(S + (1-S)\,
  \exp(-z_\alpha\,\text{SI})\bigr)
}

where \eqn{\text{SI} = \sqrt{V_P - 1} / (1 - S)}, \eqn{V_P} is the
cumulative Greenwood variance product, and \eqn{z_\alpha} is the
normal quantile for the requested confidence level.
}
\examples{
data(cabgkul)
km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)
head(km)

\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  ggplot(km, aes(time)) +
    geom_step(aes(y = survival * 100)) +
    geom_ribbon(aes(ymin = cl_lower * 100, ymax = cl_upper * 100),
                stat = "identity", alpha = 0.2) +
    labs(x = "Months", y = "Survival (\%)") +
    theme_minimal()
}
}

}
\seealso{
\code{\link[=hzr_gof]{hzr_gof()}} for parametric vs. nonparametric comparison.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_free_vcov}
\alias{.hzr_free_vcov}
\title{Free-parameter vcov submatrix for the delta-method sandwich}
\usage{
.hzr_free_vcov(vcov_mat, p)
}
\arguments{
\item{vcov_mat}{The fitted vcov (or NULL / wrong shape).}

\item{p}{Length of the parameter vector.}
}
\value{
\code{list(vcov_use, free_idx)}, or \code{NULL} if unusable.
}
\description{
Fixed parameters (e.g. \code{fixed = "shapes"}) leave NA rows/cols in the expanded
vcov. Treat them as known-with-zero-variance: restrict the sandwich to the
free submatrix. (The CoE-conserved \code{log_mu} normally participates -- CoE fits
use the full-information vcov -- but it leaves an NA row, like a fixed
parameter, when that recomputation was unavailable.) Returns \code{NULL} (with a
warning) when CLs cannot be computed. Shared by the aggregate and decomposed
se.fit paths.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_gof}
\alias{hzr_gof}
\title{Goodness-of-fit: observed vs. predicted events}
\usage{
hzr_gof(object, time_grid = NULL)
}
\arguments{
\item{object}{A fitted \code{hazard} object (with \code{fit = TRUE}).}

\item{time_grid}{Optional numeric vector of time points at which to
evaluate the parametric model.
If \code{NULL} (default), uses the sorted unique event times from the
fitted data.}
}
\value{
A data frame with one row per time point and columns:
\describe{
\item{time}{Evaluation time.}
\item{n_risk}{Number at risk (Kaplan-Meier).}
\item{n_event}{Number of events at this time.}
\item{n_censor}{Number censored at this time.}
\item{km_surv}{Kaplan-Meier survival estimate.}
\item{km_cumhaz}{Kaplan-Meier cumulative hazard
(\eqn{-\log(\text{km\_surv})}).}
\item{par_surv}{Parametric survival from the fitted model.}
\item{par_cumhaz}{Parametric cumulative hazard.}
\item{cum_observed}{Cumulative observed events to this time.}
\item{cum_expected}{Cumulative expected events (sum of individual
cumulative hazards for observations exiting the risk set).}
\item{residual}{Expected minus observed
(\code{cum_expected - cum_observed}).}
}

For multiphase models, additional columns are appended for each
phase: \code{par_cumhaz_<phase>}.

An attribute \code{"summary"} is attached with scalar diagnostics:
total observed events, total expected events, and the final residual.
}
\description{
Compare a fitted hazard model against the nonparametric Kaplan-Meier
estimate by computing observed and expected (parametric) event counts
at each distinct event time.  This is the R equivalent of the SAS
\code{hazplot.sas} macro and implements the conservation-of-events
diagnostic.
}
\details{
At each observed event time the function computes:
\itemize{
\item The Kaplan-Meier survival and cumulative hazard.
\item The parametric survival and cumulative hazard from the fitted
model (and per-phase components for multiphase models).
\item Cumulative observed events vs. cumulative expected events
(sum of individual cumulative hazards for those exiting the risk
set at each time).
\item The running residual (expected minus observed).
}

Perfect model fit implies the expected and observed event counts track
each other (residual near zero).  This is the conservation-of-events
principle.
}
\examples{
\donttest{
data(avc)
avc <- na.omit(avc)
fit <- hazard(
  survival::Surv(int_dead, dead) ~ age + mal,
  data  = avc,
  dist  = "weibull",
  theta = c(mu = 0.01, nu = 0.5, beta_age = 0, beta_mal = 0),
  fit   = TRUE
)
gof <- hzr_gof(fit)
print(gof)

# Plot observed vs expected events
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  ggplot(gof, aes(x = time)) +
    geom_line(aes(y = cum_observed), colour = "#D55E00") +
    geom_line(aes(y = cum_expected), colour = "#0072B2") +
    labs(x = "Time", y = "Cumulative events") +
    theme_minimal()
}
}

}
\seealso{
\code{\link[=hzr_deciles]{hzr_deciles()}} for decile-of-risk calibration,
\code{\link[=predict.hazard]{predict.hazard()}} for prediction types.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hzr_kaplan}
\alias{hzr_kaplan}
\title{Kaplan-Meier survival with exact logit confidence limits}
\usage{
hzr_kaplan(time, status, conf_level = 0.95, event_only = TRUE)
}
\arguments{
\item{time}{Numeric vector of follow-up times.}

\item{status}{Numeric event indicator (1 = event, 0 = censored).}

\item{conf_level}{Confidence level for the interval (default 0.95).
The SAS default of 0.68268948 corresponds to a 1-SD interval.}

\item{event_only}{Logical; if \code{TRUE} (default), only return rows at
event times (where \code{n_event > 0}).
If \code{FALSE}, return rows at all times reported by
\code{survival::survfit()} (events and censoring times).}
}
\value{
A data frame with one row per time point and columns:
\describe{
\item{time}{Event/censoring time.}
\item{n_risk}{Number at risk at start of interval.}
\item{n_event}{Number of events at this time.}
\item{n_censor}{Number censored at this time.}
\item{survival}{Kaplan-Meier survival estimate.}
\item{std_err}{Standard error of survival (Greenwood).}
\item{cl_lower}{Lower confidence limit (logit-transformed).}
\item{cl_upper}{Upper confidence limit (logit-transformed).}
\item{cumhaz}{Cumulative hazard \eqn{= -\log(S)}.}
\item{hazard}{Interval hazard rate
\eqn{= \log(S_{t-1} / S_t) / \Delta t}.}
\item{density}{Probability density estimate
\eqn{= (S_{t-1} - S_t) / \Delta t}.}
\item{life}{Restricted mean survival time (area under curve to
this time).}
}
}
\description{
Compute the product-limit (Kaplan-Meier) survival estimate with
logit-transformed confidence limits that respect the \eqn{[0, 1]}
boundary.
This is the R equivalent of the SAS \code{kaplan.sas} macro.
}
\details{
The standard Greenwood confidence interval can exceed \eqn{[0, 1]} in the
tails. The logit-transformed interval avoids this by working on the
log-odds scale:

\deqn{
  \text{CL}_{\text{lower}} = S / \bigl(S + (1-S)\,
  \exp(z_\alpha\,\text{SI})\bigr)
}
\deqn{
  \text{CL}_{\text{upper}} = S / \bigl(S + (1-S)\,
  \exp(-z_\alpha\,\text{SI})\bigr)
}

where \eqn{\text{SI} = \sqrt{V_P - 1} / (1 - S)}, \eqn{V_P} is the
cumulative Greenwood variance product, and \eqn{z_\alpha} is the
normal quantile for the requested confidence level.
}
\examples{
data(cabgkul)
km <- hzr_kaplan(cabgkul$int_dead, cabgkul$dead)
head(km)

\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  ggplot(km, aes(time)) +
    geom_step(aes(y = survival * 100)) +
    geom_ribbon(aes(ymin = cl_lower * 100, ymax = cl_upper * 100),
                stat = "identity", alpha = 0.2) +
    labs(x = "Months", y = "Survival (\%)") +
    theme_minimal()
}
}

}
\references{
Kaplan EL, Meier P (1958). Nonparametric estimation from incomplete
observations. \emph{J Am Stat Assoc} 53(282):457--481.
\doi{10.1080/01621459.1958.10501452}

Greenwood M (1926). The natural duration of cancer. \emph{Reports on Public
Health and Medical Subjects} 33:1--26.
}
\seealso{
\code{\link[=hzr_gof]{hzr_gof()}} for parametric vs. nonparametric comparison.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_with_se_decomposed}
\alias{.hzr_predict_with_se_decomposed}
\title{Per-phase + total cumulative-hazard predictions with delta-method CLs}
\usage{
.hzr_predict_with_se_decomposed(
  object,
  time,
  x_list,
  cov_counts,
  phases,
  level = 0.95
)
}
\arguments{
\item{object}{Fitted multiphase \code{hazard} object.}

\item{time}{Numeric prediction times (length n).}

\item{x_list}{Named list of per-phase design matrices.}

\item{cov_counts}{Named integer covariate counts.}

\item{phases}{Named list of \code{hzr_phase} objects.}

\item{level}{Confidence level.}
}
\value{
Long \code{data.frame(time, component, fit, se.fit, lower, upper)};
\code{component} is an ordered factor with levels \code{c("total", names(phases))}.
}
\description{
Long-format companion to \code{\link[=.hzr_predict_with_se]{.hzr_predict_with_se()}} for multiphase models under
\code{decompose = TRUE}. Computes log-scale CLs for the total cumulative hazard
and for each phase's additive contribution \verb{H_j(t) = mu_j(x) Phi_j(t)}.
Per-phase CLs use only that phase's parameter block, so they do NOT sum to
the total CL (cross-phase covariance contributes only to the total).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{omc}
\alias{omc}
\title{OMC: Open Mitral Commissurotomy}
\format{
A data frame with 339 rows and 7 variables:
\describe{
\item{study}{Patient identifier}
\item{te1}{Indicator for first thromboembolic event}
\item{te2}{Indicator for second thromboembolic event}
\item{te3}{Indicator for third thromboembolic event}
\item{int_dead}{Follow-up interval to death or last contact (months)}
\item{dead}{Death indicator (1 = dead, 0 = censored)}
\item{opdjul}{Operation date (Julian)}
}
}
\source{
University of Alabama Birmingham cardiac surgery registry.
}
\usage{
omc
}
\description{
Data for 339 patients who underwent open mitral commissurotomy at the
University of Alabama Birmingham. Contains repeated thromboembolic events
(up to 3 per patient) with left censoring, exercising the interval
censoring likelihood.
}
\seealso{
Other datasets:
\code{\link{avc}},
\code{\link{cabgkul}},
\code{\link{tga}},
\code{\link{valves}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{coef.hazard}
\alias{coef.hazard}
\title{Extract coefficients from hazard model}
\usage{
\method{coef}{hazard}(object, ...)
}
\arguments{
\item{object}{A \code{hazard} object.}

\item{...}{Unused; for S3 compatibility.}
}
\value{
A named numeric vector of fitted parameter estimates, or \code{NULL}
if the model has not been fitted (\code{fit = FALSE}).
}
\description{
Extract coefficients from hazard model
}
\examples{
fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
              theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
coef(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_multiphase_hazard}
\alias{.hzr_multiphase_hazard}
\title{Compute total instantaneous hazard h(t|x) for multiphase model}
\usage{
.hzr_multiphase_hazard(time, theta, phases, covariate_counts, x_list)
}
\arguments{
\item{time}{Numeric vector of times (n).}

\item{theta}{Full parameter vector (internal scale).}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector of covariate counts per phase.}

\item{x_list}{Named list of design matrices, one per phase. Each element is
an n x p_j matrix or NULL.}
}
\value{
Numeric vector of length n.
}
\description{
Compute total instantaneous hazard h(t|x) for multiphase model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_gradient_multiphase}
\alias{.hzr_gradient_multiphase}
\title{Gradient of the multiphase log-likelihood}
\usage{
.hzr_gradient_multiphase(
  theta,
  time,
  status,
  time_lower = NULL,
  time_upper = NULL,
  x = NULL,
  weights = NULL,
  phases,
  covariate_counts,
  x_list,
  ...
)
}
\arguments{
\item{theta}{Full parameter vector (internal scale).}

\item{time}{Numeric vector of follow-up times (n).}

\item{status}{Numeric event indicator: 1 = event, 0 = right-censored,
-1 = left-censored, 2 = interval-censored.}

\item{time_lower}{Optional lower bounds for interval censoring.}

\item{time_upper}{Optional upper bounds for left/interval censoring.}

\item{x}{Design matrix (unused directly; kept for interface compatibility).}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector of per-phase covariate counts.}

\item{x_list}{Named list of per-phase design matrices.}

\item{...}{Ignored.}
}
\value{
Numeric vector of length \code{length(theta)} -- the gradient.
Returns a zero vector if any component is non-finite (guards optimizer).
}
\description{
Computes the score vector \eqn{d\ell / d\theta} using analytic chain-rule
formulas for \code{log_mu} and \code{beta} parameters, and central-difference
derivatives (via \code{.hzr_phase_derivatives()}) for shape parameters
(\code{log_t_half}, \code{nu}, \code{m}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{.hzr_phase_theta_names}
\alias{.hzr_phase_theta_names}
\title{Generate parameter names for a phase's sub-vector}
\usage{
.hzr_phase_theta_names(phase, phase_name, covariate_names = character(0))
}
\arguments{
\item{phase}{An \code{hzr_phase} object.}

\item{phase_name}{Character label for the phase (e.g. \code{"early"}, \code{"phase_1"}).}

\item{covariate_names}{Character vector of covariate column names that
this phase uses.  Can be length 0 if no covariates.}
}
\value{
Character vector of parameter names.
}
\description{
Builds the named labels for the parameter block that belongs to a single
phase in the full theta vector.  The block layout is:
}
\details{
\itemize{
\item For \code{"cdf"}/\code{"hazard"}: \verb{[log_mu, log_t_half, nu, m, beta_1, ..., beta_p]}
\item For \code{"constant"}:       \verb{[log_mu, beta_1, ..., beta_p]}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{print.hzr_kaplan}
\alias{print.hzr_kaplan}
\title{Print method for hzr_kaplan}
\usage{
\method{print}{hzr_kaplan}(x, digits = 4, n = 20, ...)
}
\arguments{
\item{x}{An \code{hzr_kaplan} object.}

\item{digits}{Number of decimal places for formatting.}

\item{n}{Maximum rows to print (default 20).}

\item{...}{Additional arguments (ignored).}
}
\value{
The object \code{x} of class \code{c("hzr_kaplan", "data.frame")},
invisibly. The data frame has one row per event time (or all times when
\code{event_only = FALSE}) and columns:
\code{time} (follow-up time),
\code{n_risk} (number at risk),
\code{n_event} (events in interval),
\code{n_censor} (censored observations in interval),
\code{survival} (Kaplan-Meier survival estimate),
\code{std_err} (Greenwood standard error on log-hazard scale),
\code{cl_lower}, \code{cl_upper} (logit-transformed confidence limits
on the survival scale),
\code{cumhaz} (Nelson-Aalen cumulative hazard),
\code{hazard} (interval hazard estimate),
\code{density} (estimated event density),
\code{life} (life-table life expectancy contribution).
}
\description{
Print method for hzr_kaplan
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{print.hzr_calibrate}
\alias{print.hzr_calibrate}
\title{Print method for hzr_calibrate}
\usage{
\method{print}{hzr_calibrate}(x, digits = 3, ...)
}
\arguments{
\item{x}{An \code{hzr_calibrate} object.}

\item{digits}{Number of decimal places for formatting.}

\item{...}{Additional arguments (ignored).}
}
\value{
The object \code{x} of class \code{c("hzr_calibrate", "data.frame")},
invisibly. The data frame has one row per quantile group and columns:
\code{group} (group index),
\code{n} (group size),
\code{events} (event count),
\code{mean}, \code{min}, \code{max} (covariate summary within group),
\code{prob} (observed event probability),
\code{link_value} (transformed probability on the link scale).
When stratified via the \code{by} argument, a \code{by} column is also
present. Attributes: \code{"link"} (the transform applied,
e.g. \code{"logit"}) and \code{"groups"} (number of quantile bins).
}
\description{
Print method for hzr_calibrate
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{vcov.hazard}
\alias{vcov.hazard}
\title{Extract variance-covariance matrix from hazard model}
\usage{
\method{vcov}{hazard}(object, ...)
}
\arguments{
\item{object}{A \code{hazard} object.}

\item{...}{Unused; for S3 compatibility.}
}
\value{
A numeric matrix containing the estimated variance-covariance matrix
of the fitted coefficients, with rows and columns named by the coefficient
labels (phase-prefixed for multiphase models, e.g. \code{early.x}). Rows
and columns for parameters held fixed (e.g. fixed shape parameters) are
\code{NA} because they carry no variance; the finite free-parameter block
is still usable. For Conservation-of-Events fits the conserved phase
\code{log_mu} \emph{normally} carries a variance: it is removed from the
optimizer search but the vcov is the full-information matrix at the optimum
(the CoE solution is the unconstrained MLE). That recomputation requires
\pkg{numDeriv} and an invertible Hessian; if either is unavailable the fit
emits a warning and the conserved \code{log_mu} stays \code{NA} (the rest
of the matrix is unaffected). Returns a scalar \code{NA} only when the
model has not been fitted or no covariance matrix is available.
}
\description{
Returns the estimated variance-covariance matrix of the fitted coefficients.
}
\examples{
fit <- hazard(time = rexp(30, 0.5), status = rep(1L, 30),
              theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
vcov(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cabgkul}
\alias{cabgkul}
\title{CABGKUL: Primary Isolated Coronary Artery Bypass Grafting (KU Leuven)}
\format{
A data frame with 5880 rows and 2 variables:
\describe{
\item{int_dead}{Follow-up interval to death or last contact (months)}
\item{dead}{Death indicator (1 = dead, 0 = censored)}
}
}
\source{
KU Leuven cardiac surgery registry. Primary benchmark dataset for
C binary parity testing.
}
\usage{
cabgkul
}
\description{
Survival data for 5,880 patients who underwent primary isolated CABG
at KU Leuven, Belgium, between 1971 and July 1987. The simplest dataset
structure (intercept-only, right-censored) with large sample size
exercising all three temporal hazard phases.
}
\examples{
data(cabgkul)

# Kaplan-Meier survival
km <- survival::survfit(survival::Surv(int_dead, dead) ~ 1, data = cabgkul)
plot(km, xlab = "Months after CABG", ylab = "Survival",
     main = "CABGKUL: Kaplan-Meier survival (n = 5,880)")

\donttest{
# Single-phase Weibull fit with parametric overlay
fit <- hazard(survival::Surv(int_dead, dead) ~ 1, data = cabgkul,
              dist = "weibull", theta = c(mu = 0.10, nu = 1.0), fit = TRUE)
t_grid <- seq(0.01, max(cabgkul$int_dead) * 0.9, length.out = 200)
surv   <- predict(fit, newdata = data.frame(time = t_grid),
                  type = "survival")
plot(km, xlab = "Months after CABG", ylab = "Survival",
     main = "CABGKUL: Weibull vs. Kaplan-Meier")
lines(t_grid, surv, col = "blue", lwd = 2)
legend("bottomleft", c("KM", "Weibull"), col = c("black", "blue"),
       lty = 1, lwd = c(1, 2))
}

}
\seealso{
\code{vignette("fitting-hazard-models")}

Other datasets:
\code{\link{avc}},
\code{\link{omc}},
\code{\link{tga}},
\code{\link{valves}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_conserve_events}
\alias{.hzr_conserve_events}
\title{Apply the Conservation of Events adjustment to one phase's log_mu}
\usage{
.hzr_conserve_events(
  theta,
  fixmu_phase,
  fixmu_pos,
  time,
  status,
  phases,
  covariate_counts,
  x_list,
  total_events,
  weights = NULL
)
}
\arguments{
\item{theta}{Full parameter vector (internal scale).}

\item{fixmu_phase}{Character: name of the phase whose log_mu is solved.}

\item{fixmu_pos}{Integer: position of that log_mu in theta.}

\item{time}{Numeric vector of follow-up times.}

\item{status}{Numeric event indicator.}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector.}

\item{x_list}{Named list of per-phase design matrices.}

\item{total_events}{Numeric: sum of observed events (precomputed, weighted
when \code{weights} is supplied).}

\item{weights}{Optional numeric vector of row weights (length n). Defaults to
unit weights. Applied when summing per-phase cumhaz so Turner's adjustment
is computed on the same scale as \code{total_events}.}
}
\value{
Updated theta vector with fixmu phase's log_mu adjusted.
}
\description{
Given the current theta vector, analytically solve the fixmu phase's
log_mu so that total predicted events = total observed events.
}
\details{
This is called BEFORE each likelihood evaluation inside the optimizer,
matching the C HAZARD \code{CONSRV} entry point.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_jacobian_numeric}
\alias{.hzr_predict_jacobian_numeric}
\title{Numeric Jacobian of a single-distribution prediction w.r.t. theta}
\usage{
.hzr_predict_jacobian_numeric(predict_fn, theta)
}
\arguments{
\item{predict_fn}{Function(theta) -> numeric vector of length n.}

\item{theta}{MLE parameter vector.}
}
\value{
Numeric n x p Jacobian.
}
\description{
Numeric Jacobian of a single-distribution prediction w.r.t. theta
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_primitives.R
\name{hzr_log1mexp}
\alias{hzr_log1mexp}
\title{Numerically stable log(1 - exp(-x)) for x > 0}
\usage{
hzr_log1mexp(x)
}
\arguments{
\item{x}{Numeric vector with positive values.}
}
\value{
Numeric vector with element-wise \code{log(1 - exp(-x))}.
}
\description{
Numerically stable log(1 - exp(-x)) for x > 0
}
\examples{
hzr_log1mexp(c(0.01, 0.5, 5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_conserve_events}
\alias{.hzr_conserve_events}
\title{Apply the Conservation of Events adjustment to one phase's log_mu}
\usage{
.hzr_conserve_events(
  theta,
  fixmu_phase,
  fixmu_pos,
  time,
  status,
  phases,
  covariate_counts,
  x_list,
  total_events,
  weights = NULL,
  time_lower = NULL
)
}
\arguments{
\item{theta}{Full parameter vector (internal scale).}

\item{fixmu_phase}{Character: name of the phase whose log_mu is solved.}

\item{fixmu_pos}{Integer: position of that log_mu in theta.}

\item{time}{Numeric vector of follow-up times.}

\item{status}{Numeric event indicator.}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector.}

\item{x_list}{Named list of per-phase design matrices.}

\item{total_events}{Numeric: sum of observed events (precomputed, weighted
when \code{weights} is supplied).}

\item{weights}{Optional numeric vector of row weights (length n). Defaults to
unit weights. Applied when summing per-phase cumhaz so Turner's adjustment
is computed on the same scale as \code{total_events}.}

\item{time_lower}{Optional numeric vector of counting-process entry (start)
times. When supplied, conservation is enforced on the entry-time scale --
\verb{Sum E = Sum [H(stop) - H(start)]} -- by subtracting the entry-time
cumulative hazard, matching the multiphase likelihood (and C HAZARD
\code{setcoe} under \code{LCENSOR}/\code{STARTTME}). \code{NULL} (the default) means no
truncation, i.e. \code{H(start) = 0}.}
}
\value{
Updated theta vector with fixmu phase's log_mu adjusted.
}
\description{
Given the current theta vector, analytically solve the fixmu phase's
log_mu so that total predicted events = total observed events.
}
\details{
This is called BEFORE each likelihood evaluation inside the optimizer,
matching the C HAZARD \code{CONSRV} entry point.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_jacobian_weibull}
\alias{.hzr_predict_jacobian_weibull}
\title{Jacobian of Weibull predictions with respect to theta}
\usage{
.hzr_predict_jacobian_weibull(type, theta, time, x, p)
}
\arguments{
\item{type}{Prediction type (see \code{.hzr_predict_with_se}).}

\item{theta}{MLE parameter vector \code{c(mu, nu, beta_1, ...)}.}

\item{time}{Prediction times (may be NULL for \code{hazard} / \code{linear_predictor}).}

\item{x}{Design matrix (n x p_cov) or NULL.}

\item{p}{Length of theta.}
}
\value{
Numeric n x p Jacobian.
}
\description{
Jacobian of Weibull predictions with respect to theta
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_primitives.R
\name{hzr_log1mexp}
\alias{hzr_log1mexp}
\title{Numerically stable log(1 - exp(-x)) for x > 0}
\usage{
hzr_log1mexp(x)
}
\arguments{
\item{x}{Numeric vector with positive values.}
}
\value{
Numeric vector with element-wise \eqn{\log(1 - e^{-x})}.
}
\description{
Computes
\deqn{\mathrm{log1mexp}(x) = \log\!\bigl(1 - e^{-x}\bigr), \qquad x > 0}
accurately across the full range.  Following \enc{Mächler}{Machler} (2012), the evaluation
switches at \eqn{x = \log 2}: for small \eqn{x} it uses
\eqn{\log(-\mathrm{expm1}(-x))} (avoiding cancellation as \eqn{x \to 0}), and
for larger \eqn{x} it uses \eqn{\mathrm{log1p}(-e^{-x})} (avoiding loss of
precision as \eqn{x \to \infty}).  Non-positive or non-finite inputs return
\code{NA}.
}
\examples{
hzr_log1mexp(c(0.01, 0.5, 5))
}
\references{
\enc{Mächler}{Machler} M (2012). \emph{Accurately Computing \eqn{\log(1 - e^{-|a|})}.} R package
Rmpfr vignette.
\url{https://CRAN.R-project.org/package=Rmpfr/vignettes/log1mexp-note.pdf}
}
\seealso{
\code{\link[=hzr_log1pexp]{hzr_log1pexp()}} for the softplus \eqn{\log(1 + e^{x})},
\code{\link[=hzr_clamp_prob]{hzr_clamp_prob()}} for boundary-safe probabilities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula-helpers.R
\name{.hzr_parse_formula}
\alias{.hzr_parse_formula}
\title{Parse Surv() formula for hazard modeling}
\usage{
.hzr_parse_formula(formula, data)
}
\arguments{
\item{formula}{A formula object with Surv() on the LHS.}

\item{data}{A data frame containing variables referenced in the formula.}
}
\value{
A list with elements: time, status, time_lower, time_upper, x
}
\description{
Extracts time, status, time_lower, time_upper, and predictors from a formula
of the form \code{Surv(time, status) ~ x1 + x2 + ...}.
Supports right-censored, left-censored, interval-censored, and
counting-process (start-stop) data.
}
\details{
For counting-process (start-stop) data, use \code{Surv(start, stop, event)}.
The start times are returned as \code{time_lower} and stop times as \code{time},
enabling the likelihood to compute \code{H(stop) - H(start)} per epoch.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{valves}
\alias{valves}
\title{Valves: Primary Heart Valve Replacement}
\format{
A data frame with 1533 rows and 19 variables:
\describe{
\item{age_cop}{Age at operation (years)}
\item{nyha}{NYHA functional class (1--4)}
\item{mitral}{Mitral valve position indicator (0/1)}
\item{double_}{Double valve replacement indicator (0/1)}
\item{ao_pinc}{Aortic position, incompetence (0/1)}
\item{black}{Black race indicator (0/1)}
\item{i_path}{Ischemic pathology indicator (0/1)}
\item{nve}{Native valve endocarditis indicator (0/1)}
\item{mechvalv}{Mechanical valve indicator (0/1)}
\item{male}{Male sex indicator (0/1)}
\item{int_dead}{Follow-up interval to death or last contact (months)}
\item{dead}{Death indicator (1 = dead, 0 = censored)}
\item{int_pve}{Follow-up interval to PVE or last contact (months)}
\item{pve}{PVE indicator (1 = PVE, 0 = censored)}
\item{bio}{Bioprosthesis indicator (0/1)}
\item{int_rdg}{Follow-up interval to degeneration or last contact (months)}
\item{reop_dg}{Reoperation for degeneration indicator (0/1)}
\item{int_reop}{Follow-up interval to reoperation or last contact (months)}
\item{reop}{Reoperation indicator (0/1)}
}
}
\source{
Cleveland Clinic Foundation heart valve replacement registry.
}
\usage{
valves
}
\description{
Data for 1,533 patients who underwent primary heart valve replacement.
The largest multivariable example dataset with multiple endpoints
including death, prosthetic valve endocarditis (PVE), bioprosthesis
degeneration, and reoperation.
}
\examples{
data(valves)
valves_cc <- na.omit(valves)

# Kaplan-Meier for two endpoints
km_death <- survival::survfit(
  survival::Surv(int_dead, dead) ~ 1, data = valves_cc)
km_pve <- survival::survfit(
  survival::Surv(int_pve, pve) ~ 1, data = valves_cc)

plot(km_death, xlab = "Months after valve replacement", ylab = "Survival",
     main = "Valves: Death and PVE endpoints")
lines(km_pve, col = "red")
legend("bottomleft", c("Death", "PVE"), col = c("black", "red"), lty = 1)

}
\seealso{
\code{vignette("fitting-hazard-models")},
\code{vignette("prediction-visualization")}

Other datasets:
\code{\link{avc}},
\code{\link{cabgkul}},
\code{\link{omc}},
\code{\link{tga}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomposition.R
\name{hzr_decompos}
\alias{hzr_decompos}
\title{Generalized temporal decomposition}
\usage{
hzr_decompos(time, t_half, nu, m)
}
\arguments{
\item{time}{Numeric vector of times (must be > 0).}

\item{t_half}{Half-life: time at which \eqn{G(t_{1/2}) = 0.5}.
Must be > 0.}

\item{nu}{Time exponent controlling rate dynamics.
SAS early: \code{NU}.  SAS late: relates to \code{GAMMA}/\code{ETA}.}

\item{m}{Shape exponent controlling the distributional form.
SAS early: \code{M}.  SAS late: relates to \code{GAMMA}/\code{ALPHA}.}
}
\value{
A named list with three numeric vectors, each the same length
as \code{time}:
\describe{
\item{G}{Cumulative distribution \eqn{G(t) \in [0, 1]}.}
\item{g}{Density \eqn{g(t) = dG/dt \ge 0}.  The "early" phase
temporal pattern.}
\item{h}{Hazard \eqn{h(t) = g(t)/(1 - G(t)) \ge 0}.  The "late"
phase temporal pattern.}
}
}
\description{
Computes the cumulative distribution \eqn{G(t)}, density \eqn{g(t)}, and
hazard \eqn{h(t) = g(t)/(1 - G(t))} for the parametric family defined by
half-life, time exponent, and shape.  This single function generates all
temporal phase shapes used in multiphase hazard models.
}
\section{Parameter mapping from SAS/C HAZARD}{


The original C code used separate parameterizations for early (DELTA,
RHO/THALF, NU, M) and late (TAU, GAMMA, ALPHA, ETA) phases.  Both
collapse onto the three parameters here.  See
\code{\link[=hzr_argument_mapping]{hzr_argument_mapping()}} for the full translation table.
}

\section{Valid parameter combinations}{


Six cases are defined by the signs of \code{nu} and \code{m}:

\tabular{lll}{
\strong{Case} \tab \strong{Sign} \tab \strong{Behavior} \cr
1     \tab m > 0, nu > 0 \tab Standard sigmoidal \cr
1L    \tab m = 0, nu > 0 \tab Exponential-like (Weibull CDF) \cr
2     \tab m < 0, nu > 0 \tab Heavy-tailed \cr
2L    \tab m < 0, nu = 0 \tab Exponential decay \cr
3     \tab m > 0, nu < 0 \tab Bounded cumulative \cr
3L    \tab m = 0, nu < 0 \tab Bounded exponential \cr
}

The combination m < 0 \strong{and} nu < 0 is undefined and raises an error.
}

\examples{
t_grid <- seq(0.1, 10, by = 0.1)

# Case 1: standard sigmoidal (m > 0, nu > 0)
d1 <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 1)
plot(t_grid, d1$G, type = "l", main = "CDF (m=1, nu=2)")

# Case 1L: Weibull-like (m = 0, nu > 0)
d1L <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 0)

# Case 2: heavy-tailed (m < 0, nu > 0)
d2 <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = -1)

}
\references{
Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. \emph{J Am Stat Assoc.} 1986;81(395):615--624.
\doi{10.1080/01621459.1986.10478314}

Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
Probability of atrial fibrillation after ablation: Using a parametric
nonlinear temporal decomposition mixed effects model. \emph{Stat Methods Med Res.}
2018;27(1):126--141. \doi{10.1177/0962280215623583}
}
\seealso{
\code{\link[=hzr_phase_cumhaz]{hzr_phase_cumhaz()}} for the phase-level cumulative hazard
contribution, \code{\link[=hzr_argument_mapping]{hzr_argument_mapping()}} for SAS/C parameter mapping,
\code{\link[=hzr_phase]{hzr_phase()}} for specifying phases in \code{\link[=hazard]{hazard()}} models.

\code{vignette("mf-mathematical-foundations")} for the full derivation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{.hzr_phase_n_shape}
\alias{.hzr_phase_n_shape}
\title{Number of shape parameters for a phase}
\usage{
.hzr_phase_n_shape(phase)
}
\arguments{
\item{phase}{An \code{hzr_phase} object.}
}
\value{
Integer: 3 or 0.
}
\description{
Returns 3 (t_half, nu, m) for \code{"cdf"} and \code{"hazard"} phases, 0 for
\code{"constant"}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_primitives.R
\name{hzr_log1pexp}
\alias{hzr_log1pexp}
\title{Numerically stable log(1 + exp(x))}
\usage{
hzr_log1pexp(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Numeric vector with element-wise \code{log(1 + exp(x))}.
}
\description{
Numerically stable log(1 + exp(x))
}
\examples{
hzr_log1pexp(c(-50, 0, 50))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomposition.R
\name{hzr_decompos}
\alias{hzr_decompos}
\title{Generalized temporal decomposition}
\usage{
hzr_decompos(time, t_half, nu, m)
}
\arguments{
\item{time}{Numeric vector of times (must be > 0).}

\item{t_half}{Half-life: time at which \eqn{G(t_{1/2}) = 0.5}.
Must be > 0.}

\item{nu}{Time exponent controlling rate dynamics.
SAS early: \code{NU}.  SAS late: relates to \code{GAMMA}/\code{ETA}.}

\item{m}{Shape exponent controlling the distributional form.
SAS early: \code{M}.  SAS late: relates to \code{GAMMA}/\code{ALPHA}.}
}
\value{
A named list with three numeric vectors, each the same length
as \code{time}:
\describe{
\item{G}{Cumulative distribution \eqn{G(t) \in [0, 1]}.}
\item{g}{Density \eqn{g(t) = dG/dt \ge 0}.  The "early" phase
temporal pattern.}
\item{h}{Hazard \eqn{h(t) = g(t)/(1 - G(t)) \ge 0}.  The "late"
phase temporal pattern.}
}
}
\description{
Computes the cumulative distribution \eqn{G(t)}, density \eqn{g(t)}, and
hazard \eqn{h(t) = g(t)/(1 - G(t))} for the parametric family defined by
half-life, time exponent, and shape.  This single function generates all
temporal phase shapes used in multiphase hazard models.
}
\section{Parameter mapping from SAS/C HAZARD}{


The original C code used separate parameterizations for early (DELTA,
RHO/THALF, NU, M) and late (TAU, GAMMA, ALPHA, ETA) phases.  Both
collapse onto the three parameters here.  See
\code{\link[=hzr_argument_mapping]{hzr_argument_mapping()}} for the full translation table.
}

\section{Valid parameter combinations}{


Six cases are defined by the signs of \code{nu} and \code{m}:

\tabular{lll}{
\strong{Case} \tab \strong{Sign} \tab \strong{Behavior} \cr
1     \tab m > 0, nu > 0 \tab Standard sigmoidal \cr
1L    \tab m = 0, nu > 0 \tab Exponential-like (Weibull CDF) \cr
2     \tab m < 0, nu > 0 \tab Heavy-tailed \cr
2L    \tab m < 0, nu = 0 \tab Exponential decay \cr
3     \tab m > 0, nu < 0 \tab Bounded cumulative \cr
3L    \tab m = 0, nu < 0 \tab Bounded exponential \cr
}

The combination m < 0 \strong{and} nu < 0 is undefined and raises an error.
\code{nu = 0} is supported only with m < 0 (Case 2L, the exponential-decay
limit); \code{nu = 0} with m >= 0 has no usable limiting form and raises an
error.
}

\section{Mathematical form}{


The construction fixes a rate \eqn{\rho} so that \eqn{G(t_{1/2}) = 0.5}
exactly.  For the base case (\eqn{m > 0,\ \nu > 0}):

\deqn{\rho = \nu \, t_{1/2} \left(\frac{2^m - 1}{m}\right)^{\!\nu},
      \qquad b(t) = \frac{\nu t}{\rho}}

The CDF and density are then

\deqn{G(t) = \bigl(1 + m \, b(t)^{-1/\nu}\bigr)^{-1/m}, \qquad
      g(t) = \bigl(1 + m \, b(t)^{-1/\nu}\bigr)^{-1/m - 1}
             \, b(t)^{-1/\nu - 1} / \rho}

and the hazard is \eqn{h(t) = g(t) / (1 - G(t))}.  The remaining five cases
in the table above arise as limits (\eqn{m \to 0}, \eqn{\nu \to 0}) or sign
reflections of this base form; the implementation dispatches to the
appropriate branch after inspecting the signs of \code{nu} and \code{m}.  See
\code{vignette("mf-mathematical-foundations")} for the full derivation of every
case.
}

\examples{
t_grid <- seq(0.1, 10, by = 0.1)

# Case 1: standard sigmoidal (m > 0, nu > 0)
d1 <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 1)
plot(t_grid, d1$G, type = "l", main = "CDF (m=1, nu=2)")

# Case 1L: Weibull-like (m = 0, nu > 0)
d1L <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = 0)

# Case 2: heavy-tailed (m < 0, nu > 0)
d2 <- hzr_decompos(t_grid, t_half = 3, nu = 2, m = -1)

}
\references{
Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. \emph{J Am Stat Assoc.} 1986;81(395):615--624.
\doi{10.1080/01621459.1986.10478314}

Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
Probability of atrial fibrillation after ablation: Using a parametric
nonlinear temporal decomposition mixed effects model. \emph{Stat Methods Med Res.}
2018;27(1):126--141. \doi{10.1177/0962280215623583}
}
\seealso{
\code{\link[=hzr_phase_cumhaz]{hzr_phase_cumhaz()}} for the phase-level cumulative hazard
contribution, \code{\link[=hzr_argument_mapping]{hzr_argument_mapping()}} for SAS/C parameter mapping,
\code{\link[=hzr_phase]{hzr_phase()}} for specifying phases in \code{\link[=hazard]{hazard()}} models.

\code{vignette("mf-mathematical-foundations")} for the full derivation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tga}
\alias{tga}
\title{TGA: Transposition of the Great Arteries}
\format{
A data frame with 470 rows and 14 variables:
\describe{
\item{study}{Patient identifier}
\item{simple}{Simple TGA indicator (0/1)}
\item{dextroin}{D-looped transposition indicator (0/1)}
\item{ca_1rl2c}{Coronary artery pattern indicator}
\item{hyaaproc}{Hybrid approach procedure indicator (0/1)}
\item{no_tca}{No total circulatory arrest indicator (0/1)}
\item{tca_time}{Total circulatory arrest time (minutes)}
\item{age_days}{Age at operation (days)}
\item{arciopyr}{Aortic cross-clamp time per year}
\item{dead}{Death indicator (1 = dead, 0 = censored)}
\item{int_dead}{Follow-up interval to death or last contact (months)}
\item{source}{Source institution (BCH or CHOP)}
\item{ca1_2_l}{Coronary artery configuration (1/2/L)}
\item{opyear}{Year of operation}
}
}
\source{
Boston Children's Hospital and Children's Hospital of Philadelphia.
}
\usage{
tga
}
\description{
Survival data for 470 patients who underwent the arterial switch operation
for transposition of the great arteries at Boston Children's Hospital and
Children's Hospital of Philadelphia. Used for sensitivity analysis and
internal validation examples.
}
\seealso{
Other datasets:
\code{\link{avc}},
\code{\link{cabgkul}},
\code{\link{omc}},
\code{\link{valves}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_jacobian_multiphase}
\alias{.hzr_predict_jacobian_multiphase}
\title{Jacobian of multiphase cumulative-hazard predictions}
\usage{
.hzr_predict_jacobian_multiphase(
  theta,
  time,
  phases,
  covariate_counts,
  x_list,
  p
)
}
\arguments{
\item{theta}{MLE parameter vector.}

\item{time}{Prediction times (length n).}

\item{phases}{Named list of \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector.}

\item{x_list}{Named list of per-phase design matrices.}

\item{p}{Length of theta.}
}
\value{
Numeric n x p Jacobian of H(t|x).
}
\description{
Only \code{cumulative_hazard} and \code{survival} are delegated here -- \code{hazard}
and \code{linear_predictor} are rejected upstream for multiphase models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_primitives.R
\name{hzr_log1pexp}
\alias{hzr_log1pexp}
\title{Numerically stable log(1 + exp(x))}
\usage{
hzr_log1pexp(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Numeric vector with element-wise \eqn{\log(1 + e^{x})}.
}
\description{
Computes the "softplus" function
\deqn{\mathrm{log1pexp}(x) = \log\!\bigl(1 + e^{x}\bigr)}
without overflow.  A naive \code{log(1 + exp(x))} overflows to \code{Inf} for
\eqn{x \gtrsim 710} even though the true value is \eqn{\approx x}.  The
identity \eqn{\log(1 + e^{x}) = x + \log(1 + e^{-x})} for \eqn{x > 0} keeps
every intermediate finite.
}
\examples{
hzr_log1pexp(c(-50, 0, 50))
}
\references{
\enc{Mächler}{Machler} M (2012). \emph{Accurately Computing \eqn{\log(1 - e^{-|a|})}.} R package
Rmpfr vignette.
\url{https://CRAN.R-project.org/package=Rmpfr/vignettes/log1mexp-note.pdf}
}
\seealso{
\code{\link[=hzr_log1mexp]{hzr_log1mexp()}} for the complementary \eqn{\log(1 - e^{-x})},
\code{\link[=hzr_clamp_prob]{hzr_clamp_prob()}} for boundary-safe probabilities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{is_hzr_phase}
\alias{is_hzr_phase}
\title{Test if an object is an hzr_phase}
\usage{
is_hzr_phase(x)
}
\arguments{
\item{x}{Object to test.}
}
\value{
Logical scalar.
}
\description{
Test if an object is an hzr_phase
}
\examples{
is_hzr_phase(hzr_phase("cdf"))
is_hzr_phase("not a phase")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_log_mu_positions}
\alias{.hzr_log_mu_positions}
\title{Find the position of each phase's log_mu in the full theta vector}
\usage{
.hzr_log_mu_positions(phases, covariate_counts)
}
\arguments{
\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector of per-phase covariate counts.}
}
\value{
Named integer vector: position of log_mu for each phase.
}
\description{
Find the position of each phase's log_mu in the full theta vector
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict-cl.R
\name{.hzr_predict_jacobian_multiphase}
\alias{.hzr_predict_jacobian_multiphase}
\title{Jacobian of multiphase cumulative-hazard predictions}
\usage{
.hzr_predict_jacobian_multiphase(
  theta,
  time,
  phases,
  covariate_counts,
  x_list,
  p,
  per_phase = FALSE
)
}
\arguments{
\item{theta}{MLE parameter vector.}

\item{time}{Prediction times (length n).}

\item{phases}{Named list of \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector.}

\item{x_list}{Named list of per-phase design matrices.}

\item{p}{Length of theta.}

\item{per_phase}{Logical; if \code{TRUE}, return a named list of per-phase
\verb{n x p} Jacobians (each with only that phase's columns nonzero) instead
of their sum.}
}
\value{
Numeric \verb{n x p} Jacobian of \code{H(t|x)} (default), or a named list of
per-phase \verb{n x p} Jacobians when \code{per_phase = TRUE}.
}
\description{
Only \code{cumulative_hazard} and \code{survival} are delegated here -- \code{hazard}
and \code{linear_predictor} are rejected upstream for multiphase models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_unpack_phase_theta}
\alias{.hzr_unpack_phase_theta}
\title{Extract parameters from a phase sub-vector (internal scale)}
\usage{
.hzr_unpack_phase_theta(theta_j, phase)
}
\arguments{
\item{theta_j}{Numeric sub-vector for one phase.}

\item{phase}{An \code{hzr_phase} object.}
}
\value{
Named list with \code{log_mu}, \code{log_t_half}, \code{nu}, \code{m}, \code{beta}.
}
\description{
Extract parameters from a phase sub-vector (internal scale)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{predict.hazard}
\alias{predict.hazard}
\title{Predict from a hazard model object}
\usage{
\method{predict}{hazard}(
  object,
  newdata = NULL,
  type = c("hazard", "linear_predictor", "survival", "cumulative_hazard"),
  decompose = FALSE,
  se.fit = FALSE,
  level = 0.95,
  ...
)
}
\arguments{
\item{object}{A \code{hazard} object.}

\item{newdata}{Optional matrix or data frame of predictors. For types requiring
time (e.g., "survival", "cumulative_hazard"), newdata should include a \code{time}
column, or time will be taken from the fitted object's data.}

\item{type}{Prediction type:
\itemize{
\item \code{"linear_predictor"}: Linear predictor eta = x*beta (not available for multiphase)
\item \code{"hazard"}: Hazard scale exp(eta) (not available for multiphase)
\item \code{"survival"}: Survival probability S(t|x) = exp(-H(t|x))
\item \code{"cumulative_hazard"}: Cumulative hazard H(t|x) at event times
}}

\item{decompose}{Logical; if \code{TRUE} and the model is multiphase, return a
data frame with per-phase cumulative hazard contributions alongside the
total.  Ignored for single-distribution models.  Default \code{FALSE}.}

\item{se.fit}{Logical; if \code{TRUE}, compute delta-method standard errors and
confidence limits for each prediction.  The return value becomes a data
frame with columns \code{fit}, \code{se.fit}, \code{lower}, \code{upper}.  Default \code{FALSE}.
CLs are computed on the log-hazard / log-cumhaz scale and on the
log(-log(survival)) scale so lower/upper stay inside the valid range
of each prediction type; \code{linear_predictor} uses symmetric natural-scale
CLs.  Not compatible with \code{decompose = TRUE}.}

\item{level}{Numeric confidence level in \verb{(0, 1)}; default \code{0.95}.
Only used when \code{se.fit = TRUE}.}

\item{...}{Unused; included for S3 compatibility.}
}
\value{
Numeric vector of predictions.
}
\description{
Produces prediction outputs from a \code{hazard} object. Supports multiple prediction
types including linear predictor, hazard, survival probability, and cumulative hazard.
}
\details{
For Weibull models with survival or cumulative_hazard predictions:
\itemize{
\item Cumulative hazard: H(t|x) = (mu*t)^nu * exp(eta)
\item Survival: S(t|x) = exp(-H(t|x))
}

Time values must be positive and finite. If newdata contains a \code{time} column,
it will be used; otherwise, the time vector from the fitted object is used.
For models fit with \code{time_windows}, predictions for \code{type = "linear_predictor"}
or \code{"hazard"} also require time values (via \code{newdata$time} or fitted-time fallback)
so window-specific coefficients can be selected.
}
\examples{
# -- Basic predictions ------------------------------------------------
set.seed(1)
fit <- hazard(time = rexp(50, 0.3), status = rep(1L, 50),
              theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
predict(fit, type = "survival")
predict(fit, newdata = data.frame(time = c(1, 2, 5)),
        type = "cumulative_hazard")

# -- Patient-specific survival curves ---------------------------------
set.seed(1001)
n   <- 180
dat <- data.frame(
  time   = rexp(n, rate = 0.35) + 0.05,
  status = rbinom(n, size = 1, prob = 0.6),
  age    = rnorm(n, mean = 62, sd = 11),
  nyha   = sample(1:4, n, replace = TRUE),
  shock  = rbinom(n, size = 1, prob = 0.18)
)
fit2 <- hazard(
  survival::Surv(time, status) ~ age + nyha + shock,
  data  = dat,
  theta = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
  dist  = "weibull", fit = TRUE
)

new_patients <- data.frame(
  time = c(0.5, 1.5, 3.0),
  age  = c(50, 65, 75),
  nyha = c(1, 3, 4),
  shock = c(0, 0, 1)
)
# Compute predictions from the clean covariate frame before adding columns
surv   <- predict(fit2, newdata = new_patients, type = "survival")
cumhaz <- predict(fit2, newdata = new_patients, type = "cumulative_hazard")
new_patients$survival          <- surv
new_patients$cumulative_hazard <- cumhaz
new_patients

\donttest{
# -- Grouped survival curves ---------------------------------------
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)

  t_grid <- seq(0.05, max(dat$time), length.out = 80)
  profiles <- data.frame(
    label = c("Low risk (age 50, NYHA I)",
              "High risk (age 75, NYHA IV)"),
    age   = c(50, 75),
    nyha  = c(1, 4),
    shock = c(0, 1)
  )

  curve_list <- lapply(seq_len(nrow(profiles)), function(i) {
    nd <- data.frame(
      time  = t_grid,
      age   = profiles$age[i],
      nyha  = profiles$nyha[i],
      shock = profiles$shock[i]
    )
    nd$survival <- predict(fit2, newdata = nd, type = "survival") * 100
    nd$profile  <- profiles$label[i]
    nd
  })
  curve_df <- do.call(rbind, curve_list)

  ggplot(curve_df, aes(time, survival, colour = profile)) +
    geom_line() +
    scale_y_continuous(limits = c(0, 100)) +
    labs(x = "Months after surgery",
         y = "Freedom from death (\%)",
         title = "Predicted survival by risk profile",
         colour = NULL) +
    theme_minimal()
}
}

\donttest{
# -- Multiphase predictions with decomposition --------------------
set.seed(42)
n   <- 200
dat <- data.frame(
  time   = rexp(n, rate = 0.25) + 0.01,
  status = rbinom(n, size = 1, prob = 0.65)
)
fit_mp <- hazard(
  survival::Surv(time, status) ~ 1,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                       fixed = "shapes"),
    late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
                       fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)

t_grid <- seq(0.01, max(dat$time) * 0.9, length.out = 100)
nd     <- data.frame(time = t_grid)

# Overall survival
predict(fit_mp, newdata = nd, type = "survival")

# Per-phase decomposed cumulative hazard
decomp <- predict(fit_mp, newdata = nd,
                  type = "cumulative_hazard", decompose = TRUE)
head(decomp)
}

}
\seealso{
\code{\link[=hazard]{hazard()}} for model fitting,
\code{\link[=summary.hazard]{summary.hazard()}} for model summaries,
\code{\link[=hzr_phase]{hzr_phase()}} for multiphase temporal shapes.

\code{vignette("prediction-visualization")} for detailed prediction
workflows including decomposed hazard plots and patient-specific curves.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_logl_multiphase}
\alias{.hzr_logl_multiphase}
\title{Log-likelihood for multiphase additive hazard model}
\usage{
.hzr_logl_multiphase(
  theta,
  time,
  status,
  time_lower = NULL,
  time_upper = NULL,
  x = NULL,
  weights = NULL,
  phases,
  covariate_counts,
  x_list,
  return_gradient = FALSE,
  return_hessian = FALSE,
  ...
)
}
\arguments{
\item{theta}{Full parameter vector (internal scale).}

\item{time}{Numeric vector of follow-up times (n).}

\item{status}{Numeric event indicator: 1 = event, 0 = right-censored,
-1 = left-censored, 2 = interval-censored.}

\item{time_lower}{Optional lower bounds for interval censoring.}

\item{time_upper}{Optional upper bounds for left/interval censoring.}

\item{x}{Design matrix (unused directly; kept for interface compatibility).}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector of per-phase covariate counts.}

\item{x_list}{Named list of per-phase design matrices.}

\item{return_gradient}{Logical (ignored; gradient via separate function).}

\item{return_hessian}{Logical (ignored).}

\item{...}{Ignored.}
}
\value{
Scalar log-likelihood. Returns -Inf for infeasible parameters.
}
\description{
Log-likelihood for multiphase additive hazard model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_select_fixmu_phase}
\alias{.hzr_select_fixmu_phase}
\title{Select the phase whose log_mu will be solved by conservation}
\usage{
.hzr_select_fixmu_phase(
  theta,
  time,
  status,
  phases,
  covariate_counts,
  x_list,
  weights = NULL
)
}
\arguments{
\item{theta}{Full parameter vector (internal scale).}

\item{time}{Numeric vector of follow-up times.}

\item{status}{Numeric event indicator.}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector.}

\item{x_list}{Named list of per-phase design matrices.}

\item{weights}{Optional numeric vector of row weights (length n). Defaults to
unit weights. Applied when summing per-phase cumhaz so that selection
happens on the same scale as the (weighted) observed event count.}
}
\value{
Character: name of the phase to fix.
}
\description{
Chooses the phase contributing the largest share of total cumulative
hazard, matching the C HAZARD SETCOE strategy.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazard_api.R
\name{predict.hazard}
\alias{predict.hazard}
\title{Predict from a hazard model object}
\usage{
\method{predict}{hazard}(
  object,
  newdata = NULL,
  type = c("hazard", "linear_predictor", "survival", "cumulative_hazard"),
  decompose = FALSE,
  se.fit = FALSE,
  level = 0.95,
  conf.type = c("log-log", "logit"),
  ...
)
}
\arguments{
\item{object}{A \code{hazard} object.}

\item{newdata}{Optional matrix or data frame of predictors. For types requiring
time (e.g., "survival", "cumulative_hazard"), newdata should include a \code{time}
column, or time will be taken from the fitted object's data.}

\item{type}{Prediction type:
\itemize{
\item \code{"linear_predictor"}: Linear predictor eta = x*beta (not available for multiphase)
\item \code{"hazard"}: Instantaneous hazard. Single-distribution models return the
hazard scale exp(eta); multiphase models return the additive hazard
h(t|x) = sum_j mu_j(x) phi_j'(t) and so require time values (like
\code{"survival"}/\code{"cumulative_hazard"}). \code{decompose} is not supported for
\code{"hazard"}.
\item \code{"survival"}: Survival probability S(t|x) = exp(-H(t|x))
\item \code{"cumulative_hazard"}: Cumulative hazard H(t|x) at event times
}}

\item{decompose}{Logical; if \code{TRUE} and the model is multiphase, return a
data frame with per-phase cumulative hazard contributions alongside the
total.  Ignored for single-distribution models.  Default \code{FALSE}.}

\item{se.fit}{Logical; if \code{TRUE}, compute delta-method standard errors and
confidence limits for each prediction.  The return value becomes a data
frame with columns \code{fit}, \code{se.fit}, \code{lower}, \code{upper}.  Default \code{FALSE}.
CLs are computed on the log-hazard / log-cumhaz scale and on the
log(-log(survival)) scale so lower/upper stay inside the valid range
of each prediction type; \code{linear_predictor} uses symmetric natural-scale
CLs.
For multiphase models, \code{se.fit = TRUE} combines with \code{decompose = TRUE}
when \code{type = "cumulative_hazard"}: the result is a long data frame with one
row per prediction time and component (\code{component} in \code{"total"} plus each
phase name) and columns \code{fit}, \code{se.fit}, \code{lower}, \code{upper}. Per-phase CLs
use only that phase's parameters, so they do not sum to the total CL.
The combination is not available for \code{type = "survival"} (per-phase
survival is not additive).}

\item{level}{Numeric confidence level in \verb{(0, 1)}; default \code{0.95}.
Only used when \code{se.fit = TRUE}.}

\item{conf.type}{Transform for \code{type = "survival"} confidence limits when
\code{se.fit = TRUE}: \code{"log-log"} (default) builds them on \verb{log(-log S)} (the
\code{survival::survfit} standard); \code{"logit"} builds them on \code{logit(1 - S)},
reproducing SAS HAZARD's \code{HAZPRED} survival limits. Other types are
unaffected (hazard/cumulative-hazard use a log scale that already matches
HAZPRED). Only used when \code{se.fit = TRUE}.}

\item{...}{Unused; included for S3 compatibility.}
}
\value{
When \code{se.fit = FALSE} (default), a numeric vector of predictions.
When \code{se.fit = TRUE}, a data frame with columns \code{fit}, \code{se.fit}, \code{lower},
\code{upper} (delta-method point estimate, standard error, and confidence
limits at \code{level}). For multiphase \code{type = "cumulative_hazard"} with
\code{decompose = TRUE}, a long data frame (\code{time}, \code{component}, \code{fit},
\code{se.fit}, \code{lower}, \code{upper}); with \code{decompose = TRUE} and \code{se.fit = FALSE},
a wide data frame of per-phase contributions.
}
\description{
Produces prediction outputs from a \code{hazard} object. Supports multiple prediction
types including linear predictor, hazard, survival probability, and cumulative hazard.
}
\details{
For Weibull models with survival or cumulative_hazard predictions:
\itemize{
\item Cumulative hazard: H(t|x) = (mu*t)^nu * exp(eta)
\item Survival: S(t|x) = exp(-H(t|x))
}

Time values must be positive and finite. If newdata contains a \code{time} column,
it will be used; otherwise, the time vector from the fitted object is used.
For models fit with \code{time_windows}, predictions for \code{type = "linear_predictor"}
or \code{"hazard"} also require time values (via \code{newdata$time} or fitted-time fallback)
so window-specific coefficients can be selected.
}
\examples{
# -- Basic predictions ------------------------------------------------
set.seed(1)
fit <- hazard(time = rexp(50, 0.3), status = rep(1L, 50),
              theta = c(0.3, 1.0), dist = "weibull", fit = TRUE)
predict(fit, type = "survival")
predict(fit, newdata = data.frame(time = c(1, 2, 5)),
        type = "cumulative_hazard")

# -- Patient-specific survival curves ---------------------------------
set.seed(1001)
n   <- 180
dat <- data.frame(
  time   = rexp(n, rate = 0.35) + 0.05,
  status = rbinom(n, size = 1, prob = 0.6),
  age    = rnorm(n, mean = 62, sd = 11),
  nyha   = sample(1:4, n, replace = TRUE),
  shock  = rbinom(n, size = 1, prob = 0.18)
)
fit2 <- hazard(
  survival::Surv(time, status) ~ age + nyha + shock,
  data  = dat,
  theta = c(mu = 0.25, nu = 1.10, beta1 = 0, beta2 = 0, beta3 = 0),
  dist  = "weibull", fit = TRUE
)

new_patients <- data.frame(
  time = c(0.5, 1.5, 3.0),
  age  = c(50, 65, 75),
  nyha = c(1, 3, 4),
  shock = c(0, 0, 1)
)
# Compute predictions from the clean covariate frame before adding columns
surv   <- predict(fit2, newdata = new_patients, type = "survival")
cumhaz <- predict(fit2, newdata = new_patients, type = "cumulative_hazard")
new_patients$survival          <- surv
new_patients$cumulative_hazard <- cumhaz
new_patients

\donttest{
# -- Grouped survival curves ---------------------------------------
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)

  t_grid <- seq(0.05, max(dat$time), length.out = 80)
  profiles <- data.frame(
    label = c("Low risk (age 50, NYHA I)",
              "High risk (age 75, NYHA IV)"),
    age   = c(50, 75),
    nyha  = c(1, 4),
    shock = c(0, 1)
  )

  curve_list <- lapply(seq_len(nrow(profiles)), function(i) {
    nd <- data.frame(
      time  = t_grid,
      age   = profiles$age[i],
      nyha  = profiles$nyha[i],
      shock = profiles$shock[i]
    )
    nd$survival <- predict(fit2, newdata = nd, type = "survival") * 100
    nd$profile  <- profiles$label[i]
    nd
  })
  curve_df <- do.call(rbind, curve_list)

  ggplot(curve_df, aes(time, survival, colour = profile)) +
    geom_line() +
    scale_y_continuous(limits = c(0, 100)) +
    labs(x = "Months after surgery",
         y = "Freedom from death (\%)",
         title = "Predicted survival by risk profile",
         colour = NULL) +
    theme_minimal()
}
}

\donttest{
# -- Multiphase predictions with decomposition --------------------
set.seed(42)
n   <- 200
dat <- data.frame(
  time   = rexp(n, rate = 0.25) + 0.01,
  status = rbinom(n, size = 1, prob = 0.65)
)
fit_mp <- hazard(
  survival::Surv(time, status) ~ 1,
  data   = dat,
  dist   = "multiphase",
  phases = list(
    early = hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                       fixed = "shapes"),
    late  = hzr_phase("cdf", t_half = 5,   nu = 1, m = 0,
                       fixed = "shapes")
  ),
  fit     = TRUE,
  control = list(n_starts = 5, maxit = 1000)
)

t_grid <- seq(0.01, max(dat$time) * 0.9, length.out = 100)
nd     <- data.frame(time = t_grid)

# Overall survival
predict(fit_mp, newdata = nd, type = "survival")

# Per-phase decomposed cumulative hazard
decomp <- predict(fit_mp, newdata = nd,
                  type = "cumulative_hazard", decompose = TRUE)
head(decomp)
}

}
\seealso{
\code{\link[=hazard]{hazard()}} for model fitting,
\code{\link[=summary.hazard]{summary.hazard()}} for model summaries,
\code{\link[=hzr_phase]{hzr_phase()}} for multiphase temporal shapes.

\code{vignette("prediction-visualization")} for detailed prediction
workflows including decomposed hazard plots and patient-specific curves.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{print.hzr_deciles}
\alias{print.hzr_deciles}
\title{Print method for hzr_deciles}
\usage{
\method{print}{hzr_deciles}(x, digits = 3, ...)
}
\arguments{
\item{x}{An \code{hzr_deciles} object.}

\item{digits}{Number of decimal places for formatting.}

\item{...}{Additional arguments (ignored).}
}
\value{
The object \code{x} of class \code{c("hzr_deciles", "data.frame")},
invisibly. The data frame has one row per risk group and columns:
\code{group} (integer group index, 1 = lowest risk),
\code{n} (group size),
\code{events} (observed event count),
\code{expected} (expected event count from model predictions),
\code{observed_rate}, \code{expected_rate} (events / n),
\code{chi_sq} (per-group (O-E)^2/E contribution),
\code{p_value} (1-df chi-square upper-tail p),
\code{mean_survival}, \code{mean_cumhaz} (mean predicted values in group).
An \code{"overall"} attribute contains the omnibus chi-square test
(fields: \code{chi_sq}, \code{df}, \code{p_value}, \code{time},
\code{groups}, \code{total_events}, \code{total_expected},
\code{n_included}, \code{n_excluded}).
}
\description{
Print method for hzr_deciles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{.hzr_phase_free_mask}
\alias{.hzr_phase_free_mask}
\title{Build a logical mask of free (non-fixed) parameters in the full theta vector}
\usage{
.hzr_phase_free_mask(phases, covariate_counts)
}
\arguments{
\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector of per-phase covariate counts.}
}
\value{
Logical vector of length \code{sum(.hzr_phase_n_params(...))}.
}
\description{
Returns a logical vector the same length as the full theta where \code{TRUE}
means the parameter is free (to be optimized) and \code{FALSE} means it is
held fixed at its starting value.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_select_fixmu_phase}
\alias{.hzr_select_fixmu_phase}
\title{Select the phase whose log_mu will be solved by conservation}
\usage{
.hzr_select_fixmu_phase(
  theta,
  time,
  status,
  phases,
  covariate_counts,
  x_list,
  weights = NULL,
  time_lower = NULL
)
}
\arguments{
\item{theta}{Full parameter vector (internal scale).}

\item{time}{Numeric vector of follow-up times.}

\item{status}{Numeric event indicator.}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector.}

\item{x_list}{Named list of per-phase design matrices.}

\item{weights}{Optional numeric vector of row weights (length n). Defaults to
unit weights. Applied when summing per-phase cumhaz so that selection
happens on the same scale as the (weighted) observed event count.}

\item{time_lower}{Optional numeric vector of counting-process entry (start)
times. When supplied, phases are ranked by entry-time cumulative hazard
\code{H(stop) - H(start)}, the scale on which events are conserved. \code{NULL} (the
default) means no truncation, i.e. \code{H(start) = 0}.}
}
\value{
Character: name of the phase to fix.
}
\description{
Chooses the phase contributing the largest share of total cumulative
hazard at the current theta, matching the C HAZARD SETCOE strategy.
}
\details{
When one phase dominates by an extreme margin (>10x the median contribution)
it is typically due to poorly-scaled shape parameters at the starting theta
rather than a genuine signal that the phase should absorb all events.
Selecting such a phase as the fixmu phase traps the optimizer: CoE keeps
rescaling that phase's log_mu upward while the optimizer tries to drive it
to near-zero.  We therefore exclude extreme outliers and select among the
remaining phases.  If all phases are outliers (or only one phase exists)
the largest is used as a fallback.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stepwise.R
\name{hzr_stepwise}
\alias{hzr_stepwise}
\alias{print.hzr_stepwise}
\alias{summary.hzr_stepwise}
\alias{print.summary.hzr_stepwise}
\alias{as.data.frame.hzr_stepwise}
\title{Stepwise covariate selection for a parametric hazard model}
\usage{
hzr_stepwise(
  fit,
  scope = NULL,
  data,
  direction = c("both", "forward", "backward"),
  criterion = c("wald", "aic"),
  slentry = 0.3,
  slstay = 0.2,
  max_steps = 50L,
  max_move = 4L,
  force_in = character(),
  force_out = character(),
  trace = TRUE,
  ...
)

\method{print}{hzr_stepwise}(x, ...)

\method{summary}{hzr_stepwise}(object, ...)

\method{print}{summary.hzr_stepwise}(x, ...)

\method{as.data.frame}{hzr_stepwise}(x, ...)
}
\arguments{
\item{fit}{A fitted \code{hazard} object built via the
\verb{formula = Surv(...) ~ predictors, data = df} interface.}

\item{scope}{Candidate set.  \code{NULL} (default) uses every data-frame
column not already in the model for every phase.  For
single-distribution fits, pass a one-sided formula
(\code{~ age + nyha}) or a character vector of names.  For multiphase
fits, pass a named list of one-sided formulas keyed by phase.}

\item{data}{Data frame the base fit was built on.  Required for
refits.}

\item{direction}{One of \code{"both"} (default), \code{"forward"},
\code{"backward"}.}

\item{criterion}{One of \code{"wald"} (default) or \code{"aic"}.  SAS-style
p-value thresholds apply to Wald; AIC uses \code{DeltaAIC < 0} uniformly.}

\item{slentry}{Entry p-value threshold for the Wald criterion.
Default \code{0.30} matches SAS \code{SLENTRY}.}

\item{slstay}{Retention p-value threshold for the Wald criterion.
Default \code{0.20} matches SAS \code{SLSTAY}.}

\item{max_steps}{Hard cap on total accepted actions.  Emits a
\code{warning()} if hit.  Default \code{50}.}

\item{max_move}{Per-variable oscillation cap.  When a variable has
entered + exited more than \code{max_move} times it is frozen for the
remainder of the run.  Default \code{4}.}

\item{force_in}{Character vector of variables that must remain in
the model.  Such variables are still scored and reported in the
selection trace, but are never dropped.}

\item{force_out}{Character vector of variables that may never be
considered as candidates.}

\item{trace}{Logical; print step-by-step progress to the console.
Default \code{TRUE}.}

\item{...}{Unused.}

\item{x}{An \code{hzr_stepwise} object.}

\item{object}{An \code{hzr_stepwise} object.}
}
\value{
An object of class \code{c("hzr_stepwise", "hazard")} -- the
final fit augmented with:
\describe{
\item{\code{steps}}{Data frame with one row per accepted /
frozen action; see Details.}
\item{\code{scope}}{Record of the candidate scope, plus
\code{force_in}, \code{force_out}, and the frozen set.}
\item{\code{criteria}}{Named list of the threshold / direction
settings actually applied.}
\item{\code{trace_msg}}{Character vector of the trace lines,
captured regardless of the \code{trace} flag.}
\item{\code{elapsed}}{\code{difftime} from start to finish.}
\item{\code{final_call}}{The call that produced this result.}
}

\code{print.hzr_stepwise} returns \code{x} invisibly.

\code{summary.hzr_stepwise} returns a \code{summary.hzr_stepwise} object
(extends \code{summary.hazard}) with \verb{$stepwise_steps} and \verb{$stepwise_trace}
appended.

\code{print.summary.hzr_stepwise} returns \code{x} invisibly.

\code{as.data.frame.hzr_stepwise} returns the \verb{$steps} data frame.
}
\description{
Run forward, backward, or two-way stepwise selection on an existing
\code{hazard} fit using Wald p-values or AIC deltas as the entry /
retention criterion.  Phase-specific entry is supported for
multiphase models: a covariate can enter one phase and not another.
}
\details{
The \code{steps} data frame has columns:

\describe{
\item{\code{step_num}}{Integer sequence starting at 1.}
\item{\code{action}}{\code{"enter"}, \code{"drop"}, or \code{"frozen"}.}
\item{\code{variable}}{Variable affected.}
\item{\code{phase}}{Phase name (multiphase) or \code{NA_character_}.}
\item{\code{criterion}}{\code{"wald"} or \code{"aic"}.}
\item{\code{score}}{Winning score used for the decision.}
\item{\code{stat}, \code{df}}{Wald statistic and degrees of
freedom.}
\item{\code{p_value}, \code{delta_aic}}{Always populated when
computable, regardless of the active criterion.}
\item{\code{logLik}, \code{aic}, \code{n_coef}}{Goodness-of-fit
diagnostics of the model \emph{after} this step.}
}
}
\examples{
data(avc)
avc <- na.omit(avc)
base <- hazard(survival::Surv(int_dead, dead) ~ age,
               data = avc, dist = "weibull", fit = TRUE)
\donttest{
sw <- hzr_stepwise(base, scope = ~ age + nyha,
                   data = avc, direction = "forward",
                   control = list(n_starts = 1))
print(sw)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TemporalHazard-package.R
\docType{package}
\name{TemporalHazard-package}
\alias{TemporalHazard}
\alias{TemporalHazard-package}
\title{TemporalHazard: Temporal Parametric Hazard Modeling}
\description{
Native R implementation of the multiphase parametric hazard model of
Blackstone, Naftel, and Turner (1986), with a focus on behavioral parity,
transparent numerics, and reproducible validation against the original
\sQuote{C}/\sQuote{SAS} HAZARD program.  The package fits time-varying hazards
as an additive sum of parametric \emph{phases} --- early, constant, and late risk
streams --- each carrying its own covariate effects.
}
\section{The multiphase model}{


The total cumulative hazard decomposes additively across \eqn{J} phases:

\deqn{H(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \,
      \Phi_j(t)}

where \eqn{\mu_j(\mathbf{x}) = \exp(\alpha_j + \mathbf{x}_j^\top
\boldsymbol{\beta}_j)} is the phase-specific log-linear scale (intercept plus
covariate effects) and \eqn{\Phi_j(t)} is the temporal shape contributed by
phase \eqn{j}, with its own parameters depending on the phase \code{type} (see the
phase vocabulary below).  The instantaneous hazard and survival follow
directly:

\deqn{h(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \,
      \varphi_j(t), \qquad
      S(t \mid \mathbf{x}) = \exp\!\bigl(-H(t \mid \mathbf{x})\bigr)}

with \eqn{\varphi_j = d\Phi_j/dt}.  Each phase is specified with
\code{\link[=hzr_phase]{hzr_phase()}} and the model is fit by maximum likelihood with \code{\link[=hazard]{hazard()}}.
}

\section{Phase vocabulary}{


\tabular{llll}{
\strong{Phase type} \tab \strong{\eqn{\Phi_j(t)}} \tab \strong{Domain} \tab \strong{Use} \cr
\code{"cdf"} \tab \eqn{G(t)} \tab \eqn{[0, 1]} \tab Early risk that resolves over time \cr
\code{"hazard"} \tab \eqn{-\log(1 - G(t))} \tab \eqn{[0, \infty)} \tab Late or aging risk that accumulates \cr
\code{"g3"} \tab \eqn{G_3(t)} \tab \eqn{[0, \infty)} \tab Unbounded late risk (original C/SAS late phase) \cr
\code{"constant"} \tab \eqn{t} \tab \eqn{[0, \infty)} \tab Flat background rate (no shape parameters) \cr
}

Here \eqn{G(t)} is the generalized temporal decomposition CDF computed by
\code{\link[=hzr_decompos]{hzr_decompos()}}, and \eqn{G_3(t)} is the unbounded late-phase intensity from
\code{\link[=hzr_decompos_g3]{hzr_decompos_g3()}}.  See \code{vignette("mf-mathematical-foundations")} for the
full derivation.
}

\section{SAS/C HAZARD bridge}{


The classic three-phase HAZARD model maps directly onto \code{hzr_phase()} calls:

\tabular{lll}{
\strong{HAZARD phase} \tab \strong{Role} \tab \strong{R equivalent} \cr
G1 (early)    \tab Early resolving risk \tab \code{hzr_phase("cdf", ...)} \cr
G2 (constant) \tab Flat background rate \tab \code{hzr_phase("constant")} \cr
G3 (late)     \tab Rising late risk     \tab \code{hzr_phase("g3", ...)} \cr
}

TemporalHazard generalizes the fixed three-phase structure to \eqn{N} phases
of any type.  \code{\link[=hzr_argument_mapping]{hzr_argument_mapping()}} gives the full parameter translation
table between the SAS/C parameterization and the R arguments.
}

\section{Main entry points}{


\describe{
\item{Model fitting}{\code{\link[=hazard]{hazard()}} --- build and fit single- or multiphase
models; \code{\link[=hzr_phase]{hzr_phase()}} --- specify one phase; \code{\link[=hzr_stepwise]{hzr_stepwise()}} --- forward,
backward, or bidirectional covariate selection.}
\item{Prediction}{\code{predict()} on a fitted \code{hazard} object --- survival,
cumulative hazard, and per-phase decomposed hazard; \code{summary()} ---
coefficient tables with Wald inference.}
\item{Parametric family}{\code{\link[=hzr_decompos]{hzr_decompos()}} --- the early-phase (G1)
decomposition \eqn{G(t)}, \eqn{g(t)}, \eqn{h(t)}; \code{\link[=hzr_decompos_g3]{hzr_decompos_g3()}} ---
the late-phase (G3) intensity; \code{\link[=hzr_phase_cumhaz]{hzr_phase_cumhaz()}} and
\code{\link[=hzr_phase_hazard]{hzr_phase_hazard()}} --- per-phase \eqn{\Phi(t)} and \eqn{\varphi(t)}.}
\item{Diagnostics}{\code{\link[=hzr_kaplan]{hzr_kaplan()}}, \code{\link[=hzr_nelson]{hzr_nelson()}} --- nonparametric
references; \code{\link[=hzr_gof]{hzr_gof()}} --- goodness of fit; \code{\link[=hzr_calibrate]{hzr_calibrate()}},
\code{\link[=hzr_deciles]{hzr_deciles()}} --- calibration; \code{\link[=hzr_bootstrap]{hzr_bootstrap()}} --- resampling CIs;
\code{\link[=hzr_competing_risks]{hzr_competing_risks()}} --- cumulative incidence.}
}
}

\section{Vignettes}{


\describe{
\item{\code{vignette("getting-started")}}{First fit, end to end.}
\item{\code{vignette("mf-mathematical-foundations")}}{The decomposition family
and multiphase model, with derivations.}
\item{\code{vignette("fitting-hazard-models")}}{Single-phase through multiphase
fitting.}
\item{\code{vignette("prediction-visualization")}}{Prediction types and
decomposed-hazard plots.}
\item{\code{vignette("inference-diagnostics")}}{Bootstrap CIs and diagnostics.}
\item{\code{vignette("sas-to-r-migration")}}{Translating SAS/C HAZARD code.}
}
}

\references{
Blackstone EH, Naftel DC, Turner ME Jr. The decomposition of time-varying
hazard into phases, each incorporating a separate stream of concomitant
information. \emph{J Am Stat Assoc.} 1986;81(395):615--624.
\doi{10.1080/01621459.1986.10478314}

Rajeswaran J, Blackstone EH, Ehrlinger J, Li L, Ishwaran H, Parides MK.
Probability of atrial fibrillation after ablation: Using a parametric
nonlinear temporal decomposition mixed effects model. \emph{Stat Methods Med Res.}
2018;27(1):126--141. \doi{10.1177/0962280215623583}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ehrlinger.github.io/temporal_hazard/}
  \item \url{https://github.com/ehrlinger/temporal_hazard}
  \item Report bugs at \url{https://github.com/ehrlinger/temporal_hazard/issues}
}

}
\author{
\strong{Maintainer}: John Ehrlinger \email{john.ehrlinger@gmail.com} [copyright holder]

Authors:
\itemize{
  \item John Ehrlinger \email{john.ehrlinger@gmail.com} [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_multiphase_cumhaz}
\alias{.hzr_multiphase_cumhaz}
\title{Compute total cumulative hazard H(t|x) for multiphase model}
\usage{
.hzr_multiphase_cumhaz(
  time,
  theta,
  phases,
  covariate_counts,
  x_list,
  per_phase = FALSE
)
}
\arguments{
\item{time}{Numeric vector of times (n).}

\item{theta}{Full parameter vector (internal scale).}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector of covariate counts per phase.}

\item{x_list}{Named list of design matrices, one per phase. Each element is
an n x p_j matrix or NULL.}

\item{per_phase}{Logical; if TRUE return a list with total and per-phase
contributions.}
}
\value{
If \code{per_phase = FALSE}: numeric vector of length n (total H(t|x)).
If \code{per_phase = TRUE}: named list with \verb{$total} and one element per phase.
}
\description{
Compute total cumulative hazard H(t|x) for multiphase model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomposition.R
\name{hzr_phase_hazard}
\alias{hzr_phase_hazard}
\title{Instantaneous hazard contribution from a single phase}
\usage{
hzr_phase_hazard(
  time,
  t_half = 1,
  nu = 1,
  m = 0,
  type = c("cdf", "hazard", "constant")
)
}
\arguments{
\item{time}{Numeric vector of times (> 0).}

\item{t_half}{Half-life parameter (> 0).}

\item{nu}{Time exponent.}

\item{m}{Shape parameter.}

\item{type}{Phase type: \code{"cdf"} (early -- uses \eqn{G(t)}),
\code{"hazard"} (late -- uses cumulative hazard from \eqn{h(t)}), or
\code{"constant"} (flat rate -- \eqn{\Phi = t}).}
}
\value{
Numeric vector of instantaneous hazard contributions \eqn{\phi(t)},
same length as \code{time}.
}
\description{
Computes \eqn{\phi_j(t) = d\Phi_j/dt} for one phase -- the derivative of
the cumulative hazard contribution returned by \code{\link[=hzr_phase_cumhaz]{hzr_phase_cumhaz()}}.
}
\details{
\itemize{
\item \code{"cdf"}: \eqn{\phi(t) = g(t)} (density).
\item \code{"hazard"}: \eqn{\phi(t) = h(t) = g(t)/(1-G(t))}.
\item \code{"constant"}: \eqn{\phi(t) = 1}.
}
}
\examples{
t_grid <- seq(0.1, 10, by = 0.1)
phi_early <- hzr_phase_hazard(t_grid, t_half = 2, nu = 2, m = 0,
                               type = "cdf")
phi_late  <- hzr_phase_hazard(t_grid, t_half = 5, nu = 1, m = 0,
                               type = "hazard")

}
\seealso{
\code{\link[=hzr_decompos]{hzr_decompos()}} for the underlying parametric family,
\code{\link[=hzr_phase_cumhaz]{hzr_phase_cumhaz()}} for the cumulative version.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_split_theta}
\alias{.hzr_split_theta}
\title{Split the full theta vector into per-phase sub-vectors}
\usage{
.hzr_split_theta(theta, phases, covariate_counts)
}
\arguments{
\item{theta}{Numeric vector -- full parameter vector (internal scale).}

\item{phases}{Named list of validated \code{hzr_phase} objects.}

\item{covariate_counts}{Named integer vector -- number of covariates per phase.}
}
\value{
Named list of numeric vectors, one per phase.
}
\description{
Split the full theta vector into per-phase sub-vectors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{hzr_phase}
\alias{hzr_phase}
\alias{print.hzr_phase}
\title{Specify a single hazard phase}
\usage{
hzr_phase(
  type = c("cdf", "hazard", "constant", "g3"),
  t_half = 1,
  nu = 1,
  m = 0,
  tau = 1,
  gamma = 1,
  alpha = 1,
  eta = 1,
  formula = NULL,
  fixed = character(0)
)

\method{print}{hzr_phase}(x, ...)
}
\arguments{
\item{type}{Character; phase type: \code{"cdf"}, \code{"hazard"}, \code{"g3"}, or
\code{"constant"}.}

\item{t_half}{Positive scalar; initial half-life (time at which
\eqn{G(t_{1/2}) = 0.5}).  Used for \code{"cdf"} and \code{"hazard"} phases.
SAS early: \code{THALF}/\code{RHO}.}

\item{nu}{Numeric scalar; initial time exponent.  Used for \code{"cdf"} and
\code{"hazard"} phases.  SAS early: \code{NU}.}

\item{m}{Numeric scalar; initial shape exponent.  Used for \code{"cdf"} and
\code{"hazard"} phases.  SAS early: \code{M}.}

\item{tau}{Positive scalar; scale parameter for \code{"g3"} phases.
SAS late: \code{TAU}.}

\item{gamma}{Positive scalar; time exponent for \code{"g3"} phases.
SAS late: \code{GAMMA}.}

\item{alpha}{Non-negative scalar; shape parameter for \code{"g3"} phases.
When \code{alpha > 0}, the generic G3 formula is used; \code{alpha = 0} gives the
exponential limiting case.  SAS late: \code{ALPHA}.}

\item{eta}{Positive scalar; outer exponent for \code{"g3"} phases.
SAS late: \code{ETA}.}

\item{formula}{Optional one-sided formula (e.g. \code{~ age + nyha}) for
phase-specific covariates.  When \code{NULL} (default), the phase inherits
the global formula from \code{\link[=hazard]{hazard()}}.}

\item{fixed}{Character vector naming shape parameters to hold fixed during
optimization.  Valid names for \code{"cdf"}/\code{"hazard"}: \code{"t_half"}, \code{"nu"},
\code{"m"}, or \code{"shapes"} (shorthand for all three).  Valid names for \code{"g3"}:
\code{"tau"}, \code{"gamma"}, \code{"alpha"}, \code{"eta"}, or \code{"shapes"} (shorthand for all
four).  Fixed parameters are held at their starting values; only \code{mu}
(and covariates) are estimated.  Ignored for \code{"constant"} phases.
This mirrors the SAS/C HAZARD workflow where shapes are typically fixed
and only scale parameters are estimated.}

\item{x}{An \code{hzr_phase} object (for \code{print.hzr_phase()}).}

\item{...}{Additional arguments (ignored).}
}
\value{
An S3 object of class \code{"hzr_phase"} with elements:
\describe{
\item{type}{Phase type string.}
\item{t_half}{Initial half-life (cdf/hazard phases).}
\item{nu}{Initial time exponent (cdf/hazard phases).}
\item{m}{Initial shape exponent (cdf/hazard phases).}
\item{tau}{Scale parameter (g3 phases).}
\item{gamma}{Time exponent (g3 phases).}
\item{alpha}{Shape parameter (g3 phases).}
\item{eta}{Outer exponent (g3 phases).}
\item{formula}{Phase-specific formula or \code{NULL}.}
\item{fixed}{Character vector of fixed parameter names (may be empty).}
}
}
\description{
Creates an \code{hzr_phase} object describing one term in a multiphase additive
cumulative hazard model.  Pass a list of these to the \code{phases} argument of
\code{\link[=hazard]{hazard()}} when \code{dist = "multiphase"}.
}
\section{Phase types}{


\describe{
\item{\code{"cdf"}}{Early risk that resolves over time.
\eqn{\Phi(t) = G(t)}, bounded \eqn{[0, 1]}.
SAS equivalent: Early / G1 phase.}
\item{\code{"hazard"}}{Late or aging risk that accumulates.
\eqn{\Phi(t) = -\log(1 - G(t))}, monotone increasing.
SAS equivalent: Late / G3 phase.}
\item{\code{"constant"}}{Flat background hazard rate.
\eqn{\Phi(t) = t}. No shape parameters are estimated.
SAS equivalent: Constant / G2 phase.}
}
}

\examples{
# Classic 3-phase Blackstone pattern
early <- hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0)
const <- hzr_phase("constant")
late  <- hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1)

# Fix all shapes (C/SAS-style: only estimate mu)
early_fixed <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                          fixed = "shapes")
late_fixed  <- hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")

# Fix only some parameters
early_partial <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                            fixed = c("nu", "m"))

# Phase with specific covariates
early_cov <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                        formula = ~ age + shock)

# Use in hazard():
# hazard(Surv(time, status) ~ age, data = dat,
#        dist = "multiphase",
#        phases = list(early = early, constant = const, late = late))

}
\seealso{
\code{\link[=hazard]{hazard()}} for fitting multiphase models,
\code{\link[=hzr_decompos]{hzr_decompos()}} for the underlying parametric family,
\code{\link[=hzr_phase_cumhaz]{hzr_phase_cumhaz()}} and \code{\link[=hzr_phase_hazard]{hzr_phase_hazard()}} for computing
\eqn{\Phi(t)} and \eqn{\phi(t)} from these specifications.

\code{vignette("fitting-hazard-models")} for multiphase fitting examples,
\code{vignette("mf-mathematical-foundations")} for the mathematical framework.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stepwise.R
\name{hzr_stepwise}
\alias{hzr_stepwise}
\alias{print.hzr_stepwise}
\alias{summary.hzr_stepwise}
\alias{print.summary.hzr_stepwise}
\alias{as.data.frame.hzr_stepwise}
\title{Stepwise covariate selection for a parametric hazard model}
\usage{
hzr_stepwise(
  fit,
  scope = NULL,
  data,
  direction = c("both", "forward", "backward"),
  criterion = c("wald", "aic"),
  slentry = 0.3,
  slstay = 0.2,
  max_steps = 50L,
  max_move = 4L,
  force_in = character(),
  force_out = character(),
  trace = TRUE,
  ...
)

\method{print}{hzr_stepwise}(x, ...)

\method{summary}{hzr_stepwise}(object, ...)

\method{print}{summary.hzr_stepwise}(x, ...)

\method{as.data.frame}{hzr_stepwise}(x, ...)
}
\arguments{
\item{fit}{A fitted \code{hazard} object built via the
\verb{formula = Surv(...) ~ predictors, data = df} interface.}

\item{scope}{Candidate set.  \code{NULL} (default) uses every data-frame
column not already in the model for every phase.  For
single-distribution fits, pass a one-sided formula
(\code{~ age + nyha}) or a character vector of names.  For multiphase
fits, pass a named list of one-sided formulas keyed by phase.}

\item{data}{Data frame the base fit was built on.  Required for
refits.}

\item{direction}{Search strategy --- one of \code{"both"} (default),
\code{"forward"}, or \code{"backward"}.  Controls whether variables may only
enter, only leave, or both.  See the \strong{Selection direction and
criterion} section.}

\item{criterion}{Entry / retention rule --- one of \code{"wald"} (default)
or \code{"aic"}.  \code{"wald"} applies SAS-style p-value thresholds
(\code{slentry} / \code{slstay}); \code{"aic"} adds or drops whenever it lowers the
AIC.  See the \strong{Selection direction and criterion} section.}

\item{slentry}{Entry p-value threshold for the Wald criterion.
Default \code{0.30} matches SAS \code{SLENTRY}.}

\item{slstay}{Retention p-value threshold for the Wald criterion.
Default \code{0.20} matches SAS \code{SLSTAY}.}

\item{max_steps}{Hard cap on total accepted actions.  Emits a
\code{warning()} if hit.  Default \code{50}.}

\item{max_move}{Per-variable oscillation cap.  When a variable has
entered + exited more than \code{max_move} times it is frozen for the
remainder of the run.  Default \code{4}.}

\item{force_in}{Character vector of variables that must remain in
the model.  Such variables are still scored and reported in the
selection trace, but are never dropped.}

\item{force_out}{Character vector of variables that may never be
considered as candidates.}

\item{trace}{Logical; print step-by-step progress to the console.
Default \code{TRUE}.}

\item{...}{Unused.}

\item{x}{An \code{hzr_stepwise} object.}

\item{object}{An \code{hzr_stepwise} object.}
}
\value{
An object of class \code{c("hzr_stepwise", "hazard")} -- the
final fit augmented with:
\describe{
\item{\code{steps}}{Data frame with one row per accepted /
frozen action; see Details.}
\item{\code{scope}}{Record of the candidate scope, plus
\code{force_in}, \code{force_out}, and the frozen set.}
\item{\code{criteria}}{Named list of the threshold / direction
settings actually applied.}
\item{\code{trace_msg}}{Character vector of the trace lines,
captured regardless of the \code{trace} flag.}
\item{\code{elapsed}}{\code{difftime} from start to finish.}
\item{\code{final_call}}{The call that produced this result.}
}

\code{print.hzr_stepwise} returns \code{x} invisibly.

\code{summary.hzr_stepwise} returns a \code{summary.hzr_stepwise} object
(extends \code{summary.hazard}) with \verb{$stepwise_steps} and \verb{$stepwise_trace}
appended.

\code{print.summary.hzr_stepwise} returns \code{x} invisibly.

\code{as.data.frame.hzr_stepwise} returns the \verb{$steps} data frame.
}
\description{
Run forward, backward, or two-way stepwise selection on an existing
\code{hazard} fit using Wald p-values or AIC deltas as the entry /
retention criterion.  Phase-specific entry is supported for
multiphase models: a covariate can enter one phase and not another.
}
\details{
The \code{steps} data frame has columns:

\describe{
\item{\code{step_num}}{Integer sequence starting at 1.}
\item{\code{action}}{\code{"enter"}, \code{"drop"}, or \code{"frozen"}.}
\item{\code{variable}}{Variable affected.}
\item{\code{phase}}{Phase name (multiphase) or \code{NA_character_}.}
\item{\code{criterion}}{\code{"wald"} or \code{"aic"}.}
\item{\code{score}}{Winning score used for the decision.}
\item{\code{stat}, \code{df}}{Wald statistic and degrees of
freedom.}
\item{\code{p_value}, \code{delta_aic}}{Always populated when
computable, regardless of the active criterion.}
\item{\code{logLik}, \code{aic}, \code{n_coef}}{Goodness-of-fit
diagnostics of the model \emph{after} this step.}
}
}
\section{Selection direction and criterion}{


Two arguments shape the search.  \code{direction} decides which moves are
allowed at each step; \code{criterion} decides how a candidate move is scored
and whether it is accepted.

\describe{
\item{\code{direction = "forward"}}{Start from the base model and only
\emph{add} variables --- the best eligible candidate enters each step
until none clears the entry rule.  Variables never leave once in.}
\item{\code{direction = "backward"}}{Start from the full candidate model and
only \emph{drop} variables --- the weakest term leaves each step until all
survivors clear the retention rule.}
\item{\code{direction = "both"} (default)}{Two-way stepwise: after each
entry, already-selected variables are re-tested and may be dropped.
This is the SAS \code{SELECTION = STEPWISE} strategy.  \code{max_move} caps how
often a single variable may oscillate before it is frozen.}
}

\describe{
\item{\code{criterion = "wald"} (default)}{Accept moves on SAS-style
significance thresholds, using the Wald \eqn{\chi^2} of the affected
coefficient(s): a candidate enters if its p-value is below \code{slentry}, and
a term is dropped if its p-value rises above \code{slstay}.  Entry candidates
are scored from a refit that adds the candidate (so its new coefficient
can be tested); drop candidates are scored from the \emph{current} model's
Wald p-values without a per-candidate refit, and a single refit is run
only after a drop is chosen.  Note this differs algorithmically from
C/SAS HAZARD, which selects on a \emph{score} (Q) statistic evaluated without
refitting; the two can take different step paths even when they converge
to a similar final model.}
\item{\code{criterion = "aic"}}{Accept any move with
\eqn{\Delta\mathrm{AIC} < 0} (a strictly better penalised fit), ignoring
\code{slentry} / \code{slstay}.  Entry candidates use the actual
\eqn{\Delta\mathrm{AIC}} from the candidate refit; drop candidates use a
Wald-to-likelihood-ratio approximation,
\eqn{\Delta\mathrm{AIC} \approx W - 2\,\mathrm{df}}, computed from the
current model without a per-candidate refit (the chosen drop is refit
afterwards).  Use this for a non-significance-based,
information-criterion search.}
}
}

\examples{
data(avc)
avc <- na.omit(avc)
base <- hazard(survival::Surv(int_dead, dead) ~ age,
               data = avc, dist = "weibull", fit = TRUE)
\donttest{
sw <- hzr_stepwise(base, scope = ~ age + nyha,
                   data = avc, direction = "forward",
                   control = list(n_starts = 1))
print(sw)
}

}
\seealso{
\code{\link[=hazard]{hazard()}} for the base model and \code{\link[=hzr_phase]{hzr_phase()}} for multiphase
scopes; \code{\link[=stepwise_trace]{stepwise_trace()}} to retrieve the captured selection log.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomposition.R
\name{.log_expm1}
\alias{.log_expm1}
\title{Compute log(exp(x) - 1) with numerical stability}
\usage{
.log_expm1(x)
}
\arguments{
\item{x}{Numeric vector (must be > 0 for exp(x) > 1).}
}
\value{
Numeric vector: log(exp(x) - 1).
}
\description{
Compute log(exp(x) - 1) with numerical stability
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likelihood-multiphase.R
\name{.hzr_g3_phase_derivatives}
\alias{.hzr_g3_phase_derivatives}
\title{Finite-difference derivatives of G3 phase Phi and phi w.r.t. shape params}
\usage{
.hzr_g3_phase_derivatives(time, tau, gamma, alpha, eta, h = 1e-05)
}
\arguments{
\item{time}{Numeric vector of positive times.}

\item{tau}{Positive scalar scale parameter.}

\item{gamma}{Positive scalar time exponent.}

\item{alpha}{Non-negative scalar shape parameter.}

\item{eta}{Positive scalar outer exponent.}

\item{h}{Relative step size for finite differences (default 1e-5).}
}
\value{
Named list with Phi, phi, and 8 derivative vectors.
}
\description{
Computes the G3 cumulative intensity, its time derivative, and their
partial derivatives with respect to log_tau, gamma, alpha, and eta using
central finite differences.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase-spec.R
\name{hzr_phase}
\alias{hzr_phase}
\alias{print.hzr_phase}
\title{Specify a single hazard phase}
\usage{
hzr_phase(
  type = c("cdf", "hazard", "constant", "g3"),
  t_half = 1,
  nu = 1,
  m = 0,
  tau = 1,
  gamma = 1,
  alpha = 1,
  eta = 1,
  formula = NULL,
  fixed = character(0)
)

\method{print}{hzr_phase}(x, ...)
}
\arguments{
\item{type}{Character; the phase's temporal shape --- one of \code{"cdf"}
(early resolving risk), \code{"hazard"} (accumulating G1 aging risk), \code{"g3"}
(late rising risk, the original C/SAS late phase), or \code{"constant"} (flat
background rate).  See the \strong{Phase types} section for what each means and
when to use it.}

\item{t_half}{Positive scalar; initial half-life (time at which
\eqn{G(t_{1/2}) = 0.5}).  Used for \code{"cdf"} and \code{"hazard"} phases.
SAS early: \code{THALF}/\code{RHO}.}

\item{nu}{Numeric scalar; initial time exponent.  Used for \code{"cdf"} and
\code{"hazard"} phases.  SAS early: \code{NU}.}

\item{m}{Numeric scalar; initial shape exponent.  Used for \code{"cdf"} and
\code{"hazard"} phases.  SAS early: \code{M}.}

\item{tau}{Positive scalar; scale parameter for \code{"g3"} phases.
SAS late: \code{TAU}.}

\item{gamma}{Positive scalar; time exponent for \code{"g3"} phases.
SAS late: \code{GAMMA}.}

\item{alpha}{Non-negative scalar; shape parameter for \code{"g3"} phases.
When \code{alpha > 0}, the generic G3 formula is used; \code{alpha = 0} gives the
exponential limiting case.  SAS late: \code{ALPHA}.}

\item{eta}{Positive scalar; outer exponent for \code{"g3"} phases.
SAS late: \code{ETA}.}

\item{formula}{Optional one-sided formula (e.g. \code{~ age + nyha}) for
phase-specific covariates.  When \code{NULL} (default), the phase inherits
the global formula from \code{\link[=hazard]{hazard()}}.}

\item{fixed}{Character vector naming shape parameters to hold fixed during
optimization.  Valid names for \code{"cdf"}/\code{"hazard"}: \code{"t_half"}, \code{"nu"},
\code{"m"}, or \code{"shapes"} (shorthand for all three).  Valid names for \code{"g3"}:
\code{"tau"}, \code{"gamma"}, \code{"alpha"}, \code{"eta"}, or \code{"shapes"} (shorthand for all
four).  Fixed parameters are held at their starting values; only \code{mu}
(and covariates) are estimated.  Ignored for \code{"constant"} phases.
This mirrors the SAS/C HAZARD workflow where shapes are typically fixed
and only scale parameters are estimated.}

\item{x}{An \code{hzr_phase} object (for \code{print.hzr_phase()}).}

\item{...}{Additional arguments (ignored).}
}
\value{
An S3 object of class \code{"hzr_phase"} with elements:
\describe{
\item{type}{Phase type string.}
\item{t_half}{Initial half-life (cdf/hazard phases).}
\item{nu}{Initial time exponent (cdf/hazard phases).}
\item{m}{Initial shape exponent (cdf/hazard phases).}
\item{tau}{Scale parameter (g3 phases).}
\item{gamma}{Time exponent (g3 phases).}
\item{alpha}{Shape parameter (g3 phases).}
\item{eta}{Outer exponent (g3 phases).}
\item{formula}{Phase-specific formula or \code{NULL}.}
\item{fixed}{Character vector of fixed parameter names (may be empty).}
}
}
\description{
Creates an \code{hzr_phase} object describing one term in a multiphase additive
cumulative hazard model.  Pass a list of these to the \code{phases} argument of
\code{\link[=hazard]{hazard()}} when \code{dist = "multiphase"}.
}
\section{Role in the multiphase model}{


Each phase is one term \eqn{j} in the additive cumulative hazard

\deqn{H(t \mid \mathbf{x}) = \sum_{j=1}^{J} \mu_j(\mathbf{x}) \,
      \Phi_j(t)}

where \eqn{\mu_j(\mathbf{x}) = \exp(\alpha_j + \mathbf{x}_j^\top
\boldsymbol{\beta}_j)} is the phase-specific log-linear scale and
\eqn{\Phi_j(t)} is the temporal shape selected by \code{type} (below).  The
\code{t_half}/\code{nu}/\code{m} (or g3 \code{tau}/\code{gamma}/\code{alpha}/\code{eta}) arguments set the
starting values for that shape; \code{formula} attaches the covariates
\eqn{\mathbf{x}_j} that enter \eqn{\mu_j}.
}

\section{Phase types}{


The \code{type} argument chooses the temporal shape \eqn{\Phi_j(t)} for the phase.
Each captures a qualitatively different pattern of risk over time; a typical
clinical model combines an \emph{early}, a \emph{constant}, and a \emph{late} phase so that
the total hazard can fall, level off, and rise again.

\describe{
\item{\code{"cdf"} --- early, resolving risk}{Named for the \strong{c}umulative
\strong{d}istribution \strong{f}unction: the phase contributes \eqn{\Phi(t) = G(t)},
the bounded CDF of the temporal decomposition (\eqn{0} at \eqn{t = 0},
rising to a ceiling of \eqn{1}).  Because it saturates, the \emph{hazard} it
adds, \eqn{\mu\,g(t)}, peaks early and then decays toward zero --- the
signature of a one-time insult that patients either succumb to or survive
past, e.g. peri-operative mortality.  Shape set by \code{t_half}, \code{nu}, \code{m}.
SAS/C equivalent: the Early (G1) phase.}
\item{\code{"hazard"} --- accumulating aging risk (G1 family)}{Named because the
phase contributes a \strong{cumulative hazard} built from the same G1 family:
\eqn{\Phi(t) = -\log(1 - G(t))}, which is unbounded and monotone
increasing.  Its hazard \eqn{\mu\,h(t)} rises without leveling off, so it
models risk that grows as subjects age.  This is an alternative late-risk
form derived from G1; for the original SAS/C late phase prefer \code{"g3"}.
Shape set by \code{t_half}, \code{nu}, \code{m}.}
\item{\code{"g3"} --- late, rising risk (original C/SAS late phase)}{Named for
the \strong{G3} (third) decomposition family used by the original HAZARD
program for the late phase.  It contributes \eqn{\Phi(t) = G_3(t)} from
\code{\link[=hzr_decompos_g3]{hzr_decompos_g3()}}, an unbounded intensity with its own four-parameter
shape (\code{tau}, \code{gamma}, \code{alpha}, \code{eta}) that is more flexible than the
G1-derived \code{"hazard"} form for capturing accelerating late mortality
(e.g. structural valve deterioration years after surgery).  Use this when
reproducing classic three-phase HAZARD models.  SAS/C equivalent: the
Late (G3) phase.}
\item{\code{"constant"} --- flat background rate}{A time-invariant hazard:
\eqn{\Phi(t) = t}, so the added hazard \eqn{\mu} is constant (the
exponential model).  It represents the steady, ongoing risk present at all
follow-up times, independent of how long ago the time origin was.  Takes
no shape parameters --- only its scale \eqn{\mu} (and any covariates) is
estimated.  SAS/C equivalent: the Constant (G2) phase.}
}

The shape derivative \eqn{\varphi_j = d\Phi_j/dt} (which forms the
instantaneous hazard contribution \eqn{\mu_j\,\varphi_j(t)}) is \eqn{g(t)} for
\code{"cdf"}, \eqn{h(t)} for \code{"hazard"}, \eqn{g_3(t)} for \code{"g3"}, and \eqn{1} for
\code{"constant"}.
}

\examples{
# Classic 3-phase Blackstone pattern
early <- hzr_phase("cdf",      t_half = 0.5, nu = 2, m = 0)
const <- hzr_phase("constant")
late  <- hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1)

# Fix all shapes (C/SAS-style: only estimate mu)
early_fixed <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                          fixed = "shapes")
late_fixed  <- hzr_phase("g3", tau = 1, gamma = 3, alpha = 1, eta = 1,
                          fixed = "shapes")

# Fix only some parameters
early_partial <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                            fixed = c("nu", "m"))

# Phase with specific covariates
early_cov <- hzr_phase("cdf", t_half = 0.5, nu = 2, m = 0,
                        formula = ~ age + shock)

# Use in hazard():
# hazard(Surv(time, status) ~ age, data = dat,
#        dist = "multiphase",
#        phases = list(early = early, constant = const, late = late))

}
\seealso{
\code{\link[=hazard]{hazard()}} for fitting multiphase models,
\code{\link[=hzr_decompos]{hzr_decompos()}} for the underlying parametric family,
\code{\link[=hzr_phase_cumhaz]{hzr_phase_cumhaz()}} and \code{\link[=hzr_phase_hazard]{hzr_phase_hazard()}} for computing
\eqn{\Phi(t)} and \eqn{\phi(t)} from these specifications.

\code{vignette("fitting-hazard-models")} for multiphase fitting examples,
\code{vignette("mf-mathematical-foundations")} for the mathematical framework.
}
